Package: bibliometrix
Type: Package
Title: Comprehensive Science Mapping Analysis
Version: 5.3.0
Authors@R: c(
    person(given = "Massimo",
           family = "Aria",
           role = c("cre", "aut", "cph"),
           email = "aria@unina.it",
           comment = c(ORCID = "0000-0002-8517-9411")),
    person(given = "Corrado", 
           family = "Cuccurullo", 
           role = "aut",
           email = "cuccurullocorrado@gmail.com",
           comment = c(ORCID = "0000-0002-7401-8575")))
Description: Tool for quantitative research in scientometrics and bibliometrics.
    It implements the comprehensive workflow for science mapping analysis proposed in Aria M. and 
    Cuccurullo C. (2017) <doi:10.1016/j.joi.2017.08.007>.
    'bibliometrix' provides various routines for importing bibliographic data from 'SCOPUS',
    'Clarivate Analytics Web of Science' (<https://www.webofknowledge.com/>), 'Digital Science Dimensions' 
	(<https://www.dimensions.ai/>), 'OpenAlex' (<https://openalex.org/>), 'Cochrane Library' (<https://www.cochranelibrary.com/>),  'Lens' (<https://lens.org>), 
	and 'PubMed' (<https://pubmed.ncbi.nlm.nih.gov/>) databases, performing bibliometric analysis 
    and building networks for co-citation, coupling, scientific collaboration and co-word analysis.
License: GPL-3
URL: https://www.bibliometrix.org,
        https://github.com/massimoaria/bibliometrix,
        https://www.k-synth.com
BugReports: https://github.com/massimoaria/bibliometrix/issues
LazyData: true
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: stats, grDevices, bibliometrixData, contentanalysis (>=
        1.0.0), dimensionsR, dplyr, ca, forcats, ggplot2, ggrepel,
        igraph, Matrix, methods, plotly, openalexR, openxlsx, pubmedR,
        purrr, readr, readxl, rscopus, shiny, shinycssloaders (>=
        1.1.0), SnowballC, stringdist, stringi, stringr, tibble, tidyr,
        tidytext, visNetwork
Suggests: knitr, ragg, rmarkdown, testthat (>= 3.0.0), wordcloud2
RoxygenNote: 7.3.3
NeedsCompilation: no
Config/testthat/edition: 3
Packaged: 2026-04-08 10:47:11 UTC; massimoaria
Author: Massimo Aria [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8517-9411>),
  Corrado Cuccurullo [aut] (ORCID:
    <https://orcid.org/0000-0002-7401-8575>)
Maintainer: Massimo Aria <aria@unina.it>
Repository: CRAN
Date/Publication: 2026-04-09 05:10:50 UTC
Built: R 4.5.2; ; 2026-04-09 07:32:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,bibliodendrogram)
S3method(plot,bibliometrix)
S3method(summary,bibliometrix)
S3method(summary,bibliometrix_netstat)
export(Hindex)
export(KeywordGrowth)
export(applyCitationMatching)
export(applyReferenceMatching)
export(assignEvolutionColors)
export(authorBio)
export(authorProdOverTime)
export(biblioAnalysis)
export(biblioNetwork)
export(biblioshiny)
export(bradford)
export(citations)
export(cocMatrix)
export(collabByRegionPlot)
export(completeMetadata)
export(conceptualStructure)
export(convert2df)
export(couplingMap)
export(dominance)
export(duplicatedMatching)
export(fieldByYear)
export(findAuthorWorks)
export(get_authors_summary)
export(histNetwork)
export(histPlot)
export(idByAuthor)
export(keywordAssoc)
export(lifeCycle)
export(loadBookDataset)
export(localCitations)
export(lotka)
export(mergeDbSources)
export(mergeKeywords)
export(metaTagExtraction)
export(missingData)
export(net2Pajek)
export(net2VOSviewer)
export(networkPlot)
export(networkStat)
export(normalizeCitationScore)
export(normalizeSimilarity)
export(normalize_citations)
export(plotThematicEvolution)
export(print_author_works_summary)
export(readFiles)
export(retrievalByAuthorID)
export(rpys)
export(sourceGrowth)
export(splitCommunities)
export(tableTag)
export(termExtraction)
export(thematicEvolution)
export(thematicMap)
export(threeFieldsPlot)
export(timeslice)
export(trim)
export(trim.leading)
export(trimES)
import(bibliometrixData)
import(ca)
import(contentanalysis)
import(dimensionsR)
import(forcats)
import(ggplot2)
import(ggrepel)
import(openalexR)
import(pubmedR)
import(readr)
import(readxl)
import(shiny)
import(shinycssloaders)
import(stats)
import(stringi)
import(tidytext)
import(visNetwork)
importFrom(Matrix,"%&%")
importFrom(Matrix,"diag<-")
importFrom(Matrix,Arith)
importFrom(Matrix,Cholesky)
importFrom(Matrix,Compare)
importFrom(Matrix,Diagonal)
importFrom(Matrix,Logic)
importFrom(Matrix,Math)
importFrom(Matrix,Math2)
importFrom(Matrix,Matrix)
importFrom(Matrix,MatrixClass)
importFrom(Matrix,Ops)
importFrom(Matrix,Summary)
importFrom(Matrix,abIseq)
importFrom(Matrix,abIseq1)
importFrom(Matrix,all.equal)
importFrom(Matrix,anyDuplicatedT)
importFrom(Matrix,as.array)
importFrom(Matrix,as.matrix)
importFrom(Matrix,band)
importFrom(Matrix,bandSparse)
importFrom(Matrix,bdiag)
importFrom(Matrix,cbind2)
importFrom(Matrix,chol)
importFrom(Matrix,chol2inv)
importFrom(Matrix,coerce)
importFrom(Matrix,colMeans)
importFrom(Matrix,colSums)
importFrom(Matrix,condest)
importFrom(Matrix,cov2cor)
importFrom(Matrix,crossprod)
importFrom(Matrix,det)
importFrom(Matrix,determinant)
importFrom(Matrix,diag)
importFrom(Matrix,diagN2U)
importFrom(Matrix,diagU2N)
importFrom(Matrix,diff)
importFrom(Matrix,drop)
importFrom(Matrix,drop0)
importFrom(Matrix,expand)
importFrom(Matrix,expm)
importFrom(Matrix,fac2sparse)
importFrom(Matrix,forceSymmetric)
importFrom(Matrix,format)
importFrom(Matrix,formatSpMatrix)
importFrom(Matrix,formatSparseM)
importFrom(Matrix,head)
importFrom(Matrix,image)
importFrom(Matrix,invPerm)
importFrom(Matrix,is.null.DN)
importFrom(Matrix,isDiagonal)
importFrom(Matrix,isLDL)
importFrom(Matrix,isSymmetric)
importFrom(Matrix,isTriangular)
importFrom(Matrix,kronecker)
importFrom(Matrix,lu)
importFrom(Matrix,mean)
importFrom(Matrix,nnzero)
importFrom(Matrix,norm)
importFrom(Matrix,onenormest)
importFrom(Matrix,pack)
importFrom(Matrix,print)
importFrom(Matrix,printSpMatrix)
importFrom(Matrix,printSpMatrix2)
importFrom(Matrix,qr)
importFrom(Matrix,qr.Q)
importFrom(Matrix,qr.R)
importFrom(Matrix,qr.coef)
importFrom(Matrix,qr.fitted)
importFrom(Matrix,qr.qty)
importFrom(Matrix,qr.qy)
importFrom(Matrix,qr.resid)
importFrom(Matrix,qrR)
importFrom(Matrix,rankMatrix)
importFrom(Matrix,rbind2)
importFrom(Matrix,rcond)
importFrom(Matrix,readHB)
importFrom(Matrix,readMM)
importFrom(Matrix,rep2abI)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,rsparsematrix)
importFrom(Matrix,show)
importFrom(Matrix,skewpart)
importFrom(Matrix,solve)
importFrom(Matrix,spMatrix)
importFrom(Matrix,sparse.model.matrix)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,sparseVector)
importFrom(Matrix,summary)
importFrom(Matrix,symmpart)
importFrom(Matrix,t)
importFrom(Matrix,tail)
importFrom(Matrix,tcrossprod)
importFrom(Matrix,tril)
importFrom(Matrix,triu)
importFrom(Matrix,uniqTsparse)
importFrom(Matrix,unpack)
importFrom(Matrix,update)
importFrom(Matrix,updown)
importFrom(Matrix,which)
importFrom(Matrix,writeMM)
importFrom(SnowballC,getStemLanguages)
importFrom(SnowballC,wordStem)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,anti_join)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,between)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,coalesce)
importFrom(dplyr,count)
importFrom(dplyr,cummean)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,group_split)
importFrom(dplyr,if_all)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,join_by)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pick)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,as.raster)
importFrom(grDevices,chull)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,heat.colors)
importFrom(grDevices,pdf)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,arpack_defaults)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_edgelist)
importFrom(igraph,as_long_data_frame)
importFrom(igraph,authority_score)
importFrom(igraph,betweenness)
importFrom(igraph,centr_betw)
importFrom(igraph,centr_clo)
importFrom(igraph,centr_degree)
importFrom(igraph,centr_eigen)
importFrom(igraph,closeness)
importFrom(igraph,cluster_edge_betweenness)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_infomap)
importFrom(igraph,cluster_leading_eigen)
importFrom(igraph,cluster_leiden)
importFrom(igraph,cluster_louvain)
importFrom(igraph,cluster_optimal)
importFrom(igraph,cluster_spinglass)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,count_multiple)
importFrom(igraph,decompose.graph)
importFrom(igraph,degree)
importFrom(igraph,degree_distribution)
importFrom(igraph,delete.edges)
importFrom(igraph,delete.vertices)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,diameter)
importFrom(igraph,edge_density)
importFrom(igraph,eigen_centrality)
importFrom(igraph,get.edgelist)
importFrom(igraph,graph.adjacency)
importFrom(igraph,graph_attr)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,graph_from_incidence_matrix)
importFrom(igraph,hub_score)
importFrom(igraph,induced_subgraph)
importFrom(igraph,layout.auto)
importFrom(igraph,layout.circle)
importFrom(igraph,layout.fruchterman.reingold)
importFrom(igraph,layout.kamada.kawai)
importFrom(igraph,layout.mds)
importFrom(igraph,layout.norm)
importFrom(igraph,layout.sphere)
importFrom(igraph,layout.star)
importFrom(igraph,layout_with_fr)
importFrom(igraph,mean_distance)
importFrom(igraph,membership)
importFrom(igraph,modularity)
importFrom(igraph,norm_coords)
importFrom(igraph,page.rank)
importFrom(igraph,page_rank)
importFrom(igraph,plot.igraph)
importFrom(igraph,simplify)
importFrom(igraph,transitivity)
importFrom(igraph,vcount)
importFrom(igraph,write.graph)
importFrom(methods,new)
importFrom(openxlsx,write.xlsx)
importFrom(plotly,add_annotations)
importFrom(plotly,add_lines)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,subplot)
importFrom(plotly,toRGB)
importFrom(purrr,map2_dfr)
importFrom(purrr,map_chr)
importFrom(purrr,map_df)
importFrom(purrr,map_dfr)
importFrom(rscopus,affiliation_retrieval)
importFrom(rscopus,author_df_orig)
importFrom(rscopus,author_search)
importFrom(rscopus,get_complete_author_info)
importFrom(stringdist,stringdistmatrix)
importFrom(stringr,fixed)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_locate)
importFrom(stringr,str_locate_all)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_squish)
importFrom(stringr,str_to_upper)
importFrom(stringr,str_trim)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,drop_na)
importFrom(tidyr,expand_grid)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
importFrom(tidyr,starts_with)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(utils,adist)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,write.csv)

source("utils.R", local=TRUE)
source("libraries.R", local=TRUE)
source("biblioShot.R", local=TRUE)
source("biblioAI.R", local=TRUE)

suppressMessages(libraries())

#### SERVER ####
server <- function(input, output,session){
  session$onSessionEnded(stopApp)
  
  ## suppress warnings
  options(warn = -1)
  
  if (inherits(try(pagedown::find_chrome(), silent=T), "try-error")) {
    Chrome_url <- NULL
  }else{
    Chrome_url <- pagedown::find_chrome()
  }

  #  Sys.setenv (CHROMOTE_CHROME = Chrome_url)
  
  ## chrome configuration for shinyapps server

  if (identical(Sys.getenv("R_CONFIG_ACTIVE"), "shinyapps")) {
    chromote::set_default_chromote_object(
      chromote::Chromote$new(chromote::Chrome$new(
        args = c("--disable-gpu",
                 "--no-sandbox",
                 "--disable-dev-shm-usage", # required bc the target easily crashes
                 c("--force-color-profile", "srgb"))
      ))
    )
  }
  ## end configuration
  
  ## Check if Chrome browser is installed on the computer
  if(is.null(Chrome_url)){
    showModal(modalDialog(
      title = strong("Warning message!"),
      HTML("Chrome or a Chromium-based browser is not installed on your computer.<br>
If you do not have either of these browsers installed, Biblioshiny will be unable to export graphs.<br>
To ensure the functionality of Biblioshiny,
           please download Chrome by <a href='https://www.google.com/chrome/' target='_blank' > <b>clicking here</b></a>."),
      footer = modalButton("Dismiss"),
      easyClose = TRUE
    ))
  } else {
    Sys.setenv (CHROMOTE_CHROME = Chrome_url)
  }
  
  ## file upload max size
  maxUploadSize <- 200 # default value
  maxUploadSize <- getShinyOption("maxUploadSize", maxUploadSize)
  options(shiny.maxRequestSize=maxUploadSize*1024^2)
  
  ## initial values
  data("logo",package="bibliometrix",envir=environment())
  values = reactiveValues()
  values$Chrome_url <- Chrome_url
  values$sidebar <- sidebarMenu()
  values$rest_sidebar <- FALSE
  values$list_file <- data.frame(sheet=NULL,file=NULL,n=NULL) 
  values$wb <-  openxlsx::createWorkbook()
  values$dfLabel <- dfLabel()
  values$myChoices <- "Empty Report"
  values$logo <- logo
  values$logoGrid <- grid::rasterGrob(logo,interpolate = TRUE)
  values$out <- NULL
  values$loadMenu <- NA
  
  ### column to export in TALL
  if (suppressPackageStartupMessages(!require("tall", quietly = TRUE))) {
      values$TALLmissing <- TRUE
    } else {
      values$TALLmissing <- FALSE  
    }
  values$corpusCol <- c("Title" = "TI","Abstract"="AB", "Author's Keywords"="DE")
  values$metadataCol <- c("Publication Year" = "PY","Document Type"="DT", "DOI"="DI", "Open Access"="OA", "Language"="LA", "First Author"= "AU1")
  
  
  ### setting values
  values$dpi <- 300
  values$h <- 7
  #values$w <- 14 
  values$path <- paste(getwd(),"/", sep="")
  ###
  
  values$results <- list("NA")
  values$log <- "working..."
  values$load="FALSE"
  values$field = values$cocngrams = "NA"
  values$citField=values$colField=values$citSep="NA"
  values$NetWords=values$NetRefs=values$ColNetRefs=matrix(NA,1,1)
  values$Title="Network"
  values$Histfield="NA"
  values$histlog="working..."
  values$kk=0
  values$M=data.frame(PY=0)
  values$histsearch="NA"
  values$citShortlabel="NA"
  values$S=list("NA")
  values$GR="NA"
  values$dsToken <- "Wrong account or password"
  values$dsSample <- 0
  values$dsQuery <- ""
  values$pmQuery <- " "
  values$pmSample <- 0
  values$ApiOk <- 0
  values$checkControlBar <-FALSE
  
  ## gemini api
  home <- homeFolder()
  path_gemini_key <- paste0(home,"/.biblio_gemini_key.txt", collapse="")
  # check if sub directory exists
  values$geminiAPI <- load_api_key(path_gemini_key)
  values$collection_description <- NULL
  values$gemini_additional <- NULL
  
  ## NOTIFICATION ITEM ----
  
  output$notificationMenu <- renderMenu({
    notifTot <- notifications()
    values$nots <- apply(notifTot, 1, function(row) {
      
      ## extract href from messages
      if (is.na(row[["href"]])){href <- NULL
      }else{
        href <- paste("javascript:void(window.open('",row[["href"]],"', '_blank'))", sep="")
      }
      
      ## add bold to new messages and split the long ones in two rows
      if (row[["status"]]=="danger"){  ### new messages
        textRows <- paste("tags$strong('",row[["nots"]],"')", sep="")
        textRows <- strsplit(substr(textRows,1,85), "(?<=.{48})", perl = TRUE)[[1]]
        if (length(textRows)>1){
          textRows <- paste("tags$div(",textRows[1],"',tags$br(),'",textRows[2],")", sep="")
        }else{
          textRows <- paste("tags$div(",textRows,")", sep="")
        }
      }else{ ## old messages
        textRows <- strsplit(substr(row[["nots"]],1,70), "(?<=.{35})", perl = TRUE)[[1]]
        if (length(textRows)>1){
          textRows <- paste("tags$div('",textRows[1],"',tags$br(),'",textRows[2],"')", sep="")
        }else{
          textRows <- paste("tags$div('",textRows,"')", sep="")
        }
      }
      
      notificationItem(
        text = eval(parse(text=textRows)),
        icon = if (row[["status"]]=="danger") {fa_i(name ="envelope")}else{fa_i(name ="envelope-open")},
        status = row[["status"]],
        href = href
      )
    })
    
    if ("danger" %in% notifTot[["status"]]){
      badge = "danger"
      icon_name ="envelope"
    } else {
      badge = NULL
      icon_name ="envelope-open"
    }
    
    dropdownMenu(type = "notifications", 
                 .list = values$nots, 
                 headerText ="",
                 badgeStatus = NULL, 
                 icon = fa_i(name = icon_name)
    )
  })
  
  ## SIDEBAR MENU ----
  ### Apply Data----
  
  output$rest_of_sidebar <- renderMenu({
    if (isTRUE(values$rest_sidebar)){
      sidebarMenu(.list=values$menu)
    } else {
      sidebarMenu()
    }
  })
  
  observeEvent(input$applyLoad, {
    updateTabItems(session, "sidebarmenu", "loadData")
  })
  
  observeEvent(input$apiApply, {
    updateTabItems(session, "sidebarmenu", "gathData")
  })
  
  observeEvent(values$missTags, {
    switch(values$loadMenu,
           "load"={
             updateTabItems(session, "sidebarmenu", "loadData")
           },
           "merge"={
             updateTabItems(session, "sidebarmenu", "mergeData")
           })
    values$loadMenu <- NA
  })

  observeEvent(input$applyMerge, {
    updateTabItems(session, "sidebarmenu", "mergeData")
  })
  
  ## observe Gemini copy2clipboard button
  observeEvent(input$copy_btn, {
    content <- geminiSave(values, input$sidebarmenu)
    copy_to_clipboard(content)
  })
  
  ## observe Gemini Save button
  output$save_btn <- downloadHandler(
    filename = function() {
      paste0("BiblioAI_",input$sidebarmenu,".txt")
    },
    content <- function(file) {
      txtOutput <- geminiSave(values, input$sidebarmenu)
      writeLines(txtOutput, con=file)
    },
    contentType = "txt"
  )
  

  
  ## observe gemini generate button
  
  observeEvent(input$gemini_btn, {
    values$gemini_additional <- input$gemini_additional ## additional info to Gemini prompt
    values <- geminiWaitingMessage(values, input$sidebarmenu)
    values <- geminiGenerate(values, input$sidebarmenu, values$gemini_additional,values$gemini_model_parameters, input)
  })
  
  observeEvent(input$applyLoad,
               {
                 output$collection_descriptionUI <- renderUI({
                   textAreaInput(
                     inputId = "collection_description_merge",
                     label = "Brief description about your collection",
                     placeholder = "Please provide a brief description of your bibliographic collection (e.g., type of content, domain, research hypotheses, timespan) to improve prompts for the BIBLIO AI Assistant.\n\nExample: The corpus consists of 150 academic articles from biomedical journals published between 2015 and 2020...",
                     value = NULL,
                     rows = 3,
                     width = "100%"
                   )
                 })
               })
  
  observeEvent(values$M,{
    updateTextAreaInput(
      session = getDefaultReactiveDomain(),
      inputId = "collection_description",
      label = "Brief description about your collection",
      value = values$collection_description
    )
    updateTextAreaInput(
      session = getDefaultReactiveDomain(),
      inputId = "collection_description_merge",
      label = "Brief description about your collection",
      value = values$collection_description
    )
  })
  
  observeEvent(eventExpr = {
    input$collection_description},
    handlerExpr = {
      if (input$collection_description!="" & nchar(input$collection_description)>1){
        values$collection_description <- input$collection_description
      }
    },ignoreNULL = TRUE)
  
  observeEvent(eventExpr = {
    input$collection_description_merge},
    handlerExpr = {
      if (input$collection_description_merge!="" & nchar(input$collection_description_merge)>1){
        values$collection_description <- input$collection_description_merge
      }
    },ignoreNULL = TRUE)
  
  
  ## Load Menu ----
  
  DATAloading<- eventReactive(input$applyLoad,{
    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, it will be a data frame with 'name',
    # 'size', 'type', and 'datapath' columns. The 'datapath'
    # column will contain the local filenames where the data can
    # be found.
    if (input$load=="demo"){
      data(management, package="bibliometrixData")
      values = initial(values)
      row.names(management) <- management$SR
      management <- management %>% mergeKeywords(force = T)
      values$M <- management
      values$Morig = management
      values$Histfield = "NA"
      values$results = list("NA")
      values$rest_sidebar <- TRUE
      values$missingdf <- df <- missingData(values$M)$mandatoryTags
      values$missTags <- NULL
      values$menu <- menuList(values)
      values$collection_description <- 'A collection of scientific articles about the use of bibliometric approaches in business and management disciplines. Period: 1985–2020. This collection was identified by retrieving all documents indexed under the subject categories “Management” and "Business" that contain at least one of the following terms in their topic fields: “science map”, "bibliometric*".'
        #"Dataset 'Management':\nA collection of scientific articles about the use of bibliometric approaches in business and management disciplines. Period: 1985–2020."
      
      showModal(missingModal(session))
      return()
    }
    inFile <- input$file1
    
    if (!is.null(inFile) & input$load=="import") {
      ext <- getFileNameExtension(inFile$datapath)
      switch(
        input$dbsource,
        isi = {
          switch(ext,
                 ###  WoS ZIP Files
                 zip = {
                   D <-  utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(D,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(D))
                                  M <- authorNameFormat(M, input$authorName)
                                })
                 },
                 ### WoS Txt/Bib Files
                 {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(inFile$datapath))
                                  M <- authorNameFormat(M, input$authorName)
                                })
                 })
        },
        scopus = {
          switch(ext,
                 ###  Scopus ZIP Files
                 zip = {
                   D <- utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(D,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(D))
                                  M <- authorNameFormat(M, input$authorName)
                                  if (formatDB(D)=="csv" & input$authorName=="AF") M <- AuthorNameMerge(M)
                                })
                 },
                 ### Scopus CSV/Bib Files
                 csv = {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = "csv")
                                  M <- authorNameFormat(M, input$authorName)
                                  if (input$authorName=="AF") M <- AuthorNameMerge(M)
                                })
                 },
                 bib = {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = "bibtex")
                                  M <- authorNameFormat(M, input$authorName)
                                })
                 })
        },
        openalex={
          withProgress(message = 'Conversion in progress',
                         value = 0, {
                           M <- convert2df(inFile$datapath,
                                           dbsource = input$dbsource,
                                           format = "csv")
                         })
        },
        openalex_api = {
          M <- convert2df(inFile$datapath,
            dbsource = input$dbsource,
            format = "api")
        },
        lens = {
          switch(ext,
                 ###  Lens.org ZIP Files
                 zip = {
                   D <-  utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(D,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(D))
                                })
                 },
                 ### Lens.org CSV Files
                 {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(inFile$datapath))
                                })
                 })
        },
        cochrane = {
          switch(ext,
                 ###  Cochrane ZIP Files
                 zip = {
                   D <- utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(D,
                                                  dbsource = input$dbsource,
                                                  format = formatDB(D))
                                })
                 },
                 ### Cochrane txt files
                 {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = "plaintext")
                                })
                 })
        },
        pubmed = {
          switch(ext,
                 ###  Pubmed ZIP Files
                 zip = {
                   D <- utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(D,
                                                  dbsource = input$dbsource,
                                                  format = "pubmed")
                                })
                 },
                 ### Pubmed txt Files
                 txt = {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <- convert2df(inFile$datapath,
                                                  dbsource = input$dbsource,
                                                  format = "pubmed")
                                })
                 })
        },
        dimensions = {
          switch(ext,
                 ###  Dimensions ZIP Files
                 zip = {
                   D = utils::unzip(inFile$datapath)
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <-
                                    convert2df(D,
                                               dbsource = input$dbsource,
                                               format = formatDB(D))
                                })
                 },
                 ### Dimensions Xlsx/csv Files
                 xlsx = {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <-
                                    convert2df(
                                      inFile$datapath,
                                      dbsource = "dimensions",
                                      format = "excel"
                                    )
                                })
                 },
                 csv = {
                   withProgress(message = 'Conversion in progress',
                                value = 0, {
                                  M <-
                                    convert2df(
                                      inFile$datapath,
                                      dbsource = "dimensions",
                                      format = "csv"
                                    )
                                })
                 })
          
        }
      )
    } else if (!is.null(inFile) & input$load=="load") {
      ext <- tolower(getFileNameExtension(inFile$datapath))
      switch(ext,
             ### excel format
             xlsx={
               M <- readxl::read_excel(inFile$datapath, col_types = "text") %>% as.data.frame()
               M$PY <- as.numeric(M$PY)
               M$TC <- as.numeric(M$TC)
               class(M) <- c("bibliometrixDB", "data.frame")
               ### M row names
               ### identify duplicated SRs 
               SR <- M$SR
               tab <- table(SR)
               tab2 <- table(tab)
               ind <- as.numeric(names(tab2))
               ind <- ind[which(ind>1)]
               if (length(ind)>0){
                 for (i in ind){
                   indice=names(which(tab==i))
                   for (j in indice){
                     indice2 <- which(SR==j)
                     SR[indice2] <- paste(SR[indice2],as.character(1:length(indice2)),sep=" ")
                   }
                 }
               }
               M$SR <- SR
               row.names(M) <- SR
             },
             ### RData format
             rdata={
               M <- smart_load(inFile$datapath) 
             },
             rda={
               M <- smart_load(inFile$datapath) 
             },
             rds={
               M <- readRDS(inFile$datapath) 
             })
    } else if (is.null(inFile)) {return(NULL)}
    
    values = initial(values)
    ## remove not useful columns
    ind <- which(substr(names(M),1,2)=="X.")
    if (length(ind)>0) M <- M[,-ind]
    ##
    M <- M %>% mergeKeywords(force = F)
    values$M <- M
    values$Morig = M
    values$Histfield = "NA"
    values$results = list("NA")
    if (ncol(values$M)>1){values$rest_sidebar <- TRUE}
    if (ncol(values$M)>1){
      values$loadMenu <- "load"
      showModal(missingModal(session))}
  })
  
  output$contents <- DT::renderDT({
    DATAloading()   
    MData = as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))
    MData$DOI <-
      paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
    nome = c("DOI", names(MData)[-length(names(MData))])
    MData = MData[nome]
    DTformat(MData, nrow=3, filename="Table", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, size='70%', filter="top",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, selection=FALSE, scrollX=TRUE)
  })
  
  
  ## Merge Menu ----
  DATAmerging<- eventReactive(input$applyMerge,{
    
    inFile <- input$fileMerge
    
    if (!is.null(inFile)){
      #save(inFile,file="prova.rdata")
      M <- merge_files(inFile)
    } else if (is.null(inFile)) {return(NULL)}
    
    values = initial(values)
    ## remove not useful columns
    ind <- which(substr(names(M),1,2)=="X.")
    if (length(ind)>0) M <- M[,-ind]
    ##
    
    values$M <- M
    values$Morig = M
    values$nMerge <- attr(M,"nMerge")
    values$Histfield = "NA"
    values$results = list("NA")
    if (ncol(values$M)>1){values$rest_sidebar <- TRUE}
    if (ncol(values$M)>1){
      values$loadMenu <- "merge"
      showModal(missingModal(session))}
  })
  
  output$contentsMerge <- DT::renderDT({
    DATAmerging()   
    MData = as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))
    MData$DOI <-
      paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
    nome = c("DOI", names(MData)[-length(names(MData))])
    MData = MData[nome]
    DTformat(MData, nrow=3, filename="Table", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, size='70%', filter="top",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, selection=FALSE, scrollX=TRUE)
  })
  
  observeEvent(input$applyMerge,
               {
                 output$collection_description_mergeUI <- renderUI({
                   textAreaInput(
                     inputId = "collection_description",
                     label = "Brief description about your collection",
                     placeholder = "Please provide a brief description of your bibliographic collection (e.g., source, type of content, domain) to improve prompts for the BIBLIO AI Assistant.\n\nExample: The corpus consists of 150 academic articles from biomedical journals published between 2015 and 2020...",
                     value = NULL,
                     rows = 3,
                     width = "100%"
                   )
                 })
               })
  
  ### Missing Data in Metadata ----
  output$missingDataTable <- DT::renderDT({
    values$missingdf <- df <- missingData(values$M)$mandatoryTags
    values$missTags <- df$tag[df$missing_pct>50]
    values$menu <- menuList(values)
    
    names(df) <- c("Metadata", "Description", "Missing Counts", "Missing %", "Status")
    values$missingDataTable <- DT::datatable(df,escape = FALSE,rownames = FALSE, #extensions = c("Buttons"),
                  class = 'cell-border stripe',
                  selection = 'none',
                  options = list(
                    pageLength = nrow(df),
                    info = FALSE,
                    autoWidth = FALSE, scrollX = TRUE, 
                    dom = 'rti',
                    ordering=F,
                    columnDefs = list(
                      list(
                        targets = ncol(df)-1,
                        createdCell = JS(
                          "function(td, cellData, rowData, row, col) {",
                          "  if (cellData === 'Completely missing') {",
                          "    $(td).css('background-color', '#b22222');",
                          "  } else if (cellData === 'Critical') {",
                          "    $(td).css('background-color', '#f08080');",
                          "  } else if (cellData === 'Poor') {",
                          "    $(td).css('background-color', 'lightgrey');",
                          "  } else if (cellData === 'Acceptable') {",
                          "    $(td).css('background-color', '#f0e68c');",
                          "  } else if (cellData === 'Good') {",
                          "    $(td).css('background-color', '#90ee90');",
                          "  } else if (cellData === 'Excellent') {",
                          "    $(td).css('background-color', '#32cd32');",
                          "  }",
                          "}")
                      )
                    )
                  )
    ) %>% 
      formatRound("Missing %", digits=2) %>% 
      formatStyle(
        "Status",
        textAlign = 'center'
      )
    values$missingDataTable
  })
  
  observeEvent(input$missingMessage,{
    tag <- values$missingdf$description[values$missingdf$status %in% c("Critical", "Completely missing")]
    if (length(values$out)>0){
      text <- paste("The following analyses could not be performed: <br><br>",paste("- ","<em>",values$out,"</em>","<br>", collapse=""),
                    "<br>These menu will be hidden in the Biblioshiny dashboard!",collapse="")
      type <- "warning"
    }else{
      text <- "Your metadata have no critical issues"
      type <- "success"
    }
   
    show_alert(
      title = NULL,
      #text = HTML(paste("Analyses that require the following information:<br>",paste("- ",tag,"<br>", collapse=""),"cannot be performed!",collapse="")),
      text =tagList(
        div(
          h4(HTML(text)),
          style="text-align:left")
      ),
      type = type,
      size = "s", 
      closeOnEsc = TRUE,
      closeOnClickOutside = TRUE,
      html = TRUE,
      showConfirmButton = TRUE,
      showCancelButton = FALSE,
      btn_labels = "OK",
      btn_colors = "#1d8fe1",
      timer = NULL,
      imageUrl = "",
      animation = TRUE
    )
  })
  
  output$missingTitle <- renderUI({
    ndocs <- nrow(values$M)
    if ("DB_Original" %in% names(values$M)){
      DB <- paste0(length(unique(values$M$DB_Original))," DBs")
      txt1 <- paste0("Completeness of metadata -- ", strong(ndocs)," docs merged from ", DB)
      txt2 <- paste0("Original size ",strong(values$nMerge), " docs -- Deleted ", strong(values$nMerge-ndocs), " duplicated docs")
    } else {
      DB <- firstup(values$M$DB[1])
      txt1 <- paste0("Completeness of metadata -- ", strong(ndocs)," docs from ", strong(DB))
      txt2 <- ""
    }
    
    
    tagList(
      div(
        h3(HTML(txt1)),
        br(),
        h4(HTML(txt2)),
        style="text-align:center")
    )
  })
  
  missingModal <- function(session) {
    ns <- session$ns
    modalDialog(
      uiOutput("missingTitle"),
      DT::DTOutput(ns("missingDataTable")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(label="Advice", inputId = "missingMessage",
                     icon = icon("exclamation-sign", lib = "glyphicon")),
        actionButton(label="Report", inputId = "missingReport",
                     icon = icon("plus", lib = "glyphicon")),
        actionButton(label="Save", inputId = "missingDataTable",
                     icon = icon("camera", lib = "glyphicon")),
        modalButton(label="Close")),
    )
  }
  
  observeEvent(input$missingDataTable,{
    filename = paste("missingDataTable-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$missingDataTable, filename=filename, type="plotly")
  })
  
  observeEvent(input$missingReport,{
    if (!is.null(values$missingDataTable)){
      sheetname <- "MissingData"
      ind <- which(regexpr(sheetname,values$wb$sheet_names)>-1)
      if (length(ind)>0){
        sheetname <- paste(sheetname,length(ind)+1,sep="")
      } 
      addWorksheet(wb=values$wb, sheetName=sheetname, gridLines = FALSE)
      #values$fileTFP <- screenSh(selector = "#ThreeFieldsPlot") ## screenshot
      values$fileMissingData <- screenSh(values$missingDataTable, zoom = 2, type="plotly")
      values$list_file <- rbind(values$list_file, c(sheetname,values$fileMissingData,1))
      popUp(title="Missing Data Table", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ## export functions ----
  output$collection.save <- downloadHandler(
    filename = function() {
      paste("Bibliometrix-Export-File-", Sys.Date(), ".",input$save_file, sep="")
    },
    content <- function(file) {
      tr <- FALSE
      if ("CR" %in% names(values$M)) tr <- (sum(nchar(values$M$CR)>32767, na.rm=TRUE))>0
      
      if (tr & input$save_file=="xlsx"){
        show_alert(
          text = tags$span(
            tags$h4("Some documents have too long a list of references that cannot be saved in excel (>32767 characters).",
                    style = "color: firebrick;"),
            tags$br(),
            tags$h4("Data in the column CR could be truncated.",
                    style = "color: firebrick;")
          ),
          #text = "Some documents have too long a list of references that cannot be saved in excel (>32767 characters).\nData in the column CR could be truncated",
          title = "Please save the collection using the 'RData' format",
          type = "warning",
          width =  "50%", ##NEW ----
          closeOnEsc = TRUE,
          closeOnClickOutside = TRUE,
          html = FALSE,
          showConfirmButton = TRUE,
          showCancelButton = FALSE,
          btn_labels = "OK",
          btn_colors = "#913333",
          timer = 0,
          imageUrl = "",
          animation = TRUE
        )
        suppressWarnings(openxlsx::write.xlsx(values$M, file=file))
      } else {
        switch(input$save_file,
               xlsx={suppressWarnings(openxlsx::write.xlsx(values$M, file=file))},
               RData={
                 M=values$M
                 save(M, file=file)
               })
      }
    },
    contentType = input$save_file
  )
  
  output$collection.saveMerge <- downloadHandler(
    filename = function() {
      paste("Bibliometrix-Export-File-", Sys.Date(), ".",input$save_fileMerge, sep="")
    },
    content <- function(file) {
      tr <- FALSE
      if ("CR" %in% names(values$M)) tr <- (sum(nchar(values$M$CR)>32767, na.rm=TRUE))>0
      
      if (tr & input$save_file=="xlsx"){
        show_alert(
          text = tags$span(
            tags$h4("Some documents have too long a list of references that cannot be saved in excel (>32767 characters).",
                    style = "color: firebrick;"),
            tags$br(),
            tags$h4("Data in the column CR could be truncated.",
                    style = "color: firebrick;")
          ),
          #text = "Some documents have too long a list of references that cannot be saved in excel (>32767 characters).\nData in the column CR could be truncated",
          title = "Please save the collection using the 'RData' format",
          type = "warning",
          width =  "50%", ##NEW ----
          closeOnEsc = TRUE,
          closeOnClickOutside = TRUE,
          html = FALSE,
          showConfirmButton = TRUE,
          showCancelButton = FALSE,
          btn_labels = "OK",
          btn_colors = "#913333",
          timer = 0,
          imageUrl = "",
          animation = TRUE
        )
        suppressWarnings(openxlsx::write.xlsx(values$M, file=file))
      } else {
        switch(input$save_fileMerge,
               xlsx={
                 suppressWarnings(openxlsx::write.xlsx(values$M, file=file))
                 },
               RData={
                 M=values$M
                 save(M, file=file)
               })
      }
    },
    contentType = input$save_fileMerge
  )
  
  output$collection.save_api <- downloadHandler(
    filename = function() {
      
      paste("Bibliometrix-Export-File-", Sys.Date(), ".",input$save_file_api, sep="")
    },
    content <- function(file) {
      switch(input$save_file_api,
             xlsx={suppressWarnings(openxlsx::write.xlsx(values$M, file=file))},
             RData={
               M=values$M
               save(M, file=file)
             })
    },
    contentType = input$save_file_api
  )
  
  output$textLog2 <- renderUI({  
    k=dim(values$M)[1]
    if (k==1){k=0}
    log=paste("Number of Documents ",k)
    textInput("textLog", "Conversion results", 
              value=log)
  })
  
  dsModal <- function(failed = FALSE) {
    modalDialog(
      title = "Dimensions API",
      size = "l",
      h4(em(
        strong("1) Get a token using your Dimensions credentials")
      )),
      textInput(
        "dsAccount",
        "Account",
        "",
        width = NULL,
        placeholder = NULL
      ),
      passwordInput("dsPassword",
                    "Password",
                    "",
                    width = NULL,
                    placeholder = NULL
      ),
      actionButton("dsToken", "Get a token "),
      h5(tags$b("Token")),
      verbatimTextOutput("tokenLog", placeholder = FALSE),
      tags$hr(),
      h4(em(strong("2) Create a query"))),
      textInput(
        "dsWords",
        "Words",
        "",
        width = NULL,
        placeholder = NULL
      ),
      selectInput(
        "dsFullsearch",
        label = "search field",
        choices = c("Title and Abstract only" = FALSE,
                    "Full text" = TRUE),
        selected = FALSE
      ),
      textInput(
        "dsCategories",
        "Science Categories",
        "",
        width = NULL,
        placeholder = NULL
      ),
      numericInput("dsStartYear", "Start Year", value = 1990),
      numericInput("dsEndYear", "End Year", value = as.numeric(substr(Sys.time(), 1, 4))),
      actionButton("dsQuery", "Create the query "),
      h5(tags$b("Your query")),
      verbatimTextOutput("queryLog", placeholder = FALSE),
      h5(tags$b("Documents returned using your query")),
      verbatimTextOutput("sampleLog", placeholder = FALSE),
      uiOutput("sliderLimit"),
      footer = tagList(
        modalButton("Cancel"),
        actionButton("dsok", "OK")
      )
    )
  }
  
  ### Show Dimensions modal when button is clicked.
  observeEvent(input$dsShow, {
    showModal(dsModal())
  })
  
  observeEvent(input$dsok, {
    removeModal()
    values$M <- data.frame(Message="Waiting for data")
  })
  
  output$tokenLog <- renderText({ 
    input$dsToken 
    isolate({
      capture.output(Token <- dsAuth(username = input$dsAccount, password = input$dsPassword))
      if (Token==1){
        values$dsToken <- "Wrong account or password"
      }else{
        values$dsToken <- Token
      }
      values$dsToken
    })
  })
  
  DSQUERYload<- eventReactive(input$dsQuery,{
    values$dsQuery <- dsQueryBuild(item = "publications", 
                                   words = input$dsWords, 
                                   full.search = input$dsFullsearch,
                                   type = "article", 
                                   categories = input$dsCategories, 
                                   start_year = input$dsStartYear, end_year = input$dsEndYear)
    dsSample <- 0
    capture.output(dsSample <- dsApiRequest(token = values$dsToken, query = values$dsQuery, limit = 0))
    if (class(dsSample)=="numeric"){
      values$dsSample <- 0
    }else{values$dsSample <- dsSample$total_count}
  })
  
  output$queryLog <- renderText({ 
    DSQUERYload()
    values$dsQuery
  })
  
  output$queryLog2 <- renderText({ 
    DSQUERYload()
    values$dsQuery
  })
  
  output$sampleLog <- renderText({ 
    DSQUERYload()
    mes <- paste("Dimensions returns ",values$dsSample, " documents", collapse="",sep="")
    mes
  }) 
  
  output$sampleLog2 <- renderText({ 
    if (nrow(values$M)<2) {n <- 0}else{n <- nrow(values$M)}
    mes <- paste("Dimensions API returns ",n, " documents", collapse="",sep="")
    values$ApiOk <- 0
    return(mes)
  }) 
  
  output$sliderLimit <- renderUI({
    sliderInput("sliderLimit", "Total document to download", min = 1,
                max = values$dsSample, value = values$dsSample, step = 1)
  })
  
  ### API MENU: PubMed ----
  ### PubMed modal 
  pmModal <- function(failed = FALSE) {
    modalDialog(
      title = "PubMed API",
      size = "l",
      h4(em(strong(
        "1) Generate a valid query"
      ))),
      textInput(
        "pmQueryText",
        "Search terms",
        " ",
        width = NULL,
        placeholder = NULL
      ),
      numericInput("pmStartYear", "Start Year", value = 1990),
      numericInput("pmEndYear", "End Year", value = as.numeric(substr(Sys.time(
      ), 1, 4))),
      actionButton("pmQuery", "Try the query "),
      h5(tags$b("Query Translation")),
      verbatimTextOutput("pmQueryLog", placeholder = FALSE),
      h5(tags$b("Documents returned using your query")),
      verbatimTextOutput("pmSampleLog", placeholder = FALSE),
      tags$hr(),
      h4(em(
        strong("2) Choose how many documents to download")
      )),
      uiOutput("pmSliderLimit"),
      footer = tagList(
        modalButton("Cancel"),
        actionButton("pmok", "OK")
      )
    )
  }
  
  # Show modal when button is clicked.
  observeEvent(input$pmShow, {
    showModal(pmModal())
  })
  
  observeEvent(input$pmok, {
    removeModal()
  })
  
  pmQUERYLOAD <- eventReactive(input$pmQuery,{
    query = paste(input$pmQueryText,"[Title/Abstract] AND english[LA] AND Journal Article[PT] AND "
                  ,input$pmStartYear,":",input$pmEndYear,"[DP]", sep="")
    res <- pmQueryTotalCount(query = query, api_key = NULL)
    if (class(res)=="list"){
      values$pmSample <- res$total_count
      values$pmQuery <- res$query_translation}
    values$pmQuery <- res$query_translation
    
  })
  output$pmQueryLog <- renderText({ 
    pmQUERYLOAD()
    values$pmQuery
  })
  
  output$pmQueryLog2 <- renderText({ 
    pmQUERYLOAD()
    values$pmQuery
  })
  
  output$pmSampleLog <- renderText({ 
    pmQUERYLOAD()
    mes <- paste("PubMed returns ",values$pmSample, " documents", collapse="",sep="")
    mes
    
  }) 
  output$pmSampleLog2 <- renderText({ 
    if (nrow(values$M)<2) {n <- 0}else{n <- nrow(values$M)}
    
    mes <- paste("PubMed API returns ",n, " documents", collapse="",sep="")
    values$ApiOk <- 0
    return(mes)
  }) 
  
  output$pmSliderLimit <- renderUI({
    sliderInput("pmSliderLimit", "Total document to download", min = 1,
                max = values$pmSample, value = values$pmSample, step = 1)
  })
  
  ### API MENU: Content Download ----
  APIDOWNLOAD <- eventReactive(input$apiApply,{
    values = initial(values)
    values$M <- data.frame(Message="Waiting for data")
    switch(input$dbapi,
           ds={
             if (input$dsWords!="") {
               D <-
                 dsApiRequest(
                   token = values$dsToken,
                   query = values$dsQuery,
                   limit = input$sliderLimit
                 )
               M <- convert2df(D, "dimensions", "api")
               values$ApiOk <- 1
               values$M <- M
               values$Morig = M
               if (ncol(values$M)>1){values$rest_sidebar <- TRUE}
               if (ncol(values$M)>1){showModal(missingModal(session))}
               values$Histfield = "NA"
               values$results = list("NA")
               contentTable(values)
             }
           },
           pubmed={
             if (input$pmQueryText !=" ") {
               D <-
                 pmApiRequest(
                   query = values$pmQuery,
                   limit = input$pmSliderLimit,
                   api_key = NULL
                 )
               M <- convert2df(D, "pubmed", "api")
               values$ApiOk <- 1
               values$M <- M
               values$Morig = M
               if (ncol(values$M)>1){values$rest_sidebar <- TRUE}
               if (ncol(values$M)>1){showModal(missingModal(session))}
               values$Histfield = "NA"
               values$results = list("NA")
             }
           })
  })
  
  output$apiContents <- DT::renderDT({
    APIDOWNLOAD()
    contentTable(values)
  })
  
  ### function returns a formatted data.frame ----
  contentTable <- function(values){
    MData = as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))
    MData$DOI <-
      paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
    nome = c("DOI", names(MData)[-length(names(MData))])
    MData = MData[nome]
    DTformat(MData, nrow=3, filename="Table", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, size='70%', filter="top",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, selection=FALSE,scrollX=TRUE)
  }
  
  output$textDim <-  renderUI({
    str1=paste("Documents    ",dim(values$M)[1]," of ",dim(values$Morig)[1])
    str2=paste("Sources      ",length(unique(values$M$SO))," of ", length(unique(values$Morig$SO)))
    str3=paste("Authors      ",length(unique(unlist(strsplit(values$M$AU,";"))))," of ", length(unique(unlist(strsplit(values$Morig$AU,";")))))
    HTML(paste("<pre class='tab'>",str1, str2, str3, sep = '<br/>'))
  })
  
  output$selectType <- renderUI({
    artType=sort(unique(values$Morig$DT))
    selectInput("selectType", "Document Type", 
                choices = artType,
                selected = artType,
                multiple =TRUE )
  })
  
  output$selectLA <- renderUI({
    if ("LA" %in% names(values$Morig)){
      LA <- sort(unique(values$Morig$LA))} else {
        values$Morig$LA <- "N.A."
        LA <- "N.A."
      }
    selectInput("selectLA", "Language", 
                choices = LA,
                selected = LA,
                multiple =TRUE )
  })
  
  output$sliderPY <- renderUI({
    sliderInput("sliderPY", "Publication Year", min = min(values$Morig$PY,na.rm=T),sep="",
                max = max(values$Morig$PY,na.rm=T), value = c(min(values$Morig$PY,na.rm=T),max(values$Morig$PY,na.rm=T)))
  })
  
  output$selectSource <- renderUI({
    SO=sort(unique(values$Morig$SO))
    selectInput("selectSource", "Source", 
                choices = SO,
                selected = SO,
                multiple = TRUE)
  })
  
  output$sliderTCpY <- renderUI({
    Y <- as.numeric(substr(Sys.time(),1,4))
    values$Morig <- values$Morig %>% 
      mutate(Age = Y - PY+1,
             TCpY = round(TC/Age,2)) %>% 
      group_by(PY) %>% 
      mutate(NTC = TC/mean(TC, na.rm=T)) %>% 
      as.data.frame()
    sliderInput("sliderTCpY", "Average Citations per Year", min = floor(min(values$Morig$TCpY, na.rm=T)),
                max = ceiling(max(values$Morig$TCpY,na.rm=T)), step=0.1,
                value = c(floor(min(values$Morig$TCpY, na.rm=T)),ceiling(max(values$Morig$TCpY,na.rm=T))))
  })
  
  ## Update filtered data ----
  observeEvent(input$applyFilter, {
    updateTabItems(session, "sidebarmenu", "filters")
  })
  
  DTfiltered <- eventReactive(input$applyFilter,{
    M <- values$Morig
    B <- bradford(M)$table
    M <- M %>%
      dplyr::filter(PY >= input$sliderPY[1], PY <= input$sliderPY[2]) %>%
      dplyr::filter(TCpY >= input$sliderTCpY[1], TCpY <= input$sliderTCpY[2]) %>%
      dplyr::filter(DT %in% input$selectType) %>%
      dplyr::filter(LA %in% input$selectLA)
    
    switch(input$bradfordSources,
           "core"={
             so <- B$SO[B$Zone %in% "Zone 1"]
           },
           "zone2"={
             so <- B$SO[B$Zone %in% c("Zone 1", "Zone 2")]
           },
           "all"={so <- B$SO})
    M <- M %>%
      filter(SO %in% so)
    
    values<-initial(values)
    row.names(M) <- M$SR
    class(M) <- c("bibliometrixDB", "data.frame")
    values$M <- M
    Mdisp <- values$M %>%
      mutate(across(everything(), ~ substring(., 1, 150))) %>%
      as.data.frame()
    
    if (dim(Mdisp)[1]>0){
      DTformat(Mdisp, nrow=3, filename="Filtered_DataTable", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, size='70%', filter="top",
               columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, selection=FALSE,scrollX=TRUE)
    }else{Mdisp=data.frame(Message="Empty collection", row.names = " ")}
  })
  
  output$dataFiltered <- DT::renderDT({
    DTfiltered()
  })
  
  # OVERVIEW ----
  ### Main Info ----
  output$MainInfo <- DT::renderDT({
    DTformat(values$TABvb , nrow=50, filename="Main_Information", pagelength=TRUE, left=1, right=2, numeric=NULL, dom=TRUE, size='100%', filter="none",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, selection=FALSE)
  })
  
  #### box1 ---------------
  output$Timespan <- renderValueBox({
    TAB <- ValueBoxes(values$M)
    values$TABvb <- TAB
    valueBox(value = p(TAB[TAB$Description=="Timespan", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong((TAB[TAB$Description=="Timespan", 2])), style = 'font-size:36px;color:white;', align="center"), 
             icon = fa_i(name="hourglass"), color = "blue",
             width = NULL)
  })
  
  #### box2 ---------------
  output$au <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Authors", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Authors", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="user"), color = "light-blue",
             width = NULL)
  })
  
  #### box3 ------------
  output$kw <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Author's Keywords (DE)", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Author's Keywords (DE)", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="spell-check"), color = "aqua",
             width = NULL)
  })
  
  #### box4 ---------------
  output$so <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p("Sources", style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Sources (Journals, Books, etc)", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name ="book"), color = "blue",
             width = NULL)
  })
  
  #### box5 --------------------
  output$auS1 <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Authors of single-authored docs", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Authors of single-authored docs", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="pen-fancy"), color = "light-blue",
             width = NULL)
  }) 
  
  #### box6 -------------
  output$cr <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="References", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="References", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="file"), color = "aqua",
             width = NULL)
  })
  
  #### box7 ----------------
  output$doc <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Documents", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Documents", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="layer-group"), color = "blue",
             width = NULL)
  })
  
  #### box8 ---------------
  output$col <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(strong("International Co-Authorship"), style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="International co-authorships %", 2]," %"), style = 'font-size:36px;color:white;',align="center"), 
             icon = icon("globe",lib = "glyphicon"), color = "light-blue",
             width = NULL)
  })
  
  #### box9 ---------------
  output$agePerDoc <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Document Average Age", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Document Average Age", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="calendar"), color = "aqua",
             width = NULL)
  })
  
  #### box10 ------------------
  output$cagr <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(strong("Annual Growth Rate"), style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Annual Growth Rate %", 2]," %"), style = 'font-size:36px;color:white;',align="center"), 
             icon = icon("arrow-up", lib="glyphicon"), color = "blue",
             width = NULL)
  })
  
  #### box11 ------
  output$coAuPerDoc <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Co-Authors per Doc", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Co-Authors per Doc", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = fa_i(name="users"), color = "light-blue",
             width = NULL)
  })
  
  #### box12 -------
  output$tc <- renderValueBox({
    TAB <- values$TABvb
    valueBox(value = p(TAB[TAB$Description=="Average citations per doc", 1], style = 'font-size:16px;color:white;'),
             subtitle = p(strong(TAB[TAB$Description=="Average citations per doc", 2]), style = 'font-size:36px;color:white;',align="center"), 
             icon = icon("volume-up", lib = "glyphicon"), color = "aqua",
             width = NULL)
  })
  
  observeEvent(input$reportMI,{
    if(!is.null(values$TABvb)){
      sheetname <- "MainInfo"
      list_df <- list(values$TABvb)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      values$wb <- res$wb
      popUp(title="Main Information", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # gemini button for word network
  output$MainInfoGeminiUI <- renderUI({
    #values <- geminiWaitingMessage(values, input$sidebarmenu)
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$MainInfoGemini, values)
  })
  
  # Annual Production ----
  output$CAGR <- renderText({
    paste0(values$GR," %")
  })
  
  output$AnnualProdPlot <- renderPlotly({
    res <- descriptive(values,type="tab2")
    values <-res$values
    Y <- values$TAB
    
    names(Y)=c("Year","Freq")
    x <- c(max(Y$Year)-0.02-diff(range(Y$Year))*0.125, max(Y$Year)-0.02)+1
    y <- c(min(Y$Freq),min(Y$Freq)+diff(range(Y$Freq))*0.125)
    
    g=ggplot2::ggplot(Y, aes(x = Year, y = Freq, text=paste("Year: ",Year,"\nN .of Documents: ",Freq))) +
      geom_line(aes(group="NA")) +
      #geom_area(aes(group="NA"),fill = 'grey90', alpha = .5) +
      labs(x = 'Year'
           , y = 'Articles'
           , title = "Annual Scientific Production") +
      scale_x_continuous(breaks= (Y$Year[seq(1,length(Y$Year),by=2)])) +
      theme(text = element_text(color = "#444444")
            ,panel.background = element_rect(fill = '#FFFFFF')
            ,panel.grid.minor = element_line(color = '#EFEFEF')
            ,panel.grid.major = element_line(color = '#EFEFEF')
            ,plot.title = element_text(size = 24)
            ,axis.title = element_text(size = 14, color = '#555555')
            ,axis.title.y = element_text(vjust = 1, angle = 0)
            ,axis.title.x = element_text(hjust = 0)
            ,axis.text.x = element_text(vjust = 1, angle = 90)
            ,axis.line.x = element_line(color="black",linewidth=0.5)
            ,axis.line.y = element_line(color="black",linewidth=0.5)
      ) +
      annotation_custom(values$logoGrid, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]) 
    values$ASPplot <- g
    
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.7, size=0.10)
  })
  
  observeEvent(input$reportASP,{
    if(!is.null(values$TAB)){
      list_df <- list(values$TAB)
      list_plot <- list(values$ASPplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "AnnualSciProd", wb=values$wb)
      values$wb <- wb
      popUp(title="Annual Scientific Production", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  output$ASPplot.save <- downloadHandler(
    filename = function() {
      
      paste("AnnualScientificProduction-", Sys.Date(), ".png", sep="")
    },
    
    content <- function(file) {
      ggsave(filename = file, plot = values$ASPplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$AnnualProdTable <- DT::renderDT({
    TAB <- values$TAB
    DTformat(TAB , nrow=10, filename="Annual_Production", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, size='100%', filter="none",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, selection=FALSE)
  })
  
  ## Annual Citation per Year ----
  output$AnnualTotCitperYearPlot <- renderPlotly({
    current_year = as.numeric(substr(Sys.Date(),1,4))+1
    Table2 <- values$M %>%
      group_by(PY) %>% 
      summarize(MeanTCperArt=round(mean(TC, na.rm=TRUE),2),
                N =n()) %>% 
      mutate(MeanTCperYear = round(MeanTCperArt/(current_year-PY),2),
             CitableYears = current_year-PY) %>% 
      rename(Year = PY) %>% 
      drop_na()
    values$AnnualTotCitperYear=Table2
    Table2$group="A"
    
    x <- c(max(Table2$Year)-0.02-diff(range(Table2$Year))*0.125, max(Table2$Year)-0.02)+1
    y <- c(min(Table2$MeanTCperYear),min(Table2$MeanTCperYear)+diff(range(Table2$MeanTCperYear))*0.125)
    
    g <- ggplot(Table2, aes(x = Year, y =MeanTCperYear,text=paste("Year: ",Year,"\nAverage Citations per Year: ",round(MeanTCperYear,1)))) +
      geom_line(aes(x = Year, y = MeanTCperYear, group=group)) +
      #geom_area(aes(x = Year, y = MeanTCperYear, group=group),fill = 'grey90', alpha = .5) +
      labs(x = 'Year'
           , y = 'Citations'
           , title = "Average Citations per Year")+
      scale_x_continuous(breaks= (Table2$Year[seq(1,length(Table2$Year),by=2)])) +
      theme(text = element_text(color = "#444444")
            ,panel.background = element_rect(fill = '#FFFFFF')
            ,panel.grid.minor = element_line(color = '#EFEFEF')
            ,panel.grid.major = element_line(color = '#EFEFEF')
            ,plot.title = element_text(size = 24)
            ,axis.title = element_text(size = 14, color = '#555555')
            ,axis.title.y = element_text(vjust = 1, angle = 0)
            ,axis.title.x = element_text(hjust = 0)
            ,axis.line.x = element_line(color="black",linewidth=0.5)
            ,axis.line.y = element_line(color="black",linewidth=0.5)
      ) + 
      annotation_custom(values$logoGrid, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]) 
    values$ACpYplot <- g
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.7, size=0.10)
  })
  
  observeEvent(input$reportACpY,{
    if(!is.null(values$AnnualTotCitperYear)){
      list_df <- list(values$AnnualTotCitperYear)
      list_plot <- list(values$ACpYplot)
      wb <- addSheetToReport(list_df, list_plot, sheetname = "AnnualCitPerYear", wb = values$wb)
      values$wb <- wb
      popUp(title="Average Citations per Year", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  output$ACpYplot.save <- downloadHandler(
    filename = function() {
      paste("AverageArticleCitationPerYear-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$ACpYplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$AnnualTotCitperYearTable <- DT::renderDT({
    TAB <- values$AnnualTotCitperYear
    DTformat(TAB , nrow=10, filename="Annual_Total_Citation_per_Year", pagelength=TRUE, left=NULL, right=NULL, numeric=c(2,4), dom=TRUE, size='100%', filter="none",
             columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, selection=FALSE)
  })
  
  ## Three Fields Plot ---- 
  TFP <- eventReactive(input$apply3F,{
    fields=c(input$LeftField, input$CentralField, input$RightField)
    threeFieldsPlot(values$M, fields=fields,n=c(input$LeftFieldn, input$CentralFieldn,input$RightFieldn))
  })
  
  output$ThreeFieldsPlot <- renderPlotly({
    values$TFP <- TFP()  
    
    is.reactive(values$TFP)
    values$TFP
  })
  
  
  # gemini button for Three Fields Plot
  output$TFPGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$TFPGemini, values)
    
  })
  
  observeEvent(input$reportTFP,{
    if (!is.null(values$TFP)){
      sheetname <- "ThreeFieldsPlot"
      ind <- which(regexpr(sheetname,values$wb$sheet_names)>-1)
      if (length(ind)>0){
        sheetname <- paste(sheetname, "(", length(ind) + 1, ")", sep = "")
      } 
      addWorksheet(wb=values$wb, sheetName=sheetname, gridLines = FALSE)
      values$fileTFP <- screenSh(values$TFP, zoom = 2, type="plotly")
      values$list_file <- rbind(values$list_file, c(sheetname,values$fileTFP,1))
      popUp(title="Three-Field Plot", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # SOURCES MENU ----
  ### Most Relevant Sources ----
  MRSources <- eventReactive(input$applyMRSources,{
    res <- descriptive(values,type="tab7")
    values <-res$values
    values$TABSo<-values$TAB
    xx<- values$TAB %>% 
      drop_na()
    if (input$MostRelSourcesK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostRelSourcesK}
    xx <- xx %>% 
      slice_head(n=k)
    xx$Sources=substr(xx$Sources,1,50)
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Sources", textLabx = "N. of Documents", title = "Most Relevant Sources", values)
    
    values$MRSplot <- g
    return(g)
  })
  
  output$MRSplot.save <- downloadHandler(
    filename = function() {
      paste("MostRelevantSources-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MRSplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelSourcesPlot <- renderPlotly({
    g <- MRSources()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.1, size=0.10)
  })
  
  output$MostRelSourcesTable <- DT::renderDT({
    
    g <- MRSources()
    
    TAB <- values$TABSo %>% drop_na()
    DTformat(TAB , nrow=10, filename="Most_Relevant_Sources", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMRS,{
    if(!is.null(values$TABSo)){
      list_df <- list(values$TABSo %>% drop_na())
      list_plot <- list(values$MRSplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostRelSources", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Relevant Sources", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Most Local Cited Sources ---- 
  MLCSources <- eventReactive(input$applyMLCSources,{
    values$M=metaTagExtraction(values$M,"CR_SO")
    TAB=tableTag(values$M,"CR_SO")
    TAB=data.frame(Sources=names(TAB),Articles=as.numeric(TAB))
    values$TABSoCit<-TAB
    xx<- TAB
    if (input$MostRelCitSourcesK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostRelCitSourcesK}
    xx=subset(xx, row.names(xx) %in% row.names(xx)[1:k])
    xx$Articles=as.numeric(xx$Articles)
    xx$Sources=substr(xx$Sources,1,50)
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Cited Sources", textLabx = "N. of Local Citations", title = "Most Local Cited Sources", values)
    
    values$MLCSplot <- g
    return(g)
  })
  
  output$MLCSplot.save <- downloadHandler(
    filename = function() {
      paste("MostLocalCitedSources-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MLCSplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelCitSourcesPlot <- renderPlotly({
    
    g <- MLCSources()
    
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$MostRelCitSourcesTable <- DT::renderDT({
    
    g <- MLCSources()
    TAB <- values$TABSoCit
    DTformat(TAB , nrow=10, filename="Most_Cited_Sources", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMLS,{
    if(!is.null(values$TABSoCit)){
      list_df <- list(values$TABSoCit)
      list_plot <- list(values$MLCSplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostLocCitSources", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Local Cited Sources", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Bradford's Law ---- 
  output$bradfordPlot <- renderPlotly({
    values$bradford=bradford(values$M)
    plot.ly(values$bradford$graph,flip=FALSE, side="r", aspectratio=1.6, size=0.15)
  })
  
  output$BLplot.save <- downloadHandler(
    filename = function() {
      paste("BradfordLaws-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$bradford$graph, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$bradfordTable <- DT::renderDT({
    DTformat(values$bradford$table , nrow=10, filename="Bradford_Law", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportBradford,{
    if(!is.null(values$bradford$table)){
      list_df <- list(values$bradford$table)
      list_plot <- list(values$bradford$graph)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "BradfordLaw", wb=values$wb)
      values$wb <- wb
      popUp(title="Core Sources by Bradford's Law", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Sources' Impact ----  
  Hsource <- eventReactive(input$applyHsource,{
    withProgress(message = 'Calculation in progress',
                 value = 0, {
                   res <- Hindex_plot(values,type="source", input)
                 })
    values$SIplot <- res$g
    plot.ly(res$g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$SIplot.save <- downloadHandler(
    filename = function() {
      paste("SourceImpact-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$SIplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$SourceHindexPlot <- renderPlotly({
    Hsource()
  })
  
  output$SourceHindexTable <- DT::renderDT({
    DTformat(values$H %>% rename(Source = Element) , nrow=10, filename="Source_Impact", pagelength=TRUE, left=NULL, right=NULL, numeric=4, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportSI,{
    if(!is.null(values$H)){
      list_df <- list(values$H %>% rename(Source = Element))
      list_plot <- list(values$SIplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "SourceLocImpact", wb=values$wb)
      values$wb <- wb
      popUp(title="Sources' Local Impact", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Source Growth ----  
  SOGrowth <- eventReactive(input$applySOGrowth,{
    
    if (input$cumSO=="Cum"){
      cdf=TRUE
      laby="Cumulate occurrences"
    }else{
      cdf=FALSE
      laby="Annual occurrences"} 
    
    values$PYSO=sourceGrowth(values$M,input$topSO[2], cdf=cdf)
    if (input$topSO[1]>1){
      values$PYSO <- values$PYSO[-c(2:(input$topSO[1]))]
    }
    
    term=names(values$PYSO)[-1]
    
    term=rep(term,each=dim(values$PYSO)[1])
    n=dim(values$PYSO)[1]*(dim(values$PYSO)[2]-1)
    freq=matrix(as.matrix(values$PYSO[,-1]),n,1)
    values$SODF=data.frame(Year=rep(values$PYSO$Year,(dim(values$PYSO)[2]-1)),Source=term, Freq=freq)
    
    Text <- paste(values$SODF$Source," (",values$SODF$Year,") ",values$SODF$Freq, sep="")
    
    width_scale <- 1.7 * 26 / length(unique(values$SODF$Source))
    
    x <- c(max(values$SODF$Year)-0.02-diff(range(values$SODF$Year))*0.15, max(values$SODF$Year)-0.02)+1
    y <- c(min(values$SODF$Freq),min(values$SODF$Freq)+diff(range(values$SODF$Freq))*0.15)
    
    g=ggplot(values$SODF, aes(x=values$SODF$Year,y=values$SODF$Freq, group=values$SODF$Source, color=values$SODF$Source, text=Text))+
      geom_line()+
      labs(x = 'Year'
           , y = laby
           , title = "Sources' Production over Time") +
      scale_x_continuous(breaks= (values$PYSO$Year[seq(1,length(values$PYSO$Year),by=ceiling(length(values$PYSO$Year)/20))])) +
      geom_hline(aes(yintercept=0), alpha=0.1)+
      labs(color = "Source")+
      theme(text = element_text(color = "#444444"),
            legend.text=ggplot2::element_text(size=width_scale),
            legend.box.margin = margin(6, 6, 6, 6),
            legend.title=ggplot2::element_text(size=1.5*width_scale,face="bold"),
            legend.position="bottom",
            legend.direction = "vertical",
            legend.key.size = grid::unit(width_scale/50, "inch"),
            legend.key.width = grid::unit(width_scale/50, "inch")
            ,plot.caption = element_text(size = 9, hjust = 0.5, color = "black", face = "bold")
            ,panel.background = element_rect(fill = '#FFFFFF')
            ,panel.grid.minor = element_line(color = '#EFEFEF')
            ,panel.grid.major = element_line(color = '#EFEFEF')
            ,plot.title = element_text(size = 24)
            ,axis.title = element_text(size = 14, color = '#555555')
            ,axis.title.y = element_text(vjust = 1, angle = 90)
            ,axis.title.x = element_text(hjust = 0.95, angle = 0)
            ,axis.text.x = element_text(size=10, angle = 90)
            ,axis.line.x = element_line(color="black",linewidth=0.5)
            ,axis.line.y = element_line(color="black",linewidth=0.5)
      ) + annotation_custom(values$logoGrid, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]) 
    
    values$SDplot <- g
    return(g)
  }) 
  
  output$SDplot.save <- downloadHandler(
    filename = function() {
      paste("SourceDynamics-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$SDplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$soGrowthPlot <- renderPlotly({
    
    g <- SOGrowth()
    
    leg <- list(
      orientation = 'h', 
      y = -0.15,
      font = list(
        family = "sans-serif",
        size = 10,
        color = "#000"),
      bgcolor = "#FFFFFF",
      bordercolor = "#FFFFFF",
      borderwidth = 2) 
    
    plot.ly(g, flip=FALSE, side="r", aspectratio=1.8, size=0.10) %>%
      layout(legend = leg) %>%
      config(displaylogo = FALSE,
             modeBarButtonsToRemove = c(
               'toImage',
               'sendDataToCloud',
               'pan2d', 
               'select2d', 
               'lasso2d',
               'toggleSpikelines',
               'hoverClosestCartesian',
               'hoverCompareCartesian'
             ))  %>%
      layout(hovermode = 'compare')
  })
  
  output$soGrowthtable <- DT::renderDT({
    
    g <- SOGrowth()
    soData=values$PYSO
    DTformat(soData , nrow=10, filename="Source_Prod_over_Time", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportSD,{
    if(!is.null(values$PYSO)){
      list_df <- list(values$PYSO)
      list_plot <- list(values$SDplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "SourceProdOverTime", wb=values$wb)
      values$wb <- wb
      popUp(title="Sources' Production over Time", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # AUTHORS MENU ----
  ## Authors ----
  ### Most Relevant Authors ----
  MRAuthors <- eventReactive(input$applyMRAuthors,{
    res <- descriptive(values,type="tab3")
    values <-res$values
    values$TABAu<-values$TAB
    
    xx=values$TABAu
    switch(input$AuFreqMeasure,
           t={
             lab="N. of Documents"
             xx=xx[,1:2]
           },
           p={xx=xx[,1:2]
           xx[,2]=as.numeric(xx[,2])/dim(values$M)[1]*100
           lab="N. of Documents (in %)"
           },
           f={
             xx=xx[,c(1,3)]
             lab="N. of Documents (Fractionalized)"
           })
    
    xx[,2]=as.numeric(xx[,2])
    
    if (input$MostRelAuthorsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostRelAuthorsK}
    
    xx=xx[1:k,]
    xx[,2]=round(xx[,2],1)
    xx <- xx[order(-xx[,2]),]
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Authors", textLabx = lab, title = "Most Relevant Authors", values)
    
    values$MRAplot <- g
    return(g)
  })
  
  output$MRAplot.save <- downloadHandler(
    filename = function() {
      paste("MostRelevantAuthors-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MRAplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelAuthorsPlot <- renderPlotly({
    
    g <- MRAuthors()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$MostRelAuthorsTable <- DT::renderDT({
    
    TAB <- values$TABAu
    DTformat(TAB , nrow=10, filename="Most_Relevant_Authors", pagelength=TRUE, left=NULL, right=NULL, numeric=3, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMRA,{
    if(!is.null(values$TABAu)){
      list_df <- list(values$TABAu)
      list_plot <- list(values$MRAplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostRelAuthors", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Relevant Authors", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Most Cited Authors ----  
  MLCAuthors <- eventReactive(input$applyMLCAuthors,{
    res <- descriptive(values,type="tab13")
    values <-res$values
    values$TABAuCit<-values$TAB
    
    xx <- values$TABAuCit
    lab <- "Local Citations"
    xx[,2]=as.numeric(xx[,2])
    
    if (input$MostCitAuthorsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostCitAuthorsK}
    
    xx=xx[1:k,]
    xx[,2]=round(xx[,2],1)
    xx <- xx[order(-xx[,2]),]
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Authors", textLabx = lab, title = "Most Local Cited Authors", values)
    
    values$MLCAplot <- g
    return(g)
  })
  
  output$MLCAplot.save <- downloadHandler(
    filename = function() {
      paste("MostLocalCitedAuthors-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MLCAplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostCitAuthorsPlot <- renderPlotly({
    
    g <- MLCAuthors()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$MostCitAuthorsTable <- DT::renderDT({
    
    TAB <- values$TABAuCit
    DTformat(TAB , nrow=10, filename="Most_Local_Cited_Authors", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMLCA,{
    if(!is.null(values$TABAuCit)){
      list_df <- list(values$TABAuCit)
      list_plot <- list(values$MLCAplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostLocCitAuthors", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Local Cited Authors", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Authors' Impact ----  
  HAuthors <- eventReactive(input$applyHAuthors,{
    withProgress(message = 'Calculation in progress',
                 value = 0, {
                   res <- Hindex_plot(values,type="author", input)
                 })
    values$AIplot <- res$g
    return(res)
  })
  
  output$AIplot.save <- downloadHandler(
    filename = function() {
      paste("AuthorImpact-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$AIplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$AuthorHindexPlot <- renderPlotly({
    res <- HAuthors()
    plot.ly(res$g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$AuthorHindexTable <- DT::renderDT({
    DTformat(values$H %>% rename(Author = Element), nrow=10, filename="Author_Impact", pagelength=TRUE, left=NULL, right=NULL, numeric=4, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportAI,{
    if(!is.null(values$H)){
      list_df <- list(values$H %>% rename(Author = Element))
      list_plot <- list(values$AIplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "AuthorLocImpact", wb=values$wb)
      values$wb <- wb
      popUp(title="Authors' Local Impact", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Authors Production Over Time ----  
  AUoverTime <- eventReactive(input$applyAUoverTime,{
    values$AUProdOverTime <- authorProdOverTime(values$M, k=input$TopAuthorsProdK, graph=FALSE)
  })
  
  # gemini button for Apot
  output$ApotGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$ApotGemini, values)
    
  })

  output$APOTplot.save <- downloadHandler(
    filename = function() {
      paste("AuthorsProductionOverTime-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$AUProdOverTime$graph, dpi = values$dpi, height = values$h, width = values$h*2.5, bg="white")
    },
    contentType = "png"
  )
  
  output$TopAuthorsProdPlot <- renderPlotly({
    AUoverTime()
    plot.ly(values$AUProdOverTime$graph, flip=TRUE, side="l", aspectratio=1)
  })
  
  output$TopAuthorsProdTable <- DT::renderDT({
    AUoverTime()
    
    TAB <- values$AUProdOverTime$dfAU
    DTformat(TAB , nrow=10, filename="Author_Prod_over_Time", pagelength=TRUE, left=NULL, right=NULL, numeric=5, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TopAuthorsProdTablePapers <- DT::renderDT({
    AUoverTime()
    TAB <- values$AUProdOverTime$dfPapersAU
    TAB$DOI=paste0('<a href=\"https://doi.org/',TAB$DOI,'\" target=\"_blank\">',TAB$DOI,'</a>')
    DTformat(TAB , nrow=10, filename="Author_Prod_over_Time_Docs", pagelength=TRUE, left=NULL, right=NULL, numeric=7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
    })
  
  observeEvent(input$reportAPOT,{
    if(!is.null(values$AUProdOverTime$dfPapersAU)){
      list_df <- list(values$AUProdOverTime$dfPapersAU)
      list_plot <- list(values$AUProdOverTime$graph)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "AuthorProdOverTime", wb=values$wb)
      values$wb <- wb
      popUp(title="Authors' Production over Time", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Lotka Law ----  
  output$lotkaPlot <- renderPlotly({
    values$lotka <- lotka(values$M)
    values$LLplot <- values$lotka$g
    plot.ly(values$lotka$g_shiny,flip=FALSE, side="r", aspectratio=1.4, size=0.10)
  })
  
  output$LLplot.save <- downloadHandler(
    filename = function() {
      paste("LotkaLaw-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$LLplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$lotkaTable <- DT::renderDT({

    DTformat(values$lotka$AuthorProd, nrow=10, filename="Lotka_Law", pagelength=TRUE, left=NULL, right=NULL, numeric=c(3,4), dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportLotka,{
    if(!is.null(values$lotka$AuthorProd)){
      list_df <- list(values$lotka$AuthorProd)
      list_plot <- list(values$LLplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "LotkaLaw", wb=values$wb)
      values$wb <- wb
      popUp(title="Author Productivity through Lotka's Law", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # Affiliations ----
  ### Most Relevant Affiliations ---- 
  MRAffiliations <- eventReactive(input$applyMRAffiliations,{
    if (input$disAff=="Y"){
      res <- descriptive(values,type="tab11")
    }else{
      res <- descriptive(values,type="tab12")
    }
    xx=values$TAB
    names(xx)=c("AFF","Freq")
    values <-res$values
    values$TABAff <- values$TAB
    
    if (input$MostRelAffiliationsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostRelAffiliationsK}
    
    xx=xx[1:k,]
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Affiliations", textLabx = "Articles", title = "Most Relevant Affiliations", values)
    
    values$AFFplot <- g
    return(g)
  })
  
  output$AFFplot.save <- downloadHandler(
    filename = function() {
      paste("MostRelevantAffiliations-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$AFFplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelAffiliationsPlot <- renderPlotly({
    
    g <- MRAffiliations()
    
    plot.ly(g,flip=FALSE, side="r", aspectratio=1, size=0.15)
  })
  
  output$MostRelAffiliationsTable <- DT::renderDT({
    g <- MRAffiliations()
    
    TAB <- values$TABAff
    DTformat(TAB, nrow=10, filename="Most_Relevant_Affiliations", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMRAFF,{
    if(!is.null(values$TABAff)){
      list_df <- list(values$TABAff)
      list_plot <- list(values$AFFplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostRelAffiliations", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Relevant Affiliations", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Affiliation OverTime ----  
  AFFGrowth <- eventReactive(input$applyAFFGrowth,{
    
    values <- AffiliationOverTime(values,input$topAFF)
    
  }) 
  
  output$AffOverTimeplot.save <- downloadHandler(
    filename = function() {
      paste("AffiliationOverTime-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$AffOverTimePlot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$AffOverTimePlot <- renderPlotly({
    
    AFFGrowth()
    g <- values$AffOverTimePlot
    leg <- list(
      orientation = 'h', 
      y = -0.15,
      font = list(
        family = "sans-serif",
        size = 10,
        color = "#000"),
      bgcolor = "#FFFFFF",
      bordercolor = "#FFFFFF",
      borderwidth = 2) 
    
    plot.ly(g, flip=FALSE, side="r", aspectratio=1.8, size=0.10) %>%
      layout(legend = leg) %>%
      config(displaylogo = FALSE,
             modeBarButtonsToRemove = c(
               'sendDataToCloud',
               'pan2d', 
               'select2d', 
               'lasso2d',
               'toggleSpikelines'
             )) %>%
      layout(hovermode = 'compare')
  })
  
  output$AffOverTimeTable <- DT::renderDT({
    
    AFFGrowth()
    afftimeData <- values$AffOverTime
    DTformat(afftimeData, nrow=10, filename="Affiliation_over_Time", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportAFFPOT,{
    if(!is.null(values$AffOverTime)){
      list_df <- list(values$AffOverTime)
      list_plot <- list(values$AffOverTimePlot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "AffOverTime", wb=values$wb)
      values$wb <- wb
      popUp(title="Affiliations' Production over Time", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # Countries ----
  ### Country by Corresponding Authors ----  
  CAUCountries <- eventReactive(input$applyCAUCountries,{
    res <- descriptive(values,type="tab5")
    values <-res$values
    values$TABCo <- values$TAB
    
    k=input$MostRelCountriesK
    xx <- values$TABCo %>% slice_head(n=k) %>% 
      select(Country,SCP,MCP)
    xx=xx[order(-(xx$SCP+xx$MCP)),]
    xx1=cbind(xx[,1:2],rep("SCP",k))
    names(xx1)=c("Country","Freq","Collaboration")
    xx2=cbind(xx[,c(1,3)],rep("MCP",k))
    names(xx2)=c("Country","Freq","Collaboration")
    xx=rbind(xx2,xx1)
    xx$Country=factor(xx$Country,levels=xx$Country[1:dim(xx2)[1]])
    
    xx2 <- xx %>% dplyr::group_by(Country) %>%
      dplyr::summarize(Freq = sum(Freq))
    
    x <- c(0.5,0.5+length(levels(xx2$Country))*0.125)+1
    y <- c(max(xx2$Freq)-0.02-diff(range(xx2$Freq))*0.125,max(xx2$Freq)-0.02)
    
    g=suppressWarnings(ggplot2::ggplot(data=xx, aes(x=Country, y=Freq,fill=Collaboration, text=paste("Country: ",Country,"\nN.of Documents: ",Freq))) +
                         geom_bar(aes(group="NA"),stat="identity")+
                         scale_x_discrete(limits = rev(levels(xx$Country)))+
                         scale_fill_discrete(name="Collaboration",
                                             breaks=c("SCP","MCP"))+
                         labs(title = "Corresponding Author's Countries", x = "Countries", y = "N. of Documents", 
                              caption = "SCP: Single Country Publications, MCP: Multiple Country Publications")+
                         theme(plot.caption = element_text(size = 9, hjust = 0.5,
                                                           color = "blue", face = "italic")
                               ,panel.background = element_rect(fill = '#FFFFFF')
                               ,panel.grid.major.y  = element_line(color = '#EFEFEF')
                               ,plot.title = element_text(size = 24)
                               ,axis.title = element_text(size = 14, color = '#555555')
                               ,axis.title.y = element_text(vjust = 1, angle = 0)
                               ,axis.title.x = element_text(hjust = 0)
                               ,axis.line.x = element_line(color="black",linewidth=0.5)
                         ) +
                         coord_flip()) + 
      annotation_custom(values$logoGrid, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]) 
    values$TABCo <- values$TABCo %>% 
      mutate(Freq = Freq*100,
             MCP_Ratio = MCP_Ratio*100) %>% 
      rename("Articles %" = Freq,
             "MCP %" = MCP_Ratio) %>% 
      select(Country, "Articles","Articles %", SCP, MCP, "MCP %")
      
    values$MRCOplot <- g
    return(g)
  }) 
  
  # gemini button for word network
  output$MostRelCountriesGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$MostRelCountriesGemini, values)
    
  })
  
  output$MRCOplot.save <- downloadHandler(
    filename = function() {
      paste("MostRelevantCountries-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MRCOplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelCountriesPlot <- renderPlotly({
    g <- CAUCountries()
    plot.ly(g,flip=T, side="r", aspectratio=1.4, size=0.10, data.type=1)
  })
  
  output$MostRelCountriesTable <- DT::renderDT({
    g <- CAUCountries()
    
    TAB <- values$TABCo
    DTformat(TAB, nrow=10, filename="Most_Relevant_Countries_By_Corresponding_Author", pagelength=TRUE, left=NULL, right=NULL, numeric=c(3,6), dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=1, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMRCO,{
    if(!is.null(values$TABCo)){
      list_df <- list(values$TABCo)
      list_plot <- list(values$MRCOplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "CorrAuthCountries", wb=values$wb)
      values$wb <- wb
      popUp(title="Corresponding Author's Countries", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Country Production ----  
  output$countryProdPlot <- renderPlotly({
    values$mapworld<-mapworld(values$M, values)
    plot.ly(values$mapworld$g,flip=FALSE, side="r", aspectratio=1.7, size=0.07, data.type=1,height=15)
  })
  
  output$CSPplot.save <- downloadHandler(
    filename = function() {
      paste("CountryScientificProduction-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$mapworld$g, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$countryProdTable <- DT::renderDT({
    
    TAB <- values$mapworld$tab %>% rename(Country = region)
    DTformat(TAB, nrow=10, filename="Country_Production", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportCSP,{
    if(!is.null(values$mapworld$tab)){
      list_df <- list(values$mapworld$tab)
      list_plot <- list(values$mapworld$g)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "CountrySciProd", wb=values$wb)
      values$wb <- wb
      popUp(title="Countries' Scientific Production", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Countries' Production Over Time ----  
  COGrowth <- eventReactive(input$applyCOGrowth,{
    
    values <- CountryOverTime(values,input$topCO)
    
  }) 
  
  output$CountryOverTimeplot.save <- downloadHandler(
    filename = function() {
      paste("CountryOverTime-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$CountryOverTimePlot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$CountryOverTimePlot <- renderPlotly({
    
    COGrowth()
    g <- values$CountryOverTimePlot
    leg <- list(
      orientation = 'h', 
      y = -0.15,
      font = list(
        family = "sans-serif",
        size = 10,
        color = "#000"),
      bgcolor = "#FFFFFF",
      bordercolor = "#FFFFFF",
      borderwidth = 2) 
    
    plot.ly(g, flip=FALSE, side="r", aspectratio=1.8, size=0.10) %>%
      layout(legend = leg) %>%
      config(displaylogo = FALSE,
             modeBarButtonsToRemove = c(
               'toImage',
               'sendDataToCloud',
               'pan2d', 
               'select2d', 
               'lasso2d',
               'toggleSpikelines',
               'hoverClosestCartesian',
               'hoverCompareCartesian'
             )) %>%
      layout(hovermode = 'compare')
  })
  
  output$CountryOverTimeTable <- DT::renderDT({
    
    COGrowth()
    cotimeData=values$CountryOverTime
    DTformat(cotimeData, nrow=10, filename="Countries_Production_Over_Time", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportCPOT,{
    if(!is.null(values$CountryOverTime)){
      list_df <- list(values$CountryOverTime)
      list_plot <- list(values$CountryOverTimePlot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "CountryProdOverTime", wb=values$wb)
      values$wb <- wb
      popUp(title="Countries' Production over Time", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
    
    
    
  })
  
  ### Most Cited Country ----    
  MCCountries <- eventReactive(input$applyMCCountries,{
    res <- descriptive(values,type="tab6")
    values <-res$values
    values$TABCitCo <- values$TAB
    
    xx=values$TAB
    xx[,2]=as.numeric(xx[,2])
    xx[,3]=as.numeric(xx[,3])
    if (input$MostCitCountriesK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostCitCountriesK}
    if (input$CitCountriesMeasure=="TC"){
      xx=xx[1:k,c(1,2)]
      laby="N. of Citations"
    } else {
      xx=xx[order(-xx[,3]),]
      xx=xx[1:k,c(1,3)]
      laby="Average Article Citations"
    }
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Countries", textLabx = laby, title = "Most Cited Countries", values)
    
    values$MCCplot <- g
    return(g)
  })
  
  output$MCCplot.save <- downloadHandler(
    filename = function() {
      paste("MostCitedCountries-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MCCplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostCitCountriesPlot <- renderPlotly({
    g <- MCCountries()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1.3, size=0.10)
  })
  
  output$MostCitCountriesTable <- DT::renderDT({
    g <- MCCountries()
    TAB <- values$TABCitCo
    DTformat(TAB, nrow=10, filename="Most_Cited_Countries", pagelength=TRUE, left=NULL, right=NULL, numeric=3, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMCCO,{
    if(!is.null(values$TABCitCo)){
      list_df <- list(values$TABCitCo)
      list_plot <- list(values$MCCplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostCitCountries", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Cited Countries", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # DOCUMENTS MENU ----
  ## Documents ----
  ### Most Global Cited Documents ----
  
  MGCDocuments <- eventReactive(input$applyMGCDocuments,{
    res <- descriptive(values,type="tab4")
    values <-res$values
    values$TABGlobDoc <- values$TAB
    
    if (input$CitDocsMeasure=="TC"){
      xx <- values$TABGlobDoc %>% select(1,3)
      lab="Global Citations"} else {
        xx <- values$TABGlobDoc %>% select(1,4)
        xx[,2] <- round(xx[,2],1)
        lab="Global Citations per Year"
      }
    
    if (input$MostCitDocsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostCitDocsK}
    
    xx=xx[1:k,]
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "Documents", textLabx = lab, title = "Most Global Cited Documents", values)
    
    values$MGCDplot <- g
    return(g)
  })
  
  output$MGCDplot.save <- downloadHandler(
    filename = function() {
      paste("MostGlobalCitedDocuments-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MGCDplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostCitDocsPlot <- renderPlotly({
    g <- MGCDocuments()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1, size=0.10)
  })
  
  output$MostCitDocsTable <- DT::renderDT({
    g <- MGCDocuments()
    TAB <- values$TABGlobDoc
    TAB$DOI<- paste0('<a href=\"https://doi.org/',TAB$DOI,'\" target=\"_blank\">',TAB$DOI,'</a>')
    DTformat(TAB, nrow=10, filename="Most_Global_Cited_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=4:5, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMCD,{
    if(!is.null(values$TABGlobDoc)){
      list_df <- list(values$TABGlobDoc)
      list_plot <- list(values$MGCDplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostGlobCitDocs", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Global Cited Documents", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Most Local Cited Documents ----  
  MLCDocuments <- eventReactive(input$applyMLCDocuments,{
    withProgress(message = 'Calculation in progress',
                 value = 0, {
                   TAB <-localCitations(values$M, fast.search=FALSE, sep = input$LocCitSep)$Paper
                   TAB <- TAB %>%
                     group_by(Year) %>%
                     mutate(Ratio = LCS/GCS*100,
                            NLCS = LCS/mean(LCS),
                            NGCS = GCS/mean(GCS)) %>%
                     ungroup() %>%
                     as.data.frame()
                 })
    
    xx=data.frame(Document=as.character(TAB[,1]), DOI=as.character(TAB[,2]), Year=TAB[,3], 
                  "Local Citations"=TAB[,4], "Global Citations"=TAB[,5],"LC/GC Ratio"=TAB[6], 
                  "Normalized Local Citations"=TAB[,7],"Normalized Global Citations"=TAB[,8])
    values$TABLocDoc=xx
    
    if (input$MostLocCitDocsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostLocCitDocsK}
    
    xx=xx[1:k,]
    
    g <- freqPlot(xx,x=4,y=1, textLaby = "Documents", textLabx = "Local Citations", title = "Most Local Cited Documents", values)
    
    values$MLCDplot <- g
    return(g)
  })
  
  # gemini button for word network
  output$MostLocCitDocsGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$MostLocCitDocsGemini, values)
  })

  output$MLCDplot.save <- downloadHandler(
    filename = function() {
      paste("MostLocalCitedDocuments-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MLCDplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostLocCitDocsPlot <- renderPlotly({
    g <- MLCDocuments()
    plot.ly(g,flip=FALSE, side="r", aspectratio=1, size=0.10)
  })
  
  output$MostLocCitDocsTable <- DT::renderDT({
    
    TAB <- values$TABLocDoc
    TAB$DOI <- paste0('<a href=\"https://doi.org/',TAB$DOI,'\" target=\"_blank\">',TAB$DOI,'</a>')
    
    names(TAB)[4:8] <- c("Local Citations", "Global Citations","LC/GC Ratio (%)", "Normalized Local Citations","Normalized Global Citations")
    DTformat(TAB, nrow=10, filename="Most_Local_Cited_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=6:8, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMLCD,{
    if(!is.null(values$TABLocDoc)){
      list_df <- list(values$TABLocDoc)
      list_plot <- list(values$MLCDplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostLocCitDocs", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Local Cited Documents", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ## Cited References ----
  ### Most Local Cited References ----
  MLCReferences <- eventReactive(input$applyMLCReferences,{
    CR <- citations(values$M,sep=input$CitRefsSep)$Cited
    TAB <- data.frame(names(CR),as.numeric(CR))
    names(TAB) <- c("Cited References", "Citations")
    values$TABCitRef <- TAB %>% filter(`Cited References`!="ANONYMOUS, NO TITLE CAPTURED")
    
    xx=values$TABCitRef
    if (input$MostCitRefsK>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostCitRefsK}
    
    xx=xx[1:k,]
    
    g <- freqPlot(xx,x=2,y=1, textLaby = "References", textLabx = "Local Citations", title = "Most Local Cited References", values, string.max=70)
    
    values$MLCRplot <- g
    return(g)
  })
  
  output$MLCRplot.save <- downloadHandler(
    filename = function() {
      paste("MostLocalCitedReferences-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MLCRplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostCitRefsPlot <- renderPlotly({
    g <- MLCReferences()
    plot.ly(g,flip=FALSE, side="r", aspectratio=0.6, size=0.20)
  })
  
  output$MostCitRefsTable <- DT::renderDT({
    g <- MLCReferences()
    TAB <- values$TABCitRef
    
    TAB$link <- trimES(gsub("[[:punct:]]" , " ",reduceRefs(TAB[,1])))
    TAB$link <- paste0('<a href=\"https://scholar.google.it/scholar?hl=en&as_sdt=0%2C5&q=',TAB$link,'\" target=\"_blank\">','link','</a>')
    
    TAB=TAB[,c(3,1,2)]
    names(TAB)[1]="Google Scholar"
    DTformat(TAB, nrow=10, filename="Most_Local_Cited_References", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMLCR,{
    if(!is.null(values$TABCitRef)){
      list_df <- list(values$TABCitRef)
      list_plot <- list(values$MLCRplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostLocCitRefs", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Local Cited References", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Reference Spectroscopy ---- 
  RPYS <- eventReactive(input$applyRPYS,{
    timespan <- c(-Inf,Inf)
    if (!is.na(input$rpysMinYear)){
      timespan[1] <- input$rpysMinYear
    }
    if (!is.na(input$rpysMaxYear)){
      timespan[2] <- input$rpysMaxYear
    }
    values$res <- rpys(values$M, sep=input$rpysSep, timespan=timespan, graph=FALSE)
  })
  
  output$RSplot.save <- downloadHandler(
    filename = function() {
      paste("ReferenceSpectroscopy-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$res$spectroscopy, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  # gemini button for rpys
  output$rpysGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$rpysGemini, values)
    
  })

  output$rpysPlot <- renderPlotly({
    RPYS()
    plot.ly(values$res$spectroscopy, side="l", aspectratio = 1.3, size=0.10)
  })
  
  output$rpysTable <- DT::renderDT({
    RPYS()
    rpysData=values$res$rpysTable
    DTformat(rpysData, nrow=10, filename="RPYS", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$crTable <- DT::renderDT({
    RPYS()
    crData=values$res$CR
    crData$link <- paste0('<a href=\"https://scholar.google.it/scholar?hl=en&as_sdt=0%2C5&q=',crData$Reference,'\" target=\"_blank\">','link','</a>')
    
    crData=crData[order(-as.numeric(crData$Year),-crData$Freq),]
    names(crData)=c("Year", "Reference", "Local Citations", "Google link")
    crData <- crData[,c(1,4,2,3)] 
    DTformat(crData, nrow=10, filename="RPYS_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportRPYS,{
    if(!is.null(values$res$CR)){
      list_df <- list(values$res$CR, values$res$rpysTable)
      list_plot <- list(values$res$spectroscopy)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "RPYS", wb=values$wb)
      values$wb <- wb
      popUp(title="Reference Spectroscopy", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ## Words ----
  ### Most Frequent Words ----
  
  observeEvent(input$MostRelWordsStop,{
    values$MRWremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$MostRelWordsStop, sep=input$MostRelWordsSep)))
    values$GenericSL <- values$MRWremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$MRWSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$MRWSyn, sep=input$MRWSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
      }))
    values$MRWsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$MRWsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  output$stopwordList <- renderDT({
    DTformat(values$GenericSL, nrow=Inf, filename="Stopword_List", pagelength=FALSE, left=1, right=NULL, numeric=NULL, dom="none", 
             size='90%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, 
             selection=FALSE, scrollY=TRUE)
  })
  
  output$synonymList <- renderDT({
    DTformat(values$GenericSYN, nrow=Inf, filename="Stopword_List", pagelength=FALSE, left=1, right=NULL, numeric=NULL, dom="none", 
             size='90%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, 
             selection=FALSE, scrollY=TRUE)
  })
  
  MFWords <- eventReactive(input$applyMFWords,{
    if (input$MostRelWords %in% c("TI","AB")){
      ngrams <- as.numeric(input$MRWngrams)
    }else{
      ngrams <- 1
    }
    
    ### load file with terms to remove
    if (input$MostRelWordsStopFile=="Y"){
      remove.terms <- trimws(values$MRWremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$MRWremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$MRWSynFile=="Y"){
      synonyms <- values$MRWsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$MRWsyn.terms <- synonyms
    ### end of block
    
    WR <- wordlist(values$M,Field=input$MostRelWords,n=Inf,measure="identity", ngrams=ngrams, remove.terms = remove.terms, synonyms = synonyms)$v
    
    TAB <- data.frame(names(WR),as.numeric(WR))
    names(TAB) <- c("Words", "Occurrences")
    values$TABWord <- TAB
    
    xx=values$TABWord
    if (input$MostRelWordsN>dim(xx)[1]){
      k=dim(xx)[1]
    } else {k=input$MostRelWordsN}
    
    xx=xx[1:k,]
    switch(input$MostRelWords,
           ID={lab="Keywords Plus"},
           DE={lab="Author's Keywords"},
           KW_Merged={lab="All Keywords"},
           TI={lab="Title's Words"},
           AB={lab="Abstract's Words"},
           WC={lab="Subject Categories"})
    
    g <- freqPlot(xx,x=2,y=1, textLaby = lab, textLabx = "Occurrences", title = "Most Relevant Words", values)
    
    values$MRWplot <- g
    return(g)
  })
  
  output$MRWplot.save <- downloadHandler(
    filename = function() {
      paste("MostRelevantWords-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$MRWplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$MostRelWordsPlot <- renderPlotly({
    g <- MFWords()
    plot.ly(g, side="r", aspectratio = 1.3, size=0.10)
  })
  
  output$MostRelWordsTable <- DT::renderDT({
    g <- MFWords()
    
    TAB <- values$TABWord
    DTformat(TAB, nrow=10, filename="Most_Frequent_Words", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportMFW,{
    if(!is.null(values$TABWord)){
      list_df <- list(values$TABWord)
      list_plot <- list(values$MRWplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "MostFreqWords", wb=values$wb)
      values$wb <- wb
      popUp(title="Most Frequent Words", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### WordCloud ----  
  observeEvent(input$WCStop,{
    values$WCremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$WCStop, sep=input$WCSep)))
    values$GenericSL <- values$WCremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })

  observeEvent(input$WCSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$WCSyn, sep=input$WCSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$WCsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$WCsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })

  WordCloud <- eventReactive(input$applyWordCloud,{
    if (input$summaryTerms %in% c("TI","AB")){
      ngrams <- as.numeric(input$summaryTermsngrams)
    }else{
      ngrams <- 1
    }
    
    ### load file with terms to remove
    if (input$WCStopFile=="Y"){
      remove.terms <- trimws(values$WCremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$WCremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$WCSynFile=="Y"){
      synonyms <- values$WCsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
      print(synonyms)
    }else{synonyms <- NULL}
    #values$WCsyn.terms <- synonyms
    ### end of block
    
    resW=wordlist(M=values$M, Field=input$summaryTerms, n=input$n_words, measure=input$measure, ngrams=ngrams, remove.terms = remove.terms, synonyms = synonyms)
    
    W=resW$W
    values$Words <- resW$Words
    
    values$WordCloud <- wordcloud2::wordcloud2(W, size = input$scale, minSize = 0, gridSize =  input$padding,
                           fontFamily = input$font, fontWeight = 'normal',
                           color = input$wcCol, backgroundColor = "white", #input$wcBGCol,
                           minRotation = 0, maxRotation = input$rotate/10, shuffle = TRUE,
                           rotateRatio = 0.7, shape = input$wcShape, ellipticity = input$ellipticity,
                           widgetsize = NULL, figPath = NULL, hoverFunction = NULL)
  })
  
  output$wordcloud <- wordcloud2::renderWordcloud2({
    WordCloud()
    values$WordCloud
  })
  
  observeEvent(input$reportWC,{
    if(!is.null(values$Words)){
      sheetname <- "WordCloud"
      list_df <- list(values$Words)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      values$wb <- res$wb
      #values$fileTFP <- screenSh(selector = "#wordcloud") ## screenshot
      values$fileWC <- screenSh(values$WordCloud, zoom = 2, type="plotly")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileWC,res$col))
      popUp(title="WordCloud", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### TreeMap ----  
  observeEvent(input$TreeMapStop,{
    values$TreeMapremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$TreeMapStop, sep=input$TreeMapSep)))
    values$GenericSL <- values$TreeMapremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$TreeMapSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$TreeMapSyn, sep=input$TreeMapSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$TreeMapsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$TreeMapsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  TreeMap <- eventReactive(input$applyTreeMap,{
    if (input$treeTerms %in% c("TI","AB")){
      ngrams <- as.numeric(input$treeTermsngrams)
    }else{
      ngrams <- 1
    }
    ### load file with terms to remove
    if (input$TreeMapStopFile=="Y"){
      remove.terms <- trimws(values$TreeMapremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$TreeMapremove.terms <- remove.terms
    ### end of block
    ### load file with synonyms
    if (input$TreeMapSynFile=="Y"){
      synonyms <- values$TreeMapsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$TreeMapsyn.terms <- synonyms
    ### end of block
    
    resW=wordlist(M=values$M, Field=input$treeTerms, n=input$treen_words, measure="identity", ngrams=ngrams, remove.terms=remove.terms, synonyms = synonyms)
    
    W=resW$W
    values$TreeMap <- plot_ly(
      type='treemap',
      labels=W[,1],
      parents="Tree",
      values= W[,2],
      textinfo="label+value+percent entry",
      domain=list(column=0)) %>% 
      config(displaylogo = FALSE,
             modeBarButtonsToRemove = c(
               'toImage',
               'sendDataToCloud',
               'pan2d', 
               'select2d', 
               'lasso2d',
               'toggleSpikelines',
               'hoverClosestCartesian',
               'hoverCompareCartesian'
             )) 
    
    values$WordsT=resW$Words
    return(resW$Words)
  })
  
  output$treemap <- renderPlotly({
    TreeMap()
    values$TreeMap
  })
  
  
  output$wordTable <- DT::renderDT({
    WordCloud()
    DTformat(values$Words, nrow=10, filename="Most_Frequent_Words", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$treeTable <- DT::renderDT({
    WordsT <- TreeMap()
    DTformat(values$WordsT, nrow=10, filename="Most_Frequent_Words", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  },height = 600, width = 900)
  
  observeEvent(input$reportTREEMAP,{
    if(!is.null(values$WordsT)){
      sheetname <- "TreeMap"
      list_df <- list(values$WordsT)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      values$wb <- res$wb
      #values$fileTFP <- screenSh(selector = "#treemap") ## screenshot
      values$fileTreeMap <- screenSh(values$TreeMap, zoom = 2, type="plotly")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileTreeMap,res$col))
      popUp(title="TreeMap", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Words' Frequency over Time ----   
  observeEvent(input$WDStop,{
    values$WDremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$WDStop, sep=input$WDSep)))
    values$GenericSL <- values$WDremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$WDSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$WDSyn, sep=input$WDSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$WDsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$WDsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  WDynamics <- eventReactive(input$applyWD,{
    if (input$cumTerms=="Cum"){
      cdf=TRUE
      laby="Cumulate occurrences"
    }else{
      cdf=FALSE
      laby="Annual occurrences"}
    
    ### load file with terms to remove
    if (input$WDStopFile=="Y"){
      remove.terms <- trimws(values$WDremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$WDremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$WDSynFile=="Y"){
      synonyms <- values$WDsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$WDsyn.terms <- synonyms
    ### end of block
    
    switch(input$growthTerms,
           ID={
             KW=KeywordGrowth(values$M, Tag = "ID", sep = ";", top = input$topkw[2], cdf = cdf, remove.terms=remove.terms, synonyms = synonyms)
           },
           DE={
             KW=KeywordGrowth(values$M, Tag = "DE", sep = ";", top = input$topkw[2], cdf = cdf, remove.terms=remove.terms, synonyms = synonyms)
           },
           KW_Merged={
             KW=KeywordGrowth(values$M, Tag = "KW_Merged", sep = ";", top = input$topkw[2], cdf = cdf, remove.terms=remove.terms, synonyms = synonyms)
           },
           TI={
             values$M=termExtraction(values$M,Field = "TI", verbose=FALSE, ngrams=as.numeric(input$growthTermsngrams), remove.terms=remove.terms, synonyms = synonyms)
             KW=KeywordGrowth(values$M, Tag = "TI_TM", sep = ";", top = input$topkw[2], cdf = cdf)
           },
           AB={
             values$M=termExtraction(values$M,Field = "AB", verbose=FALSE, ngrams=as.numeric(input$growthTermsngrams), remove.terms=remove.terms, synonyms = synonyms)
             KW=KeywordGrowth(values$M, Tag = "AB_TM", sep = ";", top = input$topkw[2], cdf = cdf)
           }
    )
    
    values$KW=KW[,c(1,seq(input$topkw[1],input$topkw[2])+1)]
    
    term=names(values$KW)[-1]
    term=rep(term,each=dim(values$KW)[1])
    n=dim(values$KW)[1]*(dim(values$KW)[2]-1)
    freq=matrix(as.matrix(values$KW[,-1]),n,1)
    values$DF=data.frame(Year=rep(values$KW$Year,(dim(values$KW)[2]-1)),Term=term, Freq=freq)
    
    width_scale <- 2.5 * 26 / length(unique(values$DF$Term))
    
    Text <- paste(values$DF$Term," (",values$DF$Year,") ",values$DF$Freq, sep="")
    
    x <- c(max(values$DF$Year)-0.02-diff(range(values$DF$Year))*0.20, max(values$DF$Year)-0.02)-1
    y <- c(min(values$DF$Freq),min(values$DF$Freq)+diff(range(values$DF$Freq))*0.20)
    
    g <- ggplot(values$DF, aes(x=Year,y=Freq, group=Term, color=Term, text = Text))+
      geom_line()+
      labs(x = 'Year'
           , y = laby
           , title = "Words' Frequency over Time") +
      scale_x_continuous(breaks= (values$KW$Year[seq(1,length(values$KW$Year),by=ceiling(length(values$KW$Year)/20))])) +
      geom_hline(aes(yintercept=0), alpha=0.1)+
      labs(color = "Term")+
      theme(text = element_text(color = "#444444"),
            legend.text=ggplot2::element_text(size=width_scale),
            legend.box.margin = margin(6, 6, 6, 6),
            legend.title=ggplot2::element_text(size=1.5*width_scale,face="bold"),
            legend.position="bottom",
            legend.direction = "vertical",
            legend.key.size = grid::unit(width_scale/50, "inch"),
            legend.key.width = grid::unit(width_scale/50, "inch")
            ,plot.caption = element_text(size = 9, hjust = 0.5, color = "black", face = "bold")
            ,panel.background = element_rect(fill = '#FFFFFF')
            ,panel.grid.minor = element_line(color = '#EFEFEF')
            ,panel.grid.major = element_line(color = '#EFEFEF')
            ,plot.title = element_text(size = 24)
            ,axis.title = element_text(size = 14, color = '#555555')
            ,axis.title.y = element_text(vjust = 1, angle = 90)
            ,axis.title.x = element_text(hjust = 0.95, angle = 0)
            ,axis.text.x = element_text(size=10, angle = 90)
            ,axis.line.x = element_line(color="black",linewidth=0.5)
            ,axis.line.y = element_line(color="black",linewidth=0.5)
      ) + 
      annotation_custom(values$logoGrid, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]) 
    
    values$WDplot <- g
    return(g)
  })
  
  output$WDplot.save <- downloadHandler(
    filename = function() {
      paste("WordsFrequencyOverTime-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$WDplot, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$kwGrowthPlot <- renderPlotly({
    
    g <- WDynamics()
    
    leg <- list(
      orientation = 'h', 
      y = -0.15,
      font = list(
        family = "sans-serif",
        size = 10,
        color = "#000"),
      bgcolor = "#FFFFFF",
      bordercolor = "#FFFFFF",
      borderwidth = 2) 
    
    plot.ly(g, flip=FALSE, side="r", aspectratio=1.6, size=0.10) %>%
      layout(legend = leg) %>%
      config(displaylogo = FALSE,
             modeBarButtonsToRemove = c(
               'toImage',
               'sendDataToCloud',
               'pan2d', 
               'select2d', 
               'lasso2d',
               'toggleSpikelines',
               'hoverClosestCartesian',
               'hoverCompareCartesian'
             ))  %>%
      layout(hovermode = 'compare')
  })
  
  output$kwGrowthtable <- DT::renderDT({
    g <- WDynamics()
    kwData <- values$KW
    DTformat(kwData, nrow=10, filename="Word_Dynamics", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportWD,{
    if(!is.null(values$KW)){
      list_df <- list(values$KW)
      list_plot <- list(values$WDplot)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "WordFreqOverTime", wb=values$wb)
      values$wb <- wb
      popUp(title="Words' Frequency over Time", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Trend Topics ----
  
  observeEvent(input$TTStop,{
    values$TTremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$TTStop, sep=input$TTSep)))
    values$GenericSL <- values$TTremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$TTSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$TTSyn, sep=input$TTSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$TTsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$TTsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  output$trendSliderPY <- renderUI({
    
    sliderInput("trendSliderPY", "Timespan", min = min(values$M$PY,na.rm=T),sep="",
                max = max(values$M$PY,na.rm=T), value = c(min(values$M$PY,na.rm=T),max(values$M$PY,na.rm=T)))
  })
  
  TrendTopics <- eventReactive(input$applyTrendTopics,{
    
    ### load file with terms to remove
    if (input$TTStopFile=="Y"){
      remove.terms <- trimws(values$TTremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$TTremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$TTSynFile=="Y"){
      synonyms <- values$TTsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$TTsyn.terms <- synonyms
    ### end of block
    
    if (input$trendTerms %in% c("TI","AB")){
      values$M=termExtraction(values$M, Field = input$trendTerms, stemming = input$trendStemming, verbose = FALSE, ngrams=as.numeric(input$trendTermsngrams))
      field=paste(input$trendTerms,"_TM",sep="")
    } else {field=input$trendTerms}
    values$trendTopics <- fieldByYear(values$M, field = field, timespan = input$trendSliderPY, min.freq = input$trendMinFreq,
                                      n.items = input$trendNItems, remove.terms = remove.terms, synonyms = synonyms, 
                                      dynamic.plot=TRUE, graph = FALSE)
    values$trendTopics$params <- data.frame(description=c("Textual field", "N. of words per Year"),
                                            value=c(field, input$trendNItems))
    return(values$trendTopics$graph)
  })
  
  # gemini button for word network
  output$trendTopicsGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$trendTopicsGemini, values)
  })
  
  output$TTplot.save <- downloadHandler(
    filename = function() {
      paste("TrendTopics-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$trendTopics$graph, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$trendTopicsPlot <- renderPlotly({
    g <- TrendTopics()
    plot.ly(g, flip=TRUE, side="r", size=0.1, aspectratio=1.3)
  })
  
  output$trendTopicsTable <- DT::renderDT({
    TrendTopics()
    tpData=values$trendTopics$df_graph %>% 
      rename(Term = item,
             Frequency = freq,
             "Year (Q1)" = year_q1,
             "Year (Median)" = year_med,
             "Year (Q3)" = year_q3)
    DTformat(tpData, nrow=10, filename="TrendTopic", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportTT,{
    if(!is.null(values$trendTopics$df_graph)){
      list_df <- list(values$trendTopics$df_graph %>% 
                        rename(Term = item,
                               Frequency = freq,
                               "Year (Q1)" = year_q1,
                               "Year (Median)" = year_med,
                               "Year (Q3)" = year_q3))
      list_plot <- list(values$trendTopics$graph)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "TrendTopics", wb=values$wb)
      values$wb <- wb
      popUp(title="Trend Topics", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # CLUSTERING ----
  ### Clustering by Coupling ----
  CMMAP <- eventReactive(input$applyCM,{
    
    values$CM <- couplingMap(values$M, analysis=input$CManalysis, field=input$CMfield, 
                             n=input$CMn, minfreq=input$CMfreq,
                             ngrams=as.numeric(input$CMngrams),
                             community.repulsion = input$CMrepulsion,
                             impact.measure=input$CMimpact,
                             stemming=input$CMstemming, size=input$sizeCM, 
                             label.term = input$CMlabeling,
                             n.labels=input$CMn.labels, repel=FALSE)
    values$CM$data <- values$CM$data[,c(1,5,2)]
    values$CM$clusters <- values$CM$clusters[,c(7,1:4,6)]
    validate(
      need(values$CM$nclust > 0, "\n\nNo clusters in one or more periods. Please select a different set of parameters.")
    )
  })
  
  output$CMPlot <- renderPlotly({
    CMMAP()
    plot.ly(values$CM$map, size=0.15, aspectratio = 1.3)
  })
  
  output$CMNetPlot <- renderVisNetwork({
    CMMAP()
    values$networkCM<-igraph2vis(g=values$CM$net$graph,curved=(input$coc.curved=="Yes"), 
                                 labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                 shape=input$coc.shape, net=values$CM$net,shadow=TRUE)
    values$networkCM$VIS
  })
  
  output$CMplot.save <- downloadHandler(
    filename = function() {
      paste("CouplingMap-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$CM$map, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$CMTable <- DT::renderDT({
    CMMAP()
    #cmData=values$CM$data[,c(2,1,3,5)]
    cmData <- values$CM$data
    DTformat(cmData, nrow=10, filename="CouplingMap", pagelength=TRUE, left=NULL, right=NULL, numeric=2, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$CMTableCluster <- DT::renderDT({
    CMMAP()
    #cmData=values$CM$clusters[,c(7,1:4,6)]
    cmData <- values$CM$clusters
    DTformat(cmData, nrow=10, filename="CouplingMap_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=4:5, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportCM,{
    if(!is.null(values$CM$data)){
      popUp(title=NULL, type="waiting")
      list_df <- list(values$CM$params,
                      values$CM$data,
                      values$CM$clusters)
      list_plot <- list(values$CM$map,
                        values$CM$net$graph)
      wb <- addSheetToReport(list_df, list_plot, sheetname="CouplingMap", wb=values$wb)
      values$wb <- wb
      popUp(title="Coupling Map", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # CONCEPTUAL STRUCTURE ----
  ### Network approach ----
  #### Co-occurrences network ----
  observeEvent(input$COCStop,{
    values$COCremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$COCStop, sep=input$COCSep)))
    values$GenericSL <- values$COCremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$COCSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$COCSyn, sep=input$COCSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$COCsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$COCsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  COCnetwork <- eventReactive(input$applyCoc,{
    
    values <- cocNetwork(input,values)
    values$COCnetwork<-igraph2vis(g=values$cocnet$graph,curved=(input$coc.curved=="Yes"), 
                               labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                               shape=input$coc.shape, net=values$cocnet, shadow=(input$coc.shadow=="Yes"), edgesize=input$edgesize, noOverlap=input$noOverlap)
    values$cocOverlay <- overlayPlotly(values$COCnetwork$VIS)
    values$degreePlot <- degreePlot(values$cocnet)
  })
  
  output$cocPlot <- renderVisNetwork({  
    COCnetwork()
    values$COCnetwork$VIS
  })
  
  output$cocOverlay <- renderPlotly({
    COCnetwork()
    values$cocOverlay
  })
  
  # gemini button for word network
  output$cocGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$cocGemini, values)
  })
  
  output$network.coc <- downloadHandler(
    filename = function() {
      paste("Co_occurrence_network-", Sys.Date(), ".zip", sep="")
    },
    content <- function(file) {
      tmpdir <- tempdir()
      owd <- setwd(tmpdir)
      on.exit(setwd(owd))
      # print(tmpdir)
      #igraph::write.graph(values$obj$graph_pajek,file=file, format="pajek")
      myfile <- paste("mynetwork-", Sys.Date(), sep="")
      files <- paste0(myfile, c(".net",".vec",".clu"))
      graph2Pajek(values$cocnet$graph, filename=myfile)
      # print(files)
      # print(dir())
      zip::zip(file,files)
    },
    contentType = "zip"
  )
  
  ##### save coc network image as html ####
  output$networkCoc.fig <- downloadHandler(
    filename = "network.html",
    content <- function(con) {
      savenetwork(con, values$COCnetwork$VIS)
    },
    contentType = "html"
  )
  
  output$cocTable <- DT::renderDT({
    COCnetwork()
    cocData=values$cocnet$cluster_res
    names(cocData)=c("Node", "Cluster", "Betweenness", "Closeness", "PageRank")
    DTformat(cocData, nrow=10, filename="CoWord_Network", pagelength=TRUE, left=NULL, right=NULL, numeric=3:5, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  ### Degree Plot Co-word analysis ----
  output$cocDegree <- renderPlotly({
    COCnetwork()
    #values$degreePlot <- degreePlot(values$cocnet)
    plot.ly(values$degreePlot)
  })
  
  observeEvent(input$reportCOC,{
    if(!is.null(values$cocnet$cluster_res)){
      names(values$cocnet$cluster_res)=c("Node", "Cluster", "Betweenness", "Closeness", "PageRank")
      sheetname <- "CoWordNet"
      list_df <- list(values$cocnet$cluster_res)
      list_plot <- list(values$degreePlot)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      #values$wb <- res$wb
      values$wb <- addGgplotsWb(list_plot, wb=res$wb, res$sheetname, col=res$col+16, width=10, height=7, dpi=75)
      #values$fileTFP <- screenSh(selector = "#cocPlot") ## screenshot
      values$fileCOC <- screenSh(values$COCnetwork$VIS, zoom = 2, type="vis")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileCOC,res$col))
      popUp(title="Co-occurrence Network", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Correspondence Analysis ----
  observeEvent(input$CSStop,{
    values$CSremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$CSStop, sep=input$CSSep)))
    values$GenericSL <- values$CSremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$FASyn,{
    synonyms <- trimws(readSynWordsFile(file=input$FASyn, sep=input$FASynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$FAsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$FAsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  CSfactorial <- eventReactive(input$applyCA,{
    values <- CAmap(input,values)
    values$plotCS <- ca2plotly(values$CS, method=input$method ,dimX = 1, dimY = 2, topWordPlot = Inf, threshold=0.05, labelsize = input$CSlabelsize*2, size=input$CSlabelsize*1.5)
    values$dendCS <- dend2vis(values$CS$km.res, labelsize=input$CSlabelsize, nclusters=as.numeric(input$nClustersCS), community=FALSE)
  })
  
  # gemini button for correspondence analysis
  output$CSGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$CSGemini, values)
  })
  
  output$FAplot.save <- downloadHandler(
    filename = function() {
      #
      paste("FactorialAnalysis_", Sys.Date(), ".zip", sep="")
    },
    content <- function(file) {
      #go to a temp dir to avoid permission issues
      owd <- setwd(tempdir())
      on.exit(setwd(owd))
      files <- c(paste("FactorialMap_", Sys.Date(), ".png", sep=""),
                 paste("Dendrogram_", Sys.Date(), ".png", sep="")
                 #paste("MostContribDocuments_", Sys.Date(), ".png", sep=""),
                 #paste("MostCitedDocuments_", Sys.Date(), ".png", sep="")
      )
      ggsave(filename = files[1], plot = values$CS$graph_terms, dpi = values$dpi, height = values$h, width = values$h*1.5, bg="white")
      png(filename = files[2],  height = values$h, width = values$h*2, units="in", res = values$dpi)
          plot(values$CS$graph_dendogram)
      dev.off()
      zip::zip(file,files)
    },
    contentType = "zip"
  )

  output$CSPlot1 <- renderPlotly({
    CSfactorial()
    #CS=values$CS
    #save(CS,file="provaCS.rdata")
    values$plotCS #<- ca2plotly(values$CS, method=input$method ,dimX = 1, dimY = 2, topWordPlot = Inf, threshold=0.05, labelsize = input$CSlabelsize*2, size=input$CSlabelsize*1.5)
  })
  

  output$CSPlot4 <- renderVisNetwork({
    CSfactorial()
      #dend2vis(values$CS$km.res, labelsize=input$CSlabelsize, nclusters=as.numeric(input$nClustersCS), community=FALSE)
      values$dendCS
      #values$CS$graph_dendogram)
  })
  
  output$CSTableW <- DT::renderDT({
    CSfactorial()
    WData <- values$CS$WData
    DTformat(WData, nrow=10, filename="CoWord_Factorial_Analysis_Words_By_Cluster", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$CSTableD <- DT::renderDT({
    CSfactorial()
    CSData <- values$CS$CSData
    DTformat(CSData, nrow=10, filename="CoWord_Factorial_Analysis_Articles_By_Cluster", pagelength=TRUE, left=NULL, right=NULL, numeric=2:4, dom=FALSE, 
             size='100%', filter="none", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  # add to report
  observeEvent(input$reportFA,{
    if(!is.null(values$CS$params)){
      list_df <- list(values$CS$params, values$CS$WData, values$CS$CSData)
      list_plot <- list(values$CS$graph_terms, values$CS$graph_dendogram)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "FactorialAnalysis", wb=values$wb)
      values$wb <- wb
      popUp(title="Factorial Analysis", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Thematic Map ----
  observeEvent(input$TMStop,{
    values$TMremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$TMStop, sep=input$TMSep)))
    values$GenericSL <- values$TMremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$TMapSyn,{
    synonyms <- trimws(readSynWordsFile(file=input$TMapSyn, sep=input$TMapSynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$TMapsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$TMapsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  TMAP <- eventReactive(input$applyTM,{
    if (input$TMfield %in% c("TI","AB")){
      ngrams <- as.numeric(input$TMngrams)
    }else{
      ngrams <- 1
    }
    
    ### load file with terms to remove
    if (input$TMStopFile=="Y"){
      remove.terms <- trimws(values$TMremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$TMremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$TMapSynFile=="Y"){
      synonyms <- values$TMapsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$TMapsyn.terms <- synonyms
    ### end of block
    
    values$TM <- thematicMap(values$M, field=input$TMfield, 
                             n=input$TMn, minfreq=input$TMfreq, ngrams=ngrams,
                             community.repulsion = input$TMrepulsion,
                             stemming=input$TMstemming, size=input$sizeTM, cluster=input$TMCluster,
                             n.labels=input$TMn.labels, repel=FALSE, remove.terms=remove.terms, synonyms=synonyms,
                             subgraphs=TRUE)
    values$TM$doc2clust <- values$TM$documentToClusters %>% select(Assigned_cluster, SR, pagerank) 
    
    values$TM$documentToClusters$DI<- paste0('<a href=\"https://doi.org/',values$TM$documentToClusters$DI,'\" target=\"_blank\">',values$TM$documentToClusters$DI,'</a>')
    names(values$TM$documentToClusters)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR") 
    
    values$TM$words <- values$TM$words[,-c(4,6)]
    values$TM$clusters_orig <- values$TM$clusters
    values$TM$clusters <- values$TM$clusters[,c(9,5:8,11)]
    names(values$TM$clusters) <- c("Cluster", "CallonCentrality","CallonDensity","RankCentrality","RankDensity","ClusterFrequency") 
    values$TMmap <- plot.ly(values$TM$map, size=0.07, aspectratio = 1.3, customdata=values$TM$clusters$color)
    validate(
      need(values$TM$nclust > 0, "\n\nNo topics in one or more periods. Please select a different set of parameters.")
    )
  })
  output$TMPlot <- renderPlotly({
    TMAP()
    values$TMmap
  })
  
  # gemini button for Thematic Map
  output$TMGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$TMGemini, values)
  })
  
  ### click cluster networks
  
  plotModal <- function(session) {
    ns <- session$ns
    modalDialog(
      h3(strong(("Cluster Network"))),
      visNetworkOutput(ns("cocPlotClust")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(label="Save", inputId = "cocPlotClust",
                     icon = icon("camera", lib = "glyphicon")),
        modalButton("Close")),
    )
  }
  
  observeEvent(input$cocPlotClust,{
    #Time <- format(Sys.time(),'%H%M%S')
    filename = paste("TMClusterGraph-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$plotClust , filename=filename, type="vis")
  })
  
  observeEvent(event_data("plotly_click"), {
    if (input$sidebarmenu=="thematicMap"){
      showModal(plotModal(session))
    }
  })
  
  output$cocPlotClust <- renderVisNetwork({
    values$d <- event_data("plotly_click")
    coord <- values$d[c("x","y")]
    color <- values$TM$clusters_orig %>% 
      filter(rcentrality==coord$x,rdensity==coord$y) %>% 
      select(color) %>% as.character()
    g <- values$TM$subgraphs[[color]]
    values$plotClust <- igraph2visClust(g,curved=F,labelsize=4,opacity=0.5,shape="dot", shadow=TRUE, edgesize=5)$VIS
    values$plotClust 
  })
  
  ### end click cluster subgraphs
  
  output$NetPlot <- renderVisNetwork({
    TMAP()
    values$networkTM<-igraph2vis(g=values$TM$net$graph,curved=(input$coc.curved=="Yes"), 
                                 labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                 shape=input$coc.shape, net=values$TM$net, noOverlap=input$noOverlapTM)
    values$networkTM$VIS
  })
  
  
  output$TMplot.save <- downloadHandler(
    filename = function() {
      paste("ThematicMap-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      ggsave(filename = file, plot = values$TM$map, dpi = values$dpi, height = values$h, width = values$h*1.5, bg="white")
    },
    contentType = "png"
  )
  
  output$TMTable <- DT::renderDT({
    TMAP()
    tmData=values$TM$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=FALSE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableCluster <- DT::renderDT({
    TMAP()
    tmData <- values$TM$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableDocument <- DT::renderDT({
    TMAP()
    tmDataDoc <- values$TM$documentToClusters
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportTM,{
    if(!is.null(values$TM$words)){
      popUp(title=NULL, type="waiting")
      list_df <- list(values$TM$params,
                      values$TM$words,
                      values$TM$clusters,
                      values$TM$documentToClusters)
      list_plot <- list(values$TM$map,
                        values$TM$net$graph)
      wb <- addSheetToReport(list_df, list_plot, sheetname="ThematicMap", wb=values$wb)
      values$wb <- wb
      popUp(title="Thematic Map", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Thematic Evolution ----
  observeEvent(input$TEStop,{
    values$TEremove.terms <- data.frame(stopword=trimws(readStopwordsFile(file=input$TEStop, sep=input$TESep)))
    values$GenericSL <- values$TEremove.terms
    popUpGeneric(title="Stopword list", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("stopwordList"),
                 btn_labels="OK")
    
  })
  
  observeEvent(input$TESyn,{
    synonyms <- trimws(readSynWordsFile(file=input$TESyn, sep=input$TESynSep))
    term <- unlist(lapply(strsplit(synonyms,";"),function(l){l[1]}))
    synList <- unlist(lapply(strsplit(synonyms,";"),function(l){
      paste0(trimws(l[-1]),collapse=";")
    }))
    values$TEsyn.terms <- data.frame(term=term, synonyms=synList)
    values$GenericSYN <- values$TEsyn.terms
    popUpGeneric(title="Synonym List", 
                 type=NULL, 
                 color=c("#1d8fe1"),
                 subtitle=DTOutput("synonymList"),
                 btn_labels="OK")
    
  })
  
  output$sliders <- renderUI({
    numSlices <- as.integer(input$numSlices)
    v=quantile(values$M$PY, seq(0,1,by=(1/(numSlices+1))), na.rm=TRUE)
    v=round(v[-c(1,length(v))],0)
    lapply(1:numSlices, function(i) {
      numericInput(inputId = paste0("Slice", i), label = paste("Cutting Year", i),value=v[i],min=min(values$M$PY, na.rm = TRUE)+1,max=max(values$M$PY, na.rm = TRUE)-1, step=1)
    })
  })
  
  TEMAP <- eventReactive(input$applyTE,{
    if (input$TEfield %in% c("TI","AB")){
      ngrams <- as.numeric(input$TEngrams)
    }else{
      ngrams <- 1
    }
    
    ### load file with terms to remove
    if (input$TEStopFile=="Y"){
      remove.terms <- trimws(values$TEremove.terms$stopword)
    }else{remove.terms <- NULL}
    #values$TEremove.terms <- remove.terms
    ### end of block
    
    ### load file with synonyms
    if (input$TESynFile=="Y"){
      synonyms <- values$TEsyn.terms %>% group_by(term) %>% mutate(term=paste0(term,";",synonyms)) %>% select(term)
      synonyms <- synonyms$term
    }else{synonyms <- NULL}
    #values$TEsyn.terms <- synonyms
    ### end of block
    
    values$yearSlices <- as.numeric()
    if (is.null(input$numSlices)){
      values$yearSlices <- median(values$M$PY, na.rm=TRUE)
    }else{
      for (i in 1:as.integer(input$numSlices)){
        if (length(input[[paste0("Slice", i)]])>0){values$yearSlices <- c(values$yearSlices,input[[paste0("Slice", i)]])}
      }
    }
    
    if (length(values$yearSlices)>0){
      values$nexus <- thematicEvolution(values$M, field=input$TEfield, values$yearSlices, n = input$nTE, minFreq = input$fTE, size = input$sizeTE, 
                                        cluster=input$TECluster,
                                        n.labels=input$TEn.labels, repel=FALSE, ngrams=ngrams, remove.terms = remove.terms, synonyms = synonyms)
      validate(
        need(values$nexus$check != FALSE, "\n\nNo topics in one or more periods. Please select a different set of parameters.")
      )
      for (i in 1:(length(values$yearSlices)+1)){
        values$nexus$TM[[i]]$words <- values$nexus$TM[[i]]$words[,-c(4,6)]
        values$nexus$TM[[i]]$clusters <- values$nexus$TM[[i]]$clusters[,c(9,5:8,11)]
        names(values$nexus$TM[[i]]$clusters) <- c("Cluster", "CallonCentrality","CallonDensity","RankCentrality","RankDensity","ClusterFrequency")
        
        values$nexus$TM[[i]]$documentToClusters$DI<- paste0('<a href=\"https://doi.org/',values$nexus$TM[[i]]$documentToClusters$DI,'\" target=\"_blank\">',values$nexus$TM[[i]]$documentToClusters$DI,'</a>')
        names(values$nexus$TM[[i]]$documentToClusters)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR")
      }
      values$nexus$Data <- values$nexus$Data[values$nexus$Data$Inc_index>0,-c(4,8)]
      values$TEplot <- plotThematicEvolution(Nodes = values$nexus$Nodes,Edges = values$nexus$Edges, measure = input$TEmeasure, min.flow = input$minFlowTE, label_size = input$sizeTE*20)
    }
  })
  
  output$TEPlot <- visNetwork::renderVisNetwork({
    TEMAP()
    values$TEplot
  })
  
  # gemini button for word network
  output$TEGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$TEGemini, values)
  })
 
  session$onFlushed(function() {
    shinyjs::runjs("$('#TEPlot').trigger('resize');")
  })
  
  output$TEplot.save <- downloadHandler(
    filename = function() {
      #
      paste("ThematicEvolution-", Sys.Date(), ".zip", sep="")
    },
    content <- function(file) {
      #go to a temp dir to avoid permission issues
      tmpdir <- tempdir()
      owd <- setwd(tmpdir)
      on.exit(setwd(owd))
      files <- filenameTE <- paste("ThematicEvolution_", Sys.Date(), ".png", sep="")
      
      for (i in 1:length(values$nexus$TM)){
        fileName <- paste("ThematicEvolution-Map_",i,"_",Sys.Date(), ".png", sep="")
        ggsave(filename = fileName, plot = values$nexus$TM[[i]]$map, dpi = values$dpi, height = values$h, width = values$h*1.5, bg="white")
        files <- c(fileName,files)
      }
      plot2png(values$TEplot, filename= filenameTE, 
               zoom = 2, type="vis", tmpdir=tmpdir)
      zip::zip(file,files)
    },
    contentType = "zip"
  )
  
  output$TETable <- DT::renderDT({
    TEMAP()
    TEData=values$nexus$Data
    names(TEData)=c("From", "To", "Words", "Weighted Inclusion Index", "Inclusion Index", "Occurrences", "Stability Index")
    DTformat(TEData, nrow=10, filename="Thematic_Evolution", pagelength=TRUE, left=NULL, right=NULL, numeric=c(4,5,7), dom=TRUE, 
             size='85%', filter="top", columnShort=NULL, columnSmall=NULL, round=2, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMPlot1 <-  renderPlotly({
    TEMAP()
    if (length(values$nexus$TM)>=1){
      plot.ly(values$nexus$TM[[1]]$map, size=0.07, aspectratio = 1.3)
    } else {emptyPlot("You have selected fewer periods!")}
  })
  
  output$TMPlot2 <-  renderPlotly({
    TEMAP()
    if (length(values$nexus$TM)>=2){
      plot.ly(values$nexus$TM[[2]]$map, size=0.07, aspectratio = 1.3)
    } else {emptyPlot("You have selected fewer periods!")}
  })
  
  output$TMPlot3 <-  renderPlotly({
    TEMAP()
    if (length(values$nexus$TM)>=3){
      plot.ly(values$nexus$TM[[3]]$map, size=0.07, aspectratio = 1.3)
    } else {emptyPlot("You have selected fewer periods!")}
  })
  
  output$TMPlot4 <-  renderPlotly({
    TEMAP()
    if (length(values$nexus$TM)>=4){
      plot.ly(values$nexus$TM[[4]]$map, size=0.07, aspectratio = 1.3)
    } else (emptyPlot("You have selected fewer periods!"))
  })
  
  output$TMPlot5 <-  renderPlotly({
    TEMAP()
    if (length(values$nexus$TM)>=5){
      plot.ly(values$nexus$TM[[5]]$map, size=0.07, aspectratio = 1.3)
    } else (emptyPlot("You have selected fewer periods!"))
  })
  
  output$NetPlot1 <- renderVisNetwork({
    TEMAP()
    k=1
    values$network1<-igraph2vis(g=values$nexus$Net[[k]]$graph,curved=(input$coc.curved=="Yes"), 
                                labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                shape=input$coc.shape, net=values$nexus$Net[[k]], noOverlap=input$noOverlapTE)
    values$network1$VIS
  })
  
  output$NetPlot2 <- renderVisNetwork({
    TEMAP()
    k=2
    values$network2<-igraph2vis(g=values$nexus$Net[[k]]$graph,curved=(input$coc.curved=="Yes"), 
                                labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                shape=input$coc.shape, net=values$nexus$Net[[k]], noOverlap=input$noOverlapTE)
    values$network2$VIS
  })
  
  output$NetPlot3 <- renderVisNetwork({
    TEMAP()
    k=3
    values$network3<-igraph2vis(g=values$nexus$Net[[k]]$graph,curved=(input$coc.curved=="Yes"), 
                                labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                shape=input$coc.shape, net=values$nexus$Net[[k]], noOverlap=input$noOverlapTE)
    values$network3$VIS
  })
  
  output$NetPlot4 <- renderVisNetwork({
    TEMAP()
    k=4
    values$network4<-igraph2vis(g=values$nexus$Net[[k]]$graph,curved=(input$coc.curved=="Yes"), 
                                labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                shape=input$coc.shape, net=values$nexus$Net[[k]], noOverlap=input$noOverlapTE)
    values$network4$VIS
  })
  
  output$NetPlot5 <- renderVisNetwork({
    TEMAP()
    k=5
    values$network5<-igraph2vis(g=values$nexus$Net[[k]]$graph,curved=(input$coc.curved=="Yes"), 
                                labelsize=input$labelsize, opacity=input$cocAlpha,type=input$layout,
                                shape=input$coc.shape, net=values$nexus$Net[[k]], noOverlap=input$noOverlapTE)
    values$network5$VIS
  })
  
  output$TMTable1 <- DT::renderDT({
    TEMAP()
    tmData=values$nexus$TM[[1]]$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_1_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTable2 <- DT::renderDT({
    TEMAP()
    tmData=values$nexus$TM[[2]]$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_2_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTable3 <- DT::renderDT({
    TEMAP()
    tmData=values$nexus$TM[[3]]$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_3_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTable4 <- DT::renderDT({
    TEMAP()
    tmData=values$nexus$TM[[4]]$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_4_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTable5 <- DT::renderDT({
    TEMAP()
    tmData=values$nexus$TM[[5]]$words
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_5_Terms", pagelength=TRUE, left=NULL, right=NULL, numeric=5:7, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableCluster1 <- DT::renderDT({
    TEMAP()
    tmData <- values$nexus$TM[[1]]$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_1_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableCluster2 <- DT::renderDT({
    TEMAP()
    tmData <- values$nexus$TM[[2]]$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_2_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableCluster3 <- DT::renderDT({
    TEMAP()
    tmData <- values$nexus$TM[[3]]$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_3_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableCluster4 <- DT::renderDT({
    TEMAP()
    tmData <- values$nexus$TM[[4]]$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_4_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE) 
  })
  
  output$TMTableCluster5 <- DT::renderDT({
    TEMAP()
    tmData <- values$nexus$TM[[5]]$clusters
    DTformat(tmData, nrow=10, filename="Thematic_Map_Period_5_Clusters", pagelength=TRUE, left=NULL, right=NULL, numeric=2:3, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE) 
  })
  
  output$TMTableDocument1 <- DT::renderDT(server=TRUE,{
    TEMAP()
    tmDataDoc <- values$nexus$TM[[1]]$documentToClusters
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Period_1_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableDocument2 <- DT::renderDT({
    TEMAP()
    tmDataDoc <- values$nexus$TM[[2]]$documentToClusters
    tmDataDoc$DI<- paste0('<a href=\"https://doi.org/',tmDataDoc$DI,'\" target=\"_blank\">',tmDataDoc$DI,'</a>')
    names(tmDataDoc)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR") 
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Period_2_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableDocument3 <- DT::renderDT({
    TEMAP()
    tmDataDoc <- values$nexus$TM[[3]]$documentToClusters
    tmDataDoc$DI<- paste0('<a href=\"https://doi.org/',tmDataDoc$DI,'\" target=\"_blank\">',tmDataDoc$DI,'</a>')
    names(tmDataDoc)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR") 
    
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Period_3_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableDocument4 <- DT::renderDT({
    TEMAP()
    tmDataDoc <- values$nexus$TM[[4]]$documentToClusters
    tmDataDoc$DI<- paste0('<a href=\"https://doi.org/',tmDataDoc$DI,'\" target=\"_blank\">',tmDataDoc$DI,'</a>')
    names(tmDataDoc)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR") 
    
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Period_4_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  output$TMTableDocument5 <- DT::renderDT({
    TEMAP()
    tmDataDoc <- values$nexus$TM[[5]]$documentToClusters
    tmDataDoc$DI<- paste0('<a href=\"https://doi.org/',tmDataDoc$DI,'\" target=\"_blank\">',tmDataDoc$DI,'</a>')
    names(tmDataDoc)[1:9] <- c("DOI", "Authors","Title","Source","Year","TotalCitation","TCperYear","NTC","SR") 
    
    DTformat(tmDataDoc, nrow=10, filename="Thematic_Map_Period_5_Documents", pagelength=TRUE, left=NULL, right=NULL, numeric=c(7:8,10:(ncol(tmDataDoc)-2),ncol(tmDataDoc)), dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  observeEvent(input$reportTE,{
    if(!is.null(values$nexus$Data)){
      popUp(title=NULL, type="waiting")
      sheetname <- "ThematicEvolution"
      list_df <- list(values$nexus$params, values$nexus$Data)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      #values$wb <- res$wb
      #values$fileTFP <- screenSh(selector = "#TEPlot") ## screenshot
      values$fileTEplot <- screenSh(values$TEplot, zoom = 2, type="plotly")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileTEplot,res$col))
      
      ## Periods
      L <- length(values$nexus$TM)
      wb <- res$wb
      for (l in 1:L){
        if(!is.null(values$nexus$TM[[l]]$words)){
          
          list_df <- list(values$nexus$TM[[l]]$params,
                          values$nexus$TM[[l]]$words,
                          values$nexus$TM[[l]]$clusters,
                          values$nexus$TM[[l]]$documentToClusters)
          list_plot <- list(values$nexus$TM[[l]]$map,
                            values$nexus$TM[[l]]$net$graph)
          wb <- addSheetToReport(list_df, list_plot, sheetname=paste("TE_Period_",l,sep=""), wb=wb)
          #
        }
      }
      values$wb <- wb
      popUp(title="Thematic Evolution", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # INTELLECTUAL STRUCTURE ####
  ### Co-citation network ----
  COCITnetwork <- eventReactive(input$applyCocit,{
    values <- intellectualStructure(input,values)
    values$COCITnetwork<-igraph2vis(g=values$cocitnet$graph,curved=(input$cocit.curved=="Yes"), 
                               labelsize=input$citlabelsize, opacity=0.7,type=input$citlayout,
                               shape=input$cocit.shape, net=values$cocitnet, shadow=(input$cocit.shadow=="Yes"),
                               noOverlap=input$citNoOverlap)
    values$cocitOverlay <- overlayPlotly(values$COCITnetwork$VIS)
    values$degreePlot <- degreePlot(values$cocitnet)
  })
  
  output$cocitPlot <- renderVisNetwork({  
    COCITnetwork()
    isolate(values$COCITnetwork$VIS)
  })
  
  output$cocitOverlay <- renderPlotly({
    COCITnetwork()
    values$cocitOverlay
  })
  
  # gemini button for co-citation network
  output$cocitGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$cocitGemini, values)
  })
  
  output$network.cocit <- downloadHandler(
    filename = function() {
      paste("Co_citation_network-", Sys.Date(), ".zip", sep="")
    },
    content <- function(file) {
      tmpdir <- tempdir()
      owd <- setwd(tmpdir)
      on.exit(setwd(owd))
      # print(tmpdir)
      #igraph::write.graph(values$obj$graph_pajek,file=file, format="pajek")
      myfile <- paste("mynetwork-", Sys.Date(), sep="")
      files <- paste0(myfile, c(".net",".vec",".clu"))
      graph2Pajek(values$cocitnet$graph, filename=myfile)
      # print(files)
      # print(dir())
      zip::zip(file,files)
    },
    contentType = "zip"
  )
  
  output$cocitTable <- DT::renderDT({
    COCITnetwork()
    cocitData=values$cocitnet$cluster_res
    names(cocitData)=c("Node", "Cluster", "Betweenness", "Closeness", "PageRank")
    DTformat(cocitData, nrow=10, filename="CoCitation_Network", pagelength=TRUE, left=NULL, right=NULL, numeric=3:5, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  #### save coc network image as html ----
  output$networkCocit.fig <- downloadHandler(
    filename = "network.html",
    content <- function(con) {
      savenetwork(con, values$COCITnetwork$VIS)
    },
    contentType = "html"
  )
  
  ### Degree Plot Co-citation analysis ####
  output$cocitDegree <- renderPlotly({
    COCITnetwork()
    #p <- degreePlot(values$cocitnet)
    plot.ly(values$degreePlot)
  })
  
  observeEvent(input$reportCOCIT,{
    
    if(!is.null(values$cocitnet$cluster_res)){
      names(values$cocitnet$cluster_res) <- c("Node", "Cluster", "Betweenness", "Closeness", "PageRank")
      sheetname <- "CoCitNet"
      list_df <- list(values$cocitnet$cluster_res)
      list_plot <- list(values$degreePlot)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      #values$wb <- res$wb
      values$wb <- addGgplotsWb(list_plot, wb=res$wb, res$sheetname, col=res$col+15, width=12, height=8, dpi=75)
      #values$fileTFP <- screenSh(selector = "#cocitPlot") ## screenshot
      values$fileCOCIT <- screenSh(values$COCITnetwork$VIS, zoom = 2, type="vis")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileCOCIT,res$col))
      popUp(title="Co-citation Network", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### Historiograph ----
  Hist <- eventReactive(input$applyHist,{
    withProgress(message = 'Calculation in progress',
                 value = 0, {
                   values <- historiograph(input,values)
                 })
  })
  
  output$histPlotVis <- renderVisNetwork({  
    g <- Hist()
    values$histPlotVis<-hist2vis(values$histPlot,curved=TRUE, 
                                 labelsize=input$histlabelsize, 
                                 nodesize=input$histsize,
                                 opacity=0.7,
                                 shape="dot",
                                 labeltype=input$titlelabel,
                                 timeline=FALSE)
    values$histPlotVis$VIS
  })
  
  output$histTable <- DT::renderDT({
    g <- Hist()
    Data <- values$histResults$histData
    DTformat(Data, nrow=10, filename="Historiograph_Network", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })
  
  # gemini button for word network
  output$histGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$histGemini, values)
  })
  
 observeEvent(input$reportHIST,{
    if(!is.null(values$histResults$histData)){
      sheetname <- "Historiograph"
      list_df <- list(values$histResults$histData)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      #values$fileTFP <- screenSh(selector = "#histPlotVis") ## screenshot
      values$fileHIST <- screenSh(values$histPlotVis$VIS, zoom = 2, type="vis")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileHIST,res$col))
      popUp(title="Historiograph", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # SOCIAL STRUCTURE ####
  ### Collaboration network ----
  COLnetwork <- eventReactive(input$applyCol,{
    values <- socialStructure(input,values)
    values$COLnetwork<-igraph2vis(g=values$colnet$graph,curved=(input$soc.curved=="Yes"), 
                               labelsize=input$collabelsize, opacity=input$colAlpha,type=input$collayout,
                               shape=input$col.shape, net=values$colnet, shadow=(input$col.shadow=="Yes"),
                               noOverlap=input$colNoOverlap)
    values$colOverlay <- overlayPlotly(values$COLnetwork$VIS)
    values$degreePlot <-degreePlot(values$colnet)
    if (is.null(dim(values$colnet$cluster_res))){
      values$colnet$cluster_res <- data.frame(Node=NA, Cluster=NA, Betweenness=NA, 
                                                                             Closeness = NA, PageRank = NA)
    }else{
      names(values$colnet$cluster_res) <- c("Node", "Cluster", "Betweenness", "Closeness", "PageRank")
    }
    
  })
  output$colPlot <- renderVisNetwork({  
    COLnetwork()
    values$COLnetwork$VIS
  })
  
  output$colOverlay <- renderPlotly({
    COLnetwork()
    values$colOverlay
  })
  
  # gemini button for word network
  output$colGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$colGemini, values)
  })

   output$network.col <- downloadHandler(
    filename = function() {
      paste("Collaboration_network-", Sys.Date(), ".zip", sep="")
    },
    content <- function(file) {
      tmpdir <- tempdir()
      owd <- setwd(tmpdir)
      on.exit(setwd(owd))
      # print(tmpdir)
      #igraph::write.graph(values$obj$graph_pajek,file=file, format="pajek")
      myfile <- paste("mynetwork-", Sys.Date(), sep="")
      files <- paste0(myfile, c(".net",".vec",".clu"))
      graph2Pajek(values$colnet$graph, filename=myfile)
      # print(files)
      # print(dir())
      zip::zip(file,files)
    },
    contentType = "zip"
  )
  
  output$colTable <- DT::renderDT({
    COLnetwork()
    colData=values$colnet$cluster_res
    DTformat(colData, nrow=10, filename="Collaboration_Network", pagelength=TRUE, left=NULL, right=NULL, numeric=3:5, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  })   
  
  #### save coc network image as html ####
  output$networkCol.fig <- downloadHandler(
    filename = "network.html",
    content <- function(con) {
      savenetwork(con, values$COLnetwork$VIS)
    },
    contentType = "html"
  )
  
  ### Degree Plot Collaboration analysis ####
  output$colDegree <- renderPlotly({
    COLnetwork()
    p <- degreePlot(values$colnet)
    plot.ly(p)
  })
  
  observeEvent(input$reportCOL,{
    if(!is.null(values$colnet$cluster_res)){
      sheetname <- "CollabNet"
      list_df <- list(values$colnet$params, values$colnet$cluster_res)
      list_plot <- list(values$degreePlot)
      res <- addDataScreenWb(list_df, wb=values$wb, sheetname=sheetname)
      values$wb <- addGgplotsWb(list_plot, wb=res$wb, res$sheetname, col=res$col+15, width=12, height=8, dpi=75)
      #values$fileTFP <- screenSh(selector = "#colPlot") ## screenshot
      values$fileCOL <- screenSh(values$COLnetwork$VIS, zoom = 2, type="vis")
      values$list_file <- rbind(values$list_file, c(sheetname=res$sheetname,values$fileCOL,res$col))
      popUp(title="Collaboration Network", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  ### WPPlot ----
  WMnetwork<- eventReactive(input$applyWM,{
    values$WMmap <- countrycollaboration(values$M,label=FALSE,edgesize=input$WMedgesize/2,min.edges=input$WMedges.min, values)
    values$WMmap$tab <- values$WMmap$tab[,c(1,2,9)]
    names(values$WMmap$tab)=c("From","To","Frequency")
    
  })
  
  # gemini button for World Map
  output$WMGeminiUI <- renderUI({
    values$gemini_model_parameters <- geminiParameterPrompt(values, input$sidebarmenu, input)
    geminiOutput(title = "", content = values$WMGemini, values)
  })
 
  output$CCplot.save <- downloadHandler(
    filename = function() {
      paste("CountryCollaborationMap-", Sys.Date(), ".png", sep="")
    },
    content <- function(file) {
      g <- values$WMmap$g + labs(title = "Country Collaboration Map")
      ggsave(filename = file, plot = g, dpi = values$dpi, height = values$h, width = values$h*2, bg="white")
    },
    contentType = "png"
  )
  
  output$WMPlot<- renderPlotly({
    WMnetwork()  
    plot.ly(values$WMmap$g,flip=FALSE, side="r", aspectratio=1.7, size=0.07, data.type=1,height=15)
    #plot(values$WMmap$g)
  })
  
  output$WMTable <- DT::renderDT({
    WMnetwork()  
    colData=values$WMmap$tab
    DTformat(colData, nrow=10, filename="Collaboration_WorldMap", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=TRUE, 
             size='100%', filter="top", columnShort=NULL, columnSmall=NULL, round=3, title="", button=TRUE, escape=FALSE, 
             selection=FALSE)
  }) 
  
  observeEvent(input$reportCOLW,{
    if(!is.null(values$WMmap$tab)){
      list_df <- list(values$WMmap$tab)
      list_plot <- list(values$WMmap$g)
      wb <- addSheetToReport(list_df,list_plot,sheetname = "CollabWorldMap", wb=values$wb)
      values$wb <- wb
      popUp(title="Countries' Collaboration World Map", type="success")
      values$myChoices <- sheets(values$wb)
    } else {
      popUp(type="error")
    }
  })
  
  # REPORT ----
  ### Report Save xlsx ----
  output$report.save <- downloadHandler(
    filename = function() {
      paste("BiblioshinyReport-", Sys.Date(), ".xlsx", sep="")
    },
    content <- function(file) {
      wb_export <- copyWorkbook(values$wb)
      if (nrow(values$list_file)>0){
        wb_export <- addScreenWb(df=values$list_file, wb=wb_export)#, width=10, height=7, dpi=300)
      }
      sheetToRemove <- setdiff(sheets(wb_export),input$reportSheets)
      if (length(sheetToRemove)>0) for (i in sheetToRemove) removeWorksheet(wb_export,i)
      sheetToAdd <- sheets(wb_export)
      for (i in sheetToAdd) setColWidths(wb_export,sheet=i,cols=1,widths = 30, hidden = FALSE)
      openxlsx::saveWorkbook(wb_export, file = file)
    },
    contentType = "xlsx"
  )
  
  ### Report UI elements
  observe({
    output$reportSheets <- renderUI({
      prettyCheckboxGroup(
        inputId = "reportSheets",
        label = NULL, #short2long(df=values$dfLabel, myC=values$myChoices),
        choices = short2long(df=values$dfLabel, myC=values$myChoices),
        selected = values$myChoices,
        icon = icon("check"),
        animation = "pulse",
        status = "primary",
        bigger = T,
        fill = TRUE
      )
    })
  })
  
  observe({
    updatePrettyCheckboxGroup(
      session = getDefaultReactiveDomain(), 
      inputId = "reportSheets",
      #label = short2long(df=values$dfLabel, myC=values$myChoices),
      choices = short2long(df=values$dfLabel, myC=values$myChoices),
      selected = if(!input$noSheets) values$myChoices,
      prettyOptions = list(
        animation = "pulse",
        status = "info",
        bigger = T
      )
    )
  })
  
  observe({
    updatePrettyCheckboxGroup(
      session = getDefaultReactiveDomain(), 
      inputId = "reportSheets",
      choices = short2long(df=values$dfLabel, myC=values$myChoices),
      selected = if(input$allSheets) values$myChoices,
      prettyOptions = list(
        animation = "pulse",
        status = "info",
        bigger = T
      )
    )
  })
  
  observeEvent(input$deleteAll, {
    ask_confirmation(
      inputId = "delete_confirmation",
      title = "Want to confirm?",
      text = "All the results will be removed from the report",
      type = "warning",
      btn_labels = c("CANCEL", "CONFIRM"),
    )
  })
  
  observeEvent(input$delete_confirmation, {
    if (isTRUE(input$delete_confirmation)) {
      values$myChoices <- "Empty Report"
      values$list_file <- data.frame(sheet=NULL,file=NULL,n=NULL) 
      values$wb <-  openxlsx::createWorkbook()
    }
  }, ignoreNULL = TRUE
  )
  
  ### screenshot buttons ----
  observeEvent(input$screenTFP,{
    filename = paste("ThreeFieldPlot-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$TFP, filename=filename, type="plotly")
  })
  
  observeEvent(input$screenWC,{
    filename = paste("WordCloud-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$WordCloud, filename=filename, type="plotly")
  })
  
  observeEvent(input$screenTREEMAP,{
    filename = paste("TreeMap-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$TreeMap, filename=filename, type="plotly")
  })
  
  observeEvent(input$screenCOC,{
    filename = paste("Co_occurrenceNetwork-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$COCnetwork$VIS, filename=filename, type="vis")
  })
  
  observeEvent(input$screenCOCIT,{
    filename = paste("Co_citationNetwork-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$COCITnetwork$VIS, filename=filename, type="vis")
  })
  
  observeEvent(input$screenHIST,{
    filename = paste("Historiograph-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$histPlotVis$VIS, filename=filename, type="vis")
  })
  
  observeEvent(input$screenCOL,{
    filename = paste("Collaboration_Network-", "_",  gsub(" |:","",Sys.time()), ".png", sep="")
    screenShot(values$COLnetwork$VIS, filename=filename, type="vis")
  })
  
  ## TALL EXPORT ----
  observe({
    req(values$M)
    ind <- which(values$corpusCol %in% names(values$M))
    corpusCol <- values$corpusCol[ind]
    updateMultiInput(session, 'tallFields',
                     selected = character(0),
                     choices = names(corpusCol))
  })
  
  observe({
    req(values$M)
    ind <- which(values$metadataCol %in% names(values$M))
    metadataCol <- values$metadataCol[ind]
    updateMultiInput(session, 'tallMetadata',
                     selected = character(0),
                     choices = names(metadataCol))
  })
  
  observeEvent(eventExpr = {input$tallRun},
               handlerExpr = {
                 req(input$tallFields)
    values$tallDf <- tallExport(values$M, input$tallFields, input$tallMetadata, values$metadataCol)
    
  })
  
  output$tallTable <- renderDT({
    req(values$tallDf)
      DTformat(values$tallDf, nrow=3, filename="tallDf", pagelength=TRUE, left=NULL, right=NULL, numeric=NULL, dom=FALSE, size='70%', filter="none",
               columnShort=NULL, columnSmall=NULL, round=2, title="", button=FALSE, escape=FALSE, selection=FALSE,scrollX=TRUE)
  })
  
  output$tall.save <- downloadHandler(
    filename = function() {
      paste("tallFile-", Sys.Date(), ".csv", sep="")
    },
    content <- function(file) {
      write.csv(values$tallDf, file=file, row.names = FALSE)
    },
    contentType = "csv"
  )
  
  observeEvent(eventExpr = {values$TALLmissing},
               handlerExpr = {
                 if (values$TALLmissing){
                   output$tallBttn1 <- renderUI({
                     div(style ="border-radius: 10px; border-width: 3px; font-size: 15px;",
                         align = "center",
                         width="100%",
                         actionBttn(inputId = "installTall", label = strong("Install TALL"),
                                    width = "100%", style = "pill", color = "danger",
                                    icon = icon(name ="cloud-download", lib="glyphicon"))
                     )
                   })
                   output$tallBttn2 <- renderUI("")
                 } else {
                   output$tallBttn1 <- renderUI("")
                   output$tallBttn2 <- renderUI({
                     if (require("tall", quietly = TRUE)){
                       div(style ="border-radius: 10px; border-width: 3px; font-size: 15px;",
                           align = "center",
                           width="100%",
                           actionBttn(inputId = "launchTall", label = strong("Launch TALL"),
                                      width = "100%", style = "pill", color = "primary",
                                      icon = icon(name ="play", lib="glyphicon")))
                     }
                   })
                 }
  })
  
  
  observeEvent(eventExpr = {input$installTall},
               handlerExpr = {
                 pak::pkg_install("tall")
                 popUpGeneric(title=NULL, type="success", color=c("#1d8fe1"),
                              subtitle="TALL has been successfully installed",
                              btn_labels="OK", size="40%")
                 if (suppressMessages(require("tall", quietly = TRUE))) values$TALLmissing <- FALSE
               })

  observeEvent(eventExpr = {input$launchTall},
               handlerExpr = {
                 system('Rscript -e "tall::tall()"')
               })
  
  ## SETTING ----
  observeEvent(input$dpi, {
    values$dpi <- as.numeric(input$dpi)
  })
  
  observeEvent(input$h,{
    values$h <- as.numeric(input$h)
  })
  
  output$apiStatus <- renderUI({
    if (values$geminiAPI){
      last <- showGeminiAPI()
      output$status <- renderText(paste0("✅ API key has been set: ",last))
    }
  })
  
  observeEvent(input$set_key, {
    key <- input$api_key
    last <- setGeminiAPI(key)
    
    if (!last$valid){
      output$apiStatus <- renderUI({
        output$status <- renderText(last$message)
      })
      values$geminiAPI <- FALSE
    } else {
      output$apiStatus <- renderUI({
        output$status <- renderText(paste0("✅ API key has been set: ",last$message))
      })
      values$geminiAPI <- TRUE
      home <- homeFolder()
      path_gemini_key <- paste0(home,"/.biblio_gemini_key.txt", collapse="")
      writeLines(Sys.getenv("GEMINI_API_KEY"), path_gemini_key)
    }
    
  })
  
  observeEvent(input$remove_key, {
    if (values$geminiAPI){
      home <- homeFolder()
      path_gemini_key <- paste0(home,"/.biblio_gemini_key.txt", collapse="")
      file.remove(path_gemini_key)
      values$geminiAPI <- FALSE
      output$apiStatus <- renderUI({
        output$status <- renderText(paste0("❌ API key has been removed"))
      })
    }
  })
  
}

# END ####

# ==============================================================================
# UI Component - Query Builder for PubMed API (Collapsible)
# ==============================================================================

pubmedUI <- function() {
  tagList(
    fluidRow(
      column(
        width = 12,
        # Query Builder Box (Collapsible)
        wellPanel(
          id = "pmQueryPanel",

          # Header with collapse button
          div(
            style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 15px;",
            h4(
              "PubMed Data Collection",
              style = "font-weight: bold; margin: 0;"
            ),
            actionButton(
              "pmToggleQueryPanel",
              icon("chevron-up"),
              class = "btn-sm btn-outline-secondary",
              style = "padding: 5px 10px;",
              title = "Collapse/Expand"
            )
          ),

          # Query content (collapsible)
          div(
            id = "pmQueryContent",

            # Query Builder Section
            div(
              id = "pm-query-builder-container",
              style = "margin-bottom: 20px;",

              # Initial query row (NO operator column)
              div(
                id = "pm-query-row-1",
                class = "pm-query-row",
                style = "margin-bottom: 10px;",
                fluidRow(
                  column(1),
                  column(
                    width = 2,
                    selectInput(
                      "pmField_1",
                      NULL,
                      choices = c(
                        "All Fields" = "all",
                        "Title" = "title",
                        "Abstract" = "abstract",
                        "Title/Abstract" = "tiab",
                        "Author" = "author",
                        "MeSH Terms" = "mesh",
                        "Affiliation" = "affiliation"
                      ),
                      selected = "tiab",
                      width = "100%"
                    )
                  ),
                  column(
                    width = 8,
                    textInput(
                      "pmQuery_1",
                      NULL,
                      value = "",
                      placeholder = 'Example: "machine learning" OR AI',
                      width = "100%"
                    )
                  ),
                  column(
                    width = 1,
                    div(
                      id = "pm-remove-button-1",
                      style = "visibility: hidden;",
                      actionButton(
                        "pmRemoveRow_1",
                        icon("minus-circle"),
                        class = "btn-danger btn-sm",
                        style = "margin-top: 0px;",
                        title = "Remove row"
                      )
                    )
                  )
                )
              )
            ),

            # Help text for search syntax
            div(
              style = "margin-bottom: 15px; font-size: 12px; color: #666;",
              HTML(
                "<strong>Search tips:</strong> Use quotes for exact phrases (e.g., \"machine learning\"). Boolean operators AND, OR, NOT must be UPPERCASE. Learn more at <a href='https://pubmed.ncbi.nlm.nih.gov/help/' target='_blank'>PubMed Help</a>."
              )
            ),

            # Add row and date range buttons
            div(
              style = "margin-bottom: 20px;",
              actionButton(
                "pmAddRow",
                label = "+ Add row",
                icon = icon("plus"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "pmToggleDateRange",
                label = "- Date range",
                icon = icon("calendar"),
                class = "btn-outline-primary btn-sm",
                style = "margin-left: 10px;"
              )
            ),

            # Date Range Section (VISIBLE by default)
            div(
              id = "pmDateRangeSection",
              style = "display: block;",
              wellPanel(
                style = "background-color: #f8f9fa;",
                h5("Date Range Filter", style = "font-weight: bold;"),
                fluidRow(
                  column(
                    width = 6,
                    numericInput(
                      "pmYearFrom",
                      "From Year:",
                      value = 1985,
                      min = 1800,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  ),
                  column(
                    width = 6,
                    numericInput(
                      "pmYearTo",
                      "To Year:",
                      value = as.integer(format(Sys.Date(), "%Y")),
                      min = 1800,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  )
                )
              )
            ),

            # Advanced filters toggle
            actionButton(
              "pmToggleAdvanced",
              label = "Hide advanced filters",
              icon = icon("filter"),
              class = "btn-link",
              style = "padding: 5px; margin-bottom: 10px;"
            ),

            # Additional Filters Section (VISIBLE by default)
            div(
              id = "pmAdvancedSection",
              style = "display: block;",
              wellPanel(
                style = "background-color: #f8f9fa;",
                fluidRow(
                  column(
                    width = 4,
                    selectizeInput(
                      "pmDocType",
                      "Document Type:",
                      choices = c(
                        "All" = "",
                        "Journal Article" = "Journal Article",
                        "Review" = "Review",
                        "Clinical Trial" = "Clinical Trial",
                        "Meta-Analysis" = "Meta-Analysis",
                        "Case Reports" = "Case Reports",
                        "Randomized Controlled Trial" = "Randomized Controlled Trial",
                        "Systematic Review" = "Systematic Review",
                        "Letter" = "Letter",
                        "Editorial" = "Editorial"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    selectizeInput(
                      "pmLanguage",
                      "Language:",
                      choices = c(
                        "All" = "",
                        "English" = "english",
                        "Spanish" = "spanish",
                        "French" = "french",
                        "German" = "german",
                        "Italian" = "italian",
                        "Portuguese" = "portuguese",
                        "Chinese" = "chinese",
                        "Japanese" = "japanese"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    numericInput(
                      "pmMaxRecords",
                      "Max Records:",
                      value = 1000,
                      min = 1,
                      max = 10000,
                      step = 100
                    )
                  )
                )
              )
            ),

            # Action buttons with reactive count
            div(
              style = "margin-top: 10px; display: flex; justify-content: space-between; align-items: center;",

              # Left side: Reactive query count message
              div(
                style = "display: flex; align-items: center;",
                uiOutput("pmQueryCountInfo")
              ),

              # Right side: Clear and Search buttons
              div(
                style = "display: flex; gap: 10px;",
                actionButton(
                  "pmClearQuery",
                  "Clear",
                  icon = icon("times"),
                  class = "btn-outline-secondary"
                ),
                actionButton(
                  "pmFetchData",
                  "Search",
                  icon = icon("search"),
                  class = "btn-primary"
                )
              )
            )
          )
        )
      )
    ),

    # Data Display Section (shown when data is available)
    conditionalPanel(
      condition = "output.pmDataAvailable",
      fluidRow(
        column(
          width = 12,
          wellPanel(
            h4("Downloaded Collection", style = "font-weight: bold;"),

            # Progress and results info
            uiOutput("pmProgressInfo"),

            # Export buttons
            div(
              style = "margin-bottom: 15px; margin-top: 15px;",
              downloadButton(
                "pmDownloadExcel",
                "Export to Excel",
                icon = icon("file-excel"),
                class = "btn-success"
              ),
              downloadButton(
                "pmDownloadRData",
                "Export to RData",
                icon = icon("database"),
                class = "btn-info",
                style = "margin-left: 10px;"
              )
            ),

            # Data table output
            div(
              style = "margin-top: 15px;",
              uiOutput("pmDataTable")
            )
          )
        )
      )
    ),

    # JavaScript for collapsible functionality
    tags$script(HTML(
      "
      $(document).ready(function() {
        // Initialize collapsed state
        var isCollapsed = false;
        
        // Initialize visibility states as TRUE (open)
        var dateRangeVisible = true;
        var advancedVisible = true;
        
        // Handle toggle button click for main panel
        $('#pmToggleQueryPanel').on('click', function() {
          isCollapsed = !isCollapsed;
          $('#pmQueryContent').slideToggle(300);
          
          // Update button icon
          if (isCollapsed) {
            $(this).html('<i class=\"fa fa-chevron-down\"></i>');
          } else {
            $(this).html('<i class=\"fa fa-chevron-up\"></i>');
          }
        });
        
        // Handle date range toggle
        $('#pmToggleDateRange').on('click', function() {
          dateRangeVisible = !dateRangeVisible;
          $('#pmDateRangeSection').slideToggle(300);
          
          // Update button text
          if (dateRangeVisible) {
            $(this).html('<i class=\"fa fa-calendar\"></i> - Date range');
          } else {
            $(this).html('<i class=\"fa fa-calendar\"></i> + Date range');
          }
          
          // Notify Shiny
          Shiny.setInputValue('pmShowDateRange', dateRangeVisible);
        });
        
        // Handle advanced filters toggle
        $('#pmToggleAdvanced').on('click', function() {
          advancedVisible = !advancedVisible;
          $('#pmAdvancedSection').slideToggle(300);
          
          // Update button text
          if (advancedVisible) {
            $(this).html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
          } else {
            $(this).html('<i class=\"fa fa-filter\"></i> Show advanced filters');
          }
          
          // Notify Shiny
          Shiny.setInputValue('pmShowAdvanced', advancedVisible);
        });
        
        // Set initial values in Shiny
        Shiny.setInputValue('pmShowDateRange', true);
        Shiny.setInputValue('pmShowAdvanced', true);
      });
      
      // Custom message handler to collapse panel when data is loaded
      Shiny.addCustomMessageHandler('pmCollapseQueryPanel', function(message) {
        $('#pmQueryContent').slideUp(300);
        $('#pmToggleQueryPanel').html('<i class=\"fa fa-chevron-down\"></i>');
      });
    "
    ))
  )
}


# ==============================================================================
# Server Component - Query Builder for PubMed
# ==============================================================================

pubmedServer <- function(input, output, session, values) {
  # Reactive values to track query rows and toggle states
  queryRows <- reactiveVal(1)
  showDateRange <- reactiveVal(TRUE) # TRUE by default
  showAdvanced <- reactiveVal(TRUE) # TRUE by default

  # Add new query row
  observeEvent(input$pmAddRow, {
    currentRows <- queryRows()
    newRowId <- currentRows + 1
    queryRows(newRowId)

    # Insert new row in UI (WITH operator column)
    insertUI(
      selector = "#pm-query-builder-container",
      where = "beforeEnd",
      ui = div(
        id = paste0("pm-query-row-", newRowId),
        class = "pm-query-row",
        style = "margin-bottom: 10px;",
        fluidRow(
          column(
            width = 1,
            selectInput(
              paste0("pmOperator_", newRowId),
              NULL,
              choices = c("AND" = "AND", "OR" = "OR", "NOT" = "NOT"),
              selected = "AND",
              width = "100%"
            )
          ),
          column(
            width = 2,
            selectInput(
              paste0("pmField_", newRowId),
              NULL,
              choices = c(
                "All Fields" = "all",
                "Title" = "title",
                "Abstract" = "abstract",
                "Title/Abstract" = "tiab",
                "Author" = "author",
                "MeSH Terms" = "mesh",
                "Affiliation" = "affiliation"
              ),
              selected = "tiab",
              width = "100%"
            )
          ),
          column(
            width = 8,
            textInput(
              paste0("pmQuery_", newRowId),
              NULL,
              value = "",
              placeholder = 'Example: "deep learning"',
              width = "100%"
            )
          ),
          column(
            width = 1,
            actionButton(
              paste0("pmRemoveRow_", newRowId),
              icon("minus-circle"),
              class = "btn-danger btn-sm",
              style = "margin-top: 0px;",
              title = "Remove row"
            )
          )
        )
      )
    )

    # Make remove button visible for first row if adding second row
    if (newRowId == 2) {
      runjs("$('#pm-remove-button-1').css('visibility', 'visible');")
    }

    # Add observer for the remove button of the new row
    local({
      rowId <- newRowId
      observeEvent(input[[paste0("pmRemoveRow_", rowId)]], {
        removeUI(selector = paste0("#pm-query-row-", rowId))

        # Check if only one row remains
        if (queryRows() - 1 == 1) {
          runjs("$('#pm-remove-button-1').css('visibility', 'hidden');")
        }
      })
    })
  })

  # Clear all query fields
  observeEvent(input$pmClearQuery, {
    # Reset all query inputs
    for (i in 1:queryRows()) {
      updateTextInput(session, paste0("pmQuery_", i), value = "")
      updateSelectInput(session, paste0("pmField_", i), selected = "tiab")
      if (i > 1) {
        updateSelectInput(session, paste0("pmOperator_", i), selected = "AND")
      }
    }

    # Reset filters
    updateNumericInput(session, "pmYearFrom", value = 2020)
    updateNumericInput(
      session,
      "pmYearTo",
      value = as.integer(format(Sys.Date(), "%Y"))
    )
    updateSelectizeInput(session, "pmDocType", selected = "")
    updateSelectizeInput(session, "pmLanguage", selected = "")
    updateNumericInput(session, "pmMaxRecords", value = 1000)
  })

  # Helper function to build PubMed query
  build_pubmed_query <- function(queryList) {
    if (length(queryList) == 0) {
      return("")
    }

    query_parts <- c()

    for (i in seq_along(queryList)) {
      item <- queryList[[i]]

      # Wrap query with field tag
      field_tag <- switch(
        item$field,
        "all" = "",
        "title" = "[Title]",
        "abstract" = "[Abstract]",
        "tiab" = "[Title/Abstract]",
        "author" = "[Author]",
        "mesh" = "[MeSH Terms]",
        "affiliation" = "[Affiliation]",
        ""
      )

      wrapped_query <- paste0("(", item$query, ")", field_tag)

      if (i == 1) {
        query_parts <- c(query_parts, wrapped_query)
      } else {
        operator <- item$operator
        query_parts <- c(query_parts, operator, wrapped_query)
      }
    }

    query <- paste(query_parts, collapse = " ")
    return(query)
  }

  # Reactive query count - updates automatically when query changes
  output$pmQueryCountInfo <- renderUI({
    # Make this reactive to query inputs
    lapply(1:queryRows(), function(i) {
      input[[paste0("pmQuery_", i)]]
      input[[paste0("pmField_", i)]]
      if (i > 1) input[[paste0("pmOperator_", i)]]
    })

    # Also react to filters
    input$pmYearFrom
    input$pmYearTo
    input$pmDocType
    input$pmLanguage

    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("pmQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[i]] <- list(
          query = queryText,
          field = input[[paste0("pmField_", i)]],
          operator = if (i > 1) input[[paste0("pmOperator_", i)]] else NULL
        )
      }
    }

    # If no query, show hint
    if (length(queryList) == 0) {
      return(
        div(
          style = "display: inline-block; padding: 5px 10px; color: #6c757d; font-size: 13px; font-style: italic;",
          icon("info-circle"),
          " Enter search terms to see record count"
        )
      )
    }

    # Build query and fetch count
    tryCatch(
      {
        # Build the PubMed query
        query <- build_pubmed_query(queryList)

        # Add date range
        if (showDateRange()) {
          if (!is.na(input$pmYearFrom) && !is.na(input$pmYearTo)) {
            date_filter <- sprintf(
              " AND (%s:%s[pdat])",
              input$pmYearFrom,
              input$pmYearTo
            )
            query <- paste0(query, date_filter)
          }
        }

        # Add document type filter
        if (showAdvanced()) {
          if (!is.null(input$pmDocType) && length(input$pmDocType) > 0) {
            # Rimuovi eventuali stringhe vuote
            doc_types <- input$pmDocType[input$pmDocType != ""]
            if (length(doc_types) > 0) {
              type_filters <- sapply(doc_types, function(t) {
                paste0("\"", t, "\"[Publication Type]")
              })
              type_filter <- paste0(
                " AND (",
                paste(type_filters, collapse = " OR "),
                ")"
              )
              query <- paste0(query, type_filter)
            }
          }
          # Add language filter
          if (!is.null(input$pmLanguage) && length(input$pmLanguage) > 0) {
            # Rimuovi eventuali stringhe vuote
            languages <- input$pmLanguage[input$pmLanguage != ""]
            if (length(languages) > 0) {
              lang_filters <- sapply(input$pmLanguage, function(l) {
                paste0(l, "[Language]")
              })
              lang_filter <- paste0(
                " AND (",
                paste(lang_filters, collapse = " OR "),
                ")"
              )
              query <- paste0(query, lang_filter)
            }
          }
        }

        # Get count from PubMed
        if (!require("pubmedR", quietly = TRUE)) {
          return(
            div(
              style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
              icon("exclamation-circle"),
              " pubmedR package not installed"
            )
          )
        }

        count_result <- pmQueryTotalCount(query = query, api_key = NULL)
        total_available <- count_result$total_count

        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #d4edda; color: #155724; border-radius: 4px; font-size: 13px; font-weight: 500;",
          icon("check-circle"),
          sprintf(" Found %s records", format(total_available, big.mark = ","))
        )
      },
      error = function(e) {
        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
          icon("exclamation-circle"),
          " Error: ",
          e$message
        )
      }
    )
  })

  # Download data from PubMed
  observeEvent(input$pmFetchData, {
    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("pmQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[i]] <- list(
          query = queryText,
          field = input[[paste0("pmField_", i)]],
          operator = if (i > 1) input[[paste0("pmOperator_", i)]] else NULL
        )
      }
    }

    # Validate that at least one query exists
    if (length(queryList) == 0) {
      showModal(modalDialog(
        title = "Error",
        "Please enter at least one search term.",
        easyClose = TRUE,
        footer = modalButton("OK")
      ))
      return()
    }

    # Build the PubMed query
    query <- build_pubmed_query(queryList)

    # Add date range
    if (showDateRange()) {
      if (!is.na(input$pmYearFrom) && !is.na(input$pmYearTo)) {
        date_filter <- sprintf(
          " AND (%s:%s[pdat])",
          input$pmYearFrom,
          input$pmYearTo
        )
        query <- paste0(query, date_filter)
      }
    }

    # Add document type filter
    if (showAdvanced()) {
      if (!is.null(input$pmDocType) && length(input$pmDocType) > 0) {
        # Rimuovi eventuali stringhe vuote
        doc_types <- input$pmDocType[input$pmDocType != ""]
        if (length(doc_types) > 0) {
          type_filters <- sapply(doc_types, function(t) {
            paste0("\"", t, "\"[Publication Type]")
          })
          type_filter <- paste0(
            " AND (",
            paste(type_filters, collapse = " OR "),
            ")"
          )
          query <- paste0(query, type_filter)
        }
      }
      # Add language filter
      if (!is.null(input$pmLanguage) && length(input$pmLanguage) > 0) {
        # Rimuovi eventuali stringhe vuote
        languages <- input$pmLanguage[input$pmLanguage != ""]
        if (length(languages) > 0) {
          lang_filters <- sapply(input$pmLanguage, function(l) {
            paste0(l, "[Language]")
          })
          lang_filter <- paste0(
            " AND (",
            paste(lang_filters, collapse = " OR "),
            ")"
          )
          query <- paste0(query, lang_filter)
        }
      }
    }

    max_records <- input$pmMaxRecords

    print("PubMed Query:")
    print(query)

    # Show progress modal with dynamic content
    showModal(modalDialog(
      title = "Downloading from PubMed",
      div(
        div(
          style = "text-align: center; margin: 20px 0;",
          icon("spinner", class = "fa-spin fa-3x")
        ),
        div(
          id = "pm-download-progress-text",
          style = "text-align: center; font-size: 14px;",
          "Initializing search..."
        )
      ),
      footer = NULL,
      easyClose = FALSE
    ))

    # Fetch data
    tryCatch(
      {
        # Update progress
        shinyjs::html(
          "pm-download-progress-text",
          paste("Searching PubMed with query:", query)
        )

        # Load pubmedR library
        if (!require("pubmedR", quietly = TRUE)) {
          stop(
            "pubmedR package is not installed. Please install it using: install.packages('pubmedR')"
          )
        }

        # Get total count first
        shinyjs::html(
          "pm-download-progress-text",
          "Counting available records..."
        )

        count_result <- pmQueryTotalCount(query = query, api_key = NULL)
        total_available <- count_result$total_count
        records_to_download <- min(total_available, max_records)

        # Update progress with count information
        shinyjs::html(
          "pm-download-progress-text",
          sprintf(
            "Found %s records. Downloading %s records...",
            format(total_available, big.mark = ","),
            format(records_to_download, big.mark = ",")
          )
        )

        # Fetch data from PubMed using pmApiRequest with limit parameter
        pm_data <- pmApiRequest(
          query = query,
          limit = records_to_download,
          api_key = NULL
        )

        # Update progress
        shinyjs::html(
          "pm-download-progress-text",
          sprintf(
            "Converting %s records to bibliometric format...",
            format(pm_data$records_downloaded, big.mark = ",")
          )
        )

        # Convert using bibliometrix convert2df function
        M <- convert2df(
          file = pm_data,
          dbsource = "pubmed",
          format = "api"
        )

        # Close progress modal
        removeModal()

        # Collapse query panel after successful download
        session$sendCustomMessage('pmCollapseQueryPanel', TRUE)

        # Show success message
        showModal(modalDialog(
          title = "Success",
          div(
            p(
              sprintf(
                "Successfully downloaded %s records from PubMed.",
                format(nrow(M), big.mark = ",")
              )
            ),
            if (total_available > max_records) {
              p(
                style = "color: #856404; background-color: #fff3cd; padding: 10px; border-radius: 5px;",
                sprintf(
                  "Note: %s total records found, but limited to %s records.",
                  format(total_available, big.mark = ","),
                  format(max_records, big.mark = ",")
                )
              )
            }
          ),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))

        # Initialize values
        values = initial(values)
        M <- M %>% mergeKeywords(force = FALSE)

        # Update values
        values$data_source <- "PubMed"
        values$M <- M
        values$Morig <- M
        values$SCdf <- wcTable(M)
        values$COdf <- countryTable(M)
        values$Histfield <- "NA"
        values$results <- list("NA")

        if (ncol(values$M) > 1) {
          values$rest_sidebar <- TRUE
        }
        if (ncol(values$M) > 1) {
          values$loadMenu <- "pubmed_api"
          showModal(missingModalAPI(session))
        }
      },
      error = function(e) {
        removeModal()
        showModal(modalDialog(
          title = "Error",
          paste("An error occurred:", e$message),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))
      }
    )
  })

  # Render data table
  output$pmDataTable <- renderUI({
    req(values$M)
    req(values$data_source == "PubMed")

    MData <- as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))

    # Create DOI links if available
    if ("DI" %in% names(MData)) {
      MData$DOI <- paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
      nome <- c("DOI", names(MData)[-length(names(MData))])
      MData <- MData[nome]
    }

    renderBibliobox(
      MData,
      nrow = 3,
      filename = "Table",
      pagelength = TRUE,
      left = NULL,
      right = NULL,
      numeric = NULL,
      dom = TRUE,
      size = '70%',
      filter = "top",
      columnShort = NULL,
      columnSmall = NULL,
      round = 2,
      title = "",
      button = FALSE,
      escape = FALSE,
      selection = FALSE,
      scrollX = TRUE
    )
  })

  # Check if data is available for conditional panel
  output$pmDataAvailable <- reactive({
    !is.null(values$M) && values$data_source == "PubMed"
  })
  outputOptions(output, "pmDataAvailable", suspendWhenHidden = FALSE)

  # Display progress information
  output$pmProgressInfo <- renderUI({
    if (!is.null(values$M) && values$data_source == "PubMed") {
      div(
        class = "alert alert-success",
        icon("check-circle"),
        strong(" Data loaded: "),
        sprintf(
          "%s documents from PubMed",
          format(nrow(values$M), big.mark = ",")
        )
      )
    }
  })

  # Download handler for Excel
  output$pmDownloadExcel <- downloadHandler(
    filename = function() {
      paste0("pubmed_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "PubMed")

      # Write data to Excel file
      openxlsx::write.xlsx(values$M, file = file)
    }
  )

  # Download handler for RData
  output$pmDownloadRData <- downloadHandler(
    filename = function() {
      paste0("pubmed_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".RData")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "PubMed")

      # Save data as RData file
      M <- values$M
      save(M, file = file)
    }
  )

  # Modal for missing data
  missingModalAPI <- function(session) {
    ns <- session$ns
    modalDialog(
      uiOutput("missingTitle"),
      uiOutput(ns("missingDataTable")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(
          label = "Advice",
          inputId = "missingMessage",
          icon = icon("exclamation-sign", lib = "glyphicon")
        ),
        actionButton(
          label = "Report",
          inputId = "missingReport",
          icon = icon("plus", lib = "glyphicon")
        ),
        actionButton(
          label = "Save",
          inputId = "missingDataTable",
          icon = icon("camera", lib = "glyphicon")
        ),
        modalButton(label = "Close")
      )
    )
  }
}

## UI components for content analysis in Biblioshiny ----

#' Content Analysis Tab Item with two main tabs for Biblioshiny UI
#'
#' This creates a tabItem with 2 main tabs:
#' 1. Import PDF File - Contains the PDF upload interface and analysis results (with 3 sub-tabs)
#' 2. Info & References - Contains help documentation
#'
#' @param id Character, the tab item ID (default: "content_analysis")
#' @return A tabItem object for biblioshiny

content_analysis_tab <- function(id = "content_analysis") {
  tabItem(
    tabName = id,

    # Main tabsetPanel with Import and Info & References
    tabsetPanel(
      id = "content_analysis_main_tabs",
      type = "tabs",

      # ===========================================
      # MAIN TAB 1: IMPORT PDF FILE
      # ===========================================
      tabPanel(
        title = "Import and Analyze PDF File",
        value = "tab_import_pdf",
        icon = icon("upload"),

        br(),

        # Tab header
        fluidRow(
          column(
            7,
            div(
              class = "page-header",
              h2(
                "Scientific Article Content Analysis",
                style = "color: #2E86AB; margin-bottom: 10px;"
              ),
              p(
                "Upload a PDF file and analyze citation patterns, context, and co-occurrence networks.",
                style = "color: #666; font-size: 16px; margin-bottom: 20px;"
              )
            )
          ),
          column(
            5,
            div(
              style = "text-align: right; padding-top: 15px;",

              # Document metadata display (shown after text extraction)
              conditionalPanel(
                condition = "output.metadata_available",
                uiOutput("pdf_metadata_display")
              ),

              # Export buttons (shown after analysis completion)
              conditionalPanel(
                condition = "output.analysis_completed",
                div(
                  style = "display: flex; gap: 8px; justify-content: flex-end; flex-wrap: wrap;",
                  downloadButton(
                    "download_all_results",
                    "Export All Results",
                    icon = icon("file-excel"),
                    class = "btn-success",
                    style = "font-weight: bold; margin-top: 10px;"
                  ),
                  actionButton(
                    "export_all_images",
                    "Export All Images",
                    icon = icon("image"),
                    class = "btn-info",
                    style = "font-weight: bold; margin-top: 10px;"
                  )
                )
              )
            )
          )
        ),

        fluidRow(
          # ===========================================
          # RIGHT PANEL: TABBED RESULTS (column 10)
          # ===========================================
          column(
            10,

            # NEW: Button to open modal and DOI field (visible only after extraction)
            conditionalPanel(
              condition = "output.text_extracted && !output.analysis_completed",
              div(
                style = "margin-bottom: 15px; padding: 15px; background-color: #e8f4f8; border-radius: 5px;",

                # First row: preview button and message
                fluidRow(
                  column(
                    6,
                    align = "center",
                    actionButton(
                      "open_preview_btn",
                      "View Extracted Text & PDF Preview",
                      icon = icon("file-alt"),
                      class = "btn-info",
                      onclick = "$('#previewModal').modal('show');",
                      style = "width: 70%;"
                    )
                  ),
                  column(
                    6,
                    align = "center",
                    downloadButton(
                      "save_text_file",
                      "Save Extracted Text",
                      icon = icon("save"),
                      class = "btn-success",
                      style = "width: 70%;"
                    )
                  )
                ),
                fluidRow(
                  column(
                    12,
                    div(
                      style = "margin-top: 10px;",
                      span(
                        icon("check-circle", style = "color: #27ae60;"),
                        " Text extracted successfully!",
                        style = "color: #555; font-weight: 500;"
                      )
                    ),
                    helpText(
                      "💾 The 'Save' button will save the text with DOI and citation format info for easy reloading.",
                      style = "margin-top: 5px; font-size: 11px; color: #666; font-style: italic;"
                    )
                  )
                ),

                # Second row: DOI field
                fluidRow(
                  style = "margin-top: 15px;",
                  column(
                    12,
                    div(
                      style = "background-color: white; padding: 10px; border-radius: 4px; border: 1px solid #d1e7f0;",
                      fluidRow(
                        column(
                          2,
                          tags$label(
                            "Document DOI:",
                            style = "margin-top: 8px; font-weight: bold; color: #2E86AB;"
                          )
                        ),
                        column(
                          4,
                          textInput(
                            "pdf_doi_input",
                            label = NULL,
                            value = "",
                            placeholder = "Enter or edit DOI (e.g., 10.1234/example)",
                            width = "100%"
                          )
                        ),
                        column(
                          6,
                          div(
                            style = "margin-top: 8px;",
                            uiOutput("doi_status_icon")
                          )
                        )
                      ),
                      helpText(
                        "The DOI is used to fetch reference details from Crossref. Edit if needed or add manually if not detected.",
                        style = "margin-bottom: 0px; margin-top: -5px; font-size: 11px; color: #666;"
                      )
                    )
                  )
                )
              )
            ),

            conditionalPanel(
              condition = "output.analysis_completed",

              # NEW: Info after analysis
              div(
                style = "margin-bottom: 10px;",
                fluidRow(
                  column(
                    4,
                    actionButton(
                      "reopen_preview_btn",
                      "View Original Text",
                      icon = icon("file-alt"),
                      class = "btn-default btn-sm",
                      onclick = "$('#previewModal').modal('show');"
                    )
                  ),
                  column(
                    8,
                    div(
                      style = "text-align: right; padding-top: 5px;",
                      tags$strong("DOI: ", style = "color: #2E86AB;"),
                      tags$span(
                        textOutput("doi_display_after_analysis", inline = TRUE),
                        style = "font-family: 'Courier New', monospace; color: #555; background-color: #f8f9fa; padding: 4px 8px; border-radius: 3px; border: 1px solid #dee2e6;"
                      )
                    )
                  )
                )
              ),
              tabsetPanel(
                id = "content_tabs",
                type = "tabs",

                # ===========================================
                # TAB 1: DESCRIPTIVE STATISTICS
                # ===========================================
                tabPanel(
                  title = "Descriptive Statistics",
                  value = "tab_stats",

                  br(),

                  # Summary Statistics Cards
                  fluidRow(
                    column(
                      3,
                      div(
                        class = "info-box bg-aqua",
                        span(class = "info-box-icon", icon("file-text")),
                        div(
                          class = "info-box-content",
                          span("Total Words", class = "info-box-text"),
                          span(
                            textOutput("total_words", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-green",
                        span(class = "info-box-icon", icon("quote-right")),
                        div(
                          class = "info-box-content",
                          span("Citations Found", class = "info-box-text"),
                          span(
                            textOutput("total_citations", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-yellow",
                        span(class = "info-box-icon", icon("users")),
                        div(
                          class = "info-box-content",
                          span("Narrative Citations", class = "info-box-text"),
                          span(
                            textOutput("narrative_citations", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-red",
                        span(class = "info-box-icon", icon("chart-line")),
                        div(
                          class = "info-box-content",
                          span("Citation Density", class = "info-box-text"),
                          span(
                            textOutput("citation_density", inline = TRUE),
                            class = "info-box-number"
                          ),
                          span("/1000 words", class = "info-box-more")
                        )
                      )
                    )
                  ),

                  fluidRow(
                    column(
                      3,
                      div(
                        class = "info-box bg-purple",
                        span(class = "info-box-icon", icon("graduation-cap")),
                        div(
                          class = "info-box-content",
                          span("Flesch-Kincaid Grade", class = "info-box-text"),
                          span(
                            textOutput("flesch_kincaid_grade", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-blue",
                        span(class = "info-box-icon", icon("book-reader")),
                        div(
                          class = "info-box-content",
                          span("Reading Ease", class = "info-box-text"),
                          span(
                            textOutput("flesch_reading_ease", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-teal",
                        span(class = "info-box-icon", icon("robot")),
                        div(
                          class = "info-box-content",
                          span("ARI Index", class = "info-box-text"),
                          span(
                            textOutput("ari_index", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    ),
                    column(
                      3,
                      div(
                        class = "info-box bg-maroon",
                        span(class = "info-box-icon", icon("cloud")),
                        div(
                          class = "info-box-content",
                          span("Gunning Fog Index", class = "info-box-text"),
                          span(
                            textOutput("gunning_fog_index", inline = TRUE),
                            class = "info-box-number"
                          )
                        )
                      )
                    )
                  ),
                  # Readability Indices Details
                  fluidRow(
                    column(
                      6,
                      div(
                        class = "box box-info",
                        div(
                          class = "box-header with-border",
                          h5("Readability Indices", style = "color: #3498db;")
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.analysis_completed",
                            uiOutput("readability_details_html")
                          ),
                          conditionalPanel(
                            condition = "!output.analysis_completed",
                            div(
                              style = "text-align: center; padding: 20px; color: #999;",
                              p(
                                "Readability indices will appear here after analysis."
                              )
                            )
                          )
                        )
                      )
                    ),
                    column(
                      6,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h5("Text Statistics", style = "color: #2E86AB;")
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.analysis_completed",
                            uiOutput("text_stats")
                          ),
                          conditionalPanel(
                            condition = "!output.analysis_completed",
                            div(
                              style = "text-align: center; padding: 20px; color: #999;",
                              p(
                                "Text statistics will appear here after analysis."
                              )
                            )
                          )
                        )
                      )
                    )
                  ),
                  # Detailed Analysis Tables
                  fluidRow(
                    div(
                      class = "box box-primary",
                      div(
                        class = "box-header with-border",
                        h4(
                          "N-grams Analysis",
                          class = "box-title",
                          style = "color: #2E86AB;"
                        )
                      ),
                      column(
                        4,
                        h5(
                          "Top Unigrams",
                          class = "box-title",
                          style = "color: #2E86AB;"
                        ),
                        uiOutput("frequent_words_table")
                      ),
                      column(
                        4,
                        h5("Top Bigrams", style = "color: #2E86AB;"),
                        uiOutput("bigrams_table")
                      ),
                      column(
                        4,
                        h5("Top Trigrams", style = "color: #2E86AB;"),
                        uiOutput("trigrams_table")
                      )
                    )
                  ),
                  hr(),
                  # N-grams Analysis
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box-body",
                        fluidRow(
                          column(
                            6,
                            div(
                              class = "box box-primary",
                              h5(
                                "Citation Types Distribution",
                                class = "box-title",
                                style = "color: #2E86AB;"
                              ),
                              div(
                                class = "box-header with-border",
                                div(
                                  class = "box-body",
                                  uiOutput("citation_types_table")
                                )
                              )
                            )
                          ),
                          column(
                            6,
                            div(
                              class = "box box-success",
                              h5(
                                "Citations by Section",
                                class = "box-title",
                                style = "color: #27ae60;"
                              ),
                              div(
                                class = "box-header with-border",
                                div(
                                  class = "box-body",
                                  uiOutput("citation_sections_table")
                                )
                              )
                            )
                          )
                        )
                      )
                    )
                  )
                ),

                # Add this new tabPanel AFTER the "Network Analysis" tab in the content_analysis_tab function

                tabPanel(
                  title = "Word Trends",
                  value = "tab_trends",

                  br(),

                  # Word Selection and Configuration
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Word Distribution Analysis",
                            class = "box-title",
                            style = "color: #2E86AB;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            downloadButton(
                              "download_word_trends",
                              "Export Data",
                              class = "btn btn-primary btn-sm",
                              icon = icon("download")
                            )
                          )
                        ),
                        div(
                          class = "box-body",
                          fluidRow(
                            column(
                              4,
                              selectizeInput(
                                "trend_words",
                                "Select words to track:",
                                choices = NULL,
                                selected = NULL,
                                multiple = TRUE,
                                options = list(
                                  placeholder = 'Type or select words...',
                                  maxItems = 10,
                                  plugins = list('remove_button')
                                )
                              ),
                              helpText(
                                "Select up to 10 words from the most frequent terms in your document."
                              )
                            ),
                            column(
                              3,
                              radioButtons(
                                "segmentation_type",
                                "Segmentation:",
                                choices = list(
                                  "Auto (use sections if available)" = "auto",
                                  "Document sections" = "sections",
                                  "Equal-length segments" = "segments"
                                ),
                                selected = "auto"
                              )
                            ),
                            column(
                              2,
                              conditionalPanel(
                                condition = "input.segmentation_type == 'segments'",
                                numericInput(
                                  "n_segments",
                                  "Number of segments:",
                                  value = 10,
                                  min = 5,
                                  max = 20,
                                  step = 1
                                )
                              )
                            ),
                            column(
                              3,
                              radioButtons(
                                "trend_plot_type",
                                "Visualization type:",
                                choices = list(
                                  "Line chart" = "line",
                                  "Area chart" = "area"
                                ),
                                selected = "line"
                              ),
                              checkboxInput(
                                "trend_show_points",
                                "Show data points",
                                value = TRUE
                              )
                              # ,checkboxInput("trend_smooth",
                              #               "Smooth lines",
                              #               value = TRUE
                              # )
                            )
                          ),
                          hr(),
                          fluidRow(
                            column(
                              12,
                              actionButton(
                                "update_trends",
                                "Update Visualization",
                                icon = icon("refresh"),
                                class = "btn-info",
                                style = "margin-bottom: 10px;"
                              ),
                              conditionalPanel(
                                condition = "input.segmentation_type == 'sections' && !output.sections_available",
                                div(
                                  style = "background-color: #fff3cd; padding: 10px; border-radius: 5px; margin-top: 10px; border-left: 4px solid #ffc107;",
                                  icon(
                                    "exclamation-triangle",
                                    style = "color: #856404;"
                                  ),
                                  span(
                                    " Document sections are not available. The analysis will use equal-length segments instead.",
                                    style = "color: #856404; margin-left: 8px;"
                                  )
                                )
                              )
                            )
                          )
                        )
                      )
                    )
                  ),

                  # Visualization
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-success",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Word Distribution Over Document",
                            class = "box-title",
                            style = "color: #27ae60;"
                          )
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.trends_available",
                            plotlyOutput("word_trends_plot", height = "600px")
                          ),
                          conditionalPanel(
                            condition = "!output.trends_available",
                            div(
                              style = "text-align: center; padding: 60px; color: #999;",
                              icon(
                                "chart-line",
                                style = "font-size: 48px; margin-bottom: 20px;"
                              ),
                              h4(
                                "No visualization available",
                                style = "color: #666;"
                              ),
                              p(
                                "Select words from the list above and click 'Update Visualization' to see their distribution across the document.",
                                style = "font-size: 14px;"
                              )
                            )
                          )
                        )
                      )
                    )
                  ),

                  # Statistics Table
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-info",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Distribution Statistics",
                            class = "box-title",
                            style = "color: #3498db;"
                          )
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.trends_available",
                            uiOutput("word_trends_table")
                          ),
                          conditionalPanel(
                            condition = "!output.trends_available",
                            div(
                              style = "text-align: center; padding: 20px; color: #999;",
                              p(
                                "Statistics will appear here after visualization is generated."
                              )
                            )
                          )
                        )
                      )
                    )
                  )
                ),

                # ===========================================
                # TAB 2: IN-CONTEXT CITATION ANALYSIS
                # ===========================================
                tabPanel(
                  title = "In-Context Citations",
                  value = "tab_contexts",

                  br(),

                  # Search and Filter Controls - SINGLE ROW
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h4(
                            "In-Context Citation Analysis",
                            class = "box-title",
                            style = "color: #2E86AB;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            downloadButton(
                              "download_contexts",
                              "Export Contexts",
                              class = "btn btn-primary btn-sm",
                              icon = icon("download")
                            )
                          )
                        ),
                        div(
                          class = "box-body",
                          # SINGLE ROW with all controls
                          fluidRow(
                            column(
                              4,
                              textInput(
                                "context_search",
                                "Search in contexts:",
                                placeholder = "Type to search citations or context...",
                                width = "100%"
                              )
                            ),
                            column(
                              2,
                              numericInput(
                                "context_min_words",
                                "Min context words:",
                                value = 10,
                                min = 5,
                                max = 100,
                                width = "100%"
                              )
                            ),
                            column(
                              2,
                              radioButtons(
                                "citation_segmentation",
                                "Group citations by:",
                                choices = list(
                                  "Auto (use sections if available)" = "auto",
                                  "Document sections" = "sections",
                                  "Equal-length segments" = "segments"
                                ),
                                selected = "auto",
                                inline = FALSE
                              )
                            ),
                            column(
                              2,
                              conditionalPanel(
                                condition = "input.citation_segmentation == 'segments'",
                                numericInput(
                                  "n_segments_citations",
                                  "Number of segments:",
                                  value = 10,
                                  min = 5,
                                  max = 20,
                                  step = 1
                                )
                              )
                            ),
                            column(
                              2,
                              br(),
                              actionButton(
                                "update_citation_grouping",
                                "Update Grouping",
                                icon = icon("refresh"),
                                class = "btn-info btn-block",
                                style = "margin-top: 5px;"
                              ),
                              conditionalPanel(
                                condition = "input.citation_segmentation == 'sections' && !output.sections_available",
                                div(
                                  style = "background-color: #fff3cd; padding: 5px; border-radius: 3px; margin-top: 8px; border-left: 3px solid #ffc107; font-size: 11px;",
                                  icon(
                                    "exclamation-triangle",
                                    style = "color: #856404; font-size: 10px;"
                                  ),
                                  span(
                                    " Sections not available",
                                    style = "color: #856404; margin-left: 5px;"
                                  )
                                )
                              ),
                              conditionalPanel(
                                condition = "$('html').hasClass('shiny-busy')",
                                div(
                                  style = "margin-top: 8px; text-align: center;",
                                  icon(
                                    "spinner",
                                    class = "fa-spin",
                                    style = "color: #3498db; font-size: 12px;"
                                  ),
                                  span(
                                    "Updating...",
                                    style = "color: #3498db; font-style: italic; margin-left: 3px; font-size: 11px;"
                                  )
                                )
                              )
                            )
                          )
                        )
                      )
                    )
                  ),

                  # Citation Contexts Display
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-success",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Citation Contexts Visualization",
                            class = "box-title",
                            style = "color: #27ae60;"
                          )
                        ),
                        div(
                          class = "box-body",
                          uiOutput("citation_contexts_html")
                        )
                      )
                    )
                  )
                ),

                # ===========================================
                # TAB 3: NETWORK ANALYSIS
                # ===========================================
                tabPanel(
                  title = "Network Analysis",
                  value = "tab_network",

                  br(),

                  # Network Visualization
                  fluidRow(
                    div(
                      class = "box box-primary",
                      div(
                        class = "box-header with-border",
                        h4(
                          "Citation Co-occurrence Network",
                          class = "box-title",
                          style = "color: #2E86AB;"
                        ),
                        div(
                          class = "box-tools pull-right",
                          downloadButton(
                            "download_network",
                            "Export Network",
                            class = "btn btn-primary btn-sm",
                            icon = icon("download")
                          )
                        )
                      ),
                      div(
                        class = "box-body",
                        div(
                          style = "margin-bottom: 15px; padding: 10px; background-color: #f0f8ff; border-radius: 5px; display: flex; align-items: center; flex-wrap: wrap; gap: 15px;",
                          div(
                            style = "flex: 1; min-width: 300px;",
                            icon("info-circle", style = "color: #3498db;"),
                            span(
                              " Nodes are colored by paper section. Use the slider to adjust node spacing.",
                              style = "color: #555; font-size: 13px; margin-left: 8px;"
                            )
                          ),
                          div(
                            style = "display: flex; align-items: center; gap: 12px; flex-wrap: wrap;",
                            tags$label(
                              "Max nodes/section:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            numericInput(
                              "network_max_nodes",
                              label = NULL,
                              value = 10,
                              min = 3,
                              max = 50,
                              step = 1,
                              width = "70px"
                            ),
                            tags$label(
                              "Node size:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            sliderInput(
                              "network_node_size",
                              label = NULL,
                              min = 1,
                              max = 10,
                              value = 5,
                              step = 1,
                              width = "100px",
                              ticks = FALSE
                            ),
                            tags$label(
                              "Label size:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            sliderInput(
                              "network_label_size",
                              label = NULL,
                              min = 1,
                              max = 10,
                              value = 5,
                              step = 1,
                              width = "100px",
                              ticks = FALSE
                            ),
                            tags$label(
                              "Spacing:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            sliderInput(
                              "network_spacing",
                              label = NULL,
                              min = 1,
                              max = 10,
                              value = 5,
                              step = 1,
                              width = "100px",
                              ticks = FALSE
                            ),
                            tags$label(
                              "Repulsion:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            sliderInput(
                              "network_repulsion",
                              label = NULL,
                              min = 1,
                              max = 10,
                              value = 5,
                              step = 1,
                              width = "100px",
                              ticks = FALSE
                            ),
                            tags$label(
                              "Max edge dist.:",
                              style = "margin: 0; font-size: 12px; color: #555; white-space: nowrap;"
                            ),
                            numericInput(
                              "network_edge_distance",
                              label = NULL,
                              value = 1000,
                              min = 100,
                              max = 2000,
                              step = 100,
                              width = "80px"
                            ),
                            actionButton(
                              "update_network_layout",
                              "Update",
                              icon = icon("refresh"),
                              class = "btn-primary btn-sm",
                              style = "padding: 3px 12px;"
                            )
                          )
                        ),
                        div(
                          style = "height: 75vh; min-height: 700px; border: 1px solid #ddd; border-radius: 5px;",
                          visNetworkOutput(
                            "citation_network",
                            width = "100%",
                            height = "100%"
                          )
                        )
                      )
                    )
                  ),
                  fluidRow(
                    column(
                      6,
                      div(
                        class = "box box-info",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Network Information",
                            class = "box-title",
                            style = "color: #3498db;"
                          )
                        ),
                        div(
                          class = "box-body",
                          verbatimTextOutput("network_info")
                        )
                      )
                    ),
                    column(
                      6,
                      div(
                        class = "box box-success",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Strongest Connections",
                            class = "box-title",
                            style = "color: #27ae60;"
                          )
                        ),
                        div(
                          class = "box-body",
                          uiOutput("strongest_connections")
                        )
                      )
                    )
                  )
                ),

                # ===========================================
                # TAB 4: CITATION CLUSTERS
                # ===========================================
                tabPanel(
                  title = "Citation Clusters",
                  value = "tab_clusters",

                  br(),

                  # Cluster Summary Table
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Citation Cluster Analysis",
                            class = "box-title",
                            style = "color: #2E86AB;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            downloadButton(
                              "download_clusters",
                              "Export Data",
                              class = "btn btn-primary btn-sm",
                              icon = icon("download")
                            )
                          )
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.clusters_available",
                            div(
                              style = "margin-bottom: 15px; padding: 10px; background-color: #f0f8ff; border-radius: 5px;",
                              icon("info-circle", style = "color: #3498db;"),
                              span(
                                " Citation clusters are described using TF-IDF analysis of reference titles grouped by document section.",
                                style = "color: #555; font-size: 13px; margin-left: 8px;"
                              )
                            ),
                            uiOutput("cluster_summary_table")
                          ),
                          conditionalPanel(
                            condition = "!output.clusters_available",
                            div(
                              style = "text-align: center; padding: 60px; color: #999;",
                              icon(
                                "project-diagram",
                                style = "font-size: 48px; margin-bottom: 20px;"
                              ),
                              h4(
                                "No cluster data available",
                                style = "color: #666;"
                              ),
                              p(
                                "Citation cluster analysis will appear here after the content analysis is complete.",
                                style = "font-size: 14px;"
                              ),
                              p(
                                "Clusters are based on TF-IDF analysis of reference titles grouped by document section.",
                                style = "font-size: 12px; color: #999;"
                              )
                            )
                          )
                        )
                      )
                    )
                  ),

                  # TF-IDF Bar Chart
                  conditionalPanel(
                    condition = "output.clusters_available",
                    fluidRow(
                      column(
                        12,
                        div(
                          class = "box box-success",
                          div(
                            class = "box-header with-border",
                            h4(
                              "Top TF-IDF Terms by Section",
                              class = "box-title",
                              style = "color: #27ae60;"
                            )
                          ),
                          div(
                            class = "box-body",
                            uiOutput("cluster_tfidf_bars_ui")
                          )
                        )
                      )
                    ),

                    # Heatmap and References per Section side by side
                    fluidRow(
                      column(
                        7,
                        div(
                          class = "box box-info",
                          div(
                            class = "box-header with-border",
                            h4(
                              "TF-IDF Heatmap: Terms vs Sections",
                              class = "box-title",
                              style = "color: #3498db;"
                            )
                          ),
                          div(
                            class = "box-body",
                            plotlyOutput("cluster_tfidf_heatmap", height = "600px")
                          )
                        )
                      ),
                      column(
                        5,
                        div(
                          class = "box box-warning",
                          div(
                            class = "box-header with-border",
                            h4(
                              "References per Section",
                              class = "box-title",
                              style = "color: #e67e22;"
                            )
                          ),
                          div(
                            class = "box-body",
                            plotlyOutput("cluster_refs_per_section", height = "600px")
                          )
                        )
                      )
                    ),

                    # Detailed cluster references table
                    fluidRow(
                      column(
                        12,
                        div(
                          class = "box box-default",
                          div(
                            class = "box-header with-border",
                            h4(
                              "References by Section",
                              class = "box-title",
                              style = "color: #555;"
                            )
                          ),
                          div(
                            class = "box-body",
                            uiOutput("cluster_references_detail")
                          )
                        )
                      )
                    )
                  )
                ),

                # ===========================================
                # TAB 5: REFERENCES
                # ===========================================
                tabPanel(
                  title = "References",
                  value = "tab_references",

                  br(),

                  # Header con info e download
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h4(
                            "Bibliography",
                            class = "box-title",
                            style = "color: #2E86AB;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            downloadButton(
                              "download_references",
                              "Export References",
                              class = "btn btn-primary btn-sm",
                              icon = icon("download")
                            )
                          )
                        ),
                        div(
                          class = "box-body",
                          conditionalPanel(
                            condition = "output.references_available",

                            # Summary info
                            div(
                              style = "background-color: #e8f4f8; padding: 15px; border-radius: 5px; margin-bottom: 20px;",
                              fluidRow(
                                column(
                                  3,
                                  div(
                                    style = "text-align: center;",
                                    icon(
                                      "book",
                                      style = "font-size: 24px; color: #2E86AB;"
                                    ),
                                    h4(
                                      textOutput("total_refs", inline = TRUE),
                                      style = "color: #2E86AB; margin: 10px 0 5px 0;"
                                    ),
                                    p(
                                      "Total References",
                                      style = "color: #666; margin: 0;"
                                    )
                                  )
                                ),
                                column(
                                  3,
                                  div(
                                    style = "text-align: center;",
                                    icon(
                                      "file-pdf",
                                      style = "font-size: 24px; color: #e74c3c;"
                                    ),
                                    h4(
                                      textOutput("pdf_refs", inline = TRUE),
                                      style = "color: #e74c3c; margin: 10px 0 5px 0;"
                                    ),
                                    p(
                                      "From PDF",
                                      style = "color: #666; margin: 0;"
                                    )
                                  )
                                ),
                                column(
                                  3,
                                  div(
                                    style = "text-align: center;",
                                    icon(
                                      "cloud-download-alt",
                                      style = "font-size: 24px; color: #27ae60;"
                                    ),
                                    h4(
                                      textOutput(
                                        "crossref_refs",
                                        inline = TRUE
                                      ),
                                      style = "color: #27ae60; margin: 10px 0 5px 0;"
                                    ),
                                    p(
                                      "From Crossref",
                                      style = "color: #666; margin: 0;"
                                    )
                                  )
                                ),
                                column(
                                  3,
                                  div(
                                    style = "text-align: center;",
                                    icon(
                                      "database",
                                      style = "font-size: 24px; color: #8e44ad;"
                                    ),
                                    h4(
                                      textOutput(
                                        "openalex_refs",
                                        inline = TRUE
                                      ),
                                      style = "color: #8e44ad; margin: 10px 0 5px 0;"
                                    ),
                                    p(
                                      "From OpenAlex",
                                      style = "color: #666; margin: 0;"
                                    )
                                  )
                                )
                              )
                            ),

                            # Search box
                            fluidRow(
                              column(
                                12,
                                textInput(
                                  "reference_search",
                                  label = NULL,
                                  placeholder = "Search in references (author, title, year, DOI)...",
                                  width = "100%"
                                )
                              )
                            ),

                            hr(),

                            # References list
                            uiOutput("references_html")
                          ),

                          conditionalPanel(
                            condition = "!output.references_available",
                            div(
                              style = "text-align: center; padding: 60px; color: #999;",
                              icon(
                                "book-open",
                                style = "font-size: 48px; margin-bottom: 20px;"
                              ),
                              h4(
                                "No references available",
                                style = "color: #666;"
                              ),
                              p(
                                "References will appear here after the analysis is complete.",
                                style = "font-size: 14px;"
                              ),
                              p(
                                "References can be extracted from the PDF or fetched from Crossref using the document's DOI.",
                                style = "font-size: 12px; color: #999;"
                              )
                            )
                          )
                        )
                      )
                    )
                  )
                ),

                # ===========================================
                # TAB: BIBLIOAI SUMMARY - UI
                # ===========================================

                tabPanel(
                  title = "BiblioAI Summary",
                  value = "tab_biblioai",

                  br(),

                  # Header with API status, Summary Type Selection and Generate Button
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-primary",
                        div(
                          class = "box-header with-border",
                          h4(
                            "AI-Powered Document Summarization",
                            class = "box-title",
                            style = "color: #2E86AB;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            uiOutput("gemini_api_status")
                          )
                        ),
                        div(
                          class = "box-body",
                          fluidRow(
                            column(
                              6,
                              # Summary type selection with grouped options
                              selectInput(
                                "summary_type",
                                label = div(
                                  icon(
                                    "list-alt",
                                    style = "margin-right: 5px;"
                                  ),
                                  tags$strong("Summary Type:")
                                ),
                                choices = list(
                                  "Article Summarization" = list(
                                    "Short Abstract (250 words)" = "short_abstract",
                                    "Narrative Abstract (500-600 words)" = "narrative_abstract",
                                    "IMRaD Structure Summary" = "imrad_summary",
                                    "Thematic Bibliography" = "thematic_bibliography"
                                  ),
                                  "Focus on Article Parts" = list(
                                    "Research Questions & Context" = "research_questions",
                                    "Background & Literature" = "background_literature",
                                    "Methods Summary" = "methods_summary",
                                    "Implications & Conclusions" = "implications",
                                    "Tables & Figures List" = "list_tables_figures"
                                  )
                                ),
                                selected = "short_abstract",
                                width = "100%"
                              )
                            ),
                            column(4), # empty space for future options
                            column(
                              2,
                              # Generate button aligned with select input
                              div(
                                style = "margin-top: 25px;",
                                actionButton(
                                  "generate_summary",
                                  "Generate Summary",
                                  icon = icon("magic"),
                                  class = "btn-success btn-block",
                                  style = "font-weight: bold; padding: 10px;"
                                )
                              )
                            )
                          ),
                          # Progress indicator
                          conditionalPanel(
                            condition = "$('html').hasClass('shiny-busy')",
                            div(
                              style = "text-align: center; margin-top: 10px; padding: 10px; background-color: #f0f8f0; border-radius: 4px;",
                              icon(
                                "spinner",
                                class = "fa-spin",
                                style = "color: #27ae60; font-size: 20px; margin-right: 10px;"
                              ),
                              span(
                                "Generating summary with AI...",
                                style = "color: #27ae60; font-weight: 500;"
                              )
                            )
                          ),
                          # Summary type description
                          div(
                            style = "margin-top: 10px;",
                            uiOutput("summary_type_description")
                          )
                        )
                      )
                    )
                  ),

                  # Summary Results - Full Width
                  fluidRow(
                    column(
                      12,
                      div(
                        class = "box box-info",
                        div(
                          class = "box-header with-border",
                          h4(
                            "AI-Generated Summary",
                            class = "box-title",
                            style = "color: #3498db;"
                          ),
                          div(
                            class = "box-tools pull-right",
                            conditionalPanel(
                              condition = "output.summary_available",
                              downloadButton(
                                "download_summary",
                                "Export Summary",
                                class = "btn btn-info btn-sm",
                                icon = icon("download")
                              )
                            )
                          )
                        ),
                        div(
                          class = "box-body",
                          style = "min-height: 500px;",

                          conditionalPanel(
                            condition = "output.summary_available",
                            div(
                              style = "background-color: #f8f9fa; padding: 20px; border-radius: 5px; border: 1px solid #dee2e6;",

                              # Summary metadata
                              div(
                                style = "margin-bottom: 15px; padding-bottom: 10px; border-bottom: 2px solid #3498db;",
                                fluidRow(
                                  column(
                                    6,
                                    tags$strong(
                                      "Summary Type: ",
                                      style = "color: #3498db;"
                                    ),
                                    tags$span(
                                      textOutput(
                                        "summary_type_display",
                                        inline = TRUE
                                      ),
                                      style = "font-weight: bold;"
                                    )
                                  ),
                                  column(
                                    6,
                                    div(
                                      style = "text-align: right;",
                                      tags$strong(
                                        "Generated: ",
                                        style = "color: #3498db;"
                                      ),
                                      textOutput(
                                        "summary_timestamp",
                                        inline = TRUE
                                      )
                                    )
                                  )
                                )
                              ),

                              # Summary content
                              div(
                                style = "line-height: 1.8; color: #333; font-size: 14px; white-space: pre-wrap; word-wrap: break-word;",
                                uiOutput("summary_content_display")
                              )
                            )
                          ),

                          conditionalPanel(
                            condition = "!output.summary_available",
                            div(
                              style = "text-align: center; padding: 80px 20px; color: #999;",
                              icon(
                                "robot",
                                style = "font-size: 64px; margin-bottom: 20px; color: #bbb;"
                              ),
                              h4(
                                "No summary generated yet",
                                style = "color: #666; margin-bottom: 15px;"
                              ),
                              p(
                                "Select your summary type above and click 'Generate Summary' to start.",
                                style = "font-size: 14px; margin-bottom: 10px;"
                              ),
                              p(
                                "Make sure you have uploaded a PDF document first.",
                                style = "font-size: 12px; color: #999;"
                              )
                            )
                          )
                        )
                      )
                    )
                  )
                )
              )
            ),

            # Placeholder when no analysis is done
            conditionalPanel(
              condition = "!output.analysis_completed && !output.text_extracted",
              div(
                style = "text-align: center; margin-top: 100px; color: #999;",
                icon(
                  "file-upload",
                  style = "font-size: 48px; margin-bottom: 20px;"
                ),
                h3(
                  "Upload a PDF file and start the analysis",
                  style = "color: #666;"
                ),
                p(
                  "Select a scientific article in PDF format and configure the analysis parameters to begin.",
                  style = "font-size: 16px;"
                )
              )
            )
          ),

          # ===========================================
          # LEFT PANEL: CONTROLS (column 2)
          # ===========================================
          column(
            2,

            # Import Method Selection
            div(
              class = "box box-info",
              div(
                class = "box-body",
                style = "padding: 15px;",

                div(
                  style = "text-align: center; margin-bottom: 15px;",
                  icon(
                    "file-import",
                    style = "font-size: 28px; color: #2E86AB;"
                  ),
                  h4(
                    "Choose Import Method",
                    style = "color: #2E86AB; margin-top: 10px; margin-bottom: 5px; font-weight: bold;"
                  )
                ),

                radioButtons(
                  "import_method",
                  label = NULL,
                  choices = list(
                    "Import PDF File" = "import_pdf",
                    "Load Saved Text File" = "load_txt"
                  ),
                  selected = "import_pdf",
                  width = "100%"
                ),

                helpText(
                  "Select whether to extract text from a PDF or load a previously saved text file.",
                  style = "font-size: 11px; color: #666; text-align: center; margin-top: -5px;"
                )
              )
            ),

            # PDF Import Card - visible only if import_pdf is selected
            conditionalPanel(
              condition = "input.import_method == 'import_pdf'",
              div(
                class = "box box-primary",
                div(
                  class = "box-body",

                  tags$details(
                    id = "pdf_import_details",
                    open = NA, # Open by default
                    tags$summary(
                      "1. Import PDF File ▼",
                      style = "font-weight: bold; cursor: pointer; color: #2E86AB; font-size: 16px; padding: 8px 0;"
                    ),
                    br(),

                    fileInput(
                      "pdf_file",
                      label = "Choose PDF File",
                      accept = c(".pdf"),
                      buttonLabel = "Browse...",
                      placeholder = "No file selected",
                      width = "100%"
                    ),

                    numericInput(
                      "Columns",
                      label = "Number of Columns in PDF",
                      value = NULL,
                      min = 1,
                      max = 3,
                      step = 1,
                      width = "100%"
                    ),
                    helpText(
                      "Specify if the PDF has multiple columns (e.g., 2 for typical academic articles)."
                    ),

                    # MODIFIED: Citation Type Selection - NO DEFAULT
                    radioButtons(
                      "citation_type_import",
                      label = div(
                        tags$span(
                          "Citation Format in PDF:",
                          style = "color: #d9534f; font-weight: bold;"
                        ),
                        tags$span(
                          " *",
                          style = "color: #d9534f; font-size: 16px;"
                        ),
                        tags$a(
                          icon("info-circle"),
                          href = "#",
                          onclick = "return false;",
                          style = "margin-left: 5px; color: #3498db; cursor: help;",
                          title = "Choose the citation format used in your document. This helps avoid false positives in citation detection."
                        )
                      ),
                      choices = list(
                        "Author-year (Smith, 2020)" = "author_year",
                        "Numeric brackets [1]" = "numeric_bracketed",
                        "Numeric superscript¹" = "numeric_superscript"
                        # ,"All formats (may have false positives)" = "all"
                      ),
                      selected = character(0), # NO DEFAULT SELECTION
                      width = "100%"
                    ),

                    # Warning message when no selection
                    conditionalPanel(
                      condition = "!input.citation_type_import || input.citation_type_import == ''",
                      div(
                        style = "background-color: #fcf8e3; padding: 8px 12px; border-radius: 4px; border-left: 4px solid #f0ad4e; margin-top: -8px; margin-bottom: 10px;",
                        icon("exclamation-triangle", style = "color: #8a6d3b;"),
                        span(
                          " Please select the citation format before extracting text.",
                          style = "color: #8a6d3b; margin-left: 8px; font-size: 12px; font-weight: 500;"
                        )
                      )
                    ),

                    # AI Support checkbox - visible only when Gemini API is available
                    conditionalPanel(
                      condition = "output.gemini_api_available",
                      div(
                        style = "margin-top: 12px; margin-bottom: 12px; background-color: #e8f8f5; padding: 15px; border-radius: 6px; border-left: 4px solid #17a589;",
                        div(
                          style = "display: flex; align-items: center; gap: 10px;",
                          div(
                            style = "flex-shrink: 0;",
                            checkboxInput(
                              "enable_ai_support",
                              label = NULL,
                              value = FALSE,
                              width = "auto"
                            )
                          ),
                          div(
                            style = "display: flex; align-items: center; flex-grow: 1;",
                            icon(
                              "robot",
                              style = "color: #17a589; margin-right: 10px; font-size: 22px;"
                            ),
                            span(
                              "Enable AI-Enhanced Extraction",
                              style = "font-weight: bold; color: #17a589; font-size: 15px;"
                            )
                          )
                        ),
                        helpText(
                          "Use advanced AI to improve text extraction quality and citation detection accuracy.",
                          style = "margin-top: 8px; margin-bottom: 0px; margin-left: 35px; font-size: 11px; color: #555;"
                        )
                      )
                    ),

                    # Processing spinner - visible ONLY during PDF text extraction (not during analysis)
                    conditionalPanel(
                      condition = "$('html').hasClass('shiny-busy') && (input.extract_text > 0) && !output.text_extracted",
                      div(
                        style = "margin-top: 10px; margin-bottom: 10px; background-color: #fff3cd; padding: 15px; border-radius: 5px; border-left: 4px solid #ffc107; text-align: center;",
                        div(
                          icon(
                            "spinner",
                            class = "fa-spin",
                            style = "font-size: 24px; color: #ff9800; margin-bottom: 10px;"
                          ),
                          br(),
                          tags$strong(
                            "Extracting Text from PDF...",
                            style = "color: #856404; font-size: 14px;"
                          ),
                          br(),
                          tags$span(
                            id = "extraction_status_text",
                            "Please wait while we extract the document content.",
                            style = "color: #856404; font-size: 12px; font-style: italic;"
                          )
                        )
                      )
                    ),

                    # File info display
                    conditionalPanel(
                      condition = "output.pdf_uploaded",
                      div(
                        style = "background-color: #f0f8ff; padding: 10px; border-radius: 5px; margin-top: 10px;",
                        icon("file-pdf", style = "color: #e74c3c;"),
                        span(
                          " PDF uploaded successfully!",
                          style = "color: #27ae60; font-weight: bold;"
                        ),
                        br(),
                        textOutput("pdf_info", inline = TRUE)
                      )
                    ),

                    br(),
                    actionButton(
                      "extract_text",
                      "Extract Text from PDF",
                      icon = icon("file-text"),
                      class = "btn-info btn-block",
                      style = "margin-top: 10px;"
                    )
                  )
                )
              )
            ),
            # End of conditionalPanel for import_pdf

            # Load Text File Card - visible only if load_txt is selected
            conditionalPanel(
              condition = "input.import_method == 'load_txt'",
              div(
                class = "box box-primary",
                div(
                  class = "box-body",
                  tags$details(
                    open = NA, # Open by default
                    tags$summary(
                      "1. Load Saved Text File ▼",
                      style = "font-weight: bold; color: #2E86AB; font-size: 16px; padding: 8px 0; margin-bottom: 15px; display: block;"
                    ),

                    helpText(
                      "Load a .txt file that was previously saved from this tool. The file should have DOI and citation format info in the first lines for automatic configuration.",
                      style = "font-size: 12px; color: #666; margin-bottom: 15px;"
                    ),

                    fileInput(
                      "load_text_file",
                      "Select Text File",
                      accept = c("text/plain", ".txt"),
                      buttonLabel = "Browse...",
                      placeholder = "No file selected",
                      width = "100%"
                    ),

                    # File info display for loaded text
                    conditionalPanel(
                      condition = "output.text_file_loaded",
                      div(
                        style = "background-color: #f0f8ff; padding: 10px; border-radius: 5px; margin-top: 10px;",
                        icon("file-alt", style = "color: #3498db;"),
                        span(
                          " Text file loaded successfully!",
                          style = "color: #27ae60; font-weight: bold;"
                        ),
                        br(),
                        textOutput("text_file_info", inline = TRUE)
                      )
                    )
                  )
                )
              )
            ),
            # End of conditionalPanel for load_txt

            # Analysis Parameters Card
            div(
              class = "box box-success",
              div(
                class = "box-body",

                tags$details(
                  tags$summary(
                    "2. Analysis Parameters ▼",
                    style = "font-weight: bold; cursor: pointer; color: #27ae60; font-size: 16px; padding: 8px 0;"
                  ),
                  br(),

                  # Citation context window
                  numericInput(
                    "window_size",
                    label = "Context Window Size (words)",
                    value = 20,
                    min = 5,
                    max = 50,
                    step = 1,
                    width = "100%"
                  ),
                  helpText(
                    "Number of words before and after each citation to extract."
                  ),

                  # Maximum distance for network
                  numericInput(
                    "max_distance",
                    label = "Max Distance for Network (chars)",
                    value = 800,
                    min = 200,
                    max = 2000,
                    step = 100,
                    width = "100%"
                  ),
                  helpText(
                    "Maximum character distance between citations to consider as connected."
                  ),

                  # Advanced options - OPEN BY DEFAULT
                  tags$details(
                    open = NA,
                    tags$summary(
                      "Advanced Options",
                      style = "font-weight: bold; cursor: pointer;"
                    ),
                    br(),
                    checkboxInput(
                      "parse_multiple",
                      "Parse complex multiple citations",
                      value = TRUE
                    ),
                    checkboxInput(
                      "remove_stopwords",
                      "Remove stopwords from analysis",
                      value = TRUE
                    ),
                    textInput(
                      "custom_stopwords",
                      "Custom stopwords (comma-separated)",
                      value = "",
                      width = "100%"
                    )
                  )
                )
              )
            ),

            # Analysis Button
            div(
              class = "box box-warning",
              div(
                class = "box-header with-border",
                h4(
                  "3. Run Analysis",
                  class = "box-title",
                  style = "color: #f39c12;"
                )
              ),
              div(
                class = "box-body",
                actionButton(
                  "run_analysis",
                  "Start",
                  icon = icon("chart-line"),
                  class = "btn-warning btn-block btn-lg",
                  style = "font-weight: bold; margin-bottom: 10px;"
                ),

                # Progress indicator - ONLY during analysis
                conditionalPanel(
                  condition = "$('html').hasClass('shiny-busy') && (input.run_analysis > 0) && !output.analysis_completed",
                  div(
                    style = "text-align: center; margin-top: 15px;",
                    icon(
                      "spinner",
                      class = "fa-spin",
                      style = "color: #f39c12;"
                    ),
                    br(),
                    span(
                      "Analyzing content...",
                      style = "color: #f39c12; font-style: italic;"
                    )
                  )
                ),

                # Reset button
                actionButton(
                  "reset_analysis",
                  "Reset",
                  icon = icon("refresh"),
                  class = "btn-default btn-block",
                  style = "margin-top: 10px;"
                )
              )
            )
          )
        )
        # )
      ), # Close Import PDF File tabPanel

      # ===========================================
      # MAIN TAB 2: INFO & REFERENCES
      # ===========================================
      tabPanel(
        title = "Info & References",
        value = "tab_info_references",
        icon = icon("info-circle"),
        fluidPage(
          fluidRow(
            column(1),
            column(
              10,
              br(),
              HTML(helpContent()$contentAnalysis)
            ),
            column(1)
          )
        )
      )
    ), # Close main tabsetPanel

    # Modal for Preview (Bootstrap native)
    tags$div(
      class = "modal fade",
      id = "previewModal",
      tabindex = "-1",
      role = "dialog",

      tags$div(
        class = "modal-dialog modal-lg",
        role = "document",
        style = "width: 90%; max-width: 1200px;",

        tags$div(
          class = "modal-content",

          # Header
          tags$div(
            class = "modal-header",
            tags$button(
              type = "button",
              class = "close",
              `data-dismiss` = "modal",
              `aria-label` = "Close",
              tags$span(`aria-hidden` = "true", HTML("&times;"))
            ),
            tags$h4(class = "modal-title", "Text & PDF Preview")
          ),

          # Body
          tags$div(
            class = "modal-body",
            style = "max-height: 75vh; overflow-y: auto;",

            tabsetPanel(
              id = "modal_preview_tabs",

              tabPanel(
                "Extracted Text",
                br(),
                div(
                  style = "margin-bottom: 10px;",
                  span(
                    textOutput("text_length_info", inline = TRUE),
                    style = "font-size: 12px; color: #666;"
                  )
                ),
                div(
                  style = paste0(
                    "background-color: #f8f9fa; padding: 15px; ",
                    "border: 1px solid #e9ecef; border-radius: 4px; ",
                    "font-family: 'Consolas', 'Monaco', monospace; ",
                    "font-size: 13px; line-height: 1.4; ",
                    "white-space: pre-wrap; word-wrap: break-word;"
                  ),
                  textOutput("text_preview")
                )
              ),

              tabPanel(
                "PDF Preview",
                br(),
                uiOutput("pdf_viewer")
              )
            )
          ),

          # Footer
          tags$div(
            class = "modal-footer",
            tags$button(
              type = "button",
              class = "btn btn-default",
              `data-dismiss` = "modal",
              "Close"
            )
          )
        )
      )
    ),

    # Modal to manage OpenAlex Reference Cards
    tags$div(
      class = "modal fade",
      id = "oaDetailsModal",
      tabindex = "-1",
      role = "dialog",

      tags$div(
        class = "modal-dialog modal-lg",
        role = "document",
        style = "width: 90%; max-width: 1000px;",

        tags$div(
          class = "modal-content",

          # Header
          tags$div(
            class = "modal-header",
            style = "background: linear-gradient(135deg, #1a5f7a 0%, #2874a6 100%); padding: 20px 25px;",
            tags$button(
              type = "button",
              class = "close",
              `data-dismiss` = "modal",
              `aria-label` = "Close",
              tags$span(
                `aria-hidden` = "true",
                style = "color: white !important; font-size: 32px; opacity: 0.8;",
                HTML("&times;")
              )
            ),
            tags$h4(
              class = "modal-title",
              id = "oaModalTitle",
              style = "margin: 0; padding-right: 30px; color: white !important; font-weight: 600;",
              "Document Details"
            )
          ),

          # Body
          tags$div(
            class = "modal-body",
            id = "oaModalBody",
            style = "max-height: 70vh; overflow-y: auto; padding: 20px;",
            div(
              style = "text-align: center; padding: 40px;",
              icon(
                "spinner",
                class = "fa-spin",
                style = "font-size: 32px; color: #3498db;"
              ),
              p(
                "Loading document details...",
                style = "margin-top: 15px; color: #666;"
              )
            )
          ),

          # Footer
          tags$div(
            class = "modal-footer",
            tags$button(
              type = "button",
              class = "btn btn-default",
              `data-dismiss` = "modal",
              "Close"
            )
          )
        )
      )
    ),

    tags$style(HTML(
      "
  #previewModal .modal-header,
  #oaDetailsModal .modal-header {
    background: linear-gradient(135deg, #1a5f7a 0%, #2874a6 100%) !important;
  }
  
  #previewModal .modal-header .modal-title,
  #oaDetailsModal .modal-header .modal-title,
  #previewModal .modal-header h4,
  #oaDetailsModal .modal-header h4 {
    color: white !important;
    font-weight: 600 !important;
    font-size: 18px !important;
    line-height: 1.4 !important;
  }
  
  #previewModal .modal-header .close,
  #oaDetailsModal .modal-header .close,
  #previewModal .modal-header .close span,
  #oaDetailsModal .modal-header .close span {
    color: white !important;
    opacity: 0.8 !important;
    text-shadow: none !important;
  }
  
  #previewModal .modal-header .close:hover,
  #oaDetailsModal .modal-header .close:hover {
    opacity: 1 !important;
  }
  
  /* Responsive for small screens */
  @media (max-width: 768px) {
    #oaDetailsModal .modal-header .modal-title {
      font-size: 16px !important;
    }
  }
"
    )),

    tags$style(HTML(
      "
  /* Responsive layout for statistics boxes */
  @media (max-width: 992px) {
    .small-box {
      margin-bottom: 15px;
    }
  }
  
  @media (max-width: 768px) {
    .small-box h3 {
      font-size: 28px !important;
    }
    
    .small-box i {
      font-size: 32px !important;
    }
  }
"
    )),

    # JavaScript for OpenAlex  modal
    tags$script(HTML(
      "
  // Function to show OpenAlex details
  function showOADetails(refIndex) {
    // Reset modal content
    $('#oaModalBody').html('<div style=\"text-align: center; padding: 40px;\"><i class=\"fa fa-spinner fa-spin\" style=\"font-size: 32px; color: #3498db;\"></i><p style=\"margin-top: 15px; color: #666;\">Loading document details...</p></div>');
    
    // Send the index to Shiny
    Shiny.setInputValue('selected_oa_ref_index', refIndex, {priority: 'event'});
    
    // Open the modal
    $('#oaDetailsModal').modal('show');
  }
  
  // Custom handler to update modal content
  Shiny.addCustomMessageHandler('updateOAModal', function(data) {
    $('#oaModalTitle').html(data.title);
    $('#oaModalBody').html(data.content);
  });
"
    )),

    tags$script(HTML(
      "
  // Function to open/close the PDF Import box
  Shiny.addCustomMessageHandler('togglePdfImport', function(message) {
    var details = document.getElementById('pdf_import_details');
    if (details) {
      if (message.action === 'close') {
        details.removeAttribute('open');
      } else if (message.action === 'open') {
        details.setAttribute('open', '');
      }
    }
  });
"
    ))
  )
}

helpContent <- function(){
  biblioAI <- 
    "<h3>🧠 Biblio AI: AI-Powered Bibliometric Analysis</h3>
  
  <p>Starting from version <strong>5.0</strong>, <em>Biblioshiny</em> introduces <strong>Biblio AI</strong>, a new suite of features powered by Google’s <strong>Gemini</strong> models. This integration allows users to receive <em>automatic interpretations</em>, <em>critical insights</em>, and <em>narrative summaries</em> of their bibliometric results — directly within the platform.</p>
  <br>
  <p><strong>Note:</strong> Biblio AI requires a <strong>Chrome-based browser</strong> (such as Google Chrome or Microsoft Edge) installed on your computer to work correctly.</p>
  
  <br>
  <h4>✨ What does Biblio AI do?</h4>
  <p>Biblio AI enhances the core analytical modules of <em>Biblioshiny</em> by providing contextual, AI-generated commentary on several results, as:</p>
  <ul>
  <li><strong>Overview:</strong> High-level summaries of key bibliometric indicators and collection features.</li>
  <li><strong>Three-Field Plot:</strong> Interpretation of the connections among sources, authors, and keywords.</li>
  <li><strong>Authors' Production over Time:</strong> Insights on temporal dynamics and productivity patterns of key authors.</li>
  <li><strong>Corresponding Author's Countries Collaboration:</strong> Discussion of international scientific collaboration patterns.</li>
  <li><strong>Most Local Cited Documents:</strong> Evaluation of the most influential documents within the dataset.</li>
  <li><strong>Reference Publication Year Spectroscopy:</strong> Identification and interpretation of historical citation peaks.</li>
  <li><strong>Trend Topics:</strong> Explanation of thematic evolution and detection of emerging research trends.</li>
  <li><strong>Knowledge Structures:</strong> Analysis of conceptual maps and networks such as co-citation and co-word analysis.</li>
  <li><strong>Country Collaboration World Map:</strong> AI-assisted reading of global co-authorship and geographical patterns.</li>
  </ul>
  <p>In each of these sections, users can activate the <strong>Biblio AI</strong> panel to access dynamic text explanations, perfect for use in scientific writing, presentations, or reporting.</p>
  <br>
  <h4>🔧 How to enable Biblio AI?</h4>
  <p>To enable Biblio AI, follow these simple steps:</p>
  <ol>
  <li><strong>Register</strong> at <a href='https://makersuite.google.com/' target='_blank'>Google AI Studio</a> (free access available).</li>
    <li><strong>Generate an API Key</strong> enabled for Gemini model access (Free Tier supported).</li>
    <li><strong>Enter your API Key</strong> in the <em>Settings</em> section of <em>biblioshiny</em>.</li>
    </ol>
    <p>The interface will guide you through the secure and local setup. Your API key is used only on your device to interact with the AI model.</p>
    <br>
    <h4>🎯 Why use Biblio AI?</h4>
    <ul>
    <li>Reduces time spent interpreting complex outputs.</li>
    <li>Supports scientific writing and research reporting.</li>
    <li>Helps users better understand bibliometric patterns and dynamics.</li>
    <li>Delivers explanations in natural language, accessible to both experts and newcomers.</li>
    </ul>
"
    
  info <- 
    "<h3><strong>Supported Bibliographic Databases and Suggested File Formats</strong></h3>
  
  <p><strong>Biblioshiny</strong> imports and analyzes collections exported from the following bibliographic databases:</p>
  <ul>
  <li><a href='https://www.webofscience.com' target='_blank'>Web of Science</a></li>
    <li><a href='https://www.scopus.com' target='_blank'>Scopus</a></li>
      <li><a href='https://www.openalex.org' target='_blank'>OpenAlex</a></li>
        <li><a href='https://www.dimensions.ai' target='_blank'>Dimensions</a></li>
          <li><a href='https://www.lens.org' target='_blank'>The Lens</a></li>
            <li><a href='https://pubmed.ncbi.nlm.nih.gov' target='_blank'>PubMed</a></li>
              <li><a href='https://www.cochranelibrary.com' target='_blank'>Cochrane Library</a></li>
                </ul>
                
                <p><strong>Web of Science</strong>, <strong>Scopus</strong>, and <strong>OpenAlex</strong> allow users to export the complete set of metadata, making it possible to perform all analyses implemented in <strong>Biblioshiny</strong>.</p>
                
                <p>Some other databases, such as <strong>Dimensions</strong>, <strong>PubMed</strong>, and <strong>Cochrane Library</strong>, provide only a limited set of metadata. This may impose restrictions on the range of analyses that can be conducted using those datasets.</p>
                
                <p>The following table (not included here) reports, for each supported database:</p>
                <ul>
                <li>The <strong>file formats supported</strong> by the export interface</li>
                <li>The <strong>types of metadata</strong> contained in each export option</li>
                <li>The <strong>suggested file format</strong> to use with <strong>Biblioshiny</strong></li>
                </ul>"


  
  publications <-
    "<h3><strong>Main Authors’ References (Bibliometrics)</strong></h3>
  
  <ul>
  <li><strong>Aria, M., & Cuccurullo, C. (2017).</strong> <i>bibliometrix: An R-tool for comprehensive science mapping analysis.</i> <strong>Journal of Informetrics</strong>, 11(4), 959–975. <a href='https://doi.org/10.1016/j.joi.2017.08.007' target='_blank'>https://doi.org/10.1016/j.joi.2017.08.007</a></li>
    
    <li><strong>Aria, M., Cuccurullo, C., D'Aniello, L., Misuraca, M., & Spano, M. (2024).</strong> <i>Comparative science mapping: a novel conceptual structure analysis with metadata.</i> <strong>Scientometrics</strong>. <a href='https://doi.org/10.1007/s11192-024-05161-6' target='_blank'>https://doi.org/10.1007/s11192-024-05161-6</a></li>

  <li><strong>Aria, M., Le, T., Cuccurullo, C., Belfiore, A., & Choe, J. (2023).</strong> <i>openalexR: An R-Tool for Collecting Bibliometric Data from OpenAlex.</i> <strong>R Journal</strong>, 15(4). <a href='https://doi.org/10.32614/rj-2023-089' target='_blank'>https://doi.org/10.32614/rj-2023-089</a></li>

  <li><strong>Aria, M., Misuraca, M., & Spano, M. (2020).</strong> <i>Mapping the evolution of social research and data science on 30 years of Social Indicators Research.</i> <strong>Social Indicators Research</strong>. <a href='https://doi.org/10.1007/s11205-020-02281-3' target='_blank'>https://doi.org/10.1007/s11205-020-02281-3</a></li>

  <li><strong>Aria, M., Cuccurullo, C., D’Aniello, L., Misuraca, M., & Spano, M. (2022).</strong> <i>Thematic Analysis as a New Culturomic Tool: The Social Media Coverage on COVID-19 Pandemic in Italy.</i> <strong>Sustainability</strong>, 14(6), 3643. <a href='https://doi.org/10.3390/su14063643' target='_blank'>https://doi.org/10.3390/su14063643</a></li>

  <li><strong>Aria, M., Alterisio, A., Scandurra, A., Pinelli, C., & D'Aniello, B. (2021).</strong> <i>The scholar’s best friend: research trends in dog cognitive and behavioural studies.</i> <strong>Animal Cognition</strong>. <a href='https://doi.org/10.1007/s10071-020-01448-2' target='_blank'>https://doi.org/10.1007/s10071-020-01448-2</a></li>
    
    <li><strong>Cuccurullo, C., Aria, M., & Sarto, F. (2016).</strong> <i>Foundations and trends in performance management: A twenty-five years bibliometric analysis in business and public administration domains.</i> <strong>Scientometrics</strong>. <a href='https://doi.org/10.1007/s11192-016-1948-8' target='_blank'>https://doi.org/10.1007/s11192-016-1948-8</a></li>
      
      <li><strong>Cuccurullo, C., Aria, M., & Sarto, F. (2015).</strong> <i>Twenty years of research on performance management in business and public administration domains.</i> Presented at CARME 2015. <a href='https://www.bibliometrix.org/documents/2015Carme_cuccurulloetal.pdf' target='_blank'>Link</a></li>
        
        <li><strong>Sarto, F., Cuccurullo, C., & Aria, M. (2014).</strong> <i>Exploring healthcare governance literature: systematic review and paths for future research.</i> <strong>Mecosan</strong>. <a href='https://www.francoangeli.it/Riviste/Scheda_Rivista.aspx?IDarticolo=52780&lingua=en' target='_blank'>Link</a></li>
          
          <li><strong>Cuccurullo, C., Aria, M., & Sarto, F. (2013).</strong> <i>Twenty years of research on performance management in business and public administration domains.</i> <strong>Academy of Management Proceedings</strong>, Vol. 2013, No. 1, p. 14270. <a href='https://doi.org/10.5465/AMBPP.2013.14270abstract' target='_blank'>https://doi.org/10.5465/AMBPP.2013.14270abstract</a></li>
            
            <li><strong>Belfiore, A., Salatino, A., & Osborne, F. (2022).</strong> <i>Characterising Research Areas in the field of AI.</i> <strong>arXiv</strong> preprint. <a href='https://doi.org/10.48550/arXiv.2205.13471' target='_blank'>https://doi.org/10.48550/arXiv.2205.13471</a></li>
              
              <li><strong>Belfiore, A., Cuccurullo, C., & Aria, M. (2022).</strong> <i>IoT in healthcare: A scientometric analysis.</i> <strong>Technological Forecasting and Social Change</strong>, 184, 122001. <a href='https://doi.org/10.1016/j.techfore.2022.122001' target='_blank'>https://doi.org/10.1016/j.techfore.2022.122001</a></li>
                
                <li><strong>D'Aniello, L., Spano, M., Cuccurullo, C., & Aria, M. (2022).</strong> <i>Academic Health Centers’ configurations, scientific productivity, and impact: insights from the Italian setting.</i> <strong>Health Policy</strong>. <a href='https://doi.org/10.1016/j.healthpol.2022.09.007' target='_blank'>https://doi.org/10.1016/j.healthpol.2022.09.007</a></li>

  <li><strong>Belfiore, A., Scaletti, A., Lavorato, D., & Cuccurullo, C. (2022).</strong> <i>The long process by which HTA became a paradigm: A longitudinal conceptual structure analysis.</i> <strong>Health Policy</strong>. <a href='https://doi.org/10.1016/j.healthpol.2022.12.006' target='_blank'>https://doi.org/10.1016/j.healthpol.2022.12.006</a></li>
</ul>
"

  
  return(list(biblioAI=biblioAI, info=info, publications=publications))
}

#' Generate a specific prompt for summarizing a scientific article with Gemini
#'
#' @param summary_type A character string specifying the type of summary.
#'   Valid options are: "short_abstract", "narrative_abstract", "imrad_summary",
#'   "thematic_bibliography", "research_questions", "background_literature",
#'   "methods_summary", "implications", "list_tables_figures".
#' @return A character string containing the prompt for the Gemini API.
#' @examples
#' # Get the prompt for a short abstract
#' prompt <- get_article_summary_prompt("short_abstract")
#' cat(prompt)
#'
#' # Get the prompt for extracting methods
#' prompt_methods <- get_article_summary_prompt("methods_summary")
#' cat(prompt_methods)

get_article_summary_prompt <- function(summary_type) {
  # Use a switch statement to map the input to the correct prompt
  prompt <- switch(
    summary_type,
    # Article summarization prompts
    "short_abstract" = {
      "Based on the provided scientific article, generate a concise, one-paragraph abstract (around 250 words) summarizing the study's primary objective, methods, key findings, and main conclusions. The summary should be suitable for a quick overview of the research."
    },
    "narrative_abstract" = {
      "From the attached scientific article, write a narrative extended abstract of no more than 500-600 words. Instead of a formal summary, tell the story of the research. Begin with the problem that motivated the study, describe the research process (methods), reveal the key discoveries (results), and conclude with the broader meaning and implications of the findings. The tone should be engaging and accessible."
    },
    "imrad_summary" = {
      "Please analyze the provided article and summarize it following the IMRaD structure. Create a distinct section for each of the following, using these exact headings:\n- **Introduction:** State the research problem, objectives, and its significance.\n- **Methods:** Describe the methodology, data collection, and analysis procedures.\n- **Results:** Present the main findings of the study without interpretation.\n- **Discussion:** Interpret the results, discuss their implications, mention limitations, and suggest future research."
    },
    "thematic_bibliography" = {
      "Analyze the bibliography/reference list of the attached scientific article. Identify the main research themes present in the cited works and group the references accordingly. For each theme, provide a brief description followed by a list of the corresponding citations from the article's bibliography. If the citations within the scientific article's text are numbered, report the author(s), title, and journal for each reference instead of just the number."
    },
    # Focus on article parts: questions, background, methods, implications, tables/figures
    "research_questions" = {
      "Extract the following specific information from the provided article. Present the answer under these exact headings:\n- **Research Questions:** List the explicit or implicit research questions the study aims to answer.\n- **Research Context:** Describe the setting, population, or specific environment in which the research was conducted.\n- **Motivation:** Explain the gap in knowledge, problem, or observation that motivated this research."
    },
    "background_literature" = {
      "Summarize the 'Background,' 'Literature Review,' or 'Theoretical Framework' section of the attached article. Identify and explain the key concepts, theories, and previous research that form the foundation for this study. Highlight the scholarly conversation the authors are contributing to."
    },
    "methods_summary" = {
      "Provide a detailed summary of the methodology used in the provided article. Structure your response with the following headings:\n- **Research Design:** Describe the overall approach (e.g., experimental, survey, case study).\n- **Data Collection:** Explain how the data was gathered, including the instruments or protocols used.\n- **Data Analysis:** Detail the statistical or qualitative techniques used to analyze the data.\n- **Tools:** List any specific software or equipment mentioned."
    },
    "implications" = {
      "Based on the 'Discussion' and 'Conclusion' sections of the article, identify and summarize the main conclusions and implications of the research findings. Please organize them into three distinct categories using these headings: 
**Conclusions**: What are the main findings or definitive statements presented by the authors? 
**Theoretical Implications**: How do the findings contribute to, challenge, or extend existing theories? 
**Practical Implications**: What are the real-world applications, recommendations for practice, or policy suggestions derived from the results?"
    },
    "list_tables_figures" = {
      "Scan the entire document and extract a complete list of all tables and figures. For each item, provide its number and its full title or caption exactly as it appears in the text (e.g., 'Table 1: Participant Demographics' or 'Figure 1: Conceptual Model') and a brief description of the content."
    },
    # Default case for invalid input
    {
      stop(paste(
        "Invalid 'summary_type'. Please use one of:",
        "short_abstract, narrative_abstract, imrad_summary,",
        "thematic_bibliography, research_questions, background_literature,",
        "methods_summary, implications, list_tables_figures"
      ))
    }
  )
  prompt <- paste0(
    prompt,
    " Do not include any additional commentary or information beyond what is requested. ",
    "Do not mention the prompt or instructions in your response. Do not mention that you are an AI model. "
  )
  # Return the selected prompt
  return(prompt)
}

#' Create HTML Summary for Life Cycle Analysis Results
#' 
#' Generates a nicely formatted HTML output for lifeCycle results
#' suitable for Shiny renderUI
#' 
#' @param results Output from lifeCycle() function
#' 
#' @return HTML tag list for Shiny UI
#' 
lifeCycleSummaryUI <- function(results) {
  
 # Extract data
  params <- results$parameters_real_years
  metrics <- results$metrics
  df <- results$data
  forecast_df <- results$forecast
  
  # Calculate additional info
  last_obs_year <- max(df$PY)
  last_obs_annual <- tail(df$n, 1)
  last_obs_cumulative <- max(df$cumulative)
  current_pct <- (last_obs_cumulative / params$K) * 100
  
  # Years to reach milestones
  years_to_50 <- round(params$tm_year - last_obs_year,0)
  years_to_90 <- round(params$year_90_pct - last_obs_year,0)
  years_to_99 <- round(params$year_99_pct - last_obs_year,0)
  
  # Create the UI
  shiny::tagList(
    shiny::div(class = "lifecycle-summary",
               
               # === CARD 1: MODEL OVERVIEW ===
               shiny::div(class = "lifecycle-card",
                          shiny::div(class = "lifecycle-header",
                                     shiny::icon("chart-line"),
                                     "Model Overview"
                          ),
                          shiny::div(class = "lifecycle-grid",
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "Saturation (K)"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           format(round(params$K), big.mark = ","),
                                                           shiny::span(class = "lifecycle-metric-unit", "pubs")
                                                )
                                     ),
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "Peak Year (tₘ)"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           round(params$tm_year, 0)
                                                )
                                     ),
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "Peak Annual"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           format(round(params$peak_annual,0), big.mark = ","),
                                                           shiny::span(class = "lifecycle-metric-unit", "pubs/year")
                                                )
                                     ), # delta symbol for duration 
                                     
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "Growth Duration (Δₜ)"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           round(params$delta_t, 1),
                                                           shiny::span(class = "lifecycle-metric-unit", "years")
                                                )
                                     )
                          )
               ),
               
               # === CARD 2: MODEL FIT QUALITY ===
               shiny::div(class = "lifecycle-card success",
                          shiny::div(class = "lifecycle-header success",
                                     shiny::icon("check-circle"),
                                     "Model Fit Quality",
                                     if(metrics$R_squared >= 0.9) {
                                       shiny::span(class = "lifecycle-badge excellent", "Excellent")
                                     } else if(metrics$R_squared >= 0.75) {
                                       shiny::span(class = "lifecycle-badge good", "Good")
                                     } else {
                                       shiny::span(class = "lifecycle-badge moderate", "Moderate")
                                     }
                          ),
                          shiny::div(class = "lifecycle-grid",
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "R²"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           sprintf("%.3f", metrics$R_squared)
                                                )
                                     ),
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "RMSE"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           sprintf("%.2f", metrics$RMSE)
                                                )
                                     ),
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "AIC"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           sprintf("%.1f", metrics$AIC)
                                                )
                                     ),
                                     shiny::div(class = "lifecycle-metric",
                                                shiny::div(class = "lifecycle-metric-label", "BIC"),
                                                shiny::div(class = "lifecycle-metric-value",
                                                           sprintf("%.1f", metrics$BIC)
                                                )
                                     )
                          )
               ),
               
               # === CARD 3: CURRENT STATUS ===
               fluidRow(
                 column(4,
               
               shiny::div(class = "lifecycle-card info",
                          shiny::div(class = "lifecycle-header info",
                                     shiny::icon("info-circle"),
                                     "Current Status"
                          ),
                          shiny::div(
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "Last Observed Year:"),
                                       shiny::div(class = "lifecycle-row-value", last_obs_year)
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "Annual Publications:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  format(last_obs_annual, big.mark = ","))
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "Cumulative Total:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  format(round(last_obs_cumulative), big.mark = ","))
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "Progress to Saturation:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  sprintf("%.1f%%", current_pct))
                            )
                          ),
                          shiny::div(class = "lifecycle-progress",
                                     shiny::div(class = "lifecycle-progress-bar",
                                                style = sprintf("width: %.1f%%;", min(current_pct, 100)),
                                                sprintf("%.1f%%", current_pct)
                                     )
                          )
               )
               ),
               column(4,
               # === CARD 4: MILESTONE YEARS ===
               shiny::div(class = "lifecycle-card warning",
                          shiny::div(class = "lifecycle-header warning",
                                     shiny::icon("flag-checkered"),
                                     "Milestone Years"
                          ),
                          shiny::div(
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "10% of K:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  sprintf("%.1f", params$year_10_pct))
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "50% of K (Midpoint):"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  sprintf("%.1f", params$tm_year),
                                                  if(years_to_50 > 0) {
                                                    shiny::span(style = "font-size: 12px; color: #666; margin-left: 8px;",
                                                                sprintf("(%+.0f years)", years_to_50))
                                                  }
                                       )
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "90% of K:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  sprintf("%.1f", params$year_90_pct),
                                                  if(years_to_90 > 0) {
                                                    shiny::span(style = "font-size: 12px; color: #666; margin-left: 8px;",
                                                                sprintf("(%+.0f years)", years_to_90))
                                                  }
                                       )
                            ),
                            shiny::div(class = "lifecycle-row",
                                       shiny::div(class = "lifecycle-row-label", "99% of K:"),
                                       shiny::div(class = "lifecycle-row-value", 
                                                  sprintf("%.1f", params$year_99_pct),
                                                  if(years_to_99 > 0) {
                                                    shiny::span(style = "font-size: 12px; color: #666; margin-left: 8px;",
                                                                sprintf("(%+.0f years)", years_to_99))
                                                  }
                                       )
                            )
                          ),
                          
                          # Info box about current phase
                          if(current_pct < 10) {
                            shiny::div(class = "lifecycle-info-box",
                                       shiny::icon("lightbulb"),
                                       " The topic is in the ",
                                       shiny::strong("early growth phase"),
                                       " (< 10% of saturation)."
                            )
                          } else if(current_pct < 50) {
                            shiny::div(class = "lifecycle-info-box",
                                       shiny::icon("lightbulb"),
                                       " The topic is in the ",
                                       shiny::strong("rapid growth phase"),
                                       " (10-50% of saturation)."
                            )
                          } else if(current_pct < 90) {
                            shiny::div(class = "lifecycle-info-box warning",
                                       shiny::icon("exclamation-triangle"),
                                       " The topic is in the ",
                                       shiny::strong("maturity phase"),
                                       " (50-90% of saturation). Growth is slowing."
                            )
                          } else {
                            shiny::div(class = "lifecycle-info-box warning",
                                       shiny::icon("exclamation-triangle"),
                                       " The topic has reached ",
                                       shiny::strong("saturation phase"),
                                       " (> 90%). Annual publications are declining."
                            )
                          }
               )
               ),
               column(4,
               # === CARD 5: FORECAST ===
               if(nrow(forecast_df) > 0) {
                 future_5y <- forecast_df[forecast_df$PY == (last_obs_year + 5), ]
                 future_10y <- forecast_df[forecast_df$PY == (last_obs_year + 10), ]
                 
                 shiny::div(class = "lifecycle-card",
                            shiny::div(class = "lifecycle-header",
                                       icon("rocket"),
                                       "Forecast"
                            ),
                            shiny::div(
                              shiny::div(class = "lifecycle-row",
                                         shiny::div(class = "lifecycle-row-label", "Forecast Period:"),
                                         shiny::div(class = "lifecycle-row-value", 
                                                    sprintf("%d - %d", last_obs_year + 1, max(forecast_df$PY)))
                              ),
                              if(nrow(future_5y) > 0) {
                                shiny::div(class = "lifecycle-row",
                                           shiny::div(class = "lifecycle-row-label", 
                                                      sprintf("Projection for %d:", last_obs_year + 5)),
                                           shiny::div(class = "lifecycle-row-value", 
                                                      sprintf("%.0f cumulative", future_5y$predicted_cumulative[1]),
                                                      shiny::span(style = "font-size: 12px; color: #666; margin-left: 8px;",
                                                                  sprintf("(%.0f annual)", future_5y$predicted_annual[1]))
                                           )
                                )
                              },
                              if(nrow(future_10y) > 0) {
                                shiny::div(class = "lifecycle-row",
                                           shiny::div(class = "lifecycle-row-label", 
                                                      sprintf("Projection for %d:", last_obs_year + 10)),
                                           shiny::div(class = "lifecycle-row-value", 
                                                      sprintf("%.0f cumulative", future_10y$predicted_cumulative[1]),
                                                      shiny::span(style = "font-size: 12px; color: #666; margin-left: 8px;",
                                                                  sprintf("(%.0f annual)", future_10y$predicted_annual[1]))
                                           )
                                )
                              }
                            )
                 )
               }
               )
               )
    )
  )
}

cssTags <- function() {
  ## workaround to solve visualization issues in Data Table
  tagList(
    tags$head(tags$style(HTML(
      ".has-feedback .form-control { padding-right: 0px;}"
    ))),
    ## Disable download/action buttons when analysis not yet run
    tags$head(tags$style(HTML(
      "[disabled], .disabled,
       a.shiny-download-link[disabled], a.shiny-download-link.disabled,
       a.bttn[disabled], a.bttn.disabled,
       button.bttn[disabled], button.bttn.disabled {
        pointer-events: none !important;
        opacity: 0.35 !important;
        cursor: not-allowed !important;
      }"
    ))),
    ## JS interceptor: block clicks on disabled download links
    tags$head(tags$script(HTML(
      "$(document).on('click', 'a[disabled], a.disabled', function(e) {
        e.preventDefault();
        e.stopImmediatePropagation();
        return false;
      });"
    ))),
    ### animation for filter results box
    tags$head(tags$style(HTML(
      "
  .fade-in {
    animation: fadeInAnim 0.8s ease-in-out;
  }

  @keyframes fadeInAnim {
    from { opacity: 0; transform: scale(0.98); }
    to { opacity: 1; transform: scale(1); }
  }
"
    ))),
    ## clipboard ubuntu
    tags$head(
      tags$script(HTML(
        "
    Shiny.addCustomMessageHandler('copy_to_clipboard_js', function(message) {
      if (navigator.clipboard && navigator.clipboard.writeText) {
        // Metodo moderno (Clipboard API)
        navigator.clipboard.writeText(message).then(function() {
          alert('Content sent to clipboard!');
        }, function(err) {
          console.error('Error: ', err);
        });
      } else {
        // Fallback per browser vecchi o connessioni non HTTPS
        const el = document.createElement('textarea');
        el.value = message;
        document.body.appendChild(el);
        el.select();
        document.execCommand('copy');
        document.body.removeChild(el);
        alert('Content sent to clipboard (fallback)!');
      }
    });
  "
      ))
    ),

    ### css for citation matching
    tags$head(
      tags$style(HTML(
        "
    #refMatch_htmlTable tbody tr:hover {
      background-color: #e9ecef;
      cursor: pointer;
    }
  "
      ))
    ),
    tags$head(
      tags$style(HTML(
        "
    /* Manual merge buttons styling */
    #refMatch_toggleSelection,
    #refMatch_clearSelection,
    #refMatch_confirmMerge {
      white-space: nowrap;
      overflow: hidden;
      text-overflow: ellipsis;
    }
    
    /* Status inline box */
    #refMatch_selectionStatusInline {
      transition: background-color 0.3s ease;
    }
    
    /* Responsive adjustments */
    @media (max-width: 1200px) {
      #refMatch_toggleSelection,
      #refMatch_clearSelection,
      #refMatch_confirmMerge {
        font-size: 12px;
        padding: 6px 10px;
      }
    }
  "
      ))
    ),
    tags$head(
      tags$style(HTML(
        "
    /* Loading indicator animation */
    @keyframes pulse {
      0% { opacity: 1; }
      50% { opacity: 0.5; }
      100% { opacity: 1; }
    }
    
    #refMatch_loadingIndicator .box {
      animation: pulse 2s ease-in-out infinite;
    }
    
    #refMatch_loadingIndicator h4 {
      animation: pulse 1.5s ease-in-out infinite;
    }
  "
      ))
    ),

    ### css for life cycle summary
    tags$style(shiny::HTML(
      "
      .lifecycle-tabs .nav-tabs {
        border-bottom: 2px solid #dee2e6;
      }
      .lifecycle-tabs .nav-tabs .nav-link {
        color: #495057;
        font-weight: 500;
        padding: 12px 20px;
        border: none;
        border-bottom: 3px solid transparent;
        transition: all 0.3s ease;
      }
      .lifecycle-tabs .nav-tabs .nav-link:hover {
        color: #1f77b4;
        border-bottom-color: #1f77b4;
        background: transparent;
      }
      .lifecycle-tabs .nav-tabs .nav-link.active {
        color: #1f77b4;
        background: transparent;
        border-bottom-color: #1f77b4;
      }
      .lifecycle-plot-container {
        background: white;
        padding: 20px;
        border-radius: 8px;
        box-shadow: 0 2px 8px rgba(0,0,0,0.1);
        margin-bottom: 20px;
      }
    "
    )),
    shiny::tags$style(shiny::HTML(
      "
      .lifecycle-summary {
        font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, sans-serif;
        color: #333;
      }
      .lifecycle-card {
        background: white;
        border-radius: 8px;
        padding: 20px;
        margin-bottom: 20px;
        box-shadow: 0 2px 8px rgba(0,0,0,0.1);
        border-left: 4px solid #1f77b4;
      }
      .lifecycle-card.success {
        border-left-color: #2ca02c;
      }
      .lifecycle-card.warning {
        border-left-color: #ff7f0e;
      }
      .lifecycle-card.info {
        border-left-color: #9467bd;
      }
      .lifecycle-header {
        font-size: 18px;
        font-weight: 600;
        color: #1f77b4;
        margin-bottom: 15px;
        display: flex;
        align-items: center;
      }
      .lifecycle-header.success {
        color: #2ca02c;
      }
      .lifecycle-header.warning {
        color: #ff7f0e;
      }
      .lifecycle-header.info {
        color: #9467bd;
      }
      .lifecycle-header i {
        margin-right: 10px;
        font-size: 20px;
      }
      .lifecycle-grid {
        display: grid;
        grid-template-columns: repeat(auto-fit, minmax(200px, 1fr));
        gap: 15px;
        margin-top: 15px;
      }
      .lifecycle-metric {
        background: #f8f9fa;
        padding: 12px;
        border-radius: 6px;
        text-align: center;
      }
      .lifecycle-metric-label {
        font-size: 12px;
        color: #666;
        text-transform: uppercase;
        letter-spacing: 0.5px;
        margin-bottom: 5px;
      }
      .lifecycle-metric-value {
        font-size: 24px;
        font-weight: 700;
        color: #1f77b4;
      }
      .lifecycle-metric-unit {
        font-size: 14px;
        color: #999;
        margin-left: 3px;
      }
      .lifecycle-row {
        display: flex;
        justify-content: space-between;
        align-items: center;
        padding: 10px 0;
        border-bottom: 1px solid #eee;
      }
      .lifecycle-row:last-child {
        border-bottom: none;
      }
      .lifecycle-row-label {
        font-weight: 500;
        color: #555;
      }
      .lifecycle-row-value {
        font-weight: 600;
        color: #1f77b4;
        font-size: 16px;
      }
      .lifecycle-badge {
        display: inline-block;
        padding: 4px 12px;
        border-radius: 12px;
        font-size: 12px;
        font-weight: 600;
        margin-left: 8px;
      }
      .lifecycle-badge.excellent {
        background: #d4edda;
        color: #155724;
      }
      .lifecycle-badge.good {
        background: #d1ecf1;
        color: #0c5460;
      }
      .lifecycle-badge.moderate {
        background: #fff3cd;
        color: #856404;
      }
      .lifecycle-progress {
        background: #e9ecef;
        border-radius: 10px;
        height: 20px;
        margin-top: 10px;
        overflow: hidden;
      }
      .lifecycle-progress-bar {
        background: linear-gradient(90deg, #1f77b4, #2ca02c);
        height: 100%;
        transition: width 0.3s ease;
        display: flex;
        align-items: center;
        justify-content: center;
        color: white;
        font-size: 12px;
        font-weight: 600;
      }
      .lifecycle-info-box {
        background: #e7f3ff;
        border-left: 3px solid #1f77b4;
        padding: 12px;
        border-radius: 4px;
        margin-top: 15px;
      }
      .lifecycle-info-box.warning {
        background: #fff4e6;
        border-left-color: #ff7f0e;
      }
    "
    )),

    ### css for author link
    tags$head(
      tags$style(HTML(
        "
      .author-link {
        color: #337ab7;
        text-decoration: underline;
        cursor: pointer;
      }
      .author-link:hover {
        color: #23527c;
        font-weight: bold;
      }"
      ))
    ),
    ## script to open more times the same modal ####
    tags$script(
      "
    Shiny.addCustomMessageHandler('button_id', function(value) {
    Shiny.setInputValue('button_id', value);
    });
  "
    ),
    tags$script(
      "
    Shiny.addCustomMessageHandler('selected_author', function(value) {
    Shiny.setInputValue('selected_author', value);
    });
  "
    ),
    ## script to get the dimensions of the page ####
    ###
    tags$head(
      tags$style(".fa-cloud-arrow-down {font-size: 20px}"),
      tags$style(".fa-download {font-size: 20px}"),
      tags$style(".fa-envelope {color:#FF0000; font-size: 20px}"),
      tags$style(".fa-envelope-open {font-size: 20px}"),
      tags$style(".fa-cube {font-size: 20px}"),
      tags$style(".fa-question {font-size: 20px}"),
      tags$style(".fa-comment-dollar {font-size: 20px}"),
      tags$style(".fa-bars {font-size: 20px}"),
      tags$style(".sidebar-toggle {font-size: 15px}"),
      tags$script(
        'var dimension = [0, 0];
              $(document).on("shiny:connected", function(e) {
                  dimension[0] = window.innerWidth;
                  dimension[1] = window.innerHeight;
                  Shiny.onInputChange("dimension", dimension);
              });
              $(window).resize(function(e) {
                  dimension[0] = window.innerWidth;
                  dimension[1] = window.innerHeight;
                  Shiny.onInputChange("dimension", dimension);
              });
              $(document).ready(function(){
                  $("a[data-toggle=tab]").on("show.bs.tab", function(e){
                    Shiny.setInputValue("activeTab", $(this).attr("data-value"));
                   });
            });
      '
      )
    )
  )
}

# ==============================================================================
# UI Component - WebOfScience-like Query Builder for OpenAlex (Collapsible)
# ==============================================================================

openAlexUI <- function() {
  tagList(
    fluidRow(
      column(
        width = 12,
        # Query Builder Box (Collapsible)
        wellPanel(
          id = "oaQueryPanel",

          # Header with collapse button
          div(
            style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 15px;",
            h4(
              "OpenAlex Data Collection",
              style = "font-weight: bold; margin: 0;"
            ),
            actionButton(
              "oaToggleQueryPanel",
              icon("chevron-up"),
              class = "btn-sm btn-outline-secondary",
              style = "padding: 5px 10px;",
              title = "Collapse/Expand"
            )
          ),

          # API Key Warning Banner
          uiOutput("oaApiKeyWarning"),

          # Query content (collapsible)
          div(
            id = "oaQueryContent",

            # Query Builder Section (WebOfScience-like)
            div(
              id = "query-builder-container",
              style = "margin-bottom: 20px;",

              # Initial query row (NO operator column)
              div(
                id = "query-row-1",
                class = "query-row",
                style = "margin-bottom: 10px;",
                fluidRow(
                  column(1),
                  column(
                    width = 2,
                    selectInput(
                      "oaField_1",
                      NULL,
                      choices = c(
                        "All Fields" = "default",
                        "Title" = "title",
                        "Abstract" = "abstract",
                        "Title and Abstract" = "title_abstract",
                        "Author" = "author",
                        "Concept" = "concept"
                      ),
                      selected = "title_abstract",
                      width = "100%"
                    )
                  ),
                  column(
                    width = 8,
                    textInput(
                      "oaQuery_1",
                      NULL,
                      value = "",
                      placeholder = 'Example: "machine learning" OR AI',
                      width = "100%"
                    )
                  ),
                  column(
                    width = 1,
                    div(
                      id = "remove-button-1",
                      style = "visibility: hidden;",
                      actionButton(
                        "oaRemoveRow_1",
                        icon("minus-circle"),
                        class = "btn-danger btn-sm",
                        style = "margin-top: 0px;",
                        title = "Remove row"
                      )
                    )
                  )
                )
              )
            ),

            # Help text for search syntax
            div(
              style = "margin-bottom: 15px; font-size: 12px; color: #666;",
              HTML(
                "<strong>Search tips:</strong> Use quotes for exact phrases (e.g., \"science mapping\"). Boolean operators AND, OR, NOT must be UPPERCASE."
              )
            ),

            # Add row, date range, and Search buttons
            div(
              style = "margin-bottom: 20px; display: flex; align-items: center; gap: 10px;",
              actionButton(
                "oaAddRow",
                label = "+ Add row",
                icon = icon("plus"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "oaToggleDateRange",
                label = "+ Date range",
                icon = icon("calendar"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "oaSearchQuery",
                label = "Search",
                icon = icon("search"),
                class = "btn-info btn-sm",
                style = "margin-left: auto;"
              )
            ),

            # Date Range Section (initially visible)
            div(
              id = "oaDateRangeSection",
              style = "display: block;", # Visibile di default
              wellPanel(
                style = "background-color: #f8f9fa;",
                h5("Date Range Filter", style = "font-weight: bold;"),
                fluidRow(
                  column(
                    width = 6,
                    numericInput(
                      "oaYearFrom",
                      "From Year:",
                      value = 1985,
                      min = 1900,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  ),
                  column(
                    width = 6,
                    numericInput(
                      "oaYearTo",
                      "To Year:",
                      value = as.integer(format(Sys.Date(), "%Y")),
                      min = 1900,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  )
                )
              )
            ),

            # Advanced filters toggle
            actionButton(
              "oaToggleAdvanced",
              label = "Advanced filters",
              icon = icon("filter"),
              class = "btn-link",
              style = "padding: 5px; margin-bottom: 10px;"
            ),

            # Additional Filters Section (initially visible)
            div(
              id = "oaAdvancedSection",
              style = "display: block;", # Visibile di default
              wellPanel(
                style = "background-color: #f8f9fa;",
                fluidRow(
                  column(
                    width = 4,
                    selectizeInput(
                      "oaDocType",
                      "Document Type:",
                      choices = c(
                        "All" = "",
                        "Article" = "article",
                        "Book Chapter" = "book-chapter",
                        "Book" = "book",
                        "Dataset" = "dataset",
                        "Dissertation" = "dissertation",
                        "Editorial" = "editorial",
                        "Letter" = "letter",
                        "Preprint" = "preprint",
                        "Review" = "review"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    selectizeInput(
                      "oaLanguage",
                      "Language:",
                      choices = c(
                        "All" = "",
                        "English" = "en",
                        "Spanish" = "es",
                        "French" = "fr",
                        "German" = "de",
                        "Italian" = "it",
                        "Portuguese" = "pt",
                        "Chinese" = "zh",
                        "Japanese" = "ja"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    numericInput(
                      "oaMaxRecords",
                      "Max Records:",
                      value = 1000,
                      min = 1,
                      max = 10000,
                      step = 100
                    )
                  )
                ),
                fluidRow(
                  column(
                    width = 3,
                    selectizeInput(
                      "oaOpenAccess",
                      "Open Access:",
                      choices = c(
                        "All" = "",
                        "Gold" = "gold",
                        "Green" = "green",
                        "Hybrid" = "hybrid",
                        "Bronze" = "bronze",
                        "Diamond" = "diamond",
                        "Closed" = "closed"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaSDG",
                      "SDG:",
                      choices = c(
                        "All" = "",
                        "1 - No Poverty" = "1",
                        "2 - Zero Hunger" = "2",
                        "3 - Good Health and Well-being" = "3",
                        "4 - Quality Education" = "4",
                        "5 - Gender Equality" = "5",
                        "6 - Clean Water and Sanitation" = "6",
                        "7 - Affordable and Clean Energy" = "7",
                        "8 - Decent Work and Economic Growth" = "8",
                        "9 - Industry, Innovation and Infrastructure" = "9",
                        "10 - Reduced Inequalities" = "10",
                        "11 - Sustainable Cities and Communities" = "11",
                        "12 - Responsible Consumption and Production" = "12",
                        "13 - Climate Action" = "13",
                        "14 - Life Below Water" = "14",
                        "15 - Life on Land" = "15",
                        "16 - Peace, Justice and Strong Institutions" = "16",
                        "17 - Partnerships for the Goals" = "17"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaKeyword",
                      "Keyword:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaDomain",
                      "Domain:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaJournal",
                      "Journal:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  )
                ),
                fluidRow(
                  column(
                    width = 6, offset = 6,
                    radioButtons(
                      "oaDropdownSort",
                      NULL,
                      choices = c("Sort by count" = "count", "Sort A-Z" = "alpha"),
                      selected = "count",
                      inline = TRUE
                    )
                  )
                )
              )
            ),
            # Action buttons section
            div(
              style = "margin-top: 10px; display: flex; justify-content: space-between; align-items: center;",

              # Left side: Reactive query count message + working indicator
              div(
                style = "display: flex; align-items: center; gap: 8px;",
                uiOutput("oaQueryCountInfo"),
                span(
                  id = "oaWorkingLabel",
                  style = "display: none; color: #dc3545; font-size: 13px; font-weight: 600;",
                  icon("spinner", class = "fa-spin", style = "margin-right: 4px;"),
                  "working..."
                )
              ),

              # Right side: Refs checkbox, Clear and Search buttons
              div(
                style = "display: flex; align-items: center; gap: 10px;",
                div(
                  style = "margin-bottom: 0;",
                  checkboxInput(
                    "oaFetchRefs",
                    "Download cited references",
                    value = FALSE
                  ),
                  conditionalPanel(
                    condition = "input.oaFetchRefs == true",
                    div(
                      style = "margin-left: 5px; margin-top: -5px; font-size: 12px;",
                      radioButtons(
                        "oaRefsFilter",
                        NULL,
                        choices = c(
                          "Only cited > 1 (faster)" = "multiple",
                          "All references" = "all"
                        ),
                        selected = "multiple",
                        inline = TRUE
                      )
                    )
                  )
                ),
                actionButton(
                  "oaClearQuery",
                  "Clear",
                  icon = icon("times"),
                  class = "btn-outline-secondary"
                ),
                actionButton(
                  "oaFetchData",
                  "Download",
                  icon = icon("download"),
                  class = "btn-primary"
                )
              )
            )
          )
        )
      )
    ),

    # Data Display Section (shown when data is available)
    conditionalPanel(
      condition = "output.oaDataAvailable",
      fluidRow(
        column(
          width = 12,
          wellPanel(
            h4("Downloaded Collection", style = "font-weight: bold;"),

            # Progress and results info
            uiOutput("oaProgressInfo"),

            # Export buttons
            div(
              style = "margin-bottom: 15px; margin-top: 15px;",
              downloadButton(
                "oaDownloadExcel",
                "Export to Excel",
                icon = icon("file-excel"),
                class = "btn-success"
              ),
              downloadButton(
                "oaDownloadRData",
                "Export to RData",
                icon = icon("database"),
                class = "btn-info",
                style = "margin-left: 10px;"
              )
            ),

            # Data table output
            div(
              style = "margin-top: 15px;",
              uiOutput("oaDataTable")
            )
          )
        )
      )
    ),

    # JavaScript for collapsible functionality
    tags$script(HTML(
      "
  $(document).ready(function() {
    // Initialize collapsed state
    var isCollapsed = false;
    
    // Initialize visibility states
    var dateRangeVisible = true;
    var advancedVisible = true;
    
    // Handle toggle button click for main panel
    $('#oaToggleQueryPanel').on('click', function() {
      isCollapsed = !isCollapsed;
      $('#oaQueryContent').slideToggle(300);
      
      // Update button icon
      if (isCollapsed) {
        $(this).html('<i class=\"fa fa-chevron-down\"></i>');
      } else {
        $(this).html('<i class=\"fa fa-chevron-up\"></i>');
      }
    });
    
    // Handle date range toggle
    $('#oaToggleDateRange').on('click', function() {
      dateRangeVisible = !dateRangeVisible;
      $('#oaDateRangeSection').slideToggle(300);
      
      // Update button text
      if (dateRangeVisible) {
        $(this).html('<i class=\"fa fa-calendar\"></i> - Date range');
      } else {
        $(this).html('<i class=\"fa fa-calendar\"></i> + Date range');
      }
      
      // Notify Shiny
      Shiny.setInputValue('oaShowDateRange', dateRangeVisible);
    });
    
    // Handle advanced filters toggle
    $('#oaToggleAdvanced').on('click', function() {
      advancedVisible = !advancedVisible;
      $('#oaAdvancedSection').slideToggle(300);
      
      // Update button text
      if (advancedVisible) {
        $(this).html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
      } else {
        $(this).html('<i class=\"fa fa-filter\"></i> Show advanced filters');
      }
      
      // Notify Shiny
      Shiny.setInputValue('oaShowAdvanced', advancedVisible);
    });
    
    // Set initial button labels
    $('#oaToggleDateRange').html('<i class=\"fa fa-calendar\"></i> - Date range');
    $('#oaToggleAdvanced').html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
  });
  
  // Custom message handler to collapse panel when data is loaded
  Shiny.addCustomMessageHandler('oaCollapseQueryPanel', function(message) {
    $('#oaQueryContent').slideUp(300);
    $('#oaToggleQueryPanel').html('<i class=\"fa fa-chevron-down\"></i>');
  });

  // Show 'working...' indicator when Shiny is busy (scoped to OA panel)
  $(document).on('shiny:busy', function() {
    if ($('#oaQueryPanel').is(':visible')) {
      $('#oaWorkingLabel').show();
    }
  });
  $(document).on('shiny:idle', function() {
    $('#oaWorkingLabel').hide();
  });
"
    ))
  )
}


# ==============================================================================
# Helper Functions - OpenAlex Reference Resolution
# ==============================================================================

# LTWA-like lookup for journal title word abbreviations
.journal_word_abbrev <- c(
  "JOURNAL" = "J", "INTERNATIONAL" = "INT", "AMERICAN" = "AM",
  "SCIENCE" = "SCI", "SCIENCES" = "SCI", "SCIENTIFIC" = "SCI",
  "TECHNOLOGY" = "TECHNOL", "TECHNOLOGICAL" = "TECHNOL",
  "RESEARCH" = "RES", "REVIEW" = "REV", "REVIEWS" = "REV",
  "MANAGEMENT" = "MANAGE", "INFORMATION" = "INFORM",
  "ANNUAL" = "ANN", "SOCIAL" = "SOC", "SOCIETY" = "SOC",
  "APPLIED" = "APPL", "APPLICATION" = "APPL", "APPLICATIONS" = "APPL",
  "ENGINEERING" = "ENG", "EDUCATION" = "EDUC", "EDUCATIONAL" = "EDUC",
  "ECONOMICS" = "ECON", "ECONOMIC" = "ECON",
  "ENVIRONMENTAL" = "ENVIRON", "ENVIRONMENT" = "ENVIRON",
  "PSYCHOLOGY" = "PSYCHOL", "PSYCHOLOGICAL" = "PSYCHOL",
  "BIOLOGICAL" = "BIOL", "BIOLOGY" = "BIOL",
  "MEDICAL" = "MED", "MEDICINE" = "MED", "CLINICAL" = "CLIN",
  "PHYSICAL" = "PHYS", "PHYSICS" = "PHYS",
  "CHEMISTRY" = "CHEM", "CHEMICAL" = "CHEM",
  "MATHEMATICAL" = "MATH", "MATHEMATICS" = "MATH",
  "POLITICAL" = "POLIT", "POLITICS" = "POLIT",
  "COMMUNICATION" = "COMMUN", "COMMUNICATIONS" = "COMMUN",
  "COMPUTATIONAL" = "COMPUT", "COMPUTER" = "COMPUT", "COMPUTING" = "COMPUT",
  "STATISTICAL" = "STAT", "STATISTICS" = "STAT",
  "EUROPEAN" = "EUR", "BRITISH" = "BRIT", "CANADIAN" = "CAN",
  "NATIONAL" = "NATL", "GENERAL" = "GEN",
  "AGRICULTURAL" = "AGRIC", "AGRICULTURE" = "AGRIC",
  "GEOGRAPHY" = "GEOGR", "GEOGRAPHICAL" = "GEOGR",
  "PHILOSOPHY" = "PHILOS", "PHILOSOPHICAL" = "PHILOS",
  "UNIVERSITY" = "UNIV", "ASSOCIATION" = "ASSOC",
  "DEVELOPMENT" = "DEV", "EXPERIMENTAL" = "EXP",
  "PROCEEDINGS" = "PROC", "TRANSACTIONS" = "TRANS",
  "LETTERS" = "LETT", "REPORTS" = "REP", "REPORT" = "REP",
  "ANALYSIS" = "ANAL", "ANALYTICAL" = "ANAL",
  "BEHAVIOR" = "BEHAV", "BEHAVIORAL" = "BEHAV",
  "BEHAVIOUR" = "BEHAV", "BEHAVIOURAL" = "BEHAV",
  "COGNITIVE" = "COGN", "CULTURAL" = "CULT", "CULTURE" = "CULT",
  "CURRENT" = "CURR", "DISEASE" = "DIS", "DISEASES" = "DIS",
  "ECOLOGY" = "ECOL", "ECOLOGICAL" = "ECOL",
  "ENERGY" = "ENERG", "MATERIALS" = "MATER", "MOLECULAR" = "MOL",
  "NUTRITION" = "NUTR", "NUTRITIONAL" = "NUTR",
  "OPERATIONS" = "OPER", "ORGANIZATION" = "ORGAN", "ORGANIZATIONAL" = "ORGAN",
  "PHARMACEUTICAL" = "PHARM", "PHARMACY" = "PHARM",
  "STRUCTURAL" = "STRUCT", "SUSTAINABLE" = "SUSTAIN", "SUSTAINABILITY" = "SUSTAIN",
  "THEORETICAL" = "THEOR", "THERAPY" = "THER", "THERAPEUTIC" = "THER",
  "TOURISM" = "TOUR", "OPINION" = "OPIN",
  "NETWORKS" = "NETW", "NETWORK" = "NETW",
  "DECISION" = "DECIS", "MECHANICS" = "MECH", "MECHANICAL" = "MECH",
  "HOSPITALITY" = "HOSP", "PROCESSING" = "PROCESS",
  "SYSTEMS" = "SYST", "SYSTEM" = "SYST",
  "DYNAMICS" = "DYN", "DYNAMIC" = "DYN",
  "PUBLIC" = "PUBL", "POLICY" = "POL",
  "GLOBAL" = "GLOB", "ATMOSPHERIC" = "ATMOS",
  "BIOMEDICAL" = "BIOMED", "BIOCHEMISTRY" = "BIOCHEM", "BIOCHEMICAL" = "BIOCHEM",
  "MICROBIOLOGY" = "MICROBIOL", "GENETICS" = "GENET",
  "NEUROSCIENCE" = "NEUROSCI", "NEUROLOGY" = "NEUROL",
  "ADVANCED" = "ADV", "ADVANCES" = "ADV",
  "METHODS" = "METH", "METHODOLOGY" = "METHODOL",
  "HISTORY" = "HIST", "HISTORICAL" = "HIST",
  "ELECTRONIC" = "ELECTRON", "GEOPHYSICAL" = "GEOPHYS",
  "INDUSTRIAL" = "IND", "INDUSTRY" = "IND",
  "MARINE" = "MAR", "VETERINARY" = "VET",
  "ORGANIC" = "ORG", "INORGANIC" = "INORG",
  "ZOOLOGY" = "ZOOL", "ZOOLOGICAL" = "ZOOL",
  "ASTRONOMY" = "ASTRON", "ASTROPHYSICAL" = "ASTROPHYS",
  "ARCHIVES" = "ARCH", "ARCHIVE" = "ARCH",
  "LIBRARY" = "LIBR", "PROBLEMS" = "PROBL",
  "PLANNING" = "PLAN", "REGIONAL" = "REG",
  "STUDIES" = "STUD", "QUARTERLY" = "Q",
  "ABSTRACT" = "ABSTR", "BULLETIN" = "BULL",
  "CONGRESS" = "CONGR", "CONFERENCE" = "CONF",
  "DOCUMENTATION" = "DOC", "GEOGRAPHIC" = "GEOGR",
  "ENGINEERING" = "ENG", "LANGUAGE" = "LANG",
  "LINGUISTICS" = "LINGUIST", "SURGERY" = "SURG", "SURGICAL" = "SURG"
)

# Stop words to remove from journal titles
.journal_stop_words <- c(
  "of", "the", "and", "for", "in", "on", "a", "an", "to",
  "de", "la", "le", "les", "des", "du", "et",
  "der", "die", "das", "und", "fur",
  "del", "di", "e", "il", "lo"
)

# Abbreviate a journal/source name to LTWA-like form
abbreviate_source_name <- function(name) {
  if (is.null(name) || is.na(name) || name == "") return("")

  words <- unlist(strsplit(name, "[\\s:,]+"))
  words <- words[words != ""]

  # Remove stop words (keep first word always)
  if (length(words) > 1) {
    keep <- c(TRUE, !tolower(words[-1]) %in% .journal_stop_words)
    words <- words[keep]
  }

  # Abbreviate each word
  abbreviated <- vapply(words, function(w) {
    W <- toupper(w)
    # Check lookup table
    if (W %in% names(.journal_word_abbrev)) {
      return(.journal_word_abbrev[[W]])
    }
    # Short words: keep as is
    if (nchar(W) <= 4) return(W)
    # Fallback: truncate to 5 chars
    return(substr(W, 1, 5))
  }, character(1), USE.NAMES = FALSE)

  paste(toupper(abbreviated), collapse = " ")
}

# Format first author from OpenAlex authorships list for CR string
format_cr_author <- function(authorships) {
  if (is.null(authorships) || length(authorships) == 0) return("")
  first_auth <- authorships[[1]]
  name <- first_auth$author$display_name
  if (is.null(name) || is.na(name) || name == "") return("")

  parts <- unlist(strsplit(trimws(name), "\\s+"))
  if (length(parts) == 0) return("")
  if (length(parts) == 1) return(toupper(parts[1]))

  surname <- toupper(parts[length(parts)])
  given <- parts[-length(parts)]
  initials <- paste(substr(toupper(given), 1, 1), collapse = "")

  paste0(surname, " ", initials)
}

# Build a single WOS-style CR string from an OpenAlex work (list format)
build_single_cr <- function(work) {
  parts <- character(0)

  # Author
  auth <- format_cr_author(work$authorships)
  if (auth != "") parts <- c(parts, auth)

  # Year
  year <- work$publication_year
  if (!is.null(year) && !is.na(year)) parts <- c(parts, as.character(year))

  # Journal
  source_name <- tryCatch(
    work$primary_location$source$display_name,
    error = function(e) NULL
  )
  if (!is.null(source_name) && !is.na(source_name) && source_name != "") {
    parts <- c(parts, abbreviate_source_name(source_name))
  }

  # Volume
  vol <- tryCatch(work$biblio$volume, error = function(e) NULL)
  if (!is.null(vol) && !is.na(vol) && vol != "") {
    parts <- c(parts, paste0("V", vol))
  }

  # First page
  page <- tryCatch(work$biblio$first_page, error = function(e) NULL)
  if (!is.null(page) && !is.na(page) && page != "") {
    parts <- c(parts, paste0("P", page))
  }

  # DOI
  doi <- work$doi
  if (!is.null(doi) && !is.na(doi) && doi != "") {
    doi_clean <- gsub("^https?://doi\\.org/", "", doi)
    parts <- c(parts, paste0("DOI ", toupper(doi_clean)))
  }

  if (length(parts) == 0) return(NA_character_)
  paste(parts, collapse = ", ")
}

# Resolve all references for a set of OpenAlex works
# Returns a named list: openalex_id -> CR string
resolve_openalex_references <- function(oa_data, progress_id = NULL,
                                        progress_bar_id = NULL,
                                        progress_pct_id = NULL,
                                        only_multiple = FALSE) {
  # Collect all referenced work IDs
  all_refs <- unlist(lapply(oa_data, function(w) w$referenced_works))
  all_refs <- all_refs[!is.na(all_refs) & all_refs != ""]

  if (length(all_refs) == 0) return(list())

  # Filter: keep only references cited more than once
  if (isTRUE(only_multiple)) {
    ref_freq <- table(all_refs)
    all_ref_ids <- names(ref_freq[ref_freq > 1])
  } else {
    all_ref_ids <- unique(all_refs)
  }

  if (length(all_ref_ids) == 0) return(list())

  # Update progress with actual count after filtering
  if (!is.null(progress_id)) {
    filter_label <- if (isTRUE(only_multiple)) " (cited > 1)" else ""
    shinyjs::html(
      progress_id,
      sprintf(
        "Resolving %s unique references%s...",
        format(length(all_ref_ids), big.mark = ","),
        filter_label
      )
    )
  }

  # Batch fetch referenced works (50 per batch to stay within URL limits)
  batch_size <- 50
  n_batches <- ceiling(length(all_ref_ids) / batch_size)
  ref_lookup <- list()

  for (i in seq_len(n_batches)) {
    start_idx <- (i - 1) * batch_size + 1
    end_idx <- min(i * batch_size, length(all_ref_ids))
    batch_ids <- all_ref_ids[start_idx:end_idx]

    pct <- round(i / n_batches * 100)

    # Update progress bar if available
    if (!is.null(progress_bar_id)) {
      shinyjs::runjs(sprintf(
        "$('#%s').css('width', '%d%%');", progress_bar_id, pct
      ))
    }
    if (!is.null(progress_pct_id)) {
      shinyjs::html(progress_pct_id, sprintf(
        "Batch %d/%d (%d%%)", i, n_batches, pct
      ))
    } else if (!is.null(progress_id)) {
      # Fallback: text-only progress
      shinyjs::html(
        progress_id,
        sprintf(
          "Resolving references... batch %d/%d (%s unique refs)",
          i, n_batches, format(length(all_ref_ids), big.mark = ",")
        )
      )
    }

    tryCatch({
      # Strip URL prefix for the filter
      short_ids <- gsub("https://openalex.org/", "", batch_ids)
      result <- openalexR::oa_fetch(
        entity = "works",
        openalex_id = paste(short_ids, collapse = "|"),
        output = "list",
        verbose = FALSE
      )

      if (is.list(result)) {
        for (work in result) {
          if (!is.null(work$id)) {
            cr <- build_single_cr(work)
            if (!is.na(cr)) ref_lookup[[work$id]] <- cr
          }
        }
      }
    }, error = function(e) {
      # Skip failed batches silently
    })
  }

  return(ref_lookup)
}

# Build CR column for each paper given oa_data and ref_lookup
build_cr_column <- function(oa_data, ref_lookup) {
  vapply(oa_data, function(work) {
    ref_ids <- work$referenced_works
    if (is.null(ref_ids) || length(ref_ids) == 0) return(NA_character_)

    cr_strings <- unlist(lapply(ref_ids, function(rid) {
      if (rid %in% names(ref_lookup)) ref_lookup[[rid]] else NULL
    }))

    if (length(cr_strings) == 0) return(NA_character_)
    paste(cr_strings, collapse = "; ")
  }, character(1))
}


# ==============================================================================
# Server Component - WebOfScience-like Query Builder
# ==============================================================================

openAlexServer <- function(input, output, session, values) {
  # Reactive values to track query rows and toggle states
  queryRows <- reactiveVal(1)
  showDateRange <- reactiveVal(TRUE)
  showAdvanced <- reactiveVal(TRUE)
  downloadProgress <- reactiveVal(NULL)
  journalChoices <- reactiveVal(NULL)
  submittedQueryFilters <- reactiveVal(NULL) # keyword filters submitted via Search button

  # API Key Warning Banner
  output$oaApiKeyWarning <- renderUI({
    # React to changes in the API key value
    values$oaApiKey
    oa_apikey <- Sys.getenv("openalexR.apikey", unset = "")
    if (oa_apikey == "") {
      div(
        style = "margin-bottom: 15px; padding: 12px 15px; background-color: #fff3cd; border-radius: 5px; border-left: 4px solid #f39c12; display: flex; align-items: center;",
        icon("exclamation-triangle", style = "color: #f39c12; margin-right: 10px; font-size: 18px;"),
        tags$span(
          style = "color: #856404; font-size: 13px;",
          HTML(
            "<strong>No API key configured.</strong> Without an API key you are limited to 100 credits/day (testing only). Go to <strong>Settings</strong> to set your free OpenAlex API key (100,000 credits/day)."
          )
        )
      )
    }
  })

  # Disable Keyword/Domain/Journal dropdowns at startup (no data yet)
  observeEvent(TRUE, {
    shinyjs::disable("oaKeyword")
    shinyjs::disable("oaDomain")
    shinyjs::disable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")
  }, once = TRUE)

  # Toggle date range visibility
  observeEvent(input$oaToggleDateRange, {
    current <- showDateRange()
    showDateRange(!current)
    session$sendCustomMessage('oaToggleDateRange', !current)
  })

  # Toggle advanced filters visibility
  observeEvent(input$oaToggleAdvanced, {
    current <- showAdvanced()
    showAdvanced(!current)
    session$sendCustomMessage('oaToggleAdvanced', !current)
  })

  # Add new query row
  observeEvent(input$oaAddRow, {
    currentRows <- queryRows()
    newRowId <- currentRows + 1
    queryRows(newRowId)

    # Insert new row in UI (WITH operator column)
    insertUI(
      selector = "#query-builder-container",
      where = "beforeEnd",
      ui = div(
        id = paste0("query-row-", newRowId),
        class = "query-row",
        style = "margin-bottom: 10px;",
        fluidRow(
          column(
            width = 1,
            selectInput(
              paste0("oaOperator_", newRowId),
              NULL,
              choices = c("And" = "AND", "Or" = "OR", "Not" = "NOT"),
              selected = "AND",
              width = "100%"
            )
          ),
          column(
            width = 2,
            selectInput(
              paste0("oaField_", newRowId),
              NULL,
              choices = c(
                "All Fields" = "default",
                "Title" = "title",
                "Abstract" = "abstract",
                "Title and Abstract" = "title_abstract",
                "Author" = "author",
                "Concept" = "concept"
              ),
              selected = "default",
              width = "100%"
            )
          ),
          column(
            width = 8,
            textInput(
              paste0("oaQuery_", newRowId),
              NULL,
              value = "",
              placeholder = 'Example: "deep learning"',
              width = "100%"
            )
          ),
          column(
            width = 1,
            actionButton(
              paste0("oaRemoveRow_", newRowId),
              icon("minus-circle"),
              class = "btn-danger btn-sm",
              style = "margin-top: 0px;",
              title = "Remove row"
            )
          )
        )
      )
    )

    # Make remove button visible for first row if adding second row
    if (newRowId == 2) {
      runjs("$('#remove-button-1').css('visibility', 'visible');")
    }

    # Add observer for the remove button of the new row
    local({
      rowId <- newRowId
      observeEvent(input[[paste0("oaRemoveRow_", rowId)]], {
        removeUI(selector = paste0("#query-row-", rowId))

        # Check if only one row remains
        remainingRows <- length(
          lapply(1:queryRows(), function(i) {
            input[[paste0("oaQuery_", i)]]
          })
        )

        if (remainingRows == 1) {
          runjs("$('#remove-button-1').css('visibility', 'hidden');")
        }
      })
    })
  })

  # Clear all query fields
  observeEvent(input$oaClearQuery, {
    # Reset all query inputs
    for (i in 1:queryRows()) {
      updateTextInput(session, paste0("oaQuery_", i), value = "")
      updateSelectInput(session, paste0("oaField_", i), selected = "default")
      if (i > 1) {
        updateSelectInput(session, paste0("oaOperator_", i), selected = "AND")
      }
    }

    # Reset filters
    updateNumericInput(session, "oaYearFrom", value = 2020)
    updateNumericInput(
      session,
      "oaYearTo",
      value = as.integer(format(Sys.Date(), "%Y"))
    )
    updateSelectizeInput(session, "oaDocType", selected = "")
    updateSelectizeInput(session, "oaLanguage", selected = "")
    updateNumericInput(session, "oaMaxRecords", value = 1000)
    updateSelectizeInput(session, "oaOpenAccess", selected = "")
    updateSelectizeInput(session, "oaSDG", selected = "")
    updateSelectizeInput(session, "oaKeyword", choices = character(0), selected = character(0))
    updateSelectizeInput(session, "oaDomain", choices = character(0), selected = character(0))
    updateSelectizeInput(session, "oaJournal", choices = character(0), selected = character(0))
    updateRadioButtons(session, "oaDropdownSort", selected = "count")
    updateCheckboxInput(session, "oaFetchRefs", value = FALSE)
    updateRadioButtons(session, "oaRefsFilter", selected = "multiple")
    journalChoices(NULL)
    keywordData(NULL)
    domainData(NULL)
    journalData(NULL)
    submittedQueryFilters(NULL)
  })

  # Search button: submit keyword query and activate reactive filters
  observeEvent(input$oaSearchQuery, {
    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("oaQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[length(queryList) + 1]] <- list(
          query = queryText,
          field = input[[paste0("oaField_", i)]],
          operator = if (i > 1) input[[paste0("oaOperator_", i)]] else NULL
        )
      }
    }

    if (length(queryList) == 0) {
      showNotification("Please enter at least one search term.", type = "warning", duration = 4)
      return()
    }

    search_filters <- build_query_filters(queryList)

    # Add date range filter
    if (showDateRange()) {
      if (!is.na(input$oaYearFrom)) {
        search_filters$from_publication_date <- paste0(input$oaYearFrom, "-01-01")
      }
      if (!is.na(input$oaYearTo)) {
        search_filters$to_publication_date <- paste0(input$oaYearTo, "-12-31")
      }
    }

    submittedQueryFilters(search_filters)
  })

  # Helper function to get field name for filter
  # Fields that can be filtered directly on the /works endpoint
  get_filter_name <- function(field) {
    switch(
      field,
      "title" = "title.search",
      "abstract" = "abstract.search",
      "title_abstract" = "title_and_abstract.search",
      "default" = "default.search",
      NULL
    )
  }

  # Entity-based fields require a two-step approach:
  # 1) Search the entity endpoint to resolve names to IDs
  # 2) Filter /works by those IDs
  entity_field_config <- list(
    "author" = list(
      entity = "authors",
      works_filter = "authorships.author.id"
    ),
    "concept" = list(
      entity = "concepts",
      works_filter = "concepts.id"
    )
  )

  # Resolve entity-based search terms to OpenAlex IDs
  resolve_entity_ids <- function(field, query_text) {
    config <- entity_field_config[[field]]
    if (is.null(config)) return(NULL)

    tryCatch({
      results <- openalexR::oa_fetch(
        entity = config$entity,
        display_name.search = query_text,
        verbose = FALSE
      )
      if (!is.null(results) && nrow(results) > 0) {
        ids <- gsub("https://openalex.org/", "", results$id)
        return(list(
          works_filter = config$works_filter,
          ids = ids
        ))
      }
      return(NULL)
    }, error = function(e) {
      return(NULL)
    })
  }

  # Helper function to build query filters
  build_query_filters <- function(queryList) {
    if (length(queryList) == 0) {
      return(list())
    }

    search_filters <- list()

    # Separate direct filters from entity-based filters
    direct_queries <- list()
    entity_queries <- list()

    for (i in seq_along(queryList)) {
      item <- queryList[[i]]
      field <- item$field
      filter_name <- get_filter_name(field)

      if (!is.null(filter_name)) {
        # Direct works filter (title, abstract, etc.)
        if (is.null(direct_queries[[filter_name]])) {
          direct_queries[[filter_name]] <- list(
            terms = c(item$query),
            operators = c()
          )
        } else {
          direct_queries[[filter_name]]$operators <- c(
            direct_queries[[filter_name]]$operators,
            item$operator
          )
          direct_queries[[filter_name]]$terms <- c(
            direct_queries[[filter_name]]$terms,
            item$query
          )
        }
      } else if (field %in% names(entity_field_config)) {
        # Entity-based filter (author, concept)
        if (is.null(entity_queries[[field]])) {
          entity_queries[[field]] <- list(
            terms = c(item$query),
            operators = c()
          )
        } else {
          entity_queries[[field]]$operators <- c(
            entity_queries[[field]]$operators,
            item$operator
          )
          entity_queries[[field]]$terms <- c(
            entity_queries[[field]]$terms,
            item$query
          )
        }
      }
    }

    # Build direct filters
    for (filter_name in names(direct_queries)) {
      field_data <- direct_queries[[filter_name]]

      if (length(field_data$terms) == 1) {
        search_filters[[filter_name]] <- field_data$terms[1]
      } else {
        query_parts <- c(field_data$terms[1])
        for (j in seq_along(field_data$operators)) {
          query_parts <- c(query_parts, field_data$operators[j], field_data$terms[j + 1])
        }
        search_filters[[filter_name]] <- paste(query_parts, collapse = " ")
      }
    }

    # Resolve entity-based filters to IDs
    for (field in names(entity_queries)) {
      field_data <- entity_queries[[field]]
      all_ids <- c()

      for (term in field_data$terms) {
        resolved <- resolve_entity_ids(field, term)
        if (!is.null(resolved)) {
          all_ids <- c(all_ids, resolved$ids)
        }
      }

      if (length(all_ids) > 0) {
        config <- entity_field_config[[field]]
        search_filters[[config$works_filter]] <- paste(unique(all_ids), collapse = "|")
      }
    }

    return(search_filters)
  }

  # Reactive: base search filters (without Topic and Journal)
  # Keyword filters come from submittedQueryFilters (set by Search button).
  # Advanced filters (DocType, Language, etc.) are reactive after search is submitted.
  baseSearchFilters <- reactive({
    search_filters <- submittedQueryFilters()
    if (is.null(search_filters)) return(NULL)

    # Add advanced filters (except Topic and Journal) - these are reactive
    if (showAdvanced()) {
      if (!is.null(input$oaDocType) && length(input$oaDocType) > 0) {
        doc_types <- input$oaDocType[input$oaDocType != ""]
        if (length(doc_types) > 0) search_filters$type <- doc_types
      }
      if (!is.null(input$oaLanguage) && length(input$oaLanguage) > 0) {
        languages <- input$oaLanguage[input$oaLanguage != ""]
        if (length(languages) > 0) search_filters$language <- languages
      }
      if (!is.null(input$oaOpenAccess) && length(input$oaOpenAccess) > 0) {
        oa_statuses <- input$oaOpenAccess[input$oaOpenAccess != ""]
        if (length(oa_statuses) > 0) search_filters$open_access.oa_status <- oa_statuses
      }
      if (!is.null(input$oaSDG) && length(input$oaSDG) > 0) {
        sdg_ids <- input$oaSDG[input$oaSDG != ""]
        if (length(sdg_ids) > 0) {
          search_filters$sustainable_development_goals.id <- paste0("https://metadata.un.org/sdg/", sdg_ids)
        }
      }
    }

    return(search_filters)
  })

  # Debounced version of baseSearchFilters for group_by observer (2s delay)
  baseSearchFiltersDebounced <- debounce(baseSearchFilters, 2000)

  # Raw data from group_by API (data.frames with key, key_display_name, count)
  keywordData <- reactiveVal(NULL)
  domainData <- reactiveVal(NULL)
  journalData <- reactiveVal(NULL)

  # Helper: build sorted named vector from group_by result
  sort_groupby_choices <- function(data, sort_order) {
    if (is.null(data) || nrow(data) == 0) return(character(0))
    if (sort_order == "alpha") {
      data <- data[order(data$key_display_name), ]
    } else {
      # Sort descending by count (ensure numeric comparison)
      data <- data[order(as.numeric(data$count), decreasing = TRUE), ]
    }
    setNames(data$key, paste0(data$key_display_name, " (", data$count, ")"))
  }

  # Observer: fetch Keyword, Domain and Journal raw data via group_by
  observe({
    base_filters <- baseSearchFiltersDebounced()

    if (is.null(base_filters)) {
      keywordData(NULL)
      domainData(NULL)
      journalData(NULL)
      shinyjs::disable("oaKeyword")
      shinyjs::disable("oaDomain")
      shinyjs::disable("oaJournal")
      shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
      shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
      shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")
      return()
    }

    # Grey out and disable while loading
    shinyjs::disable("oaKeyword")
    shinyjs::disable("oaDomain")
    shinyjs::disable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")

    tryCatch({
      keyword_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "keywords.id", verbose = FALSE), base_filters)
      )
      if (!is.null(keyword_result) && nrow(keyword_result) > 0) {
        keywordData(keyword_result)
      } else {
        keywordData(NULL)
      }
    }, error = function(e) {
      keywordData(NULL)
    })

    tryCatch({
      domain_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "topics.domain.id", verbose = FALSE), base_filters)
      )
      if (!is.null(domain_result) && nrow(domain_result) > 0) {
        domainData(domain_result)
      } else {
        domainData(NULL)
      }
    }, error = function(e) {
      domainData(NULL)
    })

    tryCatch({
      journal_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "primary_location.source.id", verbose = FALSE), base_filters)
      )
      if (!is.null(journal_result) && nrow(journal_result) > 0) {
        journalData(journal_result)
      } else {
        journalData(NULL)
      }
    }, error = function(e) {
      journalData(NULL)
    })

    # Re-enable after loading completes
    shinyjs::enable("oaKeyword")
    shinyjs::enable("oaDomain")
    shinyjs::enable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '1');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '1');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '1');")
  })

  # Observer: update dropdown choices when raw data or sort order changes
  observe({
    kd <- keywordData()
    dd <- domainData()
    jd <- journalData()
    sort_order <- input$oaDropdownSort

    current_keywords <- isolate(input$oaKeyword)
    current_domains <- isolate(input$oaDomain)
    current_journals <- isolate(input$oaJournal)

    kc <- sort_groupby_choices(kd, sort_order)
    if (length(kc) > 0) {
      valid_keywords <- current_keywords[current_keywords %in% kc]
      updateSelectizeInput(session, "oaKeyword", choices = kc, selected = valid_keywords)
    } else {
      updateSelectizeInput(session, "oaKeyword", choices = character(0), selected = character(0))
    }

    dc <- sort_groupby_choices(dd, sort_order)
    if (length(dc) > 0) {
      valid_domains <- current_domains[current_domains %in% dc]
      updateSelectizeInput(session, "oaDomain", choices = dc, selected = valid_domains)
    } else {
      updateSelectizeInput(session, "oaDomain", choices = character(0), selected = character(0))
    }

    jc <- sort_groupby_choices(jd, sort_order)
    if (length(jc) > 0) {
      journalChoices(jc)
      valid_journals <- current_journals[current_journals %in% jc]
      updateSelectizeInput(session, "oaJournal", choices = jc, selected = valid_journals)
    } else {
      journalChoices(NULL)
      updateSelectizeInput(session, "oaJournal", choices = character(0), selected = character(0))
    }
  })

  # Reactive: full search filters (base + Keyword + Domain + Journal)
  fullSearchFilters <- reactive({
    base_filters <- baseSearchFilters()
    if (is.null(base_filters)) return(NULL)

    search_filters <- base_filters

    if (showAdvanced()) {
      if (!is.null(input$oaKeyword) && length(input$oaKeyword) > 0) {
        keywords <- input$oaKeyword[input$oaKeyword != ""]
        if (length(keywords) > 0) search_filters$keywords.id <- keywords
      }
      if (!is.null(input$oaDomain) && length(input$oaDomain) > 0) {
        domains <- input$oaDomain[input$oaDomain != ""]
        if (length(domains) > 0) {
          # Extract numeric ID from full URL (e.g., "https://openalex.org/domains/2" -> "2")
          domains <- gsub("https://openalex.org/domains/", "", domains)
          search_filters$primary_topic.domain.id <- domains
        }
      }
      if (!is.null(input$oaJournal) && length(input$oaJournal) > 0) {
        journals <- input$oaJournal[input$oaJournal != ""]
        if (length(journals) > 0) search_filters$primary_location.source.id <- journals
      }
    }

    return(search_filters)
  })

  # Debounced version of fullSearchFilters for count query (1s delay)
  fullSearchFiltersDebounced <- debounce(fullSearchFilters, 1000)

  # Reactive query count - updates automatically when query changes
  output$oaQueryCountInfo <- renderUI({
    search_filters <- fullSearchFiltersDebounced()

    # If no query, show a hint
    if (is.null(search_filters)) {
      return(
        div(
          style = "display: inline-block; padding: 5px 10px; color: #6c757d; font-size: 13px; font-style: italic;",
          icon("info-circle"),
          " Enter search terms to see record count"
        )
      )
    }

    tryCatch(
      {
        # Get the count of available records
        count_result <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              count_only = TRUE,
              verbose = FALSE
            ),
            search_filters
          )
        )

        total_available <- as.integer(count_result$count)

        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #d4edda; color: #155724; border-radius: 4px; font-size: 13px; font-weight: 500;",
          icon("check-circle"),
          sprintf(" Found %s records", format(total_available, big.mark = ","))
        )
      },
      error = function(e) {
        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
          icon("exclamation-circle"),
          " Error: ",
          e$message
        )
      }
    )
  })

  # Download data from OpenAlex
  observeEvent(input$oaFetchData, {
    # Check if API key or polite email is configured
    oa_email <- Sys.getenv("openalexR.mailto", unset = "")
    oa_apikey <- Sys.getenv("openalexR.apikey", unset = "")
    if (oa_email == "" && oa_apikey == "") {
      showModal(modalDialog(
        title = "OpenAlex API Not Configured",
        div(
          style = "padding: 10px;",
          p(
            icon("exclamation-triangle", style = "color: #f39c12; margin-right: 8px;"),
            tags$strong("No API key or email configured for OpenAlex API.")
          ),
          p("Without an API key, you are limited to 100 credits/day (testing only). A free API key provides 100,000 credits/day."),
          p("Please go to ", tags$strong("Settings"), " and set your OpenAlex API key or email address."),
          p(
            style = "color: #666; font-size: 12px; margin-top: 15px;",
            "You can still proceed, but we strongly recommend setting an API key for better performance."
          )
        ),
        easyClose = TRUE,
        footer = tagList(
          modalButton("Cancel"),
          actionButton("oaProceedWithoutEmail", "Proceed anyway", class = "btn-warning")
        )
      ))
      return()
    }

    # Run the actual search
    oaRunSearch(search_filters = fullSearchFilters())
  })

  # Allow proceeding without email after warning
  observeEvent(input$oaProceedWithoutEmail, {
    removeModal()
    oaRunSearch(search_filters = fullSearchFilters())
  })

  # Extracted search logic into a function
  oaRunSearch <- function(search_filters) {
    # Validate that at least one query exists
    if (is.null(search_filters)) {
      showModal(modalDialog(
        title = "Error",
        "Please enter at least one search term.",
        easyClose = TRUE,
        footer = modalButton("OK")
      ))
      return()
    }

    # Show progress modal with dynamic content
    showModal(modalDialog(
      title = "Downloading from OpenAlex",
      div(
        div(
          style = "text-align: center; margin: 20px 0;",
          icon("spinner", class = "fa-spin fa-3x")
        ),
        div(
          id = "download-progress-text",
          style = "text-align: center; font-size: 14px;",
          "Initializing query..."
        )
      ),
      footer = NULL,
      easyClose = FALSE
    ))

    tryCatch(
      {
        # Calculate pages needed
        max_records <- min(input$oaMaxRecords, 10000)
        pages_needed <- ceiling(max_records / 200)

        print("Search filters:")
        print(search_filters)

        # First, get the count of available records
        count_result <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              count_only = TRUE,
              verbose = FALSE
            ),
            search_filters
          )
        )

        total_available <- as.integer(count_result$count)
        records_to_download <- min(total_available, max_records)

        # Update progress modal with count information
        shinyjs::html(
          "download-progress-text",
          sprintf(
            "Found %s records. Downloading %s records...",
            format(total_available, big.mark = ","),
            format(records_to_download, big.mark = ",")
          )
        )

        # Fetch data from OpenAlex as list
        oa_data <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              output = "list",
              abstract = TRUE,
              per_page = 200,
              pages = 1:pages_needed,
              verbose = TRUE
            ),
            search_filters
          )
        )

        # Update progress
        shinyjs::html(
          "download-progress-text",
          sprintf(
            "Converting %s records...",
            format(length(oa_data), big.mark = ",")
          )
        )

        # Create temporary directory if it doesn't exist
        temp_dir <- getWD()
        temp_file <- file.path(
          temp_dir,
          paste0("openalex_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".rdata")
        )

        # Save the list as .rdata file
        save(oa_data, file = temp_file)

        # Convert using bibliometrix convert2df function
        M <- convert2df(
          file = temp_file,
          dbsource = "openalex_api",
          format = "api"
        )

        # Clean up temporary file
        unlink(temp_file)

        # Limit to max records if necessary
        if (nrow(M) > max_records) {
          M <- M[1:max_records, ]
          oa_data <- oa_data[1:max_records]
        }

        # Resolve cited references if requested
        if (isTRUE(input$oaFetchRefs)) {
          only_multiple <- isTRUE(input$oaRefsFilter == "multiple")
          all_refs <- unlist(lapply(oa_data, function(w) w$referenced_works))
          all_refs <- all_refs[!is.na(all_refs) & all_refs != ""]
          n_total <- length(unique(all_refs))
          if (only_multiple) {
            ref_freq <- table(all_refs)
            n_to_resolve <- sum(ref_freq > 1)
          } else {
            n_to_resolve <- n_total
          }

          shinyjs::html(
            "download-progress-text",
            paste0(
              sprintf("Resolving %s of %s unique cited references...",
                      format(n_to_resolve, big.mark = ","),
                      format(n_total, big.mark = ",")),
              '<div style="margin-top: 10px; background-color: #e9ecef; border-radius: 4px; height: 20px; width: 100%;">',
              '<div id="download-progress-bar" style="background-color: #007bff; height: 100%; border-radius: 4px; width: 0%; transition: width 0.3s ease;"></div>',
              '</div>',
              '<div id="download-progress-pct" style="text-align: center; margin-top: 5px; font-size: 12px; color: #666;">0%</div>'
            )
          )

          ref_lookup <- resolve_openalex_references(oa_data,
            progress_id = "download-progress-text",
            progress_bar_id = "download-progress-bar",
            progress_pct_id = "download-progress-pct",
            only_multiple = only_multiple)

          if (length(ref_lookup) > 0) {
            shinyjs::html("download-progress-text", "Building citation strings...")
            cr_col <- build_cr_column(oa_data[1:nrow(M)], ref_lookup)
            M$CRids <- M$CR
            M$CR <- cr_col
          }
        }

        # Close progress modal
        removeModal()

        # Collapse query panel after successful download
        session$sendCustomMessage('oaCollapseQueryPanel', TRUE)

        # Show success message
        showModal(modalDialog(
          title = "Success",
          div(
            p(
              sprintf(
                "Successfully downloaded %s records from OpenAlex.",
                format(nrow(M), big.mark = ",")
              )
            ),
            if (total_available > max_records) {
              p(
                style = "color: #856404; background-color: #fff3cd; padding: 10px; border-radius: 5px;",
                sprintf(
                  "Note: %s total records found, but limited to %s records.",
                  format(total_available, big.mark = ","),
                  format(max_records, big.mark = ",")
                )
              )
            }
          ),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))

        values = initial(values)
        M <- M %>% mergeKeywords(force = F)
        # Update values
        values$data_source <- "OpenAlex"
        values$M <- M
        values$Morig = M
        values$SCdf <- wcTable(M)
        values$COdf <- countryTable(M)
        values$Histfield = "NA"
        values$results = list("NA")
        if (ncol(values$M) > 1) {
          values$rest_sidebar <- TRUE
        }
        if (ncol(values$M) > 1) {
          values$loadMenu <- "openalex_api"
          showModal(missingModalAPI(session))
        }
      },
      error = function(e) {
        removeModal()
        showModal(modalDialog(
          title = "Error",
          paste("An error occurred:", e$message),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))
      }
    )
  }

  # Render data table
  output$oaDataTable <- renderUI({
    req(values$M)
    req(values$data_source == "OpenAlex")

    MData = as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))
    MData$DOI <-
      paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
    nome = c("DOI", names(MData)[-length(names(MData))])
    MData = MData[nome]
    renderBibliobox(
      MData,
      nrow = 3,
      filename = "Table",
      pagelength = TRUE,
      left = NULL,
      right = NULL,
      numeric = NULL,
      dom = TRUE,
      size = '70%',
      filter = "top",
      columnShort = NULL,
      columnSmall = NULL,
      round = 2,
      title = "",
      button = FALSE,
      escape = FALSE,
      selection = FALSE,
      scrollX = TRUE
    )
  })

  # Check if data is available for conditional panel
  output$oaDataAvailable <- reactive({
    !is.null(values$M) && values$data_source == "OpenAlex"
  })
  outputOptions(output, "oaDataAvailable", suspendWhenHidden = FALSE)

  # Display progress information
  output$oaProgressInfo <- renderUI({
    if (!is.null(values$M) && values$data_source == "OpenAlex") {
      div(
        class = "alert alert-success",
        icon("check-circle"),
        strong(" Data loaded: "),
        sprintf(
          "%s documents from OpenAlex",
          format(nrow(values$M), big.mark = ",")
        )
      )
    }
  })

  # Download handler for Excel
  output$oaDownloadExcel <- downloadHandler(
    filename = function() {
      paste0("openalex_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "OpenAlex")

      # Write data to Excel file
      openxlsx::write.xlsx(values$M, file = file)
    }
  )

  # Download handler for RData
  output$oaDownloadRData <- downloadHandler(
    filename = function() {
      paste0("openalex_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".RData")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "OpenAlex")

      # Save data as RData file
      M <- values$M
      save(M, file = file)
    }
  )

  missingModalAPI <- function(session) {
    ns <- session$ns
    modalDialog(
      uiOutput("missingTitle"),
      uiOutput(ns("missingDataTable")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(
          label = "Advice",
          inputId = "missingMessage",
          icon = icon("exclamation-sign", lib = "glyphicon")
        ),
        actionButton(
          label = "Report",
          inputId = "missingReport",
          icon = icon("plus", lib = "glyphicon")
        ),
        actionButton(
          label = "Save",
          inputId = "missingDataTable",
          icon = icon("camera", lib = "glyphicon")
        ),
        modalButton(label = "Close")
      ),
    )
  }
}

## OpenRouter AI function
openrouter_ai <- function(
  image = NULL,
  docs = NULL,
  prompt = "Explain these images",
  model = "google/gemma-3-27b-it:free", # Nota: usa 27b, il 32b potrebbe avere ID diverso
  image_type = "png",
  retry_503 = 5,
  api_key = NULL,
  outputSize = "medium"
) {
  library(httr2)
  library(base64enc)

  # 1. Setup API Key
  if (is.null(api_key)) {
    api_key <- Sys.getenv("OPENROUTER_API_KEY")
  }

  # 2. Configurazione Tokens (Semplificata per evitare 400)
  # Molti modelli free falliscono se chiedi troppi token in uscita
  max_val <- switch(
    outputSize,
    "small" = 1000,
    "medium" = 2000,
    "large" = 4000,
    2000
  )

  # 3. Costruzione Content (Standard rigoroso OpenAI)
  content_list <- list(
    list(type = "text", text = prompt)
  )

  if (!is.null(image)) {
    for (img_path in as.vector(image)) {
      if (file.exists(img_path)) {
        # Importante: usiamo una compressione base64 pulita
        img_b64 <- base64encode(img_path)
        content_list <- append(
          content_list,
          list(
            list(
              type = "image_url",
              image_url = list(
                url = paste0("data:image/", image_type, ";base64,", img_b64)
              )
            )
          )
        )
      }
    }
  }

  # 4. Request Body
  # Rimosso 'seed' e altri parametri opzionali che possono causare 400 su certi provider
  request_body <- list(
    model = model,
    messages = list(
      list(role = "user", content = content_list)
    ),
    max_tokens = max_val,
    temperature = 0.7
  )

  # 5. Invio Richiesta
  url <- "https://openrouter.ai/api/v1/chat/completions"

  req <- request(url) |>
    req_headers(
      "Authorization" = paste("Bearer", api_key),
      "HTTP-Referer" = "https://github.com/massimoaria/bibliometrix", # Referer specifico
      "X-Title" = "Biblioshiny Client",
      "Content-Type" = "application/json"
    ) |>
    req_body_json(request_body) |>
    req_timeout(120) |>
    req_retry(max_tries = retry_503)

  resp <- req_perform(req) |>
    resp_body_json()

  if (!is.null(resp$choices)) {
    return(resp$choices[[1]]$message$content)
  } else {
    return(paste("❌ Errore API:", resp$error$message))
  }
}

## gemini AI tools
gemini_ai <- function(
  image = NULL,
  docs = NULL,
  prompt = "Explain these images",
  model = "2.5-flash",
  image_type = "png",
  retry_503 = 5,
  api_key = NULL,
  outputSize = "medium"
) {
  mime_doc_types <- list(
    pdf = "application/pdf",
    txt = "text/plain",
    html = "text/html",
    csv = "text/csv",
    rtf = "text/rtf"
  )

  switch(
    outputSize,
    "small" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "medium" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 16384, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "large" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 32768, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "huge" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 131072, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    }
  )

  # # Default config
  # generation_config <- list(
  #   temperature = 1,
  #   maxOutputTokens = 16384,#8192,
  #   topP = 0.95,
  #   topK = 40,
  #   seed = 1234
  # )

  # Gemma models on the Gemini API: no "gemini-" prefix, no `seed`,
  # and a capped output window (open Gemma models max out at 8192 tokens).
  is_gemma <- startsWith(model, "gemma")
  if (is_gemma) {
    generation_config$seed <- NULL
    generation_config$maxOutputTokens <- min(
      generation_config$maxOutputTokens,
      8192
    )
  }

  # Build URL
  model_query <- if (is_gemma) {
    paste0(model, ":generateContent")
  } else {
    paste0("gemini-", model, ":generateContent")
  }
  url <- paste0(
    "https://generativelanguage.googleapis.com/v1beta/models/",
    model_query
  )
  if (is.null(api_key)) {
    api_key <- Sys.getenv("GEMINI_API_KEY")
  }

  # Base structure of parts
  parts <- list(list(text = prompt))

  # Handle images if provided
  if (!is.null(image)) {
    if (!is.vector(image)) {
      image <- as.vector(image)
    }
    mime_type <- paste0("image/", image_type)

    for (img_path in image) {
      if (!file.exists(img_path)) {
        return(paste0("❌ Error: Image file does not exist: ", img_path))
      }

      image_data <- tryCatch(
        base64enc::base64encode(img_path),
        error = function(e) {
          return(NULL)
        }
      )

      if (is.null(image_data)) {
        return(paste0("❌ Failed to encode image: ", img_path))
      }

      parts <- append(
        parts,
        list(
          list(
            inline_data = list(
              mime_type = mime_type,
              data = image_data
            )
          )
        )
      )
    }
  }

  # Handle documents if provided
  if (!is.null(docs)) {
    if (!is.vector(docs)) {
      docs <- as.vector(docs)
    }
    for (doc_path in docs) {
      if (!file.exists(doc_path)) {
        return(paste0("❌ Error: Document file does not exist: ", doc_path))
      }

      doc_data <- tryCatch(
        base64enc::base64encode(doc_path),
        error = function(e) {
          return(NULL)
        }
      )

      if (is.null(doc_data)) {
        return(paste0("❌ Failed to encode document: ", doc_path))
      }

      doc_type <- tools::file_ext(doc_path) |> tolower()

      if (doc_type %in% names(mime_doc_types)) {
        mime_type <- mime_doc_types[[doc_type]]
      } else {
        mime_type <- "application/pdf" # Default to PDF if unknown type
      }

      parts <- append(
        parts,
        list(
          list(
            inline_data = list(
              mime_type = "application/pdf",
              data = doc_data
            )
          )
        )
      )
    }
  }

  # Assemble request body
  request_body <- list(
    contents = list(
      parts = parts
    ),
    generationConfig = generation_config
  )

  # Retry loop
  for (attempt in seq_len(retry_503)) {
    # Build and send request
    req <- request(url) |>
      req_url_query(key = api_key) |>
      req_headers("Content-Type" = "application/json") |>
      req_body_json(request_body) |>
      req_timeout(120)

    resp <- tryCatch(
      req_perform(req),
      error = function(e) {
        return(list(
          status_code = stringr::str_extract(e$message, "(?<=HTTP )\\d+") |>
            as.numeric(),
          error = TRUE,
          message = paste("❌ Request failed with error:", e$message)
        ))
      }
    )

    # # Handle connection-level error
    # if (is.list(resp) && isTRUE(resp$error)) {
    #   return(resp$message)
    # }

    # Retry on transient server errors (429 rate limit, 5xx server errors)
    if (resp$status_code %in% c(429, 500, 502, 503, 504)) {
      if (attempt < retry_503) {
        message(paste0(
          "⚠️ HTTP ", resp$status_code,
          " (transient server error) - retrying (attempt ",
          attempt,
          "/",
          retry_503,
          ")..."
        ))
        Sys.sleep(min(2^attempt, 16))
        next
      } else {
        return(
          paste0(
            "❌ HTTP ", resp$status_code,
            ": the AI service is temporarily unavailable.\n",
            "The Google Gemini servers are currently overloaded or unstable.\n",
            "All retry attempts failed (",
            retry_503,
            "). Please try again in a few minutes, or select a different AI model."
          )
        )
      }
    }

    # HTTP errors
    # 400 api key not valid
    if (resp$status_code == 400) {
      msg <- tryCatch(
        {
          parsed <- jsonlite::fromJSON(httr2::resp_body_string(resp))
          parsed$error$message
        },
        error = function(e) {
          "Please check your API key. It seems to be not valid!"
        }
      )
      return(paste0("❌ HTTP ", resp$status_code, ": ", msg))
    }
    # Other HTTP errors
    if (resp$status_code != 200) {
      msg <- tryCatch(
        {
          parsed <- jsonlite::fromJSON(httr2::resp_body_string(resp))
          parsed$error$message
        },
        error = function(e) {
          "Service unavailable or unexpected error. Please check your API key and usage limit."
        }
      )

      return(paste0("❌ HTTP ", resp$status_code, ": ", msg))
    }

    # Successful response — concatenate answer text, skipping any "thought"
    # parts (thinking models such as Gemma 4 return their reasoning in a
    # separate part flagged with thought = TRUE).
    candidates <- httr2::resp_body_json(resp)$candidates
    answer <- unlist(lapply(candidates, function(cand) {
      lapply(cand$content$parts, function(p) {
        if (isTRUE(p$thought)) NULL else p$text
      })
    }))
    answer <- answer[!is.na(answer) & nzchar(answer)]
    if (length(answer) == 0) {
      return(paste0(
        "❌ The model returned an empty response. ",
        "Please try again or select a different AI model."
      ))
    }
    return(paste(answer, collapse = "\n"))
  }
}


setGeminiAPI <- function(api_key) {
  # 1. Controllo validità dell'API key
  apiCheck <- gemini_ai(
    image = NULL,
    prompt = "Hello",
    model = "2.5-flash",
    image_type = "png",
    retry_503 = 5,
    api_key = api_key
  )

  contains_http_error <- grepl("HTTP\\s*[1-5][0-9]{2}", apiCheck)

  if (contains_http_error) {
    return(list(
      valid = FALSE,
      message = "❌ API key seems be not valid! Please, check it or your connection."
    ))
  }

  if (is.null(api_key) || !is.character(api_key) || nchar(api_key) == 0) {
    return(list(
      valid = FALSE,
      message = "❌ API key must be a non-empty string."
    ))
  }

  if (nchar(api_key) < 10) {
    return(list(
      valid = FALSE,
      message = "❌ API key seems too short. Please verify your key."
    ))
  }

  # 2. Mostra solo gli ultimi 4 caratteri per feedback
  last_chars <- 4
  last <- substr(
    api_key,
    max(1, nchar(api_key) - last_chars + 1),
    nchar(api_key)
  )

  # 3. Imposta la variabile d'ambiente
  Sys.setenv(GEMINI_API_KEY = api_key)

  return(list(
    valid = TRUE,
    message = paste0(
      paste0(rep("*", nchar(api_key) - 4), collapse = ""),
      last,
      collapse = ""
    )
  ))
}

showGeminiAPI <- function() {
  api_key <- Sys.getenv("GEMINI_API_KEY")
  last_chars <- 4
  last <- substr(
    api_key,
    max(1, nchar(api_key) - last_chars + 1),
    nchar(api_key)
  )
  last <- paste0(
    paste0(rep("*", nchar(api_key) - 4), collapse = ""),
    last,
    collapse = ""
  )
  return(last)
}

load_api_key <- function(path = path_gemini_key) {
  if (file.exists(path)) {
    key <- readLines(path, warn = FALSE)
    if (nchar(key) >= 10) {
      Sys.setenv(GEMINI_API_KEY = key)
      return(TRUE)
    }
  }
  return(FALSE)
}

loadGeminiModel = function(file) {
  # load info about model type and output size
  if (file.exists(file)) {
    model <- readLines(file, warn = FALSE)
  } else {
    model <- c("2.5-flash-lite", "medium")
  }
  if (length(model == 1)) {
    model <- c(model, "medium")
  }
  return(model)
}

saveGeminiModel = function(model, file) {
  if (file.exists(file)) {
    file.remove(file)
  }
  writeLines(model, file)
}

geminiOutput <- function(title = "", content = "", values) {
  if (is.null(content)) {
    content <- "Click the 'Ask Biblio AI' button to analyze the visual outputs and provide an automatic interpretation of your results based on the graphs.\n
This AI-powered feature leverages Google Gemini to help you understand patterns and insights emerging from your contextual analysis.\n  \n  \n \n  \n  \n  \n"
  }
  box(
    title = title,
    width = 12,
    status = "info",
    solidHeader = TRUE,
    div(
      id = "typing-box",
      style = "white-space: pre-wrap; background-color:#f9f9f9; padding:15px; border:1px solid #ccc; border-radius:5px; max-height:400px; overflow-y: auto;",
      #HTML(text_to_html(content))
      HTML(gemini_to_html(content))
    ),
    br(),
    em(
      "You can modify or enrich the proposed prompt with additional context or details about your analysis to help 'Biblio AI' generate a more accurate and meaningful interpretation."
    ),
    textAreaInput(
      inputId = "gemini_additional",
      label = NULL,
      value = values$gemini_model_parameters,
      placeholder = "You can provide additional context or details about your analysis to help 'Biblio AI' generate a more accurate and meaningful interpretation.",
      rows = 3,
      width = "100%"
    ),
    fluidRow(
      column(
        4,
        align = "center",
        actionButton(
          "gemini_btn",
          "Ask Biblio AI",
          style = "color: white;",
          icon(name = "microchip", lib = "font-awesome"),
          width = "80%"
        )
      ),
      column(
        4,
        align = "center",
        actionButton(
          "copy_btn",
          "Copy",
          style = "color: white;",
          icon = icon("clipboard"),
          width = "80%"
        )
      ),
      column(
        4,
        align = "center",
        downloadButton(
          outputId = "save_btn",
          label = "Save",
          icon = icon("download"),
          style = "width: 80%;"
        )
      )
    )
  )
}

biblioAiPrompts <- function(values, activeTab) {
  ## ROle definition for Gemini as Biblio AI assistant
  promptInitial <- "You are Biblio AI, an AI assistant integrated within Biblioshiny. Your task is to support researchers in interpreting and critically discussing the results of their bibliometric analyses, offering insights, contextual explanations, and guidance for data-driven interpretation. "

  switch(
    activeTab,
    "mainInfo" = {
      prompt <- paste0(
        "Provide an interpretation of these statistics summarizing the bibliographic collection. ",
        "Focus on key metrics such as the number of documents, sources, authors, and citations, ",
        "and discuss what they reveal about the scope, productivity, and impact of the collection. ",
        "This is the list of statistics: ",
        merge_df_to_string(values$TABvb)
      )
    },
    "lifeCycle" = {
      prompt <- paste0(
        "Analyze the provided 'life cycle' plots showing the bibliographic collection's scientific production fitted to a logistic growth model. ",
        "You have been provided with: (1) an ANNUAL plot showing yearly publication counts, (2) a CUMULATIVE plot showing the total publications over time, ",
        "and (3) the logistic model parameters and goodness-of-fit metrics.\n\n",

        "Please provide a comprehensive interpretation covering:\n",
        "1. GROWTH PATTERN: Describe the overall growth trajectory (exponential, linear, or saturation phase). ",
        "Identify the inflection point where growth rate changes.\n",
        "2. MODEL FIT: Evaluate how well the logistic curve fits the observed data based on the provided metrics (R², RMSE, etc.). ",
        "Comment on any systematic deviations.\n",
        "3. KEY PARAMETERS: Interpret the logistic model parameters:\n",
        "   - K (carrying capacity): the estimated maximum cumulative production\n",
        "   - r (growth rate): the intrinsic rate of increase\n",
        "   - t₀ (inflection point): when maximum growth rate occurred\n",
        "4. TRENDS AND PHASES: Identify distinct phases in the publication timeline:\n",
        "   - Emergence phase (slow initial growth)\n",
        "   - Expansion phase (rapid growth)\n",
        "   - Maturity/saturation phase (if present)\n",
        "5. SCIENTIFIC IMPLICATIONS: Discuss what these patterns suggest about the research field's maturity, ",
        "potential factors influencing growth (e.g., funding, technological advances, paradigm shifts), ",
        "and future trajectories. Is the field still growing or approaching saturation?\n\n",

        "Provide your analysis in a structured format with clear sections.",
        "This a the list of statistics: Parameters ",
        merge_df_to_string(values$DLC$parameters),
        ", parameters_real_years ",
        merge_df_to_string(values$DLC$parameters_real_years),
        ", and goodness_of_fit ",
        merge_df_to_string(values$DLC$metrics)
      )
    },
    "threeFieldPlot" = {
      prompt <- paste0(
        "Provide an interpretation of this Three-Field Plot. The central field represents the target metadata,",
        " while the left and the right fields are the destination metadata. The plot visualizes the connections",
        " among these three dimensions by linking the target field witgh the two other metadata.",
        " Analyze the structure of the network, and describe how these elements are interconnected."
      )
    },
    "authorsProdOverTime" = {
      docs <- apot2Docs(values$AUProdOverTime, n = 3) %>% merge_df_to_string()
      prompt <- paste0(
        "Provide an interpretation of this 'Authors’ Production Over Time' plot, which displays bibliometric trends",
        " for the top authors in the field. For each author, the red horizontal line represents their scientific timeline,",
        " indicating the span of their active publishing years. Each bubble corresponds to a publication year: its size reflects",
        " the number of articles published that year, while the color intensity represents the total number of citations per year (TC/year).",
        " Analyze the evolution of individual authors’ productivity and impact, identifying key periods of activity, citation peaks, ",
        "and possible shifts in research influence over time. A list of the most three cited-per-year articles for each author is provided: ",
        docs
      )
    },
    "correspAuthorCountry" = {
      countries <- merge_df_to_string(country2collab(values$TABCo))
      prompt <- paste0(
        "Provide an interpretation of this 'Corresponding Author’s Country Collaboration Plot'. The plot displays the number of scientific publications",
        " by corresponding authors for each country, distinguishing between Single Country Publications (SCP) and Multiple Country Publications (MCP). ",
        "SCP represents articles authored exclusively within one country, while MCP refers to collaborative works involving authors from different countries. ",
        "The MCP Ratio, calculated as MCP divided by the total number of publications, indicates the extent of international collaboration. ",
        "Focus on identifying the most productive countries, comparing their levels of international collaboration, and discussing the balance between domestic ",
        "and global research engagement. A list of the most 20 frequent countries is provided: ",
        countries
      )
    },
    "mostLocalCitDoc" = {
      docs <- localCit2docs(values$TABLocDoc, n = 20) %>% merge_df_to_string()
      prompt <- paste0(
        "Provide an interpretation of this table reporting the most locally and globally cited articles in the collection. Global Citations (GC) represent the total",
        " number of citations a document has received across all records indexed in the bibliographic database, indicating its overall scholarly impact. Local Citations (LC),",
        " on the other hand, count the number of citations a document has received within this specific dataset, reflecting its relevance to the focused research field. ",
        "Both indicators are normalized by NTC (Normalized Total Citations), which adjusts citation counts relative to the average number of citations for publications from the same year.",
        " This normalization allows for fair comparison across time. Focus on identifying articles with strong global influence, local relevance, or both.",
        "A list of the most 20 local cited articles is provided: ",
        docs
      )
    },
    "trendTopic" = {
      prompt <- paste0(
        "Provide an interpretation of this trend topics plot, where each year is associated with the k words that have the highest annual median frequency. ",
        "For each word, the light blue line represents the interquartile range (from the first to the third quartile) of its frequency time series, and the central",
        "point marks the median value. The size of the bubbles is proportional to the annual frequency, reflecting the relative prominence of each term over time. "
      )
    },
    "ReferenceSpect" = {
      references <- merge_df_to_string(values$res$peaks)
      sequenceTop5 <- values$res$Sequences %>%
        filter(Class != "") %>%
        group_by(Class) %>%
        slice_max(Freq, n = 5) %>%
        rename("Totale Citation" = "Freq")
      prompt <- paste0(
        "Provide an interpretation of this Reference Publication Year Spectroscopy plot (RPYS, Marx et al. 2014, JAIST).",
        " The black line shows the number of cited references by publication year. ",
        "The red line represents the deviation from the 5-year median citation frequency.",
        " This highlights peak years of historical significance. A list of the most cited references is provided for the first ten peak years identified by the red line,",
        " along with their citation frequencies: ",
        references,
        "\nIn addition, please highlight which references are the most important according to the following categories: Hot Paper, Life Cycle, Sleeping Beauty,",
        "
                         and Constant Performer (Thor A. et al. 2018, Identifying single influential publications in a research field: new analysis opportunities of the CRExplorer. Scientometrics).",
        " Below is the list of references with their assigned category.",
        sequenceTop5
      )
    },
    "coOccurenceNetwork" = {
      prompt <- paste0(
        "Provide an interpretation of this 'word co-occurrence' network.",
        " Focus on the structure of the network, the presence of communities (topics), and the relevance of the most connected terms."
      )
      if (!is.null(values$cocnet$doc2clust)) {
        docs <- merge_df_to_string(doc2clust(values$cocnet))
        prompt <- paste0(
          prompt,
          " Here there is the list of three most central articles for each cluster: ",
          docs,
          ". Also describe the articles belonging to each community."
        )
      }
    },
    "thematicMap" = {
      docs <- merge_df_to_string(doc2clust(values$TM))
      prompt <- paste0(
        "Provide an interpretation of this 'strategic map'",
        "Here there is the list of three most central articles for each cluster: ",
        docs,
        ". Focus on the structure of the strategic map, the description of clusters and their centrality and density measures, and the description of the articles belonging to them."
      )
    },
    "thematicEvolution" = {
      prompt <- paste0(
        "Provide an interpretation of these plots.",
        "The first plot illustrates topic evolution across different time periods, showing how thematic areas develop or shift over time. ",
        "The other plots display strategic maps for each period, highlighting the positioning of topics based on centrality and density. ",
        "Focus on identifying emerging, declining, or stable topics and their strategic importance."
      )
    },
    "factorialAnalysis" = {
      prompt <- "Provide an interpretation of this 'factorial map'. Focus on the structure of the map, the presence of clusters, and the relevance of the most contributing terms."
    },
    "coCitationNetwork" = {
      prompt <- "Provide an interpretation of this 'co-citation' network. Focus on the structure of the network, the presence of communities, and the relevance of the most connected terms."
    },
    "historiograph" = {
      titles <- hist2docs(values$histPlotVis$VIS$x$nodes) %>%
        merge_df_to_string()
      prompt <- paste0(
        "Interpret this historiograph, a temporal citation network built by mapping direct citation links among documents. ",
        "Highlight the main citation paths, pivotal works, and any notable temporal trends in knowledge development, looking also to the article titles and their topics.",
        " Focus on the temporal evolution of each cluster",
        "Here there is the list of each paper (node) and its title: ",
        titles
      )
    },
    "collabNetwork" = {
      prompt <- paste0(
        "Provide an interpretation of this 'collaboration' network",
        ". Focus on the structure of the network, the presence of communities, and the relevance of the most connected terms."
      )
    },
    "collabWorldMap" = {
      #values$WMGemini
      prompt <- paste0(
        "Provide an interpretation of this ‘Countries’ Collaboration World Map’. The map visualizes international scientific ",
        "collaboration by showing, for each country, the total number of articles with at least one contributing author. ",
        "The color intensity of each country is proportional to its research output. The connecting lines represent collaborative ",
        "links between countries, based on co-authorship across all authors (not only corresponding authors). ",
        "Focus on identifying major hubs of scientific production, key international partnerships, and global patterns of collaboration."
      )
    },
    "bradford" = {
      prompt <- paste0(
        "Provide an interpretation of this Bradford bibliograph and Bradford’s Law analysis. ",
        "The plot shows the cumulative number of articles (y-axis) against the logarithm of source rank (x-axis). ",
        "The solid black line is the empirical curve and the dashed red line is the theoretical fitted Bradford distribution: C(r) = a + b * log(r). ",
        "The three shaded zones represent the Bradford zones: Zone 1 (core journals), Zone 2 (intermediate), and Zone 3 (periphery). ",
        "Each zone contains approximately one-third of the total articles, but the number of journals increases geometrically (ratio 1:k:k^2). ",
        "Focus on: (1) the degree of concentration vs dispersion in the field’s publication landscape, ",
        "(2) the shape of the curve (steep initial drop = high concentration, flatter = more dispersion), ",
        "(3) how well the empirical data fits the theoretical Bradford distribution (R-squared and KS test), ",
        "(4) the Bradford multiplier k and what it reveals about the zone structure, ",
        "(5) identifying and discussing the core journals and their role in the field."
      )
    },
    "lotka" = {
      prompt <- paste0(
        "Provide an interpretation of this Lotka’s Law plot and analysis. ",
        "The plot shows the author productivity distribution: the percentage of authors (y-axis) who published a given number of documents (x-axis). ",
        "Three curves are displayed: (1) the solid black line is the empirical distribution, ",
        "(2) the dashed red line is the theoretical Lotka’s Law with Beta=2 (inverse square law), ",
        "(3) the dot-dash blue line is the fitted generalized inverse power law with the estimated Beta. ",
        "Lotka’s Law states that f(n) = C / n^Beta, where f(n) is the proportion of authors publishing n articles. ",
        "Focus on: (1) how well the empirical data fits the theoretical (Beta=2) and fitted distributions (KS test results), ",
        "(2) the estimated Beta value and what it reveals about productivity concentration in the field, ",
        "(3) the proportion of single-paper authors vs prolific authors, ",
        "(4) whether Beta > 2 (more concentrated, typical of niche fields) or Beta < 2 (more dispersed, typical of broad fields), ",
        "(5) implications for the field’s author productivity patterns."
      )
    },
    {
      prompt <- paste0(
        "Provide an interpretation of this plot creted with ‘bibliometrix R Package’"
      )
    }
  )
  prompt <- paste0(promptInitial, prompt)
  #if (!activeTab %in% c("mainInfo", "thematicMap", "trendTopic")) prompt <- paste0(prompt, " Provide also scientific references about the methodological description")
  return(prompt)
}


geminiParameterPrompt <- function(values, activeTab, input) {
  if ("DB_Original" %in% names(values$M)) {
    DB <- unique(values$M$DB_Original)
    if (length(DB) > 1) {
      DB[length(DB)] <- paste0("and ", DB[length(DB)])
    }
    DB <- gsub("ISI", "WOS", paste0(DB, collapse = ", "))

    DB <- paste0(
      "databases ",
      DB,
      " merged by Bibliometrix R package routines."
    )
  } else {
    if (values$M$DB[1] == "ISI") {
      DB <- "WOS"
    } else {
      DB <- values$M$DB[1]
    }
    DB <- paste0("database ", DB, ".")
  }
  txt <- paste0(
    "The analysis was performed on a collection downloaded from the following bibliographic ",
    DB
  )
  missingTags <- values$missingdf %>%
    filter(status %in% "Completely missing") %>%
    pull(description)
  if (length(missingTags) > 0) {
    if ("DB_Original" %in% names(values$M)) {
      txt1 <- paste0(
        " Some metadata fields like ",
        paste0(missingTags, collapse = ", "),
        " are missing due to limitations in the merging process and 
                                                           non-standard formats across databases."
      )
      txt <- paste0(txt, txt1)
    }
  }

  switch(
    activeTab,
    "mainInfo" = {
      req(values$TABvb)
    },
    "lifeCycle" = {
      req(values$DLC)
    },
    "threeFieldPlot" = {
      req(values$TFP)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: target field '",
        input$CentralField,
        "', right field '",
        input$RightField,
        "', and left field '",
        input$LeftField,
        "'."
      )
    },
    "trendTopic" = {
      req(values$trendTopics)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: ",
        merge_df_to_string(values$trendTopics$params)
      )
    },
    "authorsProdOverTime" = {
      req(values$AUProdOverTime)
    },
    "correspAuthorCountry" = {
      req(values$TABCo)
    },
    "mostLocalCitDoc" = {
      req(values$TABLocDoc)
    },
    "ReferenceSpect" = {
      req(values$res)
    },
    "coOccurenceNetwork" = {
      req(values$COCnetwork)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: ",
        merge_df_to_string(values$cocnet$params)
      )
    },
    "thematicMap" = {
      req(values$TMmap)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$TM$params)
      )
    },
    "thematicEvolution" = {
      req(values$nexus)
      txt <- paste0(
        txt,
        " The following parameters were used for each strategic map: ",
        merge_df_to_string(values$nexus$TM[[1]]$params)
      )
    },
    "factorialAnalysis" = {
      req(values$plotCS)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$CS$params)
      )
    },
    "coCitationNetwork" = {
      req(values$COCITnetwork)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$cocitnet$params)
      )
    },
    "historiograph" = {
      req(values$histPlotVis$VIS)
    },
    "collabNetwork" = {
      req(values$COLnetwork$VIS)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$colnet$params)
      )
    },
    "collabWorldMap" = {
      req(values$WMmap)
    },
    "bradford" = {
      req(values$bradford)
      s <- values$bradford$stat
      zs <- values$bradford$zoneSummary
      txt <- paste0(
        txt,
        " Bradford's Law analysis results: ",
        "Fitted model C(r) = ", round(s$a, 2), " + ", round(s$b, 2), " * log(r), ",
        "R-squared = ", round(s$R2, 4), ", ",
        "Bradford multiplier k = ", round(s$k, 2), ", ",
        "KS test p-value = ", format.pval(s$ks.pvalue, digits = 3), ". ",
        "Zone summary: ", merge_df_to_string(zs)
      )
    },
    "lotka" = {
      req(values$lotka)
      s <- values$lotka$stat
      txt <- paste0(
        txt,
        " Lotka's Law analysis results: ",
        "Estimated Beta = ", round(s$Beta, 4), ", ",
        "Constant C = ", round(s$C, 4), ", ",
        "R-squared = ", round(s$R2, 4), ", ",
        "KS p-value (theoretical Beta=2) = ", format.pval(s$ks.theo.pvalue, digits = 3), ", ",
        "KS p-value (fitted Beta) = ", format.pval(s$ks.fit.pvalue, digits = 3), ". ",
        "Author productivity table: ", merge_df_to_string(values$lotka$AuthorProd)
      )
    }
  )
  return(txt)
}



geminiWaitingMessage <- function(values, activeTab) {
  messageTxt <- "⌛ Thinking..."

  switch(
    activeTab,
    "mainInfo" = {
      req(values$TABvb)
      values$MainInfoGemini <- messageTxt
    },
    "lifeCycle" = {
      req(values$DLC)
      values$DLCGemini <- messageTxt
    },
    "threeFieldPlot" = {
      req(values$TFP)
      values$TFPGemini <- messageTxt
    },
    "authorsProdOverTime" = {
      req(values$AUProdOverTime)
      values$ApotGemini <- messageTxt
    },
    "correspAuthorCountry" = {
      req(values$TABCo)
      values$MostRelCountriesGemini <- messageTxt
    },
    "mostLocalCitDoc" = {
      req(values$TABLocDoc)
      values$MostLocCitDocsGemini <- messageTxt
    },
    "trendTopic" = {
      req(values$trendTopics)
      values$trendTopicsGemini <- messageTxt
    },
    "ReferenceSpect" = {
      req(values$res)
      values$rpysGemini <- messageTxt
    },
    "coOccurenceNetwork" = {
      req(values$COCnetwork)
      values$cocGemini <- messageTxt
    },
    "thematicMap" = {
      req(values$TM)
      values$TMGemini <- messageTxt
    },
    "thematicEvolution" = {
      req(values$nexus)
      values$TEGemini <- messageTxt
    },
    "factorialAnalysis" = {
      req(values$plotCS)
      values$CSGemini <- messageTxt
    },
    "coCitationNetwork" = {
      req(values$COCITnetwork)
      values$cocitGemini <- messageTxt
    },
    "historiograph" = {
      req(values$histPlotVis$VIS)
      values$histGemini <- messageTxt
    },
    "collabNetwork" = {
      req(values$COLnetwork$VIS)
      values$colGemini <- messageTxt
    },
    "collabWorldMap" = {
      req(values$WMmap)
      values$WMGemini <- messageTxt
    },
    "bradford" = {
      req(values$bradford)
      values$BradfordGemini <- messageTxt
    },
    "lotka" = {
      req(values$lotka)
      values$LotkaGemini <- messageTxt
    }
  )
  return(values)
}

## Map activeTab to reactive field name in values
geminiFieldName <- function(activeTab) {
  switch(
    activeTab,
    "mainInfo" = "MainInfoGemini",
    "lifeCycle" = "DLCGemini",
    "threeFieldPlot" = "TFPGemini",
    "authorsProdOverTime" = "ApotGemini",
    "correspAuthorCountry" = "MostRelCountriesGemini",
    "mostLocalCitDoc" = "MostLocCitDocsGemini",
    "trendTopic" = "trendTopicsGemini",
    "ReferenceSpect" = "rpysGemini",
    "coOccurenceNetwork" = "cocGemini",
    "thematicMap" = "TMGemini",
    "thematicEvolution" = "TEGemini",
    "factorialAnalysis" = "CSGemini",
    "coCitationNetwork" = "cocitGemini",
    "historiograph" = "histGemini",
    "collabNetwork" = "colGemini",
    "collabWorldMap" = "WMGemini",
    "bradford" = "BradfordGemini",
    "lotka" = "LotkaGemini",
    NULL
  )
}

## Prepare image files for Gemini (synchronous part of geminiPromptImage)
geminiPrepareImage <- function(obj, type, prompt, key, desc, values) {
  # Check internet connection
  if (!is_online()) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = '⚠️ **Note**: Biblio AI requires an active internet connection to work.'
    ))
  }
  # Check Chrome browser
  if (is.null(values$Chrome_url)) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = '⚠️ **Note**: Biblio AI requires a **Chrome-based browser** (such as Google Chrome or Microsoft Edge) installed on your computer to work correctly.'
    ))
  }

  if (!key) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = 'To access this feature, please provide a valid Gemini AI API key. You can obtain your API key by visiting the official <a href="https://aistudio.google.com/" target="_blank">Google AI Studio website</a>.'
    ))
  }

  # Build prompt with description
  if (!is.null(desc)) {
    prompt <- paste0(prompt, desc, collapse = ". ")
  }

  # Convert plot to PNG
  file_path <- tempfile(fileext = ".png", tmpdir = getWD())

  result <- tryCatch({
    switch(
      type,
      "vis" = {
        plot2pngGemini(obj, filename = file_path, type = "vis")
      },
      "plotly" = {
        plot2pngGemini(obj, filename = file_path, type = "plotly")
      },
      "multi" = {
        file_path <- obj
      },
      "text" = {
        file_path <- NULL
      },
      "ggplot2" = {
        ggsave(
          filename = file_path,
          plot = obj,
          dpi = 72,
          height = 7,
          width = 14,
          bg = "transparent"
        )
      }
    )
    NULL  # no error
  }, error = function(e) {
    conditionMessage(e)
  })

  if (!is.null(result)) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = paste("Error preparing image:", result)
    ))
  }

  return(list(
    prompt = prompt,
    image_paths = file_path,
    error_msg = NULL
  ))
}

## Cleanup temporary files after Gemini API call
geminiCleanupFiles <- function(image_paths) {
  if (!is.null(image_paths[1])) {
    tryCatch({
      filename_html <- paste0(tools::file_path_sans_ext(image_paths), ".html")
      existing <- c(image_paths, filename_html)
      existing <- existing[file.exists(existing)]
      if (length(existing) > 0) file.remove(existing)
      dirs <- paste0(tools::file_path_sans_ext(image_paths), "_files")
      dirs <- dirs[dir.exists(dirs)]
      if (length(dirs) > 0) unlink(dirs, recursive = TRUE)
    }, error = function(e) {
      # Silently ignore cleanup errors
    })
  }
}

## Prepare all data for async Gemini call (synchronous orchestration)
geminiPrepareAll <- function(values, activeTab, input) {
  field <- geminiFieldName(activeTab)
  if (is.null(field)) {
    return(list(field = "MainInfoGemini", can_proceed = FALSE,
                error_msg = "Unsupported tab for Biblio AI."))
  }

  # Build description
  gemini_additional <- values$gemini_additional
  gemini_model_parameters <- values$gemini_model_parameters
  if (!is.null(gemini_additional) && gemini_additional != "") {
    desc <- paste0(
      values$collection_description,
      gemini_additional,
      gemini_model_parameters,
      collapse = ". "
    )
  } else {
    desc <- paste0(
      values$collection_description,
      gemini_model_parameters,
      collapse = ". "
    )
  }

  # Build prompt
  prompt <- biblioAiPrompts(values, activeTab)

  # Determine obj and type for each tab
  obj <- NULL
  type <- "text"

  switch(
    activeTab,
    "mainInfo" = {
      obj <- NULL
      type <- "text"
    },
    "lifeCycle" = {
      files <- DLC2Gemini(values$DLCplotYear, values$DLCplotCum)
      obj <- files
      type <- "multi"
    },
    "threeFieldPlot" = {
      obj <- values$TFP
      type <- "plotly"
    },
    "authorsProdOverTime" = {
      obj <- values$AUProdOverTime$graph
      type <- "ggplot2"
    },
    "correspAuthorCountry" = {
      obj <- values$MRCOplot
      type <- "ggplot2"
    },
    "mostLocalCitDoc" = {
      obj <- NULL
      type <- "text"
    },
    "trendTopic" = {
      obj <- values$trendTopics$graph
      type <- "ggplot2"
    },
    "ReferenceSpect" = {
      obj <- values$res$spectroscopy
      type <- "ggplot2"
    },
    "coOccurenceNetwork" = {
      obj <- values$COCnetwork$VIS
      type <- "vis"
    },
    "thematicMap" = {
      obj <- values$TMmap
      type <- "plotly"
    },
    "thematicEvolution" = {
      files <- TE2Gemini(values$nexus, values$TEplot)
      obj <- files
      type <- "multi"
    },
    "factorialAnalysis" = {
      obj <- values$plotCS
      type <- "plotly"
    },
    "coCitationNetwork" = {
      obj <- values$COCITnetwork$VIS
      type <- "vis"
    },
    "historiograph" = {
      obj <- values$histPlotVis$VIS
      type <- "vis"
    },
    "collabNetwork" = {
      obj <- values$COLnetwork$VIS
      type <- "vis"
    },
    "collabWorldMap" = {
      obj <- values$WMmap$g
      type <- "plotly"
    },
    "bradford" = {
      obj <- values$bradford$graph
      type <- "ggplot2"
    },
    "lotka" = {
      obj <- values$lotka$g
      type <- "ggplot2"
    }
  )

  # Prepare image (synchronous)
  prep <- geminiPrepareImage(
    obj = obj,
    type = type,
    prompt = prompt,
    key = values$geminiAPI,
    desc = desc,
    values = values
  )

  if (!is.null(prep$error_msg)) {
    return(list(
      field = field,
      can_proceed = FALSE,
      error_msg = prep$error_msg,
      prompt = NULL,
      image_paths = NULL
    ))
  }

  return(list(
    field = field,
    can_proceed = TRUE,
    error_msg = NULL,
    prompt = prep$prompt,
    image_paths = prep$image_paths
  ))
}

geminiSave <- function(values, activeTab) {
  switch(
    activeTab,
    "mainInfo" = {
      gemini <- values$MainInfoGemini
    },
    "lifeCycle" = {
      gemini <- values$DLCGemini
    },
    "threeFieldPlot" = {
      gemini <- values$TFPGemini
    },
    "authorsProdOverTime" = {
      gemini <- values$ApotGemini
    },
    "correspAuthorCountry" = {
      gemini <- values$MostRelCountriesGemini
    },
    "mostLocalCitDoc" = {
      gemini <- values$MostLocCitDocsGemini
    },
    "trendTopic" = {
      gemini <- values$trendTopicsGemini
    },
    "ReferenceSpect" = {
      gemini <- values$rpysGemini
    },
    "coOccurenceNetwork" = {
      gemini <- values$cocGemini
    },
    "thematicMap" = {
      gemini <- values$TMGemini
    },
    "thematicEvolution" = {
      gemini <- values$TEGemini
    },
    "factorialAnalysis" = {
      gemini <- values$CSGemini
    },
    "coCitationNetwork" = {
      gemini <- values$cocitGemini
    },
    "historiograph" = {
      gemini <- values$histGemini
    },
    "collabNetwork" = {
      gemini <- values$colGemini
    },
    "collabWorldMap" = {
      gemini <- values$WMGemini
    },
    "bradford" = {
      gemini <- values$BradfordGemini
    },
    "lotka" = {
      gemini <- values$LotkaGemini
    }
  )
  if (is.null(gemini)) {
    gemini <- "Click 'Ask Biblio AI' for help. "
  }
  return(gemini)
}

## Summarize document with Gemini AI ---
summaryAI <- function(values, i, model) {
  # Construct the prompt for Gemini AI
  prompt <- paste0(
    "Summarize the content of the following publication",
    " in no more than 250 words. ",
    "Title: ",
    values$M$TI[i],
    "Authors: ",
    values$M$AU[i],
    "Journal: ",
    values$M$SO[i],
    "Publication Year: ",
    values$M$PY[i],
    "Abstract: ",
    values$M$AB[i],
    "DOI: ",
    values$M$DI[i],
    "Try to access to article website using its DOI and extract more information about the article content. ",
    "If the article is open access, try to extract the full text and summarize it. ",
    "If the article is not open access, try to extract more information from the article website. ",
    "If you cannot access to the article website, just summarize the content using the title and the abstract. ",
    "Do not mention that you are an AI model. ",
    "Do not mention that you cannot access to the article website. ",
    "Do not mention that you do not have access to the full text. ",
    "Just provide the summary. "
  )

  # Gemini AI call
  res <- tryCatch(
    {
      gemini_ai(
        image = NULL,
        prompt = prompt,
        model = model,
        image_type = "png",
        retry_503 = 2,
        api_key = NULL
      )
    },
    error = function(e) {
      "Error generating automatic summary."
    }
  )

  return(res)
}


## Normalize abstract text
normalize_uppercase_text <- function(text) {
  # Check that input is a character string
  if (!is.character(text) || length(text) != 1) {
    stop("Input must be a character string")
  }

  # Convert everything to lowercase
  text_lower <- tolower(text)

  # Capitalize the first letter of the text
  text_lower <- paste0(
    toupper(substring(text_lower, 1, 1)),
    substring(text_lower, 2)
  )

  # Pattern to identify end of sentence followed by space and letter
  # Considers period, exclamation mark, question mark
  text_normalized <- gsub(
    "([.!?])\\s+([a-z])",
    "\\1 \\U\\2",
    text_lower,
    perl = TRUE
  )

  # Handle common abbreviations that should not be followed by uppercase
  # Examples: Dr., Prof., etc., vs., i.e., e.g.
  common_abbrev <- c(
    "dr\\.",
    "prof\\.",
    "etc\\.",
    "vs\\.",
    "i\\.e\\.",
    "e\\.g\\.",
    "mr\\.",
    "mrs\\.",
    "ms\\.",
    "jr\\.",
    "sr\\.",
    "phd\\."
  )

  for (abbrev in common_abbrev) {
    # Convert to lowercase the letters after common abbreviations
    pattern <- paste0("(", abbrev, ")\\s+([A-Z])")
    text_normalized <- gsub(
      pattern,
      "\\1 \\L\\2",
      text_normalized,
      perl = TRUE,
      ignore.case = TRUE
    )
  }

  # Capitalize after colons (optional - common in academic titles)
  text_normalized <- gsub(
    "(:)\\s+([a-z])",
    "\\1 \\U\\2",
    text_normalized,
    perl = TRUE
  )

  return(text_normalized)
}

## To title upper case metadata fields
to_title_case <- function(texts, exclude_articles = TRUE) {
  # --- Input Validation ---
  # Check that the input is a character vector. If not, stop and show an error.
  if (!is.character(texts)) {
    stop("Input must be a character vector.")
  }

  # --- Configuration ---
  # Define a list of "small words" (articles, prepositions, conjunctions) that
  # should remain in lowercase unless they are the first or last word in the title.
  small_words <- c(
    "a",
    "an",
    "and",
    "as",
    "at",
    "but",
    "by",
    "for",
    "if",
    "in",
    "of",
    "on",
    "or",
    "the",
    "to",
    "up",
    "via",
    "with",
    "from",
    "into",
    "over",
    "upon",
    "down",
    "off",
    "out",
    "per",
    "than",
    "through",
    "under",
    "within",
    "without",
    "about",
    "across",
    "after",
    "against",
    "along",
    "among",
    "around",
    "before",
    "behind",
    "below",
    "beneath",
    "beside",
    "between",
    "beyond",
    "during",
    "except",
    "inside",
    "near",
    "since",
    "toward",
    "towards",
    "until",
    "where",
    "while"
  )

  # --- Helper Function ---
  # A helper function to capitalize the first letter of a single word.
  # It also handles hyphenated words (e.g., "state-of-the-art" -> "State-of-the-Art").
  capitalize_first <- function(word) {
    # Check for hyphens
    if (grepl("-", word)) {
      # If a hyphen exists, split the word into parts
      parts <- strsplit(word, "-")[[1]]
      # Capitalize the first letter of each part
      capitalized_parts <- sapply(parts, function(part) {
        if (nchar(part) > 0) {
          paste0(toupper(substring(part, 1, 1)), substring(part, 2))
        } else {
          part # Return empty parts as they are
        }
      })
      # Re-join the parts with a hyphen
      return(paste(capitalized_parts, collapse = "-"))
    } else {
      # If no hyphen, just capitalize the first letter of the word
      paste0(toupper(substring(word, 1, 1)), substring(word, 2))
    }
  }

  # --- Main Processing Logic ---
  # Use sapply() to apply the title-casing logic to each element of the input vector 'texts'.
  # 'sapply' iterates over the vector and collects the results into a new vector.
  # USE.NAMES = FALSE ensures the output is an unnamed vector.
  result_vector <- sapply(
    texts,
    function(single_text) {
      # Handle edge cases: return NA or empty strings without modification.
      if (is.na(single_text) || nchar(single_text) == 0) {
        return(single_text)
      }

      # Convert the entire string to lowercase to ensure consistency.
      text_lower <- tolower(single_text)

      # Split the string into a vector of words based on whitespace.
      words <- strsplit(text_lower, "\\s+")[[1]]

      # Apply capitalization based on the 'exclude_articles' flag.
      if (exclude_articles) {
        # If TRUE, apply capitalization selectively.
        capitalized_words <- sapply(seq_along(words), function(i) {
          word <- words[i]
          # Always capitalize the first word, the last word, or any word NOT in the 'small_words' list.
          if (i == 1 || i == length(words) || !word %in% small_words) {
            capitalize_first(word)
          } else {
            word # Otherwise, keep the small word in lowercase.
          }
        })
      } else {
        # If FALSE, capitalize every word.
        capitalized_words <- sapply(words, capitalize_first)
      }

      # Recombine the processed words into a single string, separated by spaces.
      paste(capitalized_words, collapse = " ")
    },
    USE.NAMES = FALSE
  )

  # Return the final vector of title-cased strings.
  return(result_vector)
}


merge_df_to_string <- function(df) {
  # Check if the input has at least two columns
  if (ncol(df) < 2) {
    stop("The data frame must have at least two columns.")
  }

  # Ensure the input is a data frame
  df <- as.data.frame(df)

  # Convert each row into a "param: value" format
  row_strings <- apply(df[, 1:2], 1, function(row) {
    paste0(row[1], ": ", row[2])
  })

  # Concatenate all row strings using "; " as separator
  final_string <- paste(row_strings, collapse = "; ")

  return(final_string)
}

plot2pngGemini <- function(p, filename, zoom = 2, type = "vis") {
  #html_name <- tempfile(fileext = ".html")
  # remove file extensions from filename
  filename_html <- tools::file_path_sans_ext(filename)
  temp_html <- paste0(filename_html, ".html")
  switch(
    type,
    vis = {
      visSave(p, temp_html)
    },
    plotly = {
      htmlwidgets::saveWidget(p, file = temp_html, selfcontained = TRUE)
    }
  )

  biblioShot(temp_html, zoom = zoom, file = filename, delay = 2.5)
}

getWD <- function() {
  home_path <- Sys.getenv("HOME")
  temp_path <- file.path(home_path, "temp")

  if (!dir.exists(temp_path)) {
    dir.create(temp_path, recursive = TRUE)
  }

  return(temp_path)
}

## save all plots of the life cycle analysis
DLC2Gemini <- function(DLCplotYear, DLCplotCum) {
  plots <- c("annual", "cumulative")
  files <- unlist(lapply(plots, function(x) {
    paste0(getWD(), "/plotDLC_", x, ".png")
  }))

  plot2pngGemini(DLCplotYear, filename = files[1], type = "plotly")
  plot2pngGemini(DLCplotCum, filename = files[2], type = "plotly")

  return(files)
}

## save all plots of the thematic evolution analysis
TE2Gemini <- function(nexus, plotTE) {
  K <- length(nexus$TM)
  periods <- nexus$Nodes %>% select(group) %>% distinct() %>% pull()
  files <- unlist(lapply(periods, function(x) {
    file.path(getWD(), paste0(x, "_period.png"))
  }))
  files <- c(file.path(getWD(), "Evolution_Plot.png"), files)

  plot2pngGemini(plotTE, filename = files[1], type = "plotly")

  for (k in 2:(K + 1)) {
    suppressMessages(ggsave(
      filename = files[k],
      plot = nexus$TM[[k - 1]]$map +
        ggplot2::labs(
          title = paste0("Period: ", periods[k - 1]),
          dpi = 72,
          height = 7,
          width = 14,
          bg = "transparent"
        )
    ))
  }
  return(files)
}

copy_to_clipboard <- function(x) {
  # Check the operating system
  sys_info <- Sys.info()
  os_type <- tolower(sys_info["sysname"])

  # Convert the object to a string if it is not already
  if (!is.character(x)) {
    x <- capture.output(print(x))
  }

  # Copy to clipboard based on the operating system
  if (os_type == "windows") {
    writeClipboard(x)
  } else if (os_type == "darwin") {
    # macOS
    con <- pipe("pbcopy", "w")
    writeLines(x, con)
    close(con)
  } else if (os_type == "linux") {
    # Use xclip or xsel, if available
    if (nzchar(Sys.which("xclip"))) {
      con <- pipe("xclip -selection clipboard", "w")
      writeLines(x, con)
      close(con)
    } else if (nzchar(Sys.which("xsel"))) {
      con <- pipe("xsel --clipboard --input", "w")
      writeLines(x, con)
      close(con)
    } else {
      stop(
        "Neither 'xclip' nor 'xsel' are available. Please install one of them to use the clipboard on Linux."
      )
    }
  } else {
    stop("Unrecognized or unsupported operating system.")
  }
}

## New function to convert gemini output as HTML blocks
gemini_to_html <- function(text, font_size = "16px") {
  # Remove original leading/trailing whitespace
  text <- trimws(text)

  # Divide text into lines
  lines <- unlist(strsplit(text, "\n", fixed = TRUE))

  # Initialize HTML output with CSS styles including font size
  html_lines <- c(
    paste0(
      "<div style='font-family: Arial, sans-serif; line-height: 1.4; margin: 0 auto; padding: 15px; font-size: ",
      font_size,
      ";'>"
    )
  )

  in_list <- FALSE
  list_type <- ""
  last_ol_number <- 0

  close_list <- function() {
    if (in_list) {
      if (list_type == "ul") {
        html_lines <<- c(html_lines, "</ul>")
      } else {
        html_lines <<- c(html_lines, "</ol>")
      }
      in_list <<- FALSE
    }
  }

  for (i in seq_along(lines)) {
    line <- trimws(lines[i])

    # Skip empty lines
    if (line == "") next

    # Skip horizontal rules (--- or ***)
    if (grepl("^[-*_]{3,}\\s*$", line)) next

    # Markdown header management (lines starting with #, ##, ###, ####)
    if (stringr::str_detect(line, "^#{1,4}\\s+")) {
      close_list()

      header_level <- nchar(stringr::str_extract(line, "^#+"))
      header_text <- stringr::str_replace(line, "^#+\\s+", "")
      header_text <- stringr::str_replace(header_text, "\\s*#+\\s*$", "")
      header_text <- stringr::str_replace_all(header_text, "\\*\\*([^*]+)\\*\\*", "\\1")

      h_tag <- paste0("h", min(header_level + 1, 5))
      h_size <- switch(as.character(header_level),
        "1" = "font-size: 1.3em;",
        "2" = "font-size: 1.15em;",
        "3" = "font-size: 1.05em;",
        "font-size: 1em;"
      )
      html_lines <- c(
        html_lines,
        paste0(
          "<", h_tag, " style='color: #333; border-bottom: 1px solid #007acc; padding-bottom: 4px; margin-bottom: 6px; margin-top: 14px; ", h_size, "'>",
          header_text,
          "</", h_tag, ">"
        )
      )
      next
    }

    # Title management (lines enclosed in ** only)
    if (stringr::str_detect(line, "^\\*\\*[^*]+\\*\\*$")) {
      close_list()

      title_text <- stringr::str_replace_all(line, "^\\*\\*(.+)\\*\\*$", "\\1")
      title_text <- stringr::str_replace(title_text, "^\\d+\\.\\s*", "")
      html_lines <- c(
        html_lines,
        paste0(
          "<h3 style='color: #333; border-bottom: 1px solid #007acc; padding-bottom: 4px; margin-bottom: 6px; margin-top: 14px;'>",
          title_text,
          "</h3>"
        )
      )
      next
    }

    # Managing bulleted lists (starting with * or -)
    if (stringr::str_detect(line, "^\\s*[*\\-]\\s+")) {
      if (!in_list || list_type != "ul") {
        close_list()
        html_lines <- c(
          html_lines,
          "<ul style='margin-bottom: 6px; margin-top: 4px;'>"
        )
        in_list <- TRUE
        list_type <- "ul"
      }

      item_text <- stringr::str_replace(line, "^\\s*[*\\-]\\s+", "")
      item_text <- format_inline_text(item_text)

      html_lines <- c(
        html_lines,
        paste0("<li style='margin-bottom: 2px;'>", item_text, "</li>")
      )
      next
    }

    # Managing numbered lists (starting with a number followed by a period)
    if (stringr::str_detect(line, "^\\s*\\d+\\.\\s+")) {
      item_number <- as.integer(stringr::str_extract(line, "^\\s*\\d+"))

      if (!in_list || list_type != "ol") {
        close_list()
        start_attr <- if (item_number > 1) paste0(" start='", item_number, "'") else ""
        html_lines <- c(
          html_lines,
          paste0("<ol style='margin-bottom: 6px; margin-top: 4px;'", start_attr, ">")
        )
        in_list <- TRUE
        list_type <- "ol"
      }

      last_ol_number <- item_number
      item_text <- stringr::str_replace(line, "^\\s*\\d+\\.\\s+", "")
      item_text <- format_inline_text(item_text)
      html_lines <- c(
        html_lines,
        paste0("<li style='margin-bottom: 2px;'>", item_text, "</li>")
      )
      next
    }

    # Non-list, non-header line: close any open list
    close_list()

    # Normal paragraph management
    formatted_line <- format_inline_text(line)
    html_lines <- c(
      html_lines,
      paste0("<p style='margin-bottom: 5px; margin-top: 2px;'>", formatted_line, "</p>")
    )
  }

  # Close any lists that remain open
  close_list()

  # Close the container div
  html_lines <- c(html_lines, "</div>")

  # Merge all lines
  html_result <- paste(html_lines, collapse = "\n")

  return(html_result)
}

# Auxiliary function for formatting inline text
format_inline_text <- function(text) {
  # Bold management (**text**)
  text <- stringr::str_replace_all(
    text,
    "\\*\\*([^*]+)\\*\\*",
    "<strong>\\1</strong>"
  )

  # Italic management (*text*) - only if it is not already in bold
  text <- stringr::str_replace_all(
    text,
    "(?<!\\*)\\*([^*]+)\\*(?!\\*)",
    "<em>\\1</em>"
  )

  # Handling text in quotation marks as inline code ("text")
  text <- stringr::str_replace_all(
    text,
    '"([^"]+)"',
    '<code style="background-color: #f4f4f4; padding: 2px 4px; border-radius: 3px; font-family: \'Courier New\', monospace;">\\1</code>'
  )

  # Handling parentheses with percentages or numerical values
  text <- stringr::str_replace_all(
    text,
    "\\(([^)]*%[^)]*)\\)",
    "<span style='color: #007acc; font-weight: bold;'>(\\1)</span>"
  )

  return(text)
}


# Thematic Map top 3 documents for each cluster
doc2clust <- function(res, n = 3) {
  df <- res$doc2clust %>%
    drop_na(Assigned_cluster) %>%
    group_by(Assigned_cluster) %>%
    slice_max(pagerank, n = n) %>%
    mutate(cluster = paste(SR, ", pagerank ", round(pagerank, 3), sep = "")) %>%
    select(Assigned_cluster, cluster)
}

# APOT top 3 Documents per author
apot2Docs <- function(res, n = 3) {
  df <- res$dfPapersAU %>%
    group_by(Author) %>%
    slice_max(TCpY, n = n) %>%
    mutate(
      SR = paste(TI, SO, year, paste0("TCpY ", round(TCpY, 1)), sep = ", ")
    ) %>%
    select(Author, SR)
}

# Country collaboration table
country2collab <- function(df, n = 20) {
  df <- df %>%
    slice_max(Articles, n = n) %>%
    mutate(
      SR = paste(
        paste0("Article ", Articles, sep = ", "),
        paste0("SCP ", SCP, sep = ", "),
        paste0("MCP ", MCP, sep = ", "),
        paste0("MCP % ", round(`MCP %`, 1), sep = ", ")
      )
    ) %>%
    select(Country, SR)
}

#Local citation table
localCit2docs <- function(df, n = 20) {
  df <- df %>%
    slice_max(Local.Citations, n = n) %>%
    mutate(
      SR = paste(
        paste0("LC ", Local.Citations, sep = ", "),
        paste0("GC ", Global.Citations, sep = ", "),
        paste0("NLC ", round(Normalized.Local.Citations, 2), sep = ", "),
        paste0("NGC ", round(Normalized.Global.Citations, 2)),
        sep = ""
      )
    ) %>%
    select(Document, SR)
}

# Historiograph doc titles
hist2docs <- function(df, n = 20) {
  df %>%
    slice_max(order_by = LCS, n = n) %>%
    select(short_label, title_orig)
}

# Top 10 peaks in RPYS
rpysPeaks <- function(res, n = 10) {
  df_peaks <- res$rpysTable %>%
    arrange(Year) %>%
    mutate(
      is_peak = (diffMedian > lag(diffMedian)) & (diffMedian > lead(diffMedian))
    ) %>%
    dplyr::filter(is_peak) %>%
    arrange(desc(diffMedian)) %>%
    slice_head(n = n)

  df2 <- res$CR %>%
    group_by(Year) %>%
    slice_max(Freq, n = 3, with_ties = FALSE) %>%
    dplyr::filter(Year %in% df_peaks$Year)

  return(peaks = df2)
}

## Wrapper function to render bibliobox in Shiny
renderBibliobox <- function(df, title = "", ...) {
  tryCatch(
    {
      data <- if (is.reactive(df)) df() else df
      HTML(htmlBoxFormat(data, title = title, ...))
    },
    error = function(e) {
      tags$div(
        class = "alert alert-danger",
        paste("Render error:", e$message)
      )
    }
  )
}

#' HTML Box Format with Filter-Aware Excel Export
htmlBoxFormat <- function(
  df,
  nrow = 10,
  filename = "Table",
  pagelength = TRUE,
  left = NULL,
  right = NULL,
  numeric = NULL,
  dom = TRUE,
  size = "85%",
  filter = "top",
  columnShort = NULL,
  columnSmall = NULL,
  round = 2,
  title = "",
  button = FALSE,
  escape = FALSE,
  selection = FALSE,
  scrollX = FALSE,
  scrollY = FALSE,
  summary = FALSE
) {
  if (is.null(df) || (is.data.frame(df) && nrow(df) == 0)) {
    return(as.character(tags$div(
      class = "alert alert-info",
      "No data available"
    )))
  }

  # CORREZIONE: Gestisci pagelength prima di usare nrow nello sprintf
  if (is.logical(pagelength) && !pagelength) {
    nrow <- nrow(df) # Mostra tutte le righe
  } else if (is.numeric(pagelength)) {
    nrow <- as.integer(pagelength)
  }
  nrow <- as.integer(nrow) # Assicura che sia sempre un intero

  table_id <- sprintf(
    "htmlbox_%s",
    paste(sample(c(0:9, letters), 8, replace = TRUE), collapse = "")
  )
  col_names <- colnames(df)

  get_align <- function(i) {
    if (i %in% left) {
      return("text-align: left;")
    }
    if (i %in% right) {
      return("text-align: right;")
    }
    if (i %in% numeric) {
      return("text-align: right;")
    }
    return("text-align: left;")
  }

  # Determine if summary button column is needed
  has_summary <- is.character(summary) && summary == "documents"

  # Header
  header_cols <- paste0(
    sprintf(
      '<th class="sortable-header" data-col="%d" style="%s background-color: #f8f9fa; border-bottom: 2px solid #dee2e6; padding: 12px 25px 12px 10px; position: relative; white-space: nowrap;">%s <span class="sort-icon" style="color: #ccc; font-size: 0.9em; position: absolute; right: 8px; top: 50%%; transform: translateY(-50%%);">⇅</span></th>',
      0:(ncol(df) - 1),
      sapply(1:ncol(df), get_align),
      col_names
    ),
    collapse = ""
  )
  if (has_summary) {
    header_cols <- paste0(
      header_cols,
      '<th style="background-color: #f8f9fa; border-bottom: 2px solid #dee2e6; padding: 12px 10px; text-align: center; white-space: nowrap;">AI Summary</th>'
    )
  }
  header_html <- paste0("<thead><tr style='cursor: pointer;'>", header_cols, "</tr>")

  if (filter == "top") {
    filter_cells <- paste0(
      sprintf(
        '<td><input type="text" class="form-control table-filter-%s" data-col="%d" placeholder="Filter..." style="width:100%%; font-size:11px; height:24px; padding:2px 5px;"></td>',
        table_id,
        0:(ncol(df) - 1)
      ),
      collapse = ""
    )
    if (has_summary) {
      filter_cells <- paste0(filter_cells, "<td></td>")
    }
    filter_html <- paste0('<tr class="filter-row">', filter_cells, "</tr>")
    header_html <- paste0(header_html, filter_html)
  }
  header_html <- paste0(header_html, "</thead>")

  # Body: prepare data as JSON for client-side rendering (only visible page in DOM)
  n_cols <- ncol(df)
  aligns <- sapply(1:n_cols, get_align)

  # Build column-oriented vectors (fast R serialization, reconstruct rows in JS)
  sort_cols <- vector("list", n_cols)
  disp_cols <- vector("list", n_cols)
  for (j in 1:n_cols) {
    vals <- as.character(df[[j]])
    sort_vals <- gsub("<.*?>", "", vals)
    if (j %in% numeric) {
      num_vals <- as.numeric(sort_vals)
      disp_cols[[j]] <- format(round(num_vals, round), nsmall = round)
      sort_cols[[j]] <- as.character(num_vals)
    } else {
      disp_cols[[j]] <- vals
      sort_cols[[j]] <- sort_vals
    }
  }

  # Column-oriented JSON: {s: [[col1_vals], [col2_vals], ...], d: [[col1_vals], [col2_vals], ...], n: nrow}
  # jsonlite serializes character vectors very efficiently in C — no nested R list overhead
  data_json <- jsonlite::toJSON(
    list(s = sort_cols, d = disp_cols, n = nrow(df)),
    auto_unbox = TRUE
  )

  # UI Structure — empty tbody, data delivered via JSON script tag
  aligns_json <- jsonlite::toJSON(aligns, auto_unbox = TRUE)

  html_out <- tagList(
    tags$script(
      src = "https://cdn.jsdelivr.net/npm/xlsx@0.18.5/dist/xlsx.full.min.js"
    ),

    tags$div(
      id = paste0("container_", table_id),
      style = sprintf(
        "font-size: %s; background: white; border: 1px solid #ddd; border-radius: 4px; margin-bottom: 20px; font-family: sans-serif;",
        size
      ),

      tags$div(
        style = "padding: 10px 15px; background: #fdfdfd; border-bottom: 1px solid #eee; display: flex; justify-content: space-between; align-items: center;",
        tags$strong(title, style = "font-size: 1.1em; color: #333;"),
        if (button) {
          tags$button(
            "Excel",
            class = "btn btn-xs btn-success",
            onclick = sprintf("exportToExcel('%s', '%s')", table_id, filename)
          )
        } else {
          ""
        }
      ),

      tags$div(
        style = if (scrollX) "overflow-x: auto;" else "",
        tags$table(
          id = table_id,
          class = "table table-hover",
          style = "width: 100%; margin-bottom: 0; table-layout: auto;",
          HTML(header_html),
          HTML("<tbody></tbody>")
        )
      ),

      tags$div(
        style = "padding: 10px; border-top: 1px solid #eee; display: flex; justify-content: space-between; align-items: center;",
        tags$div(
          id = paste0("info_", table_id),
          style = "color: #777; font-size: 12px;"
        ),
        tags$div(class = "btn-group", id = paste0("pager_", table_id))
      )
    ),

    # Embed data as JSON in a script tag (not executed, just storage)
    tags$script(
      id = paste0("data_", table_id),
      type = "application/json",
      HTML(as.character(data_json))
    ),

    tags$script(HTML(sprintf(
      '
      if (typeof window.exportToExcel !== "function") {
        window.exportToExcel = function(tid, fname) {
          var state = window["__bb_" + tid];
          if (!state) return;

          var headers = [];
          var table = document.getElementById(tid);
          var headerCells = table.querySelectorAll("thead tr:first-child th");
          headerCells.forEach(function(th) { headers.push(th.innerText.replace("\\u21C5", "").trim()); });

          var data = [headers];
          var rows = state.filteredData;
          for (var i = 0; i < rows.length; i++) {
            var rowData = [];
            for (var j = 0; j < rows[i].length; j++) {
              var val = rows[i][j].s;
              if (!isNaN(val) && val !== "") val = parseFloat(val);
              rowData.push(val);
            }
            data.push(rowData);
          }

          var ws = XLSX.utils.aoa_to_sheet(data);
          var wb = XLSX.utils.book_new();
          XLSX.utils.book_append_sheet(wb, ws, "Data");
          XLSX.writeFile(wb, fname + "_" + new Date().toISOString().split("T")[0] + ".xlsx");
        };
      }

      (function() {
        var tid = "%s";
        var pageSize = %d;
        var hasSummary = %s;
        var aligns = %s;

        function init() {
          var table = document.getElementById(tid);
          if (!table) { setTimeout(init, 100); return; }

          var dataEl = document.getElementById("data_" + tid);
          var raw = JSON.parse(dataEl.textContent);
          // Transform column-oriented {s:[[col1],[col2],...], d:[[col1],[col2],...], n:nrow}
          // into row-oriented [{s:sortVal, d:dispVal}, ...] per row — fast in JS
          var nCols = raw.s.length;
          var allData = new Array(raw.n);
          for (var r = 0; r < raw.n; r++) {
            var row = new Array(nCols);
            for (var c = 0; c < nCols; c++) {
              row[c] = {s: raw.s[c][r], d: raw.d[c][r]};
            }
            allData[r] = row;
          }
          raw = null; // free column-oriented data
          var filteredData = allData.slice();
          var currentPage = 0;
          var sortCol = -1;
          var sortAsc = true;
          var tbody = table.querySelector("tbody");

          // Expose state for Excel export
          window["__bb_" + tid] = { filteredData: filteredData };

          function renderPage() {
            var start = currentPage * pageSize;
            var end = Math.min(start + pageSize, filteredData.length);
            tbody.innerHTML = "";
            for (var i = start; i < end; i++) {
              var row = filteredData[i];
              var tr = document.createElement("tr");
              for (var j = 0; j < row.length; j++) {
                var td = document.createElement("td");
                td.setAttribute("data-sort", row[j].s);
                td.style.cssText = aligns[j] + " padding: 8px; border-bottom: 1px solid #eee;";
                td.innerHTML = row[j].d;
                tr.appendChild(td);
              }
              if (hasSummary) {
                var tdS = document.createElement("td");
                tdS.style.cssText = "text-align: center; padding: 8px; border-bottom: 1px solid #eee;";
                var safeVal = row[0].s.replace(/\x27/g, "\\\x27");
                tdS.innerHTML = "<button class=\\"btn btn-xs btn-info\\" onclick=\\"Shiny.setInputValue(\\x27button_id\\x27, \\x27" + safeVal + "\\x27, {priority: \\x27event\\x27})\\" title=\\"AI Summary\\"><i class=\\"fa fa-robot\\"></i></button>";
                tr.appendChild(tdS);
              }
              tbody.appendChild(tr);
            }
            document.getElementById("info_" + tid).innerText =
              "Showing " + (filteredData.length > 0 ? start + 1 : 0) +
              "-" + end + " of " + filteredData.length;
            renderPager();
          }

          function renderPager() {
            var pager = document.getElementById("pager_" + tid);
            var pageCount = Math.ceil(filteredData.length / pageSize);
            pager.innerHTML = "";
            if (pageCount <= 1) return;
            var createBtn = function(label, target, active, disabled) {
              var b = document.createElement("button");
              b.className = "btn btn-default btn-xs" + (active ? " active" : "");
              b.innerText = label;
              b.disabled = disabled;
              b.onclick = function() { currentPage = target; renderPage(); };
              pager.appendChild(b);
            };
            createBtn("\\u00AB", Math.max(0, currentPage - 1), false, currentPage === 0);
            for (var i = 0; i < pageCount; i++) {
              if (i === 0 || i === pageCount - 1 || (i >= currentPage - 1 && i <= currentPage + 1)) {
                createBtn(i + 1, i, i === currentPage, false);
              } else if (i === currentPage - 2 || i === currentPage + 2) {
                var s = document.createElement("span");
                s.innerText = "...";
                s.style.padding = "0 5px";
                pager.appendChild(s);
              }
            }
            createBtn("\\u00BB", Math.min(pageCount - 1, currentPage + 1), false, currentPage === pageCount - 1);
          }

          table.querySelectorAll(".sortable-header").forEach(function(th) {
            th.onclick = function() {
              var col = parseInt(this.getAttribute("data-col"));
              sortAsc = (sortCol === col) ? !sortAsc : true;
              sortCol = col;
              table.querySelectorAll(".sort-icon").forEach(function(si) { si.innerText = "\\u21C5"; si.style.color = "#ccc"; });
              this.querySelector(".sort-icon").innerText = sortAsc ? "\\u25B2" : "\\u25BC";
              this.querySelector(".sort-icon").style.color = "#333";
              filteredData.sort(function(a, b) {
                var valA = a[col].s.trim();
                var valB = b[col].s.trim();
                if (!isNaN(valA) && !isNaN(valB) && valA !== "" && valB !== "") {
                  return sortAsc ? parseFloat(valA) - parseFloat(valB) : parseFloat(valB) - parseFloat(valA);
                }
                return sortAsc ? valA.localeCompare(valB) : valB.localeCompare(valA);
              });
              currentPage = 0;
              renderPage();
            };
          });

          var filters = document.querySelectorAll(".table-filter-" + tid);
          filters.forEach(function(input) {
            input.addEventListener("input", function() {
              var fVals = Array.from(filters).map(function(f) { return f.value.toUpperCase(); });
              filteredData = allData.filter(function(row) {
                return fVals.every(function(val, idx) {
                  if (!val) return true;
                  return row[idx].s.toUpperCase().indexOf(val) > -1;
                });
              });
              window["__bb_" + tid].filteredData = filteredData;
              currentPage = 0;
              renderPage();
            });
          });

          renderPage();
        }
        init();
      })();
      ',
      table_id,
      nrow,
      ifelse(has_summary, "true", "false"),
      as.character(aligns_json)
    )))
  )

  return(as.character(html_out))
}

# ### packages for biblishiny()
libraries <- function() {
  all_ok <- TRUE

  parse_pkg <- function(pkg_str) {
    # Estrae nome e versione minima, se specificata
    matches <- regmatches(
      pkg_str,
      regexec("^([a-zA-Z0-9\\.]+)(?: \\(>= ([0-9\\.]+)\\))?$", pkg_str)
    )[[1]]
    if (length(matches) >= 2) {
      list(
        name = matches[2],
        min_version = ifelse(length(matches) == 3, matches[3], NA)
      )
    } else {
      list(name = pkg_str, min_version = NA)
    }
  }

  safe_install <- function(pkg_str) {
    pkg_info <- parse_pkg(pkg_str)
    pkg <- pkg_info$name
    min_ver <- pkg_info$min_version

    need_install <- FALSE

    if (pkg %in% rownames(installed.packages())) {
      if (!is.na(min_ver)) {
        installed_ver <- as.character(packageVersion(pkg))
        if (compareVersion(installed_ver, min_ver) < 0) {
          message(sprintf(
            "The installed version of '%s' (%s) is lower than the required (%s).",
            pkg,
            installed_ver,
            min_ver
          ))
          need_install <- TRUE
        }
      }
    } else {
      need_install <- TRUE
    }

    if (need_install) {
      install.packages(pkg)
    }

    return(require(pkg, character.only = TRUE, quietly = TRUE))
  }

  pkgs <- c(
    "httr2",
    "base64enc",
    "bibliometrix",
    "zip",
    "shiny",
    "igraph",
    "stringr",
    "contentanalysis",
    "ggplot2",
    "wordcloud2",
    "ggmap",
    "maps",
    "pdftools (>= 3.6.0)",
    "tidytext",
    "visNetwork",
    "plotly",
    "fontawesome",
    "shinydashboardPlus",
    "shinydashboard",
    "shinyjs",
    "curl (>= 6.3.0)",
    "RCurl",
    "openxlsx",
    "shinyWidgets",
    "chromote",
    "pagedown",
    "Matrix",
    "dimensionsR",
    "pubmedR",
    "dplyr",
    "tidyr",
    "sparkline",
    "tidygraph",
    "ggraph",
    "future",
    "promises"
  )

  suppressPackageStartupMessages({
    results <- vapply(pkgs, safe_install, logical(1))
    all_ok <- all(results)
  })

  return(all_ok)
}

messageItem2 <- function(
  from,
  message,
  icon = shiny::icon("user"),
  time = NULL,
  href = NULL,
  inputId = NULL
) {
  if (is.null(href)) {
    href <- "#"
  }
  shiny::tags$li(shiny::a(
    id = inputId,
    class = if (!is.null(inputId)) {
      "action-button"
    },
    href = href,
    target = "_blank",
    icon,
    shiny::h4(
      from,
      if (!is.null(time)) {
        shiny::tags$small(shiny::icon("clock-o"), time)
      }
    ),
    shiny::p(message)
  ))
}

## SERVER components for contentanalysis in Biblioshiny ----

#' Enhanced Server Logic for Content Analysis Tab with 3 Tabs
#'
#' @param input Shiny input object
#' @param output Shiny output object
#' @param session Shiny session object
#' @param values Reactive values object

#' Enhanced Server Logic for Content Analysis Tab with 3 Tabs
#'
#' @param input Shiny input object
#' @param output Shiny output object
#' @param session Shiny session object
#' @param values Reactive values object

content_analysis_server <- function(input, output, session, values) {
  # Helper function for null coalescing
  `%||%` <- function(lhs, rhs) {
    if (!is.null(lhs) && length(lhs) > 0) lhs else rhs
  }

  # Enhance citation network layout for better readability
  # Post-processes the visNetwork from create_citation_network() with:
  # - max_nodes_per_section filtering
  # - community-aware Fruchterman-Reingold layout
  # - degree-proportional font sizing
  # - label overlap removal
  enhance_citation_network <- function(network, spacing = 5,
                                       max_nodes_per_section = NULL,
                                       node_size = 5,
                                       label_size = 5,
                                       repulsion = 5,
                                       edge_max_distance = NULL) {
    if (is.null(network)) return(NULL)

    # Extract nodes and edges from visNetwork object
    nodes <- network$x$nodes
    edges <- network$x$edges
    stats_attr <- attr(network, "stats")

    if (is.null(nodes) || nrow(nodes) == 0) return(network)

    # --- Filter edges by max distance ---
    if (!is.null(edge_max_distance) && "distance" %in% names(edges)) {
      edges <- edges[edges$distance <= edge_max_distance, ]
      if (nrow(edges) == 0) return(NULL)
    }

    # --- Increase edge transparency ---
    if ("color" %in% names(edges)) {
      edges$color <- gsub(
        "rgba\\((\\d+),\\s*(\\d+),\\s*(\\d+),\\s*[0-9.]+\\)",
        "rgba(\\1, \\2, \\3, 0.12)",
        edges$color
      )
    }

    # --- Max nodes per section filtering ---
    if (!is.null(max_nodes_per_section) && max_nodes_per_section > 0 &&
        "primary_section" %in% names(nodes)) {
      nodes <- nodes %>%
        dplyr::group_by(primary_section) %>%
        dplyr::arrange(dplyr::desc(connections), .by_group = TRUE) %>%
        dplyr::slice_head(n = max_nodes_per_section) %>%
        dplyr::ungroup() %>%
        as.data.frame(stringsAsFactors = FALSE)

      # Filter edges to only valid nodes
      valid_ids <- nodes$id
      edges <- edges[edges$from %in% valid_ids & edges$to %in% valid_ids, ]

      if (nrow(edges) == 0) return(NULL)

      # Recalculate connections after filtering
      edge_counts <- c(edges$from, edges$to)
      conn_table <- as.data.frame(table(edge_counts), stringsAsFactors = FALSE)
      names(conn_table) <- c("id", "connections")
      conn_table$id <- as.integer(conn_table$id)
      conn_table$connections <- as.integer(conn_table$connections)
      nodes$connections <- conn_table$connections[match(nodes$id, conn_table$id)]
      nodes$connections[is.na(nodes$connections)] <- 0
      nodes <- nodes[nodes$connections > 0, ]
      valid_ids <- nodes$id
      edges <- edges[edges$from %in% valid_ids & edges$to %in% valid_ids, ]
    }

    if (nrow(nodes) == 0 || nrow(edges) == 0) return(NULL)

    # --- Node size scaling (sqrt for compressed range) ---
    node_size <- max(1, min(10, node_size))
    size_mult <- node_size / 5

    scalemin <- round(10 * size_mult + 5)
    scalemax <- round(35 * size_mult + 15)
    conn_min <- min(nodes$connections)
    conn_max <- max(nodes$connections)

    if (conn_max > conn_min) {
      # sqrt scale: compresses high values, expands low values
      norm_conn <- (nodes$connections - conn_min) / (conn_max - conn_min)
      font_size <- sqrt(norm_conn) * (scalemax - scalemin) + scalemin
    } else {
      font_size <- rep((scalemin + scalemax) / 2, nrow(nodes))
    }
    font_size <- pmax(scalemin, pmin(scalemax, font_size))

    label_size <- max(1, min(10, label_size))
    label_mult <- label_size / 5
    nodes$font.size <- font_size * 0.6 * label_mult
    nodes$size <- font_size * 0.5 * size_mult
    nodes$font.vadjust <- -0.7 * nodes$font.size

    # Font opacity proportional to size (sqrt already applied above)
    opacity_range <- (font_size - min(font_size)) / max(1, diff(range(font_size)))
    font_opacity <- opacity_range * 0.6 + 0.4
    nodes$font.color <- sapply(font_opacity, function(a) {
      grDevices::adjustcolor("black", alpha.f = a)
    })

    # --- Build igraph and apply community repulsion to edge weights ---
    ig <- igraph::graph_from_data_frame(
      d = edges[, c("from", "to")],
      directed = FALSE,
      vertices = data.frame(name = nodes$id)
    )

    spacing <- max(1, min(10, spacing))
    spacing_mult <- spacing / 5
    repulsion <- max(1, min(10, repulsion))

    # Apply community repulsion via edge weight manipulation (same as biblioshiny co-occurrence)
    if ("primary_section" %in% names(nodes)) {
      node_sections <- stats::setNames(nodes$primary_section, as.character(nodes$id))
      n_nodes <- nrow(nodes)
      n_sections <- length(unique(nodes$primary_section))
      section_sizes <- table(nodes$primary_section)
      avg_section_size <- mean(section_sizes)

      # Adaptive repulsion strength (from biblioshiny networkPlot.R)
      community.repulsion <- repulsion / 10  # normalize to 0-1
      scale_factor <- log10(n_nodes + 10) / log10(100)
      community_factor <- 1 + (n_sections - 2) * 0.1
      community_factor <- max(0.5, min(community_factor, 2))
      x <- community.repulsion * 10
      sigmoid_transform <- x / (1 + x)
      repulsion_strength <- sigmoid_transform * scale_factor * community_factor

      # Apply weights per edge
      edge_from_sec <- node_sections[as.character(edges$from)]
      edge_to_sec <- node_sections[as.character(edges$to)]
      same_section <- edge_from_sec == edge_to_sec

      original_weights <- if (!is.null(igraph::E(ig)$weight)) {
        igraph::E(ig)$weight
      } else {
        rep(1, nrow(edges))
      }

      new_weights <- original_weights
      # Intra-section: moderate increase with sub-linear growth
      intra_mult <- 1 + (repulsion_strength^0.7) * 1.5
      new_weights[same_section] <- original_weights[same_section] * intra_mult
      # Inter-section: gradual reduction with attenuated exponential
      inter_div <- 1 + exp(repulsion_strength * 1.2) - 1
      new_weights[!same_section] <- original_weights[!same_section] / inter_div

      igraph::E(ig)$weight <- new_weights
    }

    set.seed(values$random_seed %||% 1234)
    coords <- igraph::layout_with_fr(ig, niter = 1500)
    coords <- igraph::norm_coords(coords, xmin = -1, xmax = 1, ymin = -1, ymax = 1)

    # --- Post-layout: spread nodes within each cluster ---
    if ("primary_section" %in% names(nodes)) {
      spread_factor <- 0.15 * spacing_mult
      sections <- unique(nodes$primary_section)
      for (sec in sections) {
        idx <- which(nodes$primary_section == sec)
        if (length(idx) > 1) {
          sec_center <- colMeans(coords[idx, , drop = FALSE])
          coords[idx, 1] <- sec_center[1] + (coords[idx, 1] - sec_center[1]) * (1 + spread_factor)
          coords[idx, 2] <- sec_center[2] + (coords[idx, 2] - sec_center[2]) * (1 + spread_factor)
        }
      }
      coords <- igraph::norm_coords(coords, xmin = -1, xmax = 1, ymin = -1, ymax = 1)
    }

    # --- Label overlap removal ---
    threshold <- 0.05 * spacing_mult
    w <- data.frame(
      x = coords[, 1],
      y = coords[, 2] / 2,
      label = nodes$label,
      dotSize = font_size,
      stringsAsFactors = FALSE
    )

    labeled <- w[w$label != "", , drop = FALSE]
    labels_to_remove <- character(0)
    if (nrow(labeled) >= 2) {
      d <- as.matrix(stats::dist(labeled[, c("x", "y")], method = "manhattan"))
      diag(d) <- Inf
      while (TRUE) {
        min_idx <- which(d == min(d), arr.ind = TRUE)[1, ]
        if (d[min_idx[1], min_idx[2]] >= threshold) break
        i <- min_idx[1]; j <- min_idx[2]
        remove_idx <- if (labeled$dotSize[i] >= labeled$dotSize[j]) j else i
        labels_to_remove <- c(labels_to_remove, labeled$label[remove_idx])
        d[remove_idx, ] <- Inf
        d[, remove_idx] <- Inf
      }
    }
    nodes$label[nodes$label %in% labels_to_remove] <- ""

    # --- Rebuild visNetwork with improved layout ---
    network_enhanced <- visNetwork::visNetwork(
      nodes, edges,
      type = "full", smooth = TRUE, physics = FALSE
    ) %>%
      visNetwork::visNodes(
        shadow = TRUE, shape = "dot",
        font = list(
          color = nodes$font.color,
          size = nodes$font.size,
          vadjust = nodes$font.vadjust
        )
      ) %>%
      visNetwork::visIgraphLayout(
        layout = "layout.norm",
        layoutMatrix = coords,
        type = "full"
      ) %>%
      visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
      visNetwork::visOptions(
        highlightNearest = list(enabled = TRUE, hover = TRUE, degree = 1),
        nodesIdSelection = TRUE
      ) %>%
      visNetwork::visInteraction(
        dragNodes = TRUE, dragView = TRUE,
        hideEdgesOnDrag = TRUE, zoomView = TRUE, zoomSpeed = 0.15
      ) %>%
      visNetwork::visPhysics(enabled = FALSE)

    # Update stats
    if (!is.null(stats_attr)) {
      stats_attr$n_nodes <- nrow(nodes)
      stats_attr$n_edges <- nrow(edges)
      stats_attr$section_distribution <- nodes %>%
        dplyr::count(primary_section) %>%
        dplyr::arrange(dplyr::desc(n))
    }
    attr(network_enhanced, "stats") <- stats_attr

    return(network_enhanced)
  }

  # Cartella temporanea per i PDF
  tmpdir <- tempdir()

  # Rendi la cartella temporanea accessibile via HTTP
  addResourcePath("tmpPDF", tmpdir)

  # Preview visibility reactive value
  preview_visible <- reactiveVal(TRUE)

  # Check if text is extracted
  output$text_extracted <- reactive({
    return(!is.null(values$pdf_text) && nchar(values$pdf_text) > 0)
  })
  outputOptions(output, "text_extracted", suspendWhenHidden = FALSE)

  # Preview visibility reactive output
  output$preview_visible <- reactive({
    return(preview_visible())
  })
  outputOptions(output, "preview_visible", suspendWhenHidden = FALSE)

  # ===========================================
  # PDF UPLOAD AND TEXT EXTRACTION
  # ===========================================

  # Check if PDF is uploaded
  output$pdf_uploaded <- reactive({
    return(!is.null(input$pdf_file))
  })
  outputOptions(output, "pdf_uploaded", suspendWhenHidden = FALSE)

  # Display PDF info
  output$pdf_info <- renderText({
    if (!is.null(input$pdf_file)) {
      file_size <- round(input$pdf_file$size / 1024 / 1024, 2)
      paste("File:", input$pdf_file$name, "| Size:", file_size, "MB")
    }
  })

  ## PDF PREVIEW
  output$pdf_viewer <- renderUI({
    req(input$pdf_file)

    # Verifica che sia un PDF
    if (!grepl("\\.pdf$", input$pdf_file$name, ignore.case = TRUE)) {
      return(
        div(
          style = "color: red; padding: 20px;",
          "Errore: Il file selezionato non è un PDF"
        )
      )
    }

    # Copia il file nella cartella temporanea con nome univoco
    pdf_name <- paste0(
      "viewer_",
      format(Sys.time(), "%Y%m%d_%H%M%S"),
      "_",
      input$pdf_file$name
    )
    pdf_path <- file.path(tmpdir, pdf_name)
    file.copy(input$pdf_file$datapath, pdf_path, overwrite = TRUE)

    # Crea l'iframe che punta al file tramite il path accessibile
    tags$iframe(
      src = paste0("tmpPDF/", pdf_name),
      width = "100%",
      height = "800px",
      style = "border: 1px solid #ddd;"
    )
  })

  # Pulizia alla chiusura della sessione
  onSessionEnded(function() {
    files <- list.files(
      tmpdir,
      pattern = "^viewer_.*\\.pdf$",
      full.names = TRUE
    )
    file.remove(files)
  })

  ## END PDF PREVIEW

  # Extract text from PDF
  observeEvent(input$extract_text, {
    req(input$pdf_file)

    # VALIDATION: Check if citation type is selected
    if (
      is.null(input$citation_type_import) ||
        input$citation_type_import == "" ||
        length(input$citation_type_import) == 0
    ) {
      showNotification(
        HTML(
          "<strong>Citation Format Required</strong><br/>Please select the citation format used in your PDF before extracting text."
        ),
        type = "warning",
        duration = 5
      )

      return() # Stop execution
    }

    tryCatch(
      {
        if (is.na(input$Columns)) {
          n_columns <- NULL
        } else {
          n_columns <- input$Columns
        }

        # Get citation type from input
        citation_type <- input$citation_type_import

        # Extract text with citation type parameter
        pdf_text <- pdf2txt_auto(
          input$pdf_file$datapath,
          n_columns = n_columns,
          citation_type = citation_type,
          enable_ai_support = input$enable_ai_support,
          ai_model = values$gemini_api_model,
          api_key = NULL
        )

        values$pdf_text <- pdf_text[[1]]

        # Extract PDF metadata (authors, title, journal, year, doi)
        pdf_metadata_info <- tryCatch(
          {
            contentanalysis::extract_pdf_metadata(
              input$pdf_file$datapath,
              fields = "all"
            )
          },
          error = function(e) {
            NULL
          }
        )

        # Store metadata in values
        values$pdf_metadata <- pdf_metadata_info

        # Check if AI support is enabled
        if (input$enable_ai_support) {
          if (!is.null(pdf_text)) {
            showNotification(
              "Text extracted successfully with AI enhancement!",
              type = "message",
              duration = 3
            )
          }
        }

        values$pdf_sections <- pdf_text

        values$citation_type_used <- citation_type # Store for later use

        pdf_metadata <- unlist(pdftools::pdf_info(input$pdf_file$datapath))

        # Extract DOI
        extracted_doi <- tryCatch(
          {
            extract_doi_from_pdf(input$pdf_file$datapath)
          },
          error = function(e) {
            NULL
          }
        )

        # Store DOI and update input field
        if (
          !is.null(extracted_doi) &&
            !is.na(extracted_doi) &&
            nzchar(trimws(extracted_doi))
        ) {
          values$pdf_doi <- trimws(extracted_doi)
          updateTextInput(
            session,
            "pdf_doi_input",
            value = trimws(extracted_doi)
          )
        } else {
          values$pdf_doi <- ""
          updateTextInput(session, "pdf_doi_input", value = "")
        }

        # Create notification message based on citation type
        notification_msg <- if (
          !is.null(extracted_doi) && nzchar(trimws(extracted_doi))
        ) {
          "PDF text extracted successfully! DOI detected and populated."
        } else {
          "PDF text extracted successfully! Please enter DOI manually if needed."
        }

        # Add citation type info to notification
        citation_type_label <- switch(
          citation_type,
          "author_year" = "Author-year",
          "numeric_bracketed" = "Numeric brackets",
          "numeric_superscript" = "Numeric superscript",
          "all" = "All formats"
        )

        if (citation_type == "numeric_superscript") {
          notification_msg <- paste0(
            notification_msg,
            " Superscript citations converted to [n] format."
          )
        }

        notification_msg <- paste0(
          notification_msg,
          " (Citation format: ",
          citation_type_label,
          ")"
        )

        showNotification(
          notification_msg,
          type = "message",
          duration = 5
        )

        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
      },
      error = function(e) {
        showNotification(
          paste("Error extracting PDF text:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Disable/Enable extract button based on citation type selection
  observe({
    if (
      !is.null(input$citation_type_import) &&
        input$citation_type_import != "" &&
        length(input$citation_type_import) > 0
    ) {
      # Citation type selected - ensure button is enabled
      shinyjs::enable("extract_text")
      shinyjs::removeCssClass("extract_text", "btn-secondary")
      shinyjs::addCssClass("extract_text", "btn-info")
    } else {
      # No citation type selected - visual feedback but don't disable
      # (We handle this in the observeEvent instead)
      shinyjs::removeCssClass("extract_text", "btn-info")
      shinyjs::addCssClass("extract_text", "btn-secondary")
    }
  })

  # Display PDF metadata (authors, title, journal) - compact version for top-right
  output$pdf_metadata_display <- renderUI({
    if (!is.null(values$pdf_metadata)) {
      metadata <- values$pdf_metadata

      # Extract metadata fields with safe defaults
      authors <- if (
        !is.null(metadata$authors) &&
          nzchar(metadata$authors) &&
          !is.na(metadata$authors)
      ) {
        if (nchar(metadata$authors) > 120) {
          paste0(substr(metadata$authors, 1, 117), "...")
        } else {
          metadata$authors
        }
      } else {
        "Authors not available"
      }

      title <- if (
        !is.null(metadata$title) &&
          nzchar(metadata$title) &&
          !is.na(metadata$title)
      ) {
        # Truncate title if too long
        if (nchar(metadata$title) > 120) {
          paste0(substr(metadata$title, 1, 117), "...")
        } else {
          metadata$title
        }
      } else {
        "Title not available"
      }

      journal <- if (
        !is.null(metadata$journal) &&
          nzchar(metadata$journal) &&
          !is.na(metadata$journal)
      ) {
        metadata$journal
      } else {
        "Journal not available"
      }

      year <- if (
        !is.null(metadata$year) &&
          !is.na(metadata$year) &&
          !is.na(metadata$year)
      ) {
        as.character(metadata$year)
      } else {
        ""
      }

      # Build compact display for top-right corner
      div(
        # style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); padding: 12px 15px; border-radius: 6px; margin-bottom: 15px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); color: white; text-align: left; font-size: 12px;",
        style = "background: linear-gradient(135deg, #1D8FE1 0%, #5765B1 100%); padding: 12px 15px; border-radius: 6px; margin-bottom: 15px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); color: white; text-align: left; font-size: 12px;",

        # Title (most prominent)
        div(
          style = "margin-bottom: 6px; font-weight: 600; font-size: 13px; line-height: 1.3;",
          title
        ),

        # Authors
        div(
          style = "margin-bottom: 4px; opacity: 0.95;",
          icon("user", style = "margin-right: 5px; font-size: 11px;"),
          tags$span(authors, style = "font-size: 11px;")
        ),

        # Journal and Year
        div(
          style = "opacity: 0.9;",
          icon("book", style = "margin-right: 5px; font-size: 11px;"),
          tags$span(
            paste0(journal, if (nzchar(year)) paste0(" (", year, ")") else ""),
            style = "font-size: 11px;"
          )
        )
      )
    }
  })

  # Check if metadata is available
  output$metadata_available <- reactive({
    return(!is.null(values$pdf_metadata))
  })
  outputOptions(output, "metadata_available", suspendWhenHidden = FALSE)

  # Check if DOI was detected automatically
  output$doi_detected <- reactive({
    if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
      # Check if it's the same as what was automatically extracted
      extracted_doi <- tryCatch(
        {
          if (!is.null(input$pdf_file)) {
            extract_doi_from_pdf(input$pdf_file$datapath)
          } else {
            NULL
          }
        },
        error = function(e) NULL
      )

      return(
        !is.null(extracted_doi) &&
          nzchar(extracted_doi) &&
          extracted_doi == input$pdf_doi_input
      )
    }
    return(FALSE)
  })

  outputOptions(output, "doi_detected", suspendWhenHidden = FALSE)

  # Display DOI after analysis
  output$doi_display_after_analysis <- renderText({
    doi_value <- values$pdf_doi

    if (!is.null(doi_value) && !is.na(doi_value) && nzchar(trimws(doi_value))) {
      return(trimws(doi_value))
    } else {
      return("Not available")
    }
  })

  # DOI status icon
  output$doi_status_icon <- renderUI({
    # Verifica se il campo DOI ha un valore
    doi_value <- input$pdf_doi_input

    if (is.null(doi_value) || !nzchar(trimws(doi_value))) {
      # Nessun DOI inserito
      return(
        tagList(
          icon(
            "exclamation-triangle",
            style = "color: #e74c3c; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "DOI not found",
            style = "color: #e74c3c; font-weight: bold;"
          )
        )
      )
    }

    # C'è un DOI, verifica se è stato auto-rilevato
    auto_detected <- FALSE

    if (!is.null(input$pdf_file)) {
      extracted_doi <- tryCatch(
        {
          extract_doi_from_pdf(input$pdf_file$datapath)
        },
        error = function(e) {
          NULL
        }
      )

      # Verifica se il DOI estratto corrisponde a quello inserito
      if (
        !is.null(extracted_doi) &&
          !is.na(extracted_doi) &&
          nzchar(trimws(extracted_doi))
      ) {
        auto_detected <- (trimws(extracted_doi) == trimws(doi_value))
      }
    }

    # Restituisci l'icona appropriata
    if (auto_detected) {
      return(
        tagList(
          icon(
            "check-circle",
            style = "color: #27ae60; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "Auto-detected",
            style = "color: #27ae60; font-weight: bold;"
          )
        )
      )
    } else {
      return(
        tagList(
          icon(
            "edit",
            style = "color: #f39c12; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "Manually entered",
            style = "color: #f39c12; font-weight: bold;"
          )
        )
      )
    }
  })

  # Update DOI value when user edits the field
  observeEvent(
    input$pdf_doi_input,
    {
      if (!is.null(input$pdf_doi_input)) {
        values$pdf_doi <- trimws(input$pdf_doi_input)
      }
    },
    ignoreInit = TRUE
  )

  # ===========================================
  # LOAD PREVIOUSLY SAVED TEXT FILE
  # ===========================================

  # Check if text file is loaded
  output$text_file_loaded <- reactive({
    return(!is.null(input$load_text_file))
  })
  outputOptions(output, "text_file_loaded", suspendWhenHidden = FALSE)

  # Display text file info
  output$text_file_info <- renderText({
    if (!is.null(input$load_text_file)) {
      file_size <- round(input$load_text_file$size / 1024, 2)
      paste("File:", input$load_text_file$name, "| Size:", file_size, "KB")
    }
  })

  observeEvent(input$load_text_file, {
    req(input$load_text_file)

    tryCatch(
      {
        # Read the text file
        file_content <- readLines(input$load_text_file$datapath, warn = FALSE)

        if (length(file_content) == 0) {
          showNotification(
            "The text file is empty.",
            type = "error",
            duration = 5
          )
          return()
        }

        # Extract DOI from first line if present (format: "DOI: xxxxx")
        first_line <- file_content[1]
        extracted_doi <- ""
        text_start_line <- 1

        if (grepl("^DOI:\\s*", first_line, ignore.case = TRUE)) {
          extracted_doi <- sub("^DOI:\\s*", "", first_line, ignore.case = TRUE)
          extracted_doi <- trimws(extracted_doi)
          text_start_line <- 2
        }

        # Extract CITATION_TYPE from second line if present (format: "CITATION_TYPE: xxxxx")
        extracted_citation_type <- ""
        if (length(file_content) >= 2) {
          second_line <- file_content[2]
          if (grepl("^CITATION_TYPE:\\s*", second_line, ignore.case = TRUE)) {
            extracted_citation_type <- sub(
              "^CITATION_TYPE:\\s*",
              "",
              second_line,
              ignore.case = TRUE
            )
            extracted_citation_type <- trimws(extracted_citation_type)
            text_start_line <- 3
          }
        }

        # Get the text content (everything after metadata lines)
        if (length(file_content) >= text_start_line) {
          text_content <- reconstruct_from_txt(
            file_content[text_start_line:length(file_content)]
          )
        } else {
          text_content <- ""
        }

        # Store the text
        values$pdf_text <- unlist(text_content[1])
        values$pdf_sections <- text_content

        # Update DOI field
        if (nzchar(extracted_doi)) {
          values$pdf_doi <- extracted_doi
          updateTextInput(session, "pdf_doi_input", value = extracted_doi)
        } else {
          values$pdf_doi <- ""
          updateTextInput(session, "pdf_doi_input", value = "")
        }

        # Update citation_type field if available
        if (nzchar(extracted_citation_type)) {
          values$citation_type_used <- extracted_citation_type
          # Note: citation_type_import is only used during PDF import, not needed here
        }

        # Show notification
        notification_msg <- "Text file loaded successfully!"
        if (nzchar(extracted_doi)) {
          notification_msg <- paste0(notification_msg, " DOI: ", extracted_doi)
        }
        if (nzchar(extracted_citation_type)) {
          citation_type_label <- switch(
            extracted_citation_type,
            "author_year" = "Author-year",
            "numeric_bracketed" = "Numeric brackets",
            "numeric_superscript" = "Numeric superscript",
            "all" = "All formats",
            extracted_citation_type
          )
          notification_msg <- paste0(
            notification_msg,
            " | Citation format: ",
            citation_type_label
          )
        }

        showNotification(
          notification_msg,
          type = "message",
          duration = 7
        )

        # Enable analysis button
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
      },
      error = function(e) {
        showNotification(
          paste("Error loading text file:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # ===========================================
  # SAVE EXTRACTED TEXT TO FILE
  # ===========================================
  output$save_text_file <- downloadHandler(
    filename = function() {
      # Generate filename with DOI or timestamp
      if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
        # Clean DOI for filename (replace / with _)
        clean_doi <- gsub("/", "_", values$pdf_doi)
        clean_doi <- gsub("[^a-zA-Z0-9._-]", "", clean_doi)
        paste0("extracted_text_", clean_doi, ".txt")
      } else {
        paste0("extracted_text_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".txt")
      }
    },
    content = function(file) {
      # Prepare content with DOI and CITATION_TYPE in first lines
      file_content <- ""

      # Add DOI as first line if available
      if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
        file_content <- paste0("DOI: ", values$pdf_doi, "\n")
      } else {
        file_content <- "DOI: \n"
      }

      # Add CITATION_TYPE as second line if available
      if (
        !is.null(values$citation_type_used) && nzchar(values$citation_type_used)
      ) {
        file_content <- paste0(
          file_content,
          "CITATION_TYPE: ",
          values$citation_type_used,
          "\n"
        )
      } else {
        file_content <- paste0(file_content, "CITATION_TYPE: \n")
      }

      if (length(values$pdf_sections) > 1) {
        txt <- values$pdf_sections
        section_names <- setdiff(names(txt), "Full_text")
        # save txt in a text file adding a row with \n\nSECTION: section names\n\n before each section except Full_text
        #file_content <- ""
        for (section in section_names) {
          file_content <- paste0(
            file_content,
            "\n\nSECTION LINE SEPARATION: ",
            section,
            "\n\n",
            txt[[section]]
          )
        }
      } else {
        file_content <- paste0(file_content, unlist(values$pdf_sections[1]))
      }

      # # Add the extracted text
      # if (!is.null(values$pdf_text)) {
      #   file_content <- paste0(file_content, values$pdf_text)
      # }

      # Write to file
      writeLines(file_content, file, sep = "")
    }
  )

  # Text length information
  output$text_length_info <- renderText({
    if (!is.null(values$pdf_text)) {
      char_count <- nchar(values$pdf_text)
      word_count <- length(strsplit(values$pdf_text, "\\s+")[[1]])
      paste(
        "Characters:",
        format(char_count, big.mark = ","),
        "| Words:",
        format(word_count, big.mark = ",")
      )
    }
  })

  # Text preview output
  output$text_preview <- renderText({
    if (!is.null(values$pdf_text)) {
      preview_length <- 80000
      full_text <- values$pdf_text

      if (nchar(full_text) > preview_length) {
        truncated <- substr(full_text, 1, preview_length)
        last_period <- max(c(
          regexpr("\\. [A-Z]", truncated, perl = TRUE),
          regexpr("\\.\n", truncated, perl = TRUE),
          regexpr("\\?\n", truncated, perl = TRUE),
          regexpr("!\n", truncated, perl = TRUE)
        ))

        if (last_period > 100) {
          truncated <- substr(truncated, 1, last_period)
        }

        paste0(
          truncated,
          "\n\n[...text continues for ",
          format(nchar(full_text) - nchar(truncated), big.mark = ","),
          " more characters...]"
        )
      } else {
        full_text
      }
    }
  })

  # Toggle preview visibility
  observeEvent(input$toggle_preview, {
    preview_visible(!preview_visible())
    if (preview_visible()) {
      updateActionButton(session, "toggle_preview", "Hide Preview")
    } else {
      updateActionButton(session, "toggle_preview", "Show Preview")
    }
  })

  # ===========================================
  # CONTENT ANALYSIS
  # ===========================================

  observeEvent(input$run_analysis, {
    req(values$pdf_text)

    updateActionButton(
      session,
      "run_analysis",
      label = "Processing...",
      icon = icon("spinner", class = "fa-spin")
    )

    tryCatch(
      {
        custom_stops <- NULL
        if (
          !is.null(input$custom_stopwords) && nzchar(input$custom_stopwords)
        ) {
          custom_stops <- trimws(strsplit(input$custom_stopwords, ",")[[1]])
        }

        # Determine citation segmentation
        use_sections_cit <- switch(
          input$citation_segmentation %||% "auto",
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE,
          "auto" # default fallback
        )

        n_segs_cit <- input$n_segments_citations %||% 10

        # Get citation type (use stored value from extraction or default)
        citation_type <- values$citation_type_used
        if (is.null(citation_type) || citation_type == "") {
          citation_type <- input$citation_type_import
          if (is.null(citation_type) || citation_type == "") {
            citation_type <- "author_year" # ultimate fallback
          }
        }

        # Run analysis with citation type parameter
        values$analysis_results <- analyze_scientific_content(
          text = values$pdf_sections,
          doi = values$pdf_doi,
          citation_type = citation_type,
          window_size = input$window_size,
          parse_multiple_citations = input$parse_multiple,
          remove_stopwords = input$remove_stopwords,
          custom_stopwords = custom_stops,
          use_sections_for_citations = use_sections_cit,
          n_segments_citations = n_segs_cit
        )

        # remove doi_pattern citations
        values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
          dplyr::filter(!citation_type %in% c("doi_pattern"))

        values$references_oa <- values$analysis_results$references_oa
        section_colors <- colorlist()[
          1:length(unique(values$analysis_results$citation_contexts$section))
        ]
        names(section_colors) <- unique(
          values$analysis_results$citation_contexts$section
        )
        values$analysis_results$section_colors <- section_colors

        # Calculate readability indices
        values$readability_indices <- calculate_readability_indices(
          text = values$pdf_text,
          detailed = TRUE
        )

        # Create network if we have data
        if (
          !is.null(values$analysis_results$network_data) &&
            nrow(values$analysis_results$network_data) > 0
        ) {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        }

        # Describe citation clusters
        tryCatch(
          {
            values$cluster_description <- describe_citation_clusters(
              values$analysis_results,
              top_n = 10,
              ngram_range = c(1, 2)
            )
          },
          error = function(e) {
            values$cluster_description <- NULL
          }
        )

        preview_visible(FALSE)

        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )

        showNotification(
          paste0(
            "Content analysis completed successfully! Citation type: ",
            switch(
              citation_type,
              "author_year" = "Author-Year",
              "numeric_bracketed" = "Numeric Bracketed",
              "numeric_superscript" = "Numeric Superscript",
              "all" = "All Formats"
            )
          ),
          type = "message",
          duration = 4
        )

        # Close the PDF import box
        session$sendCustomMessage('togglePdfImport', list(action = 'close'))
      },
      error = function(e) {
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )

        showNotification(
          paste("Error in content analysis:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Check if analysis is completed
  output$analysis_completed <- reactive({
    return(!is.null(values$analysis_results))
  })
  outputOptions(output, "analysis_completed", suspendWhenHidden = FALSE)

  # Check if Gemini API is available
  output$gemini_api_available <- reactive({
    return(!is.null(values$geminiAPI) && values$geminiAPI == TRUE)
  })
  outputOptions(output, "gemini_api_available", suspendWhenHidden = FALSE)

  # ===========================================
  # TAB 1: DESCRIPTIVE STATISTICS OUTPUTS
  # ===========================================

  output$total_words <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$total_words_analyzed,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$total_citations <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$citations_extracted,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$narrative_citations <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$narrative_citations,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$citation_density <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$citation_density_per_1000_words,
        digits = 1
      )
    } else {
      "0.0"
    }
  })

  # Readability indices outputs
  output$flesch_kincaid_grade <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$flesch_kincaid_grade, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$flesch_reading_ease <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$flesch_reading_ease, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$ari_index <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$automated_readability_index, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$gunning_fog_index <- renderText({
    if (!is.null(values$readability_indices)) {
      format(round(values$readability_indices$gunning_fog_index, 1), nsmall = 1)
    } else {
      "0.0"
    }
  })

  # Text Statistics - Matching height with readability
  # Text Statistics - Matching structure with readability
  output$text_stats <- renderUI({
    if (!is.null(values$analysis_results)) {
      stats <- values$analysis_results$text_analytics$basic_stats

      # Get additional stats from readability indices
      syllables_info <- ""
      complex_info <- ""

      if (!is.null(values$readability_indices)) {
        syllables_info <- format(
          values$readability_indices$n_syllables,
          big.mark = ","
        )
        complex_info <- sprintf(
          "Complex words:      %s (%.1f%%)",
          format(values$readability_indices$n_complex_words, big.mark = ","),
          values$readability_indices$pct_complex_words
        )
      } else {
        syllables_info <- "N/A"
        complex_info <- "Complex words:      N/A"
      }

      text_content <- sprintf(
        "TEXT STATISTICS
━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

🔤 Characters:      %s
📄 Words:           %s
✏️ Sentences:       %s
🗣 Syllables:       %s

━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

%s
Avg words/sentence: %.1f
Lexical diversity:  %.3f",
        format(stats$total_characters, big.mark = ","),
        format(stats$total_words, big.mark = ","),
        format(stats$total_sentences, big.mark = ","),
        syllables_info,
        complex_info,
        round(stats$avg_words_per_sentence, 1),
        round(values$analysis_results$summary$lexical_diversity, 3)
      )

      tags$pre(
        style = "background-color: #f8f9fa; padding: 15px; border-radius: 6px; border: 1px solid #dee2e6; font-family: 'Courier New', monospace; font-size: 13px; line-height: 1.5; color: #333; margin: 0; min-height: 280px;",
        text_content
      )
    } else {
      tags$pre(
        style = "background-color: #f8f9fa; padding: 40px; border-radius: 6px; border: 1px solid #dee2e6; text-align: center; color: #999; margin: 0; min-height: 280px; display: flex; align-items: center; justify-content: center;",
        "No analysis data available"
      )
    }
  })

  # Readability Indices - Add min-height
  output$readability_details_html <- renderUI({
    if (!is.null(values$readability_indices)) {
      indices <- values$readability_indices

      # Helper to get interpretation symbol
      get_symbol <- function(index_name, value) {
        switch(
          index_name,
          "flesch_kincaid" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          },
          "flesch_ease" = if (value >= 60) {
            "✓"
          } else if (value >= 30) {
            "◆"
          } else {
            "▲"
          },
          "ari" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          },
          "gunning" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          }
        )
      }

      # Helper to get interpretation text
      get_interp_text <- function(index_name, value) {
        switch(
          index_name,
          "flesch_kincaid" = paste("Grade", round(value)),
          "flesch_ease" = if (value >= 60) {
            "Easy"
          } else if (value >= 30) {
            "Moderate"
          } else {
            "Difficult"
          },
          "ari" = paste("Grade", round(value)),
          "gunning" = if (value <= 12) {
            "Readable"
          } else if (value <= 16) {
            "Difficult"
          } else {
            "Very difficult"
          }
        )
      }

      text_content <- sprintf(
        "READABILITY INDICES
━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

📚 Flesch-Kincaid:  %5.1f  %s  %s
📖 Reading Ease:    %5.1f  %s  %s  
🤖 ARI Index:       %5.1f  %s  %s
☁️ Gunning Fog:     %5.1f  %s  %s

━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

Sentences: %s • Words: %s
Syllables: %s • Complex: %s (%.1f%%)
Avg sentence length: %.1f words",
        indices$flesch_kincaid_grade,
        get_symbol("flesch_kincaid", indices$flesch_kincaid_grade),
        get_interp_text("flesch_kincaid", indices$flesch_kincaid_grade),
        indices$flesch_reading_ease,
        get_symbol("flesch_ease", indices$flesch_reading_ease),
        get_interp_text("flesch_ease", indices$flesch_reading_ease),
        indices$automated_readability_index,
        get_symbol("ari", indices$automated_readability_index),
        get_interp_text("ari", indices$automated_readability_index),
        indices$gunning_fog_index,
        get_symbol("gunning", indices$gunning_fog_index),
        get_interp_text("gunning", indices$gunning_fog_index),
        format(indices$n_sentences, big.mark = ","),
        format(indices$n_words, big.mark = ","),
        format(indices$n_syllables, big.mark = ","),
        format(indices$n_complex_words, big.mark = ","),
        indices$pct_complex_words,
        indices$avg_sentence_length
      )

      tags$pre(
        style = "background-color: #f8f9fa; padding: 15px; border-radius: 6px; border: 1px solid #dee2e6; font-family: 'Courier New', monospace; font-size: 13px; line-height: 1.5; color: #333; margin: 0; min-height: 280px;",
        text_content
      )
    } else {
      tags$pre(
        style = "background-color: #f8f9fa; padding: 40px; border-radius: 6px; border: 1px solid #dee2e6; text-align: center; color: #999; margin: 0; min-height: 280px; display: flex; align-items: center; justify-content: center;",
        "No readability data available"
      )
    }
  })

  # Citation types table
  output$citation_types_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$citation_metrics$type_distribution)
      ) {
        values$analysis_results$citation_metrics$type_distribution
      } else {
        data.frame(
          citation_type = character(0),
          n = numeric(0),
          percentage = numeric(0)
        )
      }
    }),
    title = "Citation Types",
    nrow = 10,
    filter = "none",
    round = 2
  ) })

  # Citation sections table
  output$citation_sections_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(
            values$analysis_results$citation_metrics$section_distribution
          )
      ) {
        values$analysis_results$citation_metrics$section_distribution %>%
          filter(n > 0)
      } else {
        data.frame(
          section = character(0),
          n = numeric(0),
          percentage = numeric(0)
        )
      }
    }),
    title = "Citation Sections",
    nrow = 10,
    filter = "none",
    round = 2
  ) })

  # Frequent words table
  output$frequent_words_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (!is.null(values$analysis_results)) {
        values$analysis_results$word_frequencies %>%
          slice_head(n = 15) %>%
          select(word, n)
      } else {
        data.frame(word = character(0), n = numeric(0))
      }
    }),
    title = "Frequent Words",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Bigrams table
  output$bigrams_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          "2gram" %in% names(values$analysis_results$ngrams)
      ) {
        values$analysis_results$ngrams$`2gram` %>%
          select(ngram, n) %>%
          slice_head(n = 15)
      } else {
        data.frame(ngram = character(0), n = numeric(0))
      }
    }),
    title = "Bigrams (2-grams)",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Trigrams table
  output$trigrams_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          "3gram" %in% names(values$analysis_results$ngrams)
      ) {
        values$analysis_results$ngrams$`3gram` %>%
          select(ngram, n) %>%
          slice_head(n = 15)
      } else {
        data.frame(ngram = character(0), n = numeric(0))
      }
    }),
    title = "Trigrams (3-grams)",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Text statistics
  # output$text_stats <- renderText({
  #   if (!is.null(values$analysis_results)) {
  #     stats <- values$analysis_results$text_analytics$basic_stats
  #     paste(
  #       "Characters:", format(stats$total_characters, big.mark = ","), "\n",
  #       "Words:", format(stats$total_words, big.mark = ","), "\n",
  #       "Sentences:", format(stats$total_sentences, big.mark = ","), "\n",
  #       "Avg words/sentence:", round(stats$avg_words_per_sentence, 1), "\n",
  #       "Lexical diversity:", round(values$analysis_results$summary$lexical_diversity, 3)
  #     )
  #   } else {
  #     "No analysis data available"
  #   }
  # })

  # ===========================================
  # TAB 2: IN-CONTEXT CITATION ANALYSIS
  # ===========================================

  # ===========================================
  # UPDATE CITATION GROUPING (without re-running full analysis)
  # ===========================================

  observeEvent(input$update_citation_grouping, {
    req(values$pdf_text)
    req(values$analysis_results)

    tryCatch(
      {
        # Determine citation segmentation
        use_sections_cit <- switch(
          input$citation_segmentation %||% "auto",
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE,
          "auto"
        )

        n_segs_cit <- input$n_segments_citations %||% 10

        # Get citations without section column
        citations_for_mapping <- values$analysis_results$citations %>%
          select(-section, -segment_type)

        # Re-map citations to new segments/sections
        citations_remapped <- map_citations_to_segments(
          citations_df = citations_for_mapping,
          text = values$pdf_sections,
          use_sections = use_sections_cit,
          n_segments = n_segs_cit
        )

        # Update the citations in analysis_results
        values$analysis_results$citations <- citations_remapped %>%
          rename(section = segment)

        # Update citation contexts with new section info
        if (!is.null(values$analysis_results$citation_contexts)) {
          # Remove old section column if exists
          if ("section" %in% names(values$analysis_results$citation_contexts)) {
            values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
              select(-section)
          }

          # Add new section mapping
          values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
            left_join(
              values$analysis_results$citations %>%
                select(citation_id, section),
              by = "citation_id"
            )
          section_colors <- colorlist()[
            1:length(unique(values$analysis_results$citation_contexts$section))
          ]
          names(section_colors) <- unique(
            values$analysis_results$citation_contexts$section
          )
          values$analysis_results$section_colors <- section_colors
        }

        # Update citation metrics - section distribution
        if (!is.null(values$analysis_results$citation_metrics)) {
          # Determine expected sections/segments
          sections_to_use <- NULL

          if (
            use_sections_cit == TRUE ||
              (use_sections_cit == "auto" &&
                length(setdiff(
                  names(values$pdf_sections),
                  c("Full_text", "References")
                )) >
                  0)
          ) {
            # Using sections
            sections_to_use <- setdiff(
              names(values$pdf_sections),
              c("Full_text", "References")
            )
          } else {
            # Using segments
            sections_to_use <- paste0("Segment ", 1:n_segs_cit)
          }

          # Recalculate section distribution
          if (!is.null(sections_to_use)) {
            values$analysis_results$citation_metrics$section_distribution <-
              values$analysis_results$citations %>%
              mutate(section = factor(section, levels = sections_to_use)) %>%
              count(section, sort = FALSE, .drop = FALSE) %>%
              mutate(percentage = round(n / sum(n) * 100, 2))
          } else {
            values$analysis_results$citation_metrics$section_distribution <-
              values$analysis_results$citations %>%
              count(section, sort = TRUE) %>%
              mutate(percentage = round(n / sum(n) * 100, 2))
          }
        }

        # Recreate network with new grouping
        if (
          !is.null(values$analysis_results$network_data) &&
            nrow(values$analysis_results$network_data) > 0
        ) {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        }

        showNotification(
          "Citation grouping updated successfully!",
          type = "message",
          duration = 2
        )
      },
      error = function(e) {
        showNotification(
          paste("Error updating citation grouping:", e$message),
          type = "error",
          duration = 5
        )

        # Print error to console for debugging
        cat("Error in update_citation_grouping:\n")
        print(e)
      }
    )
  })

  # Custom HTML output for citation contexts
  output$citation_contexts_html <- renderUI({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$citation_contexts) &&
        nrow(values$analysis_results$citation_contexts) > 0
    ) {
      contexts <- values$analysis_results$citation_contexts

      # Apply filters (REMOVED context_type_filter)
      if (!is.null(input$context_search) && nzchar(input$context_search)) {
        contexts <- contexts %>%
          filter(
            str_detect(
              citation_text,
              regex(input$context_search, ignore_case = TRUE)
            ) |
              str_detect(
                full_context,
                regex(input$context_search, ignore_case = TRUE)
              )
          )
      }

      # REMOVED: type filter code

      if (!is.null(input$context_min_words)) {
        contexts <- contexts %>%
          filter(context_word_count >= input$context_min_words)
      }

      # Create HTML for each citation context
      if (nrow(contexts) > 0) {
        citation_boxes <- lapply(1:nrow(contexts), function(i) {
          context <- contexts[i, ]

          section_colors <- values$analysis_results$section_colors

          section_name <- if (
            !is.null(context$section) && !is.na(context$section)
          ) {
            context$section
          } else {
            "Unknown"
          }

          box_color <- section_colors[section_name]
          if (is.na(box_color)) {
            box_color <- "#CCCCCC"
          }

          has_reference <- FALSE
          tooltip_text <- "No reference matched for this citation"

          if ("ref_full_text" %in% names(context)) {
            ref_value <- context[["ref_full_text"]]

            if (
              length(ref_value) > 0 &&
                !is.na(ref_value) &&
                is.character(ref_value) &&
                nzchar(trimws(ref_value))
            ) {
              has_reference <- TRUE
              ref_text <- trimws(ref_value)
              ref_text <- gsub("&", "&amp;", ref_text)
              ref_text <- gsub("<", "&lt;", ref_text)
              ref_text <- gsub(">", "&gt;", ref_text)
              ref_text <- gsub('"', "&quot;", ref_text)
              ref_text <- gsub("'", "&#39;", ref_text)
              ref_text <- gsub("\n", "<br/>", ref_text)
              tooltip_text <- paste0(
                "<strong style='color: #4ECDC4;'>Reference:</strong><br/>",
                ref_text
              )
            }
          }

          citation_id <- paste0("citation_", i, "_", sample(1:10000, 1))

          div(
            style = paste0(
              "margin-bottom: 20px; padding: 15px; border-radius: 8px; ",
              "box-shadow: 0 2px 4px rgba(0,0,0,0.1); ",
              "border-left: 4px solid ",
              box_color,
              "; ",
              "background-color: #fafafa;"
            ),

            div(
              style = "margin-bottom: 10px; font-size: 12px; color: #666;",
              span(paste("Citation", i, "•"), style = "font-weight: bold;"),
              span(
                section_name,
                style = paste0(
                  "color: ",
                  box_color,
                  "; font-weight: bold; margin-left: 8px;"
                )
              ),
              span(paste("• Position:", context$citation_position_in_text))
            ),

            div(
              style = "display: flex; align-items: center; font-family: 'Courier New', monospace;",

              div(
                style = "flex: 1; text-align: right; padding-right: 15px; color: #555; font-size: 14px;",
                if (nzchar(context$words_before)) {
                  paste("...", context$words_before)
                } else {
                  "[start of text]"
                }
              ),

              div(
                id = citation_id,
                class = "citation-with-tooltip",
                style = paste0(
                  "background-color: ",
                  box_color,
                  "; color: white; ",
                  "padding: 8px 12px; border-radius: 6px; font-weight: bold; ",
                  "font-size: 14px; white-space: nowrap; max-width: 300px; ",
                  "overflow: hidden; text-overflow: ellipsis; ",
                  "cursor: ",
                  if (has_reference) "pointer" else "default",
                  "; position: relative;"
                ),
                `data-toggle` = if (has_reference) "tooltip" else NULL,
                `data-placement` = "top",
                `data-html` = "true",
                `data-original-title` = tooltip_text,
                context$citation_text
              ),

              div(
                style = "flex: 1; text-align: left; padding-left: 15px; color: #555; font-size: 14px;",
                if (nzchar(context$words_after)) {
                  paste(context$words_after, "...")
                } else {
                  "[end of text]"
                }
              )
            ),

            div(
              style = "margin-top: 10px; font-size: 11px; color: #888;",
              paste("Context words:", context$context_word_count, "•"),
              if (!is.na(context$is_narrative) && context$is_narrative) {
                span(
                  "Narrative citation",
                  style = "color: #e67e22; font-weight: bold;"
                )
              } else {
                span(
                  "Parenthetical citation",
                  style = "color: #3498db; font-weight: bold;"
                )
              },
              if (has_reference) {
                span(
                  " • ",
                  icon("book", style = "color: #27ae60;"),
                  " Reference matched (hover to view)",
                  style = "color: #27ae60; font-weight: bold;"
                )
              } else {
                span(
                  " • ",
                  icon("times-circle", style = "color: #e74c3c;"),
                  " No reference matched",
                  style = "color: #999; font-style: italic;"
                )
              }
            )
          )
        })

        tagList(
          tags$script(HTML(
            "
        $(document).ready(function(){
          $('.citation-with-tooltip').tooltip('dispose');
          
          $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip({
            container: 'body',
            trigger: 'hover focus',
            delay: { show: 200, hide: 100 },
            html: true,
            boundary: 'window'
          });
          
          setTimeout(function() {
            $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip('dispose');
            $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip({
              container: 'body',
              trigger: 'hover focus',
              delay: { show: 200, hide: 100 },
              html: true,
              boundary: 'window'
            });
          }, 800);
        });
      "
          )),

          tags$style(HTML(
            "
        .tooltip {
          font-family: Arial, sans-serif;
          pointer-events: none;
        }
        .tooltip-inner {
          max-width: 500px;
          min-width: 250px;
          text-align: left;
          background-color: rgba(0, 0, 0, 0.95);
          padding: 12px 16px;
          font-size: 13px;
          line-height: 1.6;
          border-radius: 6px;
          box-shadow: 0 4px 12px rgba(0,0,0,0.4);
        }
        .tooltip-inner strong {
          color: #4ECDC4;
          display: block;
          margin-bottom: 6px;
        }
        .tooltip.show {
          opacity: 1;
        }
        .tooltip.top .tooltip-arrow {
          border-top-color: rgba(0, 0, 0, 0.95);
        }
        .citation-with-tooltip[data-toggle]:hover {
          opacity: 0.9;
          transform: scale(1.02);
          transition: all 0.2s ease;
        }
      "
          )),

          citation_boxes
        )
      } else {
        div(
          style = "text-align: center; padding: 40px; color: #999;",
          icon("search", style = "font-size: 24px; margin-bottom: 10px;"),
          h4("No citations match your current filters"),
          p("Try adjusting your search terms or filters.")
        )
      }
    } else {
      div(
        style = "text-align: center; padding: 40px; color: #999;",
        icon("quote-right", style = "font-size: 24px; margin-bottom: 10px;"),
        h4("No citation contexts available"),
        p("Run the analysis first to see in-context citations.")
      )
    }
  })

  # ===========================================
  # TAB 3: NETWORK ANALYSIS
  # ===========================================

  # Network visualization
  output$citation_network <- renderVisNetwork({
    if (!is.null(values$network_plot)) {
      values$network_plot
    } else {
      visNetwork(
        nodes = data.frame(
          id = 1,
          label = "No citations found",
          size = 20,
          color = "#CCCCCC"
        ),
        edges = data.frame(from = numeric(0), to = numeric(0))
      ) %>%
        visOptions(highlightNearest = FALSE) %>%
        visInteraction(dragNodes = FALSE, dragView = FALSE, zoomView = FALSE)
    }
  })

  # Update network layout with new spacing
  observeEvent(input$update_network_layout, {
    req(values$analysis_results)
    if (
      !is.null(values$analysis_results$network_data) &&
        nrow(values$analysis_results$network_data) > 0
    ) {
      tryCatch(
        {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        },
        error = function(e) {
          showNotification(
            paste("Error updating layout:", e$message),
            type = "error",
            duration = 4
          )
        }
      )
    }
  })

  # Network information
  output$network_info <- renderText({
    if (!is.null(values$network_plot)) {
      stats <- attr(values$network_plot, "stats")
      if (!is.null(stats)) {
        section_info <- ""
        if (!is.null(stats$section_distribution)) {
          section_info <- paste0(
            "\n\nSection Distribution:\n",
            paste(
              stats$section_distribution$section,
              ": ",
              stats$section_distribution$n,
              collapse = "\n"
            )
          )
        }

        paste(
          "Nodes:",
          stats$n_nodes,
          "\n",
          "Edges:",
          stats$n_edges,
          "\n",
          "Avg. Distance:",
          stats$avg_distance,
          "chars\n",
          "Max Distance Filter:",
          stats$max_distance,
          "chars\n\n",
          "Network Density:",
          round(stats$n_edges / (stats$n_nodes * (stats$n_nodes - 1) / 2), 3),
          section_info
        )
      } else {
        "Network statistics not available"
      }
    } else {
      "No network data available"
    }
  })

  # Strongest connections table
  output$strongest_connections <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$network_data) &&
          nrow(values$analysis_results$network_data) > 0
      ) {
        values$analysis_results$network_data %>%
          arrange(abs(distance)) %>%
          slice_head(n = 8) %>%
          select(citation1, citation2, distance) %>%
          mutate(
            citation1 = str_trunc(citation1, 20),
            citation2 = str_trunc(citation2, 20),
            distance = paste(abs(distance), "chars")
          )
      } else {
        data.frame(
          citation1 = character(0),
          citation2 = character(0),
          distance = character(0)
        )
      }
    }),
    title = "Strongest Connections",
    nrow = 8,
    filter = "none",
    round = 0
  ) })

  # ===========================================
  # DOWNLOAD HANDLERS
  # ===========================================

  output$download_network <- downloadHandler(
    filename = function() {
      paste0("citation_network_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$network_data)
      ) {
        write.csv(values$analysis_results$network_data, file, row.names = FALSE)
      } else {
        write.csv(
          data.frame(message = "No network data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  output$download_contexts <- downloadHandler(
    filename = function() {
      paste0("citation_contexts_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$citation_contexts)
      ) {
        write.csv(
          values$analysis_results$citation_contexts,
          file,
          row.names = FALSE
        )
      } else {
        write.csv(
          data.frame(message = "No citation contexts available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 4: WORD TRENDS ANALYSIS
  # ===========================================

  # Populate word choices when analysis is complete
  observe({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$word_frequencies) &&
        nrow(values$analysis_results$word_frequencies) > 0
    ) {
      # Get top 30 most frequent words
      top_words <- values$analysis_results$word_frequencies %>%
        slice_head(n = 30) %>%
        pull(word)

      # Get top bigrams if available
      top_bigrams <- character(0)
      if (
        !is.null(values$analysis_results$ngrams) &&
          "2gram" %in% names(values$analysis_results$ngrams)
      ) {
        top_bigrams <- values$analysis_results$ngrams$`2gram` %>%
          slice_head(n = 15) %>%
          pull(ngram)
      }

      # Get top trigrams if available
      top_trigrams <- character(0)
      if (
        !is.null(values$analysis_results$ngrams) &&
          "3gram" %in% names(values$analysis_results$ngrams)
      ) {
        top_trigrams <- values$analysis_results$ngrams$`3gram` %>%
          slice_head(n = 10) %>%
          pull(ngram)
      }

      # Combine all choices with labels
      all_choices <- c(top_words, top_bigrams, top_trigrams)

      # Create named list for optgroups
      choices_list <- list(
        "Top Words (1-gram)" = top_words
      )

      if (length(top_bigrams) > 0) {
        choices_list[["Top Bigrams (2-gram)"]] <- top_bigrams
      }

      if (length(top_trigrams) > 0) {
        choices_list[["Top Trigrams (3-gram)"]] <- top_trigrams
      }

      updateSelectizeInput(
        session,
        "trend_words",
        choices = choices_list,
        selected = NULL,
        server = TRUE
      )
    }
  })

  # Check if sections are available
  output$sections_available <- reactive({
    if (!is.null(values$pdf_sections) && length(values$pdf_sections) > 0) {
      section_names <- setdiff(
        names(values$pdf_sections),
        c("Full_text", "References")
      )
      return(length(section_names) > 0)
    }
    return(FALSE)
  })
  outputOptions(output, "sections_available", suspendWhenHidden = FALSE)

  # Check if trends are available
  output$trends_available <- reactive({
    return(
      !is.null(values$word_trends_data) && nrow(values$word_trends_data) > 0
    )
  })
  outputOptions(output, "trends_available", suspendWhenHidden = FALSE)

  # Update word trends when button is clicked
  observeEvent(input$update_trends, {
    # Validate inputs
    if (is.null(values$pdf_text) || nchar(values$pdf_text) == 0) {
      showNotification(
        "Please extract text from PDF first.",
        type = "warning",
        duration = 3
      )
      return()
    }

    if (is.null(input$trend_words) || length(input$trend_words) == 0) {
      showNotification(
        "Please select at least one word to track.",
        type = "warning",
        duration = 3
      )
      return()
    }

    if (length(input$trend_words) > 10) {
      showNotification(
        "Maximum 10 words allowed. Please deselect some words.",
        type = "warning",
        duration = 3
      )
      return()
    }

    tryCatch(
      {
        # Determine use_sections parameter
        use_sections_param <- switch(
          input$segmentation_type,
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE
        )

        # Get number of segments
        n_segs <- if (!is.null(input$n_segments)) input$n_segments else 10

        # Determine which text to use
        # If user explicitly chooses segments, use Full_text only
        # Otherwise, use the sections structure
        text_input <- if (input$segmentation_type == "segments") {
          values$pdf_text # Just the full text as string
        } else {
          values$pdf_sections # List with sections
        }

        # Calculate word distribution
        values$word_trends_data <- calculate_word_distribution(
          text = text_input,
          selected_words = input$trend_words,
          use_sections = use_sections_param,
          n_segments = n_segs,
          remove_stopwords = FALSE # Don't remove stopwords for selected words
        )

        # Check if sections were requested but not available
        sections_used <- attr(values$word_trends_data, "use_sections")
        if (input$segmentation_type == "sections" && !sections_used) {
          showNotification(
            "Sections not available. Using equal-length segments instead.",
            type = "warning",
            duration = 4
          )
        }

        showNotification(
          "Word distribution calculated successfully!",
          type = "message",
          duration = 2
        )
      },
      error = function(e) {
        showNotification(
          paste("Error calculating word distribution:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Render word trends plot
  output$word_trends_plot <- renderPlotly({
    req(values$word_trends_data)

    tryCatch(
      {
        plot_word_distribution(
          word_distribution_data = values$word_trends_data,
          plot_type = input$trend_plot_type,
          smooth = TRUE,
          show_points = input$trend_show_points,
          colors = NULL # Use automatic colors
        )
      },
      error = function(e) {
        showNotification(
          paste("Error creating plot:", e$message),
          type = "error",
          duration = 5
        )
        return(NULL)
      }
    )
  })

  # Render word trends statistics table
  output$word_trends_table <- renderUI({ renderBibliobox(
    df = reactive({
      req(values$word_trends_data)

      # Create summary statistics
      stats_table <- values$word_trends_data %>%
        group_by(word) %>%
        summarise(
          `Total Occurrences` = sum(count),
          `Avg Frequency` = paste0(
            round(mean(relative_frequency) * 100, 3),
            "%"
          ),
          `Min Frequency` = paste0(
            round(min(relative_frequency) * 100, 3),
            "%"
          ),
          `Max Frequency` = paste0(
            round(max(relative_frequency) * 100, 3),
            "%"
          ),
          `Std Dev` = round(sd(relative_frequency) * 100, 3),
          `Peak Segment` = segment_name[which.max(relative_frequency)],
          .groups = "drop"
        ) %>%
        arrange(desc(`Total Occurrences`))

      # Apply HTML styling to word column (replaces formatStyle)
      stats_table <- stats_table %>%
        mutate(
          word = paste0(
            '<span style="font-weight: bold; color: #2E86AB;">',
            word,
            '</span>'
          )
        )

      stats_table
    }),
    title = "Word Trends Statistics",
    nrow = 10,
    filter = "none",
    scrollX = TRUE,
    escape = FALSE, # Important: allows HTML in cells
    round = 3
  ) })

  # Download handler for word trends
  output$download_word_trends <- downloadHandler(
    filename = function() {
      paste0("word_trends_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (!is.null(values$word_trends_data)) {
        # Prepare data for export
        export_data <- values$word_trends_data %>%
          select(
            word,
            segment_id,
            segment_name,
            count,
            relative_frequency,
            percentage,
            total_words
          ) %>%
          arrange(word, segment_id)

        write.csv(export_data, file, row.names = FALSE)
      } else {
        write.csv(
          data.frame(message = "No word trends data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 4: CITATION CLUSTERS
  # ===========================================

  # Check if cluster data is available
  output$clusters_available <- reactive({
    return(
      !is.null(values$cluster_description) &&
        !is.null(values$cluster_description$cluster_descriptions) &&
        nrow(values$cluster_description$cluster_descriptions) > 0
    )
  })
  outputOptions(output, "clusters_available", suspendWhenHidden = FALSE)

  # Cluster Summary Table
  output$cluster_summary_table <- renderUI({
    req(values$cluster_description)
    summary_data <- values$cluster_description$cluster_summary
    section_colors <- values$analysis_results$section_colors

    cards <- lapply(seq_len(nrow(summary_data)), function(i) {
      row <- summary_data[i, ]
      sec_color <- if (
        !is.null(section_colors) && row$section %in% names(section_colors)
      ) {
        section_colors[row$section]
      } else {
        "#2E86AB"
      }

      # Split top terms and create styled badges
      terms <- trimws(strsplit(row$top_terms, ",")[[1]])
      term_badges <- lapply(terms, function(term) {
        tags$span(
          term,
          style = paste0(
            "display: inline-block; padding: 3px 8px; margin: 2px;",
            "background-color: ", sec_color, "22;",
            "color: ", sec_color, ";",
            "border: 1px solid ", sec_color, "55;",
            "border-radius: 12px; font-size: 12px;"
          )
        )
      })

      div(
        style = paste0(
          "border-left: 4px solid ", sec_color, ";",
          "padding: 12px 15px; margin-bottom: 12px;",
          "background-color: #fafafa; border-radius: 0 5px 5px 0;"
        ),
        div(
          style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 8px;",
          h5(
            row$section,
            style = paste0(
              "margin: 0; color: ", sec_color, "; font-weight: bold;"
            )
          ),
          tags$span(
            paste0(row$n_references, " references"),
            style = paste0(
              "padding: 2px 10px; border-radius: 10px;",
              "background-color: ", sec_color, ";",
              "color: white; font-size: 12px; font-weight: 500;"
            )
          )
        ),
        div(
          style = "margin-top: 5px;",
          tags$strong("Top terms: ", style = "font-size: 12px; color: #666;"),
          do.call(tagList, term_badges)
        )
      )
    })

    do.call(tagList, cards)
  })

  # TF-IDF Bar Chart - dynamic height container
  # TF-IDF Bar Chart - dynamic height container (2-column grid)
  output$cluster_tfidf_bars_ui <- renderUI({
    req(values$cluster_description)
    descriptions <- values$cluster_description$cluster_descriptions
    n_sections <- length(unique(descriptions$section))
    n_rows <- ceiling(n_sections / 2)
    plot_height <- max(450, n_rows * 10 * 20 + 80)
    plotlyOutput("cluster_tfidf_bars", height = paste0(plot_height, "px"))
  })

  output$cluster_tfidf_bars <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    descriptions <- cluster_desc$cluster_descriptions

    # Preserve section order
    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% unique(descriptions$section)]
    } else {
      all_sections <- unique(descriptions$section)
    }
    n_sections <- length(all_sections)
    n_cols <- 2
    n_rows <- ceiling(n_sections / n_cols)

    # Build subplots for each section
    subplot_list <- list()
    for (i in seq_along(all_sections)) {
      sec <- all_sections[i]
      sec_data <- descriptions %>%
        dplyr::filter(section == sec) %>%
        dplyr::arrange(tf_idf)

      sec_color <- if (
        !is.null(section_colors) && sec %in% names(section_colors)
      ) {
        section_colors[sec]
      } else {
        "#2E86AB"
      }

      ngram_type <- ifelse(sec_data$ngram_size == 1, "unigram", "bigram")
      hover_text <- paste0(
        "<b>Term:</b> ", sec_data$ngram, "<br>",
        "<b>TF-IDF:</b> ", round(sec_data$tf_idf, 4), "<br>",
        "<b>Count:</b> ", sec_data$n, "<br>",
        "<b>Type:</b> ", ngram_type
      )

      p <- plotly::plot_ly(
        data = sec_data,
        y = ~factor(ngram, levels = ngram),
        x = ~tf_idf,
        type = "bar",
        orientation = "h",
        marker = list(color = sec_color),
        text = hover_text,
        hovertemplate = "%{text}<extra></extra>",
        name = sec,
        showlegend = FALSE
      ) %>%
        plotly::layout(
          yaxis = list(
            title = "",
            tickfont = list(size = 10),
            categoryorder = "array",
            categoryarray = sec_data$ngram
          )
        )

      subplot_list[[i]] <- p
    }

    # Pad with empty plots if odd number of sections
    if (n_sections %% n_cols != 0) {
      empty_p <- plotly::plot_ly(type = "bar") %>%
        plotly::layout(
          xaxis = list(visible = FALSE),
          yaxis = list(visible = FALSE)
        )
      for (k in seq_len(n_cols - (n_sections %% n_cols))) {
        subplot_list[[length(subplot_list) + 1]] <- empty_p
      }
    }

    bar_plot_height <- max(450, n_rows * 10 * 20 + 80)

    fig <- plotly::subplot(
      subplot_list,
      nrows = n_rows,
      shareX = FALSE,
      shareY = FALSE,
      titleY = TRUE,
      margin = 0.06
    )
    fig$height <- bar_plot_height

    # Build annotations for section titles
    section_annotations <- list()
    for (i in seq_along(all_sections)) {
      row_idx <- ceiling(i / n_cols)
      col_idx <- ((i - 1) %% n_cols) + 1
      x_pos <- (col_idx - 1) / n_cols + 0.5 / n_cols
      y_pos <- 1 - (row_idx - 1) / n_rows + 0.005

      ann_color <- if (
        !is.null(section_colors) && all_sections[i] %in% names(section_colors)
      ) {
        section_colors[all_sections[i]]
      } else {
        "#2E86AB"
      }

      section_annotations[[i]] <- list(
        text = paste0("<b>", all_sections[i], "</b>"),
        xref = "paper", yref = "paper",
        x = x_pos, y = y_pos,
        xanchor = "center", yanchor = "bottom",
        showarrow = FALSE,
        font = list(size = 11, color = ann_color)
      )
    }

    fig %>%
      plotly::layout(
        annotations = section_annotations,
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 120, r = 20, t = 30, b = 40)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "tfidf_bars",
          height = bar_plot_height,
          width = 1200,
          scale = 2
        )
      )
  })

  # TF-IDF Heatmap
  output$cluster_tfidf_heatmap <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    descriptions <- cluster_desc$cluster_descriptions

    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% unique(descriptions$section)]
    } else {
      all_sections <- unique(descriptions$section)
    }
    n_sections <- length(all_sections)

    # Get top terms for heatmap
    top_terms_per_section <- descriptions %>%
      dplyr::group_by(section) %>%
      dplyr::slice_head(n = max(1, floor(20 / n_sections))) %>%
      dplyr::ungroup()

    heatmap_terms <- unique(top_terms_per_section$ngram)
    if (length(heatmap_terms) < 20) {
      remaining <- descriptions %>%
        dplyr::filter(!ngram %in% heatmap_terms) %>%
        dplyr::arrange(dplyr::desc(tf_idf)) %>%
        dplyr::pull(ngram) %>%
        unique()
      heatmap_terms <- unique(c(heatmap_terms, remaining))[
        seq_len(min(20, length(unique(c(heatmap_terms, remaining)))))
      ]
    }

    # Build heatmap matrix
    heatmap_data <- expand.grid(
      ngram = heatmap_terms,
      section = all_sections,
      stringsAsFactors = FALSE
    ) %>%
      dplyr::left_join(
        descriptions %>% dplyr::select(ngram, section, tf_idf),
        by = c("ngram", "section")
      ) %>%
      dplyr::mutate(tf_idf = ifelse(is.na(tf_idf), 0, tf_idf))

    heatmap_matrix <- matrix(0, nrow = length(heatmap_terms), ncol = n_sections)
    rownames(heatmap_matrix) <- heatmap_terms
    colnames(heatmap_matrix) <- all_sections

    for (r in seq_along(heatmap_terms)) {
      for (cc in seq_along(all_sections)) {
        val <- heatmap_data$tf_idf[
          heatmap_data$ngram == heatmap_terms[r] &
            heatmap_data$section == all_sections[cc]
        ]
        if (length(val) > 0) heatmap_matrix[r, cc] <- val[1]
      }
    }

    hover_matrix <- matrix("", nrow = length(heatmap_terms), ncol = n_sections)
    for (r in seq_along(heatmap_terms)) {
      for (cc in seq_along(all_sections)) {
        hover_matrix[r, cc] <- paste0(
          "<b>Term:</b> ", heatmap_terms[r], "<br>",
          "<b>Section:</b> ", all_sections[cc], "<br>",
          "<b>TF-IDF:</b> ", round(heatmap_matrix[r, cc], 4)
        )
      }
    }

    # Truncate long section labels for x-axis display
    trunc_sections <- ifelse(nchar(all_sections) > 30,
                             paste0(substr(all_sections, 1, 27), "..."),
                             all_sections)

    plotly::plot_ly(
      x = trunc_sections,
      y = heatmap_terms,
      z = heatmap_matrix,
      type = "heatmap",
      colorscale = list(c(0, "white"), c(1, "#08519c")),
      text = hover_matrix,
      hovertemplate = "%{text}<extra></extra>",
      colorbar = list(title = "TF-IDF")
    ) %>%
      plotly::layout(
        xaxis = list(
          title = list(
            text = "Section",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickangle = -45,
          tickfont = list(size = 11),
          categoryorder = "array",
          categoryarray = trunc_sections
        ),
        yaxis = list(
          title = list(
            text = "Term",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickfont = list(size = 10),
          autorange = "reversed"
        ),
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 140, r = 40, t = 20, b = 140)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "tfidf_heatmap",
          height = 600,
          width = 1000,
          scale = 2
        )
      )
  })

  # References per Section Bar Chart
  output$cluster_refs_per_section <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    summary_data <- cluster_desc$cluster_summary

    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% summary_data$section]
    } else {
      all_sections <- summary_data$section
    }

    ref_counts <- summary_data %>%
      dplyr::mutate(section = factor(section, levels = all_sections)) %>%
      dplyr::arrange(section) %>%
      dplyr::mutate(section = as.character(section))

    bar_colors <- sapply(ref_counts$section, function(s) {
      if (!is.null(section_colors) && s %in% names(section_colors)) {
        section_colors[s]
      } else {
        "#CCCCCC"
      }
    })

    # Truncate long section labels for x-axis display
    trunc_ref_sections <- ifelse(nchar(ref_counts$section) > 30,
                                 paste0(substr(ref_counts$section, 1, 27), "..."),
                                 ref_counts$section)
    trunc_all_sections <- ifelse(nchar(all_sections) > 30,
                                 paste0(substr(all_sections, 1, 27), "..."),
                                 all_sections)

    hover_refs <- paste0(
      "<b>Section:</b> ", ref_counts$section, "<br>",
      "<b>References:</b> ", ref_counts$n_references
    )

    plotly::plot_ly(
      data = ref_counts,
      x = trunc_ref_sections,
      y = ~n_references,
      type = "bar",
      marker = list(color = bar_colors),
      text = ~n_references,
      textposition = "outside",
      textfont = list(size = 12, color = "#333"),
      hovertext = hover_refs,
      hovertemplate = "%{hovertext}<extra></extra>"
    ) %>%
      plotly::layout(
        xaxis = list(
          title = list(
            text = "Section",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickangle = -45,
          gridcolor = "#e0e0e0",
          categoryorder = "array",
          categoryarray = trunc_all_sections
        ),
        yaxis = list(
          title = list(
            text = "Number of References",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          gridcolor = "#e0e0e0"
        ),
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 60, r = 40, t = 20, b = 140)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "references_per_section",
          height = 600,
          width = 1000,
          scale = 2
        )
      )
  })

  # Detailed cluster references table
  output$cluster_references_detail <- renderUI({
    req(values$cluster_description)

    cluster_refs <- values$cluster_description$cluster_references
    section_colors <- values$analysis_results$section_colors
    section_order <- values$cluster_description$section_order

    # Order sections
    sections <- if (!is.null(section_order)) {
      section_order[section_order %in% unique(cluster_refs$section)]
    } else {
      unique(cluster_refs$section)
    }

    section_panels <- lapply(sections, function(sec) {
      sec_refs <- cluster_refs %>%
        dplyr::filter(section == sec) %>%
        dplyr::distinct(ref_full_text) %>%
        dplyr::arrange(ref_full_text)

      sec_color <- if (
        !is.null(section_colors) && sec %in% names(section_colors)
      ) {
        section_colors[sec]
      } else {
        "#2E86AB"
      }

      ref_items <- lapply(seq_len(nrow(sec_refs)), function(j) {
        div(
          style = "padding: 6px 10px; border-bottom: 1px solid #eee; font-size: 13px; color: #444;",
          tags$span(
            paste0(j, "."),
            style = "font-weight: bold; color: #999; margin-right: 8px;"
          ),
          sec_refs$ref_full_text[j]
        )
      })

      div(
        style = "margin-bottom: 15px;",
        div(
          style = paste0(
            "padding: 8px 15px; border-left: 4px solid ", sec_color, ";",
            "background-color: ", sec_color, "11; cursor: pointer;"
          ),
          onclick = paste0(
            "$(this).next().slideToggle(200);",
            "$(this).find('.toggle-icon').toggleClass('fa-chevron-down fa-chevron-right');"
          ),
          tags$span(
            class = "toggle-icon fa fa-chevron-down",
            style = paste0("color: ", sec_color, "; margin-right: 8px;")
          ),
          tags$strong(sec, style = paste0("color: ", sec_color, ";")),
          tags$span(
            paste0(" (", nrow(sec_refs), " references)"),
            style = "color: #888; font-size: 12px;"
          )
        ),
        div(
          style = "border: 1px solid #eee; border-top: none;",
          do.call(tagList, ref_items)
        )
      )
    })

    do.call(tagList, section_panels)
  })

  # Download cluster data
  output$download_clusters <- downloadHandler(
    filename = function() {
      paste0("citation_clusters_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (!is.null(values$cluster_description)) {
        write.csv(
          values$cluster_description$cluster_descriptions,
          file,
          row.names = FALSE
        )
      } else {
        write.csv(
          data.frame(message = "No cluster data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 5: REFERENCES
  # ===========================================

  # Check if references are available
  output$references_available <- reactive({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      return(nrow(values$analysis_results$parsed_references) > 0)
    }
    return(FALSE)
  })
  outputOptions(output, "references_available", suspendWhenHidden = FALSE)

  # # Total references count
  output$total_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      format(nrow(values$analysis_results$parsed_references), big.mark = ",")
    } else {
      "0"
    }
  })

  # # PDF references count
  output$pdf_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      refs <- values$analysis_results$parsed_references
      pdf_refs <- sum(tolower(refs$ref_source) == "pdf", na.rm = TRUE)
      format(pdf_refs, big.mark = ",")
    } else {
      "0"
    }
  })

  # # Crossref references count
  output$crossref_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      refs <- values$analysis_results$parsed_references
      crossref_refs <- sum(tolower(refs$ref_source) == "crossref", na.rm = TRUE)
      format(crossref_refs, big.mark = ",")
    } else {
      "0"
    }
  })

  # Openalex references count
  output$openalex_refs <- renderText({
    req(values$references_oa)

    if (is.null(values$references_oa) || nrow(values$references_oa) == 0) {
      return("0")
    }

    nrow(values$references_oa)
  })

  # References HTML display
  output$references_html <- renderUI({
    req(values$analysis_results)
    req(values$analysis_results$parsed_references)

    refs <- values$analysis_results$parsed_references

    # Apply search filter
    if (!is.null(input$reference_search) && nzchar(input$reference_search)) {
      search_term <- tolower(input$reference_search)

      refs <- refs %>%
        filter(
          if_else(
            !is.na(ref_full_text),
            str_detect(tolower(ref_full_text), fixed(search_term)),
            FALSE
          ) |
            if_else(
              !is.na(ref_authors),
              str_detect(tolower(ref_authors), fixed(search_term)),
              FALSE
            ) |
            if_else(
              !is.na(ref_year),
              str_detect(tolower(ref_year), fixed(search_term)),
              FALSE
            ) |
            if_else(
              !is.na(doi),
              str_detect(tolower(doi), fixed(search_term)),
              FALSE
            )
        )
    }

    if (nrow(refs) == 0) {
      return(
        div(
          style = "text-align: center; padding: 40px; color: #999;",
          icon("search", style = "font-size: 24px; margin-bottom: 10px;"),
          h4("No references match your search"),
          p("Try different search terms.")
        )
      )
    }

    # Create HTML for each reference
    reference_items <- lapply(1:nrow(refs), function(i) {
      ref <- refs[i, ]
      # Get reference text
      ref_text <- if (!is.na(ref$ref_full_text) && nzchar(ref$ref_full_text)) {
        ref$ref_full_text
      } else {
        "[Reference text not available]"
      }

      # Check if OpenAlex data is available for this reference
      has_oa_data <- FALSE
      oa_button <- NULL

      if (
        !is.null(values$references_oa) && !is.na(ref$doi) && nzchar(ref$doi)
      ) {
        # Cerca corrispondenza nel dataset OpenAlex
        ref_doi_clean <- tolower(trimws(ref$doi))

        # Prova diverse varianti del DOI
        oa_match_idx <- which(
          tolower(gsub("https://doi.org/", "", values$references_oa$doi)) ==
            ref_doi_clean |
            tolower(values$references_oa$doi) ==
              paste0("https://doi.org/", ref_doi_clean)
        )

        if (length(oa_match_idx) > 0) {
          has_oa_data <- TRUE

          oa_button <- tags$button(
            class = "btn btn-info btn-xs",
            style = "float: right; margin-left: 10px;",
            onclick = sprintf("showOADetails(%d)", i),
            icon("info-circle"),
            " View Details"
          )
        }
      }

      # Determine source badge
      source_badge <- if (
        !is.na(ref$ref_source) && tolower(ref$ref_source) == "crossref"
      ) {
        tags$span(
          class = "label label-success",
          style = "font-size: 11px; margin-left: 10px;",
          icon("cloud-download-alt"),
          " Crossref"
        )
      } else {
        tags$span(
          class = "label label-info",
          style = "font-size: 11px; margin-left: 10px;",
          icon("file-pdf"),
          " PDF"
        )
      }

      # OpenAlex badge if available
      oa_badge <- if (has_oa_data) {
        tags$span(
          class = "label label-primary",
          style = "font-size: 11px; margin-left: 5px;",
          icon("database"),
          " OpenAlex"
        )
      }

      # DOI link if available
      doi_link <- NULL
      if ("doi" %in% names(ref) && !is.na(ref$doi) && nzchar(ref$doi)) {
        doi_link <- tags$div(
          style = "margin-top: 8px; font-size: 12px;",
          icon("link", style = "color: #3498db;"),
          tags$a(
            href = paste0("https://doi.org/", ref$doi),
            target = "_blank",
            ref$doi,
            style = "color: #3498db; margin-left: 5px;"
          )
        )
      }

      # Author and year display
      author_year_info <- NULL
      if (!is.na(ref$ref_first_author) && !is.na(ref$ref_year)) {
        n_auth <- if (!is.na(ref$n_authors) && ref$n_authors > 1) {
          paste0(" et al. (", ref$n_authors, " authors)")
        } else {
          ""
        }

        author_year_info <- tags$div(
          style = "margin-top: 8px; font-size: 12px; color: #666;",
          icon("user", style = "color: #3498db;"),
          tags$span(
            style = "margin-left: 5px;",
            paste0(ref$ref_first_author, n_auth, " • ", ref$ref_year)
          )
        )
      }

      # Citation count if available
      citation_info <- NULL
      if (
        "citation_count" %in%
          names(ref) &&
          !is.na(ref$citation_count) &&
          ref$citation_count > 0
      ) {
        citation_info <- tags$span(
          style = "margin-left: 15px; color: #e67e22; font-size: 12px;",
          icon("quote-right"),
          sprintf(
            " Cited %d time%s in text",
            ref$citation_count,
            ifelse(ref$citation_count > 1, "s", "")
          )
        )
      }

      div(
        class = "reference-item",
        `data-ref-index` = i,
        style = "padding: 15px; margin-bottom: 15px; border-left: 4px solid #2E86AB; background-color: #fafafa; border-radius: 5px; box-shadow: 0 1px 3px rgba(0,0,0,0.1); position: relative;",

        # Reference number, badges and OA button
        div(
          style = "margin-bottom: 10px;",
          tags$span(
            style = "font-weight: bold; color: #2E86AB; font-size: 16px;",
            paste0("[", i, "]")
          ),
          source_badge,
          oa_badge,
          citation_info,
          oa_button
        ),

        # Full reference text
        div(
          style = "font-family: 'Georgia', serif; font-size: 14px; line-height: 1.6; color: #333; clear: both;",
          ref_text
        ),

        # Author and year info
        author_year_info,

        # DOI link
        doi_link
      )
    })

    tagList(
      tags$style(HTML(
        "
      .reference-item:hover {
        background-color: #f0f8ff !important;
        transition: background-color 0.2s ease;
      }
      .oa-document-summary .info-box {
        display: flex;
        align-items: center;
        border-radius: 4px;
        padding: 10px;
        color: white;
      }
      .oa-document-summary .info-box-icon {
        font-size: 32px;
        flex: 0 0 70px;
        text-align: center;
      }
      .oa-document-summary .info-box-content {
        flex: 1;
      }
      .oa-document-summary .info-box-text {
        display: block;
        font-size: 12px;
        text-transform: uppercase;
        opacity: 0.9;
      }
      .oa-document-summary .info-box-number {
        display: block;
        font-size: 20px;
        font-weight: bold;
        margin-top: 5px;
      }
    "
      )),
      reference_items
    )
  })

  # Download handler for references
  output$download_references <- downloadHandler(
    filename = function() {
      paste0("references_", Sys.Date(), ".txt")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$parsed_references)
      ) {
        refs <- values$analysis_results$parsed_references

        # Create formatted text
        ref_text <- paste0(
          "BIBLIOGRAPHY\n",
          "============\n",
          "Generated: ",
          Sys.time(),
          "\n",
          "Total references: ",
          nrow(refs),
          "\n",
          "From PDF: ",
          sum(tolower(refs$ref_source) == "pdf", na.rm = TRUE),
          "\n",
          "From Crossref: ",
          sum(tolower(refs$ref_source) == "crossref", na.rm = TRUE),
          "\n\n"
        )

        for (i in 1:nrow(refs)) {
          ref <- refs[i, ]

          source_label <- if (
            !is.na(ref$ref_source) && tolower(ref$ref_source) == "crossref"
          ) {
            "[Crossref]"
          } else {
            "[PDF]"
          }

          ref_text <- paste0(
            ref_text,
            "[",
            i,
            "] ",
            source_label,
            "\n",
            ref$ref_full_text,
            "\n"
          )

          if (!is.na(ref$ref_first_author) && !is.na(ref$ref_year)) {
            n_auth <- if (!is.na(ref$n_authors) && ref$n_authors > 1) {
              paste0(" (", ref$n_authors, " authors)")
            } else {
              ""
            }
            ref_text <- paste0(
              ref_text,
              "Author: ",
              ref$ref_first_author,
              n_auth,
              " • Year: ",
              ref$ref_year,
              "\n"
            )
          }

          if ("doi" %in% names(ref) && !is.na(ref$doi) && nzchar(ref$doi)) {
            ref_text <- paste0(ref_text, "DOI: ", ref$doi, "\n")
          }

          if ("citation_count" %in% names(ref) && !is.na(ref$citation_count)) {
            ref_text <- paste0(
              ref_text,
              "Cited ",
              ref$citation_count,
              " times in text\n"
            )
          }

          ref_text <- paste0(ref_text, "\n")
        }

        writeLines(ref_text, file)
      } else {
        writeLines("No references available", file)
      }
    }
  )

  # ===========================================
  # TAB: BIBLIOAI SUMMARY - SERVER LOGIC
  # ===========================================

  # ===========================================
  # BIBLIOAI SUMMARY INITIALIZATION
  # ===========================================

  # Initialize API key from environment on startup (run once)
  observeEvent(TRUE, {
    api_key_env <- Sys.getenv("GEMINI_API_KEY")
    if (nzchar(api_key_env) && is.null(values$gemini_api_key)) {
      values$gemini_api_key <- api_key_env
    }
  }, once = TRUE)

  # ===========================================
  # API STATUS DISPLAY
  # ===========================================

  output$gemini_api_status <- renderUI({
    api_key <- values$gemini_api_key

    if (is.null(api_key) || !nzchar(api_key)) {
      tags$span(
        class = "label label-danger",
        style = "font-size: 12px; padding: 6px 12px;",
        icon("exclamation-circle"),
        " API Key Not Configured"
      )
    } else {
      tags$span(
        class = "label label-success",
        style = "font-size: 12px; padding: 6px 12px;",
        icon("check-circle"),
        " API Key Configured"
      )
    }
  })

  # ===========================================
  # SUMMARY TYPE DESCRIPTION
  # ===========================================

  output$summary_type_description <- renderUI({
    req(input$summary_type)

    descriptions <- list(
      short_abstract = "A concise, one-paragraph abstract (around 250 words) covering objectives, methods, findings, and conclusions.",
      narrative_abstract = "An engaging narrative (500-600 words) that tells the story of the research from problem to implications.",
      imrad_summary = "A structured summary following the IMRaD format: Introduction, Methods, Results, and Discussion.",
      thematic_bibliography = "Groups the references by research themes and provides descriptions of each thematic area.",
      research_questions = "Extracts research questions, context, and motivation behind the study.",
      background_literature = "Summarizes the theoretical framework and previous research that grounds this study.",
      methods_summary = "Detailed breakdown of research design, data collection, analysis techniques, and tools used.",
      implications = "Focuses on theoretical contributions and practical applications derived from the findings.",
      list_tables_figures = "Complete list of all tables and figures with their captions as they appear in the document."
    )

    description <- descriptions[[input$summary_type]]

    div(
      style = "background-color: #f0f8ff; padding: 12px; border-radius: 4px; border-left: 3px solid #3498db;",
      tags$small(
        icon("info-circle", style = "color: #3498db; margin-right: 5px;"),
        description,
        style = "color: #555; line-height: 1.6;"
      )
    )
  })

  # ===========================================
  # GENERATE SUMMARY
  # ===========================================

  observeEvent(input$generate_summary, {
    # Validate PDF file
    if (is.null(input$pdf_file)) {
      showNotification(
        "Please upload a PDF file first.",
        type = "warning",
        duration = 4
      )
      return()
    }

    # Validate API key
    api_key <- Sys.getenv("GEMINI_API_KEY")
    if (!nzchar(api_key)) {
      showNotification(
        HTML(
          "<strong>API Key Required</strong><br/>
           Please configure your Gemini API key in Biblioshiny settings.<br/>
           You can get a free API key at: <a href='https://aistudio.google.com/apikey' target='_blank'>Google AI Studio</a>"
        ),
        type = "error",
        duration = 8
      )
      return()
    }

    # Get model from Biblioshiny settings or use default
    model <- values$gemini_api_model
    if (is.null(model) || !nzchar(model)) {
      model <- "2.0-flash-lite"
    }

    # Use "huge" as default output size
    output_size <- "huge"

    tryCatch(
      {
        # Generate prompt
        prompt <- get_article_summary_prompt(input$summary_type)

        # Show progress notification
        progress_id <- showNotification(
          "Sending request to Gemini AI...",
          type = "message",
          duration = NULL,
          closeButton = FALSE
        )

        # Call Gemini AI
        result <- gemini_ai(
          docs = input$pdf_file$datapath,
          prompt = prompt,
          model = model,
          api_key = api_key,
          outputSize = output_size,
          retry_503 = 5
        )

        # Remove progress notification
        removeNotification(progress_id)

        # Check if result is an error message
        if (is.character(result) && length(result) == 1) {
          if (
            grepl("^❌", result) || grepl("Error", result, ignore.case = TRUE)
          ) {
            showNotification(
              HTML(paste0(
                "<strong>AI Request Failed</strong><br/>",
                gsub("❌ ", "", result)
              )),
              type = "error",
              duration = 10
            )
            return()
          }
        }

        # Store results
        values$biblioai_summary <- result
        values$biblioai_summary_type <- input$summary_type
        values$biblioai_summary_timestamp <- Sys.time()

        # Get summary type label
        summary_labels <- list(
          short_abstract = "Short Abstract",
          narrative_abstract = "Narrative Abstract",
          imrad_summary = "IMRaD Summary",
          thematic_bibliography = "Thematic Bibliography",
          research_questions = "Research Questions & Context",
          background_literature = "Background & Literature",
          methods_summary = "Methods Summary",
          implications = "Implications & Conclusions",
          list_tables_figures = "Tables & Figures List"
        )

        summary_label <- summary_labels[[input$summary_type]]

        showNotification(
          HTML(paste0(
            "<strong>Summary Generated!</strong><br/>",
            "Type: ",
            summary_label
          )),
          type = "message",
          duration = 5
        )
      },
      error = function(e) {
        removeNotification(progress_id)

        showNotification(
          HTML(paste0(
            "<strong>Error Generating Summary</strong><br/>",
            e$message
          )),
          type = "error",
          duration = 8
        )

        cat("BiblioAI Summary Error:\n")
        print(e)
      }
    )
  })

  # ===========================================
  # SUMMARY AVAILABILITY
  # ===========================================

  output$summary_available <- reactive({
    return(!is.null(values$biblioai_summary))
  })
  outputOptions(output, "summary_available", suspendWhenHidden = FALSE)

  # ===========================================
  # SUMMARY DISPLAY
  # ===========================================

  output$summary_type_display <- renderText({
    req(values$biblioai_summary_type)

    summary_labels <- list(
      short_abstract = "Short Abstract (250 words)",
      narrative_abstract = "Narrative Abstract (500-600 words)",
      imrad_summary = "IMRaD Structure Summary",
      thematic_bibliography = "Thematic Bibliography",
      research_questions = "Research Questions & Context",
      background_literature = "Background & Literature Review",
      methods_summary = "Methodology Summary",
      implications = "Implications & Conclusions",
      list_tables_figures = "Tables & Figures List"
    )

    summary_labels[[values$biblioai_summary_type]]
  })

  output$summary_timestamp <- renderText({
    req(values$biblioai_summary_timestamp)
    format(values$biblioai_summary_timestamp, "%Y-%m-%d %H:%M:%S")
  })

  output$summary_content_display <- renderUI({
    req(values$biblioai_summary)

    # Combine multiple text elements if result is a vector
    summary_text <- if (
      is.character(values$biblioai_summary) &&
        length(values$biblioai_summary) > 1
    ) {
      paste(values$biblioai_summary, collapse = "\n\n")
    } else {
      values$biblioai_summary
    }

    # Convert markdown-style headers to HTML for better display
    summary_html <- summary_text

    # Convert **text** to bold
    summary_html <- gsub(
      "\\*\\*([^*]+)\\*\\*",
      "<strong>\\1</strong>",
      summary_html
    )

    # Convert *text* to italic
    summary_html <- gsub("\\*([^*]+)\\*", "<em>\\1</em>", summary_html)

    # Convert ### headers to h4
    summary_html <- gsub(
      "^### (.+)$",
      "<h4 style='color: #2E86AB; margin-top: 20px; margin-bottom: 10px;'>\\1</h4>",
      summary_html,
      perl = TRUE
    )

    # Convert ## headers to h3
    summary_html <- gsub(
      "^## (.+)$",
      "<h3 style='color: #2E86AB; margin-top: 25px; margin-bottom: 10px;'>\\1</h3>",
      summary_html,
      perl = TRUE
    )

    # Convert # headers to h2
    summary_html <- gsub(
      "^# (.+)$",
      "<h2 style='color: #2E86AB; margin-top: 30px; margin-bottom: 15px;'>\\1</h2>",
      summary_html,
      perl = TRUE
    )

    # Convert bullet points
    summary_html <- gsub("^- (.+)$", "<li>\\1</li>", summary_html, perl = TRUE)
    summary_html <- gsub(
      "^\\* (.+)$",
      "<li>\\1</li>",
      summary_html,
      perl = TRUE
    )

    # Wrap consecutive <li> in <ul>
    summary_html <- gsub(
      "(<li>.*?</li>)\n(<li>)",
      "\\1\\2",
      summary_html,
      perl = TRUE
    )
    summary_html <- gsub(
      "(<li>.*?</li>)",
      "<ul style='margin-left: 20px;'>\\1</ul>",
      summary_html,
      perl = TRUE
    )

    # Convert line breaks to <br> but not inside lists
    summary_html <- gsub("\n(?!<)", "<br/>", summary_html, perl = TRUE)

    HTML(summary_html)
  })

  # ===========================================
  # DOWNLOAD SUMMARY
  # ===========================================

  output$download_summary <- downloadHandler(
    filename = function() {
      summary_type <- values$biblioai_summary_type
      if (is.null(summary_type)) {
        summary_type <- "summary"
      }
      paste0("biblioai_", summary_type, "_", Sys.Date(), ".txt")
    },
    content = function(file) {
      if (!is.null(values$biblioai_summary)) {
        # Create header
        header <- paste0(
          "====================================\n",
          "BiblioAI Summary\n",
          "====================================\n\n",
          "Summary Type: ",
          values$biblioai_summary_type,
          "\n",
          "Generated: ",
          format(values$biblioai_summary_timestamp, "%Y-%m-%d %H:%M:%S"),
          "\n",
          "Document: ",
          input$pdf_file$name,
          "\n\n",
          "====================================\n\n"
        )

        # Combine multiple text elements if result is a vector
        summary_text <- if (
          is.character(values$biblioai_summary) &&
            length(values$biblioai_summary) > 1
        ) {
          paste(values$biblioai_summary, collapse = "\n\n")
        } else {
          values$biblioai_summary
        }

        # Write to file
        writeLines(paste0(header, summary_text), file, useBytes = TRUE)
      } else {
        writeLines("No summary available", file)
      }
    }
  )

  # ===========================================
  # RESET SUMMARY ON NEW PDF OR RESET
  # ===========================================

  # Clear summary when new PDF is uploaded
  observeEvent(input$pdf_file, {
    values$biblioai_summary <- NULL
    values$biblioai_summary_type <- NULL
    values$biblioai_summary_timestamp <- NULL
  })

  # Clear summary on reset
  observeEvent(input$reset_analysis, {
    values$pdf_metadata <- NULL
    values$biblioai_summary <- NULL
    values$biblioai_summary_type <- NULL
    values$biblioai_summary_timestamp <- NULL

    # Reset UI inputs
    updateSelectInput(session, "summary_type", selected = "short_abstract")
  })

  # Clear metadata when new PDF is uploaded
  observeEvent(input$pdf_file, {
    values$pdf_metadata <- NULL
  })

  # ===========================================
  # OPENALEX MODAL HANDLER
  # ===========================================

  observeEvent(input$selected_oa_ref_index, {
    req(values$analysis_results$parsed_references)
    req(values$references_oa)
    req(input$selected_oa_ref_index)

    tryCatch(
      {
        ref_index <- input$selected_oa_ref_index
        ref <- values$analysis_results$parsed_references[ref_index, ]

        if (is.na(ref$doi) || !nzchar(ref$doi)) {
          session$sendCustomMessage(
            "updateOAModal",
            list(
              title = as.character(span(
                "Reference Details",
                style = "font-weight: 600; font-size: 20px;"
              )),
              content = as.character(div(
                style = "text-align: center; padding: 40px; color: #999;",
                icon(
                  "exclamation-triangle",
                  style = "font-size: 32px; margin-bottom: 15px; color: #f39c12;"
                ),
                h4("No DOI available"),
                p("Cannot fetch OpenAlex data without a DOI.")
              ))
            )
          )
          return()
        }

        # Trova i dati OpenAlex corrispondenti
        ref_doi_clean <- tolower(trimws(ref$doi))

        oa_match_idx <- which(
          tolower(gsub("https://doi.org/", "", values$references_oa$doi)) ==
            ref_doi_clean |
            tolower(values$references_oa$doi) ==
              paste0("https://doi.org/", ref_doi_clean)
        )

        if (length(oa_match_idx) == 0) {
          session$sendCustomMessage(
            "updateOAModal",
            list(
              title = as.character(span(
                "Reference Details",
                style = "font-weight: 600; font-size: 20px;"
              )),
              content = as.character(div(
                style = "text-align: center; padding: 40px; color: #999;",
                icon(
                  "database",
                  style = "font-size: 32px; margin-bottom: 15px; color: #e74c3c;"
                ),
                h4("OpenAlex data not found"),
                p(paste("Could not find OpenAlex data for DOI:", ref$doi))
              ))
            )
          )
          return()
        }

        # Prendi il primo match
        oa_data <- values$references_oa[oa_match_idx[1], ]

        # Estrai il titolo per l'header
        doc_title <- if (!is.na(oa_data$title) && nzchar(oa_data$title)) {
          oa_data$title
        } else if (
          !is.na(oa_data$display_name) && nzchar(oa_data$display_name)
        ) {
          oa_data$display_name
        } else {
          "Reference Details"
        }

        # Tronca il titolo se troppo lungo
        if (nchar(doc_title) > 120) {
          doc_title <- paste0(substr(doc_title, 1, 117), "...")
        }

        # Crea l'HTML per il modal
        modal_content <- create_oa_details_html(oa_data)

        # Aggiorna il modal con il titolo del documento
        session$sendCustomMessage(
          "updateOAModal",
          list(
            title = as.character(span(
              doc_title,
              style = "font-weight: 600; font-size: 20px; line-height: 1.4;"
            )),
            content = as.character(modal_content)
          )
        )
      },
      error = function(e) {
        session$sendCustomMessage(
          "updateOAModal",
          list(
            title = as.character(span(
              "Error",
              style = "font-weight: 600; font-size: 20px;"
            )),
            content = as.character(div(
              style = "text-align: center; padding: 40px; color: #e74c3c;",
              icon(
                "exclamation-circle",
                style = "font-size: 32px; margin-bottom: 15px;"
              ),
              h4("Error loading details"),
              p(paste("Error:", e$message))
            ))
          )
        )

        cat("Error in OA modal handler:", e$message, "\n")
        print(e)
      }
    )
  })

  # ===========================================
  # RESET FUNCTIONALITY
  # ===========================================

  observeEvent(input$reset_analysis, {
    values$pdf_text <- NULL
    values$analysis_results <- NULL
    values$network_plot <- NULL
    values$analysis_running <- FALSE
    values$readability_indices <- NULL
    values$word_trends_data <- NULL
    values$cluster_description <- NULL
    values$pdf_doi <- NULL
    values$citation_type_used <- NULL

    updateSelectizeInput(
      session,
      "trend_words",
      choices = NULL,
      selected = NULL
    )

    preview_visible(TRUE)

    tryCatch(
      {
        shinyjs::reset("pdf_file")
        updateTextInput(session, "pdf_doi_input", value = "")
        # MODIFIED: Reset to no selection
        updateRadioButtons(
          session,
          "citation_type_import",
          selected = character(0)
        )
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
        updateActionButton(session, "toggle_preview", "Hide Preview")
        updateTextInput(session, "context_search", value = "")
        updateSelectInput(session, "context_type_filter", selected = "")
      },
      error = function(e) {
        cat("Reset UI elements failed:", e$message, "\n")
      }
    )

    showNotification(
      "Analysis reset successfully!",
      type = "message",
      duration = 3
    )

    # Re-open the PDF import box
    session$sendCustomMessage('togglePdfImport', list(action = 'open'))
  })

  # ===========================================
  # EXCEL RESULTS DOWNLOAD
  # ===========================================

  # Download all results as Excel file
  output$download_all_results <- downloadHandler(
    filename = function() {
      paste0("content_analysis_results_", Sys.Date(), ".xlsx")
    },
    content = function(file) {
      # Verifica che ci siano risultati disponibili
      if (is.null(values$analysis_results)) {
        showNotification(
          "No analysis results available. Please run the analysis first.",
          type = "error",
          duration = 5
        )
        return()
      }

      tryCatch(
        {
          # Create workbook
          wb_content <- openxlsx::createWorkbook()

          # === SHEET 1: SUMMARY STATISTICS ===
          openxlsx::addWorksheet(wb_content, "Summary_Statistics")

          summary_data <- data.frame(
            Metric = c(
              "Total Words",
              "Total Citations",
              "Narrative Citations",
              "Parenthetical Citations",
              "Citation Density (per 1000 words)",
              "Lexical Diversity",
              "Total Characters",
              "Total Sentences",
              "Avg Words per Sentence"
            ),
            Value = c(
              values$analysis_results$summary$total_words_analyzed,
              values$analysis_results$summary$citations_extracted,
              values$analysis_results$summary$narrative_citations,
              values$analysis_results$summary$citations_extracted -
                values$analysis_results$summary$narrative_citations,
              round(
                values$analysis_results$summary$citation_density_per_1000_words,
                2
              ),
              round(values$analysis_results$summary$lexical_diversity, 4),
              values$analysis_results$text_analytics$basic_stats$total_characters,
              values$analysis_results$text_analytics$basic_stats$total_sentences,
              round(
                values$analysis_results$text_analytics$basic_stats$avg_words_per_sentence,
                2
              )
            ),
            stringsAsFactors = FALSE
          )

          openxlsx::writeData(
            wb_content,
            "Summary_Statistics",
            summary_data,
            startRow = 1
          )
          openxlsx::addStyle(
            wb_content,
            "Summary_Statistics",
            style = openxlsx::createStyle(
              textDecoration = "bold",
              fgFill = "#2E86AB",
              fontColour = "#FFFFFF"
            ),
            rows = 1,
            cols = 1:2,
            gridExpand = TRUE
          )
          openxlsx::setColWidths(
            wb_content,
            "Summary_Statistics",
            cols = 1:2,
            widths = c(35, 20)
          )

          # === SHEET 2: READABILITY INDICES ===
          if (!is.null(values$readability_indices)) {
            openxlsx::addWorksheet(wb_content, "Readability_Indices")

            readability_data <- data.frame(
              Index = c(
                "Flesch-Kincaid Grade Level",
                "Flesch Reading Ease",
                "Automated Readability Index (ARI)",
                "Gunning Fog Index",
                "Average Sentence Length (words)",
                "Number of Sentences",
                "Number of Words",
                "Number of Syllables",
                "Number of Complex Words",
                "Percentage Complex Words"
              ),
              Value = c(
                round(values$readability_indices$flesch_kincaid_grade, 2),
                round(values$readability_indices$flesch_reading_ease, 2),
                round(
                  values$readability_indices$automated_readability_index,
                  2
                ),
                round(values$readability_indices$gunning_fog_index, 2),
                round(values$readability_indices$avg_sentence_length, 2),
                values$readability_indices$n_sentences,
                values$readability_indices$n_words,
                values$readability_indices$n_syllables,
                values$readability_indices$n_complex_words,
                paste0(
                  round(values$readability_indices$pct_complex_words, 2),
                  "%"
                )
              ),
              stringsAsFactors = FALSE
            )

            openxlsx::writeData(
              wb_content,
              "Readability_Indices",
              readability_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Readability_Indices",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#27ae60",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:2,
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Readability_Indices",
              cols = 1:2,
              widths = c(40, 20)
            )
          }

          # === SHEET 3: WORD FREQUENCIES ===
          if (
            !is.null(values$analysis_results$word_frequencies) &&
              nrow(values$analysis_results$word_frequencies) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Word_Frequencies")

            top_words <- values$analysis_results$word_frequencies %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Frequencies",
              top_words,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Frequencies",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#3498db",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(top_words),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Frequencies",
              cols = 1:ncol(top_words),
              widths = "auto"
            )
          }

          # === SHEET 4: BIGRAMS ===
          if (
            !is.null(values$analysis_results$ngrams) &&
              "2gram" %in% names(values$analysis_results$ngrams)
          ) {
            openxlsx::addWorksheet(wb_content, "Bigrams")

            bigrams_data <- values$analysis_results$ngrams$`2gram` %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Bigrams",
              bigrams_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Bigrams",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#9b59b6",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(bigrams_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Bigrams",
              cols = 1:ncol(bigrams_data),
              widths = "auto"
            )
          }

          # === SHEET 5: TRIGRAMS ===
          if (
            !is.null(values$analysis_results$ngrams) &&
              "3gram" %in% names(values$analysis_results$ngrams)
          ) {
            openxlsx::addWorksheet(wb_content, "Trigrams")

            trigrams_data <- values$analysis_results$ngrams$`3gram` %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Trigrams",
              trigrams_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Trigrams",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e67e22",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trigrams_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Trigrams",
              cols = 1:ncol(trigrams_data),
              widths = "auto"
            )
          }

          # === SHEET 6: CITATION TYPES ===
          if (
            !is.null(values$analysis_results$citation_metrics$type_distribution)
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Types")

            types_data <- values$analysis_results$citation_metrics$type_distribution %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citation_Types",
              types_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Types",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e74c3c",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(types_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Types",
              cols = 1:ncol(types_data),
              widths = "auto"
            )
          }

          # === SHEET 7: CITATIONS BY SECTION ===
          if (
            !is.null(
              values$analysis_results$citation_metrics$section_distribution
            )
          ) {
            openxlsx::addWorksheet(wb_content, "Citations_by_Section")

            sections_data <- values$analysis_results$citation_metrics$section_distribution %>%
              filter(n > 0) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citations_by_Section",
              sections_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citations_by_Section",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(sections_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citations_by_Section",
              cols = 1:ncol(sections_data),
              widths = "auto"
            )
          }

          # === SHEET 8: CITATION CONTEXTS ===
          if (
            !is.null(values$analysis_results$citation_contexts) &&
              nrow(values$analysis_results$citation_contexts) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Contexts")

            # Seleziona le colonne principali se esistono, altrimenti esporta tutte
            contexts_cols <- c(
              "citation_id",
              "citation_text",
              "section",
              "citation_type",
              "is_narrative",
              "words_before",
              "words_after",
              "full_context",
              "context_word_count",
              "citation_position_in_text"
            )

            available_cols <- intersect(
              contexts_cols,
              names(values$analysis_results$citation_contexts)
            )

            if (length(available_cols) > 0) {
              contexts_export <- values$analysis_results$citation_contexts %>%
                select(all_of(available_cols)) %>%
                as.data.frame(stringsAsFactors = FALSE)
            } else {
              # Se nessuna colonna specificata esiste, esporta tutte
              contexts_export <- values$analysis_results$citation_contexts %>%
                as.data.frame(stringsAsFactors = FALSE)
            }

            openxlsx::writeData(
              wb_content,
              "Citation_Contexts",
              contexts_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Contexts",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#2E86AB",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(contexts_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Contexts",
              cols = 1:ncol(contexts_export),
              widths = "auto"
            )
          }

          # === SHEET 9: CITATION NETWORK ===
          if (
            !is.null(values$analysis_results$network_data) &&
              nrow(values$analysis_results$network_data) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Network")

            # Esporta tutte le colonne disponibili nel network
            network_export <- values$analysis_results$network_data %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citation_Network",
              network_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Network",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#8e44ad",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(network_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Network",
              cols = 1:ncol(network_export),
              widths = "auto"
            )
          }

          # === SHEET 10: WORD TRENDS (if available) ===
          if (
            !is.null(values$word_trends_data) &&
              nrow(values$word_trends_data) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Word_Trends")

            trends_export <- values$word_trends_data %>%
              arrange(word, segment_id) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Trends",
              trends_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Trends",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#27ae60",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trends_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Trends",
              cols = 1:ncol(trends_export),
              widths = "auto"
            )

            # Add summary statistics for word trends
            openxlsx::addWorksheet(wb_content, "Word_Trends_Summary")

            trends_summary <- values$word_trends_data %>%
              group_by(word) %>%
              summarise(
                total_occurrences = sum(count),
                avg_frequency = mean(relative_frequency),
                min_frequency = min(relative_frequency),
                max_frequency = max(relative_frequency),
                std_dev = sd(relative_frequency),
                peak_segment = segment_name[which.max(relative_frequency)],
                .groups = "drop"
              ) %>%
              arrange(desc(total_occurrences)) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Trends_Summary",
              trends_summary,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Trends_Summary",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trends_summary),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Trends_Summary",
              cols = 1:ncol(trends_summary),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER TF-IDF ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_descriptions) &&
              nrow(values$cluster_description$cluster_descriptions) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_TF_IDF")

            cluster_tfidf_export <- values$cluster_description$cluster_descriptions %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_TF_IDF",
              cluster_tfidf_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_TF_IDF",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#8e44ad",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_tfidf_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_TF_IDF",
              cols = 1:ncol(cluster_tfidf_export),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER SUMMARY ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_summary) &&
              nrow(values$cluster_description$cluster_summary) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_Summary")

            cluster_summary_export <- values$cluster_description$cluster_summary %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_Summary",
              cluster_summary_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_Summary",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e67e22",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_summary_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_Summary",
              cols = 1:ncol(cluster_summary_export),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER REFERENCES ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_references) &&
              nrow(values$cluster_description$cluster_references) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_References")

            cluster_refs_export <- values$cluster_description$cluster_references %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_References",
              cluster_refs_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_References",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_refs_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_References",
              cols = 1:ncol(cluster_refs_export),
              widths = "auto"
            )
          }

          # Save workbook
          openxlsx::saveWorkbook(wb_content, file, overwrite = TRUE)
        },
        error = function(e) {
          # Log detailed error for debugging
          cat("Error in download_all_results:\n")
          cat("Error message:", e$message, "\n")
          cat("Stack trace:\n")
          print(e)

          showNotification(
            paste("Error exporting results:", e$message),
            type = "error",
            duration = 8
          )
        }
      )
    }
  )

  # ===========================================
  # EXPORT ALL IMAGES
  # ===========================================

  # Step 1: Button click triggers JS canvas capture of the live network
  observeEvent(input$export_all_images, {
    req(values$analysis_results)
    if (!is.null(values$network_plot)) {
      dpi <- values$dpi %||% 300
      shinyjs::runjs(sprintf(
        'window._captureVisCanvas("citation_network", %d, function(dataURL) {
           Shiny.setInputValue("ca_network_canvas", {data: dataURL}, {priority: "event"});
         });',
        dpi
      ))
    } else {
      # No network — proceed directly
      shinyjs::runjs(
        'Shiny.setInputValue("ca_network_canvas", {data: null}, {priority: "event"});'
      )
    }
    showNotification("Exporting images...", type = "message", duration = 3)
  })

  # Step 2: Receive canvas data and build ZIP with all images
  observeEvent(input$ca_network_canvas, {
    req(values$analysis_results)

    tmpdir_img <- tempfile("ca_images_")
    dir.create(tmpdir_img, recursive = TRUE)
    png_files <- character(0)

    dpi <- values$dpi %||% 300
    h <- values$h %||% 7
    zoom <- dpi / 96
    vwidth <- round(h * 2 * 96)
    vheight <- round(h * 96)

    # Helper: save a plotly object to PNG
    save_plotly_png <- function(p, name) {
      png_path <- file.path(tmpdir_img, paste0(name, ".png"))
      html_path <- file.path(tmpdir_img, paste0(name, ".html"))
      htmlwidgets::saveWidget(p, file = html_path, selfcontained = FALSE)
      biblioShot(
        url = html_path,
        zoom = zoom,
        vwidth = vwidth,
        vheight = vheight,
        file = png_path
      )
      unlink(html_path)
      unlink(
        paste0(tools::file_path_sans_ext(html_path), "_files"),
        recursive = TRUE
      )
      png_path
    }

    tryCatch(
      {
        # 1. Citation Network from live canvas capture
        canvas_data <- input$ca_network_canvas$data
        if (!is.null(canvas_data) && nchar(canvas_data) > 0) {
          base64_str <- sub("^data:image/png;base64,", "", canvas_data)
          raw_data <- base64enc::base64decode(base64_str)
          net_png <- file.path(tmpdir_img, "Citation_Network.png")
          writeBin(raw_data, net_png)
          if (file.exists(net_png)) png_files <- c(png_files, net_png)
        }

        # 2. Cluster plots (plotly)
        if (!is.null(values$cluster_description)) {
          cluster_plots <- plot_citation_clusters(
            values$cluster_description,
            section_colors = values$analysis_results$section_colors,
            top_n = 10
          )

          if (!is.null(cluster_plots$tfidf_bars)) {
            f <- save_plotly_png(cluster_plots$tfidf_bars, "TF_IDF_Bars")
            if (file.exists(f)) png_files <- c(png_files, f)
          }

          if (!is.null(cluster_plots$tfidf_heatmap)) {
            f <- save_plotly_png(
              cluster_plots$tfidf_heatmap, "TF_IDF_Heatmap"
            )
            if (file.exists(f)) png_files <- c(png_files, f)
          }

          if (!is.null(cluster_plots$references_per_section)) {
            f <- save_plotly_png(
              cluster_plots$references_per_section, "References_per_Section"
            )
            if (file.exists(f)) png_files <- c(png_files, f)
          }
        }

        # 3. Word Trends (plotly)
        if (
          !is.null(values$word_trends_data) &&
            nrow(values$word_trends_data) > 0
        ) {
          trends_plot <- tryCatch(
            plot_word_distribution(
              word_distribution_data = values$word_trends_data,
              plot_type = input$trend_plot_type %||% "line",
              smooth = TRUE,
              show_points = input$trend_show_points %||% TRUE,
              colors = NULL
            ),
            error = function(e) NULL
          )
          if (!is.null(trends_plot)) {
            f <- save_plotly_png(trends_plot, "Word_Trends")
            if (file.exists(f)) png_files <- c(png_files, f)
          }
        }

        # Create zip and trigger browser download
        if (length(png_files) > 0) {
          zip_path <- file.path(tmpdir_img, paste0(
            "content_analysis_images_", Sys.Date(), ".zip"
          ))
          zip::zip(
            zip_path,
            files = basename(png_files),
            root = tmpdir_img
          )
          # Trigger download via base64 data URI
          zip_data <- base64enc::dataURI(file = zip_path, mime = "application/zip")
          zip_filename <- basename(zip_path)
          shinyjs::runjs(sprintf(
            "var link = document.createElement('a'); link.href = '%s'; link.download = '%s'; document.body.appendChild(link); link.click(); document.body.removeChild(link);",
            zip_data, zip_filename
          ))
          showNotification(
            paste(length(png_files), "images exported successfully!"),
            type = "message", duration = 3
          )
        } else {
          showNotification(
            "No images available to export.",
            type = "warning",
            duration = 4
          )
        }
      },
      error = function(e) {
        showNotification(
          paste("Error exporting images:", e$message),
          type = "error",
          duration = 5
        )
      },
      finally = {
        unlink(tmpdir_img, recursive = TRUE)
      }
    )
  })

  # ===========================================
  # UTILITY FUNCTIONS
  # ===========================================

  is_analysis_valid <- reactive({
    !is.null(values$analysis_results) &&
      !is.null(values$analysis_results$citations) &&
      nrow(values$analysis_results$citations) > 0
  })

  is_network_valid <- reactive({
    !is.null(values$analysis_results) &&
      !is.null(values$analysis_results$network_data) &&
      nrow(values$analysis_results$network_data) > 0
  })

  output$debug_info <- renderText({
    if (!is.null(values$analysis_results)) {
      paste(
        "Debug Info:\n",
        "PDF text length:",
        nchar(values$pdf_text %||% ""),
        "\n",
        "Citations found:",
        nrow(values$analysis_results$citations %||% data.frame()),
        "\n",
        "Network connections:",
        nrow(values$analysis_results$network_data %||% data.frame()),
        "\n",
        "Analysis running:",
        values$analysis_running %||% FALSE
      )
    } else {
      "No analysis data available"
    }
  })

  outputOptions(output, "debug_info", suspendWhenHidden = FALSE)
}

# Helper function for HTML card OpenAlex ----
create_oa_details_html <- function(oa_data) {
  # Helper per gestire valori NULL/NA
  safe_value <- function(x, default = "Not available") {
    if (
      is.null(x) ||
        length(x) == 0 ||
        all(is.na(x)) ||
        !nzchar(trimws(as.character(x)))
    ) {
      return(default)
    }
    return(x)
  }

  # === ESTRAI AUTORI E AFFILIAZIONI INLINE ===
  authors_html <- NULL
  if (!is.null(oa_data$authorships) && length(oa_data$authorships) > 0) {
    authorships_df <- oa_data$authorships[[1]]

    if (!is.null(authorships_df) && nrow(authorships_df) > 0) {
      authors_items <- lapply(1:min(nrow(authorships_df), 15), function(i) {
        auth <- authorships_df[i, ]

        author_name <- safe_value(auth$display_name, "Unknown Author")
        author_id <- safe_value(auth$id, "")

        # Estrai affiliazioni
        affiliation_text <- ""
        if (!is.null(auth$institutions) && length(auth$institutions) > 0) {
          insts_df <- auth$institutions[[1]]
          if (
            !is.null(insts_df) &&
              nrow(insts_df) > 0 &&
              "display_name" %in% names(insts_df)
          ) {
            affiliation_text <- paste(insts_df$display_name, collapse = ", ")
          }
        }

        # Se non ci sono istituzioni, prova con affiliation_raw
        if (
          !nzchar(affiliation_text) &&
            !is.na(auth$affiliation_raw) &&
            nzchar(auth$affiliation_raw)
        ) {
          affiliation_text <- auth$affiliation_raw
        }

        # Layout inline: nome e affiliazione sulla stessa riga
        div(
          style = "margin-bottom: 8px; padding: 6px 10px; background-color: #f8f9fa; border-radius: 4px; display: flex; align-items: center; flex-wrap: wrap;",
          tags$div(
            style = "font-weight: 500; font-size: 14px; margin-right: 12px;",
            # Link OpenAlex se disponibile
            if (nzchar(author_id) && author_id != "Not available") {
              tags$a(
                href = author_id,
                target = "_blank",
                style = "color: #2E86AB; text-decoration: none;",
                author_name,
                icon(
                  "external-link-alt",
                  style = "margin-left: 4px; font-size: 10px;"
                )
              )
            } else {
              tags$span(style = "color: #2E86AB;", author_name)
            }
          ),
          if (nzchar(affiliation_text)) {
            tags$span(
              style = "color: #666; font-size: 13px; font-style: italic;",
              icon("university", style = "margin-right: 5px; font-size: 11px;"),
              affiliation_text
            )
          }
        )
      })

      authors_html <- tagList(authors_items)

      if (nrow(authorships_df) > 15) {
        authors_html <- tagList(
          authors_html,
          tags$small(
            style = "color: #999; font-style: italic; display: block; margin-top: 8px; padding-left: 10px;",
            paste("... and", nrow(authorships_df) - 15, "more authors")
          )
        )
      }
    }
  }

  # === KEYWORDS - TUTTE ===
  keywords_html <- NULL
  if (!is.null(oa_data$keywords) && length(oa_data$keywords) > 0) {
    keywords_data <- oa_data$keywords[[1]]
    if (
      !is.null(keywords_data) &&
        is.data.frame(keywords_data) &&
        nrow(keywords_data) > 0
    ) {
      if ("display_name" %in% names(keywords_data)) {
        # NESSUN LIMITE - tutte le keywords
        keywords_html <- lapply(keywords_data$display_name, function(kw) {
          tags$span(
            class = "label label-primary",
            style = "margin-right: 5px; margin-bottom: 5px; font-size: 11px; display: inline-block; background-color: #3498db; color: white; padding: 4px 10px; border-radius: 3px;",
            kw
          )
        })
      }
    }
  }

  # === ABSTRACT ===
  abstract_text <- safe_value(oa_data$abstract, "Abstract not available")

  # === TOPICS - SOLO I PRIMI 5 ===
  topics_html <- NULL
  if (!is.null(oa_data$topics) && length(oa_data$topics) > 0) {
    topics_data <- oa_data$topics[[1]]
    if (
      !is.null(topics_data) &&
        is.data.frame(topics_data) &&
        nrow(topics_data) > 0
    ) {
      if ("display_name" %in% names(topics_data)) {
        main_topics <- topics_data
        if ("type" %in% names(topics_data)) {
          main_topics <- topics_data[topics_data$type == "topic", ]
        }

        if (nrow(main_topics) > 0) {
          # LIMITE A 5 TOPICS
          topics_html <- lapply(1:min(5, nrow(main_topics)), function(i) {
            tags$span(
              class = "label label-info",
              style = "margin-right: 5px; margin-bottom: 5px; font-size: 11px; display: inline-block; background-color: #5bc0de; color: white; padding: 4px 10px; border-radius: 3px;",
              main_topics$display_name[i]
            )
          })
        }
      }
    }
  }

  # === JOURNAL INFO ===
  journal_name <- safe_value(
    oa_data$source_display_name,
    "Journal not available"
  )
  issn <- safe_value(oa_data$issn_l, "")

  # === DOCUMENT TYPE ===
  doc_type <- safe_value(oa_data$type, "Unknown")

  # === PUBLICATION INFO ===
  pub_year <- safe_value(oa_data$publication_year, "N/A")
  citations <- safe_value(oa_data$cited_by_count, 0)
  fwci <- safe_value(oa_data$fwci, NA)

  # === OPEN ACCESS INFO ===
  is_oa <- !is.na(oa_data$is_oa) && oa_data$is_oa
  oa_status <- tolower(safe_value(oa_data$oa_status, "closed")) # Normalizza in lowercase

  # === DOI ===
  doi_url <- safe_value(oa_data$doi, "")
  doi_clean <- gsub("https://doi.org/", "", doi_url)

  # === PDF URL ===
  pdf_url <- safe_value(oa_data$pdf_url, "")
  has_pdf <- nzchar(pdf_url) && !is.na(pdf_url) && pdf_url != "Not available"

  # === CREA HTML COMPLETO ===
  tagList(
    div(
      class = "oa-document-summary",

      # Metadata Cards Row
      fluidRow(
        style = "margin-bottom: 20px;",

        # Card 1: Publication Year (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #3498db 0%, #2980b9 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "calendar",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Publication Year"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  pub_year
                )
              )
            )
          )
        ),

        # Card 2: Citations (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #27ae60 0%, #229954 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "quote-right",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Citations"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  format(as.numeric(citations), big.mark = ",")
                )
              )
            )
          )
        ),

        # Card 3: Open Access Status (MODIFICATA)
        column(3, {
          # Determina colore e icona in base a oa_status
          oa_colors <- list(
            closed = list(
              bg = "#95a5a6 0%, #7f8c8d",
              icon = "lock",
              text = "Closed"
            ),
            gold = list(
              bg = "#f39c12 0%, #e67e22",
              icon = "unlock",
              text = "Gold OA"
            ),
            green = list(
              bg = "#27ae60 0%, #229954",
              icon = "unlock",
              text = "Green OA"
            ),
            bronze = list(
              bg = "#cd7f32 0%, #b8732d",
              icon = "unlock",
              text = "Bronze OA"
            ),
            hybrid = list(
              bg = "#16a085 0%, #138d75",
              icon = "unlock",
              text = "Hybrid OA"
            )
          )

          # Se oa_status non è nei valori conosciuti, usa closed come default
          current_status <- if (oa_status %in% names(oa_colors)) {
            oa_status
          } else {
            "closed"
          }
          status_info <- oa_colors[[current_status]]

          div(
            style = paste0(
              "background: linear-gradient(135deg, ",
              status_info$bg,
              " 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);"
            ),
            div(
              style = "display: flex; align-items: center;",
              icon(
                status_info$icon,
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Access"
                ),
                tags$div(
                  style = "font-size: 18px; font-weight: bold; margin-top: 2px;",
                  status_info$text
                )
              )
            )
          )
        }),

        # Card 4: FWCI (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #f39c12 0%, #e67e22 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "chart-line",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "FWCI"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  if (!is.na(fwci)) round(as.numeric(fwci), 2) else "N/A"
                )
              )
            )
          )
        )
      ),

      # Authors Section - PIÙ COMPATTA
      if (!is.null(authors_html)) {
        div(
          class = "box box-primary",
          style = "margin-bottom: 15px;",
          div(
            class = "box-header with-border",
            h5(
              icon("users", style = "margin-right: 8px;"),
              "Authors",
              style = "color: #2E86AB; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            style = "max-height: 250px; overflow-y: auto;",
            authors_html
          )
        )
      },

      # Journal Info
      div(
        class = "box box-success",
        style = "margin-bottom: 15px;",
        div(
          class = "box-header with-border",
          h5(
            icon("book", style = "margin-right: 8px;"),
            "Journal",
            style = "color: #27ae60; margin: 0; font-weight: 600;"
          )
        ),
        div(
          class = "box-body",
          style = "padding: 15px;",

          # Container principale
          div(
            style = "display: flex; align-items: center; justify-content: space-between; flex-wrap: wrap; gap: 20px;",

            # Colonna 1: Nome Journal e ISSN
            div(
              style = "flex: 1 1 300px; min-width: 250px;",
              tags$div(
                style = "font-size: 16px; font-weight: 600; color: #2c3e50; margin-bottom: 5px;",
                journal_name
              ),
              if (nzchar(issn) && issn != "Not available") {
                tags$div(
                  style = "color: #666; font-size: 13px;",
                  tags$strong("ISSN: "),
                  issn
                )
              }
            ),

            # Colonna 2: Document Type
            div(
              style = "flex: 0 0 auto; display: flex; align-items: center; gap: 8px;",
              tags$span(
                style = "color: #666; font-size: 13px; font-weight: 500;",
                "Type:"
              ),
              tags$span(
                class = "label label-info",
                style = "font-size: 12px; padding: 5px 12px; background-color: #3498db; text-transform: capitalize;",
                safe_value(oa_data$type, "Unknown")
              )
            ),

            # Colonna 3: DOI
            if (nzchar(doi_url) && doi_url != "Not available") {
              div(
                style = "flex: 0 0 auto; display: flex; align-items: center; gap: 8px;",
                tags$span(
                  style = "color: #666; font-size: 13px; font-weight: 500;",
                  "DOI:"
                ),
                tags$a(
                  href = doi_url,
                  target = "_blank",
                  style = "color: #3498db; text-decoration: none; font-family: 'Courier New', monospace; font-size: 13px; font-weight: 500;",
                  doi_clean,
                  icon(
                    "external-link-alt",
                    style = "margin-left: 5px; font-size: 11px;"
                  )
                )
              )
            }
          )
        )
      ),

      # Abstract
      div(
        class = "box box-info",
        style = "margin-bottom: 15px;",
        div(
          class = "box-header with-border",
          h5(
            icon("align-left", style = "margin-right: 8px;"),
            "Abstract",
            style = "color: #3498db; margin: 0; font-weight: 600;"
          )
        ),
        div(
          class = "box-body",
          tags$p(
            style = "text-align: justify; line-height: 1.7; color: #333;",
            abstract_text
          )
        )
      ),

      # Keywords - TUTTE
      if (!is.null(keywords_html) && length(keywords_html) > 0) {
        div(
          class = "box box-warning",
          style = "margin-bottom: 15px;",
          div(
            class = "box-header with-border",
            h5(
              icon("tags", style = "margin-right: 8px;"),
              "Keywords",
              tags$small(
                style = "color: #999; font-weight: normal; margin-left: 8px;",
                paste0("(", length(keywords_html), ")")
              ),
              style = "color: #f39c12; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            keywords_html
          )
        )
      },

      # Topics - SOLO PRIMI 5
      if (!is.null(topics_html) && length(topics_html) > 0) {
        div(
          class = "box",
          style = "margin-bottom: 15px; border-top: 3px solid #9b59b6;",
          div(
            class = "box-header with-border",
            h5(
              icon("lightbulb", style = "margin-right: 8px;"),
              "Research Topics",
              tags$small(
                style = "color: #999; font-weight: normal; margin-left: 8px;",
                "(Top 5)"
              ),
              style = "color: #9b59b6; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            topics_html
          )
        )
      },

      # Action Button - PDF
      if (has_pdf) {
        div(
          style = "margin-top: 20px; padding-top: 15px; border-top: 1px solid #ddd; text-align: center;",
          tags$a(
            class = "btn btn-danger btn-lg",
            href = pdf_url,
            target = "_blank",
            icon("file-pdf"),
            " Download Full Text PDF",
            style = "font-weight: bold; padding: 12px 30px;"
          )
        )
      }
    )
  )
}

reconstruct_from_txt <- function(file_lines) {
  reconstructed_txt <- list()
  current_section <- "Full_text"
  section_content <- ""
  for (line in file_lines) {
    if (grepl("^SECTION LINE SEPARATION: ", line)) {
      # Save the previous section content
      if (current_section != "Full_text") {
        reconstructed_txt[[current_section]] <- section_content
      }
      # Start a new section
      current_section <- sub("^SECTION LINE SEPARATION: ", "", line)
      section_content <- ""
    } else {
      section_content <- paste0(
        section_content,
        ifelse(section_content == "", "", "\n"),
        line
      )
    }
  }
  # Save the last section content
  reconstructed_txt[[current_section]] <- section_content
  if (!"Full_text" %in% names(reconstructed_txt)) {
    Full_text <- paste(reconstructed_txt, collapse = "\n\n")
    reconstructed_txt <- c(Full_text = Full_text, reconstructed_txt)
  }
  return(reconstructed_txt)
}

### COMMON FUNCTIONS ####

## Count records in a bibliographic file without full conversion ----
#' @description Lightweight record counter for bibliographic files.
#'   Returns the number of publications contained in one or more files
#'   without performing the full convert2df() conversion.
#' @param file Character vector of file paths (or a single ZIP path).
#' @param dbsource Character. The database source
#'   ("isi","scopus","openalex","openalex_api","lens","cochrane","pubmed","dimensions").
#' @param format Character. The file format
#'   ("plaintext","bibtex","csv","pubmed","excel","api","endnote").
#'   If NULL, it is auto-detected from the file extension.
#' @return Integer. The total number of records across all files.
countRecords <- function(file, dbsource = "isi", format = NULL) {

  ## ---- helper: count lines matching a pattern ----
  count_lines_matching <- function(paths, pattern) {
    n <- 0L
    for (f in paths) {
      lines <- readLines(f, warn = FALSE, encoding = "UTF-8")
      n <- n + sum(grepl(pattern, lines, perl = TRUE))
    }
    n
  }

  ## ---- helper: count CSV rows (data rows only, excluding header) ----
  count_csv_rows <- function(paths) {
    n <- 0L
    for (f in paths) {
      # Count total lines minus 1 for header
      total <- length(readLines(f, warn = FALSE))
      n <- n + max(0L, total - 1L)
    }
    n
  }

  ## ---- handle ZIP: extract to temp dir ----
  paths <- file
  if (length(file) == 1 && tolower(tools::file_ext(file)) == "zip") {
    temp_exdir <- tempfile("countRecords_")
    paths <- utils::unzip(file, exdir = temp_exdir)
    on.exit(unlink(temp_exdir, recursive = TRUE), add = TRUE)
  }

  ## ---- auto-detect format from file extension if not provided ----
  if (is.null(format)) {
    ext <- tolower(tools::file_ext(paths[1]))
    format <- switch(ext,
      txt  = if (dbsource == "pubmed") "pubmed" else "plaintext",
      csv  = "csv",
      bib  = "bibtex",
      ciw  = "plaintext",
      xlsx = "excel",
      rdata = "rdata", rda = "rdata", rds = "rds",
      "plaintext"
    )
  }

  ## ---- count records by format ----
  n <- switch(format,
    plaintext = ,
    endnote = {
      if (dbsource == "cochrane") {
        count_lines_matching(paths, "^Record #")
      } else {
        # WoS: each record starts with "PT "
        count_lines_matching(paths, "^PT ")
      }
    },
    bibtex = {
      count_lines_matching(paths, "^@")
    },
    csv = {
      count_csv_rows(paths)
    },
    pubmed = {
      count_lines_matching(paths, "^PMID- ")
    },
    excel = {
      n <- 0L
      for (f in paths) {
        df <- readxl::read_excel(f, col_types = "text",
                                 range = readxl::cell_cols(1))
        n <- n + nrow(df)
      }
      n
    },
    api = ,
    rdata = {
      n <- 0L
      for (f in paths) {
        env <- new.env(parent = emptyenv())
        var <- load(f, envir = env)
        obj <- get(var[1], envir = env)
        n <- n + if (is.data.frame(obj)) nrow(obj) else length(obj)
      }
      n
    },
    rds = {
      n <- 0L
      for (f in paths) {
        obj <- readRDS(f)
        n <- n + if (is.data.frame(obj)) nrow(obj) else length(obj)
      }
      n
    },
    0L
  )

  as.integer(n)
}

## Cache helper: lightweight fingerprint of a data.frame
data_fingerprint <- function(M) {
  nr <- nrow(M)
  if (nr == 0) return("empty")
  paste(nr, ncol(M),
        M$SR[1], M$SR[nr],
        sep = "||")
}

## Cache helper: build a cache key from a named list of params
make_cache_key <- function(...) {
  args <- list(...)
  lapply(args, function(x) {
    if (is.null(x)) "<<NULL>>" else x
  })
}

# Funzione helper aggiornata con titolo e affiliazione
createAuthorCard <- function(
  name,
  title,
  affiliation,
  url,
  photo,
  scholar = FALSE
) {
  tags$a(
    href = url,
    target = "_blank",
    style = "text-decoration: none;",
    div(
      style = "text-align: center; transition: transform 0.3s; cursor: pointer; padding: 15px; background: #f9f9f9; border-radius: 10px;",
      onmouseover = "this.style.transform='translateY(-5px)'; this.style.boxShadow='0 8px 20px rgba(0,0,0,0.15)';",
      onmouseout = "this.style.transform='translateY(0)'; this.style.boxShadow='none';",

      # Foto circolare con bordo
      div(
        style = sprintf(
          "width: 110px; height: 110px; border-radius: 50%%; background-image: url('%s'); background-size: cover; background-position: center; margin: 0 auto 15px; border: 4px solid #3c8dbc; box-shadow: 0 4px 15px rgba(0,0,0,0.2);",
          photo
        )
      ),

      # Nome
      div(
        name,
        style = "font-weight: bold; color: #2c3e50; font-size: 16px; margin-bottom: 8px;"
      ),

      # Titolo
      div(
        title,
        style = "color: #7f8c8d; font-size: 13px; font-weight: 500; margin-bottom: 5px;"
      ),

      # Affiliazione
      div(
        icon("university", style = "margin-right: 5px;"),
        affiliation,
        style = "color: #95a5a6; font-size: 12px; line-height: 1.4; margin-bottom: 10px;"
      ),

      # Link icon
      div(
        if (scholar) {
          tagList(
            icon("graduation-cap", style = "margin-right: 3px;"),
            "Google Scholar"
          )
        } else {
          tagList(
            icon("link", style = "margin-right: 3px;"),
            "Website"
          )
        },
        style = "color: #3c8dbc; font-size: 12px; font-weight: 600;"
      )
    )
  )
}


# Scroll to Top Button (Font Awesome version)
scrollToTopButton <- function() {
  tags$div(
    # CSS per il pulsante
    tags$head(
      tags$style(HTML(
        "
        #scrollToTopBtn {
          display: none;
          position: fixed;
          bottom: 20px;
          right: 20px;
          z-index: 99999;
          border: none;
          outline: none;
          background-color: rgba(68, 68, 68, 0.7);
          color: white;
          cursor: pointer;
          padding: 12px;
          border-radius: 4px;
          font-size: 16px;
          width: 45px;
          height: 45px;
          box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
          transition: all 0.3s ease;
          backdrop-filter: blur(4px);
        }

        #scrollToTopBtn:hover {
          background-color: rgba(29, 143, 225, 0.85);
          transform: translateY(-3px);
          box-shadow: 0 4px 10px rgba(0, 0, 0, 0.4);
        }

        #scrollToTopBtn:active {
          transform: translateY(-1px);
          box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
        }

        #scrollToTopBtn i {
          margin: 0;
          padding: 0;
          line-height: 1;
        }
      "
      ))
    ),

    # Il pulsante HTML
    tags$button(
      id = "scrollToTopBtn",
      icon("arrow-up", lib = "glyphicon"),
      onclick = "scrollToTop()",
      title = "Back to top"
    ),

    # JavaScript per gestire lo scroll
    tags$script(HTML(
      "
      // Funzione per mostrare/nascondere il pulsante
      window.onscroll = function() {scrollFunction()};

      function scrollFunction() {
        var btn = document.getElementById('scrollToTopBtn');
        if (document.body.scrollTop > 100 || document.documentElement.scrollTop > 100) {
          btn.style.display = 'block';
        } else {
          btn.style.display = 'none';
        }
      }

      // Funzione per scrollare in alto con animazione smooth
      function scrollToTop() {
        window.scrollTo({
          top: 0,
          behavior: 'smooth'
        });
      }
    "
    ))
  )
}

# Number format abbreviated
format_abbreviated <- function(x) {
  if (is.na(x)) {
    return("--")
  }
  if (x >= 1e6) {
    return(paste0(format(round(x / 1e6, 2), nsmall = 2), "M"))
  } else if (x >= 1e3) {
    return(paste0(format(round(x / 1e3, 0), nsmall = 0), "K"))
  } else {
    return(as.character(x))
  }
}

# total package download
total_downloads <- function(
  pkg_name = "bibliometrix",
  from = "2016-01-01",
  to = Sys.Date()
) {
  # Function to get total downloads of a package from CRAN logs
  # Args:
  #   pkg_name: Name of the package as a string
  # Returns:
  #   Total number of downloads as an integer

  if (!is_Online()) {
    return(NA)
  }

  #today <- Sys.Date()
  if (!is.character(pkg_name) || length(pkg_name) != 1) {
    stop("pkg_name must be a single string.")
  }

  url <- paste0(
    "https://cranlogs.r-pkg.org/downloads/total/",
    from,
    ":",
    to,
    "/",
    pkg_name
  )

  if (!check_online(host = url, timeout = 1, method = "http")) {
    return(NA)
  }

  json_text <- tryCatch(
    {
      readLines(url, warn = FALSE)
    },
    error = function(e) {
      return(NA)
    }
  )

  # Se già nel tryCatch è tornato "NA", esci subito
  if (identical(json_text, "NA")) {
    return(NA)
  }

  # Extract the number manually (not robust)
  txt <- unlist(strsplit(json_text, ","))
  txt <- txt[grepl("downloads", txt)]

  if (length(txt) == 0) {
    return(NA)
  }

  downloads <- gsub("[^0-9]", "", txt)

  return(as.integer(downloads))
}

# getFileNameExtension <- function(fn) {
#   # remove a path
#   splitted <- strsplit(x = fn, split = "/")[[1]]
#   # or use .Platform$file.sep in stead of '/'
#   fn <- splitted[length(splitted)]
#   ext <- ""
#   splitted <- strsplit(x = fn, split = "\\.")[[1]]
#   l <- length(splitted)
#   if (l > 1 && sum(splitted[1:(l - 1)] != "")) {
#     ext <- splitted[l]
#   }
#   # the extention must be the suffix of a non-empty name
#   ext
# }

# FILTER FUNCTIONS ----
read_journal_ranking <- function(file_path) {
  ext <- tools::file_ext(file_path)

  suppressMessages(
    journals <- switch(
      tolower(ext),
      "csv" = read.csv(file_path, header = TRUE, stringsAsFactors = FALSE),
      "xlsx" = {
        readxl::read_excel(file_path, col_names = TRUE)
      },
      stop(
        "Unsupported file format. Please upload a .csv, .txt, or .xlsx file."
      )
    )
  )
  journals <- journals %>% select(1, 2)
  # journals <- journals[!is.na(journals)]
  # journals <- toupper(trimws(journals))
  names(journals) <- c("SO", "Ranking")
  journals <- journals %>%
    mutate(SO = toupper(trimws(SO)))
  return(journals)
}

read_journal_list <- function(file_path) {
  ext <- tools::file_ext(file_path)

  suppressMessages(
    journals <- switch(
      tolower(ext),
      "csv" = read.csv(file_path, header = FALSE, stringsAsFactors = FALSE)[[
        1
      ]],
      "txt" = readLines(file_path, warn = FALSE),
      "xlsx" = {
        readxl::read_excel(file_path, col_names = FALSE)[[1]]
      },
      stop(
        "Unsupported file format. Please upload a .csv, .txt, or .xlsx file."
      )
    )
  )

  journals <- journals[!is.na(journals)]
  journals <- toupper(trimws(journals))
  return(journals)
}

wcTable <- function(M) {
  # Function to extract Science Category (WC) information from metadata
  if ("WC" %in% names(M)) {
    WC <- strsplit(M$WC, ";")
    df <- data.frame(
      SR = rep(M$SR, lengths(WC)),
      WC = unlist(WC),
      stringsAsFactors = FALSE
    )

    df$WC <- trimws(df$WC) # Remove leading and trailing whitespace
  } else {
    df <- data.frame(SR = M$SR, WC = "N.A.", stringsAsFactors = FALSE)
  }

  return(df)
}

countryTable <- function(M) {
  data("countries", envir = environment())
  # Function to extract country information from metadata
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, "AU_CO")
  }

  CO <- strsplit(M$AU_CO, ";")
  df <- data.frame(
    SR = rep(M$SR, lengths(CO)),
    CO = trimws(unlist(CO)),
    stringsAsFactors = FALSE
  )

  df$CO <- gsub("[[:digit:]]", "", df$CO)
  df$CO <- gsub(".", "", df$CO, fixed = TRUE)
  df$CO <- gsub(";;", ";", df$CO, fixed = TRUE)
  df$CO <- gsub("UNITED STATES", "USA", df$CO)
  df$CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", df$CO)
  df$CO <- gsub("TAIWAN", "CHINA", df$CO)
  df$CO <- gsub("ENGLAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("SCOTLAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("WALES", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("UK", "UNITED KINGDOM", df$CO)
  #df$CO <- gsub("KOREA", "SOUTH KOREA", df$CO)

  df <- df %>%
    left_join(
      countries %>% select(countries, continent),
      by = c("CO" = "countries")
    ) %>%
    mutate(CO = ifelse(is.na(CO), "Unknown", CO)) %>%
    mutate(continent = ifelse(is.na(continent), "Unknown", continent)) %>%
    mutate(CO = ifelse(CO == "UNKNOWN", "Unknown", CO))
}

# LOAD FUNCTIONS -----

formatDB <- function(obj) {
  ext <- sub(".*\\.", "", obj[1])
  switch(
    ext,
    txt = {
      format <- "plaintext"
    },
    csv = {
      format <- "csv"
    },
    bib = {
      format <- "bibtex"
    },
    ciw = {
      format <- "endnote"
    },
    xlsx = {
      format <- "excel"
    }
  )
  return(format)
}

## smart_load function ----
smart_load <- function(file) {
  var <- load(file)
  n <- length(var)
  if (!"M" %in% var) {
    if (n == 1) {
      eval(parse(text = paste0("M <- ", var)))
    } else {
      stop("I could not find bibliometrixDB object in your data file: ", file)
    }
  }
  rm(list = var[var != "M"])
  if (("M" %in% ls()) & inherits(M, "bibliometrixDB")) {
    return(M)
  } else {
    stop(
      "Please make sure your RData/Rda file contains a bibliometrixDB object (M)."
    )
  }
}


## merge collections ----
merge_files <- function(files) {
  ## load xlsx or rdata bibliometrix files
  if ("datapath" %in% names(files)) {
    file <- files$datapath
    ext <- unlist(lapply(file, tools::file_ext))
  }

  Mfile <- list()
  n <- 0
  for (i in 1:length(file)) {
    extF <- ext[i]
    filename <- file[i]

    switch(
      tolower(extF),
      xlsx = {
        Mfile[[i]] <- readxl::read_excel(filename, col_types = "text") %>%
          as.data.frame()
        Mfile[[i]]$PY <- as.numeric(Mfile[[i]]$PY)
        Mfile[[i]]$TC <- as.numeric(Mfile[[i]]$TC)
      },
      rdata = {
        Mfile[[i]] <- smart_load(filename)
      }
    )
    n <- n + nrow(Mfile[[i]])
  }

  # merge bibliometrix files
  M <- mergeDbSources(Mfile, remove.duplicated = T)

  # save original size as attribute
  attr(M, "nMerge") <- n

  return(M)
}

## dynamic watch emoji icons ---
watchEmoji <- function(i) {
  emoji <- c(
    "🕐",
    "🕑",
    "🕒",
    "🕓",
    "🕔",
    "🕕",
    "🕖",
    "🕗",
    "🕘",
    "🕙",
    "🕚",
    "🕛"
  )
  # i is a positive int number, reduce it to an int from 1 to 12
  multiple <- floor(i / 12)
  if (multiple > 0) {
    i <- i %% (12 * multiple)
    if (i == 0) {
      i <- 12
    }
  }
  emoji[i]
}

## Author Name Format (Fullname or Surname, N.) ----
authorNameFormat <- function(M, format) {
  if (format == "AF" & "AF" %in% names(M)) {
    M <- M %>%
      rename(
        AU_IN = AU,
        AU = AF
      )
  }
  return(M)
}

split_text_numbers <- function(input_str, UT) {
  # Split the string into components based on "; "
  components <- unlist(strsplit(input_str, "; ", fixed = TRUE))

  # Initialize two vectors to store the separated parts
  texts <- character(length(components))
  numbers <- numeric(length(components))

  # Iterate through each component to separate text and numbers
  for (i in seq_along(components)) {
    # Extract the text using regex, matching everything up to " ("
    texts[i] <- gsub("\\s\\(.*$", "", components[i])

    # Extract the numbers using regex, matching digits inside parentheses
    numbers[i] <- as.numeric(gsub(".*\\((\\d+)\\).*", "\\1", components[i]))
  }

  # Return a list with texts and numbers separated
  data.frame(Texts = texts, Numbers = numbers, UT = UT)
}


AuthorNameMerge <- function(M) {
  df_list <- list()
  for (i in 1:nrow(M)) {
    if (nchar(M$AU[i]) > 0) {
      df_list[[i]] <- split_text_numbers(M$AU[i], M$UT[i])
    }
  }

  df <- do.call(rbind, df_list)

  AU <- df %>%
    group_by(Numbers, Texts) %>%
    count() %>%
    group_by(Numbers) %>%
    arrange(desc(n)) %>%
    mutate(AU = Texts[1]) %>%
    select(-"n", -"Texts") %>%
    ungroup() %>%
    distinct()

  df <- df %>%
    left_join(AU, by = "Numbers") %>%
    group_by(UT) %>%
    summarize(
      AU = paste0(AU, collapse = ";"),
      AU_ID = paste0(Numbers, collapse = ";")
    )

  M <- M %>%
    rename(AU_original = AU) %>%
    left_join(df, by = "UT")
  return(M)
}

## RESET MODAL DIALOG INPUTS
resetModalButtons <- function(session) {
  session$sendCustomMessage("button_id", "null")
  session$sendCustomMessage("button_id2", "null")
  # session$sendCustomMessage("click", "null")
  # session$sendCustomMessage("click-dend", "null")
  # runjs("Shiny.setInputValue('plotly_click-A', null);")
  return(session)
}

### LIFE CYCLE PLOTLY FUNCTION ----

#' Plot Life Cycle Analysis Results with ggplot2
#'
#' Creates ggplot2 plots from lifeCycle analysis results
#'
#' @param results Output from lifeCycle() function
#' @param plot_type Character, either "annual" or "cumulative" to specify which plot to generate
#'
#' @return A ggplot2 object
#'
#' @export
ggplotLifeCycle <- function(results, plot_type = c("annual", "cumulative")) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required but not installed.")
  }

  plot_type <- match.arg(plot_type)

  # Extract data
  complete_curve <- results$complete_curve
  observed_data <- results$data
  params <- results$parameters_real_years
  metrics <- results$metrics
  K <- params$K
  tm_year <- params$tm_year
  R2 <- metrics$R_squared

  if (plot_type == "annual") {
    # Plot 1: Annual Publications
    max_annual <- max(complete_curve$annual, na.rm = TRUE)

    p <- ggplot() +
      geom_line(
        data = complete_curve,
        aes(x = year, y = annual),
        color = "blue",
        linewidth = 1
      ) +
      geom_point(
        data = observed_data,
        aes(x = PY, y = n),
        color = "blue",
        size = 3
      ) +
      geom_vline(
        xintercept = tm_year,
        linetype = "dashed",
        color = "red",
        linewidth = 0.7
      ) +
      annotate(
        "text",
        x = tm_year,
        y = max_annual * 0.95,
        label = sprintf("Peak: %.1f", tm_year),
        hjust = -0.1,
        color = "red",
        size = 3.5
      ) +
      annotate(
        "text",
        x = max(complete_curve$year),
        y = max_annual * 1.05,
        label = sprintf("R² = %.3f", R2),
        hjust = 1,
        size = 4
      ) +
      labs(
        title = "Life Cycle - Annual Publications",
        x = "Year",
        y = "Publications (Annual)"
      ) +
      scale_y_continuous(limits = c(0, max_annual * 1.1)) +
      theme_minimal(base_size = 12) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14),
        panel.grid.minor = element_blank()
      )

    return(p)
  } else if (plot_type == "cumulative") {
    # Plot 2: Cumulative Publications
    reference_lines <- data.frame(
      y = c(K * 0.5, K * 0.9, K * 0.99),
      label = c("50%", "90%", "99%")
    )

    p <- ggplot() +
      geom_line(
        data = complete_curve,
        aes(x = year, y = cumulative),
        color = "darkgreen",
        linewidth = 1
      ) +
      geom_point(
        data = observed_data,
        aes(x = PY, y = cumulative),
        color = "darkgreen",
        size = 3
      ) +
      geom_hline(
        data = reference_lines,
        aes(yintercept = y),
        linetype = "dotted",
        color = "gray50",
        linewidth = 0.5
      ) +
      geom_text(
        data = reference_lines,
        aes(x = min(complete_curve$year), y = y, label = label),
        hjust = 0,
        vjust = -0.5,
        size = 3,
        color = "gray50"
      ) +
      labs(
        title = "Cumulative Growth Curve",
        x = "Year",
        y = "Cumulative Publications"
      ) +
      scale_y_continuous(limits = c(0, K * 1.05)) +
      theme_minimal(base_size = 12) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14),
        panel.grid.minor = element_blank()
      )

    return(p)
  }
}

#' Plot Life Cycle Analysis with Plotly
#'
#' Creates interactive plotly visualizations from lifeCycle results
#' for use in biblioshiny
#'
#' @param results Output from lifeCycle() function
#' @param plot_type Character: "annual" or "cumulative" (default: "annual")
#'
#' @return A plotly object
#'
plotLifeCycle <- function(results, plot_type = c("annual", "cumulative")) {
  if (!requireNamespace("plotly", quietly = TRUE)) {
    stop(
      "Package 'plotly' is required. Please install it with: install.packages('plotly')"
    )
  }

  plot_type <- match.arg(plot_type)

  # Extract data
  df <- results$data
  complete <- results$complete_curve
  params <- results$parameters_real_years
  metrics <- results$metrics

  # Separate observed and forecast
  last_obs_year <- max(df$PY)
  observed <- complete[complete$year <= last_obs_year, ]
  forecast <- complete[complete$year > last_obs_year, ]

  if (plot_type == "annual") {
    # === ANNUAL PUBLICATIONS PLOT ===

    p <- plotly::plot_ly()

    # Observed curve (historical fit)
    p <- p %>%
      plotly::add_trace(
        data = observed,
        x = ~year,
        y = ~annual,
        type = 'scatter',
        mode = 'lines',
        name = 'Logistic fit',
        line = list(color = '#1f77b4', width = 2),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Annual:</b> %{y:.0f}<br>',
          '<extra></extra>'
        )
      )

    # Forecast curve
    if (nrow(forecast) > 0) {
      p <- p %>%
        plotly::add_trace(
          data = forecast,
          x = ~year,
          y = ~annual,
          type = 'scatter',
          mode = 'lines',
          name = 'Forecast',
          line = list(color = '#1f77b4', width = 2, dash = 'dash'),
          hovertemplate = paste0(
            '<b>Year:</b> %{x}<br>',
            '<b>Projected:</b> %{y:.0f}<br>',
            '<extra></extra>'
          )
        )
    }

    # Observed data points
    p <- p %>%
      plotly::add_trace(
        data = df,
        x = ~PY,
        y = ~n,
        type = 'scatter',
        mode = 'markers',
        name = 'Observed',
        marker = list(color = '#1f77b4', size = 8),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Publications:</b> %{y}<br>',
          '<extra></extra>'
        )
      )

    # Peak year line
    p <- p %>%
      plotly::add_trace(
        x = c(params$tm_year, params$tm_year),
        y = c(0, max(complete$annual) * 1.1),
        type = 'scatter',
        mode = 'lines',
        name = paste0('Peak year (', round(params$tm_year, 1), ')'),
        line = list(color = 'red', width = 1.5, dash = 'dash'),
        hoverinfo = 'name'
      )

    # Layout
    p <- p %>%
      plotly::layout(
        title = list(
          text = sprintf(
            "Life Cycle - Annual Publications<br><sup>R² = %.3f | Peak = %.0f publications in %.1f</sup>",
            metrics$R_squared,
            params$peak_annual,
            params$tm_year
          ),
          x = 0.5,
          xanchor = 'center'
        ),
        xaxis = list(
          title = "Year",
          showgrid = FALSE
        ),
        yaxis = list(
          title = "Publications (Annual)",
          showgrid = TRUE,
          gridcolor = '#f0f0f0',
          rangemode = 'tozero'
        ),
        hovermode = 'closest',
        showlegend = TRUE,
        legend = list(
          x = 0.02,
          y = 0.98,
          bgcolor = 'rgba(255, 255, 255, 0.8)',
          bordercolor = '#ddd',
          borderwidth = 1
        ),
        plot_bgcolor = 'white',
        paper_bgcolor = 'white'
      )
  } else {
    # === CUMULATIVE PUBLICATIONS PLOT ===

    K <- params$K

    p <- plotly::plot_ly()

    # Observed curve
    p <- p %>%
      plotly::add_trace(
        data = observed,
        x = ~year,
        y = ~cumulative,
        type = 'scatter',
        mode = 'lines',
        name = 'Logistic fit',
        line = list(color = '#2ca02c', width = 2),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Cumulative:</b> %{y:.0f}<br>',
          '<b>% of K:</b> %{customdata:.1f}%<br>',
          '<extra></extra>'
        ),
        customdata = ~ (cumulative / K * 100)
      )

    # Forecast curve
    if (nrow(forecast) > 0) {
      p <- p %>%
        plotly::add_trace(
          data = forecast,
          x = ~year,
          y = ~cumulative,
          type = 'scatter',
          mode = 'lines',
          name = 'Forecast',
          line = list(color = '#2ca02c', width = 2, dash = 'dash'),
          hovertemplate = paste0(
            '<b>Year:</b> %{x}<br>',
            '<b>Projected:</b> %{y:.0f}<br>',
            '<b>% of K:</b> %{customdata:.1f}%<br>',
            '<extra></extra>'
          ),
          customdata = ~ (cumulative / K * 100)
        )
    }

    # Observed data points
    p <- p %>%
      plotly::add_trace(
        data = df,
        x = ~PY,
        y = ~cumulative,
        type = 'scatter',
        mode = 'markers',
        name = 'Observed',
        marker = list(color = '#2ca02c', size = 8),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Cumulative:</b> %{y:.0f}<br>',
          '<b>% of K:</b> %{customdata:.1f}%<br>',
          '<extra></extra>'
        ),
        customdata = ~ (cumulative / K * 100)
      )

    # Reference lines (50%, 90%, 99%)
    ref_levels <- data.frame(
      level = c(0.5, 0.9, 0.99),
      label = c("50%", "90%", "99%")
    )

    for (i in 1:nrow(ref_levels)) {
      p <- p %>%
        plotly::add_trace(
          x = c(min(complete$year), max(complete$year)),
          y = c(K * ref_levels$level[i], K * ref_levels$level[i]),
          type = 'scatter',
          mode = 'lines',
          name = ref_levels$label[i],
          line = list(color = 'gray', width = 1, dash = 'dot'),
          hovertemplate = paste0(
            '<b>',
            ref_levels$label[i],
            ' of K</b><br>',
            '<b>Value:</b> ',
            round(K * ref_levels$level[i]),
            '<br>',
            '<extra></extra>'
          ),
          showlegend = FALSE
        )

      # Add annotations
      p <- p %>%
        plotly::add_annotations(
          x = min(complete$year),
          y = K * ref_levels$level[i],
          text = ref_levels$label[i],
          xanchor = 'left',
          yanchor = 'middle',
          showarrow = FALSE,
          font = list(size = 10, color = 'gray')
        )
    }

    # Layout
    p <- p %>%
      plotly::layout(
        title = list(
          text = sprintf(
            "Cumulative Growth Curve<br><sup>Saturation (K) = %.0f publications</sup>",
            K
          ),
          x = 0.5,
          xanchor = 'center'
        ),
        xaxis = list(
          title = "Year",
          showgrid = FALSE
        ),
        yaxis = list(
          title = "Cumulative Publications",
          showgrid = TRUE,
          gridcolor = '#f0f0f0',
          rangemode = 'tozero',
          range = c(0, K * 1.05)
        ),
        hovermode = 'closest',
        showlegend = TRUE,
        legend = list(
          x = 0.02,
          y = 0.98,
          bgcolor = 'rgba(255, 255, 255, 0.8)',
          bordercolor = '#ddd',
          borderwidth = 1
        ),
        plot_bgcolor = 'white',
        paper_bgcolor = 'white'
      )
  }

  return(p)
}

### AUTHOR BIO SKETCH ----

#### GLOBAL PROFILE ----
# Function to get all unique authors from papers with valid DOI only
get_all_authors <- function(df, separator = ";") {
  authors_column <- df$AU
  doi_column <- if ("DI" %in% names(df)) df$DI else NULL

  # If doi_column is provided, filter authors based on valid DOI
  if (!is.null(doi_column)) {
    # Identify entries with valid DOI (not NA and not "<NA>")
    valid_doi <- !is.na(doi_column) & doi_column != "<NA>" & doi_column != ""

    # Filter authors column based on valid DOI
    authors_column <- authors_column[valid_doi]
  }

  # Handle NA and empty strings in authors column
  valid_entries <- authors_column[!is.na(authors_column) & authors_column != ""]

  # Return empty character vector if no valid entries
  if (length(valid_entries) == 0) {
    return(character(0))
  }

  # Split all author strings using the specified separator
  all_authors <- unlist(strsplit(valid_entries, separator, fixed = TRUE))

  # Remove leading and trailing whitespace from each author name
  all_authors_clean <- trimws(all_authors)

  # Remove empty elements that might result from splitting
  all_authors_clean <- all_authors_clean[all_authors_clean != ""]

  # Remove [ANONYMOUS] entries
  all_authors_clean <- all_authors_clean[all_authors_clean != "[ANONYMOUS]"]

  # Return unique authors only
  return(unique(all_authors_clean %>% sort()))
}


authorCard <- function(selected_author, values) {
  req(selected_author)
  works_exact <- findAuthorWorks(
    selected_author,
    values$M,
    exact_match = TRUE
  ) %>%
    filter(!is.na(doi))
  if (nrow(works_exact) == 0) {
    return(HTML("No works found for this author.", type = "error"))
  }
  author_position <- works_exact$author_position[1]
  doi <- works_exact$doi[1]
  on_line <- check_online(method = "http")

  if (on_line) {
    if (!is.null(values$author_data)) {
      author_data <- values$author_data
    } else {
      author_data <- tibble(AUid = character(), display_name = character())
    }

    if (selected_author %in% author_data$AUid) {
      AU_data <- author_data %>% dplyr::filter(AUid == selected_author)
    } else {
      suppressWarnings(
        AU_data <- tryCatch(
          {
            oa_email <- Sys.getenv("openalexR.mailto", unset = "")
            authorBio(author_position = author_position, doi = doi,
                      mailto = if (nchar(oa_email) > 0) oa_email else NULL)
          },
          error = function(e) {
            NULL
          }
        )
      )
      # check if AUid is a tibble
      if (is.data.frame(AU_data)) {
        author_data <- bind_rows(
          author_data,
          AU_data %>%
            mutate(AUid = selected_author)
        )
        values$author_data <- author_data
      } else {
        return(HTML("No author data found.", type = "error"))
      }
    }
    # values$author_data <- author_data
    card <- create_author_bio_card(
      AU_data,
      width = "100%",
      show_trends = TRUE,
      show_topics = TRUE,
      max_topics = 20
    )
  } else {
    card <- HTML(
      "No internet connection. Unable to fetch author data.",
      type = "error"
    )
  }
}

create_empty_author_bio_card <- function(
  author_name = "Author Name",
  width = "100%",
  message = "Author data not found or not yet retrieved"
) {
  # Metrics cards with placeholder values
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "2yr Mean Cit.",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Empty trend chart placeholder
  trend_chart <- tagList(
    h4(
      "Publication Trends (Last 10 Years)",
      style = "margin-top: 20px; color: #95A5A6;"
    ),
    div(
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 display: flex; align-items: center; justify-content: center;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$i(
          class = "fa fa-chart-bar",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No trend data available"
      )
    )
  )

  # Empty topics section
  topics_section <- tagList(
    h4("Main Research Topics", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "topics-container",
      style = "text-align: center; padding: 20px; 
                                            background: #f8f9fa; border-radius: 8px;",
      div(
        style = "color: #95A5A6;",
        tags$i(
          class = "fa fa-tags",
          style = "font-size: 36px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No research topics available"
      )
    )
  )

  # Main card UI
  div(
    class = "author-bio-card-empty",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.1); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;
                   opacity: 0.8; border: 2px dashed #bdc3c7;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      fluidRow(
        column(
          8,
          h2(
            author_name,
            style = "margin: 0 0 10px 0; color: #95A5A6; font-weight: 600;"
          ),
          h5(
            "Institution not available",
            style = "margin: 0 0 5px 0; color: #BDC3C7; font-weight: 400;"
          ),
          p(
            "📍 Country not available",
            style = "margin: 0 0 10px 0; color: #BDC3C7;"
          )
        ),
        column(
          4,
          div(
            style = "text-align: right; padding-top: 10px;",
            div(
              tags$span("ORCID not available", style = "color: #95A5A6;"),
              style = "margin-bottom: 8px;"
            ),
            div(tags$span(
              "OpenAlex Profile not available",
              style = "color: #95A5A6;"
            ))
          )
        )
      )
    ),

    # Info message
    div(
      style = "margin: 20px 0; padding: 15px; background: #fff3cd; border: 1px solid #ffeeba; 
                 border-radius: 8px; color: #856404;",
      tags$i(class = "fa fa-info-circle", style = "margin-right: 8px;"),
      strong("Information: "),
      message
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #95A5A6;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(6, strong("i10-Index: "), span("--", style = "color: #95A5A6;")),
        column(
          6,
          strong("OpenAlex ID: "),
          span(
            "Not available",
            style = "color: #95A5A6; font-family: monospace; font-size: 12px;"
          )
        )
      )
    ),

    # Trends and topics placeholders
    trend_chart,
    topics_section,

    # Footer
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Please retrieve author data to view bibliometric information -",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

create_author_bio_card <- function(
  author_data,
  width = "100%",
  show_trends = TRUE,
  show_topics = TRUE,
  max_topics = 5
) {
  # Extract key information safely
  author_name <- author_data$display_name[1] %||% "Unknown Author"
  institution <- author_data$primary_affiliation[1] %||%
    author_data$last_known_institutions[[1]]$display_name[1] %||%
    "Institution not available"
  institution_ror <- author_data$primary_affiliation_ror[1] %||%
    author_data$last_known_institutions[[1]]$ror[1] %||%
    NA
  country <- author_data$primary_affiliation_country[1] %||%
    author_data$last_known_institutions[[1]]$country_code[1] %||%
    "Country not available"

  works_count <- author_data$works_count[1] %||% 0
  citations <- author_data$cited_by_count[1] %||% 0
  h_index <- author_data$h_index[1] %||% 0
  i10_index <- author_data$i10_index[1] %||% 0
  mean_citedness <- author_data$`2yr_mean_citedness`[1] %||% 0

  orcid <- author_data$orcid[1]
  position_type <- author_data$author_position_type[1] %||% "author"
  is_corresponding <- author_data$is_corresponding[1] %||% FALSE

  # Create ORCID link if available
  orcid_link <- if (!is.null(orcid) && !is.na(orcid)) {
    tags$a(
      href = orcid,
      target = "_blank",
      tags$img(src = "ORCID.jpg", style = "height: 16px; margin-right: 5px;"),
      "ORCID Profile",
      style = "text-decoration: none; color: #338B13;"
    )
  } else {
    tags$span("ORCID not available", style = "color: #666;")
  }

  # Create OpenAlex link
  openalex_id <- gsub("https://openalex.org/", "", author_data$id[1])
  openalex_link <- tags$a(
    href = paste0("https://openalex.org/", openalex_id),
    target = "_blank",
    tags$img(src = "openalex.jpg", style = "height: 16px; margin-right: 5px;"),
    "OpenAlex Profile",
    style = "text-decoration: none; color: #E74C3C;"
  )

  # Format numbers with thousands separator
  format_number <- function(x) {
    if (is.null(x) || is.na(x)) {
      return("0")
    }
    format(x, big.mark = ",", scientific = FALSE)
  }

  # Create metrics cards
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(works_count), style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #f093fb 0%, #f5576c 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(citations), style = "margin: 0; font-size: 24px;"),
        p(
          "Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #4facfe 0%, #00f2fe 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(h_index, style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #43e97b 0%, #38f9d7 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(round(mean_citedness, 1), style = "margin: 0; font-size: 24px;"),
        p(
          "2yr Mean Cit.",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Create publication trend chart if requested
  trend_chart <- if (show_trends && !is.null(author_data$counts_by_year[[1]])) {
    counts_data <- author_data$counts_by_year[[1]]
    recent_data <- counts_data[
      counts_data$year >= (max(counts_data$year) - 9),
    ]
    # Order by year from oldest to newest
    recent_data <- recent_data[order(recent_data$year), ]

    tagList(
      h4(
        "Publication Trends (Last 10 Years)",
        style = "margin-top: 20px; color: #2C3E50;"
      ),
      div(
        style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px;",
        # Simple bar chart representation
        div(
          style = "display: flex; align-items: end; height: 170px; gap: 3px;",
          lapply(1:nrow(recent_data), function(i) {
            height_pct <- (recent_data$works_count[i] /
              max(recent_data$works_count, na.rm = TRUE)) *
              100
            div(
              style = paste0(
                "background: linear-gradient(to top, #3498db, #2980b9); 
                                   height: ",
                height_pct,
                "%; 
                                   width: ",
                100 / nrow(recent_data) - 1,
                "%; 
                                   border-radius: 3px 3px 0 0;
                                   position: relative;"
              ),
              div(
                style = "position: absolute; bottom: -20px; font-size: 10px; 
                               width: 100%; text-align: center; color: #666;",
                recent_data$year[i]
              ),
              div(
                style = "position: absolute; top: -15px; font-size: 9px; 
                               width: 100%; text-align: center; color: #333; font-weight: bold;",
                recent_data$works_count[i]
              )
            )
          })
        )
      )
    )
  } else {
    NULL
  }

  # Create research topics section if requested
  topics_section <- if (show_topics && !is.null(author_data$topics[[1]])) {
    topics_data <- author_data$topics[[1]]
    top_topics <- topics_data[topics_data$type == "topic", ][
      1:min(max_topics, sum(topics_data$type == "topic")),
    ] %>%
      slice_sample(prop = 1)

    # Calculate font sizes based on counts
    if (nrow(top_topics) > 0) {
      min_count <- min(top_topics$count, na.rm = TRUE)
      max_count <- max(top_topics$count, na.rm = TRUE)
      min_font_size <- 10
      max_font_size <- 18

      # Calculate proportional font sizes
      font_sizes <- if (max_count == min_count) {
        rep(min_font_size, nrow(top_topics))
      } else {
        min_font_size +
          (top_topics$count - min_count) /
            (max_count - min_count) *
            (max_font_size - min_font_size)
      }
    }

    tagList(
      h4("Main Research Topics", style = "margin-top: 20px; color: #2C3E50;"),
      div(
        class = "topics-container",
        style = "text-align: center;",
        lapply(1:nrow(top_topics), function(i) {
          if (!is.na(top_topics$display_name[i]) && !is.na(top_topics$id[i])) {
            tags$a(
              href = top_topics$id[i],
              target = "_blank",
              class = "topic-badge",
              style = paste0(
                "display: inline-block; background: ",
                colorlist()[i],
                # sample(
                # c("#3498db", "#e74c3c", "#2ecc71", "#f39c12", "#9b59b6"),
                # 1),
                "; opacity: 0.7; color: white; padding: 5px 10px; margin: 3px; 
                            border-radius: 15px; font-weight: 500; text-decoration: none;
                            font-size: ",
                round(font_sizes[i], 1),
                "px;"
              ),
              paste0(top_topics$display_name[i], " (", top_topics$count[i], ")")
            )
          }
        })
      )
    )
  } else {
    NULL
  }

  institution_link <- tags$a(
    href = institution_ror,
    target = "_blank",
    institution
    # ,style = "text-decoration: none; color: #E74C3C;"
  )

  # Main card UI
  div(
    class = "author-bio-card",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.15); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      fluidRow(
        column(
          8,
          h2(
            author_name,
            style = "margin: 0 0 10px 0; color: #2C3E50; font-weight: 600;"
          ),
          h5(
            institution_link,
            style = "margin: 0 0 5px 0; color: #7F8C8D; font-weight: 400;"
          ),
          p(paste("📍", country), style = "margin: 0 0 10px 0; color: #95A5A6;")
        ),
        column(
          4,
          div(
            style = "text-align: right; padding-top: 10px;",
            div(orcid_link, style = "margin-bottom: 8px;"),
            div(openalex_link)
          )
        )
      )
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #2C3E50;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(
          6,
          strong("i10-Index: "),
          span(i10_index, style = "color: #2980b9;")
        ),
        column(
          6,
          strong("OpenAlex ID: "),
          span(
            openalex_id,
            style = "color: #666; font-family: monospace; font-size: 12px;"
          )
        )
      )
    ),

    # Trends and topics
    trend_chart,
    topics_section,

    # Footer with source information
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Data retrieved from OpenAlex on",
        format(author_data$query_timestamp[1], "%Y-%m-%d %H:%M")
      )
    )
  )
}

# Wrapper function for use in Shiny renderUI

render_author_bio_card <- function(author_data, ...) {
  renderUI({
    create_author_bio_card(author_data, ...)
  })
}

#### LOCAL PROFILE ----
create_local_author_bio_card <- function(
  local_author_data,
  selected_author,
  max_py = 2024,
  width = "100%",
  show_trends = TRUE,
  show_keywords = TRUE,
  max_keywords = 8,
  max_works_display = 100
) {
  # Extract author information
  author_name <- to_title_case(selected_author)

  # Calculate metrics from local data
  works_count <- nrow(local_author_data)
  total_citations <- sum(as.numeric(local_author_data$TC), na.rm = TRUE)
  years <- as.numeric(local_author_data$PY)
  current_year <- as.numeric(format(Sys.Date(), "%Y"))

  # Calculate mean citations per year (weighted by age of publications)
  publication_ages <- current_year - years
  publication_ages[publication_ages <= 0] <- 1 # Avoid division by zero
  mean_citations_per_year <- mean(
    as.numeric(local_author_data$TC) / publication_ages,
    na.rm = TRUE
  )

  # Calculate additional metrics
  years_active <- max(years, na.rm = TRUE) - min(years, na.rm = TRUE) + 1
  avg_citations_per_work <- total_citations / works_count

  # Calculate h-index (simplified version)
  citations_sorted <- sort(as.numeric(local_author_data$TC), decreasing = TRUE)
  h_index <- sum(citations_sorted >= seq_along(citations_sorted))

  # Get most recent years for activity
  recent_years <- years[years >= (max_py - 4)]
  recent_productivity <- length(recent_years)

  # Format numbers with thousands separator
  format_number <- function(x) {
    if (is.null(x) || is.na(x)) {
      return("0")
    }
    format(round(x), big.mark = ",", scientific = FALSE)
  }

  # Create metrics cards
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(works_count), style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #f093fb 0%, #f5576c 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(
          format_number(total_citations),
          style = "margin: 0; font-size: 24px;"
        ),
        p(
          "Total Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #4facfe 0%, #00f2fe 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(h_index, style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index (Local)",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #43e97b 0%, #38f9d7 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(
          round(mean_citations_per_year, 1),
          style = "margin: 0; font-size: 24px;"
        ),
        p(
          "Cit./Year",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Create publication trend chart if requested
  trend_chart <- if (show_trends && length(years) > 0) {
    # Create yearly publication counts
    year_counts <- table(years)
    year_df <- data.frame(
      year = as.numeric(names(year_counts)),
      count = as.numeric(year_counts)
    )
    # Get recent 10 years
    recent_years_range <- max(year_df$year) - 9
    recent_data <- year_df[year_df$year >= recent_years_range, ]
    # Fill missing years with 0
    all_years <- recent_years_range:max(year_df$year)
    complete_data <- data.frame(year = all_years)
    complete_data <- merge(complete_data, recent_data, all = TRUE)
    complete_data$count[is.na(complete_data$count)] <- 0
    complete_data <- complete_data[order(complete_data$year), ]

    if (nrow(complete_data) > 0) {
      tagList(
        h4(
          "Publication Trends (Last 10 Years)",
          style = "margin-top: 20px; color: #2C3E50;"
        ),
        div(
          style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px;",
          div(
            style = "display: flex; align-items: end; height: 170px; gap: 3px;",
            lapply(1:nrow(complete_data), function(i) {
              max_count <- max(complete_data$count, na.rm = TRUE)
              height_pct <- if (max_count > 0) {
                (complete_data$count[i] / max_count) * 100
              } else {
                0
              }
              div(
                style = paste0(
                  "background: linear-gradient(to top, #3498db, #2980b9); 
                                     height: ",
                  max(height_pct, 2),
                  "%; 
                                     width: ",
                  100 / nrow(complete_data) - 1,
                  "%; 
                                     border-radius: 3px 3px 0 0;
                                     position: relative;"
                ),
                div(
                  style = "position: absolute; bottom: -20px; font-size: 10px; 
                                 width: 100%; text-align: center; color: #666;",
                  complete_data$year[i]
                ),
                div(
                  style = "position: absolute; top: -15px; font-size: 9px; 
                                 width: 100%; text-align: center; color: #333; font-weight: bold;",
                  complete_data$count[i]
                )
              )
            })
          )
        )
      )
    }
  } else {
    NULL
  }

  # Create keywords section if requested
  keywords_section <- if (show_keywords && "DE" %in% names(local_author_data)) {
    # Extract and process keywords
    all_keywords <- unlist(strsplit(
      paste(local_author_data$DE, collapse = ";"),
      ";"
    ))
    all_keywords <- to_title_case(trimws(toupper(all_keywords)))
    all_keywords <- all_keywords[all_keywords != "" & !is.na(all_keywords)]

    if (length(all_keywords) > 0) {
      keyword_freq <- sort(table(all_keywords), decreasing = TRUE)
      top_keywords <- head(keyword_freq, max_keywords)
      top_keywords <- top_keywords[sample(length(top_keywords))] # Shuffle order

      # Calculate font sizes based on frequency
      if (length(top_keywords) > 0) {
        min_freq <- min(top_keywords)
        max_freq <- max(top_keywords)
        min_font_size <- 10
        max_font_size <- 18

        font_sizes <- if (max_freq == min_freq) {
          rep(min_font_size, length(top_keywords))
        } else {
          min_font_size +
            (top_keywords - min_freq) /
              (max_freq - min_freq) *
              (max_font_size - min_font_size)
        }
      }

      tagList(
        h4("Main Keywords", style = "margin-top: 20px; color: #2C3E50;"),
        div(
          class = "keywords-container",
          style = "text-align: center;",
          lapply(1:length(top_keywords), function(i) {
            colors <- c(
              "#3498db",
              "#e74c3c",
              "#2ecc71",
              "#f39c12",
              "#9b59b6",
              "#1abc9c",
              "#34495e",
              "#e67e22"
            )
            color <- colors[((i - 1) %% length(colors)) + 1]

            tags$span(
              class = "keyword-badge",
              style = paste0(
                "display: inline-block; background: ",
                color,
                "; opacity: 0.8; color: white; padding: 5px 12px; margin: 3px; 
                            border-radius: 15px; font-weight: 500;
                            font-size: ",
                round(font_sizes[i], 1),
                "px;"
              ),
              paste0(names(top_keywords)[i], " (", top_keywords[i], ")")
            )
          })
        )
      )
    }
  } else {
    NULL
  }

  # Create works list (scrollable)
  works_section <- if (nrow(local_author_data) > 0) {
    # Prepare works data
    works_to_show <- head(
      local_author_data[
        order(as.numeric(local_author_data$PY), decreasing = TRUE),
      ],
      max_works_display
    )

    works_list <- lapply(1:nrow(works_to_show), function(i) {
      work <- works_to_show[i, ]
      title <- work$TI %||% "Title not available"
      journal <- work$SO %||% "Journal not available"
      year <- work$PY %||% "Year not available"
      doi <- work$DI %||% ""
      citations <- as.numeric(work$TC) %||% 0

      # Create DOI link if available
      doi_link <- if (!is.null(doi) && !is.na(doi) && doi != "") {
        tags$a(
          href = paste0("https://doi.org/", doi),
          target = "_blank",
          style = "color: #3498db; text-decoration: none; font-size: 11px;",
          paste0("DOI: ", doi)
        )
      } else {
        tags$span(
          "DOI not available",
          style = "color: #95a5a6; font-size: 11px;"
        )
      }

      div(
        class = "work-item",
        style = "border-bottom: 1px solid #ecf0f1; padding: 12px 0; margin: 0;",
        div(
          style = "margin-bottom: 8px;",
          tags$h6(
            title,
            style = "margin: 0 0 5px 0; color: #2c3e50; font-weight: 600; line-height: 1.3;"
          )
        ),
        div(
          style = "margin-bottom: 8px;",
          tags$span(
            journal,
            style = "color: #7f8c8d; font-style: italic; font-size: 13px; margin-right: 15px;"
          ),
          tags$span(
            paste0("(", year, ")"),
            style = "color: #95a5a6; font-size: 13px; margin-right: 15px;"
          ),
          tags$span(
            paste0("Citations: ", format_number(citations)),
            style = "color: #e74c3c; font-weight: 500; font-size: 12px;"
          )
        ),
        div(style = "margin-top: 5px;", doi_link)
      )
    })

    tagList(
      h4(
        paste0("Publications (", nrow(local_author_data), " total)"),
        style = "margin-top: 20px; color: #2C3E50;"
      ),
      div(
        class = "works-container",
        style = "max-height: 400px; overflow-y: auto; background: #f8f9fa; 
                   border-radius: 8px; padding: 15px; border: 1px solid #e9ecef;",
        works_list
      )
    )
  } else {
    NULL
  }

  # Main card UI
  div(
    class = "local-author-bio-card",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.15); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      h2(
        author_name,
        style = "margin: 0 0 10px 0; color: #2C3E50; font-weight: 600;"
      ),
      p(
        "📊 Local Collection Profile",
        style = "margin: 0 0 10px 0; color: #7F8C8D; font-style: italic;"
      )
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Local Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #2C3E50;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(
          4,
          strong("Years Active: "),
          span(years_active, style = "color: #2980b9;")
        ),
        column(
          4,
          strong("Avg Cit./Work: "),
          span(round(avg_citations_per_work, 1), style = "color: #27ae60;")
        ),
        column(
          4,
          strong("Recent Activity (5yr): "),
          span(recent_productivity, style = "color: #e74c3c;")
        )
      )
    ),

    # Trends and keywords
    trend_chart,
    keywords_section,

    # Works section
    works_section,

    # Footer with source information
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Local collection data analyzed on",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

create_empty_local_author_bio_card <- function(
  author_name = "Author Name Not Available",
  width = "100%",
  message = "Local author data not found or not yet processed"
) {
  # Metrics cards with placeholder values
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Total Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index (Local)",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Cit./Year",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Empty trend chart placeholder
  trend_chart <- tagList(
    h4(
      "Publication Trends (Last 10 Years)",
      style = "margin-top: 20px; color: #95A5A6;"
    ),
    div(
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 display: flex; align-items: center; justify-content: center; border: 1px solid #e9ecef;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$div(
          "📊",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No trend data available"
      )
    )
  )

  # Empty keywords section
  keywords_section <- tagList(
    h4("Main Keywords", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "keywords-container",
      style = "text-align: center; padding: 20px; 
                                             background: #f8f9fa; border-radius: 8px; border: 1px solid #e9ecef;",
      div(
        style = "color: #95A5A6;",
        tags$div(
          "🏷️",
          style = "font-size: 36px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No keywords available"
      )
    )
  )

  # Empty works section
  works_section <- tagList(
    h4("Publications (0 total)", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "works-container",
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 border: 1px solid #e9ecef; display: flex; align-items: center; justify-content: center;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$div(
          "📄",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No publications available",
        br(),
        tags$small(
          "Publications will appear here when local data is processed",
          style = "font-style: italic; opacity: 0.7;"
        )
      )
    )
  )

  # Main card UI
  div(
    class = "local-author-bio-card-empty",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.1); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;
                   opacity: 0.8; border: 2px dashed #bdc3c7;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      h2(
        author_name,
        style = "margin: 0 0 10px 0; color: #95A5A6; font-weight: 600;"
      ),
      p(
        "📊 Local Collection Profile",
        style = "margin: 0 0 10px 0; color: #BDC3C7; font-style: italic;"
      )
    ),

    # Info message
    div(
      style = "margin: 20px 0; padding: 15px; background: #fff3cd; border: 1px solid #ffeeba; 
                 border-radius: 8px; color: #856404;",
      tags$span("ℹ️", style = "margin-right: 8px; font-size: 16px;"),
      strong("Information: "),
      message
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Local Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #95A5A6;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px; border: 1px solid #e9ecef;",
      fluidRow(
        column(
          4,
          strong("Years Active: "),
          span("--", style = "color: #95A5A6;")
        ),
        column(
          4,
          strong("Avg Cit./Work: "),
          span("--", style = "color: #95A5A6;")
        ),
        column(
          4,
          strong("Recent Activity (5yr): "),
          span("--", style = "color: #95A5A6;")
        )
      )
    ),

    # Trends, keywords, and works placeholders
    trend_chart,
    keywords_section,
    works_section,

    # Footer
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Please process local collection data to view bibliometric information -",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

# Helper function for safe extraction (null coalescing operator)
`%||%` <- function(x, y) if (is.null(x) || is.na(x) || length(x) == 0) y else x

# Helper: return a working PNG device function.
#
# `capabilities()` reports what R was COMPILED with, not what currently works
# on the host. Two real-world cases that bit users:
#   * Windows ARM64: capabilities("png") == TRUE but grDevices::png() fails.
#   * macOS without XQuartz on R >= 4.6: capabilities("cairo") == TRUE but
#     png(type = "cairo") aborts because libXrender is missing.
#
# We therefore PROBE each candidate by opening a 1x1 PNG to a tempfile and
# checking that a non-empty file was actually written. The first candidate
# that survives the probe is cached for the session.
.safe_png_device_cache <- new.env(parent = emptyenv())
safe_png_device <- function(force = FALSE) {
  if (!force && !is.null(.safe_png_device_cache$dev)) {
    return(.safe_png_device_cache$dev)
  }

  # Build candidate list in order of preference.
  candidates <- list()

  if (requireNamespace("ragg", quietly = TRUE)) {
    candidates[["ragg"]] <- ragg::agg_png
  }

  is_macos <- (Sys.info()[["sysname"]] == "Darwin")
  if (is_macos && capabilities("aqua")) {
    candidates[["quartz"]] <- function(filename, width, height,
                                       res = 72, units = "in", ...) {
      grDevices::png(filename = filename, width = width, height = height,
                     res = res, units = units, type = "quartz", ...)
    }
  }

  if (capabilities("png")) {
    # Default (system) png(): "quartz" on aqua macOS, "cairo" if linked, "Xlib"
    # on plain X11. R picks whichever was compiled in.
    candidates[["default"]] <- grDevices::png
  }

  if (capabilities("cairo")) {
    candidates[["cairo"]] <- function(filename, width, height,
                                      res = 72, units = "in", ...) {
      grDevices::png(filename = filename, width = width, height = height,
                     res = res, units = units, type = "cairo", ...)
    }
  }

  if (length(candidates) == 0) {
    stop("No PNG graphics device available. Install the 'ragg' package: install.packages('ragg')")
  }

  for (name in names(candidates)) {
    fn <- candidates[[name]]
    test_file <- tempfile(fileext = ".png")
    ok <- tryCatch({
      fn(filename = test_file, width = 2, height = 2, res = 72, units = "in")
      # quartz/ragg only flush a file when something is actually drawn.
      grid::grid.rect(gp = grid::gpar(col = NA, fill = "white"))
      grDevices::dev.off()
      file.exists(test_file) && file.info(test_file)$size > 0
    }, error = function(e) {
      try(grDevices::dev.off(), silent = TRUE)
      FALSE
    })
    unlink(test_file)
    if (isTRUE(ok)) {
      .safe_png_device_cache$dev <- fn
      .safe_png_device_cache$name <- name
      return(fn)
    }
  }

  stop("No working PNG graphics device available. Install the 'ragg' package: install.packages('ragg')")
}

# Safe wrapper for ggsave that uses a working PNG device
safe_ggsave <- function(..., device = NULL) {
  args <- list(...)
  filename <- args$filename %||% args[[1]]
  if (is.null(device) && grepl("\\.png$", filename, ignore.case = TRUE)) {
    device <- safe_png_device()
  }
  ggplot2::ggsave(..., device = device)
}


# from igraph to png file
igraph2PNG <- function(x, filename, width = 10, height = 7, dpi = 75) {
  V(x)$centr <- centr_betw(x)$res
  df <- data.frame(
    name = V(x)$label,
    cluster = V(x)$color,
    centr = V(x)$centr
  ) %>%
    group_by(cluster) %>%
    slice_head(n = 3)
  V(x)$label[!(V(x)$label %in% df$name)] <- ""
  png_dev <- safe_png_device()
  png_dev(
    filename = filename,
    width = width,
    height = height,
    units = "in",
    res = dpi
  )
  grid::grid.draw(plot(x))
  dev.off()
}

# from ggplot to plotly
plot.ly <- function(
  g,
  flip = FALSE,
  side = "r",
  aspectratio = 1,
  size = 0.15,
  data.type = 2,
  height = 0,
  customdata = NA
) {
  g <- g + labs(title = NULL)

  gg <- ggplotly(g, tooltip = "text") %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        "toImage",
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    )

  return(gg)
}

freqPlot <- function(
  xx,
  x,
  y,
  textLaby,
  textLabx,
  title,
  values,
  string.max = 70
) {
  xl <- c(
    max(xx[, x]) - 0.02 - diff(range(xx[, x])) * 0.125,
    max(xx[, x]) - 0.02
  ) +
    1
  yl <- c(1, 1 + length(unique(xx[, y])) * 0.125)

  Text <- paste(textLaby, ": ", xx[, y], "\n", textLabx, ": ", xx[, x])

  if (title == "Most Local Cited References" & values$M$DB[1] == "SCOPUS") {
    xx[, y] <- gsub(
      "^(.+?)\\.,.*\\((\\d{4})\\)$",
      paste0("\\1", "., ", "\\2"),
      xx[, y]
    )
  }

  xx[, y] <- sapply(substr(xx[, y], 1, string.max), function(s) {
    if (nchar(s) > 40) paste0(stringr::str_wrap(s, width = 40)) else s
  })

  # Ensure unique labels to avoid multiple points on same y-axis row
  dupes <- duplicated(xx[, y]) | duplicated(xx[, y], fromLast = TRUE)
  if (any(dupes)) {
    counts <- ave(seq_along(xx[, y]), xx[, y], FUN = seq_along)
    xx[dupes, y] <- paste0(xx[dupes, y], " (", counts[dupes], ")")
  }

  g <- ggplot(xx, aes(x = xx[, x], y = xx[, y], label = xx[, x], text = Text)) +
    geom_segment(
      aes(x = 0, y = xx[, y], xend = xx[, x], yend = xx[, y]),
      color = "grey50"
    ) +
    geom_point(aes(color = -xx[, x], size = xx[, x]), show.legend = FALSE) +
    scale_radius(range = c(5, 12)) +
    geom_text(color = "#333333", size = 3.5, fontface = "bold", hjust = 0,
              nudge_x = max(xx[, x]) * 0.04) +
    scale_y_discrete(limits = rev(xx[, y])) +
    scale_x_continuous(expand = expansion(mult = c(0, 0.15))) +
    scale_fill_continuous(type = "gradient") +
    labs(title = title, y = textLaby) +
    labs(x = textLabx) +
    expand_limits(y = c(1, length(xx[, y]) + 1)) +
    theme_minimal() +
    theme(axis.text.y = element_text(angle = 0, hjust = 1, face = "bold", size = 11)) +
    annotation_custom(
      values$logoGrid,
      xmin = xl[1],
      xmax = xl[2],
      ymin = yl[1],
      ymax = yl[2]
    )

  return(g)
}

emptyPlot <- function(errortext) {
  g <- ggplot() +
    theme_void() +
    theme(legend.position = "none") +
    annotate("text", x = 4, y = 25, label = errortext, size = 10)
  plot(g)
}

count.duplicates <- function(DF) {
  x <- do.call("paste", c(DF, sep = "\r"))
  ox <- order(x)
  rl <- rle(x[ox])
  cbind(DF[ox[cumsum(rl$lengths)], , drop = FALSE], count = rl$lengths)
}

reduceRefs <- function(A) {
  ind <- unlist(regexec("*V[0-9]", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  ind <- unlist(regexec("*DOI ", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  return(A)
}

check_online <- function(
  host = "8.8.8.8",
  timeout = 5,
  # min_success = 1,
  method = "http" # method = c("ping", "socket", "http")
) {
  #method <- match.arg(method)

  if (method == "ping") {
    # Usa solo il codice di ritorno, non analizza l'output
    ping_cmd <- if (.Platform$OS.type == "windows") {
      sprintf("ping -n 1 -w %d %s", timeout * 1000, host)
    } else {
      sprintf("ping -c 1 -W %d %s", timeout, host)
    }
    exit_code <- suppressWarnings(system(
      ping_cmd,
      ignore.stdout = TRUE,
      ignore.stderr = TRUE
    ))
    return(exit_code == 0)
  } else if (method == "socket") {
    # Connessione TCP a DNS Google (porta 53)
    tryCatch(
      {
        con <- socketConnection(
          host = host,
          port = 53,
          blocking = TRUE,
          open = "r+",
          timeout = timeout
        )
        close(con)
        return(TRUE)
      },
      error = function(e) {
        return(FALSE)
      }
    )
  } else if (method == "http") {
    # Richiesta HTTP
    tryCatch(
      {
        # check if host start with http or https and add if missing
        if (!grepl("^https?://", host)) {
          host <- paste0("https://", host)
        }

        # con <- url("https://www.google.com", open = "rb")
        con <- url(host, open = "rb")
        on.exit(close(con))
        readLines(con, n = 1, warn = FALSE)
        return(TRUE)
      },
      error = function(e) {
        return(FALSE)
      }
    )
  }
}

# check_online <- function(host = "8.8.8.8", min_success = 1) {
#   # Use ping command to test connectivity (works on Windows, Linux, Mac)
#   ping_cmd <- if (.Platform$OS.type == "windows") {
#     sprintf("ping -n %d %s", min_success, host)
#   } else {
#     sprintf("ping -c %d %s", min_success, host)
#   }
#
#   result <- suppressWarnings(system(
#     ping_cmd,
#     intern = TRUE,
#     ignore.stderr = TRUE
#   ))
#
#   success <- any(grepl("time=", result, ignore.case = TRUE))
#
#   if (success) {
#     # message("✅ Host is reachable.")
#     # Extract average latency (optional)
#     latency_line <- result[grepl("time=", result)]
#     times <- as.numeric(sub(".*time=([0-9.]+).*", "\\1", latency_line))
#     avg_time <- mean(times, na.rm = TRUE)
#     # message(sprintf("📶 Average latency: %.1f ms", avg_time))
#     if (avg_time < 200) {
#       return(TRUE)
#     } else {
#       return(FALSE)
#     }
#     #return(TRUE)
#   } else {
#     #message(FALSE)
#     return(FALSE)
#   }
# }

notifications <- function() {
  ## check connection and download notifications
  online <- check_online(host = "www.bibliometrix.org", method = "http")
  location <- "https://www.bibliometrix.org/bs_notifications/biblioshiny_notifications.csv"
  notifOnline <- NULL
  if (isTRUE(online)) {
    notifOnline <- read.csv(location, header = TRUE, sep = ",")
    # ## add check to avoid blocked app when internet connection is to slow

    if (is.null(notifOnline)) {
      online <- FALSE
    } else {
      notifOnline$href[nchar(notifOnline$href) < 6] <- NA
    }
  }

  ## check if a file exists on the local machine and load it
  home <- homeFolder()

  file <- file.path(home, "biblioshiny_notifications.csv")
  fileTrue <- file.exists(file)
  if (isTRUE(fileTrue)) {
    suppressWarnings(notifLocal <- read.csv(file, header = TRUE, sep = ","))
  }

  A <- c("noA", "A")
  B <- c("noB", "B")
  status <- paste(A[online + 1], B[fileTrue + 1], sep = "")

  switch(
    status,
    # missing both files (online and local)
    noAnoB = {
      notifTot <- data.frame(
        nots = "No notifications",
        href = NA,
        status = "info"
      ) %>%
        mutate(status = "info")
    },
    # missing online file. The local one exists.
    noAB = {
      notifTot <- notifLocal %>%
        filter(action == TRUE) %>%
        mutate(status = "info")
    },
    # missing the local file. The online one exists.
    AnoB = {
      # notifOnline <- notifOnline %>%
      #   dplyr::slice_head(n = 5)
      notifTot <- notifOnline %>%
        dplyr::filter(action == TRUE) %>%
        mutate(status = "danger") %>%
        dplyr::slice_head(n = 5)
      notifOnline %>%
        dplyr::filter(action == TRUE) %>%
        write.csv(file = file, quote = FALSE, row.names = FALSE)
    },
    # both files exist.
    AB = {
      notifTot <- left_join(
        notifOnline %>% mutate(status = "danger"),
        notifLocal %>% mutate(status = "info"),
        by = "nots"
      ) %>%
        mutate(status = tidyr::replace_na(status.y, "danger")) %>%
        rename(
          href = href.x,
          action = action.x
        ) %>%
        select(nots, href, action, status) %>%
        arrange(status) %>%
        dplyr::filter(action == TRUE) %>%
        dplyr::slice_head(n = 5)
      notifTot %>%
        select(-status) %>%
        write.csv(file = file, quote = FALSE, row.names = FALSE)
    }
  )

  return(notifTot)
}

is_Online <- function(timeout = 3, url = "https://www.bibliometrix.org") {
  #RCurl::url.exists(url, timeout = timeout)
  check_online(host = url, timeout = timeout, method = "http")
}

initial <- function(values) {
  values$results <- list("NA")
  values$log <- "working..."
  values$load <- "FALSE"
  values$field <- values$cocngrams <- "NA"
  values$citField <- values$colField <- values$citSep <- "NA"
  values$NetWords <- values$NetRefs <- values$ColNetRefs <- matrix(NA, 1, 1)
  values$Title <- "Network"
  values$Histfield <- "NA"
  values$histlog <- "working..."
  values$kk <- 0
  values$histsearch <- "NA"
  values$citShortlabel <- "NA"
  values$S <- list("NA")
  values$GR <- "NA"
  values$nMerge <- NULL
  values$collection_description <- NULL
  ### column to export in TALL
  values$corpusCol <- c(
    "Title" = "TI",
    "Abstract" = "AB",
    "Author's Keywords" = "DE"
  )
  values$metadataCol <- c(
    "Publication Year" = "PY",
    "Document Type" = "DT",
    "DOI" = "DI",
    "Open Access" = "OA",
    "Language" = "LA",
    "First Author" = "AU1"
  )

  # Chrome enviroment variable
  if (inherits(try(pagedown::find_chrome(), silent = T), "try-error")) {
    values$Chrome_url <- NULL
  } else {
    values$Chrome_url <- pagedown::find_chrome()
  }

  return(values)
}

resetAnalysis <- function() {
  # reset menus
  output$lifeCycleSummaryUIid <- renderUI({
    div()
  })
  output$DLCPlotYear <- renderUI({
    div()
  })

  output$DLCPlotCum <- renderUI({
    div()
  })
}

### TALL Export functions ----
tallExport <- function(M, tallFields, tallMetadata, metadataCol) {
  corpus <- NULL
  ## Corpus Fields ##
  if ("Abstract" %in% tallFields) {
    if (!"AB_raw" %in% names(M)) {
      M <- M %>%
        mutate(AB_raw = sapply(AB, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "AB_raw")
  }

  if ("Title" %in% tallFields) {
    if (!"TI_raw" %in% names(M)) {
      M <- M %>%
        mutate(TI_raw = sapply(TI, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "TI_raw")
  }

  if ("Author's Keywords" %in% tallFields) {
    if (!"DE_raw" %in% names(M)) {
      M <- M %>%
        mutate(DE_raw = sapply(DE, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "DE_raw")
  }

  corpus <- c(corpus, as.character(metadataCol[tallMetadata]))

  M <- M %>%
    select(SR, any_of(corpus)) %>%
    rename(doc_id = SR)
  names(M) <- gsub("_raw", "", names(M))

  return(M)
}

capitalize_after_dot <- function(text) {
  # Tutto minuscolo
  text <- tolower(text)

  # Prima lettera della stringa maiuscola
  text <- paste0(toupper(substr(text, 1, 1)), substr(text, 2, nchar(text)))

  # Maiuscola dopo punto (o ! o ?) + spazio
  text <- gsub("([\\.\\!\\?]\\s*)([a-z])", "\\1\\U\\2", text, perl = TRUE)

  return(text)
}


### ANALYSIS FUNCTIONS ####
### Descriptive functions ----

ValueBoxes <- function(M) {
  # calculate statistics for Biblioshiny ValueBoxes
  df <- data.frame(Description = rep(NA, 12), Results = rep(NA, 12))

  ## VB  1 - Time span
  df[1, ] <- c("Timespan", paste(range(M$PY, na.rm = T), collapse = ":"))

  ## VB  2 - Authors
  listAU <- (strsplit(M$AU, ";"))
  nAU <- lengths(listAU)
  listAU <- unique(trimws((unlist(listAU))))
  listAU <- listAU[!is.na(listAU)]
  df[2, ] <- c("Authors", length(listAU))

  ## VB  3 - Author's Keywords (DE)
  if (!"DE" %in% names(M)) {
    M$DE <- ""
  }
  DE <- unique(trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$DE, ";")))))
  DE <- DE[!is.na(DE)]

  df[3, ] <- c("Author's Keywords (DE)", length(DE))

  ## VB  4 - Sources
  df[4, ] <- c("Sources (Journals, Books, etc)", length(unique(M$SO)))

  ## VB  5 - Authors of single-authored docs

  df[5, ] <- c(
    "Authors of single-authored docs",
    length(unique(M$AU[nAU == 1]))
  )

  ## VB  6 - References
  CR <- trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$CR, ";"))))
  CR <- CR[nchar(CR) > 0 & !is.na(CR)]
  nCR <- length(unique(CR))
  if (nCR == 1) {
    nCR <- 0
  }
  df[6, ] <- c("References", nCR)

  ## VB  7 - Documents
  df[7, ] <- c("Documents", nrow(M))

  ## VB  8 - International Co-Authorship
  if (!"AU_CO" %in% names(M)) {
    M <- metaTagExtraction(M, "AU_CO")
  }
  AU_CO <- strsplit(M$AU_CO, ";")
  Coll <- unlist(lapply(AU_CO, function(l) {
    length(unique(l)) > 1
  }))
  Coll <- sum(Coll) / nrow(M) * 100
  df[8, ] <- c("International co-authorships %", format(Coll, digits = 4))

  ## VB  9 - Document Average Age
  age <- as.numeric(substr(Sys.Date(), 1, 4)) - M$PY
  df[9, ] <- c(
    "Document Average Age",
    format(mean(age, na.rm = TRUE), digits = 3)
  )

  ## VB 10 - Annual Growth Rate
  Y <- table(M$PY)
  ny <- diff(range(M$PY, na.rm = TRUE))
  CAGR <- as.numeric(round(((Y[length(Y)] / Y[1])^(1 / (ny)) - 1) * 100, 2))
  df[10, ] <- c("Annual Growth Rate %", CAGR)

  ## VB 11 - Co-Authors per Doc
  df[11, ] <- c("Co-Authors per Doc", format(mean(nAU, na.rm = T), digit = 3))

  ## VB 12 - Average citations per doc
  df[12, ] <- c(
    "Average citations per doc",
    format(mean(M$TC, na.rm = T), digit = 4)
  )

  DT <- M %>%
    mutate(DT = tolower(DT)) %>%
    count(DT) %>%
    rename(
      Description = DT,
      Results = n
    )

  # Indexed Keywords (ID)
  ID <- unique(trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$ID, ";")))))
  ID <- ID[!is.na(ID)]
  df[nrow(df) + 1, ] <- c("Keywords Plus (ID)", length(ID))

  # Single authored docs

  df[nrow(df) + 1, ] <- c("Single-authored docs", sum(nAU == 1))

  df2 <- data.frame(
    Description = c(
      "MAIN INFORMATION ABOUT DATA",
      "Timespan",
      "Sources (Journals, Books, etc)",
      "Documents",
      "Annual Growth Rate %",
      "Document Average Age",
      "Average citations per doc",
      "References",
      "DOCUMENT CONTENTS",
      "Keywords Plus (ID)",
      "Author's Keywords (DE)",
      "AUTHORS",
      "Authors",
      "Authors of single-authored docs",
      "AUTHORS COLLABORATION",
      "Single-authored docs",
      "Co-Authors per Doc",
      "International co-authorships %",
      "DOCUMENT TYPES"
    )
  )

  df <- left_join(df2, df, by = "Description") %>%
    rbind(DT) %>%
    mutate(Results = replace_na(Results, ""))

  return(df)
}

countryCollab <- function(M) {
  sep <- ";"
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU_CO", sep)
  }
  if (!("AU1_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU1_CO", sep)
  }

  M$nCO <- as.numeric(unlist(lapply(strsplit(M$AU_CO, ";"), function(l) {
    length(unique(l)) > 1
  })))

  M$AU1_CO <- trim(gsub("[[:digit:]]", "", M$AU1_CO))
  M$AU1_CO <- gsub("UNITED STATES", "USA", M$AU1_CO)
  M$AU1_CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", M$AU1_CO)
  M$AU1_CO <- gsub("TAIWAN", "CHINA", M$AU1_CO)
  M$AU1_CO <- gsub("ENGLAND", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("SCOTLAND", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("WALES", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", M$AU1_CO)

  df <- M %>%
    group_by(AU1_CO) %>%
    select(AU1_CO, nCO) %>%
    summarize(
      Articles = n(),
      SCP = sum(nCO == 0),
      MCP = sum(nCO == 1)
    ) %>%
    rename(Country = AU1_CO) %>%
    arrange(desc(Articles))

  return(df)
}

Hindex_plot <- function(values, type, input) {
  hindex <- function(values, type, input) {
    switch(
      type,
      author = {
        # AU <- trim(gsub(",","",names(tableTag(values$M,"AU"))))
        values$H <- Hindex(
          values$M,
          field = "author",
          elements = NULL,
          sep = ";",
          years = Inf
        )$H %>%
          arrange(desc(h_index))
      },
      source = {
        # SO <- names(sort(table(values$M$SO),decreasing = TRUE))
        values$H <- Hindex(
          values$M,
          field = "source",
          elements = NULL,
          sep = ";",
          years = Inf
        )$H %>%
          arrange(desc(h_index))
      }
    )

    return(values)
  }

  values <- hindex(values, type = type, input)

  xx <- values$H
  if (type == "author") {
    K <- input$Hkauthor
    measure <- input$HmeasureAuthors
    title <- "Authors' Local Impact"
    xn <- "Authors"
  } else {
    K <- input$Hksource
    measure <- input$HmeasureSources
    title <- "Sources' Local Impact"
    xn <- "Sources"
  }
  if (K > dim(xx)[1]) {
    k <- dim(xx)[1]
  } else {
    k <- K
  }

  switch(
    measure,
    h = {
      m <- 2
    },
    g = {
      m <- 3
    },
    m = {
      m <- 4
      xx[, m] <- round(xx[, m], 2)
    },
    tc = {
      m <- 5
    }
  )
  xx <- xx[order(-xx[, m]), ]
  xx <- xx[1:k, c(1, m)]

  g <- freqPlot(
    xx,
    x = 2,
    y = 1,
    textLaby = xn,
    textLabx = paste("Impact Measure:", toupper(measure)),
    title = paste(title, "by", toupper(measure), "index"),
    values
  )

  res <- list(values = values, g = g)
  return(res)
}

descriptive <- function(values, type) {
  switch(
    type,
    "tab2" = {
      TAB <- values$M %>%
        group_by(PY) %>%
        count() %>%
        rename(
          Year = PY,
          Articles = n
        ) %>%
        right_join(
          data.frame(
            Year = seq(
              min(values$M$PY, na.rm = TRUE),
              max(values$M$PY, na.rm = TRUE)
            )
          ),
          by = "Year"
        ) %>%
        mutate(Articles = replace_na(Articles, 0)) %>%
        arrange(Year) %>%
        as.data.frame()

      ny <- diff(range(TAB$Year))
      values$GR <- round(
        ((TAB[nrow(TAB), 2] / TAB[1, 2])^(1 / (ny)) - 1) * 100,
        digits = 2
      )
    },
    "tab3" = {
      listAU <- (strsplit(values$M$AU, ";"))
      nAU <- lengths(listAU)
      fracAU <- rep(1 / nAU, nAU)
      TAB <- tibble(Author = unlist(listAU), fracAU = fracAU) %>%
        group_by(Author) %>%
        summarize(
          Articles = n(),
          AuthorFrac = sum(fracAU)
        ) %>%
        arrange(desc(Articles)) %>%
        as.data.frame()
      names(TAB) <- c("Authors", "Articles", "Articles Fractionalized")
      # print(S$MostProdAuthors)
    },
    "tab4" = {
      y <- as.numeric(substr(Sys.Date(), 1, 4))
      TAB <- values$M %>%
        mutate(TCperYear = TC / (y + 1 - PY)) %>%
        select(SR, DI, TC, TCperYear, PY) %>%
        group_by(PY) %>%
        mutate(NTC = TC / mean(TC)) %>%
        ungroup() %>%
        select(-PY) %>%
        arrange(desc(TC)) %>%
        as.data.frame()
      names(TAB) <- c(
        "Paper",
        "DOI",
        "Total Citations",
        "TC per Year",
        "Normalized TC"
      )
    },
    "tab5" = {
      TAB <- countryCollab(values$M)
      TAB <- TAB %>%
        mutate(Freq = Articles / sum(Articles)) %>%
        mutate(MCP_Ratio = MCP / Articles) %>%
        drop_na(Country)
    },
    "tab6" = {
      if (!"AU1_CO" %in% names(values$M)) {
        values$M <- metaTagExtraction(values$M, "AU1_CO")
      }
      TAB <- values$M %>%
        select(AU1_CO, TC) %>%
        drop_na(AU1_CO) %>%
        rename(
          Country = AU1_CO,
          TotalCitation = TC
        ) %>%
        group_by(Country) %>%
        summarise(
          "TC" = sum(TotalCitation),
          "Average Article Citations" = round(
            sum(TotalCitation) / length(TotalCitation),
            1
          )
        ) %>%
        arrange(-TC) %>%
        as.data.frame(.data)
    },
    "tab7" = {
      TAB <- values$M %>%
        select(SO) %>%
        group_by(SO) %>%
        count() %>%
        arrange(desc(n)) %>%
        rename(
          Sources = SO,
          Articles = n
        ) %>%
        as.data.frame()
    },
    "tab10" = {
      TAB <- mapworld(values$M)$tab
    },
    "tab11" = {
      if (!("AU_UN" %in% names(values$M))) {
        values$M <- metaTagExtraction(values$M, Field = "AU_UN")
      }
      TAB <- data.frame(Affiliation = unlist(strsplit(values$M$AU_UN, ";"))) %>%
        group_by(Affiliation) %>%
        count() %>%
        drop_na(Affiliation) %>%
        arrange(desc(n)) %>%
        rename(Articles = n) %>%
        filter(Affiliation != "NA") %>%
        as.data.frame()
    },
    "tab12" = {
      TAB <- tableTag(values$M, "C1")
      TAB <- data.frame(Affiliations = names(TAB), Articles = as.numeric(TAB))
      TAB <- TAB[nchar(TAB[, 1]) > 4, ]
      # names(TAB)=c("Affiliations", "Articles")
    },
    "tab13" = {
      CR <- localCitations(values$M, fast.search = FALSE, verbose = FALSE)
      TAB <- CR$Authors
      # TAB=data.frame(Authors=names(CR$Authors$Author), Citations=as.numeric(CR$Cited))
    }
  )
  values$TAB <- TAB
  res <- list(values = values, TAB = TAB)
  return(res)
}

AffiliationOverTime <- function(values, n) {
  if (!("AU_UN" %in% names(values$M))) {
    values$M <- metaTagExtraction(values$M, Field = "AU_UN")
  }
  AFF <- strsplit(values$M$AU_UN, ";")
  nAFF <- lengths(AFF)

  AFFY <- data.frame(
    Affiliation = unlist(AFF),
    Year = rep(values$M$PY, nAFF)
  ) %>%
    filter(Affiliation != "NA") %>%
    drop_na(Affiliation, Year) %>%
    group_by(Affiliation, Year) %>%
    count() %>%
    group_by(Affiliation) %>%
    arrange(Year) %>%
    ungroup() %>%
    pivot_wider(Affiliation, names_from = Year, values_from = n) %>%
    mutate(across(everything(), ~ replace(., is.na(.), 0))) %>%
    pivot_longer(
      cols = !Affiliation,
      names_to = "Year",
      values_to = "Articles"
    ) %>%
    group_by(Affiliation) %>%
    mutate(Articles = cumsum(Articles))

  Affselected <- AFFY %>%
    filter(Year == max(Year)) %>%
    ungroup() %>%
    slice_max(Articles, n = n)

  values$AffOverTime <- AFFY %>%
    filter(Affiliation %in% Affselected$Affiliation) %>%
    mutate(Year = Year %>% as.numeric())

  Text <- paste(
    values$AffOverTime$Affiliation,
    " (",
    values$AffOverTime$Year,
    ") ",
    values$AffOverTime$Articles,
    sep = ""
  )
  x <- c(
    min(values$AffOverTime$Year) + 0.02,
    min(values$AffOverTime$Year) + 0.02 +
      diff(range(values$AffOverTime$Year)) * 0.15
  )
  y <- c(
    max(values$AffOverTime$Articles) -
      diff(range(values$AffOverTime$Articles)) * 0.15,
    max(values$AffOverTime$Articles)
  )

  # Reorder legend by descending max cumulative value
  values$AffOverTime$Affiliation <- factor(values$AffOverTime$Affiliation,
    levels = names(sort(tapply(values$AffOverTime$Articles, values$AffOverTime$Affiliation, max), decreasing = TRUE)))

  # Truncate long affiliation names for legend readability
  trunc_aff_labels <- setNames(
    ifelse(nchar(levels(values$AffOverTime$Affiliation)) > 35,
           paste0(substr(levels(values$AffOverTime$Affiliation), 1, 32), "..."),
           levels(values$AffOverTime$Affiliation)),
    levels(values$AffOverTime$Affiliation)
  )

  values$AffOverTimePlot <- ggplot(
    values$AffOverTime,
    aes(
      x = Year,
      y = Articles,
      group = Affiliation,
      color = Affiliation,
      text = Text
    )
  ) +
    geom_line() +
    labs(
      x = "Year",
      y = "Articles",
      title = "Affiliations' Production over Time"
    ) +
    scale_x_continuous(
      breaks = {
        yrs <- sort(unique(values$AffOverTime$Year))
        yrs[seq(1, length(yrs), by = max(1, ceiling(length(yrs) / 20)))]
      }
    ) +
    geom_hline(aes(yintercept = 0), alpha = 0.1) +
    labs(color = "Affiliation") +
    theme(
      text = element_text(color = "#444444"),
      legend.text = ggplot2::element_text(size = 9, face = "bold"),
      legend.box.margin = margin(2, 2, 2, 2),
      legend.title = ggplot2::element_text(
        size = 10,
        face = "bold"
      ),
      legend.position = "bottom",
      legend.direction = "horizontal",
      legend.key.size = grid::unit(0.4, "cm"),
      legend.key.width = grid::unit(0.6, "cm"),
      plot.caption = element_text(
        size = 9,
        hjust = 0.5,
        color = "black",
        face = "bold"
      ),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0.95, angle = 0),
      axis.text.x = element_text(size = 10, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    scale_color_discrete(labels = trunc_aff_labels) +
    guides(color = guide_legend(ncol = 2)) +
    annotation_custom(
      values$logoGrid,
      xmin = x[1],
      xmax = x[2],
      ymin = y[1],
      ymax = y[2]
    )
  return(values)
}

CountryOverTime <- function(values, n) {
  if (!("AU_CO" %in% names(values$M))) {
    values$M <- metaTagExtraction(values$M, Field = "AU_CO")
  }
  AFF <- strsplit(values$M$AU_CO, ";")
  nAFF <- lengths(AFF)

  AFFY <- data.frame(
    Affiliation = unlist(AFF),
    Year = rep(values$M$PY, nAFF)
  ) %>%
    drop_na(Affiliation, Year) %>%
    group_by(Affiliation, Year) %>%
    count() %>%
    group_by(Affiliation) %>%
    arrange(Year) %>%
    ungroup() %>%
    pivot_wider(Affiliation, names_from = Year, values_from = n) %>%
    mutate(across(everything(), ~ replace(., is.na(.), 0))) %>%
    pivot_longer(
      cols = !Affiliation,
      names_to = "Year",
      values_to = "Articles"
    ) %>%
    group_by(Affiliation) %>%
    mutate(Articles = cumsum(Articles))

  Affselected <- AFFY %>%
    filter(Year == max(Year)) %>%
    ungroup() %>%
    slice_max(Articles, n = n)

  values$CountryOverTime <- AFFY %>%
    filter(Affiliation %in% Affselected$Affiliation) %>%
    mutate(Year = Year %>% as.numeric()) %>%
    rename(Country = Affiliation)

  Text <- paste(
    values$CountryOverTime$Country,
    " (",
    values$CountryOverTime$Year,
    ") ",
    values$CountryOverTime$Articles,
    sep = ""
  )
  x <- c(
    min(values$CountryOverTime$Year) + 0.02,
    min(values$CountryOverTime$Year) + 0.02 +
      diff(range(values$CountryOverTime$Year)) * 0.15
  )
  y <- c(
    max(values$CountryOverTime$Articles) -
      diff(range(values$CountryOverTime$Articles)) * 0.15,
    max(values$CountryOverTime$Articles)
  )

  # Reorder legend by descending max cumulative value
  values$CountryOverTime$Country <- factor(values$CountryOverTime$Country,
    levels = names(sort(tapply(values$CountryOverTime$Articles, values$CountryOverTime$Country, max), decreasing = TRUE)))

  values$CountryOverTimePlot <- ggplot(
    values$CountryOverTime,
    aes(x = Year, y = Articles, group = Country, color = Country, text = Text)
  ) +
    geom_line() +
    labs(
      x = "Year",
      y = "Articles",
      title = "Country Production over Time"
    ) +
    scale_x_continuous(
      breaks = {
        yrs <- sort(unique(values$CountryOverTime$Year))
        yrs[seq(1, length(yrs), by = max(1, ceiling(length(yrs) / 20)))]
      }
    ) +
    geom_hline(aes(yintercept = 0), alpha = 0.1) +
    labs(color = "Country") +
    theme(
      text = element_text(color = "#444444"),
      legend.text = ggplot2::element_text(size = 10),
      legend.box.margin = margin(6, 6, 6, 6),
      legend.title = ggplot2::element_text(
        size = 12,
        face = "bold"
      ),
      legend.position = "right",
      legend.direction = "vertical",
      legend.key.size = grid::unit(0.4, "cm"),
      legend.key.width = grid::unit(0.6, "cm"),
      plot.caption = element_text(
        size = 9,
        hjust = 0.5,
        color = "black",
        face = "bold"
      ),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0.95, angle = 0),
      axis.text.x = element_text(size = 10, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = x[1],
      xmax = x[2],
      ymin = y[1],
      ymax = y[2]
    )
  return(values)
}

wordlist <- function(
  M,
  Field,
  n,
  measure,
  ngrams,
  remove.terms = NULL,
  synonyms = NULL
) {
  switch(
    Field,
    ID = {
      v <- tableTag(M, "ID", remove.terms = remove.terms, synonyms = synonyms)
    },
    DE = {
      v <- tableTag(M, "DE", remove.terms = remove.terms, synonyms = synonyms)
    },
    KW_Merged = {
      v <- tableTag(
        M,
        "KW_Merged",
        remove.terms = remove.terms,
        synonyms = synonyms
      )
    },
    TI = {
      if (!("TI_TM" %in% names(M))) {
        v <- tableTag(
          M,
          "TI",
          ngrams = ngrams,
          remove.terms = remove.terms,
          synonyms = synonyms
        )
      }
    },
    AB = {
      if (!("AB_TM" %in% names(M))) {
        v <- tableTag(
          M,
          "AB",
          ngrams = ngrams,
          remove.terms = remove.terms,
          synonyms = synonyms
        )
      }
    },
    WC = {
      v <- tableTag(M, "WC")
    }
  )
  names(v) <- tolower(names(v))
  # v=tableTag(values$M,"ID")
  n <- min(c(n, length(v)))
  Words <- data.frame(
    Terms = names(v)[1:n],
    Frequency = (as.numeric(v)[1:n]),
    stringsAsFactors = FALSE
  )
  W <- Words
  switch(
    measure,
    identity = {},
    sqrt = {
      W$Frequency <- sqrt(W$Frequency)
    },
    log = {
      W$Frequency <- log(W$Frequency + 1)
    },
    log10 = {
      W$Frequency <- log10(W$Frequency + 1)
    }
  )

  results <- list(v = v, W = W, Words = Words)
  return(results)
}

readStopwordsFile <- function(file, sep = ",") {
  if (!is.null(file)) {
    req(file$datapath)
    remove.terms <- unlist(strsplit(readr::read_lines(file$datapath), sep))
  } else {
    remove.terms <- NULL
  }
  return(remove.terms)
}

readSynWordsFile <- function(file, sep = ",") {
  if (!is.null(file)) {
    req(file$datapath)
    syn.terms <- readr::read_lines(file$datapath)
    if (sep != ";") syn.terms <- gsub(sep, ";", syn.terms)
  } else {
    syn.terms <- NULL
  }
  return(syn.terms)
}

mapworld <- function(M, values) {
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, "AU_CO")
  }
  CO <- as.data.frame(tableTag(M, "AU_CO"))
  CO$Tab <- gsub("[[:digit:]]", "", CO$Tab)
  CO$Tab <- gsub(".", "", CO$Tab, fixed = TRUE)
  CO$Tab <- gsub(";;", ";", CO$Tab, fixed = TRUE)
  CO$Tab <- gsub("UNITED STATES", "USA", CO$Tab)
  CO$Tab <- gsub("RUSSIAN FEDERATION", "RUSSIA", CO$Tab)
  CO$Tab <- gsub("TAIWAN", "CHINA", CO$Tab)
  CO$Tab <- gsub("ENGLAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("SCOTLAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("WALES", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("NORTH IRELAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("UNITED KINGDOM", "UK", CO$Tab)
  CO$Tab <- gsub("KOREA", "SOUTH KOREA", CO$Tab)

  map.world <- map_data("world")
  map.world$region <- toupper(map.world$region)

  # dplyr::anti_join(CO, map.world, by = c('Tab' = 'region'))

  country.prod <- dplyr::left_join(map.world, CO, by = c("region" = "Tab"))

  tab <- data.frame(
    country.prod %>%
      dplyr::group_by(region) %>%
      dplyr::summarise(Freq = mean(Freq))
  )

  tab <- tab[!is.na(tab$Freq), ]

  tab <- tab[order(-tab$Freq), ]

  # breaks=as.numeric(round(quantile(CO$Freq,c(0.2,0.4,0.6,0.8,1))))
  # names(breaks)=breaks
  # breaks=log(breaks)
  breaks <- as.numeric(cut(CO$Freq, breaks = 10))
  names(breaks) <- breaks

  g <- ggplot(
    country.prod,
    aes(
      x = long,
      y = lat,
      group = group,
      text = paste("Country: ", region, "\nN.of Documents: ", Freq)
    )
  ) +
    geom_polygon(aes(fill = Freq, group = group)) +
    scale_fill_continuous(
      low = "#87CEEB",
      high = "dodgerblue4",
      breaks = breaks,
      na.value = "grey80"
    ) +
    guides(fill = guide_legend(reverse = T)) +
    # geom_text(data=centroids, aes(label = centroids$Tab, x = centroids$long, y = centroids$lat, group=centroids$Tab)) +
    labs(
      fill = "N.Documents",
      title = "Country Scientific Production",
      x = NULL,
      y = NULL
    ) +
    theme(
      text = element_text(color = "#333333"),
      plot.title = element_text(size = 28),
      plot.subtitle = element_text(size = 14),
      axis.ticks = element_blank(),
      axis.text = element_blank(),
      panel.grid = element_blank(),
      panel.background = element_rect(fill = "#FFFFFF"), #' #333333'
      plot.background = element_rect(fill = "#FFFFFF"),
      legend.position = "none"
      # ,legend.background = element_blank()
      # ,legend.key = element_blank()
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = 143,
      xmax = 189.5,
      ymin = -69,
      ymax = -48
    )

  results <- list(g = g, tab = tab)
  return(results)
}

### Structure fuctions ----
CAmap <- function(input, values) {
  if ((input$CSfield %in% names(values$M))) {
    if (input$CSfield %in% c("TI", "AB")) {
      ngrams <- as.numeric(input$CSngrams)
    } else {
      ngrams <- 1
    }

    ### load file with terms to remove
    if (input$CSStopFile == "Y") {
      remove.terms <- trimws(values$CSremove.terms$stopword)
    } else {
      remove.terms <- NULL
    }
    # values$CSremove.terms <- remove.terms
    ### end of block
    ### load file with synonyms
    if (input$FASynFile == "Y") {
      synonyms <- values$FAsyn.terms %>%
        group_by(term) %>%
        mutate(term = paste0(term, ";", synonyms)) %>%
        select(term)
      synonyms <- synonyms$term
    } else {
      synonyms <- NULL
    }
    # values$FAsyn.terms <- synonyms
    ### end of block

    tab <- tableTag(values$M, input$CSfield, ngrams = ngrams)
    if (length(tab >= 2)) {
      minDegree <- as.numeric(tab[input$CSn])

      values$CS <- conceptualStructure(
        values$M,
        method = input$method,
        field = input$CSfield,
        minDegree = minDegree,
        clust = input$nClustersCS,
        k.max = 8,
        stemming = F,
        labelsize = input$CSlabelsize / 2,
        documents = input$CSdoc,
        graph = FALSE,
        ngrams = ngrams,
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      if (input$method != "MDS") {
        CSData <- values$CS$docCoord
        CSData <- data.frame(Documents = row.names(CSData), CSData)
        CSData$dim1 <- round(CSData$dim1, 2)
        CSData$dim2 <- round(CSData$dim2, 2)
        CSData$contrib <- round(CSData$contrib, 2)
        values$CS$CSData <- CSData
      } else {
        values$CS$CSData <- data.frame(Docuemnts = NA, dim1 = NA, dim2 = NA)
      }

      switch(
        input$method,
        CA = {
          WData <- data.frame(
            word = row.names(values$CS$km.res$data.clust),
            values$CS$km.res$data.clust,
            stringsAsFactors = FALSE
          )
          names(WData)[4] <- "cluster"
        },
        MCA = {
          WData <- data.frame(
            word = row.names(values$CS$km.res$data.clust),
            values$CS$km.res$data.clust,
            stringsAsFactors = FALSE
          )
          names(WData)[4] <- "cluster"
        },
        MDS = {
          WData <- data.frame(
            word = row.names(values$CS$res),
            values$CS$res,
            cluster = values$CS$km.res$cluster
          )
        }
      )

      WData$Dim1 <- round(WData$Dim1, 2)
      WData$Dim2 <- round(WData$Dim2, 2)
      values$CS$WData <- WData
    } else {
      emptyPlot("Selected field is not included in your data collection")
      values$CS <- list("NA")
    }
  } else {
    emptyPlot("Selected field is not included in your data collection")
    values$CS <- list("NA")
  }
  return(values)
}

historiograph <- function(input, values) {
  min.cit <- 0

  # if (values$Histfield=="NA"){
  values$histResults <- histNetwork(
    values$M,
    min.citations = min.cit,
    sep = ";"
  )
  # values$Histfield="done"
  # }
  values$histResults$histData <- values$histResults$histData %>%
    tibble::rownames_to_column(var = "SR")
  # titlelabel <- input$titlelabel
  values$histlog <- (values$histPlot <- histPlot(
    values$histResults,
    n = input$histNodes,
    size = input$histsize,
    remove.isolates = (input$hist.isolates == "yes"),
    labelsize = input$histlabelsize,
    label = input$titlelabel,
    verbose = FALSE
  ))

  values$histResults$histData$DOI <- paste0(
    '<a href=\"https://doi.org/',
    values$histResults$histData$DOI,
    '\" target=\"_blank\">',
    values$histResults$histData$DOI,
    "</a>"
  )

  values$histResults$histData <- values$histResults$histData %>%
    left_join(
      values$histPlot$layout %>%
        select(name, color),
      by = c("Paper" = "name")
    ) %>%
    drop_na(color) %>%
    mutate(cluster = match(color, unique(color))) %>%
    select(!color) %>%
    group_by(cluster) %>%
    arrange(Year, .by_group = TRUE)

  return(values)
}


### Network functions ----
degreePlot <- function(net) {
  # deg <- data.frame(node = names(net$nodeDegree), x= (1:length(net$nodeDegree)), y = net$nodeDegree)
  ma <- function(x, n = 5) {
    stats::filter(x, rep(1 / n, n), sides = 1)
  }

  deg <- net$nodeDegree %>%
    mutate(x = row_number())

  p <- ggplot(
    data = deg,
    aes(
      x = x,
      y = degree,
      text = paste(node, " - Degree ", round(degree, 3), sep = "")
    )
  ) +
    geom_point() +
    geom_line(aes(group = "NA"), color = "#002F80", alpha = .5) +
    # geom_hline(yintercept=cutting$degree, linetype="dashed",color = '#002F80', alpha = .5)+
    theme(
      text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 0),
      axis.title.x = element_text(hjust = 0),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    labs(x = "Node", y = "Cumulative Degree", title = "Node Degrees")
  return(p)
}

# Associate a Year to each Keyword
keywords2Years <- function(M, field = "DE", n = 100) {
  suppressMessages(Y <- KeywordGrowth(M, Tag = field, top = Inf, cdf = FALSE))

  ## Normalize data and exclude tot from normalization
  df <- Y %>%
    rowwise() %>%
    mutate(year_freq = sum(c_across(!matches("Year")))) %>%
    mutate(across(!matches("Year"), ~ .x / year_freq)) %>%
    mutate(across(!matches("Year"), ~ replace_na(.x, 0)))

  df_long <- df %>%
    pivot_longer(
      cols = -c(Year, year_freq),
      names_to = "Keyword",
      values_to = "Probability"
    )

  df_max_year <- df_long %>%
    group_by(Keyword) %>%
    filter(Probability == max(Probability)) %>%
    slice_max(order_by = year_freq, n = 1, with_ties = FALSE) %>%
    ungroup()

  return(df_max_year)
}

cocNetwork <- function(input, values) {
  n <- input$Nodes
  label.n <- input$Labels

  ### load file with terms to remove
  if (input$COCStopFile == "Y") {
    remove.terms <- trimws(values$COCremove.terms$stopword)
  } else {
    remove.terms <- NULL
  }
  # values$COCremove.terms <- remove.terms
  ### end of block
  ### load file with synonyms
  if (input$COCSynFile == "Y") {
    synonyms <- values$COCsyn.terms %>%
      group_by(term) %>%
      mutate(term = paste0(term, ";", synonyms)) %>%
      select(term)
    synonyms <- synonyms$term
  } else {
    synonyms <- NULL
  }
  # values$COCsyn.terms <- synonyms
  ### end of block

  if ((input$field %in% names(values$M))) {
    if (
      (dim(values$NetWords)[1]) == 1 |
        !(input$field == values$field) |
        !(input$cocngrams == values$cocngrams) |
        ((dim(values$NetWords)[1]) != input$Nodes)
    ) {
      values$field <- input$field
      values$ngrams <- input$cocngrams

      switch(
        input$field,
        ID = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "Keywords Plus Network"
        },
        DE = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "author_keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "Authors' Keywords network"
        },
        KW_Merged = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "all_keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "All Keywords network"
        },
        TI = {
          # if(!("TI_TM" %in% names(values$M))){
          values$M <- termExtraction(
            values$M,
            Field = "TI",
            verbose = FALSE,
            ngrams = as.numeric(input$cocngrams),
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          # }
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "titles",
            n = n,
            sep = ";"
          )
          values$Title <- "Title Words network"
        },
        AB = {
          # if(!("AB_TM" %in% names(values$M))){
          values$M <- termExtraction(
            values$M,
            Field = "AB",
            verbose = FALSE,
            ngrams = as.numeric(input$cocngrams),
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          # }
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "abstracts",
            n = n,
            sep = ";"
          )
          values$Title <- "Abstract Words network"
        },
        WC = {
          WSC <- cocMatrix(values$M, Field = "WC", binary = FALSE)
          values$NetWords <- crossprod(WSC, WSC)
          values$Title <- "Subject Categories network"
        }
      )
    }

    if (label.n > n) {
      label.n <- n
    }
    if (input$normalize == "none") {
      normalize <- NULL
    } else {
      normalize <- input$normalize
    }
    if (input$label.cex == "Yes") {
      label.cex <- TRUE
    } else {
      label.cex <- FALSE
    }
    if (input$coc.curved == "Yes") {
      curved <- TRUE
    } else {
      curved <- FALSE
    }

    values$cocnet <- networkPlot(
      values$NetWords,
      normalize = normalize,
      Title = values$Title,
      type = input$layout,
      size.cex = TRUE,
      size = 5,
      remove.multiple = F,
      edgesize = input$edgesize * 3,
      labelsize = input$labelsize,
      label.cex = label.cex,
      label.n = label.n,
      edges.min = input$edges.min,
      label.color = F,
      curved = curved,
      alpha = input$cocAlpha,
      cluster = input$cocCluster,
      remove.isolates = (input$coc.isolates == "yes"),
      community.repulsion = input$coc.repulsion / 2,
      seed = values$random_seed,
      verbose = FALSE
    )

    g <- values$cocnet$graph
    Y <- keywords2Years(values$M, field = input$field, n = Inf)
    label <- data.frame(Keyword = igraph::V(g)$name)
    df <- label %>%
      left_join(Y %>% mutate(Keyword = tolower(Keyword)), by = "Keyword") %>%
      rename(year_med = Year)
    igraph::V(g)$year_med <- df$year_med

    if (input$cocyears == "Yes") {
      col <- hcl.colors(
        (diff(range(df$year_med)) + 1) * 10,
        palette = "Blues 3"
      )
      igraph::V(g)$color <- col[(max(df$year_med) - df$year_med + 1) * 10]
    }
    values$cocnet$graph <- g
  } else {
    emptyPlot("Selected field is not included in your data collection")
  }
  return(values)
}

intellectualStructure <- function(input, values) {
  n <- input$citNodes
  label.n <- input$citLabels

  if (
    (dim(values$NetRefs)[1]) == 1 |
      !(input$citField == values$citField) |
      !(input$citSep == values$citSep) |
      !(input$citShortlabel == values$citShortlabel) |
      ((dim(values$NetRefs)[1]) != input$citNodes)
  ) {
    values$citField <- input$citField
    values$citSep <- input$citSep
    if (input$citShortlabel == "Yes") {
      shortlabel <- TRUE
    } else {
      shortlabel <- FALSE
    }
    values$citShortlabel <- input$citShortlabel
    switch(
      input$citField,
      CR = {
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "references",
          n = n,
          sep = input$citSep,
          shortlabel = shortlabel
        )
        values$Title <- "Cited References network"
      },
      CR_AU = {
        if (!("CR_AU" %in% names(values$M))) {
          values$M <- metaTagExtraction(
            values$M,
            Field = "CR_AU",
            sep = input$citSep
          )
        }
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "authors",
          n = n,
          sep = input$citSep
        )
        values$Title <- "Cited Authors network"
      },
      CR_SO = {
        if (!("CR_SO" %in% names(values$M))) {
          values$M <- metaTagExtraction(
            values$M,
            Field = "CR_SO",
            sep = input$citSep
          )
        }
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "sources",
          n = n,
          sep = input$citSep
        )
        values$Title <- "Cited Sources network"
      }
    )
  }

  if (label.n > n) {
    label.n <- n
  }
  if (input$citlabel.cex == "Yes") {
    label.cex <- TRUE
  } else {
    label.cex <- FALSE
  }
  if (input$cocit.curved == "Yes") {
    curved <- TRUE
  } else {
    curved <- FALSE
  }

  values$cocitnet <- networkPlot(
    values$NetRefs,
    normalize = NULL,
    Title = values$Title,
    type = input$citlayout,
    size.cex = TRUE,
    size = 5,
    remove.multiple = F,
    edgesize = input$citedgesize * 3,
    labelsize = input$citlabelsize,
    label.cex = label.cex,
    curved = curved,
    label.n = label.n,
    edges.min = input$citedges.min,
    label.color = F,
    remove.isolates = (input$cit.isolates == "yes"),
    alpha = 0.7,
    cluster = input$cocitCluster,
    community.repulsion = input$cocit.repulsion / 2,
    verbose = FALSE
  )
  return(values)
}

authors2Years <- function(M, field = "AU") {
  WAU <- cocMatrix(M, field)
  WPY <- cocMatrix(M, "PY")
  B <- crossprod(WPY, WAU) %>%
    as.matrix() %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Year")

  # create a data frame that startig from B associate at each author the year of the first non zero value
  C <- B %>%
    pivot_longer(-Year, names_to = "Item", values_to = "Value") %>%
    filter(Value > 0) %>%
    group_by(Item) %>%
    summarise(FirstYear = min(Year)) %>%
    ungroup()

  return(C)
}

socialStructure <- function(input, values) {
  n <- input$colNodes
  label.n <- input$colLabels

  if (
    (dim(values$ColNetRefs)[1]) == 1 |
      !(input$colField == values$colField) |
      ((dim(values$ColNetRefs)[1]) != input$colNodes)
  ) {
    values$colField <- input$colField

    if (!"nAU" %in% names(values$M)) {
      values$M$nAU <- str_count(values$M$AU, ";") + 1
    }

    switch(
      input$colField,
      COL_AU = {
        if (input$col.filterMaxAuthors) {
          M_AU <- values$M %>% filter(nAU <= 20)
        } else {
          M_AU <- values$M
        }
        values$ColNetRefs <- biblioNetwork(
          M_AU,
          analysis = "collaboration",
          network = "authors",
          n = n,
          sep = ";"
        )
        values$Title <- "Author Collaboration network"
        values$fieldCOL <- "AU"
      },
      COL_UN = {
        if (!("AU_UN" %in% names(values$M))) {
          values$M <- metaTagExtraction(values$M, Field = "AU_UN", sep = ";")
        }
        values$ColNetRefs <- biblioNetwork(
          values$M,
          analysis = "collaboration",
          network = "universities",
          n = n,
          sep = ";"
        )
        values$Title <- "Edu Collaboration network"
        values$fieldCOL <- "AU_UN"
      },
      COL_CO = {
        if (!("AU_CO" %in% names(values$M))) {
          values$M <- metaTagExtraction(values$M, Field = "AU_CO", sep = ";")
        }
        values$ColNetRefs <- biblioNetwork(
          values$M,
          analysis = "collaboration",
          network = "countries",
          n = n,
          sep = ";"
        )
        values$Title <- "Country Collaboration network"
        values$fieldCOL <- "AU_CO"
        # values$cluster="none"
      }
    )
  }

  if (label.n > n) {
    label.n <- n
  }
  if (input$colnormalize == "none") {
    normalize <- NULL
  } else {
    normalize <- input$colnormalize
  }
  if (input$collabel.cex == "Yes") {
    label.cex <- TRUE
  } else {
    label.cex <- FALSE
  }
  if (input$soc.curved == "Yes") {
    curved <- TRUE
  } else {
    curved <- FALSE
  }

  type <- input$collayout
  if (input$collayout == "worldmap") {
    type <- "auto"
  }

  values$colnet <- networkPlot(
    values$ColNetRefs,
    normalize = normalize,
    Title = values$Title,
    type = type,
    size.cex = TRUE,
    size = 5,
    remove.multiple = F,
    edgesize = input$coledgesize * 3,
    labelsize = input$collabelsize,
    label.cex = label.cex,
    curved = curved,
    label.n = label.n,
    edges.min = input$coledges.min,
    label.color = F,
    alpha = input$colAlpha,
    remove.isolates = (input$col.isolates == "yes"),
    cluster = input$colCluster,
    community.repulsion = input$col.repulsion / 2,
    verbose = FALSE
  )

  g <- values$colnet$graph
  Y <- authors2Years(values$M, values$fieldCOL)
  label <- data.frame(Item = igraph::V(g)$name)
  df <- label %>%
    left_join(Y %>% mutate(Item = tolower(Item)), by = "Item") %>%
    rename(year_med = FirstYear)
  igraph::V(g)$year_med <- df$year_med

  values$colnet$graph <- g

  return(values)
}

countrycollaboration_plotly <- function(
  M,
  min.edges = 1,
  edge_opacity = 0.4,
  edgesize = 5,
  min_edgesize = 0.5
) {
  if (!requireNamespace("geosphere", quietly = TRUE)) {
    install.packages("geosphere")
  }
  # library(plotly)
  # library(geosphere)

  # Prepara i dati
  M <- metaTagExtraction(M, "AU_CO")
  net <- biblioNetwork(M, analysis = "collaboration", network = "countries")
  CO <- data.frame(Tab = rownames(net), Freq = diag(net))
  bsk.network <- igraph::graph_from_adjacency_matrix(net, mode = "undirected")
  COedges <- as.data.frame(igraph::ends(
    bsk.network,
    igraph::E(bsk.network),
    names = TRUE
  ))

  # Carica coordinate
  data("countries", envir = environment())
  names(countries)[1] <- "Tab"

  # Prepara i paesi
  CO <- dplyr::inner_join(CO, countries, by = "Tab")

  # Prepara gli edge
  COedges <- dplyr::inner_join(COedges, countries, by = c("V1" = "Tab"))
  COedges <- dplyr::inner_join(COedges, countries, by = c("V2" = "Tab"))
  COedges <- COedges[COedges$V1 != COedges$V2, ]
  COedges <- tab <- count.duplicates(COedges)
  COedges <- COedges[COedges$count >= min.edges, ]

  # Calcola gli spessori degli edge normalizzati
  edge_weights <- sqrt(COedges$count)
  # Normalizza tra min_edgesize e edgesize
  if (length(unique(edge_weights)) > 1) {
    COedges$width <- min_edgesize +
      (edge_weights - min(edge_weights)) /
        (max(edge_weights) - min(edge_weights)) *
        (edgesize - min_edgesize)
  } else {
    # Se tutti gli edge hanno lo stesso peso, usa il valore medio
    COedges$width <- (min_edgesize + edgesize) / 2
  }

  # Crea la mappa base con choropleth (SENZA legenda)
  fig <- plotly::plot_geo(CO) %>%
    add_trace(
      type = "choropleth",
      locations = ~Tab,
      locationmode = "country names",
      z = ~ log1p(Freq),
      colorscale = list(
        c(0, "#FFFFFF"), # Bianco
        c(0.2, "#fee5d9"), # Rosa pallido
        c(0.4, "#fcae91"), # Salmone
        c(0.6, "#fb6a4a"), # Arancione/rosso
        c(0.8, "#de2d26"), # Rosso
        c(1, "#a50f15") # Bordeaux scuro
      ),
      text = ~ paste0("<b>", Tab, "</b><br>Publications: ", Freq),
      hoverinfo = "text",
      marker = list(line = list(color = "white", width = 0.8)),
      showscale = FALSE,
      showlegend = FALSE
    )

  # Aggiungi le linee di collaborazione curve con spessore controllato
  for (i in 1:nrow(COedges)) {
    # Crea arco geodetico
    path <- geosphere::gcIntermediate(
      c(COedges$Longitude.x[i], COedges$Latitude.x[i]),
      c(COedges$Longitude.y[i], COedges$Latitude.y[i]),
      n = 100,
      addStartEnd = TRUE,
      sp = FALSE
    )

    fig <- fig %>%
      add_trace(
        type = "scattergeo",
        lon = path[, 1],
        lat = path[, 2],
        mode = "lines",
        line = list(
          width = COedges$width[i], # USA LO SPESSORE NORMALIZZATO
          color = "#4A90E2"
        ),
        opacity = edge_opacity,
        hoverinfo = "text",
        text = paste0(
          "<b>",
          COedges$V1[i],
          " ↔ ",
          COedges$V2[i],
          "</b>",
          "<br>Collaborations: ",
          COedges$count[i]
        ),
        showlegend = FALSE
      )
  }

  # Layout migliorato
  fig <- fig %>%
    layout(
      # title = list(
      #   text = "International Scientific Collaboration Network",
      #   font = list(size = 18, color = "#333333")
      # ),
      geo = list(
        projection = list(type = "natural earth"),
        showland = TRUE,
        landcolor = "#f5f5f5",
        coastlinecolor = "#999999",
        showocean = TRUE,
        oceancolor = "#d4e6f1",
        showlakes = TRUE,
        lakecolor = "#d4e6f1",
        showcountries = TRUE,
        countrycolor = "white",
        countrywidth = 1
      ),
      paper_bgcolor = "white",
      plot_bgcolor = "white"
    )

  return(list(g = fig, tab = tab))
}

countrycollaboration <- function(M, label, edgesize, min.edges, values) {
  M <- metaTagExtraction(M, "AU_CO")
  net <- biblioNetwork(M, analysis = "collaboration", network = "countries")
  CO <- data.frame(Tab = rownames(net), Freq = diag(net))
  bsk.network <- igraph::graph_from_adjacency_matrix(net, mode = "undirected")
  COedges <- as.data.frame(igraph::ends(
    bsk.network,
    igraph::E(bsk.network),
    names = TRUE
  ))

  map.world <- map_data("world")
  map.world$region <- toupper(map.world$region)
  map.world$region <- gsub("^UK$", "UNITED KINGDOM", map.world$region)
  map.world$region <- gsub("^SOUTH KOREA$", "KOREA", map.world$region)

  country.prod <- dplyr::left_join(map.world, CO, by = c("region" = "Tab"))

  # breaks <- as.numeric(round(quantile(CO$Freq,seq(0.1,1,by=0.1))))
  breaks <- as.numeric(cut(CO$Freq, breaks = 10))
  names(breaks) <- breaks
  # breaks=breaks
  data("countries", envir = environment())
  names(countries)[1] <- "Tab"

  COedges <- dplyr::inner_join(COedges, countries, by = c("V1" = "Tab"))
  COedges <- dplyr::inner_join(COedges, countries, by = c("V2" = "Tab"))
  COedges <- COedges[COedges$V1 != COedges$V2, ]
  COedges <- count.duplicates(COedges)
  tab <- COedges
  COedges <- COedges[COedges$count >= min.edges, ]
  COedges$region <- paste(
    "\nCollaboration between\n",
    COedges$V1,
    "\n and \n",
    COedges$V2
  )

  g <- ggplot(
    country.prod,
    aes(x = long, y = lat, group = group, text = paste("Country: ", region))
  ) +
    geom_polygon(aes(fill = Freq)) +
    scale_fill_continuous(
      low = "#87CEEB",
      high = "dodgerblue4",
      breaks = breaks,
      na.value = "grey80"
    ) +
    # guides(fill = guide_legend(reverse = T)) +
    guides(colour = FALSE, fill = FALSE) +
    # geom_curve(data=COedges, aes(x = Longitude.x , y = Latitude.x, xend = Longitude.y, yend = Latitude.y,     # draw edges as arcs
    #                              color = "firebrick4", size = count, group=continent.x),
    #            curvature = 0.33,
    #            alpha = 0.5) +
    geom_segment(
      data = COedges,
      aes(
        x = Longitude.x,
        y = Latitude.x,
        xend = Longitude.y,
        yend = Latitude.y, # draw edges as arcs
        size = count,
        group = continent.x
      ),
      color = "orangered4", # FFB347",
      # curvature = 0.33,
      alpha = 0.3
    ) +
    scale_size_continuous(guide = FALSE, range = c(0.25, edgesize)) +
    labs(title = NULL, x = "Latitude", y = "Longitude") +
    theme(
      text = element_text(color = "#333333"),
      plot.title = element_text(size = 28),
      plot.subtitle = element_text(size = 14),
      axis.ticks = element_blank(),
      axis.text = element_blank(),
      panel.grid = element_blank(),
      panel.background = element_rect(fill = "#FFFFFF"), #' #333333'
      plot.background = element_rect(fill = "#FFFFFF"),
      legend.position = c(.18, .36),
      legend.background = element_blank(),
      legend.key = element_blank()
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = 143,
      xmax = 189.5,
      ymin = -69,
      ymax = -48
    )
  if (isTRUE(label)) {
    CO <- dplyr::inner_join(CO, countries, by = c("Tab" = "Tab"))
    g <- g +
      # ggrepel::geom_text_repel(data=CO, aes(x = Longitude, y = Latitude, label = Tab, group=continent),             # draw text labels
      #                          hjust = 0, nudge_x = 1, nudge_y = 4,
      #                          size = 3, color = "orange", fontface = "bold")
      ggrepel::geom_text(
        data = CO,
        aes(x = Longitude, y = Latitude, label = Tab, group = continent), # draw text labels
        hjust = 0,
        nudge_x = 1,
        nudge_y = 4,
        size = 3,
        color = "orange",
        fontface = "bold"
      )
  }

  results <- list(g = g, tab = tab)
  return(results)
}
### visNetwork tools ----
netLayout <- function(type) {
  switch(
    type,
    auto = {
      l <- "layout_nicely"
    },
    circle = {
      l <- "layout_in_circle"
    },
    mds = {
      l <- "layout_with_mds"
    },
    star = {
      l <- "layout_as_star"
    },
    sphere = {
      l <- "layout_on_sphere"
    },
    fruchterman = {
      l <- "layout_with_fr"
    },
    kamada = {
      l <- "layout_with_kk"
    }
  )
  return(l)
}

savenetwork <- function(con, VIS) {
  VIS %>%
    visOptions(height = "800px") %>%
    visNetwork::visSave(con)
}

igraph2vis <- function(
  g,
  curved,
  labelsize,
  opacity,
  type,
  shape,
  net,
  shadow = TRUE,
  edgesize = 5,
  noOverlap = TRUE
) {
  LABEL <- igraph::V(g)$name

  LABEL[igraph::V(g)$labelsize == 0] <- ""

  vn <- visNetwork::toVisNetworkData(g)

  vn$nodes$label <- LABEL
  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE

  ## opacity
  vn$nodes$color <- adjustcolor(vn$nodes$color, alpha.f = min(c(opacity, 1)))
  ## set a darkest gray for iter-cluster edges
  vn$edges$color <- paste(substr(vn$edges$color, 1, 7), "90", sep = "")
  vn$edges$color[substr(vn$edges$color, 1, 7) == "#B3B3B3"] <- "#69696960"
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity)

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  vn$edges$width <- vn$edges$width^2 / (max(vn$edges$width^2)) * (10 + edgesize)

  # if (edgesize==0){
  #   vn$edges$hidden <- TRUE
  #   }else{vn$edges$hidden <- FALSE}

  ## labelsize
  vn$nodes$font.size <- vn$nodes$deg
  scalemin <- 20
  scalemax <- 150
  Min <- min(vn$nodes$font.size)
  Max <- max(vn$nodes$font.size)
  if (Max > Min) {
    size <- (vn$nodes$font.size - Min) / (Max - Min) * 15 * labelsize + 10
  } else {
    size <- 10 * labelsize
  }
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size
  l <- netLayout(type)

  ### TO ADD SHAPE AND FONT COLOR OPTIONS
  coords <- net$layout

  vn$nodes$size <- vn$nodes$font.size * 0.7

  # vn$nodes$font.color <- adjustcolor("black", alpha.f = min(c(opacity,1)))

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  opacity_font <- sqrt(
    (vn$nodes$font.size - min(vn$nodes$font.size)) /
      diff(range(vn$nodes$font.size))
  ) *
    opacity +
    0.3
  if (is.nan(opacity_font[1])) {
    opacity_font <- rep(0.3, length(opacity_font))
  }

  if (labelsize > 0) {
    vn$nodes$font.color <- unlist(lapply(opacity_font, function(x) {
      adjustcolor("black", alpha.f = x)
    }))
  } else {
    vn$nodes$font.color <- adjustcolor("black", alpha.f = 0)
  }
  ## avoid label overlaps
  if (noOverlap) {
    threshold <- 0.05
    ymax <- diff(range(coords[, 2]))
    xmax <- diff(range(coords[, 1]))
    threshold2 <- threshold * mean(xmax, ymax)
    w <- data.frame(
      x = coords[, 1],
      y = coords[, 2],
      labelToPlot = vn$nodes$label,
      dotSize = size,
      row.names = vn$nodes$label
    )
    labelToRemove <- avoidNetOverlaps(w, threshold = threshold2)
  } else {
    labelToRemove <- ""
  }

  vn$nodes <- vn$nodes %>%
    mutate(
      label = ifelse(label %in% labelToRemove, "", label),
      title = id
    )
  ##

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = shadow,
      shape = shape,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
    visNetwork::visOptions(
      highlightNearest = list(enabled = T, hover = T, degree = 1),
      nodesIdSelection = T
    ) %>%
    visNetwork::visInteraction(
      dragNodes = TRUE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visNetwork::visOptions(
      manipulation = curved,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn, type = type, l = l, curved = curved))
}

## function to avoid label overlapping ----
avoidNetOverlaps <- function(w, threshold = 0.10) {
  w[, 2] <- w[, 2] / 2

  Ds <- dist(
    w %>%
      dplyr::filter(labelToPlot != "") %>%
      select(1:2),
    method = "manhattan",
    upper = T
  ) %>%
    dist2df() %>%
    rename(
      from = row,
      to = col,
      dist = value
    ) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("from" = "labelToPlot")
    ) %>%
    rename(w_from = dotSize) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("to" = "labelToPlot")
    ) %>%
    rename(w_to = dotSize) %>%
    filter(dist < threshold)

  if (nrow(Ds) > 0) {
    st <- TRUE
    i <- 1
    label <- NULL
    case <- "n"

    while (isTRUE(st)) {
      if (Ds$w_from[i] > Ds$w_to[i] & Ds$dist[i] < threshold) {
        case <- "y"
        lab <- Ds$to[i]
      } else if (Ds$w_from[i] <= Ds$w_to[i] & Ds$dist[i] < threshold) {
        case <- "y"
        lab <- Ds$from[i]
      }

      switch(
        case,
        "y" = {
          Ds <- Ds[Ds$from != lab, ]
          Ds <- Ds[Ds$to != lab, ]
          label <- c(label, lab)
        },
        "n" = {
          Ds <- Ds[-1, ]
        }
      )

      if (i >= nrow(Ds)) {
        st <- FALSE
      }
      case <- "n"
      # print(nrow(Ds))
    }
  } else {
    label <- NULL
  }
  label
}


## visnetwork for subgraphs
igraph2visClust <- function(
  g,
  curved = FALSE,
  labelsize = 3,
  opacity = 0.7,
  shape = "dot",
  shadow = TRUE,
  edgesize = 5
) {
  LABEL <- igraph::V(g)$name

  LABEL[igraph::V(g)$labelsize == 0] <- ""

  vn <- visNetwork::toVisNetworkData(g)

  vn$nodes$label <- LABEL
  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE

  ## opacity
  vn$nodes$color <- adjustcolor(vn$nodes$color, alpha.f = min(c(opacity, 1)))
  ## set a darkest gray for iter-cluster edges
  vn$edges$color <- paste(substr(vn$edges$color, 1, 7), "90", sep = "")
  vn$edges$color[substr(vn$edges$color, 1, 7) == "#B3B3B3"] <- "#69696960"
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity)

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  vn$edges$width <- vn$edges$width^2 / (max(vn$edges$width^2)) * (5 + edgesize)

  ## labelsize
  scalemin <- 20
  scalemax <- 100
  # aggiunta
  vn$nodes$font.size <- vn$nodes$deg
  #
  Min <- min(vn$nodes$font.size)
  Max <- max(vn$nodes$font.size)
  if (Max > Min) {
    size <- (vn$nodes$font.size - Min) / (Max - Min) * 15 * labelsize #+10
  } else {
    size <- 5 * labelsize
  }
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size
  # l<-netLayout(type)

  ### TO ADD SHAPE AND FONT COLOR OPTIONS

  vn$nodes$size <- vn$nodes$font.size * 0.4

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  opacity_font <- sqrt(
    (vn$nodes$font.size - min(vn$nodes$font.size)) /
      diff(range(vn$nodes$font.size))
  ) *
    opacity +
    0.3
  if (is.nan(opacity_font[1])) {
    opacity_font <- rep(0.3, length(opacity_font))
  }

  if (labelsize > 0) {
    vn$nodes$font.color <- unlist(lapply(opacity_font, function(x) {
      adjustcolor("black", alpha.f = x)
    }))
  } else {
    vn$nodes$font.color <- adjustcolor("black", alpha.f = 0)
  }

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = shadow,
      shape = shape,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(layout = "layout_nicely", type = "full") %>%
    visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
    visNetwork::visOptions(
      highlightNearest = list(enabled = T, hover = T, degree = 1),
      nodesIdSelection = T
    ) %>%
    visNetwork::visInteraction(
      dragNodes = TRUE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visNetwork::visOptions(
      manipulation = curved,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn))
}


hist2vis <- function(
  net,
  labelsize = 2,
  nodesize = 2,
  curved = FALSE,
  shape = "dot",
  opacity = 0.7,
  labeltype = "short",
  timeline = TRUE
) {
  LABEL <- igraph::V(net$net)$id

  LABEL[igraph::V(net$net)$labelsize == 0] <- ""

  layout <- net$layout %>%
    dplyr::select(x, y, color, name)

  vn <- visNetwork::toVisNetworkData(net$net)

  vn$nodes$short_label <- LABEL

  if (labeltype != "short") {
    vn$nodes$label <- paste0(vn$nodes$years, ": ", LABEL)
  } else {
    vn$nodes$label <- LABEL
  }

  vn$nodes <- dplyr::left_join(vn$nodes, layout, by = c("id" = "name"))

  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE
  vn$edges$color <- "grey"

  ## opacity
  vn$nodes$font.color <- vn$nodes$color

  vn$nodes$color <- adjustcolor(
    vn$nodes$color,
    alpha.f = min(c(opacity - 0.2, 1))
  )
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity - 0.2)
  vn$edges$smooth <- curved

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  ## labelsize
  scalemin <- 20
  scalemax <- 150
  size <- 10 * labelsize
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size * 0.5
  vn$nodes$size <- nodesize * 2

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  text_data <- net$graph.data %>%
    select(Label, DOI, LCS, GCS) %>%
    rename(id = Label) %>%
    filter(!duplicated(id))

  vn$nodes <- vn$nodes %>% left_join(text_data, by = "id")

  ## split node tooltips into two strings
  title <- strsplit(stringi::stri_trans_totitle(vn$nodes$title), " ")

  vn$nodes$title <- unlist(lapply(title, function(l) {
    n <- floor(length(l) / 2)
    paste0(
      paste(l[1:n], collapse = " ", sep = ""),
      "<br>",
      paste(l[(n + 1):length(l)], collapse = " ", sep = "")
    )
  }))

  vn$nodes <- vn$nodes %>%
    mutate(
      title_orig = title,
      title = paste(
        "<b>Title</b>: ",
        title,
        "<br><b>DOI</b>: ",
        paste0(
          '<a href=\"https://doi.org/',
          DOI,
          '\" target=\"_blank\">',
          # "DOI: ",
          DOI,
          "</a>"
        ),
        "<br><b>GCS</b>: ",
        GCS,
        "<br><b>LCS</b>: ",
        LCS,
        sep = ""
      )
    )

  ## add time line
  vn$nodes$group <- "normal"
  vn$nodes$shape <- "dot"
  vn$nodes$shadow <- TRUE

  # nr <- nrow(vn$nodes)
  # y <- max(vn$nodes$y)
  # vn$nodes[nr + 1, c("id", "title", "label", "color", "font.color")] <-
  #   c(rep("logo", 3), "black", "white")
  # vn$nodes$x[nr + 1] <- max(vn$nodes$x, na.rm = TRUE) + 1
  # vn$nodes$y[nr + 1] <- y
  # vn$nodes$size[nr + 1] <- vn$nodes$size[nr] * 4
  # vn$nodes$years[nr + 1] <- as.numeric(vn$nodes$x[nr + 1])
  # vn$nodes$font.size[nr + 1] <- vn$nodes$font.size[nr]
  # vn$nodes$group[nr + 1] <- "logo"
  # vn$nodes$shape[nr + 1] <- "image"
  # vn$nodes$image[nr + 1] <- "logo.jpg"
  # vn$nodes$fixed.x <- TRUE
  # vn$nodes$fixed.y <- FALSE
  # vn$nodes$fixed.y[nr + 1] <- TRUE
  # vn$nodes$shadow[nr + 1] <- FALSE

  # coords <- vn$nodes[, c("x", "y")] %>%
  #   as.matrix()
  #
  # coords[, 2] <- coords[, 2]^(1 / 2)

  tooltipStyle <- ("position: fixed;visibility:hidden;padding: 5px;white-space: nowrap;
                  font-size:12px;font-color:black;background-color:white;")

  ## Font opacity
  vn$nodes$LCS[is.na(vn$nodes$LCS)] <- max(vn$nodes$LCS, na.rm = TRUE)
  opacity_font <- sqrt(
    (vn$nodes$LCS - min(vn$nodes$LCS)) / diff(range(vn$nodes$LCS))
  ) *
    0.6 +
    0.4

  vn$nodes$size <- opacity_font * 5 * nodesize
  vn$nodes$size[nrow(vn$nodes)] <- max(5 * nodesize)

  for (i in 1:nrow(vn$nodes)) {
    vn$nodes$font.color[i] <- adjustcolor(
      vn$nodes$font.color[i],
      alpha.f = opacity_font[i]
    )
  }

  x <- vn$nodes$x
  y <- vn$nodes$y
  vn$nodes$x <- y
  vn$nodes$y <- x

  vn$nodes <- assign_horizontal_coords_clusters_adaptive(vn$nodes)

  vn$nodes$fixed.x <- FALSE
  vn$nodes$fixed.y <- TRUE

  coords <- vn$nodes[, c("x", "y")] %>%
    as.matrix()
  coords[, 2] <- coords[, 2]

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = vn$nodes$shadow,
      shape = shape,
      size = vn$nodes$size,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    #visNetwork::visEdges(smooth = list(type = "horizontal"), arrows = list(to = list(enabled = TRUE, scaleFactor = 0.5))) %>%
    visNetwork::visEdges(
      smooth = list(enabled = TRUE, type = "dynamic", roundness = 0.3),
      arrows = list(to = list(enabled = TRUE, scaleFactor = 0.5))
    ) %>%
    visNetwork::visInteraction(
      dragNodes = T,
      navigationButtons = F,
      hideEdgesOnDrag = F,
      tooltipStyle = tooltipStyle,
      zoomSpeed = 0.2
    ) %>%
    visNetwork::visOptions(
      highlightNearest = list(
        enabled = T,
        hover = T,
        degree = list(from = 1),
        algorithm = "hierarchical"
      ),
      nodesIdSelection = F,
      manipulation = FALSE,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn, type = "historiograph", curved = curved))
}

## calculate node coordinates in historiograph
assign_horizontal_coords_clusters_adaptive <- function(
  nodes_df,
  spacing_base = 1.0,
  cluster_spacing = 6,
  tol = 0.15
) {
  clusters <- nodes_df %>%
    count(color, name = "n_cluster") %>%
    arrange(desc(n_cluster)) %>%
    mutate(
      cluster_id = row_number(),
      cluster_center = (cluster_id - mean(cluster_id)) * cluster_spacing
    )

  nodes_df <- nodes_df %>%
    left_join(clusters, by = "color")

  nodes_df <- nodes_df %>%
    group_by(years, color) %>%
    mutate(
      n_nodes = n(),
      spacing = spacing_base * n_nodes, # USA direttamente il numero di nodi
      x = (cluster_center[1] + spacing[1] * (row_number() - (n() + 1) / 2)) *
        runif(1, 1 - tol, 1 + tol)
    ) %>%
    ungroup()

  return(nodes_df)
}

## Pajek Export
graph2Pajek <- function(graph, filename = "my_pajek_network") {
  nodes <- igraph::as_data_frame(graph, what = c("vertices")) %>%
    mutate(id = row_number())

  edges <- igraph::as_data_frame(graph, what = c("edges"))
  edges <- edges %>%
    left_join(nodes %>% select(id, name), by = c("from" = "name")) %>%
    rename(id_from = id) %>%
    left_join(nodes %>% select(id, name), by = c("to" = "name")) %>%
    rename(id_to = id)

  ### Creation of NET file
  file <- paste0(filename, ".net")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$id, ' "', nodes$name, '"'), file = file, append = T)

  # Edges
  write(paste0("*Edges ", nrow(nodes)), file = file, append = T)
  write(
    paste0(edges$id_from, " ", edges$id_to, " ", edges$weight),
    file = file,
    append = T
  )

  ### Creation of VEC file
  file <- paste0(filename, ".vec")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$deg), file = file, append = T)

  ### Creation of CLU file
  file <- paste0(filename, ".clu")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$community), file = file, append = T)
}


## Dendogram to Visnetwork
dend2vis <- function(hc, labelsize, nclusters = 1, community = FALSE) {
  # community = TRUE means that hc is an igraph community detection object
  # community = FALSE mean that hc is a hclust object

  # transform and plot a community igraph object using dendrogram
  if (community) {
    hc <- as.hclust(hc, use.modularity = TRUE)
  }

  h_tail <- round((max(hc$height) * 0.12), 1)

  hc$height <- hc$height + h_tail

  VIS <- visHclust(
    hc,
    cutree = nclusters,
    colorEdges = "grey60",
    horizontal = TRUE,
    export = FALSE
  )
  VIS$x$edges <- data.frame(color = unique(VIS$x$edges$color)) %>%
    mutate(new_color = colorlist()[1:nrow(.)]) %>%
    right_join(VIS$x$edges, by = "color") %>%
    select(-color) %>%
    rename(color = new_color)
  VIS$x$nodes <- VIS$x$nodes %>%
    mutate(
      label = ifelse(group != "individual", NA, label),
      group = ifelse(group == "individual", "word", group),
      title = gsub("individuals", "words", title),
      value = 1,
      scaling.min = 10,
      scaling.max = 10
    )
  coords <- VIS$x$nodes %>%
    select(x, y) %>%
    as.matrix()

  edges <- VIS$x$edges
  nodes <- VIS$x$nodes %>%
    select(id, label) %>%
    dplyr::filter(label != "1")

  VIS$x$edges <- edges %>%
    select(-id) %>%
    left_join(nodes, by = c("to" = "id")) %>%
    select(-label.x) %>%
    rename(label = label.y) %>%
    mutate(
      value = 10,
      font.color = color,
      font.size = labelsize * 10,
      font.vadjust = -0.2 * font.size,
      label = ifelse(is.na(label), "", label)
    )

  VIS <- VIS %>%
    visGroups(
      groupname = "group",
      color = "gray90",
      shape = "dot",
      size = 10
    ) %>%
    visGroups(
      groupname = "word",
      font = list(size = 0),
      color = list(
        background = "white",
        border = "#80B1D3",
        highlight = "#e2e9e9",
        hover = "orange"
      ),
      shape = "box"
    ) %>%
    visNodes(font = list(align = VIS$x$nodes$font.align)) %>%
    visNetwork::visOptions(
      highlightNearest = list(
        enabled = T,
        hover = T,
        degree = list(to = 1000, from = 0),
        algorithm = "hierarchical"
      ),
      nodesIdSelection = FALSE,
      manipulation = FALSE,
      height = "100%",
      width = "100%"
    ) %>%
    visNetwork::visInteraction(
      dragNodes = FALSE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    visEdges(font = list(align = "top", size = VIS$x$edges$font.size)) %>%
    visEvents(
      click = "function(nodes){
                  Shiny.onInputChange('click_dend', nodes.nodes[0]);
                  ;}"
    )

  for (i in 1:nrow(VIS$x$nodes)) {
    if (VIS$x$nodes$group[i] == "group") {
      old_inertia <- as.character(VIS$x$nodes$inertia[i])
      inertia <- as.character(VIS$x$nodes$inertia[i] - h_tail)
      VIS$x$nodes$title[i] <- gsub(old_inertia, inertia, VIS$x$nodes$title[i])
    }
  }

  VIS
}

## Factorial Analysis dynamic plots
ca2plotly <- function(
  CS,
  method = "MCA",
  dimX = 1,
  dimY = 2,
  topWordPlot = Inf,
  threshold = 0.10,
  labelsize = 16,
  size = 5
) {
  LABEL <- CS$WData$word
  switch(
    method,
    CA = {
      contrib <- rowSums(CS$coord$contrib %>% as.data.frame()) / 2
      wordCoord <- CS$coord$coord[, 1:2] %>%
        data.frame() %>%
        mutate(
          label = LABEL,
          contrib = contrib
        ) %>%
        select(c(3, 1, 2, 4))
      row.names(wordCoord) <- LABEL
      xlabel <- paste0("Dim 1 (", round(CS$res$eigCorr$perc[1], 2), "%)")
      ylabel <- paste0("Dim 2 (", round(CS$res$eigCorr$perc[2], 2), "%)")
    },
    MCA = {
      contrib <- rowSums(CS$coord$contrib %>% as.data.frame()) / 2
      wordCoord <- CS$coord$coord[, 1:2] %>%
        data.frame() %>%
        mutate(
          label = LABEL,
          contrib = contrib
        ) %>%
        select(c(3, 1, 2, 4))
      row.names(wordCoord) <- LABEL
      xlabel <- paste0("Dim 1 (", round(CS$res$eigCorr$perc[1], 2), "%)")
      ylabel <- paste0("Dim 2 (", round(CS$res$eigCorr$perc[2], 2), "%)")
    },
    MDS = {
      contrib <- size
      xlabel <- "Dim 1"
      ylabel <- "Dim 2"
      wordCoord <- CS$WData %>%
        data.frame() %>%
        select(1:3) %>%
        mutate(contrib = contrib / 2) %>%
        rename(label = "word")
    }
  )

  dimContrLabel <- paste0("Contrib", c(dimX, dimY))
  ymax <- diff(range((wordCoord[, 3])))
  xmax <- diff(range((wordCoord[, 2])))
  threshold2 <- threshold * mean(xmax, ymax)

  # scaled size for dots
  dotScale <- (wordCoord$contrib) + size

  # Threshold labels to plot
  thres <- sort(dotScale, decreasing = TRUE)[min(topWordPlot, nrow(wordCoord))]

  names(wordCoord)[2:3] <- c("Dim1", "Dim2")

  wordCoord <- wordCoord %>%
    mutate(
      dotSize = dotScale,
      groups = CS$km.res$cluster,
      labelToPlot = ifelse(dotSize >= thres, label, ""),
      font.color = ifelse(
        labelToPlot == "",
        NA,
        adjustcolor(colorlist()[groups], alpha.f = 0.85)
      ),
      font.size = round(dotSize * 2, 0)
    )

  ## Avoid label overlapping
  labelToRemove <- avoidOverlaps(
    wordCoord,
    threshold = threshold2,
    dimX = dimX,
    dimY = dimY
  )
  wordCoord <- wordCoord %>%
    mutate(
      labelToPlot = ifelse(labelToPlot %in% labelToRemove, "", labelToPlot)
    ) %>%
    mutate(
      label = gsub("_1", "", label),
      labelToPlot = gsub("_1", "", labelToPlot)
    )

  hoverText <- paste(
    " <b>",
    wordCoord$label,
    "</b>\n Contribute: ",
    round(wordCoord$contrib, 3),
    sep = ""
  )

  fig <- plot_ly(
    data = wordCoord,
    x = wordCoord[, "Dim1"],
    y = wordCoord[, "Dim2"], # customdata=results$wordCoord,
    type = "scatter",
    mode = "markers",
    marker = list(
      size = dotScale,
      color = adjustcolor(colorlist()[wordCoord$groups], alpha.f = 0.3), #' rgb(79, 121, 66, .5)',
      line = list(
        color = adjustcolor(colorlist()[wordCoord$groups], alpha.f = 0.3), #' rgb(79, 121, 66, .8)',
        width = 2
      )
    ),
    text = hoverText,
    hoverinfo = "text",
    alpha = .3
  )

  fig <- fig %>%
    layout(
      yaxis = list(
        title = ylabel,
        showgrid = TRUE,
        showline = FALSE,
        showticklabels = TRUE,
        domain = c(0, 1)
      ),
      xaxis = list(
        title = xlabel,
        zeroline = TRUE,
        showgrid = TRUE,
        showline = FALSE,
        showticklabels = TRUE
      ),
      plot_bgcolor = "rgba(0, 0, 0, 0)",
      paper_bgcolor = "rgba(0, 0, 0, 0)"
    )

  for (i in seq_len(max(wordCoord$groups))) {
    if (method == "MDS") {
      w <- wordCoord %>%
        dplyr::filter(groups == i) %>%
        mutate(
          Dim1 = Dim1 + 0.005,
          Dim2 = Dim2 + 0.005
        )
    } else {
      w <- wordCoord %>%
        dplyr::filter(groups == i) %>%
        mutate(
          Dim1 = Dim1 + dotSize * 0.005,
          Dim2 = Dim2 + dotSize * 0.01
        )
    }

    if (max(CS$hull_data$clust) > 1) {
      hull_df <- CS$hull_data %>% dplyr::filter(clust == i)
      fig <- fig %>%
        add_polygons(
          x = hull_df$Dim1,
          y = hull_df$Dim2,
          inherit = FALSE,
          showlegend = FALSE,
          color = I(hull_df$color[1]),
          opacity = 0.3,
          line = list(width = 2),
          text = paste0("Cluster ", i),
          hoverinfo = "text",
          hoveron = "points"
        )
    }
    fig <- fig %>%
      add_annotations(
        data = w,
        x = ~Dim1,
        y = ~Dim2,
        xref = "x1",
        yref = "y",
        text = ~labelToPlot,
        font = list(
          family = "sans serif",
          size = labelsize,
          color = w$font.color[1]
        ), #' rgb(79, 121, 66)'),
        showarrow = FALSE
      )
  }

  fig <- fig %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        #' toImage',
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    ) %>%
    event_register("plotly_selecting")
  return(fig)
}


## function to avoid label overlapping ----
avoidOverlaps <- function(w, threshold = 0.10, dimX = 1, dimY = 2) {
  w[, "Dim2"] <- w[, "Dim2"] / 3

  Ds <- dist(
    w %>%
      dplyr::filter(labelToPlot != "") %>%
      select(Dim1, Dim2),
    method = "manhattan",
    upper = T
  ) %>%
    dist2df() %>%
    rename(
      from = row,
      to = col,
      dist = value
    ) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("from" = "labelToPlot")
    ) %>%
    rename(w_from = dotSize) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("to" = "labelToPlot")
    ) %>%
    rename(w_to = dotSize) %>%
    filter(dist < threshold)

  st <- TRUE
  i <- 1
  label <- NULL
  case <- "n"

  while (isTRUE(st)) {
    if (Ds$w_from[i] > Ds$w_to[i] & Ds$dist[i] < threshold) {
      case <- "y"
      lab <- Ds$to[i]
    } else if (Ds$w_from[i] <= Ds$w_to[i] & Ds$dist[i] < threshold) {
      case <- "y"
      lab <- Ds$from[i]
    }

    switch(
      case,
      "y" = {
        Ds <- Ds[Ds$from != lab, ]
        Ds <- Ds[Ds$to != lab, ]
        label <- c(label, lab)
      },
      "n" = {
        Ds <- Ds[-1, ]
      }
    )

    if (i >= nrow(Ds)) {
      st <- FALSE
    }
    case <- "n"
    # print(nrow(Ds))
  }

  label
}

## convert a distance object into a data.frame ----
dist2df <- function(inDist) {
  if (class(inDist) != "dist") {
    stop("wrong input type")
  }
  A <- attr(inDist, "Size")
  B <- if (is.null(attr(inDist, "Labels"))) {
    sequence(A)
  } else {
    attr(inDist, "Labels")
  }
  if (isTRUE(attr(inDist, "Diag"))) {
    attr(inDist, "Diag") <- FALSE
  }
  if (isTRUE(attr(inDist, "Upper"))) {
    attr(inDist, "Upper") <- FALSE
  }
  data.frame(
    row = B[unlist(lapply(sequence(A)[-1], function(x) x:A))],
    col = rep(B[-length(B)], (length(B) - 1):1),
    value = as.vector(inDist)
  )
}

### Excel report functions
addDataWb <- function(list_df, wb, sheetname) {
  l <- length(list_df)
  startRow <- 1
  for (i in 1:l) {
    df <- list_df[[i]]
    n <- nrow(df)
    writeDataTable(
      wb,
      sheetname,
      df,
      startRow = startRow,
      startCol = 1,
      tableStyle = "TableStyleMedium20"
    )
    startRow <- startRow + n + 3
  }
  return(wb)
}

addDataScreenWb <- function(list_df, wb, sheetname) {
  ind <- which(regexpr(sheetname, wb$sheet_names) > -1)
  if (length(ind) > 0) {
    sheetname <- paste(sheetname, "(", length(ind) + 1, ")", sep = "")
  }
  addWorksheet(wb = wb, sheetName = sheetname, gridLines = FALSE)
  if (!is.null(list_df)) {
    addDataWb(list_df, wb, sheetname)
    col <- max(unlist(lapply(list_df, ncol))) + 2
  } else {
    col <- 1
  }

  results <- list(wb = wb, col = col, sheetname = sheetname)
  return(results)
}

addGgplotsWb <- function(
  list_plot,
  wb,
  sheetname,
  col,
  width = 10,
  height = 7,
  dpi = 75
) {
  l <- length(list_plot)
  startRow <- 1
  for (i in seq_len(l)) {
    p <- list_plot[[i]]

    # Defensive guards. Without them, a NULL plot (e.g. user clicks the
    # report button before the corresponding tab has finished rendering)
    # or an unrecognised class would skip every writer branch below and
    # then crash openxlsx::insertImage with "File does not exist".
    if (is.null(p)) {
      warning(sprintf("Plot %d for sheet '%s' is NULL — skipped from report.",
                      i, sheetname), call. = FALSE)
      next
    }

    fileName <- file.path(
      getWD(),
      paste0("figureImage_", i, "_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".png")
    )

    written <- tryCatch({
      if (inherits(p, "ggplot")) {
        safe_ggsave(
          plot = p,
          filename = fileName,
          width = width,
          height = height,
          units = "in",
          dpi = dpi
        )
        TRUE
      } else if (inherits(p, "igraph")) {
        igraph2PNG(
          x = p,
          filename = fileName,
          width = width,
          height = height,
          dpi = dpi
        )
        TRUE
      } else if (inherits(p, "bibliodendrogram")) {
        png_dev <- safe_png_device()
        png_dev(
          filename = fileName,
          width = width,
          height = height,
          res = 300,
          units = "in"
        )
        plot(p)
        dev.off()
        TRUE
      } else {
        warning(sprintf(
          "Plot %d for sheet '%s' has unsupported class (%s) — skipped from report.",
          i, sheetname, paste(class(p), collapse = "/")
        ), call. = FALSE)
        FALSE
      }
    }, error = function(e) {
      warning(sprintf(
        "Could not export plot %d for sheet '%s': %s",
        i, sheetname, conditionMessage(e)
      ), call. = FALSE)
      FALSE
    })

    if (!isTRUE(written) || !file.exists(fileName)) {
      next
    }

    insertImage(
      wb = wb,
      sheet = sheetname,
      file = fileName,
      width = width,
      height = height,
      startRow = startRow,
      startCol = col,
      units = "in",
      dpi = dpi
    )
    startRow <- startRow + (height * 6) + 1
  }
  return(wb)
}

screenSh <- function(p, type = "vis", dpi = 300, height = 7) {
  #tmpdir <- getWD()

  fileName <- file.path(
    getWD(),
    paste0("figureImage_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".png")
  )

  plot2png(p, filename = fileName, type = type, dpi = dpi, height = height)

  return(fileName)
}

screenShot <- function(p, filename, type, dpi = 300, height = 7) {
  #home <- homeFolder()

  # # setting up the main directory
  # if ("downloads" %in% tolower(dir(home))) {
  #   filename <- file.path(home, "downloads", filename)
  # } else {
  #   filename <- file.path(home, filename)
  # }

  plot2png(p, filename, type = type, dpi = dpi, height = height)
  filename_html <- paste0(tools::file_path_sans_ext(filename), ".html")
  file.remove(filename_html)
  unlink(
    paste0(tools::file_path_sans_ext(filename), "_files"),
    recursive = TRUE
  )
}

plot2png <- function(p, filename, type = "vis", dpi = 300, height = 7) {
  zoom <- dpi / 96
  vwidth  <- round(height * 2 * 96)
  vheight <- round(height * 96)

  filename_html <- tools::file_path_sans_ext(filename)
  html_name <- paste0(filename_html, ".html")
  switch(
    type,
    vis = {
      visSave(p, html_name)
    },
    plotly = {
      htmlwidgets::saveWidget(p, file = html_name, selfcontained = FALSE)
    },
    df2html = {
      screenHtml(p, html_name)
    }
  )
  biblioShot(url = html_name, zoom = zoom, vwidth = vwidth, vheight = vheight, file = filename)

  popUpGeneric(
    title = NULL,
    type = "success",
    color = c("#1d8fe1"),
    subtitle = paste0("Plot saved to file:\n", basename(filename)),
    btn_labels = "OK",
    size = "40%"
  )
}

## Export Plotly Sankey to PNG using Plotly.toImage() via chromote
## Chromote has a rendering bug with complex SVG paths (Sankey links
## render at ~5% opacity). Plotly.toImage() uses canvas rendering instead.
plotlySankey2png <- function(p, filename, dpi = 300, height = 7) {
  vw <- max(round(height * 2 * 96), 1344)
  vh <- max(round(height * 96), 672)
  img_scale <- dpi / 96

  export_plot <- p %>%
    htmlwidgets::onRender(sprintf("
      function(el, x) {
        var plotDiv = el.querySelector('.js-plotly-plot') || el;
        Plotly.toImage(plotDiv, {format:'png', width:%d, height:%d, scale:%f}).then(function(url) {
          document.title = url;
        });
      }
    ", vw, vh, img_scale))

  html_name <- paste0(tools::file_path_sans_ext(filename), ".html")
  htmlwidgets::saveWidget(export_plot, file = html_name, selfcontained = TRUE)

  b <- chromote::ChromoteSession$new(width = vw, height = vh)
  tryCatch({
    p_event <- b$Page$loadEventFired(wait_ = FALSE)
    html_url <- paste0("file:///", normalizePath(html_name, winslash = "/", mustWork = TRUE))
    b$Page$navigate(html_url)
    b$wait_for(p_event)
    Sys.sleep(3)
    result <- b$Runtime$evaluate("document.title")
    base64_str <- result$result$value
    if (grepl("^data:image/png;base64,", base64_str)) {
      raw_data <- base64enc::base64decode(sub("^data:image/png;base64,", "", base64_str))
      writeBin(raw_data, filename)
    } else {
      # Fallback: call Plotly.toImage directly
      result2 <- b$Runtime$evaluate(sprintf("
        new Promise(function(resolve) {
          var plotDiv = document.querySelector('.js-plotly-plot');
          if (plotDiv) {
            Plotly.toImage(plotDiv, {format:'png', width:%d, height:%d, scale:%f}).then(resolve);
          } else { resolve(''); }
        })
      ", vw, vh, img_scale), awaitPromise = TRUE)
      base64_str2 <- result2$result$value
      if (nchar(base64_str2) > 100) {
        raw_data <- base64enc::base64decode(sub("^data:image/png;base64,", "", base64_str2))
        writeBin(raw_data, filename)
      } else {
        # Last resort: use biblioShot (links may be faint)
        biblioShot(url = html_name, file = filename,
          zoom = dpi / 96, vwidth = vw, vheight = vh, delay = 2)
      }
    }
  }, finally = {
    try(b$close(), silent = TRUE)
  })
  unlink(html_name)
  unlink(paste0(tools::file_path_sans_ext(html_name), "_files"), recursive = TRUE)
}

## screenshot of html objects
screenHtml <- function(df, html_file) {
  # Genera l'HTML direttamente usando htmlBoxFormat
  html_content <- htmlBoxFormat(
    df = df,
    title = "Missing Data Analysis",
    nrow = nrow(df),
    escape = FALSE,
    scrollX = TRUE,
    dom = FALSE,
    filter = "none",
    pagelength = FALSE,
    button = FALSE,
    round = 2
  )

  # Crea una pagina HTML completa con stili
  html_page <- sprintf(
    '
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Missing Data Table</title>
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css">
  <script src="https://cdn.jsdelivr.net/npm/xlsx@0.18.5/dist/xlsx.full.min.js"></script>
  <style>
    body {
      padding: 20px;
      background-color: white;
      font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;
    }
    .container-fluid {
      max-width: 100%%;
    }
  </style>
</head>
<body>
  <div class="container-fluid">
    %s
  </div>
</body>
</html>
    ',
    html_content
  )

  # Salva il file HTML
  writeLines(html_page, html_file, useBytes = TRUE)
}

## screenshot of an Author Profile card (HTML) to PNG ----
## The card (Global or Local profile) is a static HTML tag built with inline
## styles + the Bootstrap grid. It is wrapped in a standalone HTML page and
## captured with biblioShot(), cropping to the card via a CSS selector.
authorCard2png <- function(card, filename, selector = ".author-bio-card",
                           dpi = 300, height = 7) {
  zoom <- dpi / 96
  html_file <- paste0(tools::file_path_sans_ext(filename), ".html")
  out_dir <- dirname(html_file)

  # Make the card images available next to the HTML (cards use relative src).
  for (img in c("ORCID.jpg", "openalex.jpg")) {
    src <- file.path("www", img)
    if (file.exists(src)) {
      file.copy(src, file.path(out_dir, img), overwrite = TRUE)
    }
  }

  card_html <- as.character(card)

  html_page <- sprintf(
    '<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css">
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css">
  <style>
    body { margin: 0; padding: 25px; background: #ffffff;
           font-family: "Segoe UI", Tahoma, sans-serif; }
    .biblio-card-wrap { width: 950px; }
  </style>
</head>
<body>
  <div class="biblio-card-wrap">%s</div>
</body>
</html>',
    card_html
  )
  writeLines(html_page, html_file, useBytes = TRUE)

  biblioShot(
    url = html_file,
    file = filename,
    selector = selector,
    expand = c(20, 20, 20, 20),
    zoom = zoom,
    vwidth = 1100,
    vheight = 1400,
    delay = 1.2
  )

  unlink(html_file)
  filename
}

## export screen to browser download
screen2export <- function(filename = "file", obj, type, dpi = 300, height = 7) {
  JScode_screenshot <- "
    var link = document.createElement('a');
    link.href = '%s';
    link.download = '%s';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  "

  base_name <- paste0(
    filename,
    "-",
    gsub(" |:", "", Sys.time()),
    ".png",
    sep = ""
  )
  full_path <- file.path(getWD(), base_name)
  screenShot(
    obj,
    filename = full_path,
    type = type,
    dpi = dpi,
    height = height
  )
  img_data <- base64enc::dataURI(file = full_path, mime = "image/png")
  shinyjs::runjs(sprintf(JScode_screenshot, img_data, base_name))
}

addScreenWb <- function(df, wb, width = 14, height = 8, dpi = 75) {
  names(df) <- c("sheet", "file", "n")
  if (nrow(df) > 0) {
    sheet <- unique(df$sheet)
    for (i in 1:length(sheet)) {
      sh <- sheet[i]
      df_sh <- df %>% dplyr::filter(sheet == sh)
      l <- nrow(df_sh)
      startRow <- 1
      for (j in 1:l) {
        fileName <- df_sh$file[j]
        if (is.null(fileName) || is.na(fileName) || !nzchar(fileName) ||
            !file.exists(fileName)) {
          warning(sprintf(
            "Screenshot file '%s' for sheet '%s' is missing — skipped from report.",
            fileName %||% "<NA>", sh
          ), call. = FALSE)
          next
        }
        insertImage(
          wb = wb,
          sheet = sh,
          file = fileName,
          width = width,
          height = height,
          startRow = startRow,
          startCol = df_sh$n[j],
          units = "in",
          dpi = dpi
        )
        startRow <- startRow + (height * 10) + 3
      }
    }
  }
  return(wb)
}

addSheetToReport <- function(list_df, list_plot, sheetname, wb, dpi = 75) {
  ind <- which(regexpr(sheetname, wb$sheet_names) > -1)
  if (length(ind) > 0) {
    sheetname <- paste(sheetname, "(", length(ind) + 1, ")", sep = "")
  }
  addWorksheet(wb, sheetname, gridLines = FALSE)

  if (!is.null(list_df)) {
    col <- max(unlist(lapply(list_df, ncol))) + 2
    wb <- addDataWb(list_df, wb = wb, sheetname = sheetname)
  } else {
    col <- 1
  }

  if (!is.null(list_plot)) {
    wb <- addGgplotsWb(
      list_plot,
      wb = wb,
      sheetname = sheetname,
      col = col,
      dpi = dpi
    )
  }
  # values$sheet_name <- sheetname
  return(wb)
}

short2long <- function(df, myC) {
  z <- unlist(lapply(myC, function(x) {
    y <- gsub(r"{\s*\([^\)]+\)}", "", x)
    gsub(y, df$long[df$short == y], x)
  }))
  names(myC) <- z
  return(myC)
}

dfLabel <- function() {
  short <- c(
    "Empty Report",
    "MissingData",
    "MainInfo",
    "AnnualSciProd",
    "AnnualCitPerYear",
    "LifeCycle",
    "ThreeFieldsPlot",
    "MostRelSources",
    "MostLocCitSources",
    "BradfordLaw",
    "SourceLocImpact",
    "SourceProdOverTime",
    "MostRelAuthors",
    "MostLocCitAuthors",
    "AuthorProdOverTime",
    "LotkaLaw",
    "AuthorLocImpact",
    "MostRelAffiliations",
    "AffOverTime",
    "CorrAuthCountries",
    "CountrySciProd",
    "CountryProdOverTime",
    "MostCitCountries",
    "MostGlobCitDocs",
    "MostLocCitDocs",
    "MostLocCitRefs",
    "RPYS",
    "MostFreqWords",
    "WordCloud",
    "TreeMap",
    "WordFreqOverTime",
    "TrendTopics",
    "CouplingMap",
    "CoWordNet",
    "ThematicMap",
    "ThematicEvolution",
    "TE_Period_1",
    "TE_Period_2",
    "TE_Period_3",
    "TE_Period_4",
    "TE_Period_5",
    "FactorialAnalysis",
    "CoCitNet",
    "Historiograph",
    "CollabNet",
    "CollabWorldMap",
    "PRISMA_Diagram"
  )

  long <- c(
    "Empty Report",
    "Missing Data Table",
    "Main Information",
    "Annual Scientific Production",
    "Annual Citation Per Year",
    "Life Cycle of Publications",
    "Three-Field Plot",
    "Most Relevant Sources",
    "Most Local Cited Sources",
    "Bradfords Law",
    "Sources Local Impact",
    "Sources Production over Time",
    "Most Relevant Authors",
    "Most Local Cited Authors",
    "Authors Production over Time",
    "Lotkas Law",
    "Authors Local Impact",
    "Most Relevant Affiliations",
    "Affiliations Production over Time",
    "Corresponding Authors Countries",
    "Countries Scientific Production",
    "Countries Production over Time",
    "Most Cited Countries",
    "Most Global Cited Documents",
    "Most Local Cited Documents",
    "Most Local Cited References",
    "Reference Spectroscopy",
    "Most Frequent Words",
    "WordCloud",
    "TreeMap",
    "Words Frequency over Time",
    "Trend Topics",
    "Clustering by Coupling",
    "Co-occurence Network",
    "Thematic Map",
    "Thematic Evolution",
    "TE_Period_1",
    "TE_Period_2",
    "TE_Period_3",
    "TE_Period_4",
    "TE_Period_5",
    "Factorial Analysis",
    "Co-citation Network",
    "Historiograph",
    "Collaboration Network",
    "Countries Collaboration World Map",
    "PRISMA Diagram"
  )
  data.frame(short = short, long = long)
}

## Generic PopUp
popUpGeneric <- function(
  title = NULL,
  type = "success",
  color = c("#1d8fe1", "#913333", "#FFA800"),
  subtitle = "",
  btn_labels = "OK",
  ## "40%" is not a valid show_alert() size (the enum is xs/s/m/l/xl) and was
  ## silently dropped to the default, producing a very narrow modal. "l" gives
  ## these tabular pop-ups (Synonym List, Stopword List, ...) enough width to
  ## display single-word terms without mid-word breaks.
  size = "l"
) {
  showButton <- TRUE
  timer <- NA
  show_alert(
    title = title,
    text = subtitle,
    type = type,
    size = size,
    closeOnEsc = TRUE,
    closeOnClickOutside = TRUE,
    html = FALSE,
    showConfirmButton = showButton,
    showCancelButton = FALSE,
    btn_labels = btn_labels,
    btn_colors = color,
    timer = timer,
    imageUrl = "",
    animation = TRUE
  )
}


## Ad to Report PopUp
popUp <- function(title = NULL, type = "success", btn_labels = "OK") {
  switch(
    type,
    success = {
      title <- paste(title, "\n added to report", sep = "")
      subtitle <- ""
      btn_colors <- "#1d8fe1"
      showButton <- TRUE
      timer <- 3000
    },
    error = {
      title <- "No results to add to the report "
      subtitle <- "Please Run the analysis and then Add it to the report"
      btn_colors <- "#913333"
      showButton <- TRUE
      timer <- 3000
    },
    waiting = {
      title <- "Please wait... "
      subtitle <- "Adding results to report"
      btn_colors <- "#FFA800"
      showButton <- FALSE
      btn_labels <- NA
      timer <- NA
    }
  )

  show_alert(
    title = title,
    text = subtitle,
    type = type,
    size = "s",
    closeOnEsc = TRUE,
    closeOnClickOutside = TRUE,
    html = FALSE,
    showConfirmButton = showButton,
    showCancelButton = FALSE,
    btn_labels = btn_labels,
    btn_colors = btn_colors,
    timer = timer,
    imageUrl = "",
    animation = TRUE
  )
}

colorlist <- function() {
  c(
    "#E41A1C",
    "#377EB8",
    "#4DAF4A",
    "#984EA3",
    "#FF7F00",
    "#A65628",
    "#F781BF",
    "#999999",
    "#66C2A5",
    "#FC8D62",
    "#8DA0CB",
    "#E78AC3",
    "#A6D854",
    "#FFD92F",
    "#B3B3B3",
    "#A6CEE3",
    "#1F78B4",
    "#B2DF8A",
    "#33A02C",
    "#FB9A99",
    "#E31A1C",
    "#FDBF6F",
    "#FF7F00",
    "#CAB2D6",
    "#6A3D9A",
    "#B15928",
    "#8DD3C7",
    "#BEBADA",
    "#FB8072",
    "#80B1D3",
    "#FDB462",
    "#B3DE69",
    "#D9D9D9",
    "#BC80BD",
    "#CCEBC5"
  )
}

overlayPlotly <- function(VIS) {
  # colorscale_VOS=matrix(c(0, 'rgba(66,65,135,255)', 0.1, 'rgba(34,170,134,255)',
  #                         0.3, 'rgba(202,224,31,255)',
  #                         1, 'rgba(244,227,92,255)'),4,2, byrow=T)

  # colorscale_Our=matrix(c(0, 'rgba(238,238,238,255)',
  #                         0.1, 'rgba(232,202,177,255)',
  #                         0.2, 'rgba(217,137,100,255)',
  #                         0.6, 'rgba(199,107,90,255)',
  #                         0.9, 'rgba(164,38,39,255)',
  #                         1,   'rgba(178,34,34,255)'),
  #                       6,2, byrow=T)

  Reds <- matrix(
    c(
      "0",
      "rgb(255,255,255)",
      "0.05",
      "rgb(238,238,238)",
      "0.125",
      "rgb(254,224,210)",
      "0.25",
      "rgb(252,187,161)",
      "0.375",
      "rgb(252,146,114)",
      "0.5",
      "rgb(251,106,74)",
      "0.625",
      "rgb(239,59,44)",
      "0.75",
      "rgb(203,24,29)",
      "0.875",
      "rgb(165,15,21)",
      "1",
      "rgb(103,0,13)"
    )
  )

  nodes <- VIS$x$nodes %>%
    mutate(
      y = y * (-1),
      font.size = (((font.size - min(font.size)) / diff(range(font.size))) *
        20) +
        10
    )

  colori <- c(
    "Blackbody",
    "Bluered",
    "Blues",
    "Cividis",
    "Earth",
    "Electric",
    "Greens",
    "Greys",
    "Hot",
    "Jet",
    "Picnic",
    "Portland",
    "Rainbow",
    "RdBu",
    "Reds",
    "Viridis",
    "YlGnBu",
    "YlOrRd"
  )

  nodes2 <- nodes %>%
    group_by(id) %>%
    mutate(log = ceiling(log(deg))) %>%
    slice(rep(1, each = log))

  # Compute 2D kernel density estimation for smooth heatmap
  x_range <- range(nodes2$x, na.rm = TRUE)
  y_range <- range(nodes2$y, na.rm = TRUE)
  x_pad <- diff(x_range) * 0.15
  y_pad <- diff(y_range) * 0.15

  kde <- tryCatch({
    bw_x <- diff(x_range) / 4
    bw_y <- diff(y_range) / 4
    dens <- MASS::kde2d(nodes2$x, nodes2$y, n = 300,
                        h = c(bw_x, bw_y),
                        lims = c(x_range[1] - x_pad, x_range[2] + x_pad,
                                 y_range[1] - y_pad, y_range[2] + y_pad))
    list(x = dens$x, y = dens$y, z = dens$z)
  }, error = function(e) NULL)

  if (!is.null(kde)) {
    # z from kde2d is [x_index, y_index]; plotly heatmap expects z[y_index, x_index]
    p <- plotly::plot_ly(
      x = kde$x, y = kde$y, z = t(kde$z),
      type = "heatmap",
      colorscale = Reds,
      showscale = FALSE,
      zsmooth = "best",
      hoverinfo = "none"
    )
  } else {
    # Fallback to histogram2d
    p <- plot_ly(nodes2, x = ~x, y = ~y) %>%
      add_histogram2d(
        histnorm = "density",
        zsmooth = "fast",
        colorscale = Reds,
        showscale = FALSE
      )
  }

  for (i in 1:nrow(nodes)) {
    p <- p %>%
      add_annotations(
        xref = "x1",
        yref = "y",
        x = nodes$x[i],
        y = nodes$y[i],
        text = nodes$label[i],
        xanchor = "center",
        font = list(
          family = "Arial",
          size = nodes$font.size[i],
          color = adjustcolor(nodes$font.color[i], alpha.f = 0.8)
        ),
        showarrow = FALSE
      )
  }
  p <- p %>%
    layout(
      yaxis = list(
        title = "",
        zeroline = FALSE,
        showgrid = FALSE,
        showline = FALSE,
        showticklabels = FALSE,
        domain = c(-1, 1),
        gridcolor = "#FFFFFF",
        tickvals = list(NA)
      ),
      xaxis = list(
        title = "",
        zeroline = FALSE,
        showgrid = FALSE,
        showline = FALSE,
        showticklabels = FALSE,
        domain = c(-1, 1),
        gridcolor = "#FFFFFF",
        tickvals = list(NA)
      ),
      plot_bgcolor = "rgba(0, 0, 0, 0)",
      paper_bgcolor = "rgba(0, 0, 0, 0)",
      showlegend = FALSE
    ) %>%
    style(hoverinfo = "none") %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        #' toImage',
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    )
  return(p)
}


# MENU VISIBILITY MANAGEMENT ----
# This function updates visibility of submenu items based on metadata availability
updateMenuVisibility <- function(session, values) {
  # Extract metadata availability flags
  TC <- !("TC" %in% values$missTags)
  MLCS <- MLCA <- ("ISI" %in% values$M$DB[1] & !"CR" %in% values$missTags)
  ISI <- ("ISI" %in% values$M$DB[1])
  AFF <- !("C1" %in% values$missTags)
  CR <- !("CR" %in% values$missTags)
  MCC <- (!"TC" %in% values$missTags & !"C1" %in% values$missTags)
  DB_CR <- (sum(c("SCOPUS", "ISI") %in% values$M$DB[1]) > 0)
  DB_TC <- (sum(c("SCOPUS", "ISI", "OPENALEX", "LENS") %in% values$M$DB[1]) > 0)

  # Build JavaScript code to show/hide submenu items based on metadata
  js_code <- "
    // Helper function to find submenu item by text within a parent
    function findSubmenuItem(parentText, itemText) {
      var parent = null;
      $('.sidebar-menu > .treeview').each(function() {
        if ($(this).find('> a > span').first().text().trim() === parentText) {
          parent = $(this);
          return false;
        }
      });
      
      if (parent) {
        var found = null;
        parent.find('ul.treeview-menu > li').each(function() {
          var text = $(this).find('a span').first().text().trim();
          if (text === itemText) {
            found = $(this);
            return false;
          }
        });
        return found;
      }
      return null;
    }
    
    // Helper function to find section header (text-only li)
    function findSectionHeader(parentText, headerText) {
      var parent = null;
      $('.sidebar-menu > .treeview').each(function() {
        if ($(this).find('> a > span').first().text().trim() === parentText) {
          parent = $(this);
          return false;
        }
      });
      
      if (parent) {
        var found = null;
        parent.find('ul.treeview-menu > li').each(function() {
          if ($(this).children('a').length === 0 && $(this).text().trim() === headerText) {
            found = $(this);
            return false;
          }
        });
        return found;
      }
      return null;
    }
  "

  # Add conditional visibility for Sources submenu items
  if (MLCS) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Most Local Cited Sources')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Most Local Cited Sources')?.hide();"
    )
  }

  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Sources\\' Local Impact')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Sources\\' Local Impact')?.hide();"
    )
  }

  # Add conditional visibility for Authors submenu items
  if (MLCA) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Local Cited Authors')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Local Cited Authors')?.hide();"
    )
  }

  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Authors\\' Local Impact')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Authors\\' Local Impact')?.hide();"
    )
  }

  # Affiliations section
  if (AFF) {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Authors', 'Affiliations')?.show();
    findSubmenuItem('Authors', 'Most Relevant Affiliations')?.show();
    findSubmenuItem('Authors', 'Affiliations\\' Production over Time')?.show();
    findSectionHeader('Authors', 'Countries')?.show();
    findSubmenuItem('Authors', 'Corresponding Author\\'s Countries')?.show();
    findSubmenuItem('Authors', 'Countries\\' Scientific Production')?.show();
    findSubmenuItem('Authors', 'Countries\\' Production over Time')?.show();
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Authors', 'Affiliations')?.hide();
    findSubmenuItem('Authors', 'Most Relevant Affiliations')?.hide();
    findSubmenuItem('Authors', 'Affiliations\\' Production over Time')?.hide();
    findSectionHeader('Authors', 'Countries')?.hide();
    findSubmenuItem('Authors', 'Corresponding Author\\'s Countries')?.hide();
    findSubmenuItem('Authors', 'Countries\\' Scientific Production')?.hide();
    findSubmenuItem('Authors', 'Countries\\' Production over Time')?.hide();
    "
    )
  }

  if (MCC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Cited Countries')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Cited Countries')?.hide();"
    )
  }

  # Documents submenu items
  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Global Cited Documents')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Global Cited Documents')?.hide();"
    )
  }

  if (DB_TC && CR && TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Local Cited Documents')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Local Cited Documents')?.hide();"
    )
  }

  # Cited References section
  if (DB_CR) {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Documents', 'Cited References')?.show();
    findSubmenuItem('Documents', 'Most Local Cited References')?.show();
    findSubmenuItem('Documents', 'References Spectroscopy')?.show();
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Documents', 'Cited References')?.hide();
    findSubmenuItem('Documents', 'Most Local Cited References')?.hide();
    findSubmenuItem('Documents', 'References Spectroscopy')?.hide();
    "
    )
  }

  # Intellectual Structure - hide entire menu if no CR
  if (!CR) {
    js_code <- paste0(
      js_code,
      "
    $('.sidebar-menu .treeview').each(function() {
      if ($(this).find('> a > span').first().text().trim() === 'Intellectual Structure') {
        $(this).hide();
      }
    });
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    $('.sidebar-menu .treeview').each(function() {
      if ($(this).find('> a > span').first().text().trim() === 'Intellectual Structure') {
        $(this).show();
      }
    });
    "
    )

    if (DB_TC && CR) {
      js_code <- paste0(
        js_code,
        "\nfindSubmenuItem('Intellectual Structure', 'Historiograph')?.show();"
      )
    } else {
      js_code <- paste0(
        js_code,
        "\nfindSubmenuItem('Intellectual Structure', 'Historiograph')?.hide();"
      )
    }
  }

  # Social Structure
  if (AFF) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Social Structure', 'Countries\\' Collaboration World Map')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Social Structure', 'Countries\\' Collaboration World Map')?.hide();"
    )
  }

  # Execute the JavaScript code
  shinyjs::runjs(js_code)

  # Update list of disabled menu items for navigation control
  out <- character(0)

  if (!isTRUE(TC)) {
    out <- c(
      out,
      "Average Citations per Year",
      "Sources' Local Impact",
      "Authors' Local Impact",
      "Most Global Cited Documents"
    )
  }
  if (!isTRUE(MLCS)) {
    out <- c(out, "Most Local Cited Sources")
  }
  if (!isTRUE(ISI)) {
    out <- c(out, "Most Local Cited Authors")
  }
  if (!isTRUE(AFF)) {
    out <- c(
      out,
      "Most Relevant Affiliations",
      "Affiliations' Production over Time",
      "Corresponding Author's Countries",
      "Countries' Scientific Production",
      "Countries' Production over Time",
      "Countries' Collaboration World Map"
    )
  }
  if (!isTRUE(MCC)) {
    out <- c(out, "Most Cited Countries")
  }
  if (!(isTRUE(DB_TC) & isTRUE(CR) & isTRUE(TC))) {
    out <- c(out, "Most Local Cited Documents")
  }
  if (!isTRUE(DB_CR)) {
    out <- c(out, "Most Local Cited References", "References Spectroscopy")
  }
  if (!isTRUE(CR)) {
    out <- c(out, "Co-citation Network")
  }
  if (!(isTRUE(DB_TC) & isTRUE(CR))) {
    out <- c(out, "Historiograph")
  }

  values$out <- out
}

# find home folder
homeFolder <- function() {
  switch(
    Sys.info()[["sysname"]],
    Windows = {
      home <- Sys.getenv("R_USER")
    },
    Linux = {
      home <- Sys.getenv("HOME")
    },
    Darwin = {
      home <- Sys.getenv("HOME")
    }
  )
  return(home)
}

### PRISMA Flow Diagram ----
#' Draw a PRISMA Flow Diagram for bibliometric studies
#'
#' @param db_name Character. Name of the primary database.
#' @param query Character. Search query or topic description.
#' @param n_total Integer. Total records retrieved from primary source.
#' @param other_source Character. Description of additional source (or "").
#' @param n_other Integer. Records from additional source.
#' @param steps A data.frame with columns: phase, criterion, values, n_excluded, n_remaining.
#'   phase is one of "Screening" or "Eligibility".
#' @param n_included Integer. Final number of studies included.
#' @return Invisible NULL. Draws directly on the current graphics device.
drawPrismaFlowDiagram <- function(db_name = "Web of Science",
                                   query = "",
                                   n_total = 0,
                                   other_source = "",
                                   n_other = 0,
                                   steps = data.frame(),
                                   n_included = 0) {

  format_n <- function(n) format(n, big.mark = ",")

  # Wrap long text to fit within a given character width
  wrap_text <- function(text, max_chars = 50) {
    lines <- strsplit(text, "\n")[[1]]
    wrapped <- character(0)
    for (line in lines) {
      if (nchar(line) <= max_chars) {
        wrapped <- c(wrapped, line)
      } else {
        words <- strsplit(line, " ")[[1]]
        current <- words[1]
        for (w in words[-1]) {
          if (nchar(paste(current, w)) <= max_chars) {
            current <- paste(current, w)
          } else {
            wrapped <- c(wrapped, current)
            current <- w
          }
        }
        wrapped <- c(wrapped, current)
      }
    }
    paste(wrapped, collapse = "\n")
  }

  count_lines <- function(label) length(strsplit(label, "\n")[[1]])

  # Height per line of text in npc units
  line_h <- 0.022
  # Base box padding (top + bottom)
  pad_h <- 0.025

  # Compute box height from label
  box_height <- function(label) pad_h + count_lines(label) * line_h

  # Helper: draw box with text, supporting optional secondary text (smaller font)
  draw_box <- function(x, y, w, h, label, fill = "#D6E8F5", border = "#2B5C8A",
                       font_size = 14, font_face = "plain", text_col = "black",
                       sub_label = NULL, sub_font_size = 11) {
    grid::grid.roundrect(
      x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      width = grid::unit(w, "npc"), height = grid::unit(h, "npc"),
      r = grid::unit(0.008, "npc"),
      gp = grid::gpar(fill = fill, col = border, lwd = 2.5)
    )
    if (is.null(sub_label)) {
      grid::grid.text(
        label, x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
        gp = grid::gpar(fontsize = font_size, fontface = font_face,
                         col = text_col, lineheight = 1.15),
        just = "centre"
      )
    } else {
      # Two-part text: main label on top, sub_label below in smaller font
      n_main <- count_lines(label)
      n_sub <- count_lines(sub_label)
      total_text_h <- n_main * line_h + n_sub * (line_h * sub_font_size / font_size)
      y_main <- y + total_text_h / 2 - (n_main * line_h) / 2
      y_sub <- y_main - (n_main * line_h) / 2 - (n_sub * line_h * sub_font_size / font_size) / 2 - 0.003
      grid::grid.text(
        label, x = grid::unit(x, "npc"), y = grid::unit(y_main, "npc"),
        gp = grid::gpar(fontsize = font_size, fontface = font_face,
                         col = text_col, lineheight = 1.15),
        just = "centre"
      )
      grid::grid.text(
        sub_label, x = grid::unit(x, "npc"), y = grid::unit(y_sub, "npc"),
        gp = grid::gpar(fontsize = sub_font_size, fontface = "italic",
                         col = "#444444", lineheight = 1.10),
        just = "centre"
      )
    }
  }

  draw_phase_label <- function(x, y, w, h, label) {
    grid::grid.roundrect(
      x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      width = grid::unit(w, "npc"), height = grid::unit(h, "npc"),
      r = grid::unit(0.008, "npc"),
      gp = grid::gpar(fill = "#1B3A5C", col = "#1B3A5C", lwd = 2)
    )
    grid::grid.text(
      label, x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      gp = grid::gpar(fontsize = 15, fontface = "bold", col = "white")
    )
  }

  draw_arrow <- function(x0, y0, x1, y1, col = "#2B5C8A") {
    grid::grid.lines(
      x = grid::unit(c(x0, x1), "npc"), y = grid::unit(c(y0, y1), "npc"),
      arrow = grid::arrow(length = grid::unit(0.012, "npc"), type = "closed"),
      gp = grid::gpar(col = col, fill = col, lwd = 2.5)
    )
  }

  draw_line <- function(x0, y0, x1, y1, col = "#2B5C8A") {
    grid::grid.lines(
      x = grid::unit(c(x0, x1), "npc"), y = grid::unit(c(y0, y1), "npc"),
      gp = grid::gpar(col = col, lwd = 2)
    )
  }

  # Layout constants
  cx <- 0.42
  rbox_x <- 0.81
  bw <- 0.34
  rbw <- 0.28
  fs <- 14
  fs_side <- 13
  fs_query <- 11
  plx <- 0.09
  plw <- 0.13
  plh <- 0.042
  main_wrap <- 50   # max chars per line for main boxes
  side_wrap <- 35   # max chars per line for side boxes
  query_wrap <- 55  # wider wrap for query text (smaller font)

  has_other <- nchar(trimws(other_source)) > 0 && n_other > 0
  n_steps <- nrow(steps)

  # --- Pre-compute all labels and box heights ---

  # Identification: separate main label from query (shown in smaller font)
  has_query <- nchar(trimws(query)) > 0
  id_main_label <- paste0("Records identified from ", db_name, "\n(n = ", format_n(n_total), ")")
  id_main_label <- wrap_text(id_main_label, main_wrap)
  id_query_label <- NULL

  if (has_query) {
    id_query_label <- wrap_text(query, query_wrap)
    n_main <- count_lines(id_main_label)
    n_query <- count_lines(id_query_label)
    id_bh <- pad_h + n_main * line_h + n_query * (line_h * fs_query / fs) + 0.008
  } else {
    id_bh <- box_height(id_main_label)
  }

  # Total records label
  total_n <- n_total + ifelse(has_other, n_other, 0)
  total_label <- wrap_text(paste0("Total records retrieved\n(n = ", format_n(total_n), ")"), main_wrap)
  total_bh <- box_height(total_label)

  # Step labels
  step_labels <- character(0)
  step_bhs <- numeric(0)
  excl_labels <- character(0)
  excl_bhs <- numeric(0)
  for (i in seq_len(n_steps)) {
    step <- steps[i, ]
    has_values <- nchar(trimws(step$values)) > 0
    if (has_values) {
      ml <- paste0("After ", step$criterion, " filter\n",
                    step$values, "\n(n = ", format_n(step$n_remaining), ")")
    } else {
      ml <- paste0("After ", step$criterion, " filter\n(n = ", format_n(step$n_remaining), ")")
    }
    ml <- wrap_text(ml, main_wrap)
    step_labels <- c(step_labels, ml)
    step_bhs <- c(step_bhs, box_height(ml))

    el <- wrap_text(paste0("Excluded by ", step$criterion, "\n(n = ", format_n(step$n_excluded), ")"), side_wrap)
    excl_labels <- c(excl_labels, el)
    excl_bhs <- c(excl_bhs, box_height(el))
  }

  # Inclusion label
  inc_label <- wrap_text(paste0("Studies included in the analysis\n(n = ", format_n(n_included), ")"), main_wrap)
  inc_bh <- box_height(inc_label)

  # Other source label
  if (has_other) {
    other_label <- wrap_text(paste0("Records from\n", other_source, "\n(n = ", format_n(n_other), ")"), main_wrap)
    other_bh <- box_height(other_label)
  }

  # --- Compute Y positions: distribute evenly to fill vertical space ---
  all_bhs <- c(id_bh, total_bh, step_bhs, inc_bh)
  n_rows <- length(all_bhs)

  y_top_start <- 0.91
  y_bottom_end <- 0.05
  total_space <- y_top_start - y_bottom_end
  total_box_h <- sum(all_bhs)

  # Distribute remaining space equally as gaps
  gap <- if (n_rows > 1) (total_space - total_box_h) / (n_rows - 1) else 0
  gap <- max(gap, 0.010)  # minimum gap

  # If boxes + min gaps exceed space, shrink proportionally
  if (total_box_h + (n_rows - 1) * 0.010 > total_space) {
    scale <- (total_space - (n_rows - 1) * 0.010) / total_box_h
    all_bhs <- all_bhs * scale
    id_bh <- all_bhs[1]; total_bh <- all_bhs[2]
    if (n_steps > 0) step_bhs <- all_bhs[3:(2 + n_steps)]
    inc_bh <- all_bhs[n_rows]
    gap <- 0.010
  }

  # Compute y centers top-down
  y_centers <- numeric(n_rows)
  y_cur <- y_top_start - all_bhs[1] / 2
  y_centers[1] <- y_cur
  for (j in 2:n_rows) {
    y_cur <- y_cur - all_bhs[j - 1] / 2 - gap - all_bhs[j] / 2
    y_centers[j] <- y_cur
  }

  y_id <- y_centers[1]
  y_total <- y_centers[2]
  y_steps <- if (n_steps > 0) y_centers[3:(2 + n_steps)] else numeric(0)
  y_inc <- y_centers[n_rows]

  # Phase labels y positions
  screening_ys <- if (n_steps > 0) y_steps[steps$phase == "Screening"] else numeric(0)
  eligibility_ys <- if (n_steps > 0) y_steps[steps$phase == "Eligibility"] else numeric(0)

  # --- Start drawing ---
  grid::grid.newpage()
  grid::pushViewport(grid::viewport(width = 0.96, height = 0.96))
  grid::grid.rect(gp = grid::gpar(fill = "white", col = NA))

  # Title
  grid::grid.text("PRISMA Flow Diagram",
                   x = grid::unit(0.5, "npc"), y = grid::unit(0.97, "npc"),
                   gp = grid::gpar(fontsize = 22, fontface = "bold", col = "#1B3A5C"))
  grid::grid.text("(adapted for bibliometric studies)",
                   x = grid::unit(0.5, "npc"), y = grid::unit(0.945, "npc"),
                   gp = grid::gpar(fontsize = 14, fontface = "italic", col = "#555555"))

  # Phase labels
  draw_phase_label(plx, y_id, plw, plh, "Identification")
  if (length(screening_ys) > 0)
    draw_phase_label(plx, screening_ys[1], plw, plh, "Screening")
  if (length(eligibility_ys) > 0)
    draw_phase_label(plx, eligibility_ys[1], plw, plh, "Eligibility")
  draw_phase_label(plx, y_inc, plw, plh, "Inclusion")

  # Identification boxes
  if (has_other) {
    id_left_x <- 0.32
    id_right_x <- 0.58
    id_bw <- 0.24
    draw_box(id_left_x, y_id, id_bw, id_bh, id_main_label,
             font_size = fs, sub_label = id_query_label, sub_font_size = fs_query)
    draw_box(id_right_x, y_id, id_bw, other_bh, other_label, font_size = fs - 1)
    draw_arrow(id_left_x, y_id - id_bh / 2, cx, y_total + total_bh / 2)
    draw_arrow(id_right_x, y_id - other_bh / 2, cx, y_total + total_bh / 2)
  } else {
    draw_box(cx, y_id, bw, id_bh, id_main_label,
             font_size = fs, sub_label = id_query_label, sub_font_size = fs_query)
    draw_arrow(cx, y_id - id_bh / 2, cx, y_total + total_bh / 2)
  }

  # Total records
  draw_box(cx, y_total, bw, total_bh, total_label, font_size = fs)

  # Steps
  prev_y <- y_total
  prev_bh <- total_bh
  for (i in seq_len(n_steps)) {
    cur_y <- y_steps[i]
    cur_bh <- step_bhs[i]

    draw_arrow(cx, prev_y - prev_bh / 2, cx, cur_y + cur_bh / 2)
    draw_box(cx, cur_y, bw, cur_bh, step_labels[i], font_size = fs)

    # Right exclusion box
    draw_box(rbox_x, cur_y, rbw, excl_bhs[i], excl_labels[i],
             fill = "#FFFFFF", border = "#8B2020",
             font_size = fs_side, text_col = "#8B2020")
    draw_line(cx + bw / 2, cur_y, rbox_x - rbw / 2, cur_y, col = "#8B2020")

    prev_y <- cur_y
    prev_bh <- cur_bh
  }

  # Inclusion
  draw_arrow(cx, prev_y - prev_bh / 2, cx, y_inc + inc_bh / 2)

  draw_box(cx, y_inc, bw, inc_bh, inc_label,
           fill = "#D6E8F5", border = "#1B3A5C",
           font_size = fs, font_face = "bold")

  grid::popViewport()
  invisible(NULL)
}

# Acknowledgment
# This function is based on the source code of the webshot2 package.
# We would like to acknowledge and express our gratitude to the authors
# and contributors of webshot2 for their valuable work in developing tools
# for web content capture.
#
# In this implementation, we have adapted and modified parts of the original
# webshot2 code to address an issue where PNG files were being generated as
# empty images when used on Windows systems. Our modifications aim to improve
# compatibility and ensure reliable output across different platforms.
#
# Reference:
# webshot2 package: https://github.com/rstudio/webshot2

biblioShot <- function(
  url = NULL,
  file = "biblioShot.png",
  vwidth = 992,
  vheight = 744,
  selector = NULL,
  cliprect = NULL,
  expand = NULL,
  delay = 0.3,
  zoom = 1,
  useragent = NULL,
  max_concurrent = getOption("biblioShot.concurrent", default = 6),
  verbose = FALSE
) {
  if (length(url) == 0) {
    stop("Need url.")
  }

  url <- vapply(
    url,
    function(x) {
      if (!is_url(x)) {
        file_url(x)
      } else {
        x
      }
    },
    character(1)
  )

  if (!is.null(cliprect) && !is.list(cliprect)) {
    cliprect <- list(cliprect)
  }
  if (!is.null(selector) && !is.list(selector)) {
    selector <- list(selector)
  }
  if (!is.null(expand) && !is.list(expand)) {
    expand <- list(expand)
  }

  if (is.null(selector)) {
    selector <- "html"
  }

  if (length(url) > 1 && length(file) == 1) {
    file <- vapply(1:length(url), FUN.VALUE = character(1), function(i) {
      replacement <- sprintf("%03d.\\1", i)
      gsub("\\.(.{3,4})$", replacement, file)
    })
  }

  args_all <- list(
    url = url,
    file = file,
    vwidth = vwidth,
    vheight = vheight,
    selector = selector,
    cliprect = cliprect,
    expand = expand,
    delay = delay,
    zoom = zoom,
    useragent = useragent,
    verbose = verbose
  )

  n_urls <- length(url)
  args_all <- mapply(
    args_all,
    names(args_all),
    FUN = function(arg, name) {
      if (length(arg) == 0) {
        return(vector(mode = "list", n_urls))
      } else if (length(arg) == 1) {
        return(rep(arg, n_urls))
      } else if (length(arg) == n_urls) {
        return(arg)
      } else {
        stop(
          "Argument `",
          name,
          "` should be NULL, length 1, or same length as `url`."
        )
      }
    },
    SIMPLIFY = FALSE
  )

  args_all <- long_to_wide(args_all)

  cm <- default_chromote_object()

  # A list of promises for the screenshots
  res <- lapply(args_all, function(args) {
    new_session_screenshot(
      cm,
      args$url,
      args$file,
      args$vwidth,
      args$vheight,
      args$selector,
      args$cliprect,
      args$expand,
      args$delay,
      args$zoom,
      args$useragent,
      verbose
    )
  })

  p <- promises::promise_all(.list = res)
  res <- cm$wait_for(p)
  res <- structure(unlist(res), class = "webshot")
  res
}


new_session_screenshot <- function(
  chromote,
  url,
  file,
  vwidth,
  vheight,
  selector,
  cliprect,
  expand,
  delay,
  zoom,
  useragent,
  verbose = FALSE
) {
  filetype <- tolower(tools::file_ext(file))
  if (filetype != "png" && filetype != "pdf") {
    stop("File extension must be 'png' or 'pdf'")
  }

  if (is.null(selector)) {
    selector <- "html"
  }

  if (is.character(cliprect)) {
    if (cliprect == "viewport") {
      cliprect <- c(0, 0, vwidth, vheight)
    } else {
      stop("Invalid value for cliprect: ", cliprect)
    }
  } else {
    if (
      !is.null(cliprect) && !(is.numeric(cliprect) && length(cliprect) == 4)
    ) {
      stop(
        "`cliprect` must be a vector with four numbers, or a list of such vectors"
      )
    }
  }

  s <- NULL

  p <- chromote$new_session(
    wait_ = FALSE,
    width = vwidth,
    height = vheight
  )$then(function(session) {
    s <<- session

    if (!is.null(useragent)) {
      s$Network$setUserAgentOverride(userAgent = useragent)
    }
    res <- s$Page$loadEventFired(wait_ = FALSE)
    s$Page$navigate(url, wait_ = FALSE)
    res
  })$then(function(value) {
    if (delay > 0) {
      promises::promise(function(resolve, reject) {
        later::later(
          function() {
            resolve(value)
          },
          delay
        )
      })
    } else {
      value
    }
  })$then(function(value) {
    if (filetype == "png") {
      s$screenshot(
        filename = file,
        selector = selector,
        cliprect = cliprect,
        expand = expand,
        scale = zoom,
        show = FALSE,
        wait_ = FALSE
      )
    } else if (filetype == "pdf") {
      s$screenshot_pdf(filename = file, wait_ = FALSE)
    }
  })$then(function(value) {
    if (verbose) {
      message(url, " screenshot completed")
    }
    normalizePath(value)
  })$finally(function() {
    s$close()
  })

  p
}

is_url <- function(x) {
  grepl("^[a-zA-Z]+://", x)
}

# Given the path to a file, return a file:// URL.
file_url <- function(filename) {
  if (is_windows()) {
    paste0("file://", normalizePath(filename, mustWork = TRUE))
  } else {
    enc2utf8(paste0(
      "file:///",
      normalizePath(filename, winslash = "/", mustWork = TRUE)
    ))
  }
}

is_windows <- function() .Platform$OS.type == "windows"

is_mac <- function() Sys.info()[["sysname"]] == "Darwin"

is_linux <- function() Sys.info()[["sysname"]] == "Linux"

long_to_wide <- function(x) {
  if (length(x) == 0) {
    return(x)
  }

  lapply(seq_along(x[[1]]), function(n) {
    lapply(stats::setNames(names(x), names(x)), function(name) {
      x[[name]][[n]]
    })
  })
}

#' Import and Convert bibliographic export files and API objects.
#'
#' It converts a SCOPUS, Clarivate Analytics WoS, Dimensions, Lens.org, PubMed and COCHRANE Database export files or pubmedR and dimensionsR JSON/XML
#' objects into a data frame, with cases corresponding to articles and variables to Field Tags as used in WoS.
#'
#' @param file a character array containing a sequence of filenames coming from WoS, Scopus, Dimensions, Lens.org, OpenAlex and Pubmed. Alternatively, \code{file} can be
#' an object resulting from an API query fetched from Dimensions, and PubMed databases:
#' \tabular{lll}{
#' a)\tab 'wos' \tab Clarivate Analytics WoS (in plaintext '.txt', Endnote Desktop '.ciw', or bibtex formats '.bib');\cr
#' b)\tab 'scopus' \tab SCOPUS (exclusively in bibtex format '.bib');\cr
#' c)\tab 'dimensions' \tab Digital Science Dimensions (in csv '.csv' or excel '.xlsx' formats);\cr
#' d)\tab 'lens' \tab Lens.org (in csv '.csv');\cr
#' e)\tab 'pubmed' \tab an object of the class \code{pubmedR (package pubmedR)} containing a collection obtained from a query performed with pubmedR package;\cr
#' f)\tab 'dimensions' \tab an object of the class \code{dimensionsR (package dimensionsR)} containing a collection obtained from a query performed with dimensionsR package;\cr
#' g)\tab 'openalex' \tab OpenAlex .csv file;\cr
#' h)\tab 'openalex_api' \tab the filename and path to a list object returned by openalexR package, containing a collection of works resulting from a query fetched from OpenAlex database.}
#' @param dbsource is a character indicating the bibliographic database. \code{dbsource} can be \code{dbsource = c('cochrane','dimensions','generic','isi','openalex', 'pubmed','scopus','wos', 'lens')} . Default is \code{dbsource = "isi"}.
#' @param format is a character indicating the SCOPUS, Clarivate Analytics WoS, and other databases export file format. \code{format} can be \code{c('api', 'bibtex', 'csv', 'endnote','excel','plaintext', 'pubmed')}. Default is \code{format = "plaintext"}.
#' @param remove.duplicates is logical. If TRUE, the function will remove duplicated items checking by DOI and database ID.
#' @return a data frame with cases corresponding to articles and variables to Field Tags in the original export file.
#'
#' I.e We have three files download from Web of Science in plaintext format, file will be:
#'
#' file <- c("filename1.txt", "filename2.txt", "filename3.txt")
#'
#' data frame columns are named using the standard Clarivate Analytics WoS Field Tag codify. The main field tags are:
#'
#' \tabular{lll}{
#' \code{AU}\tab   \tab Authors\cr
#' \code{TI}\tab   \tab Document Title\cr
#' \code{SO}\tab   \tab Publication Name (or Source)\cr
#' \code{JI}\tab   \tab ISO Source Abbreviation\cr
#' \code{DT}\tab   \tab Document Type\cr
#' \code{DE}\tab   \tab Authors' Keywords\cr
#' \code{ID}\tab   \tab Keywords associated by SCOPUS or WoS database \cr
#' \code{AB}\tab   \tab Abstract\cr
#' \code{C1}\tab   \tab Author Address\cr
#' \code{RP}\tab   \tab Reprint Address\cr
#' \code{CR}\tab   \tab Cited References\cr
#' \code{TC}\tab   \tab Times Cited\cr
#' \code{PY}\tab   \tab Year\cr
#' \code{SC}\tab   \tab Subject Category\cr
#' \code{UT}\tab   \tab Unique Article Identifier\cr
#' \code{DB}\tab   \tab Database\cr}
#'
#' for a complete list of field tags see: \href{https://www.bibliometrix.org/documents/Field_Tags_bibliometrix.pdf}{Field Tags used in bibliometrix}
#'
#' @examples
#'
#' # Example:
#' # Import and convert a Web of Science collection form an export file in plaintext format:
#'
#' \dontrun{
#' files <- "https://www.bibliometrix.org/datasets/wos_plaintext.txt"
#'
#' M <- convert2df(file = files, dbsource = "wos", format = "plaintext")
#' }
#'
#' @export

convert2df <- function(
  file,
  dbsource = "wos",
  format = "plaintext",
  remove.duplicates = TRUE
) {
  allowed_formats <- c(
    "api",
    "bibtex",
    "csv",
    "endnote",
    "excel",
    "plaintext",
    "pubmed"
  )
  allowed_db <- c(
    "cochrane",
    "dimensions",
    "generic",
    "isi",
    "openalex",
    "openalex_api",
    "pubmed",
    "scopus",
    "wos",
    "lens"
  )

  cat(
    "\nConverting your",
    dbsource,
    "collection into a bibliographic dataframe\n\n"
  )
  if (length(setdiff(dbsource, allowed_db)) > 0) {
    cat("\n 'dbsource' argument is not properly specified")
    cat(
      "\n 'dbsource' argument has to be a character string matching one among:",
      allowed_db,
      "\n"
    )
  }
  if (length(setdiff(format, allowed_formats)) > 0) {
    cat("\n 'format' argument is not properly specified")
    cat(
      "\n 'format' argument has to be a character string matching one among:",
      allowed_formats,
      "\n"
    )
  }

  if (dbsource == "wos") {
    dbsource <- "isi"
  }
  if (format == "endnote") {
    format <- "plaintext"
  }
  if (format == "lens") {
    format <- "csv"
  }

  switch(
    dbsource,
    ## db WOS
    isi = {
      switch(
        format,
        bibtex = {
          D <- importFiles(file)
          M <- bib2df(D, dbsource = "isi")
        },
        plaintext = {
          D <- importFiles(file)
          M <- isi2df(D)
        }
      )
    },
    ## db SCOPUS
    scopus = {
      switch(
        format,
        bibtex = {
          D <- importFiles(file)
          M <- bib2df(D, dbsource = "scopus")
        },
        csv = {
          M <- csvScopus2df(file)
        }
      )
    },
    ## db GENERIC BIBTEX
    generic = {
      D <- importFiles(file)
      M <- bib2df(D, dbsource = "generic")
    },
    ## db LENS
    lens = {
      M <- csvLens2df(file)
    },
    ## db PUBMED
    pubmed = {
      switch(
        format,
        api = {
          M <- pmApi2df(file)
          M$DB <- "PUBMED"
        },
        {
          D <- importFiles(file)
          M <- pubmed2df(D)
        }
      )
    },
    ## db COCHRANE
    cochrane = {
      D <- importFiles(file)
      M <- cochrane2df(D)
    },
    ## db DIMENSIONS
    dimensions = {
      switch(
        format,
        api = {
          M <- dsApi2df(file)
          M$DB <- "DIMENSIONS"
        },
        {
          M <- dimensions2df(file, format = format)
          M$DB <- "DIMENSIONS"
        }
      )
    },
    openalex = {
      M <- csvOA2df(file)
      M <- filter_keywords_by_score(
        M %>% rename(ID_raw = ID),
        input_col = "ID_raw",
        output_col = "ID",
        threshold = 0.5
      )
    },
    openalex_api = {
      M <- apiOA2df(file)
      M <- filter_keywords_by_score(
        M %>% rename(ID_raw = ID),
        input_col = "ID_raw",
        output_col = "ID",
        threshold = 0.5
      )
    }
  )
  if ("PY" %in% names(M)) {
    M$PY <- as.numeric(M$PY)
  } else {
    M$PY <- NA
  }

  if ("TC" %in% names(M)) {
    M$TC <- as.numeric(M$TC)
    M$TC[is.na(M$TC)] <- 0
  } else {
    M$TC <- 0
  }

  if (!("CR" %in% names(M))) {
    M$CR <- "none"
  } else {
    M$CR <- trim.leading(trimES(gsub(
      "\\[,||\\[||\\]|| \\.\\. || \\. ",
      "",
      M$CR
    ))) # remove foreign characters from CR (i.e. Chinese, Russian characters)
  }

  if (dbsource %in% c("cochrane", "openalex_api")) {
    M$AU <- gsub(intToUtf8(8217), intToUtf8(39), M$AU)
  }

  cat("Done!\n\n")

  if (!(dbsource %in% c("pubmed", "lens", "openalex_api"))) {
    ## AU_UN field creation
    if ("C1" %in% names(M)) {
      cat("\nGenerating affiliation field tag AU_UN from C1:  ")

      if (!"AU_UN" %in% names(M)) {
        M <- metaTagExtraction(M, Field = "AU_UN")
      }
      cat("Done!\n\n")
    } else {
      M$C1 <- NA
      M$AU_UN <- NA
    }

    ## AU normalization
    M$AU <- unlist(lapply(strsplit(M$AU, ";"), function(x) {
      x <- trimws(trimES(gsub("[^[:alnum:][-]']", " ", x)))
      x <- paste(x, collapse = ";")
    }))
  } else if (dbsource == "openalex_api") {
    # AU separator normalization only: apiOA2df collapses author names with "; "
    # while every other dbsource uses ";". Strip the trailing space(s) so the
    # AU delimiter is consistent across sources (issue #596). Full author-name
    # format is preserved (no regex stripping) because the API returns full
    # names that may include hyphens (e.g. U+2010), dots, and accented chars.
    M$AU <- gsub("; +", ";", M$AU)
  }

  if ((dbsource == "pubmed") & (format == "pubmed")) {
    if ("C1" %in% names(M)) {
      cat("\nGenerating affiliation field tag AU_UN from C1:  ")

      M <- metaTagExtraction(M, Field = "AU_UN")
      cat("Done!\n\n")
    } else {
      M$C1 <- NA
      M$AU_UN <- NA
    }
  }

  # remove double ;
  M <- M %>%
    mutate(across(where(is.character), ~ gsub(";;", ";", .x)))

  ### SR field creation
  if (isTRUE(remove.duplicates)) {
    switch(
      dbsource,
      isi = {
        id_field <- "UT"
      },
      scopus = {
        if (format == "csv") {
          id_field <- "UT"
        } else {
          id_field <- "TI"
        }
      },
      openalex = {
        id_field <- "id_oa"
      },
      openalex_api = {
        id_field <- "id_oa"
      },
      dimneisons = {
        id_field <- "UT"
      },
      pubmed = {
        id_field <- "PMID"
      },
      lens = {
        id_field <- "UT"
      },
      {
        id_field <- "TI"
      }
    )
    d <- duplicated(M[id_field])
    if (sum(d) > 0) {
      cat("\nRemoved ", sum(d), "duplicated documents\n")
    }
    M <- M[!d, ]
  }
  suppressWarnings(M <- metaTagExtraction(M, Field = "SR"))
  row.names(M) <- M$SR

  # Create field KW_Merged
  M <- M %>% mergeKeywords(force = T)

  ### bibliometrix>DB class
  class(M) <- c("bibliometrixDB", "data.frame")

  return(M)
}

utils::globalVariables(c("countries", "continent"))
#' Country Collaboration Networks by Region
#'
#' A function to create and plot country collaboration networks by Region
#'
#' @param NetMatrix is a country collaboration matrix obtained by the function \code{\link{biblioNetwork}}.
#' @param normalize is a character. It can be "association", "jaccard", "inclusion","salton" or "equivalence" to obtain Association Strength, Jaccard,
#' Inclusion, Salton or Equivalence similarity index respectively. The default is type = NULL.
#' @param n is an integer. It indicates the number of vertices to plot.
#' @param degree is an integer. It indicates the min frequency of a vertex. If degree is not NULL, n is ignored.
#' @param type is a character object. It indicates the network map layout:
#'
#' \tabular{lll}{
#' \code{type="auto"}\tab   \tab Automatic layout selection\cr
#' \code{type="circle"}\tab   \tab Circle layout\cr
#' \code{type="sphere"}\tab   \tab Sphere layout\cr
#' \code{type="mds"}\tab   \tab Multidimensional Scaling layout\cr
#' \code{type="fruchterman"}\tab   \tab Fruchterman-Reingold layout\cr
#' \code{type="kamada"}\tab   \tab  Kamada-Kawai layout}
#'
#' @param vos.path is a character indicating the full path where VOSviewer.jar is located.
#' @param size is integer. It defines the size of each vertex. Default is \code{size=3}.
#' @param size.cex is logical. If TRUE the size of each vertex is proportional to its degree.
#' @param noloops is logical. If TRUE loops in the network are deleted.
#' @param remove.isolates is logical. If TRUE isolates vertices are not plotted.
#' @param remove.multiple is logical. If TRUE multiple links are plotted using just one edge.
#' @param label is logical. If TRUE vertex labels are plotted.
#' @param labelsize is an integer. It indicates the label size in the plot. Default is \code{labelsize=1}
#' @param label.color is logical. If TRUE, for each vertex, the label color is the same as its cluster.
#' @param label.cex is logical. If TRUE the label size of each vertex is proportional to its degree.
#' @param halo is logical. If TRUE communities are plotted using different colors. Default is \code{halo=FALSE}
#' @param cluster is a character. It indicates the type of cluster to perform among ("none", "optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").
#' @param community.repulsion is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.1}.
#' @param curved is a logical or a number. If TRUE edges are plotted with an optimal curvature. Default is \code{curved=FALSE}. Curved values are any numbers from 0 to 1.
#' @param weighted This argument specifies whether to create a weighted graph from an adjacency matrix.
#' If it is NULL then an unweighted graph is created and the elements of the adjacency matrix gives the number of edges between the vertices.
#' If it is a character constant then for every non-zero matrix entry an edge is created and the value of the entry is added as an edge attribute
#' named by the weighted argument. If it is TRUE then a weighted graph is created and the name of the edge attribute will be weight.
#' @param edgesize is an integer. It indicates the network edge size.
#' @param edges.min is an integer. It indicates the min frequency of edges between two vertices. If edge.min=0, all edges are plotted.
#' @param label.n is an integer. It indicates the number of vertex labels to draw.
#' @param alpha is a number. Legal alpha values are any numbers from 0 (transparent) to 1 (opaque). The default alpha value usually is 0.5.
#' @param verbose is a logical. If TRUE, network will be plotted. Default is \code{verbose = TRUE}.
#' @return It is a list containing the following elements:
#' \tabular{lll}{
#' \code{graph} \tab  \tab a network object of the class \code{igraph}\cr
#' \code{cluster_obj} \tab  \tab a \code{communities} object of the package \code{igraph}\cr
#' \code{cluster_res} \tab  \tab a data frame with main results of clustering procedure.\cr}
#'
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' management <- metaTagExtraction(management, Field = "AU_CO")
#'
#' NetMatrix <- biblioNetwork(management, analysis = "collaboration", network = "countries")
#'
#' net <- collabByRegionPlot(NetMatrix,
#'   edgesize = 4, label.cex = TRUE, labelsize = 2.5,
#'   weighted = TRUE, size = 0.5, size.cex = TRUE, community.repulsion = 0,
#'   verbose = FALSE
#' )
#'
#' cbind(names(net))
#'
#' plot(net[[4]]$graph)
#' }
#'
#' @export
#'
collabByRegionPlot <- function(
    NetMatrix,
    normalize = NULL,
    n = NULL,
    degree = NULL,
    type = "auto",
    label = TRUE,
    labelsize = 1,
    label.cex = FALSE,
    label.color = FALSE,
    label.n = Inf,
    halo = FALSE,
    cluster = "walktrap",
    community.repulsion = 0,
    vos.path = NULL,
    size = 3,
    size.cex = FALSE,
    curved = FALSE,
    noloops = TRUE,
    remove.multiple = TRUE,
    remove.isolates = FALSE,
    weighted = NULL,
    edgesize = 1,
    edges.min = 0,
    alpha = 0.5,
    verbose = TRUE) {
  row.names(NetMatrix) <- colnames(NetMatrix) <- tolower(row.names(NetMatrix))
  labelCo <- data.frame(countries = row.names(NetMatrix))
  data("countries", envir = environment())
  countries <- countries %>% mutate(countries = tolower(countries))

  labelCo <- labelCo %>%
    left_join(countries, by = "countries")

  if (nrow(labelCo) < 2) {
    message("The argument NetMatrix is not a country collaboration network matrix")
    return(NA)
  }

  regions <- unique(labelCo$continent)
  n_regions <- length(regions)

  net <- list()

  for (i in regions) {
    reg_co <- labelCo %>%
      dplyr::filter(continent %in% i) %>%
      select(countries)
    NetMatrix_reg <- NetMatrix[reg_co$countries, reg_co$countries]

    if (!is.null(nrow(NetMatrix_reg))) {
      net[[i]] <- networkPlot(NetMatrix_reg,
        normalize = normalize, n = Inf, degree = degree, Title = i, type = type,
        label = label, labelsize = labelsize, label.cex = label.cex, label.color = label.color, label.n = label.n,
        halo = halo, cluster = cluster, community.repulsion = community.repulsion, vos.path = NULL, size = size,
        size.cex = size.cex, curved = curved, noloops = noloops, remove.multiple = remove.multiple, remove.isolates = remove.isolates,
        weighted = weighted, edgesize = edgesize, edges.min = edges.min, alpha = alpha, verbose = FALSE
      )
      net[[i]]$NetMatrix_reg <- NetMatrix_reg
    }
  }

  if (verbose) {
    l <- ceiling(length(net) / 2)
    par(mfrow = c(l, 2))
    # for (i in 1:length(net)){
    #   plot(net[[i]]$graph)
    # }
    lapply(net, function(x) {
      plot(x$graph)
    })
  }
  return(net)
}

utils::globalVariables(c(
  "group", "MNLCS", "name", "pagerank_centrality",
  "MNLCS2", "impact", "freq", "centrality", "color",
  "words", "rcentrality", "rimpact", "label", "Cluster",
  "w"
))
#' Coupling Analysis
#'
#' It performs a coupling network analysis and plots community detection results on a bi-dimensional map (Coupling Map).
#'
#' The analysis can be performed on three different units: documents, authors or sources and
#' the coupling strength can be measured using the classical approach (coupled by references)
#' or a novel approach based on unit contents (keywords or terms from titles and abstracts)
#'
#' The x-axis measures the cluster centrality (by Callon's Centrality index) while the y-axis measures the cluster impact
#' by Mean Normalized Local Citation Score (MNLCS).
#' The Normalized Local Citation Score (NLCS) of a document is calculated
#' by dividing the actual count of local citing items by the expected citation rate for documents with the same year of publication.
#'
#' @param M is a bibliographic dataframe.
#' @param analysis is the textual attribute used to select the unit of analysis. It can be \code{analysis = c("documents", "authors", "sources")}.
#' @param field is the textual attribute used to measure the coupling strength. It can be \code{field = c("CR", "ID","DE", "TI", "AB")}.
#' @param n is an integer. It indicates the number of units to include in the analysis.
#' @param label.term is a character. It indicates which content metadata have to use for cluster labeling. It can be \code{label.term = c("ID","DE","TI","AB")}.
#' If \code{label.term = NULL} cluster items will be use for labeling.
#' @param ngrams is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
#' An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.
#' @param impact.measure is a character. It indicates the impact measure used to rank cluster elements (documents, authors or sources).
#' It can be \code{impact.measure = c("local", "global")}.\\
#' With \code{impact.measure = "local"}, \link{couplingMap} calculates elements impact using the Normalized Local Citation Score while
#' using \code{impact.measure = "global"}, the function uses the Normalized Global Citation Score to measure elements impact.
#' @param minfreq is a integer. It indicates the minimum frequency (per thousand) of a cluster. It is a number in the range (0,1000).
#' @param community.repulsion is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.1}.
#' @param stemming is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).
#' @param size is numerical. It indicates the size of the cluster circles and is a number in the range (0.01,1).
#' @param n.labels is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.
#' @param repel is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.
#' @param cluster is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").
#' @return a list containing:
#' \tabular{lll}{
#' \code{map}\tab   \tab The coupling map as ggplot2 object\cr
#' \code{clusters}\tab   \tab Centrality and Density values for each cluster. \cr
#' \code{data}\tab   \tab A list of units following in each cluster\cr
#' \code{nclust}\tab   \tab The number of clusters\cr
#' \code{NCS}\tab     \tab The Normalized Citation Score dataframe\cr
#' \code{net}\tab    \tab A list containing the network output (as provided from the networkPlot function)}
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' res <- couplingMap(management,
#'   analysis = "authors", field = "CR", n = 250, impact.measure = "local",
#'   minfreq = 3, size = 0.5, repel = TRUE
#' )
#' plot(res$map)
#' }
#'
#' @seealso \code{\link{biblioNetwork}} function to compute a bibliographic network.
#' @seealso \code{\link{cocMatrix}} to compute a bibliographic bipartite network.
#' @seealso \code{\link{networkPlot}} to plot a bibliographic network.
#'
#' @export

couplingMap <- function(M, analysis = "documents", field = "CR", n = 500, label.term = NULL, ngrams = 1, impact.measure = "local", minfreq = 5,
                        community.repulsion = 0.1,
                        stemming = FALSE, size = 0.5, n.labels = 1, repel = TRUE, cluster = "walktrap") {
  if (!(analysis %in% c("documents", "authors", "sources"))) {
    cat('\nanalysis argument is incorrect.\n\nPlease select one of the following choices: "documents", "authors", "sources"\n\n')
    return(NA)
  }
  minfreq <- max(0, floor(minfreq * nrow(M) / 1000))


  Net <- network(M, analysis = analysis, field = field, stemming = stemming, n = n, cluster = cluster, community.repulsion = community.repulsion)

  net <- Net$graph

  NCS <- normalizeCitationScore(M, field = analysis, impact.measure = impact.measure)

  if (impact.measure == "global") {
    NCS$MNLCS <- NCS$MNGCS
    NCS$LC <- NCS$TC
  }

  NCS[, 1] <- toupper(NCS[, 1])

  ### Citation for documents
  label <- V(net)$name


  L <- tibble(id = toupper(label))
  names(L) <- analysis
  D <- left_join(L, NCS, by = analysis, copy = T)

  L <- tibble(id = tolower(label))
  names(L) <- names(Net$cluster_res)[1] <- analysis

  C <- left_join(L, Net$cluster_res, by = analysis, copy = T)


  group <- Net$cluster_obj$membership
  color <- V(net)$color
  color[is.na(color)] <- "#D3D3D3"

  D$group <- group
  D$color <- color

  DC <- cbind(D, C[, -1])
  DC$name <- DC[, 1]
  df_lab <- DC %>%
    group_by(group) %>%
    mutate(
      MNLCS2 = replace(MNLCS, MNLCS < 1, NA), ## remove NCS<1
      MNLCS = round(MNLCS, 2),
      name = tolower(name),
      freq = length(MNLCS)
    ) %>%
    arrange(desc(MNLCS), .by_group = TRUE)

  df <- df_lab %>%
    group_by(group) %>% 
    mutate(
      centrality = mean(pagerank_centrality),
      impact = mean(MNLCS2, na.rm = TRUE),
      impact = replace(impact, is.na(impact), 0)
    ) %>%
    slice_max(MNLCS, n = 10) %>%
    summarize(
      freq = freq[1],
      centrality = centrality[1] * 100,
      impact = impact[1],
      label_cluster = group[1],
      color = color[1],
      label = tolower(paste(name[1:min(n.labels, length(name))], collapse = "\n")),
      words = tolower(paste(name, MNLCS, collapse = "\n"))
    ) %>%
    mutate(
      rcentrality = rank(centrality),
      words = unlist(lapply(words, function(l) {
        l <- unlist(strsplit(l, "\\\n"))
        l <- l[1:(min(length(l), 10))]
        l <- paste0(l, collapse = "\n")
      })),
      rimpact = rank(impact)
    ) %>%
    arrange(group) %>%
    as.data.frame()

  row.names(df) <- df$label

  meandens <- mean(df$rimpact)
  meancentr <- mean(df$rcentrality)
  df <- df[df$freq >= minfreq, ]

  df_lab <- df_lab %>%
    dplyr::filter(group %in% df$group)

  rangex <- max(c(meancentr - min(df$rcentrality), max(df$rcentrality) - meancentr))
  rangey <- max(c(meandens - min(df$rimpact), max(df$rimpact) - meandens))
  xlimits <- c(meancentr - rangex - 0.5, meancentr + rangex + 0.5)
  ylimits <- c(meandens - rangey - 0.5, meandens + rangey + 0.5)


  df_lab <- df_lab[, c(1, 7, 15, 8, 4)]
  names(df_lab) <- c(analysis, "Cluster", "ClusterFrequency", "ClusterColor", "NormalizedLocalCitationScore")
  df_lab$ClusterName <- df$label[df_lab$Cluster]
  # quadrant_names=rep(" ",4) ## empty tooltips for quadrant names

  if (is.null(label.term)) {
    label.term <- "null"
  }
  if (label.term %in% c("DE", "ID", "TI", "AB")) {
    w <- labeling(M, df_lab, term = label.term, n = n, n.labels = n.labels, analysis = analysis, ngrams = ngrams)
    df$label <- w
  }

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)
  x <- c(max(df$rcentrality) - 0.02 - diff(range(df$rcentrality)) * 0.125, max(df$rcentrality) - 0.02) + 0.5
  y <- c(min(df$rimpact), min(df$rimpact) + diff(range(df$rimpact)) * 0.125)

  g <- ggplot(df, aes(x = rcentrality, y = rimpact, text = (words))) +
    geom_point(group = "NA", aes(size = log(as.numeric(freq))), shape = 20, col = adjustcolor(df$color, alpha.f = 0.5)) # Use hollow circles
  if (size > 0) {
    if (isTRUE(repel)) {
      g <- g + geom_label_repel(aes(group = "NA", label = ifelse(freq > 1, unlist(tolower(label)), "")), size = 3 * (1 + size), angle = 0)
    } else {
      g <- g + geom_text(aes(group = "NA", label = ifelse(freq > 1, unlist(tolower(label)), "")), size = 3 * (1 + size), angle = 0)
    }
  }

  g <- g + geom_hline(yintercept = meandens, linetype = 2, color = adjustcolor("black", alpha.f = 0.7)) +
    geom_vline(xintercept = meancentr, linetype = 2, color = adjustcolor("black", alpha.f = 0.7)) +
    theme(legend.position = "none") +
    scale_radius(range = c(10 * (1 + size), 30 * (1 + size))) +
    labs(x = "Centrality", y = "Impact") +
    xlim(xlimits) +
    ylim(ylimits) +
    ggtitle(paste("Clusters by ", toupper(substr(analysis, 1, 1)), substr(analysis, 2, nchar(analysis)), " Coupling", sep = "")) +
    theme(
      axis.text.x = element_blank(),
      panel.background = element_rect(fill = "#FFFFFF"),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5),
      axis.ticks.x = element_blank(),
      axis.text.y = element_blank(),
      axis.ticks.y = element_blank()
    ) +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])



  row.names(df) <- NULL
  df <- df %>% rename(items = words)

  params <- list(
    analysis = analysis,
    field = field,
    n = n,
    minfreq = minfreq,
    label.term = label.term,
    ngrams = ngrams,
    impact.measure = impact.measure,
    stemming = stemming,
    n.labels = n.labels,
    size = size,
    community.repulsion = community.repulsion,
    repel = repel,
    cluster = cluster
  )
  params <- data.frame(params = names(unlist(params)), values = unlist(params), row.names = NULL)

  results <- list(map = g, clusters = df, data = df_lab, nclust = dim(df)[1], NCS = D, net = Net, params = params)
  return(results)
}

coupling <- function(M, field, analysis) {
  if (field == "TI") {
    field <- "TI_TM"
  }
  if (field == "AB") field <- "AB_TM"
  switch(analysis,
    documents = {
      WF <- t(cocMatrix(M, Field = field, short = TRUE))
      NetMatrix <- crossprod(WF, WF)
    },
    authors = {
      WF <- cocMatrix(M, Field = field, short = TRUE)
      WA <- cocMatrix(M, Field = "AU", short = TRUE)
      FA <- t(crossprod(WA, WF))
      NetMatrix <- crossprod(FA, FA)
    },
    sources = {
      WF <- cocMatrix(M, Field = field, short = TRUE)
      WS <- cocMatrix(M, Field = "SO", short = TRUE)
      FS <- t(crossprod(WS, WF))
      NetMatrix <- crossprod(FS, FS)
    }
  )
  return(NetMatrix)
}

network <- function(M, analysis, field, stemming, n, cluster, community.repulsion) {
  switch(analysis,
    documents = {
      switch(field,
        CR = {
          NetMatrix <- biblioNetwork(M, analysis = "coupling", network = "references", short = TRUE, shortlabel = FALSE, sep = ";")
          type <- "D_CR"
        },
        {
          # field ID, DE, TI, AB
          if (field %in% c("TI", "AB")) {
            M <- termExtraction(M, Field = field, verbose = FALSE, stemming = stemming)
            type <- "D_KW"
          }
          NetMatrix <- coupling(M, field, analysis = "documents")
        }
      )
    },
    authors = {
      switch(field,
        CR = {
          NetMatrix <- biblioNetwork(M, analysis = "coupling", network = "authors", short = TRUE)
          type <- "AU_CR"
        },
        {
          # field ID, DE, TI, AB
          if (field %in% c("TI", "AB")) {
            M <- termExtraction(M, Field = field, verbose = FALSE, stemming = stemming)
            type <- "AU_KW"
          }
          NetMatrix <- coupling(M, field, analysis = "authors")
        }
      )
    },
    sources = {
      switch(field,
        CR = {
          NetMatrix <- biblioNetwork(M, analysis = "coupling", network = "sources", short = TRUE)
          type <- "SO_CR"
        },
        {
          # field ID, DE, TI, AB
          if (field %in% c("TI", "AB")) {
            M <- termExtraction(M, Field = field, verbose = FALSE, stemming = stemming)
            type <- "SO_KW"
          }
          NetMatrix <- coupling(M, field, analysis = "sources")
        }
      )
    }
  )

  # delete empty vertices
  NetMatrix <- NetMatrix[nchar(colnames(NetMatrix)) != 0, nchar(colnames(NetMatrix)) != 0]

  if (nrow(NetMatrix) > 0) {
    Net <- networkPlot(NetMatrix,
      normalize = "salton", n = n, Title = paste("Coupling network of ", analysis, " using ", field, sep = ""), type = "auto",
      labelsize = 2, halo = F, cluster = cluster, remove.isolates = TRUE, community.repulsion = community.repulsion,
      remove.multiple = FALSE, noloops = TRUE, weighted = TRUE, label.cex = T, edgesize = 5,
      size = 1, edges.min = 1, label.n = n, verbose = FALSE
    )
  } else {
    cat("\n\nNetwork matrix is empty!\nThe analysis cannot be performed\n\n")
    return()
  }

  return(Net)
}

## cluster labeling
labeling <- function(M, df_lab, term, n, n.labels, analysis, ngrams) {
  if (term %in% c("TI", "AB")) {
    M <- termExtraction(M, Field = term, ngrams = ngrams, verbose = FALSE)
    term <- paste(term, "_TM", sep = "")
  }
  switch(analysis,
    documents = {
      df <- left_join(df_lab, M, by = c("documents" = "SR"))
    },
    authors = {
      WF <- cocMatrix(M, Field = term, short = TRUE)
      WA <- cocMatrix(M, Field = "AU", n = n, short = TRUE)
      AF <- (crossprod(WA, WF))
      A <- apply(AF, 1, function(x) {
        paste(rep(names(x)[x > 0], x[x > 0]), collapse = ";")
      })
      A <- data.frame(AU = names(A), words = A)
      names(A)[2] <- term
      df <- left_join(df_lab, A, by = c("authors" = "AU"))
    },
    sources = {
      df <- left_join(df_lab, M, by = c("sources" = "SO"))
    }
  )

  # clusters <- unique(df$Cluster)
  # w <- character(length(clusters))
  df$SR <- df[, 1]
  tab_global <- tableTag(df, term)
  tab_global <- data.frame(label = names(tab_global), tot = as.numeric(tab_global), n = nrow(M))

  df <- df %>%
    group_by(Cluster) %>%
    reframe(best_lab(pick(everything()), tab_global, n.labels, term)) %>%
    as.data.frame()

  return(df$w)
}

best_lab <- function(d, tab_global, n.labels, term) {
  tab <- tableTag(d, term)
  tab <- tab[!is.na(names(tab)) & names(tab) != ""]
  
  if (length(tab) == 0) return(tibble(w = "no_label"))
  
  tab1 <- data.frame(label = names(tab), value = as.numeric(tab), stringsAsFactors = FALSE)
  
  tab1 <- tab1 %>%
    left_join(tab_global, by = "label") %>%
    mutate(
      conf = round(.data$value / .data$tot * 100, 1),
      supp = round(.data$tot / n * 100, 1),
      relevance = round(.data$conf * .data$supp / 100, 1)
    ) %>%
    arrange(desc(.data$relevance)) %>%
    slice(1:n.labels)
  
  tibble(w = tolower(paste(tab1$label, " - Conf ", tab1$conf, "%", sep = "", collapse = "\n")))
}



# best_lab <- function(d, tab_global, n.labels, term) {
#   tab <- tableTag(d, term)
#   tab1 <- data.frame(label = names(tab), value = as.numeric(tab), stringsAsFactors = FALSE)
#   print(names(tab1))
#   tab1 <- tab1 %>%
#     left_join(tab_global, by = "label") %>%
#     mutate(
#       conf = round(.data$value / .data$tot * 100, 1),
#       supp = round(.data$tot / n * 100, 1),
#       relevance = round(.data$conf * .data$supp / 100, 1)
#     ) %>%
#     arrange(desc(.data$relevance)) %>%
#     slice(1:n.labels)
# 
#   # tolower(paste(tab$label," Supp ",tab$supp,"% - Conf ", tab$conf,"%", sep="", collapse="\n"))
#   tolower(paste(tab1$label, " - Conf ", tab1$conf, "%", sep = "", collapse = "\n"))
# }

utils::globalVariables(c(
  "Year",
  "diffMedian",
  "Citations",
  "citedYears",
  "Reference",
  "citingYears",
  "benchmark",
  "status",
  "citations",
  "Freq",
  "diffMedian5",
  "CP",
  "CR",
  "Class",
  "HP",
  "LC",
  "RPY",
  "SB",
  "Z",
  "dev_median_t2_t2",
  "dev_median_t4_t",
  "diffMedian2",
  "freqO",
  "freqYC",
  "is_peak",
  "lead",
  "observed",
  "z_class",
  "everything"
))
#' Reference Publication Year Spectroscopy
#'
#' \code{rpys} computes a Reference Publication Year Spectroscopy for detecting
#' the Historical Roots of Research Fields.
#' The method was introduced by Marx et al., 2014.\cr\cr
#'
#' References:\cr\cr
#' Marx, W., Bornmann, L., Barth, A., & Leydesdorff, L. (2014).
#' Detecting the historical roots of research fields by reference publication
#' year spectroscopy (RPYS). Journal of the Association for Information Science and Technology,
#' 65(4), 751-764.\cr\cr
#' Thor A., Bornmann L., Mark W. & Mutz R.(2018).
#' Identifying single influential publications in a research field: new analysis opportunities of the CRExplorer.
#' Scientometrics, 116:591–608 https://doi.org/10.1007/s11192-018-2733-7\cr\cr
#'
#' @param M is a data frame obtained by the converting function
#'   \code{\link{convert2df}}. It is a data matrix with cases corresponding to
#'   articles and variables to Field Tag in the original ISI or SCOPUS file.
#' @param sep is the cited-references separator character. This character separates cited-references in the CR
#' column of the data frame. The default is \code{sep = ";"}.
#' @param timespan is a numeric vector c(min year,max year). The default value is NULL (the entire timespan is considered).
#' @param median.window is a character string that can be "centered" or "backward". It indicates the type of median to be used.
#' "centered" is the default value and it uses the centered 5-year median (t-2 to t+2) as proposed by Marx et al. (2014). "backward" uses the backward 5-year median (t-4 to t) as proposed by Aria and Cuccurullo (2017).
#' @param graph is a logical. If TRUE the function plot the spectroscopy otherwise the plot is created but not drawn down.
#' @return a list containing the spectroscopy (class ggplot2) and three dataframes with the number of citations
#' per year, the list of the cited references for each year, and the reference list with citations recorded year by year, respectively.
#'
#'
#' @examples
#'
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' res <- rpys(management, sep = ";", graph = TRUE)
#' }
#'
#' @seealso \code{\link{convert2df}} to import and convert an ISI or SCOPUS
#'   Export file in a data frame.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @export

rpys <- function(
  M,
  sep = ";",
  timespan = NULL,
  median.window = "centered",
  graph = T
) {
  options(dplyr.summarise.inform = FALSE)

  if (is.null(timespan)) {
    timespan <- c(max(M$PY, na.rm = TRUE) - 100, max(M$PY, na.rm = TRUE) - 3)
  }

  M$CR <- gsub("DOI;", "DOI ", as.character(M$CR))

  # Vectorized: unlist first, then trim and filter on flat vector
  Fi <- strsplit(M[, "CR"], sep)
  lens <- lengths(Fi)
  Fi <- unlist(Fi)
  Fi <- sub("^\\s+", "", Fi)
  citingYears <- rep(M$PY, lens)
  keep <- !is.na(Fi) & nchar(Fi) > 10
  Fi <- Fi[keep]
  citingYears <- citingYears[keep]

  df <- data.frame(Reference = Fi, citingYears = citingYears) %>%
    mutate(Reference = refCleaning(Reference, db = M$DB[1]))
  df$citedYears <- as.numeric(yearExtract(df$Reference, db = M$DB[1]))

  df <- df %>%
    dplyr::filter(
      !is.na(Reference) &
        citedYears > 1700 &
        citedYears <= as.numeric(substr(Sys.Date(), 1, 4))
    ) %>%
    group_by(citedYears, citingYears, Reference) %>%
    summarize(citations = n()) %>%
    ungroup() %>%
    arrange(citedYears, Reference, citingYears)

  CR <- df %>%
    group_by(citedYears, Reference) %>%
    select(-citingYears) %>%
    summarize(Freq = sum(citations))

  RPYS <- CR %>%
    select(-Reference) %>%
    group_by(citedYears) %>%
    summarize(n = sum(Freq, na.rm = TRUE)) %>%
    arrange(citedYears)

  # Create a complete sequence of years and fill missing frequencies with 0
  full_years <- tibble(
    citedYears = seq(min(RPYS$citedYears), max(RPYS$citedYears))
  )
  RPYS_full <- full_years %>%
    left_join(RPYS, by = "citedYears") %>%
    mutate(n = if_else(is.na(n), 0L, n))

  # Extract vectors for computation
  years <- RPYS_full$citedYears
  counts <- RPYS_full$n

  # Vectorized rolling medians using stats::runmed
  n_years <- length(counts)

  # Backward-looking 5-year median (t-4 to t)
  # runmed requires odd k and uses "median of available" at edges
  if (n_years >= 5) {
    # Pad front with first value to simulate backward window, then take centered median
    padded <- c(rep(counts[1], 4), counts)
    median_t4_t <- vapply(seq_along(counts), function(i) {
      median(padded[i:(i + 4)])
    }, numeric(1))
  } else {
    median_t4_t <- sapply(seq_along(years), function(i) {
      start <- pmax(1, i - 4)
      median(counts[start:i])
    })
  }

  # Centered 5-year median (t-2 to t+2) using runmed with endrule
  if (n_years >= 5) {
    median_t2_t2 <- as.numeric(stats::runmed(counts, k = 5, endrule = "median"))
  } else {
    median_t2_t2 <- sapply(seq_along(years), function(i) {
      start <- pmax(1, i - 2)
      end <- pmin(length(counts), i + 2)
      median(counts[start:end])
    })
  }

  # Add the results to the data frame
  RPYS <- RPYS_full %>%
    mutate(
      dev_median_t4_t = n - median_t4_t,
      dev_median_t2_t2 = n - median_t2_t2
    ) %>%
    select(citedYears, n, dev_median_t4_t, dev_median_t2_t2)

  if (length(timespan) == 2) {
    RPYS <- RPYS %>%
      dplyr::filter(
        citedYears >= min(timespan) &
          citedYears <= max(timespan)
      )
  }
  names(RPYS) <- c("Year", "Citations", "diffMedian5", "diffMedian2")

  # RPYS$diffMedian <- RPYS$diffMedian5

  # set a limit to negative numbers
  minLimit <- round(max(RPYS$Citations) * (-0.05))

  # choose the median windows
  if (median.window == "centered") {
    RPYS <- RPYS %>%
      mutate(diffMedian = diffMedian2)
  } else if (median.window == "backward") {
    RPYS <- RPYS %>%
      mutate(diffMedian = diffMedian5)
  }

  RPYS <- RPYS %>%
    mutate(diffMedian = if_else(diffMedian > minLimit, diffMedian, minLimit))

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  x <- c(min(RPYS$Year), min(RPYS$Year) + diff(range(RPYS$Year)) * 0.125) + 1
  y <- c(
    max(c(RPYS$Citations, RPYS$diffMedian)) -
      diff(range(c(RPYS$Citations, RPYS$diffMedian))) * 0.125,
    max(c(RPYS$Citations, RPYS$diffMedian))
  )

  RPYS <- RPYS %>%
    left_join(
      CR %>%
        group_by(citedYears) %>%
        slice_max(order_by = Freq, n = 3, with_ties = FALSE) %>%
        summarize(References = paste(firstup(Reference), collapse = "\n")),
      by = c("Year" = "citedYears")
    )

  ## Identify top 10 peaks on the deviation line ----
  peaks <- RPYS %>%
    arrange(Year) %>%
    mutate(
      is_peak = (diffMedian > dplyr::lag(diffMedian)) &
        (diffMedian > dplyr::lead(diffMedian))
    ) %>%
    dplyr::filter(is_peak) %>%
    arrange(desc(diffMedian)) %>%
    slice_head(n = 10)

  g <- ggplot(
    RPYS,
    aes(
      x = Year,
      y = Citations,
      text = paste(
        "Year: ",
        Year,
        " - Total Citations: ",
        Citations,
        "\nTop 3 References:\n",
        References
      )
    )
  ) +
    geom_line(aes(group = "NA")) +
    geom_line(aes(
      x = Year,
      y = diffMedian,
      color = "firebrick",
      group = "NA"
    )) +
    # Top 10 peaks
    geom_point(
      data = peaks,
      aes(x = Year, y = diffMedian),
      color = "#D6604D",
      size = 3,
      inherit.aes = FALSE
    ) +
    ggrepel::geom_text_repel(
      data = peaks,
      aes(x = Year, y = diffMedian, label = Year),
      color = "#D6604D",
      size = 3.5,
      fontface = "bold",
      direction = "y",
      nudge_y = max(RPYS$Citations) * 0.05,
      segment.color = "#D6604D",
      segment.size = 0.3,
      segment.alpha = 0.5,
      min.segment.length = 0,
      inherit.aes = FALSE
    ) +
    labs(
      x = "Year",
      y = "Cited References",
      title = "Reference Publication Year Spectroscopy",
      caption = "Number of Cited References (black line) - Deviation from the 5-Year Median (red line) - Top 10 peaks labeled"
    ) +
    scale_x_continuous(
      breaks = (RPYS$Year[seq(
        1,
        length(RPYS$Year),
        # max(1, .) guards against by = 0 when fewer than ~15 years are present
        by = max(1, round(length(RPYS$Year) / 30))
      )])
    ) +
    theme(
      text = element_text(color = "#444444"),
      legend.position = "none",
      plot.caption = element_text(
        size = 9,
        hjust = 0.5,
        color = "black",
        face = "bold"
      ),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0.95, angle = 0),
      axis.text.x = element_text(size = 8, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])

  if (isTRUE(graph)) {
    plot(g)
  }
  CR$Reference <- reduceRefs(CR$Reference)
  CR <- CR %>%
    rename(Year = citedYears) %>%
    ungroup()

  ## identify types of sequences
  Sequences <- sequenceTypes(M, timespan = timespan)

  RPYS <- RPYS %>% select(-References)
  CR <- CR %>% mutate(Year = as.character(Year))

  result <- list(
    spectroscopy = g,
    rpysTable = RPYS,
    CR = CR,
    df = df,
    Sequences = Sequences %>% arrange(RPY, desc(Freq), CR)
  )
  return(result)
}

yearExtract <- function(string, db) {
  string <- paste(" ", string, " ", sep = "")
  # WoS-format CR carries the year as a bare 4-digit token ("AUTHOR, YEAR, ...").
  # OpenAlex/PubMed/Dimensions CR is rewritten to this layout upstream, so they
  # use the same extraction; only SCOPUS keeps the parenthesised "(YEAR)" form.
  if (toupper(db) %in% c("ISI", "WOS", "OPENALEX", "PUBMED", "DIMENSIONS")) {
    ind <- regexpr(" [[:digit:]]{4} ", string)
    ind[is.na(ind)] <- -1
    string[ind == -1] <- " 0000 "
    ind[ind == -1] <- 1
    attr(ind[ind == -1], "match.length") <- 6
    y <- trim(unlist(regmatches(string, ind)))
  } else {
    ind <- regexpr("\\([[:digit:]]{4}\\)", string)
    ind[is.na(ind)] <- -1
    string[ind == -1] <- "(0000)"
    ind[ind == -1] <- 1
    attr(ind[ind == -1], "match.length") <- 6
    y <- unlist(regmatches(string, ind))
    y <- substr(y, 2, 5)
  }
  return(y)
}

reduceRefs <- function(A) {
  ind <- unlist(regexec("*V[0-9]", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  ind <- unlist(regexec("*DOI ", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  return(A)
}

refCleaning <- function(l, db) {
  if (db == "ISI") {
    l <- gsub("\\).*", ")", l)
  }
  l <- gsub("[,;.]", " ", l)
  l <- trimws(trimES(l))
  l <- l[nchar(l) > 0]
  return(l)
}

sequenceTypes <- function(M, timespan = NULL) {
  ## calculate Attribute matrices for CR and PY
  ACR <- cocMatrix(M, Field = "CR", sep = ";")
  APY <- cocMatrix(M, Field = "PY", sep = ";")
  APY <- APY[, order(colnames(APY))]

  B <- crossprod(ACR, APY)
  Y <- yearExtract(row.names(B), db = M$DB[1])

  rowS <- Matrix::rowSums(B)
  indR <- which(rowS > 0)
  B <- B[indR, , drop = FALSE]
  Y <- Y[indR]
  rowS <- rowS[indR]

  cr_names <- rownames(B)
  py_years <- as.numeric(colnames(B))
  rpy_vals <- suppressWarnings(as.numeric(Y))

  # Pre-filter by timespan on RPY
  if (!is.null(timespan)) {
    keep <- !is.na(rpy_vals) & Y != "" & rpy_vals >= timespan[1]
  } else {
    keep <- !is.na(rpy_vals) & Y != ""
  }

  if (sum(keep) == 0) {
    return(data.frame(
      CR = character(), RPY = character(), Freq = numeric(),
      sequence = character(), Class = character(),
      stringsAsFactors = FALSE
    ))
  }

  B <- B[keep, , drop = FALSE]
  Y <- Y[keep]
  rpy_vals <- rpy_vals[keep]
  cr_names <- cr_names[keep]
  rowS <- rowS[keep]

  # Densify once (instead of 3x pivot_longer + 2x left_join)
  obs_mat <- as.matrix(B)
  z_mat <- obs_mat

  # Compute Z-scores per RPY group
  for (y in sort(unique(Y))) {
    ind <- which(Y == y)
    if (length(ind) > 1) {
      grp <- obs_mat[ind, , drop = FALSE]
      rs <- rowSums(grp)
      cs <- colSums(grp)
      total <- sum(rs)
      if (total > 0) {
        exp_mat <- outer(rs, cs) / total
        z_mat[ind, ] <- (grp - exp_mat) / sqrt(exp_mat)
      }
    }
  }

  # Mask: only PY >= RPY for each CR
  mask <- outer(rpy_vals, py_years, "<=")
  z_mat[!mask] <- NA
  obs_mat[!mask] <- NA

  # Handle NaN from 0/0 divisions
  z_mat[is.nan(z_mat)] <- NA

  # Filter rows with no valid entries
  n_valid <- rowSums(!is.na(z_mat))
  has_data <- n_valid > 0
  z_mat <- z_mat[has_data, , drop = FALSE]
  obs_mat <- obs_mat[has_data, , drop = FALSE]
  n_valid <- n_valid[has_data]
  cr_names <- cr_names[has_data]
  Y <- Y[has_data]
  rowS <- rowS[has_data]

  # Compute freqYC and freqO directly from matrices (no pivot needed)
  freqYC <- rowSums(obs_mat > 0, na.rm = TRUE) / n_valid
  freqO <- rowSums(z_mat > -1, na.rm = TRUE) / n_valid

  # Build sequence strings (vectorized, no row-by-row apply)
  z_class_mat <- matrix(NA_character_, nrow = nrow(z_mat), ncol = ncol(z_mat))
  valid <- !is.na(z_mat)
  z_class_mat[valid & z_mat >= 1] <- "+"
  z_class_mat[valid & z_mat <= -1] <- "-"
  z_class_mat[valid & z_mat > -1 & z_mat < 1] <- "o"
  z_class_mat[is.na(z_class_mat)] <- ""
  sequences <- do.call(paste0, as.data.frame(z_class_mat, stringsAsFactors = FALSE))

  # Build result dataframe
  df <- data.frame(
    CR = cr_names, RPY = Y, Freq = rowS,
    freqYC = freqYC, freqO = freqO,
    sequence = sequences, stringsAsFactors = FALSE
  )

  # Classify sequences (based on Table 1 of Thor et al., 2018)
  # Helper: count occurrences of a character in a string
  countChar <- function(s, ch) nchar(s) - nchar(gsub(ch, "", s, fixed = TRUE))

  first3 <- substr(df$sequence, 1, 3)
  first4 <- substr(df$sequence, 1, 4)
  rest_after3 <- ifelse(
    nchar(df$sequence) > 3,
    substr(df$sequence, 4, nchar(df$sequence)),
    ""
  )
  rest_after4 <- ifelse(
    nchar(df$sequence) > 4,
    substr(df$sequence, 5, nchar(df$sequence)),
    ""
  )
  last3 <- ifelse(
    nchar(df$sequence) >= 3,
    substr(df$sequence, nchar(df$sequence) - 2, nchar(df$sequence)),
    df$sequence
  )

  # SB (Sleeping Beauty): below average ("-") in 2 of the first 3 citing years,
  # and above average ("+") in the following years at least once
  df$SB <- countChar(first3, "-") >= 2 &
    countChar(rest_after3, "+") >= 1

  # CP (Constant Performer): cited in >80% of years at least once,
  # and in >80% of years at average level or above (z > -1)
  df$CP <- df$freqO >= 0.8 & df$freqYC >= 0.8 & nchar(df$sequence) > 2

  # HP (Hot Paper): above average ("+") in 2 of the first 3 citing years
  df$HP <- countChar(first3, "+") >= 2

  # LC (Life Cycle): at least 2 of first 4 years on average or lower (not "+"),
  # at least 2 of the following years above average ("+"),
  # and last 3 years on average or lower (no "+")
  df$LC <- (nchar(df$sequence) > 6) &
    (nchar(first4) - countChar(first4, "+")) >= 2 &
    countChar(rest_after4, "+") >= 2 &
    countChar(last3, "+") == 0

  df <- df %>%
    mutate(
      SB = ifelse(SB, "Sleeping Beauty", NA),
      CP = ifelse(CP, "Constant Performer", NA),
      HP = ifelse(HP, "Hot Paper", NA),
      LC = ifelse(LC, "Life Cycle", NA)
    ) %>%
    unite("Class", c("SB", "CP", "HP", "LC"), sep = "+", na.rm = T) %>%
    select(CR, RPY, Freq, sequence, Class)
}

firstup <- function(x) {
  x <- tolower(x)
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

utils::globalVariables(c(
  "clust", "Dim1", "Dim2", "label", "id", "shape",
  "color", "contrib", "dim1", "dim2", "nomi", "TC"
))
#' Creating and plotting conceptual structure map of a scientific field
#'
#' The function \code{conceptualStructure} creates a conceptual structure map of
#' a scientific field performing Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA) or Metric Multidimensional Scaling (MDS) and Clustering
#' of a bipartite network of terms extracted from keyword, title or abstract fields.
#'
#' @param M is a data frame obtained by the converting function
#'   \code{\link{convert2df}}. It is a data matrix with cases corresponding to
#'   articles and variables to Field Tag in the original ISI or SCOPUS file.
#' @param field is a character object. It indicates one of the field tags of the
#'   standard ISI WoS Field Tag codify.
#'   field can be equal to one of these tags:
#'   \tabular{lll}{
#'   \code{ID}\tab   \tab Keywords Plus associated by ISI or SCOPUS database\cr
#'   \code{DE}\tab   \tab Author's keywords\cr
#'   \code{KW_Merged}\tab   \tab All keywords\cr
#'   \code{ID_TM}\tab   \tab Keywords Plus stemmed through the Porter's stemming algorithm\cr
#'   \code{DE_TM}\tab   \tab Author's Keywords stemmed through the Porter's stemming algorithm\cr
#'   \code{TI}\tab   \tab Terms extracted from titles\cr
#'   \code{AB}\tab   \tab Terms extracted from abstracts}
#' @param ngrams is an integer between 1 and 3. It indicates the type of n-gram to extract from texts.
#' An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.
#' @param method is a character object. It indicates the factorial method used to create the factorial map. Use \code{method="CA"} for Correspondence Analysis,
#'  \code{method="MCA"} for Multiple Correspondence Analysis or \code{method="MDS"} for Metric Multidimensional Scaling. The default is \code{method="MCA"}
#' @param minDegree is an integer. It indicates the minimum occurrences of terms to analyze and plot. The default value is 2.
#' @param clust is an integer or a character. If clust="auto", the number of cluster is chosen automatically, otherwise clust can be an integer between 2 and 8.
#' @param k.max is an integer. It indicates the maximum number of cluster to keep. The default value is 5. The max value is 20.
#' @param stemming is logical. If TRUE the Porter's Stemming algorithm is applied to all extracted terms. The default is \code{stemming = FALSE}.
#' @param labelsize is an integer. It indicates the label size in the plot. Default is \code{labelsize=10}
#' @param quali.supp is a vector indicating the indexes of the categorical supplementary variables. It is used only for CA and MCA.
#' @param quanti.supp is a vector indicating the indexes of the quantitative supplementary variables. It is used only for CA and MCA.
#' @param documents is an integer. It indicates the number of documents per cluster to plot in the factorial map. The default value is 2. It is used only for CA and MCA.
#' @param graph is logical. If TRUE the function plots the maps otherwise they are saved in the output object. Default value is TRUE
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @return It is an object of the class \code{list} containing the following components:
#'
#' \tabular{lll}{
#' net \tab  \tab bipartite network\cr
#' res \tab       \tab Results of CA, MCA or MDS method\cr
#' km.res \tab      \tab Results of cluster analysis\cr
#' graph_terms \tab      \tab Conceptual structure map (class "ggplot2")\cr
#' graph_documents_Contrib \tab      \tab Factorial map of the documents with the highest contributes (class "ggplot2")\cr
#' graph_docuemnts_TC \tab      \tab Factorial map of the most cited documents (class "ggplot2")}
#'
#' @examples
#' # EXAMPLE Conceptual Structure using Keywords Plus
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' CS <- conceptualStructure(scientometrics,
#'   field = "ID", method = "CA",
#'   stemming = FALSE, minDegree = 3, k.max = 5
#' )
#'
#' @seealso \code{\link{termExtraction}} to extract terms from a textual field (abstract, title,
#' author's keywords, etc.) of a bibliographic data frame.
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export
conceptualStructure <- function(M, field = "ID", ngrams = 1, method = "MCA", quali.supp = NULL, quanti.supp = NULL, minDegree = 2,
                                clust = "auto", k.max = 5, stemming = FALSE, labelsize = 10, documents = 2, graph = TRUE,
                                remove.terms = NULL, synonyms = NULL) {
  # cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
  cbPalette <- colorlist() # c(brewer.pal(9, 'Set1')[-6], brewer.pal(8, 'Set2')[-7], brewer.pal(12, 'Paired')[-11],brewer.pal(12, 'Set3')[-c(2,8,12)])

  # if (!is.null(quali.supp)){
  #   QSUPP=data.frame(M[,quali.supp])
  #   names(QSUPP)=names(M)[quali.supp]
  #   row.names(QSUPP)=tolower(row.names(M))
  # }
  #
  # if (!is.null(quanti.supp)){
  #   SUPP=data.frame(M[,quanti.supp])
  #   names(SUPP)=names(M)[quanti.supp]
  #   row.names(SUPP)=tolower(row.names(M))
  # }
  binary <- FALSE
  if (method == "MCA") {
    binary <- TRUE
  }

  switch(field,
    ID = {
      # Create a bipartite network of Keyword plus
      #
      # each row represents a manuscript
      # each column represents a keyword (1 if present, 0 if absent in a document)
      CW <- cocMatrix(M, Field = "ID", type = "matrix", sep = ";", binary = binary, remove.terms = remove.terms, synonyms = synonyms)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[, !(colnames(CW) %in% "NA")]
      CW <- CW[rowSums(CW) > 0, ]
    },
    DE = {
      CW <- cocMatrix(M, Field = "DE", type = "matrix", sep = ";", binary = binary, remove.terms = remove.terms, synonyms = synonyms)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[rowSums(CW) > 0, ]
      CW <- CW[, !(colnames(CW) %in% "NA")]
    },
    KW_Merged = {
      CW <- cocMatrix(M, Field = "KW_Merged", type = "matrix", sep = ";", binary = binary, remove.terms = remove.terms, synonyms = synonyms)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[rowSums(CW) > 0, ]
      CW <- CW[, !(colnames(CW) %in% "NA")]
    },
    ID_TM = {
      M <- termExtraction(M, Field = "ID", remove.numbers = TRUE, stemming = stemming, language = "english", remove.terms = remove.terms, synonyms = synonyms, keep.terms = NULL, verbose = FALSE)

      CW <- cocMatrix(M, Field = "ID_TM", type = "matrix", sep = ";", binary = binary)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      CW <- CW[, !(colnames(CW) %in% "NA")]
      # Delete empty rows
      CW <- CW[rowSums(CW) > 0, ]
    },
    DE_TM = {
      M <- termExtraction(M, Field = "DE", remove.numbers = TRUE, stemming = stemming, language = "english", remove.terms = remove.terms, synonyms = synonyms, keep.terms = NULL, verbose = FALSE)

      CW <- cocMatrix(M, Field = "DE_TM", type = "matrix", sep = ";", binary = binary)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[, !(colnames(CW) %in% "NA")]
      CW <- CW[rowSums(CW) > 0, ]
    },
    TI = {
      M <- termExtraction(M, Field = "TI", remove.numbers = TRUE, stemming = stemming, language = "english", remove.terms = remove.terms, synonyms = synonyms, keep.terms = NULL, verbose = FALSE, ngrams = ngrams)

      CW <- cocMatrix(M, Field = "TI_TM", type = "matrix", sep = ";", binary = binary)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[, !(colnames(CW) %in% "NA")]
      CW <- CW[rowSums(CW) > 0, ]
    },
    AB = {
      M <- termExtraction(M, Field = "AB", remove.numbers = TRUE, stemming = stemming, language = "english", remove.terms = remove.terms, synonyms = synonyms, keep.terms = NULL, verbose = FALSE, ngrams = ngrams)

      CW <- cocMatrix(M, Field = "AB_TM", type = "matrix", sep = ";", binary = binary)
      # Define minimum degree (number of occurrences of each Keyword)
      CW <- CW[, colSums(CW) >= minDegree]
      # Delete empty rows
      CW <- CW[rowSums(CW) > 0, ]
      CW <- CW[, !(colnames(CW) %in% "NA")]
      # Recode as dataframe
      # CW=data.frame(apply(CW,2,factor))
    }
  )

  colnames(CW) <- label <- tolower(colnames(CW))
  rownames(CW) <- tolower(rownames(CW))
  p <- dim(CW)[2]
  quali <- NULL
  quanti <- NULL

  results <- factorial(CW, method = method, quanti = quanti, quali = quali)
  res.mca <- results$res.mca
  df <- results$df
  row.names(df) <- label
  # row.names(df) <- gsub("\\."," ",row.names(df))
  docCoord <- results$docCoord
  df_quali <- results$df_quali
  df_quanti <- results$df_quanti

  ### Total Citations of documents
  if ("TC" %in% names(M) & method != "MDS") {
    docCoord$TC <- as.numeric(M[toupper(rownames(docCoord)), "TC"])
  }


  # Selection of optimal number of clusters (gap statistics)
  # a=fviz_nbclust((df), kmeans, method = "gap_stat",k.max=k.max)['data']$data$y
  km.res <- hclust(dist(df), method = "average")

  if (clust == "auto") {
    clust <- min((length(km.res$height) - which.max(diff(km.res$height)) + 1), k.max)
  } else {
    clust <- max(1, min(as.numeric(clust), k.max))
  }

  km.res$data <- df
  km.res$cluster <- cutree(km.res, k = clust)
  km.res$data.clust <- data.frame(km.res$data, clust = km.res$cluster)
  centers <- km.res$data.clust %>%
    group_by(clust) %>%
    summarise("Dim1" = mean(Dim1), "Dim2" = mean(Dim2)) %>%
    as.data.frame()

  km.res$centers <- centers[, c(2, 3, 1)]

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  df_clust <- km.res$data.clust %>%
    mutate(
      shape = "1",
      label = row.names(.)
    ) %>%
    bind_rows(km.res$centers %>% mutate(shape = "0", label = "")) %>%
    mutate(color = colorlist()[clust])

  hull_data <-
    df_clust %>%
    group_by(clust) %>%
    slice(chull(Dim1, Dim2))

  hull_data <- hull_data %>%
    bind_rows(
      hull_data %>% group_by(clust) %>% slice_head(n = 1)
    ) %>%
    mutate(id = row_number()) %>%
    arrange(clust, id)

  size <- labelsize

  b <- ggplot(df_clust, aes(x = Dim1, y = Dim2, shape = shape, color = color)) +
    geom_point() +
    geom_polygon(
      data = hull_data,
      aes(
        fill = color,
        colour = color
      ),
      alpha = 0.3,
      show.legend = FALSE
    ) +
    ggrepel::geom_text_repel(aes(label = label)) +
    theme_minimal() +
    labs(title = paste("Conceptual Structure Map - method: ", method, collapse = "", sep = "")) +
    geom_hline(yintercept = 0, linetype = "dashed", color = adjustcolor("grey40", alpha.f = 0.7)) +
    geom_vline(xintercept = 0, linetype = "dashed", color = adjustcolor("grey40", alpha.f = 0.7)) +
    theme(
      text = element_text(size = size),
      axis.title = element_text(size = size, face = "bold"),
      plot.title = element_text(size = size + 1, face = "bold"),
      panel.background = element_rect(fill = "white", colour = "white"),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5),
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank()
    )
  if (method != "MDS") {
    b <- b + xlab(paste("Dim 1 (", round(res.mca$eigCorr$perc[1], 2), "%)", sep = "")) +
      ylab(paste("Dim 2 (", round(res.mca$eigCorr$perc[2], 2), "%)", sep = ""))
  } else {
    b <- b + xlab("Dim 1") + ylab("Dim 2")
  }

  b <- b + theme(legend.position = "none")

  ## logo coordinates
  coord_b <- plotCoord(b)

  b <- b + annotation_custom(logo, xmin = coord_b[1], xmax = coord_b[2], ymin = coord_b[3], ymax = coord_b[4])

  if (isTRUE(graph)) {
    plot(b)
  }

  b_dend <- dendPlot(km.res, clust = clust, label.cex = labelsize * 0.07, graph = FALSE)

  if (isTRUE(graph)) {
    plot(b_dend)
  }

  if (method != "MDS") {
    ## Factorial map of most contributing documents


    if (documents > dim(docCoord)[1]) {
      documents <- dim(docCoord)[1]
    }

    centers <- data.frame(dim1 = km.res$centers[, 1], dim2 = km.res$centers[, 2])
    centers$color <- cbPalette[1:dim(centers)[1]]
    row.names(centers) <- paste("cluster", as.character(1:dim(centers)[1]), sep = "")
    # A=docCoord[1:documents,1:2]
    # A=docCoord[,1:2]
    A <- euclDist(docCoord[, 1:2], centers)
    docCoord$Cluster <- A$color
    # A=A[1:documents,]
    A$color <- cbPalette[A$color]

    A$contrib <- docCoord$contrib
    A <- A %>%
      mutate(names = row.names(A)) %>%
      group_by(color) %>%
      slice_max(order_by = contrib, n = documents, with_ties = FALSE) %>%
      select(!"contrib") %>%
      as.data.frame()

    row.names(A) <- A$names
    A <- A[, -4]

    names(centers) <- names(A)
    A <- rbind(A, centers)
    x <- A$dim1
    y <- A$dim2
    A[, 4] <- row.names(A)

    names(A)[4] <- "nomi"

    df_all <- rbind(as.matrix(df), as.matrix(A[, 1:2]))
    rangex <- c(min(df_all[, 1]), max(df_all[, 1]))
    rangey <- c(min(df_all[, 2]), max(df_all[, 2]))

    b_doc <- ggplot(aes(x = dim1, y = dim2, label = nomi), data = A) +
      geom_point(size = 2, color = A$color) +
      labs(title = "Factorial map of the documents with the highest contributes") +
      geom_label_repel(
        box.padding = unit(0.5, "lines"), size = (log(labelsize * 3)), fontface = "bold",
        fill = adjustcolor(A$color, alpha.f = 0.6), color = "white", segment.alpha = 0.5, segment.color = "gray"
      ) +
      scale_x_continuous(limits = rangex, breaks = seq(round(rangex[1]), round(rangex[2]), 1)) +
      scale_y_continuous(limits = rangey, breaks = seq(round(rangey[1]), round(rangey[2]), 1)) +
      geom_hline(yintercept = 0, linetype = "dashed", color = adjustcolor("grey40", alpha.f = 0.7)) +
      geom_vline(xintercept = 0, linetype = "dashed", color = adjustcolor("grey40", alpha.f = 0.7)) +
      theme(
        # panel.border =  element_rect(fill=NA, size = 0.3, linetype = 'dashed', colour = adjustcolor("gray60",alpha.f = 0.7)),
        text = element_text(size = labelsize),
        axis.title = element_text(size = labelsize, face = "bold"),
        plot.title = element_text(size = labelsize + 1, face = "bold"),
        panel.background = element_rect(fill = "white", colour = "white"),
        # panel.grid.major = element_line(size = 0.3, linetype = 'dashed', colour = adjustcolor("gray60",alpha.f = 0.7)),
        axis.line.x = element_line(color = "black", linewidth = 0.5),
        axis.line.y = element_line(color = "black", linewidth = 0.5),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank()
      )



    if (method != "MDS") {
      b_doc <- b_doc + xlab(paste("Dim 1 (", round(res.mca$eigCorr$perc[1], 2), "%)", sep = "")) +
        ylab(paste("Dim 2 (", round(res.mca$eigCorr$perc[2], 2), "%)", sep = ""))
    } else {
      b_doc <- b_doc + xlab("Dim 1") + ylab("Dim 2")
    }

    ## logo coordinates
    xl <- c(rangex[2] - 0.02 - diff(rangex) * 0.125, rangex[2] - 0.02)
    yl <- c(rangey[1], rangey[1] + diff(rangey) * 0.125) + 0.02
    b_doc <- b_doc + annotation_custom(logo, xmin = xl[1], xmax = xl[2], ymin = yl[1], ymax = yl[2])
    ##

    if (isTRUE(graph)) {
      (plot(b_doc))
    }

    ## Factorial map of the most cited documents
    docCoord <- docCoord[order(-docCoord$TC), ]
    # B=docCoord[1:documents,1:2]
    B <- euclDist(docCoord[, 1:2], centers)
    B$color <- cbPalette[B$color]

    B$TC <- docCoord$TC
    B <- B %>%
      mutate(names = row.names(B)) %>%
      group_by(color) %>%
      slice_max(order_by = TC, n = documents, with_ties = FALSE) %>%
      select(!"TC") %>%
      as.data.frame()

    row.names(B) <- B$names
    B <- B[, -4]
    B <- rbind(B, centers)
    x <- B$dim1
    y <- B$dim2
    B[, 4] <- row.names(B)
    names(B)[4] <- "nomi"
    df_all_TC <- rbind(as.matrix(df), as.matrix(B[, 1:2]))
    rangex <- c(min(df_all_TC[, 1]), max(df_all_TC[, 1]))
    rangey <- c(min(df_all_TC[, 2]), max(df_all_TC[, 2]))

    b_doc_TC <- ggplot(aes(x = dim1, y = dim2, label = nomi), data = B) +
      geom_point(size = 2, color = B$color) +
      labs(title = "Factorial map of the most cited documents") +
      geom_label_repel(
        box.padding = unit(0.5, "lines"), size = (log(labelsize * 3)), fontface = "bold",
        fill = adjustcolor(B$color, alpha.f = 0.6), color = "white", segment.alpha = 0.5, segment.color = "gray"
      ) +
      scale_x_continuous(limits = rangex, breaks = seq(round(rangex[1]), round(rangex[2]), 1)) +
      scale_y_continuous(limits = rangey, breaks = seq(round(rangey[1]), round(rangey[2]), 1)) +
      xlab(paste("Dim 1 (", round(res.mca$eigCorr$perc[1], 2), "%)", sep = "")) +
      ylab(paste("Dim 2 (", round(res.mca$eigCorr$perc[2], 2), "%)", sep = "")) +
      geom_hline(yintercept = 0, linetype = "dashed", color = adjustcolor("grey60", alpha.f = 0.7)) +
      geom_vline(xintercept = 0, linetype = "dashed", color = adjustcolor("grey60", alpha.f = 0.7)) +
      theme(
        # panel.border =  element_rect(fill=NA, size = 0.3, linetype = 'dashed', colour = adjustcolor("gray60",alpha.f = 0.7)),
        text = element_text(size = labelsize),
        axis.title = element_text(size = labelsize, face = "bold"),
        plot.title = element_text(size = labelsize + 1, face = "bold"),
        panel.background = element_rect(fill = "white", colour = "white"),
        # panel.grid.major = element_line(size = 0.3, linetype = 'dashed', colour = adjustcolor("gray60",alpha.f = 0.7)),
        axis.line.x = element_line(color = "black", linewidth = 0.5),
        axis.line.y = element_line(color = "black", linewidth = 0.5),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank()
      )

    ## logo coordinates
    xl <- c(rangex[2] - 0.02 - diff(rangex) * 0.125, rangex[2] - 0.02)
    yl <- c(rangey[1], rangey[1] + diff(rangey) * 0.125) + 0.02
    b_doc_TC <- b_doc_TC + annotation_custom(logo, xmin = xl[1], xmax = xl[2], ymin = yl[1], ymax = yl[2])
    ##


    if (isTRUE(graph)) {
      plot(b_doc_TC)
    }

    semanticResults <- list(
      net = CW, res = res.mca, km.res = km.res, graph_terms = b, graph_dendogram = b_dend,
      graph_documents_Contrib = b_doc, graph_documents_TC = b_doc_TC, docCoord = docCoord, coord = results$coord, hull_data = hull_data
    )
  } else {
    semanticResults <- list(
      net = CW, res = res.mca, km.res = km.res, graph_terms = b, graph_dendogram = b_dend,
      graph_documents_Contrib = NULL, graph_documents_TC = NULL, docCoord = NULL, coord = NA, hull_data = hull_data
    )
  }

  params <- list(
    field = field,
    ngrams = ngrams,
    method = method,
    quali.supp = quali.supp,
    quanti.supp = quanti.supp,
    minDegree = minDegree,
    clust = clust,
    k.max = k.max,
    stemming = stemming,
    labelsize = labelsize,
    documents = documents,
    graph = graph,
    remove.terms = remove.terms,
    synonyms = synonyms
  )

  semanticResults$params <- data.frame(params = names(unlist(params)), values = unlist(params), row.names = NULL)


  return(semanticResults)
}


factorial <- function(X, method, quanti = NULL, quali = NULL) {
  df_quali <- data.frame()
  df_quanti <- data.frame()
  switch(method,
    ### CORRESPONDENCE ANALYSIS ###
    CA = {
      res.mca <- ca::ca(X, nd = 2)
      # Get coordinates of keywords
      coord <- list(
        coord = res.mca$colcoord,
        contrib = data.frame((res.mca$colcoord[, 1:2]^2) * res.mca$colmass),
        cos2 = data.frame(((res.mca$colcoord[, 1:2]^2) / (res.mca$coldist)))
      )
      # df <- data.frame(coord$coord)
      coord_doc <- list(
        coord = res.mca$rowcoord,
        contrib = data.frame((res.mca$rowcoord[, 1:2]^2) * res.mca$rowmass),
        cos2 = data.frame(((res.mca$rowcoord[, 1:2]^2) / (res.mca$rowdist)))
      )
      # df_doc <- data.frame(coord_doc$coord)
    },
    ### MULTIPLE CORRESPONDENCE ANALYSIS ###
    MCA = {
      X <- data.frame(apply(X, 2, factor))
      res.mca <- ca::mjca(X, nd = 2, lambda = "indicator", ps = "_")
      # Get coordinates of keywords (we take only categories "1"")
      K <- 2
      I <- dim(res.mca$rowcoord)[1]
      J <- dim(res.mca$colcoord)[1]
      evF <- matrix(rep(res.mca$sv[1:K], I), I, K, byrow = TRUE)
      evG <- matrix(rep(res.mca$sv[1:K], J), J, K, byrow = TRUE)
      rpc <- res.mca$rowcoord[, 1:2] * evF
      cpc <- res.mca$colcoord[, 1:2] * evG

      coord <- list(
        coord = data.frame(Dim1 = cpc[, 1], Dim2 = cpc[, 2], label = res.mca$levelnames, row.names = res.mca$levelnames) %>%
          dplyr::filter(substr(label, nchar(label) - 1, nchar(label)) == "_1") %>%
          select(-"label"),
        contrib = data.frame(cpc^2 * res.mca$colmass / res.mca$sv[1:2], label = res.mca$levelnames, row.names = res.mca$levelnames) %>%
          dplyr::filter(substr(label, nchar(label) - 1, nchar(label)) == "_1") %>%
          select(-"label"),
        cos2 = data.frame(((cpc^2) / (res.mca$coldist)), label = res.mca$levelnames, row.names = res.mca$levelnames) %>%
          dplyr::filter(substr(label, nchar(label) - 1, nchar(label)) == "_1") %>%
          select(-"label")
      )
      row.names(coord$coord) <- row.names(coord$contrib) <- row.names(coord$cos2) <- substr(row.names(coord$coord), 1, nchar(row.names(coord$coord)) - 2)
      # df <- coord$coord

      coord_doc <- list(
        coord = data.frame(Dim1 = rpc[, 1], Dim2 = rpc[, 1], row.names = row.names(X)),
        contrib = data.frame((rpc[, 1:2]^2) * res.mca$rowmass / res.mca$sv[1:2]),
        cos2 = data.frame(res.mca$rowmass * rpc^2 / res.mca$rowinertia)
      )
      # df_doc <- coord_doc$coord
    },
    MDS = {
      NetMatrix <- Matrix::crossprod(X, X)
      Net <- 1 - normalizeSimilarity(NetMatrix, type = "association")
      Matrix::diag(Net) <- 0
      # Net=as.matrix(Net)
      res.mca <- Net %>%
        # dist() %>%
        cmdscale()
      colnames(res.mca) <- c("Dim1", "Dim2")
      df <- data.frame(res.mca)
      row.names(df) <- row.names(Net)
    }
  )

  if (method != "MDS") {
    #
    docCoord <- as.data.frame(cbind(coord_doc$coord, rowSums(coord_doc$contrib)))
    names(docCoord) <- c("dim1", "dim2", "contrib")
    docCoord <- docCoord[order(-docCoord$contrib), ]

    # Benzecrì eigenvalue correction
    res.mca <- eigCorrection(res.mca)

    res.mca$coord_doc <- coord_doc

    results <- list(res.mca = res.mca, df = coord$coord, df_doc = coord_doc$coord, df_quali = df_quali, df_quanti = df_quanti, docCoord = docCoord, coord = coord)
  } else {
    results <- list(res.mca = res.mca, df = df, df_doc = NA, df_quali = NA, df_quanti = NA, docCoord = NA, coord = NA)
  }
  return(results)
}

euclDist <- function(x, y) {
  df <- as.data.frame(matrix(NA, dim(x)[1], dim(y)[1]))
  row.names(df) <- row.names(x)
  colnames(df) <- row.names(y)
  for (i in 1:dim(y)[1]) {
    ref <- y[i, 1:2]
    df[, i] <- apply(x, 1, function(x) sqrt(sum((x - ref)^2)))
  }
  x$color <- apply(df, 1, function(m) {
    which(m == min(m))
  })
  return(x)
}

eigCorrection <- function(res) {
  # Benzecri correction calculation
  n <- length(res$sv)
  e <- res$sv^2
  # n <- nrow(res$eig)
  # e <- res$eig[,1]
  eigBenz <- ((n / (n - 1))^2) * ((e - (1 / n))^2)
  eigBenz[e < 1 / n] <- 0
  perc <- eigBenz / sum(eigBenz) * 100
  cumPerc <- cumsum(perc)
  res$eigCorr <- data.frame(eig = e, eigBenz = eigBenz, perc = perc, cumPerc = cumPerc)
  return(res)
}


plotCoord <- function(g, side = "b") {
  a <- ggplot_build(g)$data

  ymin <- unlist(lapply(a, function(l) {
    if ("y" %in% names(l)) {
      min(l["y"])
    }
  })) %>% min(na.rm = TRUE)

  ymax <- unlist(lapply(a, function(l) {
    if ("y" %in% names(l)) {
      max(l["y"])
    }
  })) %>% max(na.rm = TRUE)

  xmin <- unlist(lapply(a, function(l) {
    if ("x" %in% names(l)) {
      min(l["x"])
    }
  })) %>% min(na.rm = TRUE)

  xmax <- unlist(lapply(a, function(l) {
    if ("x" %in% names(l)) {
      max(l["x"])
    }
  })) %>% max(na.rm = TRUE)

  coord <- c(xmin, xmax, ymin, ymax)

  xl <- c(xmax - 0.02 - diff(c(xmin, xmax)) * 0.125, xmax - 0.02)
  if (side == "b") {
    yl <- c(ymin, ymin + diff(c(ymin, ymax)) * 0.125) + 0.02
  } else {
    yl <- c(ymax - 0.02 - diff(c(ymin, ymax)) * 0.125, ymax - 0.02)
  }
  coord <- c(xl, yl)
}


dendPlot <- function(km.res, clust, label.cex, graph = FALSE) {
  # Dendrogram object
  dend <- as.dendrogram(km.res)
  # vector of colors
  labelColors <- colorlist()[1:clust]

  # cut dendrogram in k clusters
  clusMember <- cutree(km.res, clust)

  # function to get color labels
  colLab <- function(n) {
    if (is.leaf(n)) {
      a <- attributes(n)
      labCol <- labelColors[clusMember[which(names(clusMember) == a$label)]]
      attr(n, "nodePar") <- c(a$nodePar$lab.col, list(lab.col = labCol, lab.cex = label.cex))
      # attr(n, "label_cex") <- c(a$nodePar$lab.cex, label_cex = 0.1)
    }
    n
  }

  # using dendrapply
  clusDendro <- dendrapply(dend, colLab)
  k <- clust
  n <- length(km.res$labels)
  MidPoint <- (km.res$height[n - k] + km.res$height[n - k + 1]) / 2

  plotRes <- list(dend = clusDendro, line = MidPoint)
  class(plotRes) <- c("bibliodendrogram")

  if (graph) plot(plotRes)

  return(plotRes)
}

#' Deleting extra white spaces
#'
#' Deleting extra white spaces from a \code{character} object.
#'
#' \code{tableTag} is an internal routine of \code{bibliometrics} package.
#'
#' @param x is a \code{character} object.

#' @return an object of class \code{character}
#' @examples
#'
#' char <- c("Alfred  BJ", "Mary    Beth", "John      John")
#' char
#' trimES(char)
#'
#' @export
trimES <- function(x) {
  gsub("\\s+", " ", x)
}

# Test per funzioni utility: trim, trim.leading, trimES

test_that("trim rimuove spazi iniziali e finali", {
  expect_equal(trim("  hello  "), "hello")
  expect_equal(trim("hello"), "hello")
  expect_equal(trim("  "), "")
  expect_equal(trim(c("  a ", " b")), c("a", "b"))
})

test_that("trim.leading rimuove solo spazi iniziali", {
  expect_equal(trim.leading("  hello"), "hello")
  expect_equal(trim.leading("  hello  "), "hello  ")
  expect_equal(trim.leading("hello"), "hello")
  expect_equal(trim.leading(c("  a", " b")), c("a", "b"))
})

test_that("trimES rimuove spazi extra multipli", {
  expect_equal(trimES("hello  world"), "hello world")
  expect_equal(trimES("a   b   c"), "a b c")
  expect_equal(trimES("hello world"), "hello world")
  expect_equal(trimES(c("a  b", "c   d")), c("a b", "c d"))
})

utils::globalVariables(c("num"))
#' Merge bibliographic data frames from supported bibliogtraphic DBs
#'
#' Merge bibliographic data frames from different databases (WoS,SCOPUS, Lens, Openalex, etc-) into a single one.
#'
#' bibliographic data frames are obtained by the converting function \code{\link{convert2df}}.
#' The function merges data frames identifying common tag fields and duplicated records.
#'
#' @param ... are the bibliographic data frames to merge.
#' @param remove.duplicated is logical. If TRUE duplicated documents will be deleted from the bibliographic collection.
#' @param verbose is logical.  If TRUE, information on duplicate documents is printed on the screen.
#' @return the value returned from \code{mergeDbSources} is a bibliographic data frame.
#'
#'
#' @examples
#'
#' data(isiCollection, package = "bibliometrixData")
#'
#' data(scopusCollection, package = "bibliometrixData")
#'
#' M <- mergeDbSources(isiCollection, scopusCollection, remove.duplicated = TRUE)
#'
#' dim(M)
#'
#' @seealso \code{\link{convert2df}} to import and convert an ISI or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export


mergeDbSources <- function(..., remove.duplicated = TRUE, verbose = TRUE) {
  index <- NULL

  mc <- match.call(expand.dots = TRUE)

  if (length(mc) > 3) {
    M <- dplyr::bind_rows(list(...))
  } else {
    M <- dplyr::bind_rows(...)
  }
  # create KW_Merged field 
  M <- M %>% mergeKeywords(force=TRUE)

  dbLabels <- data.frame(
    DB = toupper(c("isi", "scopus", "openalex", "lens", "dimensions", "pubmed", "cochrane")),
    num = c(1, 2, 3, 4, 5, 6, 7)
  )
  DB <- unique(M$DB)
  
  if (length(DB) >1) {
    # order by db
    M <- M %>%
      left_join(dbLabels, by = "DB") %>%
      arrange(num) %>%
      select(-num) %>%
      rename("CR_raw" = "CR") %>%
      mutate(CR = "NA")
  }
  

  if (isTRUE(remove.duplicated)) {
    # remove by DOI
    if ("DI" %in% names(M)) {
      M$DI[M$DI == ""] <- NA
      index <- which(duplicated(M$DI) & !is.na(M$DI))
      if (length(index) > 0) M <- M[-index, ]
    }

    # remove by title
    if ("TI" %in% names(M)) {
      TI <- gsub("[^[:alnum:] ]", "", M$TI)
      TI <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", TI, perl = TRUE)
      d <- duplicated(paste(TI, " ", M$PY))
      if (isTRUE(verbose)) cat("\n", sum(d) + length(index), "duplicated documents have been removed\n")
      M <- M[!d, ]
    }
  }

  if (length(unique(M$DB)) > 1) {
    M$DB_Original <- M$DB
    M$DB <- "ISI"

    ## author data cleaning
    if ("AU" %in% names(M)) {
      M$AU <- gsub(",", " ", M$AU)
      AUlist <- strsplit(M$AU, ";")
      AU <- lapply(AUlist, function(l) {
        l <- trim(l)
        name <- strsplit(l, " ")
        lastname <- unlist(lapply(name, function(ln) {
          ln[1]
        }))
        firstname <- lapply(name, function(ln) {
          f <- paste(substr(ln[-1], 1, 1), collapse = " ")
        })
        AU <- paste(lastname, unlist(firstname), sep = " ", collapse = ";")
        return(AU)
      })
      M$AU <- unlist(AU)
    }
  }

  M <- metaTagExtraction(M, "SR")
  row.names(M) <- M$SR

  class(M) <- c("bibliometrixDB", "data.frame")
  return(M)
}

utils::globalVariables(c("max_retries", "retry_delay", "sleep_time", "verbose"))

# Helper function to make API calls with retry logic - STANDALONE VERSION
#' @keywords internal
safe_oa_fetch <- function(entity,
                          identifier = NULL,
                          doi = NULL,
                          attempt = 1,
                          sleep_time = 0.5,
                          retry_delay = 2,
                          max_retries = 3,
                          verbose = FALSE,
                          mailto = NULL) {
  
  if (attempt > 1 && verbose) {
    cat("Retry attempt", attempt, "of", max_retries, "\n")
  }
  
  # Add delay before API call (except for first attempt)
  if (attempt > 1 || entity == "authors") {
    wait_time <- if (attempt == 1) sleep_time else retry_delay * (2 ^ (attempt - 2))
    if (verbose) cat("Waiting", round(wait_time, 2), "seconds before API call...\n")
    Sys.sleep(wait_time)
  }
  
  # Build common arguments for oa_fetch
  fetch_args <- list(entity = entity, output = "tibble")
  if (!is.null(mailto) && nchar(mailto) > 0) {
    fetch_args$mailto <- mailto
  }

  result <- tryCatch({
    if (!is.null(identifier)) {
      fetch_args$identifier <- identifier
      do.call(openalexR::oa_fetch, fetch_args)
    } else if (!is.null(doi)) {
      fetch_args$doi <- doi
      do.call(openalexR::oa_fetch, fetch_args)
    }
  }, error = function(e) {
    error_msg <- as.character(e$message)
    
    # Check if it's a rate limit error (429)
    if (grepl("429", error_msg) || grepl("Too Many Requests", error_msg, ignore.case = TRUE)) {
      if (attempt < max_retries) {
        if (verbose) {
          cat("Rate limit hit (HTTP 429). Retrying with exponential backoff...\n")
        }
        return(safe_oa_fetch(entity, identifier, doi, attempt + 1,
                             sleep_time, retry_delay, max_retries, verbose, mailto))
      } else {
        stop("Rate limit exceeded after ", max_retries, " attempts. ",
             "Please wait a few minutes or set an OpenAlex API key for higher rate limits. ",
             "Get a free key at: https://openalex.org/")
      }
    }
    
    # Check for other temporary errors
    if (grepl("500|502|503|504", error_msg) || grepl("timeout", error_msg, ignore.case = TRUE)) {
      if (attempt < max_retries) {
        if (verbose) {
          cat("Temporary server error. Retrying...\n")
        }
        return(safe_oa_fetch(entity, identifier, doi, attempt + 1,
                             sleep_time, retry_delay, max_retries, verbose, mailto))
      }
    }
    
    # If not a retryable error, or max retries reached, throw the error
    stop(e$message)
  })
  
  return(result)
}


#' Retrieve Author Biographical Information from OpenAlex
#'
#' This function downloads comprehensive author information from OpenAlex based on a DOI 
#' and the numerical position of the author in the co-authors list. It provides detailed 
#' biographical data, bibliometric indicators, and affiliation information.
#'
#' @param author_position Integer. The numerical position of the author in the authors list (default: 1)
#' @param doi Character. DOI of the article used to identify the authors
#' @param verbose Logical. Print informative messages during execution (default: FALSE)
#' @param return_all_authors Logical. If TRUE, returns information for all co-authors (default: FALSE)
#' @param sleep_time Numeric. Seconds to wait between API calls to respect rate limits (default: 1)
#' @param max_retries Integer. Maximum number of retry attempts for failed API calls (default: 3)
#' @param retry_delay Numeric. Base delay in seconds before retrying after an error (default: 2)
#' @param mailto Character. Email address for OpenAlex polite pool (default: NULL, uses openalexR.mailto option if set)
#'
#' @return If \code{return_all_authors = FALSE}, returns a tibble with comprehensive information 
#'   about the specified author including:
#'   \itemize{
#'     \item Basic information (name, ORCID, OpenAlex ID)
#'     \item Bibliometric indicators (works count, citations, h-index, i10-index)
#'     \item Affiliation details from both the paper and author profile
#'     \item Research topics and areas
#'     \item Paper-specific metadata (corresponding author status, position type)
#'   }
#'   If \code{return_all_authors = TRUE}, returns a list of tibbles, one for each co-author.
#'
#' @details 
#' The function first retrieves the work information using the provided DOI, then extracts
#' author IDs from the authorships data, and finally fetches detailed author profiles from
#' OpenAlex. It enriches the author data with paper-specific information such as authorship
#' position, corresponding author status, and affiliations as listed in the paper.
#' 
#' The function implements automatic retry logic with exponential backoff to handle rate limiting
#' (HTTP 429 errors) and temporary network issues. It respects OpenAlex API rate limits by adding
#' configurable delays between requests.
#' 
#' IMPORTANT: For better rate limits, set your OpenAlex API key using:
#' Sys.setenv(openalexR_apikey = "YOUR_API_KEY")
#' Get a free API key at: https://openalex.org/
#'
#' @examples
#' \dontrun{
#' # Get information for the first author
#' first_author <- authorBio(doi = "10.1016/j.joi.2017.08.007")
#' 
#' # Get information for the second author with verbose output
#' second_author <- authorBio(
#'   author_position = 2, 
#'   doi = "10.1016/j.joi.2017.08.007", 
#'   verbose = TRUE
#' )
#' 
#' # Get information for all co-authors with custom rate limiting
#' all_authors <- authorBio(
#'   doi = "10.1016/j.joi.2017.08.007", 
#'   return_all_authors = TRUE,
#'   sleep_time = 0.5,
#'   max_retries = 5
#' )
#' }
#'
#' @export
#'
authorBio <- function(author_position = 1,
                      doi = "10.1016/j.joi.2017.08.007",
                      verbose = FALSE,
                      return_all_authors = FALSE,
                      sleep_time = 1,
                      max_retries = 3,
                      retry_delay = 2,
                      mailto = NULL) {
  
  # Input validation
  if (is.null(doi) || !is.character(doi) || nchar(trimws(doi)) == 0) {
    stop("The 'doi' parameter must be a non-empty character string")
  }
  
  if (!is.numeric(author_position) || author_position < 1 || author_position != as.integer(author_position)) {
    stop("The 'author_position' parameter must be a positive integer")
  }
  
  if (!is.numeric(sleep_time) || sleep_time < 0) {
    stop("The 'sleep_time' parameter must be a non-negative number")
  }
  
  if (!is.numeric(max_retries) || max_retries < 0) {
    stop("The 'max_retries' parameter must be a non-negative integer")
  }
  
  # Check library availability
  if (!requireNamespace("openalexR", quietly = TRUE)) {
    stop("The 'openalexR' library is not available. Install it with: install.packages('openalexR')")
  }
  
  # Check if API key is set
  api_key <- Sys.getenv("openalexR_apikey", unset = "")
  if (api_key == "" && verbose) {
    cat("WARNING: No OpenAlex API key detected. Rate limits will be stricter.\n")
    cat("Get a free API key at: https://openalex.org/\n")
    cat("Set it with: Sys.setenv(openalexR_apikey = 'YOUR_API_KEY')\n\n")
  }
  
  if (verbose) cat("Retrieving article information for DOI:", doi, "\n")
  
  # Retrieve article information with error handling and retry logic
  au_work <- tryCatch({
    safe_oa_fetch(entity = "works", doi = doi,
                  sleep_time = sleep_time, retry_delay = retry_delay,
                  max_retries = max_retries, verbose = verbose,
                  mailto = mailto)
  }, error = function(e) {
    stop("Error retrieving article: ", e$message, 
         "\nPlease verify that the DOI is correct and OpenAlex is accessible")
  })
  
  # Verify that the article was found
  if (is.null(au_work) || nrow(au_work) == 0) {
    stop("No article found for the provided DOI: ", doi)
  }
  
  # Extract author information from the correct structure
  authorships <- au_work$authorships[[1]]
  
  if (is.null(authorships) || nrow(authorships) == 0) {
    stop("No author information found for this article")
  }
  
  # Verify that the requested position exists
  if (author_position > nrow(authorships)) {
    stop("Author position (", author_position, 
         ") is greater than the total number of authors (", nrow(authorships), ")")
  }
  
  if (verbose) {
    cat("Article found:", au_work$display_name[1], "\n")
    cat("Total number of authors:", nrow(authorships), "\n")
    if (nrow(authorships) > 0) {
      cat("Authors:\n")
      for (i in 1:nrow(authorships)) {
        cat("  ", i, ".", authorships$display_name[i], "\n")
      }
    }
  }
  
  # If requested, return all authors
  if (return_all_authors) {
    if (verbose) cat("\nRetrieving information for all authors...\n")
    
    all_authors <- list()
    successful_authors <- 0
    failed_authors <- 0
    
    for (i in 1:nrow(authorships)) {
      if (verbose) cat("\nProcessing author", i, "of", nrow(authorships), ":", 
                       authorships$display_name[i], "\n")
      
      author_id <- authorships$id[i]
      if (!is.na(author_id) && author_id != "") {
        # Extract only the OpenAlex ID from the full URL
        clean_id <- gsub("https://openalex.org/", "", author_id)
        
        author_info <- tryCatch({
          safe_oa_fetch(entity = "authors", identifier = clean_id,
                        sleep_time = sleep_time, retry_delay = retry_delay,
                        max_retries = max_retries, verbose = verbose,
                        mailto = mailto)
        }, error = function(e) {
          if (verbose) cat("Error for author", i, ":", e$message, "\n")
          failed_authors <<- failed_authors + 1
          NULL
        })
        
        if (!is.null(author_info) && nrow(author_info) > 0) {
          # Add additional information from the authorships structure
          author_info$author_position_in_paper <- i
          author_info$original_author_name <- authorships$display_name[i]
          author_info$is_corresponding <- authorships$is_corresponding[i]
          author_info$author_position_type <- authorships$author_position[i]
          
          # Add affiliation information if available
          if (!is.null(authorships$affiliations[[i]]) && nrow(authorships$affiliations[[i]]) > 0) {
            author_info$primary_affiliation <- authorships$affiliations[[i]]$display_name[1]
            author_info$primary_affiliation_country <- authorships$affiliations[[i]]$country_code[1]
            author_info$primary_affiliation_ror <- authorships$affiliations[[i]]$ror[1]
          } else {
            author_info$primary_affiliation <- NA
            author_info$primary_affiliation_country <- NA
            author_info$primary_affiliation_ror <- NA
          }
          
          # Add raw affiliation if available
          if (!is.null(authorships$affiliation_raw) && length(authorships$affiliation_raw) >= i) {
            author_info$affiliation_raw <- authorships$affiliation_raw[i]
          } else {
            author_info$affiliation_raw <- NA
          }
          
          all_authors[[i]] <- author_info
          successful_authors <- successful_authors + 1
          
          if (verbose) cat("Successfully retrieved data for:", author_info$display_name[1], "\n")
        }
      } else {
        if (verbose) cat("Invalid author ID for position", i, "\n")
        failed_authors <- failed_authors + 1
      }
    }
    
    if (verbose) {
      cat("\n=== Summary ===\n")
      cat("Successfully retrieved:", successful_authors, "authors\n")
      if (failed_authors > 0) {
        cat("Failed to retrieve:", failed_authors, "authors\n")
      }
    }
    
    # Combine all valid results
    valid_authors <- all_authors[!sapply(all_authors, is.null)]
    if (length(valid_authors) > 0) {
      # Add common metadata to all
      for (i in seq_along(valid_authors)) {
        valid_authors[[i]]$source_doi <- doi
        valid_authors[[i]]$source_title <- au_work$display_name[1]
        valid_authors[[i]]$query_timestamp <- Sys.time()
      }
      return(valid_authors)
    } else {
      stop("Unable to retrieve information for any author")
    }
  }
  
  # Retrieve information for the specific author
  author_id <- authorships$id[author_position]
  
  if (is.na(author_id) || author_id == "") {
    stop("Invalid author ID at position ", author_position)
  }
  
  # Extract only the OpenAlex ID from the full URL
  clean_id <- gsub("https://openalex.org/", "", author_id)
  
  if (verbose) {
    cat("\nRetrieving information for author at position", author_position, "\n")
    cat("Author name:", authorships$display_name[author_position], "\n")
    cat("OpenAlex ID:", clean_id, "\n")
    cat("Position type:", authorships$author_position[author_position], "\n")
    cat("Is corresponding author:", authorships$is_corresponding[author_position], "\n")
  }
  
  # Retrieve author biographical data with retry logic
  au_info <- tryCatch({
    safe_oa_fetch(entity = "authors", identifier = clean_id,
                  sleep_time = sleep_time, retry_delay = retry_delay,
                  max_retries = max_retries, verbose = verbose,
                  mailto = mailto)
  }, error = function(e) {
    stop("Error retrieving author information: ", e$message)
  })
  
  if (is.null(au_info) || nrow(au_info) == 0) {
    stop("No biographical information found for the author at position ", author_position)
  }
  
  # Add useful metadata from the authorships structure
  au_info$author_position_in_paper <- author_position
  au_info$original_author_name <- authorships$display_name[author_position]
  au_info$is_corresponding <- authorships$is_corresponding[author_position]
  au_info$author_position_type <- authorships$author_position[author_position]
  
  # Add affiliation information if available
  if (!is.null(authorships$affiliations[[author_position]]) && 
      nrow(authorships$affiliations[[author_position]]) > 0) {
    au_info$primary_affiliation <- authorships$affiliations[[author_position]]$display_name[1]
    au_info$primary_affiliation_country <- authorships$affiliations[[author_position]]$country_code[1]
    au_info$primary_affiliation_ror <- authorships$affiliations[[author_position]]$ror[1]
  } else {
    au_info$primary_affiliation <- NA
    au_info$primary_affiliation_country <- NA
    au_info$primary_affiliation_ror <- NA
  }
  
  # Add raw affiliation if available
  if (!is.null(authorships$affiliation_raw) && length(authorships$affiliation_raw) >= author_position) {
    au_info$affiliation_raw <- authorships$affiliation_raw[author_position]
  } else {
    au_info$affiliation_raw <- NA
  }
  
  # Add query metadata
  au_info$source_doi <- doi
  au_info$source_title <- au_work$display_name[1]
  au_info$query_timestamp <- Sys.time()
  
  if (verbose) {
    cat("\n=== Success ===\n")
    cat("Information successfully retrieved for:", au_info$display_name[1], "\n")
    cat("Number of publications:", au_info$works_count[1], "\n")
    cat("Number of citations:", au_info$cited_by_count[1], "\n")
    cat("H-index:", au_info$h_index[1], "\n")
    cat("Primary affiliation:", au_info$primary_affiliation[1], "\n")
  }
  
  return(au_info)
}

# Helper function to analyze all authors of an article
analyze_all_authors <- function(doi, verbose = FALSE, sleep_time = 0.2, max_retries = 3) {
  return(authorBio(doi = doi, return_all_authors = TRUE, verbose = verbose, 
                   sleep_time = sleep_time, max_retries = max_retries))
}

#' Get Authors Summary from OpenAlex
#'
#' Retrieves a quick summary of all authors from a paper without making additional API calls
#' for individual author profiles. Useful for getting an overview of the authorship structure.
#'
#' @param doi Character. DOI of the article
#' @param verbose Logical. Print informative messages during execution (default: FALSE)
#' @param sleep_time Numeric. Seconds to wait before API call (default: 0.2)
#' @param max_retries Integer. Maximum number of retry attempts (default: 3)
#' @param mailto Character. Email address for the OpenAlex API polite pool (default: NULL)
#'
#' @return A data frame with summary information for all authors including:
#'   \itemize{
#'     \item position: Author position in the paper
#'     \item display_name: Author name as it appears in the paper
#'     \item author_position_type: Type of position (first, last, middle)
#'     \item is_corresponding: Whether the author is a corresponding author
#'     \item orcid: ORCID identifier if available
#'     \item openalex_id: OpenAlex author identifier
#'     \item primary_affiliation: Main institutional affiliation
#'   }
#'
#' @examples
#' \dontrun{
#' # Get a quick summary of all authors
#' summary <- get_authors_summary(doi = "10.1016/j.joi.2017.08.007")
#' print(summary)
#' }
#'
#' @export
get_authors_summary <- function(doi = "10.1016/j.joi.2017.08.007",
                                verbose = FALSE,
                                sleep_time = 0.2,
                                max_retries = 3,
                                mailto = NULL) {
  if (verbose) cat("Retrieving author summary for DOI:", doi, "\n")
  
  # Helper function with retry logic (simplified for single call)
  safe_fetch <- function(attempt = 1) {
    if (attempt > 1) {
      wait_time <- sleep_time * (2 ^ (attempt - 1))
      if (verbose) cat("Waiting", round(wait_time, 2), "seconds before retry...\n")
      Sys.sleep(wait_time)
    }
    
    tryCatch({
      fetch_args <- list(entity = "works", doi = doi, output = "tibble")
      if (!is.null(mailto) && nchar(mailto) > 0) fetch_args$mailto <- mailto
      do.call(openalexR::oa_fetch, fetch_args)
    }, error = function(e) {
      if ((grepl("429", e$message) || grepl("Too Many Requests", e$message)) && 
          attempt < max_retries) {
        if (verbose) cat("Rate limit hit. Retrying...\n")
        return(safe_fetch(attempt + 1))
      }
      stop(e$message)
    })
  }
  
  au_work <- safe_fetch()
  
  if (is.null(au_work) || nrow(au_work) == 0) {
    stop("No article found for the provided DOI: ", doi)
  }
  
  authorships <- au_work$authorships[[1]]
  
  # Create a summary without additional API calls
  summary_df <- data.frame(
    position = 1:nrow(authorships),
    display_name = authorships$display_name,
    author_position_type = authorships$author_position,
    is_corresponding = authorships$is_corresponding,
    orcid = authorships$orcid,
    openalex_id = authorships$id,
    stringsAsFactors = FALSE
  )
  
  # Add affiliations if available
  summary_df$primary_affiliation <- sapply(1:nrow(authorships), function(i) {
    if (!is.null(authorships$affiliations[[i]]) && nrow(authorships$affiliations[[i]]) > 0) {
      return(authorships$affiliations[[i]]$display_name[1])
    } else {
      return(NA)
    }
  })
  
  if (verbose) {
    cat("Successfully retrieved summary for", nrow(summary_df), "authors\n")
  }
  
  return(summary_df)
}

utils::globalVariables("where")

dimensions2df <- function(file, format = "csv") {
  switch(format,
    csv = {
      for (i in 1:length(file)) {
        # D <- rio::import(file[i], quote = '"',dec = ".",skip=1)

        D <- read_csv(file[i], na = character(), quote = '"', trim_ws = FALSE, progress = show_progress(), show_col_types = FALSE) %>%
          mutate(across(where(is.numeric), as.character)) %>%
          mutate(across(where(is.character), function(x) tidyr::replace_na(x, ""))) %>%
          as.data.frame()


        if (i > 1) {
          l <- intersect(l, names(D))
          DATA <- rbind(DATA[l], D[l])
        } else {
          l <- names(D)
          DATA <- D
        }
      }
      # DATA=rio::import(file, quote = '"',dec = ".",skip=1)
    },
    excel = {
      for (i in 1:length(file)) {
        # D1 <- rio::import(file[i], skip=1)
        D <- readxl::read_excel(file[1], skip = 1)
        D <- as.data.frame(D)

        if (i > 1) {
          l <- intersect(l, names(D))
          DATA <- rbind(DATA[l], D[l])
        } else {
          l <- names(D)
          DATA <- D
        }
      }
      # DATA <- rio::import(file, skip = 1)
    }
  )
  # Encoding(DATA) <- "UTF-8"

  bibtag <- NULL
  data("bibtag", envir = environment())
  bibtag <- as.data.frame(bibtag)


  names(DATA) <- trimES(gsub("\\.|-", " ", names(DATA)))

  # i <-
  #   which(names(DATA) == "Authors Affiliations Name of Research organization")
  # if (length(i) > 0) {
  #   names(DATA)[i] = "Authors Affiliations"
  # }
  # i <- which(names(DATA) == "Source title/Anthology title")
  # if (length(i) > 0) {
  #   names(DATA)[i] = "Source title"
  # }

  fields <- names(DATA)

  for (i in 1:length(fields)) {
    # print(i)
    ind <- which(bibtag$DIMENSIONS == fields[i])

    if (length(ind) > 0) {
      fields[i] <- bibtag$TAG[ind]
      # if (length(ind)>1) print(ind)
    }
  }
  names(DATA) <- fields
  ind <- which(names(DATA) == "Source title/Anthology title")
  if (length(ind) == 1) {
    names(DATA)[ind] <- "SO"
  }
  ind <- which(names(DATA) == "Authors Affiliations Name of Research organization")
  if (length(ind) == 1) {
    names(DATA)[ind] <- "C1"
    DATA$AU_UN <- DATA$C1
  }
  ind <- which(names(DATA) == "Authors Affiliations Country of Research organization")
  if (length(ind) == 1) {
    names(DATA)[ind] <- "AU_CO"
  }


  DATA <- postprocessingDim(DATA)

  return(DATA)
}



postprocessingDim <- function(DATA) {
  # DATA <- data.frame(lapply(DATA, toupper), stringsAsFactors = FALSE)

  ## Converting original references in WOS format (AU, PY, SO, VOL, NUM, DOI)
  if ("Cited references" %in% names(DATA)) {
    aaa <- strsplit(DATA$`Cited references`, ";\\[")
    cr <- (unlist(lapply(aaa, function(l) {
      l <- gsub("\\|", "!!!", l)
      l <- strsplit(l, "!!!")

      ## first authors (of cited references)
      au <- sapply(l, `[`, 1)
      au <-
        ifelse(nchar(au) > 2,
          {
            au <- gsub("\\[", "", unlist(lapply(strsplit(au, ";"), function(x) {
              x <- x[1]
            })))
            lastname <- trim(gsub(",.*", "", au))
            firstname <- strsplit(trim(gsub(".*,", "", au)), " ")
            firstname <- lapply(firstname, function(x) {
              if (length(x) > 0) {
                x <- paste(substr(x, 1, 1), sep = "", collapse = "")
              } else {
                x <- ""
              }
              return(x)
            })
            au <- paste(lastname,
              unlist(firstname),
              sep = " "
            )
          },
          "NA"
        )

      ## publication year
      py <- sapply(l, `[`, 4)
      so <- sapply(l, `[`, 3)
      vol <-
        ifelse(nchar(sapply(l, `[`, 5)) > 0, paste("V", sapply(l, `[`, 5),
          sep =
            ""
        ), "")
      num <-
        ifelse(nchar(sapply(l, `[`, 6)) > 0, paste("N", sapply(l, `[`, 6),
          sep =
            ""
        ), "")
      ## doi
      doi <- sapply(l, `[`, 8)
      ref <- paste(au,
        py,
        so,
        vol,
        num,
        doi,
        sep = ", ",
        collapse = ";"
      )
      ref <- gsub("^,*|(?<=,),|,*$", "", ref, perl = T)
    })))

    DATA$CR <- cr
    DATA$CR <- gsub("] ];", "", cr)
  } else {
    DATA$CR <- "NA,0000,NA"
  }

  # Document Type
  if (!("DT" %in% names(DATA))) {
    DATA$DT <- "Article"
  }

  # Authors cleaning and converting in WoS format
  DATA$AF <- DATA$AU

  DATA$AU <- gsub("\\s+", " ", DATA$AU)
  DATA$AU <- gsub("\\(|\\)", "", DATA$AU)

  listAU <- strsplit(DATA$AU, "; ")

  AU <- lapply(listAU, function(l) {
    lastname <- trim(gsub(",.*", "", l))
    firstname <- strsplit(trim(gsub(".*,", "", l)), " ")
    firstname <- lapply(firstname, function(x) {
      if (length(x) > 0) {
        x <- paste(substr(x, 1, 1), sep = "", collapse = "")
      } else {
        x <- ""
      }
      return(x)
    })
    AU <- paste(lastname,
      unlist(firstname),
      sep = " ",
      collapse = ";"
    )
    return(AU)
  })


  DATA$AU <- unlist(AU)

  # keywords
  if (!("DE" %in% names(DATA)) & !("ID" %in% names(DATA))) {
    if ("MeSH terms" %in% names(DATA)) {
      DATA$DE <- DATA$ID <- DATA$`MeSH terms`
    } else {
      DATA$DE <- DATA$ID <- "NA"
    }
  }
  if (("DE" %in% names(DATA)) & !("ID" %in% names(DATA))) {
    DATA$ID <- DATA$DE
  }
  if (!("DE" %in% names(DATA)) & ("ID" %in% names(DATA))) {
    DATA$DE <- DATA$ID
  }

  # Affiliations

  RP <- lapply(regmatches(DATA$RP, gregexpr("(?<=\\().*?(?=\\))", DATA$RP, perl = T)), function(l) {
    if (length(l[1]) == 0) {
      l <- NA
    } else {
      l <- paste(l, collapse = ";")
    }
    return(l)
  })
  DATA$AU1_UN <- unlist(RP)

  i <- which(names(DATA) %in% c("Authors Affiliations"))
  if (length(i) == 1) {
    # names(DATA)[i] <- "AU_UN"
    C1 <- lapply(regmatches(DATA$C1, gregexpr("(?<=\\().*?(?=\\))", DATA$C1, perl = T)), function(l) {
      if (length(l[1]) == 0) {
        l <- NA
      } else {
        l <- paste(l, collapse = ";")
      }
      return(l)
    })
    DATA$AU_UN <- unlist(C1)
    DATA$AU_UN <- gsub("\\(|)", " ", DATA$AU_UN)
  }

  if (("SO" %in% names(DATA)) & ("Anthology title" %in% names(DATA))) {
    ind <- which(is.na(DATA$SO) | DATA$SO == "")
    DATA$SO[ind] <- DATA$`Anthology title`[ind]
    DATA$SO[is.na(DATA$SO) | DATA$SO == ""] <- "NA"
  }

  if (!("SO" %in% names(DATA))) {
    DATA$SO <- "NA"
  }

  ####
  cat("\nCreating ISO Source names...")
  DATA$JI <- DATA$SO
  # DATA$JI <- sapply(DATA$SO, AbbrevTitle, USE.NAMES = FALSE)
  DATA$J9 <- gsub("\\.", "", DATA$JI)
  ####
  AB <- DATA$AB
  TI <- DATA$TI
  DE <- DATA$DE

  DATA <- data.frame(lapply(DATA, toupper), stringsAsFactors = FALSE)

  DATA$AB_raw <- AB
  DATA$TI_raw <- TI
  DATA$DE_raw <- DE

  DATA$PY <- as.numeric(DATA$PY)

  DATA$TC <- as.numeric(DATA$TC)

  DATA$DB <- "DIMENSIONS"

  if (!"AU_CO" %in% names(DATA)) DATA <- metaTagExtraction(DATA, "AU_CO")

  DATA$AU1_CO <- unlist(lapply(strsplit(DATA$AU_CO, ";"), function(l) {
    if (length(l) > 0) {
      l <- l[1]
    } else {
      l <- "NA"
    }
    return(l)
  }))

  DATA <- metaTagExtraction(DATA, "AU1_CO")
  return(DATA)
}

utils::globalVariables(c("SR", "text", "ngram"))
#' Term extraction tool from textual fields of a manuscript
#'
#' It extracts terms from a text field (abstract, title, author's keywords, etc.) of a bibliographic data frame.
#' @param M is a data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.
#' @param Field is a character object. It indicates the field tag of textual data :
#' \tabular{lll}{
#' \code{"TI"}\tab   \tab Manuscript title\cr
#' \code{"AB"}\tab   \tab Manuscript abstract\cr
#' \code{"ID"}\tab   \tab Manuscript keywords plus\cr
#' \code{"DE"}\tab   \tab Manuscript author's keywords}
#' The default is \code{Field = "TI"}.
#'
#' @param ngrams is an integer between 1 and 3. It indicates the type of n-gram to extract from texts.
#' An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.
#' @param stemming is logical. If TRUE the Porter Stemming algorithm is applied to all extracted terms. The default is \code{stemming = FALSE}.
#' @param language is a character. It is the language of textual contents ("english", "german","italian","french","spanish"). The default is \code{language="english"}.
#' @param remove.numbers is logical. If TRUE all numbers are deleted from the documents before term extraction. The default is \code{remove.numbers = TRUE}.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the corpus after term extraction. The default is \code{remove.terms = NULL}.
#' @param keep.terms is a character vector. It contains a list of compound words "formed by two or more terms" to keep in their original form in the term extraction process. The default is \code{keep.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @param verbose is logical. If TRUE the function prints the most frequent terms extracted from documents. The default is \code{verbose=TRUE}.
#' @return the bibliometric data frame with a new column containing terms about the field tag indicated in the argument \code{Field}.
#'
#'
#' @examples
#' # Example 1: Term extraction from titles
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' # vector of compound words
#' keep.terms <- c("co-citation analysis", "bibliographic coupling")
#'
#' # term extraction
#' scientometrics <- termExtraction(scientometrics,
#'   Field = "TI", ngrams = 1,
#'   remove.numbers = TRUE, remove.terms = NULL, keep.terms = keep.terms, verbose = TRUE
#' )
#'
#' # terms extracted from the first 10 titles
#' scientometrics$TI_TM[1:10]
#'
#'
#' # Example 2: Term extraction from abstracts
#'
#' data(scientometrics)
#'
#' # term extraction
#' scientometrics <- termExtraction(scientometrics,
#'   Field = "AB", ngrams = 2,
#'   stemming = TRUE, language = "english",
#'   remove.numbers = TRUE, remove.terms = NULL, keep.terms = NULL, verbose = TRUE
#' )
#'
#' # terms extracted from the first abstract
#' scientometrics$AB_TM[1]
#'
#' # Example 3: Term extraction from keywords with synonyms
#'
#' data(scientometrics)
#'
#' # vector of synonyms
#' synonyms <- c("citation; citation analysis", "h-index; index; impact factor")
#'
#' # term extraction
#' scientometrics <- termExtraction(scientometrics,
#'   Field = "ID", ngrams = 1,
#'   synonyms = synonyms, verbose = TRUE
#' )
#'
#' @seealso \code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#'
#' @export



termExtraction <- function(M, Field = "TI", ngrams = 1, stemming = FALSE, language = "english", remove.numbers = TRUE, remove.terms = NULL, keep.terms = NULL, synonyms = NULL, verbose = TRUE) {
  # ngrams imposed = 1 for keywords
  if (Field %in% c("ID", "DE")) {
    ngrams <- 1
  }

  # load stopwords
  data("stopwords", envir = environment())
  data("stop_words", envir = environment(), package = "tidytext")
  stop_words <- stop_words %>% as.data.frame()

  if (ngrams == 2) {
    remove.terms <- c(remove.terms, stopwords$bigrams)
  }

  switch(language,
    english = {
      stopwords <- (stop_words$word)
    },
    italian = {
      stopwords <- stopwords$it
    },
    german = {
      stopwords <- stopwords$de
    },
    french = {
      stopwords <- stopwords$fr
    },
    spanish = {
      stopwords <- stopwords$es
    }
  )
  stopwords <- tolower(stopwords)

  # remove all special characters (except "-" becoming "_")
  TERMS <- M %>%
    select(SR, !!Field)

  names(TERMS) <- c("SR", "text")

  TERMS$text <- gsub(" - ", " ", TERMS$text)

  # save original multi-words keywords
  if (Field %in% c("ID", "DE")) {
    listTerms <- strsplit(TERMS$text, ";")
    TERMS$text <- unlist(lapply(listTerms, function(l) {
      l <- gsub("-", "__", trimES(trimws(l)))
      l <- tolower(paste(gsub(" ", "_", l), sep = "", collapse = ";"))
    }))
  } else {
    TERMS <- TERMS %>%
      mutate(
        text = tolower(gsub("[^[:alnum:][:blank:]\\-]", "", text)),
        text = gsub("-", "__", text)
      )
  }


  # remove numbers
  if (remove.numbers == TRUE) {
    TERMS <- TERMS %>%
      mutate(text = gsub("[[:digit:]]", "", text))
  }

  # keep terms in the vector keep.terms
  if (length(keep.terms) > 0 & is.character(keep.terms)) {
    keep.terms <- tolower(keep.terms)
    if (Field %in% c("DE", "ID")) {
      kt <- gsub(" ", "_", keep.terms)
      kt <- gsub("-", "__", keep.terms)
    } else {
      kt <- gsub("-", "__", keep.terms)
    }
    for (i in 1:length(keep.terms)) {
      TERMS <- TERMS %>%
        mutate(text = gsub(keep.terms[i], kt[i], text))
    }
  }

  if (is.null(remove.terms)) remove.terms <- ""

  TERMS <- extractNgrams(
    text = TERMS, Var = "text", nword = ngrams,
    stopwords = stopwords, custom_stopwords = tolower(remove.terms),
    stemming = stemming, language = language, synonyms = synonyms, Field = Field
  )

  TERMS <- TERMS %>%
    dplyr::filter(!(ngram %in% paste(rep("NA", ngrams), sep = "", collapse = " "))) %>%
    group_by(SR) %>%
    summarize(text = paste(ngram, collapse = ";"))

  # assign the vector to the bibliographic data frame
  col_name <- paste(Field, "_TM", sep = "")
  M <- M[!names(M) %in% col_name]

  M <- TERMS %>%
    right_join(M, by = "SR")
  names(M)[which(names(M) %in% "text")] <- col_name


  # display results
  if (verbose == TRUE) {
    s <- tableTag(M, col_name)

    if (length(s > 25)) {
      print(s[1:25])
    } else {
      print(s)
    }
  }

  class(M) <- c("bibliometrixDB", "data.frame")
  row.names(M) <- M$SR
  return(M)
}

extractNgrams <- function(text, Var, nword, stopwords, custom_stopwords, stemming, language, synonyms, Field) {
  # text is data frame containing the corpus data text = M %>% select(.data$SR,.data$AB)
  # Var is a string indicating the column name. I.e. Var = "AB"
  # nword is a integer vector indicating the ngrams to extract. I.e. nword = c(2,3)

  stopwords <- c(
    stopwords, "elsevier", "springer", "wiley", "mdpi", "emerald", "originalityvalue", "designmethodologyapproach",
    "-", " -", "-present", "-based", "-literature", "-matter"
  )
  custom_stopngrams <- c(
    custom_stopwords, "rights reserved", "john wiley", "john wiley sons", "science bv", "mdpi basel",
    "mdpi licensee", "emerald publishing", "taylor francis", "paper proposes",
    "we proposes", "paper aims", "articles published", "study aims", "research limitationsimplications"
  )
  ngram <- NULL

  # ngrams <- text %>%
  #   drop_na(any_of(Var)) %>%
  #   unnest_tokens(ngram, !!Var, token = "ngrams", n = nword) %>%
  #   separate(.data$ngram, paste("word",1:nword,sep=""), sep = " ")
  ngrams <- text %>%
    drop_na(any_of(Var)) %>%
    unnest_tokens(ngram, !!Var, token = "ngrams", n = nword)
  ind <- which(substr(ngrams$ngram, 1, 2) %in% "__")
  ngrams$ngram[ind] <- trimws(substr(ngrams$ngram[ind], 3, nchar(ngrams$ngram[ind])))

  ngrams <- ngrams %>%
    separate(ngram, paste("word", 1:nword, sep = ""), sep = " ")



  ## come back to the original multiword format
  ngrams <- ngrams %>%
    mutate(across(paste("word", seq(1, nword), sep = ""), ~ gsub("__", "-", .))) %>%
    mutate(across(paste("word", seq(1, nword), sep = ""), ~ gsub("_", " ", .)))
  ##

  ngrams <- ngrams %>% dplyr::filter(if_all(starts_with("word"), ~ !.x %in% stopwords))

  if (isTRUE(stemming)) {
    ngrams <- ngrams %>%
      mutate(across(paste("word", 1:nword, sep = ""), ~ SnowballC::wordStem(.x, language = language)))
  }
  # filter(if_all(starts_with("word"), ~ !str_detect(.x, "\\d"))) %>%
  ngrams <- ngrams %>%
    unite(ngram, paste("word", 1:nword, sep = ""), sep = " ") %>%
    dplyr::filter(!ngram %in% custom_stopngrams) %>%
    mutate(ngram = toupper(ngram))

  # Merge synonyms in the vector synonyms
  if (length(synonyms) > 0 & is.character(synonyms)) {
    s <- strsplit(toupper(synonyms), ";")
    snew <- trimws(unlist(lapply(s, function(l) l[1])))
    sold <- (lapply(s, function(l) {
      l <- trimws(l[-1])
      # l <- paste("(?<![[^[:alnum:]]|[[:alnum:]]])",l,sep="")  ### string to make an exact matching
    }))

    for (i in 1:length(s)) {
      ngrams$ngram[ngrams$ngram %in% unlist(sold[[i]])] <- snew[i]
    }
  }

  return(ngrams)
}

# Test per termExtraction, tableTag, keywordAssoc, mergeKeywords

test_that("termExtraction estrae termini dal titolo", {
  M <- load_wos_fixture()
  M2 <- suppressWarnings(suppressMessages(
    termExtraction(M, Field = "TI", ngrams = 1, verbose = FALSE)
  ))
  expect_true("TI_TM" %in% names(M2))
  expect_true(any(!is.na(M2$TI_TM)))
})

test_that("termExtraction con stemming funziona", {
  M <- load_wos_fixture()
  M2 <- expect_no_error(
    suppressWarnings(suppressMessages(
      termExtraction(M, Field = "TI", ngrams = 1, stemming = TRUE, verbose = FALSE)
    ))
  )
  expect_true("TI_TM" %in% names(M2))
})

test_that("termExtraction con ngrams=2 funziona", {
  M <- load_wos_fixture()
  M2 <- expect_no_error(
    suppressWarnings(suppressMessages(
      termExtraction(M, Field = "TI", ngrams = 2, verbose = FALSE)
    ))
  )
  expect_true("TI_TM" %in% names(M2))
})

test_that("tableTag tabula frequenze per autori", {
  M <- load_wos_fixture()
  tab <- tableTag(M, Tag = "AU")
  expect_true(is.data.frame(tab) || inherits(tab, "table"))
  expect_true(length(tab) > 0)
})

test_that("tableTag tabula frequenze per DE e ID", {
  M <- load_wos_fixture()
  tab_de <- tableTag(M, Tag = "DE")
  tab_id <- tableTag(M, Tag = "ID")
  expect_true(length(tab_de) > 0)
  expect_true(length(tab_id) > 0)
})

test_that("keywordAssoc restituisce lista di associazioni", {
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  KW <- expect_no_error(keywordAssoc(scientometrics, sep = ";", n = 5))
  expect_type(KW, "list")
})

test_that("mergeKeywords aggiunge campo KW_Merged", {
  M <- load_wos_fixture()
  M2 <- M
  M2$KW_Merged <- NULL
  M3 <- mergeKeywords(M2)
  expect_true("KW_Merged" %in% names(M3))
})

### COMMON FUNCTIONS ####

## Count records in a bibliographic file without full conversion ----
#' @description Lightweight record counter for bibliographic files.
#'   Returns the number of publications contained in one or more files
#'   without performing the full convert2df() conversion.
#' @param file Character vector of file paths (or a single ZIP path).
#' @param dbsource Character. The database source
#'   ("isi","scopus","openalex","openalex_api","lens","cochrane","pubmed","dimensions").
#' @param format Character. The file format
#'   ("plaintext","bibtex","csv","pubmed","excel","api","endnote").
#'   If NULL, it is auto-detected from the file extension.
#' @return Integer. The total number of records across all files.
countRecords <- function(file, dbsource = "isi", format = NULL) {

  ## ---- helper: count lines matching a pattern ----
  count_lines_matching <- function(paths, pattern) {
    n <- 0L
    for (f in paths) {
      lines <- readLines(f, warn = FALSE, encoding = "UTF-8")
      n <- n + sum(grepl(pattern, lines, perl = TRUE))
    }
    n
  }

  ## ---- helper: count CSV rows (data rows only, excluding header) ----
  count_csv_rows <- function(paths) {
    n <- 0L
    for (f in paths) {
      # Count total lines minus 1 for header
      total <- length(readLines(f, warn = FALSE))
      n <- n + max(0L, total - 1L)
    }
    n
  }

  ## ---- handle ZIP: extract to temp dir ----
  paths <- file
  if (length(file) == 1 && tolower(tools::file_ext(file)) == "zip") {
    temp_exdir <- tempfile("countRecords_")
    paths <- utils::unzip(file, exdir = temp_exdir)
    on.exit(unlink(temp_exdir, recursive = TRUE), add = TRUE)
  }

  ## ---- auto-detect format from file extension if not provided ----
  if (is.null(format)) {
    ext <- tolower(tools::file_ext(paths[1]))
    format <- switch(ext,
      txt  = if (dbsource == "pubmed") "pubmed" else "plaintext",
      csv  = "csv",
      bib  = "bibtex",
      ciw  = "plaintext",
      xlsx = "excel",
      rdata = "rdata", rda = "rdata", rds = "rds",
      "plaintext"
    )
  }

  ## ---- count records by format ----
  n <- switch(format,
    plaintext = ,
    endnote = {
      if (dbsource == "cochrane") {
        count_lines_matching(paths, "^Record #")
      } else {
        # WoS: each record starts with "PT "
        count_lines_matching(paths, "^PT ")
      }
    },
    bibtex = {
      count_lines_matching(paths, "^@")
    },
    csv = {
      count_csv_rows(paths)
    },
    pubmed = {
      count_lines_matching(paths, "^PMID- ")
    },
    excel = {
      n <- 0L
      for (f in paths) {
        df <- readxl::read_excel(f, col_types = "text",
                                 range = readxl::cell_cols(1))
        n <- n + nrow(df)
      }
      n
    },
    api = ,
    rdata = {
      n <- 0L
      for (f in paths) {
        env <- new.env(parent = emptyenv())
        var <- load(f, envir = env)
        obj <- get(var[1], envir = env)
        n <- n + if (is.data.frame(obj)) nrow(obj) else length(obj)
      }
      n
    },
    rds = {
      n <- 0L
      for (f in paths) {
        obj <- readRDS(f)
        n <- n + if (is.data.frame(obj)) nrow(obj) else length(obj)
      }
      n
    },
    0L
  )

  as.integer(n)
}

## Cache helper: lightweight fingerprint of a data.frame
data_fingerprint <- function(M) {
  nr <- nrow(M)
  if (nr == 0) return("empty")
  paste(nr, ncol(M),
        M$SR[1], M$SR[nr],
        sep = "||")
}

## Cache helper: build a cache key from a named list of params
make_cache_key <- function(...) {
  args <- list(...)
  lapply(args, function(x) {
    if (is.null(x)) "<<NULL>>" else x
  })
}

# Funzione helper aggiornata con titolo e affiliazione
createAuthorCard <- function(
  name,
  title,
  affiliation,
  url,
  photo,
  scholar = FALSE
) {
  tags$a(
    href = url,
    target = "_blank",
    style = "text-decoration: none;",
    div(
      style = "text-align: center; transition: transform 0.3s; cursor: pointer; padding: 15px; background: #f9f9f9; border-radius: 10px;",
      onmouseover = "this.style.transform='translateY(-5px)'; this.style.boxShadow='0 8px 20px rgba(0,0,0,0.15)';",
      onmouseout = "this.style.transform='translateY(0)'; this.style.boxShadow='none';",

      # Foto circolare con bordo
      div(
        style = sprintf(
          "width: 110px; height: 110px; border-radius: 50%%; background-image: url('%s'); background-size: cover; background-position: center; margin: 0 auto 15px; border: 4px solid #3c8dbc; box-shadow: 0 4px 15px rgba(0,0,0,0.2);",
          photo
        )
      ),

      # Nome
      div(
        name,
        style = "font-weight: bold; color: #2c3e50; font-size: 16px; margin-bottom: 8px;"
      ),

      # Titolo
      div(
        title,
        style = "color: #7f8c8d; font-size: 13px; font-weight: 500; margin-bottom: 5px;"
      ),

      # Affiliazione
      div(
        icon("university", style = "margin-right: 5px;"),
        affiliation,
        style = "color: #95a5a6; font-size: 12px; line-height: 1.4; margin-bottom: 10px;"
      ),

      # Link icon
      div(
        if (scholar) {
          tagList(
            icon("graduation-cap", style = "margin-right: 3px;"),
            "Google Scholar"
          )
        } else {
          tagList(
            icon("link", style = "margin-right: 3px;"),
            "Website"
          )
        },
        style = "color: #3c8dbc; font-size: 12px; font-weight: 600;"
      )
    )
  )
}


# Scroll to Top Button (Font Awesome version)
scrollToTopButton <- function() {
  tags$div(
    # CSS per il pulsante
    tags$head(
      tags$style(HTML(
        "
        #scrollToTopBtn {
          display: none;
          position: fixed;
          bottom: 20px;
          right: 20px;
          z-index: 99999;
          border: none;
          outline: none;
          background-color: rgba(68, 68, 68, 0.7);
          color: white;
          cursor: pointer;
          padding: 12px;
          border-radius: 4px;
          font-size: 16px;
          width: 45px;
          height: 45px;
          box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
          transition: all 0.3s ease;
          backdrop-filter: blur(4px);
        }

        #scrollToTopBtn:hover {
          background-color: rgba(29, 143, 225, 0.85);
          transform: translateY(-3px);
          box-shadow: 0 4px 10px rgba(0, 0, 0, 0.4);
        }

        #scrollToTopBtn:active {
          transform: translateY(-1px);
          box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
        }

        #scrollToTopBtn i {
          margin: 0;
          padding: 0;
          line-height: 1;
        }
      "
      ))
    ),

    # Il pulsante HTML
    tags$button(
      id = "scrollToTopBtn",
      icon("arrow-up", lib = "glyphicon"),
      onclick = "scrollToTop()",
      title = "Back to top"
    ),

    # JavaScript per gestire lo scroll
    tags$script(HTML(
      "
      // Funzione per mostrare/nascondere il pulsante
      window.onscroll = function() {scrollFunction()};

      function scrollFunction() {
        var btn = document.getElementById('scrollToTopBtn');
        if (document.body.scrollTop > 100 || document.documentElement.scrollTop > 100) {
          btn.style.display = 'block';
        } else {
          btn.style.display = 'none';
        }
      }

      // Funzione per scrollare in alto con animazione smooth
      function scrollToTop() {
        window.scrollTo({
          top: 0,
          behavior: 'smooth'
        });
      }
    "
    ))
  )
}

# Number format abbreviated
format_abbreviated <- function(x) {
  if (is.na(x)) {
    return("--")
  }
  if (x >= 1e6) {
    return(paste0(format(round(x / 1e6, 2), nsmall = 2), "M"))
  } else if (x >= 1e3) {
    return(paste0(format(round(x / 1e3, 0), nsmall = 0), "K"))
  } else {
    return(as.character(x))
  }
}

# total package download
total_downloads <- function(
  pkg_name = "bibliometrix",
  from = "2016-01-01",
  to = Sys.Date()
) {
  # Function to get total downloads of a package from CRAN logs
  # Args:
  #   pkg_name: Name of the package as a string
  # Returns:
  #   Total number of downloads as an integer

  if (!is_Online()) {
    return(NA)
  }

  #today <- Sys.Date()
  if (!is.character(pkg_name) || length(pkg_name) != 1) {
    stop("pkg_name must be a single string.")
  }

  url <- paste0(
    "https://cranlogs.r-pkg.org/downloads/total/",
    from,
    ":",
    to,
    "/",
    pkg_name
  )

  if (!check_online(host = url, timeout = 1, method = "http")) {
    return(NA)
  }

  json_text <- tryCatch(
    {
      readLines(url, warn = FALSE)
    },
    error = function(e) {
      return(NA)
    }
  )

  # Se già nel tryCatch è tornato "NA", esci subito
  if (identical(json_text, "NA")) {
    return(NA)
  }

  # Extract the number manually (not robust)
  txt <- unlist(strsplit(json_text, ","))
  txt <- txt[grepl("downloads", txt)]

  if (length(txt) == 0) {
    return(NA)
  }

  downloads <- gsub("[^0-9]", "", txt)

  return(as.integer(downloads))
}

# getFileNameExtension <- function(fn) {
#   # remove a path
#   splitted <- strsplit(x = fn, split = "/")[[1]]
#   # or use .Platform$file.sep in stead of '/'
#   fn <- splitted[length(splitted)]
#   ext <- ""
#   splitted <- strsplit(x = fn, split = "\\.")[[1]]
#   l <- length(splitted)
#   if (l > 1 && sum(splitted[1:(l - 1)] != "")) {
#     ext <- splitted[l]
#   }
#   # the extention must be the suffix of a non-empty name
#   ext
# }

# FILTER FUNCTIONS ----
read_journal_ranking <- function(file_path) {
  ext <- tools::file_ext(file_path)

  suppressMessages(
    journals <- switch(
      tolower(ext),
      "csv" = read.csv(file_path, header = TRUE, stringsAsFactors = FALSE),
      "xlsx" = {
        readxl::read_excel(file_path, col_names = TRUE)
      },
      stop(
        "Unsupported file format. Please upload a .csv, .txt, or .xlsx file."
      )
    )
  )
  journals <- journals %>% select(1, 2)
  # journals <- journals[!is.na(journals)]
  # journals <- toupper(trimws(journals))
  names(journals) <- c("SO", "Ranking")
  journals <- journals %>%
    mutate(SO = toupper(trimws(SO)))
  return(journals)
}

read_journal_list <- function(file_path) {
  ext <- tools::file_ext(file_path)

  suppressMessages(
    journals <- switch(
      tolower(ext),
      "csv" = read.csv(file_path, header = FALSE, stringsAsFactors = FALSE)[[
        1
      ]],
      "txt" = readLines(file_path, warn = FALSE),
      "xlsx" = {
        readxl::read_excel(file_path, col_names = FALSE)[[1]]
      },
      stop(
        "Unsupported file format. Please upload a .csv, .txt, or .xlsx file."
      )
    )
  )

  journals <- journals[!is.na(journals)]
  journals <- toupper(trimws(journals))
  return(journals)
}

wcTable <- function(M) {
  # Function to extract Science Category (WC) information from metadata
  if ("WC" %in% names(M)) {
    WC <- strsplit(M$WC, ";")
    df <- data.frame(
      SR = rep(M$SR, lengths(WC)),
      WC = unlist(WC),
      stringsAsFactors = FALSE
    )

    df$WC <- trimws(df$WC) # Remove leading and trailing whitespace
  } else {
    df <- data.frame(SR = M$SR, WC = "N.A.", stringsAsFactors = FALSE)
  }

  return(df)
}

countryTable <- function(M) {
  data("countries", envir = environment())
  # Function to extract country information from metadata
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, "AU_CO")
  }

  CO <- strsplit(M$AU_CO, ";")
  df <- data.frame(
    SR = rep(M$SR, lengths(CO)),
    CO = trimws(unlist(CO)),
    stringsAsFactors = FALSE
  )

  df$CO <- gsub("[[:digit:]]", "", df$CO)
  df$CO <- gsub(".", "", df$CO, fixed = TRUE)
  df$CO <- gsub(";;", ";", df$CO, fixed = TRUE)
  df$CO <- gsub("UNITED STATES", "USA", df$CO)
  df$CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", df$CO)
  df$CO <- gsub("TAIWAN", "CHINA", df$CO)
  df$CO <- gsub("ENGLAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("SCOTLAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("WALES", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", df$CO)
  df$CO <- gsub("UK", "UNITED KINGDOM", df$CO)
  #df$CO <- gsub("KOREA", "SOUTH KOREA", df$CO)

  df <- df %>%
    left_join(
      countries %>% select(countries, continent),
      by = c("CO" = "countries")
    ) %>%
    mutate(CO = ifelse(is.na(CO), "Unknown", CO)) %>%
    mutate(continent = ifelse(is.na(continent), "Unknown", continent)) %>%
    mutate(CO = ifelse(CO == "UNKNOWN", "Unknown", CO))
}

# LOAD FUNCTIONS -----

formatDB <- function(obj) {
  ext <- sub(".*\\.", "", obj[1])
  switch(
    ext,
    txt = {
      format <- "plaintext"
    },
    csv = {
      format <- "csv"
    },
    bib = {
      format <- "bibtex"
    },
    ciw = {
      format <- "endnote"
    },
    xlsx = {
      format <- "excel"
    }
  )
  return(format)
}

## smart_load function ----
smart_load <- function(file) {
  var <- load(file)
  n <- length(var)
  if (!"M" %in% var) {
    if (n == 1) {
      eval(parse(text = paste0("M <- ", var)))
    } else {
      stop("I could not find bibliometrixDB object in your data file: ", file)
    }
  }
  rm(list = var[var != "M"])
  if (("M" %in% ls()) & inherits(M, "bibliometrixDB")) {
    return(M)
  } else {
    stop(
      "Please make sure your RData/Rda file contains a bibliometrixDB object (M)."
    )
  }
}


## merge collections ----
merge_files <- function(files) {
  ## load xlsx or rdata bibliometrix files
  if ("datapath" %in% names(files)) {
    file <- files$datapath
    ext <- unlist(lapply(file, tools::file_ext))
  }

  Mfile <- list()
  n <- 0
  for (i in 1:length(file)) {
    extF <- ext[i]
    filename <- file[i]

    switch(
      tolower(extF),
      xlsx = {
        Mfile[[i]] <- readxl::read_excel(filename, col_types = "text") %>%
          as.data.frame()
        Mfile[[i]]$PY <- as.numeric(Mfile[[i]]$PY)
        Mfile[[i]]$TC <- as.numeric(Mfile[[i]]$TC)
      },
      rdata = {
        Mfile[[i]] <- smart_load(filename)
      }
    )
    n <- n + nrow(Mfile[[i]])
  }

  # merge bibliometrix files
  M <- mergeDbSources(Mfile, remove.duplicated = T)

  # save original size as attribute
  attr(M, "nMerge") <- n

  return(M)
}

## dynamic watch emoji icons ---
watchEmoji <- function(i) {
  emoji <- c(
    "🕐",
    "🕑",
    "🕒",
    "🕓",
    "🕔",
    "🕕",
    "🕖",
    "🕗",
    "🕘",
    "🕙",
    "🕚",
    "🕛"
  )
  # i is a positive int number, reduce it to an int from 1 to 12
  multiple <- floor(i / 12)
  if (multiple > 0) {
    i <- i %% (12 * multiple)
    if (i == 0) {
      i <- 12
    }
  }
  emoji[i]
}

## Author Name Format (Fullname or Surname, N.) ----
authorNameFormat <- function(M, format) {
  if (format == "AF" & "AF" %in% names(M)) {
    M <- M %>%
      rename(
        AU_IN = AU,
        AU = AF
      )
  }
  return(M)
}

split_text_numbers <- function(input_str, UT) {
  # Split the string into components based on "; "
  components <- unlist(strsplit(input_str, "; ", fixed = TRUE))

  # Initialize two vectors to store the separated parts
  texts <- character(length(components))
  numbers <- numeric(length(components))

  # Iterate through each component to separate text and numbers
  for (i in seq_along(components)) {
    # Extract the text using regex, matching everything up to " ("
    texts[i] <- gsub("\\s\\(.*$", "", components[i])

    # Extract the numbers using regex, matching digits inside parentheses
    numbers[i] <- as.numeric(gsub(".*\\((\\d+)\\).*", "\\1", components[i]))
  }

  # Return a list with texts and numbers separated
  data.frame(Texts = texts, Numbers = numbers, UT = UT)
}


AuthorNameMerge <- function(M) {
  df_list <- list()
  for (i in 1:nrow(M)) {
    if (nchar(M$AU[i]) > 0) {
      df_list[[i]] <- split_text_numbers(M$AU[i], M$UT[i])
    }
  }

  df <- do.call(rbind, df_list)

  AU <- df %>%
    group_by(Numbers, Texts) %>%
    count() %>%
    group_by(Numbers) %>%
    arrange(desc(n)) %>%
    mutate(AU = Texts[1]) %>%
    select(-"n", -"Texts") %>%
    ungroup() %>%
    distinct()

  df <- df %>%
    left_join(AU, by = "Numbers") %>%
    group_by(UT) %>%
    summarize(
      AU = paste0(AU, collapse = ";"),
      AU_ID = paste0(Numbers, collapse = ";")
    )

  M <- M %>%
    rename(AU_original = AU) %>%
    left_join(df, by = "UT")
  return(M)
}

## RESET MODAL DIALOG INPUTS
resetModalButtons <- function(session) {
  session$sendCustomMessage("button_id", "null")
  session$sendCustomMessage("button_id2", "null")
  # session$sendCustomMessage("click", "null")
  # session$sendCustomMessage("click-dend", "null")
  # runjs("Shiny.setInputValue('plotly_click-A', null);")
  return(session)
}

### LIFE CYCLE PLOTLY FUNCTION ----

#' Plot Life Cycle Analysis Results with ggplot2
#'
#' Creates ggplot2 plots from lifeCycle analysis results
#'
#' @param results Output from lifeCycle() function
#' @param plot_type Character, either "annual" or "cumulative" to specify which plot to generate
#'
#' @return A ggplot2 object
#'
#' @export
ggplotLifeCycle <- function(results, plot_type = c("annual", "cumulative")) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required but not installed.")
  }

  plot_type <- match.arg(plot_type)

  # Extract data
  complete_curve <- results$complete_curve
  observed_data <- results$data
  params <- results$parameters_real_years
  metrics <- results$metrics
  K <- params$K
  tm_year <- params$tm_year
  R2 <- metrics$R_squared

  if (plot_type == "annual") {
    # Plot 1: Annual Publications
    max_annual <- max(complete_curve$annual, na.rm = TRUE)

    p <- ggplot() +
      geom_line(
        data = complete_curve,
        aes(x = year, y = annual),
        color = "blue",
        linewidth = 1
      ) +
      geom_point(
        data = observed_data,
        aes(x = PY, y = n),
        color = "blue",
        size = 3
      ) +
      geom_vline(
        xintercept = tm_year,
        linetype = "dashed",
        color = "red",
        linewidth = 0.7
      ) +
      annotate(
        "text",
        x = tm_year,
        y = max_annual * 0.95,
        label = sprintf("Peak: %.1f", tm_year),
        hjust = -0.1,
        color = "red",
        size = 3.5
      ) +
      annotate(
        "text",
        x = max(complete_curve$year),
        y = max_annual * 1.05,
        label = sprintf("R² = %.3f", R2),
        hjust = 1,
        size = 4
      ) +
      labs(
        title = "Life Cycle - Annual Publications",
        x = "Year",
        y = "Publications (Annual)"
      ) +
      scale_y_continuous(limits = c(0, max_annual * 1.1)) +
      theme_minimal(base_size = 12) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14),
        panel.grid.minor = element_blank()
      )

    return(p)
  } else if (plot_type == "cumulative") {
    # Plot 2: Cumulative Publications
    reference_lines <- data.frame(
      y = c(K * 0.5, K * 0.9, K * 0.99),
      label = c("50%", "90%", "99%")
    )

    p <- ggplot() +
      geom_line(
        data = complete_curve,
        aes(x = year, y = cumulative),
        color = "darkgreen",
        linewidth = 1
      ) +
      geom_point(
        data = observed_data,
        aes(x = PY, y = cumulative),
        color = "darkgreen",
        size = 3
      ) +
      geom_hline(
        data = reference_lines,
        aes(yintercept = y),
        linetype = "dotted",
        color = "gray50",
        linewidth = 0.5
      ) +
      geom_text(
        data = reference_lines,
        aes(x = min(complete_curve$year), y = y, label = label),
        hjust = 0,
        vjust = -0.5,
        size = 3,
        color = "gray50"
      ) +
      labs(
        title = "Cumulative Growth Curve",
        x = "Year",
        y = "Cumulative Publications"
      ) +
      scale_y_continuous(limits = c(0, K * 1.05)) +
      theme_minimal(base_size = 12) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14),
        panel.grid.minor = element_blank()
      )

    return(p)
  }
}

#' Plot Life Cycle Analysis with Plotly
#'
#' Creates interactive plotly visualizations from lifeCycle results
#' for use in biblioshiny
#'
#' @param results Output from lifeCycle() function
#' @param plot_type Character: "annual" or "cumulative" (default: "annual")
#'
#' @return A plotly object
#'
plotLifeCycle <- function(results, plot_type = c("annual", "cumulative")) {
  if (!requireNamespace("plotly", quietly = TRUE)) {
    stop(
      "Package 'plotly' is required. Please install it with: install.packages('plotly')"
    )
  }

  plot_type <- match.arg(plot_type)

  # Extract data
  df <- results$data
  complete <- results$complete_curve
  params <- results$parameters_real_years
  metrics <- results$metrics

  # Separate observed and forecast
  last_obs_year <- max(df$PY)
  observed <- complete[complete$year <= last_obs_year, ]
  forecast <- complete[complete$year > last_obs_year, ]

  if (plot_type == "annual") {
    # === ANNUAL PUBLICATIONS PLOT ===

    p <- plotly::plot_ly()

    # Observed curve (historical fit)
    p <- p %>%
      plotly::add_trace(
        data = observed,
        x = ~year,
        y = ~annual,
        type = 'scatter',
        mode = 'lines',
        name = 'Logistic fit',
        line = list(color = '#1f77b4', width = 2),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Annual:</b> %{y:.0f}<br>',
          '<extra></extra>'
        )
      )

    # Forecast curve
    if (nrow(forecast) > 0) {
      p <- p %>%
        plotly::add_trace(
          data = forecast,
          x = ~year,
          y = ~annual,
          type = 'scatter',
          mode = 'lines',
          name = 'Forecast',
          line = list(color = '#1f77b4', width = 2, dash = 'dash'),
          hovertemplate = paste0(
            '<b>Year:</b> %{x}<br>',
            '<b>Projected:</b> %{y:.0f}<br>',
            '<extra></extra>'
          )
        )
    }

    # Observed data points
    p <- p %>%
      plotly::add_trace(
        data = df,
        x = ~PY,
        y = ~n,
        type = 'scatter',
        mode = 'markers',
        name = 'Observed',
        marker = list(color = '#1f77b4', size = 8),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Publications:</b> %{y}<br>',
          '<extra></extra>'
        )
      )

    # Peak year line
    p <- p %>%
      plotly::add_trace(
        x = c(params$tm_year, params$tm_year),
        y = c(0, max(complete$annual) * 1.1),
        type = 'scatter',
        mode = 'lines',
        name = paste0('Peak year (', round(params$tm_year, 1), ')'),
        line = list(color = 'red', width = 1.5, dash = 'dash'),
        hoverinfo = 'name'
      )

    # Layout
    p <- p %>%
      plotly::layout(
        title = list(
          text = sprintf(
            "Life Cycle - Annual Publications<br><sup>R² = %.3f | Peak = %.0f publications in %.1f</sup>",
            metrics$R_squared,
            params$peak_annual,
            params$tm_year
          ),
          x = 0.5,
          xanchor = 'center'
        ),
        xaxis = list(
          title = "Year",
          showgrid = FALSE
        ),
        yaxis = list(
          title = "Publications (Annual)",
          showgrid = TRUE,
          gridcolor = '#f0f0f0',
          rangemode = 'tozero'
        ),
        hovermode = 'closest',
        showlegend = TRUE,
        legend = list(
          x = 0.02,
          y = 0.98,
          bgcolor = 'rgba(255, 255, 255, 0.8)',
          bordercolor = '#ddd',
          borderwidth = 1
        ),
        plot_bgcolor = 'white',
        paper_bgcolor = 'white'
      )
  } else {
    # === CUMULATIVE PUBLICATIONS PLOT ===

    K <- params$K

    p <- plotly::plot_ly()

    # Observed curve
    p <- p %>%
      plotly::add_trace(
        data = observed,
        x = ~year,
        y = ~cumulative,
        type = 'scatter',
        mode = 'lines',
        name = 'Logistic fit',
        line = list(color = '#2ca02c', width = 2),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Cumulative:</b> %{y:.0f}<br>',
          '<b>% of K:</b> %{customdata:.1f}%<br>',
          '<extra></extra>'
        ),
        customdata = ~ (cumulative / K * 100)
      )

    # Forecast curve
    if (nrow(forecast) > 0) {
      p <- p %>%
        plotly::add_trace(
          data = forecast,
          x = ~year,
          y = ~cumulative,
          type = 'scatter',
          mode = 'lines',
          name = 'Forecast',
          line = list(color = '#2ca02c', width = 2, dash = 'dash'),
          hovertemplate = paste0(
            '<b>Year:</b> %{x}<br>',
            '<b>Projected:</b> %{y:.0f}<br>',
            '<b>% of K:</b> %{customdata:.1f}%<br>',
            '<extra></extra>'
          ),
          customdata = ~ (cumulative / K * 100)
        )
    }

    # Observed data points
    p <- p %>%
      plotly::add_trace(
        data = df,
        x = ~PY,
        y = ~cumulative,
        type = 'scatter',
        mode = 'markers',
        name = 'Observed',
        marker = list(color = '#2ca02c', size = 8),
        hovertemplate = paste0(
          '<b>Year:</b> %{x}<br>',
          '<b>Cumulative:</b> %{y:.0f}<br>',
          '<b>% of K:</b> %{customdata:.1f}%<br>',
          '<extra></extra>'
        ),
        customdata = ~ (cumulative / K * 100)
      )

    # Reference lines (50%, 90%, 99%)
    ref_levels <- data.frame(
      level = c(0.5, 0.9, 0.99),
      label = c("50%", "90%", "99%")
    )

    for (i in 1:nrow(ref_levels)) {
      p <- p %>%
        plotly::add_trace(
          x = c(min(complete$year), max(complete$year)),
          y = c(K * ref_levels$level[i], K * ref_levels$level[i]),
          type = 'scatter',
          mode = 'lines',
          name = ref_levels$label[i],
          line = list(color = 'gray', width = 1, dash = 'dot'),
          hovertemplate = paste0(
            '<b>',
            ref_levels$label[i],
            ' of K</b><br>',
            '<b>Value:</b> ',
            round(K * ref_levels$level[i]),
            '<br>',
            '<extra></extra>'
          ),
          showlegend = FALSE
        )

      # Add annotations
      p <- p %>%
        plotly::add_annotations(
          x = min(complete$year),
          y = K * ref_levels$level[i],
          text = ref_levels$label[i],
          xanchor = 'left',
          yanchor = 'middle',
          showarrow = FALSE,
          font = list(size = 10, color = 'gray')
        )
    }

    # Layout
    p <- p %>%
      plotly::layout(
        title = list(
          text = sprintf(
            "Cumulative Growth Curve<br><sup>Saturation (K) = %.0f publications</sup>",
            K
          ),
          x = 0.5,
          xanchor = 'center'
        ),
        xaxis = list(
          title = "Year",
          showgrid = FALSE
        ),
        yaxis = list(
          title = "Cumulative Publications",
          showgrid = TRUE,
          gridcolor = '#f0f0f0',
          rangemode = 'tozero',
          range = c(0, K * 1.05)
        ),
        hovermode = 'closest',
        showlegend = TRUE,
        legend = list(
          x = 0.02,
          y = 0.98,
          bgcolor = 'rgba(255, 255, 255, 0.8)',
          bordercolor = '#ddd',
          borderwidth = 1
        ),
        plot_bgcolor = 'white',
        paper_bgcolor = 'white'
      )
  }

  return(p)
}

### AUTHOR BIO SKETCH ----

#### GLOBAL PROFILE ----
# Function to get all unique authors from papers with valid DOI only
get_all_authors <- function(df, separator = ";") {
  authors_column <- df$AU
  doi_column <- if ("DI" %in% names(df)) df$DI else NULL

  # If doi_column is provided, filter authors based on valid DOI
  if (!is.null(doi_column)) {
    # Identify entries with valid DOI (not NA and not "<NA>")
    valid_doi <- !is.na(doi_column) & doi_column != "<NA>" & doi_column != ""

    # Filter authors column based on valid DOI
    authors_column <- authors_column[valid_doi]
  }

  # Handle NA and empty strings in authors column
  valid_entries <- authors_column[!is.na(authors_column) & authors_column != ""]

  # Return empty character vector if no valid entries
  if (length(valid_entries) == 0) {
    return(character(0))
  }

  # Split all author strings using the specified separator
  all_authors <- unlist(strsplit(valid_entries, separator, fixed = TRUE))

  # Remove leading and trailing whitespace from each author name
  all_authors_clean <- trimws(all_authors)

  # Remove empty elements that might result from splitting
  all_authors_clean <- all_authors_clean[all_authors_clean != ""]

  # Remove [ANONYMOUS] entries
  all_authors_clean <- all_authors_clean[all_authors_clean != "[ANONYMOUS]"]

  # Return unique authors only
  return(unique(all_authors_clean %>% sort()))
}


authorCard <- function(selected_author, values) {
  req(selected_author)
  works_exact <- findAuthorWorks(
    selected_author,
    values$M,
    exact_match = TRUE
  ) %>%
    filter(!is.na(doi))
  if (nrow(works_exact) == 0) {
    return(HTML("No works found for this author.", type = "error"))
  }
  author_position <- works_exact$author_position[1]
  doi <- works_exact$doi[1]
  on_line <- check_online(method = "http")

  if (on_line) {
    if (!is.null(values$author_data)) {
      author_data <- values$author_data
    } else {
      author_data <- tibble(AUid = character(), display_name = character())
    }

    if (selected_author %in% author_data$AUid) {
      AU_data <- author_data %>% dplyr::filter(AUid == selected_author)
    } else {
      suppressWarnings(
        AU_data <- tryCatch(
          {
            oa_email <- Sys.getenv("openalexR.mailto", unset = "")
            authorBio(author_position = author_position, doi = doi,
                      mailto = if (nchar(oa_email) > 0) oa_email else NULL)
          },
          error = function(e) {
            NULL
          }
        )
      )
      # check if AUid is a tibble
      if (is.data.frame(AU_data)) {
        author_data <- bind_rows(
          author_data,
          AU_data %>%
            mutate(AUid = selected_author)
        )
        values$author_data <- author_data
      } else {
        return(HTML("No author data found.", type = "error"))
      }
    }
    # values$author_data <- author_data
    card <- create_author_bio_card(
      AU_data,
      width = "100%",
      show_trends = TRUE,
      show_topics = TRUE,
      max_topics = 20
    )
  } else {
    card <- HTML(
      "No internet connection. Unable to fetch author data.",
      type = "error"
    )
  }
}

create_empty_author_bio_card <- function(
  author_name = "Author Name",
  width = "100%",
  message = "Author data not found or not yet retrieved"
) {
  # Metrics cards with placeholder values
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "2yr Mean Cit.",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Empty trend chart placeholder
  trend_chart <- tagList(
    h4(
      "Publication Trends (Last 10 Years)",
      style = "margin-top: 20px; color: #95A5A6;"
    ),
    div(
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 display: flex; align-items: center; justify-content: center;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$i(
          class = "fa fa-chart-bar",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No trend data available"
      )
    )
  )

  # Empty topics section
  topics_section <- tagList(
    h4("Main Research Topics", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "topics-container",
      style = "text-align: center; padding: 20px; 
                                            background: #f8f9fa; border-radius: 8px;",
      div(
        style = "color: #95A5A6;",
        tags$i(
          class = "fa fa-tags",
          style = "font-size: 36px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No research topics available"
      )
    )
  )

  # Main card UI
  div(
    class = "author-bio-card-empty",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.1); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;
                   opacity: 0.8; border: 2px dashed #bdc3c7;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      fluidRow(
        column(
          8,
          h2(
            author_name,
            style = "margin: 0 0 10px 0; color: #95A5A6; font-weight: 600;"
          ),
          h5(
            "Institution not available",
            style = "margin: 0 0 5px 0; color: #BDC3C7; font-weight: 400;"
          ),
          p(
            "📍 Country not available",
            style = "margin: 0 0 10px 0; color: #BDC3C7;"
          )
        ),
        column(
          4,
          div(
            style = "text-align: right; padding-top: 10px;",
            div(
              tags$span("ORCID not available", style = "color: #95A5A6;"),
              style = "margin-bottom: 8px;"
            ),
            div(tags$span(
              "OpenAlex Profile not available",
              style = "color: #95A5A6;"
            ))
          )
        )
      )
    ),

    # Info message
    div(
      style = "margin: 20px 0; padding: 15px; background: #fff3cd; border: 1px solid #ffeeba; 
                 border-radius: 8px; color: #856404;",
      tags$i(class = "fa fa-info-circle", style = "margin-right: 8px;"),
      strong("Information: "),
      message
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #95A5A6;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(6, strong("i10-Index: "), span("--", style = "color: #95A5A6;")),
        column(
          6,
          strong("OpenAlex ID: "),
          span(
            "Not available",
            style = "color: #95A5A6; font-family: monospace; font-size: 12px;"
          )
        )
      )
    ),

    # Trends and topics placeholders
    trend_chart,
    topics_section,

    # Footer
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Please retrieve author data to view bibliometric information -",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

create_author_bio_card <- function(
  author_data,
  width = "100%",
  show_trends = TRUE,
  show_topics = TRUE,
  max_topics = 5
) {
  # Extract key information safely
  author_name <- author_data$display_name[1] %||% "Unknown Author"
  institution <- author_data$primary_affiliation[1] %||%
    author_data$last_known_institutions[[1]]$display_name[1] %||%
    "Institution not available"
  institution_ror <- author_data$primary_affiliation_ror[1] %||%
    author_data$last_known_institutions[[1]]$ror[1] %||%
    NA
  country <- author_data$primary_affiliation_country[1] %||%
    author_data$last_known_institutions[[1]]$country_code[1] %||%
    "Country not available"

  works_count <- author_data$works_count[1] %||% 0
  citations <- author_data$cited_by_count[1] %||% 0
  h_index <- author_data$h_index[1] %||% 0
  i10_index <- author_data$i10_index[1] %||% 0
  mean_citedness <- author_data$`2yr_mean_citedness`[1] %||% 0

  orcid <- author_data$orcid[1]
  position_type <- author_data$author_position_type[1] %||% "author"
  is_corresponding <- author_data$is_corresponding[1] %||% FALSE

  # Create ORCID link if available
  orcid_link <- if (!is.null(orcid) && !is.na(orcid)) {
    tags$a(
      href = orcid,
      target = "_blank",
      tags$img(src = "ORCID.jpg", style = "height: 16px; margin-right: 5px;"),
      "ORCID Profile",
      style = "text-decoration: none; color: #338B13;"
    )
  } else {
    tags$span("ORCID not available", style = "color: #666;")
  }

  # Create OpenAlex link
  openalex_id <- gsub("https://openalex.org/", "", author_data$id[1])
  openalex_link <- tags$a(
    href = paste0("https://openalex.org/", openalex_id),
    target = "_blank",
    tags$img(src = "openalex.jpg", style = "height: 16px; margin-right: 5px;"),
    "OpenAlex Profile",
    style = "text-decoration: none; color: #E74C3C;"
  )

  # Format numbers with thousands separator
  format_number <- function(x) {
    if (is.null(x) || is.na(x)) {
      return("0")
    }
    format(x, big.mark = ",", scientific = FALSE)
  }

  # Create metrics cards
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(works_count), style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #f093fb 0%, #f5576c 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(citations), style = "margin: 0; font-size: 24px;"),
        p(
          "Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #4facfe 0%, #00f2fe 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(h_index, style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #43e97b 0%, #38f9d7 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(round(mean_citedness, 1), style = "margin: 0; font-size: 24px;"),
        p(
          "2yr Mean Cit.",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Create publication trend chart if requested
  trend_chart <- if (show_trends && !is.null(author_data$counts_by_year[[1]])) {
    counts_data <- author_data$counts_by_year[[1]]
    recent_data <- counts_data[
      counts_data$year >= (max(counts_data$year) - 9),
    ]
    # Order by year from oldest to newest
    recent_data <- recent_data[order(recent_data$year), ]

    tagList(
      h4(
        "Publication Trends (Last 10 Years)",
        style = "margin-top: 20px; color: #2C3E50;"
      ),
      div(
        style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px;",
        # Simple bar chart representation
        div(
          style = "display: flex; align-items: end; height: 170px; gap: 3px;",
          lapply(1:nrow(recent_data), function(i) {
            height_pct <- (recent_data$works_count[i] /
              max(recent_data$works_count, na.rm = TRUE)) *
              100
            div(
              style = paste0(
                "background: linear-gradient(to top, #3498db, #2980b9); 
                                   height: ",
                height_pct,
                "%; 
                                   width: ",
                100 / nrow(recent_data) - 1,
                "%; 
                                   border-radius: 3px 3px 0 0;
                                   position: relative;"
              ),
              div(
                style = "position: absolute; bottom: -20px; font-size: 10px; 
                               width: 100%; text-align: center; color: #666;",
                recent_data$year[i]
              ),
              div(
                style = "position: absolute; top: -15px; font-size: 9px; 
                               width: 100%; text-align: center; color: #333; font-weight: bold;",
                recent_data$works_count[i]
              )
            )
          })
        )
      )
    )
  } else {
    NULL
  }

  # Create research topics section if requested
  topics_section <- if (show_topics && !is.null(author_data$topics[[1]])) {
    topics_data <- author_data$topics[[1]]
    top_topics <- topics_data[topics_data$type == "topic", ][
      1:min(max_topics, sum(topics_data$type == "topic")),
    ] %>%
      slice_sample(prop = 1)

    # Calculate font sizes based on counts
    if (nrow(top_topics) > 0) {
      min_count <- min(top_topics$count, na.rm = TRUE)
      max_count <- max(top_topics$count, na.rm = TRUE)
      min_font_size <- 10
      max_font_size <- 18

      # Calculate proportional font sizes
      font_sizes <- if (max_count == min_count) {
        rep(min_font_size, nrow(top_topics))
      } else {
        min_font_size +
          (top_topics$count - min_count) /
            (max_count - min_count) *
            (max_font_size - min_font_size)
      }
    }

    tagList(
      h4("Main Research Topics", style = "margin-top: 20px; color: #2C3E50;"),
      div(
        class = "topics-container",
        style = "text-align: center;",
        lapply(1:nrow(top_topics), function(i) {
          if (!is.na(top_topics$display_name[i]) && !is.na(top_topics$id[i])) {
            tags$a(
              href = top_topics$id[i],
              target = "_blank",
              class = "topic-badge",
              style = paste0(
                "display: inline-block; background: ",
                colorlist()[i],
                # sample(
                # c("#3498db", "#e74c3c", "#2ecc71", "#f39c12", "#9b59b6"),
                # 1),
                "; opacity: 0.7; color: white; padding: 5px 10px; margin: 3px; 
                            border-radius: 15px; font-weight: 500; text-decoration: none;
                            font-size: ",
                round(font_sizes[i], 1),
                "px;"
              ),
              paste0(top_topics$display_name[i], " (", top_topics$count[i], ")")
            )
          }
        })
      )
    )
  } else {
    NULL
  }

  institution_link <- tags$a(
    href = institution_ror,
    target = "_blank",
    institution
    # ,style = "text-decoration: none; color: #E74C3C;"
  )

  # Main card UI
  div(
    class = "author-bio-card",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.15); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      fluidRow(
        column(
          8,
          h2(
            author_name,
            style = "margin: 0 0 10px 0; color: #2C3E50; font-weight: 600;"
          ),
          h5(
            institution_link,
            style = "margin: 0 0 5px 0; color: #7F8C8D; font-weight: 400;"
          ),
          p(paste("📍", country), style = "margin: 0 0 10px 0; color: #95A5A6;")
        ),
        column(
          4,
          div(
            style = "text-align: right; padding-top: 10px;",
            div(orcid_link, style = "margin-bottom: 8px;"),
            div(openalex_link)
          )
        )
      )
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #2C3E50;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(
          6,
          strong("i10-Index: "),
          span(i10_index, style = "color: #2980b9;")
        ),
        column(
          6,
          strong("OpenAlex ID: "),
          span(
            openalex_id,
            style = "color: #666; font-family: monospace; font-size: 12px;"
          )
        )
      )
    ),

    # Trends and topics
    trend_chart,
    topics_section,

    # Footer with source information
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Data retrieved from OpenAlex on",
        format(author_data$query_timestamp[1], "%Y-%m-%d %H:%M")
      )
    )
  )
}

# Wrapper function for use in Shiny renderUI

render_author_bio_card <- function(author_data, ...) {
  renderUI({
    create_author_bio_card(author_data, ...)
  })
}

#### LOCAL PROFILE ----
create_local_author_bio_card <- function(
  local_author_data,
  selected_author,
  max_py = 2024,
  width = "100%",
  show_trends = TRUE,
  show_keywords = TRUE,
  max_keywords = 8,
  max_works_display = 100
) {
  # Extract author information
  author_name <- to_title_case(selected_author)

  # Calculate metrics from local data
  works_count <- nrow(local_author_data)
  total_citations <- sum(as.numeric(local_author_data$TC), na.rm = TRUE)
  years <- as.numeric(local_author_data$PY)
  current_year <- as.numeric(format(Sys.Date(), "%Y"))

  # Calculate mean citations per year (weighted by age of publications)
  publication_ages <- current_year - years
  publication_ages[publication_ages <= 0] <- 1 # Avoid division by zero
  mean_citations_per_year <- mean(
    as.numeric(local_author_data$TC) / publication_ages,
    na.rm = TRUE
  )

  # Calculate additional metrics
  years_active <- max(years, na.rm = TRUE) - min(years, na.rm = TRUE) + 1
  avg_citations_per_work <- total_citations / works_count

  # Calculate h-index (simplified version)
  citations_sorted <- sort(as.numeric(local_author_data$TC), decreasing = TRUE)
  h_index <- sum(citations_sorted >= seq_along(citations_sorted))

  # Get most recent years for activity
  recent_years <- years[years >= (max_py - 4)]
  recent_productivity <- length(recent_years)

  # Format numbers with thousands separator
  format_number <- function(x) {
    if (is.null(x) || is.na(x)) {
      return("0")
    }
    format(round(x), big.mark = ",", scientific = FALSE)
  }

  # Create metrics cards
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(format_number(works_count), style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #f093fb 0%, #f5576c 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(
          format_number(total_citations),
          style = "margin: 0; font-size: 24px;"
        ),
        p(
          "Total Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #4facfe 0%, #00f2fe 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(h_index, style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index (Local)",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #43e97b 0%, #38f9d7 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center;",
        h4(
          round(mean_citations_per_year, 1),
          style = "margin: 0; font-size: 24px;"
        ),
        p(
          "Cit./Year",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Create publication trend chart if requested
  trend_chart <- if (show_trends && length(years) > 0) {
    # Create yearly publication counts
    year_counts <- table(years)
    year_df <- data.frame(
      year = as.numeric(names(year_counts)),
      count = as.numeric(year_counts)
    )
    # Get recent 10 years
    recent_years_range <- max(year_df$year) - 9
    recent_data <- year_df[year_df$year >= recent_years_range, ]
    # Fill missing years with 0
    all_years <- recent_years_range:max(year_df$year)
    complete_data <- data.frame(year = all_years)
    complete_data <- merge(complete_data, recent_data, all = TRUE)
    complete_data$count[is.na(complete_data$count)] <- 0
    complete_data <- complete_data[order(complete_data$year), ]

    if (nrow(complete_data) > 0) {
      tagList(
        h4(
          "Publication Trends (Last 10 Years)",
          style = "margin-top: 20px; color: #2C3E50;"
        ),
        div(
          style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px;",
          div(
            style = "display: flex; align-items: end; height: 170px; gap: 3px;",
            lapply(1:nrow(complete_data), function(i) {
              max_count <- max(complete_data$count, na.rm = TRUE)
              height_pct <- if (max_count > 0) {
                (complete_data$count[i] / max_count) * 100
              } else {
                0
              }
              div(
                style = paste0(
                  "background: linear-gradient(to top, #3498db, #2980b9); 
                                     height: ",
                  max(height_pct, 2),
                  "%; 
                                     width: ",
                  100 / nrow(complete_data) - 1,
                  "%; 
                                     border-radius: 3px 3px 0 0;
                                     position: relative;"
                ),
                div(
                  style = "position: absolute; bottom: -20px; font-size: 10px; 
                                 width: 100%; text-align: center; color: #666;",
                  complete_data$year[i]
                ),
                div(
                  style = "position: absolute; top: -15px; font-size: 9px; 
                                 width: 100%; text-align: center; color: #333; font-weight: bold;",
                  complete_data$count[i]
                )
              )
            })
          )
        )
      )
    }
  } else {
    NULL
  }

  # Create keywords section if requested
  keywords_section <- if (show_keywords && "DE" %in% names(local_author_data)) {
    # Extract and process keywords
    all_keywords <- unlist(strsplit(
      paste(local_author_data$DE, collapse = ";"),
      ";"
    ))
    all_keywords <- to_title_case(trimws(toupper(all_keywords)))
    all_keywords <- all_keywords[all_keywords != "" & !is.na(all_keywords)]

    if (length(all_keywords) > 0) {
      keyword_freq <- sort(table(all_keywords), decreasing = TRUE)
      top_keywords <- head(keyword_freq, max_keywords)
      top_keywords <- top_keywords[sample(length(top_keywords))] # Shuffle order

      # Calculate font sizes based on frequency
      if (length(top_keywords) > 0) {
        min_freq <- min(top_keywords)
        max_freq <- max(top_keywords)
        min_font_size <- 10
        max_font_size <- 18

        font_sizes <- if (max_freq == min_freq) {
          rep(min_font_size, length(top_keywords))
        } else {
          min_font_size +
            (top_keywords - min_freq) /
              (max_freq - min_freq) *
              (max_font_size - min_font_size)
        }
      }

      tagList(
        h4("Main Keywords", style = "margin-top: 20px; color: #2C3E50;"),
        div(
          class = "keywords-container",
          style = "text-align: center;",
          lapply(1:length(top_keywords), function(i) {
            colors <- c(
              "#3498db",
              "#e74c3c",
              "#2ecc71",
              "#f39c12",
              "#9b59b6",
              "#1abc9c",
              "#34495e",
              "#e67e22"
            )
            color <- colors[((i - 1) %% length(colors)) + 1]

            tags$span(
              class = "keyword-badge",
              style = paste0(
                "display: inline-block; background: ",
                color,
                "; opacity: 0.8; color: white; padding: 5px 12px; margin: 3px; 
                            border-radius: 15px; font-weight: 500;
                            font-size: ",
                round(font_sizes[i], 1),
                "px;"
              ),
              paste0(names(top_keywords)[i], " (", top_keywords[i], ")")
            )
          })
        )
      )
    }
  } else {
    NULL
  }

  # Create works list (scrollable)
  works_section <- if (nrow(local_author_data) > 0) {
    # Prepare works data
    works_to_show <- head(
      local_author_data[
        order(as.numeric(local_author_data$PY), decreasing = TRUE),
      ],
      max_works_display
    )

    works_list <- lapply(1:nrow(works_to_show), function(i) {
      work <- works_to_show[i, ]
      title <- work$TI %||% "Title not available"
      journal <- work$SO %||% "Journal not available"
      year <- work$PY %||% "Year not available"
      doi <- work$DI %||% ""
      citations <- as.numeric(work$TC) %||% 0

      # Create DOI link if available
      doi_link <- if (!is.null(doi) && !is.na(doi) && doi != "") {
        tags$a(
          href = paste0("https://doi.org/", doi),
          target = "_blank",
          style = "color: #3498db; text-decoration: none; font-size: 11px;",
          paste0("DOI: ", doi)
        )
      } else {
        tags$span(
          "DOI not available",
          style = "color: #95a5a6; font-size: 11px;"
        )
      }

      div(
        class = "work-item",
        style = "border-bottom: 1px solid #ecf0f1; padding: 12px 0; margin: 0;",
        div(
          style = "margin-bottom: 8px;",
          tags$h6(
            title,
            style = "margin: 0 0 5px 0; color: #2c3e50; font-weight: 600; line-height: 1.3;"
          )
        ),
        div(
          style = "margin-bottom: 8px;",
          tags$span(
            journal,
            style = "color: #7f8c8d; font-style: italic; font-size: 13px; margin-right: 15px;"
          ),
          tags$span(
            paste0("(", year, ")"),
            style = "color: #95a5a6; font-size: 13px; margin-right: 15px;"
          ),
          tags$span(
            paste0("Citations: ", format_number(citations)),
            style = "color: #e74c3c; font-weight: 500; font-size: 12px;"
          )
        ),
        div(style = "margin-top: 5px;", doi_link)
      )
    })

    tagList(
      h4(
        paste0("Publications (", nrow(local_author_data), " total)"),
        style = "margin-top: 20px; color: #2C3E50;"
      ),
      div(
        class = "works-container",
        style = "max-height: 400px; overflow-y: auto; background: #f8f9fa; 
                   border-radius: 8px; padding: 15px; border: 1px solid #e9ecef;",
        works_list
      )
    )
  } else {
    NULL
  }

  # Main card UI
  div(
    class = "local-author-bio-card",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.15); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      h2(
        author_name,
        style = "margin: 0 0 10px 0; color: #2C3E50; font-weight: 600;"
      ),
      p(
        "📊 Local Collection Profile",
        style = "margin: 0 0 10px 0; color: #7F8C8D; font-style: italic;"
      )
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Local Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #2C3E50;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px;",
      fluidRow(
        column(
          4,
          strong("Years Active: "),
          span(years_active, style = "color: #2980b9;")
        ),
        column(
          4,
          strong("Avg Cit./Work: "),
          span(round(avg_citations_per_work, 1), style = "color: #27ae60;")
        ),
        column(
          4,
          strong("Recent Activity (5yr): "),
          span(recent_productivity, style = "color: #e74c3c;")
        )
      )
    ),

    # Trends and keywords
    trend_chart,
    keywords_section,

    # Works section
    works_section,

    # Footer with source information
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Local collection data analyzed on",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

create_empty_local_author_bio_card <- function(
  author_name = "Author Name Not Available",
  width = "100%",
  message = "Local author data not found or not yet processed"
) {
  # Metrics cards with placeholder values
  metrics_cards <- fluidRow(
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Publications",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Total Citations",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "H-Index (Local)",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    ),
    column(
      3,
      div(
        class = "metric-card",
        style = "background: linear-gradient(135deg, #bdc3c7 0%, #95a5a6 100%); 
                        color: white; padding: 15px; border-radius: 8px; text-align: center; opacity: 0.7;",
        h4("--", style = "margin: 0; font-size: 24px;"),
        p(
          "Cit./Year",
          style = "margin: 5px 0 0 0; font-size: 12px; opacity: 0.9;"
        )
      )
    )
  )

  # Empty trend chart placeholder
  trend_chart <- tagList(
    h4(
      "Publication Trends (Last 10 Years)",
      style = "margin-top: 20px; color: #95A5A6;"
    ),
    div(
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 display: flex; align-items: center; justify-content: center; border: 1px solid #e9ecef;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$div(
          "📊",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No trend data available"
      )
    )
  )

  # Empty keywords section
  keywords_section <- tagList(
    h4("Main Keywords", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "keywords-container",
      style = "text-align: center; padding: 20px; 
                                             background: #f8f9fa; border-radius: 8px; border: 1px solid #e9ecef;",
      div(
        style = "color: #95A5A6;",
        tags$div(
          "🏷️",
          style = "font-size: 36px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No keywords available"
      )
    )
  )

  # Empty works section
  works_section <- tagList(
    h4("Publications (0 total)", style = "margin-top: 20px; color: #95A5A6;"),
    div(
      class = "works-container",
      style = "height: 200px; background: #f8f9fa; border-radius: 8px; padding: 15px; 
                 border: 1px solid #e9ecef; display: flex; align-items: center; justify-content: center;",
      div(
        style = "text-align: center; color: #95A5A6;",
        tags$div(
          "📄",
          style = "font-size: 48px; margin-bottom: 10px; opacity: 0.3;"
        ),
        br(),
        "No publications available",
        br(),
        tags$small(
          "Publications will appear here when local data is processed",
          style = "font-style: italic; opacity: 0.7;"
        )
      )
    )
  )

  # Main card UI
  div(
    class = "local-author-bio-card-empty",
    style = paste0(
      "width: ",
      width,
      "; background: white; 
                   border-radius: 12px; box-shadow: 0 4px 12px rgba(0,0,0,0.1); 
                   padding: 25px; margin: 15px 0; font-family: 'Segoe UI', Tahoma, sans-serif;
                   opacity: 0.8; border: 2px dashed #bdc3c7;"
    ),

    # Header section
    div(
      class = "author-header",
      style = "border-bottom: 2px solid #ecf0f1; padding-bottom: 20px;",
      h2(
        author_name,
        style = "margin: 0 0 10px 0; color: #95A5A6; font-weight: 600;"
      ),
      p(
        "📊 Local Collection Profile",
        style = "margin: 0 0 10px 0; color: #BDC3C7; font-style: italic;"
      )
    ),

    # Info message
    div(
      style = "margin: 20px 0; padding: 15px; background: #fff3cd; border: 1px solid #ffeeba; 
                 border-radius: 8px; color: #856404;",
      tags$span("ℹ️", style = "margin-right: 8px; font-size: 16px;"),
      strong("Information: "),
      message
    ),

    # Metrics section
    div(
      style = "margin: 20px 0;",
      h4(
        "Local Bibliometric Indicators",
        style = "margin-bottom: 15px; color: #95A5A6;"
      ),
      metrics_cards
    ),

    # Additional metrics
    div(
      style = "margin: 15px 0; padding: 15px; background: #f8f9fa; border-radius: 8px; border: 1px solid #e9ecef;",
      fluidRow(
        column(
          4,
          strong("Years Active: "),
          span("--", style = "color: #95A5A6;")
        ),
        column(
          4,
          strong("Avg Cit./Work: "),
          span("--", style = "color: #95A5A6;")
        ),
        column(
          4,
          strong("Recent Activity (5yr): "),
          span("--", style = "color: #95A5A6;")
        )
      )
    ),

    # Trends, keywords, and works placeholders
    trend_chart,
    keywords_section,
    works_section,

    # Footer
    div(
      style = "margin-top: 25px; padding-top: 15px; border-top: 1px solid #ecf0f1; 
                 font-size: 11px; color: #95A5A6; text-align: center;",
      paste(
        "Please process local collection data to view bibliometric information -",
        format(Sys.time(), "%Y-%m-%d %H:%M")
      )
    )
  )
}

# Helper function for safe extraction (null coalescing operator)
`%||%` <- function(x, y) if (is.null(x) || is.na(x) || length(x) == 0) y else x

# Helper: return a working PNG device function.
#
# `capabilities()` reports what R was COMPILED with, not what currently works
# on the host. Two real-world cases that bit users:
#   * Windows ARM64: capabilities("png") == TRUE but grDevices::png() fails.
#   * macOS without XQuartz on R >= 4.6: capabilities("cairo") == TRUE but
#     png(type = "cairo") aborts because libXrender is missing.
#
# We therefore PROBE each candidate by opening a 1x1 PNG to a tempfile and
# checking that a non-empty file was actually written. The first candidate
# that survives the probe is cached for the session.
.safe_png_device_cache <- new.env(parent = emptyenv())
safe_png_device <- function(force = FALSE) {
  if (!force && !is.null(.safe_png_device_cache$dev)) {
    return(.safe_png_device_cache$dev)
  }

  # Build candidate list in order of preference.
  candidates <- list()

  if (requireNamespace("ragg", quietly = TRUE)) {
    candidates[["ragg"]] <- ragg::agg_png
  }

  is_macos <- (Sys.info()[["sysname"]] == "Darwin")
  if (is_macos && capabilities("aqua")) {
    candidates[["quartz"]] <- function(filename, width, height,
                                       res = 72, units = "in", ...) {
      grDevices::png(filename = filename, width = width, height = height,
                     res = res, units = units, type = "quartz", ...)
    }
  }

  if (capabilities("png")) {
    # Default (system) png(): "quartz" on aqua macOS, "cairo" if linked, "Xlib"
    # on plain X11. R picks whichever was compiled in.
    candidates[["default"]] <- grDevices::png
  }

  if (capabilities("cairo")) {
    candidates[["cairo"]] <- function(filename, width, height,
                                      res = 72, units = "in", ...) {
      grDevices::png(filename = filename, width = width, height = height,
                     res = res, units = units, type = "cairo", ...)
    }
  }

  if (length(candidates) == 0) {
    stop("No PNG graphics device available. Install the 'ragg' package: install.packages('ragg')")
  }

  for (name in names(candidates)) {
    fn <- candidates[[name]]
    test_file <- tempfile(fileext = ".png")
    ok <- tryCatch({
      fn(filename = test_file, width = 2, height = 2, res = 72, units = "in")
      # quartz/ragg only flush a file when something is actually drawn.
      grid::grid.rect(gp = grid::gpar(col = NA, fill = "white"))
      grDevices::dev.off()
      file.exists(test_file) && file.info(test_file)$size > 0
    }, error = function(e) {
      try(grDevices::dev.off(), silent = TRUE)
      FALSE
    })
    unlink(test_file)
    if (isTRUE(ok)) {
      .safe_png_device_cache$dev <- fn
      .safe_png_device_cache$name <- name
      return(fn)
    }
  }

  stop("No working PNG graphics device available. Install the 'ragg' package: install.packages('ragg')")
}

# Safe wrapper for ggsave that uses a working PNG device
safe_ggsave <- function(..., device = NULL) {
  args <- list(...)
  filename <- args$filename %||% args[[1]]
  if (is.null(device) && grepl("\\.png$", filename, ignore.case = TRUE)) {
    device <- safe_png_device()
  }
  ggplot2::ggsave(..., device = device)
}


# from igraph to png file
igraph2PNG <- function(x, filename, width = 10, height = 7, dpi = 75) {
  V(x)$centr <- centr_betw(x)$res
  df <- data.frame(
    name = V(x)$label,
    cluster = V(x)$color,
    centr = V(x)$centr
  ) %>%
    group_by(cluster) %>%
    slice_head(n = 3)
  V(x)$label[!(V(x)$label %in% df$name)] <- ""
  png_dev <- safe_png_device()
  png_dev(
    filename = filename,
    width = width,
    height = height,
    units = "in",
    res = dpi
  )
  grid::grid.draw(plot(x))
  dev.off()
}

# from ggplot to plotly
plot.ly <- function(
  g,
  flip = FALSE,
  side = "r",
  aspectratio = 1,
  size = 0.15,
  data.type = 2,
  height = 0,
  customdata = NA
) {
  g <- g + labs(title = NULL)

  gg <- ggplotly(g, tooltip = "text") %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        "toImage",
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    )

  return(gg)
}

freqPlot <- function(
  xx,
  x,
  y,
  textLaby,
  textLabx,
  title,
  values,
  string.max = 70
) {
  xl <- c(
    max(xx[, x]) - 0.02 - diff(range(xx[, x])) * 0.125,
    max(xx[, x]) - 0.02
  ) +
    1
  yl <- c(1, 1 + length(unique(xx[, y])) * 0.125)

  Text <- paste(textLaby, ": ", xx[, y], "\n", textLabx, ": ", xx[, x])

  if (title == "Most Local Cited References" & values$M$DB[1] == "SCOPUS") {
    xx[, y] <- gsub(
      "^(.+?)\\.,.*\\((\\d{4})\\)$",
      paste0("\\1", "., ", "\\2"),
      xx[, y]
    )
  }

  xx[, y] <- sapply(substr(xx[, y], 1, string.max), function(s) {
    if (nchar(s) > 40) paste0(stringr::str_wrap(s, width = 40)) else s
  })

  # Ensure unique labels to avoid multiple points on same y-axis row
  dupes <- duplicated(xx[, y]) | duplicated(xx[, y], fromLast = TRUE)
  if (any(dupes)) {
    counts <- ave(seq_along(xx[, y]), xx[, y], FUN = seq_along)
    xx[dupes, y] <- paste0(xx[dupes, y], " (", counts[dupes], ")")
  }

  g <- ggplot(xx, aes(x = xx[, x], y = xx[, y], label = xx[, x], text = Text)) +
    geom_segment(
      aes(x = 0, y = xx[, y], xend = xx[, x], yend = xx[, y]),
      color = "grey50"
    ) +
    geom_point(aes(color = -xx[, x], size = xx[, x]), show.legend = FALSE) +
    scale_radius(range = c(5, 12)) +
    geom_text(color = "#333333", size = 3.5, fontface = "bold", hjust = 0,
              nudge_x = max(xx[, x]) * 0.04) +
    scale_y_discrete(limits = rev(xx[, y])) +
    scale_x_continuous(expand = expansion(mult = c(0, 0.15))) +
    scale_fill_continuous(type = "gradient") +
    labs(title = title, y = textLaby) +
    labs(x = textLabx) +
    expand_limits(y = c(1, length(xx[, y]) + 1)) +
    theme_minimal() +
    theme(axis.text.y = element_text(angle = 0, hjust = 1, face = "bold", size = 11)) +
    annotation_custom(
      values$logoGrid,
      xmin = xl[1],
      xmax = xl[2],
      ymin = yl[1],
      ymax = yl[2]
    )

  return(g)
}

emptyPlot <- function(errortext) {
  g <- ggplot() +
    theme_void() +
    theme(legend.position = "none") +
    annotate("text", x = 4, y = 25, label = errortext, size = 10)
  plot(g)
}

count.duplicates <- function(DF) {
  x <- do.call("paste", c(DF, sep = "\r"))
  ox <- order(x)
  rl <- rle(x[ox])
  cbind(DF[ox[cumsum(rl$lengths)], , drop = FALSE], count = rl$lengths)
}

reduceRefs <- function(A) {
  ind <- unlist(regexec("*V[0-9]", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  ind <- unlist(regexec("*DOI ", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  return(A)
}

check_online <- function(
  host = "8.8.8.8",
  timeout = 5,
  # min_success = 1,
  method = "http" # method = c("ping", "socket", "http")
) {
  #method <- match.arg(method)

  if (method == "ping") {
    # Usa solo il codice di ritorno, non analizza l'output
    ping_cmd <- if (.Platform$OS.type == "windows") {
      sprintf("ping -n 1 -w %d %s", timeout * 1000, host)
    } else {
      sprintf("ping -c 1 -W %d %s", timeout, host)
    }
    exit_code <- suppressWarnings(system(
      ping_cmd,
      ignore.stdout = TRUE,
      ignore.stderr = TRUE
    ))
    return(exit_code == 0)
  } else if (method == "socket") {
    # Connessione TCP a DNS Google (porta 53)
    tryCatch(
      {
        con <- socketConnection(
          host = host,
          port = 53,
          blocking = TRUE,
          open = "r+",
          timeout = timeout
        )
        close(con)
        return(TRUE)
      },
      error = function(e) {
        return(FALSE)
      }
    )
  } else if (method == "http") {
    # Richiesta HTTP
    tryCatch(
      {
        # check if host start with http or https and add if missing
        if (!grepl("^https?://", host)) {
          host <- paste0("https://", host)
        }

        # con <- url("https://www.google.com", open = "rb")
        con <- url(host, open = "rb")
        on.exit(close(con))
        readLines(con, n = 1, warn = FALSE)
        return(TRUE)
      },
      error = function(e) {
        return(FALSE)
      }
    )
  }
}

# check_online <- function(host = "8.8.8.8", min_success = 1) {
#   # Use ping command to test connectivity (works on Windows, Linux, Mac)
#   ping_cmd <- if (.Platform$OS.type == "windows") {
#     sprintf("ping -n %d %s", min_success, host)
#   } else {
#     sprintf("ping -c %d %s", min_success, host)
#   }
#
#   result <- suppressWarnings(system(
#     ping_cmd,
#     intern = TRUE,
#     ignore.stderr = TRUE
#   ))
#
#   success <- any(grepl("time=", result, ignore.case = TRUE))
#
#   if (success) {
#     # message("✅ Host is reachable.")
#     # Extract average latency (optional)
#     latency_line <- result[grepl("time=", result)]
#     times <- as.numeric(sub(".*time=([0-9.]+).*", "\\1", latency_line))
#     avg_time <- mean(times, na.rm = TRUE)
#     # message(sprintf("📶 Average latency: %.1f ms", avg_time))
#     if (avg_time < 200) {
#       return(TRUE)
#     } else {
#       return(FALSE)
#     }
#     #return(TRUE)
#   } else {
#     #message(FALSE)
#     return(FALSE)
#   }
# }

notifications <- function() {
  ## check connection and download notifications
  online <- check_online(host = "www.bibliometrix.org", method = "http")
  location <- "https://www.bibliometrix.org/bs_notifications/biblioshiny_notifications.csv"
  notifOnline <- NULL
  if (isTRUE(online)) {
    notifOnline <- read.csv(location, header = TRUE, sep = ",")
    # ## add check to avoid blocked app when internet connection is to slow

    if (is.null(notifOnline)) {
      online <- FALSE
    } else {
      notifOnline$href[nchar(notifOnline$href) < 6] <- NA
    }
  }

  ## check if a file exists on the local machine and load it
  home <- homeFolder()

  file <- file.path(home, "biblioshiny_notifications.csv")
  fileTrue <- file.exists(file)
  if (isTRUE(fileTrue)) {
    suppressWarnings(notifLocal <- read.csv(file, header = TRUE, sep = ","))
  }

  A <- c("noA", "A")
  B <- c("noB", "B")
  status <- paste(A[online + 1], B[fileTrue + 1], sep = "")

  switch(
    status,
    # missing both files (online and local)
    noAnoB = {
      notifTot <- data.frame(
        nots = "No notifications",
        href = NA,
        status = "info"
      ) %>%
        mutate(status = "info")
    },
    # missing online file. The local one exists.
    noAB = {
      notifTot <- notifLocal %>%
        filter(action == TRUE) %>%
        mutate(status = "info")
    },
    # missing the local file. The online one exists.
    AnoB = {
      # notifOnline <- notifOnline %>%
      #   dplyr::slice_head(n = 5)
      notifTot <- notifOnline %>%
        dplyr::filter(action == TRUE) %>%
        mutate(status = "danger") %>%
        dplyr::slice_head(n = 5)
      notifOnline %>%
        dplyr::filter(action == TRUE) %>%
        write.csv(file = file, quote = FALSE, row.names = FALSE)
    },
    # both files exist.
    AB = {
      notifTot <- left_join(
        notifOnline %>% mutate(status = "danger"),
        notifLocal %>% mutate(status = "info"),
        by = "nots"
      ) %>%
        mutate(status = tidyr::replace_na(status.y, "danger")) %>%
        rename(
          href = href.x,
          action = action.x
        ) %>%
        select(nots, href, action, status) %>%
        arrange(status) %>%
        dplyr::filter(action == TRUE) %>%
        dplyr::slice_head(n = 5)
      notifTot %>%
        select(-status) %>%
        write.csv(file = file, quote = FALSE, row.names = FALSE)
    }
  )

  return(notifTot)
}

is_Online <- function(timeout = 3, url = "https://www.bibliometrix.org") {
  #RCurl::url.exists(url, timeout = timeout)
  check_online(host = url, timeout = timeout, method = "http")
}

initial <- function(values) {
  values$results <- list("NA")
  values$log <- "working..."
  values$load <- "FALSE"
  values$field <- values$cocngrams <- "NA"
  values$citField <- values$colField <- values$citSep <- "NA"
  values$NetWords <- values$NetRefs <- values$ColNetRefs <- matrix(NA, 1, 1)
  values$Title <- "Network"
  values$Histfield <- "NA"
  values$histlog <- "working..."
  values$kk <- 0
  values$histsearch <- "NA"
  values$citShortlabel <- "NA"
  values$S <- list("NA")
  values$GR <- "NA"
  values$nMerge <- NULL
  values$collection_description <- NULL
  ### column to export in TALL
  values$corpusCol <- c(
    "Title" = "TI",
    "Abstract" = "AB",
    "Author's Keywords" = "DE"
  )
  values$metadataCol <- c(
    "Publication Year" = "PY",
    "Document Type" = "DT",
    "DOI" = "DI",
    "Open Access" = "OA",
    "Language" = "LA",
    "First Author" = "AU1"
  )

  # Chrome enviroment variable
  if (inherits(try(pagedown::find_chrome(), silent = T), "try-error")) {
    values$Chrome_url <- NULL
  } else {
    values$Chrome_url <- pagedown::find_chrome()
  }

  return(values)
}

resetAnalysis <- function() {
  # reset menus
  output$lifeCycleSummaryUIid <- renderUI({
    div()
  })
  output$DLCPlotYear <- renderUI({
    div()
  })

  output$DLCPlotCum <- renderUI({
    div()
  })
}

### TALL Export functions ----
tallExport <- function(M, tallFields, tallMetadata, metadataCol) {
  corpus <- NULL
  ## Corpus Fields ##
  if ("Abstract" %in% tallFields) {
    if (!"AB_raw" %in% names(M)) {
      M <- M %>%
        mutate(AB_raw = sapply(AB, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "AB_raw")
  }

  if ("Title" %in% tallFields) {
    if (!"TI_raw" %in% names(M)) {
      M <- M %>%
        mutate(TI_raw = sapply(TI, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "TI_raw")
  }

  if ("Author's Keywords" %in% tallFields) {
    if (!"DE_raw" %in% names(M)) {
      M <- M %>%
        mutate(DE_raw = sapply(DE, capitalize_after_dot, USE.NAMES = FALSE))
    }
    corpus <- c(corpus, "DE_raw")
  }

  corpus <- c(corpus, as.character(metadataCol[tallMetadata]))

  M <- M %>%
    select(SR, any_of(corpus)) %>%
    rename(doc_id = SR)
  names(M) <- gsub("_raw", "", names(M))

  return(M)
}

capitalize_after_dot <- function(text) {
  # Tutto minuscolo
  text <- tolower(text)

  # Prima lettera della stringa maiuscola
  text <- paste0(toupper(substr(text, 1, 1)), substr(text, 2, nchar(text)))

  # Maiuscola dopo punto (o ! o ?) + spazio
  text <- gsub("([\\.\\!\\?]\\s*)([a-z])", "\\1\\U\\2", text, perl = TRUE)

  return(text)
}


### ANALYSIS FUNCTIONS ####
### Descriptive functions ----

ValueBoxes <- function(M) {
  # calculate statistics for Biblioshiny ValueBoxes
  df <- data.frame(Description = rep(NA, 12), Results = rep(NA, 12))

  ## VB  1 - Time span
  df[1, ] <- c("Timespan", paste(range(M$PY, na.rm = T), collapse = ":"))

  ## VB  2 - Authors
  listAU <- (strsplit(M$AU, ";"))
  nAU <- lengths(listAU)
  listAU <- unique(trimws((unlist(listAU))))
  listAU <- listAU[!is.na(listAU)]
  df[2, ] <- c("Authors", length(listAU))

  ## VB  3 - Author's Keywords (DE)
  if (!"DE" %in% names(M)) {
    M$DE <- ""
  }
  DE <- unique(trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$DE, ";")))))
  DE <- DE[!is.na(DE)]

  df[3, ] <- c("Author's Keywords (DE)", length(DE))

  ## VB  4 - Sources
  df[4, ] <- c("Sources (Journals, Books, etc)", length(unique(M$SO)))

  ## VB  5 - Authors of single-authored docs

  df[5, ] <- c(
    "Authors of single-authored docs",
    length(unique(M$AU[nAU == 1]))
  )

  ## VB  6 - References
  CR <- trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$CR, ";"))))
  CR <- CR[nchar(CR) > 0 & !is.na(CR)]
  nCR <- length(unique(CR))
  if (nCR == 1) {
    nCR <- 0
  }
  df[6, ] <- c("References", nCR)

  ## VB  7 - Documents
  df[7, ] <- c("Documents", nrow(M))

  ## VB  8 - International Co-Authorship
  if (!"AU_CO" %in% names(M)) {
    M <- metaTagExtraction(M, "AU_CO")
  }
  AU_CO <- strsplit(M$AU_CO, ";")
  Coll <- unlist(lapply(AU_CO, function(l) {
    length(unique(l)) > 1
  }))
  Coll <- sum(Coll) / nrow(M) * 100
  df[8, ] <- c("International co-authorships %", format(Coll, digits = 4))

  ## VB  9 - Document Average Age
  age <- as.numeric(substr(Sys.Date(), 1, 4)) - M$PY
  df[9, ] <- c(
    "Document Average Age",
    format(mean(age, na.rm = TRUE), digits = 3)
  )

  ## VB 10 - Annual Growth Rate
  Y <- table(M$PY)
  ny <- diff(range(M$PY, na.rm = TRUE))
  CAGR <- as.numeric(round(((Y[length(Y)] / Y[1])^(1 / (ny)) - 1) * 100, 2))
  df[10, ] <- c("Annual Growth Rate %", CAGR)

  ## VB 11 - Co-Authors per Doc
  df[11, ] <- c("Co-Authors per Doc", format(mean(nAU, na.rm = T), digit = 3))

  ## VB 12 - Average citations per doc
  df[12, ] <- c(
    "Average citations per doc",
    format(mean(M$TC, na.rm = T), digit = 4)
  )

  DT <- M %>%
    mutate(DT = tolower(DT)) %>%
    count(DT) %>%
    rename(
      Description = DT,
      Results = n
    )

  # Indexed Keywords (ID)
  ID <- unique(trimws(gsub("\\s+|\\.|\\,", " ", unlist(strsplit(M$ID, ";")))))
  ID <- ID[!is.na(ID)]
  df[nrow(df) + 1, ] <- c("Keywords Plus (ID)", length(ID))

  # Single authored docs

  df[nrow(df) + 1, ] <- c("Single-authored docs", sum(nAU == 1))

  df2 <- data.frame(
    Description = c(
      "MAIN INFORMATION ABOUT DATA",
      "Timespan",
      "Sources (Journals, Books, etc)",
      "Documents",
      "Annual Growth Rate %",
      "Document Average Age",
      "Average citations per doc",
      "References",
      "DOCUMENT CONTENTS",
      "Keywords Plus (ID)",
      "Author's Keywords (DE)",
      "AUTHORS",
      "Authors",
      "Authors of single-authored docs",
      "AUTHORS COLLABORATION",
      "Single-authored docs",
      "Co-Authors per Doc",
      "International co-authorships %",
      "DOCUMENT TYPES"
    )
  )

  df <- left_join(df2, df, by = "Description") %>%
    rbind(DT) %>%
    mutate(Results = replace_na(Results, ""))

  return(df)
}

countryCollab <- function(M) {
  sep <- ";"
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU_CO", sep)
  }
  if (!("AU1_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU1_CO", sep)
  }

  M$nCO <- as.numeric(unlist(lapply(strsplit(M$AU_CO, ";"), function(l) {
    length(unique(l)) > 1
  })))

  M$AU1_CO <- trim(gsub("[[:digit:]]", "", M$AU1_CO))
  M$AU1_CO <- gsub("UNITED STATES", "USA", M$AU1_CO)
  M$AU1_CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", M$AU1_CO)
  M$AU1_CO <- gsub("TAIWAN", "CHINA", M$AU1_CO)
  M$AU1_CO <- gsub("ENGLAND", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("SCOTLAND", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("WALES", "UNITED KINGDOM", M$AU1_CO)
  M$AU1_CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", M$AU1_CO)

  df <- M %>%
    group_by(AU1_CO) %>%
    select(AU1_CO, nCO) %>%
    summarize(
      Articles = n(),
      SCP = sum(nCO == 0),
      MCP = sum(nCO == 1)
    ) %>%
    rename(Country = AU1_CO) %>%
    arrange(desc(Articles))

  return(df)
}

Hindex_plot <- function(values, type, input) {
  hindex <- function(values, type, input) {
    switch(
      type,
      author = {
        # AU <- trim(gsub(",","",names(tableTag(values$M,"AU"))))
        values$H <- Hindex(
          values$M,
          field = "author",
          elements = NULL,
          sep = ";",
          years = Inf
        )$H %>%
          arrange(desc(h_index))
      },
      source = {
        # SO <- names(sort(table(values$M$SO),decreasing = TRUE))
        values$H <- Hindex(
          values$M,
          field = "source",
          elements = NULL,
          sep = ";",
          years = Inf
        )$H %>%
          arrange(desc(h_index))
      }
    )

    return(values)
  }

  values <- hindex(values, type = type, input)

  xx <- values$H
  if (type == "author") {
    K <- input$Hkauthor
    measure <- input$HmeasureAuthors
    title <- "Authors' Local Impact"
    xn <- "Authors"
  } else {
    K <- input$Hksource
    measure <- input$HmeasureSources
    title <- "Sources' Local Impact"
    xn <- "Sources"
  }
  if (K > dim(xx)[1]) {
    k <- dim(xx)[1]
  } else {
    k <- K
  }

  switch(
    measure,
    h = {
      m <- 2
    },
    g = {
      m <- 3
    },
    m = {
      m <- 4
      xx[, m] <- round(xx[, m], 2)
    },
    tc = {
      m <- 5
    }
  )
  xx <- xx[order(-xx[, m]), ]
  xx <- xx[1:k, c(1, m)]

  g <- freqPlot(
    xx,
    x = 2,
    y = 1,
    textLaby = xn,
    textLabx = paste("Impact Measure:", toupper(measure)),
    title = paste(title, "by", toupper(measure), "index"),
    values
  )

  res <- list(values = values, g = g)
  return(res)
}

descriptive <- function(values, type) {
  switch(
    type,
    "tab2" = {
      TAB <- values$M %>%
        group_by(PY) %>%
        count() %>%
        rename(
          Year = PY,
          Articles = n
        ) %>%
        right_join(
          data.frame(
            Year = seq(
              min(values$M$PY, na.rm = TRUE),
              max(values$M$PY, na.rm = TRUE)
            )
          ),
          by = "Year"
        ) %>%
        mutate(Articles = replace_na(Articles, 0)) %>%
        arrange(Year) %>%
        as.data.frame()

      ny <- diff(range(TAB$Year))
      values$GR <- round(
        ((TAB[nrow(TAB), 2] / TAB[1, 2])^(1 / (ny)) - 1) * 100,
        digits = 2
      )
    },
    "tab3" = {
      listAU <- (strsplit(values$M$AU, ";"))
      nAU <- lengths(listAU)
      fracAU <- rep(1 / nAU, nAU)
      TAB <- tibble(Author = unlist(listAU), fracAU = fracAU) %>%
        group_by(Author) %>%
        summarize(
          Articles = n(),
          AuthorFrac = sum(fracAU)
        ) %>%
        arrange(desc(Articles)) %>%
        as.data.frame()
      names(TAB) <- c("Authors", "Articles", "Articles Fractionalized")
      # print(S$MostProdAuthors)
    },
    "tab4" = {
      y <- as.numeric(substr(Sys.Date(), 1, 4))
      TAB <- values$M %>%
        mutate(TCperYear = TC / (y + 1 - PY)) %>%
        select(SR, DI, TC, TCperYear, PY) %>%
        group_by(PY) %>%
        mutate(NTC = TC / mean(TC)) %>%
        ungroup() %>%
        select(-PY) %>%
        arrange(desc(TC)) %>%
        as.data.frame()
      names(TAB) <- c(
        "Paper",
        "DOI",
        "Total Citations",
        "TC per Year",
        "Normalized TC"
      )
    },
    "tab5" = {
      TAB <- countryCollab(values$M)
      TAB <- TAB %>%
        mutate(Freq = Articles / sum(Articles)) %>%
        mutate(MCP_Ratio = MCP / Articles) %>%
        drop_na(Country)
    },
    "tab6" = {
      if (!"AU1_CO" %in% names(values$M)) {
        values$M <- metaTagExtraction(values$M, "AU1_CO")
      }
      TAB <- values$M %>%
        select(AU1_CO, TC) %>%
        drop_na(AU1_CO) %>%
        rename(
          Country = AU1_CO,
          TotalCitation = TC
        ) %>%
        group_by(Country) %>%
        summarise(
          "TC" = sum(TotalCitation),
          "Average Article Citations" = round(
            sum(TotalCitation) / length(TotalCitation),
            1
          )
        ) %>%
        arrange(-TC) %>%
        as.data.frame(.data)
    },
    "tab7" = {
      TAB <- values$M %>%
        select(SO) %>%
        group_by(SO) %>%
        count() %>%
        arrange(desc(n)) %>%
        rename(
          Sources = SO,
          Articles = n
        ) %>%
        as.data.frame()
    },
    "tab10" = {
      TAB <- mapworld(values$M)$tab
    },
    "tab11" = {
      if (!("AU_UN" %in% names(values$M))) {
        values$M <- metaTagExtraction(values$M, Field = "AU_UN")
      }
      TAB <- data.frame(Affiliation = unlist(strsplit(values$M$AU_UN, ";"))) %>%
        group_by(Affiliation) %>%
        count() %>%
        drop_na(Affiliation) %>%
        arrange(desc(n)) %>%
        rename(Articles = n) %>%
        filter(Affiliation != "NA") %>%
        as.data.frame()
    },
    "tab12" = {
      TAB <- tableTag(values$M, "C1")
      TAB <- data.frame(Affiliations = names(TAB), Articles = as.numeric(TAB))
      TAB <- TAB[nchar(TAB[, 1]) > 4, ]
      # names(TAB)=c("Affiliations", "Articles")
    },
    "tab13" = {
      CR <- localCitations(values$M, fast.search = FALSE, verbose = FALSE)
      TAB <- CR$Authors
      # TAB=data.frame(Authors=names(CR$Authors$Author), Citations=as.numeric(CR$Cited))
    }
  )
  values$TAB <- TAB
  res <- list(values = values, TAB = TAB)
  return(res)
}

AffiliationOverTime <- function(values, n) {
  if (!("AU_UN" %in% names(values$M))) {
    values$M <- metaTagExtraction(values$M, Field = "AU_UN")
  }
  AFF <- strsplit(values$M$AU_UN, ";")
  nAFF <- lengths(AFF)

  AFFY <- data.frame(
    Affiliation = unlist(AFF),
    Year = rep(values$M$PY, nAFF)
  ) %>%
    filter(Affiliation != "NA") %>%
    drop_na(Affiliation, Year) %>%
    group_by(Affiliation, Year) %>%
    count() %>%
    group_by(Affiliation) %>%
    arrange(Year) %>%
    ungroup() %>%
    pivot_wider(Affiliation, names_from = Year, values_from = n) %>%
    mutate(across(everything(), ~ replace(., is.na(.), 0))) %>%
    pivot_longer(
      cols = !Affiliation,
      names_to = "Year",
      values_to = "Articles"
    ) %>%
    group_by(Affiliation) %>%
    mutate(Articles = cumsum(Articles))

  Affselected <- AFFY %>%
    filter(Year == max(Year)) %>%
    ungroup() %>%
    slice_max(Articles, n = n)

  values$AffOverTime <- AFFY %>%
    filter(Affiliation %in% Affselected$Affiliation) %>%
    mutate(Year = Year %>% as.numeric())

  Text <- paste(
    values$AffOverTime$Affiliation,
    " (",
    values$AffOverTime$Year,
    ") ",
    values$AffOverTime$Articles,
    sep = ""
  )
  x <- c(
    min(values$AffOverTime$Year) + 0.02,
    min(values$AffOverTime$Year) + 0.02 +
      diff(range(values$AffOverTime$Year)) * 0.15
  )
  y <- c(
    max(values$AffOverTime$Articles) -
      diff(range(values$AffOverTime$Articles)) * 0.15,
    max(values$AffOverTime$Articles)
  )

  # Reorder legend by descending max cumulative value
  values$AffOverTime$Affiliation <- factor(values$AffOverTime$Affiliation,
    levels = names(sort(tapply(values$AffOverTime$Articles, values$AffOverTime$Affiliation, max), decreasing = TRUE)))

  # Truncate long affiliation names for legend readability
  trunc_aff_labels <- setNames(
    ifelse(nchar(levels(values$AffOverTime$Affiliation)) > 35,
           paste0(substr(levels(values$AffOverTime$Affiliation), 1, 32), "..."),
           levels(values$AffOverTime$Affiliation)),
    levels(values$AffOverTime$Affiliation)
  )

  values$AffOverTimePlot <- ggplot(
    values$AffOverTime,
    aes(
      x = Year,
      y = Articles,
      group = Affiliation,
      color = Affiliation,
      text = Text
    )
  ) +
    geom_line() +
    labs(
      x = "Year",
      y = "Articles",
      title = "Affiliations' Production over Time"
    ) +
    scale_x_continuous(
      breaks = {
        yrs <- sort(unique(values$AffOverTime$Year))
        yrs[seq(1, length(yrs), by = max(1, ceiling(length(yrs) / 20)))]
      }
    ) +
    geom_hline(aes(yintercept = 0), alpha = 0.1) +
    labs(color = "Affiliation") +
    theme(
      text = element_text(color = "#444444"),
      legend.text = ggplot2::element_text(size = 9, face = "bold"),
      legend.box.margin = margin(2, 2, 2, 2),
      legend.title = ggplot2::element_text(
        size = 10,
        face = "bold"
      ),
      legend.position = "bottom",
      legend.direction = "horizontal",
      legend.key.size = grid::unit(0.4, "cm"),
      legend.key.width = grid::unit(0.6, "cm"),
      plot.caption = element_text(
        size = 9,
        hjust = 0.5,
        color = "black",
        face = "bold"
      ),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0.95, angle = 0),
      axis.text.x = element_text(size = 10, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    scale_color_discrete(labels = trunc_aff_labels) +
    guides(color = guide_legend(ncol = 2)) +
    annotation_custom(
      values$logoGrid,
      xmin = x[1],
      xmax = x[2],
      ymin = y[1],
      ymax = y[2]
    )
  return(values)
}

CountryOverTime <- function(values, n) {
  if (!("AU_CO" %in% names(values$M))) {
    values$M <- metaTagExtraction(values$M, Field = "AU_CO")
  }
  AFF <- strsplit(values$M$AU_CO, ";")
  nAFF <- lengths(AFF)

  AFFY <- data.frame(
    Affiliation = unlist(AFF),
    Year = rep(values$M$PY, nAFF)
  ) %>%
    drop_na(Affiliation, Year) %>%
    group_by(Affiliation, Year) %>%
    count() %>%
    group_by(Affiliation) %>%
    arrange(Year) %>%
    ungroup() %>%
    pivot_wider(Affiliation, names_from = Year, values_from = n) %>%
    mutate(across(everything(), ~ replace(., is.na(.), 0))) %>%
    pivot_longer(
      cols = !Affiliation,
      names_to = "Year",
      values_to = "Articles"
    ) %>%
    group_by(Affiliation) %>%
    mutate(Articles = cumsum(Articles))

  Affselected <- AFFY %>%
    filter(Year == max(Year)) %>%
    ungroup() %>%
    slice_max(Articles, n = n)

  values$CountryOverTime <- AFFY %>%
    filter(Affiliation %in% Affselected$Affiliation) %>%
    mutate(Year = Year %>% as.numeric()) %>%
    rename(Country = Affiliation)

  Text <- paste(
    values$CountryOverTime$Country,
    " (",
    values$CountryOverTime$Year,
    ") ",
    values$CountryOverTime$Articles,
    sep = ""
  )
  x <- c(
    min(values$CountryOverTime$Year) + 0.02,
    min(values$CountryOverTime$Year) + 0.02 +
      diff(range(values$CountryOverTime$Year)) * 0.15
  )
  y <- c(
    max(values$CountryOverTime$Articles) -
      diff(range(values$CountryOverTime$Articles)) * 0.15,
    max(values$CountryOverTime$Articles)
  )

  # Reorder legend by descending max cumulative value
  values$CountryOverTime$Country <- factor(values$CountryOverTime$Country,
    levels = names(sort(tapply(values$CountryOverTime$Articles, values$CountryOverTime$Country, max), decreasing = TRUE)))

  values$CountryOverTimePlot <- ggplot(
    values$CountryOverTime,
    aes(x = Year, y = Articles, group = Country, color = Country, text = Text)
  ) +
    geom_line() +
    labs(
      x = "Year",
      y = "Articles",
      title = "Country Production over Time"
    ) +
    scale_x_continuous(
      breaks = {
        yrs <- sort(unique(values$CountryOverTime$Year))
        yrs[seq(1, length(yrs), by = max(1, ceiling(length(yrs) / 20)))]
      }
    ) +
    geom_hline(aes(yintercept = 0), alpha = 0.1) +
    labs(color = "Country") +
    theme(
      text = element_text(color = "#444444"),
      legend.text = ggplot2::element_text(size = 10),
      legend.box.margin = margin(6, 6, 6, 6),
      legend.title = ggplot2::element_text(
        size = 12,
        face = "bold"
      ),
      legend.position = "right",
      legend.direction = "vertical",
      legend.key.size = grid::unit(0.4, "cm"),
      legend.key.width = grid::unit(0.6, "cm"),
      plot.caption = element_text(
        size = 9,
        hjust = 0.5,
        color = "black",
        face = "bold"
      ),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0.95, angle = 0),
      axis.text.x = element_text(size = 10, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = x[1],
      xmax = x[2],
      ymin = y[1],
      ymax = y[2]
    )
  return(values)
}

wordlist <- function(
  M,
  Field,
  n,
  measure,
  ngrams,
  remove.terms = NULL,
  synonyms = NULL
) {
  switch(
    Field,
    ID = {
      v <- tableTag(M, "ID", remove.terms = remove.terms, synonyms = synonyms)
    },
    DE = {
      v <- tableTag(M, "DE", remove.terms = remove.terms, synonyms = synonyms)
    },
    KW_Merged = {
      v <- tableTag(
        M,
        "KW_Merged",
        remove.terms = remove.terms,
        synonyms = synonyms
      )
    },
    TI = {
      if (!("TI_TM" %in% names(M))) {
        v <- tableTag(
          M,
          "TI",
          ngrams = ngrams,
          remove.terms = remove.terms,
          synonyms = synonyms
        )
      }
    },
    AB = {
      if (!("AB_TM" %in% names(M))) {
        v <- tableTag(
          M,
          "AB",
          ngrams = ngrams,
          remove.terms = remove.terms,
          synonyms = synonyms
        )
      }
    },
    WC = {
      v <- tableTag(M, "WC")
    }
  )
  names(v) <- tolower(names(v))
  # v=tableTag(values$M,"ID")
  n <- min(c(n, length(v)))
  Words <- data.frame(
    Terms = names(v)[1:n],
    Frequency = (as.numeric(v)[1:n]),
    stringsAsFactors = FALSE
  )
  W <- Words
  switch(
    measure,
    identity = {},
    sqrt = {
      W$Frequency <- sqrt(W$Frequency)
    },
    log = {
      W$Frequency <- log(W$Frequency + 1)
    },
    log10 = {
      W$Frequency <- log10(W$Frequency + 1)
    }
  )

  results <- list(v = v, W = W, Words = Words)
  return(results)
}

readStopwordsFile <- function(file, sep = ",") {
  if (!is.null(file)) {
    req(file$datapath)
    remove.terms <- unlist(strsplit(readr::read_lines(file$datapath), sep))
  } else {
    remove.terms <- NULL
  }
  return(remove.terms)
}

readSynWordsFile <- function(file, sep = ",") {
  if (!is.null(file)) {
    req(file$datapath)
    syn.terms <- readr::read_lines(file$datapath)
    if (sep != ";") syn.terms <- gsub(sep, ";", syn.terms)
  } else {
    syn.terms <- NULL
  }
  return(syn.terms)
}

mapworld <- function(M, values) {
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, "AU_CO")
  }
  CO <- as.data.frame(tableTag(M, "AU_CO"))
  CO$Tab <- gsub("[[:digit:]]", "", CO$Tab)
  CO$Tab <- gsub(".", "", CO$Tab, fixed = TRUE)
  CO$Tab <- gsub(";;", ";", CO$Tab, fixed = TRUE)
  CO$Tab <- gsub("UNITED STATES", "USA", CO$Tab)
  CO$Tab <- gsub("RUSSIAN FEDERATION", "RUSSIA", CO$Tab)
  CO$Tab <- gsub("TAIWAN", "CHINA", CO$Tab)
  CO$Tab <- gsub("ENGLAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("SCOTLAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("WALES", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("NORTH IRELAND", "UNITED KINGDOM", CO$Tab)
  CO$Tab <- gsub("UNITED KINGDOM", "UK", CO$Tab)
  CO$Tab <- gsub("KOREA", "SOUTH KOREA", CO$Tab)

  map.world <- map_data("world")
  map.world$region <- toupper(map.world$region)

  # dplyr::anti_join(CO, map.world, by = c('Tab' = 'region'))

  country.prod <- dplyr::left_join(map.world, CO, by = c("region" = "Tab"))

  tab <- data.frame(
    country.prod %>%
      dplyr::group_by(region) %>%
      dplyr::summarise(Freq = mean(Freq))
  )

  tab <- tab[!is.na(tab$Freq), ]

  tab <- tab[order(-tab$Freq), ]

  # breaks=as.numeric(round(quantile(CO$Freq,c(0.2,0.4,0.6,0.8,1))))
  # names(breaks)=breaks
  # breaks=log(breaks)
  breaks <- as.numeric(cut(CO$Freq, breaks = 10))
  names(breaks) <- breaks

  g <- ggplot(
    country.prod,
    aes(
      x = long,
      y = lat,
      group = group,
      text = paste("Country: ", region, "\nN.of Documents: ", Freq)
    )
  ) +
    geom_polygon(aes(fill = Freq, group = group)) +
    scale_fill_continuous(
      low = "#87CEEB",
      high = "dodgerblue4",
      breaks = breaks,
      na.value = "grey80"
    ) +
    guides(fill = guide_legend(reverse = T)) +
    # geom_text(data=centroids, aes(label = centroids$Tab, x = centroids$long, y = centroids$lat, group=centroids$Tab)) +
    labs(
      fill = "N.Documents",
      title = "Country Scientific Production",
      x = NULL,
      y = NULL
    ) +
    theme(
      text = element_text(color = "#333333"),
      plot.title = element_text(size = 28),
      plot.subtitle = element_text(size = 14),
      axis.ticks = element_blank(),
      axis.text = element_blank(),
      panel.grid = element_blank(),
      panel.background = element_rect(fill = "#FFFFFF"), #' #333333'
      plot.background = element_rect(fill = "#FFFFFF"),
      legend.position = "none"
      # ,legend.background = element_blank()
      # ,legend.key = element_blank()
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = 143,
      xmax = 189.5,
      ymin = -69,
      ymax = -48
    )

  results <- list(g = g, tab = tab)
  return(results)
}

### Structure fuctions ----
CAmap <- function(input, values) {
  if ((input$CSfield %in% names(values$M))) {
    if (input$CSfield %in% c("TI", "AB")) {
      ngrams <- as.numeric(input$CSngrams)
    } else {
      ngrams <- 1
    }

    ### load file with terms to remove
    if (input$CSStopFile == "Y") {
      remove.terms <- trimws(values$CSremove.terms$stopword)
    } else {
      remove.terms <- NULL
    }
    # values$CSremove.terms <- remove.terms
    ### end of block
    ### load file with synonyms
    if (input$FASynFile == "Y") {
      synonyms <- values$FAsyn.terms %>%
        group_by(term) %>%
        mutate(term = paste0(term, ";", synonyms)) %>%
        select(term)
      synonyms <- synonyms$term
    } else {
      synonyms <- NULL
    }
    # values$FAsyn.terms <- synonyms
    ### end of block

    tab <- tableTag(values$M, input$CSfield, ngrams = ngrams)
    if (length(tab >= 2)) {
      minDegree <- as.numeric(tab[input$CSn])

      values$CS <- conceptualStructure(
        values$M,
        method = input$method,
        field = input$CSfield,
        minDegree = minDegree,
        clust = input$nClustersCS,
        k.max = 8,
        stemming = F,
        labelsize = input$CSlabelsize / 2,
        documents = input$CSdoc,
        graph = FALSE,
        ngrams = ngrams,
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      if (input$method != "MDS") {
        CSData <- values$CS$docCoord
        CSData <- data.frame(Documents = row.names(CSData), CSData)
        CSData$dim1 <- round(CSData$dim1, 2)
        CSData$dim2 <- round(CSData$dim2, 2)
        CSData$contrib <- round(CSData$contrib, 2)
        values$CS$CSData <- CSData
      } else {
        values$CS$CSData <- data.frame(Docuemnts = NA, dim1 = NA, dim2 = NA)
      }

      switch(
        input$method,
        CA = {
          WData <- data.frame(
            word = row.names(values$CS$km.res$data.clust),
            values$CS$km.res$data.clust,
            stringsAsFactors = FALSE
          )
          names(WData)[4] <- "cluster"
        },
        MCA = {
          WData <- data.frame(
            word = row.names(values$CS$km.res$data.clust),
            values$CS$km.res$data.clust,
            stringsAsFactors = FALSE
          )
          names(WData)[4] <- "cluster"
        },
        MDS = {
          WData <- data.frame(
            word = row.names(values$CS$res),
            values$CS$res,
            cluster = values$CS$km.res$cluster
          )
        }
      )

      WData$Dim1 <- round(WData$Dim1, 2)
      WData$Dim2 <- round(WData$Dim2, 2)
      values$CS$WData <- WData
    } else {
      emptyPlot("Selected field is not included in your data collection")
      values$CS <- list("NA")
    }
  } else {
    emptyPlot("Selected field is not included in your data collection")
    values$CS <- list("NA")
  }
  return(values)
}

historiograph <- function(input, values) {
  min.cit <- 0

  # if (values$Histfield=="NA"){
  values$histResults <- histNetwork(
    values$M,
    min.citations = min.cit,
    sep = ";"
  )
  # values$Histfield="done"
  # }
  values$histResults$histData <- values$histResults$histData %>%
    tibble::rownames_to_column(var = "SR")
  # titlelabel <- input$titlelabel
  values$histlog <- (values$histPlot <- histPlot(
    values$histResults,
    n = input$histNodes,
    size = input$histsize,
    remove.isolates = (input$hist.isolates == "yes"),
    labelsize = input$histlabelsize,
    label = input$titlelabel,
    verbose = FALSE
  ))

  values$histResults$histData$DOI <- paste0(
    '<a href=\"https://doi.org/',
    values$histResults$histData$DOI,
    '\" target=\"_blank\">',
    values$histResults$histData$DOI,
    "</a>"
  )

  values$histResults$histData <- values$histResults$histData %>%
    left_join(
      values$histPlot$layout %>%
        select(name, color),
      by = c("Paper" = "name")
    ) %>%
    drop_na(color) %>%
    mutate(cluster = match(color, unique(color))) %>%
    select(!color) %>%
    group_by(cluster) %>%
    arrange(Year, .by_group = TRUE)

  return(values)
}


### Network functions ----
degreePlot <- function(net) {
  # deg <- data.frame(node = names(net$nodeDegree), x= (1:length(net$nodeDegree)), y = net$nodeDegree)
  ma <- function(x, n = 5) {
    stats::filter(x, rep(1 / n, n), sides = 1)
  }

  deg <- net$nodeDegree %>%
    mutate(x = row_number())

  p <- ggplot(
    data = deg,
    aes(
      x = x,
      y = degree,
      text = paste(node, " - Degree ", round(degree, 3), sep = "")
    )
  ) +
    geom_point() +
    geom_line(aes(group = "NA"), color = "#002F80", alpha = .5) +
    # geom_hline(yintercept=cutting$degree, linetype="dashed",color = '#002F80', alpha = .5)+
    theme(
      text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 0),
      axis.title.x = element_text(hjust = 0),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    labs(x = "Node", y = "Cumulative Degree", title = "Node Degrees")
  return(p)
}

# Associate a Year to each Keyword
keywords2Years <- function(M, field = "DE", n = 100) {
  suppressMessages(Y <- KeywordGrowth(M, Tag = field, top = Inf, cdf = FALSE))

  ## Normalize data and exclude tot from normalization
  df <- Y %>%
    rowwise() %>%
    mutate(year_freq = sum(c_across(!matches("Year")))) %>%
    mutate(across(!matches("Year"), ~ .x / year_freq)) %>%
    mutate(across(!matches("Year"), ~ replace_na(.x, 0)))

  df_long <- df %>%
    pivot_longer(
      cols = -c(Year, year_freq),
      names_to = "Keyword",
      values_to = "Probability"
    )

  df_max_year <- df_long %>%
    group_by(Keyword) %>%
    filter(Probability == max(Probability)) %>%
    slice_max(order_by = year_freq, n = 1, with_ties = FALSE) %>%
    ungroup()

  return(df_max_year)
}

cocNetwork <- function(input, values) {
  n <- input$Nodes
  label.n <- input$Labels

  ### load file with terms to remove
  if (input$COCStopFile == "Y") {
    remove.terms <- trimws(values$COCremove.terms$stopword)
  } else {
    remove.terms <- NULL
  }
  # values$COCremove.terms <- remove.terms
  ### end of block
  ### load file with synonyms
  if (input$COCSynFile == "Y") {
    synonyms <- values$COCsyn.terms %>%
      group_by(term) %>%
      mutate(term = paste0(term, ";", synonyms)) %>%
      select(term)
    synonyms <- synonyms$term
  } else {
    synonyms <- NULL
  }
  # values$COCsyn.terms <- synonyms
  ### end of block

  if ((input$field %in% names(values$M))) {
    if (
      (dim(values$NetWords)[1]) == 1 |
        !(input$field == values$field) |
        !(input$cocngrams == values$cocngrams) |
        ((dim(values$NetWords)[1]) != input$Nodes)
    ) {
      values$field <- input$field
      values$ngrams <- input$cocngrams

      switch(
        input$field,
        ID = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "Keywords Plus Network"
        },
        DE = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "author_keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "Authors' Keywords network"
        },
        KW_Merged = {
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "all_keywords",
            n = n,
            sep = ";",
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          values$Title <- "All Keywords network"
        },
        TI = {
          # if(!("TI_TM" %in% names(values$M))){
          values$M <- termExtraction(
            values$M,
            Field = "TI",
            verbose = FALSE,
            ngrams = as.numeric(input$cocngrams),
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          # }
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "titles",
            n = n,
            sep = ";"
          )
          values$Title <- "Title Words network"
        },
        AB = {
          # if(!("AB_TM" %in% names(values$M))){
          values$M <- termExtraction(
            values$M,
            Field = "AB",
            verbose = FALSE,
            ngrams = as.numeric(input$cocngrams),
            remove.terms = remove.terms,
            synonyms = synonyms
          )
          # }
          values$NetWords <- biblioNetwork(
            values$M,
            analysis = "co-occurrences",
            network = "abstracts",
            n = n,
            sep = ";"
          )
          values$Title <- "Abstract Words network"
        },
        WC = {
          WSC <- cocMatrix(values$M, Field = "WC", binary = FALSE)
          values$NetWords <- crossprod(WSC, WSC)
          values$Title <- "Subject Categories network"
        }
      )
    }

    if (label.n > n) {
      label.n <- n
    }
    if (input$normalize == "none") {
      normalize <- NULL
    } else {
      normalize <- input$normalize
    }
    if (input$label.cex == "Yes") {
      label.cex <- TRUE
    } else {
      label.cex <- FALSE
    }
    if (input$coc.curved == "Yes") {
      curved <- TRUE
    } else {
      curved <- FALSE
    }

    values$cocnet <- networkPlot(
      values$NetWords,
      normalize = normalize,
      Title = values$Title,
      type = input$layout,
      size.cex = TRUE,
      size = 5,
      remove.multiple = F,
      edgesize = input$edgesize * 3,
      labelsize = input$labelsize,
      label.cex = label.cex,
      label.n = label.n,
      edges.min = input$edges.min,
      label.color = F,
      curved = curved,
      alpha = input$cocAlpha,
      cluster = input$cocCluster,
      remove.isolates = (input$coc.isolates == "yes"),
      community.repulsion = input$coc.repulsion / 2,
      seed = values$random_seed,
      verbose = FALSE
    )

    g <- values$cocnet$graph
    Y <- keywords2Years(values$M, field = input$field, n = Inf)
    label <- data.frame(Keyword = igraph::V(g)$name)
    df <- label %>%
      left_join(Y %>% mutate(Keyword = tolower(Keyword)), by = "Keyword") %>%
      rename(year_med = Year)
    igraph::V(g)$year_med <- df$year_med

    if (input$cocyears == "Yes") {
      col <- hcl.colors(
        (diff(range(df$year_med)) + 1) * 10,
        palette = "Blues 3"
      )
      igraph::V(g)$color <- col[(max(df$year_med) - df$year_med + 1) * 10]
    }
    values$cocnet$graph <- g
  } else {
    emptyPlot("Selected field is not included in your data collection")
  }
  return(values)
}

intellectualStructure <- function(input, values) {
  n <- input$citNodes
  label.n <- input$citLabels

  if (
    (dim(values$NetRefs)[1]) == 1 |
      !(input$citField == values$citField) |
      !(input$citSep == values$citSep) |
      !(input$citShortlabel == values$citShortlabel) |
      ((dim(values$NetRefs)[1]) != input$citNodes)
  ) {
    values$citField <- input$citField
    values$citSep <- input$citSep
    if (input$citShortlabel == "Yes") {
      shortlabel <- TRUE
    } else {
      shortlabel <- FALSE
    }
    values$citShortlabel <- input$citShortlabel
    switch(
      input$citField,
      CR = {
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "references",
          n = n,
          sep = input$citSep,
          shortlabel = shortlabel
        )
        values$Title <- "Cited References network"
      },
      CR_AU = {
        if (!("CR_AU" %in% names(values$M))) {
          values$M <- metaTagExtraction(
            values$M,
            Field = "CR_AU",
            sep = input$citSep
          )
        }
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "authors",
          n = n,
          sep = input$citSep
        )
        values$Title <- "Cited Authors network"
      },
      CR_SO = {
        if (!("CR_SO" %in% names(values$M))) {
          values$M <- metaTagExtraction(
            values$M,
            Field = "CR_SO",
            sep = input$citSep
          )
        }
        values$NetRefs <- biblioNetwork(
          values$M,
          analysis = "co-citation",
          network = "sources",
          n = n,
          sep = input$citSep
        )
        values$Title <- "Cited Sources network"
      }
    )
  }

  if (label.n > n) {
    label.n <- n
  }
  if (input$citlabel.cex == "Yes") {
    label.cex <- TRUE
  } else {
    label.cex <- FALSE
  }
  if (input$cocit.curved == "Yes") {
    curved <- TRUE
  } else {
    curved <- FALSE
  }

  values$cocitnet <- networkPlot(
    values$NetRefs,
    normalize = NULL,
    Title = values$Title,
    type = input$citlayout,
    size.cex = TRUE,
    size = 5,
    remove.multiple = F,
    edgesize = input$citedgesize * 3,
    labelsize = input$citlabelsize,
    label.cex = label.cex,
    curved = curved,
    label.n = label.n,
    edges.min = input$citedges.min,
    label.color = F,
    remove.isolates = (input$cit.isolates == "yes"),
    alpha = 0.7,
    cluster = input$cocitCluster,
    community.repulsion = input$cocit.repulsion / 2,
    verbose = FALSE
  )
  return(values)
}

authors2Years <- function(M, field = "AU") {
  WAU <- cocMatrix(M, field)
  WPY <- cocMatrix(M, "PY")
  B <- crossprod(WPY, WAU) %>%
    as.matrix() %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Year")

  # create a data frame that startig from B associate at each author the year of the first non zero value
  C <- B %>%
    pivot_longer(-Year, names_to = "Item", values_to = "Value") %>%
    filter(Value > 0) %>%
    group_by(Item) %>%
    summarise(FirstYear = min(Year)) %>%
    ungroup()

  return(C)
}

socialStructure <- function(input, values) {
  n <- input$colNodes
  label.n <- input$colLabels

  if (
    (dim(values$ColNetRefs)[1]) == 1 |
      !(input$colField == values$colField) |
      ((dim(values$ColNetRefs)[1]) != input$colNodes)
  ) {
    values$colField <- input$colField

    if (!"nAU" %in% names(values$M)) {
      values$M$nAU <- str_count(values$M$AU, ";") + 1
    }

    switch(
      input$colField,
      COL_AU = {
        if (input$col.filterMaxAuthors) {
          M_AU <- values$M %>% filter(nAU <= 20)
        } else {
          M_AU <- values$M
        }
        values$ColNetRefs <- biblioNetwork(
          M_AU,
          analysis = "collaboration",
          network = "authors",
          n = n,
          sep = ";"
        )
        values$Title <- "Author Collaboration network"
        values$fieldCOL <- "AU"
      },
      COL_UN = {
        if (!("AU_UN" %in% names(values$M))) {
          values$M <- metaTagExtraction(values$M, Field = "AU_UN", sep = ";")
        }
        values$ColNetRefs <- biblioNetwork(
          values$M,
          analysis = "collaboration",
          network = "universities",
          n = n,
          sep = ";"
        )
        values$Title <- "Edu Collaboration network"
        values$fieldCOL <- "AU_UN"
      },
      COL_CO = {
        if (!("AU_CO" %in% names(values$M))) {
          values$M <- metaTagExtraction(values$M, Field = "AU_CO", sep = ";")
        }
        values$ColNetRefs <- biblioNetwork(
          values$M,
          analysis = "collaboration",
          network = "countries",
          n = n,
          sep = ";"
        )
        values$Title <- "Country Collaboration network"
        values$fieldCOL <- "AU_CO"
        # values$cluster="none"
      }
    )
  }

  if (label.n > n) {
    label.n <- n
  }
  if (input$colnormalize == "none") {
    normalize <- NULL
  } else {
    normalize <- input$colnormalize
  }
  if (input$collabel.cex == "Yes") {
    label.cex <- TRUE
  } else {
    label.cex <- FALSE
  }
  if (input$soc.curved == "Yes") {
    curved <- TRUE
  } else {
    curved <- FALSE
  }

  type <- input$collayout
  if (input$collayout == "worldmap") {
    type <- "auto"
  }

  values$colnet <- networkPlot(
    values$ColNetRefs,
    normalize = normalize,
    Title = values$Title,
    type = type,
    size.cex = TRUE,
    size = 5,
    remove.multiple = F,
    edgesize = input$coledgesize * 3,
    labelsize = input$collabelsize,
    label.cex = label.cex,
    curved = curved,
    label.n = label.n,
    edges.min = input$coledges.min,
    label.color = F,
    alpha = input$colAlpha,
    remove.isolates = (input$col.isolates == "yes"),
    cluster = input$colCluster,
    community.repulsion = input$col.repulsion / 2,
    verbose = FALSE
  )

  g <- values$colnet$graph
  Y <- authors2Years(values$M, values$fieldCOL)
  label <- data.frame(Item = igraph::V(g)$name)
  df <- label %>%
    left_join(Y %>% mutate(Item = tolower(Item)), by = "Item") %>%
    rename(year_med = FirstYear)
  igraph::V(g)$year_med <- df$year_med

  values$colnet$graph <- g

  return(values)
}

countrycollaboration_plotly <- function(
  M,
  min.edges = 1,
  edge_opacity = 0.4,
  edgesize = 5,
  min_edgesize = 0.5
) {
  if (!requireNamespace("geosphere", quietly = TRUE)) {
    install.packages("geosphere")
  }
  # library(plotly)
  # library(geosphere)

  # Prepara i dati
  M <- metaTagExtraction(M, "AU_CO")
  net <- biblioNetwork(M, analysis = "collaboration", network = "countries")
  CO <- data.frame(Tab = rownames(net), Freq = diag(net))
  bsk.network <- igraph::graph_from_adjacency_matrix(net, mode = "undirected")
  COedges <- as.data.frame(igraph::ends(
    bsk.network,
    igraph::E(bsk.network),
    names = TRUE
  ))

  # Carica coordinate
  data("countries", envir = environment())
  names(countries)[1] <- "Tab"

  # Prepara i paesi
  CO <- dplyr::inner_join(CO, countries, by = "Tab")

  # Prepara gli edge
  COedges <- dplyr::inner_join(COedges, countries, by = c("V1" = "Tab"))
  COedges <- dplyr::inner_join(COedges, countries, by = c("V2" = "Tab"))
  COedges <- COedges[COedges$V1 != COedges$V2, ]
  COedges <- tab <- count.duplicates(COedges)
  COedges <- COedges[COedges$count >= min.edges, ]

  # Calcola gli spessori degli edge normalizzati
  edge_weights <- sqrt(COedges$count)
  # Normalizza tra min_edgesize e edgesize
  if (length(unique(edge_weights)) > 1) {
    COedges$width <- min_edgesize +
      (edge_weights - min(edge_weights)) /
        (max(edge_weights) - min(edge_weights)) *
        (edgesize - min_edgesize)
  } else {
    # Se tutti gli edge hanno lo stesso peso, usa il valore medio
    COedges$width <- (min_edgesize + edgesize) / 2
  }

  # Crea la mappa base con choropleth (SENZA legenda)
  fig <- plotly::plot_geo(CO) %>%
    add_trace(
      type = "choropleth",
      locations = ~Tab,
      locationmode = "country names",
      z = ~ log1p(Freq),
      colorscale = list(
        c(0, "#FFFFFF"), # Bianco
        c(0.2, "#fee5d9"), # Rosa pallido
        c(0.4, "#fcae91"), # Salmone
        c(0.6, "#fb6a4a"), # Arancione/rosso
        c(0.8, "#de2d26"), # Rosso
        c(1, "#a50f15") # Bordeaux scuro
      ),
      text = ~ paste0("<b>", Tab, "</b><br>Publications: ", Freq),
      hoverinfo = "text",
      marker = list(line = list(color = "white", width = 0.8)),
      showscale = FALSE,
      showlegend = FALSE
    )

  # Aggiungi le linee di collaborazione curve con spessore controllato
  for (i in 1:nrow(COedges)) {
    # Crea arco geodetico
    path <- geosphere::gcIntermediate(
      c(COedges$Longitude.x[i], COedges$Latitude.x[i]),
      c(COedges$Longitude.y[i], COedges$Latitude.y[i]),
      n = 100,
      addStartEnd = TRUE,
      sp = FALSE
    )

    fig <- fig %>%
      add_trace(
        type = "scattergeo",
        lon = path[, 1],
        lat = path[, 2],
        mode = "lines",
        line = list(
          width = COedges$width[i], # USA LO SPESSORE NORMALIZZATO
          color = "#4A90E2"
        ),
        opacity = edge_opacity,
        hoverinfo = "text",
        text = paste0(
          "<b>",
          COedges$V1[i],
          " ↔ ",
          COedges$V2[i],
          "</b>",
          "<br>Collaborations: ",
          COedges$count[i]
        ),
        showlegend = FALSE
      )
  }

  # Layout migliorato
  fig <- fig %>%
    layout(
      # title = list(
      #   text = "International Scientific Collaboration Network",
      #   font = list(size = 18, color = "#333333")
      # ),
      geo = list(
        projection = list(type = "natural earth"),
        showland = TRUE,
        landcolor = "#f5f5f5",
        coastlinecolor = "#999999",
        showocean = TRUE,
        oceancolor = "#d4e6f1",
        showlakes = TRUE,
        lakecolor = "#d4e6f1",
        showcountries = TRUE,
        countrycolor = "white",
        countrywidth = 1
      ),
      paper_bgcolor = "white",
      plot_bgcolor = "white"
    )

  return(list(g = fig, tab = tab))
}

countrycollaboration <- function(M, label, edgesize, min.edges, values) {
  M <- metaTagExtraction(M, "AU_CO")
  net <- biblioNetwork(M, analysis = "collaboration", network = "countries")
  CO <- data.frame(Tab = rownames(net), Freq = diag(net))
  bsk.network <- igraph::graph_from_adjacency_matrix(net, mode = "undirected")
  COedges <- as.data.frame(igraph::ends(
    bsk.network,
    igraph::E(bsk.network),
    names = TRUE
  ))

  map.world <- map_data("world")
  map.world$region <- toupper(map.world$region)
  map.world$region <- gsub("^UK$", "UNITED KINGDOM", map.world$region)
  map.world$region <- gsub("^SOUTH KOREA$", "KOREA", map.world$region)

  country.prod <- dplyr::left_join(map.world, CO, by = c("region" = "Tab"))

  # breaks <- as.numeric(round(quantile(CO$Freq,seq(0.1,1,by=0.1))))
  breaks <- as.numeric(cut(CO$Freq, breaks = 10))
  names(breaks) <- breaks
  # breaks=breaks
  data("countries", envir = environment())
  names(countries)[1] <- "Tab"

  COedges <- dplyr::inner_join(COedges, countries, by = c("V1" = "Tab"))
  COedges <- dplyr::inner_join(COedges, countries, by = c("V2" = "Tab"))
  COedges <- COedges[COedges$V1 != COedges$V2, ]
  COedges <- count.duplicates(COedges)
  tab <- COedges
  COedges <- COedges[COedges$count >= min.edges, ]
  COedges$region <- paste(
    "\nCollaboration between\n",
    COedges$V1,
    "\n and \n",
    COedges$V2
  )

  g <- ggplot(
    country.prod,
    aes(x = long, y = lat, group = group, text = paste("Country: ", region))
  ) +
    geom_polygon(aes(fill = Freq)) +
    scale_fill_continuous(
      low = "#87CEEB",
      high = "dodgerblue4",
      breaks = breaks,
      na.value = "grey80"
    ) +
    # guides(fill = guide_legend(reverse = T)) +
    guides(colour = FALSE, fill = FALSE) +
    # geom_curve(data=COedges, aes(x = Longitude.x , y = Latitude.x, xend = Longitude.y, yend = Latitude.y,     # draw edges as arcs
    #                              color = "firebrick4", size = count, group=continent.x),
    #            curvature = 0.33,
    #            alpha = 0.5) +
    geom_segment(
      data = COedges,
      aes(
        x = Longitude.x,
        y = Latitude.x,
        xend = Longitude.y,
        yend = Latitude.y, # draw edges as arcs
        size = count,
        group = continent.x
      ),
      color = "orangered4", # FFB347",
      # curvature = 0.33,
      alpha = 0.3
    ) +
    scale_size_continuous(guide = FALSE, range = c(0.25, edgesize)) +
    labs(title = NULL, x = "Latitude", y = "Longitude") +
    theme(
      text = element_text(color = "#333333"),
      plot.title = element_text(size = 28),
      plot.subtitle = element_text(size = 14),
      axis.ticks = element_blank(),
      axis.text = element_blank(),
      panel.grid = element_blank(),
      panel.background = element_rect(fill = "#FFFFFF"), #' #333333'
      plot.background = element_rect(fill = "#FFFFFF"),
      legend.position = c(.18, .36),
      legend.background = element_blank(),
      legend.key = element_blank()
    ) +
    annotation_custom(
      values$logoGrid,
      xmin = 143,
      xmax = 189.5,
      ymin = -69,
      ymax = -48
    )
  if (isTRUE(label)) {
    CO <- dplyr::inner_join(CO, countries, by = c("Tab" = "Tab"))
    g <- g +
      # ggrepel::geom_text_repel(data=CO, aes(x = Longitude, y = Latitude, label = Tab, group=continent),             # draw text labels
      #                          hjust = 0, nudge_x = 1, nudge_y = 4,
      #                          size = 3, color = "orange", fontface = "bold")
      ggrepel::geom_text(
        data = CO,
        aes(x = Longitude, y = Latitude, label = Tab, group = continent), # draw text labels
        hjust = 0,
        nudge_x = 1,
        nudge_y = 4,
        size = 3,
        color = "orange",
        fontface = "bold"
      )
  }

  results <- list(g = g, tab = tab)
  return(results)
}
### visNetwork tools ----
netLayout <- function(type) {
  switch(
    type,
    auto = {
      l <- "layout_nicely"
    },
    circle = {
      l <- "layout_in_circle"
    },
    mds = {
      l <- "layout_with_mds"
    },
    star = {
      l <- "layout_as_star"
    },
    sphere = {
      l <- "layout_on_sphere"
    },
    fruchterman = {
      l <- "layout_with_fr"
    },
    kamada = {
      l <- "layout_with_kk"
    }
  )
  return(l)
}

savenetwork <- function(con, VIS) {
  VIS %>%
    visOptions(height = "800px") %>%
    visNetwork::visSave(con)
}

igraph2vis <- function(
  g,
  curved,
  labelsize,
  opacity,
  type,
  shape,
  net,
  shadow = TRUE,
  edgesize = 5,
  noOverlap = TRUE
) {
  LABEL <- igraph::V(g)$name

  LABEL[igraph::V(g)$labelsize == 0] <- ""

  vn <- visNetwork::toVisNetworkData(g)

  vn$nodes$label <- LABEL
  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE

  ## opacity
  vn$nodes$color <- adjustcolor(vn$nodes$color, alpha.f = min(c(opacity, 1)))
  ## set a darkest gray for iter-cluster edges
  vn$edges$color <- paste(substr(vn$edges$color, 1, 7), "90", sep = "")
  vn$edges$color[substr(vn$edges$color, 1, 7) == "#B3B3B3"] <- "#69696960"
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity)

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  vn$edges$width <- vn$edges$width^2 / (max(vn$edges$width^2)) * (10 + edgesize)

  # if (edgesize==0){
  #   vn$edges$hidden <- TRUE
  #   }else{vn$edges$hidden <- FALSE}

  ## labelsize
  vn$nodes$font.size <- vn$nodes$deg
  scalemin <- 20
  scalemax <- 150
  Min <- min(vn$nodes$font.size)
  Max <- max(vn$nodes$font.size)
  if (Max > Min) {
    size <- (vn$nodes$font.size - Min) / (Max - Min) * 15 * labelsize + 10
  } else {
    size <- 10 * labelsize
  }
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size
  l <- netLayout(type)

  ### TO ADD SHAPE AND FONT COLOR OPTIONS
  coords <- net$layout

  vn$nodes$size <- vn$nodes$font.size * 0.7

  # vn$nodes$font.color <- adjustcolor("black", alpha.f = min(c(opacity,1)))

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  opacity_font <- sqrt(
    (vn$nodes$font.size - min(vn$nodes$font.size)) /
      diff(range(vn$nodes$font.size))
  ) *
    opacity +
    0.3
  if (is.nan(opacity_font[1])) {
    opacity_font <- rep(0.3, length(opacity_font))
  }

  if (labelsize > 0) {
    vn$nodes$font.color <- unlist(lapply(opacity_font, function(x) {
      adjustcolor("black", alpha.f = x)
    }))
  } else {
    vn$nodes$font.color <- adjustcolor("black", alpha.f = 0)
  }
  ## avoid label overlaps
  if (noOverlap) {
    threshold <- 0.05
    ymax <- diff(range(coords[, 2]))
    xmax <- diff(range(coords[, 1]))
    threshold2 <- threshold * mean(xmax, ymax)
    w <- data.frame(
      x = coords[, 1],
      y = coords[, 2],
      labelToPlot = vn$nodes$label,
      dotSize = size,
      stringsAsFactors = FALSE
    )
    # Hidden nodes share an empty label (""), which would clash as row.names.
    # make.unique keeps the real, already-unique labels untouched (they are
    # the only ones avoidNetOverlaps() looks up) and disambiguates the rest.
    rownames(w) <- make.unique(vn$nodes$label)
    labelToRemove <- avoidNetOverlaps(w, threshold = threshold2)
  } else {
    labelToRemove <- ""
  }

  vn$nodes <- vn$nodes %>%
    mutate(
      label = ifelse(label %in% labelToRemove, "", label),
      title = id
    )
  ##

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = shadow,
      shape = shape,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
    visNetwork::visOptions(
      highlightNearest = list(enabled = T, hover = T, degree = 1),
      nodesIdSelection = T
    ) %>%
    visNetwork::visInteraction(
      dragNodes = TRUE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visNetwork::visOptions(
      manipulation = curved,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn, type = type, l = l, curved = curved))
}

## function to avoid label overlapping ----
avoidNetOverlaps <- function(w, threshold = 0.10) {
  w[, 2] <- w[, 2] / 2

  Ds <- dist(
    w %>%
      dplyr::filter(labelToPlot != "") %>%
      select(1:2),
    method = "manhattan",
    upper = T
  ) %>%
    dist2df() %>%
    rename(
      from = row,
      to = col,
      dist = value
    ) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("from" = "labelToPlot")
    ) %>%
    rename(w_from = dotSize) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("to" = "labelToPlot")
    ) %>%
    rename(w_to = dotSize) %>%
    filter(dist < threshold)

  if (nrow(Ds) > 0) {
    st <- TRUE
    i <- 1
    label <- NULL
    case <- "n"

    while (isTRUE(st)) {
      if (Ds$w_from[i] > Ds$w_to[i] & Ds$dist[i] < threshold) {
        case <- "y"
        lab <- Ds$to[i]
      } else if (Ds$w_from[i] <= Ds$w_to[i] & Ds$dist[i] < threshold) {
        case <- "y"
        lab <- Ds$from[i]
      }

      switch(
        case,
        "y" = {
          Ds <- Ds[Ds$from != lab, ]
          Ds <- Ds[Ds$to != lab, ]
          label <- c(label, lab)
        },
        "n" = {
          Ds <- Ds[-1, ]
        }
      )

      if (i >= nrow(Ds)) {
        st <- FALSE
      }
      case <- "n"
      # print(nrow(Ds))
    }
  } else {
    label <- NULL
  }
  label
}


## visnetwork for subgraphs
igraph2visClust <- function(
  g,
  curved = FALSE,
  labelsize = 3,
  opacity = 0.7,
  shape = "dot",
  shadow = TRUE,
  edgesize = 5
) {
  LABEL <- igraph::V(g)$name

  LABEL[igraph::V(g)$labelsize == 0] <- ""

  vn <- visNetwork::toVisNetworkData(g)

  vn$nodes$label <- LABEL
  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE

  ## opacity
  vn$nodes$color <- adjustcolor(vn$nodes$color, alpha.f = min(c(opacity, 1)))
  ## set a darkest gray for iter-cluster edges
  vn$edges$color <- paste(substr(vn$edges$color, 1, 7), "90", sep = "")
  vn$edges$color[substr(vn$edges$color, 1, 7) == "#B3B3B3"] <- "#69696960"
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity)

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  vn$edges$width <- vn$edges$width^2 / (max(vn$edges$width^2)) * (5 + edgesize)

  ## labelsize
  scalemin <- 20
  scalemax <- 100
  # aggiunta
  vn$nodes$font.size <- vn$nodes$deg
  #
  Min <- min(vn$nodes$font.size)
  Max <- max(vn$nodes$font.size)
  if (Max > Min) {
    size <- (vn$nodes$font.size - Min) / (Max - Min) * 15 * labelsize #+10
  } else {
    size <- 5 * labelsize
  }
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size
  # l<-netLayout(type)

  ### TO ADD SHAPE AND FONT COLOR OPTIONS

  vn$nodes$size <- vn$nodes$font.size * 0.4

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  opacity_font <- sqrt(
    (vn$nodes$font.size - min(vn$nodes$font.size)) /
      diff(range(vn$nodes$font.size))
  ) *
    opacity +
    0.3
  if (is.nan(opacity_font[1])) {
    opacity_font <- rep(0.3, length(opacity_font))
  }

  if (labelsize > 0) {
    vn$nodes$font.color <- unlist(lapply(opacity_font, function(x) {
      adjustcolor("black", alpha.f = x)
    }))
  } else {
    vn$nodes$font.color <- adjustcolor("black", alpha.f = 0)
  }

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = shadow,
      shape = shape,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(layout = "layout_nicely", type = "full") %>%
    visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
    visNetwork::visOptions(
      highlightNearest = list(enabled = T, hover = T, degree = 1),
      nodesIdSelection = T
    ) %>%
    visNetwork::visInteraction(
      dragNodes = TRUE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visNetwork::visOptions(
      manipulation = curved,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn))
}


hist2vis <- function(
  net,
  labelsize = 2,
  nodesize = 2,
  curved = FALSE,
  shape = "dot",
  opacity = 0.7,
  labeltype = "short",
  timeline = TRUE
) {
  LABEL <- igraph::V(net$net)$id

  LABEL[igraph::V(net$net)$labelsize == 0] <- ""

  layout <- net$layout %>%
    dplyr::select(x, y, color, name)

  vn <- visNetwork::toVisNetworkData(net$net)

  vn$nodes$short_label <- LABEL

  if (labeltype != "short") {
    vn$nodes$label <- paste0(vn$nodes$years, ": ", LABEL)
  } else {
    vn$nodes$label <- LABEL
  }

  vn$nodes <- dplyr::left_join(vn$nodes, layout, by = c("id" = "name"))

  vn$edges$num <- 1
  vn$edges$dashes <- FALSE
  vn$edges$dashes[vn$edges$lty == 2] <- TRUE
  vn$edges$color <- "grey"

  ## opacity
  vn$nodes$font.color <- vn$nodes$color

  vn$nodes$color <- adjustcolor(
    vn$nodes$color,
    alpha.f = min(c(opacity - 0.2, 1))
  )
  vn$edges$color <- adjustcolor(vn$edges$color, alpha.f = opacity - 0.2)
  vn$edges$smooth <- curved

  ## removing multiple edges
  vn$edges <- unique(vn$edges)

  ## labelsize
  scalemin <- 20
  scalemax <- 150
  size <- 10 * labelsize
  size[size < scalemin] <- scalemin
  size[size > scalemax] <- scalemax
  vn$nodes$font.size <- size * 0.5
  vn$nodes$size <- nodesize * 2

  if (shape %in% c("dot", "square")) {
    vn$nodes$font.vadjust <- -0.7 * vn$nodes$font.size
  } else {
    vn$nodes$font.vadjust <- 0
  }

  text_data <- net$graph.data %>%
    select(Label, DOI, LCS, GCS) %>%
    rename(id = Label) %>%
    filter(!duplicated(id))

  vn$nodes <- vn$nodes %>% left_join(text_data, by = "id")

  ## split node tooltips into two strings
  title <- strsplit(stringi::stri_trans_totitle(vn$nodes$title), " ")

  vn$nodes$title <- unlist(lapply(title, function(l) {
    n <- floor(length(l) / 2)
    paste0(
      paste(l[1:n], collapse = " ", sep = ""),
      "<br>",
      paste(l[(n + 1):length(l)], collapse = " ", sep = "")
    )
  }))

  vn$nodes <- vn$nodes %>%
    mutate(
      title_orig = title,
      title = paste(
        "<b>Title</b>: ",
        title,
        "<br><b>DOI</b>: ",
        paste0(
          '<a href=\"https://doi.org/',
          DOI,
          '\" target=\"_blank\">',
          # "DOI: ",
          DOI,
          "</a>"
        ),
        "<br><b>GCS</b>: ",
        GCS,
        "<br><b>LCS</b>: ",
        LCS,
        sep = ""
      )
    )

  ## add time line
  vn$nodes$group <- "normal"
  vn$nodes$shape <- "dot"
  vn$nodes$shadow <- TRUE

  # nr <- nrow(vn$nodes)
  # y <- max(vn$nodes$y)
  # vn$nodes[nr + 1, c("id", "title", "label", "color", "font.color")] <-
  #   c(rep("logo", 3), "black", "white")
  # vn$nodes$x[nr + 1] <- max(vn$nodes$x, na.rm = TRUE) + 1
  # vn$nodes$y[nr + 1] <- y
  # vn$nodes$size[nr + 1] <- vn$nodes$size[nr] * 4
  # vn$nodes$years[nr + 1] <- as.numeric(vn$nodes$x[nr + 1])
  # vn$nodes$font.size[nr + 1] <- vn$nodes$font.size[nr]
  # vn$nodes$group[nr + 1] <- "logo"
  # vn$nodes$shape[nr + 1] <- "image"
  # vn$nodes$image[nr + 1] <- "logo.jpg"
  # vn$nodes$fixed.x <- TRUE
  # vn$nodes$fixed.y <- FALSE
  # vn$nodes$fixed.y[nr + 1] <- TRUE
  # vn$nodes$shadow[nr + 1] <- FALSE

  # coords <- vn$nodes[, c("x", "y")] %>%
  #   as.matrix()
  #
  # coords[, 2] <- coords[, 2]^(1 / 2)

  tooltipStyle <- ("position: fixed;visibility:hidden;padding: 5px;white-space: nowrap;
                  font-size:12px;font-color:black;background-color:white;")

  ## Font opacity
  vn$nodes$LCS[is.na(vn$nodes$LCS)] <- max(vn$nodes$LCS, na.rm = TRUE)
  opacity_font <- sqrt(
    (vn$nodes$LCS - min(vn$nodes$LCS)) / diff(range(vn$nodes$LCS))
  ) *
    0.6 +
    0.4

  vn$nodes$size <- opacity_font * 5 * nodesize
  vn$nodes$size[nrow(vn$nodes)] <- max(5 * nodesize)

  for (i in 1:nrow(vn$nodes)) {
    vn$nodes$font.color[i] <- adjustcolor(
      vn$nodes$font.color[i],
      alpha.f = opacity_font[i]
    )
  }

  x <- vn$nodes$x
  y <- vn$nodes$y
  vn$nodes$x <- y
  vn$nodes$y <- x

  vn$nodes <- assign_horizontal_coords_clusters_adaptive(vn$nodes)

  vn$nodes$fixed.x <- FALSE
  vn$nodes$fixed.y <- TRUE

  coords <- vn$nodes[, c("x", "y")] %>%
    as.matrix()
  coords[, 2] <- coords[, 2]

  VIS <-
    visNetwork::visNetwork(
      nodes = vn$nodes,
      edges = vn$edges,
      type = "full",
      smooth = TRUE,
      physics = FALSE
    ) %>%
    visNetwork::visNodes(
      shadow = vn$nodes$shadow,
      shape = shape,
      size = vn$nodes$size,
      font = list(
        color = vn$nodes$font.color,
        size = vn$nodes$font.size,
        vadjust = vn$nodes$font.vadjust
      )
    ) %>%
    visNetwork::visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    #visNetwork::visEdges(smooth = list(type = "horizontal"), arrows = list(to = list(enabled = TRUE, scaleFactor = 0.5))) %>%
    visNetwork::visEdges(
      smooth = list(enabled = TRUE, type = "dynamic", roundness = 0.3),
      arrows = list(to = list(enabled = TRUE, scaleFactor = 0.5))
    ) %>%
    visNetwork::visInteraction(
      dragNodes = T,
      navigationButtons = F,
      hideEdgesOnDrag = F,
      tooltipStyle = tooltipStyle,
      zoomSpeed = 0.2
    ) %>%
    visNetwork::visOptions(
      highlightNearest = list(
        enabled = T,
        hover = T,
        degree = list(from = 1),
        algorithm = "hierarchical"
      ),
      nodesIdSelection = F,
      manipulation = FALSE,
      height = "100%",
      width = "100%"
    )

  return(list(VIS = VIS, vn = vn, type = "historiograph", curved = curved))
}

## calculate node coordinates in historiograph
assign_horizontal_coords_clusters_adaptive <- function(
  nodes_df,
  spacing_base = 1.0,
  cluster_spacing = 6,
  tol = 0.15
) {
  clusters <- nodes_df %>%
    count(color, name = "n_cluster") %>%
    arrange(desc(n_cluster)) %>%
    mutate(
      cluster_id = row_number(),
      cluster_center = (cluster_id - mean(cluster_id)) * cluster_spacing
    )

  nodes_df <- nodes_df %>%
    left_join(clusters, by = "color")

  nodes_df <- nodes_df %>%
    group_by(years, color) %>%
    mutate(
      n_nodes = n(),
      spacing = spacing_base * n_nodes, # USA direttamente il numero di nodi
      x = (cluster_center[1] + spacing[1] * (row_number() - (n() + 1) / 2)) *
        runif(1, 1 - tol, 1 + tol)
    ) %>%
    ungroup()

  return(nodes_df)
}

## Pajek Export
graph2Pajek <- function(graph, filename = "my_pajek_network") {
  nodes <- igraph::as_data_frame(graph, what = c("vertices")) %>%
    mutate(id = row_number())

  edges <- igraph::as_data_frame(graph, what = c("edges"))
  edges <- edges %>%
    left_join(nodes %>% select(id, name), by = c("from" = "name")) %>%
    rename(id_from = id) %>%
    left_join(nodes %>% select(id, name), by = c("to" = "name")) %>%
    rename(id_to = id)

  ### Creation of NET file
  file <- paste0(filename, ".net")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$id, ' "', nodes$name, '"'), file = file, append = T)

  # Edges
  write(paste0("*Edges ", nrow(nodes)), file = file, append = T)
  write(
    paste0(edges$id_from, " ", edges$id_to, " ", edges$weight),
    file = file,
    append = T
  )

  ### Creation of VEC file
  file <- paste0(filename, ".vec")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$deg), file = file, append = T)

  ### Creation of CLU file
  file <- paste0(filename, ".clu")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$community), file = file, append = T)
}


## Dendogram to Visnetwork
dend2vis <- function(hc, labelsize, nclusters = 1, community = FALSE) {
  # community = TRUE means that hc is an igraph community detection object
  # community = FALSE mean that hc is a hclust object

  # transform and plot a community igraph object using dendrogram
  if (community) {
    hc <- as.hclust(hc, use.modularity = TRUE)
  }

  h_tail <- round((max(hc$height) * 0.12), 1)

  hc$height <- hc$height + h_tail

  VIS <- visHclust(
    hc,
    cutree = nclusters,
    colorEdges = "grey60",
    horizontal = TRUE,
    export = FALSE
  )
  VIS$x$edges <- data.frame(color = unique(VIS$x$edges$color)) %>%
    mutate(new_color = colorlist()[1:nrow(.)]) %>%
    right_join(VIS$x$edges, by = "color") %>%
    select(-color) %>%
    rename(color = new_color)
  VIS$x$nodes <- VIS$x$nodes %>%
    mutate(
      label = ifelse(group != "individual", NA, label),
      group = ifelse(group == "individual", "word", group),
      title = gsub("individuals", "words", title),
      value = 1,
      scaling.min = 10,
      scaling.max = 10
    )
  coords <- VIS$x$nodes %>%
    select(x, y) %>%
    as.matrix()

  edges <- VIS$x$edges
  nodes <- VIS$x$nodes %>%
    select(id, label) %>%
    dplyr::filter(label != "1")

  VIS$x$edges <- edges %>%
    select(-id) %>%
    left_join(nodes, by = c("to" = "id")) %>%
    select(-label.x) %>%
    rename(label = label.y) %>%
    mutate(
      value = 10,
      font.color = color,
      font.size = labelsize * 10,
      font.vadjust = -0.2 * font.size,
      label = ifelse(is.na(label), "", label)
    )

  VIS <- VIS %>%
    visGroups(
      groupname = "group",
      color = "gray90",
      shape = "dot",
      size = 10
    ) %>%
    visGroups(
      groupname = "word",
      font = list(size = 0),
      color = list(
        background = "white",
        border = "#80B1D3",
        highlight = "#e2e9e9",
        hover = "orange"
      ),
      shape = "box"
    ) %>%
    visNodes(font = list(align = VIS$x$nodes$font.align)) %>%
    visNetwork::visOptions(
      highlightNearest = list(
        enabled = T,
        hover = T,
        degree = list(to = 1000, from = 0),
        algorithm = "hierarchical"
      ),
      nodesIdSelection = FALSE,
      manipulation = FALSE,
      height = "100%",
      width = "100%"
    ) %>%
    visNetwork::visInteraction(
      dragNodes = FALSE,
      navigationButtons = F,
      hideEdgesOnDrag = TRUE,
      zoomSpeed = 0.4
    ) %>%
    visIgraphLayout(
      layout = "layout.norm",
      layoutMatrix = coords,
      type = "full"
    ) %>%
    visEdges(font = list(align = "top", size = VIS$x$edges$font.size)) %>%
    visEvents(
      click = "function(nodes){
                  Shiny.onInputChange('click_dend', nodes.nodes[0]);
                  ;}"
    )

  for (i in 1:nrow(VIS$x$nodes)) {
    if (VIS$x$nodes$group[i] == "group") {
      old_inertia <- as.character(VIS$x$nodes$inertia[i])
      inertia <- as.character(VIS$x$nodes$inertia[i] - h_tail)
      VIS$x$nodes$title[i] <- gsub(old_inertia, inertia, VIS$x$nodes$title[i])
    }
  }

  VIS
}

## Factorial Analysis dynamic plots
ca2plotly <- function(
  CS,
  method = "MCA",
  dimX = 1,
  dimY = 2,
  topWordPlot = Inf,
  threshold = 0.10,
  labelsize = 16,
  size = 5
) {
  LABEL <- CS$WData$word
  switch(
    method,
    CA = {
      contrib <- rowSums(CS$coord$contrib %>% as.data.frame()) / 2
      wordCoord <- CS$coord$coord[, 1:2] %>%
        data.frame() %>%
        mutate(
          label = LABEL,
          contrib = contrib
        ) %>%
        select(c(3, 1, 2, 4))
      row.names(wordCoord) <- LABEL
      xlabel <- paste0("Dim 1 (", round(CS$res$eigCorr$perc[1], 2), "%)")
      ylabel <- paste0("Dim 2 (", round(CS$res$eigCorr$perc[2], 2), "%)")
    },
    MCA = {
      contrib <- rowSums(CS$coord$contrib %>% as.data.frame()) / 2
      wordCoord <- CS$coord$coord[, 1:2] %>%
        data.frame() %>%
        mutate(
          label = LABEL,
          contrib = contrib
        ) %>%
        select(c(3, 1, 2, 4))
      row.names(wordCoord) <- LABEL
      xlabel <- paste0("Dim 1 (", round(CS$res$eigCorr$perc[1], 2), "%)")
      ylabel <- paste0("Dim 2 (", round(CS$res$eigCorr$perc[2], 2), "%)")
    },
    MDS = {
      contrib <- size
      xlabel <- "Dim 1"
      ylabel <- "Dim 2"
      wordCoord <- CS$WData %>%
        data.frame() %>%
        select(1:3) %>%
        mutate(contrib = contrib / 2) %>%
        rename(label = "word")
    }
  )

  dimContrLabel <- paste0("Contrib", c(dimX, dimY))
  ymax <- diff(range((wordCoord[, 3])))
  xmax <- diff(range((wordCoord[, 2])))
  threshold2 <- threshold * mean(xmax, ymax)

  # scaled size for dots
  dotScale <- (wordCoord$contrib) + size

  # Threshold labels to plot
  thres <- sort(dotScale, decreasing = TRUE)[min(topWordPlot, nrow(wordCoord))]

  names(wordCoord)[2:3] <- c("Dim1", "Dim2")

  wordCoord <- wordCoord %>%
    mutate(
      dotSize = dotScale,
      groups = CS$km.res$cluster,
      labelToPlot = ifelse(dotSize >= thres, label, ""),
      font.color = ifelse(
        labelToPlot == "",
        NA,
        adjustcolor(colorlist()[groups], alpha.f = 0.85)
      ),
      font.size = round(dotSize * 2, 0)
    )

  ## Avoid label overlapping
  labelToRemove <- avoidOverlaps(
    wordCoord,
    threshold = threshold2,
    dimX = dimX,
    dimY = dimY
  )
  wordCoord <- wordCoord %>%
    mutate(
      labelToPlot = ifelse(labelToPlot %in% labelToRemove, "", labelToPlot)
    ) %>%
    mutate(
      label = gsub("_1", "", label),
      labelToPlot = gsub("_1", "", labelToPlot)
    )

  hoverText <- paste(
    " <b>",
    wordCoord$label,
    "</b>\n Contribute: ",
    round(wordCoord$contrib, 3),
    sep = ""
  )

  fig <- plot_ly(
    data = wordCoord,
    x = wordCoord[, "Dim1"],
    y = wordCoord[, "Dim2"], # customdata=results$wordCoord,
    type = "scatter",
    mode = "markers",
    marker = list(
      size = dotScale,
      color = adjustcolor(colorlist()[wordCoord$groups], alpha.f = 0.3), #' rgb(79, 121, 66, .5)',
      line = list(
        color = adjustcolor(colorlist()[wordCoord$groups], alpha.f = 0.3), #' rgb(79, 121, 66, .8)',
        width = 2
      )
    ),
    text = hoverText,
    hoverinfo = "text",
    alpha = .3
  )

  fig <- fig %>%
    layout(
      yaxis = list(
        title = ylabel,
        showgrid = TRUE,
        showline = FALSE,
        showticklabels = TRUE,
        domain = c(0, 1)
      ),
      xaxis = list(
        title = xlabel,
        zeroline = TRUE,
        showgrid = TRUE,
        showline = FALSE,
        showticklabels = TRUE
      ),
      plot_bgcolor = "rgba(0, 0, 0, 0)",
      paper_bgcolor = "rgba(0, 0, 0, 0)"
    )

  for (i in seq_len(max(wordCoord$groups))) {
    if (method == "MDS") {
      w <- wordCoord %>%
        dplyr::filter(groups == i) %>%
        mutate(
          Dim1 = Dim1 + 0.005,
          Dim2 = Dim2 + 0.005
        )
    } else {
      w <- wordCoord %>%
        dplyr::filter(groups == i) %>%
        mutate(
          Dim1 = Dim1 + dotSize * 0.005,
          Dim2 = Dim2 + dotSize * 0.01
        )
    }

    if (max(CS$hull_data$clust) > 1) {
      hull_df <- CS$hull_data %>% dplyr::filter(clust == i)
      fig <- fig %>%
        add_polygons(
          x = hull_df$Dim1,
          y = hull_df$Dim2,
          inherit = FALSE,
          showlegend = FALSE,
          color = I(hull_df$color[1]),
          opacity = 0.3,
          line = list(width = 2),
          text = paste0("Cluster ", i),
          hoverinfo = "text",
          hoveron = "points"
        )
    }
    fig <- fig %>%
      add_annotations(
        data = w,
        x = ~Dim1,
        y = ~Dim2,
        xref = "x1",
        yref = "y",
        text = ~labelToPlot,
        font = list(
          family = "sans serif",
          size = labelsize,
          color = w$font.color[1]
        ), #' rgb(79, 121, 66)'),
        showarrow = FALSE
      )
  }

  fig <- fig %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        #' toImage',
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    ) %>%
    event_register("plotly_selecting")
  return(fig)
}


## function to avoid label overlapping ----
avoidOverlaps <- function(w, threshold = 0.10, dimX = 1, dimY = 2) {
  w[, "Dim2"] <- w[, "Dim2"] / 3

  Ds <- dist(
    w %>%
      dplyr::filter(labelToPlot != "") %>%
      select(Dim1, Dim2),
    method = "manhattan",
    upper = T
  ) %>%
    dist2df() %>%
    rename(
      from = row,
      to = col,
      dist = value
    ) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("from" = "labelToPlot")
    ) %>%
    rename(w_from = dotSize) %>%
    left_join(
      w %>% dplyr::filter(labelToPlot != "") %>% select(labelToPlot, dotSize),
      by = c("to" = "labelToPlot")
    ) %>%
    rename(w_to = dotSize) %>%
    filter(dist < threshold)

  st <- TRUE
  i <- 1
  label <- NULL
  case <- "n"

  while (isTRUE(st)) {
    if (Ds$w_from[i] > Ds$w_to[i] & Ds$dist[i] < threshold) {
      case <- "y"
      lab <- Ds$to[i]
    } else if (Ds$w_from[i] <= Ds$w_to[i] & Ds$dist[i] < threshold) {
      case <- "y"
      lab <- Ds$from[i]
    }

    switch(
      case,
      "y" = {
        Ds <- Ds[Ds$from != lab, ]
        Ds <- Ds[Ds$to != lab, ]
        label <- c(label, lab)
      },
      "n" = {
        Ds <- Ds[-1, ]
      }
    )

    if (i >= nrow(Ds)) {
      st <- FALSE
    }
    case <- "n"
    # print(nrow(Ds))
  }

  label
}

## convert a distance object into a data.frame ----
dist2df <- function(inDist) {
  if (class(inDist) != "dist") {
    stop("wrong input type")
  }
  A <- attr(inDist, "Size")
  B <- if (is.null(attr(inDist, "Labels"))) {
    sequence(A)
  } else {
    attr(inDist, "Labels")
  }
  if (isTRUE(attr(inDist, "Diag"))) {
    attr(inDist, "Diag") <- FALSE
  }
  if (isTRUE(attr(inDist, "Upper"))) {
    attr(inDist, "Upper") <- FALSE
  }
  data.frame(
    row = B[unlist(lapply(sequence(A)[-1], function(x) x:A))],
    col = rep(B[-length(B)], (length(B) - 1):1),
    value = as.vector(inDist)
  )
}

### Excel report functions
addDataWb <- function(list_df, wb, sheetname) {
  l <- length(list_df)
  startRow <- 1
  for (i in 1:l) {
    df <- list_df[[i]]
    n <- nrow(df)
    writeDataTable(
      wb,
      sheetname,
      df,
      startRow = startRow,
      startCol = 1,
      tableStyle = "TableStyleMedium20"
    )
    startRow <- startRow + n + 3
  }
  return(wb)
}

addDataScreenWb <- function(list_df, wb, sheetname) {
  ind <- which(regexpr(sheetname, wb$sheet_names) > -1)
  if (length(ind) > 0) {
    sheetname <- paste(sheetname, "(", length(ind) + 1, ")", sep = "")
  }
  addWorksheet(wb = wb, sheetName = sheetname, gridLines = FALSE)
  if (!is.null(list_df)) {
    addDataWb(list_df, wb, sheetname)
    col <- max(unlist(lapply(list_df, ncol))) + 2
  } else {
    col <- 1
  }

  results <- list(wb = wb, col = col, sheetname = sheetname)
  return(results)
}

addGgplotsWb <- function(
  list_plot,
  wb,
  sheetname,
  col,
  width = 10,
  height = 7,
  dpi = 75
) {
  l <- length(list_plot)
  startRow <- 1
  for (i in seq_len(l)) {
    p <- list_plot[[i]]

    # Defensive guards. Without them, a NULL plot (e.g. user clicks the
    # report button before the corresponding tab has finished rendering)
    # or an unrecognised class would skip every writer branch below and
    # then crash openxlsx::insertImage with "File does not exist".
    if (is.null(p)) {
      warning(sprintf("Plot %d for sheet '%s' is NULL — skipped from report.",
                      i, sheetname), call. = FALSE)
      next
    }

    fileName <- file.path(
      getWD(),
      paste0("figureImage_", i, "_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".png")
    )

    written <- tryCatch({
      if (inherits(p, "ggplot")) {
        safe_ggsave(
          plot = p,
          filename = fileName,
          width = width,
          height = height,
          units = "in",
          dpi = dpi
        )
        TRUE
      } else if (inherits(p, "igraph")) {
        igraph2PNG(
          x = p,
          filename = fileName,
          width = width,
          height = height,
          dpi = dpi
        )
        TRUE
      } else if (inherits(p, "bibliodendrogram")) {
        png_dev <- safe_png_device()
        png_dev(
          filename = fileName,
          width = width,
          height = height,
          res = 300,
          units = "in"
        )
        plot(p)
        dev.off()
        TRUE
      } else {
        warning(sprintf(
          "Plot %d for sheet '%s' has unsupported class (%s) — skipped from report.",
          i, sheetname, paste(class(p), collapse = "/")
        ), call. = FALSE)
        FALSE
      }
    }, error = function(e) {
      warning(sprintf(
        "Could not export plot %d for sheet '%s': %s",
        i, sheetname, conditionMessage(e)
      ), call. = FALSE)
      FALSE
    })

    if (!isTRUE(written) || !file.exists(fileName)) {
      next
    }

    insertImage(
      wb = wb,
      sheet = sheetname,
      file = fileName,
      width = width,
      height = height,
      startRow = startRow,
      startCol = col,
      units = "in",
      dpi = dpi
    )
    startRow <- startRow + (height * 6) + 1
  }
  return(wb)
}

screenSh <- function(p, type = "vis", dpi = 300, height = 7) {
  #tmpdir <- getWD()

  fileName <- file.path(
    getWD(),
    paste0("figureImage_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".png")
  )

  plot2png(p, filename = fileName, type = type, dpi = dpi, height = height)

  return(fileName)
}

screenShot <- function(p, filename, type, dpi = 300, height = 7) {
  #home <- homeFolder()

  # # setting up the main directory
  # if ("downloads" %in% tolower(dir(home))) {
  #   filename <- file.path(home, "downloads", filename)
  # } else {
  #   filename <- file.path(home, filename)
  # }

  plot2png(p, filename, type = type, dpi = dpi, height = height)
  filename_html <- paste0(tools::file_path_sans_ext(filename), ".html")
  file.remove(filename_html)
  unlink(
    paste0(tools::file_path_sans_ext(filename), "_files"),
    recursive = TRUE
  )
}

plot2png <- function(p, filename, type = "vis", dpi = 300, height = 7) {
  zoom <- dpi / 96
  vwidth  <- round(height * 2 * 96)
  vheight <- round(height * 96)

  filename_html <- tools::file_path_sans_ext(filename)
  html_name <- paste0(filename_html, ".html")
  switch(
    type,
    vis = {
      visSave(p, html_name)
    },
    plotly = {
      htmlwidgets::saveWidget(p, file = html_name, selfcontained = FALSE)
    },
    df2html = {
      screenHtml(p, html_name)
    }
  )
  biblioShot(url = html_name, zoom = zoom, vwidth = vwidth, vheight = vheight, file = filename)

  popUpGeneric(
    title = NULL,
    type = "success",
    color = c("#1d8fe1"),
    subtitle = paste0("Plot saved to file:\n", basename(filename)),
    btn_labels = "OK",
    size = "40%"
  )
}

## Export Plotly Sankey to PNG using Plotly.toImage() via chromote
## Chromote has a rendering bug with complex SVG paths (Sankey links
## render at ~5% opacity). Plotly.toImage() uses canvas rendering instead.
plotlySankey2png <- function(p, filename, dpi = 300, height = 7) {
  vw <- max(round(height * 2 * 96), 1344)
  vh <- max(round(height * 96), 672)
  img_scale <- dpi / 96

  export_plot <- p %>%
    htmlwidgets::onRender(sprintf("
      function(el, x) {
        var plotDiv = el.querySelector('.js-plotly-plot') || el;
        Plotly.toImage(plotDiv, {format:'png', width:%d, height:%d, scale:%f}).then(function(url) {
          document.title = url;
        });
      }
    ", vw, vh, img_scale))

  html_name <- paste0(tools::file_path_sans_ext(filename), ".html")
  htmlwidgets::saveWidget(export_plot, file = html_name, selfcontained = TRUE)

  b <- chromote::ChromoteSession$new(width = vw, height = vh)
  tryCatch({
    p_event <- b$Page$loadEventFired(wait_ = FALSE)
    html_url <- paste0("file:///", normalizePath(html_name, winslash = "/", mustWork = TRUE))
    b$Page$navigate(html_url)
    b$wait_for(p_event)
    Sys.sleep(3)
    result <- b$Runtime$evaluate("document.title")
    base64_str <- result$result$value
    if (grepl("^data:image/png;base64,", base64_str)) {
      raw_data <- base64enc::base64decode(sub("^data:image/png;base64,", "", base64_str))
      writeBin(raw_data, filename)
    } else {
      # Fallback: call Plotly.toImage directly
      result2 <- b$Runtime$evaluate(sprintf("
        new Promise(function(resolve) {
          var plotDiv = document.querySelector('.js-plotly-plot');
          if (plotDiv) {
            Plotly.toImage(plotDiv, {format:'png', width:%d, height:%d, scale:%f}).then(resolve);
          } else { resolve(''); }
        })
      ", vw, vh, img_scale), awaitPromise = TRUE)
      base64_str2 <- result2$result$value
      if (nchar(base64_str2) > 100) {
        raw_data <- base64enc::base64decode(sub("^data:image/png;base64,", "", base64_str2))
        writeBin(raw_data, filename)
      } else {
        # Last resort: use biblioShot (links may be faint)
        biblioShot(url = html_name, file = filename,
          zoom = dpi / 96, vwidth = vw, vheight = vh, delay = 2)
      }
    }
  }, finally = {
    try(b$close(), silent = TRUE)
  })
  unlink(html_name)
  unlink(paste0(tools::file_path_sans_ext(html_name), "_files"), recursive = TRUE)
}

## screenshot of html objects
screenHtml <- function(df, html_file) {
  # Genera l'HTML direttamente usando htmlBoxFormat
  html_content <- htmlBoxFormat(
    df = df,
    title = "Missing Data Analysis",
    nrow = nrow(df),
    escape = FALSE,
    scrollX = TRUE,
    dom = FALSE,
    filter = "none",
    pagelength = FALSE,
    button = FALSE,
    round = 2
  )

  # Crea una pagina HTML completa con stili
  html_page <- sprintf(
    '
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Missing Data Table</title>
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css">
  <script src="https://cdn.jsdelivr.net/npm/xlsx@0.18.5/dist/xlsx.full.min.js"></script>
  <style>
    body {
      padding: 20px;
      background-color: white;
      font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;
    }
    .container-fluid {
      max-width: 100%%;
    }
  </style>
</head>
<body>
  <div class="container-fluid">
    %s
  </div>
</body>
</html>
    ',
    html_content
  )

  # Salva il file HTML
  writeLines(html_page, html_file, useBytes = TRUE)
}

## screenshot of an Author Profile card (HTML) to PNG ----
## The card (Global or Local profile) is a static HTML tag built with inline
## styles + the Bootstrap grid. It is wrapped in a standalone HTML page and
## captured with biblioShot(), cropping to the card via a CSS selector.
authorCard2png <- function(card, filename, selector = ".author-bio-card",
                           dpi = 300, height = 7) {
  zoom <- dpi / 96
  html_file <- paste0(tools::file_path_sans_ext(filename), ".html")
  out_dir <- dirname(html_file)

  # Make the card images available next to the HTML (cards use relative src).
  for (img in c("ORCID.jpg", "openalex.jpg")) {
    src <- file.path("www", img)
    if (file.exists(src)) {
      file.copy(src, file.path(out_dir, img), overwrite = TRUE)
    }
  }

  card_html <- as.character(card)

  html_page <- sprintf(
    '<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css">
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css">
  <style>
    body { margin: 0; padding: 25px; background: #ffffff;
           font-family: "Segoe UI", Tahoma, sans-serif; }
    .biblio-card-wrap { width: 950px; }
  </style>
</head>
<body>
  <div class="biblio-card-wrap">%s</div>
</body>
</html>',
    card_html
  )
  writeLines(html_page, html_file, useBytes = TRUE)

  biblioShot(
    url = html_file,
    file = filename,
    selector = selector,
    expand = c(20, 20, 20, 20),
    zoom = zoom,
    vwidth = 1100,
    vheight = 1400,
    delay = 1.2
  )

  unlink(html_file)
  filename
}

## export screen to browser download
screen2export <- function(filename = "file", obj, type, dpi = 300, height = 7) {
  JScode_screenshot <- "
    var link = document.createElement('a');
    link.href = '%s';
    link.download = '%s';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  "

  base_name <- paste0(
    filename,
    "-",
    gsub(" |:", "", Sys.time()),
    ".png",
    sep = ""
  )
  full_path <- file.path(getWD(), base_name)
  screenShot(
    obj,
    filename = full_path,
    type = type,
    dpi = dpi,
    height = height
  )
  img_data <- base64enc::dataURI(file = full_path, mime = "image/png")
  shinyjs::runjs(sprintf(JScode_screenshot, img_data, base_name))
}

addScreenWb <- function(df, wb, width = 14, height = 8, dpi = 75) {
  names(df) <- c("sheet", "file", "n")
  if (nrow(df) > 0) {
    sheet <- unique(df$sheet)
    for (i in 1:length(sheet)) {
      sh <- sheet[i]
      df_sh <- df %>% dplyr::filter(sheet == sh)
      l <- nrow(df_sh)
      startRow <- 1
      for (j in 1:l) {
        fileName <- df_sh$file[j]
        if (is.null(fileName) || is.na(fileName) || !nzchar(fileName) ||
            !file.exists(fileName)) {
          warning(sprintf(
            "Screenshot file '%s' for sheet '%s' is missing — skipped from report.",
            fileName %||% "<NA>", sh
          ), call. = FALSE)
          next
        }
        insertImage(
          wb = wb,
          sheet = sh,
          file = fileName,
          width = width,
          height = height,
          startRow = startRow,
          startCol = df_sh$n[j],
          units = "in",
          dpi = dpi
        )
        startRow <- startRow + (height * 10) + 3
      }
    }
  }
  return(wb)
}

addSheetToReport <- function(list_df, list_plot, sheetname, wb, dpi = 75) {
  ind <- which(regexpr(sheetname, wb$sheet_names) > -1)
  if (length(ind) > 0) {
    sheetname <- paste(sheetname, "(", length(ind) + 1, ")", sep = "")
  }
  addWorksheet(wb, sheetname, gridLines = FALSE)

  if (!is.null(list_df)) {
    col <- max(unlist(lapply(list_df, ncol))) + 2
    wb <- addDataWb(list_df, wb = wb, sheetname = sheetname)
  } else {
    col <- 1
  }

  if (!is.null(list_plot)) {
    wb <- addGgplotsWb(
      list_plot,
      wb = wb,
      sheetname = sheetname,
      col = col,
      dpi = dpi
    )
  }
  # values$sheet_name <- sheetname
  return(wb)
}

short2long <- function(df, myC) {
  z <- unlist(lapply(myC, function(x) {
    y <- gsub(r"{\s*\([^\)]+\)}", "", x)
    gsub(y, df$long[df$short == y], x)
  }))
  names(myC) <- z
  return(myC)
}

dfLabel <- function() {
  short <- c(
    "Empty Report",
    "MissingData",
    "MainInfo",
    "AnnualSciProd",
    "AnnualCitPerYear",
    "LifeCycle",
    "ThreeFieldsPlot",
    "MostRelSources",
    "MostLocCitSources",
    "BradfordLaw",
    "SourceLocImpact",
    "SourceProdOverTime",
    "MostRelAuthors",
    "MostLocCitAuthors",
    "AuthorProdOverTime",
    "LotkaLaw",
    "AuthorLocImpact",
    "MostRelAffiliations",
    "AffOverTime",
    "CorrAuthCountries",
    "CountrySciProd",
    "CountryProdOverTime",
    "MostCitCountries",
    "MostGlobCitDocs",
    "MostLocCitDocs",
    "MostLocCitRefs",
    "RPYS",
    "MostFreqWords",
    "WordCloud",
    "TreeMap",
    "WordFreqOverTime",
    "TrendTopics",
    "CouplingMap",
    "CoWordNet",
    "ThematicMap",
    "ThematicEvolution",
    "TE_Period_1",
    "TE_Period_2",
    "TE_Period_3",
    "TE_Period_4",
    "TE_Period_5",
    "FactorialAnalysis",
    "CoCitNet",
    "Historiograph",
    "CollabNet",
    "CollabWorldMap",
    "PRISMA_Diagram"
  )

  long <- c(
    "Empty Report",
    "Missing Data Table",
    "Main Information",
    "Annual Scientific Production",
    "Annual Citation Per Year",
    "Life Cycle of Publications",
    "Three-Field Plot",
    "Most Relevant Sources",
    "Most Local Cited Sources",
    "Bradfords Law",
    "Sources Local Impact",
    "Sources Production over Time",
    "Most Relevant Authors",
    "Most Local Cited Authors",
    "Authors Production over Time",
    "Lotkas Law",
    "Authors Local Impact",
    "Most Relevant Affiliations",
    "Affiliations Production over Time",
    "Corresponding Authors Countries",
    "Countries Scientific Production",
    "Countries Production over Time",
    "Most Cited Countries",
    "Most Global Cited Documents",
    "Most Local Cited Documents",
    "Most Local Cited References",
    "Reference Spectroscopy",
    "Most Frequent Words",
    "WordCloud",
    "TreeMap",
    "Words Frequency over Time",
    "Trend Topics",
    "Clustering by Coupling",
    "Co-occurence Network",
    "Thematic Map",
    "Thematic Evolution",
    "TE_Period_1",
    "TE_Period_2",
    "TE_Period_3",
    "TE_Period_4",
    "TE_Period_5",
    "Factorial Analysis",
    "Co-citation Network",
    "Historiograph",
    "Collaboration Network",
    "Countries Collaboration World Map",
    "PRISMA Diagram"
  )
  data.frame(short = short, long = long)
}

## Generic PopUp
popUpGeneric <- function(
  title = NULL,
  type = "success",
  color = c("#1d8fe1", "#913333", "#FFA800"),
  subtitle = "",
  btn_labels = "OK",
  ## "40%" is not a valid show_alert() size (the enum is xs/s/m/l/xl) and was
  ## silently dropped to the default, producing a very narrow modal. "l" gives
  ## these tabular pop-ups (Synonym List, Stopword List, ...) enough width to
  ## display single-word terms without mid-word breaks.
  size = "l"
) {
  showButton <- TRUE
  timer <- NA
  show_alert(
    title = title,
    text = subtitle,
    type = type,
    size = size,
    closeOnEsc = TRUE,
    closeOnClickOutside = TRUE,
    html = FALSE,
    showConfirmButton = showButton,
    showCancelButton = FALSE,
    btn_labels = btn_labels,
    btn_colors = color,
    timer = timer,
    imageUrl = "",
    animation = TRUE
  )
}


## Ad to Report PopUp
popUp <- function(title = NULL, type = "success", btn_labels = "OK") {
  switch(
    type,
    success = {
      title <- paste(title, "\n added to report", sep = "")
      subtitle <- ""
      btn_colors <- "#1d8fe1"
      showButton <- TRUE
      timer <- 3000
    },
    error = {
      title <- "No results to add to the report "
      subtitle <- "Please Run the analysis and then Add it to the report"
      btn_colors <- "#913333"
      showButton <- TRUE
      timer <- 3000
    },
    waiting = {
      title <- "Please wait... "
      subtitle <- "Adding results to report"
      btn_colors <- "#FFA800"
      showButton <- FALSE
      btn_labels <- NA
      timer <- NA
    }
  )

  show_alert(
    title = title,
    text = subtitle,
    type = type,
    size = "s",
    closeOnEsc = TRUE,
    closeOnClickOutside = TRUE,
    html = FALSE,
    showConfirmButton = showButton,
    showCancelButton = FALSE,
    btn_labels = btn_labels,
    btn_colors = btn_colors,
    timer = timer,
    imageUrl = "",
    animation = TRUE
  )
}

colorlist <- function() {
  c(
    "#E41A1C",
    "#377EB8",
    "#4DAF4A",
    "#984EA3",
    "#FF7F00",
    "#A65628",
    "#F781BF",
    "#999999",
    "#66C2A5",
    "#FC8D62",
    "#8DA0CB",
    "#E78AC3",
    "#A6D854",
    "#FFD92F",
    "#B3B3B3",
    "#A6CEE3",
    "#1F78B4",
    "#B2DF8A",
    "#33A02C",
    "#FB9A99",
    "#E31A1C",
    "#FDBF6F",
    "#FF7F00",
    "#CAB2D6",
    "#6A3D9A",
    "#B15928",
    "#8DD3C7",
    "#BEBADA",
    "#FB8072",
    "#80B1D3",
    "#FDB462",
    "#B3DE69",
    "#D9D9D9",
    "#BC80BD",
    "#CCEBC5"
  )
}

overlayPlotly <- function(VIS) {
  # colorscale_VOS=matrix(c(0, 'rgba(66,65,135,255)', 0.1, 'rgba(34,170,134,255)',
  #                         0.3, 'rgba(202,224,31,255)',
  #                         1, 'rgba(244,227,92,255)'),4,2, byrow=T)

  # colorscale_Our=matrix(c(0, 'rgba(238,238,238,255)',
  #                         0.1, 'rgba(232,202,177,255)',
  #                         0.2, 'rgba(217,137,100,255)',
  #                         0.6, 'rgba(199,107,90,255)',
  #                         0.9, 'rgba(164,38,39,255)',
  #                         1,   'rgba(178,34,34,255)'),
  #                       6,2, byrow=T)

  Reds <- matrix(
    c(
      "0",
      "rgb(255,255,255)",
      "0.05",
      "rgb(238,238,238)",
      "0.125",
      "rgb(254,224,210)",
      "0.25",
      "rgb(252,187,161)",
      "0.375",
      "rgb(252,146,114)",
      "0.5",
      "rgb(251,106,74)",
      "0.625",
      "rgb(239,59,44)",
      "0.75",
      "rgb(203,24,29)",
      "0.875",
      "rgb(165,15,21)",
      "1",
      "rgb(103,0,13)"
    )
  )

  nodes <- VIS$x$nodes %>%
    mutate(
      y = y * (-1),
      font.size = (((font.size - min(font.size)) / diff(range(font.size))) *
        20) +
        10
    )

  colori <- c(
    "Blackbody",
    "Bluered",
    "Blues",
    "Cividis",
    "Earth",
    "Electric",
    "Greens",
    "Greys",
    "Hot",
    "Jet",
    "Picnic",
    "Portland",
    "Rainbow",
    "RdBu",
    "Reds",
    "Viridis",
    "YlGnBu",
    "YlOrRd"
  )

  nodes2 <- nodes %>%
    group_by(id) %>%
    mutate(log = ceiling(log(deg))) %>%
    slice(rep(1, each = log))

  # Compute 2D kernel density estimation for smooth heatmap
  x_range <- range(nodes2$x, na.rm = TRUE)
  y_range <- range(nodes2$y, na.rm = TRUE)
  x_pad <- diff(x_range) * 0.15
  y_pad <- diff(y_range) * 0.15

  kde <- tryCatch({
    bw_x <- diff(x_range) / 4
    bw_y <- diff(y_range) / 4
    dens <- MASS::kde2d(nodes2$x, nodes2$y, n = 300,
                        h = c(bw_x, bw_y),
                        lims = c(x_range[1] - x_pad, x_range[2] + x_pad,
                                 y_range[1] - y_pad, y_range[2] + y_pad))
    list(x = dens$x, y = dens$y, z = dens$z)
  }, error = function(e) NULL)

  if (!is.null(kde)) {
    # z from kde2d is [x_index, y_index]; plotly heatmap expects z[y_index, x_index]
    p <- plotly::plot_ly(
      x = kde$x, y = kde$y, z = t(kde$z),
      type = "heatmap",
      colorscale = Reds,
      showscale = FALSE,
      zsmooth = "best",
      hoverinfo = "none"
    )
  } else {
    # Fallback to histogram2d
    p <- plot_ly(nodes2, x = ~x, y = ~y) %>%
      add_histogram2d(
        histnorm = "density",
        zsmooth = "fast",
        colorscale = Reds,
        showscale = FALSE
      )
  }

  for (i in 1:nrow(nodes)) {
    p <- p %>%
      add_annotations(
        xref = "x1",
        yref = "y",
        x = nodes$x[i],
        y = nodes$y[i],
        text = nodes$label[i],
        xanchor = "center",
        font = list(
          family = "Arial",
          size = nodes$font.size[i],
          color = adjustcolor(nodes$font.color[i], alpha.f = 0.8)
        ),
        showarrow = FALSE
      )
  }
  p <- p %>%
    layout(
      yaxis = list(
        title = "",
        zeroline = FALSE,
        showgrid = FALSE,
        showline = FALSE,
        showticklabels = FALSE,
        domain = c(-1, 1),
        gridcolor = "#FFFFFF",
        tickvals = list(NA)
      ),
      xaxis = list(
        title = "",
        zeroline = FALSE,
        showgrid = FALSE,
        showline = FALSE,
        showticklabels = FALSE,
        domain = c(-1, 1),
        gridcolor = "#FFFFFF",
        tickvals = list(NA)
      ),
      plot_bgcolor = "rgba(0, 0, 0, 0)",
      paper_bgcolor = "rgba(0, 0, 0, 0)",
      showlegend = FALSE
    ) %>%
    style(hoverinfo = "none") %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        #' toImage',
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    )
  return(p)
}


# MENU VISIBILITY MANAGEMENT ----
# This function updates visibility of submenu items based on metadata availability
updateMenuVisibility <- function(session, values) {
  # Extract metadata availability flags
  TC <- !("TC" %in% values$missTags)
  MLCS <- MLCA <- ("ISI" %in% values$M$DB[1] & !"CR" %in% values$missTags)
  ISI <- ("ISI" %in% values$M$DB[1])
  AFF <- !("C1" %in% values$missTags)
  CR <- !("CR" %in% values$missTags)
  MCC <- (!"TC" %in% values$missTags & !"C1" %in% values$missTags)
  DB_CR <- (sum(c("SCOPUS", "ISI") %in% values$M$DB[1]) > 0)
  DB_TC <- (sum(c("SCOPUS", "ISI", "OPENALEX", "LENS") %in% values$M$DB[1]) > 0)

  # Build JavaScript code to show/hide submenu items based on metadata
  js_code <- "
    // Helper function to find submenu item by text within a parent
    function findSubmenuItem(parentText, itemText) {
      var parent = null;
      $('.sidebar-menu > .treeview').each(function() {
        if ($(this).find('> a > span').first().text().trim() === parentText) {
          parent = $(this);
          return false;
        }
      });
      
      if (parent) {
        var found = null;
        parent.find('ul.treeview-menu > li').each(function() {
          var text = $(this).find('a span').first().text().trim();
          if (text === itemText) {
            found = $(this);
            return false;
          }
        });
        return found;
      }
      return null;
    }
    
    // Helper function to find section header (text-only li)
    function findSectionHeader(parentText, headerText) {
      var parent = null;
      $('.sidebar-menu > .treeview').each(function() {
        if ($(this).find('> a > span').first().text().trim() === parentText) {
          parent = $(this);
          return false;
        }
      });
      
      if (parent) {
        var found = null;
        parent.find('ul.treeview-menu > li').each(function() {
          if ($(this).children('a').length === 0 && $(this).text().trim() === headerText) {
            found = $(this);
            return false;
          }
        });
        return found;
      }
      return null;
    }
  "

  # Add conditional visibility for Sources submenu items
  if (MLCS) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Most Local Cited Sources')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Most Local Cited Sources')?.hide();"
    )
  }

  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Sources\\' Local Impact')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Sources', 'Sources\\' Local Impact')?.hide();"
    )
  }

  # Add conditional visibility for Authors submenu items
  if (MLCA) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Local Cited Authors')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Local Cited Authors')?.hide();"
    )
  }

  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Authors\\' Local Impact')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Authors\\' Local Impact')?.hide();"
    )
  }

  # Affiliations section
  if (AFF) {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Authors', 'Affiliations')?.show();
    findSubmenuItem('Authors', 'Most Relevant Affiliations')?.show();
    findSubmenuItem('Authors', 'Affiliations\\' Production over Time')?.show();
    findSectionHeader('Authors', 'Countries')?.show();
    findSubmenuItem('Authors', 'Corresponding Author\\'s Countries')?.show();
    findSubmenuItem('Authors', 'Countries\\' Scientific Production')?.show();
    findSubmenuItem('Authors', 'Countries\\' Production over Time')?.show();
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Authors', 'Affiliations')?.hide();
    findSubmenuItem('Authors', 'Most Relevant Affiliations')?.hide();
    findSubmenuItem('Authors', 'Affiliations\\' Production over Time')?.hide();
    findSectionHeader('Authors', 'Countries')?.hide();
    findSubmenuItem('Authors', 'Corresponding Author\\'s Countries')?.hide();
    findSubmenuItem('Authors', 'Countries\\' Scientific Production')?.hide();
    findSubmenuItem('Authors', 'Countries\\' Production over Time')?.hide();
    "
    )
  }

  if (MCC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Cited Countries')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Authors', 'Most Cited Countries')?.hide();"
    )
  }

  # Documents submenu items
  if (TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Global Cited Documents')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Global Cited Documents')?.hide();"
    )
  }

  if (DB_TC && CR && TC) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Local Cited Documents')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Documents', 'Most Local Cited Documents')?.hide();"
    )
  }

  # Cited References section
  if (DB_CR) {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Documents', 'Cited References')?.show();
    findSubmenuItem('Documents', 'Most Local Cited References')?.show();
    findSubmenuItem('Documents', 'References Spectroscopy')?.show();
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    findSectionHeader('Documents', 'Cited References')?.hide();
    findSubmenuItem('Documents', 'Most Local Cited References')?.hide();
    findSubmenuItem('Documents', 'References Spectroscopy')?.hide();
    "
    )
  }

  # Intellectual Structure - hide entire menu if no CR
  if (!CR) {
    js_code <- paste0(
      js_code,
      "
    $('.sidebar-menu .treeview').each(function() {
      if ($(this).find('> a > span').first().text().trim() === 'Intellectual Structure') {
        $(this).hide();
      }
    });
    "
    )
  } else {
    js_code <- paste0(
      js_code,
      "
    $('.sidebar-menu .treeview').each(function() {
      if ($(this).find('> a > span').first().text().trim() === 'Intellectual Structure') {
        $(this).show();
      }
    });
    "
    )

    if (DB_TC && CR) {
      js_code <- paste0(
        js_code,
        "\nfindSubmenuItem('Intellectual Structure', 'Historiograph')?.show();"
      )
    } else {
      js_code <- paste0(
        js_code,
        "\nfindSubmenuItem('Intellectual Structure', 'Historiograph')?.hide();"
      )
    }
  }

  # Social Structure
  if (AFF) {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Social Structure', 'Countries\\' Collaboration World Map')?.show();"
    )
  } else {
    js_code <- paste0(
      js_code,
      "\nfindSubmenuItem('Social Structure', 'Countries\\' Collaboration World Map')?.hide();"
    )
  }

  # Execute the JavaScript code
  shinyjs::runjs(js_code)

  # Update list of disabled menu items for navigation control
  out <- character(0)

  if (!isTRUE(TC)) {
    out <- c(
      out,
      "Average Citations per Year",
      "Sources' Local Impact",
      "Authors' Local Impact",
      "Most Global Cited Documents"
    )
  }
  if (!isTRUE(MLCS)) {
    out <- c(out, "Most Local Cited Sources")
  }
  if (!isTRUE(ISI)) {
    out <- c(out, "Most Local Cited Authors")
  }
  if (!isTRUE(AFF)) {
    out <- c(
      out,
      "Most Relevant Affiliations",
      "Affiliations' Production over Time",
      "Corresponding Author's Countries",
      "Countries' Scientific Production",
      "Countries' Production over Time",
      "Countries' Collaboration World Map"
    )
  }
  if (!isTRUE(MCC)) {
    out <- c(out, "Most Cited Countries")
  }
  if (!(isTRUE(DB_TC) & isTRUE(CR) & isTRUE(TC))) {
    out <- c(out, "Most Local Cited Documents")
  }
  if (!isTRUE(DB_CR)) {
    out <- c(out, "Most Local Cited References", "References Spectroscopy")
  }
  if (!isTRUE(CR)) {
    out <- c(out, "Co-citation Network")
  }
  if (!(isTRUE(DB_TC) & isTRUE(CR))) {
    out <- c(out, "Historiograph")
  }

  values$out <- out
}

# find home folder
homeFolder <- function() {
  switch(
    Sys.info()[["sysname"]],
    Windows = {
      home <- Sys.getenv("R_USER")
    },
    Linux = {
      home <- Sys.getenv("HOME")
    },
    Darwin = {
      home <- Sys.getenv("HOME")
    }
  )
  return(home)
}

### PRISMA Flow Diagram ----
#' Draw a PRISMA Flow Diagram for bibliometric studies
#'
#' @param db_name Character. Name of the primary database.
#' @param query Character. Search query or topic description.
#' @param n_total Integer. Total records retrieved from primary source.
#' @param other_source Character. Description of additional source (or "").
#' @param n_other Integer. Records from additional source.
#' @param steps A data.frame with columns: phase, criterion, values, n_excluded, n_remaining.
#'   phase is one of "Screening" or "Eligibility".
#' @param n_included Integer. Final number of studies included.
#' @return Invisible NULL. Draws directly on the current graphics device.
drawPrismaFlowDiagram <- function(db_name = "Web of Science",
                                   query = "",
                                   n_total = 0,
                                   other_source = "",
                                   n_other = 0,
                                   steps = data.frame(),
                                   n_included = 0) {

  format_n <- function(n) format(n, big.mark = ",")

  # Wrap long text to fit within a given character width
  wrap_text <- function(text, max_chars = 50) {
    lines <- strsplit(text, "\n")[[1]]
    wrapped <- character(0)
    for (line in lines) {
      if (nchar(line) <= max_chars) {
        wrapped <- c(wrapped, line)
      } else {
        words <- strsplit(line, " ")[[1]]
        current <- words[1]
        for (w in words[-1]) {
          if (nchar(paste(current, w)) <= max_chars) {
            current <- paste(current, w)
          } else {
            wrapped <- c(wrapped, current)
            current <- w
          }
        }
        wrapped <- c(wrapped, current)
      }
    }
    paste(wrapped, collapse = "\n")
  }

  count_lines <- function(label) length(strsplit(label, "\n")[[1]])

  # Height per line of text in npc units
  line_h <- 0.022
  # Base box padding (top + bottom)
  pad_h <- 0.025

  # Compute box height from label
  box_height <- function(label) pad_h + count_lines(label) * line_h

  # Helper: draw box with text, supporting optional secondary text (smaller font)
  draw_box <- function(x, y, w, h, label, fill = "#D6E8F5", border = "#2B5C8A",
                       font_size = 14, font_face = "plain", text_col = "black",
                       sub_label = NULL, sub_font_size = 11) {
    grid::grid.roundrect(
      x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      width = grid::unit(w, "npc"), height = grid::unit(h, "npc"),
      r = grid::unit(0.008, "npc"),
      gp = grid::gpar(fill = fill, col = border, lwd = 2.5)
    )
    if (is.null(sub_label)) {
      grid::grid.text(
        label, x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
        gp = grid::gpar(fontsize = font_size, fontface = font_face,
                         col = text_col, lineheight = 1.15),
        just = "centre"
      )
    } else {
      # Two-part text: main label on top, sub_label below in smaller font
      n_main <- count_lines(label)
      n_sub <- count_lines(sub_label)
      total_text_h <- n_main * line_h + n_sub * (line_h * sub_font_size / font_size)
      y_main <- y + total_text_h / 2 - (n_main * line_h) / 2
      y_sub <- y_main - (n_main * line_h) / 2 - (n_sub * line_h * sub_font_size / font_size) / 2 - 0.003
      grid::grid.text(
        label, x = grid::unit(x, "npc"), y = grid::unit(y_main, "npc"),
        gp = grid::gpar(fontsize = font_size, fontface = font_face,
                         col = text_col, lineheight = 1.15),
        just = "centre"
      )
      grid::grid.text(
        sub_label, x = grid::unit(x, "npc"), y = grid::unit(y_sub, "npc"),
        gp = grid::gpar(fontsize = sub_font_size, fontface = "italic",
                         col = "#444444", lineheight = 1.10),
        just = "centre"
      )
    }
  }

  draw_phase_label <- function(x, y, w, h, label) {
    grid::grid.roundrect(
      x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      width = grid::unit(w, "npc"), height = grid::unit(h, "npc"),
      r = grid::unit(0.008, "npc"),
      gp = grid::gpar(fill = "#1B3A5C", col = "#1B3A5C", lwd = 2)
    )
    grid::grid.text(
      label, x = grid::unit(x, "npc"), y = grid::unit(y, "npc"),
      gp = grid::gpar(fontsize = 15, fontface = "bold", col = "white")
    )
  }

  draw_arrow <- function(x0, y0, x1, y1, col = "#2B5C8A") {
    grid::grid.lines(
      x = grid::unit(c(x0, x1), "npc"), y = grid::unit(c(y0, y1), "npc"),
      arrow = grid::arrow(length = grid::unit(0.012, "npc"), type = "closed"),
      gp = grid::gpar(col = col, fill = col, lwd = 2.5)
    )
  }

  draw_line <- function(x0, y0, x1, y1, col = "#2B5C8A") {
    grid::grid.lines(
      x = grid::unit(c(x0, x1), "npc"), y = grid::unit(c(y0, y1), "npc"),
      gp = grid::gpar(col = col, lwd = 2)
    )
  }

  # Layout constants
  cx <- 0.42
  rbox_x <- 0.81
  bw <- 0.34
  rbw <- 0.28
  fs <- 14
  fs_side <- 13
  fs_query <- 11
  plx <- 0.09
  plw <- 0.13
  plh <- 0.042
  main_wrap <- 50   # max chars per line for main boxes
  side_wrap <- 35   # max chars per line for side boxes
  query_wrap <- 55  # wider wrap for query text (smaller font)

  has_other <- nchar(trimws(other_source)) > 0 && n_other > 0
  n_steps <- nrow(steps)

  # --- Pre-compute all labels and box heights ---

  # Identification: separate main label from query (shown in smaller font)
  has_query <- nchar(trimws(query)) > 0
  id_main_label <- paste0("Records identified from ", db_name, "\n(n = ", format_n(n_total), ")")
  id_main_label <- wrap_text(id_main_label, main_wrap)
  id_query_label <- NULL

  if (has_query) {
    id_query_label <- wrap_text(query, query_wrap)
    n_main <- count_lines(id_main_label)
    n_query <- count_lines(id_query_label)
    id_bh <- pad_h + n_main * line_h + n_query * (line_h * fs_query / fs) + 0.008
  } else {
    id_bh <- box_height(id_main_label)
  }

  # Total records label
  total_n <- n_total + ifelse(has_other, n_other, 0)
  total_label <- wrap_text(paste0("Total records retrieved\n(n = ", format_n(total_n), ")"), main_wrap)
  total_bh <- box_height(total_label)

  # Step labels
  step_labels <- character(0)
  step_bhs <- numeric(0)
  excl_labels <- character(0)
  excl_bhs <- numeric(0)
  for (i in seq_len(n_steps)) {
    step <- steps[i, ]
    has_values <- nchar(trimws(step$values)) > 0
    if (has_values) {
      ml <- paste0("After ", step$criterion, " filter\n",
                    step$values, "\n(n = ", format_n(step$n_remaining), ")")
    } else {
      ml <- paste0("After ", step$criterion, " filter\n(n = ", format_n(step$n_remaining), ")")
    }
    ml <- wrap_text(ml, main_wrap)
    step_labels <- c(step_labels, ml)
    step_bhs <- c(step_bhs, box_height(ml))

    el <- wrap_text(paste0("Excluded by ", step$criterion, "\n(n = ", format_n(step$n_excluded), ")"), side_wrap)
    excl_labels <- c(excl_labels, el)
    excl_bhs <- c(excl_bhs, box_height(el))
  }

  # Inclusion label
  inc_label <- wrap_text(paste0("Studies included in the analysis\n(n = ", format_n(n_included), ")"), main_wrap)
  inc_bh <- box_height(inc_label)

  # Other source label
  if (has_other) {
    other_label <- wrap_text(paste0("Records from\n", other_source, "\n(n = ", format_n(n_other), ")"), main_wrap)
    other_bh <- box_height(other_label)
  }

  # --- Compute Y positions: distribute evenly to fill vertical space ---
  all_bhs <- c(id_bh, total_bh, step_bhs, inc_bh)
  n_rows <- length(all_bhs)

  y_top_start <- 0.91
  y_bottom_end <- 0.05
  total_space <- y_top_start - y_bottom_end
  total_box_h <- sum(all_bhs)

  # Distribute remaining space equally as gaps
  gap <- if (n_rows > 1) (total_space - total_box_h) / (n_rows - 1) else 0
  gap <- max(gap, 0.010)  # minimum gap

  # If boxes + min gaps exceed space, shrink proportionally
  if (total_box_h + (n_rows - 1) * 0.010 > total_space) {
    scale <- (total_space - (n_rows - 1) * 0.010) / total_box_h
    all_bhs <- all_bhs * scale
    id_bh <- all_bhs[1]; total_bh <- all_bhs[2]
    if (n_steps > 0) step_bhs <- all_bhs[3:(2 + n_steps)]
    inc_bh <- all_bhs[n_rows]
    gap <- 0.010
  }

  # Compute y centers top-down
  y_centers <- numeric(n_rows)
  y_cur <- y_top_start - all_bhs[1] / 2
  y_centers[1] <- y_cur
  for (j in 2:n_rows) {
    y_cur <- y_cur - all_bhs[j - 1] / 2 - gap - all_bhs[j] / 2
    y_centers[j] <- y_cur
  }

  y_id <- y_centers[1]
  y_total <- y_centers[2]
  y_steps <- if (n_steps > 0) y_centers[3:(2 + n_steps)] else numeric(0)
  y_inc <- y_centers[n_rows]

  # Phase labels y positions
  screening_ys <- if (n_steps > 0) y_steps[steps$phase == "Screening"] else numeric(0)
  eligibility_ys <- if (n_steps > 0) y_steps[steps$phase == "Eligibility"] else numeric(0)

  # --- Start drawing ---
  grid::grid.newpage()
  grid::pushViewport(grid::viewport(width = 0.96, height = 0.96))
  grid::grid.rect(gp = grid::gpar(fill = "white", col = NA))

  # Title
  grid::grid.text("PRISMA Flow Diagram",
                   x = grid::unit(0.5, "npc"), y = grid::unit(0.97, "npc"),
                   gp = grid::gpar(fontsize = 22, fontface = "bold", col = "#1B3A5C"))
  grid::grid.text("(adapted for bibliometric studies)",
                   x = grid::unit(0.5, "npc"), y = grid::unit(0.945, "npc"),
                   gp = grid::gpar(fontsize = 14, fontface = "italic", col = "#555555"))

  # Phase labels
  draw_phase_label(plx, y_id, plw, plh, "Identification")
  if (length(screening_ys) > 0)
    draw_phase_label(plx, screening_ys[1], plw, plh, "Screening")
  if (length(eligibility_ys) > 0)
    draw_phase_label(plx, eligibility_ys[1], plw, plh, "Eligibility")
  draw_phase_label(plx, y_inc, plw, plh, "Inclusion")

  # Identification boxes
  if (has_other) {
    id_left_x <- 0.32
    id_right_x <- 0.58
    id_bw <- 0.24
    draw_box(id_left_x, y_id, id_bw, id_bh, id_main_label,
             font_size = fs, sub_label = id_query_label, sub_font_size = fs_query)
    draw_box(id_right_x, y_id, id_bw, other_bh, other_label, font_size = fs - 1)
    draw_arrow(id_left_x, y_id - id_bh / 2, cx, y_total + total_bh / 2)
    draw_arrow(id_right_x, y_id - other_bh / 2, cx, y_total + total_bh / 2)
  } else {
    draw_box(cx, y_id, bw, id_bh, id_main_label,
             font_size = fs, sub_label = id_query_label, sub_font_size = fs_query)
    draw_arrow(cx, y_id - id_bh / 2, cx, y_total + total_bh / 2)
  }

  # Total records
  draw_box(cx, y_total, bw, total_bh, total_label, font_size = fs)

  # Steps
  prev_y <- y_total
  prev_bh <- total_bh
  for (i in seq_len(n_steps)) {
    cur_y <- y_steps[i]
    cur_bh <- step_bhs[i]

    draw_arrow(cx, prev_y - prev_bh / 2, cx, cur_y + cur_bh / 2)
    draw_box(cx, cur_y, bw, cur_bh, step_labels[i], font_size = fs)

    # Right exclusion box
    draw_box(rbox_x, cur_y, rbw, excl_bhs[i], excl_labels[i],
             fill = "#FFFFFF", border = "#8B2020",
             font_size = fs_side, text_col = "#8B2020")
    draw_line(cx + bw / 2, cur_y, rbox_x - rbw / 2, cur_y, col = "#8B2020")

    prev_y <- cur_y
    prev_bh <- cur_bh
  }

  # Inclusion
  draw_arrow(cx, prev_y - prev_bh / 2, cx, y_inc + inc_bh / 2)

  draw_box(cx, y_inc, bw, inc_bh, inc_label,
           fill = "#D6E8F5", border = "#1B3A5C",
           font_size = fs, font_face = "bold")

  grid::popViewport()
  invisible(NULL)
}

#' Get Complete Author Information and ID from Scopus
#'
#' Uses SCOPUS API author search to identify author identification information.
#'
#' @param df is a dataframe composed of three columns:
#' \tabular{lll}{
#' \code{lastname}\tab   \tab author's last name\cr
#' \code{firstname}\tab   \tab author's first name\cr
#' \code{affiliation}\tab   \tab Part of the affiliation name (university name, city, etc.)}
#' i.e. df[1,1:3]<-c("aria","massimo","naples")
#' When affiliation is not specified, the field df$affiliation have to be NA.
#' i.e. df[2,1:3]<-c("cuccurullo","corrado", NA)
#' @param api_key is a character. It contains the Elsevier API key. Information about how to obtain an API Key \href{https://dev.elsevier.com/sc_apis.html}{Elsevier API website}
#' @return a data frame with cases corresponding to authors and variables to author's information and ID got from SCOPUS.
#' @examples
#' ## Request a personal API Key to Elsevier web page https://dev.elsevier.com/sc_apis.html
#' #
#' # api_key="your api key"
#'
#' ## create a data frame with the list of authors to get information and IDs
#' # i.e. df[1,1:3]<-c("aria","massimo","naples")
#' #      df[2,1:3]<-c("cuccurullo","corrado", NA)
#'
#' ## run idByAuthor function
#' #
#' # authorsID <- idByAuthor(df, api_key)
#'
#' @seealso \code{\link{retrievalByAuthorID}} for downloading the complete author bibliographic collection from SCOPUS
#'
#' @export

idByAuthor <- function(df, api_key) {
  n <- dim(df)[1]

  ### download authors' info
  AU_ID <- NA
  AU_AFF <- NA
  AU_count <- NA

  for (j in 1:n) {
    lastname <- tolower(df[j, 1])
    firstname <- tolower(df[j, 2])
    if (!is.na(df[j, 3])) {
      query <- paste("affil(", df[j, 3], ")", sep = "")
    } else {
      query <- NULL
    }
    cat("\nSearching author's info: ", toupper(df[j, 1]), toupper(df[j, 2]))

    AU_info <- get_complete_author_info(last_name = lastname, first_name = firstname, api_key = api_key, query = query)

    ### author id
    if (AU_info$content$`search-results`$`opensearch:totalResults` != 0) {
      AU_ID[j] <- AU_info[[2]]$`search-results`$entr[[1]]$`dc:identifier`
      AU_ID[j] <- gsub("AUTHOR_ID:", "", AU_ID[j])
      AU_info2 <- AU_info[[2]]
      aff <- AU_info2$`search-results`$entry[[1]]$`affiliation-current`
      AU_AFF[j] <- paste(aff$`affiliation-name`, ", ", aff$`affiliation-city`, ", ", aff$`affiliation-country`, sep = "")
      ### author document counts
      AU_count[j] <- AU_info[[2]]$`search-results`$entr[[1]]$`document-count`
    } else {
      AU_ID[j] <- NA
      AU_AFF[j] <- NA
      AU_count[j] <- NA
    }
  }
  authorsID <- data.frame(lastname = df[, 1], firstname = df[, 2], id = AU_ID, affiliation = AU_AFF, count = AU_count)
  return(authorsID)
}

importFiles <- function(...) {
  arguments <- unlist(list(...))
  k <- length(arguments)
  D <- list()
  # enc="UTF-8"
  # origEnc=getOption("encoding")
  # if (origEnc=="UTF-8"){options(encoding = "native.enc")}
  for (i in 1:k) {
    D[[i]] <- read_lines(
      arguments[i],
      skip = 0,
      n_max = -1L,
      locale = default_locale(),
      progress = show_progress()
    )


    # D[[i]]=suppressWarnings(
    #   iconv(readLines(arguments[i],encoding = "UTF-8"),"latin1", "ASCII", sub="")
    #   #conv(readLines(arguments[[i]]))
    # )
  }
  D <- unlist(D)
  # options(encoding = origEnc)
  # Encoding(D) <- "UTF-8"
  return(D)
}

# utils::globalVariables("where")
# utils::globalVariables("any_of")
utils::globalVariables(c("matches", "KW_Merged"))

#' @import stats
#' @import contentanalysis
#' @import dimensionsR
#' @import ggplot2
#' @import bibliometrixData
#' @import forcats
#' @import ggrepel
#' @import shinycssloaders
#' @import pubmedR
#' @import shiny
#' @import readr
#' @import readxl
#' @import stringi
#' @import tidytext
#' @import openalexR
#' @import ca
#' @import visNetwork
#' @importFrom methods new
#' @importFrom purrr map2_dfr
#' @importFrom purrr map_dfr
#' @importFrom purrr map_df
#' @importFrom purrr map_chr
#' @importFrom dplyr %>%
#' @importFrom dplyr bind_cols
#' @importFrom dplyr across
#' @importFrom dplyr row_number
#' @importFrom dplyr tibble
#' @importFrom dplyr as_tibble
#' @importFrom dplyr between
#' @importFrom dplyr case_when
#' @importFrom dplyr if_else
#' @importFrom dplyr lead
#' @importFrom dplyr pull
#' @importFrom dplyr arrange
#' @importFrom dplyr everything
#' @importFrom dplyr where
#' @importFrom dplyr pick
#' @importFrom dplyr distinct
#' @importFrom dplyr join_by
#' @importFrom dplyr n
#' @importFrom dplyr slice
#' @importFrom dplyr slice_max
#' @importFrom dplyr if_all
#' @importFrom dplyr any_of
#' @importFrom dplyr cummean
#' @importFrom dplyr count
#' @importFrom dplyr desc
#' @importFrom dplyr bind_rows
#' @importFrom dplyr group_by
#' @importFrom dplyr mutate
#' @importFrom dplyr ungroup
#' @importFrom dplyr rename
#' @importFrom dplyr rename_with
#' @importFrom dplyr rowwise
#' @importFrom dplyr summarise
#' @importFrom dplyr summarize
#' @importFrom dplyr select
#' @importFrom dplyr anti_join
#' @importFrom dplyr inner_join
#' @importFrom dplyr left_join
#' @importFrom dplyr right_join
#' @importFrom dplyr relocate
#' @importFrom dplyr slice_head
#' @importFrom dplyr slice_tail
#' @importFrom dplyr reframe
#' @importFrom dplyr coalesce group_split arrange n_distinct first
#' @importFrom stringdist stringdistmatrix
#' @importFrom plotly add_annotations
#' @importFrom plotly add_lines
#' @importFrom plotly config
#' @importFrom plotly ggplotly
#' @importFrom plotly layout
#' @importFrom plotly plot_ly
#' @importFrom plotly subplot
#' @importFrom plotly toRGB
#' @importFrom tibble rownames_to_column
# #' @import shinycssloaders
# #' @import shinythemes
#' @importFrom openxlsx write.xlsx
#' @importFrom tidyr expand_grid
#' @importFrom tidyr gather
#' @importFrom tidyr spread
#' @importFrom tidyr pivot_wider
#' @importFrom tidyr pivot_longer
#' @importFrom tidyr replace_na
#' @importFrom tidyr separate
#' @importFrom tidyr drop_na
#' @importFrom tidyr unite
#' @importFrom tidyr starts_with
#' @importFrom tidyr unnest
#' @importFrom tidyr unite
#' @importFrom graphics abline
#' @importFrom grDevices adjustcolor
#' @importFrom grDevices dev.off
#' @importFrom grDevices pdf
#' @importFrom grDevices chull
#' @importFrom grDevices heat.colors
#' @importFrom stringdist stringdistmatrix
#' @importFrom rscopus affiliation_retrieval
#' @importFrom rscopus author_df_orig
#' @importFrom rscopus author_search
#' @importFrom rscopus get_complete_author_info
#' @importFrom igraph as_long_data_frame
#' @importFrom igraph get.edgelist
#' @importFrom igraph graph_from_data_frame
#' @importFrom igraph as_adjacency_matrix
#' @importFrom igraph graph.adjacency
#' @importFrom igraph as_edgelist
#' @importFrom igraph degree
#' @importFrom igraph plot.igraph
#' @importFrom igraph delete.vertices
#' @importFrom igraph decompose.graph
#' @importFrom igraph E
#' @importFrom igraph E<-
#' @importFrom igraph V
#' @importFrom igraph V<-
#' @importFrom igraph vcount
#' @importFrom igraph graph_attr
#' @importFrom igraph edge_density
#' @importFrom igraph transitivity
#' @importFrom igraph diameter
#' @importFrom igraph degree_distribution
#' @importFrom igraph centr_degree
#' @importFrom igraph centr_clo
#' @importFrom igraph centr_betw
#' @importFrom igraph centr_eigen
#' @importFrom igraph mean_distance
#' @importFrom igraph delete_edges delete_vertices layout_with_fr modularity norm_coords
#' @importFrom igraph closeness
#' @importFrom igraph induced_subgraph
#' @importFrom igraph page.rank
#' @importFrom igraph eigen_centrality
#' @importFrom igraph arpack_defaults
#' @importFrom igraph authority_score
#' @importFrom igraph page_rank
#' @importFrom igraph hub_score
#' @importFrom igraph graph_from_incidence_matrix
#' @importFrom igraph graph_from_adjacency_matrix
#' @importFrom igraph simplify
#' @importFrom igraph layout.auto
#' @importFrom igraph layout.circle
#' @importFrom igraph layout.sphere
#' @importFrom igraph layout.mds
#' @importFrom igraph layout.kamada.kawai
#' @importFrom igraph layout.fruchterman.reingold
#' @importFrom igraph layout.star
#' @importFrom igraph write.graph
#' @importFrom igraph cluster_walktrap
#' @importFrom igraph cluster_leiden
#' @importFrom igraph cluster_optimal
#' @importFrom igraph cluster_infomap
#' @importFrom igraph cluster_edge_betweenness
#' @importFrom igraph cluster_fast_greedy
#' @importFrom igraph cluster_louvain
#' @importFrom igraph cluster_leading_eigen
#' @importFrom igraph cluster_spinglass
#' @importFrom igraph count_multiple
#' @importFrom igraph membership
#' @importFrom igraph layout.norm
#' @importFrom igraph delete.edges
#' @importFrom igraph betweenness
#' @importFrom Matrix %&%
#' @importFrom Matrix abIseq
#' @importFrom Matrix abIseq1
#' @importFrom Matrix all.equal
#' @importFrom Matrix anyDuplicatedT
#' @importFrom Matrix Arith
#' @importFrom Matrix as.array
#' @importFrom Matrix as.matrix
#' @importFrom Matrix band
#' @importFrom Matrix bandSparse
#' @importFrom Matrix bdiag
#' @importFrom Matrix cbind2
#' @importFrom Matrix chol
#' @importFrom Matrix chol2inv
#' @importFrom Matrix Cholesky
#' @importFrom Matrix coerce
#' @importFrom Matrix colMeans
#' @importFrom Matrix colSums
#' @importFrom Matrix Compare
#' @importFrom Matrix condest
#' @importFrom Matrix cov2cor
#' @importFrom Matrix crossprod
#' @importFrom Matrix det
#' @importFrom Matrix determinant
#' @importFrom Matrix diag
#' @importFrom Matrix diag<-
#' @importFrom Matrix diagN2U
#' @importFrom Matrix Diagonal
#' @importFrom Matrix diagU2N
#' @importFrom Matrix diff
#' @importFrom Matrix drop
#' @importFrom Matrix drop0
#' @importFrom Matrix expand
#' @importFrom Matrix expm
#' @importFrom Matrix fac2sparse
#' @importFrom Matrix forceSymmetric
#' @importFrom Matrix format
#' @importFrom Matrix formatSparseM
#' @importFrom Matrix formatSpMatrix
#' @importFrom Matrix head
#' @importFrom Matrix image
#' @importFrom Matrix invPerm
#' @importFrom Matrix is.null.DN
#' @importFrom Matrix isDiagonal
#' @importFrom Matrix isLDL
#' @importFrom Matrix isSymmetric
#' @importFrom Matrix isTriangular
#' @importFrom Matrix kronecker
#' @importFrom Matrix Logic
#' @importFrom Matrix lu
#' @importFrom Matrix Math
#' @importFrom Matrix Math2
#' @importFrom Matrix Matrix
#' @importFrom Matrix MatrixClass
#' @importFrom Matrix mean
#' @importFrom Matrix nnzero
#' @importFrom Matrix norm
#' @importFrom Matrix onenormest
#' @importFrom Matrix Ops
#' @importFrom Matrix pack
#' @importFrom Matrix print
#' @importFrom Matrix printSpMatrix
#' @importFrom Matrix printSpMatrix2
#' @importFrom Matrix qr
#' @importFrom Matrix qr.coef
#' @importFrom Matrix qr.fitted
#' @importFrom Matrix qr.Q
#' @importFrom Matrix qr.qty
#' @importFrom Matrix qr.qy
#' @importFrom Matrix qr.R
#' @importFrom Matrix qr.resid
#' @importFrom Matrix qrR
#' @importFrom Matrix rankMatrix
#' @importFrom Matrix rbind2
#' @importFrom Matrix rcond
#' @importFrom Matrix readHB
#' @importFrom Matrix readMM
#' @importFrom Matrix rep2abI
#' @importFrom Matrix rowMeans
#' @importFrom Matrix rowSums
#' @importFrom Matrix rsparsematrix
#' @importFrom Matrix show
#' @importFrom Matrix skewpart
#' @importFrom Matrix solve
#' @importFrom Matrix sparse.model.matrix
#' @importFrom Matrix sparseMatrix
#' @importFrom Matrix sparseVector
#' @importFrom Matrix spMatrix
#' @importFrom Matrix summary
#' @importFrom Matrix Summary
#' @importFrom Matrix symmpart
#' @importFrom Matrix t
#' @importFrom Matrix tail
#' @importFrom Matrix tcrossprod
#' @importFrom Matrix tril
#' @importFrom Matrix triu
#' @importFrom Matrix uniqTsparse
#' @importFrom Matrix unpack
#' @importFrom Matrix update
#' @importFrom Matrix updown
#' @importFrom Matrix which
#' @importFrom Matrix writeMM
#' @importFrom stringr str_detect
#' @importFrom stringr fixed
#' @importFrom stringr str_to_upper
#' @importFrom stringr str_squish regex str_remove
#' @importFrom stringr str_locate_all
#' @importFrom stringr str_extract_all str_extract
#' @importFrom stringr str_replace_all str_remove_all
#' @importFrom stringr str_trim str_locate
#' @importFrom stringr str_split
#' @importFrom graphics axis points
#' @importFrom graphics barplot
#' @importFrom graphics legend
#' @importFrom graphics lines
#' @importFrom graphics plot
#' @importFrom graphics par
#' @importFrom grDevices colorRampPalette
#' @importFrom grDevices as.raster
#' @importFrom utils capture.output
#' @importFrom utils data
#' @importFrom utils adist
#' @importFrom utils read.csv
#' @importFrom utils write.csv
#' @importFrom SnowballC wordStem
#' @importFrom SnowballC getStemLanguages
# @importFrom rio import
.onAttach <- function(...) {
  packageStartupMessage(
    "Please note that our software is open source and available for use, distributed under the MIT license.
    \nWhen it is used in a publication, we ask that authors properly cite the following references:\n\n
Aria, M. & Cuccurullo, C. (2017) bibliometrix: An R-tool for comprehensive science mapping analysis,
                        Journal of Informetrics, 11(4), pp 959-975, Elsevier.\n
Aria, M. & Cuccurullo, C. (2026) Science Mapping Analysis - A primer with Biblioshiny,
                        McGraw-Hill, ISBN: 978-88-386-2297-7.\n
Aria, M., Cuccurullo, C., D'Aniello L. & Spano, M. (2026) Biblioshiny and the SAAS Workflow:
                        An integrated framework for transparent and reproducible science mapping. 
                        A demonstration through the replication of a study,
                        Journal of Informetrics, 20(3), Elsevier.\n
                        
                        \nFailure to properly cite the software is considered a violation of the license.
                        \nFor information and bug reports:
                        - Take a look at https://www.bibliometrix.org
                        - Send an email to info@bibliometrix.org
                        - Write a post on https://github.com/massimoaria/bibliometrix/issues
                        \nHelp us to keep Bibliometrix and Biblioshiny free to download and use by contributing with a small donation to support our research team (https://bibliometrix.org/donate.html)\n
                        \nTo start with the Biblioshiny app, please digit:
biblioshiny()\n"
  )
}


# ### scale coordinates
# ### Credits to François Briatte. Function is a fork of the package ggnetwork
scale_data <- function(x, scale = diff(range(x))) {
  if (!scale) {
    x <- rep(0.5, length.out = length(x))
  } else {
    x <- scale(x, center = min(x), scale = scale)
  }
  as.vector(x)
}


# ## Plot edges using ggplot2
# ### Credits to François Briatte. Function is a fork of the package ggnetwork
geom_network_edges <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  arrow = NULL,
  curvature = 0,
  angle = 90,
  ncp = 5,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
) {
  if (!curvature) {
    geom <- ggplot2::GeomSegment
    params <- list(arrow = arrow, na.rm = na.rm, ...)
  } else {
    geom <- ggplot2::GeomCurve
    params <- list(
      arrow = arrow,
      curvature = curvature,
      angle = angle,
      ncp = ncp,
      na.rm = na.rm,
      ...
    )
  }

  ggplot2::layer(
    data = data,
    mapping = mapping,
    stat = StatEdges,
    geom = geom,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = params
  )
}

# ### Credits to François Briatte. Function is a fork of the package ggnetwork
geom_network_nodes <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
) {
  ggplot2::layer(
    data = data,
    mapping = mapping,
    stat = StatNodes,
    geom = ggplot2::GeomPoint,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      na.rm = na.rm,
      ...
    )
  )
}
#
# ### Credits to François Briatte. Function is a fork of the package ggnetwork
StatEdges <- ggplot2::ggproto(
  "StatEdges",
  ggplot2::Stat,
  compute_layer = function(data, scales, params) {
    unique(subset(data, !(x == xend & y == yend)))
  }
)

### Credits to François Briatte. Function is a fork of the package ggnetwork
StatNodes <- ggplot2::ggproto(
  "StatNodes",
  ggplot2::Stat,
  compute_layer = function(data, scales, params) {
    if (all(c("xend", "yend") %in% names(data))) {
      unique(subset(data, select = c(-xend, -yend)))
    } else {
      unique(data)
    }
  }
)

### Credits to Patrick Barks. Function is a fork of the package Abbrev
AbbrevTerm <- function(x, check = TRUE) {
  if (check == FALSE) {
    out <- x
  } else {
    # check for invalid x
    if (length(x) > 1) {
      stop("Please provide a single string (length(x) should equal 1)")
    } else if (grepl("[[:space:]]", x)) {
      warning(
        "The provided string contains spaces. This function is only designed to abbreviate a single word."
      )
    }

    # check whether x is title case
    ch1 <- substr(x, 1, 1)
    is_title <- ch1 == toupper(ch1)

    # check for whole word match
    # TODO: find way to deal with multiple whole word matches (e.g. w/ and w/o diacritics)
    int_whole_match <- stringi::stri_cmp_equiv(
      x,
      ltwa_singles$WORD,
      strength = 1
    )
    ind_whole_match <- which(int_whole_match == TRUE)

    if (length(ind_whole_match > 0)) {
      # if whole-word match is found, return corresponding abbrev (if no abbrev,
      #  return original x)
      abbrev <- ltwa_singles$ABBREVIATIONS[ind_whole_match]
      word <- ltwa_singles$WORD[ind_whole_match]
      out <- ifelse(is.na(abbrev), word, abbrev)
    } else {
      # else, find matching prefix
      # TODO: give preference to matches with higher strength
      lgl_prefix <- stringi::stri_startswith_coll(
        x,
        ltwa_prefix$WORD,
        strength = 1
      )
      ind_prefix <- which(lgl_prefix == TRUE)

      # choose abbrev based on number of matching prefixes (0, 1, or 2+)
      if (length(ind_prefix) == 0) {
        # if no matching prefixes, return original x
        out <- x
      } else if (length(ind_prefix) == 1) {
        # if one matching prefixes, return corresponding abbrev
        out <- ltwa_prefix$ABBREVIATIONS[ind_prefix]
        if (is.na(out)) out <- x
      } else {
        # if multiple matching prefixes, choose abbrev from longest match
        ind_prefix <- ind_prefix[which.max(nchar(ltwa_prefix$WORD[ind_prefix]))]
        out <- ltwa_prefix$ABBREVIATIONS[ind_prefix]
      }
    }

    # if x contains only latin chars, remove any diacritics from abbrev
    if (stringi::stri_trans_general(x, "Latin-ASCII") == x) {
      out <- stringi::stri_trans_general(out, "Latin-ASCII")
    }

    # if x is title case, convert out to title case
    if (is_title == TRUE) {
      out <- ToTitleCase(out)
    }
  }

  # return
  return(out)
}


### Credits to Patrick Barks. Function is a fork of the package Abbrev
AbbrevTitle <- function(x) {
  ind <- rank(-nchar(ltwa_phrase$WORD), ties.method = "last")
  ltwa_phrase[ind, ] <- ltwa_phrase

  # check for invalid x
  if (length(x) > 1) {
    stop("Please provide a single string (length(x) should equal 1)")
  }

  # remove parentheses and punctuation, and split string into vector of
  #  individual words/terms
  x <- gsub("[[:space:]]\\(.+\\)|\\,|\\.", "", x)
  xv <- unlist(strsplit(x, " "))

  # if title contains only one word, return that word
  if (length(xv) == 1) {
    out <- ToTitleCase(xv)
  } else {
    # otherwise... check for matching multi-word phrases

    # vector to track which elements of xv already abbreviated
    xv_which_abb <- logical(length(xv))

    # search for multi-word matches (e.g. South Pacific)
    lgl_phrase <- sapply(
      ltwa_phrase$WORD,
      function(y) grepl(y, tolower(x)),
      USE.NAMES = FALSE
    )
    ind_phrase <- which(lgl_phrase)

    # if any matching multi-word phrases
    if (length(ind_phrase) > 0) {
      for (i in 1:length(ind_phrase)) {
        match_phrase <- unlist(strsplit(
          ltwa_phrase$WORD[ind_phrase[i]],
          "[[:space:]]"
        ))
        match_abb <- ltwa_phrase$ABBREVIATIONS[ind_phrase[i]]
        ind_match <- sapply(
          match_phrase,
          function(y) grep(y, tolower(xv)),
          USE.NAMES = FALSE
        ) # should only find sequential matches
        if (length(ind_match[[1]]) > 0) {
          ind <- ind_match[[1]]:(ind_match[[1]] + length(ind_match) - 1)
          xv[ind[1]] <- match_abb
          xv <- c(xv[-ind[-1]])
          xv_which_abb[ind[1]] <- TRUE
          xv_which_abb <- c(xv_which_abb[-ind[-1]])
        }
      }
    }

    # if title fully matches multi-word phrase(s), return
    if (all(xv_which_abb == TRUE)) {
      out <- xv
    } else {
      # otherwise... deal with prepositions, articles, and conjunctions

      # remove prepositions not at beginning of word
      ind_rem_prep <- c(
        FALSE,
        mapply(CheckPrep, USE.NAMES = F, x = xv[-1], check = !xv_which_abb[-1])
      )
      xv <- xv[!ind_rem_prep]
      xv_which_abb <- xv_which_abb[!ind_rem_prep]

      # remove articles and conjunctions
      ind_rem_artcon <- mapply(
        CheckArtCon,
        USE.NAMES = F,
        x = xv,
        check = !xv_which_abb
      )
      xv <- xv[!ind_rem_artcon]
      xv_which_abb <- xv_which_abb[!ind_rem_artcon]

      # remove d' and l', if followed by character
      xv <- gsub(
        "^d'(?=[[:alpha:]])|^l'(?=[[:alpha:]])",
        "",
        xv,
        ignore.case = TRUE,
        perl = TRUE
      )

      # if title fully matches multi-word phrase(s) (minus articles/conjunctions), return
      if (all(xv_which_abb == TRUE)) {
        out <- xv
      } else {
        # otherwise... check for hyphenated words

        # check for hyphenated words
        ind_dash <- grep("[[:alpha:]]-[[:alpha:]]", xv)
        if (length(ind_dash) > 0) {
          # if hyphens, split hyphenated strings into vectors
          xv <- unlist(strsplit(xv, "-"))

          # update xv_which_abb based on hyphens
          for (i in length(ind_dash):1) {
            xv_which_abb <- append(xv_which_abb, FALSE, ind_dash[i])
          }
        }

        # abbreviate all words in title (excluding multi-word phrases)
        abbrev_full <- mapply(
          AbbrevTerm,
          x = xv,
          check = !xv_which_abb,
          USE.NAMES = F
        )

        # add dashes back in, if applicable
        if (length(ind_dash) > 0) {
          for (i in 1:length(ind_dash)) {
            dashed_terms <- abbrev_full[c(ind_dash[i], ind_dash[i] + 1)]
            abbrev_full[ind_dash[i]] <- paste(dashed_terms, collapse = "-")
            abbrev_full <- abbrev_full[-(ind_dash[i] + 1)]
          }
        }

        # collapse title to vector
        out <- paste(abbrev_full, collapse = " ")
      }
    }
  }
  return(out)
}

ToTitleCase <- function(x) {
  paste0(toupper(substr(x, 1, 1)), substr(x, 2, nchar(x)))
}

CheckPrep <- function(x, check) {
  if (check == TRUE) {
    return(ifelse(tolower(x) %in% df_prep$word, TRUE, FALSE))
  } else {
    return(FALSE)
  }
}

CheckArtCon <- function(x, check) {
  if (check == TRUE) {
    return(ifelse(tolower(x) %in% df_artcon$word, TRUE, FALSE))
  } else {
    return(FALSE)
  }
}

# color palette
colorlist <- function() {
  c(
    "#E41A1C",
    "#377EB8",
    "#4DAF4A",
    "#984EA3",
    "#FF7F00",
    "#A65628",
    "#F781BF",
    "#999999",
    "#66C2A5",
    "#FC8D62",
    "#8DA0CB",
    "#E78AC3",
    "#A6D854",
    "#FFD92F",
    "#B3B3B3",
    "#A6CEE3",
    "#1F78B4",
    "#B2DF8A",
    "#33A02C",
    "#FB9A99",
    "#E31A1C",
    "#FDBF6F",
    "#FF7F00",
    "#CAB2D6",
    "#6A3D9A",
    "#B15928",
    "#8DD3C7",
    "#BEBADA",
    "#FB8072",
    "#80B1D3",
    "#FDB462",
    "#B3DE69",
    "#D9D9D9",
    "#BC80BD",
    "#CCEBC5"
  )
}

# Initial to upper case
firstup <- function(x) {
  x <- tolower(x)
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

#' Merge DE and ID Fields into a Unified Keywords Column
#'
#' This function creates a new column `KW_Merged` by combining the contents of the `DE` (author keywords) and `ID` (keywords plus) fields
#' in a bibliographic dataframe. Duplicate keywords within each record are removed, and leading/trailing spaces are trimmed.
#' The merged keywords are separated by a semicolon (`;`).
#'
#' If the `KW_Merged` column already exists, it will not be overwritten unless `force = TRUE` is specified.
#'
#' @param M A dataframe containing at least the `DE` and/or `ID` columns, typically generated by `convert2df()` from the `bibliometrix` package.
#' @param force Logical. If `TRUE`, an existing `KW_Merged` column will be overwritten. Default is `FALSE`.
#'
#' @return A dataframe with an added (or updated) `KW_Merged` column containing deduplicated and cleaned keyword strings.
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrix")
#' M <- mergeKeywords(management)
#' head(M$KW_Merged)
#' }
#'
#' @export
mergeKeywords <- function(M, force = FALSE) {
  if (force) {
    M <- M %>% select(!matches("KW_Merged"))
  } # force to replace old KW_Merged column
  if (!"KW_Merged" %in% names(M)) {
    if (!"DE" %in% names(M)) {
      M$DE <- NA
    }
    if (!"ID" %in% names(M)) {
      M$ID <- NA
    }
    M <- M %>%
      unite(
        col = "KW_Merged",
        DE,
        ID,
        sep = "; ",
        remove = FALSE,
        na.rm = TRUE
      ) %>%
      mutate(KW_Merged = ifelse(KW_Merged == "", NA, KW_Merged)) %>%
      mutate(
        KW_Merged = map_chr(
          KW_Merged,
          ~ {
            .x %>%
              str_split(";") %>%
              unlist() %>%
              str_trim() %>%
              unique() %>%
              paste(collapse = "; ")
          }
        )
      )
  }
  ### bibliometrix>DB class
  class(M) <- c("bibliometrixDB", "data.frame")
  return(M)
}

filter_keywords_by_score <- function(
  df,
  input_col = "ID",
  output_col = "ID_filtered",
  threshold = 0.5
) {
  # Funzione per processare una singola stringa
  process_string <- function(x) {
    if (is.na(x) || x == "") {
      return("")
    }

    # Splitta per punto e virgola
    keywords <- strsplit(x, ";")[[1]]

    # Trimma spazi bianchi
    keywords <- trimws(keywords)

    # Estrai keyword e score
    filtered_kw <- sapply(keywords, function(kw) {
      # Estrai lo score tra parentesi usando regex
      score_match <- regmatches(kw, regexpr("\\(\\s*([0-9.]+)\\s*\\)", kw))

      if (length(score_match) > 0) {
        # Estrai il valore numerico dello score
        score <- as.numeric(gsub("[\\(\\)\\s]", "", score_match))

        # Se score >= threshold, estrai solo la keyword (parte prima delle parentesi)
        if (!is.na(score) && score >= threshold) {
          keyword_name <- trimws(sub("\\s*\\(.*\\)\\s*$", "", kw))
          return(keyword_name)
        } else {
          # Score presente ma < threshold
          return(NA)
        }
      } else {
        # Score NON presente: include la keyword così com'è
        return(trimws(kw))
      }
    })

    # Rimuovi NA e concatena con punto e virgola
    filtered_kw <- filtered_kw[!is.na(filtered_kw)]

    if (length(filtered_kw) > 0) {
      return(paste(filtered_kw, collapse = "; "))
    } else {
      return("")
    }
  }

  # Applica la funzione a tutte le righe
  df[[output_col]] <- sapply(df[[input_col]], process_string)

  return(df)
}

# adjust node positions in TM
adjust_positions_oblique <- function(
  df,
  xvar = "rcentrality",
  yvar = "rdensity",
  min_dist = 0.5,
  max_iter = 100,
  step_factor = 0.5,
  jitter_strength = 0.1
) {
  df_adj <- df

  for (iter in 1:max_iter) {
    moved <- FALSE
    for (i in 1:(nrow(df_adj) - 1)) {
      for (j in (i + 1):nrow(df_adj)) {
        xi <- df_adj[[xvar]][i]
        yi <- df_adj[[yvar]][i]
        xj <- df_adj[[xvar]][j]
        yj <- df_adj[[yvar]][j]

        dx <- xi - xj
        dy <- yi - yj
        dist <- sqrt(dx^2 + dy^2)

        # Se perfettamente sovrapposti, applica jitter obliquo casuale
        if (dist == 0) {
          jitter_angle <- runif(1, 0, 2 * pi)
          offset <- jitter_strength

          df_adj[[xvar]][i] <- xi + cos(jitter_angle) * offset
          df_adj[[yvar]][i] <- yi + sin(jitter_angle) * offset
          df_adj[[xvar]][j] <- xj - cos(jitter_angle) * offset
          df_adj[[yvar]][j] <- yj - sin(jitter_angle) * offset

          moved <- TRUE
        } else if (dist < min_dist) {
          angle <- atan2(dy, dx)
          offset <- (min_dist - dist) * step_factor

          df_adj[[xvar]][i] <- xi + cos(angle) * offset
          df_adj[[yvar]][i] <- yi + sin(angle) * offset
          df_adj[[xvar]][j] <- xj - cos(angle) * offset
          df_adj[[yvar]][j] <- yj - sin(angle) * offset

          moved <- TRUE
        }
      }
    }
    if (!moved) break
  }

  return(df_adj)
}

removeStrangeChar <- function(D) {
  ind <- numeric(length(D))
  for (i in 1:length(D)) {
    # print(i)
    # ind[i] <- nchar(D[i])

    res <- try(ind[i] <- nchar(D[i]), silent = TRUE)
    if (inherits(res, "try-error")) {
      ind[i] <- 0
      next
    }
  }
  D <- D[ind > 1]
}

utils::globalVariables(c("Paper", "Tag", "content", "cont"))
pubmed2df <- function(D) {
  D <- D[nchar(D) > 0] # remove empty rows

  for (i in 1:length(D)) {
    if (substr(D[i], 1, 4) == "    ") substr(D[i], 1, 4) <- substr(D[i - 1], 1, 4)
  }

  Papers <- which(regexpr("PMID-", D) == 1) # first row of each document
  nP <- length(Papers) # number of docuemnts

  rowPapers <- diff(c(Papers, length(D) + 1))

  numPapers <- rep(1:nP, rowPapers)

  DATA <- data.frame(Tag = substr(D, 1, 4), content = substr(D, 7, nchar(D)), Paper = numPapers)
  DATA$Tag <- gsub(" ", "", DATA$Tag)
  df <- DATA %>%
    group_by(Paper, Tag) %>%
    summarise(cont = paste(content, collapse = "---", sep = "")) %>%
    arrange(Tag, Paper) %>%
    pivot_wider(names_from = Tag, values_from = cont) %>%
    ungroup()
  df <- as.data.frame(df)

  # rename field tags
  error <- 0
  old_labs <- c("AD", "AUID", "FAU", "IS", "IP", "SO", "JT", "TA", "MH", "PG", "PT", "VI", "DP")
  new_labs <- c("C1", "OI", "AF", "SN", "IS", "SO2", "SO", "J9", "DE", "PP", "DT", "VL", "PY")
  lab <- names(df)
  for (j in 1:length(old_labs)) {
    i <- which(lab %in% old_labs[j])
    if (length(i) > 0) {
      lab[i] <- new_labs[j]
    } else {
      error <- 1
    }
  }
  names(df) <- lab
  if (error == 1) {
    cat("\nWarning:\nIn your file, some mandatory metadata are missing. Bibliometrix functions may not work properly!\n
Please, take a look at the vignettes:
- 'Data Importing and Converting' (https://www.bibliometrix.org/vignettes/Data-Importing-and-Converting.html)
- 'A brief introduction to bibliometrix' (https://www.bibliometrix.org/vignettes/Introduction_to_bibliometrix.html)\n\n")
  }

  # extract DOIs
  df$DI <- trimws(unlist(lapply(strsplit(df$LID, "\\["), "[", 1)))
  df$PY <- as.numeric(substr(df$PY, 1, 4))


  ### replace "---" with ";"
  tagsComma <- c("AU", "AF", "DE", "AID", "OT", "PHST", "DT")
  nolab <- setdiff(tagsComma, names(df))
  tagsComma <- tagsComma[(!(tagsComma %in% nolab))]

  df1 <- data.frame(lapply(df[tagsComma], function(x) {
    gsub("---", ";", x)
  }))

  ### replace "---" with " "
  otherTags <- setdiff(names(df), tagsComma)
  df2 <- data.frame(lapply(df[otherTags], function(x) {
    trimES(gsub("---", " ", x))
  }))
  df <- cbind(df1, df2)
  rm(df1, df2)

  df$DB <- "PUBMED"

  # remove * char from keywords
  df$DE <- df$ID <- gsub("\\*", "", df$DE)
  AB <- df$AB
  TI <- df$TI
  DE <- df$DE
  df <- data.frame(lapply(df, toupper))
  df$AB_raw <- AB
  df$TI_raw <- TI
  df$DE_raw <- DE
  # add sep ; to affiliations
  df$C1 <- gsub("\\.", ".;", df$C1)
  df$RP <- NA
  df <- df[names(df) != "Paper"]

  return(df)
}

#' Citation frequency distribution
#'
#' It calculates frequency distribution of citations.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics Web of Science file.
#' @param field is a character. It can be "article" or "author" to obtain frequency distribution of cited citations or cited authors (only first authors for WoS database) respectively. The default is \code{field = "article"}.
#' @param sep is the field separator character. This character separates citations in each string of CR column of the bibliographic data frame. The default is \code{sep = ";"}.
#' @return an object of \code{class} "list"  containing the following components:
#'
#' \tabular{lll}{
#' Cited \tab  \tab the most frequent cited manuscripts or authors\cr
#' Year \tab       \tab the publication year (only for cited article analysis)\cr
#' Source \tab      \tab the journal (only for cited article analysis)}
#'
#'
#'
#' @examples
#' ## EXAMPLE 1: Cited articles
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' CR <- citations(scientometrics, field = "article", sep = ";")
#'
#' CR$Cited[1:10]
#' CR$Year[1:10]
#' CR$Source[1:10]
#'
#' ## EXAMPLE 2: Cited first authors
#'
#' data(scientometrics)
#'
#' CR <- citations(scientometrics, field = "author", sep = ";")
#'
#' CR$Cited[1:10]
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export

citations <- function(M, field = "article", sep = ";") {
  CR <- NULL
  Year <- NULL
  SO <- NULL
  listCR <- strsplit(M$CR, sep)

  ## check for empty CR
  if (sum(nchar(listCR) > 3, na.rm = TRUE) == 0) {
    cat("\nReference metadata field 'CR' is empty!!\n\n")
    return(NA)
  }


  if (field == "author") {
    # listCR=strsplit(M$CR,sep)
    if (M$DB[1] == "ISI") {
      listCR <- lapply(listCR, function(l) {
        ListL <- lapply(strsplit(unlist(l), ","), function(x) x[1])
        l <- trimws(trimES(gsub("[[:punct:]]", " ", unlist(ListL))))
      })
    }
    if (M$DB[1] == "SCOPUS") {
      listCR <- lapply(listCR, function(l) {
        ListL <- lapply(l, function(x) {
          a <- strsplit(x, "\\., ")
          ind <- which(grepl("[[:digit:]]", a[[1]]))
          if (length(ind) == 0) ind <- 1
          x <- unlist(a[[1]][1:(ind[1] - 1)])
        })
        l <- trimws(trimES(gsub("[[:punct:]]", " ", unlist(ListL))))
      })
    }
  }


  if (field == "article") {
    # listCR=strsplit(M$CR,sep)
    listCR <- lapply(listCR, function(l) {
      l <- l[grep(",", l)]
    })
  }

  CR <- unlist(listCR)
  # CR=gsub("\\.","",CR)
  CR <- CR[nchar(CR) >= 3]
  CR <- trim.leading(CR)
  CR <- sort(table(CR), decreasing = TRUE)


  if (field == "article") {
    switch(M$DB[1],
      ISI = {
        listCR <- strsplit(rownames(CR), ",")
        Year <- unlist(lapply(listCR, function(l) {
          if (length(l) > 1) {
            l <- suppressWarnings(as.numeric(l[2]))
          } else {
            l <- NA
          }
        }))
        SO <- unlist(lapply(listCR, function(l) {
          if (length(l) > 2) {
            l <- l[3]
          } else {
            l <- NA
          }
        }))
        SO <- trim.leading(SO)
      },
      SCOPUS = {
        REF <- names(CR)
        y <- yearSoExtract(REF)
        Year <- as.numeric(y$Year)
        SO <- y$SO
      }
    )
  }
  CR <- list(Cited = CR, Year = Year, Source = SO)
  return(CR)
}


yearSoExtract <- function(string) {
  ## for scopus references
  ind <- regexpr("\\([[:digit:]]{4}\\)", string)
  ind[is.na(ind)] <- -1
  string[ind == -1] <- "(0000)"
  ind[ind == -1] <- 1
  attr(ind[ind == -1], "match.length") <- 6
  y <- unlist(regmatches(string, ind))
  Year <- substr(y, 2, 5)

  SO <- sub(",.*$", "", substr(string, ind + 7, nchar(string)))
  y <- list(Year = Year, SO = SO)
  return(y)
}

#' Number of documents published annually per Top Sources
#'
#' It calculates yearly published documents of the top sources.
#'
#' @param M is a data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to articles and variables to Field Tag in the original ISI or SCOPUS file.
#' @param top is a numeric. It indicates the number of top sources to analyze. The default value is 5.
#' @param cdf is a logical. If TRUE, the function calculates the cumulative occurrences distribution.
#' @return an object of class \code{data.frame}
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#' topSO <- sourceGrowth(scientometrics, top = 1, cdf = TRUE)
#' topSO
#'
#' # Plotting results
#' \dontrun{
#' install.packages("reshape2")
#' library(reshape2)
#' library(ggplot2)
#' DF <- melt(topSO, id = "Year")
#' ggplot(DF, aes(Year, value, group = variable, color = variable)) +
#'   geom_line()
#' }
#'
#' @export
#'
sourceGrowth <- function(M, top = 5, cdf = TRUE) {
  PY <- min(M$PY, na.rm = T):max(M$PY, na.rm = T)
  WSO <- cocMatrix(M, Field = "SO")
  if (is.null(dim(WSO))) {
    WSO <- cbind(WSO)
    colnames(WSO) <- M$SO[1]
  }
  if (top > dim(WSO)[2]) {
    top <- dim(WSO)[2]
  }

  M$PY <- as.character(M$PY)
  WPY <- cocMatrix(M, Field = "PY")
  i <- setdiff(PY, colnames(WPY))
  if (length(i) > 0) {
    WPY <- cbind(WPY, matrix(0, dim(WPY)[1], length(i)))
    colnames(WPY)[(dim(WPY)[2] - length(i) + 1):dim(WPY)[2]] <- as.character(i)
  }
  PYSO <- Matrix::crossprod(WPY, WSO)
  ind <- Matrix::colSums(PYSO)
  deg <- sort(ind, decreasing = T)[top]
  sonames <- colnames(PYSO)[ind >= deg]
  PYSO <- as.data.frame(as.matrix(PYSO[, ind >= deg]))

  PYSO <- cbind(as.numeric(colnames(WPY)), PYSO)

  PYSO <- PYSO[order(PYSO[, 1]), ]
  if (isTRUE(cdf)) {
    PYSO[, -1] <- apply(as.data.frame(PYSO[, -1]), 2, cumsum)
  }
  names(PYSO) <- c("Year", sonames)
  return(PYSO)
}

#' Find Author's Co-authored Works
#'
#' Searches for an author's name in a bibliometric dataframe and returns 
#' the DOIs and author positions of their co-authored works.
#'
#' @param author_name Character. The author's name to search for (case-insensitive)
#' @param data Data.frame. The bibliometric dataframe with AU and DI columns
#' @param partial_match Logical. If TRUE, allows partial name matching (default: TRUE)
#' @param exact_match Logical. If TRUE, requires exact name matching (default: FALSE)
#'
#' @return A data.frame with columns:
#'   \itemize{
#'     \item doi: DOI of the work
#'     \item author_position: Numerical position of the author in the author list
#'     \item total_authors: Total number of authors in the work
#'     \item all_authors: Complete list of authors for reference
#'     \item matched_name: The exact name variant that was matched
#'   }
#'
#' @details 
#' The function searches through the AU column which contains author names 
#' separated by semicolons. It identifies the position of the target author
#' and returns comprehensive information about each matching work.
#'
#' @examples
#' \dontrun{
#' # Find works by "ARIA M"
#' works <- findAuthorWorks("ARIA M", M)
#' 
#' # Find works with exact matching
#' works_exact <- findAuthorWorks("PESTANA MH", M, exact_match = TRUE)
#' 
#' # Find works with partial matching disabled
#' works_full <- findAuthorWorks("MASSIMO ARIA", M, partial_match = FALSE)
#' }
#'
#' @author Your Name
#' @export
findAuthorWorks <- function(author_name, 
                              data, 
                              partial_match = TRUE, 
                              exact_match = FALSE) {
  
  # Input validation
  if (missing(author_name) || !is.character(author_name) || nchar(trimws(author_name)) == 0) {
    stop("The 'author_name' parameter must be a non-empty character string")
  }
  
  if (missing(data) || !is.data.frame(data)) {
    stop("The 'data' parameter must be a data.frame")
  }
  
  if (!all(c("AU", "DI") %in% names(data))) {
    stop("The data.frame must contain 'AU' and 'DI' columns")
  }
  
  # Clean the input author name
  author_name <- trimws(author_name)
  
  # Initialize result dataframe
  result <- data.frame(
    doi = character(0),
    author_position = integer(0),
    total_authors = integer(0),
    all_authors = character(0),
    matched_name = character(0),
    stringsAsFactors = FALSE
  )
  
  # Process each row
  for (i in 1:nrow(data)) {
    au_string <- data$AU[i]
    doi <- data$DI[i]
    
    # Skip rows with missing AU or DI
    if (is.na(au_string) || is.na(doi) || au_string == "" || doi == "") {
      next
    }
    
    # Split authors by semicolon and clean
    authors <- trimws(unlist(strsplit(au_string, ";")))
    authors <- authors[authors != ""]  # Remove empty strings
    
    if (length(authors) == 0) {
      next
    }
    
    # Find matching positions
    matched_positions <- c()
    matched_names <- c()
    
    if (exact_match) {
      # Exact matching (case-insensitive)
      matches <- which(toupper(authors) == toupper(author_name))
      if (length(matches) > 0) {
        matched_positions <- matches
        matched_names <- authors[matches]
      }
    } else if (partial_match) {
      # Partial matching (case-insensitive)
      matches <- which(grepl(toupper(author_name), toupper(authors), fixed = TRUE))
      if (length(matches) > 0) {
        matched_positions <- matches
        matched_names <- authors[matches]
      }
    } else {
      # Full name matching (case-insensitive, but complete match)
      matches <- which(toupper(authors) == toupper(author_name))
      if (length(matches) > 0) {
        matched_positions <- matches
        matched_names <- authors[matches]
      }
    }
    
    # Add matches to result
    if (length(matched_positions) > 0) {
      for (j in seq_along(matched_positions)) {
        new_row <- data.frame(
          doi = doi,
          author_position = matched_positions[j],
          total_authors = length(authors),
          all_authors = paste(authors, collapse = "; "),
          matched_name = matched_names[j],
          stringsAsFactors = FALSE
        )
        result <- rbind(result, new_row)
      }
    }
  }
  
  # Add summary information as attributes
  if (nrow(result) > 0) {
    attr(result, "search_summary") <- list(
      searched_author = author_name,
      total_works_found = nrow(result),
      unique_dois = length(unique(result$doi)),
      search_type = ifelse(exact_match, "exact", 
                           ifelse(partial_match, "partial", "full_name")),
      unique_matched_names = unique(result$matched_name)
    )
    
    # Sort by DOI for consistency
    result <- result[order(result$doi, result$author_position), ]
    rownames(result) <- NULL
  }
  
  return(result)
}

#' Print Summary of Author Works Search
#'
#' Prints a summary of the search results from find_author_works()
#'
#' @param works_df Data.frame. Result from find_author_works()
#' @export
print_author_works_summary <- function(works_df) {
  summary_info <- attr(works_df, "search_summary")
  
  if (is.null(summary_info)) {
    cat("No search summary available. Use find_author_works() to generate results.\n")
    return()
  }
  
  cat("=== Author Works Search Summary ===\n")
  cat("Searched for:", summary_info$searched_author, "\n")
  cat("Search type:", summary_info$search_type, "\n")
  cat("Total works found:", summary_info$total_works_found, "\n")
  cat("Unique DOIs:", summary_info$unique_dois, "\n")
  cat("Matched name variants:\n")
  for (name in summary_info$unique_matched_names) {
    cat("  -", name, "\n")
  }
  cat("\n")
  
  if (nrow(works_df) > 0) {
    cat("Position distribution:\n")
    pos_table <- table(works_df$author_position)
    for (pos in names(pos_table)) {
      cat("  Position", pos, ":", pos_table[pos], "works\n")
    }
  }
}

utils::globalVariables(c("Paper", "Tag", "content", "cont"))
isi2df <- function(D) {
  # D <- D[nchar(D)>0]  # remove empty rows

  # remove empty rows and strange characters
  res <- try(D <- D[nchar(D) > 1], silent = T)
  if (inherits(res, "try-error")) {
    D <- removeStrangeChar(D)
    # next
  } else {
    D <- res
    rm(res)
  }

  D <- D[!(substr(D, 1, 3) %in% c("FN ", "VR "))]

  for (i in 1:length(D)) {
    if (substr(D[i], 1, 3) == "   ") substr(D[i], 1, 3) <- substr(D[i - 1], 1, 3)
  }
  Papers <- which(substr(D, 1, 3) == "PT ") # first row of each document
  nP <- length(Papers) # number of documents

  rowPapers <- diff(c(Papers, length(D) + 1))

  numPapers <- rep(1:nP, rowPapers)

  DATA <- data.frame(Tag = substr(D, 1, 3), content = substr(D, 4, nchar(D)), Paper = numPapers)
  DATA$Tag <- gsub(" ", "", DATA$Tag)
  df <- DATA %>%
    group_by(Paper, Tag) %>%
    summarise(cont = paste(content, collapse = "---", sep = "")) %>%
    arrange(Tag, Paper) %>%
    pivot_wider(names_from = Tag, values_from = cont) %>%
    ungroup()
  df <- as.data.frame(df)


  df$PY <- as.numeric(df$PY)

  missingTags <- setdiff(c("AU", "DE", "C1", "RP", "CR", "PY", "SO", "TI", "TC"), names(df))
  if (length(missingTags) > 0) {
    cat("\nWarning:\nIn your file, some mandatory metadata are missing. Bibliometrix functions may not work properly!\n
Please, take a look at the vignettes:
- 'Data Importing and Converting' (https://www.bibliometrix.org/vignettes/Data-Importing-and-Converting.html)
- 'A brief introduction to bibliometrix' (https://www.bibliometrix.org/vignettes/Introduction_to_bibliometrix.html)\n\n")
    cat("\nMissing fields: ", missingTags, "\n")
  }

  ### replace "---" with ";"
  tagsComma <- c("AU", "AF", "CR")

  nolab <- setdiff(tagsComma, names(df))

  tagsComma <- tagsComma[(!(tagsComma %in% nolab))]

  df1 <- data.frame(lapply(df[tagsComma], function(x) {
    gsub("---", ";", x)
  }))

  ### replace "---" with " "
  otherTags <- setdiff(names(df), tagsComma)
  df2 <- data.frame(lapply(df[otherTags], function(x) {
    trimES(gsub("---", " ", x))
  }))
  df <- cbind(df1, df2)
  rm(df1, df2)

  ### store raw affiliation format to extract link among authors and affiliations
  df$C1raw <- df$C1
  ###

  df$DB <- "ISI"

  # Authors
  df$AU <- trimES(gsub(",", " ", df$AU))

  # Toupper
  DI <- df$DI
  AB <- df$AB
  TI <- df$TI
  DE <- df$DE
  df <- data.frame(lapply(df, toupper))
  df$DI <- DI
  df$AB_raw <- AB
  df$TI_raw <- TI
  df$DE_raw <- DE

  # add sep ; to affiliations
  df$C1 <- trim(gsub("\\[.*?\\]", "", df$C1)) # to remove author info in square brackets
  df$C1 <- gsub("\\.", ".;", df$C1)

  df <- df[names(df) != "Paper"]

  return(df)
}

# Internal helpers shared by completeMetadata() and biblioshiny's
# OpenAlex query builder. They convert OpenAlex "work" list objects into
# WoS-style cited-reference strings (used by bibliometrix CR analyses).
# Promoted from inst/biblioshiny/openalex_api.R so the package code can
# reuse the same conversion path without depending on Shiny.

# ---- LTWA-style abbreviation lookup -----------------------------------------

#' @noRd
.journal_word_abbrev <- c(
  "JOURNAL" = "J", "INTERNATIONAL" = "INT", "AMERICAN" = "AM",
  "SCIENCE" = "SCI", "SCIENCES" = "SCI", "SCIENTIFIC" = "SCI",
  "TECHNOLOGY" = "TECHNOL", "TECHNOLOGICAL" = "TECHNOL",
  "RESEARCH" = "RES", "REVIEW" = "REV", "REVIEWS" = "REV",
  "MANAGEMENT" = "MANAGE", "INFORMATION" = "INFORM",
  "ANNUAL" = "ANN", "SOCIAL" = "SOC", "SOCIETY" = "SOC",
  "APPLIED" = "APPL", "APPLICATION" = "APPL", "APPLICATIONS" = "APPL",
  "ENGINEERING" = "ENG", "EDUCATION" = "EDUC", "EDUCATIONAL" = "EDUC",
  "ECONOMICS" = "ECON", "ECONOMIC" = "ECON",
  "ENVIRONMENTAL" = "ENVIRON", "ENVIRONMENT" = "ENVIRON",
  "PSYCHOLOGY" = "PSYCHOL", "PSYCHOLOGICAL" = "PSYCHOL",
  "BIOLOGICAL" = "BIOL", "BIOLOGY" = "BIOL",
  "MEDICAL" = "MED", "MEDICINE" = "MED", "CLINICAL" = "CLIN",
  "PHYSICAL" = "PHYS", "PHYSICS" = "PHYS",
  "CHEMISTRY" = "CHEM", "CHEMICAL" = "CHEM",
  "MATHEMATICAL" = "MATH", "MATHEMATICS" = "MATH",
  "POLITICAL" = "POLIT", "POLITICS" = "POLIT",
  "COMMUNICATION" = "COMMUN", "COMMUNICATIONS" = "COMMUN",
  "COMPUTATIONAL" = "COMPUT", "COMPUTER" = "COMPUT", "COMPUTING" = "COMPUT",
  "STATISTICAL" = "STAT", "STATISTICS" = "STAT",
  "EUROPEAN" = "EUR", "BRITISH" = "BRIT", "CANADIAN" = "CAN",
  "NATIONAL" = "NATL", "GENERAL" = "GEN",
  "AGRICULTURAL" = "AGRIC", "AGRICULTURE" = "AGRIC",
  "GEOGRAPHY" = "GEOGR", "GEOGRAPHICAL" = "GEOGR",
  "PHILOSOPHY" = "PHILOS", "PHILOSOPHICAL" = "PHILOS",
  "UNIVERSITY" = "UNIV", "ASSOCIATION" = "ASSOC",
  "DEVELOPMENT" = "DEV", "EXPERIMENTAL" = "EXP",
  "PROCEEDINGS" = "PROC", "TRANSACTIONS" = "TRANS",
  "LETTERS" = "LETT", "REPORTS" = "REP", "REPORT" = "REP",
  "ANALYSIS" = "ANAL", "ANALYTICAL" = "ANAL",
  "BEHAVIOR" = "BEHAV", "BEHAVIORAL" = "BEHAV",
  "BEHAVIOUR" = "BEHAV", "BEHAVIOURAL" = "BEHAV",
  "COGNITIVE" = "COGN", "CULTURAL" = "CULT", "CULTURE" = "CULT",
  "CURRENT" = "CURR", "DISEASE" = "DIS", "DISEASES" = "DIS",
  "ECOLOGY" = "ECOL", "ECOLOGICAL" = "ECOL",
  "ENERGY" = "ENERG", "MATERIALS" = "MATER", "MOLECULAR" = "MOL",
  "NUTRITION" = "NUTR", "NUTRITIONAL" = "NUTR",
  "OPERATIONS" = "OPER", "ORGANIZATION" = "ORGAN", "ORGANIZATIONAL" = "ORGAN",
  "PHARMACEUTICAL" = "PHARM", "PHARMACY" = "PHARM",
  "STRUCTURAL" = "STRUCT", "SUSTAINABLE" = "SUSTAIN", "SUSTAINABILITY" = "SUSTAIN",
  "THEORETICAL" = "THEOR", "THERAPY" = "THER", "THERAPEUTIC" = "THER",
  "TOURISM" = "TOUR", "OPINION" = "OPIN",
  "NETWORKS" = "NETW", "NETWORK" = "NETW",
  "DECISION" = "DECIS", "MECHANICS" = "MECH", "MECHANICAL" = "MECH",
  "HOSPITALITY" = "HOSP", "PROCESSING" = "PROCESS",
  "SYSTEMS" = "SYST", "SYSTEM" = "SYST",
  "DYNAMICS" = "DYN", "DYNAMIC" = "DYN",
  "PUBLIC" = "PUBL", "POLICY" = "POL",
  "GLOBAL" = "GLOB", "ATMOSPHERIC" = "ATMOS",
  "BIOMEDICAL" = "BIOMED", "BIOCHEMISTRY" = "BIOCHEM", "BIOCHEMICAL" = "BIOCHEM",
  "MICROBIOLOGY" = "MICROBIOL", "GENETICS" = "GENET",
  "NEUROSCIENCE" = "NEUROSCI", "NEUROLOGY" = "NEUROL",
  "ADVANCED" = "ADV", "ADVANCES" = "ADV",
  "METHODS" = "METH", "METHODOLOGY" = "METHODOL",
  "HISTORY" = "HIST", "HISTORICAL" = "HIST",
  "ELECTRONIC" = "ELECTRON", "GEOPHYSICAL" = "GEOPHYS",
  "INDUSTRIAL" = "IND", "INDUSTRY" = "IND",
  "MARINE" = "MAR", "VETERINARY" = "VET",
  "ORGANIC" = "ORG", "INORGANIC" = "INORG",
  "ZOOLOGY" = "ZOOL", "ZOOLOGICAL" = "ZOOL",
  "ASTRONOMY" = "ASTRON", "ASTROPHYSICAL" = "ASTROPHYS",
  "ARCHIVES" = "ARCH", "ARCHIVE" = "ARCH",
  "LIBRARY" = "LIBR", "PROBLEMS" = "PROBL",
  "PLANNING" = "PLAN", "REGIONAL" = "REG",
  "STUDIES" = "STUD", "QUARTERLY" = "Q",
  "ABSTRACT" = "ABSTR", "BULLETIN" = "BULL",
  "CONGRESS" = "CONGR", "CONFERENCE" = "CONF",
  "DOCUMENTATION" = "DOC", "GEOGRAPHIC" = "GEOGR",
  "ENGINEERING" = "ENG", "LANGUAGE" = "LANG",
  "LINGUISTICS" = "LINGUIST", "SURGERY" = "SURG", "SURGICAL" = "SURG"
)

#' @noRd
.journal_stop_words <- c(
  "of", "the", "and", "for", "in", "on", "a", "an", "to",
  "de", "la", "le", "les", "des", "du", "et",
  "der", "die", "das", "und", "fur",
  "del", "di", "e", "il", "lo"
)

# ---- Helpers ----------------------------------------------------------------

#' @noRd
abbreviate_source_name <- function(name) {
  if (is.null(name) || is.na(name) || name == "") return("")

  # NB: split on whitespace/colon/comma. The POSIX class [[:space:]] is used
  # instead of \s because R's default (TRE) regex engine does not recognise
  # \s inside a bracket expression -- "[\\s:,]" would match the literal
  # characters \, s, : and , , splitting journal names on every letter "s".
  words <- unlist(strsplit(name, "[[:space:]:,]+"))
  words <- words[words != ""]

  if (length(words) > 1) {
    keep <- c(TRUE, !tolower(words[-1]) %in% .journal_stop_words)
    words <- words[keep]
  }

  abbreviated <- vapply(words, function(w) {
    W <- toupper(w)
    if (W %in% names(.journal_word_abbrev)) {
      return(.journal_word_abbrev[[W]])
    }
    if (nchar(W) <= 4) return(W)
    return(substr(W, 1, 5))
  }, character(1), USE.NAMES = FALSE)

  paste(toupper(abbreviated), collapse = " ")
}

#' @noRd
format_cr_author <- function(authorships) {
  if (is.null(authorships) || length(authorships) == 0) return("")
  first_auth <- authorships[[1]]
  name <- first_auth$author$display_name
  if (is.null(name) || is.na(name) || name == "") return("")

  parts <- unlist(strsplit(trimws(name), "\\s+"))
  if (length(parts) == 0) return("")
  if (length(parts) == 1) return(toupper(parts[1]))

  surname <- toupper(parts[length(parts)])
  given <- parts[-length(parts)]
  initials <- paste(substr(toupper(given), 1, 1), collapse = "")

  paste0(surname, " ", initials)
}

#' @noRd
build_single_cr <- function(work) {
  parts <- character(0)

  auth <- format_cr_author(work$authorships)
  if (auth != "") parts <- c(parts, auth)

  year <- work$publication_year
  if (!is.null(year) && !is.na(year)) parts <- c(parts, as.character(year))

  source_name <- tryCatch(
    work$primary_location$source$display_name,
    error = function(e) NULL
  )
  if (!is.null(source_name) && !is.na(source_name) && source_name != "") {
    parts <- c(parts, abbreviate_source_name(source_name))
  }

  vol <- tryCatch(work$biblio$volume, error = function(e) NULL)
  if (!is.null(vol) && !is.na(vol) && vol != "") {
    parts <- c(parts, paste0("V", vol))
  }

  page <- tryCatch(work$biblio$first_page, error = function(e) NULL)
  if (!is.null(page) && !is.na(page) && page != "") {
    parts <- c(parts, paste0("P", page))
  }

  doi <- work$doi
  if (!is.null(doi) && !is.na(doi) && doi != "") {
    doi_clean <- gsub("^https?://doi\\.org/", "", doi)
    parts <- c(parts, paste0("DOI ", toupper(doi_clean)))
  }

  if (length(parts) == 0) return(NA_character_)
  paste(parts, collapse = ", ")
}

utils::globalVariables(c(
  "AU",
  "listAU",
  "logo",
  "N.Articles",
  "N.Authors",
  "Theoretical",
  "Empirical",
  "Fitted"
))
#' Lotka's law coefficient estimation
#'
#' It estimates Lotka's law coefficients for scientific productivity and tests the goodness of fit.
#'
#' Lotka's Law, first formulated by Alfred J. Lotka in 1926, describes the frequency distribution
#' of scientific productivity among authors. The law states that the number of authors producing
#' \eqn{n} publications is approximately \eqn{C / n^\beta}, where \eqn{C} is a constant and
#' \eqn{\beta} is the productivity exponent.\cr\cr
#'
#' In the original formulation, Lotka proposed that \eqn{\beta = 2}, meaning that the number of
#' authors who publish \eqn{n} papers is approximately \eqn{1/n^2} of those who publish one paper.
#' The function estimates both the empirical \eqn{\beta} via regression and tests the fit of
#' the theoretical distribution (\eqn{\beta = 2}) using a Kolmogorov-Smirnov test.\cr\cr
#'
#' Reference:\cr
#' Lotka, A. J. (1926). The frequency distribution of scientific productivity. Journal of the
#' Washington Academy of Sciences, 16(12), 317-323.\cr
#'
#' @param M is an object of the class '\code{bibliometrixDB}'.
#' @return The function \code{lotka} returns a list containing the following objects:
#' \tabular{lll}{
#' \code{AuthorProd}    \tab   \tab Authors' Productivity frequency table\cr
#' \code{g}             \tab   \tab Lotka's law plot in ggplot2 format (with logo)\cr
#' \code{g_shiny}       \tab   \tab Lotka's law plot for biblioshiny (without logo)\cr
#' \code{stat}          \tab   \tab list of statistical results (Beta, C, R2, KS tests)\cr
#' \code{Beta}          \tab   \tab Beta coefficient (estimated)\cr
#' \code{C}             \tab   \tab Constant coefficient\cr
#' \code{R2}            \tab   \tab Goodness of Fit (R-squared)\cr
#' \code{fitted}        \tab   \tab Fitted Values\cr
#' \code{p.value}       \tab   \tab p-value of KS test (theoretical Beta=2)}
#'
#' @examples
#' data(management, package = "bibliometrixData")
#' L <- lotka(management)
#' L
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{summary}} method for class '\code{bibliometrix}'
#'
#' @export

lotka <- function(M) {
  if (!inherits(M, "bibliometrixDB")) {
    cat('\n argument M have to be an object of class "bibliometrixDB"\n')
    return(NA)
  }

  ## 1. Author Productivity table ----
  AUdf <- M %>%
    mutate(
      listAU = stringr::str_split(as.character(AU), ";"),
      listAU = purrr::map(listAU, stringr::str_trim)
    ) %>%
    select(listAU) %>%
    unnest(cols = listAU) %>%
    count(listAU, sort = TRUE, name = "Freq") %>%
    rename(AU = listAU)

  AuthorProd <- aggregate(AUdf, by = list(AUdf$Freq), "length")
  AuthorProd[, 2] <- as.numeric(AuthorProd[, 2])
  AuthorProd[, 3] <- AuthorProd[, 2] / sum(AuthorProd[, 2])
  names(AuthorProd) <- c("N.Articles", "N.Authors", "Freq")

  nAuthors <- sum(AuthorProd$N.Authors)

  ## 2. Fit Lotka's Law: log10(Freq) ~ log10(N.Articles) ----
  LOTKA <- lm(log10(Freq) ~ log10(N.Articles), data = AuthorProd)
  Beta <- abs(as.numeric(coef(LOTKA)[2]))
  C <- 10^(as.numeric(coef(LOTKA)[1]))
  R2 <- summary(LOTKA)$r.squared
  fitted_vals <- 10^LOTKA$fitted.values

  ## 3. Theoretical distribution (Beta = 2) ----
  Yt <- coef(LOTKA)[1] - 2 * log10(AuthorProd[, 1])
  Theoretical_beta2 <- 10^Yt
  Theoretical_beta2 <- Theoretical_beta2 / sum(Theoretical_beta2)

  ## 4. Fitted distribution (empirical Beta) ----
  Fitted_betaEmp <- 10^(log10(C) - Beta * log10(AuthorProd[, 1]))
  Fitted_betaEmp <- Fitted_betaEmp / sum(Fitted_betaEmp)

  ## 5. KS goodness-of-fit tests ----
  # Test 1: Empirical vs theoretical (Beta = 2)
  ks_theoretical <- suppressWarnings(
    ks.test(AuthorProd[, 3], Theoretical_beta2, exact = FALSE)
  )

  # Test 2: Empirical vs fitted (empirical Beta)
  ks_fitted <- suppressWarnings(
    ks.test(AuthorProd[, 3], Fitted_betaEmp, exact = FALSE)
  )

  stat <- list(
    Beta = Beta,
    C = C,
    R2 = R2,
    ks.theo.stat = as.numeric(ks_theoretical$statistic),
    ks.theo.pvalue = ks_theoretical$p.value,
    ks.fit.stat = as.numeric(ks_fitted$statistic),
    ks.fit.pvalue = ks_fitted$p.value
  )

  ## 6. Prepare plot data ----
  AuthorProd$Theoretical <- Theoretical_beta2
  AuthorProd$Fitted <- Fitted_betaEmp

  # Subtitle with statistics
  subtitle_text <- paste0(
    "Estimated: Beta = ",
    round(Beta, 2),
    ", C = ",
    round(C, 4),
    "    |    R2 = ",
    round(R2, 4),
    "    |    KS p-value (Beta=2): ",
    format.pval(ks_theoretical$p.value, digits = 3),
    "    |    KS p-value (fitted): ",
    format.pval(ks_fitted$p.value, digits = 3)
  )

  ## 7. Plot ----
  g_shiny <- suppressWarnings(ggplot2::ggplot(AuthorProd, ggplot2::aes(x = N.Articles)) +
    # Empirical line
    ggplot2::geom_line(
      ggplot2::aes(
        y = Freq * 100,
        group = 1,
        text = paste0(
          "Documents written: ",
          N.Articles,
          "\nN. of Authors: ",
          N.Authors,
          "\n% of Authors: ",
          round(Freq * 100, 2)
        )
      ),
      color = "#1a1a1a",
      linewidth = 0.7
    ) +
    ggplot2::geom_point(
      ggplot2::aes(
        y = Freq * 100,
        text = paste0(
          "Documents written: ",
          N.Articles,
          "\nN. of Authors: ",
          N.Authors,
          "\n% of Authors: ",
          round(Freq * 100, 2)
        )
      ),
      color = "#1a1a1a",
      size = 2
    ) +
    # Theoretical line (Beta = 2)
    ggplot2::geom_line(
      ggplot2::aes(y = Theoretical * 100, group = 1),
      linetype = "dashed",
      color = "#D6604D",
      linewidth = 0.8,
      alpha = 0.8
    ) +
    # Fitted line (empirical Beta)
    ggplot2::geom_line(
      ggplot2::aes(y = Fitted * 100, group = 1),
      linetype = "dotdash",
      color = "#2171B5",
      linewidth = 0.8,
      alpha = 0.8
    ) +
    ggplot2::xlim(0, max(AuthorProd$N.Articles) + 1) +
    ggplot2::labs(
      x = "Documents Written",
      y = "% of Authors",
      title = "Lotka's Law",
      subtitle = subtitle_text
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      text = ggplot2::element_text(color = "#333333"),
      plot.title = ggplot2::element_text(size = 18, face = "bold", hjust = 0.5),
      plot.subtitle = ggplot2::element_text(
        size = 10,
        color = "#666666",
        hjust = 0.5
      ),
      axis.title = ggplot2::element_text(size = 12),
      axis.title.y = ggplot2::element_text(angle = 90, vjust = 1),
      axis.line = ggplot2::element_line(color = "#333333", linewidth = 0.4),
      panel.grid.major = ggplot2::element_line(
        color = "#EBEBEB",
        linewidth = 0.3
      ),
      panel.grid.minor = ggplot2::element_blank(),
      legend.position = "none"
    ))

  # Version with logo for export
  x_logo <- c(
    max(AuthorProd$N.Articles) - diff(range(AuthorProd$N.Articles)) * 0.10,
    max(AuthorProd$N.Articles)
  ) +
    1
  y_logo <- c(
    min(AuthorProd$Freq * 100),
    min(AuthorProd$Freq * 100) + diff(range(AuthorProd$Freq * 100)) * 0.10
  )

  data("logo", package = "bibliometrix", envir = environment())
  logoGrid <- grid::rasterGrob(logo, interpolate = TRUE)

  g <- g_shiny +
    ggplot2::annotation_custom(
      logoGrid,
      xmin = x_logo[1],
      xmax = x_logo[2],
      ymin = y_logo[1],
      ymax = y_logo[2]
    )

  ## 8. Clean table for output ----
  AuthorProd_out <- AuthorProd[, c("N.Articles", "N.Authors", "Freq")]
  names(AuthorProd_out) <- c(
    "Documents written",
    "N. of Authors",
    "Proportion of Authors"
  )

  L <- list(
    AuthorProd = AuthorProd_out,
    g = g,
    g_shiny = g_shiny,
    stat = stat,
    Beta = Beta,
    C = C,
    R2 = R2,
    fitted = fitted_vals,
    p.value = ks_theoretical$p.value
  )
  return(L)
}

# Test per lotka e bradford

test_that("lotka funziona con dati bibliometrici", {
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  class(scientometrics) <- c("bibliometrixDB", "data.frame")
  L <- lotka(scientometrics)
  expect_type(L, "list")
  expect_true(all(c("AuthorProd", "Beta", "C", "R2", "p.value") %in% names(L)))
  expect_true(L$Beta > 0)
  expect_true(L$R2 >= 0 && L$R2 <= 1)
})

test_that("bradford funziona e restituisce zone", {
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  BR <- bradford(scientometrics)
  expect_type(BR, "list")
  expect_true("table" %in% names(BR))
  expect_true(is.data.frame(BR$table))
  expect_true("Zone" %in% names(BR$table))
})

#' Open a bibliometrix network in VosViewer
#'
#' \code{net2VOSviewer} plots a network created with \code{\link{networkPlot}} using \href{https://www.vosviewer.com/}{VOSviewer} by Nees Jan van Eck and Ludo Waltman.
#'
#' The function \code{\link{networkPlot}} can plot a bibliographic network previously created by \code{\link{biblioNetwork}}.
#' The network map can be plotted using internal R routines or using \href{https://www.vosviewer.com/}{VOSviewer} by Nees Jan van Eck and Ludo Waltman.
#'
#'
#' @param net is an object created by networkPlot function.
#' @param vos.path is a character indicating the full path where VOSviewer.jar is located.
#' @return It write a .net file that can be open in VOSviewer
#'
#' @examples
#' # EXAMPLE
#'
#' # VOSviewer.jar have to be present in the working folder
#'
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # NetMatrix <- biblioNetwork(scientometrics, analysis = "co-citation",
#' # network = "references", sep = ";")
#'
#' # net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Co-Citation",labelsize=0.5)
#'
#' # net2VOSviewer(net)
#'
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @seealso \code{\link{networkPlot}} to create and plot a network object
#'
#' @export


net2VOSviewer <- function(net, vos.path = NULL) {
  net <- net$graph_pajek
  V(net)$id <- V(net)$name

  if (is.null(vos.path)) {
    vos.path <- getwd()
  }
  if (sum(dir(vos.path) %in% "VOSviewer.jar") == 0) {
    cat(
      paste(
        "VOSviewer.jar does not exist in the path",
        vos.path,
        "\n\nPlese download it from https://www.vosviewer.com/download",
        "\n(Java version for other systems)\n"
      )
    )
  } else {
    netfile <- paste(vos.path, "/", "vosnetwork.net", sep = "")
    VOScommand <- paste("java -jar ",
      vos.path,
      "/",
      "VOSviewer.jar -pajek_network ",
      netfile,
      sep = ""
    )
    write.graph(
      graph = net,
      file = netfile,
      format = "pajek"
    )
    system(VOScommand, wait = FALSE)
  }
}

# ==============================================================================
# UI Component - Query Builder for PubMed API (Collapsible)
# ==============================================================================

pubmedUI <- function() {
  tagList(
    fluidRow(
      column(
        width = 12,
        # Query Builder Box (Collapsible)
        wellPanel(
          id = "pmQueryPanel",

          # Header with collapse button
          div(
            style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 15px;",
            h4(
              "PubMed Data Collection",
              style = "font-weight: bold; margin: 0;"
            ),
            actionButton(
              "pmToggleQueryPanel",
              icon("chevron-up"),
              class = "btn-sm btn-outline-secondary",
              style = "padding: 5px 10px;",
              title = "Collapse/Expand"
            )
          ),

          # Query content (collapsible)
          div(
            id = "pmQueryContent",

            # Query Builder Section
            div(
              id = "pm-query-builder-container",
              style = "margin-bottom: 20px;",

              # Initial query row (NO operator column)
              div(
                id = "pm-query-row-1",
                class = "pm-query-row",
                style = "margin-bottom: 10px;",
                fluidRow(
                  column(1),
                  column(
                    width = 2,
                    selectInput(
                      "pmField_1",
                      NULL,
                      choices = c(
                        "All Fields" = "all",
                        "Title" = "title",
                        "Abstract" = "abstract",
                        "Title/Abstract" = "tiab",
                        "Author" = "author",
                        "MeSH Terms" = "mesh",
                        "Affiliation" = "affiliation"
                      ),
                      selected = "tiab",
                      width = "100%"
                    )
                  ),
                  column(
                    width = 8,
                    textInput(
                      "pmQuery_1",
                      NULL,
                      value = "",
                      placeholder = 'Example: "machine learning" OR AI',
                      width = "100%"
                    )
                  ),
                  column(
                    width = 1,
                    div(
                      id = "pm-remove-button-1",
                      style = "visibility: hidden;",
                      actionButton(
                        "pmRemoveRow_1",
                        icon("minus-circle"),
                        class = "btn-danger btn-sm",
                        style = "margin-top: 0px;",
                        title = "Remove row"
                      )
                    )
                  )
                )
              )
            ),

            # Help text for search syntax
            div(
              style = "margin-bottom: 15px; font-size: 12px; color: #666;",
              HTML(
                "<strong>Search tips:</strong> Use quotes for exact phrases (e.g., \"machine learning\"). Boolean operators AND, OR, NOT must be UPPERCASE. Learn more at <a href='https://pubmed.ncbi.nlm.nih.gov/help/' target='_blank'>PubMed Help</a>."
              )
            ),

            # Add row and date range buttons
            div(
              style = "margin-bottom: 20px;",
              actionButton(
                "pmAddRow",
                label = "+ Add row",
                icon = icon("plus"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "pmToggleDateRange",
                label = "- Date range",
                icon = icon("calendar"),
                class = "btn-outline-primary btn-sm",
                style = "margin-left: 10px;"
              )
            ),

            # Date Range Section (VISIBLE by default)
            div(
              id = "pmDateRangeSection",
              style = "display: block;",
              wellPanel(
                style = "background-color: #f8f9fa;",
                h5("Date Range Filter", style = "font-weight: bold;"),
                fluidRow(
                  column(
                    width = 6,
                    numericInput(
                      "pmYearFrom",
                      "From Year:",
                      value = 1985,
                      min = 1800,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  ),
                  column(
                    width = 6,
                    numericInput(
                      "pmYearTo",
                      "To Year:",
                      value = as.integer(format(Sys.Date(), "%Y")),
                      min = 1800,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  )
                )
              )
            ),

            # Advanced filters toggle
            actionButton(
              "pmToggleAdvanced",
              label = "Hide advanced filters",
              icon = icon("filter"),
              class = "btn-link",
              style = "padding: 5px; margin-bottom: 10px;"
            ),

            # Additional Filters Section (VISIBLE by default)
            div(
              id = "pmAdvancedSection",
              style = "display: block;",
              wellPanel(
                style = "background-color: #f8f9fa;",
                fluidRow(
                  column(
                    width = 4,
                    selectizeInput(
                      "pmDocType",
                      "Document Type:",
                      choices = c(
                        "All" = "",
                        "Journal Article" = "Journal Article",
                        "Review" = "Review",
                        "Clinical Trial" = "Clinical Trial",
                        "Meta-Analysis" = "Meta-Analysis",
                        "Case Reports" = "Case Reports",
                        "Randomized Controlled Trial" = "Randomized Controlled Trial",
                        "Systematic Review" = "Systematic Review",
                        "Letter" = "Letter",
                        "Editorial" = "Editorial"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    selectizeInput(
                      "pmLanguage",
                      "Language:",
                      choices = c(
                        "All" = "",
                        "English" = "english",
                        "Spanish" = "spanish",
                        "French" = "french",
                        "German" = "german",
                        "Italian" = "italian",
                        "Portuguese" = "portuguese",
                        "Chinese" = "chinese",
                        "Japanese" = "japanese"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    numericInput(
                      "pmMaxRecords",
                      "Max Records:",
                      value = 1000,
                      min = 1,
                      max = 10000,
                      step = 100
                    )
                  )
                )
              )
            ),

            # Action buttons with reactive count
            div(
              style = "margin-top: 10px; display: flex; justify-content: space-between; align-items: center;",

              # Left side: Reactive query count message
              div(
                style = "display: flex; align-items: center;",
                uiOutput("pmQueryCountInfo")
              ),

              # Right side: Clear and Search buttons
              div(
                style = "display: flex; gap: 10px;",
                actionButton(
                  "pmClearQuery",
                  "Clear",
                  icon = icon("times"),
                  class = "btn-outline-secondary"
                ),
                actionButton(
                  "pmFetchData",
                  "Search",
                  icon = icon("search"),
                  class = "btn-primary"
                )
              )
            )
          )
        )
      )
    ),

    # Data Display Section (shown when data is available)
    conditionalPanel(
      condition = "output.pmDataAvailable",
      fluidRow(
        column(
          width = 12,
          wellPanel(
            h4("Downloaded Collection", style = "font-weight: bold;"),

            # Progress and results info
            uiOutput("pmProgressInfo"),

            # Export buttons
            div(
              style = "margin-bottom: 15px; margin-top: 15px;",
              downloadButton(
                "pmDownloadExcel",
                "Export to Excel",
                icon = icon("file-excel"),
                class = "btn-success"
              ),
              downloadButton(
                "pmDownloadRData",
                "Export to RData",
                icon = icon("database"),
                class = "btn-info",
                style = "margin-left: 10px;"
              )
            ),

            # Data table output
            div(
              style = "margin-top: 15px;",
              uiOutput("pmDataTable")
            )
          )
        )
      )
    ),

    # JavaScript for collapsible functionality
    tags$script(HTML(
      "
      $(document).ready(function() {
        // Initialize collapsed state
        var isCollapsed = false;
        
        // Initialize visibility states as TRUE (open)
        var dateRangeVisible = true;
        var advancedVisible = true;
        
        // Handle toggle button click for main panel
        $('#pmToggleQueryPanel').on('click', function() {
          isCollapsed = !isCollapsed;
          $('#pmQueryContent').slideToggle(300);
          
          // Update button icon
          if (isCollapsed) {
            $(this).html('<i class=\"fa fa-chevron-down\"></i>');
          } else {
            $(this).html('<i class=\"fa fa-chevron-up\"></i>');
          }
        });
        
        // Handle date range toggle
        $('#pmToggleDateRange').on('click', function() {
          dateRangeVisible = !dateRangeVisible;
          $('#pmDateRangeSection').slideToggle(300);
          
          // Update button text
          if (dateRangeVisible) {
            $(this).html('<i class=\"fa fa-calendar\"></i> - Date range');
          } else {
            $(this).html('<i class=\"fa fa-calendar\"></i> + Date range');
          }
          
          // Notify Shiny
          Shiny.setInputValue('pmShowDateRange', dateRangeVisible);
        });
        
        // Handle advanced filters toggle
        $('#pmToggleAdvanced').on('click', function() {
          advancedVisible = !advancedVisible;
          $('#pmAdvancedSection').slideToggle(300);
          
          // Update button text
          if (advancedVisible) {
            $(this).html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
          } else {
            $(this).html('<i class=\"fa fa-filter\"></i> Show advanced filters');
          }
          
          // Notify Shiny
          Shiny.setInputValue('pmShowAdvanced', advancedVisible);
        });
        
        // Set initial values in Shiny
        Shiny.setInputValue('pmShowDateRange', true);
        Shiny.setInputValue('pmShowAdvanced', true);
      });
      
      // Custom message handler to collapse panel when data is loaded
      Shiny.addCustomMessageHandler('pmCollapseQueryPanel', function(message) {
        $('#pmQueryContent').slideUp(300);
        $('#pmToggleQueryPanel').html('<i class=\"fa fa-chevron-down\"></i>');
      });
    "
    ))
  )
}


# ==============================================================================
# Server Component - Query Builder for PubMed
# ==============================================================================

pubmedServer <- function(input, output, session, values) {
  # Reactive values to track query rows and toggle states
  queryRows <- reactiveVal(1)
  showDateRange <- reactiveVal(TRUE) # TRUE by default
  showAdvanced <- reactiveVal(TRUE) # TRUE by default

  # Add new query row
  observeEvent(input$pmAddRow, {
    currentRows <- queryRows()
    newRowId <- currentRows + 1
    queryRows(newRowId)

    # Insert new row in UI (WITH operator column)
    insertUI(
      selector = "#pm-query-builder-container",
      where = "beforeEnd",
      ui = div(
        id = paste0("pm-query-row-", newRowId),
        class = "pm-query-row",
        style = "margin-bottom: 10px;",
        fluidRow(
          column(
            width = 1,
            selectInput(
              paste0("pmOperator_", newRowId),
              NULL,
              choices = c("AND" = "AND", "OR" = "OR", "NOT" = "NOT"),
              selected = "AND",
              width = "100%"
            )
          ),
          column(
            width = 2,
            selectInput(
              paste0("pmField_", newRowId),
              NULL,
              choices = c(
                "All Fields" = "all",
                "Title" = "title",
                "Abstract" = "abstract",
                "Title/Abstract" = "tiab",
                "Author" = "author",
                "MeSH Terms" = "mesh",
                "Affiliation" = "affiliation"
              ),
              selected = "tiab",
              width = "100%"
            )
          ),
          column(
            width = 8,
            textInput(
              paste0("pmQuery_", newRowId),
              NULL,
              value = "",
              placeholder = 'Example: "deep learning"',
              width = "100%"
            )
          ),
          column(
            width = 1,
            actionButton(
              paste0("pmRemoveRow_", newRowId),
              icon("minus-circle"),
              class = "btn-danger btn-sm",
              style = "margin-top: 0px;",
              title = "Remove row"
            )
          )
        )
      )
    )

    # Make remove button visible for first row if adding second row
    if (newRowId == 2) {
      runjs("$('#pm-remove-button-1').css('visibility', 'visible');")
    }

    # Add observer for the remove button of the new row
    local({
      rowId <- newRowId
      observeEvent(input[[paste0("pmRemoveRow_", rowId)]], {
        removeUI(selector = paste0("#pm-query-row-", rowId))

        # Check if only one row remains
        if (queryRows() - 1 == 1) {
          runjs("$('#pm-remove-button-1').css('visibility', 'hidden');")
        }
      })
    })
  })

  # Clear all query fields
  observeEvent(input$pmClearQuery, {
    # Reset all query inputs
    for (i in 1:queryRows()) {
      updateTextInput(session, paste0("pmQuery_", i), value = "")
      updateSelectInput(session, paste0("pmField_", i), selected = "tiab")
      if (i > 1) {
        updateSelectInput(session, paste0("pmOperator_", i), selected = "AND")
      }
    }

    # Reset filters
    updateNumericInput(session, "pmYearFrom", value = 2020)
    updateNumericInput(
      session,
      "pmYearTo",
      value = as.integer(format(Sys.Date(), "%Y"))
    )
    updateSelectizeInput(session, "pmDocType", selected = "")
    updateSelectizeInput(session, "pmLanguage", selected = "")
    updateNumericInput(session, "pmMaxRecords", value = 1000)
  })

  # Helper function to build PubMed query
  build_pubmed_query <- function(queryList) {
    if (length(queryList) == 0) {
      return("")
    }

    query_parts <- c()

    for (i in seq_along(queryList)) {
      item <- queryList[[i]]

      # Wrap query with field tag
      field_tag <- switch(
        item$field,
        "all" = "",
        "title" = "[Title]",
        "abstract" = "[Abstract]",
        "tiab" = "[Title/Abstract]",
        "author" = "[Author]",
        "mesh" = "[MeSH Terms]",
        "affiliation" = "[Affiliation]",
        ""
      )

      wrapped_query <- paste0("(", item$query, ")", field_tag)

      if (i == 1) {
        query_parts <- c(query_parts, wrapped_query)
      } else {
        operator <- item$operator
        query_parts <- c(query_parts, operator, wrapped_query)
      }
    }

    query <- paste(query_parts, collapse = " ")
    return(query)
  }

  # Reactive query count - updates automatically when query changes
  output$pmQueryCountInfo <- renderUI({
    # Make this reactive to query inputs
    lapply(1:queryRows(), function(i) {
      input[[paste0("pmQuery_", i)]]
      input[[paste0("pmField_", i)]]
      if (i > 1) input[[paste0("pmOperator_", i)]]
    })

    # Also react to filters
    input$pmYearFrom
    input$pmYearTo
    input$pmDocType
    input$pmLanguage

    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("pmQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[i]] <- list(
          query = queryText,
          field = input[[paste0("pmField_", i)]],
          operator = if (i > 1) input[[paste0("pmOperator_", i)]] else NULL
        )
      }
    }

    # If no query, show hint
    if (length(queryList) == 0) {
      return(
        div(
          style = "display: inline-block; padding: 5px 10px; color: #6c757d; font-size: 13px; font-style: italic;",
          icon("info-circle"),
          " Enter search terms to see record count"
        )
      )
    }

    # Build query and fetch count
    tryCatch(
      {
        # Build the PubMed query
        query <- build_pubmed_query(queryList)

        # Add date range
        if (showDateRange()) {
          if (!is.na(input$pmYearFrom) && !is.na(input$pmYearTo)) {
            date_filter <- sprintf(
              " AND (%s:%s[pdat])",
              input$pmYearFrom,
              input$pmYearTo
            )
            query <- paste0(query, date_filter)
          }
        }

        # Add document type filter
        if (showAdvanced()) {
          if (!is.null(input$pmDocType) && length(input$pmDocType) > 0) {
            # Rimuovi eventuali stringhe vuote
            doc_types <- input$pmDocType[input$pmDocType != ""]
            if (length(doc_types) > 0) {
              type_filters <- sapply(doc_types, function(t) {
                paste0("\"", t, "\"[Publication Type]")
              })
              type_filter <- paste0(
                " AND (",
                paste(type_filters, collapse = " OR "),
                ")"
              )
              query <- paste0(query, type_filter)
            }
          }
          # Add language filter
          if (!is.null(input$pmLanguage) && length(input$pmLanguage) > 0) {
            # Rimuovi eventuali stringhe vuote
            languages <- input$pmLanguage[input$pmLanguage != ""]
            if (length(languages) > 0) {
              lang_filters <- sapply(input$pmLanguage, function(l) {
                paste0(l, "[Language]")
              })
              lang_filter <- paste0(
                " AND (",
                paste(lang_filters, collapse = " OR "),
                ")"
              )
              query <- paste0(query, lang_filter)
            }
          }
        }

        # Get count from PubMed
        if (!require("pubmedR", quietly = TRUE)) {
          return(
            div(
              style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
              icon("exclamation-circle"),
              " pubmedR package not installed"
            )
          )
        }

        count_result <- pmQueryTotalCount(query = query, api_key = NULL)
        total_available <- count_result$total_count

        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #d4edda; color: #155724; border-radius: 4px; font-size: 13px; font-weight: 500;",
          icon("check-circle"),
          sprintf(" Found %s records", format(total_available, big.mark = ","))
        )
      },
      error = function(e) {
        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
          icon("exclamation-circle"),
          " Error: ",
          e$message
        )
      }
    )
  })

  # Download data from PubMed
  observeEvent(input$pmFetchData, {
    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("pmQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[i]] <- list(
          query = queryText,
          field = input[[paste0("pmField_", i)]],
          operator = if (i > 1) input[[paste0("pmOperator_", i)]] else NULL
        )
      }
    }

    # Validate that at least one query exists
    if (length(queryList) == 0) {
      showModal(modalDialog(
        title = "Error",
        "Please enter at least one search term.",
        easyClose = TRUE,
        footer = modalButton("OK")
      ))
      return()
    }

    # Build the PubMed query
    query <- build_pubmed_query(queryList)

    # Add date range
    if (showDateRange()) {
      if (!is.na(input$pmYearFrom) && !is.na(input$pmYearTo)) {
        date_filter <- sprintf(
          " AND (%s:%s[pdat])",
          input$pmYearFrom,
          input$pmYearTo
        )
        query <- paste0(query, date_filter)
      }
    }

    # Add document type filter
    if (showAdvanced()) {
      if (!is.null(input$pmDocType) && length(input$pmDocType) > 0) {
        # Rimuovi eventuali stringhe vuote
        doc_types <- input$pmDocType[input$pmDocType != ""]
        if (length(doc_types) > 0) {
          type_filters <- sapply(doc_types, function(t) {
            paste0("\"", t, "\"[Publication Type]")
          })
          type_filter <- paste0(
            " AND (",
            paste(type_filters, collapse = " OR "),
            ")"
          )
          query <- paste0(query, type_filter)
        }
      }
      # Add language filter
      if (!is.null(input$pmLanguage) && length(input$pmLanguage) > 0) {
        # Rimuovi eventuali stringhe vuote
        languages <- input$pmLanguage[input$pmLanguage != ""]
        if (length(languages) > 0) {
          lang_filters <- sapply(input$pmLanguage, function(l) {
            paste0(l, "[Language]")
          })
          lang_filter <- paste0(
            " AND (",
            paste(lang_filters, collapse = " OR "),
            ")"
          )
          query <- paste0(query, lang_filter)
        }
      }
    }

    max_records <- input$pmMaxRecords

    print("PubMed Query:")
    print(query)

    # Show progress modal with dynamic content
    showModal(modalDialog(
      title = "Downloading from PubMed",
      div(
        div(
          style = "text-align: center; margin: 20px 0;",
          icon("spinner", class = "fa-spin fa-3x")
        ),
        div(
          id = "pm-download-progress-text",
          style = "text-align: center; font-size: 14px;",
          "Initializing search..."
        )
      ),
      footer = NULL,
      easyClose = FALSE
    ))

    # Fetch data
    tryCatch(
      {
        # Update progress
        shinyjs::html(
          "pm-download-progress-text",
          paste("Searching PubMed with query:", query)
        )

        # Load pubmedR library
        if (!require("pubmedR", quietly = TRUE)) {
          stop(
            "pubmedR package is not installed. Please install it using: install.packages('pubmedR')"
          )
        }

        # Get total count first
        shinyjs::html(
          "pm-download-progress-text",
          "Counting available records..."
        )

        count_result <- pmQueryTotalCount(query = query, api_key = NULL)
        total_available <- count_result$total_count
        records_to_download <- min(total_available, max_records)

        # Update progress with count information
        shinyjs::html(
          "pm-download-progress-text",
          sprintf(
            "Found %s records. Downloading %s records...",
            format(total_available, big.mark = ","),
            format(records_to_download, big.mark = ",")
          )
        )

        # Fetch data from PubMed using pmApiRequest with limit parameter
        pm_data <- pmApiRequest(
          query = query,
          limit = records_to_download,
          api_key = NULL
        )

        # Update progress
        shinyjs::html(
          "pm-download-progress-text",
          sprintf(
            "Converting %s records to bibliometric format...",
            format(pm_data$records_downloaded, big.mark = ",")
          )
        )

        # Convert using bibliometrix convert2df function
        M <- convert2df(
          file = pm_data,
          dbsource = "pubmed",
          format = "api"
        )

        # Close progress modal
        removeModal()

        # Collapse query panel after successful download
        session$sendCustomMessage('pmCollapseQueryPanel', TRUE)

        # Show success message
        showModal(modalDialog(
          title = "Success",
          div(
            p(
              sprintf(
                "Successfully downloaded %s records from PubMed.",
                format(nrow(M), big.mark = ",")
              )
            ),
            if (total_available > max_records) {
              p(
                style = "color: #856404; background-color: #fff3cd; padding: 10px; border-radius: 5px;",
                sprintf(
                  "Note: %s total records found, but limited to %s records.",
                  format(total_available, big.mark = ","),
                  format(max_records, big.mark = ",")
                )
              )
            }
          ),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))

        # Initialize values
        values = initial(values)
        M <- M %>% mergeKeywords(force = FALSE)

        # Update values
        values$data_source <- "PubMed"
        values$M <- M
        values$Morig <- M
        values$SCdf <- wcTable(M)
        values$COdf <- countryTable(M)
        values$Histfield <- "NA"
        values$results <- list("NA")

        if (ncol(values$M) > 1) {
          values$rest_sidebar <- TRUE
        }
        if (ncol(values$M) > 1) {
          values$loadMenu <- "pubmed_api"
          showModal(missingModalAPI(session))
        }
      },
      error = function(e) {
        removeModal()
        showModal(modalDialog(
          title = "Error",
          paste("An error occurred:", e$message),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))
      }
    )
  })

  # Render data table
  output$pmDataTable <- renderUI({
    req(values$M)
    req(values$data_source == "PubMed")

    MData <- as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))

    # Create DOI links if available
    if ("DI" %in% names(MData)) {
      MData$DOI <- paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
      nome <- c("DOI", names(MData)[-length(names(MData))])
      MData <- MData[nome]
    }

    renderBibliobox(
      MData,
      nrow = 3,
      filename = "Table",
      pagelength = TRUE,
      left = NULL,
      right = NULL,
      numeric = NULL,
      dom = TRUE,
      size = '70%',
      filter = "top",
      columnShort = NULL,
      columnSmall = NULL,
      round = 2,
      title = "",
      button = FALSE,
      escape = FALSE,
      selection = FALSE,
      scrollX = TRUE
    )
  })

  # Check if data is available for conditional panel
  output$pmDataAvailable <- reactive({
    !is.null(values$M) && values$data_source == "PubMed"
  })
  outputOptions(output, "pmDataAvailable", suspendWhenHidden = FALSE)

  # Display progress information
  output$pmProgressInfo <- renderUI({
    if (!is.null(values$M) && values$data_source == "PubMed") {
      div(
        class = "alert alert-success",
        icon("check-circle"),
        strong(" Data loaded: "),
        sprintf(
          "%s documents from PubMed",
          format(nrow(values$M), big.mark = ",")
        )
      )
    }
  })

  # Download handler for Excel
  output$pmDownloadExcel <- downloadHandler(
    filename = function() {
      paste0("pubmed_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "PubMed")

      # Write data to Excel file
      openxlsx::write.xlsx(values$M, file = file)
    }
  )

  # Download handler for RData
  output$pmDownloadRData <- downloadHandler(
    filename = function() {
      paste0("pubmed_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".RData")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "PubMed")

      # Save data as RData file
      M <- values$M
      save(M, file = file)
    }
  )

  # Modal for missing data (mirrors the main missingModal so users can also
  # trigger completeMetadata() right after an API download).
  missingModalAPI <- function(session) {
    ns <- session$ns
    modalDialog(
      uiOutput("missingTitle"),
      uiOutput(ns("missingDataTable")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(
          label = "Advice",
          inputId = "missingMessage",
          icon = icon("exclamation-sign", lib = "glyphicon")
        ),
        actionButton(
          label = "Complete",
          inputId = "missingComplete",
          icon = icon("download", lib = "glyphicon")
        ),
        actionButton(
          label = "Undo",
          inputId = "missingCompleteUndo",
          icon = icon("repeat", lib = "glyphicon")
        ),
        actionButton(
          label = "Report",
          inputId = "missingReport",
          icon = icon("plus", lib = "glyphicon")
        ),
        actionButton(
          label = "Save",
          inputId = "missingDataSave",
          icon = icon("camera", lib = "glyphicon")
        ),
        modalButton(label = "Close")
      )
    )
  }
}

#' Deleting leading white spaces
#'
#' Deleting leading white spaces from a \code{character} object.
#'
#' \code{tableTag} is an internal routine of \code{bibliometrics} package.
#'
#' @param x is a \code{character} object.

#' @return an object of class \code{character}
#' @examples
#'
#' char <- c("  Alfred", "Mary", " John")
#' char
#' trim.leading(char)
#'
#' @export
trim.leading <- function(x) {
  sub("^\\s+", "", x) ## function to delete leading spaces in a string
}

#' Creating Bibliographic networks
#'
#' \code{biblioNetwork} creates different bibliographic networks from a bibliographic data frame.
#'
#' The function \code{\link{biblioNetwork}} can create a collection of bibliographic networks
#' following the approach proposed by Batagelj & Cerinsek (2013) and Aria & cuccurullo (2017).\cr\cr
#' Typical networks output of \code{biblioNetwork} are:\cr\cr
#' #### Collaboration Networks ############\cr
#' -- Authors collaboration (analysis = "collaboration", network = "authors")\cr
#' -- University collaboration (analysis = "collaboration", network = universities")\cr
#' -- Country collaboration (analysis = "collaboration", network = "countries")\cr\cr
#' #### Co-citation Networks ##############\cr
#' -- Authors co-citation (analysis = "co-citation", network = "authors")\cr
#' -- Reference co-citation (analysis = "co-citation", network = "references")\cr
#' -- Source co-citation (analysis = "co-citation", network = "sources")\cr\cr
#' #### Coupling Networks ################\cr
#' -- Manuscript coupling (analysis = "coupling", network = "references")\cr
#' -- Authors coupling (analysis = "coupling", network = "authors")\cr
#' -- Source coupling (analysis = "coupling", network = "sources")\cr
#' -- Country coupling (analysis = "coupling", network = "countries")\cr\cr
#' #### Co-occurrences Networks ################\cr
#' -- Authors co-occurrences (analysis = "co-occurrences", network = "authors")\cr
#' -- Source co-occurrences (analysis = "co-occurrences", network = "sources")\cr
#' -- Keyword co-occurrences (analysis = "co-occurrences", network = "keywords")\cr
#' -- Author-Keyword co-occurrences (analysis = "co-occurrences", network = "author_keywords")\cr
#' -- Title content co-occurrences (analysis = "co-occurrences", network = "titles")\cr
#' -- Abstract content co-occurrences (analysis = "co-occurrences", network = "abstracts")\cr\cr
#'
#' References:\cr
#' Batagelj, V., & Cerinsek, M. (2013). On bibliographic networks. Scientometrics, 96(3), 845-864.\cr
#' Aria, M., & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive science mapping analysis. Journal of Informetrics, 11(4), 959-975.\cr
#'
#' @param M is a bibliographic data frame obtained by the converting function
#'   \code{\link{convert2df}}. It is a data matrix with cases corresponding to
#'   manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param analysis is a character object. It indicates the type of analysis can be performed.
#'   \code{analysis} argument can be \code{"collaboration"}, \code{"coupling"}, \code{"co-occurrences"}  or \code{"co-citation"}.
#'   Default is \code{analysis = "coupling"}.
#' @param network is a character object. It indicates the network typology. The \code{network} argument can be
#' \code{"authors"}, \code{"references"}, \code{"sources"}, \code{"countries"},\code{"keywords"}, \code{"author_keywords"},\code{"all_keywords"}, \code{"titles"}, or \code{"abstracts"}.
#' Default is \code{network = "authors"}.
#' @param n is an integer. It indicates the number of items to select. If \code{N = NULL}, all items are selected.
#' @param sep is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.
#' @param short is a logical. If TRUE all items with frequency<2 are deleted to reduce the matrix size.
#' @param shortlabel is logical. IF TRUE, reference labels are stored in a short format. Default is \code{shortlabel=TRUE}.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @return It is a squared network matrix. It is an object of class \code{dgMatrix} of the package \code{Matrix}.
#' @examples
#' # EXAMPLE 1: Authors collaboration network
#'
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # NetMatrix <- biblioNetwork(scientometrics, analysis = "collaboration",
#' # network = "authors", sep = ";")
#'
#' # net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Collaboration",labelsize=0.5)
#'
#'
#' # EXAMPLE 2: Co-citation network
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' NetMatrix <- biblioNetwork(scientometrics,
#'   analysis = "co-citation",
#'   network = "references", sep = ";"
#' )
#'
#' net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Co-Citation", labelsize = 0.5)
#'
#' @seealso \code{\link{convert2df}} to import and convert a SCOPUS and Thomson
#'   Reuters' ISI Web of Knowledge export file in a data frame.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export

biblioNetwork <-
  function(M,
           analysis = "coupling",
           network = "authors",
           n = NULL,
           sep = ";",
           short = FALSE,
           shortlabel = TRUE,
           remove.terms = NULL,
           synonyms = NULL) {
    crossprod <- Matrix::crossprod
    NetMatrix <- NA
    if (analysis == "coupling") {
      switch(network,
        authors = {
          WA <- cocMatrix(M, Field = "AU", type = "sparse", n, sep, short = short)
          WCR <- cocMatrix(M, Field = "CR", type = "sparse", n, sep, short = short)
          CRA <- crossprod(WCR, WA)
          NetMatrix <- crossprod(CRA, CRA)
        },
        references = {
          WCR <- Matrix::t(cocMatrix(M, Field = "CR", type = "sparse", n, sep, short = short))
          NetMatrix <- crossprod(WCR, WCR)
        },
        sources = {
          WSO <- cocMatrix(M, Field = "SO", type = "sparse", n, sep, short = short)
          WCR <- cocMatrix(M, Field = "CR", type = "sparse", n, sep, short = short)
          CRSO <- crossprod(WCR, WSO)
          NetMatrix <- crossprod(CRSO, CRSO)
        },
        countries = {
          WCO <- cocMatrix(M, Field = "AU_CO", type = "sparse", n, sep, short = short)
          WCR <- cocMatrix(M, Field = "CR", type = "sparse", n, sep, short = short)
          CRCO <- crossprod(WCR, WCO)
          NetMatrix <- crossprod(CRCO, CRCO)
        }
      )
    }

    if (analysis == "co-occurrences") {
      switch(network,
        authors = {
          WA <- cocMatrix(M, Field = "AU", type = "sparse", n, sep, short = short)
        },
        keywords = {
          WA <- cocMatrix(M, Field = "ID", type = "sparse", n, sep, short = short, remove.terms = remove.terms, synonyms = synonyms)
        },
        author_keywords = {
          WA <- cocMatrix(M, Field = "DE", type = "sparse", n, sep, short = short, remove.terms = remove.terms, synonyms = synonyms)
        },
        all_keywords = {
          WA <- cocMatrix(M, Field = "KW_Merged", type = "sparse", n, sep, short = short, remove.terms = remove.terms, synonyms = synonyms)
        },
        titles = {
          WA <- cocMatrix(M, Field = "TI_TM", type = "sparse", n, sep, short = short, remove.terms = remove.terms, synonyms = synonyms)
        },
        abstracts = {
          WA <- cocMatrix(M, Field = "AB_TM", type = "sparse", n, sep, short = short, remove.terms = remove.terms, synonyms = synonyms)
        },
        sources = {
          WA <- cocMatrix(M, Field = "SO", type = "sparse", n, sep, short = short)
        }
      )
      NetMatrix <- crossprod(WA, WA)
    }



    if (analysis == "co-citation") {
      switch(network,
        authors = {
          WA <- cocMatrix(M, Field = "CR_AU", type = "sparse", n, sep, short = short)
        },
        references = {
          WA <- cocMatrix(M, Field = "CR", type = "sparse", n, sep, short = short)
        },
        sources = {
          WA <- cocMatrix(M, Field = "CR_SO", type = "sparse", n, sep, short = short)
        }
      )
      NetMatrix <- crossprod(WA, WA)
    }

    if (analysis == "collaboration") {
      switch(network,
        authors = {
          WA <- cocMatrix(M, Field = "AU", type = "sparse", n, sep, short = short)
        },
        universities = {
          WA <- cocMatrix(M, Field = "AU_UN", type = "sparse", n, sep, short = short)
        },
        countries = {
          WA <- cocMatrix(M, Field = "AU_CO", type = "sparse", n, sep, short = short)
        }
      )
      NetMatrix <- crossprod(WA, WA)
    }


    # delete empty vertices
    NetMatrix <- NetMatrix[nchar(colnames(NetMatrix)) != 0, nchar(colnames(NetMatrix)) != 0]

    # short label for scopus references
    if (network == "references" & M$DB[1] == "SCOPUS") {
      ind <- which(regexpr("[A-Za-z]", substr(colnames(NetMatrix), 1, 1)) == 1)
      NetMatrix <- NetMatrix[ind, ind]
    }
    if (network == "references" & isTRUE(shortlabel)) {
      LABEL <- labelShort(NetMatrix, db = tolower(M$DB[1]))
      LABEL <- removeDuplicatedlabels(LABEL)
      colnames(NetMatrix) <- rownames(NetMatrix) <- LABEL
    }
    # short label for scopus references
    # if (network == "references") {
    #   ind <- which(regexpr("[A-Za-z]", substr(colnames(NetMatrix), 1, 1)) == 1)
    #   NetMatrix <- NetMatrix[ind, ind]
    #   if (isTRUE(shortlabel)) {
    #     LABEL <- labelShort(NetMatrix, db = tolower(M$DB[1]))
    #     LABEL <- removeDuplicatedlabels(LABEL)
    #     colnames(NetMatrix) <- rownames(NetMatrix) <-  LABEL
    #   }
    #
    # }

    # if (analysis != "coupling") {
    #   attr(NetMatrix, "PY") <- attr(WA, "PY")
    # }
    return(NetMatrix)
  }

### shortlabel
labelShort <- function(NET, db = "isi") {
  LABEL <- colnames(NET)
  YEAR <- suppressWarnings(as.numeric(sub(".*(\\d{4}).*", "\\1", LABEL)))
  YEAR[is.na(YEAR)] <- ""
  switch(db,
    isi = {
      AU <- strsplit(LABEL, " ")
      AU <- unlist(lapply(AU, function(l) {
        paste(l[1], " ", l[2], sep = "")
      }))
      LABEL <- paste0(AU, " ", YEAR, sep = "")
    },
    scopus = {
      AU <- strsplit(LABEL, "\\. ")
      AU <- unlist(lapply(AU, function(l) {
        l[1]
      }))
      LABEL <- paste0(AU, ". ", YEAR, sep = "")
    }
  )

  return(LABEL)
}

removeDuplicatedlabels <- function(LABEL) {
  ## assign an unique name to each label
  tab <- sort(table(LABEL), decreasing = T)
  dup <- names(tab[tab > 1])
  for (i in 1:length(dup)) {
    ind <- which(LABEL %in% dup[i])
    if (length(ind) > 0) {
      LABEL[ind] <- paste0(LABEL[ind], "-", as.character(1:length(ind)), sep = "")
    }
  }
  return(LABEL)
}

# Test per biblioNetwork, cocMatrix, networkStat, networkPlot, normalizeSimilarity

test_that("biblioNetwork crea matrice co-citation per references", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "co-citation", network = "references", sep = ";")
  expect_true(inherits(NetMatrix, "Matrix") || inherits(NetMatrix, "matrix"))
  expect_equal(nrow(NetMatrix), ncol(NetMatrix))
  expect_true(nrow(NetMatrix) > 0)
})

test_that("biblioNetwork crea matrice collaboration per autori", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "collaboration", network = "authors", sep = ";")
  expect_true(nrow(NetMatrix) > 0)
  expect_equal(nrow(NetMatrix), ncol(NetMatrix))
})

test_that("biblioNetwork crea matrice co-occurrences per keywords", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "co-occurrences", network = "keywords", sep = ";")
  expect_true(nrow(NetMatrix) > 0)
  expect_equal(nrow(NetMatrix), ncol(NetMatrix))
})

test_that("biblioNetwork crea matrice coupling per references", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "coupling", network = "references", sep = ";")
  expect_true(nrow(NetMatrix) > 0)
  expect_equal(nrow(NetMatrix), ncol(NetMatrix))
})

test_that("cocMatrix crea matrice bipartita sparsa", {
  M <- load_wos_fixture()
  WA <- cocMatrix(M, Field = "AU", type = "sparse", sep = ";")
  expect_true(inherits(WA, "dgCMatrix") || inherits(WA, "Matrix") || inherits(WA, "sparseMatrix"))
  expect_equal(nrow(WA), nrow(M))
})

test_that("networkStat calcola statistiche di rete", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "co-citation", network = "references", sep = ";")
  ns <- networkStat(NetMatrix)
  expect_type(ns, "list")
  expect_true("network" %in% names(ns))
})

test_that("normalizeSimilarity calcola indici di similarita", {
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "co-occurrences", network = "keywords", sep = ";")
  S <- normalizeSimilarity(NetMatrix, type = "association")
  expect_true(inherits(S, "Matrix") || is.matrix(S))
})

test_that("networkPlot genera output senza errori", {
  skip_on_cran()
  M <- load_wos_fixture()
  NetMatrix <- biblioNetwork(M, analysis = "co-citation", network = "references", sep = ";")
  net <- expect_no_error(
    suppressWarnings(networkPlot(NetMatrix, n = 10, type = "auto", verbose = FALSE))
  )
  expect_true("graph" %in% names(net))
  expect_true(inherits(net$graph, "igraph"))
})

utils::globalVariables(c(
  "from", "to", "color", "from_name", "from_title", "from_keywords",
  "from_keywordsplus", "from_id", "from_size", "from_years",
  "to_name", "to_title", "to_keywords", "from2", "x", "y", "color.x",
  "to_keywordsplus", "to_id", "to_size", "to_years", "color.y",
  "Title", "DOI", "LCS", "GCS", "xend", "yend", "text", "color_v", "id"
))
#' Plotting historical co-citation network
#'
#' \code{histPlot} plots a historical co-citation network.
#'
#' The function \code{\link{histPlot}} can plot a historical co-citation network previously created by \code{\link{histNetwork}}.
#' @param histResults is an object of \code{class} "list" containing the following components:
#'
#' \tabular{lll}{
#' NetMatrix \tab  \tab the historical citation network matrix\cr
#' Degree \tab       \tab the min degree of the network\cr
#' histData \tab      \tab the set of n most cited references\cr
#' M \tab      \tab the bibliographic data frame}
#'
#' is a network matrix obtained by the function \code{\link{histNetwork}}.
#' @param n is integer. It defines the number of vertices to plot.
#' @param size is an integer. It defines the point size of the vertices. Default value is 5.
#' @param labelsize is an integer. It indicates the label size in the plot. Default is \code{labelsize=5}.
#' @param remove.isolates is logical. If TRUE isolates vertices are not plotted.
#' @param title_as_label is a logical. DEPRECATED
#' @param label is a character. It indicates which label type to use as node id in the historiograph. It can be \code{label=c("short", "title", "keywords", "keywordsplus")}.
#' Default is \code{label = "short"}.
#' @param verbose is logical. If TRUE, results and plots are printed on screen.
#' @return It is list containing: a network object of the class \code{igraph} and a plot object of the class \code{ggraph}.
#'
#' @examples
#' # EXAMPLE Citation network
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' histResults <- histNetwork(management, sep = ";")
#'
#' net <- histPlot(histResults, n = 20, labelsize = 5)
#' }
#'
#' @seealso \code{\link{histNetwork}} to compute a historical co-citation network.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export
histPlot <- function(histResults, n = 20, size = 5, labelsize = 5, remove.isolates = TRUE, title_as_label = FALSE, label = "short", verbose = TRUE) {
  params <- list(
    n = n,
    size = size,
    labelsize = labelsize,
    title_as_label = title_as_label,
    label = label
  )

  colorlist <- colorlist()
  # c(brewer.pal(9, 'Set1')[-6], brewer.pal(8, 'Set2')[-7], brewer.pal(12, 'Paired')[-11],brewer.pal(12, 'Set3')[-c(2,8,12)])
  ## legacy with old argument size
  if (isTRUE(size)) {
    size <- 5
  }

  # LCS=histResults$LCS
  LCS <- colSums(histResults$NetMatrix)
  NET <- histResults$NetMatrix
  ## selecting the first n vertices or all if smaller
  s <- sort(LCS, decreasing = TRUE)[min(n, length(LCS))]
  ind <- which(LCS >= s)
  # NET=NET[a,names(ind)]
  NET <- NET[names(ind), names(ind)]
  LCS <- LCS[ind]

  # Create igraph object
  bsk.network <- graph_from_adjacency_matrix(NET, mode = c("directed"), weighted = NULL)

  R <- strsplit(names(V(bsk.network)), ",")
  RR <- lapply(R, function(l) {
    l <- l[1:2]
    l <- paste(l[1], l[2], sep = ",")
  })

  # add label to nodes
  V(bsk.network)$title <- histResults$histData$Title[ind]
  V(bsk.network)$keywords <- histResults$histData$Author_Keywords[ind]
  V(bsk.network)$keywordsplus <- histResults$histData$KeywordsPlus[ind]

  switch(label,
    title = {
      title <- strsplit(stringi::stri_trans_totitle(V(bsk.network)$title), " ")
      V(bsk.network)$id <- unlist(lapply(title, function(l) {
        n <- floor(length(l) / 2)
        paste0(paste(l[1:n], collapse = " ", sep = ""), "\n", paste(l[(n + 1):length(l)], collapse = " ", sep = ""))
      }))
    },
    keywords = {
      kw <- strsplit(stringi::stri_trans_totitle(V(bsk.network)$keywords), ";")
      kw[is.na(kw)] <- "Not Available"
      V(bsk.network)$id <- unlist(lapply(kw, function(l) {
        if (length(l) > 1) {
          n <- floor(length(l) / 2)
          l <- trimws(l)
          paste0(paste(l[1:n], collapse = "; ", sep = ""), "\n", paste(l[(n + 1):length(l)], collapse = "; ", sep = ""))
        } else {
          l
        }
      }))
    },
    keywordsplus = {
      kw <- strsplit(stringi::stri_trans_totitle(V(bsk.network)$keywordsplus), ";")
      kw[is.na(kw)] <- "Not Available"
      V(bsk.network)$id <- unlist(lapply(kw, function(l) {
        if (length(l) > 1) {
          n <- floor(length(l) / 2)
          l <- trimws(l)
          paste0(paste(l[1:n], collapse = "; ", sep = ""), "\n", paste(l[(n + 1):length(l)], collapse = "; ", sep = ""))
        } else {
          l
        }
      }))
    },
    {
      V(bsk.network)$id <- tolower(unlist(RR))
    }
  )

  # Compute node degrees (#links) and use that to set node size:
  deg <- LCS
  V(bsk.network)$size <- size
  # rep(size,length(V(bsk.network)))}

  # Years=histResults$histData$Year[ind]
  Years <- as.numeric(unlist(stringi::stri_extract_all_regex(unlist(RR), "[[:digit:]]{4}$")))
  V(bsk.network)$years <- Years

  # Remove loops
  bsk.network <- igraph::simplify(bsk.network, remove.multiple = T, remove.loops = T)


  # define network layout

  E(bsk.network)$color <- "slategray1"

  if (isTRUE(remove.isolates)) bsk.network <- delete.isolates(bsk.network)

  if (length(V(bsk.network)) == 0) {
    if (isTRUE(verbose)) cat("\nNo direct citations found among the documents in the collection.\n")
    return(list(net = bsk.network, g = NULL, graph.data = NULL, layout = NULL, axis = NULL, params = params))
  }

  dg <- decompose.graph(bsk.network)

  layout_m <- as.data.frame(layout.fruchterman.reingold(bsk.network))
  names(layout_m) <- c("x", "y")
  layout_m$name <- V(bsk.network)$name
  layout_m$years <- V(bsk.network)$years
  layout_m$cluster <- 0
  rr <- 0
  for (k in 1:length(dg)) {
    bsk <- dg[[k]]

    a <- ifelse(layout_m$name %in% V(bsk)$name, k, 0)
    layout_m$cluster <- layout_m$cluster + a
    Min <- min(layout_m$y[layout_m$cluster == k]) - 1
    layout_m$y[layout_m$cluster == k] <- layout_m$y[layout_m$cluster == k] + (rr - Min)
    rr <- max(layout_m$y[layout_m$cluster == k])
  }
  # bsk <- bsk.network
  wp <- membership(cluster_infomap(bsk.network, modularity = FALSE))
  layout_m$color <- colorlist[wp]
  layout_m$x <- layout_m$years
  layout_m$y <- (diff(range(layout_m$x)) / diff(range(layout_m$y))) * layout_m$y

  df_net <- igraph::as_long_data_frame(bsk.network)
  df_net$color <- "slategray1"

  ID <- setdiff(df_net$to, df_net$from)

  df_from <- df_net %>%
    select(from, to, color, from_name, from_title, from_keywords, from_keywordsplus, from_id, from_size, from_years)

  df_to <- df_net %>%
    dplyr::filter(to %in% ID) %>%
    mutate(from2 = to) %>%
    select(from2, to, color, to_name, to_title, to_keywords, to_keywordsplus, to_id, to_size, to_years)

  df_to <- df_to[!duplicated(df_to$to), ]

  label <- c("from", "to", "color", "name", "title", "keywords", "keywordsplus", "id", "size", "years")

  names(df_from) <- label
  names(df_to) <- label

  df_net <- rbind(df_from, df_to)

  layout_norm <- layout_m %>%
    mutate(
      x = (x - min(x)) / (max(x) - min(x)),
      y = (y - min(y)) / (max(y) - min(y))
    )
  df_net <- left_join(df_net, layout_norm[c("name", "color", "x", "y")], by = c("name" = "name")) %>%
    rename(
      color = color.x,
      color_v = color.y
    )

  df_coord <- layout_norm %>%
    mutate(to = row_number()) %>%
    select(to, x, y) %>%
    rename(
      xend = x,
      yend = y
    )

  df_net <- df_net %>%
    left_join(df_coord, by = "to")

  ylength <- diff(range(df_net$years)) + 1
  Ylabel <- (as.character(seq(min(df_net$years), max(df_net$years), length.out = ylength)))
  Breaks <- (seq(0, 1, length.out = ylength))

  df_net <- df_net %>%
    left_join(histResults$histData, by = c("name" = "Paper")) # %>%

  #  Title <- strsplit(df_net$title, "(?<=.{40})", perl = TRUE)
  Title <- gsub("(.{40})", "\\1\n", df_net$title)
  df_net$Title <- unlist(lapply(Title, function(x) {
    paste(x, "\n", collapse = "", sep = "")
  }))

  df_net <- df_net %>%
    mutate(text = paste(tolower(Title), "doi: ",
      DOI, "\nLCS: ",
      LCS, "    GCS: ",
      GCS,
      sep = ""
    ))

  g <- ggplot(df_net, aes(x = x, y = y, xend = xend, yend = yend, text = text)) +
    geom_network_edges(color = "grey", size = 0.4, alpha = 0.4) +
    geom_network_nodes(aes(color = color_v), size = size, alpha = 0.5) +
    geom_text(aes(label = id, color = color_v),
      size = labelsize,
      nudge_x = 0,
      nudge_y = 0.02,
      check_overlap = FALSE, alpha = 0.7
    ) +
    scale_x_continuous(labels = Ylabel, breaks = Breaks) +
    guides(size = "none", color = "none") +
    theme_minimal() +
    theme(
      legend.position = "none", panel.background = element_rect(fill = "white", color = "white"),
      axis.line.y = element_blank(), axis.text.y = element_blank(), axis.ticks.y = element_blank(),
      axis.title.y = element_blank(), axis.title.x = element_blank(),
      panel.grid.minor.y = element_blank(), panel.grid.major.y = element_blank(),
      panel.grid.major.x = element_line(adjustcolor(col = "grey", alpha.f = 0.2), linetype = 2, linewidth = 0.5),
      panel.grid.minor.x = element_blank(),
      axis.text.x = element_text(face = "bold", angle = 90, size = labelsize + 4)
    ) +
    labs(title = "Historical Direct Citation Network")

  ### logo coordinates
  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  a <- ggplot_build(g)$data

  ymin <- unlist(lapply(a, function(l) {
    if ("y" %in% names(l)) {
      min(l["y"])
    }
  })) %>% min(na.rm = TRUE)

  ymax <- unlist(lapply(a, function(l) {
    if ("y" %in% names(l)) {
      max(l["y"])
    }
  })) %>% max(na.rm = TRUE)

  xmin <- unlist(lapply(a, function(l) {
    if ("x" %in% names(l)) {
      min(l["x"])
    }
  })) %>% min(na.rm = TRUE)

  xmax <- unlist(lapply(a, function(l) {
    if ("x" %in% names(l)) {
      max(l["x"])
    }
  })) %>% max(na.rm = TRUE)

  x <- c(xmax - 0.02 - diff(c(xmin, xmax)) * 0.125, xmax - 0.02)
  y <- c(ymin, ymin + diff(c(ymin, ymax)) * 0.125) + 0.02

  g <- g +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])

  label <- data.frame(Label = names(V(bsk.network)))
  Data <- histResults$histData

  Data <- left_join(label, Data, by = c("Label" = "Paper"))

  if (isTRUE(verbose)) {
    plot(g)

    cat("\n Legend\n\n")

    print(Data[, -2])
  }

  results <- list(net = bsk.network, g = g, graph.data = Data, layout = layout_m, axis = data.frame(label = Ylabel, values = Breaks), params = params)
  return(results)
}


delete.isolates <- function(graph, mode = "all") {
  isolates <- which(degree(graph, mode = mode) == 0) - 1
  delete.vertices(graph, names(isolates))
}

#' Summarizing network analysis results
#'
#' \code{summary} method for class '\code{bibliometrix_netstat}'
#' @param object is the object for which a summary is desired.
#' @param ... can accept two arguments:\cr
#' \code{k} integer, used for table formatting (number of rows). Default value is 10.\cr
#' @return The function \code{summary} computes and returns on display several statistics both at network and vertex level.
#'
#'
#'
#' @examples
#'
#' # to run the example, please remove # from the beginning of the following lines
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # NetMatrix <- biblioNetwork(scientometrics, analysis = "collaboration",
#' #                   network = "authors", sep = ";")
#' # netstat <- networkStat(NetMatrix, stat = "all", type = "degree")
#' # summary(netstat)
#'
#' @method summary bibliometrix_netstat
#' @export

summary.bibliometrix_netstat <- function(object, ...) {
  if (!inherits(object, "bibliometrix_netstat")) {
    cat('\n argument "object" have to be an object of class "netstat"\n')
    return(NA)
  }

  arguments <- list(...)
  if (sum(names(arguments) == "k") == 0) {
    k <- 10
  } else {
    k <- arguments$k
  }

  # Main Statistics about network
  MainStatNet <- ("\n\nMain statistics about the network\n\n")
  MainStatNet[2] <- paste("Size                                 ", object$network$networkSize, "\n")
  MainStatNet[3] <- paste("Density                              ", round(object$network$networkDensity, 3), "\n")
  MainStatNet[4] <- paste("Transitivity                         ", round(object$network$networkTransitivity, 3), "\n")
  MainStatNet[5] <- paste("Diameter                             ", round(object$network$networkDiameter, 3), "\n")
  switch(object$type,
    degree = {
      MainStatNet[6] <- paste("Degree Centralization                ", round(object$network$networkCentrDegree, 3), "\n")
    },
    pagerank = {
      MainStatNet[6] <- paste("Degree Centralization                ", round(object$network$networkCentrDegree, 3), "\n")
    },
    hub = {
      MainStatNet[6] <- paste("Degree Centralization                ", round(object$network$networkCentrDegree, 3), "\n")
    },
    authority = {
      MainStatNet[6] <- paste("Degree Centralization                ", round(object$network$networkCentrDegree, 3), "\n")
    },
    closeness = {
      MainStatNet[6] <- paste("Closeness Centralization             ", round(object$network$networkCentrCloseness, 3), "\n")
    },
    betweenness = {
      MainStatNet[6] <- paste("Betweenness Centralization           ", round(object$network$networkCentrbetweenness, 3), "\n")
    },
    eigenvector = {
      MainStatNet[6] <- paste("Eigenvector Centralization           ", round(object$network$networkCentrEigen, 3), "\n")
    }
  )

  MainStatNet[7] <- paste("Average path length                  ", round(object$network$NetworkAverPathLeng, 3), "\n")
  MainStatNet[8] <- paste("\n")
  cat(MainStatNet)
  cat("\n\n\n")

  if (object$stat == "all") {
    switch(object$type,
      degree = {
        # Main measures of centrality and prestige of vertices
        cat("\n\nMain measures of centrality and prestige of vertices\n\n")
        # Centrality Degree
        cat("\nDegree Centrality: Top vertices\n\n")
        CD <- object$vertex[, 1:2]

        A <- CD[order(-CD$vertexCentrDegree), ]
        names(A) <- c("Vertex ID             ", "Degree Centrality")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      closeness = {
        # Centrality Closeness
        cat("\nCloseness Centrality: Top vertices\n\n")
        CD <- object$vertex[, c(1, 3)]

        A <- CD[order(-CD$vertexCentrCloseness), ]
        names(A) <- c("Vertex ID             ", "Closeness Centrality")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      eigenvector = {
        # Centrality Eigenvectors
        cat("\nEigenvector Centrality: Top vertices\n\n")
        CD <- object$vertex[, c(1, 4)]

        A <- CD[order(-CD$vertexCentrEigen), ]
        names(A) <- c("Vertex ID             ", "Eigenvector Centrality")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      betweenness = {
        # Centrality betweeness
        cat("\nBetweenness Centrality: Top vertices\n\n")
        CD <- object$vertex[, c(1, 5)]

        A <- CD[order(-CD$vertexCentrBetweenness), ]
        names(A) <- c("Vertex ID             ", "Betweenness Centrality")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      pagerank = {
        # pagerank
        cat("\nPageRank Score: Top vertices\n\n")
        CD <- object$vertex[, c(1, 6)]

        A <- CD[order(-CD$vertexPageRank), ]
        names(A) <- c("Vertex ID             ", "Pagerank Score")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      hub = {
        # hub
        cat("\nHub Score: Top vertices\n\n")
        CD <- object$vertex[, c(1, 7)]

        A <- CD[order(-CD$vertexHub), ]
        names(A) <- c("Vertex ID             ", "Hub Score")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      },
      authority = {
        # Authority
        cat("\nAuthority Score: Top vertices\n\n")
        CD <- object$vertex[, c(1, 8)]

        A <- CD[order(-CD$vertexAuthority), ]
        names(A) <- c("Vertex ID             ", "Authority Score")
        A <- format(A[1:k, ], justify = "left", digits = 3)
        row.names(A) <- 1:k
        print(A, row.names = TRUE)
        cat("\n")
      }
    )
  }

  invisible(TRUE)
}

#' List of English stopwords.
#'
#' A character vector containing a complete list of English stopwords\cr
#' Data are used by  \code{\link{biblioAnalysis}} function
#' to extract Country Field of Cited References and Authors.
#'
#' @format A character vector with 665 rows.\cr
#'
#' @name stopwords
NULL

#' Plotting dendrogram resulting from Conceptual Structure Analysis
#'
#' \code{plot} method for class '\code{bibliodendrogram}'
#' @param x is the object for which plots are desired.
#' @param ... is a generic param for plot functions.
#' @return The function \code{plot} draws a dendrogram.
#'
#'
#' @method plot bibliodendrogram
#' @export


plot.bibliodendrogram <- function(x, ...) {
  plot(x$dend)
  abline(h = x$line, lty = 2)
}

#' Custom Theme variables for Biblioshiny.
#'
#' List containing a set of custom theme variables for Biblioshiny.
#'
#' @format A list with 3 elements:
#'  \describe{
#'     \item{name}{object name}
#'     \item{attribs}{attributes}
#'     \item{children}{CSS style}
#'     }
#'
#' @name customTheme
NULL

utils::globalVariables(c("Paper", "Tag", "content", "cont"))

bib2df <- function(D, dbsource = "isi") {
  # D <- D[nchar(D)>1]

  # remove empty rows and strange characters
  res <- try(D <- D[nchar(D) > 1], silent = T)
  if (inherits(res, "try-error")) {
    D <- removeStrangeChar(D)
    # next
  } else {
    D <- res
    rm(res)
  }

  D <- gsub("\\{\\[\\}", "[", D)
  D <- gsub("\\{\\]\\}", "]", D)
  Papers <- which(substr(D, 1, 1) == "@") # # first row of each document
  if (Papers[1] > 1) {
    D <- D[-(1:(Papers[1] - 1))]
    Papers <- Papers - (Papers[1] - 1)
  }

  if (dbsource == "isi") D <- gsub(" = \\{", "={", D)

  D <- gsub("\\\t", "", gsub(" = \\{", "=\\{", D)) # to work also with new scopus bib format

  D[Papers] <- paste("Paper={", D[Papers], sep = "")

  ind <- regexpr("=\\{", D) # sep among tags and contents
  ind[Papers] <- 6

  nP <- length(Papers) # number of documents

  for (i in 1:length(D)) {
    if (ind[i] == -1) {
      D[i] <- trimES(paste(substr(D[i - 1], 1, ind[i - 1] + 1), D[i], collapse = " "))
      ind[i] <- ind[i - 1]
    }
  }

  rowPapers <- diff(c(Papers, length(D) + 1))

  numPapers <- rep(1:nP, rowPapers)

  DATA <- data.frame(Tag = substr(D, 1, ind + 1), content = substr(D, ind + 2, nchar(D)), Paper = numPapers)
  DATA$content <- gsub("\\}|\\},|\\{", "", DATA$content)
  # DATA$content <- gsub("\\{","",DATA$content)
  df <- DATA %>%
    group_by(Paper, Tag) %>%
    summarise(cont = paste(content, collapse = "---", sep = "")) %>%
    arrange(Tag, Paper) %>%
    pivot_wider(names_from = Tag, values_from = cont) %>%
    ungroup()

  df <- as.data.frame(df)

  rm(DATA)
  bibtag <- NULL
  data("bibtag", envir = environment())
  bibtag <- as.data.frame(bibtag)

  Tag <- tolower(names(df))
  switch(dbsource,
    scopus = {
      bibtag <- bibtag[(bibtag[, "SCOPUS"] %in% Tag), ]
      for (i in 1:nrow(bibtag)) {
        Tag[Tag == bibtag$SCOPUS[i]] <- bibtag$TAG[i]
      }
    },
    isi = {
      bibtag <- bibtag[(bibtag[, "ISI"] %in% Tag), ]
      for (i in 1:nrow(bibtag)) {
        Tag[Tag == bibtag$ISI[i]] <- bibtag$TAG[i]
      }
    },
    generic = {
      bibtag <- bibtag[(bibtag[, "GENERIC"] %in% Tag), ]
      for (i in 1:nrow(bibtag)) {
        Tag[Tag == bibtag$GENERIC[i]] <- bibtag$TAG[i]
      }
    }
  )



  names(df) <- gsub("=\\{", "", Tag)

  ### replace "---" with ";"
  tagsComma <- c("AU", "DE", "ID", "C1", "CR")
  nolab <- setdiff(tagsComma, names(df))
  if (length(nolab) > 0) {
    cat("\nWarning:\nIn your file, some mandatory metadata are missing. Bibliometrix functions may not work properly!\n
Please, take a look at the vignettes:
- 'Data Importing and Converting' (https://www.bibliometrix.org/vignettes/Data-Importing-and-Converting.html)
- 'A brief introduction to bibliometrix' (https://www.bibliometrix.org/vignettes/Introduction_to_bibliometrix.html)\n\n")
    cat("\nMissing fields: ", nolab, "\n")
  }

  tagsComma <- tagsComma[(!(tagsComma %in% nolab))]
  df1 <- data.frame(lapply(df[tagsComma], function(x) {
    gsub("---", ";", x)
  }))

  df1$AU <- gsub(" and;| and ", ";", df1$AU)

  ### replace "---" with " "
  otherTags <- setdiff(names(df), tagsComma)
  df2 <- data.frame(lapply(df[otherTags], function(x) {
    trimES(gsub("---", " ", x))
  }), stringsAsFactors = FALSE)
  df <- cbind(df1, df2)
  rm(df1, df2)

  # Funding info
  ind <- which(regexpr("funding_text", names(df)) > -1)
  if (!("FX" %in% Tag) & length(ind) > 0) {
    df$FX <- apply(
      (as.data.frame(df[, ind], stringsAsFactors = FALSE)), 1,
      function(x) paste(x, collapse = " ")
    )
    df <- df[, -ind]
  }

  df <- postprocessing(df, dbsource)

  df <- df[!names(df) %in% c("Paper", "paper")]

  return(df)
}

### DATA FRAME postprocessing
postprocessing <- function(DATA, dbsource) {
  # Authors' names cleaning (surname and initials)
  # remove ; and 2 or more spaces
  DATA$AU <- gsub("\\s+", " ", DATA$AU)
  DATA$AF <- gsub("\\.|,", "", DATA$AU)

  listAU <- strsplit(DATA$AU, ";")

  AU <- lapply(listAU, function(l) {
    lastname <- trim(gsub(",.*", "", l))
    firstname <- strsplit(trim(gsub(".*,", "", l)), " ")
    firstname <- gsub("[^:A-Z:]", "", firstname)
    AU <- paste(lastname, unlist(firstname), sep = " ", collapse = ";")
    return(AU)
  })


  DATA$AU <- unlist(AU)

  # TC post-processing
  if ("TC" %in% names(DATA)) {
    DATA$TC <- as.numeric(sub("\\D*(\\d+).*", "\\1", DATA$TC))
  }
  # CR post-processing
  if ("CR" %in% names(DATA)) {
    # reomve dots after DOIs
    DATA$CR <- gsub("\\.;", ";", DATA$CR)
    DATA$CR <- substr(DATA$CR, 1, (nchar(DATA$CR) - 1))
  }
  # Year
  if ("PY" %in% names(DATA)) {
    DATA$PY <- as.numeric(sub("\\D*(\\d+).*", "\\1", DATA$PY))
  }

  if ("UT" %in% names(DATA)) {
    DATA$UT <- gsub(":", "", DATA$UT, fixed = TRUE)
  }

  if (!("RP" %in% names(DATA)) & ("C1" %in% names(DATA))) {
    DATA$RP <- unlist(lapply(strsplit(DATA$C1, "\\."), function(l) l[1]))
  }

  # keywords post-processing (missing ";" in some rows)
  if ("ID" %in% names(DATA)) {
    DATA$ID <- gsub("   |,", ";", DATA$ID)
    # DATA$ID <- gsub(",",";",DATA$ID)
  }

  if ("DE" %in% names(DATA)) {
    DATA$DE <- gsub("   |,", ";", DATA$DE)
    # DATA$DE <- gsub(",",";",DATA$DE)
  }
  # row.names(DATA)=DATA$UT

  ### merge Sources and Proceedings
  if (("SO" %in% names(DATA)) & ("BO" %in% names(DATA))) {
    ind <- which(is.na(DATA$SO))
    DATA$SO[ind] <- DATA$BO[ind]
  }

  if ("PN" %in% names(DATA)) {
    DATA$PN <- as.numeric(gsub("[^0-9]", "", DATA$PN))
  }

  if (dbsource != "generic") {
    DATA$DB <- dbsource
  } else {
    DATA$DB <- "SCOPUS"
  }


  # Toupper
  DI <- DATA$DI
  URL <- DATA$url
  AB <- DATA$AB
  TI <- DATA$TI
  DE <- DATA$DE
  DATA <- data.frame(lapply(DATA, toupper), stringsAsFactors = FALSE)
  if ("JI" %in% names(DATA)) {
    DATA$J9 <- gsub("\\.", "", DATA$JI)
  } else {
    DATA$J9 <- DATA$JI <- sapply(DATA$SO, AbbrevTitle, USE.NAMES = FALSE)
  }
  DATA$DI <- DI
  DATA$url <- URL
  DATA$AB_raw <- AB
  DATA$TI_raw <- TI
  DATA$DE_raw <- DE
  return(DATA)
}

# helper-load-fixtures.R
# Caricato automaticamente da testthat prima dell'esecuzione dei test.
# Fornisce funzioni helper per caricare i dataset fixture pre-convertiti.

fixture_path <- function(filename) {
  testthat::test_path("fixtures", filename)
}

# Carica e converte i fixture nei diversi formati.
# Ogni funzione restituisce un data.frame di classe "bibliometrixDB".

load_wos_fixture <- function() {
  suppressWarnings(suppressMessages(
    convert2df(fixture_path("wos_sample.txt"), dbsource = "wos", format = "plaintext")
  ))
}

load_scopus_fixture <- function() {
  suppressWarnings(suppressMessages(
    convert2df(fixture_path("scopus_sample.csv"), dbsource = "scopus", format = "csv")
  ))
}

load_openalex_fixture <- function() {
  suppressWarnings(suppressMessages(
    convert2df(fixture_path("openalex_sample.csv"), dbsource = "openalex", format = "csv")
  ))
}

load_lens_fixture <- function() {
  suppressWarnings(suppressMessages(
    convert2df(fixture_path("lens_sample.csv"), dbsource = "lens", format = "csv")
  ))
}

# Test per convert2df: import e conversione da diversi database/formati

test_that("convert2df importa WoS plaintext correttamente", {
  M <- load_wos_fixture()
  expect_s3_class(M, "bibliometrixDB")
  expect_s3_class(M, "data.frame")
  expect_equal(nrow(M), 8)
  expect_true(all(c("AU", "TI", "SO", "PY", "TC", "CR", "DB", "SR") %in% names(M)))
  expect_equal(unique(M$DB), "ISI")
  expect_true(is.numeric(M$PY))
  expect_true(is.numeric(M$TC))
})

test_that("convert2df importa Scopus CSV correttamente", {
  M <- load_scopus_fixture()
  expect_s3_class(M, "bibliometrixDB")
  expect_s3_class(M, "data.frame")
  expect_equal(nrow(M), 8)
  expect_true(all(c("AU", "TI", "SO", "PY", "TC", "DB", "SR") %in% names(M)))
  expect_equal(unique(M$DB), "SCOPUS")
  expect_true(is.numeric(M$PY))
  expect_true(is.numeric(M$TC))
})

test_that("convert2df importa OpenAlex CSV correttamente", {
  M <- load_openalex_fixture()
  expect_s3_class(M, "bibliometrixDB")
  expect_s3_class(M, "data.frame")
  expect_equal(nrow(M), 8)
  expect_true(all(c("AU", "TI", "SO", "PY", "TC", "DB", "SR") %in% names(M)))
  expect_equal(unique(M$DB), "OPENALEX")
  expect_true(is.numeric(M$PY))
  expect_true(is.numeric(M$TC))
})

test_that("convert2df importa Lens CSV correttamente", {
  M <- load_lens_fixture()
  expect_s3_class(M, "bibliometrixDB")
  expect_s3_class(M, "data.frame")
  expect_equal(nrow(M), 8)
  expect_true(all(c("AU", "TI", "SO", "PY", "TC", "DB", "SR") %in% names(M)))
  expect_equal(unique(M$DB), "LENS")
  expect_true(is.numeric(M$PY))
  expect_true(is.numeric(M$TC))
})

test_that("convert2df genera SR univoci senza duplicati", {
  M <- load_wos_fixture()
  expect_false(any(duplicated(M$SR)))
})

test_that("convert2df crea campo KW_Merged", {
  M <- load_wos_fixture()
  expect_true("KW_Merged" %in% names(M))
})

test_that("convert2df crea AU_UN per sorgenti con affiliazioni", {
  M <- load_wos_fixture()
  expect_true("AU_UN" %in% names(M))
})

test_that("convert2df segnala dbsource non valido", {
  expect_error(
    suppressWarnings(suppressMessages(
      convert2df("fake.txt", dbsource = "invalid_db", format = "plaintext")
    ))
  )
})

# Test per biblioAnalysis, summary e plot

test_that("biblioAnalysis restituisce oggetto bibliometrix", {
  M <- load_wos_fixture()
  res <- biblioAnalysis(M)
  expect_s3_class(res, "bibliometrix")
  expect_true(res$Articles > 0)
  expect_true(res$nAuthors > 0)
  expect_true(is.numeric(res$Years))
})

test_that("biblioAnalysis contiene tutti i componenti attesi", {
  M <- load_wos_fixture()
  res <- biblioAnalysis(M)
  expected <- c("Articles", "Authors", "AuthorsFrac", "FirstAuthors",
                "nAUperPaper", "Appearances", "nAuthors", "MostCitedPapers",
                "Years", "Sources", "DE", "ID")
  expect_true(all(expected %in% names(res)))
})

test_that("biblioAnalysis funziona con dati Scopus", {
  M <- load_scopus_fixture()
  res <- biblioAnalysis(M)
  expect_s3_class(res, "bibliometrix")
  expect_equal(res$Articles, nrow(M))
})

test_that("biblioAnalysis funziona con dati OpenAlex", {
  M <- load_openalex_fixture()
  res <- biblioAnalysis(M)
  expect_s3_class(res, "bibliometrix")
  expect_equal(res$Articles, nrow(M))
})

test_that("summary.bibliometrix funziona senza errori", {
  M <- load_wos_fixture()
  res <- biblioAnalysis(M)
  expect_no_error(
    capture.output(summary(res, k = 5, pause = FALSE, verbose = FALSE))
  )
})

test_that("plot.bibliometrix funziona senza errori", {
  M <- load_wos_fixture()
  res <- biblioAnalysis(M)
  expect_no_error(plot(res, k = 5, pause = FALSE))
})

# Test per fieldByYear, authorProdOverTime, threeFieldsPlot, histNetwork, histPlot

test_that("fieldByYear genera output con grafico", {
  skip_on_cran()
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  res <- expect_no_error(
    suppressWarnings(suppressMessages(
      fieldByYear(scientometrics, field = "ID", timespan = NULL,
                  min.freq = 5, n.items = 5, graph = TRUE)
    ))
  )
  expect_type(res, "list")
  expect_true("graph" %in% names(res))
})

test_that("authorProdOverTime produce risultati corretti", {
  skip_on_cran()
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  res <- expect_no_error(
    suppressWarnings(suppressMessages(
      authorProdOverTime(scientometrics, k = 5, graph = FALSE)
    ))
  )
  expect_type(res, "list")
  expect_true(all(c("dfAU", "dfPapersAU", "graph") %in% names(res)))
  expect_true(is.data.frame(res$dfAU))
})

test_that("threeFieldsPlot genera sankey plot senza errori", {
  skip_on_cran()
  M <- load_wos_fixture()
  expect_no_error(
    suppressWarnings(suppressMessages(
      threeFieldsPlot(M, fields = c("AU", "DE", "SO"), n = c(5, 5, 5))
    ))
  )
})

test_that("histNetwork costruisce rete storica", {
  skip_on_cran()
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  histResults <- expect_no_error(
    suppressWarnings(suppressMessages(
      histNetwork(scientometrics, sep = ";", verbose = FALSE)
    ))
  )
  expect_type(histResults, "list")
})

# Test per citations, localCitations, Hindex

test_that("citations calcola distribuzione citazioni per articolo", {
  M <- load_wos_fixture()
  CR <- citations(M, field = "article", sep = ";")
  expect_type(CR, "list")
  expect_true("Cited" %in% names(CR))
  expect_true(length(CR$Cited) > 0)
})

test_that("citations calcola distribuzione per autore", {
  M <- load_wos_fixture()
  CR <- citations(M, field = "author", sep = ";")
  expect_type(CR, "list")
  expect_true("Cited" %in% names(CR))
  expect_true(length(CR$Cited) > 0)
})

test_that("citations funziona con dati Scopus", {
  M <- load_scopus_fixture()
  CR <- citations(M, field = "article", sep = ";")
  expect_type(CR, "list")
  expect_true(length(CR$Cited) > 0)
})

test_that("localCitations calcola citazioni locali", {
  M <- load_wos_fixture()
  LC <- expect_no_error(
    suppressWarnings(suppressMessages(localCitations(M, fast.search = TRUE, verbose = FALSE)))
  )
  expect_type(LC, "list")
  expect_true(all(c("Authors", "Papers", "M") %in% names(LC)))
  expect_true(is.data.frame(LC$Authors))
  expect_true(is.data.frame(LC$Papers))
})

test_that("Hindex calcola indici per autore", {
  M <- load_wos_fixture()
  first_au <- trimws(strsplit(M$AU[1], ";")[[1]][1])
  H <- Hindex(M, field = "author", elements = first_au, sep = ";")
  expect_type(H, "list")
  expect_true("H" %in% names(H))
  expect_true("CitationList" %in% names(H))
  expect_true(is.data.frame(H$H))
  expect_true(all(c("h_index", "g_index", "m_index") %in% names(H$H)))
})

test_that("Hindex calcola indici per source", {
  M <- load_wos_fixture()
  first_so <- M$SO[1]
  H <- Hindex(M, field = "source", elements = first_so, sep = ";")
  expect_true(is.data.frame(H$H))
})

# Test per duplicatedMatching e mergeDbSources

test_that("duplicatedMatching rimuove duplicati esatti", {
  M <- load_wos_fixture()
  M_dup <- rbind(M, M[1:3, ])
  class(M_dup) <- class(M)
  M_clean <- suppressWarnings(suppressMessages(
    duplicatedMatching(M_dup, Field = "TI", tol = 0.95)
  ))
  expect_true(nrow(M_clean) <= nrow(M_dup))
  expect_true(nrow(M_clean) >= nrow(M))
})

test_that("mergeDbSources unisce dati da WoS e Scopus", {
  M_wos <- load_wos_fixture()
  M_scopus <- load_scopus_fixture()
  M_merged <- expect_no_error(
    suppressWarnings(suppressMessages(
      mergeDbSources(M_wos, M_scopus, remove.duplicated = TRUE)
    ))
  )
  expect_s3_class(M_merged, "data.frame")
  expect_true(nrow(M_merged) > 0)
  expect_true("SR" %in% names(M_merged))
  expect_true(all(c("AU", "TI", "SO", "PY") %in% names(M_merged)))
})

test_that("mergeDbSources unisce dati da tre sorgenti", {
  M_wos <- load_wos_fixture()
  M_scopus <- load_scopus_fixture()
  M_oa <- load_openalex_fixture()
  M_merged <- expect_no_error(
    suppressWarnings(suppressMessages(
      mergeDbSources(M_wos, M_scopus, M_oa, remove.duplicated = TRUE)
    ))
  )
  expect_true(nrow(M_merged) > 0)
})

# Tests for completeMetadata: vacancy predicate, DOI normaliser, JATS stripper,
# Crossref reference -> CR string conversion, fill-only merge contract.
# All tests run offline (no network calls); the Crossref client itself is
# exercised only indirectly via stubbed responses.

# These tests target the unexported helpers via :::.
.cm <- function(name) get(name, envir = asNamespace("bibliometrix"))

test_that(".normalize_doi strips prefixes and validates", {
  fn <- .cm(".normalize_doi")
  inp <- c(
    "10.1000/abc",
    "https://doi.org/10.1000/ABC",
    "http://dx.doi.org/10.1234/zz/yy",
    "doi:10.55555/x",
    "10.1/x",        # too few digits in registrant
    "not a doi",
    "",
    NA_character_
  )
  out <- fn(inp)
  expect_equal(out[1], "10.1000/abc")
  expect_equal(out[2], "10.1000/abc")
  expect_equal(out[3], "10.1234/zz/yy")
  expect_equal(out[4], "10.55555/x")
  expect_true(is.na(out[5]))
  expect_true(is.na(out[6]))
  expect_true(is.na(out[7]))
  expect_true(is.na(out[8]))
})

test_that(".is_vacant matches missingData()'s predicate", {
  fn <- .cm(".is_vacant")
  v <- c(NA, "", "NA", "none", "NA,0000,NA", "real value", " spaced ")
  out <- fn(v)
  expect_equal(out, c(TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE))
})

test_that(".strip_jats_abstract removes JATS markup", {
  fn <- .cm(".strip_jats_abstract")
  inp <- "<jats:p>Hello <jats:italic>world</jats:italic>.</jats:p>"
  out <- fn(inp)
  expect_match(out, "Hello world", fixed = TRUE)
  expect_false(grepl("<", out, fixed = TRUE))
  expect_true(is.na(fn(NA_character_)))
  expect_true(is.na(fn("")))
})

test_that(".crossref_authors formats AU as WoS-style 'LASTNAME I'", {
  fn <- .cm(".crossref_authors")
  authors <- list(
    list(family = "Aria", given = "Massimo",
         sequence = "first",
         affiliation = list(list(name = "Univ Naples"))),
    list(family = "Cuccurullo", given = "Corrado",
         affiliation = list())
  )
  out <- fn(authors)
  expect_equal(out$AU, "ARIA M; CUCCURULLO C")
  expect_equal(out$C1, "UNIV NAPLES")
  expect_equal(out$RP, "UNIV NAPLES")
})

test_that(".crossref_refs_to_cr_string builds WoS-style refs from structured items", {
  fn <- .cm(".crossref_refs_to_cr_string")
  refs <- list(
    list(author = "Smith J",
         year = "2010",
         `journal-title` = "J Sci",
         volume = "1",
         `first-page` = "5",
         DOI = "10.1234/abc"),
    list(unstructured = "Doe (2020) Some random ref; with semicolons; 10.5678/xyz")
  )
  out <- fn(refs)
  expect_match(out, "SMITH J, 2010, J SCI, V1, P5, DOI 10.1234/abc", fixed = TRUE)
  expect_match(out, "; ", fixed = TRUE)
  expect_match(out, "2020", fixed = TRUE)
  expect_match(out, "10.5678/xyz", fixed = TRUE)
  ## Semicolons inside an unstructured ref must be neutralised so they do not
  ## fragment the CR field.
  un_part <- sub(".*; ", "", out)
  expect_false(grepl(";.*;", un_part))
})

test_that(".crossref_refs_to_cr_string returns NA on empty input", {
  fn <- .cm(".crossref_refs_to_cr_string")
  expect_true(is.na(fn(NULL)))
  expect_true(is.na(fn(list())))
})

test_that(".crossref_doctype maps common Crossref types", {
  fn <- .cm(".crossref_doctype")
  expect_equal(fn("journal-article"), "ARTICLE")
  expect_equal(fn("book-chapter"), "BOOK CHAPTER")
  expect_equal(fn("posted-content"), "PREPRINT")
  expect_equal(fn("custom-type"), "CUSTOM-TYPE")  ## verbatim fallback uppercased
})

test_that(".merge_enrichment fills only missing cells and records provenance", {
  merge_fn <- .cm(".merge_enrichment")
  M <- data.frame(
    SR = c("paper a", "paper b", "paper c"),
    DI = c("10.1234/a", "10.1234/b", "10.1234/c"),
    AB = c(NA, "existing abstract", ""),
    TI = c("", NA, "existing title"),
    AU = c(NA_character_, NA_character_, NA_character_),
    stringsAsFactors = FALSE
  )
  enriched <- list(
    by_doi = list(
      "10.1234/a" = list(DI = "10.1234/a",
                      AB = "new abstract A",
                      TI = "new title A",
                      AU = "ARIA M"),
      "10.1234/b" = list(DI = "10.1234/b",
                      AB = "should NOT overwrite",
                      TI = "new title B",
                      AU = "CUCCURULLO C"),
      "10.1234/c" = list(DI = "10.1234/c",
                      AB = "new abstract C",
                      TI = "should NOT overwrite",
                      AU = "TEST X")
    ),
    fail_count = 0L
  )
  res <- merge_fn(M, enriched, eligible_idx = 1:3,
                  fields = c("AB","TI","AU"), source_label = "crossref")
  expect_equal(res$M$AB[1], "new abstract A")
  expect_equal(res$M$AB[2], "existing abstract")          ## preserved
  expect_equal(res$M$AB[3], "new abstract C")
  expect_equal(res$M$TI[1], "new title A")
  expect_equal(res$M$TI[2], "new title B")
  expect_equal(res$M$TI[3], "existing title")             ## preserved
  expect_equal(res$M$AU[1], "ARIA M")
  expect_equal(res$M$AU[2], "CUCCURULLO C")
  expect_equal(res$M$AU[3], "TEST X")

  expect_equal(res$filled[["AB"]], 2L)
  expect_equal(res$filled[["TI"]], 2L)
  expect_equal(res$filled[["AU"]], 3L)
  expect_equal(nrow(res$prov), 7L)
  expect_true(all(res$prov$source == "crossref"))
})

test_that(".merge_enrichment is a no-op when nothing is missing", {
  merge_fn <- .cm(".merge_enrichment")
  M <- data.frame(
    SR = "paper a",
    DI = "10.1234/a",
    AB = "already there",
    TI = "already there",
    stringsAsFactors = FALSE
  )
  enriched <- list(
    by_doi = list("10.1234/a" = list(DI = "10.1234/a",
                                  AB = "x", TI = "y")),
    fail_count = 0L
  )
  res <- merge_fn(M, enriched, eligible_idx = 1L,
                  fields = c("AB","TI"), source_label = "crossref")
  expect_identical(res$M, M)
  expect_null(res$prov)
  expect_equal(sum(res$filled), 0L)
})

test_that("completeMetadata is a no-op on a complete fixture (offline)", {
  ## Skip if we have no fixture or no internet check is needed.
  M <- tryCatch(load_wos_fixture(), error = function(e) NULL)
  skip_if(is.null(M), "no WoS fixture available")
  ## Block any HTTP traffic by passing an empty source set; the function
  ## must short-circuit and return the original collection unchanged.
  res <- completeMetadata(M, sources = character(0), verbose = FALSE)
  expect_identical(res$M, M)
  expect_equal(nrow(res$report), 0L)
})

test_that("completeMetadata short-circuits when no eligible records", {
  M <- data.frame(
    SR = c("a", "b"),
    DI = c(NA_character_, NA_character_),
    AB = c("x", "y"),
    AU = c("X", "Y"),
    TC = c(0, 0),
    stringsAsFactors = FALSE
  )
  res <- completeMetadata(M, sources = "crossref",
                          fields = c("AB","AU"), verbose = FALSE)
  expect_identical(res$M, M)
  expect_equal(nrow(res$report), 0L)
})

# ----- Phase 2: OpenAlex source -----------------------------------------------

test_that("completeMetadata auto-disables OpenAlex when M$DB is OPENALEX", {
  ## We construct a one-row collection with DB=OPENALEX and a DOI that
  ## triggers eligibility. With sources=c("openalex","crossref") and an
  ## offline-impossible Crossref endpoint, we just verify that we DO NOT
  ## crash on missing openalexR network and that the function attempts only
  ## Crossref. We block real traffic by setting fields to ones the local
  ## fallback path can short-circuit.
  M <- data.frame(
    SR = "a",
    DI = "10.1234/anything",
    AB = NA_character_,
    DB = "OPENALEX",
    TC = "0",
    stringsAsFactors = FALSE
  )
  ## Use sources = "openalex" alone -> auto-disabled -> empty source set ->
  ## function returns the original collection unchanged with empty report.
  res <- completeMetadata(M, sources = "openalex",
                          fields = c("AB","TC"), verbose = FALSE)
  expect_identical(res$M, M)
  expect_equal(nrow(res$report), 0L)
})

test_that("completeMetadata silently drops TC when only Crossref is selected", {
  M <- data.frame(
    SR = "a",
    DI = "10.1234/x",
    AB = NA_character_,
    TC = "0",
    DB = "ISI",
    stringsAsFactors = FALSE
  )
  ## Without an OpenAlex pass, TC must be dropped. Run with empty sources to
  ## avoid network: explicit sources = character(0) returns immediately.
  res <- completeMetadata(M, sources = character(0),
                          fields = c("AB","TC"), verbose = FALSE)
  expect_identical(res$M, M)
})

test_that(".openalex_authors formats AU as WoS-style 'LASTNAME I'", {
  fn <- .cm(".openalex_authors")
  authorships <- list(
    list(
      author = list(display_name = "Massimo Aria"),
      author_position = "first",
      is_corresponding = TRUE,
      institutions = list(list(display_name = "Univ Naples"))
    ),
    list(
      author = list(display_name = "Corrado Cuccurullo"),
      author_position = "middle",
      institutions = list(list(display_name = "Univ Vanvitelli"))
    )
  )
  out <- fn(authorships)
  expect_equal(out$AU, "ARIA M; CUCCURULLO C")
  expect_match(out$C1, "UNIV NAPLES", fixed = TRUE)
  expect_match(out$C1, "UNIV VANVITELLI", fixed = TRUE)
  expect_equal(out$RP, "UNIV NAPLES")
})

test_that(".openalex_doctype maps common OpenAlex types", {
  fn <- .cm(".openalex_doctype")
  expect_equal(fn("article"), "ARTICLE")
  expect_equal(fn("book-chapter"), "BOOK CHAPTER")
  expect_equal(fn("preprint"), "PREPRINT")
  expect_equal(fn("custom-type"), "CUSTOM-TYPE")
})

test_that(".openalex_work_to_row extracts the requested fields", {
  fn <- .cm(".openalex_work_to_row")
  work <- list(
    doi = "https://doi.org/10.1234/X",
    title = "A Title",
    publication_year = 2020,
    language = "en",
    type = "article",
    primary_location = list(source = list(display_name = "J Test")),
    abstract_inverted_index = list("Hello" = list(0L), "world" = list(1L)),
    cited_by_count = 7,
    authorships = list(
      list(
        author = list(display_name = "John Smith"),
        author_position = "first",
        is_corresponding = TRUE,
        institutions = list(list(display_name = "MIT"))
      )
    ),
    referenced_works = character(0)
  )
  out <- fn(work,
            fields = c("AB","AU","C1","DT","LA","PY","RP","SO","TC","TI","CR"),
            ref_lookup = list())
  expect_equal(out$DI, "10.1234/x")
  expect_equal(out$TI, "A Title")
  expect_equal(out$PY, "2020")
  expect_equal(out$LA, "en")
  expect_equal(out$DT, "ARTICLE")
  expect_equal(out$SO, "J Test")
  expect_equal(out$TC, "7")
  expect_equal(out$AU, "SMITH J")
  expect_equal(out$C1, "MIT")
  expect_equal(out$RP, "MIT")
  expect_match(out$AB, "Hello world", fixed = TRUE)
  ## CR key is absent (not NA) because referenced_works was empty.
  expect_true(is.null(out$CR))
})

test_that(".recompute_eligible drops rows fully populated by an earlier pass", {
  fn <- .cm(".recompute_eligible")
  M <- data.frame(
    SR = c("a","b","c"),
    AB = c("present", NA_character_, NA_character_),
    TI = c(NA_character_, "present", NA_character_),
    stringsAsFactors = FALSE
  )
  ## Initial eligibility was all 3 rows. After OpenAlex filled AB and TI on
  ## row a (so a now has TI=NA but AB=present), Crossref should still see
  ## row a as eligible because TI is missing.
  out <- fn(M, fields = c("AB","TI"), base_idx = 1:3)
  expect_equal(out, c(1L, 2L, 3L))
  ## Now mark row 'a' as fully populated:
  M2 <- M; M2$TI[1] <- "now present"
  out2 <- fn(M2, fields = c("AB","TI"), base_idx = 1:3)
  expect_equal(out2, c(2L, 3L))
})

test_that("completeMetadata report aggregates rows per source", {
  ## Stub the two .enrich_* helpers via local function-scope mocks so we
  ## can exercise the multi-source aggregation code path offline.
  M <- data.frame(
    SR = c("p1","p2"),
    DI = c("10.1234/aaa","10.1234/bbb"),
    AB = c(NA_character_, NA_character_),
    TI = c(NA_character_, NA_character_),
    TC = c("0","0"),
    DB = "ISI",
    stringsAsFactors = FALSE
  )

  ## Patch openalexR to be undetected -> .enrich_from_openalex returns empty
  ## (the requireNamespace branch). Patch httr2 + crossref endpoint by
  ## replacing .enrich_from_crossref via local() trace.
  orig_cr <- get(".enrich_from_crossref", envir = asNamespace("bibliometrix"))
  orig_oa <- get(".enrich_from_openalex", envir = asNamespace("bibliometrix"))
  local_unlock <- function(name, env) {
    if (exists(name, envir = env, inherits = FALSE)) {
      try(unlockBinding(name, env), silent = TRUE)
    }
  }
  ns <- asNamespace("bibliometrix")
  local_unlock(".enrich_from_crossref", ns)
  local_unlock(".enrich_from_openalex", ns)
  on.exit({
    assign(".enrich_from_crossref", orig_cr, envir = ns)
    assign(".enrich_from_openalex", orig_oa, envir = ns)
  }, add = TRUE)

  assign(".enrich_from_crossref",
         function(dois, fields, email, batch_size = 20,
                  progress = NULL, verbose = TRUE) {
           list(
             by_doi = list(
               "10.1234/aaa" = list(DI = "10.1234/aaa", TI = "T from CR"),
               "10.1234/bbb" = list(DI = "10.1234/bbb", AB = "AB from CR")
             ),
             fail_count = 0L
           )
         },
         envir = ns)
  assign(".enrich_from_openalex",
         function(dois, fields, email, oa_apikey,
                  progress = NULL, verbose = TRUE) {
           list(
             by_doi = list(
               "10.1234/aaa" = list(DI = "10.1234/aaa",
                                    AB = "AB from OA",
                                    TC = "42")
             ),
             fail_count = 0L
           )
         },
         envir = ns)

  res <- completeMetadata(
    M,
    sources = c("openalex", "crossref"),
    fields  = c("AB","TI","TC"),
    verbose = FALSE
  )

  ## Row 1: AB filled by OpenAlex (first); TI filled by Crossref (residual).
  expect_equal(res$M$AB[1], "AB from OA")
  expect_equal(res$M$TI[1], "T from CR")
  expect_equal(res$M$TC[1], "42")
  ## Row 2: AB filled by Crossref (OA had no record); TI was missing from
  ## both, stays NA.
  expect_equal(res$M$AB[2], "AB from CR")
  expect_true(is.na(res$M$TI[2]))

  ## Report should have rows for both sources.
  expect_setequal(unique(res$report$source), c("openalex", "crossref"))

  ## Provenance should record both sources too.
  prov <- attr(res$M, "enrichment")
  expect_setequal(unique(prov$source), c("openalex", "crossref"))

  ## M$ENRICH must summarise per row what was filled and from where.
  expect_true("ENRICH" %in% names(res$M))
  ## Row 1 had AB filled by OpenAlex, TC filled by OpenAlex, TI filled by Crossref.
  expect_match(res$M$ENRICH[1], "AB:OA", fixed = TRUE)
  expect_match(res$M$ENRICH[1], "TC:OA", fixed = TRUE)
  expect_match(res$M$ENRICH[1], "TI:CR", fixed = TRUE)
  ## Row 2 only had AB filled by Crossref.
  expect_match(res$M$ENRICH[2], "AB:CR", fixed = TRUE)
  expect_false(grepl(":OA", res$M$ENRICH[2], fixed = TRUE))
})

utils::globalVariables(c(
  "Rank",
  "SO",
  "Freq",
  "cumFreq",
  "Zone",
  "Theoretical",
  "logRank",
  "xmin",
  "ymin"
))
#' Bradford's law
#'
#' It estimates Bradford's law source distribution and tests the goodness of fit.
#'
#' Bradford's Law of Scattering, first formulated by Samuel C. Bradford in 1934,
#' describes the phenomenon of concentration and dispersion in scientific publishing:
#' a small number of core journals account for a disproportionately large share of the
#' literature on a given topic, while the remaining literature is scattered across an
#' increasingly large number of peripheral journals.\cr\cr
#'
#' If journals are ranked in decreasing order of productivity and partitioned into three
#' zones, each containing roughly one-third of the total articles, the number of journals
#' in each zone follows the ratio 1:n:n^2, where n is the Bradford multiplier.\cr\cr
#'
#' The Bradford distribution models the cumulative number of articles C(r) contributed
#' by the top r sources as: C(r) = a + b * log(r)\cr\cr
#'
#' Reference:\cr
#' Bradford, S. C. (1934). Sources of information on specific subjects. Engineering, 137, 85-86.\cr
#'
#' @param M is a bibliographic dataframe.
#' @return The function \code{bradford} returns a list containing the following objects:
#' \tabular{lll}{
#' \code{table}       \tab   \tab a dataframe with the source distribution partitioned in the three zones\cr
#' \code{graph}       \tab   \tab the Bradford bibliograph plot in ggplot2 format\cr
#' \code{graph_shiny} \tab   \tab the Bradford bibliograph plot for biblioshiny (without logo)\cr
#' \code{zoneSummary} \tab   \tab a dataframe summarizing the three Bradford zones\cr
#' \code{stat}        \tab   \tab a list of statistical results (coefficients, R2, KS test, Bradford multiplier)}
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' BR <- bradford(management)
#' }
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{summary}} method for class '\code{bibliometrix}'
#'
#' @export

bradford <- function(M) {
  ## 1. Sort sources by decreasing productivity ----
  SO <- sort(table(M$SO), decreasing = TRUE)
  nSO <- length(SO)
  N <- sum(SO)

  df <- data.frame(
    SO = names(SO),
    Rank = seq_len(nSO),
    Freq = as.numeric(SO),
    cumFreq = as.numeric(cumsum(SO)),
    stringsAsFactors = FALSE
  )

  ## 2. Fit Bradford's distribution: C(r) = a + b * log(r) ----
  df$logRank <- log(df$Rank)
  fit <- lm(cumFreq ~ logRank, data = df)
  a_coef <- as.numeric(coef(fit)[1])
  b_coef <- as.numeric(coef(fit)[2])
  R2 <- summary(fit)$r.squared
  df$Theoretical <- as.numeric(fitted(fit))

  ## 3. Zone assignment: each zone contains ~N/3 articles ----
  zone1_end <- sum(df$cumFreq <= N / 3) + 1
  zone2_end <- sum(df$cumFreq <= 2 * N / 3) + 1
  zone1_end <- min(zone1_end, nSO)
  zone2_end <- min(zone2_end, nSO)

  Z <- rep("Zone 3", nSO)
  Z[seq_len(zone1_end)] <- "Zone 1"
  if (zone1_end < nSO) {
    Z[(zone1_end + 1):zone2_end] <- "Zone 2"
  }
  df$Zone <- Z

  ## 4. Zone summary ----
  n1 <- sum(Z == "Zone 1")
  n2 <- sum(Z == "Zone 2")
  n3 <- sum(Z == "Zone 3")
  freq1 <- sum(df$Freq[Z == "Zone 1"])
  freq2 <- sum(df$Freq[Z == "Zone 2"])
  freq3 <- sum(df$Freq[Z == "Zone 3"])

  zoneSummary <- data.frame(
    Zone = c("Zone 1", "Zone 2", "Zone 3", "Total"),
    `N. Sources` = c(n1, n2, n3, nSO),
    `% Sources` = round(c(n1, n2, n3, nSO) / nSO * 100, 1),
    `N. Articles` = c(freq1, freq2, freq3, N),
    `% Articles` = round(c(freq1, freq2, freq3, N) / N * 100, 1),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  ## 5. Bradford multiplier ----
  # k estimated as geometric mean of consecutive zone ratios
  if (n1 > 0 && n2 > 0 && n3 > 0) {
    k <- (n2 / n1 + n3 / n2) / 2
  } else {
    k <- NA
  }

  ## 6. Kolmogorov-Smirnov goodness-of-fit test ----
  # Comparing empirical vs theoretical cumulative proportions
  empirical_prop <- df$cumFreq / N
  theoretical_prop <- pmin(pmax(df$Theoretical / N, 0), 1)
  ks_result <- suppressWarnings(
    ks.test(empirical_prop, theoretical_prop, exact = FALSE)
  )

  stat <- list(
    a = a_coef,
    b = b_coef,
    R2 = R2,
    k = k,
    ks.stat = as.numeric(ks_result$statistic),
    ks.pvalue = ks_result$p.value
  )

  ## 7. Bradford bibliograph plot ----
  zone_colors <- c(
    "Zone 1" = "#2171B5",
    "Zone 2" = "#6BAED6",
    "Zone 3" = "#BDD7E7"
  )
  zone_fills <- c(
    "Zone 1" = "#2171B5",
    "Zone 2" = "#6BAED6",
    "Zone 3" = "#BDD7E7"
  )

  # Zone boundary x positions (log scale)
  xz1 <- df$logRank[zone1_end]
  xz2 <- df$logRank[zone2_end]
  xmax <- max(df$logRank)
  ymax <- max(df$cumFreq)

  # Zone rects
  zone_rects <- data.frame(
    xmin = c(0, xz1, xz2),
    xmax = c(xz1, xz2, xmax),
    ymin = 0,
    ymax = ymax,
    Zone = c("Zone 1", "Zone 2", "Zone 3"),
    stringsAsFactors = FALSE
  )

  # Zone labels
  zone_labels <- data.frame(
    x = c(xz1 / 2, (xz1 + xz2) / 2, (xz2 + xmax) / 2),
    y = ymax * 0.95,
    label = c(
      paste0("Core\n(", n1, " sources)"),
      paste0("Zone 2\n(", n2, " sources)"),
      paste0("Zone 3\n(", n3, " sources)")
    ),
    stringsAsFactors = FALSE
  )

  # Subtitle with fit statistics
  subtitle_text <- paste0(
    "C(r) = ",
    round(a_coef, 1),
    " + ",
    round(b_coef, 1),
    " * log(r)",
    "    |    R2 = ",
    round(R2, 4),
    "    |    Bradford multiplier k = ",
    round(k, 2),
    "    |    KS p-value = ",
    format.pval(ks_result$p.value, digits = 3)
  )

  g_shiny <- suppressWarnings(ggplot2::ggplot(df) +
    # Zone background shading
    ggplot2::geom_rect(
      data = zone_rects,
      ggplot2::aes(
        xmin = xmin,
        xmax = xmax,
        ymin = ymin,
        ymax = ymax,
        fill = Zone
      ),
      alpha = 0.15,
      inherit.aes = FALSE
    ) +
    # Zone boundary lines
    ggplot2::geom_vline(
      xintercept = xz1,
      linetype = "dashed",
      color = "#666666",
      linewidth = 0.4
    ) +
    ggplot2::geom_vline(
      xintercept = xz2,
      linetype = "dashed",
      color = "#666666",
      linewidth = 0.4
    ) +
    # Theoretical fitted line
    ggplot2::geom_line(
      ggplot2::aes(x = logRank, y = Theoretical, group = 1),
      linetype = "dashed",
      color = "#D6604D",
      linewidth = 0.8,
      alpha = 0.8
    ) +
    # Empirical curve
    ggplot2::geom_line(
      ggplot2::aes(
        x = logRank,
        y = cumFreq,
        group = 1,
        text = paste0(
          "Source: ",
          SO,
          "\nRank: ",
          Rank,
          "\nArticles: ",
          Freq,
          "\nCumulative: ",
          cumFreq,
          "\nZone: ",
          Zone
        )
      ),
      color = "#1a1a1a",
      linewidth = 0.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(
        x = logRank,
        y = cumFreq,
        text = paste0(
          "Source: ",
          SO,
          "\nRank: ",
          Rank,
          "\nArticles: ",
          Freq,
          "\nCumulative: ",
          cumFreq,
          "\nZone: ",
          Zone
        )
      ),
      color = "#1a1a1a",
      size = 0.8,
      alpha = 0.5
    ) +
    # Zone labels
    ggplot2::annotate(
      "text",
      x = zone_labels$x,
      y = zone_labels$y,
      label = zone_labels$label,
      fontface = "bold",
      size = 3.5,
      color = "#333333",
      alpha = 0.7,
      vjust = 1
    ) +
    ggplot2::scale_fill_manual(values = zone_fills, guide = "none") +
    ggplot2::labs(
      x = "Source log(Rank)",
      y = "Cumulative N. of Articles",
      title = "Bradford's Law",
      subtitle = subtitle_text
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      text = ggplot2::element_text(color = "#333333"),
      plot.title = ggplot2::element_text(size = 18, face = "bold", hjust = 0.5),
      plot.subtitle = ggplot2::element_text(
        size = 10,
        face = "bold",
        color = "#666666",
        hjust = 0.5
      ),
      axis.title = ggplot2::element_text(size = 12, face = "bold"),
      axis.title.y = ggplot2::element_text(angle = 90, vjust = 1),
      axis.text = ggplot2::element_text(face = "bold", size = 11),
      axis.line = ggplot2::element_line(color = "#333333", linewidth = 0.4),
      panel.grid.major = ggplot2::element_line(
        color = "#EBEBEB",
        linewidth = 0.3
      ),
      panel.grid.minor = ggplot2::element_blank(),
      legend.position = "none"
    ))

  # Version with logo for export
  x_logo <- c(xmax - 0.02 - (xmax - 0) * 0.10, xmax - 0.02)
  y_logo <- c(0, ymax * 0.08)
  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  g <- g_shiny +
    ggplot2::annotation_custom(
      logo,
      xmin = x_logo[1],
      xmax = x_logo[2],
      ymin = y_logo[1],
      ymax = y_logo[2]
    )

  ## 8. Clean table for output ----
  df_out <- df[, c("SO", "Rank", "Freq", "cumFreq", "Zone")]
  names(df_out) <- c("SO", "Rank", "Freq", "cumFreq", "Zone")

  results <- list(
    table = df_out,
    graph = g,
    graph_shiny = g_shiny,
    zoneSummary = zoneSummary,
    stat = stat
  )
  return(results)
}

## SERVER components for contentanalysis in Biblioshiny ----

#' Enhanced Server Logic for Content Analysis Tab with 3 Tabs
#'
#' @param input Shiny input object
#' @param output Shiny output object
#' @param session Shiny session object
#' @param values Reactive values object

#' Enhanced Server Logic for Content Analysis Tab with 3 Tabs
#'
#' @param input Shiny input object
#' @param output Shiny output object
#' @param session Shiny session object
#' @param values Reactive values object

content_analysis_server <- function(input, output, session, values) {
  # Helper function for null coalescing
  `%||%` <- function(lhs, rhs) {
    if (!is.null(lhs) && length(lhs) > 0) lhs else rhs
  }

  # Enhance citation network layout for better readability
  # Post-processes the visNetwork from create_citation_network() with:
  # - max_nodes_per_section filtering
  # - community-aware Fruchterman-Reingold layout
  # - degree-proportional font sizing
  # - label overlap removal
  enhance_citation_network <- function(network, spacing = 5,
                                       max_nodes_per_section = NULL,
                                       node_size = 5,
                                       label_size = 5,
                                       repulsion = 5,
                                       edge_max_distance = NULL) {
    if (is.null(network)) return(NULL)

    # Extract nodes and edges from visNetwork object
    nodes <- network$x$nodes
    edges <- network$x$edges
    stats_attr <- attr(network, "stats")

    if (is.null(nodes) || nrow(nodes) == 0) return(network)

    # --- Filter edges by max distance ---
    if (!is.null(edge_max_distance) && "distance" %in% names(edges)) {
      edges <- edges[edges$distance <= edge_max_distance, ]
      if (nrow(edges) == 0) return(NULL)
    }

    # --- Increase edge transparency ---
    if ("color" %in% names(edges)) {
      edges$color <- gsub(
        "rgba\\((\\d+),\\s*(\\d+),\\s*(\\d+),\\s*[0-9.]+\\)",
        "rgba(\\1, \\2, \\3, 0.12)",
        edges$color
      )
    }

    # --- Max nodes per section filtering ---
    if (!is.null(max_nodes_per_section) && max_nodes_per_section > 0 &&
        "primary_section" %in% names(nodes)) {
      nodes <- nodes %>%
        dplyr::group_by(primary_section) %>%
        dplyr::arrange(dplyr::desc(connections), .by_group = TRUE) %>%
        dplyr::slice_head(n = max_nodes_per_section) %>%
        dplyr::ungroup() %>%
        as.data.frame(stringsAsFactors = FALSE)

      # Filter edges to only valid nodes
      valid_ids <- nodes$id
      edges <- edges[edges$from %in% valid_ids & edges$to %in% valid_ids, ]

      if (nrow(edges) == 0) return(NULL)

      # Recalculate connections after filtering
      edge_counts <- c(edges$from, edges$to)
      conn_table <- as.data.frame(table(edge_counts), stringsAsFactors = FALSE)
      names(conn_table) <- c("id", "connections")
      conn_table$id <- as.integer(conn_table$id)
      conn_table$connections <- as.integer(conn_table$connections)
      nodes$connections <- conn_table$connections[match(nodes$id, conn_table$id)]
      nodes$connections[is.na(nodes$connections)] <- 0
      nodes <- nodes[nodes$connections > 0, ]
      valid_ids <- nodes$id
      edges <- edges[edges$from %in% valid_ids & edges$to %in% valid_ids, ]
    }

    if (nrow(nodes) == 0 || nrow(edges) == 0) return(NULL)

    # --- Node size scaling (sqrt for compressed range) ---
    node_size <- max(1, min(10, node_size))
    size_mult <- node_size / 5

    scalemin <- round(10 * size_mult + 5)
    scalemax <- round(35 * size_mult + 15)
    conn_min <- min(nodes$connections)
    conn_max <- max(nodes$connections)

    if (conn_max > conn_min) {
      # sqrt scale: compresses high values, expands low values
      norm_conn <- (nodes$connections - conn_min) / (conn_max - conn_min)
      font_size <- sqrt(norm_conn) * (scalemax - scalemin) + scalemin
    } else {
      font_size <- rep((scalemin + scalemax) / 2, nrow(nodes))
    }
    font_size <- pmax(scalemin, pmin(scalemax, font_size))

    label_size <- max(1, min(10, label_size))
    label_mult <- label_size / 5
    nodes$font.size <- font_size * 0.6 * label_mult
    nodes$size <- font_size * 0.5 * size_mult
    nodes$font.vadjust <- -0.7 * nodes$font.size

    # Font opacity proportional to size (sqrt already applied above)
    opacity_range <- (font_size - min(font_size)) / max(1, diff(range(font_size)))
    font_opacity <- opacity_range * 0.6 + 0.4
    nodes$font.color <- sapply(font_opacity, function(a) {
      grDevices::adjustcolor("black", alpha.f = a)
    })

    # --- Build igraph and apply community repulsion to edge weights ---
    ig <- igraph::graph_from_data_frame(
      d = edges[, c("from", "to")],
      directed = FALSE,
      vertices = data.frame(name = nodes$id)
    )

    spacing <- max(1, min(10, spacing))
    spacing_mult <- spacing / 5
    repulsion <- max(1, min(10, repulsion))

    # Apply community repulsion via edge weight manipulation (same as biblioshiny co-occurrence)
    if ("primary_section" %in% names(nodes)) {
      node_sections <- stats::setNames(nodes$primary_section, as.character(nodes$id))
      n_nodes <- nrow(nodes)
      n_sections <- length(unique(nodes$primary_section))
      section_sizes <- table(nodes$primary_section)
      avg_section_size <- mean(section_sizes)

      # Adaptive repulsion strength (from biblioshiny networkPlot.R)
      community.repulsion <- repulsion / 10  # normalize to 0-1
      scale_factor <- log10(n_nodes + 10) / log10(100)
      community_factor <- 1 + (n_sections - 2) * 0.1
      community_factor <- max(0.5, min(community_factor, 2))
      x <- community.repulsion * 10
      sigmoid_transform <- x / (1 + x)
      repulsion_strength <- sigmoid_transform * scale_factor * community_factor

      # Apply weights per edge
      edge_from_sec <- node_sections[as.character(edges$from)]
      edge_to_sec <- node_sections[as.character(edges$to)]
      same_section <- edge_from_sec == edge_to_sec

      original_weights <- if (!is.null(igraph::E(ig)$weight)) {
        igraph::E(ig)$weight
      } else {
        rep(1, nrow(edges))
      }

      new_weights <- original_weights
      # Intra-section: moderate increase with sub-linear growth
      intra_mult <- 1 + (repulsion_strength^0.7) * 1.5
      new_weights[same_section] <- original_weights[same_section] * intra_mult
      # Inter-section: gradual reduction with attenuated exponential
      inter_div <- 1 + exp(repulsion_strength * 1.2) - 1
      new_weights[!same_section] <- original_weights[!same_section] / inter_div

      igraph::E(ig)$weight <- new_weights
    }

    set.seed(values$random_seed %||% 1234)
    coords <- igraph::layout_with_fr(ig, niter = 1500)
    coords <- igraph::norm_coords(coords, xmin = -1, xmax = 1, ymin = -1, ymax = 1)

    # --- Post-layout: spread nodes within each cluster ---
    if ("primary_section" %in% names(nodes)) {
      spread_factor <- 0.15 * spacing_mult
      sections <- unique(nodes$primary_section)
      for (sec in sections) {
        idx <- which(nodes$primary_section == sec)
        if (length(idx) > 1) {
          sec_center <- colMeans(coords[idx, , drop = FALSE])
          coords[idx, 1] <- sec_center[1] + (coords[idx, 1] - sec_center[1]) * (1 + spread_factor)
          coords[idx, 2] <- sec_center[2] + (coords[idx, 2] - sec_center[2]) * (1 + spread_factor)
        }
      }
      coords <- igraph::norm_coords(coords, xmin = -1, xmax = 1, ymin = -1, ymax = 1)
    }

    # --- Label overlap removal ---
    threshold <- 0.05 * spacing_mult
    w <- data.frame(
      x = coords[, 1],
      y = coords[, 2] / 2,
      label = nodes$label,
      dotSize = font_size,
      stringsAsFactors = FALSE
    )

    labeled <- w[w$label != "", , drop = FALSE]
    labels_to_remove <- character(0)
    if (nrow(labeled) >= 2) {
      d <- as.matrix(stats::dist(labeled[, c("x", "y")], method = "manhattan"))
      diag(d) <- Inf
      while (TRUE) {
        min_idx <- which(d == min(d), arr.ind = TRUE)[1, ]
        if (d[min_idx[1], min_idx[2]] >= threshold) break
        i <- min_idx[1]; j <- min_idx[2]
        remove_idx <- if (labeled$dotSize[i] >= labeled$dotSize[j]) j else i
        labels_to_remove <- c(labels_to_remove, labeled$label[remove_idx])
        d[remove_idx, ] <- Inf
        d[, remove_idx] <- Inf
      }
    }
    nodes$label[nodes$label %in% labels_to_remove] <- ""

    # --- Rebuild visNetwork with improved layout ---
    network_enhanced <- visNetwork::visNetwork(
      nodes, edges,
      type = "full", smooth = TRUE, physics = FALSE
    ) %>%
      visNetwork::visNodes(
        shadow = TRUE, shape = "dot",
        font = list(
          color = nodes$font.color,
          size = nodes$font.size,
          vadjust = nodes$font.vadjust
        )
      ) %>%
      visNetwork::visIgraphLayout(
        layout = "layout.norm",
        layoutMatrix = coords,
        type = "full"
      ) %>%
      visNetwork::visEdges(smooth = list(type = "horizontal")) %>%
      visNetwork::visOptions(
        highlightNearest = list(enabled = TRUE, hover = TRUE, degree = 1),
        nodesIdSelection = TRUE
      ) %>%
      visNetwork::visInteraction(
        dragNodes = TRUE, dragView = TRUE,
        hideEdgesOnDrag = TRUE, zoomView = TRUE, zoomSpeed = 0.15
      ) %>%
      visNetwork::visPhysics(enabled = FALSE)

    # Update stats
    if (!is.null(stats_attr)) {
      stats_attr$n_nodes <- nrow(nodes)
      stats_attr$n_edges <- nrow(edges)
      stats_attr$section_distribution <- nodes %>%
        dplyr::count(primary_section) %>%
        dplyr::arrange(dplyr::desc(n))
    }
    attr(network_enhanced, "stats") <- stats_attr

    return(network_enhanced)
  }

  # Cartella temporanea per i PDF
  tmpdir <- tempdir()

  # Rendi la cartella temporanea accessibile via HTTP
  addResourcePath("tmpPDF", tmpdir)

  # Preview visibility reactive value
  preview_visible <- reactiveVal(TRUE)

  # Check if text is extracted
  output$text_extracted <- reactive({
    return(!is.null(values$pdf_text) && nchar(values$pdf_text) > 0)
  })
  outputOptions(output, "text_extracted", suspendWhenHidden = FALSE)

  # Preview visibility reactive output
  output$preview_visible <- reactive({
    return(preview_visible())
  })
  outputOptions(output, "preview_visible", suspendWhenHidden = FALSE)

  # ===========================================
  # PDF UPLOAD AND TEXT EXTRACTION
  # ===========================================

  # Check if PDF is uploaded
  output$pdf_uploaded <- reactive({
    return(!is.null(input$pdf_file))
  })
  outputOptions(output, "pdf_uploaded", suspendWhenHidden = FALSE)

  # Display PDF info
  output$pdf_info <- renderText({
    if (!is.null(input$pdf_file)) {
      file_size <- round(input$pdf_file$size / 1024 / 1024, 2)
      paste("File:", input$pdf_file$name, "| Size:", file_size, "MB")
    }
  })

  ## PDF PREVIEW
  output$pdf_viewer <- renderUI({
    req(input$pdf_file)

    # Verifica che sia un PDF
    if (!grepl("\\.pdf$", input$pdf_file$name, ignore.case = TRUE)) {
      return(
        div(
          style = "color: red; padding: 20px;",
          "Errore: Il file selezionato non è un PDF"
        )
      )
    }

    # Copia il file nella cartella temporanea con nome univoco
    pdf_name <- paste0(
      "viewer_",
      format(Sys.time(), "%Y%m%d_%H%M%S"),
      "_",
      input$pdf_file$name
    )
    pdf_path <- file.path(tmpdir, pdf_name)
    file.copy(input$pdf_file$datapath, pdf_path, overwrite = TRUE)

    # Crea l'iframe che punta al file tramite il path accessibile
    tags$iframe(
      src = paste0("tmpPDF/", pdf_name),
      width = "100%",
      height = "800px",
      style = "border: 1px solid #ddd;"
    )
  })

  # Pulizia alla chiusura della sessione
  onSessionEnded(function() {
    files <- list.files(
      tmpdir,
      pattern = "^viewer_.*\\.pdf$",
      full.names = TRUE
    )
    file.remove(files)
  })

  ## END PDF PREVIEW

  # Extract text from PDF
  observeEvent(input$extract_text, {
    req(input$pdf_file)

    # VALIDATION: Check if citation type is selected
    if (
      is.null(input$citation_type_import) ||
        input$citation_type_import == "" ||
        length(input$citation_type_import) == 0
    ) {
      showNotification(
        HTML(
          "<strong>Citation Format Required</strong><br/>Please select the citation format used in your PDF before extracting text."
        ),
        type = "warning",
        duration = 5
      )

      return() # Stop execution
    }

    tryCatch(
      {
        if (is.na(input$Columns)) {
          n_columns <- NULL
        } else {
          n_columns <- input$Columns
        }

        # Get citation type from input
        citation_type <- input$citation_type_import

        # Extract text with citation type parameter
        pdf_text <- pdf2txt_auto(
          input$pdf_file$datapath,
          n_columns = n_columns,
          citation_type = citation_type,
          enable_ai_support = input$enable_ai_support,
          ai_model = values$gemini_api_model,
          api_key = NULL
        )

        values$pdf_text <- pdf_text[[1]]

        # Extract PDF metadata (authors, title, journal, year, doi)
        pdf_metadata_info <- tryCatch(
          {
            contentanalysis::extract_pdf_metadata(
              input$pdf_file$datapath,
              fields = "all"
            )
          },
          error = function(e) {
            NULL
          }
        )

        # Store metadata in values
        values$pdf_metadata <- pdf_metadata_info

        # Check if AI support is enabled
        if (input$enable_ai_support) {
          if (!is.null(pdf_text)) {
            showNotification(
              "Text extracted successfully with AI enhancement!",
              type = "message",
              duration = 3
            )
          }
        }

        values$pdf_sections <- pdf_text

        values$citation_type_used <- citation_type # Store for later use

        pdf_metadata <- unlist(pdftools::pdf_info(input$pdf_file$datapath))

        # Extract DOI
        extracted_doi <- tryCatch(
          {
            extract_doi_from_pdf(input$pdf_file$datapath)
          },
          error = function(e) {
            NULL
          }
        )

        # Store DOI and update input field
        if (
          !is.null(extracted_doi) &&
            !is.na(extracted_doi) &&
            nzchar(trimws(extracted_doi))
        ) {
          values$pdf_doi <- trimws(extracted_doi)
          updateTextInput(
            session,
            "pdf_doi_input",
            value = trimws(extracted_doi)
          )
        } else {
          values$pdf_doi <- ""
          updateTextInput(session, "pdf_doi_input", value = "")
        }

        # Create notification message based on citation type
        notification_msg <- if (
          !is.null(extracted_doi) && nzchar(trimws(extracted_doi))
        ) {
          "PDF text extracted successfully! DOI detected and populated."
        } else {
          "PDF text extracted successfully! Please enter DOI manually if needed."
        }

        # Add citation type info to notification
        citation_type_label <- switch(
          citation_type,
          "author_year" = "Author-year",
          "numeric_bracketed" = "Numeric brackets",
          "numeric_superscript" = "Numeric superscript",
          "all" = "All formats"
        )

        if (citation_type == "numeric_superscript") {
          notification_msg <- paste0(
            notification_msg,
            " Superscript citations converted to [n] format."
          )
        }

        notification_msg <- paste0(
          notification_msg,
          " (Citation format: ",
          citation_type_label,
          ")"
        )

        showNotification(
          notification_msg,
          type = "message",
          duration = 5
        )

        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
      },
      error = function(e) {
        showNotification(
          paste("Error extracting PDF text:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Disable/Enable extract button based on citation type selection
  observe({
    if (
      !is.null(input$citation_type_import) &&
        input$citation_type_import != "" &&
        length(input$citation_type_import) > 0
    ) {
      # Citation type selected - ensure button is enabled
      shinyjs::enable("extract_text")
      shinyjs::removeCssClass("extract_text", "btn-secondary")
      shinyjs::addCssClass("extract_text", "btn-info")
    } else {
      # No citation type selected - visual feedback but don't disable
      # (We handle this in the observeEvent instead)
      shinyjs::removeCssClass("extract_text", "btn-info")
      shinyjs::addCssClass("extract_text", "btn-secondary")
    }
  })

  # Display PDF metadata (authors, title, journal) - compact version for top-right
  output$pdf_metadata_display <- renderUI({
    if (!is.null(values$pdf_metadata)) {
      metadata <- values$pdf_metadata

      # Extract metadata fields with safe defaults
      authors <- if (
        !is.null(metadata$authors) &&
          nzchar(metadata$authors) &&
          !is.na(metadata$authors)
      ) {
        if (nchar(metadata$authors) > 120) {
          paste0(substr(metadata$authors, 1, 117), "...")
        } else {
          metadata$authors
        }
      } else {
        "Authors not available"
      }

      title <- if (
        !is.null(metadata$title) &&
          nzchar(metadata$title) &&
          !is.na(metadata$title)
      ) {
        # Truncate title if too long
        if (nchar(metadata$title) > 120) {
          paste0(substr(metadata$title, 1, 117), "...")
        } else {
          metadata$title
        }
      } else {
        "Title not available"
      }

      journal <- if (
        !is.null(metadata$journal) &&
          nzchar(metadata$journal) &&
          !is.na(metadata$journal)
      ) {
        metadata$journal
      } else {
        "Journal not available"
      }

      year <- if (
        !is.null(metadata$year) &&
          !is.na(metadata$year) &&
          !is.na(metadata$year)
      ) {
        as.character(metadata$year)
      } else {
        ""
      }

      # Build compact display for top-right corner
      div(
        # style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); padding: 12px 15px; border-radius: 6px; margin-bottom: 15px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); color: white; text-align: left; font-size: 12px;",
        style = "background: linear-gradient(135deg, #1D8FE1 0%, #5765B1 100%); padding: 12px 15px; border-radius: 6px; margin-bottom: 15px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); color: white; text-align: left; font-size: 12px;",

        # Title (most prominent)
        div(
          style = "margin-bottom: 6px; font-weight: 600; font-size: 13px; line-height: 1.3;",
          title
        ),

        # Authors
        div(
          style = "margin-bottom: 4px; opacity: 0.95;",
          icon("user", style = "margin-right: 5px; font-size: 11px;"),
          tags$span(authors, style = "font-size: 11px;")
        ),

        # Journal and Year
        div(
          style = "opacity: 0.9;",
          icon("book", style = "margin-right: 5px; font-size: 11px;"),
          tags$span(
            paste0(journal, if (nzchar(year)) paste0(" (", year, ")") else ""),
            style = "font-size: 11px;"
          )
        )
      )
    }
  })

  # Check if metadata is available
  output$metadata_available <- reactive({
    return(!is.null(values$pdf_metadata))
  })
  outputOptions(output, "metadata_available", suspendWhenHidden = FALSE)

  # Check if DOI was detected automatically
  output$doi_detected <- reactive({
    if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
      # Check if it's the same as what was automatically extracted
      extracted_doi <- tryCatch(
        {
          if (!is.null(input$pdf_file)) {
            extract_doi_from_pdf(input$pdf_file$datapath)
          } else {
            NULL
          }
        },
        error = function(e) NULL
      )

      return(
        !is.null(extracted_doi) &&
          nzchar(extracted_doi) &&
          extracted_doi == input$pdf_doi_input
      )
    }
    return(FALSE)
  })

  outputOptions(output, "doi_detected", suspendWhenHidden = FALSE)

  # Display DOI after analysis
  output$doi_display_after_analysis <- renderText({
    doi_value <- values$pdf_doi

    if (!is.null(doi_value) && !is.na(doi_value) && nzchar(trimws(doi_value))) {
      return(trimws(doi_value))
    } else {
      return("Not available")
    }
  })

  # DOI status icon
  output$doi_status_icon <- renderUI({
    # Verifica se il campo DOI ha un valore
    doi_value <- input$pdf_doi_input

    if (is.null(doi_value) || !nzchar(trimws(doi_value))) {
      # Nessun DOI inserito
      return(
        tagList(
          icon(
            "exclamation-triangle",
            style = "color: #e74c3c; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "DOI not found",
            style = "color: #e74c3c; font-weight: bold;"
          )
        )
      )
    }

    # C'è un DOI, verifica se è stato auto-rilevato
    auto_detected <- FALSE

    if (!is.null(input$pdf_file)) {
      extracted_doi <- tryCatch(
        {
          extract_doi_from_pdf(input$pdf_file$datapath)
        },
        error = function(e) {
          NULL
        }
      )

      # Verifica se il DOI estratto corrisponde a quello inserito
      if (
        !is.null(extracted_doi) &&
          !is.na(extracted_doi) &&
          nzchar(trimws(extracted_doi))
      ) {
        auto_detected <- (trimws(extracted_doi) == trimws(doi_value))
      }
    }

    # Restituisci l'icona appropriata
    if (auto_detected) {
      return(
        tagList(
          icon(
            "check-circle",
            style = "color: #27ae60; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "Auto-detected",
            style = "color: #27ae60; font-weight: bold;"
          )
        )
      )
    } else {
      return(
        tagList(
          icon(
            "edit",
            style = "color: #f39c12; font-size: 18px; margin-right: 5px;"
          ),
          tags$small(
            "Manually entered",
            style = "color: #f39c12; font-weight: bold;"
          )
        )
      )
    }
  })

  # Update DOI value when user edits the field
  observeEvent(
    input$pdf_doi_input,
    {
      if (!is.null(input$pdf_doi_input)) {
        values$pdf_doi <- trimws(input$pdf_doi_input)
      }
    },
    ignoreInit = TRUE
  )

  # ===========================================
  # LOAD PREVIOUSLY SAVED TEXT FILE
  # ===========================================

  # Check if text file is loaded
  output$text_file_loaded <- reactive({
    return(!is.null(input$load_text_file))
  })
  outputOptions(output, "text_file_loaded", suspendWhenHidden = FALSE)

  # Display text file info
  output$text_file_info <- renderText({
    if (!is.null(input$load_text_file)) {
      file_size <- round(input$load_text_file$size / 1024, 2)
      paste("File:", input$load_text_file$name, "| Size:", file_size, "KB")
    }
  })

  observeEvent(input$load_text_file, {
    req(input$load_text_file)

    tryCatch(
      {
        # Read the text file
        file_content <- readLines(input$load_text_file$datapath, warn = FALSE)

        if (length(file_content) == 0) {
          showNotification(
            "The text file is empty.",
            type = "error",
            duration = 5
          )
          return()
        }

        # Extract DOI from first line if present (format: "DOI: xxxxx")
        first_line <- file_content[1]
        extracted_doi <- ""
        text_start_line <- 1

        if (grepl("^DOI:\\s*", first_line, ignore.case = TRUE)) {
          extracted_doi <- sub("^DOI:\\s*", "", first_line, ignore.case = TRUE)
          extracted_doi <- trimws(extracted_doi)
          text_start_line <- 2
        }

        # Extract CITATION_TYPE from second line if present (format: "CITATION_TYPE: xxxxx")
        extracted_citation_type <- ""
        if (length(file_content) >= 2) {
          second_line <- file_content[2]
          if (grepl("^CITATION_TYPE:\\s*", second_line, ignore.case = TRUE)) {
            extracted_citation_type <- sub(
              "^CITATION_TYPE:\\s*",
              "",
              second_line,
              ignore.case = TRUE
            )
            extracted_citation_type <- trimws(extracted_citation_type)
            text_start_line <- 3
          }
        }

        # Get the text content (everything after metadata lines)
        if (length(file_content) >= text_start_line) {
          text_content <- reconstruct_from_txt(
            file_content[text_start_line:length(file_content)]
          )
        } else {
          text_content <- ""
        }

        # Store the text
        values$pdf_text <- unlist(text_content[1])
        values$pdf_sections <- text_content

        # Update DOI field
        if (nzchar(extracted_doi)) {
          values$pdf_doi <- extracted_doi
          updateTextInput(session, "pdf_doi_input", value = extracted_doi)
        } else {
          values$pdf_doi <- ""
          updateTextInput(session, "pdf_doi_input", value = "")
        }

        # Update citation_type field if available
        if (nzchar(extracted_citation_type)) {
          values$citation_type_used <- extracted_citation_type
          # Note: citation_type_import is only used during PDF import, not needed here
        }

        # Show notification
        notification_msg <- "Text file loaded successfully!"
        if (nzchar(extracted_doi)) {
          notification_msg <- paste0(notification_msg, " DOI: ", extracted_doi)
        }
        if (nzchar(extracted_citation_type)) {
          citation_type_label <- switch(
            extracted_citation_type,
            "author_year" = "Author-year",
            "numeric_bracketed" = "Numeric brackets",
            "numeric_superscript" = "Numeric superscript",
            "all" = "All formats",
            extracted_citation_type
          )
          notification_msg <- paste0(
            notification_msg,
            " | Citation format: ",
            citation_type_label
          )
        }

        showNotification(
          notification_msg,
          type = "message",
          duration = 7
        )

        # Enable analysis button
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
      },
      error = function(e) {
        showNotification(
          paste("Error loading text file:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # ===========================================
  # SAVE EXTRACTED TEXT TO FILE
  # ===========================================
  output$save_text_file <- downloadHandler(
    filename = function() {
      # Generate filename with DOI or timestamp
      if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
        # Clean DOI for filename (replace / with _)
        clean_doi <- gsub("/", "_", values$pdf_doi)
        clean_doi <- gsub("[^a-zA-Z0-9._-]", "", clean_doi)
        paste0("extracted_text_", clean_doi, ".txt")
      } else {
        paste0("extracted_text_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".txt")
      }
    },
    content = function(file) {
      # Prepare content with DOI and CITATION_TYPE in first lines
      file_content <- ""

      # Add DOI as first line if available
      if (!is.null(values$pdf_doi) && nzchar(values$pdf_doi)) {
        file_content <- paste0("DOI: ", values$pdf_doi, "\n")
      } else {
        file_content <- "DOI: \n"
      }

      # Add CITATION_TYPE as second line if available
      if (
        !is.null(values$citation_type_used) && nzchar(values$citation_type_used)
      ) {
        file_content <- paste0(
          file_content,
          "CITATION_TYPE: ",
          values$citation_type_used,
          "\n"
        )
      } else {
        file_content <- paste0(file_content, "CITATION_TYPE: \n")
      }

      if (length(values$pdf_sections) > 1) {
        txt <- values$pdf_sections
        section_names <- setdiff(names(txt), "Full_text")
        # save txt in a text file adding a row with \n\nSECTION: section names\n\n before each section except Full_text
        #file_content <- ""
        for (section in section_names) {
          file_content <- paste0(
            file_content,
            "\n\nSECTION LINE SEPARATION: ",
            section,
            "\n\n",
            txt[[section]]
          )
        }
      } else {
        file_content <- paste0(file_content, unlist(values$pdf_sections[1]))
      }

      # # Add the extracted text
      # if (!is.null(values$pdf_text)) {
      #   file_content <- paste0(file_content, values$pdf_text)
      # }

      # Write to file
      writeLines(file_content, file, sep = "")
    }
  )

  # Text length information
  output$text_length_info <- renderText({
    if (!is.null(values$pdf_text)) {
      char_count <- nchar(values$pdf_text)
      word_count <- length(strsplit(values$pdf_text, "\\s+")[[1]])
      paste(
        "Characters:",
        format(char_count, big.mark = ","),
        "| Words:",
        format(word_count, big.mark = ",")
      )
    }
  })

  # Text preview output
  output$text_preview <- renderText({
    if (!is.null(values$pdf_text)) {
      preview_length <- 80000
      full_text <- values$pdf_text

      if (nchar(full_text) > preview_length) {
        truncated <- substr(full_text, 1, preview_length)
        last_period <- max(c(
          regexpr("\\. [A-Z]", truncated, perl = TRUE),
          regexpr("\\.\n", truncated, perl = TRUE),
          regexpr("\\?\n", truncated, perl = TRUE),
          regexpr("!\n", truncated, perl = TRUE)
        ))

        if (last_period > 100) {
          truncated <- substr(truncated, 1, last_period)
        }

        paste0(
          truncated,
          "\n\n[...text continues for ",
          format(nchar(full_text) - nchar(truncated), big.mark = ","),
          " more characters...]"
        )
      } else {
        full_text
      }
    }
  })

  # Toggle preview visibility
  observeEvent(input$toggle_preview, {
    preview_visible(!preview_visible())
    if (preview_visible()) {
      updateActionButton(session, "toggle_preview", "Hide Preview")
    } else {
      updateActionButton(session, "toggle_preview", "Show Preview")
    }
  })

  # ===========================================
  # CONTENT ANALYSIS
  # ===========================================

  observeEvent(input$run_analysis, {
    req(values$pdf_text)

    updateActionButton(
      session,
      "run_analysis",
      label = "Processing...",
      icon = icon("spinner", class = "fa-spin")
    )

    tryCatch(
      {
        custom_stops <- NULL
        if (
          !is.null(input$custom_stopwords) && nzchar(input$custom_stopwords)
        ) {
          custom_stops <- trimws(strsplit(input$custom_stopwords, ",")[[1]])
        }

        # Determine citation segmentation
        use_sections_cit <- switch(
          input$citation_segmentation %||% "auto",
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE,
          "auto" # default fallback
        )

        n_segs_cit <- input$n_segments_citations %||% 10

        # Get citation type (use stored value from extraction or default)
        citation_type <- values$citation_type_used
        if (is.null(citation_type) || citation_type == "") {
          citation_type <- input$citation_type_import
          if (is.null(citation_type) || citation_type == "") {
            citation_type <- "author_year" # ultimate fallback
          }
        }

        # Run analysis with citation type parameter
        values$analysis_results <- analyze_scientific_content(
          text = values$pdf_sections,
          doi = values$pdf_doi,
          citation_type = citation_type,
          window_size = input$window_size,
          parse_multiple_citations = input$parse_multiple,
          remove_stopwords = input$remove_stopwords,
          custom_stopwords = custom_stops,
          use_sections_for_citations = use_sections_cit,
          n_segments_citations = n_segs_cit
        )

        # remove doi_pattern citations
        values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
          dplyr::filter(!citation_type %in% c("doi_pattern"))

        values$references_oa <- values$analysis_results$references_oa
        section_colors <- colorlist()[
          1:length(unique(values$analysis_results$citation_contexts$section))
        ]
        names(section_colors) <- unique(
          values$analysis_results$citation_contexts$section
        )
        values$analysis_results$section_colors <- section_colors

        # Calculate readability indices
        values$readability_indices <- calculate_readability_indices(
          text = values$pdf_text,
          detailed = TRUE
        )

        # Create network if we have data
        if (
          !is.null(values$analysis_results$network_data) &&
            nrow(values$analysis_results$network_data) > 0
        ) {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        }

        # Describe citation clusters
        tryCatch(
          {
            values$cluster_description <- describe_citation_clusters(
              values$analysis_results,
              top_n = 10,
              ngram_range = c(1, 2)
            )
          },
          error = function(e) {
            values$cluster_description <- NULL
          }
        )

        preview_visible(FALSE)

        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )

        showNotification(
          paste0(
            "Content analysis completed successfully! Citation type: ",
            switch(
              citation_type,
              "author_year" = "Author-Year",
              "numeric_bracketed" = "Numeric Bracketed",
              "numeric_superscript" = "Numeric Superscript",
              "all" = "All Formats"
            )
          ),
          type = "message",
          duration = 4
        )

        # Close the PDF import box
        session$sendCustomMessage('togglePdfImport', list(action = 'close'))
      },
      error = function(e) {
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )

        showNotification(
          paste("Error in content analysis:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Check if analysis is completed
  output$analysis_completed <- reactive({
    return(!is.null(values$analysis_results))
  })
  outputOptions(output, "analysis_completed", suspendWhenHidden = FALSE)

  # Check if Gemini API is available
  output$gemini_api_available <- reactive({
    return(!is.null(values$geminiAPI) && values$geminiAPI == TRUE)
  })
  outputOptions(output, "gemini_api_available", suspendWhenHidden = FALSE)

  # ===========================================
  # TAB 1: DESCRIPTIVE STATISTICS OUTPUTS
  # ===========================================

  output$total_words <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$total_words_analyzed,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$total_citations <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$citations_extracted,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$narrative_citations <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$narrative_citations,
        big.mark = ","
      )
    } else {
      "0"
    }
  })

  output$citation_density <- renderText({
    if (!is.null(values$analysis_results)) {
      format(
        values$analysis_results$summary$citation_density_per_1000_words,
        digits = 1
      )
    } else {
      "0.0"
    }
  })

  # Readability indices outputs
  output$flesch_kincaid_grade <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$flesch_kincaid_grade, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$flesch_reading_ease <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$flesch_reading_ease, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$ari_index <- renderText({
    if (!is.null(values$readability_indices)) {
      format(
        round(values$readability_indices$automated_readability_index, 1),
        nsmall = 1
      )
    } else {
      "0.0"
    }
  })

  output$gunning_fog_index <- renderText({
    if (!is.null(values$readability_indices)) {
      format(round(values$readability_indices$gunning_fog_index, 1), nsmall = 1)
    } else {
      "0.0"
    }
  })

  # Text Statistics - Matching height with readability
  # Text Statistics - Matching structure with readability
  output$text_stats <- renderUI({
    if (!is.null(values$analysis_results)) {
      stats <- values$analysis_results$text_analytics$basic_stats

      # Get additional stats from readability indices
      syllables_info <- ""
      complex_info <- ""

      if (!is.null(values$readability_indices)) {
        syllables_info <- format(
          values$readability_indices$n_syllables,
          big.mark = ","
        )
        complex_info <- sprintf(
          "Complex words:      %s (%.1f%%)",
          format(values$readability_indices$n_complex_words, big.mark = ","),
          values$readability_indices$pct_complex_words
        )
      } else {
        syllables_info <- "N/A"
        complex_info <- "Complex words:      N/A"
      }

      text_content <- sprintf(
        "TEXT STATISTICS
━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

🔤 Characters:      %s
📄 Words:           %s
✏️ Sentences:       %s
🗣 Syllables:       %s

━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

%s
Avg words/sentence: %.1f
Lexical diversity:  %.3f",
        format(stats$total_characters, big.mark = ","),
        format(stats$total_words, big.mark = ","),
        format(stats$total_sentences, big.mark = ","),
        syllables_info,
        complex_info,
        round(stats$avg_words_per_sentence, 1),
        round(values$analysis_results$summary$lexical_diversity, 3)
      )

      tags$pre(
        style = "background-color: #f8f9fa; padding: 15px; border-radius: 6px; border: 1px solid #dee2e6; font-family: 'Courier New', monospace; font-size: 13px; line-height: 1.5; color: #333; margin: 0; min-height: 280px;",
        text_content
      )
    } else {
      tags$pre(
        style = "background-color: #f8f9fa; padding: 40px; border-radius: 6px; border: 1px solid #dee2e6; text-align: center; color: #999; margin: 0; min-height: 280px; display: flex; align-items: center; justify-content: center;",
        "No analysis data available"
      )
    }
  })

  # Readability Indices - Add min-height
  output$readability_details_html <- renderUI({
    if (!is.null(values$readability_indices)) {
      indices <- values$readability_indices

      # Helper to get interpretation symbol
      get_symbol <- function(index_name, value) {
        switch(
          index_name,
          "flesch_kincaid" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          },
          "flesch_ease" = if (value >= 60) {
            "✓"
          } else if (value >= 30) {
            "◆"
          } else {
            "▲"
          },
          "ari" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          },
          "gunning" = if (value <= 12) {
            "✓"
          } else if (value <= 16) {
            "◆"
          } else {
            "▲"
          }
        )
      }

      # Helper to get interpretation text
      get_interp_text <- function(index_name, value) {
        switch(
          index_name,
          "flesch_kincaid" = paste("Grade", round(value)),
          "flesch_ease" = if (value >= 60) {
            "Easy"
          } else if (value >= 30) {
            "Moderate"
          } else {
            "Difficult"
          },
          "ari" = paste("Grade", round(value)),
          "gunning" = if (value <= 12) {
            "Readable"
          } else if (value <= 16) {
            "Difficult"
          } else {
            "Very difficult"
          }
        )
      }

      text_content <- sprintf(
        "READABILITY INDICES
━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

📚 Flesch-Kincaid:  %5.1f  %s  %s
📖 Reading Ease:    %5.1f  %s  %s  
🤖 ARI Index:       %5.1f  %s  %s
☁️ Gunning Fog:     %5.1f  %s  %s

━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

Sentences: %s • Words: %s
Syllables: %s • Complex: %s (%.1f%%)
Avg sentence length: %.1f words",
        indices$flesch_kincaid_grade,
        get_symbol("flesch_kincaid", indices$flesch_kincaid_grade),
        get_interp_text("flesch_kincaid", indices$flesch_kincaid_grade),
        indices$flesch_reading_ease,
        get_symbol("flesch_ease", indices$flesch_reading_ease),
        get_interp_text("flesch_ease", indices$flesch_reading_ease),
        indices$automated_readability_index,
        get_symbol("ari", indices$automated_readability_index),
        get_interp_text("ari", indices$automated_readability_index),
        indices$gunning_fog_index,
        get_symbol("gunning", indices$gunning_fog_index),
        get_interp_text("gunning", indices$gunning_fog_index),
        format(indices$n_sentences, big.mark = ","),
        format(indices$n_words, big.mark = ","),
        format(indices$n_syllables, big.mark = ","),
        format(indices$n_complex_words, big.mark = ","),
        indices$pct_complex_words,
        indices$avg_sentence_length
      )

      tags$pre(
        style = "background-color: #f8f9fa; padding: 15px; border-radius: 6px; border: 1px solid #dee2e6; font-family: 'Courier New', monospace; font-size: 13px; line-height: 1.5; color: #333; margin: 0; min-height: 280px;",
        text_content
      )
    } else {
      tags$pre(
        style = "background-color: #f8f9fa; padding: 40px; border-radius: 6px; border: 1px solid #dee2e6; text-align: center; color: #999; margin: 0; min-height: 280px; display: flex; align-items: center; justify-content: center;",
        "No readability data available"
      )
    }
  })

  # Citation types table
  output$citation_types_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$citation_metrics$type_distribution)
      ) {
        values$analysis_results$citation_metrics$type_distribution
      } else {
        data.frame(
          citation_type = character(0),
          n = numeric(0),
          percentage = numeric(0)
        )
      }
    }),
    title = "Citation Types",
    nrow = 10,
    filter = "none",
    round = 2
  ) })

  # Citation sections table
  output$citation_sections_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(
            values$analysis_results$citation_metrics$section_distribution
          )
      ) {
        values$analysis_results$citation_metrics$section_distribution %>%
          filter(n > 0)
      } else {
        data.frame(
          section = character(0),
          n = numeric(0),
          percentage = numeric(0)
        )
      }
    }),
    title = "Citation Sections",
    nrow = 10,
    filter = "none",
    round = 2
  ) })

  # Frequent words table
  output$frequent_words_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (!is.null(values$analysis_results)) {
        values$analysis_results$word_frequencies %>%
          slice_head(n = 15) %>%
          select(word, n)
      } else {
        data.frame(word = character(0), n = numeric(0))
      }
    }),
    title = "Frequent Words",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Bigrams table
  output$bigrams_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          "2gram" %in% names(values$analysis_results$ngrams)
      ) {
        values$analysis_results$ngrams$`2gram` %>%
          select(ngram, n) %>%
          slice_head(n = 15)
      } else {
        data.frame(ngram = character(0), n = numeric(0))
      }
    }),
    title = "Bigrams (2-grams)",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Trigrams table
  output$trigrams_table <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          "3gram" %in% names(values$analysis_results$ngrams)
      ) {
        values$analysis_results$ngrams$`3gram` %>%
          select(ngram, n) %>%
          slice_head(n = 15)
      } else {
        data.frame(ngram = character(0), n = numeric(0))
      }
    }),
    title = "Trigrams (3-grams)",
    nrow = 15,
    filter = "none",
    round = 0
  ) })

  # Text statistics
  # output$text_stats <- renderText({
  #   if (!is.null(values$analysis_results)) {
  #     stats <- values$analysis_results$text_analytics$basic_stats
  #     paste(
  #       "Characters:", format(stats$total_characters, big.mark = ","), "\n",
  #       "Words:", format(stats$total_words, big.mark = ","), "\n",
  #       "Sentences:", format(stats$total_sentences, big.mark = ","), "\n",
  #       "Avg words/sentence:", round(stats$avg_words_per_sentence, 1), "\n",
  #       "Lexical diversity:", round(values$analysis_results$summary$lexical_diversity, 3)
  #     )
  #   } else {
  #     "No analysis data available"
  #   }
  # })

  # ===========================================
  # TAB 2: IN-CONTEXT CITATION ANALYSIS
  # ===========================================

  # ===========================================
  # UPDATE CITATION GROUPING (without re-running full analysis)
  # ===========================================

  observeEvent(input$update_citation_grouping, {
    req(values$pdf_text)
    req(values$analysis_results)

    tryCatch(
      {
        # Determine citation segmentation
        use_sections_cit <- switch(
          input$citation_segmentation %||% "auto",
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE,
          "auto"
        )

        n_segs_cit <- input$n_segments_citations %||% 10

        # Get citations without section column
        citations_for_mapping <- values$analysis_results$citations %>%
          select(-section, -segment_type)

        # Re-map citations to new segments/sections
        citations_remapped <- map_citations_to_segments(
          citations_df = citations_for_mapping,
          text = values$pdf_sections,
          use_sections = use_sections_cit,
          n_segments = n_segs_cit
        )

        # Update the citations in analysis_results
        values$analysis_results$citations <- citations_remapped %>%
          rename(section = segment)

        # Update citation contexts with new section info
        if (!is.null(values$analysis_results$citation_contexts)) {
          # Remove old section column if exists
          if ("section" %in% names(values$analysis_results$citation_contexts)) {
            values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
              select(-section)
          }

          # Add new section mapping
          values$analysis_results$citation_contexts <- values$analysis_results$citation_contexts %>%
            left_join(
              values$analysis_results$citations %>%
                select(citation_id, section),
              by = "citation_id"
            )
          section_colors <- colorlist()[
            1:length(unique(values$analysis_results$citation_contexts$section))
          ]
          names(section_colors) <- unique(
            values$analysis_results$citation_contexts$section
          )
          values$analysis_results$section_colors <- section_colors
        }

        # Update citation metrics - section distribution
        if (!is.null(values$analysis_results$citation_metrics)) {
          # Determine expected sections/segments
          sections_to_use <- NULL

          if (
            use_sections_cit == TRUE ||
              (use_sections_cit == "auto" &&
                length(setdiff(
                  names(values$pdf_sections),
                  c("Full_text", "References")
                )) >
                  0)
          ) {
            # Using sections
            sections_to_use <- setdiff(
              names(values$pdf_sections),
              c("Full_text", "References")
            )
          } else {
            # Using segments
            sections_to_use <- paste0("Segment ", 1:n_segs_cit)
          }

          # Recalculate section distribution
          if (!is.null(sections_to_use)) {
            values$analysis_results$citation_metrics$section_distribution <-
              values$analysis_results$citations %>%
              mutate(section = factor(section, levels = sections_to_use)) %>%
              count(section, sort = FALSE, .drop = FALSE) %>%
              mutate(percentage = round(n / sum(n) * 100, 2))
          } else {
            values$analysis_results$citation_metrics$section_distribution <-
              values$analysis_results$citations %>%
              count(section, sort = TRUE) %>%
              mutate(percentage = round(n / sum(n) * 100, 2))
          }
        }

        # Recreate network with new grouping
        if (
          !is.null(values$analysis_results$network_data) &&
            nrow(values$analysis_results$network_data) > 0
        ) {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        }

        showNotification(
          "Citation grouping updated successfully!",
          type = "message",
          duration = 2
        )
      },
      error = function(e) {
        showNotification(
          paste("Error updating citation grouping:", e$message),
          type = "error",
          duration = 5
        )

        # Print error to console for debugging
        cat("Error in update_citation_grouping:\n")
        print(e)
      }
    )
  })

  # Custom HTML output for citation contexts
  output$citation_contexts_html <- renderUI({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$citation_contexts) &&
        nrow(values$analysis_results$citation_contexts) > 0
    ) {
      contexts <- values$analysis_results$citation_contexts

      # Apply filters (REMOVED context_type_filter)
      if (!is.null(input$context_search) && nzchar(input$context_search)) {
        contexts <- contexts %>%
          filter(
            str_detect(
              citation_text,
              regex(input$context_search, ignore_case = TRUE)
            ) |
              str_detect(
                full_context,
                regex(input$context_search, ignore_case = TRUE)
              )
          )
      }

      # REMOVED: type filter code

      if (!is.null(input$context_min_words)) {
        contexts <- contexts %>%
          filter(context_word_count >= input$context_min_words)
      }

      # Create HTML for each citation context
      if (nrow(contexts) > 0) {
        citation_boxes <- lapply(1:nrow(contexts), function(i) {
          context <- contexts[i, ]

          section_colors <- values$analysis_results$section_colors

          section_name <- if (
            !is.null(context$section) && !is.na(context$section)
          ) {
            context$section
          } else {
            "Unknown"
          }

          box_color <- section_colors[section_name]
          if (is.na(box_color)) {
            box_color <- "#CCCCCC"
          }

          has_reference <- FALSE
          tooltip_text <- "No reference matched for this citation"

          if ("ref_full_text" %in% names(context)) {
            ref_value <- context[["ref_full_text"]]

            if (
              length(ref_value) > 0 &&
                !is.na(ref_value) &&
                is.character(ref_value) &&
                nzchar(trimws(ref_value))
            ) {
              has_reference <- TRUE
              ref_text <- trimws(ref_value)
              ref_text <- gsub("&", "&amp;", ref_text)
              ref_text <- gsub("<", "&lt;", ref_text)
              ref_text <- gsub(">", "&gt;", ref_text)
              ref_text <- gsub('"', "&quot;", ref_text)
              ref_text <- gsub("'", "&#39;", ref_text)
              ref_text <- gsub("\n", "<br/>", ref_text)
              tooltip_text <- paste0(
                "<strong style='color: #4ECDC4;'>Reference:</strong><br/>",
                ref_text
              )
            }
          }

          citation_id <- paste0("citation_", i, "_", sample(1:10000, 1))

          div(
            style = paste0(
              "margin-bottom: 20px; padding: 15px; border-radius: 8px; ",
              "box-shadow: 0 2px 4px rgba(0,0,0,0.1); ",
              "border-left: 4px solid ",
              box_color,
              "; ",
              "background-color: #fafafa;"
            ),

            div(
              style = "margin-bottom: 10px; font-size: 12px; color: #666;",
              span(paste("Citation", i, "•"), style = "font-weight: bold;"),
              span(
                section_name,
                style = paste0(
                  "color: ",
                  box_color,
                  "; font-weight: bold; margin-left: 8px;"
                )
              ),
              span(paste("• Position:", context$citation_position_in_text))
            ),

            div(
              style = "display: flex; align-items: center; font-family: 'Courier New', monospace;",

              div(
                style = "flex: 1; text-align: right; padding-right: 15px; color: #555; font-size: 14px;",
                if (nzchar(context$words_before)) {
                  paste("...", context$words_before)
                } else {
                  "[start of text]"
                }
              ),

              div(
                id = citation_id,
                class = "citation-with-tooltip",
                style = paste0(
                  "background-color: ",
                  box_color,
                  "; color: white; ",
                  "padding: 8px 12px; border-radius: 6px; font-weight: bold; ",
                  "font-size: 14px; white-space: nowrap; max-width: 300px; ",
                  "overflow: hidden; text-overflow: ellipsis; ",
                  "cursor: ",
                  if (has_reference) "pointer" else "default",
                  "; position: relative;"
                ),
                `data-toggle` = if (has_reference) "tooltip" else NULL,
                `data-placement` = "top",
                `data-html` = "true",
                `data-original-title` = tooltip_text,
                context$citation_text
              ),

              div(
                style = "flex: 1; text-align: left; padding-left: 15px; color: #555; font-size: 14px;",
                if (nzchar(context$words_after)) {
                  paste(context$words_after, "...")
                } else {
                  "[end of text]"
                }
              )
            ),

            div(
              style = "margin-top: 10px; font-size: 11px; color: #888;",
              paste("Context words:", context$context_word_count, "•"),
              if (!is.na(context$is_narrative) && context$is_narrative) {
                span(
                  "Narrative citation",
                  style = "color: #e67e22; font-weight: bold;"
                )
              } else {
                span(
                  "Parenthetical citation",
                  style = "color: #3498db; font-weight: bold;"
                )
              },
              if (has_reference) {
                span(
                  " • ",
                  icon("book", style = "color: #27ae60;"),
                  " Reference matched (hover to view)",
                  style = "color: #27ae60; font-weight: bold;"
                )
              } else {
                span(
                  " • ",
                  icon("times-circle", style = "color: #e74c3c;"),
                  " No reference matched",
                  style = "color: #999; font-style: italic;"
                )
              }
            )
          )
        })

        tagList(
          tags$script(HTML(
            "
        $(document).ready(function(){
          $('.citation-with-tooltip').tooltip('dispose');
          
          $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip({
            container: 'body',
            trigger: 'hover focus',
            delay: { show: 200, hide: 100 },
            html: true,
            boundary: 'window'
          });
          
          setTimeout(function() {
            $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip('dispose');
            $('.citation-with-tooltip[data-toggle=\"tooltip\"]').tooltip({
              container: 'body',
              trigger: 'hover focus',
              delay: { show: 200, hide: 100 },
              html: true,
              boundary: 'window'
            });
          }, 800);
        });
      "
          )),

          tags$style(HTML(
            "
        .tooltip {
          font-family: Arial, sans-serif;
          pointer-events: none;
        }
        .tooltip-inner {
          max-width: 500px;
          min-width: 250px;
          text-align: left;
          background-color: rgba(0, 0, 0, 0.95);
          padding: 12px 16px;
          font-size: 13px;
          line-height: 1.6;
          border-radius: 6px;
          box-shadow: 0 4px 12px rgba(0,0,0,0.4);
        }
        .tooltip-inner strong {
          color: #4ECDC4;
          display: block;
          margin-bottom: 6px;
        }
        .tooltip.show {
          opacity: 1;
        }
        .tooltip.top .tooltip-arrow {
          border-top-color: rgba(0, 0, 0, 0.95);
        }
        .citation-with-tooltip[data-toggle]:hover {
          opacity: 0.9;
          transform: scale(1.02);
          transition: all 0.2s ease;
        }
      "
          )),

          citation_boxes
        )
      } else {
        div(
          style = "text-align: center; padding: 40px; color: #999;",
          icon("search", style = "font-size: 24px; margin-bottom: 10px;"),
          h4("No citations match your current filters"),
          p("Try adjusting your search terms or filters.")
        )
      }
    } else {
      div(
        style = "text-align: center; padding: 40px; color: #999;",
        icon("quote-right", style = "font-size: 24px; margin-bottom: 10px;"),
        h4("No citation contexts available"),
        p("Run the analysis first to see in-context citations.")
      )
    }
  })

  # ===========================================
  # TAB 3: NETWORK ANALYSIS
  # ===========================================

  # Network visualization
  output$citation_network <- renderVisNetwork({
    if (!is.null(values$network_plot)) {
      values$network_plot
    } else {
      visNetwork(
        nodes = data.frame(
          id = 1,
          label = "No citations found",
          size = 20,
          color = "#CCCCCC"
        ),
        edges = data.frame(from = numeric(0), to = numeric(0))
      ) %>%
        visOptions(highlightNearest = FALSE) %>%
        visInteraction(dragNodes = FALSE, dragView = FALSE, zoomView = FALSE)
    }
  })

  # Update network layout with new spacing
  observeEvent(input$update_network_layout, {
    req(values$analysis_results)
    if (
      !is.null(values$analysis_results$network_data) &&
        nrow(values$analysis_results$network_data) > 0
    ) {
      tryCatch(
        {
          raw_network <- create_citation_network(
            values$analysis_results,
            max_distance = input$max_distance,
            show_labels = TRUE
          )
          values$network_plot <- enhance_citation_network(
            raw_network,
            spacing = input$network_spacing %||% 5,
            max_nodes_per_section = input$network_max_nodes %||% 10,
            node_size = input$network_node_size %||% 5,
            label_size = input$network_label_size %||% 5,
            repulsion = input$network_repulsion %||% 5,
            edge_max_distance = input$network_edge_distance %||% 1000
          )
        },
        error = function(e) {
          showNotification(
            paste("Error updating layout:", e$message),
            type = "error",
            duration = 4
          )
        }
      )
    }
  })

  # Network information
  output$network_info <- renderText({
    if (!is.null(values$network_plot)) {
      stats <- attr(values$network_plot, "stats")
      if (!is.null(stats)) {
        section_info <- ""
        if (!is.null(stats$section_distribution)) {
          section_info <- paste0(
            "\n\nSection Distribution:\n",
            paste(
              stats$section_distribution$section,
              ": ",
              stats$section_distribution$n,
              collapse = "\n"
            )
          )
        }

        paste(
          "Nodes:",
          stats$n_nodes,
          "\n",
          "Edges:",
          stats$n_edges,
          "\n",
          "Avg. Distance:",
          stats$avg_distance,
          "chars\n",
          "Max Distance Filter:",
          stats$max_distance,
          "chars\n\n",
          "Network Density:",
          round(stats$n_edges / (stats$n_nodes * (stats$n_nodes - 1) / 2), 3),
          section_info
        )
      } else {
        "Network statistics not available"
      }
    } else {
      "No network data available"
    }
  })

  # Strongest connections table
  output$strongest_connections <- renderUI({ renderBibliobox(
    df = reactive({
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$network_data) &&
          nrow(values$analysis_results$network_data) > 0
      ) {
        values$analysis_results$network_data %>%
          arrange(abs(distance)) %>%
          slice_head(n = 8) %>%
          select(citation1, citation2, distance) %>%
          mutate(
            citation1 = str_trunc(citation1, 20),
            citation2 = str_trunc(citation2, 20),
            distance = paste(abs(distance), "chars")
          )
      } else {
        data.frame(
          citation1 = character(0),
          citation2 = character(0),
          distance = character(0)
        )
      }
    }),
    title = "Strongest Connections",
    nrow = 8,
    filter = "none",
    round = 0
  ) })

  # ===========================================
  # DOWNLOAD HANDLERS
  # ===========================================

  output$download_network <- downloadHandler(
    filename = function() {
      paste0("citation_network_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$network_data)
      ) {
        write.csv(values$analysis_results$network_data, file, row.names = FALSE)
      } else {
        write.csv(
          data.frame(message = "No network data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  output$download_contexts <- downloadHandler(
    filename = function() {
      paste0("citation_contexts_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$citation_contexts)
      ) {
        write.csv(
          values$analysis_results$citation_contexts,
          file,
          row.names = FALSE
        )
      } else {
        write.csv(
          data.frame(message = "No citation contexts available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 4: WORD TRENDS ANALYSIS
  # ===========================================

  # Populate word choices when analysis is complete
  observe({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$word_frequencies) &&
        nrow(values$analysis_results$word_frequencies) > 0
    ) {
      # Get top 30 most frequent words
      top_words <- values$analysis_results$word_frequencies %>%
        slice_head(n = 30) %>%
        pull(word)

      # Get top bigrams if available
      top_bigrams <- character(0)
      if (
        !is.null(values$analysis_results$ngrams) &&
          "2gram" %in% names(values$analysis_results$ngrams)
      ) {
        top_bigrams <- values$analysis_results$ngrams$`2gram` %>%
          slice_head(n = 15) %>%
          pull(ngram)
      }

      # Get top trigrams if available
      top_trigrams <- character(0)
      if (
        !is.null(values$analysis_results$ngrams) &&
          "3gram" %in% names(values$analysis_results$ngrams)
      ) {
        top_trigrams <- values$analysis_results$ngrams$`3gram` %>%
          slice_head(n = 10) %>%
          pull(ngram)
      }

      # Combine all choices with labels
      all_choices <- c(top_words, top_bigrams, top_trigrams)

      # Create named list for optgroups
      choices_list <- list(
        "Top Words (1-gram)" = top_words
      )

      if (length(top_bigrams) > 0) {
        choices_list[["Top Bigrams (2-gram)"]] <- top_bigrams
      }

      if (length(top_trigrams) > 0) {
        choices_list[["Top Trigrams (3-gram)"]] <- top_trigrams
      }

      updateSelectizeInput(
        session,
        "trend_words",
        choices = choices_list,
        selected = NULL,
        server = TRUE
      )
    }
  })

  # Check if sections are available
  output$sections_available <- reactive({
    if (!is.null(values$pdf_sections) && length(values$pdf_sections) > 0) {
      section_names <- setdiff(
        names(values$pdf_sections),
        c("Full_text", "References")
      )
      return(length(section_names) > 0)
    }
    return(FALSE)
  })
  outputOptions(output, "sections_available", suspendWhenHidden = FALSE)

  # Check if trends are available
  output$trends_available <- reactive({
    return(
      !is.null(values$word_trends_data) && nrow(values$word_trends_data) > 0
    )
  })
  outputOptions(output, "trends_available", suspendWhenHidden = FALSE)

  # Update word trends when button is clicked
  observeEvent(input$update_trends, {
    # Validate inputs
    if (is.null(values$pdf_text) || nchar(values$pdf_text) == 0) {
      showNotification(
        "Please extract text from PDF first.",
        type = "warning",
        duration = 3
      )
      return()
    }

    if (is.null(input$trend_words) || length(input$trend_words) == 0) {
      showNotification(
        "Please select at least one word to track.",
        type = "warning",
        duration = 3
      )
      return()
    }

    if (length(input$trend_words) > 10) {
      showNotification(
        "Maximum 10 words allowed. Please deselect some words.",
        type = "warning",
        duration = 3
      )
      return()
    }

    tryCatch(
      {
        # Determine use_sections parameter
        use_sections_param <- switch(
          input$segmentation_type,
          "auto" = "auto",
          "sections" = TRUE,
          "segments" = FALSE
        )

        # Get number of segments
        n_segs <- if (!is.null(input$n_segments)) input$n_segments else 10

        # Determine which text to use
        # If user explicitly chooses segments, use Full_text only
        # Otherwise, use the sections structure
        text_input <- if (input$segmentation_type == "segments") {
          values$pdf_text # Just the full text as string
        } else {
          values$pdf_sections # List with sections
        }

        # Calculate word distribution
        values$word_trends_data <- calculate_word_distribution(
          text = text_input,
          selected_words = input$trend_words,
          use_sections = use_sections_param,
          n_segments = n_segs,
          remove_stopwords = FALSE # Don't remove stopwords for selected words
        )

        # Check if sections were requested but not available
        sections_used <- attr(values$word_trends_data, "use_sections")
        if (input$segmentation_type == "sections" && !sections_used) {
          showNotification(
            "Sections not available. Using equal-length segments instead.",
            type = "warning",
            duration = 4
          )
        }

        showNotification(
          "Word distribution calculated successfully!",
          type = "message",
          duration = 2
        )
      },
      error = function(e) {
        showNotification(
          paste("Error calculating word distribution:", e$message),
          type = "error",
          duration = 5
        )
      }
    )
  })

  # Render word trends plot
  output$word_trends_plot <- renderPlotly({
    req(values$word_trends_data)

    tryCatch(
      {
        plot_word_distribution(
          word_distribution_data = values$word_trends_data,
          plot_type = input$trend_plot_type,
          smooth = TRUE,
          show_points = input$trend_show_points,
          colors = NULL # Use automatic colors
        )
      },
      error = function(e) {
        showNotification(
          paste("Error creating plot:", e$message),
          type = "error",
          duration = 5
        )
        return(NULL)
      }
    )
  })

  # Render word trends statistics table
  output$word_trends_table <- renderUI({ renderBibliobox(
    df = reactive({
      req(values$word_trends_data)

      # Create summary statistics
      stats_table <- values$word_trends_data %>%
        group_by(word) %>%
        summarise(
          `Total Occurrences` = sum(count),
          `Avg Frequency` = paste0(
            round(mean(relative_frequency) * 100, 3),
            "%"
          ),
          `Min Frequency` = paste0(
            round(min(relative_frequency) * 100, 3),
            "%"
          ),
          `Max Frequency` = paste0(
            round(max(relative_frequency) * 100, 3),
            "%"
          ),
          `Std Dev` = round(sd(relative_frequency) * 100, 3),
          `Peak Segment` = segment_name[which.max(relative_frequency)],
          .groups = "drop"
        ) %>%
        arrange(desc(`Total Occurrences`))

      # Apply HTML styling to word column (replaces formatStyle)
      stats_table <- stats_table %>%
        mutate(
          word = paste0(
            '<span style="font-weight: bold; color: #2E86AB;">',
            word,
            '</span>'
          )
        )

      stats_table
    }),
    title = "Word Trends Statistics",
    nrow = 10,
    filter = "none",
    scrollX = TRUE,
    escape = FALSE, # Important: allows HTML in cells
    round = 3
  ) })

  # Download handler for word trends
  output$download_word_trends <- downloadHandler(
    filename = function() {
      paste0("word_trends_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (!is.null(values$word_trends_data)) {
        # Prepare data for export
        export_data <- values$word_trends_data %>%
          select(
            word,
            segment_id,
            segment_name,
            count,
            relative_frequency,
            percentage,
            total_words
          ) %>%
          arrange(word, segment_id)

        write.csv(export_data, file, row.names = FALSE)
      } else {
        write.csv(
          data.frame(message = "No word trends data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 4: CITATION CLUSTERS
  # ===========================================

  # Check if cluster data is available
  output$clusters_available <- reactive({
    return(
      !is.null(values$cluster_description) &&
        !is.null(values$cluster_description$cluster_descriptions) &&
        nrow(values$cluster_description$cluster_descriptions) > 0
    )
  })
  outputOptions(output, "clusters_available", suspendWhenHidden = FALSE)

  # Cluster Summary Table
  output$cluster_summary_table <- renderUI({
    req(values$cluster_description)
    summary_data <- values$cluster_description$cluster_summary
    section_colors <- values$analysis_results$section_colors

    cards <- lapply(seq_len(nrow(summary_data)), function(i) {
      row <- summary_data[i, ]
      sec_color <- if (
        !is.null(section_colors) && row$section %in% names(section_colors)
      ) {
        section_colors[row$section]
      } else {
        "#2E86AB"
      }

      # Split top terms and create styled badges
      terms <- trimws(strsplit(row$top_terms, ",")[[1]])
      term_badges <- lapply(terms, function(term) {
        tags$span(
          term,
          style = paste0(
            "display: inline-block; padding: 3px 8px; margin: 2px;",
            "background-color: ", sec_color, "22;",
            "color: ", sec_color, ";",
            "border: 1px solid ", sec_color, "55;",
            "border-radius: 12px; font-size: 12px;"
          )
        )
      })

      div(
        style = paste0(
          "border-left: 4px solid ", sec_color, ";",
          "padding: 12px 15px; margin-bottom: 12px;",
          "background-color: #fafafa; border-radius: 0 5px 5px 0;"
        ),
        div(
          style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 8px;",
          h5(
            row$section,
            style = paste0(
              "margin: 0; color: ", sec_color, "; font-weight: bold;"
            )
          ),
          tags$span(
            paste0(row$n_references, " references"),
            style = paste0(
              "padding: 2px 10px; border-radius: 10px;",
              "background-color: ", sec_color, ";",
              "color: white; font-size: 12px; font-weight: 500;"
            )
          )
        ),
        div(
          style = "margin-top: 5px;",
          tags$strong("Top terms: ", style = "font-size: 12px; color: #666;"),
          do.call(tagList, term_badges)
        )
      )
    })

    do.call(tagList, cards)
  })

  # TF-IDF Bar Chart - dynamic height container
  # TF-IDF Bar Chart - dynamic height container (2-column grid)
  output$cluster_tfidf_bars_ui <- renderUI({
    req(values$cluster_description)
    descriptions <- values$cluster_description$cluster_descriptions
    n_sections <- length(unique(descriptions$section))
    n_rows <- ceiling(n_sections / 2)
    plot_height <- max(450, n_rows * 10 * 20 + 80)
    plotlyOutput("cluster_tfidf_bars", height = paste0(plot_height, "px"))
  })

  output$cluster_tfidf_bars <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    descriptions <- cluster_desc$cluster_descriptions

    # Preserve section order
    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% unique(descriptions$section)]
    } else {
      all_sections <- unique(descriptions$section)
    }
    n_sections <- length(all_sections)
    n_cols <- 2
    n_rows <- ceiling(n_sections / n_cols)

    # Build subplots for each section
    subplot_list <- list()
    for (i in seq_along(all_sections)) {
      sec <- all_sections[i]
      sec_data <- descriptions %>%
        dplyr::filter(section == sec) %>%
        dplyr::arrange(tf_idf)

      sec_color <- if (
        !is.null(section_colors) && sec %in% names(section_colors)
      ) {
        section_colors[sec]
      } else {
        "#2E86AB"
      }

      ngram_type <- ifelse(sec_data$ngram_size == 1, "unigram", "bigram")
      hover_text <- paste0(
        "<b>Term:</b> ", sec_data$ngram, "<br>",
        "<b>TF-IDF:</b> ", round(sec_data$tf_idf, 4), "<br>",
        "<b>Count:</b> ", sec_data$n, "<br>",
        "<b>Type:</b> ", ngram_type
      )

      p <- plotly::plot_ly(
        data = sec_data,
        y = ~factor(ngram, levels = ngram),
        x = ~tf_idf,
        type = "bar",
        orientation = "h",
        marker = list(color = sec_color),
        text = hover_text,
        hovertemplate = "%{text}<extra></extra>",
        name = sec,
        showlegend = FALSE
      ) %>%
        plotly::layout(
          yaxis = list(
            title = "",
            tickfont = list(size = 10),
            categoryorder = "array",
            categoryarray = sec_data$ngram
          )
        )

      subplot_list[[i]] <- p
    }

    # Pad with empty plots if odd number of sections
    if (n_sections %% n_cols != 0) {
      empty_p <- plotly::plot_ly(type = "bar") %>%
        plotly::layout(
          xaxis = list(visible = FALSE),
          yaxis = list(visible = FALSE)
        )
      for (k in seq_len(n_cols - (n_sections %% n_cols))) {
        subplot_list[[length(subplot_list) + 1]] <- empty_p
      }
    }

    bar_plot_height <- max(450, n_rows * 10 * 20 + 80)

    fig <- plotly::subplot(
      subplot_list,
      nrows = n_rows,
      shareX = FALSE,
      shareY = FALSE,
      titleY = TRUE,
      margin = 0.06
    )
    fig$height <- bar_plot_height

    # Build annotations for section titles
    section_annotations <- list()
    for (i in seq_along(all_sections)) {
      row_idx <- ceiling(i / n_cols)
      col_idx <- ((i - 1) %% n_cols) + 1
      x_pos <- (col_idx - 1) / n_cols + 0.5 / n_cols
      y_pos <- 1 - (row_idx - 1) / n_rows + 0.005

      ann_color <- if (
        !is.null(section_colors) && all_sections[i] %in% names(section_colors)
      ) {
        section_colors[all_sections[i]]
      } else {
        "#2E86AB"
      }

      section_annotations[[i]] <- list(
        text = paste0("<b>", all_sections[i], "</b>"),
        xref = "paper", yref = "paper",
        x = x_pos, y = y_pos,
        xanchor = "center", yanchor = "bottom",
        showarrow = FALSE,
        font = list(size = 11, color = ann_color)
      )
    }

    fig %>%
      plotly::layout(
        annotations = section_annotations,
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 120, r = 20, t = 30, b = 40)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "tfidf_bars",
          height = bar_plot_height,
          width = 1200,
          scale = 2
        )
      )
  })

  # TF-IDF Heatmap
  output$cluster_tfidf_heatmap <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    descriptions <- cluster_desc$cluster_descriptions

    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% unique(descriptions$section)]
    } else {
      all_sections <- unique(descriptions$section)
    }
    n_sections <- length(all_sections)

    # Get top terms for heatmap
    top_terms_per_section <- descriptions %>%
      dplyr::group_by(section) %>%
      dplyr::slice_head(n = max(1, floor(20 / n_sections))) %>%
      dplyr::ungroup()

    heatmap_terms <- unique(top_terms_per_section$ngram)
    if (length(heatmap_terms) < 20) {
      remaining <- descriptions %>%
        dplyr::filter(!ngram %in% heatmap_terms) %>%
        dplyr::arrange(dplyr::desc(tf_idf)) %>%
        dplyr::pull(ngram) %>%
        unique()
      heatmap_terms <- unique(c(heatmap_terms, remaining))[
        seq_len(min(20, length(unique(c(heatmap_terms, remaining)))))
      ]
    }

    # Build heatmap matrix
    heatmap_data <- expand.grid(
      ngram = heatmap_terms,
      section = all_sections,
      stringsAsFactors = FALSE
    ) %>%
      dplyr::left_join(
        descriptions %>% dplyr::select(ngram, section, tf_idf),
        by = c("ngram", "section")
      ) %>%
      dplyr::mutate(tf_idf = ifelse(is.na(tf_idf), 0, tf_idf))

    heatmap_matrix <- matrix(0, nrow = length(heatmap_terms), ncol = n_sections)
    rownames(heatmap_matrix) <- heatmap_terms
    colnames(heatmap_matrix) <- all_sections

    for (r in seq_along(heatmap_terms)) {
      for (cc in seq_along(all_sections)) {
        val <- heatmap_data$tf_idf[
          heatmap_data$ngram == heatmap_terms[r] &
            heatmap_data$section == all_sections[cc]
        ]
        if (length(val) > 0) heatmap_matrix[r, cc] <- val[1]
      }
    }

    hover_matrix <- matrix("", nrow = length(heatmap_terms), ncol = n_sections)
    for (r in seq_along(heatmap_terms)) {
      for (cc in seq_along(all_sections)) {
        hover_matrix[r, cc] <- paste0(
          "<b>Term:</b> ", heatmap_terms[r], "<br>",
          "<b>Section:</b> ", all_sections[cc], "<br>",
          "<b>TF-IDF:</b> ", round(heatmap_matrix[r, cc], 4)
        )
      }
    }

    # Truncate long section labels for x-axis display
    trunc_sections <- ifelse(nchar(all_sections) > 30,
                             paste0(substr(all_sections, 1, 27), "..."),
                             all_sections)

    plotly::plot_ly(
      x = trunc_sections,
      y = heatmap_terms,
      z = heatmap_matrix,
      type = "heatmap",
      colorscale = list(c(0, "white"), c(1, "#08519c")),
      text = hover_matrix,
      hovertemplate = "%{text}<extra></extra>",
      colorbar = list(title = "TF-IDF")
    ) %>%
      plotly::layout(
        xaxis = list(
          title = list(
            text = "Section",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickangle = -45,
          tickfont = list(size = 11),
          categoryorder = "array",
          categoryarray = trunc_sections
        ),
        yaxis = list(
          title = list(
            text = "Term",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickfont = list(size = 10),
          autorange = "reversed"
        ),
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 140, r = 40, t = 20, b = 140)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "tfidf_heatmap",
          height = 600,
          width = 1000,
          scale = 2
        )
      )
  })

  # References per Section Bar Chart
  output$cluster_refs_per_section <- renderPlotly({
    req(values$cluster_description)

    cluster_desc <- values$cluster_description
    section_colors <- values$analysis_results$section_colors
    summary_data <- cluster_desc$cluster_summary

    if (!is.null(cluster_desc$section_order)) {
      all_sections <- cluster_desc$section_order
      all_sections <- all_sections[all_sections %in% summary_data$section]
    } else {
      all_sections <- summary_data$section
    }

    ref_counts <- summary_data %>%
      dplyr::mutate(section = factor(section, levels = all_sections)) %>%
      dplyr::arrange(section) %>%
      dplyr::mutate(section = as.character(section))

    bar_colors <- sapply(ref_counts$section, function(s) {
      if (!is.null(section_colors) && s %in% names(section_colors)) {
        section_colors[s]
      } else {
        "#CCCCCC"
      }
    })

    # Truncate long section labels for x-axis display
    trunc_ref_sections <- ifelse(nchar(ref_counts$section) > 30,
                                 paste0(substr(ref_counts$section, 1, 27), "..."),
                                 ref_counts$section)
    trunc_all_sections <- ifelse(nchar(all_sections) > 30,
                                 paste0(substr(all_sections, 1, 27), "..."),
                                 all_sections)

    hover_refs <- paste0(
      "<b>Section:</b> ", ref_counts$section, "<br>",
      "<b>References:</b> ", ref_counts$n_references
    )

    plotly::plot_ly(
      data = ref_counts,
      x = trunc_ref_sections,
      y = ~n_references,
      type = "bar",
      marker = list(color = bar_colors),
      text = ~n_references,
      textposition = "outside",
      textfont = list(size = 12, color = "#333"),
      hovertext = hover_refs,
      hovertemplate = "%{hovertext}<extra></extra>"
    ) %>%
      plotly::layout(
        xaxis = list(
          title = list(
            text = "Section",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          tickangle = -45,
          gridcolor = "#e0e0e0",
          categoryorder = "array",
          categoryarray = trunc_all_sections
        ),
        yaxis = list(
          title = list(
            text = "Number of References",
            font = list(size = 12, family = "Arial, sans-serif", color = "#333")
          ),
          gridcolor = "#e0e0e0"
        ),
        plot_bgcolor = "#fafafa",
        paper_bgcolor = "white",
        margin = list(l = 60, r = 40, t = 20, b = 140)
      ) %>%
      plotly::config(
        displayModeBar = TRUE,
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("select2d", "lasso2d", "autoScale2d"),
        toImageButtonOptions = list(
          format = "png",
          filename = "references_per_section",
          height = 600,
          width = 1000,
          scale = 2
        )
      )
  })

  # Detailed cluster references table
  output$cluster_references_detail <- renderUI({
    req(values$cluster_description)

    cluster_refs <- values$cluster_description$cluster_references
    section_colors <- values$analysis_results$section_colors
    section_order <- values$cluster_description$section_order

    # Order sections
    sections <- if (!is.null(section_order)) {
      section_order[section_order %in% unique(cluster_refs$section)]
    } else {
      unique(cluster_refs$section)
    }

    section_panels <- lapply(sections, function(sec) {
      sec_refs <- cluster_refs %>%
        dplyr::filter(section == sec) %>%
        dplyr::distinct(ref_full_text) %>%
        dplyr::arrange(ref_full_text)

      sec_color <- if (
        !is.null(section_colors) && sec %in% names(section_colors)
      ) {
        section_colors[sec]
      } else {
        "#2E86AB"
      }

      ref_items <- lapply(seq_len(nrow(sec_refs)), function(j) {
        div(
          style = "padding: 6px 10px; border-bottom: 1px solid #eee; font-size: 13px; color: #444;",
          tags$span(
            paste0(j, "."),
            style = "font-weight: bold; color: #999; margin-right: 8px;"
          ),
          sec_refs$ref_full_text[j]
        )
      })

      div(
        style = "margin-bottom: 15px;",
        div(
          style = paste0(
            "padding: 8px 15px; border-left: 4px solid ", sec_color, ";",
            "background-color: ", sec_color, "11; cursor: pointer;"
          ),
          onclick = paste0(
            "$(this).next().slideToggle(200);",
            "$(this).find('.toggle-icon').toggleClass('fa-chevron-down fa-chevron-right');"
          ),
          tags$span(
            class = "toggle-icon fa fa-chevron-down",
            style = paste0("color: ", sec_color, "; margin-right: 8px;")
          ),
          tags$strong(sec, style = paste0("color: ", sec_color, ";")),
          tags$span(
            paste0(" (", nrow(sec_refs), " references)"),
            style = "color: #888; font-size: 12px;"
          )
        ),
        div(
          style = "border: 1px solid #eee; border-top: none;",
          do.call(tagList, ref_items)
        )
      )
    })

    do.call(tagList, section_panels)
  })

  # Download cluster data
  output$download_clusters <- downloadHandler(
    filename = function() {
      paste0("citation_clusters_", Sys.Date(), ".csv")
    },
    content = function(file) {
      if (!is.null(values$cluster_description)) {
        write.csv(
          values$cluster_description$cluster_descriptions,
          file,
          row.names = FALSE
        )
      } else {
        write.csv(
          data.frame(message = "No cluster data available"),
          file,
          row.names = FALSE
        )
      }
    }
  )

  # ===========================================
  # TAB 5: REFERENCES
  # ===========================================

  # Check if references are available
  output$references_available <- reactive({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      return(nrow(values$analysis_results$parsed_references) > 0)
    }
    return(FALSE)
  })
  outputOptions(output, "references_available", suspendWhenHidden = FALSE)

  # # Total references count
  output$total_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      format(nrow(values$analysis_results$parsed_references), big.mark = ",")
    } else {
      "0"
    }
  })

  # # PDF references count
  output$pdf_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      refs <- values$analysis_results$parsed_references
      pdf_refs <- sum(tolower(refs$ref_source) == "pdf", na.rm = TRUE)
      format(pdf_refs, big.mark = ",")
    } else {
      "0"
    }
  })

  # # Crossref references count
  output$crossref_refs <- renderText({
    if (
      !is.null(values$analysis_results) &&
        !is.null(values$analysis_results$parsed_references)
    ) {
      refs <- values$analysis_results$parsed_references
      crossref_refs <- sum(tolower(refs$ref_source) == "crossref", na.rm = TRUE)
      format(crossref_refs, big.mark = ",")
    } else {
      "0"
    }
  })

  # Openalex references count
  output$openalex_refs <- renderText({
    req(values$references_oa)

    if (is.null(values$references_oa) || nrow(values$references_oa) == 0) {
      return("0")
    }

    nrow(values$references_oa)
  })

  # References HTML display
  output$references_html <- renderUI({
    req(values$analysis_results)
    req(values$analysis_results$parsed_references)

    refs <- values$analysis_results$parsed_references

    # Apply search filter
    if (!is.null(input$reference_search) && nzchar(input$reference_search)) {
      search_term <- tolower(input$reference_search)

      refs <- refs %>%
        filter(
          if_else(
            !is.na(ref_full_text),
            str_detect(tolower(ref_full_text), fixed(search_term)),
            FALSE
          ) |
            if_else(
              !is.na(ref_authors),
              str_detect(tolower(ref_authors), fixed(search_term)),
              FALSE
            ) |
            if_else(
              !is.na(ref_year),
              str_detect(tolower(ref_year), fixed(search_term)),
              FALSE
            ) |
            if_else(
              !is.na(doi),
              str_detect(tolower(doi), fixed(search_term)),
              FALSE
            )
        )
    }

    if (nrow(refs) == 0) {
      return(
        div(
          style = "text-align: center; padding: 40px; color: #999;",
          icon("search", style = "font-size: 24px; margin-bottom: 10px;"),
          h4("No references match your search"),
          p("Try different search terms.")
        )
      )
    }

    # Create HTML for each reference
    reference_items <- lapply(1:nrow(refs), function(i) {
      ref <- refs[i, ]
      # Get reference text
      ref_text <- if (!is.na(ref$ref_full_text) && nzchar(ref$ref_full_text)) {
        ref$ref_full_text
      } else {
        "[Reference text not available]"
      }

      # Check if OpenAlex data is available for this reference
      has_oa_data <- FALSE
      oa_button <- NULL

      if (
        !is.null(values$references_oa) && !is.na(ref$doi) && nzchar(ref$doi)
      ) {
        # Cerca corrispondenza nel dataset OpenAlex
        ref_doi_clean <- tolower(trimws(ref$doi))

        # Prova diverse varianti del DOI
        oa_match_idx <- which(
          tolower(gsub("https://doi.org/", "", values$references_oa$doi)) ==
            ref_doi_clean |
            tolower(values$references_oa$doi) ==
              paste0("https://doi.org/", ref_doi_clean)
        )

        if (length(oa_match_idx) > 0) {
          has_oa_data <- TRUE

          oa_button <- tags$button(
            class = "btn btn-info btn-xs",
            style = "float: right; margin-left: 10px;",
            onclick = sprintf("showOADetails(%d)", i),
            icon("info-circle"),
            " View Details"
          )
        }
      }

      # Determine source badge
      source_badge <- if (
        !is.na(ref$ref_source) && tolower(ref$ref_source) == "crossref"
      ) {
        tags$span(
          class = "label label-success",
          style = "font-size: 11px; margin-left: 10px;",
          icon("cloud-download-alt"),
          " Crossref"
        )
      } else {
        tags$span(
          class = "label label-info",
          style = "font-size: 11px; margin-left: 10px;",
          icon("file-pdf"),
          " PDF"
        )
      }

      # OpenAlex badge if available
      oa_badge <- if (has_oa_data) {
        tags$span(
          class = "label label-primary",
          style = "font-size: 11px; margin-left: 5px;",
          icon("database"),
          " OpenAlex"
        )
      }

      # DOI link if available
      doi_link <- NULL
      if ("doi" %in% names(ref) && !is.na(ref$doi) && nzchar(ref$doi)) {
        doi_link <- tags$div(
          style = "margin-top: 8px; font-size: 12px;",
          icon("link", style = "color: #3498db;"),
          tags$a(
            href = paste0("https://doi.org/", ref$doi),
            target = "_blank",
            ref$doi,
            style = "color: #3498db; margin-left: 5px;"
          )
        )
      }

      # Author and year display
      author_year_info <- NULL
      if (!is.na(ref$ref_first_author) && !is.na(ref$ref_year)) {
        n_auth <- if (!is.na(ref$n_authors) && ref$n_authors > 1) {
          paste0(" et al. (", ref$n_authors, " authors)")
        } else {
          ""
        }

        author_year_info <- tags$div(
          style = "margin-top: 8px; font-size: 12px; color: #666;",
          icon("user", style = "color: #3498db;"),
          tags$span(
            style = "margin-left: 5px;",
            paste0(ref$ref_first_author, n_auth, " • ", ref$ref_year)
          )
        )
      }

      # Citation count if available
      citation_info <- NULL
      if (
        "citation_count" %in%
          names(ref) &&
          !is.na(ref$citation_count) &&
          ref$citation_count > 0
      ) {
        citation_info <- tags$span(
          style = "margin-left: 15px; color: #e67e22; font-size: 12px;",
          icon("quote-right"),
          sprintf(
            " Cited %d time%s in text",
            ref$citation_count,
            ifelse(ref$citation_count > 1, "s", "")
          )
        )
      }

      div(
        class = "reference-item",
        `data-ref-index` = i,
        style = "padding: 15px; margin-bottom: 15px; border-left: 4px solid #2E86AB; background-color: #fafafa; border-radius: 5px; box-shadow: 0 1px 3px rgba(0,0,0,0.1); position: relative;",

        # Reference number, badges and OA button
        div(
          style = "margin-bottom: 10px;",
          tags$span(
            style = "font-weight: bold; color: #2E86AB; font-size: 16px;",
            paste0("[", i, "]")
          ),
          source_badge,
          oa_badge,
          citation_info,
          oa_button
        ),

        # Full reference text
        div(
          style = "font-family: 'Georgia', serif; font-size: 14px; line-height: 1.6; color: #333; clear: both;",
          ref_text
        ),

        # Author and year info
        author_year_info,

        # DOI link
        doi_link
      )
    })

    tagList(
      tags$style(HTML(
        "
      .reference-item:hover {
        background-color: #f0f8ff !important;
        transition: background-color 0.2s ease;
      }
      .oa-document-summary .info-box {
        display: flex;
        align-items: center;
        border-radius: 4px;
        padding: 10px;
        color: white;
      }
      .oa-document-summary .info-box-icon {
        font-size: 32px;
        flex: 0 0 70px;
        text-align: center;
      }
      .oa-document-summary .info-box-content {
        flex: 1;
      }
      .oa-document-summary .info-box-text {
        display: block;
        font-size: 12px;
        text-transform: uppercase;
        opacity: 0.9;
      }
      .oa-document-summary .info-box-number {
        display: block;
        font-size: 20px;
        font-weight: bold;
        margin-top: 5px;
      }
    "
      )),
      reference_items
    )
  })

  # Download handler for references
  output$download_references <- downloadHandler(
    filename = function() {
      paste0("references_", Sys.Date(), ".txt")
    },
    content = function(file) {
      if (
        !is.null(values$analysis_results) &&
          !is.null(values$analysis_results$parsed_references)
      ) {
        refs <- values$analysis_results$parsed_references

        # Create formatted text
        ref_text <- paste0(
          "BIBLIOGRAPHY\n",
          "============\n",
          "Generated: ",
          Sys.time(),
          "\n",
          "Total references: ",
          nrow(refs),
          "\n",
          "From PDF: ",
          sum(tolower(refs$ref_source) == "pdf", na.rm = TRUE),
          "\n",
          "From Crossref: ",
          sum(tolower(refs$ref_source) == "crossref", na.rm = TRUE),
          "\n\n"
        )

        for (i in 1:nrow(refs)) {
          ref <- refs[i, ]

          source_label <- if (
            !is.na(ref$ref_source) && tolower(ref$ref_source) == "crossref"
          ) {
            "[Crossref]"
          } else {
            "[PDF]"
          }

          ref_text <- paste0(
            ref_text,
            "[",
            i,
            "] ",
            source_label,
            "\n",
            ref$ref_full_text,
            "\n"
          )

          if (!is.na(ref$ref_first_author) && !is.na(ref$ref_year)) {
            n_auth <- if (!is.na(ref$n_authors) && ref$n_authors > 1) {
              paste0(" (", ref$n_authors, " authors)")
            } else {
              ""
            }
            ref_text <- paste0(
              ref_text,
              "Author: ",
              ref$ref_first_author,
              n_auth,
              " • Year: ",
              ref$ref_year,
              "\n"
            )
          }

          if ("doi" %in% names(ref) && !is.na(ref$doi) && nzchar(ref$doi)) {
            ref_text <- paste0(ref_text, "DOI: ", ref$doi, "\n")
          }

          if ("citation_count" %in% names(ref) && !is.na(ref$citation_count)) {
            ref_text <- paste0(
              ref_text,
              "Cited ",
              ref$citation_count,
              " times in text\n"
            )
          }

          ref_text <- paste0(ref_text, "\n")
        }

        writeLines(ref_text, file)
      } else {
        writeLines("No references available", file)
      }
    }
  )

  # ===========================================
  # TAB: BIBLIOAI SUMMARY - SERVER LOGIC
  # ===========================================

  # ===========================================
  # BIBLIOAI SUMMARY INITIALIZATION
  # ===========================================

  # Initialize API key from environment on startup (run once)
  observeEvent(TRUE, {
    api_key_env <- Sys.getenv("GEMINI_API_KEY")
    if (nzchar(api_key_env) && is.null(values$gemini_api_key)) {
      values$gemini_api_key <- api_key_env
    }
  }, once = TRUE)

  # ===========================================
  # API STATUS DISPLAY
  # ===========================================

  output$gemini_api_status <- renderUI({
    api_key <- values$gemini_api_key

    if (is.null(api_key) || !nzchar(api_key)) {
      tags$span(
        class = "label label-danger",
        style = "font-size: 12px; padding: 6px 12px;",
        icon("exclamation-circle"),
        " API Key Not Configured"
      )
    } else {
      tags$span(
        class = "label label-success",
        style = "font-size: 12px; padding: 6px 12px;",
        icon("check-circle"),
        " API Key Configured"
      )
    }
  })

  # ===========================================
  # SUMMARY TYPE DESCRIPTION
  # ===========================================

  output$summary_type_description <- renderUI({
    req(input$summary_type)

    descriptions <- list(
      short_abstract = "A concise, one-paragraph abstract (around 250 words) covering objectives, methods, findings, and conclusions.",
      narrative_abstract = "An engaging narrative (500-600 words) that tells the story of the research from problem to implications.",
      imrad_summary = "A structured summary following the IMRaD format: Introduction, Methods, Results, and Discussion.",
      thematic_bibliography = "Groups the references by research themes and provides descriptions of each thematic area.",
      research_questions = "Extracts research questions, context, and motivation behind the study.",
      background_literature = "Summarizes the theoretical framework and previous research that grounds this study.",
      methods_summary = "Detailed breakdown of research design, data collection, analysis techniques, and tools used.",
      implications = "Focuses on theoretical contributions and practical applications derived from the findings.",
      list_tables_figures = "Complete list of all tables and figures with their captions as they appear in the document."
    )

    description <- descriptions[[input$summary_type]]

    div(
      style = "background-color: #f0f8ff; padding: 12px; border-radius: 4px; border-left: 3px solid #3498db;",
      tags$small(
        icon("info-circle", style = "color: #3498db; margin-right: 5px;"),
        description,
        style = "color: #555; line-height: 1.6;"
      )
    )
  })

  # ===========================================
  # GENERATE SUMMARY
  # ===========================================

  observeEvent(input$generate_summary, {
    # Validate PDF file
    if (is.null(input$pdf_file)) {
      showNotification(
        "Please upload a PDF file first.",
        type = "warning",
        duration = 4
      )
      return()
    }

    # Validate API key
    api_key <- Sys.getenv("GEMINI_API_KEY")
    if (!nzchar(api_key)) {
      showNotification(
        HTML(
          "<strong>API Key Required</strong><br/>
           Please configure your Gemini API key in Biblioshiny settings.<br/>
           You can get a free API key at: <a href='https://aistudio.google.com/apikey' target='_blank'>Google AI Studio</a>"
        ),
        type = "error",
        duration = 8
      )
      return()
    }

    # Get model from Biblioshiny settings or use default
    model <- values$gemini_api_model
    if (is.null(model) || !nzchar(model)) {
      model <- "2.5-flash-lite"
    }

    # Use "huge" as default output size
    output_size <- "huge"

    tryCatch(
      {
        # Generate prompt
        prompt <- get_article_summary_prompt(input$summary_type)

        # Show progress notification
        progress_id <- showNotification(
          "Sending request to Gemini AI...",
          type = "message",
          duration = NULL,
          closeButton = FALSE
        )

        # Call Gemini AI
        result <- gemini_ai(
          docs = input$pdf_file$datapath,
          prompt = prompt,
          model = model,
          api_key = api_key,
          outputSize = output_size,
          retry_503 = 5
        )

        # Remove progress notification
        removeNotification(progress_id)

        # Check if result is an error message
        if (is.character(result) && length(result) == 1) {
          if (
            grepl("^❌", result) || grepl("Error", result, ignore.case = TRUE)
          ) {
            showNotification(
              HTML(paste0(
                "<strong>AI Request Failed</strong><br/>",
                gsub("❌ ", "", result)
              )),
              type = "error",
              duration = 10
            )
            return()
          }
        }

        # Store results
        values$biblioai_summary <- result
        values$biblioai_summary_type <- input$summary_type
        values$biblioai_summary_timestamp <- Sys.time()

        # Get summary type label
        summary_labels <- list(
          short_abstract = "Short Abstract",
          narrative_abstract = "Narrative Abstract",
          imrad_summary = "IMRaD Summary",
          thematic_bibliography = "Thematic Bibliography",
          research_questions = "Research Questions & Context",
          background_literature = "Background & Literature",
          methods_summary = "Methods Summary",
          implications = "Implications & Conclusions",
          list_tables_figures = "Tables & Figures List"
        )

        summary_label <- summary_labels[[input$summary_type]]

        showNotification(
          HTML(paste0(
            "<strong>Summary Generated!</strong><br/>",
            "Type: ",
            summary_label
          )),
          type = "message",
          duration = 5
        )
      },
      error = function(e) {
        removeNotification(progress_id)

        showNotification(
          HTML(paste0(
            "<strong>Error Generating Summary</strong><br/>",
            e$message
          )),
          type = "error",
          duration = 8
        )

        cat("BiblioAI Summary Error:\n")
        print(e)
      }
    )
  })

  # ===========================================
  # SUMMARY AVAILABILITY
  # ===========================================

  output$summary_available <- reactive({
    return(!is.null(values$biblioai_summary))
  })
  outputOptions(output, "summary_available", suspendWhenHidden = FALSE)

  # ===========================================
  # SUMMARY DISPLAY
  # ===========================================

  output$summary_type_display <- renderText({
    req(values$biblioai_summary_type)

    summary_labels <- list(
      short_abstract = "Short Abstract (250 words)",
      narrative_abstract = "Narrative Abstract (500-600 words)",
      imrad_summary = "IMRaD Structure Summary",
      thematic_bibliography = "Thematic Bibliography",
      research_questions = "Research Questions & Context",
      background_literature = "Background & Literature Review",
      methods_summary = "Methodology Summary",
      implications = "Implications & Conclusions",
      list_tables_figures = "Tables & Figures List"
    )

    summary_labels[[values$biblioai_summary_type]]
  })

  output$summary_timestamp <- renderText({
    req(values$biblioai_summary_timestamp)
    format(values$biblioai_summary_timestamp, "%Y-%m-%d %H:%M:%S")
  })

  output$summary_content_display <- renderUI({
    req(values$biblioai_summary)

    # Combine multiple text elements if result is a vector
    summary_text <- if (
      is.character(values$biblioai_summary) &&
        length(values$biblioai_summary) > 1
    ) {
      paste(values$biblioai_summary, collapse = "\n\n")
    } else {
      values$biblioai_summary
    }

    # Convert markdown-style headers to HTML for better display
    summary_html <- summary_text

    # Convert **text** to bold
    summary_html <- gsub(
      "\\*\\*([^*]+)\\*\\*",
      "<strong>\\1</strong>",
      summary_html
    )

    # Convert *text* to italic
    summary_html <- gsub("\\*([^*]+)\\*", "<em>\\1</em>", summary_html)

    # Convert ### headers to h4
    summary_html <- gsub(
      "^### (.+)$",
      "<h4 style='color: #2E86AB; margin-top: 20px; margin-bottom: 10px;'>\\1</h4>",
      summary_html,
      perl = TRUE
    )

    # Convert ## headers to h3
    summary_html <- gsub(
      "^## (.+)$",
      "<h3 style='color: #2E86AB; margin-top: 25px; margin-bottom: 10px;'>\\1</h3>",
      summary_html,
      perl = TRUE
    )

    # Convert # headers to h2
    summary_html <- gsub(
      "^# (.+)$",
      "<h2 style='color: #2E86AB; margin-top: 30px; margin-bottom: 15px;'>\\1</h2>",
      summary_html,
      perl = TRUE
    )

    # Convert bullet points
    summary_html <- gsub("^- (.+)$", "<li>\\1</li>", summary_html, perl = TRUE)
    summary_html <- gsub(
      "^\\* (.+)$",
      "<li>\\1</li>",
      summary_html,
      perl = TRUE
    )

    # Wrap consecutive <li> in <ul>
    summary_html <- gsub(
      "(<li>.*?</li>)\n(<li>)",
      "\\1\\2",
      summary_html,
      perl = TRUE
    )
    summary_html <- gsub(
      "(<li>.*?</li>)",
      "<ul style='margin-left: 20px;'>\\1</ul>",
      summary_html,
      perl = TRUE
    )

    # Convert line breaks to <br> but not inside lists
    summary_html <- gsub("\n(?!<)", "<br/>", summary_html, perl = TRUE)

    HTML(summary_html)
  })

  # ===========================================
  # DOWNLOAD SUMMARY
  # ===========================================

  output$download_summary <- downloadHandler(
    filename = function() {
      summary_type <- values$biblioai_summary_type
      if (is.null(summary_type)) {
        summary_type <- "summary"
      }
      paste0("biblioai_", summary_type, "_", Sys.Date(), ".txt")
    },
    content = function(file) {
      if (!is.null(values$biblioai_summary)) {
        # Create header
        header <- paste0(
          "====================================\n",
          "BiblioAI Summary\n",
          "====================================\n\n",
          "Summary Type: ",
          values$biblioai_summary_type,
          "\n",
          "Generated: ",
          format(values$biblioai_summary_timestamp, "%Y-%m-%d %H:%M:%S"),
          "\n",
          "Document: ",
          input$pdf_file$name,
          "\n\n",
          "====================================\n\n"
        )

        # Combine multiple text elements if result is a vector
        summary_text <- if (
          is.character(values$biblioai_summary) &&
            length(values$biblioai_summary) > 1
        ) {
          paste(values$biblioai_summary, collapse = "\n\n")
        } else {
          values$biblioai_summary
        }

        # Write to file
        writeLines(paste0(header, summary_text), file, useBytes = TRUE)
      } else {
        writeLines("No summary available", file)
      }
    }
  )

  # ===========================================
  # RESET SUMMARY ON NEW PDF OR RESET
  # ===========================================

  # Clear summary when new PDF is uploaded
  observeEvent(input$pdf_file, {
    values$biblioai_summary <- NULL
    values$biblioai_summary_type <- NULL
    values$biblioai_summary_timestamp <- NULL
  })

  # Clear summary on reset
  observeEvent(input$reset_analysis, {
    values$pdf_metadata <- NULL
    values$biblioai_summary <- NULL
    values$biblioai_summary_type <- NULL
    values$biblioai_summary_timestamp <- NULL

    # Reset UI inputs
    updateSelectInput(session, "summary_type", selected = "short_abstract")
  })

  # Clear metadata when new PDF is uploaded
  observeEvent(input$pdf_file, {
    values$pdf_metadata <- NULL
  })

  # ===========================================
  # OPENALEX MODAL HANDLER
  # ===========================================

  observeEvent(input$selected_oa_ref_index, {
    req(values$analysis_results$parsed_references)
    req(values$references_oa)
    req(input$selected_oa_ref_index)

    tryCatch(
      {
        ref_index <- input$selected_oa_ref_index
        ref <- values$analysis_results$parsed_references[ref_index, ]

        if (is.na(ref$doi) || !nzchar(ref$doi)) {
          session$sendCustomMessage(
            "updateOAModal",
            list(
              title = as.character(span(
                "Reference Details",
                style = "font-weight: 600; font-size: 20px;"
              )),
              content = as.character(div(
                style = "text-align: center; padding: 40px; color: #999;",
                icon(
                  "exclamation-triangle",
                  style = "font-size: 32px; margin-bottom: 15px; color: #f39c12;"
                ),
                h4("No DOI available"),
                p("Cannot fetch OpenAlex data without a DOI.")
              ))
            )
          )
          return()
        }

        # Trova i dati OpenAlex corrispondenti
        ref_doi_clean <- tolower(trimws(ref$doi))

        oa_match_idx <- which(
          tolower(gsub("https://doi.org/", "", values$references_oa$doi)) ==
            ref_doi_clean |
            tolower(values$references_oa$doi) ==
              paste0("https://doi.org/", ref_doi_clean)
        )

        if (length(oa_match_idx) == 0) {
          session$sendCustomMessage(
            "updateOAModal",
            list(
              title = as.character(span(
                "Reference Details",
                style = "font-weight: 600; font-size: 20px;"
              )),
              content = as.character(div(
                style = "text-align: center; padding: 40px; color: #999;",
                icon(
                  "database",
                  style = "font-size: 32px; margin-bottom: 15px; color: #e74c3c;"
                ),
                h4("OpenAlex data not found"),
                p(paste("Could not find OpenAlex data for DOI:", ref$doi))
              ))
            )
          )
          return()
        }

        # Prendi il primo match
        oa_data <- values$references_oa[oa_match_idx[1], ]

        # Estrai il titolo per l'header
        doc_title <- if (!is.na(oa_data$title) && nzchar(oa_data$title)) {
          oa_data$title
        } else if (
          !is.na(oa_data$display_name) && nzchar(oa_data$display_name)
        ) {
          oa_data$display_name
        } else {
          "Reference Details"
        }

        # Tronca il titolo se troppo lungo
        if (nchar(doc_title) > 120) {
          doc_title <- paste0(substr(doc_title, 1, 117), "...")
        }

        # Crea l'HTML per il modal
        modal_content <- create_oa_details_html(oa_data)

        # Aggiorna il modal con il titolo del documento
        session$sendCustomMessage(
          "updateOAModal",
          list(
            title = as.character(span(
              doc_title,
              style = "font-weight: 600; font-size: 20px; line-height: 1.4;"
            )),
            content = as.character(modal_content)
          )
        )
      },
      error = function(e) {
        session$sendCustomMessage(
          "updateOAModal",
          list(
            title = as.character(span(
              "Error",
              style = "font-weight: 600; font-size: 20px;"
            )),
            content = as.character(div(
              style = "text-align: center; padding: 40px; color: #e74c3c;",
              icon(
                "exclamation-circle",
                style = "font-size: 32px; margin-bottom: 15px;"
              ),
              h4("Error loading details"),
              p(paste("Error:", e$message))
            ))
          )
        )

        cat("Error in OA modal handler:", e$message, "\n")
        print(e)
      }
    )
  })

  # ===========================================
  # RESET FUNCTIONALITY
  # ===========================================

  observeEvent(input$reset_analysis, {
    values$pdf_text <- NULL
    values$analysis_results <- NULL
    values$network_plot <- NULL
    values$analysis_running <- FALSE
    values$readability_indices <- NULL
    values$word_trends_data <- NULL
    values$cluster_description <- NULL
    values$pdf_doi <- NULL
    values$citation_type_used <- NULL

    updateSelectizeInput(
      session,
      "trend_words",
      choices = NULL,
      selected = NULL
    )

    preview_visible(TRUE)

    tryCatch(
      {
        shinyjs::reset("pdf_file")
        updateTextInput(session, "pdf_doi_input", value = "")
        # MODIFIED: Reset to no selection
        updateRadioButtons(
          session,
          "citation_type_import",
          selected = character(0)
        )
        updateActionButton(
          session,
          "run_analysis",
          label = "Start",
          icon = icon("play")
        )
        updateActionButton(session, "toggle_preview", "Hide Preview")
        updateTextInput(session, "context_search", value = "")
        updateSelectInput(session, "context_type_filter", selected = "")
      },
      error = function(e) {
        cat("Reset UI elements failed:", e$message, "\n")
      }
    )

    showNotification(
      "Analysis reset successfully!",
      type = "message",
      duration = 3
    )

    # Re-open the PDF import box
    session$sendCustomMessage('togglePdfImport', list(action = 'open'))
  })

  # ===========================================
  # EXCEL RESULTS DOWNLOAD
  # ===========================================

  # Download all results as Excel file
  output$download_all_results <- downloadHandler(
    filename = function() {
      paste0("content_analysis_results_", Sys.Date(), ".xlsx")
    },
    content = function(file) {
      # Verifica che ci siano risultati disponibili
      if (is.null(values$analysis_results)) {
        showNotification(
          "No analysis results available. Please run the analysis first.",
          type = "error",
          duration = 5
        )
        return()
      }

      tryCatch(
        {
          # Create workbook
          wb_content <- openxlsx::createWorkbook()

          # === SHEET 1: SUMMARY STATISTICS ===
          openxlsx::addWorksheet(wb_content, "Summary_Statistics")

          summary_data <- data.frame(
            Metric = c(
              "Total Words",
              "Total Citations",
              "Narrative Citations",
              "Parenthetical Citations",
              "Citation Density (per 1000 words)",
              "Lexical Diversity",
              "Total Characters",
              "Total Sentences",
              "Avg Words per Sentence"
            ),
            Value = c(
              values$analysis_results$summary$total_words_analyzed,
              values$analysis_results$summary$citations_extracted,
              values$analysis_results$summary$narrative_citations,
              values$analysis_results$summary$citations_extracted -
                values$analysis_results$summary$narrative_citations,
              round(
                values$analysis_results$summary$citation_density_per_1000_words,
                2
              ),
              round(values$analysis_results$summary$lexical_diversity, 4),
              values$analysis_results$text_analytics$basic_stats$total_characters,
              values$analysis_results$text_analytics$basic_stats$total_sentences,
              round(
                values$analysis_results$text_analytics$basic_stats$avg_words_per_sentence,
                2
              )
            ),
            stringsAsFactors = FALSE
          )

          openxlsx::writeData(
            wb_content,
            "Summary_Statistics",
            summary_data,
            startRow = 1
          )
          openxlsx::addStyle(
            wb_content,
            "Summary_Statistics",
            style = openxlsx::createStyle(
              textDecoration = "bold",
              fgFill = "#2E86AB",
              fontColour = "#FFFFFF"
            ),
            rows = 1,
            cols = 1:2,
            gridExpand = TRUE
          )
          openxlsx::setColWidths(
            wb_content,
            "Summary_Statistics",
            cols = 1:2,
            widths = c(35, 20)
          )

          # === SHEET 2: READABILITY INDICES ===
          if (!is.null(values$readability_indices)) {
            openxlsx::addWorksheet(wb_content, "Readability_Indices")

            readability_data <- data.frame(
              Index = c(
                "Flesch-Kincaid Grade Level",
                "Flesch Reading Ease",
                "Automated Readability Index (ARI)",
                "Gunning Fog Index",
                "Average Sentence Length (words)",
                "Number of Sentences",
                "Number of Words",
                "Number of Syllables",
                "Number of Complex Words",
                "Percentage Complex Words"
              ),
              Value = c(
                round(values$readability_indices$flesch_kincaid_grade, 2),
                round(values$readability_indices$flesch_reading_ease, 2),
                round(
                  values$readability_indices$automated_readability_index,
                  2
                ),
                round(values$readability_indices$gunning_fog_index, 2),
                round(values$readability_indices$avg_sentence_length, 2),
                values$readability_indices$n_sentences,
                values$readability_indices$n_words,
                values$readability_indices$n_syllables,
                values$readability_indices$n_complex_words,
                paste0(
                  round(values$readability_indices$pct_complex_words, 2),
                  "%"
                )
              ),
              stringsAsFactors = FALSE
            )

            openxlsx::writeData(
              wb_content,
              "Readability_Indices",
              readability_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Readability_Indices",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#27ae60",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:2,
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Readability_Indices",
              cols = 1:2,
              widths = c(40, 20)
            )
          }

          # === SHEET 3: WORD FREQUENCIES ===
          if (
            !is.null(values$analysis_results$word_frequencies) &&
              nrow(values$analysis_results$word_frequencies) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Word_Frequencies")

            top_words <- values$analysis_results$word_frequencies %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Frequencies",
              top_words,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Frequencies",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#3498db",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(top_words),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Frequencies",
              cols = 1:ncol(top_words),
              widths = "auto"
            )
          }

          # === SHEET 4: BIGRAMS ===
          if (
            !is.null(values$analysis_results$ngrams) &&
              "2gram" %in% names(values$analysis_results$ngrams)
          ) {
            openxlsx::addWorksheet(wb_content, "Bigrams")

            bigrams_data <- values$analysis_results$ngrams$`2gram` %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Bigrams",
              bigrams_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Bigrams",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#9b59b6",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(bigrams_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Bigrams",
              cols = 1:ncol(bigrams_data),
              widths = "auto"
            )
          }

          # === SHEET 5: TRIGRAMS ===
          if (
            !is.null(values$analysis_results$ngrams) &&
              "3gram" %in% names(values$analysis_results$ngrams)
          ) {
            openxlsx::addWorksheet(wb_content, "Trigrams")

            trigrams_data <- values$analysis_results$ngrams$`3gram` %>%
              slice_head(n = 50) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Trigrams",
              trigrams_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Trigrams",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e67e22",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trigrams_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Trigrams",
              cols = 1:ncol(trigrams_data),
              widths = "auto"
            )
          }

          # === SHEET 6: CITATION TYPES ===
          if (
            !is.null(values$analysis_results$citation_metrics$type_distribution)
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Types")

            types_data <- values$analysis_results$citation_metrics$type_distribution %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citation_Types",
              types_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Types",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e74c3c",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(types_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Types",
              cols = 1:ncol(types_data),
              widths = "auto"
            )
          }

          # === SHEET 7: CITATIONS BY SECTION ===
          if (
            !is.null(
              values$analysis_results$citation_metrics$section_distribution
            )
          ) {
            openxlsx::addWorksheet(wb_content, "Citations_by_Section")

            sections_data <- values$analysis_results$citation_metrics$section_distribution %>%
              filter(n > 0) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citations_by_Section",
              sections_data,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citations_by_Section",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(sections_data),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citations_by_Section",
              cols = 1:ncol(sections_data),
              widths = "auto"
            )
          }

          # === SHEET 8: CITATION CONTEXTS ===
          if (
            !is.null(values$analysis_results$citation_contexts) &&
              nrow(values$analysis_results$citation_contexts) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Contexts")

            # Seleziona le colonne principali se esistono, altrimenti esporta tutte
            contexts_cols <- c(
              "citation_id",
              "citation_text",
              "section",
              "citation_type",
              "is_narrative",
              "words_before",
              "words_after",
              "full_context",
              "context_word_count",
              "citation_position_in_text"
            )

            available_cols <- intersect(
              contexts_cols,
              names(values$analysis_results$citation_contexts)
            )

            if (length(available_cols) > 0) {
              contexts_export <- values$analysis_results$citation_contexts %>%
                select(all_of(available_cols)) %>%
                as.data.frame(stringsAsFactors = FALSE)
            } else {
              # Se nessuna colonna specificata esiste, esporta tutte
              contexts_export <- values$analysis_results$citation_contexts %>%
                as.data.frame(stringsAsFactors = FALSE)
            }

            openxlsx::writeData(
              wb_content,
              "Citation_Contexts",
              contexts_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Contexts",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#2E86AB",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(contexts_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Contexts",
              cols = 1:ncol(contexts_export),
              widths = "auto"
            )
          }

          # === SHEET 9: CITATION NETWORK ===
          if (
            !is.null(values$analysis_results$network_data) &&
              nrow(values$analysis_results$network_data) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Citation_Network")

            # Esporta tutte le colonne disponibili nel network
            network_export <- values$analysis_results$network_data %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Citation_Network",
              network_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Citation_Network",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#8e44ad",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(network_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Citation_Network",
              cols = 1:ncol(network_export),
              widths = "auto"
            )
          }

          # === SHEET 10: WORD TRENDS (if available) ===
          if (
            !is.null(values$word_trends_data) &&
              nrow(values$word_trends_data) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Word_Trends")

            trends_export <- values$word_trends_data %>%
              arrange(word, segment_id) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Trends",
              trends_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Trends",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#27ae60",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trends_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Trends",
              cols = 1:ncol(trends_export),
              widths = "auto"
            )

            # Add summary statistics for word trends
            openxlsx::addWorksheet(wb_content, "Word_Trends_Summary")

            trends_summary <- values$word_trends_data %>%
              group_by(word) %>%
              summarise(
                total_occurrences = sum(count),
                avg_frequency = mean(relative_frequency),
                min_frequency = min(relative_frequency),
                max_frequency = max(relative_frequency),
                std_dev = sd(relative_frequency),
                peak_segment = segment_name[which.max(relative_frequency)],
                .groups = "drop"
              ) %>%
              arrange(desc(total_occurrences)) %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Word_Trends_Summary",
              trends_summary,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Word_Trends_Summary",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(trends_summary),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Word_Trends_Summary",
              cols = 1:ncol(trends_summary),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER TF-IDF ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_descriptions) &&
              nrow(values$cluster_description$cluster_descriptions) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_TF_IDF")

            cluster_tfidf_export <- values$cluster_description$cluster_descriptions %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_TF_IDF",
              cluster_tfidf_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_TF_IDF",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#8e44ad",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_tfidf_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_TF_IDF",
              cols = 1:ncol(cluster_tfidf_export),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER SUMMARY ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_summary) &&
              nrow(values$cluster_description$cluster_summary) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_Summary")

            cluster_summary_export <- values$cluster_description$cluster_summary %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_Summary",
              cluster_summary_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_Summary",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#e67e22",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_summary_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_Summary",
              cols = 1:ncol(cluster_summary_export),
              widths = "auto"
            )
          }

          # === SHEET: CLUSTER REFERENCES ===
          if (
            !is.null(values$cluster_description) &&
              !is.null(values$cluster_description$cluster_references) &&
              nrow(values$cluster_description$cluster_references) > 0
          ) {
            openxlsx::addWorksheet(wb_content, "Cluster_References")

            cluster_refs_export <- values$cluster_description$cluster_references %>%
              as.data.frame(stringsAsFactors = FALSE)

            openxlsx::writeData(
              wb_content,
              "Cluster_References",
              cluster_refs_export,
              startRow = 1
            )
            openxlsx::addStyle(
              wb_content,
              "Cluster_References",
              style = openxlsx::createStyle(
                textDecoration = "bold",
                fgFill = "#16a085",
                fontColour = "#FFFFFF"
              ),
              rows = 1,
              cols = 1:ncol(cluster_refs_export),
              gridExpand = TRUE
            )
            openxlsx::setColWidths(
              wb_content,
              "Cluster_References",
              cols = 1:ncol(cluster_refs_export),
              widths = "auto"
            )
          }

          # Save workbook
          openxlsx::saveWorkbook(wb_content, file, overwrite = TRUE)
        },
        error = function(e) {
          # Log detailed error for debugging
          cat("Error in download_all_results:\n")
          cat("Error message:", e$message, "\n")
          cat("Stack trace:\n")
          print(e)

          showNotification(
            paste("Error exporting results:", e$message),
            type = "error",
            duration = 8
          )
        }
      )
    }
  )

  # ===========================================
  # EXPORT ALL IMAGES
  # ===========================================

  # Step 1: Button click triggers JS canvas capture of the live network
  observeEvent(input$export_all_images, {
    req(values$analysis_results)
    if (!is.null(values$network_plot)) {
      dpi <- values$dpi %||% 300
      shinyjs::runjs(sprintf(
        'window._captureVisCanvas("citation_network", %d, function(dataURL) {
           Shiny.setInputValue("ca_network_canvas", {data: dataURL}, {priority: "event"});
         });',
        dpi
      ))
    } else {
      # No network — proceed directly
      shinyjs::runjs(
        'Shiny.setInputValue("ca_network_canvas", {data: null}, {priority: "event"});'
      )
    }
    showNotification("Exporting images...", type = "message", duration = 3)
  })

  # Step 2: Receive canvas data and build ZIP with all images
  observeEvent(input$ca_network_canvas, {
    req(values$analysis_results)

    tmpdir_img <- tempfile("ca_images_")
    dir.create(tmpdir_img, recursive = TRUE)
    png_files <- character(0)

    dpi <- values$dpi %||% 300
    h <- values$h %||% 7
    zoom <- dpi / 96
    vwidth <- round(h * 2 * 96)
    vheight <- round(h * 96)

    # Helper: save a plotly object to PNG
    save_plotly_png <- function(p, name) {
      png_path <- file.path(tmpdir_img, paste0(name, ".png"))
      html_path <- file.path(tmpdir_img, paste0(name, ".html"))
      htmlwidgets::saveWidget(p, file = html_path, selfcontained = FALSE)
      biblioShot(
        url = html_path,
        zoom = zoom,
        vwidth = vwidth,
        vheight = vheight,
        file = png_path
      )
      unlink(html_path)
      unlink(
        paste0(tools::file_path_sans_ext(html_path), "_files"),
        recursive = TRUE
      )
      png_path
    }

    tryCatch(
      {
        # 1. Citation Network from live canvas capture
        canvas_data <- input$ca_network_canvas$data
        if (!is.null(canvas_data) && nchar(canvas_data) > 0) {
          base64_str <- sub("^data:image/png;base64,", "", canvas_data)
          raw_data <- base64enc::base64decode(base64_str)
          net_png <- file.path(tmpdir_img, "Citation_Network.png")
          writeBin(raw_data, net_png)
          if (file.exists(net_png)) png_files <- c(png_files, net_png)
        }

        # 2. Cluster plots (plotly)
        if (!is.null(values$cluster_description)) {
          cluster_plots <- plot_citation_clusters(
            values$cluster_description,
            section_colors = values$analysis_results$section_colors,
            top_n = 10
          )

          if (!is.null(cluster_plots$tfidf_bars)) {
            f <- save_plotly_png(cluster_plots$tfidf_bars, "TF_IDF_Bars")
            if (file.exists(f)) png_files <- c(png_files, f)
          }

          if (!is.null(cluster_plots$tfidf_heatmap)) {
            f <- save_plotly_png(
              cluster_plots$tfidf_heatmap, "TF_IDF_Heatmap"
            )
            if (file.exists(f)) png_files <- c(png_files, f)
          }

          if (!is.null(cluster_plots$references_per_section)) {
            f <- save_plotly_png(
              cluster_plots$references_per_section, "References_per_Section"
            )
            if (file.exists(f)) png_files <- c(png_files, f)
          }
        }

        # 3. Word Trends (plotly)
        if (
          !is.null(values$word_trends_data) &&
            nrow(values$word_trends_data) > 0
        ) {
          trends_plot <- tryCatch(
            plot_word_distribution(
              word_distribution_data = values$word_trends_data,
              plot_type = input$trend_plot_type %||% "line",
              smooth = TRUE,
              show_points = input$trend_show_points %||% TRUE,
              colors = NULL
            ),
            error = function(e) NULL
          )
          if (!is.null(trends_plot)) {
            f <- save_plotly_png(trends_plot, "Word_Trends")
            if (file.exists(f)) png_files <- c(png_files, f)
          }
        }

        # Create zip and trigger browser download
        if (length(png_files) > 0) {
          zip_path <- file.path(tmpdir_img, paste0(
            "content_analysis_images_", Sys.Date(), ".zip"
          ))
          zip::zip(
            zip_path,
            files = basename(png_files),
            root = tmpdir_img
          )
          # Trigger download via base64 data URI
          zip_data <- base64enc::dataURI(file = zip_path, mime = "application/zip")
          zip_filename <- basename(zip_path)
          shinyjs::runjs(sprintf(
            "var link = document.createElement('a'); link.href = '%s'; link.download = '%s'; document.body.appendChild(link); link.click(); document.body.removeChild(link);",
            zip_data, zip_filename
          ))
          showNotification(
            paste(length(png_files), "images exported successfully!"),
            type = "message", duration = 3
          )
        } else {
          showNotification(
            "No images available to export.",
            type = "warning",
            duration = 4
          )
        }
      },
      error = function(e) {
        showNotification(
          paste("Error exporting images:", e$message),
          type = "error",
          duration = 5
        )
      },
      finally = {
        unlink(tmpdir_img, recursive = TRUE)
      }
    )
  })

  # ===========================================
  # UTILITY FUNCTIONS
  # ===========================================

  is_analysis_valid <- reactive({
    !is.null(values$analysis_results) &&
      !is.null(values$analysis_results$citations) &&
      nrow(values$analysis_results$citations) > 0
  })

  is_network_valid <- reactive({
    !is.null(values$analysis_results) &&
      !is.null(values$analysis_results$network_data) &&
      nrow(values$analysis_results$network_data) > 0
  })

  output$debug_info <- renderText({
    if (!is.null(values$analysis_results)) {
      paste(
        "Debug Info:\n",
        "PDF text length:",
        nchar(values$pdf_text %||% ""),
        "\n",
        "Citations found:",
        nrow(values$analysis_results$citations %||% data.frame()),
        "\n",
        "Network connections:",
        nrow(values$analysis_results$network_data %||% data.frame()),
        "\n",
        "Analysis running:",
        values$analysis_running %||% FALSE
      )
    } else {
      "No analysis data available"
    }
  })

  outputOptions(output, "debug_info", suspendWhenHidden = FALSE)
}

# Helper function for HTML card OpenAlex ----
create_oa_details_html <- function(oa_data) {
  # Helper per gestire valori NULL/NA
  safe_value <- function(x, default = "Not available") {
    if (
      is.null(x) ||
        length(x) == 0 ||
        all(is.na(x)) ||
        !nzchar(trimws(as.character(x)))
    ) {
      return(default)
    }
    return(x)
  }

  # === ESTRAI AUTORI E AFFILIAZIONI INLINE ===
  authors_html <- NULL
  if (!is.null(oa_data$authorships) && length(oa_data$authorships) > 0) {
    authorships_df <- oa_data$authorships[[1]]

    if (!is.null(authorships_df) && nrow(authorships_df) > 0) {
      authors_items <- lapply(1:min(nrow(authorships_df), 15), function(i) {
        auth <- authorships_df[i, ]

        author_name <- safe_value(auth$display_name, "Unknown Author")
        author_id <- safe_value(auth$id, "")

        # Estrai affiliazioni
        affiliation_text <- ""
        if (!is.null(auth$institutions) && length(auth$institutions) > 0) {
          insts_df <- auth$institutions[[1]]
          if (
            !is.null(insts_df) &&
              nrow(insts_df) > 0 &&
              "display_name" %in% names(insts_df)
          ) {
            affiliation_text <- paste(insts_df$display_name, collapse = ", ")
          }
        }

        # Se non ci sono istituzioni, prova con affiliation_raw
        if (
          !nzchar(affiliation_text) &&
            !is.na(auth$affiliation_raw) &&
            nzchar(auth$affiliation_raw)
        ) {
          affiliation_text <- auth$affiliation_raw
        }

        # Layout inline: nome e affiliazione sulla stessa riga
        div(
          style = "margin-bottom: 8px; padding: 6px 10px; background-color: #f8f9fa; border-radius: 4px; display: flex; align-items: center; flex-wrap: wrap;",
          tags$div(
            style = "font-weight: 500; font-size: 14px; margin-right: 12px;",
            # Link OpenAlex se disponibile
            if (nzchar(author_id) && author_id != "Not available") {
              tags$a(
                href = author_id,
                target = "_blank",
                style = "color: #2E86AB; text-decoration: none;",
                author_name,
                icon(
                  "external-link-alt",
                  style = "margin-left: 4px; font-size: 10px;"
                )
              )
            } else {
              tags$span(style = "color: #2E86AB;", author_name)
            }
          ),
          if (nzchar(affiliation_text)) {
            tags$span(
              style = "color: #666; font-size: 13px; font-style: italic;",
              icon("university", style = "margin-right: 5px; font-size: 11px;"),
              affiliation_text
            )
          }
        )
      })

      authors_html <- tagList(authors_items)

      if (nrow(authorships_df) > 15) {
        authors_html <- tagList(
          authors_html,
          tags$small(
            style = "color: #999; font-style: italic; display: block; margin-top: 8px; padding-left: 10px;",
            paste("... and", nrow(authorships_df) - 15, "more authors")
          )
        )
      }
    }
  }

  # === KEYWORDS - TUTTE ===
  keywords_html <- NULL
  if (!is.null(oa_data$keywords) && length(oa_data$keywords) > 0) {
    keywords_data <- oa_data$keywords[[1]]
    if (
      !is.null(keywords_data) &&
        is.data.frame(keywords_data) &&
        nrow(keywords_data) > 0
    ) {
      if ("display_name" %in% names(keywords_data)) {
        # NESSUN LIMITE - tutte le keywords
        keywords_html <- lapply(keywords_data$display_name, function(kw) {
          tags$span(
            class = "label label-primary",
            style = "margin-right: 5px; margin-bottom: 5px; font-size: 11px; display: inline-block; background-color: #3498db; color: white; padding: 4px 10px; border-radius: 3px;",
            kw
          )
        })
      }
    }
  }

  # === ABSTRACT ===
  abstract_text <- safe_value(oa_data$abstract, "Abstract not available")

  # === TOPICS - SOLO I PRIMI 5 ===
  topics_html <- NULL
  if (!is.null(oa_data$topics) && length(oa_data$topics) > 0) {
    topics_data <- oa_data$topics[[1]]
    if (
      !is.null(topics_data) &&
        is.data.frame(topics_data) &&
        nrow(topics_data) > 0
    ) {
      if ("display_name" %in% names(topics_data)) {
        main_topics <- topics_data
        if ("type" %in% names(topics_data)) {
          main_topics <- topics_data[topics_data$type == "topic", ]
        }

        if (nrow(main_topics) > 0) {
          # LIMITE A 5 TOPICS
          topics_html <- lapply(1:min(5, nrow(main_topics)), function(i) {
            tags$span(
              class = "label label-info",
              style = "margin-right: 5px; margin-bottom: 5px; font-size: 11px; display: inline-block; background-color: #5bc0de; color: white; padding: 4px 10px; border-radius: 3px;",
              main_topics$display_name[i]
            )
          })
        }
      }
    }
  }

  # === JOURNAL INFO ===
  journal_name <- safe_value(
    oa_data$source_display_name,
    "Journal not available"
  )
  issn <- safe_value(oa_data$issn_l, "")

  # === DOCUMENT TYPE ===
  doc_type <- safe_value(oa_data$type, "Unknown")

  # === PUBLICATION INFO ===
  pub_year <- safe_value(oa_data$publication_year, "N/A")
  citations <- safe_value(oa_data$cited_by_count, 0)
  fwci <- safe_value(oa_data$fwci, NA)

  # === OPEN ACCESS INFO ===
  is_oa <- !is.na(oa_data$is_oa) && oa_data$is_oa
  oa_status <- tolower(safe_value(oa_data$oa_status, "closed")) # Normalizza in lowercase

  # === DOI ===
  doi_url <- safe_value(oa_data$doi, "")
  doi_clean <- gsub("https://doi.org/", "", doi_url)

  # === PDF URL ===
  pdf_url <- safe_value(oa_data$pdf_url, "")
  has_pdf <- nzchar(pdf_url) && !is.na(pdf_url) && pdf_url != "Not available"

  # === CREA HTML COMPLETO ===
  tagList(
    div(
      class = "oa-document-summary",

      # Metadata Cards Row
      fluidRow(
        style = "margin-bottom: 20px;",

        # Card 1: Publication Year (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #3498db 0%, #2980b9 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "calendar",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Publication Year"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  pub_year
                )
              )
            )
          )
        ),

        # Card 2: Citations (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #27ae60 0%, #229954 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "quote-right",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Citations"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  format(as.numeric(citations), big.mark = ",")
                )
              )
            )
          )
        ),

        # Card 3: Open Access Status (MODIFICATA)
        column(3, {
          # Determina colore e icona in base a oa_status
          oa_colors <- list(
            closed = list(
              bg = "#95a5a6 0%, #7f8c8d",
              icon = "lock",
              text = "Closed"
            ),
            gold = list(
              bg = "#f39c12 0%, #e67e22",
              icon = "unlock",
              text = "Gold OA"
            ),
            green = list(
              bg = "#27ae60 0%, #229954",
              icon = "unlock",
              text = "Green OA"
            ),
            bronze = list(
              bg = "#cd7f32 0%, #b8732d",
              icon = "unlock",
              text = "Bronze OA"
            ),
            hybrid = list(
              bg = "#16a085 0%, #138d75",
              icon = "unlock",
              text = "Hybrid OA"
            )
          )

          # Se oa_status non è nei valori conosciuti, usa closed come default
          current_status <- if (oa_status %in% names(oa_colors)) {
            oa_status
          } else {
            "closed"
          }
          status_info <- oa_colors[[current_status]]

          div(
            style = paste0(
              "background: linear-gradient(135deg, ",
              status_info$bg,
              " 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);"
            ),
            div(
              style = "display: flex; align-items: center;",
              icon(
                status_info$icon,
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "Access"
                ),
                tags$div(
                  style = "font-size: 18px; font-weight: bold; margin-top: 2px;",
                  status_info$text
                )
              )
            )
          )
        }),

        # Card 4: FWCI (invariata)
        column(
          3,
          div(
            style = "background: linear-gradient(135deg, #f39c12 0%, #e67e22 100%); border-radius: 8px; padding: 15px; color: white; min-height: 85px; display: flex; flex-direction: column; justify-content: center; box-shadow: 0 2px 4px rgba(0,0,0,0.1);",
            div(
              style = "display: flex; align-items: center;",
              icon(
                "chart-line",
                style = "font-size: 28px; margin-right: 12px; opacity: 0.9;"
              ),
              div(
                tags$div(
                  style = "font-size: 10px; text-transform: uppercase; opacity: 0.85; letter-spacing: 0.5px;",
                  "FWCI"
                ),
                tags$div(
                  style = "font-size: 24px; font-weight: bold; margin-top: 2px;",
                  if (!is.na(fwci)) round(as.numeric(fwci), 2) else "N/A"
                )
              )
            )
          )
        )
      ),

      # Authors Section - PIÙ COMPATTA
      if (!is.null(authors_html)) {
        div(
          class = "box box-primary",
          style = "margin-bottom: 15px;",
          div(
            class = "box-header with-border",
            h5(
              icon("users", style = "margin-right: 8px;"),
              "Authors",
              style = "color: #2E86AB; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            style = "max-height: 250px; overflow-y: auto;",
            authors_html
          )
        )
      },

      # Journal Info
      div(
        class = "box box-success",
        style = "margin-bottom: 15px;",
        div(
          class = "box-header with-border",
          h5(
            icon("book", style = "margin-right: 8px;"),
            "Journal",
            style = "color: #27ae60; margin: 0; font-weight: 600;"
          )
        ),
        div(
          class = "box-body",
          style = "padding: 15px;",

          # Container principale
          div(
            style = "display: flex; align-items: center; justify-content: space-between; flex-wrap: wrap; gap: 20px;",

            # Colonna 1: Nome Journal e ISSN
            div(
              style = "flex: 1 1 300px; min-width: 250px;",
              tags$div(
                style = "font-size: 16px; font-weight: 600; color: #2c3e50; margin-bottom: 5px;",
                journal_name
              ),
              if (nzchar(issn) && issn != "Not available") {
                tags$div(
                  style = "color: #666; font-size: 13px;",
                  tags$strong("ISSN: "),
                  issn
                )
              }
            ),

            # Colonna 2: Document Type
            div(
              style = "flex: 0 0 auto; display: flex; align-items: center; gap: 8px;",
              tags$span(
                style = "color: #666; font-size: 13px; font-weight: 500;",
                "Type:"
              ),
              tags$span(
                class = "label label-info",
                style = "font-size: 12px; padding: 5px 12px; background-color: #3498db; text-transform: capitalize;",
                safe_value(oa_data$type, "Unknown")
              )
            ),

            # Colonna 3: DOI
            if (nzchar(doi_url) && doi_url != "Not available") {
              div(
                style = "flex: 0 0 auto; display: flex; align-items: center; gap: 8px;",
                tags$span(
                  style = "color: #666; font-size: 13px; font-weight: 500;",
                  "DOI:"
                ),
                tags$a(
                  href = doi_url,
                  target = "_blank",
                  style = "color: #3498db; text-decoration: none; font-family: 'Courier New', monospace; font-size: 13px; font-weight: 500;",
                  doi_clean,
                  icon(
                    "external-link-alt",
                    style = "margin-left: 5px; font-size: 11px;"
                  )
                )
              )
            }
          )
        )
      ),

      # Abstract
      div(
        class = "box box-info",
        style = "margin-bottom: 15px;",
        div(
          class = "box-header with-border",
          h5(
            icon("align-left", style = "margin-right: 8px;"),
            "Abstract",
            style = "color: #3498db; margin: 0; font-weight: 600;"
          )
        ),
        div(
          class = "box-body",
          tags$p(
            style = "text-align: justify; line-height: 1.7; color: #333;",
            abstract_text
          )
        )
      ),

      # Keywords - TUTTE
      if (!is.null(keywords_html) && length(keywords_html) > 0) {
        div(
          class = "box box-warning",
          style = "margin-bottom: 15px;",
          div(
            class = "box-header with-border",
            h5(
              icon("tags", style = "margin-right: 8px;"),
              "Keywords",
              tags$small(
                style = "color: #999; font-weight: normal; margin-left: 8px;",
                paste0("(", length(keywords_html), ")")
              ),
              style = "color: #f39c12; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            keywords_html
          )
        )
      },

      # Topics - SOLO PRIMI 5
      if (!is.null(topics_html) && length(topics_html) > 0) {
        div(
          class = "box",
          style = "margin-bottom: 15px; border-top: 3px solid #9b59b6;",
          div(
            class = "box-header with-border",
            h5(
              icon("lightbulb", style = "margin-right: 8px;"),
              "Research Topics",
              tags$small(
                style = "color: #999; font-weight: normal; margin-left: 8px;",
                "(Top 5)"
              ),
              style = "color: #9b59b6; margin: 0; font-weight: 600;"
            )
          ),
          div(
            class = "box-body",
            topics_html
          )
        )
      },

      # Action Button - PDF
      if (has_pdf) {
        div(
          style = "margin-top: 20px; padding-top: 15px; border-top: 1px solid #ddd; text-align: center;",
          tags$a(
            class = "btn btn-danger btn-lg",
            href = pdf_url,
            target = "_blank",
            icon("file-pdf"),
            " Download Full Text PDF",
            style = "font-weight: bold; padding: 12px 30px;"
          )
        )
      }
    )
  )
}

reconstruct_from_txt <- function(file_lines) {
  reconstructed_txt <- list()
  current_section <- "Full_text"
  section_content <- ""
  for (line in file_lines) {
    if (grepl("^SECTION LINE SEPARATION: ", line)) {
      # Save the previous section content
      if (current_section != "Full_text") {
        reconstructed_txt[[current_section]] <- section_content
      }
      # Start a new section
      current_section <- sub("^SECTION LINE SEPARATION: ", "", line)
      section_content <- ""
    } else {
      section_content <- paste0(
        section_content,
        ifelse(section_content == "", "", "\n"),
        line
      )
    }
  }
  # Save the last section content
  reconstructed_txt[[current_section]] <- section_content
  if (!"Full_text" %in% names(reconstructed_txt)) {
    Full_text <- paste(reconstructed_txt, collapse = "\n\n")
    reconstructed_txt <- c(Full_text = Full_text, reconstructed_txt)
  }
  return(reconstructed_txt)
}

utils::globalVariables(c("Country", "TotalCitation", "TC"))
#' Summarizing bibliometric analysis results
#'
#' \code{summary} method for class '\code{bibliometrix}'
#' @param object is the object for which a summary is desired.
#' @param ... can accept two arguments:\cr
#' \code{k} integer, used for table formatting (number of rows). Default value is 10.\cr
#' \code{pause} logical, used to allow pause in screen scrolling of results. Default value is \code{pause = FALSE}.\cr
#' \code{width} integer, used to define screen output width. Default value is \code{width = 120}.
#' \code{verbose} logical, used to allow screen output. Default is TRUE.
#' @return The function \code{summary} computes and returns a list of summary statistics of the object of class \code{bibliometrics}.
#'
#' the list contains the following objects:
#' \tabular{lll}{
#' \code{MainInformation}   \tab   \tab Main Information about Data\cr
#' \code{AnnualProduction}  \tab   \tab Annual Scientific Production\cr
#' \code{AnnualGrowthRate}  \tab   \tab Annual Percentage Growth Rate\cr
#' \code{MostProdAuthors}   \tab   \tab Most Productive Authors\cr
#' \code{MostCitedPapers}   \tab   \tab Top manuscripts per number of citations\cr
#' \code{MostProdCountries} \tab   \tab Corresponding Author's Countries\cr
#' \code{TCperCountries}    \tab   \tab Total Citation per Countries\cr
#' \code{MostRelSources}    \tab   \tab Most Relevant Sources\cr
#' \code{MostRelKeywords}   \tab   \tab Most Relevant Keywords}
#'
#'
#' @examples
#' data(scientometrics, package = "bibliometrixData")
#'
#' results <- biblioAnalysis(scientometrics)
#'
#' summary(results)
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @method summary bibliometrix
#' @export

summary.bibliometrix <- function(object, ...) {
  if (!inherits(object, "bibliometrix")) {
    cat('\n argument "object" have to be an object of class "bibliometrix"\n')
    return(NA)
  }

  options(width = 130)

  arguments <- list(...)
  if (sum(names(arguments) == "k") == 0) {
    k <- 10
  } else {
    k <- arguments$k
  }
  if (sum(names(arguments) == "pause") == 0) {
    pause <- FALSE
  } else {
    pause <- arguments$pause
  }
  if (sum(names(arguments) == "width") == 0) {
    options(width = 120)
  } else {
    options(width = arguments$width)
  }
  if (sum(names(arguments) == "verbose") == 0) {
    verbose <- TRUE
  } else {
    verbose <- FALSE
  }
  K <- k
  Co <- NULL
  AC <- NULL

  TCm <- format(mean(as.numeric(object$TotalCitation), na.rm = TRUE), digits = 4)
  TCmy <- format(mean(as.numeric(object$MostCitedPapers$TCperYear), na.rm = TRUE), digits = 4)
  CollIndex <- format(object$AuMultiAuthoredArt / sum(object$nAUperPaper > 1), digits = 3) # Collaboration Index
  MYfP <- as.numeric(substr(Sys.time(), 1, 4)) - mean(object$Years, na.rm = TRUE)

  # CAGR
  Y <- table(object$Years)
  ny <- diff(range(as.numeric(names(Y)))) # dim(Y)[1]
  CAGR <- as.numeric(round(((Y[length(Y)] / Y[1])^(1 / (ny)) - 1) * 100, 2))
  #

  # IntColl<- sum(object$CountryCollaboration$MCP)/object$Articles*100

  # Main Information about data
  MainInfo <- toupper("\n\nMain Information about data\n\n")
  MainInfo[length(MainInfo) + 1] <- paste("Timespan                             ", min(object$Years, na.rm = T), ":", max(object$Years, na.rm = T), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Sources (Journals, Books, etc)       ", length(object$Sources), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Documents                            ", object$Articles, "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Annual Growth Rate %                 ", format(CAGR, digits = 4), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Document Average Age                 ", format(MYfP, digits = 3), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Average citations per doc            ", format(TCm, digits = 3), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Average citations per year per doc   ", format(TCmy, digits = 3), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("References                           ", object$nReferences, "\n")
  if (!is.na(object$Documents[1])) {
    MainInfo[length(MainInfo) + 1] <- toupper(paste("\nDocument types                    ", "\n"))
    for (i in 1:length(object$Documents)) {
      MainInfo[length(MainInfo) + 1] <- tolower(paste(names(object$Documents)[i], as.numeric(object$Documents)[i], "\n"))
    }
  }
  MainInfo[length(MainInfo) + 1] <- toupper("\nDocument Contents\n")
  MainInfo[length(MainInfo) + 1] <- paste("Keywords Plus (ID)                   ", length(object$ID), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Author's Keywords (DE)               ", length(object$DE), "\n")
  MainInfo[length(MainInfo) + 1] <- toupper("\nAuthors\n")
  MainInfo[length(MainInfo) + 1] <- paste("Authors                              ", object$nAuthors, "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Author Appearances                   ", object$Appearances, "\n")
  MainInfo[length(MainInfo) + 1] <- paste("Authors of single-authored docs      ", object$AuSingleAuthoredArt, "\n")
  # MainInfo[length(MainInfo)+1]=paste("Authors of multi-authored documents  ",object$AuMultiAuthoredArt,"\n")
  MainInfo[length(MainInfo) + 1] <- toupper("\nAuthors Collaboration\n")
  MainInfo[length(MainInfo) + 1] <- paste("Single-authored docs                 ", as.character(round(sum(object$nAUperPaper == 1), 0)), "\n") # format(sum(object$nAUperPaper==1),digits=0),"\n")
  MainInfo[length(MainInfo) + 1] <- paste("Documents per Author                 ", format(object$Articles / object$nAuthors, digits = 3), "\n")
  # MainInfo[length(MainInfo)+1]=paste("Authors per Document                 ",format(object$nAuthors/object$Articles,digits=3),"\n")
  MainInfo[length(MainInfo) + 1] <- paste("Co-Authors per Doc                   ", format(mean(object$nAUperPaper), digits = 3), "\n")
  MainInfo[length(MainInfo) + 1] <- paste("International co-authorships %       ", format(object$IntColl, digits = 4), "\n")
  # MainInfo[length(MainInfo)+1]=paste("Collaboration Index                  ",CollIndex,"\n")
  MainInfo[length(MainInfo) + 1] <- paste("\n")


  ## DF table for biblioshiny
  Description <- gsub("[0-9]+", "", MainInfo)
  Description <- gsub("\\\n", "", Description)
  Description <- gsub(":", "", Description)
  Description <- trimws(gsub("\\.", "", Description))
  Results <- gsub("[^0-9\\.:]", "", MainInfo)
  MainInfoDF <- data.frame("Description" = Description, "Results" = Results)

  if (isTRUE(verbose)) {
    cat(MainInfo)
  }

  if (pause == TRUE & isTRUE(verbose)) {
    cat("Hit <Return> to see next table: ")
    line <- readline()
  }

  if (isTRUE(verbose)) cat("\nAnnual Scientific Production\n\n")
  Y <- data.frame(table(object$Years))
  names(Y) <- c("Year   ", "Articles")
  if (isTRUE(verbose)) {
    print(Y, row.names = FALSE)
    cat("\n")
  }
  #  ny=dim(Y)[1]
  # ny=max(as.numeric(levels(Y[,1])),na.rm=TRUE)-min(as.numeric(levels(Y[,1])),na.rm=TRUE)
  # GR=((Y[nrow(Y),2]/Y[1,2])^(1/(ny))-1)*100
  if (isTRUE(verbose)) {
    cat("Annual Percentage Growth Rate", CAGR, "\n\n")
  }


  if (pause == TRUE & isTRUE(verbose)) {
    cat("Hit <Return> to see next table: ")
    line <- readline()
  }


  # Most Productive Authors
  if (isTRUE(verbose)) {
    cat("\nMost Productive Authors\n\n")
  }
  if (K == Inf) {
    k <- length(object$Authors)
  }
  A <- data.frame(cbind(object$Authors[1:k]))
  A$MPA <- row.names(A)
  A <- A[, c(2, 1)]
  A[, 3:4] <- object$AuthorsFrac[1:k, ]
  names(A) <- c("Authors       ", "Articles", "Authors       ", "Articles Fractionalized")
  A <- format(A, justify = "left", digits = 3)
  row.names(A) <- 1:k
  if (isTRUE(verbose)) {
    print(A, row.names = TRUE)
    cat("\n")
  }

  if (pause == TRUE & isTRUE(verbose)) {
    cat("Hit <Return> to see next table: ")
    line <- readline()
  }

  # Most Cited Manuscipts
  if (isTRUE(verbose)) {
    cat("\nTop manuscripts per citations\n\n")
  }
  if (K == Inf) {
    k <- dim(object$MostCitedPapers)[1]
  }
  MostCitedPapers <- object$MostCitedPapers[1:k, ]
  MostCitedPapers <- format(MostCitedPapers, justify = "left", digits = 3)
  row.names(MostCitedPapers) <- 1:k
  if (isTRUE(verbose)) {
    print(MostCitedPapers, row.names = TRUE)
    cat("\n")
  }

  if (pause == TRUE & isTRUE(verbose)) {
    cat("Hit <Return> to see next table: ")
    line <- readline()
  }

  if (K == Inf) {
    k <- length(object$Countries)
  }
  kk <- k
  if (length(object$Countries) > 0) {
    # Most Productive Countries
    if (isTRUE(verbose)) {
      cat("\nCorresponding Author's Countries\n\n")
    }

    if (length(object$Countries) < k) {
      kk <- length(object$Countries)
    }

    Co <- data.frame(Country = names(object$Countries[1:kk]), Articles = as.numeric(object$Countries)[1:kk], Freq = 0)
    Co$Freq <- as.numeric(Co[, 2]) / sum(object$Countries)
    Co <- cbind(Co, object$CountryCollaboration[1:kk, 2:3])
    Co$MCP_Ratio <- Co$MCP / Co$Articles
    Co <- format(Co, justify = "left", digits = 3)
    row.names(Co) <- 1:kk
    if (isTRUE(verbose)) {
      print(Co, row.names = TRUE)
      cat("\n")
    }
    if (isTRUE(verbose)) {
      cat("\nSCP: Single Country Publications\n\nMCP: Multiple Country Publications\n\n")
    }


    if (pause == TRUE & isTRUE(verbose)) {
      cat("Hit <Return> to see next table: ")
      line <- readline()
    }

    # Total Citation per Country
    if (isTRUE(verbose)) {
      cat("\nTotal Citations per Country\n\n")
    }

    Co2 <- data.frame(Country = object$CO, TotalCitation = object$TotalCitation)
    Co2 <- Co2[!is.na(Co2[, 1]), ]
    AC <- Co2 %>%
      group_by(Country) %>%
      summarise("TC" = sum(TotalCitation), "Average Article Citations" = sum(TotalCitation) / length(TotalCitation)) %>%
      arrange(-TC) # %>% as.data.frame(.data)

    AC <- as.data.frame(AC)

    names(AC) <- c("Country     ", "Total Citations", "Average Article Citations")
    AC <- format(AC, justify = "left", digits = 3)[1:kk, ]
    row.names(AC) <- 1:kk
    if (isTRUE(verbose)) {
      print(AC, row.names = TRUE)
      cat("\n")
    }


    if (pause == TRUE & isTRUE(verbose)) {
      cat("Hit <Return> to see next table: ")
      line <- readline()
    }
  }

  if (!is.null(object$Sources)) {
    # Most relevant Sources
    if (isTRUE(verbose)) {
      cat("\nMost Relevant Sources\n\n")
    }
    if (K == Inf) {
      k <- length(object$Sources)
    }
    kk <- k
    if (length(object$Sources) < k) {
      kk <- length(object$Sources)
    }
    AA <- data.frame(cbind(object$Sources[1:kk]))
    AA$MPA <- row.names(AA)
    AA <- AA[, c(2, 1)]
    names(AA) <- c("Sources       ", "Articles")
    AA <- format(AA, justify = "left", digits = 3)
    row.names(AA) <- 1:kk
    if (isTRUE(verbose)) {
      print(AA, row.names = TRUE)
      cat("\n")
    }


    if (pause == TRUE & isTRUE(verbose)) {
      cat("Hit <Return> to see next table: ")
      line <- readline()
    }
  }

  if (!is.null(object$ID) & !is.null(object$DE)) {
    # Most relevant Keywords
    if (isTRUE(verbose)) {
      cat("\nMost Relevant Keywords\n\n")
    }
    if (K == Inf) {
      k <- min(c(length(object$DE), length(object$ID)))
    }
    AAA <- data.frame(cbind(object$DE[1:k]))
    AAA$MPA <- row.names(AAA)
    AAA <- AAA[, c(2, 1)]
    names(AAA) <- c("DE Keywords     ", "Articles")
    A2 <- data.frame(cbind(object$ID[1:k]))
    A2$MPA <- row.names(A2)
    A2 <- A2[, c(2, 1)]
    AAA[, c(3, 4)] <- A2
    names(AAA) <- c("Author Keywords (DE)     ", "Articles", "Keywords-Plus (ID)    ", "Articles")
    AAA <- format(AAA, justify = "left", digits = 3)
    row.names(AAA) <- 1:k
    if (isTRUE(verbose)) {
      print(AAA, row.names = TRUE)
      cat("\n")
    }
  } else {
    (AAA <- NULL)
  }

  summaryresults <- list(MainInformation = MainInfo, MainInformationDF = MainInfoDF, AnnualProduction = Y, AnnualGrowthRate = CAGR, MostProdAuthors = A, MostCitedPapers = MostCitedPapers, MostProdCountries = Co, TCperCountries = AC, MostRelSources = AA, MostRelKeywords = AAA)

  invisible(summaryresults)
}

utils::globalVariables(c("id", "name"))
#' Save a network graph object as Pajek files
#'
#'
#' The function \code{\link{net2Pajek}} save a bibliographic network previously created by \code{\link{networkPlot}} as pajek files.
#'
#' @param net is a network graph object returned by the function \code{\link{networkPlot}}.
#' @param filename is a character. It indicates the filename for Pajek export files.
#' @param path is a character. It indicates the path where the files will be saved. When path="NULL, the files will be saved in the current folder. Default is NULL.
#' @return The function returns no object but will save three Pajek files in the folder given in the "path" argument with the name "filename.clu," "filename.vec," and "filename.net."
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' NetMatrix <- biblioNetwork(management,
#'   analysis = "co-occurrences",
#'   network = "keywords", sep = ";"
#' )
#'
#' net <- networkPlot(NetMatrix, n = 30, type = "auto", Title = "Co-occurrence Network", labelsize = 1)
#'
#' net2Pajek(net, filename = "pajekfiles", path = NULL)
#' }
#' @seealso \code{\link{net2VOSviewer}} to export and plot the network with VOSviewer software.
#'
#' @export
net2Pajek <- function(net, filename = "my_pajek_network", path = NULL) {
  graph <- net$graph

  nodes <- igraph::as_data_frame(graph, what = c("vertices")) %>%
    mutate(id = row_number())

  edges <- igraph::as_data_frame(graph, what = c("edges"))
  edges <- edges %>%
    left_join(nodes %>% select(id, name), by = c("from" = "name")) %>%
    rename(id_from = id) %>%
    left_join(nodes %>% select(id, name), by = c("to" = "name")) %>%
    rename(id_to = id)

  ### Creation of NET file
  if (!is.null(path)) {
    if (substr(path, nchar(path), nchar(path)) != "/") {
      path <- paste0(path, "/")
    }
  }
  filename <- paste0(path, filename)

  file <- paste0(filename, ".net")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$id, ' "', nodes$name, '"'), file = file, append = T)

  # Edges
  write(paste0("*Edges ", nrow(nodes)), file = file, append = T)
  write(paste0(edges$id_from, " ", edges$id_to, " ", edges$weight), file = file, append = T)

  ### Creation of VEC file
  file <- paste0(filename, ".vec")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$deg), file = file, append = T)

  ### Creation of CLU file
  file <- paste0(filename, ".clu")

  # Nodes
  write(paste0("*Vertices ", nrow(nodes)), file = file)
  write(paste0(nodes$community), file = file, append = T)
}

#' Meta-Field Tag Extraction
#'
#' It extracts other field tags, different from the standard WoS/SCOPUS codify.
#' @param M is a data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.
#' @param Field is a character object. New tag extracted from aggregated data is specified by this string.
#' Field can be equal to one of these tags:
#' \tabular{lll}{
#' \code{"CR_AU"}\tab   \tab First Author of each cited reference\cr
#' \code{"CR_SO"}\tab   \tab Source of each cited reference\cr
#' \code{"AU_CO"}\tab   \tab Country of affiliation for co-authors \cr
#' \code{"AU1_CO"}\tab   \tab Country of affiliation for the first author\cr
#' \code{"AU_UN"}\tab   \tab University of affiliation for each co-author and the corresponding author (AU1_UN)\cr
#' \code{"SR"}\tab     \tab Short tag of the document (as used in reference lists)}
#'
#' @param sep is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.
#' @param aff.disamb is a logical. If TRUE and Field="AU_UN", then a disambiguation algorithm is used to identify and match scientific affiliations
#' (univ, research centers, etc.). The default is \code{aff.disamb=TRUE}.
#' @return the bibliometric data frame with a new column containing data about new field tag indicated in the argument \code{Field}.
#'
#'
#'
#' @examples
#' # Example 1: First Authors for each cited reference
#'
#' data(scientometrics, package = "bibliometrixData")
#' scientometrics <- metaTagExtraction(scientometrics, Field = "CR_AU", sep = ";")
#' unlist(strsplit(scientometrics$CR_AU[1], ";"))
#'
#'
#' # Example 2: Source for each cited reference
#'
#' data(scientometrics)
#' scientometrics <- metaTagExtraction(scientometrics, Field = "CR_SO", sep = ";")
#' unlist(strsplit(scientometrics$CR_SO[1], ";"))
#'
#' # Example 3: Affiliation country for co-authors
#'
#' data(scientometrics)
#' scientometrics <- metaTagExtraction(scientometrics, Field = "AU_CO", sep = ";")
#' scientometrics$AU_CO[1:10]
#'
#' @seealso \code{\link{convert2df}} for importing and converting bibliographic files into a data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#'
#' @export

metaTagExtraction <- function(M, Field = "CR_AU", sep = ";", aff.disamb = TRUE) {
  ### data cleaning
  if ("CR" %in% names(M)) {
    M$CR <- gsub("DOI;", "DOI ", as.character(M$CR))
  }


  ### SR field creation

  if (Field == "SR") {
    M <- SR(M)
  }


  if (Field == "CR_AU") {
    M <- CR_AU(M, sep)
  }

  ### CR_SO field creation

  if (Field == "CR_SO") {
    M <- CR_SO(M, sep)
  }
  ### AU_CO field creation

  if (Field == "AU_CO") {
    M <- AU_CO(M)
  }

  if (Field == "AU1_CO") {
    M <- AU1_CO(M, sep)
  }


  # UNIVERSITY AFFILIATION OF ALL AUTHORS AND CORRESPONDING AUTHOR
  if (Field == "AU_UN") {
    ### with disambiguation
    if (isTRUE(aff.disamb)) {
      M <- AU_UN(M, sep)
    } else {
      ### without disambiguation
      M$AU_UN <- gsub("\\[.*?\\] ", "", M$C1)
      M$AU1_UN <- unlist(lapply(strsplit(M$RP, sep), function(l) {
        l <- l[1]
        return(l)
      }))
      ind <- regexpr("\\),", M$AU1_UN)
      a <- which(ind > -1)
      M$AU1_UN[a] <- trim(substr(M$AU1_UN[a], ind[a] + 2, nchar(M$AU1_UN[a])))
    }
  }

  return(M)
}


### SR field
SR <- function(M) {
  listAU <- strsplit(as.character(M$AU), ";")
  listAU <- lapply(listAU, function(l) trim.leading(l))
  if (M$DB[1] == "scopus") {
    listAU <- lapply(listAU, function(l) {
      l <- trim(l)
      l <- sub(" ", ",", l, fixed = TRUE)
      l <- sub(",,", ",", l, fixed = TRUE)
      l <- gsub(" ", "", l, fixed = TRUE)
    })
  }
  FirstAuthors <- gsub(",", " ", unlist(lapply(listAU, function(l) {
    if (length(l > 0)) {
      l <- l[[1]]
    } else {
      (l <- "NA")
    }
    return(l)
  })))

  if (!is.null(M$J9)) {
    ## replace full title in no iso names
    no_art <- which(is.na(M$J9) & is.na(M$JI))
    M$J9[no_art] <- M$SO[no_art]
    ## repleace NA in J9 with JIO
    ind <- which(is.na(M$J9))
    M$J9[ind] <- trim(gsub("\\.", " ", M$JI[ind]))
    SR <- paste(FirstAuthors, M$PY, M$J9, sep = ", ")
  } else {
    no_art <- which(is.na(M$JI))
    M$JI[no_art] <- M$SO[no_art]
    J9 <- trim(gsub("\\.", " ", M$JI))
    SR <- paste(FirstAuthors, M$PY, J9, sep = ", ")
  }

  M$SR_FULL <- gsub("\\s+", " ", SR)

  ## assign an unique name to each document
  SR <- gsub("\\s+", " ", SR)
  st <- i <- 0
  while (st == 0) {
    ind <- which(duplicated(SR))
    if (length(ind) > 0) {
      i <- i + 1
      SR[ind] <- paste0(SR[ind], "-", letters[i], sep = "")
    } else {
      st <- 1
    }
  }
  M$SR <- SR
  # M$SR<- gsub("\\s+", " ", SR)
  return(M)
}

### CR_AU field
CR_AU <- function(M, sep) {
  FCAU <- list(NULL)
  CCR <- NULL
  size <- dim(M)
  CR <- M$CR
  listCAU <- strsplit(as.character(CR), sep)
  listCAU <- lapply(listCAU, function(l) l <- l[nchar(l) > 10]) ## delete not congruent references

  # vector of cited authors
  for (i in 1:size[1]) {
    FCAU[[i]] <- gsub("[[:punct:]]", "", trim.leading(sub(",.*", "", listCAU[[i]])))
    CCR[i] <- paste(FCAU[[i]], collapse = ";")
  }

  M$CR_AU <- CCR
  return(M)
}

### CR_SO field
CR_SO <- function(M, sep) {
  size <- dim(M)
  # Pre-size the result so M$CR_SO is ALWAYS created, even for DBs whose CR
  # cannot be parsed. Otherwise tableTag()'s dplyr::pull() fails with a
  # cryptic "must select exactly one column" error on the missing column.
  CCR <- rep(NA_character_, size[1])
  listCAU <- strsplit(as.character(M$CR), sep)

  db <- toupper(M$DB[1])

  # vector of cited Journals
  if (db == "SCOPUS") {
    # SCOPUS reference layout: "AUTHORS, TITLE (YEAR) JOURNAL, VOL, PAGE"
    # -> after dropping everything up to ") ", the journal is the 1st field.
    for (i in seq_len(size[1])) {
      refs <- gsub(".*?\\) ", "", listCAU[[i]])
      elem <- strsplit(as.character(refs), ",")
      ind <- lengths(elem)
      if (length(ind) > 0 && max(ind) > 2) {
        elem <- elem[ind > 2]
        CCR[i] <- paste(trim.leading(unlist(lapply(elem, function(l) l[[1]]))),
                        collapse = ";")
      }
    }
  } else if (db %in% c("ISI", "WOS", "OPENALEX", "PUBMED", "DIMENSIONS")) {
    # WoS reference layout: "AUTHOR, YEAR, JOURNAL, VOL, PAGE, DOI".
    # OpenAlex/PubMed/Dimensions CR is rewritten to this layout upstream
    # (build_single_cr / completeMetadata / postprocessingDim), so the
    # journal is the 3rd comma-separated field for all of them.
    for (i in seq_len(size[1])) {
      elem <- strsplit(as.character(listCAU[[i]]), ",")
      ind <- lengths(elem)
      if (length(ind) > 0 && max(ind) > 2) {
        elem <- elem[ind > 2]
        CCR[i] <- paste(trim.leading(unlist(lapply(elem, function(l) l[[3]]))),
                        collapse = ";")
      }
    }
  }
  # Other DBs (e.g. LENS) have no reliable CR source -> column stays all-NA.

  M$CR_SO <- CCR
  return(M)
}

### AU_CO field
AU_CO <- function(M) {
  # Countries
  size <- dim(M)[1]
  data("countries", envir = environment())
  countries <- as.character(countries[[1]])
  if (M$DB[1] %in% c("ISI", "PUBMED")) {
    countries <- as.character(sapply(countries, function(s) paste0(s, ".", collapse = "")))
  } else if (M$DB[1] == "SCOPUS") {
    countries <- as.character(sapply(countries, function(s) paste0(s, ";", collapse = "")))
  } else if (M$DB[1] == "DIMENSIONS") {
    countries <- as.character(sapply(countries, function(s) paste0(s, ")", collapse = "")))
  }


  M$AU_CO <- NA
  C1 <- M$C1
  ## must replace all NA before "removing reprint info", or NA_character_ became string "NA"
  if ("RP" %in% names(M)) {
    C1[which(is.na(C1))] <- M$RP[which(is.na(C1))]
  } else {
    M$RP <- NA
  }

  ## remove reprint information from C1
  C1 <- unlist(lapply(C1, function(l) {
    if (!is.na(l)) {
      l <- unlist(strsplit(l, ";"))
      # l=l[regexpr("REPRINT AUTHOR",l)==-1]
      l <- paste0(l, collapse = ";")
    } else {
      "NA"
    }
  }))
  ## above changes all NA to 'NA'

  C1 <- gsub("\\[.*?\\] ", "", C1)
  ## change 'NA' back to NA
  C1[which(C1 == "NA")] <- NA
  if (M$DB[1] == "ISI") {
    C1 <- lastChar(C1, last = ".")
  }
  if (M$DB[1] == "SCOPUS") {
    C1 <- lastChar(C1, last = ";")
  }
  if (M$DB[1] == "DIMENSIONS") {
    C1 <- gsub("\\.)", ")", C1)
  }

  # C1=gsub("[[:punct:][:blank:]]+", " ", C1)
  RP <- M$RP
  # RP[which(is.na(RP))]=M$RRP)
  RP <- paste(RP, ";", sep = "")
  # RP = gsub("[[:punct:][:blank:]]+", " ", RP)
  ## this will make gregexpr(l, RP[i], fixed=TRUE) fail as countries has the format '_COUNTRY_.' or '_COUNTRY_;'

  for (i in 1:size[1]) {
    if (!is.na(C1[i])) {
      ind <- unlist(sapply(countries, function(l) (gregexpr(l, C1[i], fixed = TRUE))))
      if (sum(ind > -1) > 0) {
        M$AU_CO[i] <- paste(unique(names(ind[ind > -1])), collapse = ";")
      }
    }
    if (is.na(M$AU_CO[i])) {
      ind <- unlist(sapply(countries, function(l) (gregexpr(l, RP[i], fixed = TRUE))))
      if (sum(ind > -1) > 0) {
        M$AU_CO[i] <- paste(unique(names(ind[ind > -1])), collapse = ";")
      }
    }
  }

  M$AU_CO <- gsub("[[:digit:]]", "", M$AU_CO)
  M$AU_CO <- gsub("\\(|)", "", M$AU_CO)
  M$AU_CO <- gsub(".", "", M$AU_CO, fixed = TRUE)
  M$AU_CO <- gsub(";;", ";", M$AU_CO, fixed = TRUE)
  M$AU_CO <- gsub("UNITED STATES", "USA", M$AU_CO)
  M$AU_CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", M$AU_CO)
  M$AU_CO <- gsub("TAIWAN", "CHINA", M$AU_CO)
  M$AU_CO <- gsub("ENGLAND", "UNITED KINGDOM", M$AU_CO)
  M$AU_CO <- gsub("SCOTLAND", "UNITED KINGDOM", M$AU_CO)
  M$AU_CO <- gsub("WALES", "UNITED KINGDOM", M$AU_CO)
  M$AU_CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", M$AU_CO)
  M$AU_CO <- gsub("TURKIYE", "TURKEY", M$AU_CO)
  M$AU_CO <- gsub("ESWATINI", "SWAZILAND", M$AU_CO)
  M$AU_CO <- gsub("CZECHIA", "CZECH REPUBLIC", M$AU_CO)

  if (M$DB[1] == "ISI") {
    M$AU_CO <- removeLastChar(M$AU_CO, last = ".")
  }
  if (M$DB[1] == "SCOPUS") {
    M$AU_CO <- removeLastChar(M$AU_CO, last = ";")
  }

  return(M)
}

### AU1_CO field
AU1_CO <- function(M, sep) {
  if (M$DB[1] == "DIMENSIONS") {
    M2 <- AU_CO(M)
    M$AU1_CO <- unlist(lapply(strsplit(M2$AU_CO, ";"), function(l) l[1]))
  } else {
    size <- dim(M)[1]
    # Countries
    data("countries", envir = environment())
    countries <- as.character(countries[[1]])
    countries <- paste(" ", countries, " ", sep = "")
    M$AU1_CO <- NA
    C1 <- M$C1
    C1[which(!is.na(M$RP))] <- M$RP[which(!is.na(M$RP))]
    ## do this before strsplit(), otherwise entries with multiple (reprint) author would be split between first group of authors
    C1 <- gsub("\\[.*?\\] ", "", C1)
    ## remove string before the first "(REPRINT AUTHOR)", otherwise C1 may get split between first group of authors, thus removing address, forcing it to default to RP.



    C1 <- gsub("^.*?\\(REPRINT\\sAUTHOR\\)", "", C1)
    C1 <- unlist(lapply(strsplit(C1, sep), function(l) l[1]))
    ## remove all characters before the last comma, thus constantly leaving only country, or in the case of US, state + zip_code + country.
    ## this way the need to distinguish Georgia and Georgia, US is eliminated.
    C1 <- gsub("^(.+)?,", "", C1)
    C1 <- gsub("[[:punct:][:blank:]]+", " ", C1)
    C1 <- paste(trim(C1), " ", sep = "")
    if (M$DB[1] != "PUBMED") {
      RP <- M$RP
      # RP[which(is.na(RP))]=M$RRP)
      RP <- paste(RP, ";", sep = "")
      RP <- gsub("[[:punct:][:blank:]]+", " ", RP)
    } else {
      RP <- C1 <- paste(" ", gsub("[[:punct:]]", "", C1), sep = "")
    }

    for (i in 1:size[1]) {
      if (!is.na(C1[i])) {
        ind <- unlist(sapply(countries, function(l) (gregexpr(l, C1[i], fixed = TRUE))))
        if (sum(ind > -1) > 0) {
          M$AU1_CO[i] <- paste(unique(names(ind[ind > -1][1])), collapse = ";")
          # print(i)
          # print(M$AU1_CO[i])
        }
      }
      if (is.na(M$AU1_CO[i])) {
        ind <- unlist(sapply(countries, function(l) (gregexpr(l, RP[i], fixed = TRUE))))
        if (sum(ind > -1) > 0) {
          M$AU1_CO[i] <- paste(unique(names(ind[ind > -1][1])), collapse = ";")
        }
      }
    }
    M$AU1_CO <- trim(gsub("[[:digit:]]", "", M$AU1_CO))
    M$AU1_CO <- gsub("UNITED STATES", "USA", M$AU1_CO)
    M$AU1_CO <- gsub("RUSSIAN FEDERATION", "RUSSIA", M$AU1_CO)
    M$AU1_CO <- gsub("TAIWAN", "CHINA", M$AU1_CO)
    M$AU1_CO <- gsub("ENGLAND", "UNITED KINGDOM", M$AU1_CO)
    M$AU1_CO <- gsub("SCOTLAND", "UNITED KINGDOM", M$AU1_CO)
    M$AU1_CO <- gsub("WALES", "UNITED KINGDOM", M$AU1_CO)
    M$AU1_CO <- gsub("NORTH IRELAND", "UNITED KINGDOM", M$AU1_CO)
    M$AU1_CO <- gsub("TURKIYE", "TURKEY", M$AU1_CO)
    M$AU1_CO <- gsub("ESWATINI", "SWAZILAND", M$AU1_CO)
    M$AU1_CO <- gsub("CZECHIA", "CZECH REPUBLIC", M$AU1_CO)
    # M$AU1_CO=gsub(".", "", M$AU1_CO, fixed = TRUE)
    # M$AU1_CO=gsub(";;", ";", M$AU1_CO, fixed = TRUE)
  }
  return(M)
}

### AU_UN field
AU_UN <- function(M, sep) {
  ## remove reprint information from C1
  C1 <- M$C1
  if (!("RP" %in% names(M))) {
    M$RP <- NA
  }
  # C1=unlist(lapply(C1,function(l){
  #   l=unlist(strsplit(l,";"))
  #   #l=l[regexpr("REPRINT AUTHOR",l)==-1]
  #   l=paste0(l,collapse=";")
  # }))
  ###
  AFF <- gsub("\\[.*?\\] ", "", C1)
  indna <- which(is.na(AFF))
  if (length(indna) > 0) {
    AFF[indna] <- M$RP[indna]
  }
  nc <- nchar(AFF)
  AFF[nc == 0] <- NA

  listAFF <- strsplit(AFF, sep, fixed = TRUE)

  uTags <- c(
    "UNIV", "COLL", "SCH", "INST", "ACAD", "ECOLE", "CTR", "SCI", "CENTRE", "CENTER", "CENTRO", "HOSP", "ASSOC", "COUNCIL",
    "FONDAZ", "FOUNDAT", "ISTIT", "LAB", "TECH", "RES", "CNR", "ARCH", "SCUOLA", "PATENT OFF", "CENT LIB", "HEALTH", "NATL",
    "LIBRAR", "CLIN", "FDN", "OECD", "FAC", "WORLD BANK", "POLITECN", "INT MONETARY FUND", "CLIMA", "METEOR", "OFFICE", "ENVIR",
    "CONSORTIUM", "OBSERVAT", "AGRI", "MIT ", "INFN", "SUNY "
  )

  AFFL <- lapply(listAFF, function(l) {
    # l=gsub(","," ,",l)
    l <- gsub("\\(REPRINT AUTHOR\\)", "", l)
    index <- NULL

    for (i in 1:length(l)) {
      # ind=list()
      affL <- unlist(strsplit(l[i], ",", fixed = TRUE))

      indd <- unlist(lapply(uTags, function(x) which(regexpr(x, affL, fixed = TRUE) != -1)))


      # if (length(indd)==0){index=append(index,"NR")
      #   } else if (grepl("[[:digit:]]", affL[indd[1]])){index=append(index,"ND")
      #   } else {index=append(index,affL[indd[1]])}

      if (length(indd) == 0) {
        index <- append(index, "NOTREPORTED")
      } else if (isTRUE(ND(affL, indd)$cond)) {
        index <- append(index, "NOTDECLARED")
      } else {
        index <- append(index, ND(affL, indd)$affL)
      }
    }
    # index=unique(c(ind1,ind2,ind3,ind4,ind5,ind6,ind7,ind8))
    x <- ""
    if (length(index) > 0) {
      # x=paste0(trim.leading((affL[index])),collapse=",")
      x <- paste0(trim.leading(index), collapse = ";")
      x <- gsub(" ,", ";", x)
    }
    return(x)
  })
  AFFL <- unlist(AFFL)
  M$AU_UN <- AFFL
  if (M$DB[1] %in% c("ISI", "OPENALEX") & "C3" %in% names(M)) {
    M$AU_UN[!is.na(M$C3) & M$C3 != ""] <- M$C3[!is.na(M$C3) & M$C3 != ""]
    M$AU_UN <- unlist(lapply(strsplit(M$AU_UN, sep), function(l) {
      l <- paste(trim(l), collapse = sep)
    }))
  }
  M$AU_UN <- gsub("\\\\&", "AND", M$AU_UN)
  M$AU_UN <- gsub("\\&", "AND", M$AU_UN)


  ## identification of Corresponding author affiliation
  RP <- M$RP
  RP[is.na(RP)] <- M$C1[is.na(RP)]
  AFF <- gsub("\\[.*?\\] ", "", RP)
  indna <- which(is.na(AFF))
  if (length(indna) > 0) {
    AFF[indna] <- M$RP[indna]
  }
  nc <- nchar(AFF)
  AFF[nc == 0] <- NA

  listAFF <- strsplit(AFF, sep, fixed = TRUE)

  AFFL <- lapply(listAFF, function(l) {
    # l=gsub(","," ,",l)
    l <- gsub("\\(REPRINT AUTHOR\\)", "", l)
    index <- NULL

    for (i in 1:length(l)) {
      # ind=list()
      affL <- unlist(strsplit(l[i], ",", fixed = TRUE))

      indd <- unlist(lapply(uTags, function(x) which(regexpr(x, affL, fixed = TRUE) != -1)))

      if (length(indd) == 0) {
        index <- append(index, "NOTREPORTED")
      } else if (grepl("[[:digit:]]", affL[indd[1]])) {
        index <- append(index, "NOTDECLARED")
      } else {
        index <- append(index, affL[indd[1]])
      }
    }
    # index=unique(c(ind1,ind2,ind3,ind4,ind5,ind6,ind7,ind8))
    x <- ""
    if (length(index) > 0) {
      # x=paste0(trim.leading((affL[index])),collapse=",")
      x <- paste0(trim.leading(index), collapse = ";")
      x <- gsub(" ,", ";", x)
    }
    return(x)
  })
  AFFL <- unlist(AFFL)
  M$AU1_UN <- AFFL
  M$AU1_UN <- gsub("\\\\&", "AND", M$AU1_UN)
  M$AU1_UN <- gsub("\\&", "AND", M$AU1_UN)

  ## identification of NR affiliations
  M$AU_UN_NR <- NA
  listAFF2 <- strsplit(M$AU_UN, sep)
  cont <- lapply(listAFF2, function(l) {
    l <- unlist(l)
    ind <- which(l %in% "NR")
  })

  for (i in 1:length(cont)) {
    if (length(cont[[i]]) > 0) {
      M$AU_UN_NR[i] <- paste(trim(listAFF[[i]][cont[[i]]]), collapse = ";")
    }
  }
  M$AU_UN[is.na(AFF)] <- NA
  M$AU_UN[M$AU_UN == "NOTDECLARED"] <- NA
  M$AU_UN[M$AU_UN == "NOTREPORTED"] <- NA
  M$AU_UN <- gsub("NOTREPORTED;", "", M$AU_UN)
  M$AU_UN <- gsub(";NOTREPORTED", "", M$AU_UN)
  M$AU_UN <- gsub("NOTDECLARED;", "", M$AU_UN)
  M$AU_UN <- gsub("NOTDECLARED", "", M$AU_UN)

  return(M)
}


### tools
lastChar <- function(C, last = ".") {
  A <- substr(C, nchar(C), nchar(C))
  ind <- which((A != last) & (!is.na(A)))
  C[ind] <- paste0(C[ind], last)
  return(C)
}

removeLastChar <- function(C, last = ".") {
  A <- substr(C, nchar(C), nchar(C))
  ind <- which((A == last) & (!is.na(A)))
  C[ind] <- substr(C[ind], 1, (nchar(C[ind]) - 1))
  return(C)
}
### remove non interesting field
ND <- function(affL, indd) {
  aff <- affL[!grepl("[[:digit:]]", affL)]
  ind <- indd[!grepl("[[:digit:]]", affL[indd])]
  cond <- length(ind) < 1
  r <- list(affL = aff[ind[1]], cond = cond)
  return(r)
}

# Test per missingData, metaTagExtraction, normalizeCitationScore

test_that("missingData restituisce informazioni sulla completezza", {
  M <- load_wos_fixture()
  res <- missingData(M)
  expect_type(res, "list")
  expect_true(all(c("allTags", "mandatoryTags") %in% names(res)))
  expect_true(is.data.frame(res$allTags))
  expect_true(is.data.frame(res$mandatoryTags))
  expect_true("status" %in% names(res$mandatoryTags))
  valid_statuses <- c("Excellent", "Good", "Acceptable", "Poor", "Critical", "Completely missing")
  expect_true(all(res$mandatoryTags$status %in% valid_statuses))
})

test_that("missingData funziona con dati Scopus", {
  M <- load_scopus_fixture()
  res <- missingData(M)
  expect_type(res, "list")
  expect_true(is.data.frame(res$mandatoryTags))
})

test_that("metaTagExtraction estrae CR_AU", {
  M <- load_wos_fixture()
  M2 <- metaTagExtraction(M, Field = "CR_AU", sep = ";")
  expect_true("CR_AU" %in% names(M2))
})

test_that("metaTagExtraction estrae CR_SO", {
  M <- load_wos_fixture()
  M2 <- metaTagExtraction(M, Field = "CR_SO", sep = ";")
  expect_true("CR_SO" %in% names(M2))
})

test_that("metaTagExtraction estrae AU_CO (paese autore)", {
  M <- load_wos_fixture()
  M2 <- metaTagExtraction(M, Field = "AU_CO", sep = ";")
  expect_true("AU_CO" %in% names(M2))
})

test_that("metaTagExtraction estrae AU1_CO (paese primo autore)", {
  M <- load_wos_fixture()
  M2 <- metaTagExtraction(M, Field = "AU1_CO", sep = ";")
  expect_true("AU1_CO" %in% names(M2))
})

test_that("metaTagExtraction estrae SR (short reference)", {
  M <- load_wos_fixture()
  M2 <- suppressWarnings(metaTagExtraction(M, Field = "SR", sep = ";"))
  expect_true("SR" %in% names(M2))
})

test_that("normalizeCitationScore calcola NCS", {
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  NCS <- expect_no_error(
    suppressWarnings(suppressMessages(
      normalizeCitationScore(scientometrics, field = "documents", impact.measure = "global")
    ))
  )
  expect_true(is.data.frame(NCS))
})

utils::globalVariables(c(
  "nLABEL", "LABEL", "GCS", "CITING", "CIT_PY", "paper", "TC",
  "AU", "Page.start", "Page.end", "PP", "SR", "Included",
  "PP.y", "PP.x", "toRemove", "SR_cited", "LCS", "SR_FULL",
  "TI", "DE", "ID", "DI", "Year", "SR_citing", "ref", "n",
  "id_oa", "UT", "PY","TI_clean","CR_clean","is_match", "cited_SR", "citing_SR"
))
#' Historical co-citation network
#'
#' \code{histNetwork} creates a historical citation network from a bibliographic
#' data frame.
#'
#' @param M is a bibliographic data frame obtained by the converting function
#'   \code{\link{convert2df}}. It is a data matrix with cases corresponding to
#'   manuscripts and variables to Field Tag in the original SCOPUS, OpenAlex, Lens.org and Clarivate
#'   Analytics Web of Science file.
#' @param min.citations DEPRECATED. New algorithm does not use this parameters. It will be remove in the next version of bibliometrix.
#' @param sep is the field separator character. This character separates strings
#'   in CR column of the data frame. The default is \code{sep = ";"}.
#' @param network is logical. If TRUE, function calculates and returns also the direct citation network. If FALSE,
#' the function returns only the local citation table.
#' @param verbose is logical. If TRUE, results are printed on screen.
#' @return \code{histNetwork} returns an object of \code{class} "list"
#'   containing the following components:
#'
#'   \tabular{lll}{ NetMatrix \tab  \tab the historical co-citation network
#'   matrix\cr histData \tab      \tab the set of n most cited references\cr M
#'   \tab      \tab the bibliographic data frame}
#'
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' histResults <- histNetwork(management, sep = ";")
#' }
#'
#' @seealso \code{\link{convert2df}} to import and convert a supported
#'   export file in a bibliographic data frame.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#'
#' @export

histNetwork <- function(M, min.citations, sep = ";", network = TRUE, verbose = TRUE) {
  min.citations <- 0
  db <- M$DB[1]
  if (!("DI" %in% names(M))) {
    M$DI <- ""
  } else {
    M$DI[is.na(M$DI)] <- ""
  }
  has_cr <- "CR" %in% names(M) && any(!is.na(M$CR) & M$CR != "")
  has_crids <- "CRids" %in% names(M) && any(!is.na(M$CRids) & M$CRids != "")
  if (!has_cr && !has_crids) {
    cat("\nYour collection does not contain Cited References metadata (Field CR is missing)\n")
    return(NA)
  }

  M$TC[is.na(M$TC)] <- 0

  if (db == "ISI") db <- "WOS"
  switch(db,
    WOS = {
      results <- wos(M = M, min.citations = min.citations, sep = sep, network = network, verbose = verbose)
    },
    SCOPUS = {
      results <- scopus(M = M, min.citations = min.citations, sep = sep, network = network, verbose = verbose)
    },
    OPENALEX = {
      results <- openalex(M = M, min.citations = min.citations, sep = sep, network = network, verbose = verbose)
    },
    LENS = {
      results <- lens(M = M, min.citations = min.citations, sep = sep, network = network, verbose = verbose)
    },
    {
      cat("\nDatabase not compatible with direct citation analysis\n")
    }
  )

  return(results)
}


wos <- function(M, min.citations, sep, network, verbose) {
  if (isTRUE(verbose)) {
    cat("\nWOS DB:\nSearching local citations (LCS) by reference items (SR) and DOIs...\n")
  }
  if (!("SR_FULL" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "SR")
  }

  M <- M[order(M$PY), ]
  M$Paper <- 1:nrow(M)
  M_orig <- M
  M$nLABEL <- 1:nrow(M)
  # papers <- M$nLABEL[M$TC >= min.citations]

  # Reference list and citing papers
  CR <- strsplit(M$CR, sep)

  CR <- lapply(seq_along(CR), function(i) {
    l <- data.frame(
      ref = CR[[i]],
      paper = i
    )
  })
  CR <- (do.call(rbind, CR))
  CR$DI <-
    trimws(unlist(lapply(
      strsplit(CR$ref, "DOI", fixed = TRUE), "[", 2
    )))
  CR$DI[is.na(CR$DI) | CR$DI == "NA"] <- ""
  CR$AU <-
    trimws(gsub("[ ]{2,}", "", (gsub(
      "\\.", " ", unlist(lapply(strsplit(CR$ref, ",", fixed = TRUE), "[", 1))
    ))))
  CR$PY <-
    trimws(unlist(lapply(strsplit(CR$ref, ",", fixed = TRUE), "[", 2)))
  CR$SO <-
    trimws(unlist(lapply(strsplit(CR$ref, ",", fixed = TRUE), "[", 3)))
  CR$SR <- paste(CR$AU, ", ", CR$PY, ", ", CR$SO, sep = "")

  if (isTRUE(verbose)) {
    cat("\nAnalyzing", nrow(CR), "reference items...\n")
  }
  # Local cited documents by DOI and reference item
  # M=M[papers,]
  M$LABEL <- paste(M$SR_FULL, "DOI", toupper(M$DI))

  CR$LABEL <- paste(CR$SR, "DOI", CR$DI)

  # By reference
  L <- left_join(M, CR, by = c("LABEL"))
  L <- L[!is.na(L$paper), ]
  L$CITING <- M$LABEL[L$paper]
  L$nCITING <- M$nLABEL[L$paper]
  L$CIT_PY <- M$PY[L$paper]

  LCS <- L %>%
    group_by(nLABEL) %>%
    summarize(
      LABEL = LABEL[1],
      n = length(nLABEL)
    ) %>%
    as.data.frame()

  M$LCS <- 0
  M[LCS$nLABEL, "LCS"] <- LCS$n
  M_orig$LCS <- M$LCS

  histData <- M[c("LABEL", "TI", "DE", "ID", "DI", "PY", "LCS", "TC")]
  names(histData) <- c("Paper", "Title", "Author_Keywords", "KeywordsPlus", "DOI", "Year", "LCS", "GCS")
  histData <- histData %>%
    dplyr::filter(GCS >= min.citations)

  if (isTRUE(network)) {
    # Citing data frame
    CITING <- L %>%
      group_by(CITING) %>%
      summarize(
        LCR = paste(LABEL, collapse = ";"),
        PY = CIT_PY[1],
        Paper = paper[1]
      ) %>%
      ungroup() %>%
      arrange(PY) %>%
      as.data.frame()

    M_orig$LCR <- NA
    M_orig$LCR[CITING$Paper] <- CITING$LCR
    M_orig$LABEL <- M$LABEL
    M <- M_orig %>%
      dplyr::filter(TC >= min.citations)


    ## assign an unique name to each document
    st <- i <- 0
    while (st == 0) {
      ind <- which(duplicated(M$LABEL))
      if (length(ind) > 0) {
        i <- i + 1
        M$LABEL[ind] <- paste0(M$LABEL[ind], "-", letters[i], sep = "")
      } else {
        st <- 1
      }
    }
    row.names(M) <- M$LABEL

    # NetMatrix
    WLCR <- cocMatrix(M, "LCR", sep = ";")
    missingLABEL <- setdiff((M$LABEL), colnames(WLCR))
    colLab <- c(colnames(WLCR), missingLABEL)
    WLCR <- cbind(WLCR, matrix(0, nrow(WLCR), length(missingLABEL)))
    WLCR <- as.data.frame(as.matrix(WLCR))
    colnames(WLCR) <- colLab
    LABEL <- (row.names(WLCR))
    WLCR <- as.matrix(WLCR[LABEL])
    # row.names(WLCR) <- LABEL
  } else {
    WLCR <- NULL
  }

  if (isTRUE(verbose)) {
    cat(
      "\nFound",
      length(M$LCS[M$LCS > 0]),
      "documents with no empty Local Citations (LCS)\n"
    )
  }
  results <-
    list(
      NetMatrix = WLCR,
      histData = histData,
      M = M_orig,
      LCS = M$LCS
    )

  return(results)
}

# New algorithm for Scopus
scopus <- function(M, min.citations, sep, network, verbose) {
  if (!("SR_FULL" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "SR")
  }
  
  CR <- match_citations_fast(
    titles_df=M %>% select(SR,TI),
    references_df=M %>% select(SR,CR)
  )

  LCS <- CR %>% 
    group_by(SR_cited) %>%
    count(name = "LCS")

  M <- M %>%
    left_join(LCS, by = c("SR" = "SR_cited")) %>%
    mutate(LCS = ifelse(is.na(LCS), 0, LCS))

  histData <- M %>%
    select(SR_FULL, TI, DE, ID, DI, PY, LCS, TC) %>%
    rename(
      Paper = SR_FULL,
      Title = TI,
      Author_Keywords = DE,
      KeywordsPlus = ID,
      DOI = DI,
      Year = PY,
      GCS = TC
    ) %>%
    arrange(Year) %>%
    as.data.frame()

  names(histData) <- c("Paper", "Title", "Author_Keywords", "KeywordsPlus", "DOI", "Year", "LCS", "GCS")

  if (isTRUE(network)) {
    CRadd <- data.frame(SR_citing = unique(M$SR), SR_cited = unique(M$SR), value = 1)

    WLCR <- CR %>%
      select(SR_citing, SR_cited) %>%
      mutate(value = 1) %>%
      bind_rows(CRadd) %>%
      distinct() %>%
      pivot_wider(names_from = "SR_cited", values_from = "value", values_fill = 0) %>%
      dplyr::filter(SR_citing %in% CRadd$SR_cited)

    SRrow <- WLCR$SR_citing
    SRcol <- colnames(WLCR)[-1]

    WLCR <- as.matrix(WLCR %>% select(-1))
    row.names(WLCR) <- SRrow
    colnames(WLCR) <- SRcol
  } else {
    WLCR <- NULL
  }

  if (isTRUE(verbose)) {
    cat(
      "\nFound",
      length(M$LCS[M$LCS > 0]),
      "documents with no empty Local Citations (LCS)\n"
    )
  }

  results <-
    list(
      NetMatrix = WLCR,
      histData = histData,
      M = M,
      LCS = M$LCS
    )
}

openalex <- function(M, min.citations = min.citations, sep = sep, network = network, verbose = verbose) {
  ids <- M$id_oa

  # Use CRids (OpenAlex IDs) if available, otherwise fall back to CR
  if ("CRids" %in% names(M) && any(!is.na(M$CRids) & M$CRids != "")) {
    cr_col <- M$CRids
  } else {
    cr_col <- M$CR
  }
  cr_col[is.na(cr_col) | cr_col == ""] <- "none"

  CR <- strsplit(cr_col, ";")
  CR <- data.frame(id_oa = rep(M$id_oa, lengths(CR)), ref = trimws(unlist(CR))) %>%
    dplyr::filter(ref %in% ids)

  LCS <- CR %>%
    count(id_oa = ref) %>%
    rename(LCS = n)

  histData <- M %>%
    left_join(LCS, by = c("id_oa")) %>%
    mutate(
      LCS = ifelse(is.na(LCS), 0, LCS),
      DE = ID
    ) %>%
    rename(
      LABEL = SR,
      GCS = TC
    ) %>%
    select(c("LABEL", "TI", "DE", "ID", "DI", "PY", "LCS", "GCS")) %>%
    dplyr::filter(GCS >= min.citations)

  names(histData) <- c("Paper", "Title", "Author_Keywords", "KeywordsPlus", "DOI", "Year", "LCS", "GCS")

  if (isTRUE(network)) {
    CRadd <- data.frame(id_oa = unique(M$id_oa), ref = unique(M$id_oa), value = 1)
    WLCR <- CR %>%
      mutate(value = 1) %>%
      bind_rows(CRadd) %>%
      distinct() %>%
      pivot_wider(names_from = "ref", values_from = "value", values_fill = 0) %>%
      dplyr::filter(id_oa %in% CRadd$ref)

    SRrow <- WLCR %>%
      select(id_oa) %>%
      left_join(
        M %>%
          select(id_oa, SR),
        by = "id_oa"
      )

    SR_col <- data.frame(id_oa = colnames(WLCR)[-1]) %>%
      left_join(
        M %>%
          select(id_oa, SR),
        by = "id_oa"
      )

    WLCR <- as.matrix(WLCR %>% select(-1))
    row.names(WLCR) <- SRrow$SR
    colnames(WLCR) <- SR_col$SR
    WLCR <- WLCR[colnames(WLCR), colnames(WLCR)]
  } else {
    WLCR <- NULL
  }

  results <-
    list(
      NetMatrix = WLCR,
      histData = histData,
      M = M %>%
        left_join(LCS, by = "id_oa") %>%
        replace_na(list(LCS = 0)),
      LCS = M$LCS
    )
}

lens <- function(M, min.citations = min.citations, sep = sep, network = network, verbose = verbose) {
  M$CR[is.na(M$CR)] <- "none"
  ids <- M$UT
  CR <- lapply(strsplit(M$CR, ";"), trimws)
  CR <- data.frame(UT = rep(M$UT, lengths(CR)), ref = unlist(CR)) %>%
    dplyr::filter(ref %in% ids)

  LCS <- CR %>%
    count(UT = ref) %>%
    rename(LCS = n)

  histData <- M %>%
    left_join(LCS, by = c("UT")) %>%
    mutate(
      LCS = ifelse(is.na(LCS), 0, LCS),
      DE = ID
    ) %>%
    rename(
      LABEL = SR,
      GCS = TC
    ) %>%
    select(c("LABEL", "TI", "DE", "ID", "DI", "PY", "LCS", "GCS")) %>%
    dplyr::filter(GCS >= min.citations)

  names(histData) <- c("Paper", "Title", "Author_Keywords", "KeywordsPlus", "DOI", "Year", "LCS", "GCS")

  if (isTRUE(network)) {
    CRadd <- data.frame(UT = unique(M$UT), ref = unique(M$UT), value = 1)
    WLCR <- CR %>%
      mutate(value = 1) %>%
      bind_rows(CRadd) %>%
      distinct() %>%
      pivot_wider(names_from = "ref", values_from = "value", values_fill = 0) %>%
      dplyr::filter(UT %in% CRadd$ref)

    SRrow <- WLCR %>%
      select(UT) %>%
      left_join(
        M %>%
          select(UT, SR),
        by = "UT"
      )

    SR_col <- data.frame(UT = colnames(WLCR)[-1]) %>%
      left_join(
        M %>%
          select(UT, SR),
        by = "UT"
      )

    WLCR <- as.matrix(WLCR %>% select(-1))
    row.names(WLCR) <- SRrow$SR
    colnames(WLCR) <- SR_col$SR
    WLCR <- WLCR[colnames(WLCR), colnames(WLCR)]
  } else {
    WLCR <- NULL
  }

  results <-
    list(
      NetMatrix = WLCR,
      histData = histData,
      M = M %>%
        left_join(LCS, by = "UT") %>%
        replace_na(list(LCS = 0)),
      LCS = M$LCS
    )
}

# Funzione alternativa più veloce per dataset molto grandi
# match_citations_fast <- function(titles_df, references_df) {
#   
#   # Normalizza
#   titles_norm <- titles_df %>%
#     mutate(TI_clean = normalize_text(TI))
#   
#   refs_norm <- references_df %>%
#     mutate(CR_clean = normalize_text(CR))
#   
#   # Crea una matrice di matching usando stringdist
#   results <- expand_grid(
#     SR_cited = titles_norm$SR,
#     SR_citing = refs_norm$SR
#   ) %>%
#     left_join(titles_norm %>% select(SR, TI_clean), by = c("SR_cited" = "SR")) %>%
#     left_join(refs_norm %>% select(SR, CR_clean), by = c("SR_citing" = "SR")) %>%
#     mutate(
#       is_match = str_detect(CR_clean, fixed(TI_clean))
#     ) %>%
#     filter(is_match) %>%
#     select(SR_cited, SR_citing)
#   
#   return(results)
# }
# 
# # Funzione to normalize text
# normalize_text <- function(text) {
#   text %>%
#     str_to_upper() %>%
#     str_replace_all("[^A-Z0-9\\s]", " ") %>%
#     str_squish()
# }
match_citations_fast <- function(titles_df, references_df) {
  
  # Normalizza i titoli
  titles_norm <- titles_df %>%
    dplyr::mutate(TI_clean = stringr::str_to_upper(TI) %>%
                    stringr::str_replace_all("[^A-Z0-9\\s]", " ") %>%
                    stringr::str_squish())
  
  # Normalizza le bibliografie
  refs_norm <- references_df %>%
    dplyr::mutate(CR_clean = stringr::str_to_upper(CR) %>%
                    stringr::str_replace_all("[^A-Z0-9\\s]", " ") %>%
                    stringr::str_squish())
  
  # Crea tutte le possibili combinazioni
  all_combinations <- tidyr::expand_grid(
    cited_SR = titles_norm$SR,
    citing_SR = refs_norm$SR
  )
  
  # Aggiungi i titoli normalizzati
  all_combinations <- dplyr::left_join(
    all_combinations, 
    titles_norm %>% dplyr::select(SR, TI_clean), 
    by = c("cited_SR" = "SR")
  )
  
  # Aggiungi le bibliografie normalizzate
  all_combinations <- dplyr::left_join(
    all_combinations,
    refs_norm %>% dplyr::select(SR, CR_clean), 
    by = c("citing_SR" = "SR")
  )
  
  # Trova i match
  all_combinations$is_match <- stringr::str_detect(
    all_combinations$CR_clean, 
    stringr::fixed(all_combinations$TI_clean)
  )
  
  # Filtra solo i match
  matches <- all_combinations %>%
    dplyr::filter(is_match) %>%
    dplyr::select(cited_SR, citing_SR) %>% 
    rename("SR_cited" = cited_SR, "SR_citing" = citing_SR)
  
  return(matches)
}

#' Get Author Content on SCOPUS by ID
#'
#' Uses SCOPUS API search to get information about documents on a set of authors using SCOPUS ID.
#'
#' @param id is a vector of characters containing the author's SCOPUS IDs.
#' SCOPUS IDs con be obtained using the function \code{\link{idByAuthor}}.
#' @param api_key is a character. It contains the Elsvier API key. Information about how to obtain an API Key \href{https://dev.elsevier.com/sc_apis.html}{Elsevier API website}
#' @param remove.duplicated is logical. If TRUE duplicated documents will be deleted from the bibliographic collection.
#' @param country is logical. If TRUE authors' country information will be downloaded from SCOPUS.
#' @return a list containing two objects: (i) M which is a data frame with cases corresponding to articles and variables to main Field Tags named using the standard ISI WoS Field Tag codify.
#' M includes the entire bibliographic collection downloaded from SCOPUS.
#' The main field tags are:
#'
#' \tabular{lll}{
#' \code{AU}\tab   \tab Authors\cr
#' \code{TI}\tab   \tab Document Title\cr
#' \code{SO}\tab   \tab Publication Name (or Source)\cr
#' \code{DT}\tab   \tab Document Type\cr
#' \code{DE}\tab   \tab Authors' Keywords\cr
#' \code{ID}\tab   \tab Keywords associated by SCOPUS or ISI database \cr
#' \code{AB}\tab   \tab Abstract\cr
#' \code{C1}\tab   \tab Author Address\cr
#' \code{RP}\tab   \tab Reprint Address\cr
#' \code{TC}\tab   \tab Times Cited\cr
#' \code{PY}\tab   \tab Year\cr
#' \code{UT}\tab   \tab Unique Article Identifier\cr
#' \code{DB}\tab   \tab Database\cr}
#' (ii) authorDocuments which is a list containing a bibliographic data frame for each author.
#'
#' LIMITATIONS:
#' Currently, SCOPUS API does not allow to download document references.
#' As consequence, it is not possible to perform co-citation analysis (the field CR is empty).
#'
#' @examples
#' ## Request a personal API Key to Elsevier web page https://dev.elsevier.com/sc_apis.html
#'
#' ## api_key="your api key"
#'
#' ## create a data frame with the list of authors to get information and IDs
#' # i.e. df[1,1:3] <- c("aria","massimo","naples")
#' #      df[2,1:3] <- c("cuccurullo","corrado", "naples")
#'
#' ## run idByAuthor function
#' #
#' # authorsID <- idByAuthor(df, api_key)
#' #
#'
#' ## extract the IDs
#' #
#' # id <- authorsID[,3]
#' #
#'
#' ## create the bibliographic collection
#' #
#' # res <- retrievalByAuthorID(id, api_key)
#' #
#' # M <- res$M  # the entire bibliographic data frame
#' # M <- res$authorDocuments # the list containing a bibliographic data frame for each author
#'
#' @seealso \code{\link{idByAuthor}} for downloading author information and SCOPUS ID.
#'
#' @export
#'
retrievalByAuthorID <- function(id, api_key, remove.duplicated = TRUE, country = TRUE) {
  id <- id[!is.na(id)]
  M_list <- list()
  n <- length(id)
  nomi <- c(
    "au_id", "name", "affil_id", "affilname", "n_auth", "n_affils", "citations", "journal", "description",
    "title", "pii", "doi", "eid", "cover_date", "cover_display_date", "prism_url", "dc_identifier",
    "dc_creator", "prism_issn", "prism_eIssn", "prism_pageRange", "dc_description", "prism_aggregationType",
    "subtype", "authkeywords", "source_id"
  )
  M <- data.frame(matrix(NA, 1, length(nomi)))
  names(M) <- nomi

  for (j in 1:n) {
    AU_ID <- id[j]
    cat("\n Query n. ", j, "   Author ID: ", AU_ID)
    ### documents of an author

    AU_S <- tryCatch(
      author_df_orig(
        au_id = AU_ID,
        api_key = api_key,
        all_author_info = TRUE,
        verbose = FALSE
      ),
      error = function(e) err <- 1
    )

    if (!is.numeric(class(AU_S))) {
      AU_S$cover_date <- substr(as.character(AU_S$cover_date), 1, 4)

      for (i in 1:dim(AU_S)[2]) {
        if (is.factor(AU_S[[i]])) {
          AU_S[[i]] <- as.character(AU_S[[i]])
        }
      }

      M_AU <- data.frame(AU_S)

      if (dim(M_AU)[2] <= dim(M)[2]) {
        M_AU[setdiff(names(M), names(M_AU))] <- NA
      }
      M <- rbind(M, M_AU[names(M)])
      M_list[[j]] <- M_AU
      names(M_list)[j] <- id[j]
    } else {
      cat("\n Error in id:", AU_ID, "retrieval\n")
    }
  }
  M <- M[-1, ] ### remove first empty row
  names(M) <- c("AU_ID", "AU", "C1_ID", "C1", "nAU", "nC1", "TC", "SO", "DT", "TI", "PII", "DI", "EID", "PY", "CDD", "URL", "UT", "AU1", "ISSN", "EISSN", "PAG", "AB", "PT", "SUBTYPE", "DE", "SO_ID")
  if (isTRUE(remove.duplicated)) {
    d <- duplicated(gsub("[^[:alnum:] ]", "", M$UT))
    cat("\n", sum(d), "duplicated documents have been removed\n")
    M <- M[!d, ]
  }
  M$CR <- NA
  M$DB <- "SCOPUS"
  M$DE <- gsub("\\| ", ";", M$DE)
  M$ID <- M$DE


  ### da rivedere ###
  if (isTRUE(country)) {
    M$AU_CO <- paste(M$C1_ID, ";", sep = "")

    ### country retrieval
    cat("\nAuthors' country retrieval\n\n")
    aff_id <- sort(unique(unlist(strsplit(M$C1_ID, ";"))))
    aff_id <- aff_id[nchar(aff_id) > 1]

    AFF <- data.frame(ID = NA, NAME = NA, CO = NA)
    for (i in 1:length(aff_id)) {
      a <- affiliation_retrieval(aff_id[i], api_key = api_key, verbose = FALSE)
      AFF[i, 1] <- aff_id[i]
      if (length(a$content$`affiliation-retrieval-response`$`affiliation-name`) > 0) {
        AFF[i, 2] <- a$content$`affiliation-retrieval-response`$`affiliation-name`
      }
      if (length(a$content$`affiliation-retrieval-response`$country) > 0) {
        AFF[i, 3] <- a$content$`affiliation-retrieval-response`$country
      }
      cat("\nAffiliation ID: ", AFF[i, 1], "   Name: ", AFF[i, 2], ",", AFF[i, 3])
      M$AU_CO <- gsub(paste(aff_id[i], ";", sep = ""), paste(AFF[i, 3], ";", sep = ""), M$AU_CO)
    }
    M$AU_CO <- gsub(";;", ";", M$AU_CO)
    M$AU_CO[nchar(M$AU_CO) < 3] <- NA
    M$AU1_CO <- unlist(lapply(strsplit(M$AU_CO, ";"), function(l) {
      l <- l[1]
    }))
    UN <- strsplit(M$C1, ";")
    CO <- strsplit(M$AU_CO, ";")

    for (i in 1:length(UN)) {
      M$C1[i] <- paste(paste(UN[[i]], ", ", CO[[i]], sep = ""), collapse = ";")
    }
    #########
  }
  M <- data.frame(lapply(M, toupper))
  M$TC <- as.numeric(M$TC)
  M$PY <- as.numeric(M$PY)
  M$DB <- "SCOPUS"
  M$RP <- unlist(lapply(strsplit(M$C1, ";"), function(l) {
    l <- l[1]
  }))
  M$CR <- NA
  M$J9 <- M$JI <- M$SO
  ### SR field creation
  suppressWarnings(M <- metaTagExtraction(M, Field = "SR"))
  ### identify duplicated SRs
  SR <- M$SR
  tab <- table(SR)
  tab2 <- table(tab)
  ind <- as.numeric(names(tab2))
  ind <- ind[which(ind > 1)]
  if (length(ind) > 0) {
    for (i in ind) {
      indice <- names(which(tab == i))
      for (j in indice) {
        indice2 <- which(SR == j)
        SR[indice2] <- paste(SR[indice2], as.character(1:length(indice2)), sep = " ")
      }
    }
  }

  row.names(M) <- SR

  results <- list(M = M, authorDocuments = M_list)
  return(results)
}

utils::globalVariables(c("TC", "AUs", "PY", "h_index", "PY_start", "Element"))
#' h-index calculation
#'
#' It calculates the authors' h-index and its variants.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param field is character. It can be equal to c("author", "source"). field indicates if H-index have to be calculated for a list of authors or for a list of sources. Default
#' value is \code{field = "author"}.
#' @param elements is a character vector. It contains the authors' names list or the source list for which you want to calculate the H-index. When the field is
#' "author", the argument has the form C("SURNAME1 N","SURNAME2 N",...), in other words, for each author: surname and initials separated by one blank space. If elements=NULL, the function calculates impact indices for all elements contained in the data frame.
#' i.e for the authors SEMPRONIO TIZIO CAIO and ARIA MASSIMO \code{elements} argument is \code{elements = c("SEMPRONIO TC", "ARIA M")}.
#' @param sep is the field separator character. This character separates authors in each string of AU column of the bibliographic data frame. The default is \code{sep = ";"}.
#' @param years is a integer. It indicates the number of years to consider for Hindex calculation. Default is Inf.
#' @return an object of \code{class} "list". It contains two elements: H is a data frame with h-index, g-index and m-index for each author; CitationList is a list with the bibliographic collection for each author.
#'
#'
#' @examples
#'
#' ### EXAMPLE 1: ###
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' authors <- c("SMALL H", "CHEN DZ")
#'
#' Hindex(scientometrics, field = "author", elements = authors, sep = ";")$H
#'
#' Hindex(scientometrics, field = "source", elements = "SCIENTOMETRICS", sep = ";")$H
#'
#' ### EXAMPLE 2: Garfield h-index###
#'
#' data(garfield, package = "bibliometrixData")
#'
#' indices <- Hindex(garfield, field = "author", elements = "GARFIELD E", years = Inf, sep = ";")
#'
#' # h-index, g-index and m-index of Eugene Garfield
#' indices$H
#'
#' # Papers and total citations
#' head(indices$CitationList[[1]])
#'
#' @seealso \code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export

Hindex <- function(M, field = "author", elements = NULL, sep = ";", years = Inf) {
  # elements=paste("\\\b",elements,"\\\b",sep="")
  M$TC <- as.numeric(M$TC)
  M$PY <- as.numeric(M$PY)
  M <- M %>% dplyr::filter(!is.na(TC) | !is.na(TC))
  # M <- M[M$TC>0,]

  Today <- as.numeric(substr(Sys.time(), 1, 4))
  past <- Today - years
  if (min(M$PY, na.rm = TRUE) < past) {
    M <- M[M$PY >= past, ]
  }

  switch(field,
    author = {
      AU <- M$AU
      AU <- trimES(gsub(",", " ", AU))
      listAU <- strsplit(AU, split = sep)
      l <- lengths(listAU)
      index <- rep(row.names(M), l)
      df <- M[index, ]
      df$AUs <- trimws(unlist(listAU))
    },
    source = {
      df <- M
      df$AUs <- M$SO
    }
  )
  rm(M)

  h_calc <- function(x) {
    h <- tail(which(1:length(x) <= sort(x, decreasing = T)), 1) # [1]-1
    return(h)
  }

  g_calc <- function(x) {
    g <- tail(which(1:length(x) <= cummean(sort(x, decreasing = T))), 1)
    return(g)
  }

  H <- df %>%
    group_by(AUs) %>%
    reframe(
      h_index = h_calc(TC),
      g_index = g_calc(TC),
      PY_start = min(PY),
      TC = sum(TC),
      NP = length(AUs)
    ) %>%
    mutate(
      m_index = h_index / (Today - PY_start + 1)
    ) %>%
    rename(Element = AUs) %>%
    as.data.frame()

  if (!is.null(elements)) {
    H <- H %>%
      dplyr::filter(Element %in% elements) %>%
      dplyr::select("Element", "h_index", "g_index", "m_index", "TC", "NP", "PY_start")

    df <- df %>% dplyr::filter(AUs %in% elements)
  } else {
    H <- H %>%
      dplyr::select("Element", "h_index", "g_index", "m_index", "TC", "NP", "PY_start")
  }


  # H <- H[c("Element","h_index","g_index","m_index","TC","NP","PY_start")]

  vars <- intersect(c("AU", "SO", "PY", "TC", "DI"), names(df))


  CitationList <- split(df[vars], f = df$AUs)

  results <- list(H = H, CitationList = CitationList)

  return(results)
}

#' Calculate similarity indices
#'
#' It calculates a relative measure of bibliographic co-occurrences.
#'
#' \code{couplingSimilarity} calculates Association strength, Inclusion, Jaccard or Salton similarity from a co-occurrence bibliographic matrix.
#'
#' The association strength is used by Van Eck and Waltman (2007) and Van Eck et al. (2006). Several works refer to the measure as the proximity index,
#' while Leydesdorff (2008)and Zitt et al. (2000) refer to it as the probabilistic affinity (or activity) index.
#'
#' The inclusion index, also called Simpson coefficient, is an overlap measure used in information retrieval.
#'
#' The Jaccard index (or Jaccard similarity coefficient) gives us a relative measure of the overlap of two sets.
#' It is calculated as the ratio between the intersection and the union of the reference lists (of two manuscripts).
#'
#' The Salton index, instead, relates the intersection of the two lists to the geometric mean of the size of both sets.
#' The square of Salton index is also called Equivalence index.
#'
#' The indices are equal to zero if the intersection of the reference lists is empty.\cr\cr
#'
#' References\cr\cr
#' Leydesdorff, L. (2008). On the normalization and visualization of author Cocitation data: Salton's cosine versus the Jaccard index.
#' Journal of the American Society for Information Science and Technology, 59(1), 77– 85.\cr
#' Van Eck, N.J., Waltman, L., Van den Berg, J., & Kaymak, U. (2006). Visualizing the computational intelligence field.
#' IEEE Computational Intelligence Magazine, 1(4), 6– 10.\cr
#' Van Eck, N.J., & Waltman, L. (2007). Bibliometric mapping of the computational intelligence field.
#' International Journal of Uncertainty, Fuzziness and Knowledge-Based Systems, 15(5), 625– 645\cr.
#' Van Eck, N. J., & Waltman, L. (2009). How to normalize cooccurrence data? An analysis of some well-known similarity measures.
#' Journal of the American society for information science and technology, 60(8), 1635-1651.\cr
#' Zitt, M., Bassecoulard, E., & Okubo, Y. (2000). Shadows of the past in international cooperation:
#' Collaboration profiles of the top five producers of science. Scientometrics, 47(3), 627– 657.\cr
#'
#'
#' @param NetMatrix is a coupling matrix obtained by the network functions \code{\link{biblioNetwork}} or \code{\link{cocMatrix}}.
#' @param type is a character. It can be "association", "jaccard", "inclusion","salton" or "equivalence" to obtain Association Strength, Jaccard,
#' Inclusion, Salton or Equivalence similarity index respectively. The default is \code{type = "association"}.
#' @return a similarity matrix.
#'
#'
#'
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#' NetMatrix <- biblioNetwork(scientometrics,
#'   analysis = "co-occurrences",
#'   network = "keywords", sep = ";"
#' )
#' S <- normalizeSimilarity(NetMatrix, type = "association")
#'
#' @seealso \code{\link{biblioNetwork}} function to compute a bibliographic network.
#' @seealso \code{\link{cocMatrix}} to compute a bibliographic bipartite network.
#'
#' @export

normalizeSimilarity <- function(NetMatrix, type = "association") {
  diag <- Matrix::diag
  D <- diag(NetMatrix)
  # S=NetMatrix
  switch(type,
    association = {
      S <- NetMatrix / ((outer(D, D, "*")))
    },
    inclusion = {
      S <- NetMatrix / outer(D, D, function(a, b) {
        mapply(min, a, b)
      })
    },
    jaccard = {
      S <- NetMatrix / (outer(D, D, "+") - NetMatrix)
    },
    salton = {
      S <- NetMatrix / (sqrt(outer(D, D, "*")))
    },
    equivalence = {
      S <- (NetMatrix / sqrt(outer(D, D, "*")))^2
    }
  )

  S <- as.matrix(S)
  S[is.nan(S)] <- 0
  S <- Matrix(S, sparse = TRUE)
  # if (class(S)!="dgCMatrix"){S=as.matrix(S)}

  return(S)
}

#' Tabulate elements from a Tag Field column
#'
#' It tabulates elements from a Tag Field column of a bibliographic data frame.
#'
#' \code{tableTag} is an internal routine of main function \code{\link{biblioAnalysis}}.
#'
#' @param M is a data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.
#' @param Tag is a character object. It indicates one of the field tags of the
#'   standard ISI WoS Field Tag codify.
#' @param sep is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.
#' @param ngrams is an integer between 1 and 3. It indicates the type of n-gram to extract from titles or abstracts.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @return an object of class \code{table}
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#' Tab <- tableTag(scientometrics, Tag = "CR", sep = ";")
#' Tab[1:10]
#'
#' @export
tableTag <- function(M, Tag = "CR", sep = ";", ngrams = 1, remove.terms = NULL, synonyms = NULL) {
  ## check and remove duplicated
  M <- M[!duplicated(M$SR), ]

  if (Tag %in% c("AB", "TI")) {
    M <- termExtraction(M, Field = Tag, stemming = F, verbose = FALSE, ngrams = ngrams, remove.terms = remove.terms, synonyms = synonyms)
    i <- which(names(M) == paste(Tag, "_TM", sep = ""))
    remove.terms <- NULL
    synonyms <- NULL
  } else {
    i <- which(names(M) == Tag)
  }

  if (Tag == "C1") {
    M$C1 <- gsub("\\[.+?]", "", M$C1)
  }

  Tab <- unlist(strsplit(as.character(M %>% dplyr::pull(i)), sep))

  ### inserted to remove punct and extra spaces ####
  Tab <- trimws(gsub("\\s+|\\.|\\,", " ", Tab))
  ####
  # Tab<-Tab[Tab!=""]
  Tab <- Tab[nchar(Tab) > 0]

  # Merge synonyms in the vector synonyms
  if (length(synonyms) > 0 & is.character(synonyms)) {
    s <- strsplit(toupper(synonyms), ";")
    snew <- trimws(unlist(lapply(s, function(l) l[1])))
    sold <- (lapply(s, function(l) trimws(l[-1])))
    for (i in 1:length(s)) {
      Tab[Tab %in% unlist(sold[[i]])] <- snew[i]
    }
  }

  Tab <- sort(table(Tab), decreasing = TRUE)
  # remove terms from ID and DE
  if ((Tag %in% c("DE", "ID", "KW_Merged")) & (!is.null(remove.terms))) {
    term <- setdiff(names(Tab), toupper(remove.terms))
    Tab <- Tab[term]
  }

  return(Tab)
}

#' Index of ltwa.
#'
#' Data frame containing a normalized index of words used in journal names and their ISO4 abbreviations.
#'
#' @format A data frame with 56463 rows and 3 variables:
#'  \describe{
#'     \item{WORD}{word from journal names}
#'     \item{ABBREVIATION}{ISO4 abbreviation}
#'     \item{LANGUAGES}{Language of the journal name}
#'     }
#'
#' @name ltwa
NULL

utils::globalVariables(c("Paper", "Tag", "content", "cont"))
cochrane2df <- function(D) {
  D <- D[nchar(D) > 0] # remove empty rows

  Papers <- which(substr(D, 1, 8) == "Record #") # first row of each document
  nP <- length(Papers) # number of documents

  rowPapers <- diff(c(Papers, length(D) + 1))

  numPapers <- rep(1:nP, rowPapers)

  DATA <- data.frame(Tag = substr(D, 1, 4), content = substr(D, 5, nchar(D)), Paper = numPapers)
  DATA$Tag <- gsub(" ", "", gsub(":", "", DATA$Tag))
  df <- DATA %>%
    group_by(Paper, Tag) %>%
    summarise(cont = paste(content, collapse = "---", sep = "")) %>%
    arrange(Tag, Paper) %>%
    pivot_wider(names_from = Tag, values_from = cont) %>%
    ungroup() %>%
    rename(
      "PY" = "YR",
      "UT" = "ID",
      "ID" = "KY",
      "URL" = "US",
      "DI" = "DOI",
      "NR" = "NO"
    )
  df <- as.data.frame(df)

  df$PY <- as.numeric(df$PY)

  ### replace "---" with ";"
  tagsComma <- c("AU", "ID")
  df1 <- data.frame(lapply(df[tagsComma], function(x) {
    gsub("---", ";", x)
  }))

  ### replace "---" with " "
  otherTags <- setdiff(names(df), tagsComma)
  df2 <- data.frame(lapply(df[otherTags], function(x) {
    trimES(gsub("---", " ", x))
  }), stringsAsFactors = FALSE)
  df <- cbind(df1, df2)
  rm(df1, df2)

  df$ID <- gsub(" ;", ";", gsub("; ", ";", gsub("\\[[^\\]]*\\]", "", df$ID, perl = TRUE)))

  df$DB <- "COCHRANE"

  # Authors
  # df$AU <- trimES(gsub("-","",df$AU))

  # Toupper
  DI <- df$DI
  df <- data.frame(lapply(df, toupper), stringsAsFactors = FALSE)
  df$DI <- gsub(" ", "", DI)

  df <- df[!(names(df) %in% c("Paper", "Reco"))]

  df$DE <- df$ID

  df$JI <- df$J9 <- df$SO

  return(df)
}

## Wrapper function to render bibliobox in Shiny
renderBibliobox <- function(df, title = "", ...) {
  tryCatch(
    {
      data <- if (is.reactive(df)) df() else df
      HTML(htmlBoxFormat(data, title = title, ...))
    },
    error = function(e) {
      tags$div(
        class = "alert alert-danger",
        paste("Render error:", e$message)
      )
    }
  )
}

#' HTML Box Format with Filter-Aware Excel Export
htmlBoxFormat <- function(
  df,
  nrow = 10,
  filename = "Table",
  pagelength = TRUE,
  left = NULL,
  right = NULL,
  numeric = NULL,
  dom = TRUE,
  size = "85%",
  filter = "top",
  columnShort = NULL,
  columnSmall = NULL,
  round = 2,
  title = "",
  button = FALSE,
  escape = FALSE,
  selection = FALSE,
  scrollX = FALSE,
  scrollY = FALSE,
  summary = FALSE
) {
  if (is.null(df) || (is.data.frame(df) && nrow(df) == 0)) {
    return(as.character(tags$div(
      class = "alert alert-info",
      "No data available"
    )))
  }

  # CORREZIONE: Gestisci pagelength prima di usare nrow nello sprintf
  if (is.logical(pagelength) && !pagelength) {
    nrow <- nrow(df) # Mostra tutte le righe
  } else if (is.numeric(pagelength)) {
    nrow <- as.integer(pagelength)
  }
  nrow <- as.integer(nrow) # Assicura che sia sempre un intero

  table_id <- sprintf(
    "htmlbox_%s",
    paste(sample(c(0:9, letters), 8, replace = TRUE), collapse = "")
  )
  col_names <- colnames(df)
  n_cols <- ncol(df)

  # Detect numeric columns: explicit (via `numeric` arg) plus auto-detected
  # (every non-empty stripped value parses cleanly as a number). Used for
  # right-alignment AND for client-side filter behavior.
  num_re <- "^-?[0-9]+(\\.[0-9]+)?$"
  is_numeric_col <- logical(n_cols)
  for (j in seq_len(n_cols)) {
    if (j %in% numeric) {
      is_numeric_col[j] <- TRUE
      next
    }
    raw <- as.character(df[[j]])
    stripped <- gsub("<.*?>", "", raw)
    stripped <- stripped[!is.na(stripped) & nzchar(stripped)]
    is_numeric_col[j] <- length(stripped) > 0 &&
      all(grepl(num_re, stripped))
  }

  get_align <- function(i) {
    if (i %in% left) {
      return("text-align: left;")
    }
    if (i %in% right) {
      return("text-align: right;")
    }
    if (is_numeric_col[i]) {
      return("text-align: right;")
    }
    return("text-align: left;")
  }

  # Determine if summary button column is needed
  has_summary <- is.character(summary) && summary == "documents"

  # Header
  header_cols <- paste0(
    sprintf(
      '<th class="sortable-header" data-col="%d" style="%s background-color: #f8f9fa; border-bottom: 2px solid #dee2e6; padding: 12px 25px 12px 10px; position: relative; white-space: nowrap;">%s <span class="sort-icon" style="color: #ccc; font-size: 0.9em; position: absolute; right: 8px; top: 50%%; transform: translateY(-50%%);">⇅</span></th>',
      0:(ncol(df) - 1),
      sapply(1:ncol(df), get_align),
      col_names
    ),
    collapse = ""
  )
  if (has_summary) {
    header_cols <- paste0(
      header_cols,
      '<th style="background-color: #f8f9fa; border-bottom: 2px solid #dee2e6; padding: 12px 10px; text-align: center; white-space: nowrap;">AI Summary</th>'
    )
  }
  header_html <- paste0("<thead><tr style='cursor: pointer;'>", header_cols, "</tr>")

  if (filter == "top") {
    filter_cells <- paste0(
      sprintf(
        '<td><input type="text" class="form-control table-filter-%s" data-col="%d" placeholder="Filter..." style="width:100%%; font-size:11px; height:24px; padding:2px 5px;"></td>',
        table_id,
        0:(ncol(df) - 1)
      ),
      collapse = ""
    )
    if (has_summary) {
      filter_cells <- paste0(filter_cells, "<td></td>")
    }
    filter_html <- paste0('<tr class="filter-row">', filter_cells, "</tr>")
    header_html <- paste0(header_html, filter_html)
  }
  header_html <- paste0(header_html, "</thead>")

  # Body: prepare data as JSON for client-side rendering (only visible page in DOM)
  aligns <- sapply(1:n_cols, get_align)
  numeric_cols_idx <- which(is_numeric_col) - 1L # 0-indexed for JS

  # Build column-oriented vectors (fast R serialization, reconstruct rows in JS)
  sort_cols <- vector("list", n_cols)
  disp_cols <- vector("list", n_cols)
  for (j in 1:n_cols) {
    vals <- as.character(df[[j]])
    vals[is.na(vals)] <- ""
    sort_vals <- gsub("<.*?>", "", vals)
    # Apply decimal formatting to:
    #  - explicit numeric columns (always — user opted in)
    #  - auto-detected numeric columns that contain at least one decimal value
    #    (so integers in a "rounded" column display as "29.0" alongside "30.5",
    #    while pure-integer columns like years stay as "2020" instead of "2020.00")
    apply_format <- (j %in% numeric) ||
      (is_numeric_col[j] && round > 0 && any(grepl(".", sort_vals, fixed = TRUE)))
    if (apply_format) {
      num_vals <- suppressWarnings(as.numeric(sort_vals))
      disp_cols[[j]] <- ifelse(
        is.na(num_vals),
        "",
        formatC(num_vals, format = "f", digits = round)
      )
      sort_cols[[j]] <- ifelse(is.na(num_vals), "", as.character(num_vals))
    } else {
      disp_cols[[j]] <- vals
      sort_cols[[j]] <- sort_vals
    }
  }

  # Column-oriented JSON: {s: [[col1_vals], [col2_vals], ...], d: [[col1_vals], [col2_vals], ...], n: nrow}
  # jsonlite serializes character vectors very efficiently in C — no nested R list overhead
  data_json <- jsonlite::toJSON(
    list(s = sort_cols, d = disp_cols, n = nrow(df)),
    auto_unbox = TRUE,
    na = "string"
  )

  # UI Structure — empty tbody, data delivered via JSON script tag
  aligns_json <- jsonlite::toJSON(aligns, auto_unbox = TRUE)
  numeric_cols_json <- jsonlite::toJSON(numeric_cols_idx, auto_unbox = FALSE)

  html_out <- tagList(
    tags$script(
      src = "https://cdn.jsdelivr.net/npm/xlsx@0.18.5/dist/xlsx.full.min.js"
    ),

    tags$div(
      id = paste0("container_", table_id),
      style = sprintf(
        "font-size: %s; background: white; border: 1px solid #ddd; border-radius: 4px; margin-bottom: 20px; font-family: sans-serif;",
        size
      ),

      tags$div(
        style = "padding: 10px 15px; background: #fdfdfd; border-bottom: 1px solid #eee; display: flex; justify-content: space-between; align-items: center;",
        tags$strong(title, style = "font-size: 1.1em; color: #333;"),
        if (button) {
          tags$button(
            "Excel",
            class = "btn btn-xs btn-success",
            onclick = sprintf("exportToExcel('%s', '%s')", table_id, filename)
          )
        } else {
          ""
        }
      ),

      tags$div(
        style = if (scrollX) "overflow-x: auto;" else "",
        tags$table(
          id = table_id,
          class = "table table-hover",
          style = "width: 100%; margin-bottom: 0; table-layout: auto;",
          HTML(header_html),
          HTML("<tbody></tbody>")
        )
      ),

      tags$div(
        style = "padding: 10px; border-top: 1px solid #eee; display: flex; justify-content: space-between; align-items: center;",
        tags$div(
          id = paste0("info_", table_id),
          style = "color: #777; font-size: 12px;"
        ),
        tags$div(class = "btn-group", id = paste0("pager_", table_id))
      )
    ),

    # Embed data as JSON in a script tag (not executed, just storage)
    tags$script(
      id = paste0("data_", table_id),
      type = "application/json",
      HTML(as.character(data_json))
    ),

    tags$script(HTML({
      js_tmpl <- '
      if (typeof window.exportToExcel !== "function") {
        window.exportToExcel = function(tid, fname) {
          var state = window["__bb_" + tid];
          if (!state) return;

          var headers = [];
          var table = document.getElementById(tid);
          var headerCells = table.querySelectorAll("thead tr:first-child th");
          headerCells.forEach(function(th) { headers.push(th.innerText.replace("\\u21C5", "").trim()); });

          var data = [headers];
          var rows = state.filteredData;
          for (var i = 0; i < rows.length; i++) {
            var rowData = [];
            for (var j = 0; j < rows[i].length; j++) {
              var val = rows[i][j].s;
              if (!isNaN(val) && val !== "") val = parseFloat(val);
              rowData.push(val);
            }
            data.push(rowData);
          }

          var ws = XLSX.utils.aoa_to_sheet(data);
          var wb = XLSX.utils.book_new();
          XLSX.utils.book_append_sheet(wb, ws, "Data");
          XLSX.writeFile(wb, fname + "_" + new Date().toISOString().split("T")[0] + ".xlsx");
        };
      }

      (function() {
        var tid = "@@TID@@";
        var pageSize = @@PAGESIZE@@;
        var hasSummary = @@HASSUMMARY@@;
        var aligns = @@ALIGNS@@;
        var numericColsList = @@NUMERIC@@;

        function init() {
          var table = document.getElementById(tid);
          if (!table) { setTimeout(init, 100); return; }

          var dataEl = document.getElementById("data_" + tid);
          var raw = JSON.parse(dataEl.textContent);
          // Transform column-oriented {s:[[col1],[col2],...], d:[[col1],[col2],...], n:nrow}
          // into row-oriented [{s:sortVal, d:dispVal}, ...] per row — fast in JS
          var nCols = raw.s.length;
          var allData = new Array(raw.n);
          for (var r = 0; r < raw.n; r++) {
            var row = new Array(nCols);
            for (var c = 0; c < nCols; c++) {
              var sVal = raw.s[c][r];
              var dVal = raw.d[c][r];
              row[c] = {
                s: (sVal == null ? "" : String(sVal)),
                d: (dVal == null ? "" : String(dVal))
              };
            }
            allData[r] = row;
          }
          raw = null; // free column-oriented data
          var filteredData = allData.slice();

          // R has already determined which columns are numeric (explicit + auto)
          var numRe = /^-?\\d+(?:\\.\\d+)?$/;
          var isNumericCol = new Array(nCols);
          for (var c2 = 0; c2 < nCols; c2++) isNumericCol[c2] = false;
          for (var k = 0; k < numericColsList.length; k++) isNumericCol[numericColsList[k]] = true;
          var currentPage = 0;
          var sortCol = -1;
          var sortAsc = true;
          var tbody = table.querySelector("tbody");

          // Expose state for Excel export
          window["__bb_" + tid] = { filteredData: filteredData };

          function renderPage() {
            var start = currentPage * pageSize;
            var end = Math.min(start + pageSize, filteredData.length);
            tbody.innerHTML = "";
            for (var i = start; i < end; i++) {
              var row = filteredData[i];
              var tr = document.createElement("tr");
              for (var j = 0; j < row.length; j++) {
                var td = document.createElement("td");
                td.setAttribute("data-sort", row[j].s);
                td.style.cssText = aligns[j] + " padding: 8px; border-bottom: 1px solid #eee;";
                td.innerHTML = row[j].d;
                tr.appendChild(td);
              }
              if (hasSummary) {
                var tdS = document.createElement("td");
                tdS.style.cssText = "text-align: center; padding: 8px; border-bottom: 1px solid #eee;";
                var safeVal = row[0].s.replace(/\x27/g, "\\\x27");
                tdS.innerHTML = "<button class=\\"btn btn-xs btn-info\\" onclick=\\"Shiny.setInputValue(\\x27button_id\\x27, \\x27" + safeVal + "\\x27, {priority: \\x27event\\x27})\\" title=\\"AI Summary\\"><i class=\\"fa fa-robot\\"></i></button>";
                tr.appendChild(tdS);
              }
              tbody.appendChild(tr);
            }
            document.getElementById("info_" + tid).innerText =
              "Showing " + (filteredData.length > 0 ? start + 1 : 0) +
              "-" + end + " of " + filteredData.length;
            renderPager();
          }

          function renderPager() {
            var pager = document.getElementById("pager_" + tid);
            var pageCount = Math.ceil(filteredData.length / pageSize);
            pager.innerHTML = "";
            if (pageCount <= 1) return;
            var createBtn = function(label, target, active, disabled) {
              var b = document.createElement("button");
              b.className = "btn btn-default btn-xs" + (active ? " active" : "");
              b.innerText = label;
              b.disabled = disabled;
              b.onclick = function() { currentPage = target; renderPage(); };
              pager.appendChild(b);
            };
            createBtn("\\u00AB", Math.max(0, currentPage - 1), false, currentPage === 0);
            for (var i = 0; i < pageCount; i++) {
              if (i === 0 || i === pageCount - 1 || (i >= currentPage - 1 && i <= currentPage + 1)) {
                createBtn(i + 1, i, i === currentPage, false);
              } else if (i === currentPage - 2 || i === currentPage + 2) {
                var s = document.createElement("span");
                s.innerText = "...";
                s.style.padding = "0 5px";
                pager.appendChild(s);
              }
            }
            createBtn("\\u00BB", Math.min(pageCount - 1, currentPage + 1), false, currentPage === pageCount - 1);
          }

          table.querySelectorAll(".sortable-header").forEach(function(th) {
            th.onclick = function() {
              var col = parseInt(this.getAttribute("data-col"));
              sortAsc = (sortCol === col) ? !sortAsc : true;
              sortCol = col;
              table.querySelectorAll(".sort-icon").forEach(function(si) { si.innerText = "\\u21C5"; si.style.color = "#ccc"; });
              this.querySelector(".sort-icon").innerText = sortAsc ? "\\u25B2" : "\\u25BC";
              this.querySelector(".sort-icon").style.color = "#333";
              filteredData.sort(function(a, b) {
                var valA = (a[col].s || "").trim();
                var valB = (b[col].s || "").trim();
                if (!isNaN(valA) && !isNaN(valB) && valA !== "" && valB !== "") {
                  return sortAsc ? parseFloat(valA) - parseFloat(valB) : parseFloat(valB) - parseFloat(valA);
                }
                return sortAsc ? valA.localeCompare(valB) : valB.localeCompare(valA);
              });
              currentPage = 0;
              renderPage();
            };
          });

          var filters = document.querySelectorAll(".table-filter-" + tid);
          // Hint users that numeric columns accept operators
          filters.forEach(function(input) {
            var col = parseInt(input.getAttribute("data-col"));
            if (isNumericCol[col]) input.setAttribute("placeholder", "= > < range");
          });

          var rangeRe = /^(-?\\d+(?:\\.\\d+)?)\\s*(?:-|\\.\\.)\\s*(-?\\d+(?:\\.\\d+)?)$/;
          var opRe = /^(>=|<=|>|<|=)\\s*(-?\\d+(?:\\.\\d+)?)$/;

          function matchNumeric(cellSort, raw) {
            if (cellSort === "" || cellSort === "NA") return false;
            var n = parseFloat(cellSort);
            if (isNaN(n)) return false;
            var fv = raw.trim();
            var m;
            if ((m = rangeRe.exec(fv))) {
              var lo = parseFloat(m[1]), hi = parseFloat(m[2]);
              if (lo > hi) { var t = lo; lo = hi; hi = t; }
              return n >= lo && n <= hi;
            }
            if ((m = opRe.exec(fv))) {
              var op = m[1], v = parseFloat(m[2]);
              if (op === ">") return n > v;
              if (op === "<") return n < v;
              if (op === ">=") return n >= v;
              if (op === "<=") return n <= v;
              return n === v;
            }
            if (numRe.test(fv)) return n === parseFloat(fv);
            // Unrecognized expression: filter out everything until input is valid.
            return false;
          }

          filters.forEach(function(input) {
            input.addEventListener("input", function() {
              var fRaw = Array.from(filters).map(function(f) { return f.value; });
              var fUpper = fRaw.map(function(v) { return v.toUpperCase(); });
              filteredData = allData.filter(function(row) {
                for (var idx = 0; idx < fRaw.length; idx++) {
                  var val = fRaw[idx];
                  if (!val || !val.trim()) continue;
                  var cell = row[idx].s;
                  if (isNumericCol[idx]) {
                    if (!matchNumeric(cell, val)) return false;
                  } else {
                    if (cell.toUpperCase().indexOf(fUpper[idx]) === -1) return false;
                  }
                }
                return true;
              });
              window["__bb_" + tid].filteredData = filteredData;
              currentPage = 0;
              renderPage();
            });
          });

          renderPage();
        }
        init();
      })();
      '
      js_filled <- js_tmpl
      js_filled <- gsub("@@TID@@", table_id, js_filled, fixed = TRUE)
      js_filled <- gsub(
        "@@PAGESIZE@@",
        as.character(nrow),
        js_filled,
        fixed = TRUE
      )
      js_filled <- gsub(
        "@@HASSUMMARY@@",
        ifelse(has_summary, "true", "false"),
        js_filled,
        fixed = TRUE
      )
      js_filled <- gsub(
        "@@ALIGNS@@",
        as.character(aligns_json),
        js_filled,
        fixed = TRUE
      )
      js_filled <- gsub(
        "@@NUMERIC@@",
        as.character(numeric_cols_json),
        js_filled,
        fixed = TRUE
      )
      js_filled
    }))
  )

  return(as.character(html_out))
}

#' Index of Countries.
#'
#' Data frame containing a normalized index of countries.\cr
#' Data are used by \code{\link{biblioAnalysis}} function
#' to extract Country Field of Cited References and Authors.
#'
#' @format A data frame with 202 rows and 5 variables:
#'  \describe{
#'     \item{countries}{country names}
#'     \item{continent}{continent names}
#'     \item{iso2}{country ISO 3166-1 alpha-2 code}
#'     \item{Longitude}{country centroid longitude}
#'     \item{Latitude}{country centroid latitude}
#'     }
#'
#' @name countries
NULL

#' Tag list and bibtex fields.
#'
#' Data frame containing a list of tags and corresponding: WoS, SCOPUS and generic bibtex fields; and Dimensions.ai csv and xlsx fields.
#'
#' @format A data frame with 44 rows and 6 variables:
#'  \describe{
#'     \item{TAG}{Tag Fields}
#'     \item{SCOPUS}{Scopus bibtex fields}
#'     \item{ISI}{WOS/ISI bibtex fields}
#'     \item{GENERIC}{Generic bibtex fields}
#'     \item{DIMENSIONS_OLD}{DIMENSIONS cvs/xlsx old fields}
#'     \item{DIMENSIONS}{DIMENSIONS cvs/xlsx fields}
#'     }
#'
#' @name bibtag
NULL

#' Authors' dominance ranking
#'
#' It calculates the authors' dominance ranking from an object of the class '\code{bibliometrix}' as proposed by Kumar & Kumar, 2008.
#' @param results is an object of the class '\code{bibliometrix}' for which the analysis of the authors' dominance ranking is desired.
#' @param k is an integer, used for table formatting (number of authors). Default value is 10.
#' @return The function \code{dominance} returns a data frame with cases corresponding to the first \code{k} most productive authors and variables to typical field of a dominance analysis.
#'
#' the data frame variables are:
#' \tabular{lll}{
#' \code{Author} \tab   \tab Author's name\cr
#' \code{Dominance Factor}  \tab   \tab Dominance Factor (DF = FAA / MAA)\cr
#' \code{Tot Articles}   \tab   \tab N. of Authored Articles (TAA)\cr
#' \code{Single Authored}   \tab   \tab N. of Single-Authored Articles (SAA)\cr
#' \code{Multi Authored}   \tab   \tab N. of Multi-Authored Articles (MAA=TAA-SAA)\cr
#' \code{First Authored} \tab   \tab N. of First Authored Articles (FAA)\cr
#' \code{Rank by Articles}    \tab   \tab Author Ranking by N. of Articles\cr
#' \code{Rank by DF}    \tab   \tab Author Ranking by Dominance Factor}
#'
#'
#'
#' @examples
#' data(scientometrics, package = "bibliometrixData")
#' results <- biblioAnalysis(scientometrics)
#' DF <- dominance(results)
#' DF
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{summary}} method for class '\code{bibliometrix}'
#'
#' @export

dominance <- function(results, k = 10) {
  # Author Rank by Dominance Rank  (Kumar & Kumar, 2008)

  # options(warn=-1)

  if (!inherits(results, "bibliometrix")) {
    cat('\n argument "results" have to be an object of class "bibliometrix"\n')
    return(NA)
  }

  Nmf <- table(results$FirstAuthors[results$nAUperPaper > 1])
  FA <- names(Nmf)
  # FA=gsub(" ", "", FA, fixed = TRUE)  # delete spaces

  AU <- names(results$Authors)


  Tot <- Single <- rep(NA, length(FA))
  for (i in 1:length(FA)) {
    Single[i] <- sum(results$FirstAuthors[results$nAUperPaper == 1] == FA[i])
    Tot[i] <- results$Authors[FA[i] == AU]
  }
  Dominance <- Nmf / (Tot - Single)

  D <- data.frame("Author" = FA, "Dominance Factor" = as.numeric(Dominance), "Articles" = Tot, "Single-Authored" = Single, "Multi-Authored" = Tot - Single, "First-Author" = as.numeric(Nmf))
  D <- D[order(-D[, 3]), ]
  D <- D[1:k, ]
  D$RankbyArticles <- rank(-D$Articles, ties.method = "min")
  D <- D[order(-D$Dominance.Factor), ]
  D$RankDF <- rank(-D$Dominance.Factor, ties.method = "min")
  names(D) <- c("Author", "Dominance Factor", "Tot Articles", "Single-Authored", "Multi-Authored", "First-Authored", "Rank by Articles", "Rank by DF")
  row.names(D) <- 1:k
  return(D)
}

utils::globalVariables("where")

csvLens2df <- function(file) {
  options(readr.num_columns = 0)

  ## import all files in a single data frame
  for (i in 1:length(file)) {
    D <- read_csv(file[i], na = character(), quote = '"', trim_ws = FALSE, progress = show_progress(), show_col_types = FALSE) %>%
      mutate(across(where(is.numeric), as.character)) %>%
      mutate(across(where(is.character), function(x) tidyr::replace_na(x, ""))) %>%
      as.data.frame()

    if (i > 1) {
      l <- intersect(l, names(D))
      DATA <- rbind(DATA[l], D[l])
    } else {
      l <- names(D)
      DATA <- D
    }
  }
  rm(D)

  ## Post-Processing

  # column re-labelling
  DATA <- relabelling_lens(DATA)

  if ("TC" %in% names(DATA)) DATA$TC <- as.numeric(DATA$TC)

  # Authors' names cleaning (surname and initials)
  DATA$AF <- DATA$AU

  # Authors' names cleaning (surname and initials)
  # remove ; and 2 or more spaces
  # DATA$AU <- gsub("\\s+", " ", DATA$AU)

  listAU <- strsplit(DATA$AU, split = "; ")

  AU <- lapply(listAU, function(l) {
    lastname <- sub(".*\\s", "", trimws(l))
    firstname <- sub("\\s+[^ ]+$", "", l)
    firstname <- gsub("[^:A-Z:]", "", firstname)
    AU <- paste(lastname, firstname, sep = " ", collapse = ";")
    return(AU)
  })


  DATA$AU <- unlist(AU)
  # DATA$AU=gsub("\\.", "", DATA$AU)
  # DATA$AU=gsub(",", ";", DATA$AU)

  # Affiliation
  # DATA$C1 <- "Unknown"

  # Iso Source Titles
  DATA$SO[DATA$SO == ""] <- DATA$Publisher[DATA$SO == ""]
  # DATA$JI <- sapply(DATA$SO, AbbrevTitle, USE.NAMES = FALSE)
  # DATA$J9 <- gsub("\\.","",DATA$JI)
  DATA$JI <- DATA$J9 <- DATA$SO
  DATA$ID <- DATA$DE
  DI <- DATA$DI
  URL <- DATA$URL
  AB <- DATA$AB
  TI <- DATA$TI
  DE <- DATA$DE
  DATA <- data.frame(lapply(DATA, toupper))
  DATA$AB_raw <- AB
  DATA$TI_raw <- TI
  DATA$DE_raw <- DE
  DATA$DI <- DI
  DATA$URL <- URL
  DATA$AU_CO <- "NA"
  DATA$DB <- "LENS"
  return(DATA)
}




relabelling_lens <- function(DATA) {
  ## column re-labelling
  label <- names(DATA)
  label <- gsub("Source Title", "SO", label)
  # label <- gsub("Authors with affiliations","C1",label)
  label <- gsub("Author/s", "AU", label)
  label <- gsub("Publication.Type", "DT", label)
  label <- gsub("Title", "TI", label)
  label <- gsub("Publication Year", "PY", label)
  label <- gsub("Volume", "VL", label)
  label <- gsub("Issue Number", "IS", label)
  label <- gsub("Source Country", "SO_CO", label)
  label <- gsub("Scholarly Citation Count", "TC", label)
  label <- gsub("DOI", "DI", label)
  label <- gsub("Source URLs", "URL", label)
  label <- gsub("Abstract", "AB", label)
  label <- gsub("Keywords", "DE", label)
  label <- gsub("MeSH Terms", "MESH", label)
  label <- gsub("Funding Details", "FU", label)
  label <- gsub("Funding", "FX", label)
  label <- gsub("References", "CR", label)
  # label <- gsub("Correspondence Address","RP",label)
  label <- gsub("Fields of Study", "SC", label)
  label <- gsub("Language of Original Document", "LA", label)
  label <- gsub("Document Type", "DT", label)
  label <- gsub("Source", "DB", label)
  label <- gsub("Lens ID", "UT", label)
  label <- gsub("Citing Works Count", "TC", label)
  names(DATA) <- label

  return(DATA)
}

#' Completeness of bibliographic metadata
#'
#' It calculates the percentage of missing data in the metadata of a bibliographic data frame.
#'
#' Each metadata is assigned a status c("Excellent," "Good," "Acceptable", "Poor", "Critical," "Completely missing")
#' depending on the percentage of missing data. In particular, the column *status* classifies the percentage of missing
#' value in 5 categories: "Excellent" (0%), "Good" (0.01% to 10.00%), "Acceptable" (from 10.01% to 20.00%),
#' "Poor" (from 20.01% to 50.00%), "Critical" (from 50.01% to 99.99%), "Completely missing" (100%).
#'
#' The results of the function allow us to understand which analyses can be performed with bibliometrix
#' and which cannot based on the completeness (or status) of different metadata.
#' @param M is a bibliographic data frame obtained by \code{\link{convert2df}} function.
#'
#' @return The function \code{missingData} returns a list containing two objects:
#' \tabular{lll}{
#' \code{allTags}  \tab   \tab is a data frame including results for all original metadata tags from the collection\cr
#' \code{mandatoryTags}\tab    \tab is a data frame that included only the tags needed for analysis with bibliometrix and biblioshiny.}
#'
#' @examples
#' data(scientometrics, package = "bibliometrixData")
#' res <- missingData(scientometrics)
#' print(res$mandatoryTags)
#'
#' @export
#'
missingData <- function(M) {
  cols <- names(M)
  # count the number of missing values in each column
  missing_counts <- sapply(cols, function(x) {
    sum(is.na(M[, x]) | M[, x] %in% c("NA,0000,NA", "NA", "", "none"))
  })

  if (sum(as.numeric(M$TC), na.rm = T) == 0) {
    missing_counts["TC"] <- nrow(M)
  }
  # calculate the percentage of missing values in each column
  missing_pct <- round(missing_counts / nrow(M) * 100, 2)
  # create a dataframe with the column names, number of missing values and percentage of missing values
  df_all <- data.frame(cols, missing_counts, missing_pct)

  # create a vector with the tags
  tag <- unlist(
    strsplit(
      "AB,AU,C1,CR,DE,DI,DT,ID,LA,PY,RP,SO,TC,TI,WC", ","
    )
  )
  # create a vector with the description of the tags
  description <- trimws(unlist(
    strsplit(
      "Abstract, Author,Affiliation,Cited References,Keywords,DOI,Document Type,Keywords Plus,Language,
      Publication Year,Corresponding Author, Journal, Total Citation, Title, Science Categories", ","
    )
  ))

  # create a dataframe with the column names, number of missing values, percentage of missing values and status
  df_all <- df_all %>%
    mutate(status = status(missing_pct)) %>%
    replace_na(replace = list(missing_counts = nrow(M), missing_pct = 100))

  df_tags <- data.frame(tag, description) %>%
    left_join(df_all, by = c("tag" = "cols")) %>%
    replace_na(replace = list(missing_counts = nrow(M), missing_pct = 100, status = "Completely missing")) %>%
    arrange(missing_pct, description)

  results <- list(allTags = df_all, mandatoryTags = df_tags)
  return(results)
}

status <- function(x) {
  y <- character(length(x))
  y[x == 0] <- "Excellent"
  y[x > 0 & x <= 10] <- "Good"
  y[x > 10 & x <= 20] <- "Acceptable"
  y[x > 20 & x <= 50] <- "Poor"
  y[x > 50 & x < 100] <- "Critical"
  y[is.na(x) | x == 100] <- "Completely missing"
  return(y)
}

#' Calculating network summary statistics
#'
#' \code{networkStat} calculates main network statistics.
#'
#' The function \code{\link{networkStat}} can calculate the main network statistics from a bibliographic network previously created by \code{\link{biblioNetwork}}.
#' @param object is a network matrix obtained by the function \code{\link{biblioNetwork}} or an graph object of the class \code{igraph}.
#' @param stat is a character. It indicates which statistics are to be calculated. \code{stat = "network"} calculates the statistics related to the network;
#' \code{stat = "all"} calculates the statistics related to the network and the individual nodes that compose it. Default value is \code{stat = "network"}.
#' @param type is a character. It indicates which centrality index is calculated. type values can be c("degree", "closeness", "betweenness","eigenvector","pagerank","hub","authority", "all"). Default is "degree".
#' @return It is a list containing the following elements:
#' \tabular{lll}{
#' \code{graph} \tab  \tab a network object of the class \code{igraph}\cr
#' \code{network} \tab  \tab a \code{communities} a list with the main statistics of the network\cr
#' \code{vertex} \tab  \tab a data frame with the main measures of centrality and prestige of vertices.\cr}
#'
#'
#' @examples
#' # EXAMPLE Co-citation network
#'
#' # to run the example, please remove # from the beginning of the following lines
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # NetMatrix <- biblioNetwork(scientometrics, analysis = "co-citation",
#' #      network = "references", sep = ";")
#'
#' # netstat <- networkStat(NetMatrix, stat = "all", type = "degree")
#'
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export
networkStat <- function(object, stat = "network", type = "degree") {
  if (!inherits(object, "igraph")) {
    # Create igraph object
    net <- graph_from_adjacency_matrix(object, mode = "undirected", weighted = NULL)
    V(net)$id <- colnames(object)
  } else {
    net <- object
    V(net)$id <- V(net)$name
  }

  net <- simplify(net, remove.loops = T)

  ### network statistics
  networkSize <- vcount(net)

  # The proportion of present edges from all possible edges in the network.
  networkDensity <- edge_density(net, loops = FALSE)

  # Transitivity
  # global - ratio of triangles (direction disregarded) to connected triples.
  TR <- transitivity(net, type = "global")
  # local - ratio of triangles to connected triples each vertex is part of.
  # TRL=transitivity(net, type="local")

  # Diameter
  # A network diameter is the longest geodesic distance
  # (length of the shortest path between two nodes) in the network
  DIAM <- diameter(net, directed = F, weights = NA)

  # Degree distribution
  deg <- degree(net, mode = "all")
  deg.dist <- degree_distribution(net, cumulative = T, mode = "all")

  # plot( x=0:max(deg), y=1-deg.dist, pch=19, cex=1.2, col="orange",
  #      xlab="Degree", ylab="Cumulative Frequency")


  # Network centralization
  # Degree (number of ties)degree
  NCD <- NCC <- NCB <- NCE <- pagerank <- hub <- authority <- NA
  switch(type,
    degree = {
      NCD <- centr_degree(net, mode = "all", normalized = T)$centralization
    },
    closeness = { # Closeness (centrality based on distance to others in the graph)
      NCC <- suppressWarnings(centr_clo(net, mode = "all", normalized = T)$centralization)
    },
    betweenness = { # Betweenness (centrality based on a broker position connecting others)
      NCB <- centr_betw(net, directed = F, normalized = T)$centralization
    },
    eigenvector = { # Eigenvector (centrality proportional to the sum of connection centralities)
      NCE <- centr_eigen(net, directed = F, normalized = T)$centralization
    },
    pagerank = {
      NCD <- centr_degree(net, mode = "all", normalized = T)$centralization
    },
    hub = {
      NCD <- centr_degree(net, mode = "all", normalized = T)$centralization
    },
    authority = {
      NCD <- centr_degree(net, mode = "all", normalized = T)$centralization
    }
  )
  # Average path length
  # the mean of the shortest distance between each pair of nodes in the network (in both directions for directed graphs).
  meanDistance <- mean_distance(net, directed = F)

  networkResults <- list(
    networkSize = networkSize,
    networkDensity = networkDensity,
    networkTransitivity = TR,
    networkDiameter = DIAM,
    networkDegreeDist = deg.dist,
    networkCentrDegree = NCD,
    networkCentrCloseness = NCC,
    networkCentrEigen = NCE,
    networkCentrbetweenness = NCB,
    NetworkAverPathLeng = meanDistance
  )

  ### Centrality and Prestige of vertices
  if (stat == "all") {
    DC <- CC <- BC <- EC <- PR <- HS <- AS <- NA

    switch(type,
      degree = {
        # Degree centrality.
        # The simplest way to quantify a node'simportance is to consider the number of nodes it is incident with,
        # with high numbers interpreted to be of higher importance.
        # standardized index is: Ei/(N-1)
        DC <- degree(net, v = V(net), mode = c("all"), loops = TRUE, normalized = TRUE)
      },
      closeness = {
        # Closeness centrality.
        # Nodes can also be indexed by con-sidering their geodesic distance to each other.
        CC <- suppressWarnings(closeness(net, vids = V(net), mode = c("all"), normalized = TRUE))
      },
      betweenness = {
        # Betweenness centrality.
        # Another way to gauge a node's influence is to consider its role in linking other nodes together in the network.
        BC <- betweenness(net, v = V(net), directed = FALSE, weights = NULL, normalized = TRUE)
      },
      eigenvector = {
        # Eigenvector centrality.
        # Eigenvector centrality is another measure of centrality.
        # The eigenvector centrality ofeach node can be found by computing the leading
        EC <- eigen_centrality(net, directed = FALSE, scale = TRUE, weights = NULL, options = arpack_defaults)$vector
      },
      pagerank = {
        # PageRank ranking of vertices
        PR <- page_rank(net,
          algo = c("prpack"), vids = V(net),
          directed = FALSE, damping = 0.85, personalized = NULL, weights = NULL,
          options = NULL
        )$vector
      },
      hub = {
        ### Hubs and authorities
        # The hubs and authorities algorithm developed by Jon Kleinberg was initially used to examine web pages.
        # Hubs were expected to contain catalogs with a large number of outgoing links; while authorities would get
        # many incoming links from hubs, presumably because of their high-quality relevant information.

        # Hubs
        HS <- hub_score(net, weights = NA)$vector
      },
      authority = {
        # Authorities
        AS <- authority_score(net, weights = NA)$vector
      },
      all = {
        DC <- degree(net, v = V(net), mode = c("all"), loops = TRUE, normalized = TRUE)
        CC <- suppressWarnings(closeness(net, vids = V(net), mode = c("all"), normalized = TRUE))
        BC <- betweenness(net, v = V(net), directed = FALSE, weights = NULL, normalized = TRUE)
        EC <- eigen_centrality(net, directed = FALSE, scale = TRUE, weights = NULL, options = arpack_defaults)$vector
        PR <- page_rank(net,
          algo = c("prpack"), vids = V(net),
          directed = FALSE, damping = 0.85, personalized = NULL, weights = NULL,
          options = NULL
        )$vector
        HS <- hub_score(net, weights = NA)$vector
        AS <- authority_score(net, weights = NA)$vector
      }
    )

    vertexResults <- data.frame(
      vertexID = V(net)$id,
      vertexCentrDegree = DC,
      vertexCentrCloseness = CC,
      vertexCentrEigen = EC,
      vertexCentrBetweenness = BC,
      vertexPageRank = PR,
      vertexHub = HS,
      vertexAuthority = AS
    )

    # res=PCA(vertexResults[,-1],graph=FALSE)

    # R=rank(-res$ind$coord[,1]-min(res$ind$coord[,1]))

    # vertexResults$Ranking=R
  } else {
    vertexResults <- NA
  }
  netstat <- list(graph = net, network = networkResults, vertex = vertexResults, stat = stat, type = type)
  class(netstat) <- "bibliometrix_netstat"
  return(netstat)
}

#' Shiny UI for bibliometrix package
#'
#' \code{biblioshiny} performs science mapping analysis using the main functions of the bibliometrix package.
#'
#' @param port is the TCP port that the application should listen on. If the port is not specified,
#' and the shiny.port option is set (with options(shiny.port = XX)), then that port will be used.
#' Otherwise, use the default port 3838.
#'
#' @param launch.browser If true, the system's default web browser will be launched automatically
#' after the app is started. Defaults to true in interactive sessions only. This value of
#' this parameter can also be a function to call with the application's URL.
#'
#' @param host The IPv4 address that the application should listen on.
#' Defaults to the shiny.host option, if set, or "127.0.0.1" if not.
#'
#' @param maxUploadSize is a integer. The max upload file size argument. Default value is 200 (megabyte)
#'
#' @param max.rows is an integer or Inf. The maximum number of records (publications) allowed
#' when importing or loading a file. Files exceeding this limit will be rejected with an
#' informative message. Default value is \code{Inf} (no limit).
#'
#' @examples
#'
#' # biblioshiny()
#'
#' @export

biblioshiny <- function(
  host = "127.0.0.1",
  port = 3838,
  launch.browser = TRUE,
  maxUploadSize = 500,
  max.rows = Inf
) {
  shinyOptions(maxUploadSize = maxUploadSize)
  shinyOptions(biblioshiny.max.rows = max.rows)

  runApp(
    system.file("biblioshiny", package = "bibliometrix"),
    launch.browser = launch.browser,
    port = port,
    host = getOption("shiny.host", host)
  )
}

#' Searching of duplicated records in a bibliographic database
#'
#' Search duplicated records in a dataframe.
#'
#' A bibliographic data frame is obtained by the converting function \code{\link{convert2df}}.
#' It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' The function identifies duplicated records in a bibliographic data frame and deletes them.
#' Duplicate entries are identified through the restricted Damerau-Levenshtein distance.
#' Two manuscripts that have a relative similarity measure greater than \code{tol} argument are stored in the output data frame only once.
#'
#' @param M is the bibliographic data frame.
#' @param Field is a character object. It indicates one of the field tags used to identify duplicated records. Field can be equal to one of these tags: TI (title), AB (abstract), UT (manuscript ID).
#' @param exact is logical. If exact = TRUE the function searches duplicates using exact matching. If exact=FALSE,
#' the function uses the restricted Damerau-Levenshtein distance to find duplicated documents.
#' @param tol is a numeric value giving the minimum relative similarity to match two manuscripts. Default value is \code{tol = 0.95}.
#' To use the restricted Damerau-Levenshtein distance, exact argument has to be set as FALSE.
#' @return the value returned from \code{duplicatedMatching} is a data frame without duplicated records.
#'
#'
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' M <- rbind(scientometrics[1:20, ], scientometrics[10:30, ])
#'
#' newM <- duplicatedMatching(M, Field = "TI", exact = FALSE, tol = 0.95)
#'
#' dim(newM)
#'
#' @seealso \code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export
duplicatedMatching <- function(M, Field = "TI", exact = FALSE, tol = 0.95) {
  if (!(Field %in% names(M))) {
    cat("\nField", Field, "is not a valid column name of your bibliographic data frame\n ")
    return(M)
  }
  if (isTRUE(exact)) {
    exact <- "true"
  } else {
    exact <- "false"
  }
  switch(exact,
    true = {
      M <- M[!duplicated(M[Field]), ]
    },
    false = {
      a <- b <- M[[Field]]
      an <- nchar(a)
      A <- matrix(an, length(an), length(an))
      A[is.na(A)] <- 0
      B <- t(A)
      C <- A
      C[B > A] <- B[B > A]
      D <- as.matrix(stringdistmatrix(a))
      Dn <- 1 - (D / C)
      Dn[Dn > tol] <- 2
      M <- M[!duplicated(Dn), ]
    }
  )

  return(M)
}

utils::globalVariables(c("degree"))
#' Plotting Bibliographic networks
#'
#' \code{networkPlot} plots a bibliographic network.
#'
#' The function \code{\link{networkPlot}} can plot a bibliographic network previously created by \code{\link{biblioNetwork}}.
#'
#' @param NetMatrix is a network matrix obtained by the function \code{\link{biblioNetwork}}.
#' @param normalize is a character. It can be "association", "jaccard", "inclusion","salton" or "equivalence" to obtain Association Strength, Jaccard,
#' Inclusion, Salton or Equivalence similarity index respectively. The default is type = NULL.
#' @param n is an integer. It indicates the number of vertices to plot.
#' @param degree is an integer. It indicates the min frequency of a vertex. If degree is not NULL, n is ignored.
#' @param type is a character object. It indicates the network map layout:
#'
#' \tabular{lll}{
#' \code{type="auto"}\tab   \tab Automatic layout selection\cr
#' \code{type="circle"}\tab   \tab Circle layout\cr
#' \code{type="sphere"}\tab   \tab Sphere layout\cr
#' \code{type="mds"}\tab   \tab Multidimensional Scaling layout\cr
#' \code{type="fruchterman"}\tab   \tab Fruchterman-Reingold layout\cr
#' \code{type="kamada"}\tab   \tab  Kamada-Kawai layout}
#'
#' @param Title is a character indicating the plot title.
#' @param vos.path is a character indicating the full path where VOSviewer.jar is located.
#' @param size is integer. It defines the size of each vertex. Default is \code{size=3}.
#' @param size.cex is logical. If TRUE the size of each vertex is proportional to its degree.
#' @param noloops is logical. If TRUE loops in the network are deleted.
#' @param remove.isolates is logical. If TRUE isolates vertices are not plotted.
#' @param remove.multiple is logical. If TRUE multiple links are plotted using just one edge.
#' @param label is logical. If TRUE vertex labels are plotted.
#' @param labelsize is an integer. It indicates the label size in the plot. Default is \code{labelsize=1}
#' @param label.color is logical. If TRUE, for each vertex, the label color is the same as its cluster.
#' @param label.cex is logical. If TRUE the label size of each vertex is proportional to its degree.
#' @param halo is logical. If TRUE communities are plotted using different colors. Default is \code{halo=FALSE}
#' @param cluster is a character. It indicates the type of cluster to perform among ("none", "optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").
#' @param community.repulsion is a real number between 0 and 1.
#'   Controls the separation between communities with adaptive scaling.
#'   The algorithm automatically adapts to network size and structure.
#'   Recommended values:
#'   \tabular{ll}{
#'   \code{0.0}\tab No repulsion - communities may overlap\cr
#'   \code{0.2-0.4}\tab Light separation - suitable for dense networks\cr
#'   \code{0.5-0.7}\tab Moderate separation - general use (RECOMMENDED)\cr
#'   \code{0.8-1.0}\tab Strong separation - for many small communities}
#'   Default is \code{community.repulsion = 0.5}.
#' @param curved is a logical or a number. If TRUE edges are plotted with an optimal curvature. Default is \code{curved=FALSE}. Curved values are any numbers from 0 to 1.
#' @param weighted This argument specifies whether to create a weighted graph from an adjacency matrix.
#' If it is NULL then an unweighted graph is created and the elements of the adjacency matrix gives the number of edges between the vertices.
#' If it is a character constant then for every non-zero matrix entry an edge is created and the value of the entry is added as an edge attribute
#' named by the weighted argument. If it is TRUE then a weighted graph is created and the name of the edge attribute will be weight.
#' @param edgesize is an integer. It indicates the network edge size.
#' @param edges.min is an integer. It indicates the min frequency of edges between two vertices. If edge.min=0, all edges are plotted.
#' @param label.n is an integer. It indicates the number of vertex labels to draw.
#' @param alpha is a number. Legal alpha values are any numbers from 0 (transparent) to 1 (opaque). The default alpha value usually is 0.5.
#' @param seed is an integer. It indicates the random seed for clustering reproducibility. Default is \code{seed = 123}.
#' @param verbose is a logical. If TRUE, network will be plotted. Default is \code{verbose = TRUE}.
#' @return It is a list containing the following elements:
#' \tabular{lll}{
#' \code{graph} \tab  \tab a network object of the class \code{igraph}\cr
#' \code{cluster_obj} \tab  \tab a \code{communities} object of the package \code{igraph}\cr
#' \code{cluster_res} \tab  \tab a data frame with main results of clustering procedure.\cr}
#'
#'
#' @examples
#' # EXAMPLE Keywordd co-occurrence network
#'
#' data(management, package = "bibliometrixData")
#'
#' NetMatrix <- biblioNetwork(management,
#'   analysis = "co-occurrences",
#'   network = "keywords", sep = ";"
#' )
#'
#' net <- networkPlot(NetMatrix, n = 30, type = "auto", Title = "Co-occurrence Network", labelsize = 1)
#'
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @seealso \code{\link{net2VOSviewer}} to export and plot the network with VOSviewer software.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export
networkPlot <-
  function(
    NetMatrix,
    normalize = NULL,
    n = NULL,
    degree = NULL,
    Title = "Plot",
    type = "auto",
    label = TRUE,
    labelsize = 1,
    label.cex = FALSE,
    label.color = FALSE,
    label.n = NULL,
    halo = FALSE,
    cluster = "walktrap",
    community.repulsion = 0.5,
    vos.path = NULL,
    size = 3,
    size.cex = FALSE,
    curved = FALSE,
    noloops = TRUE,
    remove.multiple = TRUE,
    remove.isolates = FALSE,
    weighted = NULL,
    edgesize = 1,
    edges.min = 0,
    alpha = 0.5,
    seed = 123,
    verbose = TRUE
  ) {
    S <- NULL

    colnames(NetMatrix) <- rownames(NetMatrix) <- tolower(colnames(NetMatrix))
    bsk.S <- TRUE
    l <- NA
    net_groups <- list()
    # if (type == "vosviewer") {
    #  cluster <- "none"
    # }

    if (!is.null(normalize)) {
      S <- normalizeSimilarity(NetMatrix, type = normalize)
      bsk.S <- graph_from_adjacency_matrix(S, mode = "undirected", weighted = T)
    }

    ## legacy with version <1.9.4
    if (isTRUE(size)) {
      size <- 20
      size.cex <- T
    }

    if (alpha < 0 & alpha > 1) {
      alpha <- 0.5
    }

    # Create igraph object
    bsk.network <-
      graph_from_adjacency_matrix(
        NetMatrix,
        mode = "undirected",
        weighted = weighted
      )

    # vertex labels
    V(bsk.network)$name <- colnames(NetMatrix)

    # node degree plot
    # deg <- igraph::degree_distribution(bsk.network, cumulative=T, mode="all")
    deg <- degree(bsk.network, mode = "all")
    deg.dist <- data.frame(node = V(bsk.network)$name, degree = deg) %>%
      arrange(desc(degree)) %>%
      mutate(degree = degree / max(degree))

    # Compute node degrees (#links) and use that to set node size:
    # deg <- degree(bsk.network, mode = "all")
    V(bsk.network)$deg <- deg
    if (isTRUE(size.cex)) {
      V(bsk.network)$size <- (deg / max(deg)[1]) * size
    } else {
      V(bsk.network)$size <- rep(size, length(V(bsk.network)))
    }

    # label size
    if (isTRUE(label.cex)) {
      lsize <- log(1 + (deg / max(deg)[1])) * labelsize
      lsize[lsize < 0.5] <- 0.5 ### min labelsize is fixed to 0.5
      V(bsk.network)$label.cex <- lsize
    } else {
      V(bsk.network)$label.cex <- labelsize
    }

    # Select number of vertices to plot
    if (!is.null(degree)) {
      Deg <- deg - diag(NetMatrix)
      Vind <- Deg < degree
      if (sum(!Vind) == 0) {
        cat("\ndegree argument is to high!\n\n")
        return()
      }
      bsk.network <- delete_vertices(bsk.network, which(Vind))
      if (!isTRUE(bsk.S)) {
        bsk.S <- delete_vertices(bsk.S, which(Vind))
      }
    } else if (!is.null(n)) {
      if (n > dim(NetMatrix)[1]) {
        n <- dim(NetMatrix)[1]
      }
      nodes <- names(sort(deg, decreasing = TRUE)[1:n])

      bsk.network <- delete_vertices(
        bsk.network,
        which(!(V(bsk.network)$name %in% nodes))
      )
      if (!isTRUE(bsk.S)) {
        bsk.S <- delete_vertices(bsk.S, which(!(V(bsk.S)$name %in% nodes)))
      }
    }

    # Remove loops and multiple edges
    if (edges.min > 1) {
      remove.multiple <- FALSE
    }
    bsk.network <-
      simplify(
        bsk.network,
        remove.multiple = remove.multiple,
        remove.loops = noloops
      )
    if (!isTRUE(bsk.S)) {
      bsk.S <-
        simplify(
          bsk.S,
          remove.multiple = remove.multiple,
          remove.loops = noloops
        )
    }

    ### graph to write in pajek format ###
    bsk.save <- bsk.network
    V(bsk.save)$id <- V(bsk.save)$name

    ###

    # Clustering
    # if (type != "vosviewer") {
    ## Edge size
    E(bsk.network)$num <- E(bsk.save)$num <- count_multiple(
      bsk.network,
      eids = E(bsk.network)
    )
    if (is.null(weighted)) {
      E(bsk.save)$weight <- E(bsk.save)$num
    }

    if (!is.null(weighted)) {
      E(bsk.network)$width <-
        (E(bsk.network)$weight + min(E(bsk.network)$weight)) /
        max(E(bsk.network)$weight + min(E(bsk.network)$weight)) *
        edgesize
    } else {
      if (isTRUE(remove.multiple)) {
        E(bsk.network)$width <- edgesize
      } else {
        edges <- E(bsk.network)$num
        E(bsk.network)$width <- edges / max(edges) * edgesize
      }
    }

    bsk.network <- delete_edges(
      bsk.network,
      which(E(bsk.network)$num < edges.min)
    )
    if (!isTRUE(bsk.S)) {
      bsk.S <- delete_edges(bsk.S, which(E(bsk.network)$num < edges.min))
    }

    # delete not linked vertices
    if (isTRUE(remove.isolates)) {
      bsk.network <- delete.isolates(bsk.network, mode = "all")
      if (!isTRUE(bsk.S)) {
        bsk.S <-
          delete_vertices(
            bsk.S,
            which(
              V(bsk.S)$name %in%
                setdiff(
                  V(bsk.S)$name,
                  V(bsk.network)$name
                )
            )
          )
      }
    }

    # Community Detection

    cl <- clusteringNetwork(bsk.network, cluster, seed = seed)

    bsk.network <- cl$bsk.network
    if (!isTRUE(bsk.S)) {
      V(bsk.S)$color <- V(bsk.network)$color
      V(bsk.S)$community <- V(bsk.network)$community
    }
    net_groups <- cl$net_groups

    # Choose Network layout
    if (!isTRUE(bsk.S)) {
      layout_results <- switchLayout(bsk.S, type, community.repulsion)
      bsk.S <- layout_results$bsk.S
    } else {
      layout_results <- switchLayout(bsk.network, type, community.repulsion)
      bsk.network <- layout_results$bsk.network
    }
    l <- layout_results$l

    ## Labelling the network
    LABEL <- ""
    if (isTRUE(label)) {
      LABEL <- V(bsk.network)$name
      if (!is.null(label.n)) {
        q <- 1 - (label.n / length(V(bsk.network)$deg))
        if (q <= 0) {
          # LABEL <- rep("", length(LABEL))
          V(bsk.network)$labelsize <- 10
        } else {
          if (q > 1) {
            q <- 1
          }
          q <- quantile(V(bsk.network)$deg, q)
          LABEL[V(bsk.network)$deg < q] <- ""
          V(bsk.network)$labelsize <- 10
          V(bsk.network)$labelsize[V(bsk.network)$deg < q] <- 0
        }
      }
    }

    if (isTRUE(label.color)) {
      lab.color <- V(bsk.network)$color
    } else {
      lab.color <- "black"
    }

    # l <- layout.norm(l)

    ## Setting Network Attributes
    igraph::graph_attr(bsk.network, "alpha") <- alpha
    igraph::graph_attr(bsk.network, "ylim") <- c(-1, 1)
    igraph::graph_attr(bsk.network, "xlim") <- c(-1, 1)
    igraph::graph_attr(bsk.network, "rescale") <- TRUE
    igraph::graph_attr(bsk.network, "asp") <- 0
    igraph::graph_attr(bsk.network, "layout") <- l
    igraph::graph_attr(bsk.network, "main") <- Title
    E(bsk.network)$curved <- curved
    V(bsk.network)$label.dist <- 0.7
    V(bsk.network)$frame.color <- adjustcolor("black", alpha)
    V(bsk.network)$color <- adjustcolor(V(bsk.network)$color, alpha)
    V(bsk.network)$label.color <- adjustcolor("black", min(c(1, alpha + 0.1)))
    V(bsk.network)$label.font <- 2
    V(bsk.network)$label <- LABEL

    ## Plot the network
    if (isTRUE(halo) & cluster != "none") {
      if (isTRUE(verbose)) {
        plot(net_groups, bsk.network)
      }
    } else {
      E(bsk.network)$color <- adjustcolor(E(bsk.network)$color, alpha / 2)

      if (isTRUE(verbose)) {
        plot(bsk.network)
      }
    }

    ## Output
    if (cluster != "none") {
      cluster_res <- data.frame(
        net_groups$names,
        net_groups$membership,
        as.numeric(betweenness(
          bsk.network,
          directed = F,
          normalized = F
        )),
        suppressWarnings(as.numeric(closeness(
          bsk.network
        ))),
        as.numeric(page.rank(bsk.network)$vector)
      )
      names(cluster_res) <- c(
        "vertex",
        "cluster",
        "btw_centrality",
        "clos_centrality",
        "pagerank_centrality"
      )
      cluster_res <- cluster_res[order(cluster_res$cluster), ]
    } else {
      cluster_res <- NA
    }

    params <- list(
      normalize = normalize,
      n = n,
      degree = degree,
      Title = Title,
      type = type,
      label = label,
      labelsize = labelsize,
      label.cex = label.cex,
      label.color = label.color,
      label.n = label.n,
      halo = halo,
      cluster = cluster,
      community.repulsion = community.repulsion,
      vos.path = vos.path,
      size = size,
      size.cex = size.cex,
      curved = curved,
      noloops = noloops,
      remove.multiple = remove.multiple,
      remove.isolates = remove.isolates,
      weighted = weighted,
      edgesize = edgesize,
      edges.min = edges.min,
      alpha = alpha,
      verbose = verbose
    )
    params <- data.frame(
      params = names(unlist(params)),
      values = unlist(params),
      row.names = NULL
    )

    ## Modularity of community detection
    Q <- modularity(bsk.network, V(bsk.network)$community)

    net <- list(
      graph = bsk.network,
      graph_pajek = bsk.save,
      cluster_obj = net_groups,
      cluster_res = cluster_res,
      community_obj = cl$net_groups,
      layout = l,
      S = S,
      nodeDegree = deg.dist,
      modularity = Q,
      params = params
    )

    return(net)
  }


### internal functions:

### deleteIsolates

delete.isolates <- function(graph, mode = "all") {
  isolates <- which(degree(graph, mode = mode) == 0) - 1
  delete_vertices(graph, names(isolates))
}

### clusteringNetwork

clusteringNetwork <- function(bsk.network, cluster, seed = NULL, n_runs = 10) {
  colorlist <- colorlist()

  # For stochastic algorithms, perform multiple runs and select the best one
  if (cluster %in% c("louvain", "leiden") && !is.null(seed)) {
    best_modularity <- -Inf
    best_result <- NULL

    for (i in 1:n_runs) {
      set.seed(seed + i - 1) # Different seed for each run

      if (cluster == "louvain") {
        result <- cluster_louvain(bsk.network)
      } else if (cluster == "leiden") {
        result <- cluster_leiden(
          bsk.network,
          objective_function = "modularity",
          n_iterations = 3,
          resolution_parameter = 0.75
        )
      }

      current_modularity <- modularity(bsk.network, result$membership)

      if (current_modularity > best_modularity) {
        best_modularity <- current_modularity
        best_result <- result
      }
    }

    net_groups <- best_result
  } else {
    # For other algorithms, use standard logic
    # if (!is.null(seed) && cluster %in% c("spinglass", "leading_eigen","walktrap")) {
    set.seed(seed)
    # }

    switch(
      cluster,
      none = {
        net_groups <- list(membership = rep(1, vcount(bsk.network)))
      },
      optimal = {
        net_groups <- cluster_optimal(bsk.network)
      },
      leiden = {
        set.seed(seed)
        net_groups <- cluster_leiden(
          bsk.network,
          objective_function = "modularity",
          n_iterations = 3,
          resolution_parameter = 0.75
        )
      },
      louvain = {
        set.seed(seed)
        net_groups <- cluster_louvain(bsk.network)
      },
      fast_greedy = {
        net_groups <- cluster_fast_greedy(bsk.network)
      },
      leading_eigen = {
        if (!is.null(seed)) {
          set.seed(seed)
        }
        net_groups <- cluster_leading_eigen(bsk.network)
      },
      spinglass = {
        if (!is.null(seed)) {
          set.seed(seed)
        }
        net_groups <- cluster_spinglass(bsk.network)
      },
      infomap = {
        net_groups <- cluster_infomap(bsk.network)
      },
      edge_betweenness = {
        net_groups <- cluster_edge_betweenness(bsk.network)
      },
      walktrap = {
        net_groups <- cluster_walktrap(bsk.network)
      },
      {
        cat("\nUnknown cluster argument. Using default algorithm\n")
        net_groups <- cluster_walktrap(bsk.network)
      }
    )
  }

  V(bsk.network)$color <- colorlist[net_groups$membership]
  V(bsk.network)$community <- net_groups$membership
  El <- as.data.frame(get.edgelist(bsk.network, names = F))

  colorlist <- colorlist()
  E(bsk.network)$color <- apply(El, 1, function(x) {
    if (V(bsk.network)$community[x[1]] == V(bsk.network)$community[x[2]]) {
      C <- colorlist[V(bsk.network)$community[x[1]]]
    } else {
      C <- "gray70"
    }
    return(C)
  })
  E(bsk.network)$lty <- 1
  E(bsk.network)$lty[E(bsk.network)$color == "gray70"] <- 5

  cl <- list()
  cl$bsk.network <- bsk.network
  cl$net_groups <- net_groups
  return(cl)
}


### switchLayout

switchLayout <- function(bsk.network, type, community.repulsion) {
  if (community.repulsion > 0) {
    # Extract community structure information
    membership <- V(bsk.network)$community
    names(membership) <- V(bsk.network)$name
    n_communities <- length(unique(membership))
    n_nodes <- vcount(bsk.network)

    # Calculate statistics for adaptive normalization
    community_sizes <- table(membership)
    avg_community_size <- mean(community_sizes)

    # Calculate adaptive repulsion strength
    repulsion_strength <- adaptive_repulsion_strength(
      community.repulsion,
      n_nodes,
      n_communities,
      avg_community_size
    )

    # Extract edgelist
    row <- get.edgelist(bsk.network)

    # Save or initialize original weights
    if (is.null(E(bsk.network)$weight[1])) {
      original_weights <- rep(1, nrow(row))
    } else {
      original_weights <- E(bsk.network)$weight
    }

    # Apply new weighting scheme with gradual growth
    new_weights <- numeric(nrow(row))

    for (i in 1:nrow(row)) {
      node1 <- row[i, 1]
      node2 <- row[i, 2]

      comm1 <- membership[which(names(membership) == node1)]
      comm2 <- membership[which(names(membership) == node2)]

      if (comm1 == comm2) {
        # INTRA-COMMUNITY Edge
        # Moderate increase with sub-linear growth
        multiplier <- 1 + (repulsion_strength^0.7) * 1.5
        new_weights[i] <- original_weights[i] * multiplier
      } else {
        # INTER-COMMUNITY Edge
        # Gradual reduction with attenuated exponential function
        divisor <- 1 + exp(repulsion_strength * 1.2) - 1
        new_weights[i] <- original_weights[i] / divisor
      }
    }

    # Apply new weights
    E(bsk.network)$weight <- new_weights
  }

  # Apply the chosen layout
  switch(
    type,
    auto = {
      l <- layout.auto(bsk.network)
    },
    circle = {
      l <- layout.circle(bsk.network)
    },
    star = {
      l <- layout.star(bsk.network)
    },
    sphere = {
      l <- layout.sphere(bsk.network)
    },
    mds = {
      l <- layout.mds(bsk.network)
    },
    fruchterman = {
      l <- layout_with_fr(bsk.network)
    },
    kamada = {
      l <- layout.kamada.kawai(bsk.network)
    }
  )

  # Normalize the layout
  l <- norm_coords(l)

  layout_results <- list(l = l, bsk.network = bsk.network)
  return(layout_results)
}


adaptive_repulsion_strength <- function(
  community.repulsion,
  n_nodes,
  n_communities,
  avg_community_size
) {
  # Scale factor based on network size
  scale_factor <- log10(n_nodes + 10) / log10(100)

  # Correction factor based on the number of communities
  community_factor <- 1 + (n_communities - 2) * 0.1
  community_factor <- max(0.5, min(community_factor, 2))

  # Sigmoidal transformation of the user parameter
  x <- community.repulsion * 10
  sigmoid_transform <- x / (1 + x)

  # Combine the factors
  strength <- sigmoid_transform * scale_factor * community_factor

  return(strength)
}

# Community repulsion

# weight.community <- function(row, membership, weigth.within, weight.between) {
#   if (as.numeric(membership[which(names(membership) == row[1])]) == as.numeric(membership[which(names(membership) == row[2])])) {
#     weight <- weigth.within
#   } else {
#     weight <- weight.between
#   }
#   return(weight)
# }

# # louvain community detection
# louvain <- function(g){
#   cl <- 50 %>%
#     rerun(cluster_louvain(g)$membership) %>%
#     do.call(rbind, .) %>%
#     data.frame() %>%
#     summarize_all(Mode)
#   cl <- list(membership=as.numeric(cl), names=V(g)$name)
#   return(cl)
# }

utils::globalVariables(c("PY", "TC"))

#' Bibliometric Analysis
#'
#' It performs a bibliometric analysis of a dataset imported from SCOPUS and Clarivate Analytics Web of Science databases.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics Web of Science file.
#' @param sep is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.
#' @return \code{biblioAnalysis} returns an object of \code{class} "bibliometrix".
#'
#' The functions \code{\link{summary}} and \code{\link{plot}} are used to obtain or print a summary and some useful plots of the results.
#'
#' An object of \code{class} "bibliometrix" is a list containing the following components:
#'
#' \tabular{lll}{
#' Articles \tab  \tab the total number of manuscripts\cr
#' Authors \tab       \tab the authors' frequency distribution\cr
#' AuthorsFrac \tab      \tab the authors' frequency distribution (fractionalized)\cr
#' FirstAuthors \tab      \tab corresponding author of each manuscript\cr
#' nAUperPaper \tab      \tab the number of authors per manuscript\cr
#' Appearances \tab      \tab the number of author appearances\cr
#' nAuthors \tab       \tab the number of authors\cr
#' AuMultiAuthoredArt \tab      \tab the number of authors of multi-authored articles\cr
#' MostCitedPapers \tab      \tab the list of manuscripts sorted by citations\cr
#' Years \tab      \tab publication year of each manuscript\cr
#' FirstAffiliation \tab      \tab the affiliation of the first author\cr
#' Affiliations \tab      \tab the frequency distribution of affiliations (of all co-authors for each paper)\cr
#' Aff_frac \tab      \tab the fractionalized frequency distribution of affiliations (of all co-authors for each paper)\cr
#' CO \tab      \tab the affiliation country of the first author\cr
#' Countries \tab      \tab the affiliation countries' frequency distribution\cr
#' CountryCollaboration \tab      \tab Intra-country (SCP) and intercountry (MCP) collaboration indices\cr
#' TotalCitation \tab      \tab the number of times each manuscript has been cited\cr
#' TCperYear \tab      \tab the yearly average number of times each manuscript has been cited\cr
#' Sources \tab      \tab the frequency distribution of sources (journals, books, etc.)\cr
#' DE \tab      \tab the frequency distribution of authors' keywords\cr
#' ID \tab      \tab the frequency distribution of keywords associated to the manuscript by SCOPUS and Clarivate Analytics Web of Science database}
#'
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#'
#' results <- biblioAnalysis(management)
#'
#' summary(results, k = 10, pause = FALSE)
#' }
#'
#' @seealso \code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export

biblioAnalysis <- function(M, sep = ";") {
  # initialize variables
  Authors <- NULL
  Authors_frac <- NULL
  FirstAuthors <- NULL
  PY <- NULL
  FAffiliation <- NULL
  Affiliation <- NULL
  Affiliation_frac <- NULL
  CO <- rep(NA, dim(M)[1])
  TC <- NULL
  TCperYear <- NULL
  SO <- NULL
  Country <- NULL
  DE <- NULL
  ID <- NULL
  MostCitedPapers <- NULL





  # M is the bibliographic dataframe
  Tags <- names(M)

  if (!("SR" %in% Tags)) {
    M <- metaTagExtraction(M, "SR")
  }

  # temporal analyis

  if ("PY" %in% Tags) {
    PY <- as.numeric(M$PY)
  }

  # Author's distribution

  if ("AU" %in% Tags) {
    listAU <- strsplit(as.character(M$AU), sep)
    listAU <- lapply(listAU, function(l) trim(l))
    # nAU=unlist(lapply(listAU,length))  # num. of authors per paper
    nAU <- lengths(listAU)
    # fracAU=unlist(lapply(nAU,function(x){rep(1/x,x)}))  # fractional frequencies
    fracAU <- rep(1 / nAU, nAU)
    AU <- unlist(listAU)

    Authors <- sort(table(AU), decreasing = TRUE)
    Authors_frac <- aggregate(fracAU, by = list(AU), "sum")
    names(Authors_frac) <- c("Author", "Frequency")
    Authors_frac <- Authors_frac[order(-Authors_frac$Frequency), ]
    FirstAuthors <- unlist(lapply(listAU, function(l) {
      if (length(l) > 0) {
        l <- l[[1]]
      } else {
        l <- NA
      }
      return(l)
    }))

    AuSingleAuthoredArt <- length(unique(FirstAuthors[nAU == 1]))
    AuMultiAuthoredArt <- length(Authors) - AuSingleAuthoredArt
  }
  # PY=as.numeric(M$PY)

  # Total Citation Distribution
  if ("TC" %in% Tags) {
    TC <- as.numeric(M$TC)
    CurrentYear <- as.numeric(format(Sys.Date(), "%Y"))
    TCperYear <- TC / (CurrentYear - PY + 1)
    if (!("DI" %in% names(M))) M$DI <- ""
    MostCitedPapers <- data.frame(M$SR, M$DI, TC, TCperYear, PY) %>%
      group_by(PY) %>%
      mutate(NTC = TC / mean(TC)) %>%
      ungroup() %>%
      select(-PY) %>%
      arrange(desc(TC)) %>%
      as.data.frame()

    names(MostCitedPapers) <- c("Paper         ", "DOI", "TC", "TCperYear", "NTC")
  }

  # References
  nReferences <- 0
  if ("CR" %in% Tags) {
    CR <- tableTag(M, "CR", sep)
    nReferences <- length(CR)
  }

  # ID Keywords
  if ("ID" %in% Tags) {
    ID <- tableTag(M, "ID", sep)
  }

  # DE Keywords
  if ("DE" %in% Tags) {
    DE <- tableTag(M, "DE", sep)
  }

  # Sources
  if ("SO" %in% Tags) {
    SO <- gsub(",", "", M$SO, fixed = TRUE)
    SO <- sort(table(SO), decreasing = TRUE)
  }

  # All Affiliations, First Affiliation and Countries
  if (("C1" %in% Tags) & (sum(!is.na(M$C1)) > 0)) {
    if (!("AU_UN" %in% Tags)) {
      M <- metaTagExtraction(M, Field = "AU_UN")
    }
    AFF <- M$AU_UN
    listAFF <- strsplit(AFF, sep, fixed = TRUE)
    nAFF <- unlist(lapply(listAFF, length))
    listAFF[nAFF == 0] <- "NA"
    fracAFF <- unlist(sapply(nAFF, function(x) {
      if (x > 0) {
        x <- rep(1 / x, x)
      } else {
        x <- 0
      }
    })) # fractional frequencies
    AFF <- trim.leading(unlist(listAFF)) # delete spaces
    Affiliation <- sort(table(AFF), decreasing = TRUE)
    Affiliation_frac <- aggregate(fracAFF, by = list(AFF), "sum")
    names(Affiliation_frac) <- c("Affiliation", "Frequency")
    Affiliation_frac <- Affiliation_frac[order(-Affiliation_frac$Frequency), ]

    # First Affiliation
    FAffiliation <- lapply(listAFF, function(l) l[1])

    # Countries
    data("countries", envir = environment())
    countries <- as.character(countries[[1]])

    ### new code{
    if (!("AU1_CO" %in% names(M))) {
      M <- metaTagExtraction(M, Field = "AU1_CO", sep)
    }
    CO <- M$AU1_CO

    Country <- tableTag(M, "AU1_CO")

    SCP_MCP <- countryCollaboration(M, Country, k = length(Country), sep)
  } else {
    M$AU1_CO <- NA
    SCP_MCP <- data.frame(Country = rep(NA, 1), SCP = rep(NA, 1))
  }
  if ("DT" %in% names(M)) {
    Documents <- table(M$DT)
    n <- max(nchar(names(Documents)))
    names(Documents) <- substr(paste(names(Documents), "                                              ", sep = ""), 1, n + 5)
  } else {
    Documents <- NA
  }

  # international collaboration
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU_CO", sep)
  }
  Coll <- unlist(lapply(strsplit(M$AU_CO, sep), function(l) {
    length(unique(l)) > 1
  }))

  results <- list(
    Articles = dim(M)[1], # Articles
    Authors = Authors, # Authors' frequency distribution
    AuthorsFrac = Authors_frac, # Authors' frequency distribution (fractionalized)
    FirstAuthors = FirstAuthors, # First Author's list
    nAUperPaper = nAU, # N. Authors per Paper
    Appearances = sum(nAU), # Author appearances
    nAuthors = dim(Authors), # N. of Authors
    AuMultiAuthoredArt = AuMultiAuthoredArt, # N. of Authors of multi-authored articles
    AuSingleAuthoredArt = AuSingleAuthoredArt, # N. of Authors of single-authored articles
    MostCitedPapers = MostCitedPapers, # Papers sorted by citations
    Years = PY, # Years
    FirstAffiliation = unlist(FAffiliation), # Affiliation of First Author
    Affiliations = Affiliation, # Affiliations of all authors
    Aff_frac = Affiliation_frac, # Affiliations of all authors (fractionalized)
    CO = CO, # Country of each paper
    Countries = Country, # Countries' frequency distribution
    CountryCollaboration = SCP_MCP, # Intracountry (SCP) and intercountry (MCP) collaboration
    TotalCitation = TC, # Total Citations
    TCperYear = TCperYear, # Total Citations per year
    Sources = SO, # Sources
    DE = DE, # Keywords
    ID = ID, # Authors' keywords
    Documents = Documents,
    IntColl = sum(Coll) / nrow(M) * 100,
    nReferences = nReferences, # N. of References
    DB = M$DB[1]
  )
  class(results) <- "bibliometrix"

  return(results)
}
countryCollaboration <- function(M, Country, k, sep) {
  if (!("AU_CO" %in% names(M))) {
    M <- metaTagExtraction(M, Field = "AU_CO", sep)
  }
  M$SCP <- 0
  M$SCP_CO <- NA
  for (i in 1:dim(M)[1]) {
    if (!is.na(M$AU_CO[i])) {
      co <- M$AU_CO[i]
      co <- table(unlist(strsplit(co, ";")))
      if (length(co) == 1) {
        M$SCP[i] <- 1
      }
      M$SCP_CO[i] <- M$AU1_CO[i]
    } else {
      M$SCP[i] <- NA
    }
  }

  if (k == 0) {
    return(data.frame(Country = character(0), SCP = numeric(0), MCP = numeric(0)))
  }

  CO <- names(Country)[1:k]

  df <- data.frame(Country = rep(NA, k), SCP = rep(0, k))
  for (i in 1:length(CO)) {
    co <- CO[i]
    df$Country[i] <- co
    df$SCP[i] <- sum(M$SCP[M$SCP_CO == co], na.rm = T)
  }
  df$MCP <- as.numeric(tableTag(M, "AU1_CO")[1:k]) - df$SCP
  return(df)
}

#' Bibliographic data frame time slice
#'
#' Divide a bibliographic data frame into time slice
#'
#'
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param breaks is a numeric vector of two or more unique cut points.
#' @param k is an integer value giving the number of intervals into which the data frame is to be cut. \code{k} is used only in case \code{breaks} argument is not provided. The default is \code{k = 5}.
#' @return the value returned from \code{split} is a list containing the data frames for each sub-period.
#'
#'
#'
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' list_df <- timeslice(scientometrics, breaks = c(1995, 2005))
#'
#' names(list_df)
#'
#' @seealso \code{\link{convert2df}} to import and convert an ISI or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export
timeslice <- function(M, breaks = NA, k = 5) {
  M$PY <- as.numeric(M$PY)
  period <- list()
  if (is.na(breaks[1]) & is.numeric(k)) {
    breaks <- (floor(seq(min(M$PY, na.rm = TRUE) - 1, max(M$PY, na.rm = TRUE), length.out = k + 1)))
  } else {
    breaks <- c(min(M$PY, na.rm = TRUE) - 1, breaks, max(M$PY, na.rm = TRUE))
  }
  df <- cut(M$PY, breaks)
  N <- levels(df)
  ind <- as.numeric(df)
  df <- split(M, ind)
  names(df) <- N
  return(df)
}

utils::globalVariables(c(
  "corresponding_institution_ids",
  "corresponding_author_name",
  "corresponding_author_affiliation",
  "corresponding_author_country",
  "C1_ID"
))


importOAFiles <- function(file) {
  objName <- load(file)
  if (!isTRUE(inherits(eval(parse(text = objName)), c("list")))) {
    message(
      "the rdata file does not contain a valid object!\nopenalexR API requests have to be exported as 'list'.\nPlease set argument output='list' when use oafetch.\n

    ## Example ##
  works_from_dois <- oa_fetch(\n
  entity = 'works',\n
  doi = c('10.1016/j.joi.2017.08.007', 'https://doi.org/10.1007/s11192-013-1221-3'),\n
  output = 'list',
  verbose = TRUE\n
  )
    "
    )
    return(NA)
  }
  if (length(objName) != 1) {
    message("the rdata file contains more than an object!")
    return(NA)
  }
  return(eval(parse(text = objName)))
}


# Function to apply the extraction to the whole list of items.
apiOA2df <- function(file) {
  DATA <- importOAFiles(file)

  if (inherits(DATA, "list")) {
    type <- "list"
  } else if (inherits(DATA, "data.frame")) {
    type <- "data.frame"
  }

  switch(
    type,
    "list" = {
      df <- purrr::map_dfr(DATA, extract_all_metadata)
      df <- relabelling_OA_API(df)
    },
    "data.frame" = {}
  )

  df$AF <- df$AU
  df$AU_UN <- df$C3 <- df$C1

  # move all char strings to Upper
  ind <- apply(df, 2, function(x) {
    sum(regexpr("https://", x) > -1, na.rm = TRUE) > 0
  })
  label <- names(ind)[ind == FALSE & !is.na(ind)]
  AB <- df$AB
  TI <- df$TI
  DE <- df$DE
  if ("DI" %in% names(df)) {
    df <- df %>%
      mutate(
        across(all_of(label), toupper),
        DI = gsub("https://doi.org/", "", DI),
        DI = ifelse(DI == "null", NA, DI)
      )
  } else {
    df <- df %>%
      mutate(across(all_of(label), toupper))
  }
  df$DB <- "OPENALEX"
  df$AB_raw <- AB
  df$TI_raw <- TI
  df$DE_raw <- DE

  ## transform Country code in names
  CO <- strsplit(df$AU_CO, ";")
  CO <- data.frame(
    Alpha2 = trimws(unlist(CO)),
    id_oa = rep(df$id_oa, lengths(CO))
  )
  CO <- CO %>%
    left_join(
      openalexR::countrycode %>% select("Alpha2", "Country"),
      by = "Alpha2"
    ) %>%
    mutate(Country = toupper(Country)) %>%
    group_by(id_oa) %>%
    summarize(
      AU_CO = paste0(Country, collapse = ";")
    )
  df <- df %>%
    select(-"AU_CO") %>%
    left_join(CO, by = "id_oa")

  ## transform Corresponding Country code in names
  CO <- strsplit(df$AU1_CO, ";")
  CO <- data.frame(
    Alpha2 = trimws(unlist(CO)),
    id_oa = rep(df$id_oa, lengths(CO))
  )
  CO <- CO %>%
    left_join(
      openalexR::countrycode %>% select("Alpha2", "Country"),
      by = "Alpha2"
    ) %>%
    mutate(Country = toupper(Country)) %>%
    group_by(id_oa) %>%
    summarize(
      AU1_CO = paste0(Country, collapse = ";")
    )
  df <- df %>%
    select(-"AU1_CO") %>%
    left_join(CO, by = "id_oa")

  df$id_oa <- gsub("https://openalex.org/", "", df$id_oa)

  df <- df %>% as.data.frame()
  df <- .flagOAAuthorship(df)
  return(df)
}

# ---------------------------------------------------------------------------
# Authorship data-quality flag
# ---------------------------------------------------------------------------

#' Flag OpenAlex records whose author list looks corrupted
#'
#' OpenAlex occasionally absorbs a paper's cited-reference authors into its
#' \code{authorships} array -- typically for low-quality / predatory journals
#' whose PDFs are auto-parsed. Such records carry an implausibly long \code{AU}
#' list of which only a small share have an OpenAlex \code{author.id}. A single
#' such record turns the co-authorship network into a giant artificial clique.
#'
#' Records are NOT removed: a blanket author-count cap would also corrupt
#' legitimate hyper-authorship papers (physics, genomics consortia). They are
#' flagged in \code{df$AU_FLAG} (logical) and reported through a warning, so the
#' user can inspect and, if appropriate, exclude them before running
#' collaboration analyses.
#'
#' @param df A data frame produced by \code{csvOA2df()} / \code{apiOA2df()}.
#' @return \code{df} with a logical \code{AU_FLAG} column added.
#' @keywords internal
#' @noRd
.flagOAAuthorship <- function(df) {
  if (is.null(df) || !"AU" %in% names(df) || nrow(df) == 0) return(df)

  n_tokens <- function(x) {
    vapply(strsplit(as.character(x), ";", fixed = TRUE),
           function(t) sum(nzchar(trimws(t))), integer(1))
  }
  n_au   <- n_tokens(df$AU)
  n_auid <- if ("AU_ID" %in% names(df)) n_tokens(df$AU_ID) else rep(0L, nrow(df))

  # Suspect = many authors, yet a large share carry no OpenAlex author id.
  # AU and AU_ID are both built from the same authorships array, so for a
  # clean record they are (nearly) equal in length; a wide gap signals that
  # reference authors leaked into the author list.
  flag <- n_au >= 10 & n_auid < 0.6 * n_au
  flag[is.na(flag)] <- FALSE
  df$AU_FLAG <- flag

  if (any(flag)) {
    warning(sprintf(
      paste0("%d record(s) have a suspicious authorship list (likely OpenAlex ",
             "source corruption: cited-reference authors absorbed into the ",
             "author field). Flagged in M$AU_FLAG; inspect row(s) %s before ",
             "running collaboration networks."),
      sum(flag), paste(which(flag), collapse = ", ")),
      call. = FALSE)
  }
  df
}

relabelling_OA_API <- function(DATA) {
  ## column re-labelling
  label <- names(DATA)
  label[label %in% "id"] <- "id_oa"
  label[label %in% "doi"] <- "DI"
  label[label %in% "title"] <- "TI"
  label[label %in% "language"] <- "LA"
  label[label %in% "publication_year"] <- "PY"
  label[label %in% "type"] <- "DT"
  label[label %in% "referenced_works_count"] <- "NR"
  label[label %in% "cited_by_count"] <- "TC"
  label[label %in% "author_id"] <- "AU_ID"
  label[label %in% "name"] <- "AU"
  label[label %in% "orcid"] <- "OI"
  label[label %in% "position"] <- "AU_POSITION"
  label[label %in% "institutions"] <- "C1"
  label[label %in% "institution_ids"] <- "C1_ID"
  label[label %in% "countries"] <- "AU_CO"
  label[label %in% "corresponding_institutions"] <- "RP"
  label[label %in% "journal_name"] <- "SO"
  label[label %in% "issn"] <- "IS"
  label[label %in% "abstract"] <- "AB"
  label[label %in% "cited_references"] <- "CR"
  label[label %in% "keywords"] <- "DE"
  label[label %in% "concepts"] <- "ID"
  label[label %in% "oa_url"] <- "URL"
  label[label %in% "sdg_display_name"] <- "SDG"
  label[label %in% "mesh_terms"] <- "MESH"

  names(DATA) <- label
  return(DATA)
}


# Basic metadata extraction
extract_basic_info <- function(article) {
  tibble(
    id = article$id %||% NA_character_,
    doi = article$doi %||% NA_character_,
    title = article$title %||% NA_character_,
    language = article$language %||% NA_character_,
    display_name = article$display_name %||% NA_character_,
    publication_year = article$publication_year %||% NA_integer_,
    publication_date = article$publication_date %||% NA_character_,
    type = article$type %||% NA_character_,
    countries_distinct_count = article$countries_distinct_count %||% NA_integer_,
    institutions_distinct_count = article$institutions_distinct_count %||% NA_integer_,
    referenced_works_count = article$referenced_works_count %||% NA_integer_,
    cited_by_count = article$cited_by_count %||% NA_integer_
  )
}

# Extraction of author information
extract_authors <- function(article) {
  authors <- article$authorships
  if (length(authors) > 0) {
    map_df(
      authors,
      ~ tibble(
        # Remove the prefix from the author ID
        author_id = if (!is.null(.x$author$id)) {
          gsub("https://openalex.org/", "", .x$author$id)
        } else {
          NA_character_
        },
        # Author's name
        name = if (!is.null(.x$author$display_name)) {
          .x$author$display_name
        } else {
          NA_character_
        },
        # ORCID (if available)
        orcid = if (!is.null(.x$author$orcid)) {
          .x$author$orcid
        } else {
          NA_character_
        },
        # Author's position
        position = if (!is.null(.x$author_position)) {
          .x$author_position
        } else {
          NA_character_
        },
        # Affiliations: Names of institutions
        institutions = if (
          !is.null(.x$institutions) && length(.x$institutions) > 0
        ) {
          inst_names <- map_chr(.x$institutions, function(inst) {
            if (!is.null(inst$display_name)) {
              inst$display_name
            } else {
              NA_character_
            }
          })
          paste(inst_names, collapse = "; ")
        } else {
          NA_character_
        },
        # Affiliations: Institution IDs (removing prefix)
        institution_ids = if (
          !is.null(.x$institutions) && length(.x$institutions) > 0
        ) {
          inst_ids <- map_chr(.x$institutions, function(inst) {
            if (!is.null(inst$id)) {
              gsub("https://openalex.org/", "", inst$id)
            } else {
              NA_character_
            }
          })
          paste(inst_ids, collapse = "; ")
        } else {
          NA_character_
        },
        # Countries of affiliations
        countries = if (!is.null(.x$countries) && length(.x$countries) > 0) {
          paste(.x$countries, collapse = "; ")
        } else {
          NA_character_
        }
      )
    )
  } else {
    tibble(
      author_id = NA_character_,
      name = NA_character_,
      orcid = NA_character_,
      position = NA_character_,
      institutions = NA_character_,
      institution_ids = NA_character_,
      countries = NA_character_
    )
  }
}

compress_author_affiliation_info <- function(authors_info) {
  # Helper function to concatenate only non-NA values
  concat_non_na <- function(x) {
    if (!is.null(x) && all(is.na(x))) {
      return(NA)
    } else if (is.null(x)) {
      return(NA)
    } else {
      return(paste(na.omit(x), collapse = "; "))
    }
  }

  # Check for the existence of columns before concatenating
  compressed_info <- tibble(
    author_id = if ("author_id" %in% names(authors_info)) {
      concat_non_na(authors_info$author_id)
    } else {
      NA
    },
    name = if ("name" %in% names(authors_info)) {
      concat_non_na(authors_info$name)
    } else {
      NA
    },
    orcid = if ("orcid" %in% names(authors_info)) {
      concat_non_na(authors_info$orcid)
    } else {
      NA
    },
    position = if ("position" %in% names(authors_info)) {
      concat_non_na(authors_info$position)
    } else {
      NA
    },
    institutions = if ("institutions" %in% names(authors_info)) {
      concat_non_na(authors_info$institutions)
    } else {
      NA
    },
    institution_ids = if ("institution_ids" %in% names(authors_info)) {
      concat_non_na(authors_info$institution_ids)
    } else {
      NA
    },
    countries = if ("countries" %in% names(authors_info)) {
      concat_non_na(authors_info$countries)
    } else {
      NA
    }
  )

  return(compressed_info)
}

# Extract journal information
extract_journal_info <- function(article) {
  primary_loc <- article$primary_location

  if (!is.null(primary_loc$source)) {
    journal_name <- primary_loc$source$display_name %||% NA_character_
    issn <- primary_loc$source$issn_l %||% NA_character_
  } else {
    journal_name <- NA_character_
    issn <- NA_character_
  }

  tibble(
    journal_name = journal_name,
    issn = issn,
    is_oa = primary_loc$is_oa %||% NA,
    oa_status = article$open_access$oa_status %||% NA_character_,
    oa_url = article$open_access$oa_url %||% NA_character_
  )
}

# Extract abstract
extract_abstracts <- function(article) {
  abstract <- if (!is.null(article$abstract_inverted_index)) {
    abstract_build(article$abstract_inverted_index)
  } else {
    NA
  }

  tibble(
    abstract = abstract
  )
}

abstract_build <- function(ab) {
  if (is.null(ab)) {
    return(NA)
  }
  w <- rep(names(ab), lengths(ab))
  ind <- unlist(ab)
  if (is.null(ind)) {
    return("")
  }

  paste(w[order(ind)], collapse = " ", sep = "")
}

# Extract referenced citations
extract_cited_references <- function(article) {
  if (
    !is.null(article$referenced_works) && length(article$referenced_works) > 0
  ) {
    cited_references <- article$referenced_works %>%
      map_chr(~ gsub("https://openalex.org/", "", .x)) %>%
      paste(collapse = "; ")
  } else {
    cited_references <- NA
  }

  tibble(cited_references = cited_references)
}

# Extract grants
extract_grants <- function(article) {
  if (!is.null(article$grants) && length(article$grants) > 0) {
    grants_info <- article$grants %>%
      map_chr(
        ~ paste(
          ifelse(!is.null(.x$award_id), .x$award_id, NA),
          ifelse(!is.null(.x$funding_agency), .x$funding_agency, NA),
          sep = ": "
        )
      ) %>%
      paste(collapse = "; ")
  } else {
    grants_info <- NA
  }

  tibble(grants = grants_info)
}

# Extract SDGs
extract_sdg <- function(article) {
  if (
    !is.null(article$sustainable_development_goals) &&
      length(article$sustainable_development_goals) > 0
  ) {
    tibble(
      sdg_display_name = article$sustainable_development_goals %>%
        map_chr(~ ifelse(!is.null(.x$display_name), .x$display_name, NA)) %>%
        paste(collapse = "; "),
      sdg_id = article$sustainable_development_goals %>%
        map_chr(~ ifelse(!is.null(.x$id), .x$id, NA)) %>%
        paste(collapse = "; "),
      sdg_score = article$sustainable_development_goals %>%
        map_chr(~ as.character(ifelse(!is.null(.x$score), .x$score, NA))) %>%
        paste(collapse = "; ")
    )
  } else {
    tibble(
      sdg_display_name = NA,
      sdg_id = NA,
      sdg_score = NA
    )
  }
}

# Extract Mesh Terms
extract_mesh_terms <- function(article) {
  if (!is.null(article$mesh) && length(article$mesh) > 0) {
    mesh_terms <- article$mesh %>%
      map_chr(
        ~ ifelse(!is.null(.x$descriptor_name), .x$descriptor_name, NA)
      ) %>%
      paste(collapse = "; ")
  } else {
    mesh_terms <- NA
  }

  tibble(mesh_terms = mesh_terms)
}

# Extract keywords
extract_keywords <- function(article) {
  if (!is.null(article$keywords) && length(article$keywords) > 0) {
    keywords_info <- article$keywords %>%
      map_chr(~ ifelse(!is.null(.x$display_name), .x$display_name, NA)) %>%
      paste(collapse = "; ")
  } else {
    keywords_info <- NA
  }

  tibble(keywords = keywords_info)
}

# Extract concepts
extract_concepts <- function(article) {
  if (!is.null(article$concepts) && length(article$concepts) > 0) {
    concepts_info <- article$concepts %>%
      map_chr(
        ~ paste(
          ifelse(!is.null(.x$display_name), .x$display_name, NA),
          "(",
          ifelse(!is.null(.x$score), .x$score, NA),
          ")"
        )
      ) %>%
      paste(collapse = "; ")
  } else {
    concepts_info <- NA
  }

  tibble(concepts = concepts_info)
}

# Extract topics
extract_topics <- function(article) {
  if (!is.null(article$topics) && length(article$topics) > 0) {
    topics_info <- article$topics %>%
      map_chr(~ ifelse(!is.null(.x$display_name), .x$display_name, NA)) %>%
      paste(collapse = "; ")
  } else {
    topics_info <- NA
  }

  tibble(topics = topics_info)
}

# Extract additional information (citations per year, FWCI, etc.)
extract_additional_info <- function(article) {
  yearly_citations <- if (!is.null(article$counts_by_year)) {
    paste(
      sapply(article$counts_by_year, function(count) {
        paste(count$year, count$cited_by_count, sep = ": ")
      }),
      collapse = "; "
    )
  } else {
    NA
  }

  tibble(
    yearly_citations = yearly_citations,
    fwci = article$fwci %||% NA_real_,
    apc_value = article$apc_list$value %||% NA_real_,
    apc_currency = article$apc_list$currency %||% NA_character_,
    has_fulltext = article$has_fulltext %||% NA
  )
}

extract_corresponding_info <- function(authorships) {
  # Trova se esiste un corresponding author
  if (is.null(authorships) || length(authorships) == 0) {
    return(tibble(
      AU_CORR = NA,
      AU_CORR_ID = NA,
      AU1_CO = NA,
      AU1_UN = NA,
      RP = NA
    ))
  }
  corr_idx <- which(sapply(authorships, function(a) isTRUE(a$is_corresponding)))

  if (length(corr_idx) > 0) {
    # Se esiste almeno un corresponding author, prendi il primo
    selected_author <- authorships[[corr_idx[1]]]
  } else {
    # Altrimenti prendi il primo autore
    first_idx <- which(sapply(authorships, function(a) {
      a$author_position == "first"
    }))
    selected_author <- authorships[[first_idx[1]]]
  }

  # Estrai nome e ID autore
  display_name <- selected_author$author$display_name %||% NA_character_
  id <- selected_author$author$id %||% NA_character_

  # Estrai il paese se disponibile
  if (
    !is.null(selected_author$countries) && length(selected_author$countries) > 0
  ) {
    country <- selected_author$countries[[1]]
  } else if (
    !is.null(selected_author$institutions) &&
      length(selected_author$institutions) > 0 &&
      !is.null(selected_author$institutions[[1]]$country_code)
  ) {
    country <- selected_author$institutions[[1]]$country_code
  } else {
    country <- NA
  }

  # Estrai l'affiliazione se disponibile
  if (
    !is.null(selected_author$affiliations) &&
      length(selected_author$affiliations) > 0 &&
      !is.null(selected_author$affiliations[[1]]$raw_affiliation_string)
  ) {
    affiliation <- selected_author$affiliations[[1]]$raw_affiliation_string
  } else {
    affiliation <- NA
  }

  return(tibble(
    AU_CORR = display_name,
    AU_CORR_ID = id,
    AU1_CO = country,
    AU1_UN = affiliation,
    RP = affiliation
  ))
}

# Function to combine all extractions
extract_all_metadata <- function(article) {
  authors_info <- extract_authors(article)
  bind_cols(
    extract_basic_info(article),
    compress_author_affiliation_info(authors_info),
    extract_corresponding_info(article$authorships),
    extract_journal_info(article),
    extract_abstracts(article),
    extract_cited_references(article),
    extract_grants(article),
    extract_sdg(article),
    extract_mesh_terms(article),
    extract_keywords(article),
    extract_concepts(article),
    extract_topics(article),
    extract_additional_info(article)
  )
}

utils::globalVariables(c("level"))
#' Three Fields Plot
#'
#' Visualize the main items of three fields (e.g. authors, keywords, journals), and how they are related through a Sankey diagram.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param fields is a character vector. It indicates the fields to analyze using the standard WoS field tags.
#'        Default is \code{fields = c("AU","DE", "SO")}.
#' @param n is a integer vector. It indicates how many items to plot, for each of the three fields.
#'        Default is \code{n = c(20, 20, 20)}
#' @return a sankeyPlot
#'
#'
#' @examples
#'
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # threeFieldsPlot(scientometrics, fields=c("DE","AU","CR"),n=c(20,20,20))
#'
#' @export
#'
threeFieldsPlot <- function(M, fields = c("DE", "AU", "SO"), n = c(20, 20, 20)) {
  if ("CR_SO" %in% fields) {
    M <- metaTagExtraction(M, "CR_SO")
  }
  if ("AU_UN" %in% fields) {
    M <- metaTagExtraction(M, "AU_UN")
  }
  if ("AB_TM" %in% fields) {
    M <- termExtraction(M, "AB")
  }
  if ("TI_TM" %in% fields) {
    M <- termExtraction(M, "TI")
  }
  if ("AU_CO" %in% fields) {
    M <- metaTagExtraction(M, "AU_CO")
  }


  Binary <- rep(FALSE, 3)
  ind <- which(fields %in% "CR")
  if (length(ind) > 0) Binary[ind] <- TRUE

  ### Document x Attribute matrix Field MIDDLE
  WM <- cocMatrix(M, fields[2], binary = Binary[2], n = n[2])
  n2 <- min(n[2], ncol(WM))
  TopM <- colnames(WM)

  docInd <- which(Matrix::rowSums(WM[, TopM]) > 0)

  ### Document x Attribute matrix Field LEFT
  WL <- cocMatrix(M, fields[1], binary = Binary[1], n = n[1])
  n1 <- min(n[1], ncol(WL))
  # TopL=names(sort(Matrix::colSums(WL),decreasing = TRUE))[1:n1]
  TopL <- colnames(WL)
  # WL=WL[,TopL]



  ### Document x Attribute matrix Field RIGHT
  WR <- cocMatrix(M, fields[3], binary = Binary[3], n = n[3])
  n3 <- min(n[3], ncol(WR))
  # TopR=names(sort(Matrix::colSums(WR),decreasing = TRUE))[1:n3]
  TopR <- colnames(WR)
  # WR=WR[,TopR]

  ### Co-Occurrence Matrices
  LM <- Matrix::crossprod(WL[docInd, ], WM[docInd, ])
  MR <- Matrix::crossprod(WM[docInd, ], WR[docInd, ])

  row.names(LM) <- 1:n1
  colnames(LM) <- row.names(MR) <- (n1 + 1):(n2 + n1)
  colnames(MR) <- (n2 + n1 + 1):(n1 + n2 + n3)

  LMm <- meltx(as.matrix(LM))
  LMm$group <- NA
  MRm <- meltx(as.matrix(MR))
  MRm$group <- NA

  Edges <- rbind(LMm, MRm)
  Edges$Var1 <- as.numeric(Edges$Var1)
  Edges$Var2 <- as.numeric(Edges$Var2)
  names(Edges) <- c("from", "to", "Value", "group")
  Edges <- Edges[!is.na(Edges$to) & !is.na(Edges$from), ]
  Edges$from <- Edges$from - 1
  Edges$to <- Edges$to - 1
  Edges <- Edges[, -4]

  Nodes <- tolower(c(TopL, TopM, TopR))

  Nodes <- data.frame(
    Nodes = Nodes,
    group = c(rep(fields[1], length(TopL)), rep(fields[2], length(TopM)), rep(fields[3], length(TopR))),
    level = c(rep(1, length(TopL)), rep(2, length(TopM)), rep(3, length(TopR)))
  )


  min.flow <- 1
  names(Edges)[3] <- "weight"
  Edges <- Edges[Edges$weight >= min.flow, ]
  # ind=which(!((0:(sum(n)-1)) %in% c(Edges$from,Edges$to)))
  # Nodes[ind,]=c("","")

  ###########
  Kx <- length(table(Nodes$group))
  Ky <- nrow(Nodes)
  Nodes <- Nodes %>%
    # mutate(coordX=factor(group, labels = seq(from= 0, to= 1, by= 1/(Kx-0.8)))) %>%  # before: seq(from= 0, to= 1, by= 1/(Kx-1))
    mutate(
      coordX = rep(seq(from = 0, to = 1, by = 1 / (Kx - 0.8)), as.numeric(table(level))),
      coordY = rep(0.1, Ky)
    )

  colornodes <- c("#9E0142", "#D53E4F", "#F46D43", "#FDAE61", "#FEE08B", "#E6F598", "#ABDDA4", "#66C2A5", "#3288BD", "#5E4FA2")
  Nodes$color <- colorRampPalette(colornodes)(nrow(Nodes))
  Nodes$id <- (1:nrow(Nodes)) - 1

  ## identify and remove nodes with empty edges
  ind <- setdiff(Nodes$id, unique(c(Edges$from, Edges$to)))
  if (length(ind) > 0) {
    Nodes <- Nodes[-(ind + 1), ]
    Nodes$idnew <- (1:nrow(Nodes)) - 1
    ## replace old edge ids with new ones
    for (i in 1:nrow(Nodes)) {
      old <- Nodes$id[i]
      new <- Nodes$idnew[i]
      Edges$from[Edges$from == old] <- new
      Edges$to[Edges$to == old] <- new
    }
  }

  ## Build sankey plot
  m <- list(
    l = 50,
    r = 50,
    b = 100,
    t = 100,
    pad = 2
  )

  plotly::plot_ly(
    type = "sankey",
    arrangement = "snap",
    node = list(
      label = Nodes$Nodes,
      x = Nodes$coordX,
      y = Nodes$coordY,
      # color = "black",
      color = Nodes$color
      # colors = colorRampPalette(brewer.pal(10,"Spectral"))(nrow(Nodes)),
      , pad = 4
    ), # 10 Pixel
    link = list(
      source = Edges$from,
      target = Edges$to,
      value = Edges$weight
      # ,color = adjustcolor(Edges$color,alpha.f=0.4)
    )
  ) %>%
    layout(margin = m) %>%
    plotly::add_annotations(
      x = Nodes$coordX,
      y = 1.08,
      text = factor(Nodes$group),
      showarrow = F, xanchor = "center",
      font = list(
        color = "Dark",
        family = "TimesNewRoman",
        size = 14
      )
    ) %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        "toImage",
        "sendDataToCloud",
        "pan2d",
        "select2d",
        "lasso2d",
        "toggleSpikelines",
        "hoverClosestCartesian",
        "hoverCompareCartesian"
      )
    )

  # networkD3::sankeyNetwork(Links = Edges, Nodes = Nodes, Source = "from", Target = "to",
  #                          NodeID = "Nodes", Value = "weight", width = width,height=height,fontSize = 12,
  #                          nodeWidth = 30,  NodeGroup = "group",LinkGroup = "group")
}



## function to melt data
meltx <- function(LM) {
  var1 <- rep((1:nrow(LM)), ncol(LM))
  var2 <- sort(var1)
  LMM <-
    data.frame(
      Var1 = rownames(LM)[var1],
      Var2 = colnames(LM)[var2],
      value = matrix(LM, length(LM), 1)
    )
  return(LMM)
}

#' Load the Biblioshiny Book Dataset
#'
#' Downloads and caches the management collection dataset used in the book
#' "Science Mapping Analysis - A primer with Biblioshiny"
#' by Massimo Aria and Corrado Cuccurullo (McGraw-Hill, 2026).
#' The dataset is downloaded from GitHub on first use and
#' cached locally in \code{~/.bibliometrix/} for subsequent calls.
#'
#' The dataset contains a comprehensive collection of original research articles,
#' written in English, that employ bibliometric or scientometric methods within
#' management and business fields, as indexed by Web of Science, over the period
#' 1985--2025.
#'
#' @param force_download Logical. If \code{TRUE}, re-downloads the dataset even
#'   if a cached copy exists locally. Default is \code{FALSE}.
#' @importFrom utils download.file
#'
#' @return A bibliometrix data frame containing the book dataset.
#'
#' @references
#' Aria, M., & Cuccurullo, C. (2026). \emph{Science Mapping Analysis - A primer
#' with Biblioshiny}. McGraw-Hill. ISBN: 978-88-386-2297-7.
#'
#' @examples
#' \dontrun{
#' M <- loadBookDataset()
#' }
#'
#' @export
loadBookDataset <- function(force_download = FALSE) {
  cache_dir <- file.path(path.expand("~"), ".bibliometrix")
  cache_file <- file.path(cache_dir, "management_collection.rdata")
  url <- "https://github.com/massimoaria/biblioshiny-book/raw/main/datasets/management_collection.rdata"

  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }

  if (isTRUE(force_download) || !file.exists(cache_file)) {
    tryCatch(
      download.file(url, cache_file, mode = "wb", quiet = TRUE),
      error = function(e) {
        if (file.exists(cache_file)) file.remove(cache_file)
        stop("Failed to download the book dataset: ", e$message, call. = FALSE)
      }
    )
  }

  env <- new.env(parent = emptyenv())
  load(cache_file, envir = env)
  obj_names <- ls(env)

  if (length(obj_names) == 0) {
    stop("The downloaded file does not contain any R objects.", call. = FALSE)
  }

  get(obj_names[1], envir = env)
}

utils::globalVariables(c(
  "params",
  "Cluster_Label",
  "Occurrences",
  "Words",
  "Cluster_Label.y",
  "Cluster_Label.x",
  "min",
  "tot",
  "len.x",
  "len.y",
  "Occurrences.x",
  "Occurrences.y",
  "tot.x",
  "tot.y",
  "Cluster.x",
  "Cluster.y",
  "Occ",
  "name",
  "group",
  "Inc_Weighted",
  "PageRank",
  "PageRankIndex",
  "PageRankProd",
  "PageRankTot.x",
  "PageRankTot.y",
  "PageRankXY",
  "PageRankXYTot",
  "has_natural_successor",
  "idnew",
  "is_natural_lineage",
  "lineage_strength",
  "new old_combined",
  "pagerank_assigned",
  "pagerank_centrality.x",
  "pagerank_centrality.y",
  "rel_freq_temp",
  "separate_longer_delim",
  "total_pagerank"
))

#' Perform a Thematic Evolution Analysis
#'
#' It performs a Thematic Evolution Analysis based on co-word network analysis and clustering.
#' The methodology is inspired by the proposal of Cobo et al. (2011).
#'
#' \code{\link{thematicEvolution}} starts from two or more thematic maps created by \code{\link{thematicMap}} function.\cr\cr
#'
#' Reference:\cr
#' Cobo, M. J., Lopez-Herrera, A. G., Herrera-Viedma, E., & Herrera, F. (2011). An approach for detecting, quantifying,
#' and visualizing the evolution of a research field: A practical application to the fuzzy sets theory field. Journal of Informetrics, 5(1), 146-166.\cr
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#' @param field is a character object. It indicates the content field to use. Field can be one of c=("ID","DE","KW_Merged","TI","AB"). Default value is \code{field="ID"}.
#' @param years is a numeric vector of one or more unique cut points.
#' @param n is numerical. It indicates the number of words to use in the network analysis
#' @param minFreq is numerical. It indicates the min frequency of words included in to a cluster.
#' @param ngrams is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
#' An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.
#' @param stemming is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).
#' @param size is numerical. It indicates del size of the cluster circles and is a number in the range (0.01,1).
#' @param n.labels is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.
#' @param repel is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @param cluster is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").
#' @param seed is numerical. It indicates the seed for random number generator to obtain always the same results. Default value is \code{seed = 1234}.
#' @param assign.evolution.colors is a list. If \code{assignEvolutionColors = list(assign = TRUE)}, colors are assigned to lineages based on the highest weighted inclusion value. If a list is provided, it must contain the arguments \code{assignEvolutionColors = list(assign = c(TRUE, FALSE), alpha=0.5)}.
#' Default is \code{assign.evolution.colors = list(assign=TRUE, alpha = 0.5)}. If assign = FALSE, measure argument is ignored.
#' @return a list containing:
#' \tabular{lll}{
#' \code{nets}\tab   \tab The thematic nexus graph for each comparison\cr
#' \code{incMatrix}\tab   \tab Some useful statistics about the thematic nexus}
#'
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' years=c(2004,2008,2015)
#'
#' nexus <- thematicEvolution(management,field="DE",years=years,n=100,minFreq=2)
#' }
#'
#' @seealso \code{\link{thematicMap}} function to create a thematic map based on co-word network analysis and clustering.
#' @seealso \code{\link{cocMatrix}} to compute a bibliographic bipartite network.
#' @seealso \code{\link{networkPlot}} to plot a bibliographic network.
#'
#' @export

thematicEvolution <- function(
  M,
  field = "ID",
  years,
  n = 250,
  minFreq = 2,
  size = 0.5,
  ngrams = 1,
  stemming = FALSE,
  n.labels = 1,
  repel = TRUE,
  remove.terms = NULL,
  synonyms = NULL,
  cluster = "louvain",
  seed = 1234,
  assign.evolution.colors = list(assign = TRUE, alpha = 0.5)
) {
  list_df <- timeslice(M, breaks = years)
  K <- length(list_df)
  S <- net <- res <- list()
  Y <- NULL

  for (k in 1:K) {
    Mk <- list_df[[k]]
    Y[k] <- paste(min(Mk$PY), "-", max(Mk$PY), sep = "", collapse = "")
    resk <- thematicMap(
      Mk,
      field = field,
      n = n,
      minfreq = minFreq,
      ngrams = ngrams,
      stemming = stemming,
      size = size,
      n.labels = n.labels,
      repel = repel,
      remove.terms = remove.terms,
      synonyms = synonyms,
      cluster = cluster,
      subgraphs = FALSE,
      seed = seed
    )
    resk$params <- resk$params %>% dplyr::filter(params != "minfreq")
    res[[k]] <- resk
    net[[k]] <- resk$net
  }

  if (K < 2) {
    print("Error")
    return()
  }
  incMatrix <- list()
  for (k in 2:K) {
    res1 <- res[[(k - 1)]]
    res2 <- res[[(k)]]
    if (res1$nclust == 0 | res2$nclust == 0) {
      cat(paste(
        "\nNo topics in the period ",
        k - 1,
        " with this set of input parameters\n\n"
      ))
      return(list(check = FALSE))
    }
    res1$words$Cluster <- paste(
      res1$clusters$name[res1$words$Cluster],
      "--",
      Y[k - 1],
      sep = ""
    )
    res1$clusters$label <- paste(res1$clusters$name, "--", Y[k - 1], sep = "")
    res2$words$Cluster <- paste(
      res2$clusters$name[res2$words$Cluster],
      "--",
      Y[k],
      sep = ""
    )
    res2$clusters$label <- paste(res2$clusters$name, "--", Y[k], sep = "")
    cluster1 <- res1$words %>%
      group_by(Cluster_Label) %>%
      mutate(
        len = length(Words),
        tot = sum(Occurrences),
        PageRankTot = sum(pagerank_centrality, na.rm = TRUE)
      )
    cluster2 <- res2$words %>%
      group_by(Cluster_Label) %>%
      mutate(
        len = length(Words),
        tot = sum(Occurrences),
        PageRankTot = sum(pagerank_centrality, na.rm = TRUE)
      )
    A <- inner_join(cluster1, cluster2, by = "Words") %>%
      group_by(Cluster_Label.x, Cluster_Label.y) %>%
      rowwise() %>%
      mutate(
        min = min(
          Occurrences.x,
          Occurrences.y
        ),
        Occ = sum(Occurrences.x),
        tot = min(tot.x, tot.y),
        PageRankTot.x = min(PageRankTot.x),
        PageRankTot.y = min(PageRankTot.y),
        PageRankXY = pagerank_centrality.x * pagerank_centrality.y
      ) %>%
      ungroup() %>%
      mutate(PageRankProd = PageRankTot.x * PageRankTot.y)
    B <- A %>%
      group_by(Cluster_Label.x, Cluster_Label.y) %>%
      reframe(
        CL1 = Cluster.x[1],
        CL2 = Cluster.y[1],
        Words = paste0(Words, collapse = ";", sep = ""),
        sum = sum(min),
        Inc_Weighted = sum(min) / min(tot),
        Inc_index = length(Words) /
          min(
            len.x,
            len.y
          ),
        Occ = Occ[1],
        Tot = tot[1],
        Stability = length(Words) /
          (len.x[1] +
            len.y[1] -
            length(Words)),
        PageRank1 = sum(pagerank_centrality.x),
        PageRank2 = sum(pagerank_centrality.y),
        PageRankXYTot = sum(PageRankXY),
        PageRankIndex = sqrt(PageRankXYTot / PageRankProd)
      ) %>%
      data.frame() %>%
      distinct()
    incMatrix[[k - 1]] <- B
  }
  INC <- incMatrix[[1]]
  if (length(incMatrix) > 1) {
    for (i in 2:length(incMatrix)) {
      INC <- rbind(INC, incMatrix[[i]])
    }
  }
  edges <- INC[, c(
    "CL1",
    "CL2",
    "Inc_index",
    "Inc_Weighted",
    "Stability",
    "PageRankIndex"
  )] #%>%
  #distinct()
  # edges = edges[edges[, 3] > 0, ]
  nodes <- data.frame(name = unique(c(edges$CL1, edges$CL2)))
  nodes$group <- nodes$name

  cont <- 0
  edges[, 7] <- edges[, 1]
  for (i in nodes$name) {
    ind <- which(edges[, 1] == i)
    edges[ind, 1] <- cont
    ind1 <- which(edges[, 2] == i)
    edges[ind1, 2] <- cont
    cont <- cont + 1
  }
  names(edges) <- c(
    "from",
    "to",
    "Inclusion",
    "Inc_Weighted",
    "Stability",
    "PageRankIndex",
    "group"
  )
  edges$from <- as.numeric(edges$from)
  edges$to <- as.numeric(edges$to)

  ### for colors
  nodes <- nodes %>%
    mutate(label = name) %>%
    separate(sep = "--", col = "name", into = c("name", "group")) %>%
    mutate(slice = factor(group, labels = 1:K))
  Nodes <- data.frame()
  for (i in 1:K) {
    Nodes <- rbind(
      Nodes,
      left_join(
        subset(nodes, nodes$slice == i),
        res[[i]]$clusters[c("color", "name")],
        by = "name"
      )
    )
  }

  # Preparing data for plot
  Nodes$id <- 0:(nrow(Nodes) - 1)
  Nodes <- Nodes %>%
    left_join(
      rbind(
        INC[, -c(1, 2)] %>% select(CL1, sum) %>% rename(label = CL1),
        INC[, -c(1, 2)] %>% select(CL2, sum) %>% rename(label = CL2)
      ) %>%
        group_by(label) %>%
        reframe(sum = max(sum)),
      by = "label"
    )
  Nodes <- Nodes %>%
    group_by(slice) %>%
    mutate(sum = sum / sum(sum, na.rm = T)) %>%
    ungroup()

  params <- list(
    field = field,
    years = years,
    n = n,
    minFreq = minFreq,
    size = size,
    ngrams = ngrams,
    stemming = stemming,
    n.labels = n.labels,
    repel = repel,
    remove.terms = remove.terms,
    synonyms = synonyms,
    cluster = cluster
  )

  params <- data.frame(
    params = names(unlist(params)),
    values = unlist(params),
    row.names = NULL
  )

  periods <- sort(unique(Nodes$group))

  clusters <- data.frame(name = NULL, freq = NULL, group = NULL)

  for (i in 1:length(res)) {
    df <- res[[i]]$clusters %>%
      select(name, freq)
    df$group <- periods[i]
    clusters <- bind_rows(clusters, df)
  }

  Nodes <- Nodes %>%
    left_join(clusters, by = c("group", "name"))

  edges$color <- "#D3D3D380"

  nexus <- list(
    Nodes = Nodes,
    Edges = edges,
    clusters = clusters,
    Data = INC[, -c(1, 2)],
    check = TRUE,
    TM = res,
    Net = net,
    params = params
  )

  if (!is.list(assign.evolution.colors)) {
    assign.evolution.colors <- list(assign = TRUE, alpha = 0.5)
  }

  if (is.null(assign.evolution.colors$assign)) {
    assign.evolution.colors <- list(assign = TRUE, alpha = 0.5)
  }

  if (is.null(assign.evolution.colors$alpha)) {
    assign.evolution.colors$alpha = 0.5
  }

  if (assign.evolution.colors$assign) {
    ## Assign colors based on lineages
    nexus <- assignEvolutionColors(
      nexus = nexus,
      threshold = 0.5,
      palette = NULL,
      alpha = assign.evolution.colors$alpha
    )
  }

  return(nexus)
}

# Test per thematicMap e thematicEvolution
# Queste funzioni richiedono dataset grandi per generare cluster significativi

test_that("thematicMap funziona con dataset bibliometrixData", {
  skip_on_cran()
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  scientometrics$PY <- as.numeric(scientometrics$PY)
  scientometrics$TC <- as.numeric(scientometrics$TC)
  res <- expect_no_error(
    suppressWarnings(suppressMessages(
      thematicMap(scientometrics, field = "ID", n = 50, minfreq = 5,
                  stemming = FALSE, size = 0.5, repel = TRUE)
    ))
  )
  expect_type(res, "list")
  expect_true(all(c("map", "clusters", "words", "nclust") %in% names(res)))
  expect_true(res$nclust > 0)
})

test_that("thematicEvolution funziona con cut temporali", {
  skip_on_cran()
  skip_if_not_installed("bibliometrixData")
  data(scientometrics, package = "bibliometrixData")
  scientometrics$PY <- as.numeric(scientometrics$PY)
  scientometrics$TC <- as.numeric(scientometrics$TC)
  years <- range(scientometrics$PY, na.rm = TRUE)
  mid_year <- floor(mean(years))
  nexus <- expect_no_error(
    suppressWarnings(suppressMessages(
      thematicEvolution(scientometrics, field = "ID", years = mid_year,
                        n = 50, minFreq = 2)
    ))
  )
  expect_type(nexus, "list")
  expect_true("Nodes" %in% names(nexus))
  expect_true("Edges" %in% names(nexus))
})

utils::globalVariables(c(
  "AU", "Freq", "Country", "Collaboration", "Year",
  "MeanTCperYear", "MeanTCperArt"
))
#' Plotting bibliometric analysis results
#'
#' \code{plot} method for class '\code{bibliometrix}'
#' @param x is the object for which plots are desired.
#' @param ... can accept two arguments:\cr
#' \code{k} is an integer, used for plot formatting (number of objects). Default value is 10.\cr
#' \code{pause} is a logical, used to allow pause in screen scrolling of results. Default value is \code{pause = FALSE}.
#' @return The function \code{plot} returns a list of plots of class \code{ggplot2}.
#'
#'
#' @examples
#' data(scientometrics, package = "bibliometrixData")
#'
#' results <- biblioAnalysis(scientometrics)
#'
#' plot(results, k = 10, pause = FALSE)
#'
#' @seealso The bibliometric analysis function \code{\link{biblioAnalysis}}.
#' @seealso \code{\link{summary}} to compute a list of summary statistics of the object of class \code{bibliometrix}.
#'
#' @method plot bibliometrix
#' @export


plot.bibliometrix <- function(x, ...) {
  data("logo", package = "bibliometrix", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  if (!inherits(x, "bibliometrix")) {
    cat('\n argument "x" have to be an object of class "bibliometrix"\n')
    return(NA)
  }
  graphs <- list()

  arguments <- list(...)
  if (sum(names(arguments) == "k") == 0) {
    k <- 10
  } else {
    k <- arguments$k
  }
  if (sum(names(arguments) == "pause") == 0) {
    pause <- FALSE
  } else {
    pause <- arguments$pause
  }

  if (pause == TRUE) {
    cat("Hit <Return> to see next plot: ")
    line <- readline()
  }

  xx <- as.data.frame(x$Authors[1:k])
  xcoord <- c(k - 0.2 - (k) * 0.15, k - 0.02) + 1
  ycoord <- c(max(xx$Freq), max(xx$Freq) - diff(range(xx$Freq)) * 0.15)
  # Authors


  g <- ggplot(data = xx, aes(x = AU, y = Freq)) +
    geom_bar(stat = "identity", fill = "grey90") +
    labs(title = "Most productive Authors", x = "Authors") +
    labs(y = "N. of Documents") +
    theme(
      text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 18),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0),
      axis.text.x = element_text(size = 10),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(logo, xmin = xcoord[1], xmax = xcoord[2], ymin = ycoord[1], ymax = ycoord[2]) +
    coord_flip()
  plot(g)

  graphs$MostProdAuthors <- g

  if (pause == TRUE) {
    cat("Hit <Return> to see next plot: ")
    line <- readline()
  }
  if (!is.na(x$CountryCollaboration[1, 1])) {
    # Countries
    xx <- x$CountryCollaboration[1:k, ] 
    xx <- xx[!is.na(xx$Country),]
    xx <- xx[order(-(xx$SCP + xx$MCP)), ]
    xx1 <- cbind(xx[, 1:2], rep("SCP", nrow(xx)))
    names(xx1) <- c("Country", "Freq", "Collaboration")
    xx2 <- cbind(xx[, c(1, 3)], rep("MCP", nrow(xx)))
    names(xx2) <- c("Country", "Freq", "Collaboration")
    xx <- rbind(xx2, xx1)
    xx$Country <- factor(xx$Country, levels = xx$Country[1:dim(xx2)[1]])

    Freq <- x$CountryCollaboration$SCP[1:(nrow(xx)/2)] + x$CountryCollaboration$MCP[1:(nrow(xx)/2)]
    st <- floor(nrow(xx) / 10)
    # xcoord <- c(st-0.2-(st)*0.85, 0.02)+1
    xcoord <- c(1, max(st, 3))
    ycoord <- c(max(Freq), max(Freq) - diff(range(Freq)) * 0.15)

    g <- suppressWarnings(ggplot(data = xx, aes(x = Country, y = Freq, fill = Collaboration)) +
      geom_bar(stat = "identity") +
      scale_x_discrete(limits = rev(levels(xx$Country))) +
      scale_fill_discrete(
        name = "Collaboration",
        breaks = c("SCP", "MCP")
      ) +
      labs(
        title = "Most Productive Countries", x = "Countries", y = "N. of Documents",
        caption = "SCP: Single Country Publications, MCP: Multiple Country Publications"
      ) +
      theme(
        text = element_text(color = "#444444"),
        panel.background = element_rect(fill = "#FFFFFF"),
        panel.grid.minor = element_line(color = "#EFEFEF"),
        panel.grid.major = element_line(color = "#EFEFEF"),
        plot.title = element_text(size = 18),
        axis.title = element_text(size = 14, color = "#555555"),
        axis.title.y = element_text(vjust = 1, angle = 90),
        axis.title.x = element_text(hjust = 0),
        axis.text.x = element_text(size = 10),
        axis.line.x = element_line(color = "black", linewidth = 0.5),
        axis.line.y = element_line(color = "black", linewidth = 0.5)
      ) +
      annotation_custom(logo, xmin = xcoord[1], xmax = xcoord[2], ymin = ycoord[1], ymax = ycoord[2]) +
      coord_flip())


    plot(g)
    graphs$MostProdCountries <- g
  } else {
    graphs$MostProdCountries <- NA
  }

  if (pause == TRUE) {
    cat("Hit <Return> to see next plot: ")
    line <- readline()
  }

  # Articles per Year

  Tab <- table(x$Years)

  ## inserting missing years
  YY <- setdiff(seq(min(x$Years, na.rm = TRUE), max(x$Years, na.rm = TRUE)), names(Tab))
  Y <- data.frame(Year = as.numeric(c(names(Tab), YY)), Freq = c(as.numeric(Tab), rep(0, length(YY))))
  Y <- Y[order(Y$Year), ]

  names(Y) <- c("Year", "Freq")

  xcoord <- c(max(Y$Year) - 0.02 - diff(range(Y$Year)) * 0.15, max(Y$Year) - 0.02) + 1
  ycoord <- c(min(Y$Freq), min(Y$Freq) + diff(range(Y$Freq)) * 0.15)

  g <- ggplot(Y, aes(x = Year, y = Freq)) +
    geom_line() +
    geom_area(fill = "grey90", alpha = .5) +
    labs(
      x = "Year",
      y = "Articles",
      title = "Annual Scientific Production"
    ) +
    scale_x_continuous(breaks = (Y$Year[seq(1, length(Y$Year), by = 2)])) +
    theme(
      text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.minor = element_line(color = "#EFEFEF"),
      panel.grid.major = element_line(color = "#EFEFEF"),
      plot.title = element_text(size = 18),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90),
      axis.title.x = element_text(hjust = 0),
      axis.text.x = element_text(size = 10, angle = 90),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(logo, xmin = xcoord[1], xmax = xcoord[2], ymin = ycoord[1], ymax = ycoord[2])

  plot(g)
  graphs$AnnualScientProd <- g


  Table2 <- NA
  if (!(x$DB %in% c("COCHRANE", "PUBMED"))) {
    if (pause == TRUE) {
      cat("Hit <Return> to see next plot: ")
      line <- readline()
    }


    # Total Citation Plot
    Table2 <- aggregate(x$TotalCitation, by = list(x$Years), length)
    Table2$xx <- aggregate(x$TotalCitation, by = list(x$Years), mean)$x
    Table2$Annual <- NA
    d <- date()
    d <- as.numeric(substring(d, nchar(d) - 3, nchar(d)))
    Table2$Years <- d - Table2$Group.1
    Table2$Annual <- Table2$xx / Table2$Years
    names(Table2) <- c("Year", "N", "MeanTCperArt", "MeanTCperYear", "CitableYears")

    ## inserting missing years
    YY <- setdiff(seq(min(x$Years, na.rm = TRUE), max(x$Years, na.rm = TRUE)), Table2$Year)
    if (length(YY > 0)) {
      YY <- data.frame(YY, 0, 0, 0, 0)
      names(YY) <- c("Year", "N", "MeanTCperArt", "MeanTCperYear", "CitableYears")
      Table2 <- rbind(Table2, YY)
      Table2 <- Table2[order(Table2$Year), ]
      row.names(Table2) <- Table2$Year
    }

    xcoord <- c(max(Table2$Year) - 0.02 - diff(range(Table2$Year)) * 0.15, max(Table2$Year) - 0.02) + 1
    Table2$MeanTCperYear[is.nan(Table2$MeanTCperYear)] <- 0
    ycoord <- c(min(Table2$MeanTCperYear), min(Table2$MeanTCperYear) + diff(range(Table2$MeanTCperYear)) * 0.15)

    g <- ggplot(Table2, aes(x = Year, y = MeanTCperYear)) +
      geom_line() +
      geom_area(fill = "grey90", alpha = .5) +
      labs(
        x = "Year",
        y = "Citations",
        title = "Average Article Citations per Year"
      ) +
      scale_x_continuous(breaks = (Table2$Year[seq(1, length(Table2$Year), by = 2)])) +
      theme(
        text = element_text(color = "#444444"),
        panel.background = element_rect(fill = "#FFFFFF"),
        panel.grid.minor = element_line(color = "#EFEFEF"),
        panel.grid.major = element_line(color = "#EFEFEF"),
        plot.title = element_text(size = 18),
        axis.title = element_text(size = 14, color = "#555555"),
        axis.title.y = element_text(vjust = 1, angle = 90),
        axis.title.x = element_text(hjust = 0),
        axis.text.x = element_text(size = 10, angle = 90),
        axis.line.x = element_line(color = "black", linewidth = 0.5),
        axis.line.y = element_line(color = "black", linewidth = 0.5)
      ) +
      annotation_custom(logo, xmin = xcoord[1], xmax = xcoord[2], ymin = ycoord[1], ymax = ycoord[2])

    plot(g)
    graphs$AverArtCitperYear <- g

    if (pause == TRUE) {
      cat("Hit <Return> to see next plot: ")
      line <- readline()
    }

    xcoord <- c(max(Table2$Year) - 0.02 - diff(range(Table2$Year)) * 0.15, max(Table2$Year) - 0.02) + 1
    ycoord <- c(min(Table2$MeanTCperArt), min(Table2$MeanTCperArt) + diff(range(Table2$MeanTCperArt)) * 0.15)

    g <- ggplot(Table2, aes(x = Year, y = MeanTCperArt)) +
      geom_line() +
      geom_area(fill = "grey90", alpha = .5) +
      labs(
        x = "Year",
        y = "Citations",
        title = "Average Total Citations per Year"
      ) +
      scale_x_continuous(breaks = (Table2$Year[seq(1, length(Table2$Year), by = 2)])) +
      theme(
        text = element_text(color = "#444444"),
        panel.background = element_rect(fill = "#FFFFFF"),
        panel.grid.minor = element_line(color = "#EFEFEF"),
        panel.grid.major = element_line(color = "#EFEFEF"),
        plot.title = element_text(size = 18),
        axis.title = element_text(size = 14, color = "#555555"),
        axis.title.y = element_text(vjust = 1, angle = 90),
        axis.title.x = element_text(hjust = 0),
        axis.text.x = element_text(size = 10, angle = 90),
        axis.line.x = element_line(color = "black", linewidth = 0.5),
        axis.line.y = element_line(color = "black", linewidth = 0.5)
      ) +
      annotation_custom(logo, xmin = xcoord[1], xmax = xcoord[2], ymin = ycoord[1], ymax = ycoord[2])
    plot(g)
    graphs$AverTotCitperYear <- g
  } else {
    graphs$AverArtCitperYear <- NA
    graphs$AverTotCitperYear <- NA
  }
  invisible(graphs)
}

utils::globalVariables(c(
  "sC",
  "groups",
  "words",
  "words1",
  "eij",
  "groups.x",
  "groups.y",
  "color",
  "freq",
  "cluster_label",
  "words2",
  "groups2",
  "ext",
  "n",
  "centrality",
  "density",
  "rcentrality",
  "rdensity",
  "name_full",
  "Cluster",
  "Color",
  "Occurrences",
  "pagerank_centrality",
  "terms",
  "p_c",
  "p_w",
  "Cluster_Label",
  "SR",
  "pagerank",
  "weigth",
  "p",
  "Assigned_cluster",
  "TC",
  "PY",
  "DI",
  "AU",
  "TI",
  "SO",
  "TCpY",
  "NTC",
  ".",
  "wordlist",
  "vertexID",
  "score",
  "PageRank",
  "rel_freq",
  "vertexPageRank",
  "old_combined"
))
#' Create a thematic map
#'
#' It creates a thematic map based on co-word network analysis and clustering.
#' The methodology is inspired by the proposal of Cobo et al. (2011).
#'
#' \code{thematicMap} starts from a co-occurrence keyword network to plot in a
#' two-dimensional map the typological themes of a domain.\cr\cr
#'
#' Reference:\cr
#' Cobo, M. J., Lopez-Herrera, A. G., Herrera-Viedma, E., & Herrera, F. (2011). An approach for detecting, quantifying,
#' and visualizing the evolution of a research field: A practical application to the fuzzy sets theory field. Journal of Informetrics, 5(1), 146-166.\cr
#'
#'
#' @param M is a bibliographic dataframe.
#' @param field is the textual attribute used to build up the thematic map. It can be \code{field = c("ID","DE","KW_Merged","TI", "AB")}.
#' \code{\link{biblioNetwork}} or \code{\link{cocMatrix}}.
#' @param n is an integer. It indicates the number of terms to include in the analysis.
#' @param minfreq is a integer. It indicates the minimum frequency (per thousand) of a cluster. It is a number in the range (0,1000).
#' @param ngrams is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
#' An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.
#' @param stemming is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).
#' @param size is numerical. It indicates del size of the cluster circles and is a number in the range (0.01,1).
#' @param n.labels is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.
#' @param community.repulsion is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.5}.
#' @param repel is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @param cluster is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").
#' @param subgraphs is a logical. If TRUE cluster subgraphs are returned.
#' @param alpha is a real. it indicates the balancing between word frequency and pagerank centrality to identify cluster labels. Default is \code{alpha = 0.5}.
#' @param seed is an integer. It indicates the seed for random number generation. Default is \code{seed = 1234}.
#' @return a list containing:
#' \tabular{lll}{
#' \code{map}\tab   \tab The thematic map as ggplot2 object\cr
#' \code{clusters}\tab   \tab Centrality and Density values for each cluster. \cr
#' \code{words}\tab   \tab A list of words following in each cluster\cr
#' \code{nclust}\tab   \tab The number of clusters\cr
#' \code{net}\tab    \tab A list containing the network output (as provided from the networkPlot function)}
#'
#'
#' @examples
#'
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' res <- thematicMap(management, field = "ID", n = 250, minfreq = 5, size = 0.5, repel = TRUE)
#' plot(res$map)
#' plot(res$net$graph)
#' }
#'
#' @seealso \code{\link{biblioNetwork}} function to compute a bibliographic network.
#' @seealso \code{\link{cocMatrix}} to compute a bibliographic bipartite network.
#' @seealso \code{\link{networkPlot}} to plot a bibliographic network.
#'
#' @export

thematicMap <- function(
  M,
  field = "ID",
  n = 250,
  minfreq = 5,
  ngrams = 1,
  stemming = FALSE,
  size = 0.5,
  n.labels = 1,
  community.repulsion = 0.5,
  repel = TRUE,
  remove.terms = NULL,
  synonyms = NULL,
  cluster = "louvain",
  subgraphs = FALSE,
  alpha = 0.5,
  seed = 1234
) {
  minfreq <- max(2, floor(minfreq * nrow(M) / 1000))

  switch(
    field,
    ID = {
      NetMatrix <- biblioNetwork(
        M,
        analysis = "co-occurrences",
        network = "keywords",
        n = n,
        sep = ";",
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      TERMS <- tolower(M$ID)
    },
    DE = {
      NetMatrix <- biblioNetwork(
        M,
        analysis = "co-occurrences",
        network = "author_keywords",
        n = n,
        sep = ";",
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      TERMS <- tolower(M$DE)
    },
    KW_Merged = {
      NetMatrix <- biblioNetwork(
        M,
        analysis = "co-occurrences",
        network = "all_keywords",
        n = n,
        sep = ";",
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      TERMS <- tolower(M$KW_Merged)
    },
    TI = {
      M = termExtraction(
        M,
        Field = "TI",
        ngrams = ngrams,
        verbose = FALSE,
        stemming = stemming,
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      NetMatrix <- biblioNetwork(
        M,
        analysis = "co-occurrences",
        network = "titles",
        n = n,
        sep = ";"
      )
    },
    AB = {
      M = termExtraction(
        M,
        Field = "AB",
        ngrams = ngrams,
        verbose = FALSE,
        stemming = stemming,
        remove.terms = remove.terms,
        synonyms = synonyms
      )
      NetMatrix <- biblioNetwork(
        M,
        analysis = "co-occurrences",
        network = "abstracts",
        n = n,
        sep = ";"
      )
    }
  )

  if (nrow(NetMatrix) > 0) {
    Net <- networkPlot(
      NetMatrix,
      normalize = "association",
      Title = "Keyword co-occurrences",
      type = "auto",
      labelsize = 2,
      halo = F,
      cluster = cluster,
      remove.isolates = TRUE,
      community.repulsion = community.repulsion,
      remove.multiple = FALSE,
      noloops = TRUE,
      weighted = TRUE,
      label.cex = T,
      edgesize = 5,
      size = 1,
      edges.min = 1,
      label.n = n,
      seed = seed,
      verbose = FALSE
    )
  } else {
    cat("\n\nNetwork matrix is empty!\nThe analysis cannot be performed\n\n")
    return()
  }
  #dev.off();file.remove(t) ### end of trick
  S = Net$S

  row.names(NetMatrix) = colnames(NetMatrix) = tolower(row.names(NetMatrix))
  net = Net$graph
  net_groups <- Net$cluster_obj
  group = net_groups$membership
  word = net_groups$name
  color = V(net)$color
  color[is.na(color)] = "#D3D3D3"

  ###
  W = intersect(row.names(NetMatrix), word)
  index = which(row.names(NetMatrix) %in% W)
  ii = which(word %in% W)
  word = word[ii]
  group = group[ii]
  color = color[ii]
  ###

  C = diag(NetMatrix)

  sEij = S[index, index]
  #dim(sEij)
  sC = (C[index])

  ### centrality and density
  label_cluster = unique(group)
  word_cluster = word[group]

  centr <- networkStat(Net$graph, stat = "all", type = "pagerank")$vertex %>%
    rename(pagerank = vertexPageRank)
  df_lab <- data.frame(
    sC = sC,
    words = word,
    groups = group,
    color = color,
    cluster_label = "NA"
  ) %>%
    left_join(
      centr %>% select(vertexID, pagerank),
      by = c("words" = "vertexID")
    ) %>%
    group_by(groups) %>%
    mutate(rel_freq = sC / sum(sC)) %>%
    ungroup() %>%
    mutate(score = (alpha * rel_freq + (1 - alpha) * pagerank) * 100)

  ## new code using tidyvverse
  df_lab <- df_lab %>%
    dplyr::filter(sC >= minfreq) %>%
    group_by(groups) %>%
    mutate(freq = sum(sC), cluster_label = words[which.max(score)])

  sEij <- triu(sEij)
  df_lab_top <- df_lab %>% select(words, groups)

  sEij_df <- as.matrix(sEij) %>%
    data.frame() %>%
    rename_with(~ row.names(sEij), .cols = colnames(.)) %>%
    mutate(words1 = row.names(sEij)) %>%
    pivot_longer(cols = !words1, names_to = "words2", values_to = "eij") %>%
    dplyr::filter(eij > 0) %>%
    left_join(df_lab_top, by = c("words1" = "words")) %>%
    left_join(df_lab_top, by = c("words2" = "words")) %>%
    rename(groups = groups.x, groups2 = groups.y)

  df_lab_top <- df_lab %>%
    select(groups, cluster_label, color, freq) %>%
    slice_head(n = 1)

  df <- sEij_df %>%
    dplyr::filter(
      words1 %in% unique(df_lab$words) & words2 %in% unique(df_lab$words)
    ) %>%
    group_by(groups) %>%
    mutate(ext = as.numeric(groups != groups2)) %>%
    summarize(
      n = length(unique(words1)),
      centrality = sum(eij * ext),
      density = sum((eij * (1 - ext)) / n) * 100
    ) %>%
    mutate(rcentrality = rank(centrality), rdensity = rank(density)) %>%
    left_join(., df_lab_top, by = "groups") %>%
    rename(label = cluster_label)

  meandens = mean(df$rdensity)
  meancentr = mean(df$rcentrality)
  rangex = max(c(
    meancentr - min(df$rcentrality),
    max(df$rcentrality) - meancentr
  ))
  rangey = max(c(meandens - min(df$rdensity), max(df$rdensity) - meandens))

  df <- df_lab %>%
    group_by(groups) %>% #dplyr::filter(sC>1) %>%
    arrange(desc(score), .by_group = TRUE) %>%
    dplyr::slice_max(n = 10, score, with_ties = FALSE) %>%
    summarise(
      wordlist = paste(words, round(score, 1), collapse = "\n"),
      name_full = paste(words[1:min(n.labels, n())], collapse = "\n")
    ) %>%
    right_join(., df, by = "groups") %>%
    rename(name = label, words = wordlist)

  xlimits = c(meancentr - (rangex * 1.2), meancentr + rangex * 1.2)
  ylimits = c(meandens - (rangey * 1.2), meandens + rangey * 1.2)

  annotations <- data.frame(
    xpos = sort(c(xlimits, xlimits)),
    ypos = c(ylimits, ylimits),
    words = c(
      "Emerging or\nDeclining Themes",
      "Niche Themes",
      "Basic Themes ",
      "Motor Themes "
    ),
    hjustvar = c(0, 0, 1, 1),
    vjustvar = c(0, 1.0, 0, 1)
  )

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  x <- c(
    max(df$rcentrality) - 0.02 - diff(range(df$rcentrality)) * 0.125,
    max(df$rcentrality) - 0.02
  ) +
    0.6
  y <- c(min(df$rdensity), min(df$rdensity) + diff(range(df$rdensity)) * 0.125)

  if (!isTRUE(repel)) {
    df <- adjust_positions_oblique(
      df,
      xvar = "rcentrality",
      yvar = "rdensity",
      min_dist = 1
    )
  }

  g <- buildTMPlot(
    df,
    meandens,
    meancentr,
    xlimits,
    ylimits,
    annotations,
    size,
    repel
  ) +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])

  names(df_lab) = c(
    "Occurrences",
    "Words",
    "Cluster",
    "Color",
    "Cluster_Label",
    "Cluster_Frequency",
    "PageRank",
    "Score",
    "Freq"
  )

  df_lab <- df_lab %>%
    arrange(Cluster) %>%
    dplyr::filter(!is.na(Color)) %>%
    data.frame() %>%
    mutate(Cluster = as.numeric(factor(Cluster)))

  ## Add centrality measure to words
  cluster_res <- Net$cluster_res %>%
    dplyr::select(!cluster)

  df_lab <- df_lab %>%
    dplyr::left_join(cluster_res, by = c("Words" = "vertex"))

  documentToClusters <- clusterAssignment(
    M,
    words = df_lab,
    field,
    remove.terms,
    synonyms,
    threshold = 0.5
  )

  params <- list(
    field = field,
    n = n,
    minfreq = minfreq,
    ngrams = ngrams,
    stemming = stemming,
    size = size,
    n.labels = n.labels,
    community.repulsion = community.repulsion,
    repel = repel,
    remove.terms = remove.terms,
    synonyms = synonyms,
    cluster = cluster
  )
  params <- data.frame(
    params = names(unlist(params)),
    values = unlist(params),
    row.names = NULL
  )

  ## cluster subgraphs
  if (isTRUE(subgraphs)) {
    gcl <- list()
    color <- unique(df$color)

    for (i in color) {
      ind <- which(V(Net$graph)$color == i)
      gcl[[i]] <- induced_subgraph(
        graph = Net$graph,
        vids = ind,
        impl = "create_from_scratch"
      )
    }
  } else {
    gcl <- NA
  }

  results = list(
    map = g,
    axisOrigin = c(meancentr, meandens),
    axisLimits = c(xlimits, ylimits),
    annotations = annotations,
    clusters = df,
    words = df_lab %>%
      rename(
        Cluster_PageRank = PageRank
      ),
    nclust = dim(df)[1],
    net = Net,
    modularity = Net$modularity,
    subgraphs = gcl,
    documentToClusters = documentToClusters,
    params = params
  )
  return(results)
}

# Probability calculation for cluster assignment ----
clusterAssignment <- function(
  M,
  words,
  field,
  remove.terms,
  synonyms,
  threshold
) {
  # 1. Normalizzazione campo di input
  if (field %in% c("AB", "TI")) {
    field <- paste0(field, "_TM")
  }

  # 2. Tokenizzazione
  allField <- M %>%
    select(SR, terms = !!sym(field)) %>%
    tidyr::separate_longer_delim(terms, delim = ";") %>%
    mutate(terms = trimws(terms)) %>%
    dplyr::filter(terms != "" & !is.na(terms))

  # 3. Pulizia termini
  if (!is.null(remove.terms)) {
    remove_vec <- toupper(trimws(remove.terms))
    allField <- allField %>%
      dplyr::filter(!(toupper(terms) %in% remove_vec))
  }

  # 4. Gestione Sinonimi
  if (!is.null(synonyms)) {
    syn_df <- tibble(raw = synonyms) %>%
      separate(
        raw,
        into = c("new", "old_combined"),
        sep = ";",
        extra = "merge"
      ) %>%
      tidyr::separate_longer_delim(old_combined, delim = ";") %>%
      mutate(across(everything(), ~ trimws(toupper(.x)))) %>%
      rename(old = old_combined)

    allField <- allField %>%
      left_join(syn_df, by = c("terms" = "old")) %>%
      mutate(terms = coalesce(new, terms)) %>%
      select(SR, terms)
  }

  # 5. Preparazione tabella 'words'
  words_clean <- words %>%
    mutate(
      terms = tolower(Words),
      p_w = pagerank_centrality / Occurrences #1 / Occurrences
    ) %>%
    select(terms, Cluster_Label, p_w, p_c = pagerank_centrality)

  # 6. Calcolo Probabilità
  TERMS_processed <- allField %>%
    mutate(terms = tolower(terms)) %>%
    inner_join(words_clean, by = "terms") %>%
    group_by(SR) %>%
    mutate(total_pagerank = sum(p_c, na.rm = TRUE)) %>%
    group_by(SR, Cluster_Label) %>%
    summarise(
      weight = sum(p_w),
      pagerank = first(total_pagerank),
      .groups = "drop_last"
    ) %>%
    group_by(SR) %>%
    mutate(p = weight / sum(weight, na.rm = TRUE)) %>%
    ungroup()

  # 7. Assegnazione Cluster
  TERMS_Max <- TERMS_processed %>%
    dplyr::filter(p >= threshold) %>%
    group_by(SR) %>%
    slice_max(p, n = 1, with_ties = FALSE) %>%
    select(
      SR,
      Assigned_cluster = Cluster_Label,
      pagerank_assigned = pagerank
    ) %>%
    ungroup()

  # 8. Preparazione M
  if (!"DI" %in% names(M)) {
    M$DI <- NA_character_
  }

  current_year <- as.numeric(format(Sys.Date(), "%Y")) + 1

  # 9. Join Finale e Gestione NA Tipizzata (Risolve l'errore)
  TERMS_wide <- TERMS_processed %>%
    select(SR, Cluster_Label, p) %>%
    pivot_wider(
      names_from = Cluster_Label,
      values_from = p,
      values_fill = 0
    )
  cluster_cols <- setdiff(names(TERMS_wide), "SR")
  if (length(cluster_cols) > 0) {
    k <- suppressWarnings(as.numeric(gsub("\\D+", "", cluster_cols)))
    cluster_cols <- cluster_cols[order(is.na(k), k, cluster_cols)]
    TERMS_wide <- TERMS_wide[, c("SR", cluster_cols)]
  }

  TERMS_final <- M %>%
    select(DI, AU, TI, SO, PY, TC, SR) %>%
    mutate(
      TC = as.numeric(TC),
      PY = as.numeric(PY),
      TCpY = TC / (current_year - PY),
      NTC = if_else(TC == 0, 0, TC / mean(TC, na.rm = TRUE))
    ) %>%
    left_join(
      TERMS_wide %>%
        left_join(TERMS_Max, by = "SR"),
      by = "SR"
    ) %>%
    # Sostituzione NA sicura per tipo di dato
    mutate(
      across(where(is.numeric), ~ replace_na(.x, 0)),
      across(where(is.character), ~ replace_na(.x, "n.a."))
    ) %>%
    group_by(Assigned_cluster) %>%
    arrange(desc(TC), desc(PY), desc(SR), .by_group = TRUE) %>%
    ungroup() %>%
    rename(pagerank = pagerank_assigned)

  return(TERMS_final)
}

# Build TM Plot -----

#' Build Thematic Map Plot
#'
#' Internal function to generate a strategic diagram (thematic map) visualization
#' plotting themes according to their centrality and density dimensions.
#'
#' @param df A data.frame containing the thematic structure with columns:
#'   \itemize{
#'     \item \code{rcentrality}: Relevance degree (normalized centrality values)
#'     \item \code{rdensity}: Development degree (normalized density values)
#'     \item \code{freq}: Frequency of each theme
#'     \item \code{words}: Theme labels for hover text
#'     \item \code{name_full}: Full theme names for labels
#'     \item \code{color}: Color assignments for each theme
#'   }
#' @param size Numeric. Label size multiplier. If 0, labels are not displayed.
#'   Values > 0 increase label size proportionally.
#' @param repel Logical. If \code{TRUE}, uses \code{ggrepel::geom_label_repel()}
#'   to prevent label overlapping. If \code{FALSE}, uses standard \code{geom_text()}.
#'
#' @return A ggplot object representing the thematic map with:
#'   \itemize{
#'     \item Themes plotted as points sized by log-frequency
#'     \item Quadrant divisions at mean centrality and density
#'     \item Optional theme labels (frequency > 1)
#'     \item Quadrant annotations
#'   }
#'
#' @details
#' The function creates a strategic diagram dividing the thematic space into four
#' quadrants based on mean centrality (x-axis) and mean density (y-axis):
#' \itemize{
#'   \item \strong{Upper-right}: Motor themes (high centrality, high density)
#'   \item \strong{Upper-left}: Niche themes (low centrality, high density)
#'   \item \strong{Lower-right}: Basic themes (high centrality, low density)
#'   \item \strong{Lower-left}: Emerging/declining themes (low centrality, low density)
#' }
#'
#' The function uses several variables from the parent environment:
#' \code{meandens}, \code{meancentr}, \code{xlimits}, \code{ylimits}, and
#' \code{annotations}.
#'
#' @keywords internal
#' @noRd
buildTMPlot <- function(
  df,
  meandens,
  meancentr,
  xlimits,
  ylimits,
  annotations,
  size,
  repel
) {
  g <- ggplot(df, aes(x = rcentrality, y = rdensity, text = c(words))) +
    geom_point(
      group = "NA",
      aes(size = log(as.numeric(freq))),
      shape = 20,
      col = adjustcolor(df$color, alpha.f = 0.5)
    ) # Use hollow circles
  if (size > 0) {
    if (isTRUE(repel)) {
      g <- g +
        geom_label_repel(
          aes(
            group = "NA",
            label = ifelse(freq > 1, unlist(tolower(name_full)), '')
          ),
          size = 3 * (1 + size),
          angle = 0
        )
    } else {
      g <- g +
        geom_text(
          aes(
            group = "NA",
            label = ifelse(freq > 1, unlist(tolower(name_full)), '')
          ),
          color = adjustcolor("black", alpha.f = 0.7),
          size = 3 * (1 + size),
          angle = 0
        )
    }
  }

  g <- g +
    geom_hline(
      yintercept = meandens,
      linetype = 2,
      color = adjustcolor("black", alpha.f = 0.7)
    ) +
    geom_vline(
      xintercept = meancentr,
      linetype = 2,
      color = adjustcolor("black", alpha.f = 0.7)
    ) +
    theme(legend.position = "none") +
    scale_radius(range = c(5 * (1 + size), 30 * (1 + size))) +
    labs(
      x = "Relevance degree\n(Centrality)",
      y = "Development degree\n(Density)"
    ) +
    xlim(xlimits) +
    ylim(ylimits) +
    annotate(
      "text",
      x = annotations$xpos,
      y = annotations$ypos,
      hjust = annotations$hjustvar,
      vjust = annotations$vjustvar,
      label = annotations$words,
      color = adjustcolor("gray20", alpha.f = 0.5),
      size = 3 * (1 + size)
    ) +
    theme(
      axis.text.x = element_blank(),
      panel.background = element_rect(fill = '#FFFFFF'),
      axis.line.x = element_line(color = "black", linewidth = 0.5),
      axis.line.y = element_line(color = "black", linewidth = 0.5),
      axis.ticks.x = element_blank(),
      axis.text.y = element_blank(),
      axis.ticks.y = element_blank()
    ) #+
  #annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])

  return(g)
}

utils::globalVariables(c("item", "freq", "year_q1", "year_med", "year_q3"))
#' Field Tag distribution by Year
#'
#' It calculates the median year for each item of a field tag.
#' @param M is a bibliographic data frame obtained by \code{\link{convert2df}} function.
#' @param field is a character object. It indicates one of the field tags of the
#'   standard ISI WoS Field Tag codify.
#' @param timespan is a vector with the min and max year. If it is = NULL, the analysis is performed on the entire period. Default is \code{timespan = NULL}.
#' @param min.freq is an integer. It indicates the min frequency of the items to include in the analysis
#' @param n.items is an integer. I indicates the maximum number of items per year to include in the plot.
#' @param labelsize is deprecated argument. It will be removed in the next update.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @param dynamic.plot is a logical. If TRUE plot aesthetics are optimized for plotly package.
#' @param graph is logical. If TRUE the function plots Filed Tag distribution by Year graph. Default is \code{graph = TRUE}.
#' @return The function \code{fieldByYear} returns a list containing threeobjects:
#' \tabular{lll}{
#' \code{df}  \tab   \tab is a data frame\cr
#' \code{df_graph}\tab    \tab is a data frame with data used to build the graph\cr
#' \code{graph}   \tab   \tab a ggplot object}
#'
#' @examples
#' data(management, package = "bibliometrixData")
#' timespan <- c(2005, 2015)
#' res <- fieldByYear(management,
#'   field = "ID", timespan = timespan,
#'   min.freq = 5, n.items = 5, graph = TRUE
#' )

#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{summary}} method for class '\code{bibliometrix}'
#'
#' @export
#'

fieldByYear <- function(M,
                        field = "ID",
                        timespan = NULL,
                        min.freq = 2,
                        n.items = 5,
                        labelsize = NULL,
                        remove.terms = NULL,
                        synonyms = NULL,
                        dynamic.plot = FALSE,
                        graph = TRUE) {
  A <- cocMatrix(M, Field = field, binary = FALSE, remove.terms = remove.terms, synonyms = synonyms)
  n <- colSums(as.array(A))

  # A=tdIdf(A)


  trend_med <- apply(A, 2, function(x) {
    round(quantile(rep(M$PY, x), c(0.25, 0.50, 0.75), na.rm = TRUE))
  })

  trend_med <- as_tibble(t(trend_med)) %>%
    rename("year_q1" = "25%", "year_med" = "50%", "year_q3" = "75%") %>%
    mutate(item = rownames(t(trend_med)), freq = n) %>%
    relocate(c(item, freq), year_q1)

  # if timespan is null, timespan is set to the whole period
  if (is.null(timespan) | length(timespan) != 2) {
    timespan <- as.numeric(range(trend_med$year_med, na.rm = TRUE))
  }

  df <- trend_med %>%
    mutate(item = tolower(item)) %>%
    group_by(year_med) %>%
    arrange(desc(freq), item) %>%
    arrange(desc(year_med)) %>%
    dplyr::slice_head(n = n.items) %>%
    dplyr::filter(freq >= min.freq) %>%
    dplyr::filter(between(year_med, timespan[1], timespan[2])) %>%
    mutate(item = fct_reorder(item, freq))

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  yrange <- range(unlist(df[, which(regexpr("year", names(df)) > -1)]))

  x <- c(0 + 0.5, 0.05 + length(levels(df$item)) * 0.125) + 1
  y <- c(yrange[2] - 0.02 - diff(yrange) * 0.125, yrange[2] - 0.02)

  g <- ggplot(df, aes(
    x = item, y = year_med,
    text = paste(
      "Term: ", item, "\nYear: ",
      year_med, "\nTerm frequency: ", freq
    )
  )) +
    geom_point(aes(size = freq), alpha = 0.6, color = "dodgerblue4") +
    scale_size(range = c(2, 6)) +
    # scale_alpha(range=c(0.3,1))+
    scale_y_continuous(breaks = seq(min(df$year_q1), max(df$year_q3), by = 2)) +
    guides(size = guide_legend(order = 1, "Term frequency"), alpha = guide_legend(order = 2, "Term frequency")) +
    theme(
      legend.position = "right"
      # ,aspect.ratio = 1
      , text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      panel.grid.major.x = element_blank(),
      panel.grid.major.y = element_line(color = "grey95"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90, face = "bold"),
      axis.title.x = element_text(hjust = .95),
      axis.text.x = element_text(face = "bold", angle = 90) # , size=labelsize)
      , axis.text.y = element_text(face = "bold", ),
      axis.line.x = element_line(color = "black", linewidth = 0.5)
    ) +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])

  if (!isTRUE(dynamic.plot)) {
    g <- g + geom_vline(xintercept = nrow(df) - (which(c(diff(df$year_med)) == -1) - 0.5), color = "grey70", alpha = 0.6, linetype = 6) +
      geom_point(aes(y = year_q1), alpha = 0.6, size = 3, color = "royalblue4", shape = "|") +
      geom_point(aes(y = year_q3), alpha = 0.6, size = 3, color = "royalblue4", shape = "|")
  }

  g <- g +
    labs(
      title = "Trend Topics",
      x = "Term",
      y = "Year"
    ) +
    geom_segment(data = df, aes(x = item, y = year_q1, xend = item, yend = year_q3), linewidth = 1.0, color = "royalblue4", alpha = 0.3) +
    coord_flip()

  if (isTRUE(graph)) {
    print(g)
  }



  results <- list(df = trend_med, df_graph = df, graph = g)

  return(results)
}

utils::globalVariables(c(
  "CL1",
  "CL2",
  "color_assigned",
  "color_final",
  "color_from",
  "color_lineage",
  "color_to",
  "freq",
  "measure",
  "total_in",
  "weight"
))

#' Plot a Thematic Evolution Analysis
#'
#' It plot a Thematic Evolution Analysis performed using the \code{\link{thematicEvolution}} function.
#'
#' @param Nodes is a list of nodes obtained by \code{\link{thematicEvolution}} function.
#' @param Edges is a list of edges obtained by \code{\link{thematicEvolution}} function.
#' @param measure is a character. It is deprecated and has no effect. The weight of the flows is always calculated using lineage_strength.
#' @param min.flow is numerical. It indicates the minimum value of measure to plot a flow.
#' @return a sankeyPlot
#'
#'
#' @examples
#'
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' years=c(2004,2008,2015)
#'
#' nexus <- thematicEvolution(management,field="DE",years=years,n=100,minFreq=2)
#'
#' plotThematicEvolution(nexus$Nodes,nexus$Edges)
#' }
#'
#' @seealso \code{\link{thematicMap}} function to create a thematic map based on co-word network analysis and clustering.
#' @seealso \code{\link{thematicEvolution}} function to perform a thematic evolution analysis.
#' @seealso \code{\link{networkPlot}} to plot a bibliographic network.
#'
#' @export
#'
plotThematicEvolution <- function(
  Nodes,
  Edges,
  measure = "lineage_strength",
  min.flow = 0
) {
  # Calculate relative frequency for nodes if not already present
  if (!"rel_freq" %in% names(Nodes)) {
    Nodes <- Nodes %>%
      group_by(group) %>%
      mutate(rel_freq = freq / sum(freq, na.rm = TRUE) * 100) %>%
      ungroup()
  } else {
    # Convert to percentage if in [0,1] range
    if (max(Nodes$rel_freq, na.rm = TRUE) <= 1) {
      Nodes <- Nodes %>%
        mutate(rel_freq = rel_freq * 100)
    }
  }

  # Calculate lineage_strength if not present in Edges
  if (!"lineage_strength" %in% names(Edges)) {
    Edges <- Edges %>%
      # left_join(node_rel_freq, by = c("from" = "id")) %>%
      mutate(lineage_strength = Inc_Weighted)
  }

  # Store original lineage_strength for tooltips
  Edges$lineage_strength_display <- Edges$lineage_strength

  # Flag flows below the threshold. They are NOT removed: plotly assigns the
  # sankey nodes to columns from the graph topology, so dropping edges would
  # collapse the period layering and a cluster could be drawn in the wrong
  # period (e.g. a 2016-2025 topic shown among the 2011-2015 ones). We keep
  # every edge so the topology stays intact and hide the weak flows visually
  # instead (zero width + transparent colour, see below).
  Edges$is_weak <- !is.na(Edges$lineage_strength) &
    Edges$lineage_strength < min.flow

  # Use lineage_strength as weight
  Edges <- Edges %>%
    mutate(weight = lineage_strength)

  # Calculate coordinates
  Kx <- length(unique(Nodes$group))
  group_coords <- setNames(
    seq(from = 0, to = 1, length.out = Kx),
    sort(unique(Nodes$group))
  )

  Nodes <- Nodes %>%
    mutate(
      coordX = group_coords[as.character(group)],
      coordY = 0.1
    )

  # Scale weights for visualization
  Edges$weight <- Edges$weight * 100

  # Weak flows are kept only to preserve the layout topology: give them a
  # negligible width so they take no visible space.
  Edges$weight[Edges$is_weak] <- 1e-6

  # NOW reassign node IDs for plotly
  Nodes$old_id <- Nodes$id
  Nodes$id <- (1:nrow(Nodes)) - 1

  # Create mapping old_id -> new_id
  id_mapping <- setNames(Nodes$id, Nodes$old_id)

  # Update edge references using the mapping
  Edges$from <- id_mapping[as.character(Edges$from)]
  Edges$to <- id_mapping[as.character(Edges$to)]

  # Remove any edges with NA
  Edges <- Edges[!is.na(Edges$from) & !is.na(Edges$to), ]

  # Default edge color if not present
  if (!"color" %in% names(Edges)) {
    Edges$color <- "#D3D3D380"
  }

  # Hide the weak flows: kept for topology only, rendered fully transparent.
  Edges$color[Edges$is_weak] <- "rgba(0,0,0,0)"

  node_labels <- Nodes$name

  # Clusters whose flows are all below the threshold: hide them from the plot.
  # They are kept in the trace (so the period layering/topology is preserved
  # and the other clusters stay in the right period) but rendered invisible -
  # transparent node and empty label.
  strong_ids <- unique(c(
    Edges$from[!Edges$is_weak],
    Edges$to[!Edges$is_weak]
  ))
  hidden_node <- !(Nodes$id %in% strong_ids)
  node_labels[hidden_node] <- ""
  Nodes$color[hidden_node] <- "rgba(0,0,0,0)"

  # Prepare tooltips
  node_customdata <- matrix(
    c(Nodes$freq, Nodes$rel_freq),
    nrow = nrow(Nodes),
    ncol = 2
  )

  node_hovertemplate <- paste0(
    "<b>%{label}</b><br>",
    "Frequency: %{customdata[0]:.0f}<br>",
    "Relative Frequency: %{customdata[1]:.2f}%<br>",
    "<extra></extra>"
  )

  # Prepare edge labels
  edge_labels <- character(nrow(Edges))
  edge_lineage <- numeric(nrow(Edges))

  for (i in 1:nrow(Edges)) {
    from_idx <- which(Nodes$id == Edges$from[i])[1]
    to_idx <- which(Nodes$id == Edges$to[i])[1]

    if (!is.na(from_idx) && !is.na(to_idx)) {
      edge_labels[i] <- paste0(
        Nodes$name[from_idx],
        " \u2192 ",
        Nodes$name[to_idx]
      )
      edge_lineage[i] <- Edges$lineage_strength_display[i]
    }
  }

  edge_customdata <- lapply(1:nrow(Edges), function(i) {
    list(edge_labels[i], edge_lineage[i])
  })

  # Plotly margins
  m <- list(l = 50, r = 50, b = 100, t = 100, pad = 4)

  # Create Sankey
  fig <- plotly::plot_ly(
    type = "sankey",
    arrangement = "snap",
    node = list(
      label = node_labels,
      x = Nodes$coordX,
      y = Nodes$coordY,
      color = Nodes$color,
      pad = 4,
      customdata = node_customdata,
      hovertemplate = node_hovertemplate
    ),
    link = list(
      source = Edges$from,
      target = Edges$to,
      value = Edges$weight,
      color = Edges$color,
      customdata = edge_customdata,
      hovertemplate = paste0(
        "<b>%{customdata[0]}</b><br>",
        "Lineage Strength: %{customdata[1]:.4f}<br>",
        "<extra></extra>"
      )
    )
  )

  fig <- fig %>%
    layout(margin = m) %>%
    plotly::add_annotations(
      x = unique(Nodes$coordX),
      y = 1.08,
      text = sort(unique(Nodes$group)),
      showarrow = FALSE,
      xanchor = "center",
      font = list(color = 'Dark', family = "TimesNewRoman", size = 18)
    ) %>%
    config(
      displaylogo = FALSE,
      modeBarButtonsToRemove = c(
        'toImage',
        'sendDataToCloud',
        'pan2d',
        'select2d',
        'lasso2d',
        'toggleSpikelines',
        'hoverClosestCartesian',
        'hoverCompareCartesian'
      )
    )

  return(fig)
}

#' DEPRECATED: Load a sequence of ISI or SCOPUS Export files into a large character object
#'
#' The function readFiled is deprecated. You can import and convert your export files directly using the function \code{\link{convert2df}}.
#'
#' @param ... is a sequence of names of files downloaded from WOS.(in plain text or bibtex format) or SCOPUS Export file (exclusively in bibtex format).
#' @return a character vector of length the number of lines read.
#'
#' @examples
#' # WoS or SCOPUS Export files can be read using \code{\link{readFiles}} function:
#'
#' # largechar <- readFiles('filename1.txt','filename2.txt','filename3.txt')
#'
#' # filename1.txt, filename2.txt and filename3.txt are ISI or SCOPUS Export file
#' # in plain text or bibtex format.
#'
#' # D <- readFiles('https://www.bibliometrix.org/datasets/bibliometrics_articles.txt')
#'
#' @seealso \code{\link{convert2df}} for converting SCOPUS of ISI Export file into a dataframe
#'
#' @export

readFiles <- function(...) {
  cat("\nFrom version 3.0.0, the function readFiles has been dropped.\nPlease use the function 'convert2df' to import and convert your export files")
  # arguments <- unlist(list(...))
  # k=length(arguments)
  # D=list()
  # enc="UTF-8"
  # origEnc=getOption("encoding")
  # if (origEnc=="UTF-8"){options(encoding = "native.enc")}
  #   for (i in 1:k){
  #     D[[i]]=suppressWarnings(
  #       iconv(readLines(arguments[i],encoding = "UTF-8"),"latin1", "ASCII", sub="")
  #       #conv(readLines(arguments[[i]]))
  #       )
  #     }
  # D=unlist(D)
  # options(encoding = origEnc)
  # Encoding(D) <- "UTF-8"
  # return(D)
  return(NULL)
}

utils::globalVariables(c(
  "CR_canonical",
  "CR_id",
  "CR_normalized",
  "CR_original",
  "blocking_key",
  "cluster_id",
  "completeness_score",
  "first_author",
  "journal",
  "merge_key",
  "n_cluster",
  "pages",
  "temp_cluster",
  "volume",
  "doi",
  "n_exact",
  "ABBREVIATION",
  "ABBR_CLEAN",
  "LANGUAGES",
  "NO_ABBR",
  "WORD",
  "WORD_CLEAN",
  "cur_group_id",
  "journal_iso4",
  "journal_original",
  "len",
  "page_start",
  "str_starts",
  "CR_stripped",
  "n_stripped",
  "temp_citation",
  "variant_freq",
  "wos_key",
  "author_surname"
))

#' Normalize journal names to ISO4 abbreviated form
#'
#' Converts all journal names to their ISO4 abbreviated form using LTWA.
#' Only uses English language entries from LTWA to avoid foreign word matches.
#'
#' @param journal_name Character string with journal name
#' @param ltwa_db Data frame with LTWA database
#'
#' @return Normalized journal name in ISO4 abbreviated form
#' @keywords internal

normalize_journal_to_iso4 <- function(journal_name, ltwa_db) {
  if (is.na(journal_name) || journal_name == "" || is.null(journal_name)) {
    return(NA_character_)
  }

  # Clean and prepare
  journal_upper <- journal_name %>%
    toupper() %>%
    str_replace_all("&", "AND") %>%
    str_trim()

  # Split into words
  words_with_punct <- str_split(journal_upper, "\\s+")[[1]]
  words <- str_remove_all(words_with_punct, "[[:punct:]]")
  words <- words[nchar(words) > 0]

  if (length(words) == 0) {
    return(NA_character_)
  }

  # Prepare LTWA lookup - ONLY English entries
  ltwa_english <- ltwa_db %>%
    dplyr::filter(
      !is.na(WORD),
      !is.na(ABBREVIATION),
      ABBREVIATION != "",
      ABBREVIATION != "n.a.",
      # Filter for English language
      str_detect(LANGUAGES, regex("english", ignore_case = TRUE)) |
        LANGUAGES == "Multiple Languages"
    ) %>%
    mutate(
      WORD_CLEAN = toupper(str_remove_all(WORD, "[[:punct:]]")) %>%
        str_squish(),
      ABBR_CLEAN = toupper(str_remove_all(ABBREVIATION, "[[:punct:]]")) %>%
        str_squish()
    ) %>%
    dplyr::filter(WORD_CLEAN != "", ABBR_CLEAN != "") %>%
    select(WORD_CLEAN, ABBR_CLEAN) %>%
    distinct()

  # Create lookup: word -> abbreviation
  word_to_abbr <- setNames(ltwa_english$ABBR_CLEAN, ltwa_english$WORD_CLEAN)

  # Also create reverse for words that are already abbreviated
  abbr_set <- unique(ltwa_english$ABBR_CLEAN)

  # Process each word
  abbreviated_words <- sapply(
    words,
    function(word) {
      # Skip very short words (articles, prepositions, conjunctions)
      if (
        word %in%
          c(
            "OF",
            "THE",
            "A",
            "AN",
            "AND",
            "OR",
            "FOR",
            "IN",
            "ON",
            "AT",
            "TO",
            "BY"
          )
      ) {
        return(word)
      }

      # Check if word is already an abbreviation in LTWA
      if (word %in% abbr_set) {
        return(word)
      }

      # Try to abbreviate using LTWA
      if (word %in% names(word_to_abbr)) {
        return(word_to_abbr[[word]])
      }

      # Word not found in LTWA - check if it's a common single-letter abbreviation
      if (nchar(word) == 1) {
        return(word)
      }

      # For words not in LTWA, keep original
      # (might be proper nouns, acronyms, etc.)
      return(word)
    },
    USE.NAMES = FALSE
  )

  # Reconstruct journal in ISO4 form
  result <- paste(abbreviated_words, collapse = " ")

  return(result)
}


#' Prepare LTWA database for efficient lookup
#'
#' Pre-processes LTWA database into optimized lookup tables
#'
#' @param ltwa_db LTWA database data frame
#' @return List with singles, prefix, and phrase lookup tables
#' @keywords internal

prepare_ltwa_lookup <- function(ltwa_db) {
  # Filter for English language entries only
  ltwa_english <- ltwa_db %>%
    dplyr::filter(
      !is.na(WORD),
      !is.na(ABBREVIATION),
      str_detect(LANGUAGES, regex("english", ignore_case = TRUE)) |
        LANGUAGES == "Multiple Languages"
    ) %>%
    mutate(
      WORD_CLEAN = toupper(str_remove_all(WORD, "[[:punct:]]")) %>%
        str_squish(),
      ABBR_CLEAN = toupper(str_remove_all(ABBREVIATION, "[[:punct:]]")) %>%
        str_squish(),
      # Mark words that should not be abbreviated
      NO_ABBR = ABBREVIATION == "" | ABBREVIATION == "n.a."
    ) %>%
    dplyr::filter(WORD_CLEAN != "")

  # Separate into single words and multi-word phrases
  ltwa_singles <- ltwa_english %>%
    dplyr::filter(!str_detect(WORD_CLEAN, "\\s")) %>%
    select(WORD = WORD_CLEAN, ABBREVIATION = ABBR_CLEAN, NO_ABBR)

  ltwa_phrases <- ltwa_english %>%
    dplyr::filter(str_detect(WORD_CLEAN, "\\s")) %>%
    select(WORD = WORD_CLEAN, ABBREVIATION = ABBR_CLEAN, NO_ABBR)

  # Create prefix lookup (for partial matching)
  ltwa_prefix <- ltwa_singles %>%
    dplyr::filter(!NO_ABBR, nchar(WORD) >= 4) %>%
    select(WORD, ABBREVIATION)

  list(
    singles = ltwa_singles,
    phrases = ltwa_phrases,
    prefix = ltwa_prefix
  )
}


#' Articles, prepositions, and conjunctions to be removed (ISO 4 standard)
#'
#' @keywords internal

get_iso4_stop_words <- function() {
  list(
    # Articles (removed in most positions)
    articles = c("A", "AN", "THE"),

    # Prepositions (removed when not at start)
    prepositions = c(
      "OF",
      "IN",
      "ON",
      "AT",
      "TO",
      "FOR",
      "WITH",
      "FROM",
      "BY",
      "ABOUT",
      "AS",
      "INTO",
      "LIKE",
      "THROUGH",
      "AFTER",
      "OVER",
      "BETWEEN",
      "OUT",
      "AGAINST",
      "DURING",
      "WITHOUT",
      "BEFORE",
      "UNDER",
      "AROUND",
      "AMONG"
    ),

    # Conjunctions (removed)
    conjunctions = c("AND", "OR", "BUT", "NOR", "YET", "SO"),

    # Common scientific terms that should be abbreviated
    common_abbr = c(
      "JOURNAL" = "J",
      "JOURNALS" = "J",
      "RESEARCH" = "RES",
      "SCIENCE" = "SCI",
      "SCIENCES" = "SCI",
      "SCIENTIFIC" = "SCI",
      "TECHNOLOGY" = "TECHNOL",
      "TECHNOLOGICAL" = "TECHNOL",
      "INTERNATIONAL" = "INT",
      "NATIONAL" = "NATL",
      "AMERICAN" = "AM",
      "EUROPEAN" = "EUR",
      "MANAGEMENT" = "MANAG",
      "ACADEMY" = "ACAD",
      "SOCIETY" = "SOC",
      "ASSOCIATION" = "ASSOC",
      "ORGANIZATION" = "ORGAN",
      "ENVIRONMENTAL" = "ENV",
      "ENGINEERING" = "ENG",
      "APPLIED" = "APPL",
      "THEORETICAL" = "THEOR",
      "EXPERIMENTAL" = "EXP",
      "CLINICAL" = "CLIN",
      "MEDICAL" = "MED",
      "BIOLOGICAL" = "BIOL",
      "CHEMICAL" = "CHEM",
      "PHYSICAL" = "PHYS",
      "MATHEMATICAL" = "MATH",
      "MATHEMATICS" = "MATH",
      "ECONOMICS" = "ECON",
      "ECONOMIC" = "ECON",
      "POLICY" = "POLICY",
      "POLICIES" = "POLICIES",
      "BUSINESS" = "BUS",
      "STRATEGIC" = "STRATEG",
      "STRATEGY" = "STRATEG",
      "QUARTERLY" = "Q",
      "ANNUAL" = "ANNU",
      "ANNALS" = "ANN",
      "REVIEW" = "REV",
      "REVIEWS" = "REV",
      "STUDIES" = "STUD",
      "ANALYSIS" = "ANAL",
      "DEVELOPMENT" = "DEV",
      "PRODUCTION" = "PROD",
      "MANUFACTURING" = "MANUF",
      "OPERATIONS" = "OPER",
      "OPERATIONAL" = "OPER",
      "SYSTEMS" = "SYST",
      "SYSTEM" = "SYST",
      "INFORMATION" = "INF",
      "COMPUTING" = "COMPUT",
      "COMPUTER" = "COMPUT",
      "COMMUNICATIONS" = "COMMUN",
      "COMMUNICATION" = "COMMUN",
      "MATERIALS" = "MATER",
      "MATERIAL" = "MATER",
      "LETTERS" = "LETT",
      "PROCEEDINGS" = "PROC",
      "TRANSACTIONS" = "TRANS",
      "BULLETIN" = "BULL",
      "ANNALS" = "ANN",
      "ARCHIVES" = "ARCH",
      "REPORT" = "REP",
      "REPORTS" = "REPORTS",
      "ADVANCES" = "ADV",
      "INNOVATION" = "INNOV",
      "INNOVATIONS" = "INNOV",
      "SUSTAINABILITY" = "SUSTAIN",
      "SUSTAINABLE" = "SUSTAIN",
      "CLEANER" = "CLEAN",
      "PLANNING" = "PLAN"
    )
  )
}


#' Abbreviate a single term using LTWA
#'
#' @param word Single word to abbreviate
#' @param ltwa_lookup Pre-processed LTWA lookup tables
#' @param common_abbr Named vector of common abbreviations
#' @param check Logical, whether to check for abbreviation
#'
#' @return Abbreviated form of word
#' @keywords internal

abbreviate_term <- function(word, ltwa_lookup, common_abbr, check = TRUE) {
  if (!check || is.na(word) || word == "") {
    return(word)
  }

  word_upper <- toupper(word)

  # Strategy 1: Check common scientific terms
  if (word_upper %in% names(common_abbr)) {
    return(common_abbr[[word_upper]])
  }

  # Strategy 2: Check for exact whole-word match in LTWA
  exact_match <- ltwa_lookup$singles %>%
    dplyr::filter(WORD == word_upper)

  if (nrow(exact_match) > 0) {
    if (exact_match$NO_ABBR[1]) {
      return(word_upper)
    } else {
      return(exact_match$ABBREVIATION[1])
    }
  }

  # Strategy 3: Check for prefix match (for partial words)
  if (nchar(word_upper) >= 4) {
    prefix_matches <- ltwa_lookup$prefix %>%
      dplyr::filter(str_starts(word_upper, fixed(WORD)))

    if (nrow(prefix_matches) > 0) {
      # Choose longest matching prefix
      best_match <- prefix_matches %>%
        mutate(len = nchar(WORD)) %>%
        arrange(desc(len)) %>%
        slice(1)

      return(best_match$ABBREVIATION)
    }
  }

  # Strategy 4: No match found - return original
  return(word_upper)
}


#' Abbreviate journal title to ISO 4 standard
#'
#' Converts a full journal title to its ISO 4 abbreviated form using LTWA.
#' Removes articles, prepositions, and conjunctions according to ISO 4 rules.
#' Returns result WITHOUT periods (dots).
#'
#' @param title Journal title string
#' @param ltwa_lookup Pre-processed LTWA lookup tables (from prepare_ltwa_lookup)
#'
#' @return Abbreviated journal title in ISO 4 format (without periods)
#' @keywords internal

abbreviate_journal_title <- function(title, ltwa_lookup) {
  if (is.na(title) || title == "") {
    return(NA_character_)
  }

  # Clean title - REMOVE ALL PUNCTUATION including periods
  title_clean <- title %>%
    toupper() %>%
    str_replace_all("&", "AND") %>%
    # Remove parenthetical content
    str_replace_all("\\s*\\([^)]+\\)", "") %>%
    # Remove ALL punctuation (periods, commas, etc.)
    str_remove_all("[[:punct:]]") %>%
    # Normalize whitespace
    str_squish()

  # Get stop words
  stop_words <- get_iso4_stop_words()

  # Split into words
  words <- str_split(title_clean, "\\s+")[[1]]
  words <- words[nchar(words) > 0]

  if (length(words) == 0) {
    return(NA_character_)
  }

  # Single word titles - just abbreviate and return
  if (length(words) == 1) {
    return(abbreviate_term(words[1], ltwa_lookup, stop_words$common_abbr))
  }

  # Check for multi-word phrases first
  words_to_keep <- rep(TRUE, length(words))

  if (nrow(ltwa_lookup$phrases) > 0) {
    title_lower <- tolower(title_clean)

    for (i in 1:nrow(ltwa_lookup$phrases)) {
      phrase <- ltwa_lookup$phrases$WORD[i]
      phrase_lower <- tolower(phrase)

      if (str_detect(title_lower, fixed(phrase_lower))) {
        phrase_words <- str_split(phrase, "\\s+")[[1]]
        # Find matching positions
        for (j in 1:(length(words) - length(phrase_words) + 1)) {
          if (
            all(
              tolower(words[j:(j + length(phrase_words) - 1)]) ==
                tolower(phrase_words)
            )
          ) {
            # Mark these positions
            words[j] <- ltwa_lookup$phrases$ABBREVIATION[i]
            if (length(phrase_words) > 1) {
              words_to_keep[(j + 1):(j + length(phrase_words) - 1)] <- FALSE
            }
            break
          }
        }
      }
    }

    words <- words[words_to_keep]
  }

  # Remove articles and conjunctions (except first word)
  to_remove <- rep(FALSE, length(words))

  for (i in seq_along(words)) {
    # Keep first word always
    if (i == 1) {
      next
    }

    # Remove prepositions (not at start)
    if (words[i] %in% stop_words$prepositions) {
      to_remove[i] <- TRUE
    }

    # Remove articles
    if (words[i] %in% stop_words$articles) {
      to_remove[i] <- TRUE
    }

    # Remove conjunctions
    if (words[i] %in% stop_words$conjunctions) {
      to_remove[i] <- TRUE
    }
  }

  words <- words[!to_remove]

  # Handle hyphenated words
  words_expanded <- character()
  for (word in words) {
    if (str_detect(word, "-")) {
      # Split hyphenated word and abbreviate each part
      parts <- str_split(word, "-")[[1]]
      parts_abbr <- sapply(parts, function(p) {
        abbreviate_term(p, ltwa_lookup, stop_words$common_abbr)
      })
      words_expanded <- c(words_expanded, paste(parts_abbr, collapse = "-"))
    } else {
      words_expanded <- c(words_expanded, word)
    }
  }

  # Abbreviate each word
  abbreviated <- sapply(
    words_expanded,
    function(w) {
      # Check if already abbreviated (from multi-word phrase)
      if (str_detect(w, "^[A-Z]+$") && nchar(w) <= 6 && !str_detect(w, "-")) {
        # Might already be abbreviated, but try anyway
        abbreviate_term(w, ltwa_lookup, stop_words$common_abbr)
      } else if (str_detect(w, "-")) {
        # Already processed hyphenated word
        w
      } else {
        abbreviate_term(w, ltwa_lookup, stop_words$common_abbr)
      }
    },
    USE.NAMES = FALSE
  )

  # Collapse and return - NO PERIODS
  result <- paste(abbreviated, collapse = " ")

  # Final cleanup: ensure no periods in result
  result <- str_remove_all(result, "\\.")

  return(result)
}


#' Create ISO4 journal normalization lookup table
#'
#' @param journal_vector Character vector of journal names
#' @param ltwa_db LTWA database data frame
#'
#' @return Data frame with journal_original and journal_iso4 columns
#' @keywords internal

create_journal_iso4_lookup <- function(journal_vector, ltwa_db) {
  unique_journals <- unique(journal_vector)
  unique_journals <- unique_journals[
    !is.na(unique_journals) & unique_journals != ""
  ]

  if (length(unique_journals) == 0) {
    return(tibble(
      journal_original = character(0),
      journal_iso4 = character(0)
    ))
  }

  cat("  Preparing LTWA lookup tables...\n")
  ltwa_lookup <- prepare_ltwa_lookup(ltwa_db)

  cat("  Converting", length(unique_journals), "unique journals to ISO4...\n")

  show_progress <- length(unique_journals) > 100
  pb_step <- if (show_progress) {
    max(1, floor(length(unique_journals) / 20))
  } else {
    Inf
  }

  normalized <- tibble(journal_original = unique_journals) %>%
    mutate(
      journal_iso4 = sapply(seq_along(journal_original), function(i) {
        if (show_progress && i %% pb_step == 0) {
          cat("\r    Progress:", round(100 * i / length(journal_original)), "%")
        }

        tryCatch(
          {
            iso4 <- abbreviate_journal_title(journal_original[i], ltwa_lookup)
            if (is.na(iso4) || iso4 == "") {
              toupper(str_remove_all(journal_original[i], "[[:punct:]]")) %>%
                str_squish()
            } else {
              iso4
            }
          },
          error = function(e) {
            toupper(str_remove_all(journal_original[i], "[[:punct:]]")) %>%
              str_squish()
          }
        )
      })
    )

  if (show_progress) {
    cat("\n")
  }

  n_changed <- sum(
    normalized$journal_original != normalized$journal_iso4,
    na.rm = TRUE
  )
  cat(
    "  Converted",
    n_changed,
    "journals to ISO4 (",
    round(100 * n_changed / nrow(normalized), 1),
    "%)\n"
  )

  return(normalized)
}

#' Remove diacritics from string with robust fallback
#' @keywords internal
remove_diacritics <- function(x) {
  if (is.na(x) || x == "") {
    return(x)
  }

  # Try iconv first
  result <- tryCatch(
    {
      temp <- iconv(x, from = "UTF-8", to = "ASCII//TRANSLIT")
      if (!is.na(temp)) {
        # Clean up artifacts that iconv might add (like " or ')
        temp <- str_remove_all(temp, '[\'\"`^~]')
        temp
      } else {
        NULL
      }
    },
    error = function(e) {
      NULL
    }
  )

  if (is.null(result) || is.na(result)) {
    result <- x %>%
      # German umlauts
      str_replace_all("\u00C4", "AE") %>% # Ä
      str_replace_all("\u00D6", "OE") %>% # Ö
      str_replace_all("\u00DC", "UE") %>% # Ü
      str_replace_all("\u00E4", "ae") %>% # ä
      str_replace_all("\u00F6", "oe") %>% # ö
      str_replace_all("\u00FC", "ue") %>% # ü
      str_replace_all("\u00DF", "ss") %>% # ß
      # French/Spanish accents - vowels with accents
      str_replace_all("\u00C0|\u00C1|\u00C2|\u00C3|\u00C5", "A") %>% # À|Á|Â|Ã|Å
      str_replace_all("\u00C8|\u00C9|\u00CA|\u00CB", "E") %>% # È|É|Ê|Ë
      str_replace_all("\u00CC|\u00CD|\u00CE|\u00CF", "I") %>% # Ì|Í|Î|Ï
      str_replace_all("\u00D2|\u00D3|\u00D4|\u00D5", "O") %>% # Ò|Ó|Ô|Õ
      str_replace_all("\u00D9|\u00DA|\u00DB", "U") %>% # Ù|Ú|Û
      str_replace_all("\u00DD", "Y") %>% # Ý
      str_replace_all("\u00E0|\u00E1|\u00E2|\u00E3|\u00E5", "a") %>% # à|á|â|ã|å
      str_replace_all("\u00E8|\u00E9|\u00EA|\u00EB", "e") %>% # è|é|ê|ë
      str_replace_all("\u00EC|\u00ED|\u00EE|\u00EF", "i") %>% # ì|í|î|ï
      str_replace_all("\u00F2|\u00F3|\u00F4|\u00F5", "o") %>% # ò|ó|ô|õ
      str_replace_all("\u00F9|\u00FA|\u00FB", "u") %>% # ù|ú|û
      str_replace_all("\u00FD|\u00FF", "y") %>% # ý|ÿ
      # Spanish
      str_replace_all("\u00D1", "N") %>% # Ñ
      str_replace_all("\u00F1", "n") %>% # ñ
      # Czech, Polish, etc.
      str_replace_all("\u010C|\u0106|\u00C7", "C") %>% # Č|Ć|Ç
      str_replace_all("\u010D|\u0107|\u00E7", "c") %>% # č|ć|ç
      str_replace_all("\u0160", "S") %>% # Š
      str_replace_all("\u0161", "s") %>% # š
      str_replace_all("\u017D", "Z") %>% # Ž
      str_replace_all("\u017E", "z") %>% # ž
      str_replace_all("\u0141", "L") %>% # Ł
      str_replace_all("\u0142", "l") %>% # ł
      # Scandinavian
      str_replace_all("\u00D8", "O") %>% # Ø
      str_replace_all("\u00F8", "o") %>% # ø
      str_replace_all("\u00C6", "AE") %>% # Æ
      str_replace_all("\u00E6", "ae") %>% # æ
      str_replace_all("\u00C5", "A") %>% # Å
      str_replace_all("\u00E5", "a") %>% # å
      # Turkish
      str_replace_all("\u0130|\u012A", "I") %>% # İ|Ī
      str_replace_all("\u0131|\u012B", "i") %>% # ı|ī
      str_replace_all("\u015E", "S") %>% # Ş
      str_replace_all("\u015F", "s") %>% # ş
      str_replace_all("\u011E", "G") %>% # Ğ
      str_replace_all("\u011F", "g") # ğ
  }

  return(result)
}

#' Convert new Scopus citation format to classic format
#'
#' Scopus has introduced a new citation format where the publication year appears
#' at the end in parentheses instead of after the title. This function converts
#' citations from the new format to the classic format by moving the year from
#' the end to after the title.
#'
#' @param citation Character string containing a bibliographic citation
#'
#' @return Character string with citation in classic Scopus format
#'
#' @details
#' New Scopus format: AUTHOR, TITLE, JOURNAL, VOLUME, ISSUE, PAGES, (YEAR)
#' Classic Scopus format: AUTHOR, TITLE (YEAR) JOURNAL, VOLUME, PAGES
#'
#' The function uses a robust approach:
#' \itemize{
#'   \item Extracts year from end (YYYY)
#'   \item Extracts first author from beginning
#'   \item Extracts pages (PP. xxx-xxx or PP. xxx)
#'   \item Extracts volume and issue numbers
#'   \item Extracts journal name (text before volume/issue/pages)
#'   \item Deduces title as remaining text after author
#' }
#'
#' @keywords internal

convert_scopus_new_to_classic <- function(citation) {
  # Check if citation ends with year in parentheses: (YYYY)
  if (!str_detect(citation, "\\(\\d{4}\\)\\s*$")) {
    return(citation) # Not new format, return unchanged
  }

  # Extract and remove year from the end
  year_match <- str_extract(citation, "\\(\\d{4}\\)\\s*$")
  year <- str_extract(year_match, "\\d{4}")
  citation_no_year <- str_remove(citation, "\\s*\\(\\d{4}\\)\\s*$") %>%
    str_trim()

  # Extract first author (everything before first comma)
  first_comma_pos <- str_locate(citation_no_year, ",")[1, "start"]
  if (is.na(first_comma_pos)) {
    return(citation) # Cannot parse without commas
  }

  first_author <- substr(citation_no_year, 1, first_comma_pos - 1) %>%
    str_trim()
  after_author <- substr(
    citation_no_year,
    first_comma_pos + 1,
    nchar(citation_no_year)
  ) %>%
    str_trim()

  # Extract pages (PP. xxx-xxx or PP. xxx) - usually near the end
  pages_pattern <- "PP\\.?\\s*\\d+\\s*[-\u2013\u2014]?\\s*\\d*"
  pages_match <- str_extract(
    after_author,
    regex(pages_pattern, ignore_case = TRUE)
  )

  # Remove pages from string to simplify further parsing
  if (!is.na(pages_match)) {
    after_author_no_pages <- str_remove(
      after_author,
      regex(pages_pattern, ignore_case = TRUE)
    ) %>%
      str_trim() %>%
      str_remove(",\\s*$") %>% # Remove trailing comma if present
      str_trim()
  } else {
    after_author_no_pages <- after_author
    pages_match <- ""
  }

  # Split remaining text by commas
  parts <- str_split(after_author_no_pages, ",")[[1]] %>%
    str_trim()

  if (length(parts) < 2) {
    # Not enough parts to parse properly
    return(citation)
  }

  # Working backwards from the end to identify known components:
  # Last part (or second-to-last if pages removed) should be issue or volume
  # Before that should be volume (if issue present) or journal
  # Everything else at the beginning is the title

  # Try to identify volume and issue
  # Volume: usually a number by itself or with V prefix
  # Issue: usually a number by itself, comes after volume

  # Check last parts for numeric patterns
  volume <- NA_character_
  issue <- NA_character_
  journal_idx <- NA_integer_

  # Search from the end backwards
  n_parts <- length(parts)

  # Strategy: Find numeric values from the end
  # In new Scopus format: ..., JOURNAL, VOLUME, ISSUE, (YEAR)
  # After removing year and pages: ..., JOURNAL, VOLUME, ISSUE
  # So last 1-2 parts are likely volume/issue, before that is journal

  numeric_positions <- c()
  for (i in n_parts:1) {
    part <- parts[i]
    # Check if this part is purely numeric (not a year, which we already removed)
    if (str_detect(part, "^\\d+$") && nchar(part) <= 4) {
      numeric_positions <- c(numeric_positions, i)
    }
  }

  # Identify volume and issue based on position
  if (length(numeric_positions) >= 2) {
    # Last two numeric parts are likely issue and volume (in that order from end)
    issue <- parts[numeric_positions[1]]
    volume <- parts[numeric_positions[2]]
    # Journal is the part before volume
    journal_idx <- numeric_positions[2] - 1
  } else if (length(numeric_positions) == 1) {
    # Only one numeric part - it's the volume
    volume <- parts[numeric_positions[1]]
    issue <- NA_character_
    # Journal is the part before volume
    journal_idx <- numeric_positions[1] - 1
  } else {
    # No clear numeric volume/issue found
    # Assume last part is journal
    journal_idx <- n_parts
  }

  # Journal is the part just before volume/issue
  if (journal_idx >= 1 && journal_idx <= length(parts)) {
    journal <- parts[journal_idx]
  } else {
    journal <- ""
  }

  # Title is everything from start up to (but not including) journal
  if (journal_idx > 1) {
    title <- paste(parts[1:(journal_idx - 1)], collapse = ", ")
  } else {
    title <- ""
  }

  # Reconstruct in classic format: AUTHOR, TITLE (YEAR) JOURNAL, VOLUME, ISSUE, PAGES
  result_parts <- c()

  # Add author
  result_parts <- c(result_parts, first_author)

  # Add title with year: TITLE (YEAR)
  if (nchar(title) > 0) {
    result_parts <- c(result_parts, paste0(title, " (", year, ")"))
  } else {
    result_parts <- c(result_parts, paste0("(", year, ")"))
  }

  # Now add journal and rest WITHOUT commas, space-separated like classic Scopus
  # Classic Scopus: AUTHOR, TITLE (YEAR) JOURNAL, VOLUME, PAGES

  # Build the "after year" part
  after_year_parts <- c()

  if (nchar(journal) > 0) {
    after_year_parts <- c(after_year_parts, journal)
  }

  if (!is.na(volume)) {
    after_year_parts <- c(after_year_parts, volume)
  }

  if (!is.na(issue)) {
    after_year_parts <- c(after_year_parts, issue)
  }

  if (nchar(pages_match) > 0) {
    after_year_parts <- c(after_year_parts, pages_match)
  }

  # Join the parts correctly:
  # AUTHOR, TITLE (YEAR) JOURNAL, VOLUME, ISSUE, PAGES
  if (length(result_parts) > 0 && length(after_year_parts) > 0) {
    classic_format <- paste0(
      paste(result_parts, collapse = ", "),
      " ",
      paste(after_year_parts, collapse = ", ")
    )
  } else if (length(result_parts) > 0) {
    classic_format <- paste(result_parts, collapse = ", ")
  } else {
    classic_format <- citation # Fallback to original
  }

  return(classic_format)
}


#' Normalize and match bibliographic citations
#'
#' This function performs advanced normalization and fuzzy matching of bibliographic
#' citations to identify and group citations that refer to the same work but are
#' formatted differently. It uses a multi-phase approach combining string normalization,
#' blocking strategies, hierarchical clustering, and post-processing to achieve both
#' speed and accuracy on large citation datasets.
#'
#' @param CR_vector Character vector containing bibliographic citations to be normalized and matched.
#' @param threshold Numeric value between 0 and 1 indicating the similarity threshold
#'   for matching citations. Higher values (e.g., 0.90-0.95) produce more conservative
#'   matching, while lower values (e.g., 0.75-0.80) produce more aggressive matching.
#'   Default is 0.90, which provides a good balance between precision and recall.
#' @param method String distance method to use for fuzzy matching. Options include:
#'   \itemize{
#'     \item "jw" (default): Jaro-Winkler distance, optimized for bibliographic strings
#'     \item "lv": Levenshtein distance
#'     \item "osa": Optimal String Alignment distance
#'     \item "lcs": Longest Common Subsequence distance
#'     \item Other methods supported by \code{\link[stringdist]{stringdistmatrix}}
#'   }
#' @param min_chars Minimum characters for valid citations (default: 20).
#' @param max_block_size Integer. Blocks with at least this many unique normalized
#'   strings skip within-block fuzzy matching and fall back to exact matching only,
#'   to bound the cost of the pairwise distance matrix (default: 100).
#' @param use_iso4 Logical. If \code{TRUE} (default), normalize journal names to their
#'   ISO 4 abbreviated form via the LTWA database (Phase 1.5). Set to \code{FALSE} to
#'   disable ISO 4 / LTWA journal normalization (used for ablation analyses).
#' @param use_doi Logical. If \code{TRUE} (default), perform exact matching on DOIs
#'   (part of Phase 2). Set to \code{FALSE} to disable DOI-based matching.
#' @param use_exact Logical. If \code{TRUE} (default), perform exact normalized-string
#'   and punctuation-invariant matching (Phase 2). Set to \code{FALSE} to disable them.
#' @param fuzzy Logical. If \code{TRUE} (default), perform within-block matching
#'   (Phase 4: WoS deterministic key matching and Scopus fuzzy clustering). Set to
#'   \code{FALSE} to disable within-block matching, keeping only the exact phases.
#' @param use_postproc Logical. If \code{TRUE} (default), perform Phase 4.5
#'   metadata-based post-processing merge. Set to \code{FALSE} to disable it.
#' @param title_guard Logical. If \code{TRUE}, run an optional Phase 4.6 that
#'   purifies clusters by detecting series part markers in titles: distinct
#'   works that share author, year, journal and volume but differ only in a
#'   series designator (e.g. "... Part I" / "... Part II", "... I." / "... II.")
#'   are split into separate clusters. This step only splits clusters, never
#'   merges; it relies solely on the part marker (not on full-title similarity)
#'   so it does not interfere with robustness to title typos. Default
#'   \code{FALSE} (legacy behaviour).
#'
#' @details
#' The function implements a five-phase matching algorithm:
#'
#' \strong{Phase 1: Normalization and Feature Extraction}
#' \itemize{
#'   \item Converts text to uppercase
#'   \item Removes issue numbers and page numbers (which often contain typos)
#'   \item Removes punctuation and normalizes whitespace
#'   \item Expands common journal abbreviations (e.g., "J. CLEAN. PROD." -> "JOURNAL OF CLEANER PRODUCTION")
#'   \item Extracts key features: first author, year, journal, volume, pages
#' }
#'
#' \strong{Phase 1.5: Journal Normalization}
#' The function uses the LTWA (List of Title Word Abbreviations) database from
#' ISO 4 standards to normalize journal names. This ensures that abbreviated
#' forms (e.g., "J. Clean. Prod.") and full forms (e.g., "Journal of Cleaner
#' Production") are recognized as the same journal and matched together.
#'
#' The LTWA database is included in the bibliometrix package. If not found,
#' the function attempts to download it from ISSN.org. Journal normalization
#' can be disabled by ensuring the LTWA database is not available.
#'
#' \strong{Phase 2: Blocking}
#' Citations are grouped into blocks by first author and year. This dramatically
#' reduces computational complexity from O(n^2) to approximately O(k*m^2), where k is
#' the number of blocks and m is the average block size.
#'
#' \strong{Phase 3: Within-Block Matching}
#' Within each block, citations are compared using string distance metrics and
#' hierarchical clustering. For blocks with at least \code{max_block_size} unique
#' normalized strings (default 100), exact matching on normalized strings is used
#' instead to maintain performance.
#'
#' \strong{Phase 4: Canonical Representative Selection}
#' For each cluster, the most complete citation (prioritizing those with volume
#' and page information) is selected as the canonical representative.
#'
#' \strong{Phase 5: Post-Processing}
#' Citations sharing the same first author, year, journal, and volume are merged
#' into a single cluster, even if they weren't matched in Phase 3. This catches
#' cases where minor title variations prevented matching.
#'
#' @return A data frame with the following columns:
#' \itemize{
#'   \item \code{CR_original}: Original citation string
#'   \item \code{CR_canonical}: Canonical (representative) citation for the cluster
#'   \item \code{cluster_id}: Unique identifier for each citation cluster
#'   \item \code{n_cluster}: Number of citations in the cluster
#'   \item \code{first_author}: First author surname
#'   \item \code{year}: Publication year
#'   \item \code{journal_iso4}: Journal name normalized to ISO4 abbreviated form
#'   \item \code{journal_original}: Original journal name as extracted from citation
#'   \item \code{volume}: Volume number
#'   \item \code{doi}: Digital Object Identifier (when available)
#'   \item \code{blocking_key}: Internal key used for blocking (author_year_journal)
#' }
#'
#' @examples
#' \dontrun{
#' # Load bibliometrix data
#' data(scientometrics, package = "bibliometrixData")
#'
#' # Extract and normalize citations
#' CR_vector <- unlist(strsplit(scientometrics$CR, ";"))
#' CR_vector <- trimws(CR_vector)
#'
#' # Perform normalization with default threshold
#' matched <- normalize_citations(CR_vector)
#'
#' # View matching statistics
#' table(matched$n_cluster)
#'
#' # Find all variants of a specific citation
#' subset(matched, cluster_id == matched$cluster_id[1])
#'
#' # Use more conservative matching
#' matched_conservative <- normalize_citations(CR_vector, threshold = 0.90)
#' }
#'
#' @seealso
#' \code{\link{applyReferenceMatching}} for direct application to bibliometrix data frames
#'
#' @references
#' Aria, M. & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive
#' science mapping analysis. Journal of Informetrics, 11(4), 959-975.
#'
#' @export

normalize_citations <- function(
  CR_vector,
  threshold = 0.90,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
) {
  # Detect citation format
  detect_format <- function(x) {
    # New Scopus format: ends with (YEAR)
    if (str_detect(x, "\\(\\d{4}\\)\\s+[A-Z]")) {
      # Classic Scopus format: AUTHOR, TITLE (YEAR) JOURNAL, V, PP
      return("scopus")
    } else if (str_detect(x, ",\\s*\\d{4},\\s*[A-Z]")) {
      # WoS format: AUTHOR, YEAR, JOURNAL, V, P, DOI
      return("wos")
    } else {
      return("unknown")
    }
  }

  # Base normalization function
  normalize_string <- function(x) {
    x %>%
      # Remove diacritics FIRST (before uppercasing)
      remove_diacritics() %>%
      toupper() %>%
      str_replace_all("\\s*:\\s*", " ") %>% # Remove colons with surrounding spaces
      str_replace_all("\\s*;\\s*", " ") %>% # Remove semicolons with surrounding spaces
      str_replace_all("\\s*-\\s*", " ") %>% # Normalize hyphens
      str_replace_all("\\(\\d+\\)", "") %>% # Remove issue numbers
      str_replace_all("PP\\.?\\s*\\d+\\s*[-\u2013\u2014]\\s*\\d+", "") %>% # Remove page numbers
      str_replace_all("P\\.?\\s*\\d+\\s*[-\u2013\u2014]\\s*\\d+", "") %>%
      # remove quotes symbols
      str_replace_all("<e2><80><9c>", " ") %>%
      str_replace_all('"', " ") %>%
      str_replace_all("[[:punct:]]", " ") %>% # Remove punctuation
      str_replace_all("\\s+", " ") %>% # Normalize whitespace
      str_trim()
  }

  # Extract DOI (for WoS format)
  extract_doi <- function(x) {
    doi_pattern <- "DOI\\s*:?\\s*(10\\.\\d{4,}/[^,\\s]+)"
    doi <- str_extract(x, regex(doi_pattern, ignore_case = TRUE))
    if (!is.na(doi)) {
      doi <- str_remove(doi, regex("^DOI\\s*:?\\s*", ignore_case = TRUE))
      doi <- str_remove(doi, "[,;\\s]+$")
    }
    return(doi)
  }

  # Extract key features - unified for both formats
  extract_key_features <- function(x) {
    format <- detect_format(x)

    # First author - IMPROVED PATTERN for both WoS and Scopus
    # WoS: "Adams CA," or "Atkins J," or "Atkins JF,"
    # Scopus: "ADAMS, C." or similar
    first_author <- str_extract(x, "^[A-Z][A-Za-z\\s\\-']+(?=,)") %>%
      str_trim()

    # If extraction failed, try alternative pattern
    if (is.na(first_author)) {
      first_author <- str_extract(x, "^[^,]+") %>%
        str_trim()
    }

    # Clean up: remove diacritics, trailing periods, normalize
    if (!is.na(first_author)) {
      first_author <- first_author %>%
        remove_diacritics() %>%
        str_remove_all("\\.$") %>%
        str_squish() %>%
        toupper()
    }

    # Year
    year <- str_extract(x, "\\d{4}") %>%
      head(1) # Take first year found

    # DOI extraction
    doi <- extract_doi(x)

    if (format == "scopus" || format == "scopus_new") {
      # Scopus: AUTHOR1, AUTHOR2, ..., TITLE (YEAR) JOURNAL, VOLUME, PAGES

      # Extract everything between first comma and year
      temp_title <- str_extract(x, "(?<=,\\s)(.+?)(?=\\s*\\(\\d{4}\\))")

      if (!is.na(temp_title)) {
        # Strategy: Find the LAST occurrence of author pattern
        # Pattern needs to match:
        # - SURNAME, I., (with comma after) - for middle authors
        # - SURNAME, I. (without comma after) - for last author

        # Try pattern WITH comma after (for all authors except last)
        author_pattern_with_comma <- "[A-Z][A-Za-z\\-']+,\\s*[A-Z][A-Z\\.\\s]*\\.?,\\s*"
        matches_with_comma <- str_locate_all(
          temp_title,
          author_pattern_with_comma
        )[[1]]

        # Try pattern WITHOUT comma after (for potentially last author)
        author_pattern_no_comma <- "[A-Z][A-Za-z\\-']+,\\s*[A-Z][A-Z\\.\\s]*\\.?\\s+"
        matches_no_comma <- str_locate_all(
          temp_title,
          author_pattern_no_comma
        )[[1]]

        # Combine all matches and find the one that ends last
        last_end <- 0

        if (nrow(matches_with_comma) > 0) {
          last_end <- max(last_end, max(matches_with_comma[, "end"]))
        }

        if (nrow(matches_no_comma) > 0) {
          last_end <- max(last_end, max(matches_no_comma[, "end"]))
        }

        if (last_end > 0) {
          # Extract title starting after last author
          title <- substr(temp_title, last_end + 1, nchar(temp_title))
        } else {
          # No author pattern found - use full temp_title
          title <- temp_title
        }

        # Clean up
        title <- str_replace_all(title, '"|<e2><80><9c>', " ")
        title <- str_trim(title)

        # Safety: remove any remaining author-like fragments at start
        # (single letter followed by dot and space)
        title <- str_remove(title, "^[A-Z]\\.?,?\\s+")
        title <- str_trim(title)
      } else {
        title <- NA
      }

      # Rest of Scopus extraction...
      journal <- str_extract(
        x,
        "(?<=\\d{4}\\)\\s)([A-Z][A-Z\\s&\\.-]+?)(?=,\\s*\\d+|,\\s*PP)"
      )

      # Volume
      after_year <- str_extract(x, "\\d{4}\\)(.+)$")
      if (!is.na(after_year)) {
        parts <- str_split(after_year, ",")[[1]]
        if (length(parts) >= 2) {
          volume <- str_extract(parts[2], "^\\s*(\\d+)") %>%
            str_extract("\\d+")
        } else {
          volume <- NA
        }
      } else {
        volume <- NA
      }

      # Pages
      pages <- str_extract(
        x,
        "PP\\.?\\s*\\d+[-\u2013\u2014]\\d+|PP\\.?\\s*\\d+"
      )

      # Extract significant words from CLEANED title
      if (!is.na(title) && nchar(title) > 0) {
        title_words <- str_extract_all(title, "\\b[A-Z]{3,}\\b")[[1]]

        # Filter stop words
        stop_words <- c(
          "THE",
          "AND",
          "FOR",
          "FROM",
          "WITH",
          "ABOUT",
          "THAT",
          "THIS",
          "BETWEEN",
          "THROUGH",
          "DURING",
          "BEFORE",
          "AFTER",
          "WHICH",
          "THEIR",
          "THESE",
          "THOSE",
          "WHEN",
          "WHERE",
          "WHAT"
        )
        title_words <- title_words[!title_words %in% stop_words]

        # Take first 4 significant words
        title_words <- head(title_words, 4) %>%
          paste(collapse = " ")

        # Safety check
        if (is.na(title_words) || nchar(title_words) < 3) {
          title_words <- NA_character_
        }
      } else {
        title_words <- NA_character_
      }
    } else {
      # WoS: AUTHOR, YEAR, JOURNAL, V##, P##, DOI

      # Extract journal (after year, before V or P or DOI)
      journal <- str_extract(
        x,
        "(?<=\\d{4},\\s)([A-Z][A-Z\\s&\\-]+?)(?=,\\s*V\\d+|,\\s*P\\d+|,\\s*DOI)"
      )

      # Volume - look for V followed by digits
      volume <- str_extract(x, "V(\\d+)") %>%
        str_extract("\\d+")

      # Pages - look for P followed by digits
      pages <- str_extract(x, "P(\\d+)") %>%
        str_remove("P")

      # For WoS, NO title extraction - not available in format
      title_words <- NA_character_
    }

    list(
      format = format,
      first_author = first_author,
      year = year,
      doi = doi,
      journal = journal,
      volume = volume,
      pages = pages,
      title_words = title_words,
      full_normalized = normalize_string(x)
    )
  }

  cat("Phase 1: Cleaning and feature extraction...\n")

  # Remove invalid citations
  df <- tibble(
    CR_original = CR_vector,
    CR_id = seq_along(CR_vector)
  ) %>%
    dplyr::filter(
      !is.na(CR_original),
      CR_original != "",
      nchar(CR_original) >= min_chars,
      !grepl("^NO TITLE CAPTURED$", CR_original, ignore.case = TRUE),
      !grepl("^ANONYMOUS", CR_original, ignore.case = TRUE),
      !grepl("^\\d+-[A-Z#]+$", CR_original),
      !grepl("^[A-Z\\s]{1,10}$", CR_original) # Remove very short strings
    )

  # Extract features
  features <- lapply(df$CR_original, extract_key_features)

  df <- df %>%
    mutate(
      format = sapply(features, function(x) x$format %||% NA),
      first_author = sapply(features, function(x) x$first_author %||% NA),
      year = sapply(features, function(x) x$year %||% NA),
      doi = sapply(features, function(x) x$doi %||% NA),
      journal = sapply(features, function(x) x$journal %||% NA),
      volume = sapply(features, function(x) x$volume %||% NA),
      pages = sapply(features, function(x) x$pages %||% NA),
      title_words = sapply(features, function(x) x$title_words %||% NA),
      CR_normalized = sapply(features, function(x) x$full_normalized)
    ) %>% # exclude rows with NA in first_author and year
    dplyr::filter(!is.na(first_author), !is.na(year)) %>%
    # exclude rows with year < 1700 and > current year+1
    dplyr::filter(
      as.numeric(year) >= 1700 &
        as.numeric(year) <= as.numeric(format(Sys.Date(), "%Y")) + 1
    )

  cat("  Filtered out", length(CR_vector) - nrow(df), "invalid citations\n")

  cat(
    "  Detected formats: WoS =",
    sum(df$format == "wos", na.rm = TRUE),
    ", Scopus =",
    sum(df$format == "scopus", na.rm = TRUE),
    ", Scopus New =",
    sum(df$format == "scopus_new", na.rm = TRUE),
    ", Unknown =",
    sum(df$format == "unknown", na.rm = TRUE),
    "\n"
  )

  cat("Phase 1.5: Normalizing journal names to ISO4 format...\n")

  # Load LTWA database
  ltwa <- NULL

  # Try to load from bibliometrix package data
  data("ltwa", package = "bibliometrix", envir = environment())

  # Apply ISO4 normalization if LTWA is available (and not disabled via use_iso4)
  if (use_iso4 && !is.null(ltwa) && nrow(ltwa) > 0) {
    # Create ISO4 lookup table for efficiency
    journal_iso4_lookup <- create_journal_iso4_lookup(df$journal, ltwa)

    # Apply normalization
    df <- df %>%
      left_join(journal_iso4_lookup, by = c("journal" = "journal_original")) %>%
      mutate(
        journal_original = journal, # Preserve original
        journal = coalesce(journal_iso4, journal) # Use ISO4, fallback to original
      ) %>%
      select(-journal_iso4)

    cat("  Journal ISO4 normalization completed\n")

    # *** CRITICAL: Recreate CR_normalized with ISO4 journal names ***
    cat("  Recreating normalized strings with ISO4 journal names...\n")
    df <- df %>%
      mutate(
        # Reconstruct the citation string with ISO4 journal
        temp_citation = case_when(
          format %in% c("scopus", "scopus_new") ~
            paste0(
              first_author,
              ", ",
              year,
              " ",
              journal,
              " ",
              ifelse(!is.na(volume), paste0("V", volume), ""),
              ifelse(!is.na(pages), paste0(" ", pages), "")
            ),
          format == "wos" ~
            paste0(
              first_author,
              ", ",
              year,
              ", ",
              journal,
              ifelse(!is.na(volume), paste0(", V", volume), ""),
              ifelse(!is.na(pages), paste0(", P", pages), ""),
              ifelse(!is.na(doi), paste0(", DOI ", doi), "")
            ),
          TRUE ~ CR_original
        ),
        # Use sapply to normalize each string individually
        CR_normalized = sapply(
          temp_citation,
          normalize_string,
          USE.NAMES = FALSE
        )
      ) %>%
      select(-temp_citation)
  } else {
    cat("  Proceeding without journal normalization\n")
    df <- df %>%
      mutate(journal_original = journal)
  }

  cat("Phase 2: Exact matching by DOI and normalized string...\n")

  # Initialize cluster_id
  df <- df %>%
    mutate(cluster_id = NA_character_)

  # DOI-based matching (only for WoS with valid DOI)
  valid_dois <- df %>%
    dplyr::filter(
      !is.na(doi),
      str_detect(doi, "^10\\.\\d{4,}/"),
      nchar(doi) >= 10
    )

  if (use_doi && nrow(valid_dois) > 0) {
    doi_clusters <- valid_dois %>%
      group_by(doi) %>%
      mutate(cluster_id = paste0("DOI_", min(CR_id))) %>%
      ungroup()

    df$cluster_id[df$CR_id %in% doi_clusters$CR_id] <- doi_clusters$cluster_id

    cat(
      "  Matched",
      nrow(doi_clusters),
      "citations via",
      n_distinct(doi_clusters$doi),
      "unique DOIs\n"
    )
  }

  # Exact normalized string matching (fast pre-clustering)
  unmatched_df <- df %>%
    dplyr::filter(is.na(cluster_id))

  if (use_exact && nrow(unmatched_df) > 0) {
    exact_matches <- unmatched_df %>%
      group_by(CR_normalized) %>%
      mutate(
        n_exact = n(),
        cluster_id = if_else(
          n_exact > 1,
          paste0("EXACT_", min(CR_id)),
          NA_character_
        )
      ) %>%
      ungroup() %>%
      dplyr::filter(!is.na(cluster_id))

    if (nrow(exact_matches) > 0) {
      df$cluster_id[
        df$CR_id %in% exact_matches$CR_id
      ] <- exact_matches$cluster_id
      cat(
        "  Matched",
        nrow(exact_matches),
        "citations via exact normalization\n"
      )
    }
  }

  # Punctuation-invariant matching: additional exact matching pass using
  # a stripped form of the original string (all non-alphanumeric characters
  # removed, case-folded, whitespace collapsed). This catches citation
  # pairs that differ only in punctuation, spacing, or formatting.
  unmatched_df2 <- df %>%
    dplyr::filter(is.na(cluster_id))

  if (use_exact && nrow(unmatched_df2) > 0) {
    unmatched_df2 <- unmatched_df2 %>%
      mutate(
        CR_stripped = sapply(CR_original, function(x) {
          x %>%
            remove_diacritics() %>%
            toupper() %>%
            str_replace_all("[^A-Z0-9]", "")
        }, USE.NAMES = FALSE)
      )

    stripped_matches <- unmatched_df2 %>%
      group_by(CR_stripped) %>%
      mutate(
        n_stripped = n(),
        cluster_id = if_else(
          n_stripped > 1,
          paste0("STRIP_", min(CR_id)),
          NA_character_
        )
      ) %>%
      ungroup() %>%
      dplyr::filter(!is.na(cluster_id))

    if (nrow(stripped_matches) > 0) {
      df$cluster_id[
        df$CR_id %in% stripped_matches$CR_id
      ] <- stripped_matches$cluster_id
      cat(
        "  Matched",
        nrow(stripped_matches),
        "citations via punctuation-invariant matching\n"
      )
    }
  }

  cat("Phase 3: Blocking by author + year...\n")

  # Block by author surname + year to allow matching across journal/name variants
  # Use only surname (first word) so "ARIA M" and "ARIA MASSIMO" share a block
  df <- df %>%
    mutate(
      blocking_key = paste0(
        coalesce(gsub("\\s.*", "", first_author), "UNK"),
        "_",
        coalesce(year, "0000")
      )
    )

  block_sizes <- table(df$blocking_key)
  cat("  Created", length(block_sizes), "blocks\n")
  cat("  Average block size:", round(mean(block_sizes), 1), "\n")

  cat("Phase 4: Fuzzy matching within blocks...\n")

  # Function to match within a block
  match_within_block <- function(block_df) {
    # Only work with unmatched citations
    unmatched <- block_df %>% dplyr::filter(is.na(cluster_id))

    if (nrow(unmatched) <= 1) {
      if (nrow(unmatched) == 1) {
        block_df$cluster_id[is.na(
          block_df$cluster_id
        )] <- as.character(unmatched$CR_id[1])
      }
      return(block_df)
    }

    # Ablation: when fuzzy matching is disabled, skip within-block matching
    # entirely (both the WoS deterministic key and the Scopus fuzzy branch);
    # each still-unmatched citation is kept as its own singleton cluster.
    if (!fuzzy) {
      for (i in seq_len(nrow(unmatched))) {
        if (is.na(block_df$cluster_id[block_df$CR_id == unmatched$CR_id[i]])) {
          block_df$cluster_id[block_df$CR_id == unmatched$CR_id[i]] <-
            as.character(unmatched$CR_id[i])
        }
      }
      return(block_df)
    }

    # Check if block is predominantly WoS or Scopus
    format_counts <- table(unmatched$format)
    predominant_format <- names(which.max(format_counts))

    # For WoS citations, use more deterministic matching based on metadata
    if (predominant_format == "wos") {
      # WoS: Match by first_author + year + journal + volume + page_start
      unmatched <- unmatched %>%
        mutate(
          page_start = str_extract(pages, "\\d+"),
          wos_key = paste0(
            coalesce(gsub("\\s.*", "", first_author), "UNK"),
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(gsub("[^A-Z0-9]", "", toupper(journal)), "UNK"),
            "_",
            coalesce(volume, "NA"),
            "_",
            coalesce(page_start, "NOPAGE")
          )
        ) %>%
        group_by(wos_key) %>%
        mutate(
          new_cluster_id = paste0(
            unique(block_df$blocking_key)[1],
            "_C",
            cur_group_id()
          )
        ) %>%
        ungroup() %>%
        select(-wos_key, -page_start)

      block_df$cluster_id[
        block_df$CR_id %in% unmatched$CR_id
      ] <- unmatched$new_cluster_id
      return(block_df)
    }

    # For Scopus citations, use fuzzy matching as before
    unique_norm <- unique(unmatched$CR_normalized)

    if (length(unique_norm) == 1) {
      cluster_id <- as.character(unmatched$CR_id[1])
      block_df$cluster_id[is.na(block_df$cluster_id)] <- cluster_id
      return(block_df)
    }

    # Fuzzy matching for Scopus (or mixed blocks)
    if (length(unique_norm) > 1 && length(unique_norm) < max_block_size) {
      dist_matrix <- stringdist::stringdistmatrix(
        unique_norm,
        unique_norm,
        method = method
      )

      max_dist <- max(dist_matrix)
      if (max_dist > 0) {
        sim_matrix <- 1 - (dist_matrix / max_dist)
      } else {
        sim_matrix <- matrix(
          1,
          nrow = length(unique_norm),
          ncol = length(unique_norm)
        )
      }

      if (nrow(sim_matrix) > 1) {
        hc <- hclust(as.dist(1 - sim_matrix), method = "complete")
        clusters <- cutree(hc, h = 1 - threshold)

        cluster_map <- tibble(
          CR_normalized = unique_norm,
          temp_cluster = clusters
        )

        unmatched <- unmatched %>%
          left_join(cluster_map, by = "CR_normalized") %>%
          mutate(
            new_cluster_id = paste0(
              unique(block_df$blocking_key)[1],
              "_C",
              temp_cluster
            )
          )

        block_df$cluster_id[
          block_df$CR_id %in% unmatched$CR_id
        ] <- unmatched$new_cluster_id
      }
    } else {
      # For very large blocks, keep as separate
      for (i in 1:nrow(unmatched)) {
        if (is.na(block_df$cluster_id[block_df$CR_id == unmatched$CR_id[i]])) {
          block_df$cluster_id[block_df$CR_id == unmatched$CR_id[i]] <-
            as.character(unmatched$CR_id[i])
        }
      }
    }

    return(block_df)
  }

  # Apply matching per block
  df_matched <- df %>%
    group_by(blocking_key) %>%
    group_split() %>%
    lapply(match_within_block) %>%
    bind_rows()

  # Ensure all have cluster_id
  df_matched <- df_matched %>%
    mutate(
      cluster_id = ifelse(is.na(cluster_id), as.character(CR_id), cluster_id)
    )

  cat(
    "Phase 4.5: Post-processing - Merging clusters with identical metadata...\n"
  )

  n_clusters_before_merge <- n_distinct(df_matched$cluster_id)

  # Different merge strategies based on format
  # Journal is excluded from merge keys: author + year + volume + page_start
  # (+ title_words for Scopus) is already highly discriminating
  if (use_postproc) {
  df_matched <- df_matched %>%
    mutate(
      # Extract starting page number
      page_start = str_extract(pages, "^\\d+"),
      # Use surname only for merge keys (handles "ARIA M" vs "ARIA MASSIMO")
      author_surname = coalesce(gsub("\\s.*", "", first_author), "UNK"),

      merge_key = case_when(
        # WoS format: DOI is most reliable
        format == "wos" & !is.na(doi) ~ paste0("DOI_", doi),

        # WoS format without DOI: use metadata + pages
        format == "wos" ~
          paste0(
            author_surname,
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(volume, "NA"),
            "_",
            coalesce(page_start, "NOPAGE")
          ),

        # Scopus format: use metadata + pages + title fingerprint
        format %in%
          c("scopus", "scopus_new") &
          !is.na(page_start) &
          !is.na(title_words) ~
          paste0(
            author_surname,
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(volume, "NA"),
            "_",
            coalesce(page_start, "NOPAGE"),
            "_",
            title_words
          ),

        # Scopus with pages but no title
        format %in% c("scopus", "scopus_new") & !is.na(page_start) ~
          paste0(
            author_surname,
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(volume, "NA"),
            "_",
            coalesce(page_start, "NOPAGE")
          ),

        # Scopus with title but no pages (rare)
        format %in% c("scopus", "scopus_new") & !is.na(title_words) ~
          paste0(
            author_surname,
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(volume, "NA"),
            "_",
            title_words
          ),

        # Fallback with title: use title_words to distinguish
        !is.na(title_words) ~
          paste0(
            author_surname,
            "_",
            coalesce(year, "0000"),
            "_",
            coalesce(volume, "NA"),
            "_",
            title_words
          ),

        # Last resort: no pages, no title - keep original cluster (no merge)
        TRUE ~ paste0("NOMERGE_", cluster_id)
      )
    ) %>%
    group_by(merge_key) %>%
    mutate(
      cluster_id = min(cluster_id, na.rm = TRUE)
    ) %>%
    ungroup() %>%
    select(-merge_key, -page_start, -author_surname)
  }

  n_clusters_after_merge <- n_distinct(df_matched$cluster_id)
  n_additional_matches <- n_clusters_before_merge - n_clusters_after_merge

  cat("  Clusters before metadata merge:", n_clusters_before_merge, "\n")
  cat("  Clusters after metadata merge:", n_clusters_after_merge, "\n")
  cat("  Additional matches found:", n_additional_matches, "\n")

  # ------------------------------------------------------------------
  # Phase 4.6 (optional): Title-based cluster purification.
  # Prevents over-merging of DISTINCT works that share author + year +
  # journal + volume but differ in title (e.g. multi-part series such as
  # "... Part I" / "... Part II", or several same-author/same-year/same-
  # journal articles whose titles were dropped from the matching string).
  # This step ONLY SPLITS existing clusters; it never merges. Disabled by
  # default to preserve legacy behaviour.
  # ------------------------------------------------------------------
  if (isTRUE(title_guard)) {
    cat("Phase 4.6: Title-based cluster purification...\n")

    # Extract a normalised title from a raw Scopus reference (text between the
    # author block and the year), returning NA for WoS / unparseable strings.
    .get_title <- function(x, fmt) {
      if (is.na(x) || is.na(fmt) || !fmt %in% c("scopus", "scopus_new")) {
        return(NA_character_)
      }
      t <- str_extract(x, "^.*?(?=\\s*\\(\\d{4}\\))")
      if (is.na(t)) {
        return(NA_character_)
      }
      t2 <- str_extract(t, "(?<=,\\s).*$") # drop leading first-author chunk
      if (!is.na(t2)) {
        t <- t2
      }
      t <- toupper(remove_diacritics(t))
      t <- gsub("[^A-Z0-9 ]", " ", t)
      t <- str_trim(gsub("\\s+", " ", t))
      if (nchar(t) < 4) {
        return(NA_character_)
      }
      t
    }

    # Extract a series-part marker (roman numeral or digit, optionally after
    # "PART") and map it to a canonical part number; NA when none is present.
    .get_part <- function(t) {
      if (is.na(t)) {
        return(NA_character_)
      }
      m <- regmatches(
        t,
        regexpr("\\b(PART\\s+)?(VIII|VII|VI|IV|IX|III|II|I|V|X|[1-9])\\b", t)
      )
      if (length(m) == 0 || m == "") {
        return(NA_character_)
      }
      r <- sub("PART\\s+", "", m)
      rmap <- c(
        I = "1", II = "2", III = "3", IV = "4", V = "5",
        VI = "6", VII = "7", VIII = "8", IX = "9", X = "10"
      )
      if (r %in% names(rmap)) {
        return(unname(rmap[r]))
      }
      r
    }

    ti_vec <- mapply(
      .get_title,
      df_matched$CR_original,
      df_matched$format,
      USE.NAMES = FALSE
    )
    pt_vec <- vapply(ti_vec, .get_part, character(1), USE.NAMES = FALSE)
    df_matched$.title_sig <- ti_vec
    df_matched$.part_sig <- pt_vec

    n_split <- 0L

    # Process only multi-member clusters; split into part-marker-compatible
    # sub-clusters via complete linkage on the 0/1 dissimilarity below.
    cl_sizes <- table(df_matched$cluster_id)
    multi_ids <- names(cl_sizes)[cl_sizes > 1]

    for (cid in multi_ids) {
      rows <- which(df_matched$cluster_id == cid)
      ti <- df_matched$.title_sig[rows]
      pt <- df_matched$.part_sig[rows]
      m <- length(rows)
      # Need at least two members carrying a title to consider splitting.
      if (sum(!is.na(ti)) < 2) {
        next
      }

      # Pairwise 0/1 dissimilarity: two members are INCOMPATIBLE (distance 1)
      # only when they carry DIFFERENT series part markers (e.g. "Part I" vs
      # "Part II"). This is the single title signal that distinguishes genuine
      # distinct works without being confounded by title typos (a character
      # typo virtually never alters a short roman-numeral/part token, and an
      # original and its perturbed variant always share the same marker), so it
      # is safe for the typo-robustness the pipeline is designed for. Title-less
      # members and same-marker members impose no constraint. Complete linkage
      # guarantees that no incompatible pair ever shares a sub-cluster.
      D <- matrix(0, m, m)
      for (a in 1:(m - 1)) {
        for (b in (a + 1):m) {
          if (!is.na(pt[a]) && !is.na(pt[b]) && pt[a] != pt[b]) {
            D[a, b] <- 1
            D[b, a] <- 1
          }
        }
      }

      if (any(D == 1)) {
        hc <- hclust(as.dist(D), method = "complete")
        comp <- cutree(hc, h = 0.5)
        if (length(unique(comp)) > 1) {
          # Keep the largest sub-cluster on the original id; suffix the rest.
          ord <- names(sort(table(comp), decreasing = TRUE))
          new_ids <- df_matched$cluster_id[rows]
          for (k in seq_along(ord)) {
            if (k > 1) {
              new_ids[comp == as.integer(ord[k])] <- paste0(cid, "_T", k)
            }
          }
          df_matched$cluster_id[rows] <- new_ids
          n_split <- n_split + (length(ord) - 1L)
        }
      }
    }

    df_matched$.title_sig <- NULL
    df_matched$.part_sig <- NULL
    cat("  Title-guard split operations:", n_split, "\n")
    cat(
      "  Clusters after purification:",
      n_distinct(df_matched$cluster_id),
      "\n"
    )
  }

  cat("Phase 5: Selecting canonical representatives...\n")

  result <- df_matched %>%
    # Count frequency of each variant within its cluster
    group_by(cluster_id, CR_original) %>%
    mutate(variant_freq = n()) %>%
    ungroup() %>%
    group_by(cluster_id) %>%
    mutate(
      n_cluster = n(),
      # Scoring: frequency first, then completeness as tiebreaker
      completeness_score = variant_freq * 1000 +
        (!is.na(doi)) * 100 +
        (!is.na(volume)) * 10 +
        (!is.na(pages)) * 5 +
        nchar(CR_original) * 0.01,
      CR_canonical = CR_original[which.max(completeness_score)][1]
    ) %>%
    select(-variant_freq) %>%
    ungroup() %>%
    arrange(desc(n_cluster), cluster_id) %>%
    select(
      CR_original,
      CR_canonical,
      cluster_id,
      n_cluster,
      format,
      first_author,
      year,
      journal_iso4 = journal, # Rename for clarity
      journal_original, # Keep original
      volume,
      doi,
      blocking_key
    )

  cat(
    "Completed! Found",
    length(unique(result$cluster_id)),
    "unique clusters from",
    nrow(result),
    "valid citations.\n"
  )
  cat("  Clusters with >1 citation:", sum(result$n_cluster > 1), "\n")
  cat(
    "  Total variants found:",
    sum(result$n_cluster) - length(unique(result$cluster_id)),
    "\n"
  )

  # Add back filtered citations with basic feature extraction
  filtered_citations <- tibble(
    CR_original = CR_vector[!CR_vector %in% result$CR_original]
  ) %>%
    dplyr::filter(CR_original != "", !is.na(CR_original))

  if (nrow(filtered_citations) > 0) {
    # Try to extract basic features even for filtered citations
    filtered_features <- lapply(filtered_citations$CR_original, function(x) {
      tryCatch(
        {
          # Basic extraction without full validation
          first_author <- str_extract(x, "^[A-Z][A-Z\\s-\\.]+(?=,)") %>%
            str_remove_all("\\.$") %>%
            str_trim()

          year <- str_extract(x, "\\(?\\d{4}\\)?") %>%
            str_remove_all("[()]")

          # Detect format to extract journal
          if (str_detect(x, "\\(\\d{4}\\)\\s+[A-Z]")) {
            # Scopus format
            journal <- str_extract(
              x,
              "(?<=\\d{4}\\)\\s)([A-Z][A-Z\\s&\\.-]+?)(?=,)"
            )
          } else if (str_detect(x, ",\\s*\\d{4},\\s*[A-Z]")) {
            # WoS format
            journal <- str_extract(x, "(?<=\\d{4},\\s)([A-Z][A-Z\\s&-]+?)(?=,)")
          } else {
            journal <- NA_character_
          }

          list(first_author = first_author, year = year, journal = journal)
        },
        error = function(e) {
          list(
            first_author = NA_character_,
            year = NA_character_,
            journal = NA_character_
          )
        }
      )
    })

    filtered_citations <- filtered_citations %>%
      mutate(
        CR_canonical = CR_original,
        cluster_id = paste0("FILTERED_", row_number()),
        n_cluster = 1L,
        format = NA_character_,
        first_author = sapply(filtered_features, function(x) {
          x$first_author %||% NA
        }),
        year = sapply(filtered_features, function(x) x$year %||% NA),
        journal_iso4 = NA_character_, # No ISO4 normalization for filtered
        journal_original = sapply(filtered_features, function(x) {
          x$journal %||% NA
        }),
        volume = NA_character_,
        doi = NA_character_,
        blocking_key = "FILTERED"
      )

    result <- bind_rows(result, filtered_citations)
    cat(
      "Added",
      nrow(filtered_citations),
      "filtered citations as separate entries\n"
    )
  }

  return(result)
}


#' Apply reference normalization to bibliometrix data frame
#'
#' This is a convenience wrapper function that applies \code{\link{normalize_citations}}
#' to a bibliometrix data frame (typically loaded with \code{\link{convert2df}}). It
#' extracts citations from the CR field, performs normalization and matching, and
#' returns comprehensive results including per-paper citation lists and summary statistics.
#'
#' The function automatically handles the new Scopus citation format (where the year
#' appears at the end in parentheses) by converting it to the classic format before
#' processing.
#'
#' @param M A bibliometrix data frame, typically created by \code{\link{convert2df}}.
#'   Must contain the columns:
#'   \itemize{
#'     \item \code{SR}: Short reference identifier for each document
#'     \item \code{CR}: Cited references field (citations separated by semicolons)
#'     \item \code{DB}: (Optional) Database source identifier for format detection
#'   }
#' @param threshold Numeric value between 0 and 1 indicating the similarity threshold
#'   for matching citations. Default is 0.90. See \code{\link{normalize_citations}}
#'   for details on selecting appropriate thresholds.
#' @param method String distance method to use for fuzzy matching. Options include:
#'   \itemize{
#'     \item "jw" (default): Jaro-Winkler distance, optimized for bibliographic strings
#'     \item "lv": Levenshtein distance
#'     \item "osa": Optimal String Alignment distance
#'     \item "lcs": Longest Common Subsequence distance
#'     \item Other methods supported by \code{\link[stringdist]{stringdistmatrix}}
#'   }
#' @param min_chars Minimum characters for valid citations (default: 20).
#' @param max_block_size Integer. Blocks with at least this many unique normalized
#'   strings skip within-block fuzzy matching and fall back to exact matching only
#'   (default: 100). Passed to \code{\link{normalize_citations}}.
#' @param use_iso4 Logical. Enable ISO 4 / LTWA journal normalization (default \code{TRUE}).
#' @param use_doi Logical. Enable DOI-based exact matching (default \code{TRUE}).
#' @param use_exact Logical. Enable exact normalized-string and punctuation-invariant
#'   matching (default \code{TRUE}).
#' @param fuzzy Logical. Enable within-block (WoS deterministic and Scopus fuzzy)
#'   matching (default \code{TRUE}).
#' @param use_postproc Logical. Enable Phase 4.5 metadata-based post-processing
#'   merge (default \code{TRUE}). These five logical switches are intended primarily
#'   for ablation analyses and are passed through to \code{\link{normalize_citations}}.
#' @param title_guard Logical. Enable the optional Phase 4.6 series part-marker
#'   cluster purification that splits over-merged multi-part works (default
#'   \code{FALSE}). Passed through to \code{\link{normalize_citations}}.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Splits the CR field by semicolons to extract individual citations
#'   \item Detects and converts new Scopus format citations to classic format
#'   \item Trims whitespace from each citation
#'   \item Applies \code{\link{normalize_citations}} to identify duplicate citations
#'   \item Links normalized citations back to source documents (SR)
#'   \item Generates summary statistics and reconstructs normalized CR fields
#' }
#'
#' The normalized CR field can be used to replace the original CR field in subsequent
#' bibliometric analyses, ensuring that citation counts and network analyses are not
#' inflated by duplicate citations with minor formatting differences.
#'
#' @return A list with four elements:
#' \describe{
#'   \item{full_data}{A data frame with columns:
#'     \itemize{
#'       \item \code{SR}: Source document identifier
#'       \item \code{CR}: Original citation string
#'       \item \code{CR_canonical}: Canonical (normalized) citation
#'       \item \code{cluster_id}: Unique cluster identifier
#'       \item \code{n_cluster}: Size of the citation cluster
#'       \item \code{first_author}, \code{year}, \code{journal}, \code{volume}: Extracted metadata
#'     }
#'   }
#'   \item{summary}{A data frame summarizing citation frequencies with columns:
#'     \itemize{
#'       \item \code{CR_canonical}: The canonical citation for each cluster
#'       \item \code{n}: Total number of times this work was cited
#'       \item \code{n_variants}: Number of different formatting variants found
#'       \item \code{variants_example}: Sample of variant formats (up to 3 examples)
#'     }
#'     Sorted by citation frequency (n) in descending order.
#'   }
#'   \item{matched_citations}{Complete output from \code{\link{normalize_citations}},
#'     useful for detailed analysis of the matching process.}
#'   \item{CR_normalized}{A data frame with columns:
#'     \itemize{
#'       \item \code{SR}: Source document identifier
#'       \item \code{CR}: Reconstructed CR field with normalized citations (semicolon-separated)
#'       \item \code{n_references}: Number of unique references after normalization
#'     }
#'     This can be merged back with M to replace the original CR field.
#'   }
#' }
#'
#' @examples
#' \dontrun{
#' # Load bibliometric data
#' file <- "https://www.bibliometrix.org/datasets/savedrecs.txt"
#' M <- convert2df(file, dbsource = "wos", format = "plaintext")
#'
#' # Apply reference normalization
#' results <- applyReferenceMatching(M, threshold = 0.85)
#'
#' # View top cited works (after normalization)
#' head(results$summary, 20)
#'
#' # See how many variants were found for the top citation
#' top_citation <- results$summary$CR_canonical[1]
#' variants <- subset(results$full_data, CR_canonical == top_citation)
#' unique(variants$CR)
#'
#' # Replace original CR with normalized CR in the data frame
#' M_normalized <- M %>%
#'   rename(CR_orig = CR) %>%
#'   left_join(results$CR_normalized, by = "SR")
#'
#' # Compare citation counts before and after normalization
#' original_citations <- strsplit(M$CR, ";") %>%
#'   unlist() %>%
#'   trimws() %>%
#'   table() %>%
#'   length()
#'
#' normalized_citations <- nrow(results$summary)
#'
#' cat("Original unique citations:", original_citations, "\n")
#' cat("After normalization:", normalized_citations, "\n")
#' cat("Duplicates found:", original_citations - normalized_citations, "\n")
#'
#' # Use normalized data for further analysis
#' CR_analysis <- citations(M_normalized, field = "article", sep = ";")
#' }
#'
#' @seealso
#' \code{\link{normalize_citations}} for the underlying normalization algorithm
#' \code{\link{citations}} for citation analysis
#' \code{\link{localCitations}} for local citation analysis
#'
#' @references
#' Aria, M. & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive
#' science mapping analysis. Journal of Informetrics, 11(4), 959-975.
#'
#' @export

applyReferenceMatching <- function(
  M,
  threshold = 0.90,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
) {
  # Extract citations
  CR <- strsplit(M$CR, ";")
  CR_df <- tibble(
    SR = rep(M$SR, lengths(CR)),
    CR = trimws(unlist(CR))
  )

  # === NEW: Pre-processing for new Scopus format ===
  if ("DB" %in% names(M)) {
    # Identify citations from Scopus
    is_scopus <- M$DB == "SCOPUS"
    scopus_sr <- M$SR[is_scopus]

    if (length(scopus_sr) > 0) {
      # Convert only citations from Scopus database
      CR_df <- CR_df %>%
        mutate(
          CR = if_else(
            SR %in% scopus_sr,
            sapply(CR, convert_scopus_new_to_classic, USE.NAMES = FALSE),
            CR
          )
        )

      # Report conversions
      n_scopus_new <- sum(
        sapply(CR_df$CR[CR_df$SR %in% scopus_sr], function(x) {
          # Count how many were actually converted (now show as "scopus" format)
          str_detect(x, "\\(\\d{4}\\)\\s+[A-Z]")
        })
      )

      if (n_scopus_new > 0) {
        cat(
          "Pre-processing: Converted",
          n_scopus_new,
          "citations from new Scopus format to classic format\n"
        )
      }
    }
  }

  # Apply normalization (rest remains unchanged)
  cat("\n=== CITATION NORMALIZATION ===\n")
  matched <- normalize_citations(
    CR_df$CR,
    threshold = threshold,
    method = method,
    min_chars = min_chars,
    max_block_size = max_block_size,
    use_iso4 = use_iso4,
    use_doi = use_doi,
    use_exact = use_exact,
    fuzzy = fuzzy,
    use_postproc = use_postproc,
    title_guard = title_guard
  )

  # Join with SR
  result <- CR_df %>%
    left_join(matched, by = c("CR" = "CR_original"))

  # Remove invalid entries: filtered citations and short canonical forms
  result <- result %>%
    dplyr::filter(
      !is.na(cluster_id),
      !grepl("^FILTERED_", cluster_id),
      nchar(CR_canonical) >= min_chars
    )

  # Reference citation counts
  citation_count <- result %>% distinct() %>% count(CR_canonical, sort = TRUE)

  # Create summary
  summary <- result %>%
    group_by(CR_canonical, cluster_id) %>%
    summarise(
      n_variants = n_distinct(CR),
      format = first(format),
      first_author = first(first_author),
      year = first(year),
      journal_iso4 = first(journal_iso4), # Add ISO4 journal
      journal_original = first(journal_original), # Add original journal
      variants_example = paste(head(unique(CR), 3), collapse = " | "),
      .groups = "drop"
    ) %>%
    ungroup() %>%
    left_join(citation_count, by = "CR_canonical") %>%
    arrange(desc(n))

  # Reconstruct CR field
  cat("Reconstructing CR field for each paper...\n")
  CR_by_paper <- result %>%
    group_by(SR) %>%
    summarise(
      CR = paste(unique(CR_canonical), collapse = "; "),
      n_references = n_distinct(CR_canonical),
      .groups = "drop"
    )

  # Summary statistics
  cat("\n=== SUMMARY STATISTICS ===\n")
  cat("Total citations processed:", nrow(CR_df), "\n")
  cat("Valid citations analyzed:", length(unique(result$CR)), "\n")
  cat("Unique works identified:", length(unique(result$CR_canonical)), "\n")
  cat("Citations with variants (n>1):", sum(summary$n_variants > 1), "\n")
  cat(
    "Duplicate citations removed:",
    length(unique(result$CR)) - length(unique(result$CR_canonical)),
    "\n"
  )

  return(list(
    full_data = result %>% distinct(),
    summary = summary,
    matched_citations = matched,
    CR_normalized = CR_by_paper
  ))
}


#' @title applyCitationMatching (Deprecated)
#'
#' @description
#' \strong{Deprecated} in bibliometrix 6.0. This function has been renamed to
#' \code{\link{applyReferenceMatching}}. Please use \code{applyReferenceMatching()} instead.
#'
#' @inheritParams applyReferenceMatching
#'
#' @return Same as \code{\link{applyReferenceMatching}}.
#'
#' @export
applyCitationMatching <- function(
  M,
  threshold = 0.90,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
) {
  warning(
    "applyCitationMatching() is deprecated as of bibliometrix 6.0. ",
    "Please use applyReferenceMatching() instead.",
    call. = FALSE
  )
  applyReferenceMatching(
    M = M,
    threshold = threshold,
    method = method,
    min_chars = min_chars,
    max_block_size = max_block_size,
    use_iso4 = use_iso4,
    use_doi = use_doi,
    use_exact = use_exact,
    fuzzy = fuzzy,
    use_postproc = use_postproc,
    title_guard = title_guard
  )
}

#' ID and DE keyword associations
#'
#' It associates authors' keywords to keywords plus.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param sep is the field separator character. This character separates keywords in each string of ID and DE columns of the bibliographic data frame. The default is \code{sep = ";"}.
#' @param excludeKW is character vector. It contains authors' keywords to exclude from the analysis.
#' @param n is a integer. It indicates the number of authors' keywords to associate to each keyword plus. The default is \code{n = 10}.
#' @return an object of \code{class} "list".
#'
#'
#'
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' KWlist <- keywordAssoc(scientometrics, sep = ";", n = 10, excludeKW = NA)
#'
#' # list of first 10 Keywords plus
#' names(KWlist)
#'
#' # list of first 10 authors' keywords associated to the first Keyword plus
#' KWlist[[1]][1:10]
#'
#' @seealso \code{\link{convert2df}} to import and convert a WoS or SCOPUS Export file in a bibliographic data frame.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export
keywordAssoc <- function(M, sep = ";", n = 10, excludeKW = NA) {
  excludeKW <- toupper(excludeKW)

  WDE <- cocMatrix(M, Field = "DE", type = "sparse", sep = sep)
  WID <- cocMatrix(M, Field = "ID", type = "sparse", sep = sep)

  NetMatrix <- Matrix::crossprod(WID, WDE)
  if (!is.na(excludeKW)) {
    NetMatrix <- NetMatrix[!(row.names(NetMatrix) %in% excludeKW), !(colnames(NetMatrix) %in% excludeKW)]
  }
  NetMatrix <- NetMatrix[!is.na(row.names(NetMatrix)), !is.na(colnames(NetMatrix))]
  NetMatrix <- NetMatrix[nchar(row.names(NetMatrix)) > 0, nchar(colnames(NetMatrix)) > 0]
  rS <- Matrix::rowSums(NetMatrix)

  NetDegree <- sort(rS, decreasing = TRUE)[n]
  NET <- NetMatrix[rS >= NetDegree, Matrix::colSums(NetMatrix) > 1]
  KW <- apply(NET, 1, function(x) {
    i <- sort(x, decreasing = TRUE)[n]
    x <- sort(x[x >= i], decreasing = TRUE)
    return(x)
  })

  return(KW)
}

#' Yearly occurrences of top keywords/terms
#'
#' It calculates yearly occurrences of top keywords/terms.
#'
#' @param M is a data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.
#' @param Tag is a character object. It indicates one of the keyword field tags of the
#'   standard ISI WoS Field Tag codify (ID, DE, KW_Merged) or a field tag created by \code{\link{termExtraction}} function (TI_TM, AB_TM, etc.).
#' @param sep is the field separator character. This character separates strings in each keyword column of the data frame. The default is \code{sep = ";"}.
#' @param top is a numeric. It indicates the number of top keywords to analyze. The default value is 10.
#' @param cdf is a logical. If TRUE, the function calculates the cumulative occurrences distribution.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @return an object of class \code{data.frame}
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#' topKW <- KeywordGrowth(scientometrics, Tag = "ID", sep = ";", top = 5, cdf = TRUE)
#' topKW
#'
#' # Plotting results
#' \dontrun{
#' install.packages("reshape2")
#' library(reshape2)
#' library(ggplot2)
#' DF <- melt(topKW, id = "Year")
#' ggplot(DF, aes(Year, value, group = variable, color = variable)) + geom_line
#' }
#'
#' @export
KeywordGrowth <- function(M, Tag = "ID", sep = ";", top = 10, cdf = TRUE, remove.terms = NULL, synonyms = NULL) {
  i <- which(names(M) == Tag)
  PY <- as.numeric(M$PY)
  Tab <- lapply(strsplit(as.character(M[, i]), sep),
                function(x) unique(trimws(x)))
  Y <- rep(PY, lengths(Tab))
  A <- data.frame(Tab = unlist(Tab), Y = Y)
  A <- A[A$Tab != "", ]
  A <- A[!is.na(A$Y), ]

  ### remove terms
  terms <- data.frame(Tab = toupper(remove.terms))
  A <- anti_join(A, terms)
  # end of block

  ### Merge synonyms in the vector synonyms
  if (length(synonyms) > 0 & is.character(synonyms)) {
    s <- strsplit(toupper(synonyms), ";")
    snew <- trimws(unlist(lapply(s, function(l) l[1])))
    sold <- (lapply(s, function(l) trimws(l[-1])))
    for (i in 1:length(s)) {
      A <- A %>%
        mutate(
          # Tab = str_replace_all(Tab, paste(sold[[i]], collapse="|",sep=""),snew[i])
          # Tab= str_replace_all(Tab, str_replace_all(str_replace_all(paste(sold[[i]], collapse="|",sep=""),"\\(","\\\\("),"\\)","\\\\)"),snew[i]),
          Tab = stringi::stri_replace_all_regex(Tab, stringi::stri_replace_all_regex(stringi::stri_replace_all_regex(paste(sold[[i]], collapse = "|", sep = ""), "\\(", "\\\\("), "\\)", "\\\\)"), snew[i])
        )
    }
  }
  # end of block

  Ymin <- min(A$Y)
  Ymax <- max(A$Y)
  Year <- Ymin:Ymax
  if (top==Inf) top <- length(unique(A$Tab))
  Tab <- names(sort(table(A$Tab), decreasing = TRUE))[1:top]

  words <- matrix(0, length(Year), top + 1)
  words <- data.frame(words)
  names(words) <- c("Year", Tab)
  words[, 1] <- Year
  for (j in 1:length(Tab)) {
    word <- (table(A[A$Tab %in% Tab[j], 2]))
    words[, j + 1] <- trim.years(word, Year, cdf)
  }
  return(words)
}

trim.years <- function(w, Year, cdf) {
  Y <- as.numeric(names(w))
  W <- matrix(0, length(Year), 1)

  for (i in 1:length(Year)) {
    if (length(Y) > 0 && Y[1] == Year[i]) {
      W[i, 1] <- w[1]
      Y <- Y[-1]
      w <- w[-1]
    }
  }
  if (isTRUE(cdf)) W <- cumsum(W)
  names(W) <- Year
  W <- data.frame(W)
  return(W)
}

utils::globalVariables(c(
  "PY", "LCS", "TC", "SR", "NGCS", "NLCS", "Author",
  "year", "SO"
))
#' Calculate the normalized citation score metric
#'
#' It calculates the normalized citation score for documents, authors and sources using both global and local citations.
#'
#' The document Normalized Citation Score (NCS) of a document is calculated by dividing the actual count of citing items by the expected
#' citation rate for documents with the same year of publication.
#'
#' The MNCS of a set of documents, for example the collected works of an individual, or published on a journal, is the average of the NCS values for all the documents in the set.
#'
#' The NGCS is the NCS calculated using the global citations (total citations that a document received considering the whole bibliographic database).
#'
#' The NLCS is the NCS calculated using the local citations (total citations that a document received from a set of documents included in the same collection).
#'
#'
#'
#' @param M is a bibliographic data frame obtained by \code{\link{convert2df}} function.
#' @param field is a character. It indicates the unit of analysis on which calculate the NCS. It can be equal to \code{field = c("documents", "authors", "sources")}. Default is \code{field = "documents"}.
#' @param impact.measure is a character. It indicates the impact measure used to rank cluster elements (documents, authors or sources).
#' It can be \code{impact.measure = c("local", "global")}.\\
#' With \code{impact.measure = "local"}, \link{normalizeCitationScore} calculates elements impact using the Normalized Local Citation Score while
#' using \code{impact.measure = "global"}, the function uses the Normalized Global Citation Score to measure elements impact.
#' @return a dataframe.
#'
#'
#'
#' @examples
#' \dontrun{
#' data(management, package = "bibliometrixData")
#' NCS <- normalizeCitationScore(management, field = "authors", impact.measure = "local")
#' }
#'
#' @export
normalizeCitationScore <- function(M, field = "documents", impact.measure = "local") {
  if (!(field %in% c("documents", "authors", "sources"))) {
    cat('\nfield argument is incorrect.\n\nPlease select one of the following choices: "documents", "authors", "sources"\n\n')
    return(NA)
  }
  M$TC <- as.numeric(M$TC)
  M$PY <- as.numeric(M$PY)

  if (impact.measure == "local") {
    log <- capture.output(M <- localCitations(M, fast.search = FALSE, sep = ";")$M)
  } else {
    M$LCS <- 0
  }

  M <- M %>%
    group_by(PY) %>%
    mutate(
      LCS = replace(LCS, LCS == 0, 1),
      NGCS = TC / mean(TC, na.rm = TRUE),
      NLCS = LCS / mean(LCS, na.rm = TRUE)
    ) %>%
    ungroup() %>%
    as.data.frame()


  switch(field,
    documents = {
      #### Documents Impact by Normalized Local Citations ####
      NCS <- M %>%
        select(SR, PY, NGCS, NLCS, TC, LCS) %>%
        rename(
          MNGCS = NGCS,
          MNLCS = NLCS,
          LC = LCS
        ) %>%
        rename(documents = SR) %>%
        as.data.frame()
    },
    authors = {
      #### Authors Impact by Normalized Local Citations ####
      AU <- names(tableTag(M, "AU"))
      df <- data.frame("Author" = "NA", "SR" = NA, "year" = NA, "TC" = NA, "LCS" = NA, "NGCS" = NA, "NLCS" = NA)

      if (!("DI" %in% names(M))) {
        M$DI <- "NA"
      }
      for (i in 1:length(AU)) {
        ind <- which(regexpr(AU[i], M$AU) > -1)
        dfAU <- data.frame(
          "Author" = rep(AU[i], length(ind)), "SR" = M$PY[ind], "year" = M$PY[ind],
          "TC" = M$TC[ind], "LCS" = M$LCS[ind], "NGCS" = M$NGCS[ind],
          "NLCS" = M$NLCS[ind]
        )
        df <- rbind(df, dfAU)
      }
      df <- df[-1, ]

      NCS <- df %>%
        group_by(Author) %>%
        summarize(
          NP = length(year),
          MNGCS = mean(NGCS, na.rm = TRUE),
          MNLCS = mean(NLCS, na.rm = TRUE),
          TC = mean(TC, na.rm = TRUE),
          LC = mean(LCS, na.rm = TRUE)
        ) %>%
        rename(authors = Author) %>%
        as.data.frame()
    },
    sources = {
      #### Source Impact by Normalized Local Citations ####
      NCS <- M %>%
        group_by(SO) %>%
        summarize(
          NP = length(PY),
          MNGCS = mean(NGCS, na.rm = TRUE),
          MNLCS = mean(NLCS, na.rm = TRUE),
          TC = mean(TC, na.rm = TRUE),
          LC = mean(LCS, na.rm = TRUE)
        ) %>%
        rename(sources = SO) %>%
        as.data.frame()
    }
  )

  if (impact.measure == "global") {
    NCS <- NCS[, -c(4, 6)]
  } else {
    NCS$MNLCS[is.na(NCS$MNLCS)] <- 0
  }

  return(NCS)
}

utils::globalVariables(c("X1", "X2", "tag", "orig"))

csvScopus2df <- function(file) {
  options(readr.num_columns = 0)

  ## import all files in a single data frame
  for (i in 1:length(file)) {
    D <- read_csv(file[i],
      na = character(), quote = '"', trim_ws = FALSE, progress = show_progress(),
      col_types = cols(.default = col_character())
    ) %>% # Introduced to remove cols parsing errors
      # mutate(across(!where(is.numeric), as.character)) %>%   # not yet necessary with the inclusion of previuos line
      mutate(across(where(is.character), function(x) tidyr::replace_na(x, ""))) %>% as.data.frame()

    if (i > 1) {
      l <- intersect(l, names(D))
      DATA <- rbind(DATA[l], D[l])
    } else {
      l <- names(D)
      DATA <- D
    }
  }

  ## Post-Processing

  # column re-labelling
  DATA <- labelling(DATA)

  # Authors' names cleaning (surname and initials)
  DATA$AU <- gsub("\\.", "", DATA$AU)
  #DATA$AU <- gsub(",", ";", DATA$AU)
  DATA$AU <- gsub(",", "", DATA$AU)

  ### store raw affiliation format to extract link among authors and affiliations
  DATA$C1raw <- DATA$C1
  ###

  # Affiliation
  if (!("C1" %in% names(DATA))) {
    DATA$C1 <- NA
  } else {
    DATA$C1 <- unlist(lapply(strsplit(DATA$C1, ";"), function(l) {
      l <- paste(gsub(".*\\., ", "", l), collapse = ";", sep = "")
    }))
  }
  # Iso Source Titles
  if ("JI" %in% names(DATA)) {
    DATA$J9 <- gsub("\\.", "", DATA$JI)
  } else {
    DATA$J9 <- DATA$JI <- sapply(DATA$SO, AbbrevTitle, USE.NAMES = FALSE)
  }

  # Fix new Scopus CSV format: replace ";" between authors with ","
  # so that ";" remains only as separator between different references
  if ("CR" %in% names(DATA)) {
    DATA$CR <- fix_scopus_author_separator(DATA$CR)
  }

  DI <- DATA$DI
  URL <- DATA$URL
  AB <- DATA$AB
  TI <- DATA$TI
  DE <- DATA$DE
  DATA <- data.frame(lapply(DATA, toupper))
  DATA$AB_raw <- AB
  DATA$TI_raw <- TI
  DATA$DE_raw <- DE
  DATA$DI <- DI
  DATA$URL <- URL
  return(DATA)
}


#' Fix author separators in new Scopus CSV reference format
#'
#' In the new Scopus CSV format, both author separators and reference
#' separators use ";". This function replaces ";" between authors within
#' each reference with "," so that ";" only separates different references.
#'
#' @param cr_vec Character vector of CR strings
#' @return Character vector with fixed separators
#' @noRd
fix_scopus_author_separator <- function(cr_vec) {
  is_new_format <- grepl("\\(\\d{4}\\)", cr_vec)

  cr_vec[is_new_format] <- vapply(
    cr_vec[is_new_format],
    function(cr) {
      if (is.na(cr) || nchar(trimws(cr)) == 0) return(cr)

      # Split on reference boundaries: (YYYY) followed by ;
      cr_split <- gsub("\\((\\d{4})\\)\\s*;\\s*", "(\\1)|||", cr)
      refs <- unlist(strsplit(cr_split, "\\|\\|\\|"))
      refs <- trimws(refs)

      # Within each reference, replace ";" (author separator) with ","
      refs <- vapply(refs, function(ref) {
        gsub(";", ",", ref)
      }, character(1), USE.NAMES = FALSE)

      # Rejoin references with ";"
      paste(refs, collapse = "; ")
    },
    character(1),
    USE.NAMES = FALSE
  )

  cr_vec
}


labelling <- function(DATA) {
  ## column re-labelling

  df_tag <- data.frame(
    rbind(
      c("Abbreviated Source Title", "JI"),
      c("Affiliations", "C1"),
      c("Authors with affiliations", "C1_raw"),
      c("Author Addresses", "C1_raw"),
      c("Authors", "AU"),
      c("Author Names", "AU"),
      c("Author full names", "AF"),
      c("Source title", "SO"),
      c("Titles", "TI"),
      c("Title", "TI"),
      c("Publication Year", "PY"),
      c("Year", "PY"),
      c("Volume", "VL"),
      c("Issue", "IS"),
      c("Page count", "PP"),
      c("Cited by", "TC"),
      c("DOI", "DI"),
      c("Link", "URL"),
      c("Abstract", "AB"),
      c("Author Keywords", "DE"),
      c("Indexed Keywords", "ID"),
      c("Index Keywords", "ID"),
      c("Funding Details", "FU"),
      c("Funding Texts", "FX"),
      c("Funding Text 1", "FX"),
      c("References", "CR"),
      c("Correspondence Address", "RP"),
      c("Publisher", "PU"),
      c("Open Access", "OA"),
      c("Language of Original Document", "LA"),
      c("Document Type", "DT"),
      c("Source", "DB"),
      c("EID", "UT")
    )
  ) %>%
    rename(
      orig = X1,
      tag = X2
    )

  label <- data.frame(orig = names(DATA)) %>%
    left_join(df_tag, by = "orig") %>%
    mutate(tag = ifelse(is.na(tag), orig, tag))

  names(DATA) <- label$tag
  
  if (!"C1" %in% names(DATA)) {
    if ("C1_raw" %in% names(DATA)) {
      DATA$C1 <- DATA$C1_raw
    } else {
      DATA$C1 <- NA
    }
  }

  return(DATA)
}

utils::globalVariables(c("item", "SR"))
#' Bibliographic bipartite network matrices
#'
#' \code{cocMatrix} computes occurrences between elements of a Tag Field from a bibliographic data frame. Manuscript is the unit of analysis.
#'
#' This occurrence matrix represents a bipartite network which can be transformed into a collection of bibliographic
#' networks such as coupling, co-citation, etc..
#'
#' The function follows the approach proposed by Batagelj & Cerinsek (2013) and Aria & cuccurullo (2017).\cr\cr
#'
#' References:\cr
#' Batagelj, V., & Cerinsek, M. (2013). On bibliographic networks. Scientometrics, 96(3), 845-864.\cr
#' Aria, M., & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive science mapping analysis. Journal of Informetrics, 11(4), 959-975.\cr
#'
#' @param M is a data frame obtained by the converting function
#'   \code{\link{convert2df}}. It is a data matrix with cases corresponding to
#'   articles and variables to Field Tag in the original WoS or SCOPUS file.
#' @param Field is a character object. It indicates one of the field tags of the
#'   standard ISI WoS Field Tag codify. Field can be equal to one of these tags:
#'   \tabular{lll}{ \code{AU}\tab   \tab Authors\cr \code{SO}\tab   \tab
#'   Publication Name (or Source)\cr \code{JI}\tab   \tab ISO Source
#'   Abbreviation\cr \code{DE}\tab   \tab Author Keywords\cr \code{ID}\tab
#'   \tab Keywords associated by WoS or SCOPUS database \cr 
#'   \code{KW_Merged}\tab    \tab All Keywords (merged by DE and ID) \cr
#'   \code{CR}\tab   \tab Cited References}
#'
#'   for a complete list of filed tags see:
#'   \href{https://www.bibliometrix.org/documents/Field_Tags_bibliometrix.pdf}{Field Tags used in bibliometrix}\cr\cr
#'
#' @param type indicates the output format of co-occurrences: \tabular{lll}{
#'   \code{type = "matrix"} \tab   \tab produces an object of class
#'   \code{matrix}\cr \code{type = "sparse"} \tab   \tab produces an object of
#'   class \code{dgMatrix} of the package \code{Matrix}. "sparse"
#'   argument generates a compact representation of the matrix.}
#' @param n is an integer. It indicates the number of items to select. If \code{N = NULL}, all items are selected.
#' @param sep is the field separator character. This character separates strings in each
#' column of the data frame. The default is \code{sep = ";"}.
#' @param binary is a logical. If TRUE each cell contains a 0/1. if FALSE each cell contains the frequency.
#' @param short is a logical. If TRUE all items with frequency<2 are deleted to reduce the matrix size.
#' @param remove.terms is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.
#' @param synonyms is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.
#' @return a bipartite network matrix with cases corresponding to manuscripts and variables to the
#'   objects extracted from the Tag \code{Field}.
#'
#' @examples
#' # EXAMPLE 1: Articles x Authors occurrence matrix
#'
#' data(scientometrics, package = "bibliometrixData")
#' WA <- cocMatrix(scientometrics, Field = "AU", type = "sparse", sep = ";")
#'
#' # EXAMPLE 2: Articles x Cited References occurrence matrix
#'
#' # data(scientometrics, package = "bibliometrixData")
#'
#' # WCR <- cocMatrix(scientometrics, Field = "CR", type = "sparse", sep = ";")
#'
#' # EXAMPLE 3: Articles x Cited First Authors occurrence matrix
#'
#' # data(scientometrics, package = "bibliometrixData")
#' # scientometrics <- metaTagExtraction(scientometrics, Field = "CR_AU", sep = ";")
#' # WCR <- cocMatrix(scientometrics, Field = "CR_AU", type = "sparse", sep = ";")
#'
#' @seealso \code{\link{convert2df}} to import and convert an ISI or SCOPUS
#'   Export file in a data frame.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @export

cocMatrix <- function(M, Field = "AU", type = "sparse", n = NULL, sep = ";", binary = TRUE,
                      short = FALSE, remove.terms = NULL, synonyms = NULL) {
  #
  # The function creates occurrences data between Works and Field
  #
  # type indicates the output format of occurrences:
  #   "matrix" argument generates a W x Field sparse matrix
  #   "sparse" argument generates a compact representation of the matrix (using the package Matrix)
  #    it represents a compact representation of a occurrences matrix.
  # Field indicates the ISI Tag
  # if Field is AU -> WA (Works x Authors)
  # if Field is CR -> WR (Works x References)
  # if Field is DE -> WK (Works x Keywords)
  # etc.
  # crossprod <- Matrix::crossprod
  
  size <- dim(M)
  if (!"LABEL" %in% names(M)) row.names(M) <- M$SR
  RowNames <- row.names(M)

  if (Field == "KW_Merged") M <- M %>% mergeKeywords(force = FALSE)
  ### REMOVE TERMS AND MERGE SYNONYMS
  if (Field %in% c("ID", "DE","KW_Merged","TI", "TI_TM", "AB", "AB_TM")) {
    # Crete df with all terms

    Fi <- strsplit(M[, Field], sep)
    TERMS <- data.frame(item = trimws(unlist(Fi)), SR = rep(M$SR, lengths(Fi)))

    # Merge synonyms in the vector synonyms
    if (length(synonyms) > 0 & is.character(synonyms)) {
      s <- strsplit(toupper(synonyms), ";")
      snew <- trimws(unlist(lapply(s, function(l) l[1])))
      sold <- (lapply(s, function(l) {
        l <- trimws(l[-1])
      }))

      for (i in 1:length(s)) {
        TERMS$item[TERMS$item %in% unlist(sold[[i]])] <- snew[i]
      }
    }

    TERMS <- TERMS %>%
      anti_join(data.frame(item = trimws(toupper(remove.terms))), by = "item") %>%
      mutate(item = trimws(item))

    TERMS <- TERMS %>%
      group_by(SR) %>%
      summarize(item = paste(item, collapse = ";"))

    M <- M %>%
      left_join(TERMS, by = "SR")
    M[, Field] <- M$item
    row.names(M) <- RowNames
  }

  if (Field == "CR") {
    M$CR <- gsub("DOI;", "DOI ", as.character(M$CR))
  }

  if (Field %in% names(M)) {
    Fi <- strsplit(as.character(M[, Field]), sep)
  } else {
    return(print(paste("Field", Field, "is not a column name of input data frame")))
  }
  Fi <- lapply(Fi, trim.leading)
  if (Field == "CR") {
    Fi <- lapply(Fi, function(l) l <- l[nchar(l) > 10])
  } ## delete not congruent references

  ## Scelta dell'informazione contenuta in CR da utilizzare (Reference, Autore, Affiliation, ecc.)

  # vector of unique units
  allField <- unlist(Fi)
  allField <- allField[!is.na(allField)]

  if (Field == "CR") {
    ind <- which(substr(allField, 1, 1) != "(")
    S <- allField
    S[ind] <- gsub("\\).*", ")", allField[ind])
    S[-ind] <- substr(S[-ind], 1, 100)
    S <- gsub(",", " ", S)
    S <- gsub(";", " ", S)
    S <- reduceRefs(S)
    allField <- trimES(S)
    Fi <- lapply(Fi, function(l) {
      l <- gsub("\\).*", ")", l)
      l <- gsub(",", " ", l)
      l <- gsub(";", " ", l)
      l <- l[nchar(l) > 0]
      l <- reduceRefs(l)
      l <- trimES(l)
      return(l)
    })
  } else {
    # normalize reference names
    S <- gsub("\\,", ";", allField)
    S <- sub("\\;", ",", S)
    S <- sub("\\;", ",", S)
    S <- gsub("\\;.*", "", S)
    allField <- trimws(S)
    Fi <- lapply(Fi, function(l) {
      l <- gsub("\\,", ";", l)
      l <- sub("\\;", ",", l)
      l <- sub("\\;", ",", l)
      l <- gsub("\\;.*", "", l)
      l <- l[nchar(l) > 0]
      l <- trimws(l)
      return(l)
    })
  }
  tabField <- sort(table(allField), decreasing = TRUE)
  uniqueField <- names(tabField)
  # select n items
  if (!is.null(n)) {
    if ("NA" %in% uniqueField[1:n]) n <- n + 1
    uniqueField <- uniqueField[1:n]
  } else if (isTRUE(short)) {
    uniqueField <- names(tabField[tabField > 1]) # remove items with frequency<2
  }

  if (length(uniqueField) < 1) {
    print("Matrix is empty!!")
    return(NA)
  }

  # Build sparse matrix directly from (i, j, x) triplets — O(total_references)
  doc_ids <- rep(seq_along(Fi), lengths(Fi))
  terms <- unlist(Fi)
  col_ids <- match(terms, uniqueField)
  valid <- !is.na(col_ids)
  doc_ids <- doc_ids[valid]
  col_ids <- col_ids[valid]

  if (isTRUE(binary)) {
    # Remove duplicate (doc, term) pairs for binary counting
    dup <- duplicated(paste(doc_ids, col_ids, sep = "_"))
    WF <- Matrix::sparseMatrix(
      i = doc_ids[!dup], j = col_ids[!dup],
      x = 1, dims = c(size[1], length(uniqueField)),
      dimnames = list(rownames(M), uniqueField)
    )
  } else {
    # Full counting: aggregate duplicates by summing
    WF <- Matrix::sparseMatrix(
      i = doc_ids, j = col_ids,
      x = 1, dims = c(size[1], length(uniqueField)),
      dimnames = list(rownames(M), uniqueField)
    )
  }

  if (type == "matrix") {
    WF <- as.matrix(WF)
  }


  WF <- WF[, !is.na(uniqueField)]
  ind <- which(colnames(WF) == "NA")
  if (length(ind) > 0) {
    WF <- WF[, -ind]
  }

  return(WF)
}

reduceRefs <- function(A) {
  ind <- unlist(regexec("*V[0-9]", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  ind <- unlist(regexec("*DOI ", A))
  A[ind > -1] <- substr(A[ind > -1], 1, (ind[ind > -1] - 1))
  return(A)
}

#' Complete missing metadata via DOI lookup against Crossref and OpenAlex
#'
#' Given a bibliometrix collection produced by \code{\link{convert2df}}, this
#' function takes the subset of records that have a DOI but are missing one
#' or more of the analysis-relevant fields, queries the Crossref REST API
#' (\url{https://api.crossref.org/works}) and/or OpenAlex (via
#' \pkg{openalexR}) using the DOI as the lookup key, and fills the gaps
#' with the values returned by those sources. Existing non-empty values
#' are never overwritten.
#'
#' When both sources are enabled, OpenAlex runs first (broader coverage of
#' AB/CR/C1/TC) and Crossref then fills the residual gaps. If the input
#' collection was originally imported from OpenAlex (\code{M$DB[1] ==
#' "OPENALEX"}), the OpenAlex pass is automatically skipped because re-querying
#' it would not add information.
#'
#' The vacancy predicate matches the one used by \code{\link{missingData}}:
#' a cell is considered missing when it is \code{NA} or one of
#' \code{c("", "NA", "none", "NA,0000,NA")}.
#'
#' Crossref cannot supply author keywords (DE), Keywords Plus (ID), Web of
#' Science categories (WC), or citation counts (TC). OpenAlex covers TC,
#' AB, AU, C1, CR, DT, LA, PY, RP, SO, TI well; OpenAlex \code{keywords}
#' are AI-derived topic labels and not author keywords, so DE is off by
#' default. ID and WC are always skipped.
#'
#' @param M Bibliometrix data frame produced by \code{\link{convert2df}}.
#' @param sources Character vector of enrichment sources.
#'   Default \code{c("openalex", "crossref")}. Order is irrelevant; OpenAlex
#'   always runs before Crossref. \code{"openalex"} is skipped if
#'   \code{M$DB[1] == "OPENALEX"}.
#' @param fields Character vector of WoS-codified fields to attempt to fill.
#'   Default \code{c("AB","AU","C1","CR","DT","LA","PY","RP","SO","TC","TI")}.
#'   \code{TC} is filled only by OpenAlex.
#' @param email Optional contact email used as the Crossref polite-pool
#'   identifier and OpenAlex \code{mailto}. If \code{NULL}, the function
#'   falls back to the env var \code{BIBLIOMETRIX_EMAIL} or the persisted
#'   file \code{~/.biblio_openalex_email.txt}.
#' @param oa_apikey Optional OpenAlex API key. If \code{NULL}, the function
#'   reads \code{Sys.getenv("openalexR.apikey")} and falls back to
#'   \code{~/.biblio_openalex_apikey.txt}. The OpenAlex pass works without a
#'   key (lower rate limit).
#' @param batch_size Number of DOIs per Crossref batch request (default 20).
#'   OpenAlex uses a fixed batch size of 50 (the maximum that keeps URLs
#'   under length limits).
#' @param max_records Optional cap on the number of records to enrich
#'   (useful for previewing). Default \code{Inf}.
#' @param progress Optional callback \code{function(done, total, label)}
#'   invoked after each batch. Used by biblioshiny to drive a progress bar.
#' @param verbose Logical. Print progress messages to the console.
#'
#' @return A list with components:
#' \describe{
#'   \item{\code{M}}{The enriched collection (same class as the input).}
#'   \item{\code{report}}{Long-format \code{data.frame} with one row per
#'     (field, source) summarising attempts, fills, and failures.}
#'   \item{\code{before}}{The \code{mandatoryTags} table from
#'     \code{missingData(M)} before enrichment.}
#'   \item{\code{after}}{The \code{mandatoryTags} table from
#'     \code{missingData(M)} after enrichment.}
#' }
#' Provenance is attached to the returned collection as
#' \code{attr(M, "enrichment")}, a long-format \code{data.frame} with columns
#' \code{SR}, \code{field}, \code{source}, \code{timestamp}.
#'
#' @examples
#' \dontrun{
#' data(scientometrics, package = "bibliometrixData")
#' res <- completeMetadata(scientometrics, email = "you@example.com")
#' res$report
#' res$after
#' }
#'
#' @export
completeMetadata <- function(
  M,
  sources = c("openalex", "crossref"),
  fields = c("AB", "AU", "C1", "CR", "DT", "LA", "PY", "RP", "SO", "TC", "TI"),
  email = NULL,
  oa_apikey = NULL,
  batch_size = 20,
  max_records = Inf,
  progress = NULL,
  verbose = TRUE
) {
  stopifnot(is.data.frame(M))
  ## Source filtering
  sources <- tolower(unique(sources))
  unknown <- setdiff(sources, c("crossref", "openalex"))
  if (length(unknown) > 0) {
    stop("Unsupported sources: ", paste(unknown, collapse = ", "))
  }
  ## If the collection itself comes from OpenAlex, re-querying it makes no
  ## sense -- transparently drop the OpenAlex pass.
  if (
    "openalex" %in%
      sources &&
      "DB" %in% names(M) &&
      length(M$DB) > 0 &&
      !is.na(M$DB[1]) &&
      toupper(M$DB[1]) == "OPENALEX"
  ) {
    if (isTRUE(verbose)) {
      message(
        "[completeMetadata] Source DB is OPENALEX; skipping OpenAlex pass."
      )
    }
    sources <- setdiff(sources, "openalex")
  }

  ## Field filtering -- always-skip set
  always_skip <- c("ID", "WC")
  skipped <- intersect(fields, always_skip)
  if (length(skipped) > 0) {
    if (isTRUE(verbose)) {
      message(
        "[completeMetadata] No source provides: ",
        paste(skipped, collapse = ", "),
        ". Skipping."
      )
    }
    fields <- setdiff(fields, skipped)
  }
  ## TC requires OpenAlex; if OpenAlex is disabled, drop TC silently.
  if (!"openalex" %in% sources) {
    if ("TC" %in% fields && isTRUE(verbose)) {
      message("[completeMetadata] TC requires OpenAlex; skipping.")
    }
    fields <- setdiff(fields, "TC")
  }
  if (length(sources) == 0 || length(fields) == 0) {
    return(list(
      M = M,
      report = .empty_enrichment_report(),
      before = missingData(M)$mandatoryTags,
      after = missingData(M)$mandatoryTags
    ))
  }

  before <- missingData(M)$mandatoryTags

  ## TC normalization to match missingData()'s collection-level semantics.
  ## missingData() reports TC as 100% missing only when sum(M$TC) == 0;
  ## otherwise per-cell zeros are treated as legitimate (a real 0 citation
  ## count). To stay consistent, we treat TC = 0 as vacant for enrichment
  ## ONLY when the entire column sums to zero, by mapping those zeros to NA
  ## here so the standard vacancy predicate flags them. If even one row has
  ## TC > 0, all the zeros are left untouched (genuine zeros).
  if ("TC" %in% fields && "TC" %in% names(M)) {
    tc_sum <- suppressWarnings(sum(as.numeric(M$TC), na.rm = TRUE))
    if (!is.na(tc_sum) && tc_sum == 0) {
      tc_zeros <- !is.na(M$TC) & trimws(as.character(M$TC)) %in% c("0", "0.0")
      if (any(tc_zeros)) M$TC[tc_zeros] <- NA_character_
    }
  }

  ## Snapshot eligible rows: DOI present and at least one target field empty.
  doi_norm <- .normalize_doi(M$DI)
  target_present <- intersect(fields, names(M))
  if (length(target_present) == 0) {
    return(list(
      M = M,
      report = .empty_enrichment_report(),
      before = before,
      after = before
    ))
  }
  any_missing <- rep(FALSE, nrow(M))
  for (f in target_present) {
    any_missing <- any_missing | .is_vacant(M[[f]])
  }
  eligible_idx <- which(!is.na(doi_norm) & any_missing)
  if (length(eligible_idx) == 0) {
    if (isTRUE(verbose)) {
      message("[completeMetadata] No eligible records.")
    }
    return(list(
      M = M,
      report = .empty_enrichment_report(),
      before = before,
      after = before
    ))
  }
  if (length(eligible_idx) > max_records) {
    eligible_idx <- eligible_idx[seq_len(max_records)]
  }

  ## Resolve email and OpenAlex API key.
  email <- .resolve_email(email)
  oa_apikey <- .resolve_oa_apikey(oa_apikey)

  M_out <- M
  prov_all <- list()
  report_rows <- list()
  filled_total <- stats::setNames(integer(length(fields)), fields)

  ## ---- Pass 1: OpenAlex (preferred when available) ------------------------
  if ("openalex" %in% sources) {
    eligible_idx_oa <- .recompute_eligible(M_out, fields, eligible_idx)
    if (length(eligible_idx_oa) > 0) {
      oa_dois <- .normalize_doi(M_out$DI)[eligible_idx_oa]
      enriched_oa <- .enrich_from_openalex(
        oa_dois,
        fields,
        email,
        oa_apikey,
        progress = progress,
        verbose = verbose
      )
      merged_oa <- .merge_enrichment(
        M_out,
        enriched_oa,
        eligible_idx_oa,
        fields,
        source_label = "openalex"
      )
      M_out <- merged_oa$M
      if (!is.null(merged_oa$prov)) {
        prov_all[[length(prov_all) + 1]] <- merged_oa$prov
      }
      report_rows[[length(report_rows) + 1]] <- .build_enrichment_report(
        merged_oa$filled,
        attempted = length(eligible_idx_oa),
        source_label = "openalex",
        fail_count = enriched_oa$fail_count
      )
      filled_total <- filled_total + merged_oa$filled[names(filled_total)]
    }
  }

  ## ---- Pass 2: Crossref (fills residual gaps) -----------------------------
  if ("crossref" %in% sources) {
    cr_fields <- setdiff(fields, "TC") # Crossref cannot provide TC
    eligible_idx_cr <- .recompute_eligible(M_out, cr_fields, eligible_idx)
    if (length(eligible_idx_cr) > 0 && length(cr_fields) > 0) {
      cr_dois <- .normalize_doi(M_out$DI)[eligible_idx_cr]
      enriched_cr <- .enrich_from_crossref(
        cr_dois,
        cr_fields,
        email,
        batch_size = batch_size,
        progress = progress,
        verbose = verbose
      )
      merged_cr <- .merge_enrichment(
        M_out,
        enriched_cr,
        eligible_idx_cr,
        cr_fields,
        source_label = "crossref"
      )
      M_out <- merged_cr$M
      if (!is.null(merged_cr$prov)) {
        prov_all[[length(prov_all) + 1]] <- merged_cr$prov
      }
      report_rows[[length(report_rows) + 1]] <- .build_enrichment_report(
        merged_cr$filled,
        attempted = length(eligible_idx_cr),
        source_label = "crossref",
        fail_count = enriched_cr$fail_count
      )
      filled_total[names(merged_cr$filled)] <-
        filled_total[names(merged_cr$filled)] + merged_cr$filled
    }
  }

  ## Drop derived columns whose source field has been touched, so the user
  ## can rerun metaTagExtraction() to refresh them. Classic bibliometrix
  ## works on flat data frames, so no relational rebuild is needed.
  touched <- names(filled_total)[filled_total > 0]
  if ("C1" %in% touched || "AU" %in% touched) {
    for (col in c("AU_UN", "AU_CO", "AU1_UN", "AU1_CO")) {
      if (col %in% names(M_out)) M_out[[col]] <- NULL
    }
  }

  ## Attach concatenated provenance and project it onto a
  ## per-row M$ENRICH column so downstream consumers (e.g. biblioshiny's
  ## contents table) can see what was filled and from where without digging
  ## into the attribute.
  if (length(prov_all) > 0) {
    prov_df <- do.call(rbind, prov_all)
    attr(M_out, "enrichment") <- prov_df

    src_short <- c(openalex = "OA", crossref = "CR")
    tag_per_row <- paste0(
      prov_df$field,
      ":",
      ifelse(
        prov_df$source %in% names(src_short),
        src_short[prov_df$source],
        toupper(prov_df$source)
      )
    )
    enrich_per_sr <- tapply(tag_per_row, prov_df$SR, function(v) {
      paste(unique(v), collapse = "; ")
    })
    if ("SR" %in% names(M_out)) {
      M_out$ENRICH <- unname(enrich_per_sr[as.character(M_out$SR)])
    }
  }

  after <- missingData(M_out)$mandatoryTags
  report <- if (length(report_rows) > 0) {
    do.call(rbind, report_rows)
  } else {
    .empty_enrichment_report()
  }

  list(M = M_out, report = report, before = before, after = after)
}

#' @noRd
.recompute_eligible <- function(M, fields, base_idx) {
  ## Among the rows we initially flagged eligible, keep only those that still
  ## have at least one missing target field. This makes Pass 2 (Crossref)
  ## skip rows that Pass 1 (OpenAlex) already fully populated.
  if (length(base_idx) == 0) {
    return(base_idx)
  }
  fields_present <- intersect(fields, names(M))
  if (length(fields_present) == 0) {
    return(integer(0))
  }
  any_missing <- rep(FALSE, length(base_idx))
  for (f in fields_present) {
    any_missing <- any_missing | .is_vacant(M[[f]][base_idx])
  }
  base_idx[any_missing]
}

# -- DOI helpers ---------------------------------------------------------------

#' @noRd
.normalize_doi <- function(x) {
  if (is.null(x)) {
    return(character(0))
  }
  s <- as.character(x)
  s <- trimws(s)
  s[s == "" | s == "NA" | s == "none"] <- NA_character_
  s <- sub("^https?://(dx\\.)?doi\\.org/", "", s, ignore.case = TRUE)
  s <- sub("^doi:\\s*", "", s, ignore.case = TRUE)
  ok <- grepl("^10\\.[0-9]{4,9}/", s)
  s[!ok] <- NA_character_
  tolower(s)
}

#' @noRd
.is_vacant <- function(x) {
  is.na(x) | x %in% c("NA,0000,NA", "NA", "", "none")
}

#' @noRd
.empty_enrichment_report <- function() {
  data.frame(
    field = character(0),
    source = character(0),
    n_attempted = integer(0),
    n_filled = integer(0),
    n_failed = integer(0),
    stringsAsFactors = FALSE
  )
}

#' @noRd
.build_enrichment_report <- function(
  filled_per_field,
  attempted,
  source_label,
  fail_count
) {
  fields <- names(filled_per_field)
  if (length(fields) == 0) {
    return(.empty_enrichment_report())
  }
  data.frame(
    field = fields,
    source = source_label,
    n_attempted = attempted,
    n_filled = as.integer(filled_per_field),
    n_failed = fail_count,
    stringsAsFactors = FALSE
  )
}

#' @noRd
.resolve_email <- function(email) {
  if (!is.null(email) && nzchar(email)) {
    return(email)
  }
  env <- Sys.getenv("BIBLIOMETRIX_EMAIL", unset = "")
  if (nzchar(env)) {
    return(env)
  }
  env2 <- Sys.getenv("openalexR.mailto", unset = "")
  if (nzchar(env2)) {
    return(env2)
  }
  pf <- path.expand("~/.biblio_openalex_email.txt")
  if (file.exists(pf)) {
    e <- trimws(readLines(pf, warn = FALSE)[1])
    if (length(e) == 1 && nzchar(e)) return(e)
  }
  NULL
}

#' @noRd
.resolve_oa_apikey <- function(key) {
  if (!is.null(key) && nzchar(key)) {
    return(key)
  }
  env <- Sys.getenv("openalexR.apikey", unset = "")
  if (nzchar(env)) {
    return(env)
  }
  pf <- path.expand("~/.biblio_openalex_apikey.txt")
  if (file.exists(pf)) {
    k <- trimws(readLines(pf, warn = FALSE)[1])
    if (length(k) == 1 && nzchar(k) && nchar(k) >= 10) return(k)
  }
  NULL
}

# -- Crossref client -----------------------------------------------------------

#' @noRd
.enrich_from_crossref <- function(
  dois,
  fields,
  email,
  batch_size = 20,
  progress = NULL,
  verbose = TRUE
) {
  if (
    !requireNamespace("httr2", quietly = TRUE) ||
      !requireNamespace("jsonlite", quietly = TRUE)
  ) {
    stop("completeMetadata() requires the 'httr2' and 'jsonlite' packages.")
  }

  ## We deduplicate DOIs because two rows may share a DOI (rare but possible)
  uniq <- unique(dois)
  chunks <- split(uniq, ceiling(seq_along(uniq) / batch_size))

  ua <- .crossref_user_agent(email)
  base <- "https://api.crossref.org/works"
  results <- list()
  fail_count <- 0L

  for (i in seq_along(chunks)) {
    chunk <- chunks[[i]]
    items <- tryCatch(
      .crossref_fetch_batch(chunk, base, ua),
      error = function(e) {
        if (isTRUE(verbose)) {
          message(sprintf(
            "[completeMetadata] Batch %d failed: %s",
            i,
            conditionMessage(e)
          ))
        }
        NULL
      }
    )
    if (is.null(items)) {
      fail_count <- fail_count + length(chunk)
    } else {
      results <- c(results, items)
    }
    if (is.function(progress)) {
      progress(
        i * batch_size,
        length(uniq),
        sprintf("Crossref batch %d / %d", i, length(chunks))
      )
    }
    if (isTRUE(verbose)) {
      message(sprintf(
        "[completeMetadata] Crossref %d/%d batches done.",
        i,
        length(chunks)
      ))
    }
  }

  ## items is a named list keyed by lowercase DOI; build a per-DOI parsed frame
  parsed <- lapply(results, function(it) .crossref_item_to_row(it, fields))
  parsed <- parsed[!vapply(parsed, is.null, logical(1))]
  if (length(parsed) == 0) {
    return(list(by_doi = list(), fail_count = fail_count))
  }
  list(
    by_doi = stats::setNames(
      parsed,
      vapply(parsed, function(r) r$DI, character(1))
    ),
    fail_count = fail_count
  )
}

#' @noRd
.crossref_user_agent <- function(email) {
  ver <- tryCatch(
    as.character(utils::packageVersion("bibliometrix")),
    error = function(e) "dev"
  )
  if (!is.null(email) && nzchar(email)) {
    sprintf(
      "bibliometrix/%s (https://www.bibliometrix.org; mailto:%s)",
      ver,
      email
    )
  } else {
    sprintf("bibliometrix/%s (https://www.bibliometrix.org)", ver)
  }
}

#' @noRd
.crossref_fetch_batch <- function(dois, base, ua) {
  ## Build filter=doi:DOI1,doi:DOI2,... ; ask only for the per-item fields we need.
  filter_val <- paste0("doi:", dois, collapse = ",")
  req <- httr2::request(base) %>%
    httr2::req_url_query(
      filter = filter_val,
      rows = as.character(length(dois))
    ) %>%
    httr2::req_user_agent(ua) %>%
    httr2::req_retry(max_tries = 4, backoff = function(i) 2^i) %>%
    httr2::req_timeout(60)

  resp <- httr2::req_perform(req)
  raw <- httr2::resp_body_string(resp)
  parsed <- jsonlite::fromJSON(raw, simplifyVector = FALSE)
  if (is.null(parsed$message$items)) {
    return(list())
  }
  parsed$message$items
}

# -- Crossref item parser ------------------------------------------------------

#' @noRd
.crossref_item_to_row <- function(it, fields) {
  doi <- tolower(.coalesce_chr(it$DOI))
  if (is.na(doi) || !nzchar(doi)) {
    return(NULL)
  }

  out <- list(DI = doi)

  if ("TI" %in% fields) {
    out$TI <- .coalesce_chr(.first(it$title))
  }
  if ("SO" %in% fields) {
    so <- .first(it$`container-title`)
    if (!nzchar(so %||% "")) {
      so <- .first(it$`short-container-title`)
    }
    out$SO <- .coalesce_chr(so)
  }
  if ("PY" %in% fields) {
    out$PY <- .crossref_year(it)
  }
  if ("LA" %in% fields) {
    out$LA <- .coalesce_chr(it$language)
  }
  if ("DT" %in% fields) {
    out$DT <- .crossref_doctype(.coalesce_chr(it$type))
  }
  if ("AB" %in% fields) {
    out$AB <- .strip_jats_abstract(.coalesce_chr(it$abstract))
  }
  if ("AU" %in% fields || "C1" %in% fields || "RP" %in% fields) {
    au_info <- .crossref_authors(it$author)
    if ("AU" %in% fields) {
      out$AU <- au_info$AU
    }
    if ("C1" %in% fields) {
      out$C1 <- au_info$C1
    }
    if ("RP" %in% fields) out$RP <- au_info$RP
  }
  if ("CR" %in% fields) {
    out$CR <- .crossref_refs_to_cr_string(it$reference)
  }

  ## Replace empty strings with NA so the merge step ignores them.
  out <- lapply(out, function(v) {
    if (is.character(v) && length(v) == 1 && (is.na(v) || !nzchar(v))) {
      NA_character_
    } else {
      v
    }
  })
  out
}

#' @noRd
.first <- function(x) {
  if (is.null(x) || length(x) == 0) {
    return(NA_character_)
  }
  if (is.list(x)) {
    return(as.character(x[[1]]))
  }
  as.character(x[[1]])
}

#' @noRd
.coalesce_chr <- function(x) {
  if (is.null(x) || length(x) == 0) {
    return(NA_character_)
  }
  v <- as.character(x[[1]])
  if (is.na(v) || !nzchar(v)) {
    return(NA_character_)
  }
  v
}

#' @noRd
`%||%` <- function(a, b) {
  if (is.null(a) || (length(a) == 1 && is.na(a))) b else a
}

#' @noRd
.crossref_year <- function(it) {
  ## Try issued -> published-print -> published-online -> created
  for (k in c("issued", "published-print", "published-online", "created")) {
    parts <- it[[k]]$`date-parts`
    if (!is.null(parts) && length(parts) > 0) {
      y <- suppressWarnings(as.integer(parts[[1]][[1]]))
      if (!is.na(y) && y >= 1500 && y <= 2200) return(as.character(y))
    }
  }
  NA_character_
}

#' @noRd
.crossref_doctype <- function(x) {
  if (is.null(x) || is.na(x)) {
    return(NA_character_)
  }
  ## Map Crossref types onto the WoS-codified DT vocabulary used elsewhere
  ## in bibliometrix. Anything we cannot map confidently is uppercased verbatim
  ## so downstream filtering still works.
  x <- tolower(x)
  toupper(switch(
    x,
    "journal-article" = "ARTICLE",
    "proceedings-article" = "PROCEEDINGS PAPER",
    "book-chapter" = "BOOK CHAPTER",
    "book" = "BOOK",
    "monograph" = "BOOK",
    "edited-book" = "EDITED BOOK",
    "reference-book" = "BOOK",
    "report" = "REPORT",
    "posted-content" = "PREPRINT",
    "dissertation" = "DISSERTATION",
    "dataset" = "DATA",
    "review" = "REVIEW",
    "journal-issue" = "EDITORIAL",
    x
  ))
}

#' @noRd
.crossref_authors <- function(authors) {
  if (is.null(authors) || length(authors) == 0) {
    return(list(AU = NA_character_, C1 = NA_character_, RP = NA_character_))
  }
  fmt <- vapply(
    authors,
    function(a) {
      fam <- toupper(a$family %||% "")
      giv <- a$given %||% ""
      inits <- ""
      if (nzchar(giv)) {
        tokens <- strsplit(giv, "[ \\.\\-]+")[[1]]
        tokens <- tokens[nzchar(tokens)]
        inits <- paste(toupper(substr(tokens, 1, 1)), collapse = "")
      }
      if (!nzchar(fam)) {
        return(NA_character_)
      }
      trimws(paste(fam, inits))
    },
    character(1)
  )
  fmt <- fmt[!is.na(fmt) & nzchar(fmt)]
  AU <- if (length(fmt) > 0) paste(fmt, collapse = "; ") else NA_character_

  affs <- lapply(authors, function(a) {
    aa <- a$affiliation
    if (is.null(aa) || length(aa) == 0) {
      return(character(0))
    }
    vapply(
      aa,
      function(z) {
        if (is.list(z) && !is.null(z$name)) {
          as.character(z$name)
        } else {
          NA_character_
        }
      },
      character(1)
    )
  })
  affs <- unlist(affs, use.names = FALSE)
  affs <- affs[!is.na(affs) & nzchar(affs)]
  affs <- unique(affs)
  C1 <- if (length(affs) > 0) {
    toupper(paste(affs, collapse = "; "))
  } else {
    NA_character_
  }

  ## Corresponding author guess: first author flagged sequence == "first",
  ## otherwise the first author. Crossref's corresponding flag is unreliable.
  rp_idx <- which(vapply(
    authors,
    function(a) isTRUE(a$sequence == "first"),
    logical(1)
  ))
  if (length(rp_idx) == 0) {
    rp_idx <- 1L
  }
  rp_aff <- authors[[rp_idx[1]]]$affiliation
  RP <- NA_character_
  if (!is.null(rp_aff) && length(rp_aff) > 0) {
    nm <- vapply(
      rp_aff,
      function(z) {
        if (is.list(z) && !is.null(z$name)) {
          as.character(z$name)
        } else {
          NA_character_
        }
      },
      character(1)
    )
    nm <- nm[!is.na(nm) & nzchar(nm)]
    if (length(nm) > 0) RP <- toupper(paste(nm, collapse = "; "))
  }

  list(AU = AU, C1 = C1, RP = RP)
}

#' @noRd
.strip_jats_abstract <- function(x) {
  if (is.null(x) || length(x) == 0 || is.na(x) || !nzchar(x)) {
    return(NA_character_)
  }
  out <- NA_character_
  if (requireNamespace("xml2", quietly = TRUE)) {
    out <- tryCatch(
      {
        ## Wrap to ensure a single root and tolerate JATS namespaces.
        doc <- xml2::read_xml(paste0(
          "<root xmlns:jats=\"http://jats.nlm.nih.gov\">",
          x,
          "</root>"
        ))
        txt <- xml2::xml_text(doc)
        trimws(gsub("\\s+", " ", txt))
      },
      error = function(e) NA_character_
    )
  }
  if (is.na(out) || !nzchar(out)) {
    ## Fallback: strip tags and decode common entities.
    out <- gsub("<[^>]+>", " ", x)
    out <- gsub("&amp;", "&", out, fixed = TRUE)
    out <- gsub("&lt;", "<", out, fixed = TRUE)
    out <- gsub("&gt;", ">", out, fixed = TRUE)
    out <- gsub("&quot;", "\"", out, fixed = TRUE)
    out <- gsub("&#x2013;", "-", out, fixed = TRUE)
    out <- trimws(gsub("\\s+", " ", out))
  }
  if (!nzchar(out)) NA_character_ else out
}

#' @noRd
.crossref_refs_to_cr_string <- function(refs) {
  if (is.null(refs) || length(refs) == 0) {
    return(NA_character_)
  }
  parts <- vapply(refs, .one_crossref_ref, character(1))
  parts <- parts[!is.na(parts) & nzchar(parts)]
  if (length(parts) == 0) {
    return(NA_character_)
  }
  paste(parts, collapse = "; ")
}

#' @noRd
.one_crossref_ref <- function(r) {
  if (is.null(r) || length(r) == 0) {
    return(NA_character_)
  }
  ## Prefer structured representation when available.
  au <- r$author %||% ""
  yr <- r$year %||% ""
  jrn <- r$`journal-title` %||% (r$`series-title` %||% "")
  vol <- r$volume %||% ""
  page <- r$`first-page` %||% ""
  doi <- r$DOI %||% ""
  art <- r$`article-title` %||% ""

  has_struct <- any(nzchar(c(au, yr, jrn, vol, page, doi)))
  if (has_struct) {
    tokens <- c(
      if (nzchar(au)) toupper(.first_token(au)) else NULL,
      if (nzchar(yr)) yr else NULL,
      if (nzchar(jrn)) toupper(jrn) else NULL,
      if (nzchar(vol)) paste0("V", vol) else NULL,
      if (nzchar(page)) paste0("P", page) else NULL,
      if (nzchar(doi)) paste0("DOI ", tolower(doi)) else NULL
    )
    out <- paste(tokens, collapse = ", ")
    return(out)
  }

  ## Unstructured fallback: lift year and DOI via regex; keep the rest verbatim.
  un <- r$unstructured %||% ""
  if (!nzchar(un) && nzchar(art)) {
    un <- art
  }
  if (!nzchar(un)) {
    return(NA_character_)
  }
  yr_m <- regmatches(un, regexpr("\\b(19|20)[0-9]{2}\\b", un))
  doi_m <- regmatches(un, regexpr("10\\.[0-9]{4,9}/[^\\s,;]+", un, perl = TRUE))
  un_clean <- gsub(";", ",", un, fixed = TRUE) ## protect CR separator
  tokens <- c(
    toupper(un_clean),
    if (length(yr_m)) yr_m else NULL,
    if (length(doi_m)) paste0("DOI ", tolower(doi_m)) else NULL
  )
  paste(tokens, collapse = ", ")
}

#' @noRd
.first_token <- function(x) {
  s <- as.character(x[[1]])
  s <- trimws(s)
  s <- sub("[,;].*$", "", s)
  s
}

# -- Fill-only merge -----------------------------------------------------------

#' @noRd
.merge_enrichment <- function(
  M,
  enriched,
  eligible_idx,
  fields,
  source_label = "crossref"
) {
  filled <- stats::setNames(integer(length(fields)), fields)
  prov_rows <- list()

  if (length(enriched$by_doi) == 0) {
    return(list(M = M, prov = NULL, filled = filled))
  }

  doi_norm <- .normalize_doi(M$DI)
  ts <- format(Sys.time(), "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")

  ## Make sure SR exists for provenance
  has_sr <- "SR" %in% names(M)

  for (i in eligible_idx) {
    di <- doi_norm[i]
    if (is.na(di)) {
      next
    }
    rec <- enriched$by_doi[[di]]
    if (is.null(rec)) {
      next
    }
    for (f in fields) {
      if (!(f %in% names(M))) {
        next
      }
      cur <- M[[f]][i]
      if (!.is_vacant(cur)) {
        next
      }
      val <- rec[[f]]
      if (is.null(val) || length(val) == 0) {
        next
      }
      v <- as.character(val[[1]])
      if (is.na(v) || !nzchar(v)) {
        next
      }
      ## Coerce target column to character if needed (e.g. all-NA logical).
      if (!is.character(M[[f]])) {
        M[[f]] <- as.character(M[[f]])
      }
      M[[f]][i] <- v
      filled[f] <- filled[f] + 1L
      prov_rows[[length(prov_rows) + 1]] <- data.frame(
        SR = if (has_sr) M$SR[i] else NA_character_,
        field = f,
        source = source_label,
        timestamp = ts,
        stringsAsFactors = FALSE
      )
    }
  }

  prov <- if (length(prov_rows) > 0) do.call(rbind, prov_rows) else NULL
  list(M = M, prov = prov, filled = filled)
}

# -- OpenAlex client -----------------------------------------------------------
# Uses openalexR::oa_fetch with output = "list" so we can reuse the helpers
# in R/resolveOAReferences.R (same code path biblioshiny uses) without going
# through openalexR's tibble post-processing. Returns a per-DOI list aligned
# with the Crossref output so .merge_enrichment can treat them uniformly.

#' @noRd
.enrich_from_openalex <- function(
  dois,
  fields,
  email,
  oa_apikey,
  progress = NULL,
  verbose = TRUE
) {
  if (!requireNamespace("openalexR", quietly = TRUE)) {
    if (isTRUE(verbose)) {
      message(
        "[completeMetadata] openalexR is not installed; skipping OpenAlex pass."
      )
    }
    return(list(by_doi = list(), fail_count = length(unique(dois))))
  }

  ## OpenAlex polite pool / API key plumbing -- set as env vars + options for
  ## the duration of the call (openalexR reads both).
  if (!is.null(email) && nzchar(email)) {
    op_old <- options(openalexR.mailto = email)
    on.exit(options(op_old), add = TRUE)
    Sys.setenv(openalexR.mailto = email)
  }
  if (!is.null(oa_apikey) && nzchar(oa_apikey)) {
    op_old2 <- options(openalexR.apikey = oa_apikey)
    on.exit(options(op_old2), add = TRUE)
    Sys.setenv(openalexR.apikey = oa_apikey)
  }

  uniq <- unique(dois)
  ## OpenAlex accepts up to 50 IDs per filter request.
  oa_batch <- 50L
  chunks <- split(uniq, ceiling(seq_along(uniq) / oa_batch))

  works <- list()
  fail_count <- 0L

  for (i in seq_along(chunks)) {
    chunk <- chunks[[i]]
    fetched <- tryCatch(
      openalexR::oa_fetch(
        entity = "works",
        doi = chunk,
        output = "list",
        verbose = FALSE
      ),
      error = function(e) {
        if (isTRUE(verbose)) {
          message(sprintf(
            "[completeMetadata] OpenAlex batch %d failed: %s",
            i,
            conditionMessage(e)
          ))
        }
        NULL
      }
    )
    if (is.null(fetched)) {
      fail_count <- fail_count + length(chunk)
    } else if (is.list(fetched)) {
      works <- c(works, fetched)
    }
    if (is.function(progress)) {
      progress(
        i * oa_batch,
        length(uniq),
        sprintf("OpenAlex batch %d / %d", i, length(chunks))
      )
    }
    if (isTRUE(verbose)) {
      message(sprintf(
        "[completeMetadata] OpenAlex %d/%d batches done.",
        i,
        length(chunks)
      ))
    }
  }

  if (length(works) == 0) {
    return(list(by_doi = list(), fail_count = fail_count))
  }

  ## CR field requires resolving referenced_works (a second batch of fetches).
  ## We build a lookup ID -> WoS-style CR string and then assemble per-paper
  ## CR strings in the per-DOI parser.
  ref_lookup <- if ("CR" %in% fields) {
    .openalex_resolve_refs(
      works,
      email,
      oa_apikey,
      progress = progress,
      verbose = verbose
    )
  } else {
    list()
  }

  parsed <- lapply(works, function(w) {
    .openalex_work_to_row(w, fields, ref_lookup)
  })
  parsed <- parsed[!vapply(parsed, is.null, logical(1))]
  if (length(parsed) == 0) {
    return(list(by_doi = list(), fail_count = fail_count))
  }

  list(
    by_doi = stats::setNames(
      parsed,
      vapply(parsed, function(r) r$DI, character(1))
    ),
    fail_count = fail_count
  )
}

#' @noRd
.openalex_resolve_refs <- function(
  works,
  email,
  oa_apikey,
  progress = NULL,
  verbose = TRUE
) {
  all_refs <- unlist(
    lapply(works, function(w) w$referenced_works),
    use.names = FALSE
  )
  all_refs <- all_refs[!is.na(all_refs) & nzchar(all_refs)]
  if (length(all_refs) == 0) {
    return(list())
  }
  uniq_refs <- unique(all_refs)

  batch <- 50L
  n_chunks <- ceiling(length(uniq_refs) / batch)
  out <- list()
  for (i in seq_len(n_chunks)) {
    rng <- ((i - 1) * batch + 1):min(i * batch, length(uniq_refs))
    short_ids <- gsub("^https://openalex\\.org/", "", uniq_refs[rng])
    fetched <- tryCatch(
      openalexR::oa_fetch(
        entity = "works",
        openalex_id = paste(short_ids, collapse = "|"),
        output = "list",
        verbose = FALSE
      ),
      error = function(e) NULL
    )
    if (is.list(fetched)) {
      for (w in fetched) {
        if (!is.null(w$id)) {
          cr <- build_single_cr(w)
          if (!is.na(cr)) out[[w$id]] <- cr
        }
      }
    }
    if (is.function(progress)) {
      progress(
        i * batch,
        length(uniq_refs),
        sprintf("OpenAlex refs %d / %d", i, n_chunks)
      )
    }
  }
  out
}

#' @noRd
.openalex_work_to_row <- function(w, fields, ref_lookup) {
  doi <- .coalesce_chr(w$doi)
  if (is.na(doi)) {
    return(NULL)
  }
  doi <- tolower(sub(
    "^https?://(dx\\.)?doi\\.org/",
    "",
    doi,
    ignore.case = TRUE
  ))
  if (!nzchar(doi)) {
    return(NULL)
  }

  out <- list(DI = doi)

  if ("TI" %in% fields) {
    out$TI <- .coalesce_chr(w$title %||% w$display_name)
  }
  if ("PY" %in% fields) {
    py <- w$publication_year
    out$PY <- if (!is.null(py) && !is.na(py)) {
      as.character(py)
    } else {
      NA_character_
    }
  }
  if ("LA" %in% fields) {
    out$LA <- .coalesce_chr(w$language)
  }
  if ("DT" %in% fields) {
    out$DT <- .openalex_doctype(.coalesce_chr(w$type))
  }
  if ("SO" %in% fields) {
    out$SO <- .coalesce_chr(
      tryCatch(w$primary_location$source$display_name, error = function(e) NA)
    )
  }
  if ("AB" %in% fields && !is.null(w$abstract_inverted_index)) {
    out$AB <- abstract_build(w$abstract_inverted_index)
  }
  if ("TC" %in% fields) {
    tc <- w$cited_by_count
    out$TC <- if (!is.null(tc) && !is.na(tc)) {
      as.character(tc)
    } else {
      NA_character_
    }
  }
  if (any(c("AU", "C1", "RP") %in% fields)) {
    info <- .openalex_authors(w$authorships)
    if ("AU" %in% fields) {
      out$AU <- info$AU
    }
    if ("C1" %in% fields) {
      out$C1 <- info$C1
    }
    if ("RP" %in% fields) out$RP <- info$RP
  }
  if ("CR" %in% fields) {
    refs <- w$referenced_works
    if (!is.null(refs) && length(refs) > 0 && length(ref_lookup) > 0) {
      strs <- unlist(
        lapply(refs, function(rid) ref_lookup[[rid]]),
        use.names = FALSE
      )
      strs <- strs[!is.na(strs) & nzchar(strs)]
      if (length(strs) > 0) out$CR <- paste(strs, collapse = "; ")
    }
  }

  ## Replace empty-strings with NA so the merge step ignores them.
  out <- lapply(out, function(v) {
    if (is.character(v) && length(v) == 1 && (is.na(v) || !nzchar(v))) {
      NA_character_
    } else {
      v
    }
  })
  out
}

#' @noRd
.openalex_doctype <- function(x) {
  if (is.null(x) || is.na(x)) {
    return(NA_character_)
  }
  x <- tolower(x)
  toupper(switch(
    x,
    "article" = "ARTICLE",
    "review" = "REVIEW",
    "book" = "BOOK",
    "book-chapter" = "BOOK CHAPTER",
    "report" = "REPORT",
    "preprint" = "PREPRINT",
    "dissertation" = "DISSERTATION",
    "dataset" = "DATA",
    "editorial" = "EDITORIAL",
    "letter" = "LETTER",
    "paratext" = "PARATEXT",
    "reference-entry" = "REFERENCE",
    "standard" = "STANDARD",
    x
  ))
}

#' @noRd
.openalex_authors <- function(authorships) {
  if (is.null(authorships) || length(authorships) == 0) {
    return(list(AU = NA_character_, C1 = NA_character_, RP = NA_character_))
  }
  ## AU: WoS-style "LASTNAME I" -- match the convention used by csvScopus2df.
  fmt <- vapply(
    authorships,
    function(a) {
      nm <- a$author$display_name %||% ""
      if (!nzchar(nm)) {
        return(NA_character_)
      }
      parts <- unlist(strsplit(trimws(nm), "\\s+"))
      if (length(parts) == 0) {
        return(NA_character_)
      }
      if (length(parts) == 1) {
        return(toupper(parts[1]))
      }
      surname <- toupper(parts[length(parts)])
      given <- parts[-length(parts)]
      inits <- paste(toupper(substr(given, 1, 1)), collapse = "")
      trimws(paste(surname, inits))
    },
    character(1)
  )
  fmt <- fmt[!is.na(fmt) & nzchar(fmt)]
  AU <- if (length(fmt) > 0) paste(fmt, collapse = "; ") else NA_character_

  ## C1: union of institution display names across all authors.
  inst_names <- unlist(
    lapply(authorships, function(a) {
      if (is.null(a$institutions) || length(a$institutions) == 0) {
        return(character(0))
      }
      vapply(
        a$institutions,
        function(inst) {
          if (!is.null(inst$display_name)) inst$display_name else NA_character_
        },
        character(1)
      )
    }),
    use.names = FALSE
  )
  inst_names <- inst_names[!is.na(inst_names) & nzchar(inst_names)]
  inst_names <- unique(inst_names)
  C1 <- if (length(inst_names) > 0) {
    toupper(paste(inst_names, collapse = "; "))
  } else {
    NA_character_
  }

  ## RP: corresponding author affiliation. Prefer is_corresponding flag,
  ## fall back to first authorship.
  corr_idx <- which(vapply(
    authorships,
    function(a) isTRUE(a$is_corresponding),
    logical(1)
  ))
  if (length(corr_idx) == 0) {
    first_idx <- which(vapply(
      authorships,
      function(a) isTRUE(a$author_position == "first"),
      logical(1)
    ))
    corr_idx <- if (length(first_idx) > 0) first_idx[1] else 1L
  } else {
    corr_idx <- corr_idx[1]
  }
  rp_inst <- authorships[[corr_idx]]$institutions
  RP <- NA_character_
  if (!is.null(rp_inst) && length(rp_inst) > 0) {
    nm <- vapply(
      rp_inst,
      function(z) {
        if (!is.null(z$display_name)) z$display_name else NA_character_
      },
      character(1)
    )
    nm <- nm[!is.na(nm) & nzchar(nm)]
    if (length(nm) > 0) RP <- toupper(paste(nm, collapse = "; "))
  }
  if (is.na(RP)) {
    raw_aff <- authorships[[corr_idx]]$affiliations
    if (!is.null(raw_aff) && length(raw_aff) > 0) {
      raw <- raw_aff[[1]]$raw_affiliation_string
      if (!is.null(raw) && nzchar(raw)) RP <- toupper(raw)
    }
  }

  list(AU = AU, C1 = C1, RP = RP)
}

utils::globalVariables(c(
  "cluster",
  "color_original",
  "values",
  "vertex"
))

#' Assign Colors to Thematic Evolution Nodes Based on Lineages
#'
#' This function assigns colors to nodes in a thematic evolution analysis based on
#' their lineages across time periods. Nodes connected by strong edges (above a threshold)
#' receive the same color to visualize thematic continuity. Nodes with the same name
#' across periods that are not strongly connected to other nodes are also colored identically.
#'
#' @param nexus A list object returned by \code{\link{thematicEvolution}} containing:
#'   \itemize{
#'     \item \code{Nodes}: data frame with node information (name, group, id, sum, freq, etc.)
#'     \item \code{Edges}: data frame with edge information (from, to, weight measures)
#'     \item \code{TM}: list of thematic maps for each period
#'   }
#' @param threshold Numeric. The minimum weight value for an edge to be considered a
#'   "strong connection" (default: 0.5). Edges with weights >= threshold will propagate
#'   the same color to connected nodes across periods.
#' @param palette Character vector. Optional custom color palette as hex codes. If NULL,
#'   uses a default palette of 50+ distinct colors. Colors are assigned sequentially
#'   without reuse.
#' @param alpha Numeric. Balancing parameters between inclusion and importance measures. Default value is 0.5.
#'
#' @return Returns the modified \code{nexus} object with updated colors:
#'   \itemize{
#'     \item \code{Nodes$color}: updated with lineage-based colors
#'     \item \code{Edges$color}: edges connecting same-colored nodes receive the node color,
#'       others are grey
#'     \item \code{TM}: thematic maps updated with new cluster colors
#'   }
#'
#' @details
#' The function uses a multi-phase algorithm:
#' \enumerate{
#'   \item \strong{Phase 1}: Identifies lineages by following strong connections (weight >= threshold)
#'     from the first period forward. When a node has multiple strong connections, the
#'     strongest one determines the lineage.
#'   \item \strong{Phase 1.5}: Assigns the same lineage to nodes with identical names across
#'     periods if they are not already part of different strong connections.
#'   \item \strong{Phase 2}: Assigns unique colors from the palette to each identified lineage.
#'   \item \strong{Phase 3}: Assigns unique colors to isolated nodes (those without any lineage).
#'   \item \strong{Phase 4}: Colors edges based on their connected nodes - same color if both
#'     nodes share a color, grey otherwise.
#'   \item \strong{Final}: Updates thematic maps with the new color scheme.
#' }
#'
#' Each lineage receives a unique color from the palette. No color is reused across
#' different lineages, ensuring clear visual distinction between independent thematic streams.
#'
#' @examples
#' \dontrun{
#' data(scientometrics, package = "bibliometrixData")
#' years <- c(2000, 2010)
#'
#' nexus <- thematicEvolution(scientometrics, field = "ID",
#'                             years = years, n = 100, minFreq = 2)
#'
#'
#' # Use custom threshold and measure
#' nexus <- assignEvolutionColors(nexus, threshold = 0.6, use_measure = "weighted")
#'
#' }
#'
#' @seealso \code{\link{thematicEvolution}} to perform thematic evolution analysis.
#' @seealso \code{\link{plotThematicEvolution}} to visualize the colored evolution.
#'
#' @export

assignEvolutionColors <- function(
  nexus,
  threshold = 0.5,
  palette = NULL,
  alpha = 0.5
) {
  # Prepare data structure
  Nodes <- nexus$Nodes %>%
    mutate(
      color_assigned = FALSE,
      color_lineage = NA_integer_,
      color_final = NA_character_,
      has_natural_successor = FALSE,
      is_natural_lineage = FALSE # NUOVO FLAG
    )

  Edges <- nexus$Edges

  # # Calculate relative frequency for each node
  # Nodes <- Nodes %>%
  #   group_by(group) %>%
  #   mutate(rel_freq = freq / sum(freq, na.rm = TRUE)) %>%
  #   ungroup()

  # Add lineage strength to edges
  Edges <- Edges %>%
    # left_join(
    #   Nodes %>% select(id, rel_freq_from = rel_freq),
    #   by = c("from" = "id")
    # ) %>%
    mutate(
      lineage_strength = (alpha * Inc_Weighted) + (1 - alpha) * PageRankIndex
    )

  strong_edges <- Edges %>%
    dplyr::filter(Inc_Weighted >= threshold) %>%
    select(from, to, weight = lineage_strength, inc_weighted = Inc_Weighted)

  if (is.null(palette)) {
    palette <- c(
      "#E41A1C",
      "#377EB8",
      "#4DAF4A",
      "#984EA3",
      "#FF7F00",
      "#A65628",
      "#F781BF",
      "#66C2A5",
      "#FC8D62",
      "#8DA0CB",
      "#E78AC3",
      "#A6D854",
      "#FFD92F",
      "#A6CEE3",
      "#1F78B4",
      "#B2DF8A",
      "#33A02C",
      "#FB9A99",
      "#E31A1C",
      "#FDBF6F",
      "#FF7F00",
      "#CAB2D6",
      "#6A3D9A",
      "#B15928",
      "#8DD3C7",
      "#BEBADA",
      "#FB8072",
      "#80B1D3",
      "#FDB462",
      "#B3DE69",
      "#BC80BD",
      "#CCEBC5",
      "#FFB6C1",
      "#DDA0DD",
      "#F0E68C",
      "#87CEEB",
      "#98D8C8",
      "#F7DC6F",
      "#FAD7A0",
      "#D7BDE2",
      "#A9DFBF",
      "#F8B88B",
      "#AED6F1",
      "#F9E79F",
      "#C39BD3",
      "#73C6B6",
      "#F5B7B1"
    )
  }

  periods <- unique(Nodes$group)
  lineage_id <- 1

  # ============================================================================
  # PHASE 1: Natural lineages (same label across periods)
  # ============================================================================

  unique_names <- unique(Nodes$name)

  for (node_name in unique_names) {
    same_name_nodes <- Nodes %>%
      dplyr::filter(name == node_name) %>%
      arrange(group)

    if (nrow(same_name_nodes) > 1) {
      # Assign the same lineage to all nodes with this name
      Nodes$color_lineage[Nodes$name == node_name] <- lineage_id
      Nodes$is_natural_lineage[Nodes$name == node_name] <- TRUE # NUOVO

      # Mark nodes that have a natural successor
      for (j in 1:(nrow(same_name_nodes) - 1)) {
        node_id <- same_name_nodes$id[j]
        Nodes$has_natural_successor[Nodes$id == node_id] <- TRUE
      }

      lineage_id <- lineage_id + 1
    }
  }

  # ============================================================================
  # PHASE 2: Algorithm-based lineages using lineage strength
  # ============================================================================

  for (i in 1:(length(periods) - 1)) {
    current_period <- periods[i]
    next_period <- periods[i + 1]

    current_nodes <- Nodes %>% dplyr::filter(group == current_period)

    for (node_id in current_nodes$id) {
      # SKIP nodes that already have a natural successor
      if (Nodes$has_natural_successor[Nodes$id == node_id]) {
        next
      }

      connections <- strong_edges %>%
        dplyr::filter(from == node_id) %>%
        arrange(desc(weight))

      if (nrow(connections) > 0) {
        main_successor <- connections$to[1]
        main_weight <- connections$weight[1]

        current_lineage <- Nodes$color_lineage[Nodes$id == node_id]
        successor_idx <- which(Nodes$id == main_successor)
        successor_lineage <- Nodes$color_lineage[successor_idx]
        successor_is_natural <- Nodes$is_natural_lineage[successor_idx] # NUOVO

        if (is.na(current_lineage)) {
          if (is.na(successor_lineage)) {
            Nodes$color_lineage[Nodes$id == node_id] <- lineage_id
            Nodes$color_lineage[successor_idx] <- lineage_id
            lineage_id <- lineage_id + 1
          } else {
            # NON sovrascrivere se il successore ha un lineage naturale
            if (!successor_is_natural) {
              # MODIFICATO
              Nodes$color_lineage[Nodes$id == node_id] <- successor_lineage
            }
          }
        } else {
          if (is.na(successor_lineage)) {
            Nodes$color_lineage[successor_idx] <- current_lineage
          } else if (successor_lineage != current_lineage) {
            # NON sovrascrivere se il successore ha un lineage naturale
            if (!successor_is_natural) {
              # NUOVO CONTROLLO
              competing_edges <- strong_edges %>%
                dplyr::filter(to == main_successor)

              if (nrow(competing_edges) > 0) {
                existing_weight <- max(competing_edges$weight, na.rm = TRUE)

                if (
                  main_weight > existing_weight ||
                    (main_weight == existing_weight &&
                      !any(
                        competing_edges$from[
                          competing_edges$weight == existing_weight
                        ] !=
                          node_id
                      ))
                ) {
                  Nodes$color_lineage[successor_idx] <- current_lineage
                }
              }
            }
          }
        }
      }
    }
  }

  # ============================================================================
  # PHASE 3: Assign unique colors to lineages
  # ============================================================================

  unique_lineages <- unique(na.omit(Nodes$color_lineage))

  for (i in seq_along(unique_lineages)) {
    lineage <- unique_lineages[i]

    if (i <= length(palette)) {
      color <- paste0(palette[i], "80")
    } else {
      color <- paste0(
        sprintf(
          "#%02X%02X%02X",
          sample(50:255, 1),
          sample(50:255, 1),
          sample(50:255, 1)
        ),
        "80"
      )
    }

    Nodes$color_final[Nodes$color_lineage == lineage] <- color
  }

  # ============================================================================
  # PHASE 4: Assign unique colors to isolated nodes
  # ============================================================================

  isolated_nodes <- which(is.na(Nodes$color_lineage))
  color_idx <- length(unique_lineages) + 1

  for (idx in isolated_nodes) {
    if (color_idx <= length(palette)) {
      color <- paste0(palette[color_idx], "80")
    } else {
      color <- paste0(
        sprintf(
          "#%02X%02X%02X",
          sample(50:255, 1),
          sample(50:255, 1),
          sample(50:255, 1)
        ),
        "80"
      )
    }

    Nodes$color_final[idx] <- color
    color_idx <- color_idx + 1
  }

  # ============================================================================
  # PHASE 5: Assign colors to edges
  # ============================================================================

  Edges <- Edges %>%
    left_join(
      Nodes %>% select(id, color_from = color_final),
      by = c("from" = "id")
    ) %>%
    left_join(
      Nodes %>% select(id, color_to = color_final),
      by = c("to" = "id")
    ) %>%
    mutate(color = if_else(
      color_from == color_to,
      paste0(substr(color_from, 1, 7), "80"),
      "#D3D3D380"
    )) %>%
    select(-color_from, -color_to)

  Nodes <- Nodes %>%
    mutate(color = color_final) %>%
    select(
      -color_assigned,
      -color_lineage,
      -color_final,
      # -rel_freq,
      -has_natural_successor,
      -is_natural_lineage
    ) # AGGIORNATO

  nexus$Nodes <- Nodes
  nexus$Edges <- Edges

  # ============================================================================
  # Update Thematic Maps
  # ============================================================================

  for (i in 1:length(nexus$TM)) {
    res <- nexus$TM[[i]]
    newdf <- Nodes %>% dplyr::filter(slice == i) %>% select(name, color)
    df <- res$clusters %>% select(-color) %>% left_join(newdf, by = "name")
    df$color[is.na(df$color)] <- "#D3D3D3"

    gplot <- buildTMPlot(
      df = df,
      meandens = res$axisOrigin[2],
      meancentr = res$axisOrigin[1],
      xlimits = res$axisLimits[1:2],
      ylimits = res$axisLimits[3:4],
      annotations = res$annotations,
      size = res$params %>%
        dplyr::filter(params == "size") %>%
        pull(values) %>%
        as.numeric(),
      repel = nexus$TM[[i]]$params %>%
        dplyr::filter(params == "repel") %>%
        pull(values) %>%
        as.logical()
    )
    nexus$TM[[i]]$map <- gplot

    g <- nexus$Net[[i]]$graph
    clusters <- nexus$Net[[i]]$cluster_res %>%
      group_by(cluster) %>%
      slice_head(n = 1) %>%
      select(vertex, cluster) %>%
      rename(name = vertex, slice = cluster) %>%
      ungroup() %>%
      left_join(
        data.frame(name = V(g)$name, color_original = V(g)$color),
        by = "name"
      ) %>%
      left_join(newdf, by = "name")

    add_colors <- setdiff(palette, substr(clusters$color, 1, 7))
    clusters$color[is.na(clusters$color)] <- paste0(
      add_colors[1:length(which(is.na(clusters$color)))],
      80
    )

    V(g)$color <- clusters$color[match(V(g)$community, clusters$slice)]

    old_edge_colors <- data.frame(
      from = as.character(igraph::as_edgelist(g)[, 1]),
      to = as.character(igraph::as_edgelist(g)[, 2]),
      color_original = E(g)$color
    )
    new_edge_colors <- old_edge_colors %>%
      left_join(
        clusters %>%
          select(color_original, color) %>%
          mutate(color_original = paste0(substr(color_original, 1, 7), "40")),
        by = "color_original"
      ) %>%
      mutate(
        color = ifelse(is.na(color), "#B3B3B380", color),
        color = ifelse(color_original == "#B3B3B340", "#B3B3B380", color)
      )

    color_map <- setNames(
      paste0(substr(new_edge_colors$color, 1, 7), "40"),
      new_edge_colors$color_original
    )

    E(g)$color <- color_map[E(g)$color]
    nexus$Net[[i]]$graph <- g
  }

  return(nexus)
}

utils::globalVariables(c("all_of", "corr", "DI", "C1", "id_oa", "RP", "UN", "AU_ID", "corresponding_author_ids", "References", "AU_CORRESPONDING", "AU_CO", "AU"))

csvOA2df <- function(file) {
  options(readr.num_columns = 0)

  ## import all files in a single data frame
  for (i in 1:length(file)) {
    # D <- read.csv(file[i], quote='"', check.names = F, stringsAsFactors = F) #fileEncoding = "UTF-8-BOM")
    # D <- read_csv(file[i], na=character(), quote='"', trim_ws = FALSE, progress = show_progress(), show_col_types = FALSE) %>%
    #   mutate(across(where(is.numeric), as.character)) %>%
    #   mutate(across(where(is.character), \(x) tidyr::replace_na(x,""))) %>%
    #   as.data.frame()
    D <- read_csv(file[i], na = character(), quote = '"', trim_ws = FALSE, progress = show_progress(), show_col_types = FALSE) %>%
      mutate(across(where(is.numeric), as.character)) %>%
      mutate(across(where(is.character), function(x) tidyr::replace_na(x, ""))) %>%
      as.data.frame()

    if (i > 1) {
      l <- intersect(l, names(D))
      DATA <- rbind(DATA[l], D[l])
    } else {
      l <- names(D)
      DATA <- D
    }
  }
  rm(D)

  ## Post-Processing

  # column re-labelling
  DATA <- relabelling_OA(DATA)

  # Save original column names before pipe replacement (for dual-path detection)
  DATA_ORIG_NAMES <- names(DATA)

  DATA <- DATA %>% distinct(id_oa, .keep_all = TRUE)

  # replace | with ;
  # OpenAlex CSV uses '|' as the multi-value separator; bibliometrix uses ';'.
  # In multi-value columns, any ';' already present INSIDE a value (e.g. author
  # names mangled by OpenAlex such as "Francisco-Javier; Cabrerizo") must be
  # neutralised BEFORE converting '|' to ';', otherwise the stray ';' becomes a
  # spurious field boundary and inflates the author/affiliation count. Columns
  # that contain no '|' (free text such as abstract/title) are left untouched.
  DATA <- DATA %>%
    mutate(across(where(is.character), function(x) {
      if (!any(grepl("|", x, fixed = TRUE), na.rm = TRUE)) return(x)
      x <- gsub(";", ",", x, fixed = TRUE)
      gsub("|", ";", x, fixed = TRUE)
    }))

  # Defensive checks for columns that may be missing in new format
  missing_cols <- c()
  if (!"CR" %in% names(DATA)) {
    DATA$CR <- NA
    missing_cols <- c(missing_cols, "Cited References (CR)")
  }
  if (!"NR" %in% names(DATA)) DATA$NR <- NA
  if (!"VL" %in% names(DATA)) DATA$VL <- NA
  if (!"IS" %in% names(DATA)) DATA$IS <- NA
  if (!"DE" %in% names(DATA)) DATA$DE <- NA
  if (!"ID" %in% names(DATA)) DATA$ID <- NA
  if (!"TOPICS" %in% names(DATA)) DATA$TOPICS <- NA
  if (!"WC" %in% names(DATA)) DATA$WC <- NA
  if (!"SDG" %in% names(DATA)) DATA$SDG <- NA
  if (!"MESH" %in% names(DATA)) DATA$MESH <- NA
  if (!"SO_ID" %in% names(DATA)) DATA$SO_ID <- NA
  if (!"AU_CO" %in% names(DATA)) DATA$AU_CO <- NA

  if (length(missing_cols) > 0) {
    warning(
      "\nThe OpenAlex CSV does not contain the following fields: ",
      paste(missing_cols, collapse = ", "), ".",
      "\nSome analyses (e.g., citation network, co-citation, bibliographic coupling) may not be available.",
      "\nTo get full metadata, consider using the OpenAlex API import instead.",
      call. = FALSE
    )
  }

  DATA$AF <- DATA$AU
  if (!"AB" %in% names(DATA)) DATA$AB <- ""

  # Protected gsub for OpenAlex URL prefixes
  if ("CR" %in% names(DATA) && !all(is.na(DATA$CR))) {
    DATA$CR <- gsub("https://openalex.org/", "", DATA$CR)
  }
  if ("AU_ID" %in% names(DATA)) {
    DATA$AU_ID <- gsub("https://openalex.org/", "", DATA$AU_ID)
  }
  DATA$id_oa <- gsub("https://openalex.org/", "", DATA$id_oa)
  if (!all(is.na(DATA$SO_ID))) {
    DATA$JI <- DATA$J9 <- gsub("https://openalex.org/", "", DATA$SO_ID)
  } else {
    DATA$JI <- DATA$J9 <- NA
  }
  if ("corresponding_author_ids" %in% names(DATA)) {
    DATA$corresponding_author_ids <- gsub("https://openalex.org/", "", DATA$corresponding_author_ids)
  }
  DATA$DB <- "OPENALEX"

  ## Affiliations — dual path: old vs new format
  if ("authorships.affiliations" %in% DATA_ORIG_NAMES) {
    # Old format: parse JSON-like affiliations string
    DATA <- extract_collapsed_affiliations(DATA$authorships.affiliations, DATA$id_oa) %>%
      right_join(DATA, by = "id_oa")
  } else if ("C1_NAMES" %in% names(DATA)) {
    # New format: pipe-separated values already converted to ; by mutate above
    DATA$C1 <- DATA$C1_NAMES
    DATA$C1_ID <- gsub("https://openalex.org/", "", DATA$C1_IDS)
  } else {
    DATA$C1 <- NA
    DATA$C1_ID <- NA
  }

  ## Corresponding author — dual path: old vs new format
  if ("corresponding_author_ids" %in% DATA_ORIG_NAMES) {
    # Old format: use replace_corresponding_info()
    CORR_INFO <- replace_corresponding_info(DATA[, c("corresponding_author_ids", "corresponding_institution_ids", "AU", "AU_ID", "C1", "C1_ID", "AU_CO")]) %>%
      select("RP", "AU1_CO", "corresponding_author_name")
    DATA <- bind_cols(DATA, CORR_INFO)
  } else if ("AU_CORRESPONDING" %in% names(DATA)) {
    # New format: find corresponding author from is_corresponding flags
    DATA <- DATA %>%
      rowwise() %>%
      mutate(
        corresponding_author_name = {
          corr_flags <- trimws(unlist(strsplit(AU_CORRESPONDING, ";")))
          authors <- trimws(unlist(strsplit(AU, ";")))
          idx <- which(toupper(corr_flags) == "TRUE")
          if (length(idx) > 0 && idx[1] <= length(authors)) authors[idx[1]] else NA_character_
        },
        RP = {
          corr_flags <- trimws(unlist(strsplit(AU_CORRESPONDING, ";")))
          affils <- trimws(unlist(strsplit(ifelse(is.na(C1), "", C1), ";")))
          idx <- which(toupper(corr_flags) == "TRUE")
          if (length(idx) > 0 && idx[1] <= length(affils)) affils[idx[1]] else NA_character_
        },
        AU1_CO = {
          corr_flags <- trimws(unlist(strsplit(AU_CORRESPONDING, ";")))
          countries_list <- trimws(unlist(strsplit(ifelse(is.na(AU_CO), "", AU_CO), ";")))
          idx <- which(toupper(corr_flags) == "TRUE")
          if (length(idx) > 0 && idx[1] <= length(countries_list)) countries_list[idx[1]] else NA_character_
        }
      ) %>%
      ungroup()
  } else {
    DATA$corresponding_author_name <- NA
    DATA$RP <- NA
    DATA$AU1_CO <- NA
  }

  # move all char strings to Upper
  ind <- apply(DATA, 2, function(x) {
    sum(regexpr("https://", x) > -1, na.rm = TRUE) > 0
  })
  label <- names(ind)[ind == FALSE & !is.na(ind)]

  AB <- DATA$AB
  TI <- DATA$TI
  DE <- DATA$DE
  DATA <- DATA %>%
    mutate(across(all_of(label), toupper),
      DI = gsub("https://doi.org/", "", DI),
      DI = ifelse(DI == "null", NA, DI)
    )
  DATA$SO <- toupper(DATA$SO)
  DATA$SO[DATA$SO == "NAN"] <- NA
  if ("C1" %in% names(DATA)) DATA$C1 <- toupper(DATA$C1)
  if ("RP" %in% names(DATA)) DATA$RP <- toupper(DATA$RP)
  if ("AU_CO" %in% names(DATA) && !all(is.na(DATA$AU_CO))) DATA$AU_CO <- toupper(DATA$AU_CO)
  if ("AU1_CO" %in% names(DATA) && !all(is.na(DATA$AU1_CO))) DATA$AU1_CO <- toupper(DATA$AU1_CO)
  data("countries", envir = environment())
  DATA$AU_CO <- convert_iso2_to_country(DATA$AU_CO, countries)
  DATA$AU1_CO <- convert_iso2_to_country(DATA$AU1_CO, countries)
  DATA$AB_raw <- AB
  DATA$TI_raw <- TI
  DATA$DE_raw <- DE

  DATA <- .flagOAAuthorship(DATA %>% as.data.frame())
  return(DATA)
}

relabelling_OA <- function(DATA) {
  ## column re-labelling
  label <- names(DATA)
  label[label %in% "id"] <- "id_oa"
  label[label %in% "title"] <- "TI"
  label[label %in% "primary_location.source.display_name"] <- "SO"
  # label[label %in% "locations.source.display_name"] <- "SO"
  label[label %in% "primary_location.source.id"] <- "SO_ID"
  # label[label %in% "locations.source.id"] <- "SO_ID"
  label[label %in% "primary_location.source.host_organization_name"] <- "PU"
  label[label %in% "primary_location.source.issn"] <- "ISSN"
  label[label %in% "primary_location.source.issn_l"] <- "ISSN_I"
  label[label %in% "primary_location.landing_page_url"] <- "URL"
  label[label %in% "primary_location.pdf_url"] <- "URL_PDF"
  # label[label %in% "author_ids"] <- "AU_ID"
  label[label %in% "authorships.author.id"] <- "AU_ID"
  label[label %in% "authorships.countries"] <- "AU_CO"
  # label[label %in% "author_names"] <- "AU"
  label[label %in% "authorships.author.display_name"] <- "AU"
  label[label %in% "authorships.author.orcid"] <- "OI"
  # label[label %in% "author_institution_names"] <- "C3"
  label[label %in% "cited_by_count"] <- "TC"
  label[label %in% "publication_year"] <- "PY"
  label[label %in% "type"] <- "DT"
  label[label %in% "biblio.issue"] <- "IS"
  label[label %in% "biblio.volume"] <- "VL"
  label[label %in% "referenced_works"] <- "CR"
  # label[label %in% "keywords_display_name"] <- "DE"
  label[label %in% "keywords.display_name"] <- "DE"
  label[label %in% "abstract"] <- "AB"
  label[label %in% "concepts.display_name"] <- "ID"
  label[label %in% "topics.display_name"] <- "TOPICS"
  label[label %in% "sustainable_development_goals.display_name"] <- "SDG"
  label[label %in% "topics.field.display_name"] <- "WC"
  label[label %in% "mesh.descriptor_name"] <- "MESH"
  label[label %in% "referenced_works_count"] <- "NR"
  label[label %in% "language"] <- "LA"
  label[label %in% "authorships.author_position"] <- "AU_POSITION"
  # label[label %in% "authorships.countries"] <- "C1"
  label[label %in% "doi"] <- "DI"

  # New OpenAlex CSV format mappings (backward-compatible)
  label[label %in% "display_name"] <- "TI"
  label[label %in% "primary_topic.display_name"] <- "WC"
  label[label %in% "fwci"] <- "FWCI"
  label[label %in% "ids.pmid"] <- "PMID"
  label[label %in% "is_retracted"] <- "RETRACTED"
  label[label %in% "funders.display_name"] <- "FU"
  label[label %in% "publication_date"] <- "PD"
  label[label %in% "authorships.institutions.display_name"] <- "C1_NAMES"
  label[label %in% "authorships.institutions.id"] <- "C1_IDS"
  label[label %in% "authorships.is_corresponding"] <- "AU_CORRESPONDING"
  label[label %in% "best_oa_location.license"] <- "OA_LICENSE"
  label[label %in% "open_access.is_oa"] <- "OA"
  label[label %in% "open_access.oa_status"] <- "OA_STATUS"

  names(DATA) <- label
  return(DATA)
}

TrimMult <- function(x, char = " ") {
  return(gsub(paste0("^", char, "*|(?<=", char, ")", char, "|", char, "*$"),
    "", x,
    perl = T
  ))
}

# Funzione aggiornata per estrarre affiliazioni, includere i paesi e rimuovere i caratteri `"`
extract_collapsed_affiliations <- function(affiliations, id_oa) {
  map2_dfr(affiliations, id_oa, ~ {
    # Rimuovi `{}`, `;`, e `'`, quindi dividi per `raw_affiliation_string: `
    affil_split <- stringr::str_split(stringr::str_replace_all(.x, "\\{|\\}|\\'|;", ""), "raw_affiliation_string: ")[[1]]

    # Rimuovi stringhe vuote e processa ciascun elemento per ottenere nome, ID e paese
    affil_df <- map_df(affil_split[affil_split != ""], function(affil) {
      # Estrai il nome completo dell'affiliazione
      affiliation_name <- stringr::str_trim(stringr::str_extract(affil, "^.*?(?=, institution_ids|$)"))

      # Estrai `institution_ids` e rimuovi il prefisso `https://openalex.org/`
      institution_ids <- stringr::str_extract(affil, "(?<=institution_ids: \\[)[^\\]]+") %>%
        stringr::str_replace_all("https://openalex.org/", "")

      # Estrai il paese come ultima parola dopo l'ultima virgola della stringa
      # country <- stringr::str_trim(stringr::str_extract(affiliation_name, "(?<=, )[^,]+$"))

      tibble(
        article_id = .y, # Assegna l'ID dell'articolo a ogni riga
        affiliation_name = ifelse(is.na(affiliation_name) || affiliation_name == "", NA, affiliation_name),
        institution_ids = ifelse(is.na(institution_ids) || institution_ids == "", NA, institution_ids) # ,
        # countries = ifelse(is.na(country) || country == "", NA, country)
      )
    })

    # Collassa tutte le affiliazioni, gli ID istituzionali e i paesi in singole stringhe separate da `;`
    df <- tibble(
      id_oa = .y, # Includi l'ID dell'articolo
      C1 = ifelse(nrow(affil_df) == 0, NA, paste(affil_df$affiliation_name, collapse = "; ")),
      C1_ID = ifelse(nrow(affil_df) == 0, NA, paste(affil_df$institution_ids, collapse = "; ")) # ,
      # AU_CO = ifelse(nrow(affil_df) == 0, NA, paste(affil_df$countries[!is.na(affil_df$countries)], collapse = "; "))
    )

    # Rimozione dei caratteri `"` da tutte le colonne
    df <- df %>% mutate(across(everything(), ~ stringr::str_replace_all(., "\"", "")))
  })
}

# Funzione aggiornata per sostituire corresponding_author_ids e corresponding_institution_ids con nome e affiliazione
replace_corresponding_info <- function(data) {
  data$corresponding_author_ids <- unlist(lapply(strsplit(data$corresponding_author_ids, ";"), function(l) l[1]))
  data %>%
    rowwise() %>%
    mutate(
      # Trova il nome dell'autore corrispondente
      corresponding_author_name = ifelse(
        !is.na(corresponding_author_ids) && corresponding_author_ids != "",
        {
          # Ottieni la lista di autori e ID
          author_names <- stringr::str_split(AU, ";")[[1]]
          author_ids <- stringr::str_split(AU_ID, ";")[[1]]
          # Trova l'indice dell'ID autore corrispondente
          matching_index <- which(author_ids == corresponding_author_ids, "")
          if (length(matching_index) > 0) author_names[matching_index] else NA_character_
        },
        NA_character_
      ),

      # Trova l'affiliazione dell'autore corrispondente
      corresponding_author_affiliation = ifelse(
        !is.na(corresponding_institution_ids) && corresponding_institution_ids != "",
        {
          # Ottieni la lista di affiliazioni e ID
          institution_names <- stringr::str_split(C1, ";")[[1]]
          institution_ids <- stringr::str_split(C1_ID, ";")[[1]] %>% stringr::str_trim()
          # Rimuovi il prefisso dell'ID istituzionale per la corrispondenza
          corresponding_ids <- stringr::str_split(corresponding_institution_ids, ";")[[1]] %>%
            stringr::str_replace_all("https://openalex.org/", "")
          # Trova le affiliazioni corrispondenti e uniscile
          matching_affiliations <- institution_names[institution_ids %in% corresponding_ids]
          if (length(matching_affiliations) > 0) paste(matching_affiliations, collapse = "; ") else NA_character_
        },
        NA_character_
      ),

      # Trova la nazione dell'autore corrispondente
      corresponding_author_country = ifelse(
        !is.na(corresponding_author_ids) && corresponding_author_ids != "",
        {
          # Ottieni la lista delle nazioni e degli ID
          author_countries <- stringr::str_split(AU_CO, ";")[[1]]
          author_ids <- stringr::str_split(AU_ID, ";")[[1]]
          # Trova l'indice dell'ID autore corrispondente
          matching_index <- which(author_ids == corresponding_author_ids)
          if (length(matching_index) > 0) author_countries[matching_index] else NA_character_
        },
        NA_character_
      )
    ) %>%
    # Rimuove le colonne originali degli ID e rinomina
    select(-corresponding_author_ids, -corresponding_institution_ids) %>%
    rename(
      corresponding_author_name = corresponding_author_name,
      RP = corresponding_author_affiliation,
      AU1_CO = corresponding_author_country
    ) %>%
    ungroup()
}

convert_iso2_to_country <- function(df_column, countries_df) {
  # Crea una named vector per la conversione
  iso_to_country <- setNames(countries_df$countries, countries_df$iso2)
  
  # Funzione per tradurre un singolo elemento (es: "IT;IT;FR")
  translate_entry <- function(entry) {
    if (is.na(entry) || entry == "" || grepl("^;+\\s*$", entry)) return(NA)
    iso_codes <- unlist(strsplit(entry, ";"))
    country_names <- iso_to_country[iso_codes]
    # Rimuove NA e unifica nomi duplicati
    unique_names <- unique(na.omit(country_names))
    paste(unique_names, collapse = "; ")
  }
  
  # Applica la funzione a tutta la colonna
  sapply(df_column, translate_entry)
}

# ==============================================================================
# UI Component - WebOfScience-like Query Builder for OpenAlex (Collapsible)
# ==============================================================================

openAlexUI <- function() {
  tagList(
    fluidRow(
      column(
        width = 12,
        # Query Builder Box (Collapsible)
        wellPanel(
          id = "oaQueryPanel",

          # Header with collapse button
          div(
            style = "display: flex; justify-content: space-between; align-items: center; margin-bottom: 15px;",
            h4(
              "OpenAlex Data Collection",
              style = "font-weight: bold; margin: 0;"
            ),
            actionButton(
              "oaToggleQueryPanel",
              icon("chevron-up"),
              class = "btn-sm btn-outline-secondary",
              style = "padding: 5px 10px;",
              title = "Collapse/Expand"
            )
          ),

          # API Key Warning Banner
          uiOutput("oaApiKeyWarning"),

          # Query content (collapsible)
          div(
            id = "oaQueryContent",

            # Query Builder Section (WebOfScience-like)
            div(
              id = "query-builder-container",
              style = "margin-bottom: 20px;",

              # Initial query row (NO operator column)
              div(
                id = "query-row-1",
                class = "query-row",
                style = "margin-bottom: 10px;",
                fluidRow(
                  column(1),
                  column(
                    width = 2,
                    selectInput(
                      "oaField_1",
                      NULL,
                      choices = c(
                        "All Fields" = "default",
                        "Title" = "title",
                        "Abstract" = "abstract",
                        "Title and Abstract" = "title_abstract",
                        "Author" = "author",
                        "Concept" = "concept"
                      ),
                      selected = "title_abstract",
                      width = "100%"
                    )
                  ),
                  column(
                    width = 8,
                    textInput(
                      "oaQuery_1",
                      NULL,
                      value = "",
                      placeholder = 'Example: "machine learning" OR AI',
                      width = "100%"
                    )
                  ),
                  column(
                    width = 1,
                    div(
                      id = "remove-button-1",
                      style = "visibility: hidden;",
                      actionButton(
                        "oaRemoveRow_1",
                        icon("minus-circle"),
                        class = "btn-danger btn-sm",
                        style = "margin-top: 0px;",
                        title = "Remove row"
                      )
                    )
                  )
                )
              )
            ),

            # Help text for search syntax
            div(
              style = "margin-bottom: 15px; font-size: 12px; color: #666;",
              HTML(
                "<strong>Search tips:</strong> Use quotes for exact phrases (e.g., \"science mapping\"). Boolean operators AND, OR, NOT must be UPPERCASE."
              )
            ),

            # Add row, date range, and Search buttons
            div(
              style = "margin-bottom: 20px; display: flex; align-items: center; gap: 10px;",
              actionButton(
                "oaAddRow",
                label = "+ Add row",
                icon = icon("plus"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "oaToggleDateRange",
                label = "+ Date range",
                icon = icon("calendar"),
                class = "btn-outline-primary btn-sm"
              ),
              actionButton(
                "oaSearchQuery",
                label = "Search",
                icon = icon("search"),
                class = "btn-info btn-sm",
                style = "margin-left: auto;"
              )
            ),

            # Date Range Section (initially visible)
            div(
              id = "oaDateRangeSection",
              style = "display: block;", # Visibile di default
              wellPanel(
                style = "background-color: #f8f9fa;",
                h5("Date Range Filter", style = "font-weight: bold;"),
                fluidRow(
                  column(
                    width = 6,
                    numericInput(
                      "oaYearFrom",
                      "From Year:",
                      value = 1985,
                      min = 1900,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  ),
                  column(
                    width = 6,
                    numericInput(
                      "oaYearTo",
                      "To Year:",
                      value = as.integer(format(Sys.Date(), "%Y")),
                      min = 1900,
                      max = as.integer(format(Sys.Date(), "%Y"))
                    )
                  )
                )
              )
            ),

            # Advanced filters toggle
            actionButton(
              "oaToggleAdvanced",
              label = "Advanced filters",
              icon = icon("filter"),
              class = "btn-link",
              style = "padding: 5px; margin-bottom: 10px;"
            ),

            # Additional Filters Section (initially visible)
            div(
              id = "oaAdvancedSection",
              style = "display: block;", # Visibile di default
              wellPanel(
                style = "background-color: #f8f9fa;",
                fluidRow(
                  column(
                    width = 4,
                    selectizeInput(
                      "oaDocType",
                      "Document Type:",
                      choices = c(
                        "All" = "",
                        "Article" = "article",
                        "Book Chapter" = "book-chapter",
                        "Book" = "book",
                        "Dataset" = "dataset",
                        "Dissertation" = "dissertation",
                        "Editorial" = "editorial",
                        "Letter" = "letter",
                        "Preprint" = "preprint",
                        "Review" = "review"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    selectizeInput(
                      "oaLanguage",
                      "Language:",
                      choices = c(
                        "All" = "",
                        "English" = "en",
                        "Spanish" = "es",
                        "French" = "fr",
                        "German" = "de",
                        "Italian" = "it",
                        "Portuguese" = "pt",
                        "Chinese" = "zh",
                        "Japanese" = "ja"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 4,
                    numericInput(
                      "oaMaxRecords",
                      "Max Records:",
                      value = 1000,
                      min = 1,
                      max = 10000,
                      step = 100
                    )
                  )
                ),
                fluidRow(
                  column(
                    width = 3,
                    selectizeInput(
                      "oaOpenAccess",
                      "Open Access:",
                      choices = c(
                        "All" = "",
                        "Gold" = "gold",
                        "Green" = "green",
                        "Hybrid" = "hybrid",
                        "Bronze" = "bronze",
                        "Diamond" = "diamond",
                        "Closed" = "closed"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaSDG",
                      "SDG:",
                      choices = c(
                        "All" = "",
                        "1 - No Poverty" = "1",
                        "2 - Zero Hunger" = "2",
                        "3 - Good Health and Well-being" = "3",
                        "4 - Quality Education" = "4",
                        "5 - Gender Equality" = "5",
                        "6 - Clean Water and Sanitation" = "6",
                        "7 - Affordable and Clean Energy" = "7",
                        "8 - Decent Work and Economic Growth" = "8",
                        "9 - Industry, Innovation and Infrastructure" = "9",
                        "10 - Reduced Inequalities" = "10",
                        "11 - Sustainable Cities and Communities" = "11",
                        "12 - Responsible Consumption and Production" = "12",
                        "13 - Climate Action" = "13",
                        "14 - Life Below Water" = "14",
                        "15 - Life on Land" = "15",
                        "16 - Peace, Justice and Strong Institutions" = "16",
                        "17 - Partnerships for the Goals" = "17"
                      ),
                      selected = "",
                      multiple = TRUE
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaKeyword",
                      "Keyword:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaDomain",
                      "Domain:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  ),
                  column(
                    width = 3,
                    selectizeInput(
                      "oaJournal",
                      "Journal:",
                      choices = NULL,
                      multiple = TRUE,
                      options = list(placeholder = "Enter a query first...")
                    )
                  )
                ),
                fluidRow(
                  column(
                    width = 6, offset = 6,
                    radioButtons(
                      "oaDropdownSort",
                      NULL,
                      choices = c("Sort by count" = "count", "Sort A-Z" = "alpha"),
                      selected = "count",
                      inline = TRUE
                    )
                  )
                )
              )
            ),
            # Action buttons section
            div(
              style = "margin-top: 10px; display: flex; justify-content: space-between; align-items: center;",

              # Left side: Reactive query count message + working indicator
              div(
                style = "display: flex; align-items: center; gap: 8px;",
                uiOutput("oaQueryCountInfo"),
                span(
                  id = "oaWorkingLabel",
                  style = "display: none; color: #dc3545; font-size: 13px; font-weight: 600;",
                  icon("spinner", class = "fa-spin", style = "margin-right: 4px;"),
                  "working..."
                )
              ),

              # Right side: Refs checkbox, Clear and Search buttons
              div(
                style = "display: flex; align-items: center; gap: 10px;",
                div(
                  style = "margin-bottom: 0;",
                  checkboxInput(
                    "oaFetchRefs",
                    "Download cited references",
                    value = FALSE
                  ),
                  conditionalPanel(
                    condition = "input.oaFetchRefs == true",
                    div(
                      style = "margin-left: 5px; margin-top: -5px; font-size: 12px;",
                      radioButtons(
                        "oaRefsFilter",
                        NULL,
                        choices = c(
                          "Only cited > 1 (faster)" = "multiple",
                          "All references" = "all"
                        ),
                        selected = "multiple",
                        inline = TRUE
                      )
                    )
                  )
                ),
                actionButton(
                  "oaClearQuery",
                  "Clear",
                  icon = icon("times"),
                  class = "btn-outline-secondary"
                ),
                actionButton(
                  "oaFetchData",
                  "Download",
                  icon = icon("download"),
                  class = "btn-primary"
                )
              )
            )
          )
        )
      )
    ),

    # Data Display Section (shown when data is available)
    conditionalPanel(
      condition = "output.oaDataAvailable",
      fluidRow(
        column(
          width = 12,
          wellPanel(
            h4("Downloaded Collection", style = "font-weight: bold;"),

            # Progress and results info
            uiOutput("oaProgressInfo"),

            # Export buttons
            div(
              style = "margin-bottom: 15px; margin-top: 15px;",
              downloadButton(
                "oaDownloadExcel",
                "Export to Excel",
                icon = icon("file-excel"),
                class = "btn-success"
              ),
              downloadButton(
                "oaDownloadRData",
                "Export to RData",
                icon = icon("database"),
                class = "btn-info",
                style = "margin-left: 10px;"
              )
            ),

            # Data table output
            div(
              style = "margin-top: 15px;",
              uiOutput("oaDataTable")
            )
          )
        )
      )
    ),

    # JavaScript for collapsible functionality
    tags$script(HTML(
      "
  $(document).ready(function() {
    // Initialize collapsed state
    var isCollapsed = false;
    
    // Initialize visibility states
    var dateRangeVisible = true;
    var advancedVisible = true;
    
    // Handle toggle button click for main panel
    $('#oaToggleQueryPanel').on('click', function() {
      isCollapsed = !isCollapsed;
      $('#oaQueryContent').slideToggle(300);
      
      // Update button icon
      if (isCollapsed) {
        $(this).html('<i class=\"fa fa-chevron-down\"></i>');
      } else {
        $(this).html('<i class=\"fa fa-chevron-up\"></i>');
      }
    });
    
    // Handle date range toggle
    $('#oaToggleDateRange').on('click', function() {
      dateRangeVisible = !dateRangeVisible;
      $('#oaDateRangeSection').slideToggle(300);
      
      // Update button text
      if (dateRangeVisible) {
        $(this).html('<i class=\"fa fa-calendar\"></i> - Date range');
      } else {
        $(this).html('<i class=\"fa fa-calendar\"></i> + Date range');
      }
      
      // Notify Shiny
      Shiny.setInputValue('oaShowDateRange', dateRangeVisible);
    });
    
    // Handle advanced filters toggle
    $('#oaToggleAdvanced').on('click', function() {
      advancedVisible = !advancedVisible;
      $('#oaAdvancedSection').slideToggle(300);
      
      // Update button text
      if (advancedVisible) {
        $(this).html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
      } else {
        $(this).html('<i class=\"fa fa-filter\"></i> Show advanced filters');
      }
      
      // Notify Shiny
      Shiny.setInputValue('oaShowAdvanced', advancedVisible);
    });
    
    // Set initial button labels
    $('#oaToggleDateRange').html('<i class=\"fa fa-calendar\"></i> - Date range');
    $('#oaToggleAdvanced').html('<i class=\"fa fa-filter\"></i> Hide advanced filters');
  });
  
  // Custom message handler to collapse panel when data is loaded
  Shiny.addCustomMessageHandler('oaCollapseQueryPanel', function(message) {
    $('#oaQueryContent').slideUp(300);
    $('#oaToggleQueryPanel').html('<i class=\"fa fa-chevron-down\"></i>');
  });

  // Show 'working...' indicator when Shiny is busy (scoped to OA panel)
  $(document).on('shiny:busy', function() {
    if ($('#oaQueryPanel').is(':visible')) {
      $('#oaWorkingLabel').show();
    }
  });
  $(document).on('shiny:idle', function() {
    $('#oaWorkingLabel').hide();
  });
"
    ))
  )
}


# ==============================================================================
# Helper Functions - OpenAlex Reference Resolution
# ==============================================================================
# Constants and small helpers (.journal_word_abbrev, .journal_stop_words,
# abbreviate_source_name, format_cr_author, build_single_cr) live in
# R/resolveOAReferences.R so they can be shared with completeMetadata().
# Aliased into the biblioshiny global env so the rest of this file is unchanged.
.journal_word_abbrev   <- bibliometrix:::.journal_word_abbrev
.journal_stop_words    <- bibliometrix:::.journal_stop_words
abbreviate_source_name <- bibliometrix:::abbreviate_source_name
format_cr_author       <- bibliometrix:::format_cr_author
build_single_cr        <- bibliometrix:::build_single_cr

# Resolve all references for a set of OpenAlex works
# Returns a named list: openalex_id -> CR string
resolve_openalex_references <- function(oa_data, progress_id = NULL,
                                        progress_bar_id = NULL,
                                        progress_pct_id = NULL,
                                        only_multiple = FALSE) {
  # Collect all referenced work IDs
  all_refs <- unlist(lapply(oa_data, function(w) w$referenced_works))
  all_refs <- all_refs[!is.na(all_refs) & all_refs != ""]

  if (length(all_refs) == 0) return(list())

  # Filter: keep only references cited more than once
  if (isTRUE(only_multiple)) {
    ref_freq <- table(all_refs)
    all_ref_ids <- names(ref_freq[ref_freq > 1])
  } else {
    all_ref_ids <- unique(all_refs)
  }

  if (length(all_ref_ids) == 0) return(list())

  # Update progress with actual count after filtering
  if (!is.null(progress_id)) {
    filter_label <- if (isTRUE(only_multiple)) " (cited > 1)" else ""
    shinyjs::html(
      progress_id,
      sprintf(
        "Resolving %s unique references%s...",
        format(length(all_ref_ids), big.mark = ","),
        filter_label
      )
    )
  }

  # Batch fetch referenced works (50 per batch to stay within URL limits)
  batch_size <- 50
  n_batches <- ceiling(length(all_ref_ids) / batch_size)
  ref_lookup <- list()

  for (i in seq_len(n_batches)) {
    start_idx <- (i - 1) * batch_size + 1
    end_idx <- min(i * batch_size, length(all_ref_ids))
    batch_ids <- all_ref_ids[start_idx:end_idx]

    pct <- round(i / n_batches * 100)

    # Update progress bar if available
    if (!is.null(progress_bar_id)) {
      shinyjs::runjs(sprintf(
        "$('#%s').css('width', '%d%%');", progress_bar_id, pct
      ))
    }
    if (!is.null(progress_pct_id)) {
      shinyjs::html(progress_pct_id, sprintf(
        "Batch %d/%d (%d%%)", i, n_batches, pct
      ))
    } else if (!is.null(progress_id)) {
      # Fallback: text-only progress
      shinyjs::html(
        progress_id,
        sprintf(
          "Resolving references... batch %d/%d (%s unique refs)",
          i, n_batches, format(length(all_ref_ids), big.mark = ",")
        )
      )
    }

    tryCatch({
      # Strip URL prefix for the filter
      short_ids <- gsub("https://openalex.org/", "", batch_ids)
      result <- openalexR::oa_fetch(
        entity = "works",
        openalex_id = paste(short_ids, collapse = "|"),
        output = "list",
        verbose = FALSE
      )

      if (is.list(result)) {
        for (work in result) {
          if (!is.null(work$id)) {
            cr <- build_single_cr(work)
            if (!is.na(cr)) ref_lookup[[work$id]] <- cr
          }
        }
      }
    }, error = function(e) {
      # Skip failed batches silently
    })
  }

  return(ref_lookup)
}

# Build CR column for each paper given oa_data and ref_lookup
build_cr_column <- function(oa_data, ref_lookup) {
  vapply(oa_data, function(work) {
    ref_ids <- work$referenced_works
    if (is.null(ref_ids) || length(ref_ids) == 0) return(NA_character_)

    cr_strings <- unlist(lapply(ref_ids, function(rid) {
      if (rid %in% names(ref_lookup)) ref_lookup[[rid]] else NULL
    }))

    if (length(cr_strings) == 0) return(NA_character_)
    paste(cr_strings, collapse = "; ")
  }, character(1))
}


# ==============================================================================
# Server Component - WebOfScience-like Query Builder
# ==============================================================================

openAlexServer <- function(input, output, session, values) {
  # Reactive values to track query rows and toggle states
  queryRows <- reactiveVal(1)
  showDateRange <- reactiveVal(TRUE)
  showAdvanced <- reactiveVal(TRUE)
  downloadProgress <- reactiveVal(NULL)
  journalChoices <- reactiveVal(NULL)
  submittedQueryFilters <- reactiveVal(NULL) # keyword filters submitted via Search button

  # API Key Warning Banner
  output$oaApiKeyWarning <- renderUI({
    # React to changes in the API key value
    values$oaApiKey
    oa_apikey <- Sys.getenv("openalexR.apikey", unset = "")
    if (oa_apikey == "") {
      div(
        style = "margin-bottom: 15px; padding: 12px 15px; background-color: #fff3cd; border-radius: 5px; border-left: 4px solid #f39c12; display: flex; align-items: center;",
        icon("exclamation-triangle", style = "color: #f39c12; margin-right: 10px; font-size: 18px;"),
        tags$span(
          style = "color: #856404; font-size: 13px;",
          HTML(
            "<strong>No API key configured.</strong> Without an API key you are limited to 100 credits/day (testing only). Go to <strong>Settings</strong> to set your free OpenAlex API key (100,000 credits/day)."
          )
        )
      )
    }
  })

  # Disable Keyword/Domain/Journal dropdowns at startup (no data yet)
  observeEvent(TRUE, {
    shinyjs::disable("oaKeyword")
    shinyjs::disable("oaDomain")
    shinyjs::disable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")
  }, once = TRUE)

  # Toggle date range visibility
  observeEvent(input$oaToggleDateRange, {
    current <- showDateRange()
    showDateRange(!current)
    session$sendCustomMessage('oaToggleDateRange', !current)
  })

  # Toggle advanced filters visibility
  observeEvent(input$oaToggleAdvanced, {
    current <- showAdvanced()
    showAdvanced(!current)
    session$sendCustomMessage('oaToggleAdvanced', !current)
  })

  # Add new query row
  observeEvent(input$oaAddRow, {
    currentRows <- queryRows()
    newRowId <- currentRows + 1
    queryRows(newRowId)

    # Insert new row in UI (WITH operator column)
    insertUI(
      selector = "#query-builder-container",
      where = "beforeEnd",
      ui = div(
        id = paste0("query-row-", newRowId),
        class = "query-row",
        style = "margin-bottom: 10px;",
        fluidRow(
          column(
            width = 1,
            selectInput(
              paste0("oaOperator_", newRowId),
              NULL,
              choices = c("And" = "AND", "Or" = "OR", "Not" = "NOT"),
              selected = "AND",
              width = "100%"
            )
          ),
          column(
            width = 2,
            selectInput(
              paste0("oaField_", newRowId),
              NULL,
              choices = c(
                "All Fields" = "default",
                "Title" = "title",
                "Abstract" = "abstract",
                "Title and Abstract" = "title_abstract",
                "Author" = "author",
                "Concept" = "concept"
              ),
              selected = "default",
              width = "100%"
            )
          ),
          column(
            width = 8,
            textInput(
              paste0("oaQuery_", newRowId),
              NULL,
              value = "",
              placeholder = 'Example: "deep learning"',
              width = "100%"
            )
          ),
          column(
            width = 1,
            actionButton(
              paste0("oaRemoveRow_", newRowId),
              icon("minus-circle"),
              class = "btn-danger btn-sm",
              style = "margin-top: 0px;",
              title = "Remove row"
            )
          )
        )
      )
    )

    # Make remove button visible for first row if adding second row
    if (newRowId == 2) {
      runjs("$('#remove-button-1').css('visibility', 'visible');")
    }

    # Add observer for the remove button of the new row
    local({
      rowId <- newRowId
      observeEvent(input[[paste0("oaRemoveRow_", rowId)]], {
        removeUI(selector = paste0("#query-row-", rowId))

        # Check if only one row remains
        remainingRows <- length(
          lapply(1:queryRows(), function(i) {
            input[[paste0("oaQuery_", i)]]
          })
        )

        if (remainingRows == 1) {
          runjs("$('#remove-button-1').css('visibility', 'hidden');")
        }
      })
    })
  })

  # Clear all query fields
  observeEvent(input$oaClearQuery, {
    # Reset all query inputs
    for (i in 1:queryRows()) {
      updateTextInput(session, paste0("oaQuery_", i), value = "")
      updateSelectInput(session, paste0("oaField_", i), selected = "default")
      if (i > 1) {
        updateSelectInput(session, paste0("oaOperator_", i), selected = "AND")
      }
    }

    # Reset filters
    updateNumericInput(session, "oaYearFrom", value = 2020)
    updateNumericInput(
      session,
      "oaYearTo",
      value = as.integer(format(Sys.Date(), "%Y"))
    )
    updateSelectizeInput(session, "oaDocType", selected = "")
    updateSelectizeInput(session, "oaLanguage", selected = "")
    updateNumericInput(session, "oaMaxRecords", value = 1000)
    updateSelectizeInput(session, "oaOpenAccess", selected = "")
    updateSelectizeInput(session, "oaSDG", selected = "")
    updateSelectizeInput(session, "oaKeyword", choices = character(0), selected = character(0))
    updateSelectizeInput(session, "oaDomain", choices = character(0), selected = character(0))
    updateSelectizeInput(session, "oaJournal", choices = character(0), selected = character(0))
    updateRadioButtons(session, "oaDropdownSort", selected = "count")
    updateCheckboxInput(session, "oaFetchRefs", value = FALSE)
    updateRadioButtons(session, "oaRefsFilter", selected = "multiple")
    journalChoices(NULL)
    keywordData(NULL)
    domainData(NULL)
    journalData(NULL)
    submittedQueryFilters(NULL)
  })

  # Search button: submit keyword query and activate reactive filters
  observeEvent(input$oaSearchQuery, {
    # Collect all query rows
    queryList <- list()
    for (i in 1:queryRows()) {
      queryText <- input[[paste0("oaQuery_", i)]]
      if (!is.null(queryText) && queryText != "") {
        queryList[[length(queryList) + 1]] <- list(
          query = queryText,
          field = input[[paste0("oaField_", i)]],
          operator = if (i > 1) input[[paste0("oaOperator_", i)]] else NULL
        )
      }
    }

    if (length(queryList) == 0) {
      showNotification("Please enter at least one search term.", type = "warning", duration = 4)
      return()
    }

    search_filters <- build_query_filters(queryList)

    # Add date range filter
    if (showDateRange()) {
      if (!is.na(input$oaYearFrom)) {
        search_filters$from_publication_date <- paste0(input$oaYearFrom, "-01-01")
      }
      if (!is.na(input$oaYearTo)) {
        search_filters$to_publication_date <- paste0(input$oaYearTo, "-12-31")
      }
    }

    submittedQueryFilters(search_filters)
  })

  # Helper function to get field name for filter
  # Fields that can be filtered directly on the /works endpoint
  get_filter_name <- function(field) {
    switch(
      field,
      "title" = "title.search",
      "abstract" = "abstract.search",
      "title_abstract" = "title_and_abstract.search",
      "default" = "default.search",
      NULL
    )
  }

  # Entity-based fields require a two-step approach:
  # 1) Search the entity endpoint to resolve names to IDs
  # 2) Filter /works by those IDs
  entity_field_config <- list(
    "author" = list(
      entity = "authors",
      works_filter = "authorships.author.id"
    ),
    "concept" = list(
      entity = "concepts",
      works_filter = "concepts.id"
    )
  )

  # Resolve entity-based search terms to OpenAlex IDs
  resolve_entity_ids <- function(field, query_text) {
    config <- entity_field_config[[field]]
    if (is.null(config)) return(NULL)

    tryCatch({
      results <- openalexR::oa_fetch(
        entity = config$entity,
        display_name.search = query_text,
        verbose = FALSE
      )
      if (!is.null(results) && nrow(results) > 0) {
        ids <- gsub("https://openalex.org/", "", results$id)
        return(list(
          works_filter = config$works_filter,
          ids = ids
        ))
      }
      return(NULL)
    }, error = function(e) {
      return(NULL)
    })
  }

  # Helper function to build query filters
  build_query_filters <- function(queryList) {
    if (length(queryList) == 0) {
      return(list())
    }

    search_filters <- list()

    # Separate direct filters from entity-based filters
    direct_queries <- list()
    entity_queries <- list()

    for (i in seq_along(queryList)) {
      item <- queryList[[i]]
      field <- item$field
      filter_name <- get_filter_name(field)

      if (!is.null(filter_name)) {
        # Direct works filter (title, abstract, etc.)
        if (is.null(direct_queries[[filter_name]])) {
          direct_queries[[filter_name]] <- list(
            terms = c(item$query),
            operators = c()
          )
        } else {
          direct_queries[[filter_name]]$operators <- c(
            direct_queries[[filter_name]]$operators,
            item$operator
          )
          direct_queries[[filter_name]]$terms <- c(
            direct_queries[[filter_name]]$terms,
            item$query
          )
        }
      } else if (field %in% names(entity_field_config)) {
        # Entity-based filter (author, concept)
        if (is.null(entity_queries[[field]])) {
          entity_queries[[field]] <- list(
            terms = c(item$query),
            operators = c()
          )
        } else {
          entity_queries[[field]]$operators <- c(
            entity_queries[[field]]$operators,
            item$operator
          )
          entity_queries[[field]]$terms <- c(
            entity_queries[[field]]$terms,
            item$query
          )
        }
      }
    }

    # Build direct filters
    for (filter_name in names(direct_queries)) {
      field_data <- direct_queries[[filter_name]]

      if (length(field_data$terms) == 1) {
        search_filters[[filter_name]] <- field_data$terms[1]
      } else {
        query_parts <- c(field_data$terms[1])
        for (j in seq_along(field_data$operators)) {
          query_parts <- c(query_parts, field_data$operators[j], field_data$terms[j + 1])
        }
        search_filters[[filter_name]] <- paste(query_parts, collapse = " ")
      }
    }

    # Resolve entity-based filters to IDs
    for (field in names(entity_queries)) {
      field_data <- entity_queries[[field]]
      all_ids <- c()

      for (term in field_data$terms) {
        resolved <- resolve_entity_ids(field, term)
        if (!is.null(resolved)) {
          all_ids <- c(all_ids, resolved$ids)
        }
      }

      if (length(all_ids) > 0) {
        config <- entity_field_config[[field]]
        search_filters[[config$works_filter]] <- paste(unique(all_ids), collapse = "|")
      }
    }

    return(search_filters)
  }

  # Reactive: base search filters (without Topic and Journal)
  # Keyword filters come from submittedQueryFilters (set by Search button).
  # Advanced filters (DocType, Language, etc.) are reactive after search is submitted.
  baseSearchFilters <- reactive({
    search_filters <- submittedQueryFilters()
    if (is.null(search_filters)) return(NULL)

    # Add advanced filters (except Topic and Journal) - these are reactive
    if (showAdvanced()) {
      if (!is.null(input$oaDocType) && length(input$oaDocType) > 0) {
        doc_types <- input$oaDocType[input$oaDocType != ""]
        if (length(doc_types) > 0) search_filters$type <- doc_types
      }
      if (!is.null(input$oaLanguage) && length(input$oaLanguage) > 0) {
        languages <- input$oaLanguage[input$oaLanguage != ""]
        if (length(languages) > 0) search_filters$language <- languages
      }
      if (!is.null(input$oaOpenAccess) && length(input$oaOpenAccess) > 0) {
        oa_statuses <- input$oaOpenAccess[input$oaOpenAccess != ""]
        if (length(oa_statuses) > 0) search_filters$open_access.oa_status <- oa_statuses
      }
      if (!is.null(input$oaSDG) && length(input$oaSDG) > 0) {
        sdg_ids <- input$oaSDG[input$oaSDG != ""]
        if (length(sdg_ids) > 0) {
          search_filters$sustainable_development_goals.id <- paste0("https://metadata.un.org/sdg/", sdg_ids)
        }
      }
    }

    return(search_filters)
  })

  # Debounced version of baseSearchFilters for group_by observer (2s delay)
  baseSearchFiltersDebounced <- debounce(baseSearchFilters, 2000)

  # Raw data from group_by API (data.frames with key, key_display_name, count)
  keywordData <- reactiveVal(NULL)
  domainData <- reactiveVal(NULL)
  journalData <- reactiveVal(NULL)

  # Helper: build sorted named vector from group_by result
  sort_groupby_choices <- function(data, sort_order) {
    if (is.null(data) || nrow(data) == 0) return(character(0))
    if (sort_order == "alpha") {
      data <- data[order(data$key_display_name), ]
    } else {
      # Sort descending by count (ensure numeric comparison)
      data <- data[order(as.numeric(data$count), decreasing = TRUE), ]
    }
    setNames(data$key, paste0(data$key_display_name, " (", data$count, ")"))
  }

  # Observer: fetch Keyword, Domain and Journal raw data via group_by
  observe({
    base_filters <- baseSearchFiltersDebounced()

    if (is.null(base_filters)) {
      keywordData(NULL)
      domainData(NULL)
      journalData(NULL)
      shinyjs::disable("oaKeyword")
      shinyjs::disable("oaDomain")
      shinyjs::disable("oaJournal")
      shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
      shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
      shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")
      return()
    }

    # Grey out and disable while loading
    shinyjs::disable("oaKeyword")
    shinyjs::disable("oaDomain")
    shinyjs::disable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '0.5');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '0.5');")

    tryCatch({
      keyword_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "keywords.id", verbose = FALSE), base_filters)
      )
      if (!is.null(keyword_result) && nrow(keyword_result) > 0) {
        keywordData(keyword_result)
      } else {
        keywordData(NULL)
      }
    }, error = function(e) {
      keywordData(NULL)
    })

    tryCatch({
      domain_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "topics.domain.id", verbose = FALSE), base_filters)
      )
      if (!is.null(domain_result) && nrow(domain_result) > 0) {
        domainData(domain_result)
      } else {
        domainData(NULL)
      }
    }, error = function(e) {
      domainData(NULL)
    })

    tryCatch({
      journal_result <- do.call(
        openalexR::oa_fetch,
        c(list(entity = "works", group_by = "primary_location.source.id", verbose = FALSE), base_filters)
      )
      if (!is.null(journal_result) && nrow(journal_result) > 0) {
        journalData(journal_result)
      } else {
        journalData(NULL)
      }
    }, error = function(e) {
      journalData(NULL)
    })

    # Re-enable after loading completes
    shinyjs::enable("oaKeyword")
    shinyjs::enable("oaDomain")
    shinyjs::enable("oaJournal")
    shinyjs::runjs("$('#oaKeyword').closest('.form-group').css('opacity', '1');")
    shinyjs::runjs("$('#oaDomain').closest('.form-group').css('opacity', '1');")
    shinyjs::runjs("$('#oaJournal').closest('.form-group').css('opacity', '1');")
  })

  # Observer: update dropdown choices when raw data or sort order changes
  observe({
    kd <- keywordData()
    dd <- domainData()
    jd <- journalData()
    sort_order <- input$oaDropdownSort

    current_keywords <- isolate(input$oaKeyword)
    current_domains <- isolate(input$oaDomain)
    current_journals <- isolate(input$oaJournal)

    kc <- sort_groupby_choices(kd, sort_order)
    if (length(kc) > 0) {
      valid_keywords <- current_keywords[current_keywords %in% kc]
      updateSelectizeInput(session, "oaKeyword", choices = kc, selected = valid_keywords)
    } else {
      updateSelectizeInput(session, "oaKeyword", choices = character(0), selected = character(0))
    }

    dc <- sort_groupby_choices(dd, sort_order)
    if (length(dc) > 0) {
      valid_domains <- current_domains[current_domains %in% dc]
      updateSelectizeInput(session, "oaDomain", choices = dc, selected = valid_domains)
    } else {
      updateSelectizeInput(session, "oaDomain", choices = character(0), selected = character(0))
    }

    jc <- sort_groupby_choices(jd, sort_order)
    if (length(jc) > 0) {
      journalChoices(jc)
      valid_journals <- current_journals[current_journals %in% jc]
      updateSelectizeInput(session, "oaJournal", choices = jc, selected = valid_journals)
    } else {
      journalChoices(NULL)
      updateSelectizeInput(session, "oaJournal", choices = character(0), selected = character(0))
    }
  })

  # Reactive: full search filters (base + Keyword + Domain + Journal)
  fullSearchFilters <- reactive({
    base_filters <- baseSearchFilters()
    if (is.null(base_filters)) return(NULL)

    search_filters <- base_filters

    if (showAdvanced()) {
      if (!is.null(input$oaKeyword) && length(input$oaKeyword) > 0) {
        keywords <- input$oaKeyword[input$oaKeyword != ""]
        if (length(keywords) > 0) search_filters$keywords.id <- keywords
      }
      if (!is.null(input$oaDomain) && length(input$oaDomain) > 0) {
        domains <- input$oaDomain[input$oaDomain != ""]
        if (length(domains) > 0) {
          # Extract numeric ID from full URL (e.g., "https://openalex.org/domains/2" -> "2")
          domains <- gsub("https://openalex.org/domains/", "", domains)
          search_filters$primary_topic.domain.id <- domains
        }
      }
      if (!is.null(input$oaJournal) && length(input$oaJournal) > 0) {
        journals <- input$oaJournal[input$oaJournal != ""]
        if (length(journals) > 0) search_filters$primary_location.source.id <- journals
      }
    }

    return(search_filters)
  })

  # Debounced version of fullSearchFilters for count query (1s delay)
  fullSearchFiltersDebounced <- debounce(fullSearchFilters, 1000)

  # Reactive query count - updates automatically when query changes
  output$oaQueryCountInfo <- renderUI({
    search_filters <- fullSearchFiltersDebounced()

    # If no query, show a hint
    if (is.null(search_filters)) {
      return(
        div(
          style = "display: inline-block; padding: 5px 10px; color: #6c757d; font-size: 13px; font-style: italic;",
          icon("info-circle"),
          " Enter search terms to see record count"
        )
      )
    }

    tryCatch(
      {
        # Get the count of available records
        count_result <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              count_only = TRUE,
              verbose = FALSE
            ),
            search_filters
          )
        )

        total_available <- as.integer(count_result$count)

        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #d4edda; color: #155724; border-radius: 4px; font-size: 13px; font-weight: 500;",
          icon("check-circle"),
          sprintf(" Found %s records", format(total_available, big.mark = ","))
        )
      },
      error = function(e) {
        div(
          style = "display: inline-block; padding: 5px 10px; background-color: #f8d7da; color: #721c24; border-radius: 4px; font-size: 13px;",
          icon("exclamation-circle"),
          " Error: ",
          e$message
        )
      }
    )
  })

  # Download data from OpenAlex
  observeEvent(input$oaFetchData, {
    # Check if API key or polite email is configured
    oa_email <- Sys.getenv("openalexR.mailto", unset = "")
    oa_apikey <- Sys.getenv("openalexR.apikey", unset = "")
    if (oa_email == "" && oa_apikey == "") {
      showModal(modalDialog(
        title = "OpenAlex API Not Configured",
        div(
          style = "padding: 10px;",
          p(
            icon("exclamation-triangle", style = "color: #f39c12; margin-right: 8px;"),
            tags$strong("No API key or email configured for OpenAlex API.")
          ),
          p("Without an API key, you are limited to 100 credits/day (testing only). A free API key provides 100,000 credits/day."),
          p("Please go to ", tags$strong("Settings"), " and set your OpenAlex API key or email address."),
          p(
            style = "color: #666; font-size: 12px; margin-top: 15px;",
            "You can still proceed, but we strongly recommend setting an API key for better performance."
          )
        ),
        easyClose = TRUE,
        footer = tagList(
          modalButton("Cancel"),
          actionButton("oaProceedWithoutEmail", "Proceed anyway", class = "btn-warning")
        )
      ))
      return()
    }

    # Run the actual search
    oaRunSearch(search_filters = fullSearchFilters())
  })

  # Allow proceeding without email after warning
  observeEvent(input$oaProceedWithoutEmail, {
    removeModal()
    oaRunSearch(search_filters = fullSearchFilters())
  })

  # Extracted search logic into a function
  oaRunSearch <- function(search_filters) {
    # Validate that at least one query exists
    if (is.null(search_filters)) {
      showModal(modalDialog(
        title = "Error",
        "Please enter at least one search term.",
        easyClose = TRUE,
        footer = modalButton("OK")
      ))
      return()
    }

    # Show progress modal with dynamic content
    showModal(modalDialog(
      title = "Downloading from OpenAlex",
      div(
        div(
          style = "text-align: center; margin: 20px 0;",
          icon("spinner", class = "fa-spin fa-3x")
        ),
        div(
          id = "download-progress-text",
          style = "text-align: center; font-size: 14px;",
          "Initializing query..."
        )
      ),
      footer = NULL,
      easyClose = FALSE
    ))

    tryCatch(
      {
        # Calculate pages needed
        max_records <- min(input$oaMaxRecords, 10000)
        pages_needed <- ceiling(max_records / 200)

        print("Search filters:")
        print(search_filters)

        # First, get the count of available records
        count_result <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              count_only = TRUE,
              verbose = FALSE
            ),
            search_filters
          )
        )

        total_available <- as.integer(count_result$count)
        records_to_download <- min(total_available, max_records)

        # Update progress modal with count information
        shinyjs::html(
          "download-progress-text",
          sprintf(
            "Found %s records. Downloading %s records...",
            format(total_available, big.mark = ","),
            format(records_to_download, big.mark = ",")
          )
        )

        # Fetch data from OpenAlex as list
        oa_data <- do.call(
          openalexR::oa_fetch,
          c(
            list(
              entity = "works",
              output = "list",
              abstract = TRUE,
              per_page = 200,
              pages = 1:pages_needed,
              verbose = TRUE
            ),
            search_filters
          )
        )

        # Update progress
        shinyjs::html(
          "download-progress-text",
          sprintf(
            "Converting %s records...",
            format(length(oa_data), big.mark = ",")
          )
        )

        # Create temporary directory if it doesn't exist
        temp_dir <- getWD()
        temp_file <- file.path(
          temp_dir,
          paste0("openalex_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".rdata")
        )

        # Save the list as .rdata file
        save(oa_data, file = temp_file)

        # Convert using bibliometrix convert2df function
        M <- convert2df(
          file = temp_file,
          dbsource = "openalex_api",
          format = "api"
        )

        # Clean up temporary file
        unlink(temp_file)

        # Limit to max records if necessary
        if (nrow(M) > max_records) {
          M <- M[1:max_records, ]
          oa_data <- oa_data[1:max_records]
        }

        # Resolve cited references if requested
        if (isTRUE(input$oaFetchRefs)) {
          only_multiple <- isTRUE(input$oaRefsFilter == "multiple")
          all_refs <- unlist(lapply(oa_data, function(w) w$referenced_works))
          all_refs <- all_refs[!is.na(all_refs) & all_refs != ""]
          n_total <- length(unique(all_refs))
          if (only_multiple) {
            ref_freq <- table(all_refs)
            n_to_resolve <- sum(ref_freq > 1)
          } else {
            n_to_resolve <- n_total
          }

          shinyjs::html(
            "download-progress-text",
            paste0(
              sprintf("Resolving %s of %s unique cited references...",
                      format(n_to_resolve, big.mark = ","),
                      format(n_total, big.mark = ",")),
              '<div style="margin-top: 10px; background-color: #e9ecef; border-radius: 4px; height: 20px; width: 100%;">',
              '<div id="download-progress-bar" style="background-color: #007bff; height: 100%; border-radius: 4px; width: 0%; transition: width 0.3s ease;"></div>',
              '</div>',
              '<div id="download-progress-pct" style="text-align: center; margin-top: 5px; font-size: 12px; color: #666;">0%</div>'
            )
          )

          ref_lookup <- resolve_openalex_references(oa_data,
            progress_id = "download-progress-text",
            progress_bar_id = "download-progress-bar",
            progress_pct_id = "download-progress-pct",
            only_multiple = only_multiple)

          if (length(ref_lookup) > 0) {
            shinyjs::html("download-progress-text", "Building citation strings...")
            cr_col <- build_cr_column(oa_data[1:nrow(M)], ref_lookup)
            M$CRids <- M$CR
            M$CR <- cr_col
          }
        }

        # Close progress modal
        removeModal()

        # Collapse query panel after successful download
        session$sendCustomMessage('oaCollapseQueryPanel', TRUE)

        # Show success message
        showModal(modalDialog(
          title = "Success",
          div(
            p(
              sprintf(
                "Successfully downloaded %s records from OpenAlex.",
                format(nrow(M), big.mark = ",")
              )
            ),
            if (total_available > max_records) {
              p(
                style = "color: #856404; background-color: #fff3cd; padding: 10px; border-radius: 5px;",
                sprintf(
                  "Note: %s total records found, but limited to %s records.",
                  format(total_available, big.mark = ","),
                  format(max_records, big.mark = ",")
                )
              )
            }
          ),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))

        values = initial(values)
        M <- M %>% mergeKeywords(force = F)
        # Update values
        values$data_source <- "OpenAlex"
        values$M <- M
        values$Morig = M
        values$SCdf <- wcTable(M)
        values$COdf <- countryTable(M)
        values$Histfield = "NA"
        values$results = list("NA")
        if (ncol(values$M) > 1) {
          values$rest_sidebar <- TRUE
        }
        if (ncol(values$M) > 1) {
          values$loadMenu <- "openalex_api"
          showModal(missingModalAPI(session))
        }
      },
      error = function(e) {
        removeModal()
        showModal(modalDialog(
          title = "Error",
          paste("An error occurred:", e$message),
          easyClose = TRUE,
          footer = modalButton("OK")
        ))
      }
    )
  }

  # Render data table
  output$oaDataTable <- renderUI({
    req(values$M)
    req(values$data_source == "OpenAlex")

    MData = as.data.frame(apply(values$M, 2, function(x) {
      substring(x, 1, 150)
    }))
    MData$DOI <-
      paste0(
        '<a href=\"https://doi.org/',
        MData$DI,
        '\" target=\"_blank\">',
        MData$DI,
        '</a>'
      )
    nome = c("DOI", names(MData)[-length(names(MData))])
    MData = MData[nome]
    renderBibliobox(
      MData,
      nrow = 3,
      filename = "Table",
      pagelength = TRUE,
      left = NULL,
      right = NULL,
      numeric = NULL,
      dom = TRUE,
      size = '70%',
      filter = "top",
      columnShort = NULL,
      columnSmall = NULL,
      round = 2,
      title = "",
      button = FALSE,
      escape = FALSE,
      selection = FALSE,
      scrollX = TRUE
    )
  })

  # Check if data is available for conditional panel
  output$oaDataAvailable <- reactive({
    !is.null(values$M) && values$data_source == "OpenAlex"
  })
  outputOptions(output, "oaDataAvailable", suspendWhenHidden = FALSE)

  # Display progress information
  output$oaProgressInfo <- renderUI({
    if (!is.null(values$M) && values$data_source == "OpenAlex") {
      div(
        class = "alert alert-success",
        icon("check-circle"),
        strong(" Data loaded: "),
        sprintf(
          "%s documents from OpenAlex",
          format(nrow(values$M), big.mark = ",")
        )
      )
    }
  })

  # Download handler for Excel
  output$oaDownloadExcel <- downloadHandler(
    filename = function() {
      paste0("openalex_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "OpenAlex")

      # Write data to Excel file
      openxlsx::write.xlsx(values$M, file = file)
    }
  )

  # Download handler for RData
  output$oaDownloadRData <- downloadHandler(
    filename = function() {
      paste0("openalex_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".RData")
    },
    content = function(file) {
      req(values$M)
      req(values$data_source == "OpenAlex")

      # Save data as RData file
      M <- values$M
      save(M, file = file)
    }
  )

  missingModalAPI <- function(session) {
    ns <- session$ns
    modalDialog(
      uiOutput("missingTitle"),
      uiOutput(ns("missingDataTable")),
      size = "l",
      easyClose = TRUE,
      footer = tagList(
        actionButton(
          label = "Advice",
          inputId = "missingMessage",
          icon = icon("exclamation-sign", lib = "glyphicon")
        ),
        actionButton(
          label = "Complete",
          inputId = "missingComplete",
          icon = icon("download", lib = "glyphicon")
        ),
        actionButton(
          label = "Undo",
          inputId = "missingCompleteUndo",
          icon = icon("repeat", lib = "glyphicon")
        ),
        actionButton(
          label = "Report",
          inputId = "missingReport",
          icon = icon("plus", lib = "glyphicon")
        ),
        actionButton(
          label = "Save",
          inputId = "missingDataSave",
          icon = icon("camera", lib = "glyphicon")
        ),
        modalButton(label = "Close")
      ),
    )
  }
}

utils::globalVariables(c(
  "AU", "n", "TC", "PY", "AU.x", "year", "Author",
  "TCpY", "freq"
))

#' Top-Authors' Productivity over Time
#'
#' It calculates and plots the author production (in terms of number of publications) over the time.
#' @param M is a bibliographic data frame obtained by \code{\link{convert2df}} function.
#' @param k is a integer. It is the number of top authors to analyze and plot. Default is \code{k = 10}.
#' @param graph is logical. If TRUE the function plots the author production over time graph. Default is \code{graph = TRUE}.
#' @return The function \code{authorProdOverTime} returns a list containing two objects:
#' \tabular{lll}{
#' \code{dfAU}  \tab   \tab is a data frame\cr
#' \code{dfpapersAU}\tab    \tab is a data frame\cr
#' \code{graph}   \tab   \tab a ggplot object}
#'
#' @examples
#' data(scientometrics, package = "bibliometrixData")
#' res <- authorProdOverTime(scientometrics, k = 10)
#' print(res$dfAU)
#' plot(res$graph)
#'
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis
#' @seealso \code{\link{summary}} method for class '\code{bibliometrix}'
#'
#' @export
#'
authorProdOverTime <- function(M, k = 10, graph = TRUE) {
  if (!("DI" %in% names(M))) {
    M$DI <- "NA"
  }
  M$TC <- as.numeric(M$TC)
  M$PY <- as.numeric(M$PY)
  M <- M[!is.na(M$PY), ] # remove rows with missing value in PY

  Y <- as.numeric(substr(Sys.time(), 1, 4))
  listAU <- (strsplit(M$AU, ";"))
  nAU <- lengths(listAU)
  df <- data.frame(AU = trimws(unlist(listAU)), SR = rep(M$SR, nAU))
  AU <- df %>%
    group_by(AU) %>%
    count() %>%
    arrange(desc(n)) %>%
    ungroup()
  k <- min(k, nrow(AU))
  AU <- AU %>%
    slice_head(n = k)

  df <- df %>%
    right_join(AU, by = "AU") %>%
    left_join(M, by = "SR") %>%
    select("AU.x", "PY", "TI", "SO", "DI", "TC") %>%
    mutate(TCpY = TC / (Y - PY + 1)) %>%
    group_by(AU.x) %>%
    mutate(n = length(AU.x)) %>%
    ungroup() %>%
    rename(
      Author = AU.x,
      year = PY,
      DOI = DI
    ) %>%
    arrange(desc(n), desc(year)) %>%
    select(-n)

  df2 <- dplyr::group_by(df, Author, year) %>%
    dplyr::summarise(freq = length(year), TC = sum(TC), TCpY = sum(TCpY)) %>%
    as.data.frame()

  df2$Author <- factor(df2$Author, levels = AU$AU[1:k])

  x <- c(0.5, 1.5 * k / 10)
  y <- c(min(df$year), min(df$year) + diff(range(df2$year)) * 0.125)

  data("logo", envir = environment())
  logo <- grid::rasterGrob(logo, interpolate = TRUE)

  g <- ggplot(df2, aes(x = Author, y = year, text = paste("Author: ", Author, "\nYear: ", year, "\nN. of Articles: ", freq, "\nTotal Citations per Year: ", round(TCpY, 2)))) +
    geom_point(aes(alpha = TCpY, size = freq), color = "dodgerblue4") +
    scale_size(range = c(2, 6)) +
    scale_alpha(range = c(0.3, 1)) +
    scale_y_continuous(breaks = seq(min(df2$year), max(df2$year), by = 2)) +
    guides(size = guide_legend(order = 1, "N.Articles"), alpha = guide_legend(order = 2, "TC per Year")) +
    theme(
      legend.position = "right",
      text = element_text(color = "#444444"),
      panel.background = element_rect(fill = "#FFFFFF"),
      plot.title = element_text(size = 24),
      axis.title = element_text(size = 14, color = "#555555"),
      axis.title.y = element_text(vjust = 1, angle = 90) # , face="bold")
      , axis.title.x = element_text(hjust = .95) # ,face="bold")
      , axis.text.x = element_text(face = "bold", angle = 90),
      axis.text.y = element_text(face = "bold")
      # ,axis.line.x = element_line(color="black", size=1)
      , axis.line.x = element_line(color = "grey50", linewidth = 0.5),
      panel.grid.major.x = element_blank(),
      panel.grid.major.y = element_line(linewidth = .2, color = "grey90")
    ) +
    # coord_fixed(ratio = 2/1) +
    labs(
      title = "Authors' Production over Time",
      x = "Author",
      y = "Year"
    ) +
    geom_line(data = df2, aes(x = Author, y = year, group = Author), linewidth = 1.0, color = "firebrick4", alpha = 0.3) +
    scale_x_discrete(limits = rev(levels(df2$Author))) +
    coord_flip() +
    annotation_custom(logo, xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2])


  df$DOI <- as.character(df$DOI)
  res <- list(dfAU = df2, dfPapersAU = df, graph = g)
  if (isTRUE(graph)) {
    plot(g)
  }
  return(res)
}

#' Author local citations
#'
#' It calculates local citations (LCS) of authors and documents of a bibliographic collection.
#'
#' Local citations measure how many times an author (or a document) included in this collection have been cited by the documents also included in the collection.
#'
#' @param M is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
#'        It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
#' @param sep is the field separator character. This character separates citations in each string of CR column of the bibliographic data frame. The default is \code{sep = ";"}.
#' @param fast.search is logical. If true, the function calculates local citations only for 25 percent top cited documents.
#' @param verbose is a logical.  If TRUE, results are printed on screen.
#' @return an object of \code{class} "list" containing author local citations and document local citations.
#'
#'
#' @examples
#'
#' data(scientometrics, package = "bibliometrixData")
#'
#' CR <- localCitations(scientometrics, sep = ";")
#'
#' CR$Authors[1:10, ]
#' CR$Papers[1:10, ]
#'
#' @seealso \code{\link{citations}} function for citation frequency distribution.
#' @seealso \code{\link{biblioAnalysis}} function for bibliometric analysis.
#' @seealso \code{\link{summary}} to obtain a summary of the results.
#' @seealso \code{\link{plot}} to draw some useful plots of the results.
#'
#' @export

localCitations <- function(M, fast.search = FALSE, sep = ";", verbose = FALSE) {
  M$TC[is.na(M$TC)] <- 0
  if (isTRUE(fast.search)) {
    loccit <- quantile(as.numeric(M$TC), 0.75, na.rm = TRUE)
  } else {
    loccit <- 1
  }

  H <- histNetwork(M, min.citations = loccit, sep = sep, network = FALSE, verbose = verbose)
  LCS <- H$histData
  M <- H$M
  rm(H)
  AU <- strsplit(M$AU, split = ";")
  n <- lengths(AU)

  df <- data.frame(AU = unlist(AU), LCS = rep(M$LCS, n))
  AU <- aggregate(df$LCS, by = list(df$AU), FUN = "sum")
  names(AU) <- c("Author", "LocalCitations")
  AU <- AU[order(-AU$LocalCitations), ]

  if ("SR" %in% names(M)) {
    LCS <- data.frame(Paper = M$SR, DOI = M$DI, Year = M$PY, LCS = M$LCS, GCS = M$TC)
    LCS <- LCS[order(-LCS$LCS), ]
  }
  CR <- list(Authors = AU, Papers = LCS, M = M)
  return(CR)
}

## OpenRouter AI function
openrouter_ai <- function(
  image = NULL,
  docs = NULL,
  prompt = "Explain these images",
  model = "google/gemma-3-27b-it:free", # Nota: usa 27b, il 32b potrebbe avere ID diverso
  image_type = "png",
  retry_503 = 5,
  api_key = NULL,
  outputSize = "medium"
) {
  library(httr2)
  library(base64enc)

  # 1. Setup API Key
  if (is.null(api_key)) {
    api_key <- Sys.getenv("OPENROUTER_API_KEY")
  }

  # 2. Configurazione Tokens (Semplificata per evitare 400)
  # Molti modelli free falliscono se chiedi troppi token in uscita
  max_val <- switch(
    outputSize,
    "small" = 1000,
    "medium" = 2000,
    "large" = 4000,
    2000
  )

  # 3. Costruzione Content (Standard rigoroso OpenAI)
  content_list <- list(
    list(type = "text", text = prompt)
  )

  if (!is.null(image)) {
    for (img_path in as.vector(image)) {
      if (file.exists(img_path)) {
        # Importante: usiamo una compressione base64 pulita
        img_b64 <- base64encode(img_path)
        content_list <- append(
          content_list,
          list(
            list(
              type = "image_url",
              image_url = list(
                url = paste0("data:image/", image_type, ";base64,", img_b64)
              )
            )
          )
        )
      }
    }
  }

  # 4. Request Body
  # Rimosso 'seed' e altri parametri opzionali che possono causare 400 su certi provider
  request_body <- list(
    model = model,
    messages = list(
      list(role = "user", content = content_list)
    ),
    max_tokens = max_val,
    temperature = 0.7
  )

  # 5. Invio Richiesta
  url <- "https://openrouter.ai/api/v1/chat/completions"

  req <- request(url) |>
    req_headers(
      "Authorization" = paste("Bearer", api_key),
      "HTTP-Referer" = "https://github.com/massimoaria/bibliometrix", # Referer specifico
      "X-Title" = "Biblioshiny Client",
      "Content-Type" = "application/json"
    ) |>
    req_body_json(request_body) |>
    req_timeout(120) |>
    req_retry(max_tries = retry_503)

  resp <- req_perform(req) |>
    resp_body_json()

  if (!is.null(resp$choices)) {
    return(resp$choices[[1]]$message$content)
  } else {
    return(paste("❌ Errore API:", resp$error$message))
  }
}

## gemini AI tools
gemini_ai <- function(
  image = NULL,
  docs = NULL,
  prompt = "Explain these images",
  model = "2.5-flash",
  image_type = "png",
  retry_503 = 5,
  api_key = NULL,
  outputSize = "medium"
) {
  mime_doc_types <- list(
    pdf = "application/pdf",
    txt = "text/plain",
    html = "text/html",
    csv = "text/csv",
    rtf = "text/rtf"
  )

  switch(
    outputSize,
    "small" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "medium" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 16384, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "large" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 32768, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    },
    "huge" = {
      generation_config <- list(
        temperature = 1,
        maxOutputTokens = 131072, #8192,
        topP = 0.95,
        topK = 40,
        seed = 1234
      )
    }
  )

  # # Default config
  # generation_config <- list(
  #   temperature = 1,
  #   maxOutputTokens = 16384,#8192,
  #   topP = 0.95,
  #   topK = 40,
  #   seed = 1234
  # )

  # Gemma models on the Gemini API: no "gemini-" prefix, no `seed`,
  # and a capped output window (open Gemma models max out at 8192 tokens).
  is_gemma <- startsWith(model, "gemma")
  if (is_gemma) {
    generation_config$seed <- NULL
    generation_config$maxOutputTokens <- min(
      generation_config$maxOutputTokens,
      8192
    )
  }

  # Build URL
  model_query <- if (is_gemma) {
    paste0(model, ":generateContent")
  } else {
    paste0("gemini-", model, ":generateContent")
  }
  url <- paste0(
    "https://generativelanguage.googleapis.com/v1beta/models/",
    model_query
  )
  if (is.null(api_key)) {
    api_key <- Sys.getenv("GEMINI_API_KEY")
  }

  # Base structure of parts
  parts <- list(list(text = prompt))

  # Handle images if provided
  if (!is.null(image)) {
    if (!is.vector(image)) {
      image <- as.vector(image)
    }
    mime_type <- paste0("image/", image_type)

    for (img_path in image) {
      if (!file.exists(img_path)) {
        return(paste0("❌ Error: Image file does not exist: ", img_path))
      }

      image_data <- tryCatch(
        base64enc::base64encode(img_path),
        error = function(e) {
          return(NULL)
        }
      )

      if (is.null(image_data)) {
        return(paste0("❌ Failed to encode image: ", img_path))
      }

      parts <- append(
        parts,
        list(
          list(
            inline_data = list(
              mime_type = mime_type,
              data = image_data
            )
          )
        )
      )
    }
  }

  # Handle documents if provided
  if (!is.null(docs)) {
    if (!is.vector(docs)) {
      docs <- as.vector(docs)
    }
    for (doc_path in docs) {
      if (!file.exists(doc_path)) {
        return(paste0("❌ Error: Document file does not exist: ", doc_path))
      }

      doc_data <- tryCatch(
        base64enc::base64encode(doc_path),
        error = function(e) {
          return(NULL)
        }
      )

      if (is.null(doc_data)) {
        return(paste0("❌ Failed to encode document: ", doc_path))
      }

      doc_type <- tools::file_ext(doc_path) |> tolower()

      if (doc_type %in% names(mime_doc_types)) {
        mime_type <- mime_doc_types[[doc_type]]
      } else {
        mime_type <- "application/pdf" # Default to PDF if unknown type
      }

      parts <- append(
        parts,
        list(
          list(
            inline_data = list(
              mime_type = "application/pdf",
              data = doc_data
            )
          )
        )
      )
    }
  }

  # Assemble request body
  request_body <- list(
    contents = list(
      parts = parts
    ),
    generationConfig = generation_config
  )

  # Helper: pull the real error message out of a Gemini error response body,
  # returning NULL when the body can't be parsed or carries no message.
  extract_api_error <- function(resp) {
    tryCatch(
      {
        parsed <- jsonlite::fromJSON(httr2::resp_body_string(resp))
        msg <- parsed$error$message
        if (is.null(msg) || !nzchar(msg)) NULL else msg
      },
      error = function(e) NULL
    )
  }

  # Helper: parse a Gemini 429 body into the human message, the server-suggested
  # retry delay (seconds, from the RetryInfo detail) and whether the violated
  # quota is a per-DAY cap — which a retry cannot clear, unlike a per-minute one.
  parse_api_error <- function(resp) {
    out <- list(message = NULL, retry_delay = NA_real_, is_daily = FALSE)
    tryCatch(
      {
        err <- jsonlite::fromJSON(
          httr2::resp_body_string(resp),
          simplifyVector = FALSE
        )$error
        if (!is.null(err$message) && nzchar(err$message)) {
          out$message <- err$message
        }
        for (d in err$details) {
          dtype <- if (is.null(d[["@type"]])) "" else d[["@type"]]
          if (grepl("RetryInfo", dtype) && !is.null(d$retryDelay)) {
            out$retry_delay <- suppressWarnings(
              as.numeric(sub("s$", "", d$retryDelay))
            )
          }
          if (grepl("QuotaFailure", dtype)) {
            ids <- vapply(d$violations, function(v) {
              if (is.null(v$quotaId)) "" else v$quotaId
            }, character(1))
            if (any(grepl("PerDay", ids, ignore.case = TRUE))) {
              out$is_daily <- TRUE
            }
          }
        }
        out
      },
      error = function(e) out
    )
  }

  # Retry loop
  for (attempt in seq_len(retry_503)) {
    # Build and send request. Disable httr2's default "throw on HTTP error
    # status" so 4xx/5xx come back as normal response objects: this lets us
    # read the actual error message Gemini returns instead of masking every
    # failure as an invalid API key.
    req <- request(url) |>
      req_url_query(key = api_key) |>
      req_headers("Content-Type" = "application/json") |>
      req_body_json(request_body) |>
      req_timeout(120) |>
      req_error(is_error = function(resp) FALSE)

    resp <- tryCatch(
      req_perform(req),
      error = function(e) {
        # Transport-level failure (timeout, DNS, no network) — not an HTTP
        # status. Flag it so it is handled separately from HTTP errors.
        structure(
          list(connection_error = TRUE, message = e$message),
          class = "gemini_conn_error"
        )
      }
    )

    # Connection-level error: retry if attempts remain, otherwise report it.
    if (inherits(resp, "gemini_conn_error")) {
      if (attempt < retry_503) {
        message(paste0(
          "⚠️ Connection error - retrying (attempt ",
          attempt, "/", retry_503, ")..."
        ))
        Sys.sleep(min(2^attempt, 16))
        next
      }
      return(paste0(
        "❌ Request failed (connection error): ", resp$message,
        "\nPlease check your internet connection and try again."
      ))
    }

    # 429 = RESOURCE_EXHAUSTED: a quota / rate-limit response, NOT server
    # overload. On a paid tier this is typically the per-MINUTE request cap
    # (RPM) hit by a burst — low-RPM models such as Gemini Pro (~25 RPM) trip
    # far sooner than Flash (1000+ RPM) — or shared-capacity throttling on the
    # newest flagship models. Gemini reports how long to wait (RetryInfo) and
    # whether the cap is per-day (a retry cannot clear that), so honour both
    # instead of hammering with a fixed backoff and masking the real cause.
    if (resp$status_code == 429) {
      err <- parse_api_error(resp)
      msg <- err$message

      # Depleted prepayment / billing credits: a hard 429 that a retry cannot
      # clear. The paid "Pro" models (e.g. gemini-2.5-pro, gemini-3.1-pro)
      # draw on prepay credits; when those run out Google returns this even
      # though Flash models on the same key keep working.
      is_billing <- !is.null(msg) && grepl(
        "prepay|credits? (are )?depleted|billing|insufficient",
        msg,
        ignore.case = TRUE
      )
      if (is_billing) {
        return(paste0(
          "❌ HTTP 429: ",
          paste0(msg, "\n\n"),
          "This model needs available billing/prepayment credits on the key's ",
          "Google Cloud project — retrying will not help. Top up or enable ",
          "billing at https://ai.studio/projects , or switch to a Flash model ",
          "(e.g. Gemini 2.5 Flash), which is often covered separately. Note ",
          "that restricted promotional/EDU credits may not apply to the Pro ",
          "models' prepay balance."
        ))
      }

      # Per-day quota exhausted: retrying is futile — report and stop.
      if (isTRUE(err$is_daily)) {
        return(paste0(
          "❌ HTTP 429: ",
          if (!is.null(msg)) paste0(msg, "\n\n") else "",
          "You have reached the per-DAY request quota for this model. ",
          "Retrying will not help until it resets (around midnight Pacific ",
          "time). Select a different model or try again tomorrow."
        ))
      }

      # Per-minute / capacity throttling: wait the server-suggested delay
      # (capped) and retry while attempts remain.
      if (attempt < retry_503) {
        wait <- if (!is.na(err$retry_delay) && err$retry_delay > 0) {
          min(err$retry_delay + 1, 35)
        } else {
          min(2^attempt, 16)
        }
        message(paste0(
          "⚠️ HTTP 429 (rate limited) - waiting ", round(wait),
          "s before retry (attempt ", attempt, "/", retry_503, ")..."
        ))
        Sys.sleep(wait)
        next
      }

      return(paste0(
        "❌ HTTP 429: ",
        if (!is.null(err$message)) paste0(err$message, "\n\n") else "",
        "This is a per-minute rate-limit (RPM) or shared-capacity response, ",
        "not a server outage. The request was retried ", retry_503,
        " times without success. Try a higher-throughput model such as ",
        "Gemini 2.5 Flash (much higher RPM than the Pro models), run the ",
        "analysis less frequently, or wait a minute and retry."
      ))
    }

    # Retry on transient server errors (5xx).
    if (resp$status_code %in% c(500, 502, 503, 504)) {
      if (attempt < retry_503) {
        message(paste0(
          "⚠️ HTTP ", resp$status_code,
          " (transient server error) - retrying (attempt ",
          attempt,
          "/",
          retry_503,
          ")..."
        ))
        Sys.sleep(min(2^attempt, 16))
        next
      } else {
        return(
          paste0(
            "❌ HTTP ", resp$status_code,
            ": the AI service is temporarily unavailable.\n",
            "The Google Gemini servers are currently overloaded or unstable.\n",
            "All retry attempts failed (",
            retry_503,
            "). Please try again in a few minutes, or select a different AI model."
          )
        )
      }
    }

    # HTTP errors
    # 400 api key not valid
    if (resp$status_code == 400) {
      msg <- tryCatch(
        {
          parsed <- jsonlite::fromJSON(httr2::resp_body_string(resp))
          parsed$error$message
        },
        error = function(e) {
          "Please check your API key. It seems to be not valid!"
        }
      )
      return(paste0("❌ HTTP ", resp$status_code, ": ", msg))
    }
    # Other HTTP errors
    if (resp$status_code != 200) {
      msg <- tryCatch(
        {
          parsed <- jsonlite::fromJSON(httr2::resp_body_string(resp))
          parsed$error$message
        },
        error = function(e) {
          "Service unavailable or unexpected error. Please check your API key and usage limit."
        }
      )

      return(paste0("❌ HTTP ", resp$status_code, ": ", msg))
    }

    # Successful response — concatenate answer text, skipping any "thought"
    # parts (thinking models such as Gemma 4 return their reasoning in a
    # separate part flagged with thought = TRUE).
    candidates <- httr2::resp_body_json(resp)$candidates
    answer <- unlist(lapply(candidates, function(cand) {
      lapply(cand$content$parts, function(p) {
        if (isTRUE(p$thought)) NULL else p$text
      })
    }))
    answer <- answer[!is.na(answer) & nzchar(answer)]
    if (length(answer) == 0) {
      return(paste0(
        "❌ The model returned an empty response. ",
        "Please try again or select a different AI model."
      ))
    }
    return(paste(answer, collapse = "\n"))
  }
}


setGeminiAPI <- function(api_key) {
  # 1. Controllo validità dell'API key
  apiCheck <- gemini_ai(
    image = NULL,
    prompt = "Hello",
    model = "2.5-flash",
    image_type = "png",
    retry_503 = 5,
    api_key = api_key
  )

  contains_http_error <- grepl("HTTP\\s*[1-5][0-9]{2}", apiCheck)

  if (contains_http_error) {
    return(list(
      valid = FALSE,
      message = "❌ API key seems be not valid! Please, check it or your connection."
    ))
  }

  if (is.null(api_key) || !is.character(api_key) || nchar(api_key) == 0) {
    return(list(
      valid = FALSE,
      message = "❌ API key must be a non-empty string."
    ))
  }

  if (nchar(api_key) < 10) {
    return(list(
      valid = FALSE,
      message = "❌ API key seems too short. Please verify your key."
    ))
  }

  # 2. Mostra solo gli ultimi 4 caratteri per feedback
  last_chars <- 4
  last <- substr(
    api_key,
    max(1, nchar(api_key) - last_chars + 1),
    nchar(api_key)
  )

  # 3. Imposta la variabile d'ambiente
  Sys.setenv(GEMINI_API_KEY = api_key)

  return(list(
    valid = TRUE,
    message = paste0(
      paste0(rep("*", nchar(api_key) - 4), collapse = ""),
      last,
      collapse = ""
    )
  ))
}

showGeminiAPI <- function() {
  api_key <- Sys.getenv("GEMINI_API_KEY")
  last_chars <- 4
  last <- substr(
    api_key,
    max(1, nchar(api_key) - last_chars + 1),
    nchar(api_key)
  )
  last <- paste0(
    paste0(rep("*", nchar(api_key) - 4), collapse = ""),
    last,
    collapse = ""
  )
  return(last)
}

load_api_key <- function(path = path_gemini_key) {
  if (file.exists(path)) {
    key <- readLines(path, warn = FALSE)
    if (nchar(key) >= 10) {
      Sys.setenv(GEMINI_API_KEY = key)
      return(TRUE)
    }
  }
  return(FALSE)
}

loadGeminiModel = function(file) {
  # load info about model type and output size
  if (file.exists(file)) {
    model <- readLines(file, warn = FALSE)
  } else {
    model <- c("2.5-flash-lite", "medium")
  }
  if (length(model == 1)) {
    model <- c(model, "medium")
  }
  return(model)
}

saveGeminiModel = function(model, file) {
  if (file.exists(file)) {
    file.remove(file)
  }
  writeLines(model, file)
}

geminiOutput <- function(title = "", content = "", values) {
  if (is.null(content)) {
    content <- "Click the 'Ask Biblio AI' button to analyze the visual outputs and provide an automatic interpretation of your results based on the graphs.\n
This AI-powered feature leverages Google Gemini to help you understand patterns and insights emerging from your contextual analysis.\n  \n  \n \n  \n  \n  \n"
  }
  box(
    title = title,
    width = 12,
    status = "info",
    solidHeader = TRUE,
    div(
      id = "typing-box",
      style = "white-space: pre-wrap; background-color:#f9f9f9; padding:15px; border:1px solid #ccc; border-radius:5px; max-height:400px; overflow-y: auto;",
      #HTML(text_to_html(content))
      HTML(gemini_to_html(content))
    ),
    br(),
    em(
      "You can modify or enrich the proposed prompt with additional context or details about your analysis to help 'Biblio AI' generate a more accurate and meaningful interpretation."
    ),
    textAreaInput(
      inputId = "gemini_additional",
      label = NULL,
      value = values$gemini_model_parameters,
      placeholder = "You can provide additional context or details about your analysis to help 'Biblio AI' generate a more accurate and meaningful interpretation.",
      rows = 3,
      width = "100%"
    ),
    fluidRow(
      column(
        4,
        align = "center",
        actionButton(
          "gemini_btn",
          "Ask Biblio AI",
          style = "color: white;",
          icon(name = "microchip", lib = "font-awesome"),
          width = "80%"
        )
      ),
      column(
        4,
        align = "center",
        actionButton(
          "copy_btn",
          "Copy",
          style = "color: white;",
          icon = icon("clipboard"),
          width = "80%"
        )
      ),
      column(
        4,
        align = "center",
        downloadButton(
          outputId = "save_btn",
          label = "Save",
          icon = icon("download"),
          style = "width: 80%;"
        )
      )
    )
  )
}

biblioAiPrompts <- function(values, activeTab) {
  ## ROle definition for Gemini as Biblio AI assistant
  promptInitial <- "You are Biblio AI, an AI assistant integrated within Biblioshiny. Your task is to support researchers in interpreting and critically discussing the results of their bibliometric analyses, offering insights, contextual explanations, and guidance for data-driven interpretation. "

  switch(
    activeTab,
    "mainInfo" = {
      prompt <- paste0(
        "Provide an interpretation of these statistics summarizing the bibliographic collection. ",
        "Focus on key metrics such as the number of documents, sources, authors, and citations, ",
        "and discuss what they reveal about the scope, productivity, and impact of the collection. ",
        "This is the list of statistics: ",
        merge_df_to_string(values$TABvb)
      )
    },
    "lifeCycle" = {
      prompt <- paste0(
        "Analyze the provided 'life cycle' plots showing the bibliographic collection's scientific production fitted to a logistic growth model. ",
        "You have been provided with: (1) an ANNUAL plot showing yearly publication counts, (2) a CUMULATIVE plot showing the total publications over time, ",
        "and (3) the logistic model parameters and goodness-of-fit metrics.\n\n",

        "Please provide a comprehensive interpretation covering:\n",
        "1. GROWTH PATTERN: Describe the overall growth trajectory (exponential, linear, or saturation phase). ",
        "Identify the inflection point where growth rate changes.\n",
        "2. MODEL FIT: Evaluate how well the logistic curve fits the observed data based on the provided metrics (R², RMSE, etc.). ",
        "Comment on any systematic deviations.\n",
        "3. KEY PARAMETERS: Interpret the logistic model parameters:\n",
        "   - K (carrying capacity): the estimated maximum cumulative production\n",
        "   - r (growth rate): the intrinsic rate of increase\n",
        "   - t₀ (inflection point): when maximum growth rate occurred\n",
        "4. TRENDS AND PHASES: Identify distinct phases in the publication timeline:\n",
        "   - Emergence phase (slow initial growth)\n",
        "   - Expansion phase (rapid growth)\n",
        "   - Maturity/saturation phase (if present)\n",
        "5. SCIENTIFIC IMPLICATIONS: Discuss what these patterns suggest about the research field's maturity, ",
        "potential factors influencing growth (e.g., funding, technological advances, paradigm shifts), ",
        "and future trajectories. Is the field still growing or approaching saturation?\n\n",

        "Provide your analysis in a structured format with clear sections.",
        "This a the list of statistics: Parameters ",
        merge_df_to_string(values$DLC$parameters),
        ", parameters_real_years ",
        merge_df_to_string(values$DLC$parameters_real_years),
        ", and goodness_of_fit ",
        merge_df_to_string(values$DLC$metrics)
      )
    },
    "threeFieldPlot" = {
      prompt <- paste0(
        "Provide an interpretation of this Three-Field Plot. The central field represents the target metadata,",
        " while the left and the right fields are the destination metadata. The plot visualizes the connections",
        " among these three dimensions by linking the target field witgh the two other metadata.",
        " Analyze the structure of the network, and describe how these elements are interconnected."
      )
    },
    "authorsProdOverTime" = {
      docs <- apot2Docs(values$AUProdOverTime, n = 3) %>% merge_df_to_string()
      prompt <- paste0(
        "Provide an interpretation of this 'Authors’ Production Over Time' plot, which displays bibliometric trends",
        " for the top authors in the field. For each author, the red horizontal line represents their scientific timeline,",
        " indicating the span of their active publishing years. Each bubble corresponds to a publication year: its size reflects",
        " the number of articles published that year, while the color intensity represents the total number of citations per year (TC/year).",
        " Analyze the evolution of individual authors’ productivity and impact, identifying key periods of activity, citation peaks, ",
        "and possible shifts in research influence over time. A list of the most three cited-per-year articles for each author is provided: ",
        docs
      )
    },
    "correspAuthorCountry" = {
      countries <- merge_df_to_string(country2collab(values$TABCo))
      prompt <- paste0(
        "Provide an interpretation of this 'Corresponding Author’s Country Collaboration Plot'. The plot displays the number of scientific publications",
        " by corresponding authors for each country, distinguishing between Single Country Publications (SCP) and Multiple Country Publications (MCP). ",
        "SCP represents articles authored exclusively within one country, while MCP refers to collaborative works involving authors from different countries. ",
        "The MCP Ratio, calculated as MCP divided by the total number of publications, indicates the extent of international collaboration. ",
        "Focus on identifying the most productive countries, comparing their levels of international collaboration, and discussing the balance between domestic ",
        "and global research engagement. A list of the most 20 frequent countries is provided: ",
        countries
      )
    },
    "mostLocalCitDoc" = {
      docs <- localCit2docs(values$TABLocDoc, n = 20) %>% merge_df_to_string()
      prompt <- paste0(
        "Provide an interpretation of this table reporting the most locally and globally cited articles in the collection. Global Citations (GC) represent the total",
        " number of citations a document has received across all records indexed in the bibliographic database, indicating its overall scholarly impact. Local Citations (LC),",
        " on the other hand, count the number of citations a document has received within this specific dataset, reflecting its relevance to the focused research field. ",
        "Both indicators are normalized by NTC (Normalized Total Citations), which adjusts citation counts relative to the average number of citations for publications from the same year.",
        " This normalization allows for fair comparison across time. Focus on identifying articles with strong global influence, local relevance, or both.",
        "A list of the most 20 local cited articles is provided: ",
        docs
      )
    },
    "trendTopic" = {
      prompt <- paste0(
        "Provide an interpretation of this trend topics plot, where each year is associated with the k words that have the highest annual median frequency. ",
        "For each word, the light blue line represents the interquartile range (from the first to the third quartile) of its frequency time series, and the central",
        "point marks the median value. The size of the bubbles is proportional to the annual frequency, reflecting the relative prominence of each term over time. "
      )
    },
    "ReferenceSpect" = {
      references <- merge_df_to_string(values$res$peaks)
      sequenceTop5 <- values$res$Sequences %>%
        filter(Class != "") %>%
        group_by(Class) %>%
        slice_max(Freq, n = 5) %>%
        rename("Totale Citation" = "Freq")
      prompt <- paste0(
        "Provide an interpretation of this Reference Publication Year Spectroscopy plot (RPYS, Marx et al. 2014, JAIST).",
        " The black line shows the number of cited references by publication year. ",
        "The red line represents the deviation from the 5-year median citation frequency.",
        " This highlights peak years of historical significance. A list of the most cited references is provided for the first ten peak years identified by the red line,",
        " along with their citation frequencies: ",
        references,
        "\nIn addition, please highlight which references are the most important according to the following categories: Hot Paper, Life Cycle, Sleeping Beauty,",
        "
                         and Constant Performer (Thor A. et al. 2018, Identifying single influential publications in a research field: new analysis opportunities of the CRExplorer. Scientometrics).",
        " Below is the list of references with their assigned category.",
        sequenceTop5
      )
    },
    "coOccurenceNetwork" = {
      prompt <- paste0(
        "Provide an interpretation of this 'word co-occurrence' network.",
        " Focus on the structure of the network, the presence of communities (topics), and the relevance of the most connected terms."
      )
      if (!is.null(values$cocnet$doc2clust)) {
        docs <- merge_df_to_string(doc2clust(values$cocnet))
        prompt <- paste0(
          prompt,
          " Here there is the list of three most central articles for each cluster: ",
          docs,
          ". Also describe the articles belonging to each community."
        )
      }
    },
    "thematicMap" = {
      docs <- merge_df_to_string(doc2clust(values$TM))
      prompt <- paste0(
        "Provide an interpretation of this 'strategic map'",
        "Here there is the list of three most central articles for each cluster: ",
        docs,
        ". Focus on the structure of the strategic map, the description of clusters and their centrality and density measures, and the description of the articles belonging to them."
      )
    },
    "thematicEvolution" = {
      prompt <- paste0(
        "Provide an interpretation of these plots.",
        "The first plot illustrates topic evolution across different time periods, showing how thematic areas develop or shift over time. ",
        "The other plots display strategic maps for each period, highlighting the positioning of topics based on centrality and density. ",
        "Focus on identifying emerging, declining, or stable topics and their strategic importance."
      )
    },
    "factorialAnalysis" = {
      prompt <- "Provide an interpretation of this 'factorial map'. Focus on the structure of the map, the presence of clusters, and the relevance of the most contributing terms."
    },
    "coCitationNetwork" = {
      prompt <- "Provide an interpretation of this 'co-citation' network. Focus on the structure of the network, the presence of communities, and the relevance of the most connected terms."
    },
    "historiograph" = {
      titles <- hist2docs(values$histPlotVis$VIS$x$nodes) %>%
        merge_df_to_string()
      prompt <- paste0(
        "Interpret this historiograph, a temporal citation network built by mapping direct citation links among documents. ",
        "Highlight the main citation paths, pivotal works, and any notable temporal trends in knowledge development, looking also to the article titles and their topics.",
        " Focus on the temporal evolution of each cluster",
        "Here there is the list of each paper (node) and its title: ",
        titles
      )
    },
    "collabNetwork" = {
      prompt <- paste0(
        "Provide an interpretation of this 'collaboration' network",
        ". Focus on the structure of the network, the presence of communities, and the relevance of the most connected terms."
      )
    },
    "collabWorldMap" = {
      #values$WMGemini
      prompt <- paste0(
        "Provide an interpretation of this ‘Countries’ Collaboration World Map’. The map visualizes international scientific ",
        "collaboration by showing, for each country, the total number of articles with at least one contributing author. ",
        "The color intensity of each country is proportional to its research output. The connecting lines represent collaborative ",
        "links between countries, based on co-authorship across all authors (not only corresponding authors). ",
        "Focus on identifying major hubs of scientific production, key international partnerships, and global patterns of collaboration."
      )
    },
    "bradford" = {
      prompt <- paste0(
        "Provide an interpretation of this Bradford bibliograph and Bradford’s Law analysis. ",
        "The plot shows the cumulative number of articles (y-axis) against the logarithm of source rank (x-axis). ",
        "The solid black line is the empirical curve and the dashed red line is the theoretical fitted Bradford distribution: C(r) = a + b * log(r). ",
        "The three shaded zones represent the Bradford zones: Zone 1 (core journals), Zone 2 (intermediate), and Zone 3 (periphery). ",
        "Each zone contains approximately one-third of the total articles, but the number of journals increases geometrically (ratio 1:k:k^2). ",
        "Focus on: (1) the degree of concentration vs dispersion in the field’s publication landscape, ",
        "(2) the shape of the curve (steep initial drop = high concentration, flatter = more dispersion), ",
        "(3) how well the empirical data fits the theoretical Bradford distribution (R-squared and KS test), ",
        "(4) the Bradford multiplier k and what it reveals about the zone structure, ",
        "(5) identifying and discussing the core journals and their role in the field."
      )
    },
    "lotka" = {
      prompt <- paste0(
        "Provide an interpretation of this Lotka’s Law plot and analysis. ",
        "The plot shows the author productivity distribution: the percentage of authors (y-axis) who published a given number of documents (x-axis). ",
        "Three curves are displayed: (1) the solid black line is the empirical distribution, ",
        "(2) the dashed red line is the theoretical Lotka’s Law with Beta=2 (inverse square law), ",
        "(3) the dot-dash blue line is the fitted generalized inverse power law with the estimated Beta. ",
        "Lotka’s Law states that f(n) = C / n^Beta, where f(n) is the proportion of authors publishing n articles. ",
        "Focus on: (1) how well the empirical data fits the theoretical (Beta=2) and fitted distributions (KS test results), ",
        "(2) the estimated Beta value and what it reveals about productivity concentration in the field, ",
        "(3) the proportion of single-paper authors vs prolific authors, ",
        "(4) whether Beta > 2 (more concentrated, typical of niche fields) or Beta < 2 (more dispersed, typical of broad fields), ",
        "(5) implications for the field’s author productivity patterns."
      )
    },
    {
      prompt <- paste0(
        "Provide an interpretation of this plot creted with ‘bibliometrix R Package’"
      )
    }
  )
  prompt <- paste0(promptInitial, prompt)
  #if (!activeTab %in% c("mainInfo", "thematicMap", "trendTopic")) prompt <- paste0(prompt, " Provide also scientific references about the methodological description")
  return(prompt)
}


geminiParameterPrompt <- function(values, activeTab, input) {
  if ("DB_Original" %in% names(values$M)) {
    DB <- unique(values$M$DB_Original)
    if (length(DB) > 1) {
      DB[length(DB)] <- paste0("and ", DB[length(DB)])
    }
    DB <- gsub("ISI", "WOS", paste0(DB, collapse = ", "))

    DB <- paste0(
      "databases ",
      DB,
      " merged by Bibliometrix R package routines."
    )
  } else {
    if (values$M$DB[1] == "ISI") {
      DB <- "WOS"
    } else {
      DB <- values$M$DB[1]
    }
    DB <- paste0("database ", DB, ".")
  }
  txt <- paste0(
    "The analysis was performed on a collection downloaded from the following bibliographic ",
    DB
  )
  missingTags <- values$missingdf %>%
    filter(status %in% "Completely missing") %>%
    pull(description)
  if (length(missingTags) > 0) {
    if ("DB_Original" %in% names(values$M)) {
      txt1 <- paste0(
        " Some metadata fields like ",
        paste0(missingTags, collapse = ", "),
        " are missing due to limitations in the merging process and 
                                                           non-standard formats across databases."
      )
      txt <- paste0(txt, txt1)
    }
  }

  switch(
    activeTab,
    "mainInfo" = {
      req(values$TABvb)
    },
    "lifeCycle" = {
      req(values$DLC)
    },
    "threeFieldPlot" = {
      req(values$TFP)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: target field '",
        input$CentralField,
        "', right field '",
        input$RightField,
        "', and left field '",
        input$LeftField,
        "'."
      )
    },
    "trendTopic" = {
      req(values$trendTopics)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: ",
        merge_df_to_string(values$trendTopics$params)
      )
    },
    "authorsProdOverTime" = {
      req(values$AUProdOverTime)
    },
    "correspAuthorCountry" = {
      req(values$TABCo)
    },
    "mostLocalCitDoc" = {
      req(values$TABLocDoc)
    },
    "ReferenceSpect" = {
      req(values$res)
    },
    "coOccurenceNetwork" = {
      req(values$COCnetwork)
      txt <- paste0(
        txt,
        " This graph was generated with the following parameters: ",
        merge_df_to_string(values$cocnet$params)
      )
    },
    "thematicMap" = {
      req(values$TMmap)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$TM$params)
      )
    },
    "thematicEvolution" = {
      req(values$nexus)
      txt <- paste0(
        txt,
        " The following parameters were used for each strategic map: ",
        merge_df_to_string(values$nexus$TM[[1]]$params)
      )
    },
    "factorialAnalysis" = {
      req(values$plotCS)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$CS$params)
      )
    },
    "coCitationNetwork" = {
      req(values$COCITnetwork)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$cocitnet$params)
      )
    },
    "historiograph" = {
      req(values$histPlotVis$VIS)
    },
    "collabNetwork" = {
      req(values$COLnetwork$VIS)
      txt <- paste0(
        txt,
        "  This graph was generated with the following parameters: ",
        merge_df_to_string(values$colnet$params)
      )
    },
    "collabWorldMap" = {
      req(values$WMmap)
    },
    "bradford" = {
      req(values$bradford)
      s <- values$bradford$stat
      zs <- values$bradford$zoneSummary
      txt <- paste0(
        txt,
        " Bradford's Law analysis results: ",
        "Fitted model C(r) = ", round(s$a, 2), " + ", round(s$b, 2), " * log(r), ",
        "R-squared = ", round(s$R2, 4), ", ",
        "Bradford multiplier k = ", round(s$k, 2), ", ",
        "KS test p-value = ", format.pval(s$ks.pvalue, digits = 3), ". ",
        "Zone summary: ", merge_df_to_string(zs)
      )
    },
    "lotka" = {
      req(values$lotka)
      s <- values$lotka$stat
      txt <- paste0(
        txt,
        " Lotka's Law analysis results: ",
        "Estimated Beta = ", round(s$Beta, 4), ", ",
        "Constant C = ", round(s$C, 4), ", ",
        "R-squared = ", round(s$R2, 4), ", ",
        "KS p-value (theoretical Beta=2) = ", format.pval(s$ks.theo.pvalue, digits = 3), ", ",
        "KS p-value (fitted Beta) = ", format.pval(s$ks.fit.pvalue, digits = 3), ". ",
        "Author productivity table: ", merge_df_to_string(values$lotka$AuthorProd)
      )
    }
  )
  return(txt)
}



geminiWaitingMessage <- function(values, activeTab) {
  messageTxt <- "⌛ Thinking..."

  switch(
    activeTab,
    "mainInfo" = {
      req(values$TABvb)
      values$MainInfoGemini <- messageTxt
    },
    "lifeCycle" = {
      req(values$DLC)
      values$DLCGemini <- messageTxt
    },
    "threeFieldPlot" = {
      req(values$TFP)
      values$TFPGemini <- messageTxt
    },
    "authorsProdOverTime" = {
      req(values$AUProdOverTime)
      values$ApotGemini <- messageTxt
    },
    "correspAuthorCountry" = {
      req(values$TABCo)
      values$MostRelCountriesGemini <- messageTxt
    },
    "mostLocalCitDoc" = {
      req(values$TABLocDoc)
      values$MostLocCitDocsGemini <- messageTxt
    },
    "trendTopic" = {
      req(values$trendTopics)
      values$trendTopicsGemini <- messageTxt
    },
    "ReferenceSpect" = {
      req(values$res)
      values$rpysGemini <- messageTxt
    },
    "coOccurenceNetwork" = {
      req(values$COCnetwork)
      values$cocGemini <- messageTxt
    },
    "thematicMap" = {
      req(values$TM)
      values$TMGemini <- messageTxt
    },
    "thematicEvolution" = {
      req(values$nexus)
      values$TEGemini <- messageTxt
    },
    "factorialAnalysis" = {
      req(values$plotCS)
      values$CSGemini <- messageTxt
    },
    "coCitationNetwork" = {
      req(values$COCITnetwork)
      values$cocitGemini <- messageTxt
    },
    "historiograph" = {
      req(values$histPlotVis$VIS)
      values$histGemini <- messageTxt
    },
    "collabNetwork" = {
      req(values$COLnetwork$VIS)
      values$colGemini <- messageTxt
    },
    "collabWorldMap" = {
      req(values$WMmap)
      values$WMGemini <- messageTxt
    },
    "bradford" = {
      req(values$bradford)
      values$BradfordGemini <- messageTxt
    },
    "lotka" = {
      req(values$lotka)
      values$LotkaGemini <- messageTxt
    }
  )
  return(values)
}

## Map activeTab to reactive field name in values
geminiFieldName <- function(activeTab) {
  switch(
    activeTab,
    "mainInfo" = "MainInfoGemini",
    "lifeCycle" = "DLCGemini",
    "threeFieldPlot" = "TFPGemini",
    "authorsProdOverTime" = "ApotGemini",
    "correspAuthorCountry" = "MostRelCountriesGemini",
    "mostLocalCitDoc" = "MostLocCitDocsGemini",
    "trendTopic" = "trendTopicsGemini",
    "ReferenceSpect" = "rpysGemini",
    "coOccurenceNetwork" = "cocGemini",
    "thematicMap" = "TMGemini",
    "thematicEvolution" = "TEGemini",
    "factorialAnalysis" = "CSGemini",
    "coCitationNetwork" = "cocitGemini",
    "historiograph" = "histGemini",
    "collabNetwork" = "colGemini",
    "collabWorldMap" = "WMGemini",
    "bradford" = "BradfordGemini",
    "lotka" = "LotkaGemini",
    NULL
  )
}

## Prepare image files for Gemini (synchronous part of geminiPromptImage)
geminiPrepareImage <- function(obj, type, prompt, key, desc, values) {
  # Check internet connection
  if (!is_online()) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = '⚠️ **Note**: Biblio AI requires an active internet connection to work.'
    ))
  }
  # Check Chrome browser
  if (is.null(values$Chrome_url)) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = '⚠️ **Note**: Biblio AI requires a **Chrome-based browser** (such as Google Chrome or Microsoft Edge) installed on your computer to work correctly.'
    ))
  }

  if (!key) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = 'To access this feature, please provide a valid Gemini AI API key. You can obtain your API key by visiting the official <a href="https://aistudio.google.com/" target="_blank">Google AI Studio website</a>.'
    ))
  }

  # Build prompt with description
  if (!is.null(desc)) {
    prompt <- paste0(prompt, desc, collapse = ". ")
  }

  # Convert plot to PNG
  file_path <- tempfile(fileext = ".png", tmpdir = getWD())

  result <- tryCatch({
    switch(
      type,
      "vis" = {
        plot2pngGemini(obj, filename = file_path, type = "vis")
      },
      "plotly" = {
        plot2pngGemini(obj, filename = file_path, type = "plotly")
      },
      "multi" = {
        file_path <- obj
      },
      "text" = {
        file_path <- NULL
      },
      "ggplot2" = {
        ggsave(
          filename = file_path,
          plot = obj,
          dpi = 72,
          height = 7,
          width = 14,
          bg = "transparent"
        )
      }
    )
    NULL  # no error
  }, error = function(e) {
    conditionMessage(e)
  })

  if (!is.null(result)) {
    return(list(
      prompt = NULL,
      image_paths = NULL,
      error_msg = paste("Error preparing image:", result)
    ))
  }

  return(list(
    prompt = prompt,
    image_paths = file_path,
    error_msg = NULL
  ))
}

## Cleanup temporary files after Gemini API call
geminiCleanupFiles <- function(image_paths) {
  if (!is.null(image_paths[1])) {
    tryCatch({
      filename_html <- paste0(tools::file_path_sans_ext(image_paths), ".html")
      existing <- c(image_paths, filename_html)
      existing <- existing[file.exists(existing)]
      if (length(existing) > 0) file.remove(existing)
      dirs <- paste0(tools::file_path_sans_ext(image_paths), "_files")
      dirs <- dirs[dir.exists(dirs)]
      if (length(dirs) > 0) unlink(dirs, recursive = TRUE)
    }, error = function(e) {
      # Silently ignore cleanup errors
    })
  }
}

## Prepare all data for async Gemini call (synchronous orchestration)
geminiPrepareAll <- function(values, activeTab, input) {
  field <- geminiFieldName(activeTab)
  if (is.null(field)) {
    return(list(field = "MainInfoGemini", can_proceed = FALSE,
                error_msg = "Unsupported tab for Biblio AI."))
  }

  # Build description
  gemini_additional <- values$gemini_additional
  gemini_model_parameters <- values$gemini_model_parameters
  if (!is.null(gemini_additional) && gemini_additional != "") {
    desc <- paste0(
      values$collection_description,
      gemini_additional,
      gemini_model_parameters,
      collapse = ". "
    )
  } else {
    desc <- paste0(
      values$collection_description,
      gemini_model_parameters,
      collapse = ". "
    )
  }

  # Build prompt
  prompt <- biblioAiPrompts(values, activeTab)

  # Determine obj and type for each tab
  obj <- NULL
  type <- "text"

  switch(
    activeTab,
    "mainInfo" = {
      obj <- NULL
      type <- "text"
    },
    "lifeCycle" = {
      files <- DLC2Gemini(values$DLCplotYear, values$DLCplotCum)
      obj <- files
      type <- "multi"
    },
    "threeFieldPlot" = {
      obj <- values$TFP
      type <- "plotly"
    },
    "authorsProdOverTime" = {
      obj <- values$AUProdOverTime$graph
      type <- "ggplot2"
    },
    "correspAuthorCountry" = {
      obj <- values$MRCOplot
      type <- "ggplot2"
    },
    "mostLocalCitDoc" = {
      obj <- NULL
      type <- "text"
    },
    "trendTopic" = {
      obj <- values$trendTopics$graph
      type <- "ggplot2"
    },
    "ReferenceSpect" = {
      obj <- values$res$spectroscopy
      type <- "ggplot2"
    },
    "coOccurenceNetwork" = {
      obj <- values$COCnetwork$VIS
      type <- "vis"
    },
    "thematicMap" = {
      obj <- values$TMmap
      type <- "plotly"
    },
    "thematicEvolution" = {
      files <- TE2Gemini(values$nexus, values$TEplot)
      obj <- files
      type <- "multi"
    },
    "factorialAnalysis" = {
      obj <- values$plotCS
      type <- "plotly"
    },
    "coCitationNetwork" = {
      obj <- values$COCITnetwork$VIS
      type <- "vis"
    },
    "historiograph" = {
      obj <- values$histPlotVis$VIS
      type <- "vis"
    },
    "collabNetwork" = {
      obj <- values$COLnetwork$VIS
      type <- "vis"
    },
    "collabWorldMap" = {
      obj <- values$WMmap$g
      type <- "plotly"
    },
    "bradford" = {
      obj <- values$bradford$graph
      type <- "ggplot2"
    },
    "lotka" = {
      obj <- values$lotka$g
      type <- "ggplot2"
    }
  )

  # Prepare image (synchronous)
  prep <- geminiPrepareImage(
    obj = obj,
    type = type,
    prompt = prompt,
    key = values$geminiAPI,
    desc = desc,
    values = values
  )

  if (!is.null(prep$error_msg)) {
    return(list(
      field = field,
      can_proceed = FALSE,
      error_msg = prep$error_msg,
      prompt = NULL,
      image_paths = NULL
    ))
  }

  return(list(
    field = field,
    can_proceed = TRUE,
    error_msg = NULL,
    prompt = prep$prompt,
    image_paths = prep$image_paths
  ))
}

geminiSave <- function(values, activeTab) {
  switch(
    activeTab,
    "mainInfo" = {
      gemini <- values$MainInfoGemini
    },
    "lifeCycle" = {
      gemini <- values$DLCGemini
    },
    "threeFieldPlot" = {
      gemini <- values$TFPGemini
    },
    "authorsProdOverTime" = {
      gemini <- values$ApotGemini
    },
    "correspAuthorCountry" = {
      gemini <- values$MostRelCountriesGemini
    },
    "mostLocalCitDoc" = {
      gemini <- values$MostLocCitDocsGemini
    },
    "trendTopic" = {
      gemini <- values$trendTopicsGemini
    },
    "ReferenceSpect" = {
      gemini <- values$rpysGemini
    },
    "coOccurenceNetwork" = {
      gemini <- values$cocGemini
    },
    "thematicMap" = {
      gemini <- values$TMGemini
    },
    "thematicEvolution" = {
      gemini <- values$TEGemini
    },
    "factorialAnalysis" = {
      gemini <- values$CSGemini
    },
    "coCitationNetwork" = {
      gemini <- values$cocitGemini
    },
    "historiograph" = {
      gemini <- values$histGemini
    },
    "collabNetwork" = {
      gemini <- values$colGemini
    },
    "collabWorldMap" = {
      gemini <- values$WMGemini
    },
    "bradford" = {
      gemini <- values$BradfordGemini
    },
    "lotka" = {
      gemini <- values$LotkaGemini
    }
  )
  if (is.null(gemini)) {
    gemini <- "Click 'Ask Biblio AI' for help. "
  }
  return(gemini)
}

## Summarize document with Gemini AI ---
summaryAI <- function(values, i, model) {
  # Construct the prompt for Gemini AI
  prompt <- paste0(
    "Summarize the content of the following publication",
    " in no more than 250 words. ",
    "Title: ",
    values$M$TI[i],
    "Authors: ",
    values$M$AU[i],
    "Journal: ",
    values$M$SO[i],
    "Publication Year: ",
    values$M$PY[i],
    "Abstract: ",
    values$M$AB[i],
    "DOI: ",
    values$M$DI[i],
    "Try to access to article website using its DOI and extract more information about the article content. ",
    "If the article is open access, try to extract the full text and summarize it. ",
    "If the article is not open access, try to extract more information from the article website. ",
    "If you cannot access to the article website, just summarize the content using the title and the abstract. ",
    "Do not mention that you are an AI model. ",
    "Do not mention that you cannot access to the article website. ",
    "Do not mention that you do not have access to the full text. ",
    "Just provide the summary. "
  )

  # Gemini AI call
  res <- tryCatch(
    {
      gemini_ai(
        image = NULL,
        prompt = prompt,
        model = model,
        image_type = "png",
        retry_503 = 2,
        api_key = NULL
      )
    },
    error = function(e) {
      "Error generating automatic summary."
    }
  )

  return(res)
}


## Normalize abstract text
normalize_uppercase_text <- function(text) {
  # Check that input is a character string
  if (!is.character(text) || length(text) != 1) {
    stop("Input must be a character string")
  }

  # Convert everything to lowercase
  text_lower <- tolower(text)

  # Capitalize the first letter of the text
  text_lower <- paste0(
    toupper(substring(text_lower, 1, 1)),
    substring(text_lower, 2)
  )

  # Pattern to identify end of sentence followed by space and letter
  # Considers period, exclamation mark, question mark
  text_normalized <- gsub(
    "([.!?])\\s+([a-z])",
    "\\1 \\U\\2",
    text_lower,
    perl = TRUE
  )

  # Handle common abbreviations that should not be followed by uppercase
  # Examples: Dr., Prof., etc., vs., i.e., e.g.
  common_abbrev <- c(
    "dr\\.",
    "prof\\.",
    "etc\\.",
    "vs\\.",
    "i\\.e\\.",
    "e\\.g\\.",
    "mr\\.",
    "mrs\\.",
    "ms\\.",
    "jr\\.",
    "sr\\.",
    "phd\\."
  )

  for (abbrev in common_abbrev) {
    # Convert to lowercase the letters after common abbreviations
    pattern <- paste0("(", abbrev, ")\\s+([A-Z])")
    text_normalized <- gsub(
      pattern,
      "\\1 \\L\\2",
      text_normalized,
      perl = TRUE,
      ignore.case = TRUE
    )
  }

  # Capitalize after colons (optional - common in academic titles)
  text_normalized <- gsub(
    "(:)\\s+([a-z])",
    "\\1 \\U\\2",
    text_normalized,
    perl = TRUE
  )

  return(text_normalized)
}

## To title upper case metadata fields
to_title_case <- function(texts, exclude_articles = TRUE) {
  # --- Input Validation ---
  # Check that the input is a character vector. If not, stop and show an error.
  if (!is.character(texts)) {
    stop("Input must be a character vector.")
  }

  # --- Configuration ---
  # Define a list of "small words" (articles, prepositions, conjunctions) that
  # should remain in lowercase unless they are the first or last word in the title.
  small_words <- c(
    "a",
    "an",
    "and",
    "as",
    "at",
    "but",
    "by",
    "for",
    "if",
    "in",
    "of",
    "on",
    "or",
    "the",
    "to",
    "up",
    "via",
    "with",
    "from",
    "into",
    "over",
    "upon",
    "down",
    "off",
    "out",
    "per",
    "than",
    "through",
    "under",
    "within",
    "without",
    "about",
    "across",
    "after",
    "against",
    "along",
    "among",
    "around",
    "before",
    "behind",
    "below",
    "beneath",
    "beside",
    "between",
    "beyond",
    "during",
    "except",
    "inside",
    "near",
    "since",
    "toward",
    "towards",
    "until",
    "where",
    "while"
  )

  # --- Helper Function ---
  # A helper function to capitalize the first letter of a single word.
  # It also handles hyphenated words (e.g., "state-of-the-art" -> "State-of-the-Art").
  capitalize_first <- function(word) {
    # Check for hyphens
    if (grepl("-", word)) {
      # If a hyphen exists, split the word into parts
      parts <- strsplit(word, "-")[[1]]
      # Capitalize the first letter of each part
      capitalized_parts <- sapply(parts, function(part) {
        if (nchar(part) > 0) {
          paste0(toupper(substring(part, 1, 1)), substring(part, 2))
        } else {
          part # Return empty parts as they are
        }
      })
      # Re-join the parts with a hyphen
      return(paste(capitalized_parts, collapse = "-"))
    } else {
      # If no hyphen, just capitalize the first letter of the word
      paste0(toupper(substring(word, 1, 1)), substring(word, 2))
    }
  }

  # --- Main Processing Logic ---
  # Use sapply() to apply the title-casing logic to each element of the input vector 'texts'.
  # 'sapply' iterates over the vector and collects the results into a new vector.
  # USE.NAMES = FALSE ensures the output is an unnamed vector.
  result_vector <- sapply(
    texts,
    function(single_text) {
      # Handle edge cases: return NA or empty strings without modification.
      if (is.na(single_text) || nchar(single_text) == 0) {
        return(single_text)
      }

      # Convert the entire string to lowercase to ensure consistency.
      text_lower <- tolower(single_text)

      # Split the string into a vector of words based on whitespace.
      words <- strsplit(text_lower, "\\s+")[[1]]

      # Apply capitalization based on the 'exclude_articles' flag.
      if (exclude_articles) {
        # If TRUE, apply capitalization selectively.
        capitalized_words <- sapply(seq_along(words), function(i) {
          word <- words[i]
          # Always capitalize the first word, the last word, or any word NOT in the 'small_words' list.
          if (i == 1 || i == length(words) || !word %in% small_words) {
            capitalize_first(word)
          } else {
            word # Otherwise, keep the small word in lowercase.
          }
        })
      } else {
        # If FALSE, capitalize every word.
        capitalized_words <- sapply(words, capitalize_first)
      }

      # Recombine the processed words into a single string, separated by spaces.
      paste(capitalized_words, collapse = " ")
    },
    USE.NAMES = FALSE
  )

  # Return the final vector of title-cased strings.
  return(result_vector)
}


merge_df_to_string <- function(df) {
  # Check if the input has at least two columns
  if (ncol(df) < 2) {
    stop("The data frame must have at least two columns.")
  }

  # Ensure the input is a data frame
  df <- as.data.frame(df)

  # Convert each row into a "param: value" format
  row_strings <- apply(df[, 1:2], 1, function(row) {
    paste0(row[1], ": ", row[2])
  })

  # Concatenate all row strings using "; " as separator
  final_string <- paste(row_strings, collapse = "; ")

  return(final_string)
}

plot2pngGemini <- function(p, filename, zoom = 2, type = "vis") {
  #html_name <- tempfile(fileext = ".html")
  # remove file extensions from filename
  filename_html <- tools::file_path_sans_ext(filename)
  temp_html <- paste0(filename_html, ".html")
  switch(
    type,
    vis = {
      visSave(p, temp_html)
    },
    plotly = {
      htmlwidgets::saveWidget(p, file = temp_html, selfcontained = TRUE)
    }
  )

  biblioShot(temp_html, zoom = zoom, file = filename, delay = 2.5)
}

getWD <- function() {
  home_path <- Sys.getenv("HOME")
  temp_path <- file.path(home_path, "temp")

  if (!dir.exists(temp_path)) {
    dir.create(temp_path, recursive = TRUE)
  }

  return(temp_path)
}

## save all plots of the life cycle analysis
DLC2Gemini <- function(DLCplotYear, DLCplotCum) {
  plots <- c("annual", "cumulative")
  files <- unlist(lapply(plots, function(x) {
    paste0(getWD(), "/plotDLC_", x, ".png")
  }))

  plot2pngGemini(DLCplotYear, filename = files[1], type = "plotly")
  plot2pngGemini(DLCplotCum, filename = files[2], type = "plotly")

  return(files)
}

## save all plots of the thematic evolution analysis
TE2Gemini <- function(nexus, plotTE) {
  K <- length(nexus$TM)
  periods <- nexus$Nodes %>% select(group) %>% distinct() %>% pull()
  files <- unlist(lapply(periods, function(x) {
    file.path(getWD(), paste0(x, "_period.png"))
  }))
  files <- c(file.path(getWD(), "Evolution_Plot.png"), files)

  plot2pngGemini(plotTE, filename = files[1], type = "plotly")

  for (k in 2:(K + 1)) {
    suppressMessages(ggsave(
      filename = files[k],
      plot = nexus$TM[[k - 1]]$map +
        ggplot2::labs(
          title = paste0("Period: ", periods[k - 1]),
          dpi = 72,
          height = 7,
          width = 14,
          bg = "transparent"
        )
    ))
  }
  return(files)
}

copy_to_clipboard <- function(x) {
  # Check the operating system
  sys_info <- Sys.info()
  os_type <- tolower(sys_info["sysname"])

  # Convert the object to a string if it is not already
  if (!is.character(x)) {
    x <- capture.output(print(x))
  }

  # Copy to clipboard based on the operating system
  if (os_type == "windows") {
    writeClipboard(x)
  } else if (os_type == "darwin") {
    # macOS
    con <- pipe("pbcopy", "w")
    writeLines(x, con)
    close(con)
  } else if (os_type == "linux") {
    # Use xclip or xsel, if available
    if (nzchar(Sys.which("xclip"))) {
      con <- pipe("xclip -selection clipboard", "w")
      writeLines(x, con)
      close(con)
    } else if (nzchar(Sys.which("xsel"))) {
      con <- pipe("xsel --clipboard --input", "w")
      writeLines(x, con)
      close(con)
    } else {
      stop(
        "Neither 'xclip' nor 'xsel' are available. Please install one of them to use the clipboard on Linux."
      )
    }
  } else {
    stop("Unrecognized or unsupported operating system.")
  }
}

## New function to convert gemini output as HTML blocks
gemini_to_html <- function(text, font_size = "16px") {
  # Remove original leading/trailing whitespace
  text <- trimws(text)

  # Divide text into lines
  lines <- unlist(strsplit(text, "\n", fixed = TRUE))

  # Initialize HTML output with CSS styles including font size
  html_lines <- c(
    paste0(
      "<div style='font-family: Arial, sans-serif; line-height: 1.4; margin: 0 auto; padding: 15px; font-size: ",
      font_size,
      ";'>"
    )
  )

  in_list <- FALSE
  list_type <- ""
  last_ol_number <- 0

  close_list <- function() {
    if (in_list) {
      if (list_type == "ul") {
        html_lines <<- c(html_lines, "</ul>")
      } else {
        html_lines <<- c(html_lines, "</ol>")
      }
      in_list <<- FALSE
    }
  }

  for (i in seq_along(lines)) {
    line <- trimws(lines[i])

    # Skip empty lines
    if (line == "") next

    # Skip horizontal rules (--- or ***)
    if (grepl("^[-*_]{3,}\\s*$", line)) next

    # Markdown header management (lines starting with #, ##, ###, ####)
    if (stringr::str_detect(line, "^#{1,4}\\s+")) {
      close_list()

      header_level <- nchar(stringr::str_extract(line, "^#+"))
      header_text <- stringr::str_replace(line, "^#+\\s+", "")
      header_text <- stringr::str_replace(header_text, "\\s*#+\\s*$", "")
      header_text <- stringr::str_replace_all(header_text, "\\*\\*([^*]+)\\*\\*", "\\1")

      h_tag <- paste0("h", min(header_level + 1, 5))
      h_size <- switch(as.character(header_level),
        "1" = "font-size: 1.3em;",
        "2" = "font-size: 1.15em;",
        "3" = "font-size: 1.05em;",
        "font-size: 1em;"
      )
      html_lines <- c(
        html_lines,
        paste0(
          "<", h_tag, " style='color: #333; border-bottom: 1px solid #007acc; padding-bottom: 4px; margin-bottom: 6px; margin-top: 14px; ", h_size, "'>",
          header_text,
          "</", h_tag, ">"
        )
      )
      next
    }

    # Title management (lines enclosed in ** only)
    if (stringr::str_detect(line, "^\\*\\*[^*]+\\*\\*$")) {
      close_list()

      title_text <- stringr::str_replace_all(line, "^\\*\\*(.+)\\*\\*$", "\\1")
      title_text <- stringr::str_replace(title_text, "^\\d+\\.\\s*", "")
      html_lines <- c(
        html_lines,
        paste0(
          "<h3 style='color: #333; border-bottom: 1px solid #007acc; padding-bottom: 4px; margin-bottom: 6px; margin-top: 14px;'>",
          title_text,
          "</h3>"
        )
      )
      next
    }

    # Managing bulleted lists (starting with * or -)
    if (stringr::str_detect(line, "^\\s*[*\\-]\\s+")) {
      if (!in_list || list_type != "ul") {
        close_list()
        html_lines <- c(
          html_lines,
          "<ul style='margin-bottom: 6px; margin-top: 4px;'>"
        )
        in_list <- TRUE
        list_type <- "ul"
      }

      item_text <- stringr::str_replace(line, "^\\s*[*\\-]\\s+", "")
      item_text <- format_inline_text(item_text)

      html_lines <- c(
        html_lines,
        paste0("<li style='margin-bottom: 2px;'>", item_text, "</li>")
      )
      next
    }

    # Managing numbered lists (starting with a number followed by a period)
    if (stringr::str_detect(line, "^\\s*\\d+\\.\\s+")) {
      item_number <- as.integer(stringr::str_extract(line, "^\\s*\\d+"))

      if (!in_list || list_type != "ol") {
        close_list()
        start_attr <- if (item_number > 1) paste0(" start='", item_number, "'") else ""
        html_lines <- c(
          html_lines,
          paste0("<ol style='margin-bottom: 6px; margin-top: 4px;'", start_attr, ">")
        )
        in_list <- TRUE
        list_type <- "ol"
      }

      last_ol_number <- item_number
      item_text <- stringr::str_replace(line, "^\\s*\\d+\\.\\s+", "")
      item_text <- format_inline_text(item_text)
      html_lines <- c(
        html_lines,
        paste0("<li style='margin-bottom: 2px;'>", item_text, "</li>")
      )
      next
    }

    # Non-list, non-header line: close any open list
    close_list()

    # Normal paragraph management
    formatted_line <- format_inline_text(line)
    html_lines <- c(
      html_lines,
      paste0("<p style='margin-bottom: 5px; margin-top: 2px;'>", formatted_line, "</p>")
    )
  }

  # Close any lists that remain open
  close_list()

  # Close the container div
  html_lines <- c(html_lines, "</div>")

  # Merge all lines
  html_result <- paste(html_lines, collapse = "\n")

  return(html_result)
}

# Auxiliary function for formatting inline text
format_inline_text <- function(text) {
  # Bold management (**text**)
  text <- stringr::str_replace_all(
    text,
    "\\*\\*([^*]+)\\*\\*",
    "<strong>\\1</strong>"
  )

  # Italic management (*text*) - only if it is not already in bold
  text <- stringr::str_replace_all(
    text,
    "(?<!\\*)\\*([^*]+)\\*(?!\\*)",
    "<em>\\1</em>"
  )

  # Handling text in quotation marks as inline code ("text")
  text <- stringr::str_replace_all(
    text,
    '"([^"]+)"',
    '<code style="background-color: #f4f4f4; padding: 2px 4px; border-radius: 3px; font-family: \'Courier New\', monospace;">\\1</code>'
  )

  # Handling parentheses with percentages or numerical values
  text <- stringr::str_replace_all(
    text,
    "\\(([^)]*%[^)]*)\\)",
    "<span style='color: #007acc; font-weight: bold;'>(\\1)</span>"
  )

  return(text)
}


# Thematic Map top 3 documents for each cluster
doc2clust <- function(res, n = 3) {
  df <- res$doc2clust %>%
    drop_na(Assigned_cluster) %>%
    group_by(Assigned_cluster) %>%
    slice_max(pagerank, n = n) %>%
    mutate(cluster = paste(SR, ", pagerank ", round(pagerank, 3), sep = "")) %>%
    select(Assigned_cluster, cluster)
}

# APOT top 3 Documents per author
apot2Docs <- function(res, n = 3) {
  df <- res$dfPapersAU %>%
    group_by(Author) %>%
    slice_max(TCpY, n = n) %>%
    mutate(
      SR = paste(TI, SO, year, paste0("TCpY ", round(TCpY, 1)), sep = ", ")
    ) %>%
    select(Author, SR)
}

# Country collaboration table
country2collab <- function(df, n = 20) {
  df <- df %>%
    slice_max(Articles, n = n) %>%
    mutate(
      SR = paste(
        paste0("Article ", Articles, sep = ", "),
        paste0("SCP ", SCP, sep = ", "),
        paste0("MCP ", MCP, sep = ", "),
        paste0("MCP % ", round(`MCP %`, 1), sep = ", ")
      )
    ) %>%
    select(Country, SR)
}

#Local citation table
localCit2docs <- function(df, n = 20) {
  df <- df %>%
    slice_max(Local.Citations, n = n) %>%
    mutate(
      SR = paste(
        paste0("LC ", Local.Citations, sep = ", "),
        paste0("GC ", Global.Citations, sep = ", "),
        paste0("NLC ", round(Normalized.Local.Citations, 2), sep = ", "),
        paste0("NGC ", round(Normalized.Global.Citations, 2)),
        sep = ""
      )
    ) %>%
    select(Document, SR)
}

# Historiograph doc titles
hist2docs <- function(df, n = 20) {
  df %>%
    slice_max(order_by = LCS, n = n) %>%
    select(short_label, title_orig)
}

# Top 10 peaks in RPYS
rpysPeaks <- function(res, n = 10) {
  df_peaks <- res$rpysTable %>%
    arrange(Year) %>%
    mutate(
      is_peak = (diffMedian > lag(diffMedian)) & (diffMedian > lead(diffMedian))
    ) %>%
    dplyr::filter(is_peak) %>%
    arrange(desc(diffMedian)) %>%
    slice_head(n = n)

  df2 <- res$CR %>%
    group_by(Year) %>%
    slice_max(Freq, n = 3, with_ties = FALSE) %>%
    dplyr::filter(Year %in% df_peaks$Year)

  return(peaks = df2)
}

utils::globalVariables(c("group", "size"))
#' Splitting Network communities
#'
#' \code{networkPlot} Create a network plot with separated communities.
#'
#' The function \code{\link{splitCommunities}} splits communities in separated subnetworks from a bibliographic network plot previously created by \code{\link{networkPlot}}.
#'
#' @param graph is a network plot obtained by the function \code{\link{networkPlot}}.
#' @param n is an integer. It indicates the number of vertices to plot for each community.
#' @return It is a network object of the class \code{igraph}
#'
#'
#' @examples
#' # EXAMPLE Keywordd co-occurrence network
#'
#' data(management, package = "bibliometrixData")
#'
#' NetMatrix <- biblioNetwork(management,
#'   analysis = "co-occurrences",
#'   network = "keywords", sep = ";"
#' )
#'
#' net <- networkPlot(NetMatrix,
#'   n = 30, type = "auto",
#'   Title = "Co-occurrence Network", labelsize = 1, verbose = FALSE
#' )
#'
#' graph <- splitCommunities(net$graph, n = 30)
#'
#' @seealso \code{\link{biblioNetwork}} to compute a bibliographic network.
#' @seealso \code{\link{networkPlot}} to plot a bibliographic network.
#' @seealso \code{\link{net2VOSviewer}} to export and plot the network with VOSviewer software.
#' @seealso \code{\link{cocMatrix}} to compute a co-occurrence matrix.
#' @seealso \code{\link{biblioAnalysis}} to perform a bibliometric analysis.
#'
#' @export
splitCommunities <- function(graph, n = NULL) {
  df <- data.frame(label = V(graph)$name, size = V(graph)$deg, group = V(graph)$community)

  if (!is.null(n)) {
    labels <- df %>%
      group_by(group) %>%
      slice_max(order_by = size, n = n, with_ties = FALSE) %>%
      as.data.frame()
  } else {
    labels <- df %>%
      group_by(group) %>%
      as.data.frame()
  }

  # remove inter-cluster edges
  ind <- which(E(graph)$color == adjustcolor("gray70", alpha.f = graph$alpha / 2))
  coGraph <- igraph::delete_edges(graph, E(graph)[ind])
  ind <- which(V(coGraph)$name %in% labels$label)
  V(coGraph)$label[-ind] <- ""
  igraph::graph_attr(coGraph, "layout") <- igraph::layout_with_fr(coGraph)
  # plot(coGraph)
  return(coGraph)
}

#' Life Cycle Analysis with Logistic Growth Model
#' 
#' Estimates logistic growth model for annual (non-cumulative) publications
#' following Meyer et al. (1999) methodology 
#' 
#' @param data Data frame with columns: year (PY) and number of publications (n)
#' @param forecast_years Number of years to forecast beyond saturation
#' @param plot Logical, if TRUE produces plots
#' @param verbose Logical, if TRUE prints detailed output
#' 
#' @return List containing parameters, forecasts and metrics
#' 
#' @export
lifeCycle <- function(data, 
                      forecast_years = 5,
                      plot = TRUE,
                      verbose = FALSE) {
  
  # Prepare data
  # Drop rows with missing publication year or count: records with NA PY
  # (common in OpenAlex/Scopus exports) would otherwise turn min(df$PY) and
  # all derived quantities into NA, making optim() fail with a non-finite value.
  data <- as.data.frame(data)
  data <- data[is.finite(data$PY) & is.finite(data$n), , drop = FALSE]

  if (nrow(data) < 3) {
    stop("lifeCycle: not enough valid data points. At least 3 years with a ",
         "non-missing publication year are required to fit the logistic model.")
  }

  df <- data[order(data$PY), ]
  df$year_index <- df$PY - min(df$PY) + 1
  
  t <- df$year_index
  N_annual <- df$n  # Annual publications (NOT cumulative)
  base_year <- min(df$PY)
  
  # Logistic cumulative function
  logistic <- function(t, K, tm, delta_t) {
    exponent <- -(log(81)/delta_t) * (t - tm)
    exponent <- pmax(pmin(exponent, 700), -700)
    K / (1 + exp(exponent))
  }
  
  # Derivative: annual publications (rate of change)
  logistic_derivative <- function(t, K, tm, delta_t) {
    exponent <- -(log(81)/delta_t) * (t - tm)
    exponent <- pmax(pmin(exponent, 700), -700)
    constant <- log(81)/delta_t
    K * constant * exp(exponent) / (1 + exp(exponent))^2
  }
  
  # === ESTIMATE INITIAL PARAMETERS ===
  # Use cumulative for initial parameter estimation
  cumulative <- cumsum(N_annual)
  
  estimate_initial_params <- function(t, cum_N, annual_N) {
    # Estimate K from cumulative
    K_est <- max(cum_N) * 2.5
    
    # Find approximate midpoint (where annual publications peak)
    peak_idx <- which.max(annual_N)
    tm_est <- t[peak_idx]
    
    # Estimate delta_t from spread of data
    # Use the range where we have significant growth
    threshold <- max(annual_N) * 0.1
    growth_range <- t[annual_N > threshold]
    if(length(growth_range) > 1) {
      delta_t_est <- (max(growth_range) - min(growth_range)) / 2
    } else {
      delta_t_est <- length(t) / 3
    }
    
    list(K = K_est, tm = tm_est, delta_t = max(1, delta_t_est))
  }
  
  init <- estimate_initial_params(t, cumulative, N_annual)
  
  # === OPTIMIZATION: Fit on ANNUAL publications ===
  objective <- function(params) {
    K <- params[1]
    tm <- params[2]
    delta_t <- params[3]
    
    if (K <= max(cumulative) || delta_t <= 0) {
      return(1e10)
    }
    
    # Predict annual publications using derivative
    predicted <- logistic_derivative(t, K, tm, delta_t)
    
    if (any(!is.finite(predicted))) {
      return(1e10)
    }
    
    # Sum of squared errors on ANNUAL data
    sum((N_annual - predicted)^2)
  }
  
  result <- optim(
    par = c(init$K, init$tm, init$delta_t),
    fn = objective,
    method = "L-BFGS-B",
    lower = c(max(cumulative) * 1.01, min(t) * 0.5, 0.1),
    upper = c(max(cumulative) * 10, max(t) * 3, length(t) * 5),
    control = list(maxit = 2000)
  )
  
  if (result$convergence != 0) {
    result <- optim(
      par = c(init$K, init$tm, init$delta_t),
      fn = objective,
      method = "Nelder-Mead",
      control = list(maxit = 10000)
    )
  }
  
  # Extract parameters
  K <- as.numeric(result$par[1])
  tm <- as.numeric(result$par[2])
  delta_t <- as.numeric(result$par[3])
  
  params <- setNames(c(K, tm, delta_t), c("K", "tm", "delta_t"))
  
  # Fitted values for annual publications
  df$fitted_annual <- logistic_derivative(t, K, tm, delta_t)
  df$fitted_cumulative <- logistic(t, K, tm, delta_t)
  df$cumulative <- cumulative
  
  # Calculate actual years for interpretation
  tm_year <- base_year + tm - 1
  year_10_pct <- base_year + (tm - delta_t/2) - 1
  year_90_pct <- base_year + (tm + delta_t/2) - 1
  year_99_pct <- base_year + (tm + log(99) * delta_t / log(81)) - 1
  
  # === FORECAST ===
  years_to_99pct <- ceiling(year_99_pct - max(df$PY)) + forecast_years
  
  future_t <- seq(max(t) + 1, length.out = years_to_99pct, by = 1)
  future_years <- seq(max(df$PY) + 1, length.out = years_to_99pct, by = 1)
  
  forecast_annual <- logistic_derivative(future_t, K, tm, delta_t)
  forecast_cumulative <- logistic(future_t, K, tm, delta_t)
  
  forecast_df <- data.frame(
    PY = future_years,
    year_index = future_t,
    predicted_annual = forecast_annual,
    predicted_cumulative = forecast_cumulative
  )
  
  # Complete time series for plotting
  all_years <- c(df$PY, forecast_df$PY)
  all_t <- c(df$year_index, forecast_df$year_index)
  all_annual <- logistic_derivative(all_t, K, tm, delta_t)
  all_cumulative <- logistic(all_t, K, tm, delta_t)
  
  # === METRICS ===
  residuals <- N_annual - df$fitted_annual
  R2 <- 1 - sum(residuals^2) / sum((N_annual - mean(N_annual))^2)
  RMSE <- sqrt(mean(residuals^2))
  n_params <- length(params)
  n_obs <- length(N_annual)
  AIC_val <- n_obs * log(sum(residuals^2)/n_obs) + 2 * n_params
  BIC_val <- n_obs * log(sum(residuals^2)/n_obs) + n_params * log(n_obs)
  
  metrics <- data.frame(
    R_squared = R2,
    RMSE = RMSE,
    AIC = AIC_val,
    BIC = BIC_val
  )
  
  # === PLOTS ===
  if (plot) {
    par(mfrow = c(2, 2), mar = c(4, 4, 3, 1))
    
    # Plot 1: Annual Publications (MAIN - like Loglet Lab)
    max_annual <- max(c(N_annual, all_annual), na.rm = TRUE)
    
    plot(all_years, all_annual, type = "l", col = "blue", lwd = 2,
         xlab = "Year", ylab = "Publications (Annual)",
         main = "Life Cycle - Annual Publications",
         ylim = c(0, max_annual * 1.1))
    
    # Add observed points
    points(df$PY, N_annual, pch = 19, col = "blue", cex = 1.2)
    
    # Mark the peak
    abline(v = tm_year, lty = 2, col = "red")
    text(tm_year, max_annual * 0.95, 
         sprintf("Peak: %.1f", tm_year), 
         pos = 4, cex = 0.8, col = "red")
    
    legend("topleft", 
           legend = c("Observed", "Logistic fit", "Peak year"),
           col = c("blue", "blue", "red"), 
           pch = c(19, NA, NA),
           lty = c(NA, 1, 2),
           lwd = c(NA, 2, 1),
           bty = "n")
    
    text(max(all_years), max_annual * 1.05, 
         sprintf("R2 = %.3f", R2),
         adj = 1, cex = 0.9)
    
    # Plot 2: Cumulative Publications
    plot(all_years, all_cumulative, type = "l", col = "darkgreen", lwd = 2,
         xlab = "Year", ylab = "Cumulative Publications",
         main = "Cumulative Growth Curve",
         ylim = c(0, K * 1.05))
    
    points(df$PY, df$cumulative, pch = 19, col = "darkgreen", cex = 1.2)
    
    # Add reference lines
    abline(h = K * 0.5, lty = 3, col = "gray50")
    abline(h = K * 0.9, lty = 3, col = "gray50")
    abline(h = K * 0.99, lty = 3, col = "gray50")
    
    text(min(all_years), K * 0.5, "50%", pos = 4, cex = 0.7, col = "gray50")
    text(min(all_years), K * 0.9, "90%", pos = 4, cex = 0.7, col = "gray50")
    text(min(all_years), K * 0.99, "99%", pos = 4, cex = 0.7, col = "gray50")
    
    legend("topleft", 
           legend = c("Observed", "Logistic fit"),
           col = c("darkgreen", "darkgreen"), 
           pch = c(19, NA),
           lty = c(NA, 1),
           lwd = c(NA, 2),
           bty = "n")
    
    # Plot 3: Fisher-Pry Transform
    ratio_all <- all_cumulative / (K - all_cumulative)
    ratio_all[ratio_all <= 0] <- NA
    fisher_pry_all <- log10(ratio_all)
    
    valid_fp <- is.finite(fisher_pry_all) & all_cumulative < K * 0.99
    
    if (sum(valid_fp) > 2) {
      plot(all_years[valid_fp], fisher_pry_all[valid_fp], 
           type = "l", col = "blue", lwd = 2,
           xlab = "Year", 
           ylab = expression(log[10]*"[F/(1-F)]"),
           main = "Fisher-Pry Transform",
           ylim = c(-2, 2))
      
      valid_obs <- is.finite(fisher_pry_all[1:nrow(df)])
      points(df$PY[valid_obs], fisher_pry_all[1:nrow(df)][valid_obs], 
             pch = 19, col = "blue", cex = 1.2)
      
      axis(4, at = log10(c(1/99, 1/9, 1, 9, 99)), 
           labels = c("1%", "10%", "50%", "90%", "99%"),
           las = 1, cex.axis = 0.8)
      
      abline(h = log10(c(1/99, 1/9, 1, 9, 99)), lty = 3, col = "gray70")
      
      valid_obs_fp <- is.finite(fisher_pry_all[1:nrow(df)])
      if(sum(valid_obs_fp) > 2) {
        fit_lm <- lm(fisher_pry_all[1:nrow(df)][valid_obs_fp] ~ df$PY[valid_obs_fp])
        r2_fp <- summary(fit_lm)$r.squared
        text(min(df$PY), max(fisher_pry_all[valid_fp], na.rm = TRUE), 
             sprintf("R2 = %.3f", r2_fp), adj = 0, cex = 0.9)
                 
      }
    }
    
    # Plot 4: Residuals
    plot(df$PY, residuals, type = "h", col = "darkgray", lwd = 2,
         xlab = "Year", ylab = "Residuals (Annual)", 
         main = "Model Residuals")
    abline(h = 0, col = "red", lty = 2)
    points(df$PY, residuals, pch = 19, col = "steelblue", cex = 1.2)
    
    text(max(df$PY), max(abs(residuals)) * 0.9,
         sprintf("RMSE = %.2f", RMSE),
         adj = 1, cex = 0.9)
    
    par(mfrow = c(1, 1))
  }
  
  if (verbose){
    # === REPORT ===
    cat("\n=== LIFE CYCLE ANALYSIS (LOGISTIC MODEL) ===\n")
    cat("Model: Logistic (fitted on annual publications)\n\n")
    cat("Estimated Parameters:\n")
    cat(sprintf("  K (saturation limit): %.2f\n", params["K"]))
    cat(sprintf("  tm (midpoint index): %.2f\n", params["tm"]))
    cat(sprintf("  delta_t (duration 10-90%%): %.2f\n", params["delta_t"]))
    cat("\nFit Metrics:\n")
    print(round(metrics, 3))
    
    cat("\nInterpretation:\n")
    cat(sprintf("- Estimated saturation (K): %.0f cumulative publications\n", params["K"]))
    cat(sprintf("- Peak year (tm): %.1f\n", tm_year))
    cat(sprintf("- Peak annual publications: %.0f\n", 
                logistic_derivative(tm, K, tm, delta_t)))
    cat(sprintf("- Growth duration 10-90%% (delta_t): %.1f years\n", params["delta_t"]))
    cat(sprintf("- Year reaching 50%% of K: %.1f\n", tm_year))
    cat(sprintf("- Year reaching 90%% of K: %.1f\n", year_90_pct))
    cat(sprintf("- Year reaching 99%% of K: %.1f\n", year_99_pct))
    
    # Forecast summary
    cat("\n--- Forecast Summary ---\n")
    cat(sprintf("Last observed year: %d (%.0f annual / %.0f cumulative)\n", 
                max(df$PY), tail(N_annual, 1), max(df$cumulative)))
    cat(sprintf("Forecast extends to: %d\n", max(forecast_df$PY)))
  }
  
  next_5_year <- max(df$PY) + 5
  if(next_5_year <= max(forecast_df$PY)) {
    next_5_val <- forecast_df$predicted_cumulative[forecast_df$PY == next_5_year]
    if (verbose) cat(sprintf("\nProjection for %d: %.0f cumulative publications\n", 
                next_5_year, next_5_val))
  }
  
  
  results <- list(
    parameters = params,
    parameters_real_years = data.frame(
      K = K,
      tm_year = tm_year,
      delta_t = delta_t,
      peak_annual = logistic_derivative(tm, K, tm, delta_t),
      year_10_pct = year_10_pct,
      year_90_pct = year_90_pct,
      year_99_pct = year_99_pct
    ),
    data = df,
    forecast = forecast_df,
    complete_curve = data.frame(
      year = all_years,
      year_index = all_t,
      annual = all_annual,
      cumulative = all_cumulative
    ),
    metrics = metrics,
    residuals = residuals,
    base_year = base_year
  )
  
  return(invisible(results))
}





# Example usage:
# data <- data.frame(PY = 2000:2024, n = c(5, 8, 12, 20, 35, 55, 80, ...))
# results <- lifeCycle(data, forecast_years = 20, plot = FALSE)
# 
# # For biblioshiny
# p1 <- plotLifeCycle(results, plot_type = "annual")
# p2 <- plotLifeCycle(results, plot_type = "cumulative")
# 
# # In Shiny app:
# output$lifeCyclePlot1 <- plotly::renderPlotly({ p1 })
# output$lifeCyclePlot2 <- plotly::renderPlotly({ p2 })

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net2VOSviewer.R
\name{net2VOSviewer}
\alias{net2VOSviewer}
\title{Open a bibliometrix network in VosViewer}
\usage{
net2VOSviewer(net, vos.path = NULL)
}
\arguments{
\item{net}{is an object created by networkPlot function.}

\item{vos.path}{is a character indicating the full path where VOSviewer.jar is located.}
}
\value{
It write a .net file that can be open in VOSviewer
}
\description{
\code{net2VOSviewer} plots a network created with \code{\link{networkPlot}} using \href{https://www.vosviewer.com/}{VOSviewer} by Nees Jan van Eck and Ludo Waltman.
}
\details{
The function \code{\link{networkPlot}} can plot a bibliographic network previously created by \code{\link{biblioNetwork}}.
The network map can be plotted using internal R routines or using \href{https://www.vosviewer.com/}{VOSviewer} by Nees Jan van Eck and Ludo Waltman.
}
\examples{
# EXAMPLE

# VOSviewer.jar have to be present in the working folder

# data(scientometrics, package = "bibliometrixData")

# NetMatrix <- biblioNetwork(scientometrics, analysis = "co-citation",
# network = "references", sep = ";")

# net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Co-Citation",labelsize=0.5)

# net2VOSviewer(net)

}
\seealso{
\code{\link{biblioNetwork}} to compute a bibliographic network.

\code{\link{networkPlot}} to create and plot a network object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{applyCitationMatching}
\alias{applyCitationMatching}
\title{applyCitationMatching (Deprecated)}
\usage{
applyCitationMatching(
  M,
  threshold = 0.9,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
)
}
\arguments{
\item{M}{A bibliometrix data frame, typically created by \code{\link{convert2df}}.
Must contain the columns:
\itemize{
  \item \code{SR}: Short reference identifier for each document
  \item \code{CR}: Cited references field (citations separated by semicolons)
  \item \code{DB}: (Optional) Database source identifier for format detection
}}

\item{threshold}{Numeric value between 0 and 1 indicating the similarity threshold
for matching citations. Default is 0.90. See \code{\link{normalize_citations}}
for details on selecting appropriate thresholds.}

\item{method}{String distance method to use for fuzzy matching. Options include:
\itemize{
  \item "jw" (default): Jaro-Winkler distance, optimized for bibliographic strings
  \item "lv": Levenshtein distance
  \item "osa": Optimal String Alignment distance
  \item "lcs": Longest Common Subsequence distance
  \item Other methods supported by \code{\link[stringdist]{stringdistmatrix}}
}}

\item{min_chars}{Minimum characters for valid citations (default: 20).}

\item{max_block_size}{Integer. Blocks with at least this many unique normalized
strings skip within-block fuzzy matching and fall back to exact matching only
(default: 100). Passed to \code{\link{normalize_citations}}.}

\item{use_iso4}{Logical. Enable ISO 4 / LTWA journal normalization (default \code{TRUE}).}

\item{use_doi}{Logical. Enable DOI-based exact matching (default \code{TRUE}).}

\item{use_exact}{Logical. Enable exact normalized-string and punctuation-invariant
matching (default \code{TRUE}).}

\item{fuzzy}{Logical. Enable within-block (WoS deterministic and Scopus fuzzy)
matching (default \code{TRUE}).}

\item{use_postproc}{Logical. Enable Phase 4.5 metadata-based post-processing
merge (default \code{TRUE}). These five logical switches are intended primarily
for ablation analyses and are passed through to \code{\link{normalize_citations}}.}

\item{title_guard}{Logical. Enable the optional Phase 4.6 series part-marker
cluster purification that splits over-merged multi-part works (default
\code{FALSE}). Passed through to \code{\link{normalize_citations}}.}
}
\value{
Same as \code{\link{applyReferenceMatching}}.
}
\description{
\strong{Deprecated} in bibliometrix 6.0. This function has been renamed to
\code{\link{applyReferenceMatching}}. Please use \code{applyReferenceMatching()} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bradford.R
\name{bradford}
\alias{bradford}
\title{Bradford's law}
\usage{
bradford(M)
}
\arguments{
\item{M}{is a bibliographic dataframe.}
}
\value{
The function \code{bradford} returns a list containing the following objects:
\tabular{lll}{
\code{table}       \tab   \tab a dataframe with the source distribution partitioned in the three zones\cr
\code{graph}       \tab   \tab the Bradford bibliograph plot in ggplot2 format\cr
\code{graph_shiny} \tab   \tab the Bradford bibliograph plot for biblioshiny (without logo)\cr
\code{zoneSummary} \tab   \tab a dataframe summarizing the three Bradford zones\cr
\code{stat}        \tab   \tab a list of statistical results (coefficients, R2, KS test, Bradford multiplier)}
}
\description{
It estimates Bradford's law source distribution and tests the goodness of fit.
}
\details{
Bradford's Law of Scattering, first formulated by Samuel C. Bradford in 1934,
describes the phenomenon of concentration and dispersion in scientific publishing:
a small number of core journals account for a disproportionately large share of the
literature on a given topic, while the remaining literature is scattered across an
increasingly large number of peripheral journals.\cr\cr

If journals are ranked in decreasing order of productivity and partitioned into three
zones, each containing roughly one-third of the total articles, the number of journals
in each zone follows the ratio 1:n:n^2, where n is the Bradford multiplier.\cr\cr

The Bradford distribution models the cumulative number of articles C(r) contributed
by the top r sources as: C(r) = a + b * log(r)\cr\cr

Reference:\cr
Bradford, S. C. (1934). Sources of information on specific subjects. Engineering, 137, 85-86.\cr
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")

BR <- bradford(management)
}

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{summary}} method for class '\code{bibliometrix}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadBookDataset.R
\name{loadBookDataset}
\alias{loadBookDataset}
\title{Load the Biblioshiny Book Dataset}
\usage{
loadBookDataset(force_download = FALSE)
}
\arguments{
\item{force_download}{Logical. If \code{TRUE}, re-downloads the dataset even
if a cached copy exists locally. Default is \code{FALSE}.}
}
\value{
A bibliometrix data frame containing the book dataset.
}
\description{
Downloads and caches the management collection dataset used in the book
"Science Mapping Analysis - A primer with Biblioshiny"
by Massimo Aria and Corrado Cuccurullo (McGraw-Hill, 2026).
The dataset is downloaded from GitHub on first use and
cached locally in \code{~/.bibliometrix/} for subsequent calls.
}
\details{
The dataset contains a comprehensive collection of original research articles,
written in English, that employ bibliometric or scientometric methods within
management and business fields, as indexed by Web of Science, over the period
1985--2025.
}
\examples{
\dontrun{
M <- loadBookDataset()
}

}
\references{
Aria, M., & Cuccurullo, C. (2026). \emph{Science Mapping Analysis - A primer
with Biblioshiny}. McGraw-Hill. ISBN: 978-88-386-2297-7.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keywordAssoc.R
\name{keywordAssoc}
\alias{keywordAssoc}
\title{ID and DE keyword associations}
\usage{
keywordAssoc(M, sep = ";", n = 10, excludeKW = NA)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{sep}{is the field separator character. This character separates keywords in each string of ID and DE columns of the bibliographic data frame. The default is \code{sep = ";"}.}

\item{n}{is a integer. It indicates the number of authors' keywords to associate to each keyword plus. The default is \code{n = 10}.}

\item{excludeKW}{is character vector. It contains authors' keywords to exclude from the analysis.}
}
\value{
an object of \code{class} "list".
}
\description{
It associates authors' keywords to keywords plus.
}
\examples{

data(scientometrics, package = "bibliometrixData")

KWlist <- keywordAssoc(scientometrics, sep = ";", n = 10, excludeKW = NA)

# list of first 10 Keywords plus
names(KWlist)

# list of first 10 authors' keywords associated to the first Keyword plus
KWlist[[1]][1:10]

}
\seealso{
\code{\link{convert2df}} to import and convert a WoS or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeSimilarity.R
\name{normalizeSimilarity}
\alias{normalizeSimilarity}
\title{Calculate similarity indices}
\usage{
normalizeSimilarity(NetMatrix, type = "association")
}
\arguments{
\item{NetMatrix}{is a coupling matrix obtained by the network functions \code{\link{biblioNetwork}} or \code{\link{cocMatrix}}.}

\item{type}{is a character. It can be "association", "jaccard", "inclusion","salton" or "equivalence" to obtain Association Strength, Jaccard,
Inclusion, Salton or Equivalence similarity index respectively. The default is \code{type = "association"}.}
}
\value{
a similarity matrix.
}
\description{
It calculates a relative measure of bibliographic co-occurrences.
}
\details{
\code{couplingSimilarity} calculates Association strength, Inclusion, Jaccard or Salton similarity from a co-occurrence bibliographic matrix.

The association strength is used by Van Eck and Waltman (2007) and Van Eck et al. (2006). Several works refer to the measure as the proximity index,
while Leydesdorff (2008)and Zitt et al. (2000) refer to it as the probabilistic affinity (or activity) index.

The inclusion index, also called Simpson coefficient, is an overlap measure used in information retrieval.

The Jaccard index (or Jaccard similarity coefficient) gives us a relative measure of the overlap of two sets.
It is calculated as the ratio between the intersection and the union of the reference lists (of two manuscripts).

The Salton index, instead, relates the intersection of the two lists to the geometric mean of the size of both sets.
The square of Salton index is also called Equivalence index.

The indices are equal to zero if the intersection of the reference lists is empty.\cr\cr

References\cr\cr
Leydesdorff, L. (2008). On the normalization and visualization of author Cocitation data: Salton's cosine versus the Jaccard index.
Journal of the American Society for Information Science and Technology, 59(1), 77– 85.\cr
Van Eck, N.J., Waltman, L., Van den Berg, J., & Kaymak, U. (2006). Visualizing the computational intelligence field.
IEEE Computational Intelligence Magazine, 1(4), 6– 10.\cr
Van Eck, N.J., & Waltman, L. (2007). Bibliometric mapping of the computational intelligence field.
International Journal of Uncertainty, Fuzziness and Knowledge-Based Systems, 15(5), 625– 645\cr.
Van Eck, N. J., & Waltman, L. (2009). How to normalize cooccurrence data? An analysis of some well-known similarity measures.
Journal of the American society for information science and technology, 60(8), 1635-1651.\cr
Zitt, M., Bassecoulard, E., & Okubo, Y. (2000). Shadows of the past in international cooperation:
Collaboration profiles of the top five producers of science. Scientometrics, 47(3), 627– 657.\cr
}
\examples{

data(scientometrics, package = "bibliometrixData")
NetMatrix <- biblioNetwork(scientometrics,
  analysis = "co-occurrences",
  network = "keywords", sep = ";"
)
S <- normalizeSimilarity(NetMatrix, type = "association")

}
\seealso{
\code{\link{biblioNetwork}} function to compute a bibliographic network.

\code{\link{cocMatrix}} to compute a bibliographic bipartite network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thematicEvolution.R
\name{thematicEvolution}
\alias{thematicEvolution}
\title{Perform a Thematic Evolution Analysis}
\usage{
thematicEvolution(
  M,
  field = "ID",
  years,
  n = 250,
  minFreq = 2,
  size = 0.5,
  ngrams = 1,
  stemming = FALSE,
  n.labels = 1,
  repel = TRUE,
  remove.terms = NULL,
  synonyms = NULL,
  cluster = "louvain",
  seed = 1234,
  assign.evolution.colors = list(assign = TRUE, alpha = 0.5)
)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.}

\item{field}{is a character object. It indicates the content field to use. Field can be one of c=("ID","DE","KW_Merged","TI","AB"). Default value is \code{field="ID"}.}

\item{years}{is a numeric vector of one or more unique cut points.}

\item{n}{is numerical. It indicates the number of words to use in the network analysis}

\item{minFreq}{is numerical. It indicates the min frequency of words included in to a cluster.}

\item{size}{is numerical. It indicates del size of the cluster circles and is a number in the range (0.01,1).}

\item{ngrams}{is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.}

\item{stemming}{is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).}

\item{n.labels}{is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.}

\item{repel}{is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}

\item{cluster}{is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").}

\item{seed}{is numerical. It indicates the seed for random number generator to obtain always the same results. Default value is \code{seed = 1234}.}

\item{assign.evolution.colors}{is a list. If \code{assignEvolutionColors = list(assign = TRUE)}, colors are assigned to lineages based on the highest weighted inclusion value. If a list is provided, it must contain the arguments \code{assignEvolutionColors = list(assign = c(TRUE, FALSE), alpha=0.5)}.
Default is \code{assign.evolution.colors = list(assign=TRUE, alpha = 0.5)}. If assign = FALSE, measure argument is ignored.}
}
\value{
a list containing:
\tabular{lll}{
\code{nets}\tab   \tab The thematic nexus graph for each comparison\cr
\code{incMatrix}\tab   \tab Some useful statistics about the thematic nexus}
}
\description{
It performs a Thematic Evolution Analysis based on co-word network analysis and clustering.
The methodology is inspired by the proposal of Cobo et al. (2011).
}
\details{
\code{\link{thematicEvolution}} starts from two or more thematic maps created by \code{\link{thematicMap}} function.\cr\cr

Reference:\cr
Cobo, M. J., Lopez-Herrera, A. G., Herrera-Viedma, E., & Herrera, F. (2011). An approach for detecting, quantifying,
and visualizing the evolution of a research field: A practical application to the fuzzy sets theory field. Journal of Informetrics, 5(1), 146-166.\cr
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")
years=c(2004,2008,2015)

nexus <- thematicEvolution(management,field="DE",years=years,n=100,minFreq=2)
}

}
\seealso{
\code{\link{thematicMap}} function to create a thematic map based on co-word network analysis and clustering.

\code{\link{cocMatrix}} to compute a bibliographic bipartite network.

\code{\link{networkPlot}} to plot a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/idByAuthor.R
\name{idByAuthor}
\alias{idByAuthor}
\title{Get Complete Author Information and ID from Scopus}
\usage{
idByAuthor(df, api_key)
}
\arguments{
\item{df}{is a dataframe composed of three columns:
\tabular{lll}{
\code{lastname}\tab   \tab author's last name\cr
\code{firstname}\tab   \tab author's first name\cr
\code{affiliation}\tab   \tab Part of the affiliation name (university name, city, etc.)}
i.e. df[1,1:3]<-c("aria","massimo","naples")
When affiliation is not specified, the field df$affiliation have to be NA.
i.e. df[2,1:3]<-c("cuccurullo","corrado", NA)}

\item{api_key}{is a character. It contains the Elsevier API key. Information about how to obtain an API Key \href{https://dev.elsevier.com/sc_apis.html}{Elsevier API website}}
}
\value{
a data frame with cases corresponding to authors and variables to author's information and ID got from SCOPUS.
}
\description{
Uses SCOPUS API author search to identify author identification information.
}
\examples{
## Request a personal API Key to Elsevier web page https://dev.elsevier.com/sc_apis.html
#
# api_key="your api key"

## create a data frame with the list of authors to get information and IDs
# i.e. df[1,1:3]<-c("aria","massimo","naples")
#      df[2,1:3]<-c("cuccurullo","corrado", NA)

## run idByAuthor function
#
# authorsID <- idByAuthor(df, api_key)

}
\seealso{
\code{\link{retrievalByAuthorID}} for downloading the complete author bibliographic collection from SCOPUS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conceptualStructure.R
\name{conceptualStructure}
\alias{conceptualStructure}
\title{Creating and plotting conceptual structure map of a scientific field}
\usage{
conceptualStructure(
  M,
  field = "ID",
  ngrams = 1,
  method = "MCA",
  quali.supp = NULL,
  quanti.supp = NULL,
  minDegree = 2,
  clust = "auto",
  k.max = 5,
  stemming = FALSE,
  labelsize = 10,
  documents = 2,
  graph = TRUE,
  remove.terms = NULL,
  synonyms = NULL
)
}
\arguments{
\item{M}{is a data frame obtained by the converting function
\code{\link{convert2df}}. It is a data matrix with cases corresponding to
articles and variables to Field Tag in the original ISI or SCOPUS file.}

\item{field}{is a character object. It indicates one of the field tags of the
standard ISI WoS Field Tag codify.
field can be equal to one of these tags:
\tabular{lll}{
\code{ID}\tab   \tab Keywords Plus associated by ISI or SCOPUS database\cr
\code{DE}\tab   \tab Author's keywords\cr
\code{KW_Merged}\tab   \tab All keywords\cr
\code{ID_TM}\tab   \tab Keywords Plus stemmed through the Porter's stemming algorithm\cr
\code{DE_TM}\tab   \tab Author's Keywords stemmed through the Porter's stemming algorithm\cr
\code{TI}\tab   \tab Terms extracted from titles\cr
\code{AB}\tab   \tab Terms extracted from abstracts}}

\item{ngrams}{is an integer between 1 and 3. It indicates the type of n-gram to extract from texts.
An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.}

\item{method}{is a character object. It indicates the factorial method used to create the factorial map. Use \code{method="CA"} for Correspondence Analysis,
\code{method="MCA"} for Multiple Correspondence Analysis or \code{method="MDS"} for Metric Multidimensional Scaling. The default is \code{method="MCA"}}

\item{quali.supp}{is a vector indicating the indexes of the categorical supplementary variables. It is used only for CA and MCA.}

\item{quanti.supp}{is a vector indicating the indexes of the quantitative supplementary variables. It is used only for CA and MCA.}

\item{minDegree}{is an integer. It indicates the minimum occurrences of terms to analyze and plot. The default value is 2.}

\item{clust}{is an integer or a character. If clust="auto", the number of cluster is chosen automatically, otherwise clust can be an integer between 2 and 8.}

\item{k.max}{is an integer. It indicates the maximum number of cluster to keep. The default value is 5. The max value is 20.}

\item{stemming}{is logical. If TRUE the Porter's Stemming algorithm is applied to all extracted terms. The default is \code{stemming = FALSE}.}

\item{labelsize}{is an integer. It indicates the label size in the plot. Default is \code{labelsize=10}}

\item{documents}{is an integer. It indicates the number of documents per cluster to plot in the factorial map. The default value is 2. It is used only for CA and MCA.}

\item{graph}{is logical. If TRUE the function plots the maps otherwise they are saved in the output object. Default value is TRUE}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}
}
\value{
It is an object of the class \code{list} containing the following components:

\tabular{lll}{
net \tab  \tab bipartite network\cr
res \tab       \tab Results of CA, MCA or MDS method\cr
km.res \tab      \tab Results of cluster analysis\cr
graph_terms \tab      \tab Conceptual structure map (class "ggplot2")\cr
graph_documents_Contrib \tab      \tab Factorial map of the documents with the highest contributes (class "ggplot2")\cr
graph_docuemnts_TC \tab      \tab Factorial map of the most cited documents (class "ggplot2")}
}
\description{
The function \code{conceptualStructure} creates a conceptual structure map of
a scientific field performing Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA) or Metric Multidimensional Scaling (MDS) and Clustering
of a bipartite network of terms extracted from keyword, title or abstract fields.
}
\examples{
# EXAMPLE Conceptual Structure using Keywords Plus

data(scientometrics, package = "bibliometrixData")

CS <- conceptualStructure(scientometrics,
  field = "ID", method = "CA",
  stemming = FALSE, minDegree = 3, k.max = 5
)

}
\seealso{
\code{\link{termExtraction}} to extract terms from a textual field (abstract, title,
author's keywords, etc.) of a bibliographic data frame.

\code{\link{biblioNetwork}} to compute a bibliographic network.

\code{\link{cocMatrix}} to compute a co-occurrence matrix.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/networkStat.R
\name{networkStat}
\alias{networkStat}
\title{Calculating network summary statistics}
\usage{
networkStat(object, stat = "network", type = "degree")
}
\arguments{
\item{object}{is a network matrix obtained by the function \code{\link{biblioNetwork}} or an graph object of the class \code{igraph}.}

\item{stat}{is a character. It indicates which statistics are to be calculated. \code{stat = "network"} calculates the statistics related to the network;
\code{stat = "all"} calculates the statistics related to the network and the individual nodes that compose it. Default value is \code{stat = "network"}.}

\item{type}{is a character. It indicates which centrality index is calculated. type values can be c("degree", "closeness", "betweenness","eigenvector","pagerank","hub","authority", "all"). Default is "degree".}
}
\value{
It is a list containing the following elements:
\tabular{lll}{
\code{graph} \tab  \tab a network object of the class \code{igraph}\cr
\code{network} \tab  \tab a \code{communities} a list with the main statistics of the network\cr
\code{vertex} \tab  \tab a data frame with the main measures of centrality and prestige of vertices.\cr}
}
\description{
\code{networkStat} calculates main network statistics.
}
\details{
The function \code{\link{networkStat}} can calculate the main network statistics from a bibliographic network previously created by \code{\link{biblioNetwork}}.
}
\examples{
# EXAMPLE Co-citation network

# to run the example, please remove # from the beginning of the following lines
# data(scientometrics, package = "bibliometrixData")

# NetMatrix <- biblioNetwork(scientometrics, analysis = "co-citation",
#      network = "references", sep = ";")

# netstat <- networkStat(NetMatrix, stat = "all", type = "degree")

}
\seealso{
\code{\link{biblioNetwork}} to compute a bibliographic network.

\code{\link{cocMatrix}} to compute a co-occurrence matrix.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stopwords.R
\name{stopwords}
\alias{stopwords}
\title{List of English stopwords.}
\format{
A character vector with 665 rows.\cr
}
\description{
A character vector containing a complete list of English stopwords\cr
Data are used by  \code{\link{biblioAnalysis}} function
to extract Country Field of Cited References and Authors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimES.R
\name{trimES}
\alias{trimES}
\title{Deleting extra white spaces}
\usage{
trimES(x)
}
\arguments{
\item{x}{is a \code{character} object.}
}
\value{
an object of class \code{character}
}
\description{
Deleting extra white spaces from a \code{character} object.
}
\details{
\code{tableTag} is an internal routine of \code{bibliometrics} package.
}
\examples{

char <- c("Alfred  BJ", "Mary    Beth", "John      John")
char
trimES(char)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splitCommunities.R
\name{splitCommunities}
\alias{splitCommunities}
\title{Splitting Network communities}
\usage{
splitCommunities(graph, n = NULL)
}
\arguments{
\item{graph}{is a network plot obtained by the function \code{\link{networkPlot}}.}

\item{n}{is an integer. It indicates the number of vertices to plot for each community.}
}
\value{
It is a network object of the class \code{igraph}
}
\description{
\code{networkPlot} Create a network plot with separated communities.
}
\details{
The function \code{\link{splitCommunities}} splits communities in separated subnetworks from a bibliographic network plot previously created by \code{\link{networkPlot}}.
}
\examples{
# EXAMPLE Keywordd co-occurrence network

data(management, package = "bibliometrixData")

NetMatrix <- biblioNetwork(management,
  analysis = "co-occurrences",
  network = "keywords", sep = ";"
)

net <- networkPlot(NetMatrix,
  n = 30, type = "auto",
  Title = "Co-occurrence Network", labelsize = 1, verbose = FALSE
)

graph <- splitCommunities(net$graph, n = 30)

}
\seealso{
\code{\link{biblioNetwork}} to compute a bibliographic network.

\code{\link{networkPlot}} to plot a bibliographic network.

\code{\link{net2VOSviewer}} to export and plot the network with VOSviewer software.

\code{\link{cocMatrix}} to compute a co-occurrence matrix.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threeFieldsPlot.R
\name{threeFieldsPlot}
\alias{threeFieldsPlot}
\title{Three Fields Plot}
\usage{
threeFieldsPlot(M, fields = c("DE", "AU", "SO"), n = c(20, 20, 20))
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{fields}{is a character vector. It indicates the fields to analyze using the standard WoS field tags.
Default is \code{fields = c("AU","DE", "SO")}.}

\item{n}{is a integer vector. It indicates how many items to plot, for each of the three fields.
Default is \code{n = c(20, 20, 20)}}
}
\value{
a sankeyPlot
}
\description{
Visualize the main items of three fields (e.g. authors, keywords, journals), and how they are related through a Sankey diagram.
}
\examples{

# data(scientometrics, package = "bibliometrixData")

# threeFieldsPlot(scientometrics, fields=c("DE","AU","CR"),n=c(20,20,20))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logo.R
\name{logo}
\alias{logo}
\title{Bibliometrix logo.}
\format{
A matrix with 927 rows and 800 columns.
}
\description{
The matrix contains the rgb format of the bibliometrix official logo.\cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/citations.R
\name{citations}
\alias{citations}
\title{Citation frequency distribution}
\usage{
citations(M, field = "article", sep = ";")
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics Web of Science file.}

\item{field}{is a character. It can be "article" or "author" to obtain frequency distribution of cited citations or cited authors (only first authors for WoS database) respectively. The default is \code{field = "article"}.}

\item{sep}{is the field separator character. This character separates citations in each string of CR column of the bibliographic data frame. The default is \code{sep = ";"}.}
}
\value{
an object of \code{class} "list"  containing the following components:

\tabular{lll}{
Cited \tab  \tab the most frequent cited manuscripts or authors\cr
Year \tab       \tab the publication year (only for cited article analysis)\cr
Source \tab      \tab the journal (only for cited article analysis)}
}
\description{
It calculates frequency distribution of citations.
}
\examples{
## EXAMPLE 1: Cited articles

data(scientometrics, package = "bibliometrixData")

CR <- citations(scientometrics, field = "article", sep = ";")

CR$Cited[1:10]
CR$Year[1:10]
CR$Source[1:10]

## EXAMPLE 2: Cited first authors

data(scientometrics)

CR <- citations(scientometrics, field = "author", sep = ";")

CR$Cited[1:10]

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifeCycle.R
\name{lifeCycle}
\alias{lifeCycle}
\title{Life Cycle Analysis with Logistic Growth Model}
\usage{
lifeCycle(data, forecast_years = 5, plot = TRUE, verbose = FALSE)
}
\arguments{
\item{data}{Data frame with columns: year (PY) and number of publications (n)}

\item{forecast_years}{Number of years to forecast beyond saturation}

\item{plot}{Logical, if TRUE produces plots}

\item{verbose}{Logical, if TRUE prints detailed output}
}
\value{
List containing parameters, forecasts and metrics
}
\description{
Estimates logistic growth model for annual (non-cumulative) publications
following Meyer et al. (1999) methodology
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lotka.R
\name{lotka}
\alias{lotka}
\title{Lotka's law coefficient estimation}
\usage{
lotka(M)
}
\arguments{
\item{M}{is an object of the class '\code{bibliometrixDB}'.}
}
\value{
The function \code{lotka} returns a list containing the following objects:
\tabular{lll}{
\code{AuthorProd}    \tab   \tab Authors' Productivity frequency table\cr
\code{g}             \tab   \tab Lotka's law plot in ggplot2 format (with logo)\cr
\code{g_shiny}       \tab   \tab Lotka's law plot for biblioshiny (without logo)\cr
\code{stat}          \tab   \tab list of statistical results (Beta, C, R2, KS tests)\cr
\code{Beta}          \tab   \tab Beta coefficient (estimated)\cr
\code{C}             \tab   \tab Constant coefficient\cr
\code{R2}            \tab   \tab Goodness of Fit (R-squared)\cr
\code{fitted}        \tab   \tab Fitted Values\cr
\code{p.value}       \tab   \tab p-value of KS test (theoretical Beta=2)}
}
\description{
It estimates Lotka's law coefficients for scientific productivity and tests the goodness of fit.
}
\details{
Lotka's Law, first formulated by Alfred J. Lotka in 1926, describes the frequency distribution
of scientific productivity among authors. The law states that the number of authors producing
\eqn{n} publications is approximately \eqn{C / n^\beta}, where \eqn{C} is a constant and
\eqn{\beta} is the productivity exponent.\cr\cr

In the original formulation, Lotka proposed that \eqn{\beta = 2}, meaning that the number of
authors who publish \eqn{n} papers is approximately \eqn{1/n^2} of those who publish one paper.
The function estimates both the empirical \eqn{\beta} via regression and tests the fit of
the theoretical distribution (\eqn{\beta = 2}) using a Kolmogorov-Smirnov test.\cr\cr

Reference:\cr
Lotka, A. J. (1926). The frequency distribution of scientific productivity. Journal of the
Washington Academy of Sciences, 16(12), 317-323.\cr
}
\examples{
data(management, package = "bibliometrixData")
L <- lotka(management)
L

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{summary}} method for class '\code{bibliometrix}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biblioAnalysis.R
\name{biblioAnalysis}
\alias{biblioAnalysis}
\title{Bibliometric Analysis}
\usage{
biblioAnalysis(M, sep = ";")
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics Web of Science file.}

\item{sep}{is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.}
}
\value{
\code{biblioAnalysis} returns an object of \code{class} "bibliometrix".

The functions \code{\link{summary}} and \code{\link{plot}} are used to obtain or print a summary and some useful plots of the results.

An object of \code{class} "bibliometrix" is a list containing the following components:

\tabular{lll}{
Articles \tab  \tab the total number of manuscripts\cr
Authors \tab       \tab the authors' frequency distribution\cr
AuthorsFrac \tab      \tab the authors' frequency distribution (fractionalized)\cr
FirstAuthors \tab      \tab corresponding author of each manuscript\cr
nAUperPaper \tab      \tab the number of authors per manuscript\cr
Appearances \tab      \tab the number of author appearances\cr
nAuthors \tab       \tab the number of authors\cr
AuMultiAuthoredArt \tab      \tab the number of authors of multi-authored articles\cr
MostCitedPapers \tab      \tab the list of manuscripts sorted by citations\cr
Years \tab      \tab publication year of each manuscript\cr
FirstAffiliation \tab      \tab the affiliation of the first author\cr
Affiliations \tab      \tab the frequency distribution of affiliations (of all co-authors for each paper)\cr
Aff_frac \tab      \tab the fractionalized frequency distribution of affiliations (of all co-authors for each paper)\cr
CO \tab      \tab the affiliation country of the first author\cr
Countries \tab      \tab the affiliation countries' frequency distribution\cr
CountryCollaboration \tab      \tab Intra-country (SCP) and intercountry (MCP) collaboration indices\cr
TotalCitation \tab      \tab the number of times each manuscript has been cited\cr
TCperYear \tab      \tab the yearly average number of times each manuscript has been cited\cr
Sources \tab      \tab the frequency distribution of sources (journals, books, etc.)\cr
DE \tab      \tab the frequency distribution of authors' keywords\cr
ID \tab      \tab the frequency distribution of keywords associated to the manuscript by SCOPUS and Clarivate Analytics Web of Science database}
}
\description{
It performs a bibliometric analysis of a dataset imported from SCOPUS and Clarivate Analytics Web of Science databases.
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")

results <- biblioAnalysis(management)

summary(results, k = 10, pause = FALSE)
}

}
\seealso{
\code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rpys.R
\name{rpys}
\alias{rpys}
\title{Reference Publication Year Spectroscopy}
\usage{
rpys(M, sep = ";", timespan = NULL, median.window = "centered", graph = T)
}
\arguments{
\item{M}{is a data frame obtained by the converting function
\code{\link{convert2df}}. It is a data matrix with cases corresponding to
articles and variables to Field Tag in the original ISI or SCOPUS file.}

\item{sep}{is the cited-references separator character. This character separates cited-references in the CR
column of the data frame. The default is \code{sep = ";"}.}

\item{timespan}{is a numeric vector c(min year,max year). The default value is NULL (the entire timespan is considered).}

\item{median.window}{is a character string that can be "centered" or "backward". It indicates the type of median to be used.
"centered" is the default value and it uses the centered 5-year median (t-2 to t+2) as proposed by Marx et al. (2014). "backward" uses the backward 5-year median (t-4 to t) as proposed by Aria and Cuccurullo (2017).}

\item{graph}{is a logical. If TRUE the function plot the spectroscopy otherwise the plot is created but not drawn down.}
}
\value{
a list containing the spectroscopy (class ggplot2) and three dataframes with the number of citations
per year, the list of the cited references for each year, and the reference list with citations recorded year by year, respectively.
}
\description{
\code{rpys} computes a Reference Publication Year Spectroscopy for detecting
the Historical Roots of Research Fields.
The method was introduced by Marx et al., 2014.\cr\cr
}
\details{
References:\cr\cr
Marx, W., Bornmann, L., Barth, A., & Leydesdorff, L. (2014).
Detecting the historical roots of research fields by reference publication
year spectroscopy (RPYS). Journal of the Association for Information Science and Technology,
65(4), 751-764.\cr\cr
Thor A., Bornmann L., Mark W. & Mutz R.(2018).
Identifying single influential publications in a research field: new analysis opportunities of the CRExplorer.
Scientometrics, 116:591–608 https://doi.org/10.1007/s11192-018-2733-7\cr\cr
}
\examples{

\dontrun{
data(management, package = "bibliometrixData")
res <- rpys(management, sep = ";", graph = TRUE)
}

}
\seealso{
\code{\link{convert2df}} to import and convert an ISI or SCOPUS
  Export file in a data frame.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.

\code{\link{biblioNetwork}} to compute a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.bibliodendrogram.R
\name{plot.bibliodendrogram}
\alias{plot.bibliodendrogram}
\title{Plotting dendrogram resulting from Conceptual Structure Analysis}
\usage{
\method{plot}{bibliodendrogram}(x, ...)
}
\arguments{
\item{x}{is the object for which plots are desired.}

\item{...}{is a generic param for plot functions.}
}
\value{
The function \code{plot} draws a dendrogram.
}
\description{
\code{plot} method for class '\code{bibliodendrogram}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/authorBio.r
\name{get_authors_summary}
\alias{get_authors_summary}
\title{Get Authors Summary from OpenAlex}
\usage{
get_authors_summary(
  doi = "10.1016/j.joi.2017.08.007",
  verbose = FALSE,
  sleep_time = 0.2,
  max_retries = 3,
  mailto = NULL
)
}
\arguments{
\item{doi}{Character. DOI of the article}

\item{verbose}{Logical. Print informative messages during execution (default: FALSE)}

\item{sleep_time}{Numeric. Seconds to wait before API call (default: 0.2)}

\item{max_retries}{Integer. Maximum number of retry attempts (default: 3)}

\item{mailto}{Character. Email address for the OpenAlex API polite pool (default: NULL)}
}
\value{
A data frame with summary information for all authors including:
  \itemize{
    \item position: Author position in the paper
    \item display_name: Author name as it appears in the paper
    \item author_position_type: Type of position (first, last, middle)
    \item is_corresponding: Whether the author is a corresponding author
    \item orcid: ORCID identifier if available
    \item openalex_id: OpenAlex author identifier
    \item primary_affiliation: Main institutional affiliation
  }
}
\description{
Retrieves a quick summary of all authors from a paper without making additional API calls
for individual author profiles. Useful for getting an overview of the authorship structure.
}
\examples{
\dontrun{
# Get a quick summary of all authors
summary <- get_authors_summary(doi = "10.1016/j.joi.2017.08.007")
print(summary)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/termExtraction.R
\name{termExtraction}
\alias{termExtraction}
\title{Term extraction tool from textual fields of a manuscript}
\usage{
termExtraction(
  M,
  Field = "TI",
  ngrams = 1,
  stemming = FALSE,
  language = "english",
  remove.numbers = TRUE,
  remove.terms = NULL,
  keep.terms = NULL,
  synonyms = NULL,
  verbose = TRUE
)
}
\arguments{
\item{M}{is a data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.}

\item{Field}{is a character object. It indicates the field tag of textual data :
\tabular{lll}{
\code{"TI"}\tab   \tab Manuscript title\cr
\code{"AB"}\tab   \tab Manuscript abstract\cr
\code{"ID"}\tab   \tab Manuscript keywords plus\cr
\code{"DE"}\tab   \tab Manuscript author's keywords}
The default is \code{Field = "TI"}.}

\item{ngrams}{is an integer between 1 and 3. It indicates the type of n-gram to extract from texts.
An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.}

\item{stemming}{is logical. If TRUE the Porter Stemming algorithm is applied to all extracted terms. The default is \code{stemming = FALSE}.}

\item{language}{is a character. It is the language of textual contents ("english", "german","italian","french","spanish"). The default is \code{language="english"}.}

\item{remove.numbers}{is logical. If TRUE all numbers are deleted from the documents before term extraction. The default is \code{remove.numbers = TRUE}.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the corpus after term extraction. The default is \code{remove.terms = NULL}.}

\item{keep.terms}{is a character vector. It contains a list of compound words "formed by two or more terms" to keep in their original form in the term extraction process. The default is \code{keep.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}

\item{verbose}{is logical. If TRUE the function prints the most frequent terms extracted from documents. The default is \code{verbose=TRUE}.}
}
\value{
the bibliometric data frame with a new column containing terms about the field tag indicated in the argument \code{Field}.
}
\description{
It extracts terms from a text field (abstract, title, author's keywords, etc.) of a bibliographic data frame.
}
\examples{
# Example 1: Term extraction from titles

data(scientometrics, package = "bibliometrixData")

# vector of compound words
keep.terms <- c("co-citation analysis", "bibliographic coupling")

# term extraction
scientometrics <- termExtraction(scientometrics,
  Field = "TI", ngrams = 1,
  remove.numbers = TRUE, remove.terms = NULL, keep.terms = keep.terms, verbose = TRUE
)

# terms extracted from the first 10 titles
scientometrics$TI_TM[1:10]


# Example 2: Term extraction from abstracts

data(scientometrics)

# term extraction
scientometrics <- termExtraction(scientometrics,
  Field = "AB", ngrams = 2,
  stemming = TRUE, language = "english",
  remove.numbers = TRUE, remove.terms = NULL, keep.terms = NULL, verbose = TRUE
)

# terms extracted from the first abstract
scientometrics$AB_TM[1]

# Example 3: Term extraction from keywords with synonyms

data(scientometrics)

# vector of synonyms
synonyms <- c("citation; citation analysis", "h-index; index; impact factor")

# term extraction
scientometrics <- termExtraction(scientometrics,
  Field = "ID", ngrams = 1,
  synonyms = synonyms, verbose = TRUE
)

}
\seealso{
\code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{prepare_ltwa_lookup}
\alias{prepare_ltwa_lookup}
\title{Prepare LTWA database for efficient lookup}
\usage{
prepare_ltwa_lookup(ltwa_db)
}
\arguments{
\item{ltwa_db}{LTWA database data frame}
}
\value{
List with singles, prefix, and phrase lookup tables
}
\description{
Pre-processes LTWA database into optimized lookup tables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histNetwork.R
\name{histNetwork}
\alias{histNetwork}
\title{Historical co-citation network}
\usage{
histNetwork(M, min.citations, sep = ";", network = TRUE, verbose = TRUE)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function
\code{\link{convert2df}}. It is a data matrix with cases corresponding to
manuscripts and variables to Field Tag in the original SCOPUS, OpenAlex, Lens.org and Clarivate
Analytics Web of Science file.}

\item{min.citations}{DEPRECATED. New algorithm does not use this parameters. It will be remove in the next version of bibliometrix.}

\item{sep}{is the field separator character. This character separates strings
in CR column of the data frame. The default is \code{sep = ";"}.}

\item{network}{is logical. If TRUE, function calculates and returns also the direct citation network. If FALSE,
the function returns only the local citation table.}

\item{verbose}{is logical. If TRUE, results are printed on screen.}
}
\value{
\code{histNetwork} returns an object of \code{class} "list"
  containing the following components:

  \tabular{lll}{ NetMatrix \tab  \tab the historical co-citation network
  matrix\cr histData \tab      \tab the set of n most cited references\cr M
  \tab      \tab the bibliographic data frame}
}
\description{
\code{histNetwork} creates a historical citation network from a bibliographic
data frame.
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")

histResults <- histNetwork(management, sep = ";")
}

}
\seealso{
\code{\link{convert2df}} to import and convert a supported
  export file in a bibliographic data frame.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.

\code{\link{biblioNetwork}} to compute a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/authorProdOverTime.R
\name{authorProdOverTime}
\alias{authorProdOverTime}
\title{Top-Authors' Productivity over Time}
\usage{
authorProdOverTime(M, k = 10, graph = TRUE)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by \code{\link{convert2df}} function.}

\item{k}{is a integer. It is the number of top authors to analyze and plot. Default is \code{k = 10}.}

\item{graph}{is logical. If TRUE the function plots the author production over time graph. Default is \code{graph = TRUE}.}
}
\value{
The function \code{authorProdOverTime} returns a list containing two objects:
\tabular{lll}{
\code{dfAU}  \tab   \tab is a data frame\cr
\code{dfpapersAU}\tab    \tab is a data frame\cr
\code{graph}   \tab   \tab a ggplot object}
}
\description{
It calculates and plots the author production (in terms of number of publications) over the time.
}
\examples{
data(scientometrics, package = "bibliometrixData")
res <- authorProdOverTime(scientometrics, k = 10)
print(res$dfAU)
plot(res$graph)

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{summary}} method for class '\code{bibliometrix}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{remove_diacritics}
\alias{remove_diacritics}
\title{Remove diacritics from string with robust fallback}
\usage{
remove_diacritics(x)
}
\description{
Remove diacritics from string with robust fallback
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findAuthorWorks.r
\name{print_author_works_summary}
\alias{print_author_works_summary}
\title{Print Summary of Author Works Search}
\usage{
print_author_works_summary(works_df)
}
\arguments{
\item{works_df}{Data.frame. Result from find_author_works()}
}
\description{
Prints a summary of the search results from find_author_works()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sourceGrowth.R
\name{sourceGrowth}
\alias{sourceGrowth}
\title{Number of documents published annually per Top Sources}
\usage{
sourceGrowth(M, top = 5, cdf = TRUE)
}
\arguments{
\item{M}{is a data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to articles and variables to Field Tag in the original ISI or SCOPUS file.}

\item{top}{is a numeric. It indicates the number of top sources to analyze. The default value is 5.}

\item{cdf}{is a logical. If TRUE, the function calculates the cumulative occurrences distribution.}
}
\value{
an object of class \code{data.frame}
}
\description{
It calculates yearly published documents of the top sources.
}
\examples{

data(scientometrics, package = "bibliometrixData")
topSO <- sourceGrowth(scientometrics, top = 1, cdf = TRUE)
topSO

# Plotting results
\dontrun{
install.packages("reshape2")
library(reshape2)
library(ggplot2)
DF <- melt(topSO, id = "Year")
ggplot(DF, aes(Year, value, group = variable, color = variable)) +
  geom_line()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readFiles.R
\name{readFiles}
\alias{readFiles}
\title{DEPRECATED: Load a sequence of ISI or SCOPUS Export files into a large character object}
\usage{
readFiles(...)
}
\arguments{
\item{...}{is a sequence of names of files downloaded from WOS.(in plain text or bibtex format) or SCOPUS Export file (exclusively in bibtex format).}
}
\value{
a character vector of length the number of lines read.
}
\description{
The function readFiled is deprecated. You can import and convert your export files directly using the function \code{\link{convert2df}}.
}
\examples{
# WoS or SCOPUS Export files can be read using \code{\link{readFiles}} function:

# largechar <- readFiles('filename1.txt','filename2.txt','filename3.txt')

# filename1.txt, filename2.txt and filename3.txt are ISI or SCOPUS Export file
# in plain text or bibtex format.

# D <- readFiles('https://www.bibliometrix.org/datasets/bibliometrics_articles.txt')

}
\seealso{
\code{\link{convert2df}} for converting SCOPUS of ISI Export file into a dataframe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/completeMetadata.R
\name{completeMetadata}
\alias{completeMetadata}
\title{Complete missing metadata via DOI lookup against Crossref and OpenAlex}
\usage{
completeMetadata(
  M,
  sources = c("openalex", "crossref"),
  fields = c("AB", "AU", "C1", "CR", "DT", "LA", "PY", "RP", "SO", "TC", "TI"),
  email = NULL,
  oa_apikey = NULL,
  batch_size = 20,
  max_records = Inf,
  progress = NULL,
  verbose = TRUE
)
}
\arguments{
\item{M}{Bibliometrix data frame produced by \code{\link{convert2df}}.}

\item{sources}{Character vector of enrichment sources.
Default \code{c("openalex", "crossref")}. Order is irrelevant; OpenAlex
always runs before Crossref. \code{"openalex"} is skipped if
\code{M$DB[1] == "OPENALEX"}.}

\item{fields}{Character vector of WoS-codified fields to attempt to fill.
Default \code{c("AB","AU","C1","CR","DT","LA","PY","RP","SO","TC","TI")}.
\code{TC} is filled only by OpenAlex.}

\item{email}{Optional contact email used as the Crossref polite-pool
identifier and OpenAlex \code{mailto}. If \code{NULL}, the function
falls back to the env var \code{BIBLIOMETRIX_EMAIL} or the persisted
file \code{~/.biblio_openalex_email.txt}.}

\item{oa_apikey}{Optional OpenAlex API key. If \code{NULL}, the function
reads \code{Sys.getenv("openalexR.apikey")} and falls back to
\code{~/.biblio_openalex_apikey.txt}. The OpenAlex pass works without a
key (lower rate limit).}

\item{batch_size}{Number of DOIs per Crossref batch request (default 20).
OpenAlex uses a fixed batch size of 50 (the maximum that keeps URLs
under length limits).}

\item{max_records}{Optional cap on the number of records to enrich
(useful for previewing). Default \code{Inf}.}

\item{progress}{Optional callback \code{function(done, total, label)}
invoked after each batch. Used by biblioshiny to drive a progress bar.}

\item{verbose}{Logical. Print progress messages to the console.}
}
\value{
A list with components:
\describe{
  \item{\code{M}}{The enriched collection (same class as the input).}
  \item{\code{report}}{Long-format \code{data.frame} with one row per
    (field, source) summarising attempts, fills, and failures.}
  \item{\code{before}}{The \code{mandatoryTags} table from
    \code{missingData(M)} before enrichment.}
  \item{\code{after}}{The \code{mandatoryTags} table from
    \code{missingData(M)} after enrichment.}
}
Provenance is attached to the returned collection as
\code{attr(M, "enrichment")}, a long-format \code{data.frame} with columns
\code{SR}, \code{field}, \code{source}, \code{timestamp}.
}
\description{
Given a bibliometrix collection produced by \code{\link{convert2df}}, this
function takes the subset of records that have a DOI but are missing one
or more of the analysis-relevant fields, queries the Crossref REST API
(\url{https://api.crossref.org/works}) and/or OpenAlex (via
\pkg{openalexR}) using the DOI as the lookup key, and fills the gaps
with the values returned by those sources. Existing non-empty values
are never overwritten.
}
\details{
When both sources are enabled, OpenAlex runs first (broader coverage of
AB/CR/C1/TC) and Crossref then fills the residual gaps. If the input
collection was originally imported from OpenAlex (\code{M$DB[1] ==
"OPENALEX"}), the OpenAlex pass is automatically skipped because re-querying
it would not add information.

The vacancy predicate matches the one used by \code{\link{missingData}}:
a cell is considered missing when it is \code{NA} or one of
\code{c("", "NA", "none", "NA,0000,NA")}.

Crossref cannot supply author keywords (DE), Keywords Plus (ID), Web of
Science categories (WC), or citation counts (TC). OpenAlex covers TC,
AB, AU, C1, CR, DT, LA, PY, RP, SO, TI well; OpenAlex \code{keywords}
are AI-derived topic labels and not author keywords, so DE is off by
default. ID and WC are always skipped.
}
\examples{
\dontrun{
data(scientometrics, package = "bibliometrixData")
res <- completeMetadata(scientometrics, email = "you@example.com")
res$report
res$after
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableTag.R
\name{tableTag}
\alias{tableTag}
\title{Tabulate elements from a Tag Field column}
\usage{
tableTag(
  M,
  Tag = "CR",
  sep = ";",
  ngrams = 1,
  remove.terms = NULL,
  synonyms = NULL
)
}
\arguments{
\item{M}{is a data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.}

\item{Tag}{is a character object. It indicates one of the field tags of the
standard ISI WoS Field Tag codify.}

\item{sep}{is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.}

\item{ngrams}{is an integer between 1 and 3. It indicates the type of n-gram to extract from titles or abstracts.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}
}
\value{
an object of class \code{table}
}
\description{
It tabulates elements from a Tag Field column of a bibliographic data frame.
}
\details{
\code{tableTag} is an internal routine of main function \code{\link{biblioAnalysis}}.
}
\examples{

data(scientometrics, package = "bibliometrixData")
Tab <- tableTag(scientometrics, Tag = "CR", sep = ";")
Tab[1:10]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/customTheme.R
\name{customTheme}
\alias{customTheme}
\title{Custom Theme variables for Biblioshiny.}
\format{
A list with 3 elements:
 \describe{
    \item{name}{object name}
    \item{attribs}{attributes}
    \item{children}{CSS style}
    }
}
\description{
List containing a set of custom theme variables for Biblioshiny.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{abbreviate_term}
\alias{abbreviate_term}
\title{Abbreviate a single term using LTWA}
\usage{
abbreviate_term(word, ltwa_lookup, common_abbr, check = TRUE)
}
\arguments{
\item{word}{Single word to abbreviate}

\item{ltwa_lookup}{Pre-processed LTWA lookup tables}

\item{common_abbr}{Named vector of common abbreviations}

\item{check}{Logical, whether to check for abbreviation}
}
\value{
Abbreviated form of word
}
\description{
Abbreviate a single term using LTWA
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrievalByAuthorID.R
\name{retrievalByAuthorID}
\alias{retrievalByAuthorID}
\title{Get Author Content on SCOPUS by ID}
\usage{
retrievalByAuthorID(id, api_key, remove.duplicated = TRUE, country = TRUE)
}
\arguments{
\item{id}{is a vector of characters containing the author's SCOPUS IDs.
SCOPUS IDs con be obtained using the function \code{\link{idByAuthor}}.}

\item{api_key}{is a character. It contains the Elsvier API key. Information about how to obtain an API Key \href{https://dev.elsevier.com/sc_apis.html}{Elsevier API website}}

\item{remove.duplicated}{is logical. If TRUE duplicated documents will be deleted from the bibliographic collection.}

\item{country}{is logical. If TRUE authors' country information will be downloaded from SCOPUS.}
}
\value{
a list containing two objects: (i) M which is a data frame with cases corresponding to articles and variables to main Field Tags named using the standard ISI WoS Field Tag codify.
M includes the entire bibliographic collection downloaded from SCOPUS.
The main field tags are:

\tabular{lll}{
\code{AU}\tab   \tab Authors\cr
\code{TI}\tab   \tab Document Title\cr
\code{SO}\tab   \tab Publication Name (or Source)\cr
\code{DT}\tab   \tab Document Type\cr
\code{DE}\tab   \tab Authors' Keywords\cr
\code{ID}\tab   \tab Keywords associated by SCOPUS or ISI database \cr
\code{AB}\tab   \tab Abstract\cr
\code{C1}\tab   \tab Author Address\cr
\code{RP}\tab   \tab Reprint Address\cr
\code{TC}\tab   \tab Times Cited\cr
\code{PY}\tab   \tab Year\cr
\code{UT}\tab   \tab Unique Article Identifier\cr
\code{DB}\tab   \tab Database\cr}
(ii) authorDocuments which is a list containing a bibliographic data frame for each author.

LIMITATIONS:
Currently, SCOPUS API does not allow to download document references.
As consequence, it is not possible to perform co-citation analysis (the field CR is empty).
}
\description{
Uses SCOPUS API search to get information about documents on a set of authors using SCOPUS ID.
}
\examples{
## Request a personal API Key to Elsevier web page https://dev.elsevier.com/sc_apis.html

## api_key="your api key"

## create a data frame with the list of authors to get information and IDs
# i.e. df[1,1:3] <- c("aria","massimo","naples")
#      df[2,1:3] <- c("cuccurullo","corrado", "naples")

## run idByAuthor function
#
# authorsID <- idByAuthor(df, api_key)
#

## extract the IDs
#
# id <- authorsID[,3]
#

## create the bibliographic collection
#
# res <- retrievalByAuthorID(id, api_key)
#
# M <- res$M  # the entire bibliographic data frame
# M <- res$authorDocuments # the list containing a bibliographic data frame for each author

}
\seealso{
\code{\link{idByAuthor}} for downloading author information and SCOPUS ID.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keywordGrowth.R
\name{KeywordGrowth}
\alias{KeywordGrowth}
\title{Yearly occurrences of top keywords/terms}
\usage{
KeywordGrowth(
  M,
  Tag = "ID",
  sep = ";",
  top = 10,
  cdf = TRUE,
  remove.terms = NULL,
  synonyms = NULL
)
}
\arguments{
\item{M}{is a data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.}

\item{Tag}{is a character object. It indicates one of the keyword field tags of the
standard ISI WoS Field Tag codify (ID, DE, KW_Merged) or a field tag created by \code{\link{termExtraction}} function (TI_TM, AB_TM, etc.).}

\item{sep}{is the field separator character. This character separates strings in each keyword column of the data frame. The default is \code{sep = ";"}.}

\item{top}{is a numeric. It indicates the number of top keywords to analyze. The default value is 10.}

\item{cdf}{is a logical. If TRUE, the function calculates the cumulative occurrences distribution.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}
}
\value{
an object of class \code{data.frame}
}
\description{
It calculates yearly occurrences of top keywords/terms.
}
\examples{

data(scientometrics, package = "bibliometrixData")
topKW <- KeywordGrowth(scientometrics, Tag = "ID", sep = ";", top = 5, cdf = TRUE)
topKW

# Plotting results
\dontrun{
install.packages("reshape2")
library(reshape2)
library(ggplot2)
DF <- melt(topKW, id = "Year")
ggplot(DF, aes(Year, value, group = variable, color = variable)) + geom_line
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collabByRegionPlot.R
\name{collabByRegionPlot}
\alias{collabByRegionPlot}
\title{Country Collaboration Networks by Region}
\usage{
collabByRegionPlot(
  NetMatrix,
  normalize = NULL,
  n = NULL,
  degree = NULL,
  type = "auto",
  label = TRUE,
  labelsize = 1,
  label.cex = FALSE,
  label.color = FALSE,
  label.n = Inf,
  halo = FALSE,
  cluster = "walktrap",
  community.repulsion = 0,
  vos.path = NULL,
  size = 3,
  size.cex = FALSE,
  curved = FALSE,
  noloops = TRUE,
  remove.multiple = TRUE,
  remove.isolates = FALSE,
  weighted = NULL,
  edgesize = 1,
  edges.min = 0,
  alpha = 0.5,
  verbose = TRUE
)
}
\arguments{
\item{NetMatrix}{is a country collaboration matrix obtained by the function \code{\link{biblioNetwork}}.}

\item{normalize}{is a character. It can be "association", "jaccard", "inclusion","salton" or "equivalence" to obtain Association Strength, Jaccard,
Inclusion, Salton or Equivalence similarity index respectively. The default is type = NULL.}

\item{n}{is an integer. It indicates the number of vertices to plot.}

\item{degree}{is an integer. It indicates the min frequency of a vertex. If degree is not NULL, n is ignored.}

\item{type}{is a character object. It indicates the network map layout:

\tabular{lll}{
\code{type="auto"}\tab   \tab Automatic layout selection\cr
\code{type="circle"}\tab   \tab Circle layout\cr
\code{type="sphere"}\tab   \tab Sphere layout\cr
\code{type="mds"}\tab   \tab Multidimensional Scaling layout\cr
\code{type="fruchterman"}\tab   \tab Fruchterman-Reingold layout\cr
\code{type="kamada"}\tab   \tab  Kamada-Kawai layout}}

\item{label}{is logical. If TRUE vertex labels are plotted.}

\item{labelsize}{is an integer. It indicates the label size in the plot. Default is \code{labelsize=1}}

\item{label.cex}{is logical. If TRUE the label size of each vertex is proportional to its degree.}

\item{label.color}{is logical. If TRUE, for each vertex, the label color is the same as its cluster.}

\item{label.n}{is an integer. It indicates the number of vertex labels to draw.}

\item{halo}{is logical. If TRUE communities are plotted using different colors. Default is \code{halo=FALSE}}

\item{cluster}{is a character. It indicates the type of cluster to perform among ("none", "optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").}

\item{community.repulsion}{is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.1}.}

\item{vos.path}{is a character indicating the full path where VOSviewer.jar is located.}

\item{size}{is integer. It defines the size of each vertex. Default is \code{size=3}.}

\item{size.cex}{is logical. If TRUE the size of each vertex is proportional to its degree.}

\item{curved}{is a logical or a number. If TRUE edges are plotted with an optimal curvature. Default is \code{curved=FALSE}. Curved values are any numbers from 0 to 1.}

\item{noloops}{is logical. If TRUE loops in the network are deleted.}

\item{remove.multiple}{is logical. If TRUE multiple links are plotted using just one edge.}

\item{remove.isolates}{is logical. If TRUE isolates vertices are not plotted.}

\item{weighted}{This argument specifies whether to create a weighted graph from an adjacency matrix.
If it is NULL then an unweighted graph is created and the elements of the adjacency matrix gives the number of edges between the vertices.
If it is a character constant then for every non-zero matrix entry an edge is created and the value of the entry is added as an edge attribute
named by the weighted argument. If it is TRUE then a weighted graph is created and the name of the edge attribute will be weight.}

\item{edgesize}{is an integer. It indicates the network edge size.}

\item{edges.min}{is an integer. It indicates the min frequency of edges between two vertices. If edge.min=0, all edges are plotted.}

\item{alpha}{is a number. Legal alpha values are any numbers from 0 (transparent) to 1 (opaque). The default alpha value usually is 0.5.}

\item{verbose}{is a logical. If TRUE, network will be plotted. Default is \code{verbose = TRUE}.}
}
\value{
It is a list containing the following elements:
\tabular{lll}{
\code{graph} \tab  \tab a network object of the class \code{igraph}\cr
\code{cluster_obj} \tab  \tab a \code{communities} object of the package \code{igraph}\cr
\code{cluster_res} \tab  \tab a data frame with main results of clustering procedure.\cr}
}
\description{
A function to create and plot country collaboration networks by Region
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")

management <- metaTagExtraction(management, Field = "AU_CO")

NetMatrix <- biblioNetwork(management, analysis = "collaboration", network = "countries")

net <- collabByRegionPlot(NetMatrix,
  edgesize = 4, label.cex = TRUE, labelsize = 2.5,
  weighted = TRUE, size = 0.5, size.cex = TRUE, community.repulsion = 0,
  verbose = FALSE
)

cbind(names(net))

plot(net[[4]]$graph)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biblioNetwork.R
\name{biblioNetwork}
\alias{biblioNetwork}
\title{Creating Bibliographic networks}
\usage{
biblioNetwork(
  M,
  analysis = "coupling",
  network = "authors",
  n = NULL,
  sep = ";",
  short = FALSE,
  shortlabel = TRUE,
  remove.terms = NULL,
  synonyms = NULL
)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function
\code{\link{convert2df}}. It is a data matrix with cases corresponding to
manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{analysis}{is a character object. It indicates the type of analysis can be performed.
\code{analysis} argument can be \code{"collaboration"}, \code{"coupling"}, \code{"co-occurrences"}  or \code{"co-citation"}.
Default is \code{analysis = "coupling"}.}

\item{network}{is a character object. It indicates the network typology. The \code{network} argument can be
\code{"authors"}, \code{"references"}, \code{"sources"}, \code{"countries"},\code{"keywords"}, \code{"author_keywords"},\code{"all_keywords"}, \code{"titles"}, or \code{"abstracts"}.
Default is \code{network = "authors"}.}

\item{n}{is an integer. It indicates the number of items to select. If \code{N = NULL}, all items are selected.}

\item{sep}{is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.}

\item{short}{is a logical. If TRUE all items with frequency<2 are deleted to reduce the matrix size.}

\item{shortlabel}{is logical. IF TRUE, reference labels are stored in a short format. Default is \code{shortlabel=TRUE}.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}
}
\value{
It is a squared network matrix. It is an object of class \code{dgMatrix} of the package \code{Matrix}.
}
\description{
\code{biblioNetwork} creates different bibliographic networks from a bibliographic data frame.
}
\details{
The function \code{\link{biblioNetwork}} can create a collection of bibliographic networks
following the approach proposed by Batagelj & Cerinsek (2013) and Aria & cuccurullo (2017).\cr\cr
Typical networks output of \code{biblioNetwork} are:\cr\cr
#### Collaboration Networks ############\cr
-- Authors collaboration (analysis = "collaboration", network = "authors")\cr
-- University collaboration (analysis = "collaboration", network = universities")\cr
-- Country collaboration (analysis = "collaboration", network = "countries")\cr\cr
#### Co-citation Networks ##############\cr
-- Authors co-citation (analysis = "co-citation", network = "authors")\cr
-- Reference co-citation (analysis = "co-citation", network = "references")\cr
-- Source co-citation (analysis = "co-citation", network = "sources")\cr\cr
#### Coupling Networks ################\cr
-- Manuscript coupling (analysis = "coupling", network = "references")\cr
-- Authors coupling (analysis = "coupling", network = "authors")\cr
-- Source coupling (analysis = "coupling", network = "sources")\cr
-- Country coupling (analysis = "coupling", network = "countries")\cr\cr
#### Co-occurrences Networks ################\cr
-- Authors co-occurrences (analysis = "co-occurrences", network = "authors")\cr
-- Source co-occurrences (analysis = "co-occurrences", network = "sources")\cr
-- Keyword co-occurrences (analysis = "co-occurrences", network = "keywords")\cr
-- Author-Keyword co-occurrences (analysis = "co-occurrences", network = "author_keywords")\cr
-- Title content co-occurrences (analysis = "co-occurrences", network = "titles")\cr
-- Abstract content co-occurrences (analysis = "co-occurrences", network = "abstracts")\cr\cr

References:\cr
Batagelj, V., & Cerinsek, M. (2013). On bibliographic networks. Scientometrics, 96(3), 845-864.\cr
Aria, M., & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive science mapping analysis. Journal of Informetrics, 11(4), 959-975.\cr
}
\examples{
# EXAMPLE 1: Authors collaboration network

# data(scientometrics, package = "bibliometrixData")

# NetMatrix <- biblioNetwork(scientometrics, analysis = "collaboration",
# network = "authors", sep = ";")

# net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Collaboration",labelsize=0.5)


# EXAMPLE 2: Co-citation network

data(scientometrics, package = "bibliometrixData")

NetMatrix <- biblioNetwork(scientometrics,
  analysis = "co-citation",
  network = "references", sep = ";"
)

net <- networkPlot(NetMatrix, n = 30, type = "kamada", Title = "Co-Citation", labelsize = 0.5)

}
\seealso{
\code{\link{convert2df}} to import and convert a SCOPUS and Thomson
  Reuters' ISI Web of Knowledge export file in a data frame.

\code{\link{cocMatrix}} to compute a co-occurrence matrix.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thematicMap.R
\name{thematicMap}
\alias{thematicMap}
\title{Create a thematic map}
\usage{
thematicMap(
  M,
  field = "ID",
  n = 250,
  minfreq = 5,
  ngrams = 1,
  stemming = FALSE,
  size = 0.5,
  n.labels = 1,
  community.repulsion = 0.5,
  repel = TRUE,
  remove.terms = NULL,
  synonyms = NULL,
  cluster = "louvain",
  subgraphs = FALSE,
  alpha = 0.5,
  seed = 1234
)
}
\arguments{
\item{M}{is a bibliographic dataframe.}

\item{field}{is the textual attribute used to build up the thematic map. It can be \code{field = c("ID","DE","KW_Merged","TI", "AB")}.
\code{\link{biblioNetwork}} or \code{\link{cocMatrix}}.}

\item{n}{is an integer. It indicates the number of terms to include in the analysis.}

\item{minfreq}{is a integer. It indicates the minimum frequency (per thousand) of a cluster. It is a number in the range (0,1000).}

\item{ngrams}{is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.}

\item{stemming}{is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).}

\item{size}{is numerical. It indicates del size of the cluster circles and is a number in the range (0.01,1).}

\item{n.labels}{is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.}

\item{community.repulsion}{is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.5}.}

\item{repel}{is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}

\item{cluster}{is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").}

\item{subgraphs}{is a logical. If TRUE cluster subgraphs are returned.}

\item{alpha}{is a real. it indicates the balancing between word frequency and pagerank centrality to identify cluster labels. Default is \code{alpha = 0.5}.}

\item{seed}{is an integer. It indicates the seed for random number generation. Default is \code{seed = 1234}.}
}
\value{
a list containing:
\tabular{lll}{
\code{map}\tab   \tab The thematic map as ggplot2 object\cr
\code{clusters}\tab   \tab Centrality and Density values for each cluster. \cr
\code{words}\tab   \tab A list of words following in each cluster\cr
\code{nclust}\tab   \tab The number of clusters\cr
\code{net}\tab    \tab A list containing the network output (as provided from the networkPlot function)}
}
\description{
It creates a thematic map based on co-word network analysis and clustering.
The methodology is inspired by the proposal of Cobo et al. (2011).
}
\details{
\code{thematicMap} starts from a co-occurrence keyword network to plot in a
two-dimensional map the typological themes of a domain.\cr\cr

Reference:\cr
Cobo, M. J., Lopez-Herrera, A. G., Herrera-Viedma, E., & Herrera, F. (2011). An approach for detecting, quantifying,
and visualizing the evolution of a research field: A practical application to the fuzzy sets theory field. Journal of Informetrics, 5(1), 146-166.\cr
}
\examples{

\dontrun{
data(management, package = "bibliometrixData")
res <- thematicMap(management, field = "ID", n = 250, minfreq = 5, size = 0.5, repel = TRUE)
plot(res$map)
plot(res$net$graph)
}

}
\seealso{
\code{\link{biblioNetwork}} function to compute a bibliographic network.

\code{\link{cocMatrix}} to compute a bibliographic bipartite network.

\code{\link{networkPlot}} to plot a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fieldByYear.R
\name{fieldByYear}
\alias{fieldByYear}
\title{Field Tag distribution by Year}
\usage{
fieldByYear(
  M,
  field = "ID",
  timespan = NULL,
  min.freq = 2,
  n.items = 5,
  labelsize = NULL,
  remove.terms = NULL,
  synonyms = NULL,
  dynamic.plot = FALSE,
  graph = TRUE
)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by \code{\link{convert2df}} function.}

\item{field}{is a character object. It indicates one of the field tags of the
standard ISI WoS Field Tag codify.}

\item{timespan}{is a vector with the min and max year. If it is = NULL, the analysis is performed on the entire period. Default is \code{timespan = NULL}.}

\item{min.freq}{is an integer. It indicates the min frequency of the items to include in the analysis}

\item{n.items}{is an integer. I indicates the maximum number of items per year to include in the plot.}

\item{labelsize}{is deprecated argument. It will be removed in the next update.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}

\item{dynamic.plot}{is a logical. If TRUE plot aesthetics are optimized for plotly package.}

\item{graph}{is logical. If TRUE the function plots Filed Tag distribution by Year graph. Default is \code{graph = TRUE}.}
}
\value{
The function \code{fieldByYear} returns a list containing threeobjects:
\tabular{lll}{
\code{df}  \tab   \tab is a data frame\cr
\code{df_graph}\tab    \tab is a data frame with data used to build the graph\cr
\code{graph}   \tab   \tab a ggplot object}
}
\description{
It calculates the median year for each item of a field tag.
}
\examples{
data(management, package = "bibliometrixData")
timespan <- c(2005, 2015)
res <- fieldByYear(management,
  field = "ID", timespan = timespan,
  min.freq = 5, n.items = 5, graph = TRUE
)

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{summary}} method for class '\code{bibliometrix}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ltwa.R
\name{ltwa}
\alias{ltwa}
\title{Index of ltwa.}
\format{
A data frame with 56463 rows and 3 variables:
 \describe{
    \item{WORD}{word from journal names}
    \item{ABBREVIATION}{ISO4 abbreviation}
    \item{LANGUAGES}{Language of the journal name}
    }
}
\description{
Data frame containing a normalized index of words used in journal names and their ISO4 abbreviations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaTagExtraction.R
\name{metaTagExtraction}
\alias{metaTagExtraction}
\title{Meta-Field Tag Extraction}
\usage{
metaTagExtraction(M, Field = "CR_AU", sep = ";", aff.disamb = TRUE)
}
\arguments{
\item{M}{is a data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to articles and variables to Field Tag in the original WoS or SCOPUS file.}

\item{Field}{is a character object. New tag extracted from aggregated data is specified by this string.
Field can be equal to one of these tags:
\tabular{lll}{
\code{"CR_AU"}\tab   \tab First Author of each cited reference\cr
\code{"CR_SO"}\tab   \tab Source of each cited reference\cr
\code{"AU_CO"}\tab   \tab Country of affiliation for co-authors \cr
\code{"AU1_CO"}\tab   \tab Country of affiliation for the first author\cr
\code{"AU_UN"}\tab   \tab University of affiliation for each co-author and the corresponding author (AU1_UN)\cr
\code{"SR"}\tab     \tab Short tag of the document (as used in reference lists)}}

\item{sep}{is the field separator character. This character separates strings in each column of the data frame. The default is \code{sep = ";"}.}

\item{aff.disamb}{is a logical. If TRUE and Field="AU_UN", then a disambiguation algorithm is used to identify and match scientific affiliations
(univ, research centers, etc.). The default is \code{aff.disamb=TRUE}.}
}
\value{
the bibliometric data frame with a new column containing data about new field tag indicated in the argument \code{Field}.
}
\description{
It extracts other field tags, different from the standard WoS/SCOPUS codify.
}
\examples{
# Example 1: First Authors for each cited reference

data(scientometrics, package = "bibliometrixData")
scientometrics <- metaTagExtraction(scientometrics, Field = "CR_AU", sep = ";")
unlist(strsplit(scientometrics$CR_AU[1], ";"))


# Example 2: Source for each cited reference

data(scientometrics)
scientometrics <- metaTagExtraction(scientometrics, Field = "CR_SO", sep = ";")
unlist(strsplit(scientometrics$CR_SO[1], ";"))

# Example 3: Affiliation country for co-authors

data(scientometrics)
scientometrics <- metaTagExtraction(scientometrics, Field = "AU_CO", sep = ";")
scientometrics$AU_CO[1:10]

}
\seealso{
\code{\link{convert2df}} for importing and converting bibliographic files into a data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.bibliometrix.R
\name{plot.bibliometrix}
\alias{plot.bibliometrix}
\title{Plotting bibliometric analysis results}
\usage{
\method{plot}{bibliometrix}(x, ...)
}
\arguments{
\item{x}{is the object for which plots are desired.}

\item{...}{can accept two arguments:\cr
\code{k} is an integer, used for plot formatting (number of objects). Default value is 10.\cr
\code{pause} is a logical, used to allow pause in screen scrolling of results. Default value is \code{pause = FALSE}.}
}
\value{
The function \code{plot} returns a list of plots of class \code{ggplot2}.
}
\description{
\code{plot} method for class '\code{bibliometrix}'
}
\examples{
data(scientometrics, package = "bibliometrixData")

results <- biblioAnalysis(scientometrics)

plot(results, k = 10, pause = FALSE)

}
\seealso{
The bibliometric analysis function \code{\link{biblioAnalysis}}.

\code{\link{summary}} to compute a list of summary statistics of the object of class \code{bibliometrix}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/couplingMap.R
\name{couplingMap}
\alias{couplingMap}
\title{Coupling Analysis}
\usage{
couplingMap(
  M,
  analysis = "documents",
  field = "CR",
  n = 500,
  label.term = NULL,
  ngrams = 1,
  impact.measure = "local",
  minfreq = 5,
  community.repulsion = 0.1,
  stemming = FALSE,
  size = 0.5,
  n.labels = 1,
  repel = TRUE,
  cluster = "walktrap"
)
}
\arguments{
\item{M}{is a bibliographic dataframe.}

\item{analysis}{is the textual attribute used to select the unit of analysis. It can be \code{analysis = c("documents", "authors", "sources")}.}

\item{field}{is the textual attribute used to measure the coupling strength. It can be \code{field = c("CR", "ID","DE", "TI", "AB")}.}

\item{n}{is an integer. It indicates the number of units to include in the analysis.}

\item{label.term}{is a character. It indicates which content metadata have to use for cluster labeling. It can be \code{label.term = c("ID","DE","TI","AB")}.
If \code{label.term = NULL} cluster items will be use for labeling.}

\item{ngrams}{is an integer between 1 and 4. It indicates the type of n-gram to extract from texts.
An n-gram is a contiguous sequence of n terms. The function can extract n-grams composed by 1, 2, 3 or 4 terms. Default value is \code{ngrams=1}.}

\item{impact.measure}{is a character. It indicates the impact measure used to rank cluster elements (documents, authors or sources).
It can be \code{impact.measure = c("local", "global")}.\\
With \code{impact.measure = "local"}, \link{couplingMap} calculates elements impact using the Normalized Local Citation Score while
using \code{impact.measure = "global"}, the function uses the Normalized Global Citation Score to measure elements impact.}

\item{minfreq}{is a integer. It indicates the minimum frequency (per thousand) of a cluster. It is a number in the range (0,1000).}

\item{community.repulsion}{is a real. It indicates the repulsion force among network communities. It is a real number between 0 and 1. Default is \code{community.repulsion = 0.1}.}

\item{stemming}{is logical. If it is TRUE the word (from titles or abstracts) will be stemmed (using the Porter's algorithm).}

\item{size}{is numerical. It indicates the size of the cluster circles and is a number in the range (0.01,1).}

\item{n.labels}{is integer. It indicates how many labels associate to each cluster. Default is \code{n.labels = 1}.}

\item{repel}{is logical. If it is TRUE ggplot uses geom_label_repel instead of geom_label.}

\item{cluster}{is a character. It indicates the type of cluster to perform among ("optimal", "louvain","leiden", "infomap","edge_betweenness","walktrap", "spinglass", "leading_eigen", "fast_greedy").}
}
\value{
a list containing:
\tabular{lll}{
\code{map}\tab   \tab The coupling map as ggplot2 object\cr
\code{clusters}\tab   \tab Centrality and Density values for each cluster. \cr
\code{data}\tab   \tab A list of units following in each cluster\cr
\code{nclust}\tab   \tab The number of clusters\cr
\code{NCS}\tab     \tab The Normalized Citation Score dataframe\cr
\code{net}\tab    \tab A list containing the network output (as provided from the networkPlot function)}
}
\description{
It performs a coupling network analysis and plots community detection results on a bi-dimensional map (Coupling Map).
}
\details{
The analysis can be performed on three different units: documents, authors or sources and
the coupling strength can be measured using the classical approach (coupled by references)
or a novel approach based on unit contents (keywords or terms from titles and abstracts)

The x-axis measures the cluster centrality (by Callon's Centrality index) while the y-axis measures the cluster impact
by Mean Normalized Local Citation Score (MNLCS).
The Normalized Local Citation Score (NLCS) of a document is calculated
by dividing the actual count of local citing items by the expected citation rate for documents with the same year of publication.
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")
res <- couplingMap(management,
  analysis = "authors", field = "CR", n = 250, impact.measure = "local",
  minfreq = 3, size = 0.5, repel = TRUE
)
plot(res$map)
}

}
\seealso{
\code{\link{biblioNetwork}} function to compute a bibliographic network.

\code{\link{cocMatrix}} to compute a bibliographic bipartite network.

\code{\link{networkPlot}} to plot a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.bibliometrix.R
\name{summary.bibliometrix}
\alias{summary.bibliometrix}
\title{Summarizing bibliometric analysis results}
\usage{
\method{summary}{bibliometrix}(object, ...)
}
\arguments{
\item{object}{is the object for which a summary is desired.}

\item{...}{can accept two arguments:\cr
\code{k} integer, used for table formatting (number of rows). Default value is 10.\cr
\code{pause} logical, used to allow pause in screen scrolling of results. Default value is \code{pause = FALSE}.\cr
\code{width} integer, used to define screen output width. Default value is \code{width = 120}.
\code{verbose} logical, used to allow screen output. Default is TRUE.}
}
\value{
The function \code{summary} computes and returns a list of summary statistics of the object of class \code{bibliometrics}.

the list contains the following objects:
\tabular{lll}{
\code{MainInformation}   \tab   \tab Main Information about Data\cr
\code{AnnualProduction}  \tab   \tab Annual Scientific Production\cr
\code{AnnualGrowthRate}  \tab   \tab Annual Percentage Growth Rate\cr
\code{MostProdAuthors}   \tab   \tab Most Productive Authors\cr
\code{MostCitedPapers}   \tab   \tab Top manuscripts per number of citations\cr
\code{MostProdCountries} \tab   \tab Corresponding Author's Countries\cr
\code{TCperCountries}    \tab   \tab Total Citation per Countries\cr
\code{MostRelSources}    \tab   \tab Most Relevant Sources\cr
\code{MostRelKeywords}   \tab   \tab Most Relevant Keywords}
}
\description{
\code{summary} method for class '\code{bibliometrix}'
}
\examples{
data(scientometrics, package = "bibliometrixData")

results <- biblioAnalysis(scientometrics)

summary(results)

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{normalize_citations}
\alias{normalize_citations}
\title{Normalize and match bibliographic citations}
\usage{
normalize_citations(
  CR_vector,
  threshold = 0.9,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
)
}
\arguments{
\item{CR_vector}{Character vector containing bibliographic citations to be normalized and matched.}

\item{threshold}{Numeric value between 0 and 1 indicating the similarity threshold
for matching citations. Higher values (e.g., 0.90-0.95) produce more conservative
matching, while lower values (e.g., 0.75-0.80) produce more aggressive matching.
Default is 0.90, which provides a good balance between precision and recall.}

\item{method}{String distance method to use for fuzzy matching. Options include:
\itemize{
  \item "jw" (default): Jaro-Winkler distance, optimized for bibliographic strings
  \item "lv": Levenshtein distance
  \item "osa": Optimal String Alignment distance
  \item "lcs": Longest Common Subsequence distance
  \item Other methods supported by \code{\link[stringdist]{stringdistmatrix}}
}}

\item{min_chars}{Minimum characters for valid citations (default: 20).}

\item{max_block_size}{Integer. Blocks with at least this many unique normalized
strings skip within-block fuzzy matching and fall back to exact matching only,
to bound the cost of the pairwise distance matrix (default: 100).}

\item{use_iso4}{Logical. If \code{TRUE} (default), normalize journal names to their
ISO 4 abbreviated form via the LTWA database (Phase 1.5). Set to \code{FALSE} to
disable ISO 4 / LTWA journal normalization (used for ablation analyses).}

\item{use_doi}{Logical. If \code{TRUE} (default), perform exact matching on DOIs
(part of Phase 2). Set to \code{FALSE} to disable DOI-based matching.}

\item{use_exact}{Logical. If \code{TRUE} (default), perform exact normalized-string
and punctuation-invariant matching (Phase 2). Set to \code{FALSE} to disable them.}

\item{fuzzy}{Logical. If \code{TRUE} (default), perform within-block matching
(Phase 4: WoS deterministic key matching and Scopus fuzzy clustering). Set to
\code{FALSE} to disable within-block matching, keeping only the exact phases.}

\item{use_postproc}{Logical. If \code{TRUE} (default), perform Phase 4.5
metadata-based post-processing merge. Set to \code{FALSE} to disable it.}

\item{title_guard}{Logical. If \code{TRUE}, run an optional Phase 4.6 that
purifies clusters by detecting series part markers in titles: distinct
works that share author, year, journal and volume but differ only in a
series designator (e.g. "... Part I" / "... Part II", "... I." / "... II.")
are split into separate clusters. This step only splits clusters, never
merges; it relies solely on the part marker (not on full-title similarity)
so it does not interfere with robustness to title typos. Default
\code{FALSE} (legacy behaviour).}
}
\value{
A data frame with the following columns:
\itemize{
  \item \code{CR_original}: Original citation string
  \item \code{CR_canonical}: Canonical (representative) citation for the cluster
  \item \code{cluster_id}: Unique identifier for each citation cluster
  \item \code{n_cluster}: Number of citations in the cluster
  \item \code{first_author}: First author surname
  \item \code{year}: Publication year
  \item \code{journal_iso4}: Journal name normalized to ISO4 abbreviated form
  \item \code{journal_original}: Original journal name as extracted from citation
  \item \code{volume}: Volume number
  \item \code{doi}: Digital Object Identifier (when available)
  \item \code{blocking_key}: Internal key used for blocking (author_year_journal)
}
}
\description{
This function performs advanced normalization and fuzzy matching of bibliographic
citations to identify and group citations that refer to the same work but are
formatted differently. It uses a multi-phase approach combining string normalization,
blocking strategies, hierarchical clustering, and post-processing to achieve both
speed and accuracy on large citation datasets.
}
\details{
The function implements a five-phase matching algorithm:

\strong{Phase 1: Normalization and Feature Extraction}
\itemize{
  \item Converts text to uppercase
  \item Removes issue numbers and page numbers (which often contain typos)
  \item Removes punctuation and normalizes whitespace
  \item Expands common journal abbreviations (e.g., "J. CLEAN. PROD." -> "JOURNAL OF CLEANER PRODUCTION")
  \item Extracts key features: first author, year, journal, volume, pages
}

\strong{Phase 1.5: Journal Normalization}
The function uses the LTWA (List of Title Word Abbreviations) database from
ISO 4 standards to normalize journal names. This ensures that abbreviated
forms (e.g., "J. Clean. Prod.") and full forms (e.g., "Journal of Cleaner
Production") are recognized as the same journal and matched together.

The LTWA database is included in the bibliometrix package. If not found,
the function attempts to download it from ISSN.org. Journal normalization
can be disabled by ensuring the LTWA database is not available.

\strong{Phase 2: Blocking}
Citations are grouped into blocks by first author and year. This dramatically
reduces computational complexity from O(n^2) to approximately O(k*m^2), where k is
the number of blocks and m is the average block size.

\strong{Phase 3: Within-Block Matching}
Within each block, citations are compared using string distance metrics and
hierarchical clustering. For blocks with at least \code{max_block_size} unique
normalized strings (default 100), exact matching on normalized strings is used
instead to maintain performance.

\strong{Phase 4: Canonical Representative Selection}
For each cluster, the most complete citation (prioritizing those with volume
and page information) is selected as the canonical representative.

\strong{Phase 5: Post-Processing}
Citations sharing the same first author, year, journal, and volume are merged
into a single cluster, even if they weren't matched in Phase 3. This catches
cases where minor title variations prevented matching.
}
\examples{
\dontrun{
# Load bibliometrix data
data(scientometrics, package = "bibliometrixData")

# Extract and normalize citations
CR_vector <- unlist(strsplit(scientometrics$CR, ";"))
CR_vector <- trimws(CR_vector)

# Perform normalization with default threshold
matched <- normalize_citations(CR_vector)

# View matching statistics
table(matched$n_cluster)

# Find all variants of a specific citation
subset(matched, cluster_id == matched$cluster_id[1])

# Use more conservative matching
matched_conservative <- normalize_citations(CR_vector, threshold = 0.90)
}

}
\references{
Aria, M. & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive
science mapping analysis. Journal of Informetrics, 11(4), 959-975.
}
\seealso{
\code{\link{applyReferenceMatching}} for direct application to bibliometrix data frames
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeDbSources.R
\name{mergeDbSources}
\alias{mergeDbSources}
\title{Merge bibliographic data frames from supported bibliogtraphic DBs}
\usage{
mergeDbSources(..., remove.duplicated = TRUE, verbose = TRUE)
}
\arguments{
\item{...}{are the bibliographic data frames to merge.}

\item{remove.duplicated}{is logical. If TRUE duplicated documents will be deleted from the bibliographic collection.}

\item{verbose}{is logical.  If TRUE, information on duplicate documents is printed on the screen.}
}
\value{
the value returned from \code{mergeDbSources} is a bibliographic data frame.
}
\description{
Merge bibliographic data frames from different databases (WoS,SCOPUS, Lens, Openalex, etc-) into a single one.
}
\details{
bibliographic data frames are obtained by the converting function \code{\link{convert2df}}.
The function merges data frames identifying common tag fields and duplicated records.
}
\examples{

data(isiCollection, package = "bibliometrixData")

data(scopusCollection, package = "bibliometrixData")

M <- mergeDbSources(isiCollection, scopusCollection, remove.duplicated = TRUE)

dim(M)

}
\seealso{
\code{\link{convert2df}} to import and convert an ISI or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/missingData.R
\name{missingData}
\alias{missingData}
\title{Completeness of bibliographic metadata}
\usage{
missingData(M)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by \code{\link{convert2df}} function.}
}
\value{
The function \code{missingData} returns a list containing two objects:
\tabular{lll}{
\code{allTags}  \tab   \tab is a data frame including results for all original metadata tags from the collection\cr
\code{mandatoryTags}\tab    \tab is a data frame that included only the tags needed for analysis with bibliometrix and biblioshiny.}
}
\description{
It calculates the percentage of missing data in the metadata of a bibliographic data frame.
}
\details{
Each metadata is assigned a status c("Excellent," "Good," "Acceptable", "Poor", "Critical," "Completely missing")
depending on the percentage of missing data. In particular, the column *status* classifies the percentage of missing
value in 5 categories: "Excellent" (0%), "Good" (0.01% to 10.00%), "Acceptable" (from 10.01% to 20.00%),
"Poor" (from 20.01% to 50.00%), "Critical" (from 50.01% to 99.99%), "Completely missing" (100%).

The results of the function allow us to understand which analyses can be performed with bibliometrix
and which cannot based on the completeness (or status) of different metadata.
}
\examples{
data(scientometrics, package = "bibliometrixData")
res <- missingData(scientometrics)
print(res$mandatoryTags)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duplicatedMatching.R
\name{duplicatedMatching}
\alias{duplicatedMatching}
\title{Searching of duplicated records in a bibliographic database}
\usage{
duplicatedMatching(M, Field = "TI", exact = FALSE, tol = 0.95)
}
\arguments{
\item{M}{is the bibliographic data frame.}

\item{Field}{is a character object. It indicates one of the field tags used to identify duplicated records. Field can be equal to one of these tags: TI (title), AB (abstract), UT (manuscript ID).}

\item{exact}{is logical. If exact = TRUE the function searches duplicates using exact matching. If exact=FALSE,
the function uses the restricted Damerau-Levenshtein distance to find duplicated documents.}

\item{tol}{is a numeric value giving the minimum relative similarity to match two manuscripts. Default value is \code{tol = 0.95}.
To use the restricted Damerau-Levenshtein distance, exact argument has to be set as FALSE.}
}
\value{
the value returned from \code{duplicatedMatching} is a data frame without duplicated records.
}
\description{
Search duplicated records in a dataframe.
}
\details{
A bibliographic data frame is obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.
The function identifies duplicated records in a bibliographic data frame and deletes them.
Duplicate entries are identified through the restricted Damerau-Levenshtein distance.
Two manuscripts that have a relative similarity measure greater than \code{tol} argument are stored in the output data frame only once.
}
\examples{

data(scientometrics, package = "bibliometrixData")

M <- rbind(scientometrics[1:20, ], scientometrics[10:30, ])

newM <- duplicatedMatching(M, Field = "TI", exact = FALSE, tol = 0.95)

dim(newM)

}
\seealso{
\code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{get_iso4_stop_words}
\alias{get_iso4_stop_words}
\title{Articles, prepositions, and conjunctions to be removed (ISO 4 standard)}
\usage{
get_iso4_stop_words()
}
\description{
Articles, prepositions, and conjunctions to be removed (ISO 4 standard)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net2Pajek.R
\name{net2Pajek}
\alias{net2Pajek}
\title{Save a network graph object as Pajek files}
\usage{
net2Pajek(net, filename = "my_pajek_network", path = NULL)
}
\arguments{
\item{net}{is a network graph object returned by the function \code{\link{networkPlot}}.}

\item{filename}{is a character. It indicates the filename for Pajek export files.}

\item{path}{is a character. It indicates the path where the files will be saved. When path="NULL, the files will be saved in the current folder. Default is NULL.}
}
\value{
The function returns no object but will save three Pajek files in the folder given in the "path" argument with the name "filename.clu," "filename.vec," and "filename.net."
}
\description{
The function \code{\link{net2Pajek}} save a bibliographic network previously created by \code{\link{networkPlot}} as pajek files.
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")

NetMatrix <- biblioNetwork(management,
  analysis = "co-occurrences",
  network = "keywords", sep = ";"
)

net <- networkPlot(NetMatrix, n = 30, type = "auto", Title = "Co-occurrence Network", labelsize = 1)

net2Pajek(net, filename = "pajekfiles", path = NULL)
}
}
\seealso{
\code{\link{net2VOSviewer}} to export and plot the network with VOSviewer software.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timeslice.R
\name{timeslice}
\alias{timeslice}
\title{Bibliographic data frame time slice}
\usage{
timeslice(M, breaks = NA, k = 5)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{breaks}{is a numeric vector of two or more unique cut points.}

\item{k}{is an integer value giving the number of intervals into which the data frame is to be cut. \code{k} is used only in case \code{breaks} argument is not provided. The default is \code{k = 5}.}
}
\value{
the value returned from \code{split} is a list containing the data frames for each sub-period.
}
\description{
Divide a bibliographic data frame into time slice
}
\examples{

data(scientometrics, package = "bibliometrixData")

list_df <- timeslice(scientometrics, breaks = c(1995, 2005))

names(list_df)

}
\seealso{
\code{\link{convert2df}} to import and convert an ISI or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cocMatrix.R
\name{cocMatrix}
\alias{cocMatrix}
\title{Bibliographic bipartite network matrices}
\usage{
cocMatrix(
  M,
  Field = "AU",
  type = "sparse",
  n = NULL,
  sep = ";",
  binary = TRUE,
  short = FALSE,
  remove.terms = NULL,
  synonyms = NULL
)
}
\arguments{
\item{M}{is a data frame obtained by the converting function
\code{\link{convert2df}}. It is a data matrix with cases corresponding to
articles and variables to Field Tag in the original WoS or SCOPUS file.}

\item{Field}{is a character object. It indicates one of the field tags of the
  standard ISI WoS Field Tag codify. Field can be equal to one of these tags:
  \tabular{lll}{ \code{AU}\tab   \tab Authors\cr \code{SO}\tab   \tab
  Publication Name (or Source)\cr \code{JI}\tab   \tab ISO Source
  Abbreviation\cr \code{DE}\tab   \tab Author Keywords\cr \code{ID}\tab
  \tab Keywords associated by WoS or SCOPUS database \cr 
  \code{KW_Merged}\tab    \tab All Keywords (merged by DE and ID) \cr
  \code{CR}\tab   \tab Cited References}

  for a complete list of filed tags see:
  \href{https://www.bibliometrix.org/documents/Field_Tags_bibliometrix.pdf}{Field Tags used in bibliometrix}\cr\cr}

\item{type}{indicates the output format of co-occurrences: \tabular{lll}{
\code{type = "matrix"} \tab   \tab produces an object of class
\code{matrix}\cr \code{type = "sparse"} \tab   \tab produces an object of
class \code{dgMatrix} of the package \code{Matrix}. "sparse"
argument generates a compact representation of the matrix.}}

\item{n}{is an integer. It indicates the number of items to select. If \code{N = NULL}, all items are selected.}

\item{sep}{is the field separator character. This character separates strings in each
column of the data frame. The default is \code{sep = ";"}.}

\item{binary}{is a logical. If TRUE each cell contains a 0/1. if FALSE each cell contains the frequency.}

\item{short}{is a logical. If TRUE all items with frequency<2 are deleted to reduce the matrix size.}

\item{remove.terms}{is a character vector. It contains a list of additional terms to delete from the documents before term extraction. The default is \code{remove.terms = NULL}.}

\item{synonyms}{is a character vector. Each element contains a list of synonyms, separated by ";",  that will be merged into a single term (the first word contained in the vector element). The default is \code{synonyms = NULL}.}
}
\value{
a bipartite network matrix with cases corresponding to manuscripts and variables to the
  objects extracted from the Tag \code{Field}.
}
\description{
\code{cocMatrix} computes occurrences between elements of a Tag Field from a bibliographic data frame. Manuscript is the unit of analysis.
}
\details{
This occurrence matrix represents a bipartite network which can be transformed into a collection of bibliographic
networks such as coupling, co-citation, etc..

The function follows the approach proposed by Batagelj & Cerinsek (2013) and Aria & cuccurullo (2017).\cr\cr

References:\cr
Batagelj, V., & Cerinsek, M. (2013). On bibliographic networks. Scientometrics, 96(3), 845-864.\cr
Aria, M., & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive science mapping analysis. Journal of Informetrics, 11(4), 959-975.\cr
}
\examples{
# EXAMPLE 1: Articles x Authors occurrence matrix

data(scientometrics, package = "bibliometrixData")
WA <- cocMatrix(scientometrics, Field = "AU", type = "sparse", sep = ";")

# EXAMPLE 2: Articles x Cited References occurrence matrix

# data(scientometrics, package = "bibliometrixData")

# WCR <- cocMatrix(scientometrics, Field = "CR", type = "sparse", sep = ";")

# EXAMPLE 3: Articles x Cited First Authors occurrence matrix

# data(scientometrics, package = "bibliometrixData")
# scientometrics <- metaTagExtraction(scientometrics, Field = "CR_AU", sep = ";")
# WCR <- cocMatrix(scientometrics, Field = "CR_AU", type = "sparse", sep = ";")

}
\seealso{
\code{\link{convert2df}} to import and convert an ISI or SCOPUS
  Export file in a data frame.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.

\code{\link{biblioNetwork}} to compute a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histPlot.R
\name{histPlot}
\alias{histPlot}
\title{Plotting historical co-citation network}
\usage{
histPlot(
  histResults,
  n = 20,
  size = 5,
  labelsize = 5,
  remove.isolates = TRUE,
  title_as_label = FALSE,
  label = "short",
  verbose = TRUE
)
}
\arguments{
\item{histResults}{is an object of \code{class} "list" containing the following components:

\tabular{lll}{
NetMatrix \tab  \tab the historical citation network matrix\cr
Degree \tab       \tab the min degree of the network\cr
histData \tab      \tab the set of n most cited references\cr
M \tab      \tab the bibliographic data frame}

is a network matrix obtained by the function \code{\link{histNetwork}}.}

\item{n}{is integer. It defines the number of vertices to plot.}

\item{size}{is an integer. It defines the point size of the vertices. Default value is 5.}

\item{labelsize}{is an integer. It indicates the label size in the plot. Default is \code{labelsize=5}.}

\item{remove.isolates}{is logical. If TRUE isolates vertices are not plotted.}

\item{title_as_label}{is a logical. DEPRECATED}

\item{label}{is a character. It indicates which label type to use as node id in the historiograph. It can be \code{label=c("short", "title", "keywords", "keywordsplus")}.
Default is \code{label = "short"}.}

\item{verbose}{is logical. If TRUE, results and plots are printed on screen.}
}
\value{
It is list containing: a network object of the class \code{igraph} and a plot object of the class \code{ggraph}.
}
\description{
\code{histPlot} plots a historical co-citation network.
}
\details{
The function \code{\link{histPlot}} can plot a historical co-citation network previously created by \code{\link{histNetwork}}.
}
\examples{
# EXAMPLE Citation network
\dontrun{
data(management, package = "bibliometrixData")

histResults <- histNetwork(management, sep = ";")

net <- histPlot(histResults, n = 20, labelsize = 5)
}

}
\seealso{
\code{\link{histNetwork}} to compute a historical co-citation network.

\code{\link{cocMatrix}} to compute a co-occurrence matrix.

\code{\link{biblioAnalysis}} to perform a bibliometric analysis.
}

\name{bibliometrix-package}
\alias{bibliometrix-package}
\alias{bibliometrix}
\docType{package}
\title{
\packageTitle{bibliometrix}
}
\description{
\packageDescription{bibliometrix}
}
\details{

INSTALLATION

- Stable version from CRAN:

install.packages("bibliometrix")

- Or development version from GitHub:

install.packages("devtools")
devtools::install_github("massimoaria/bibliometrix")

- Load "bibliometrix"

library('bibliometrix')



DATA LOADING AND CONVERTING

The export file can be imported and converted by R using the function *convert2df*:

file <- ("https://www.bibliometrix.org/datasets/savedrecs.txt")

M <- convert2df(file, dbsource = "wos", format = "bibtex")

*convert2df* creates a bibliographic data frame with cases corresponding to manuscripts and variables to Field Tag in the original export file.
Each manuscript contains several elements, such as authors' names, title, keywords and other information. All these elements constitute the bibliographic attributes of a document, also called metadata.
Data frame columns are named using the standard Clarivate Analytics WoS Field Tag codify. 



BIBLIOMETRIC ANALYSIS

The first step is to perform a descriptive analysis of the bibliographic data frame.
The function *biblioAnalysis* calculates main bibliometric measures using this syntax:

results <- biblioAnalysis(M, sep = ";")

The function *biblioAnalysis* returns an object of class "bibliometrix".

To summarize main results of the bibliometric analysis, use the generic function *summary*.
It displays main information about the bibliographic data frame and several tables, such as annual scientific production, top manuscripts per number of citations, most productive authors, most productive countries, total citation per country, most relevant sources (journals) and most relevant keywords.
*summary* accepts two additional arguments. *k* is a formatting value that indicates the number of rows of each table. *pause* is a logical value (TRUE or FALSE) used to allow (or not) pause in screen scrolling.
Choosing k=10 you decide to see the first 10 Authors, the first 10 sources, etc.

S <- summary(object = results, k = 10, pause = FALSE)

Some basic plots can be drawn using the generic function plot:

plot(x = results, k = 10, pause = FALSE)



BIBLIOGRAPHIC NETWORK MATRICES

Manuscript's attributes are connected to each other through the manuscript itself: author(s) to journal, keywords to publication date, etc.
These connections of different attributes generate bipartite networks that can be represented as rectangular matrices (Manuscripts x Attributes).
Furthermore, scientific publications regularly contain references to
other scientific works. This generates a further network, namely, co-citation or coupling network.
These networks are analyzed in order to capture meaningful properties of the underlying research system, and in particular to determine the influence of bibliometric units such as scholars and journals.


*biblioNetwork* function

The function *biblioNetwork* calculates, starting from a bibliographic data frame, the most frequently used networks: Coupling, Co-citation, Co-occurrences, and Collaboration.
*biblioNetwork* uses two arguments to define the network to compute:
- *analysis* argument can be "co-citation", "coupling", "collaboration",  or "co-occurrences".
- *network* argument can be "authors", "references", "sources", "countries", "universities", "keywords", "author_keywords", "titles" and "abstracts".

i.e. the following code calculates a classical co-citation network:

NetMatrix <- biblioNetwork(M, analysis = "co-citation", network = "references", sep = ";")



VISUALIZING BIBLIOGRAPHIC NETWORKS 

All bibliographic networks can be graphically visualized or modeled.
Using the function *networkPlot*, you can plot a network created by *biblioNetwork* using R routines.

The main argument of *networkPlot* is type. It indicates the network map layout: circle, kamada-kawai, mds, etc.

In the following, we propose some examples.



### Country Scientific Collaboration

# Create a country collaboration network

M <- metaTagExtraction(M, Field = "AU_CO", sep = ";")

NetMatrix <- biblioNetwork(M, analysis = "collaboration", network = "countries", sep = ";")

# Plot the network

net=networkPlot(NetMatrix, n = dim(NetMatrix)[1], Title = "Country Collaboration", type = "circle", size=TRUE, remove.multiple=FALSE,labelsize=0.8)



### Co-Citation Network

# Create a co-citation network

NetMatrix <- biblioNetwork(M, analysis = "co-citation", network = "references", sep = ";")

# Plot the network

net=networkPlot(NetMatrix, n = 30, Title = "Co-Citation Network", type = "fruchterman", size=T, remove.multiple=FALSE, labelsize=0.7,edgesize = 5)


### Keyword co-occurrences

# Create keyword co-occurrences network

NetMatrix <- biblioNetwork(M, analysis = "co-occurrences", network = "keywords", sep = ";")

# Plot the network

net=networkPlot(NetMatrix, normalize="association", weighted=T, n = 30, Title = "Keyword Co-occurrences", type = "fruchterman", size=T,edgesize = 5,labelsize=0.7)



CO-WORD ANALYSIS: THE CONCEPTUAL STRUCTURE OF A FIELD 

The aim of the co-word analysis is to map the conceptual structure of a framework using the word co-occurrences in a bibliographic collection.
The analysis can be performed through dimensionality reduction techniques such as Multidimensional Scaling (MDS), Correspondence Analysis (CA) or Multiple Correspondence Analysis (MCA). 
Here, we show an example using the function *conceptualStructure* that performs a CA or MCA to draw a conceptual structure of the field and K-means clustering to identify clusters of documents which express common concepts. Results are plotted on a two-dimensional map.
*conceptualStructure* includes natural language processing (NLP) routines (see the function *termExtraction*) to extract terms from titles and abstracts.  In addition, it implements the Porter's stemming algorithm to reduce inflected (or sometimes derived) words to their word stem, base or root form.

# Conceptual Structure using keywords (method="MCA")

CS <- conceptualStructure(M,field="ID", method="MCA", minDegree=4, clust=4 ,k.max=8, stemming=FALSE, labelsize=10, documents=10)



HISTORICAL DIRECT CITATION NETWORK

The historiographic map is a graph proposed by E. Garfield to represent a chronological network map of most relevant direct citations resulting from a bibliographic collection.
The function histNetwork generates a chronological direct citation network matrix which can be plotted using *histPlot*:

# Create a historical citation network

histResults <- histNetwork(M, sep = ";")

# Plot a historical co-citation network

net <- histPlot(histResults, size = 10)

}

\author{
\packageAuthor{bibliometrix}

Maintainer: \packageMaintainer{bibliometrix}
}
\references{
Aria, M. & Cuccurullo, C. (2017).  *bibliometrix*: An R-tool for comprehensive science mapping analysis, *Journal of Informetrics*, 11(4), pp 959-975, Elsevier, DOI: 10.1016/j.joi.2017.08.007 (https://doi.org/10.1016/j.joi.2017.08.007).

Cuccurullo, C., Aria, M., & Sarto, F. (2016). Foundations and trends in performance management. A twenty-five years bibliometric analysis in business and public administration domains, *Scientometrics*, DOI: 10.1007/s11192-016-1948-8 (https://doi.org/10.1007/s11192-016-1948-8).

Cuccurullo, C., Aria, M., & Sarto, F.  (2015). Twenty years of research on performance management in business and public administration domains. Presentation at the *Correspondence Analysis and Related Methods conference (CARME 2015)* in September 2015 (https://www.bibliometrix.org/documents/2015Carme_cuccurulloetal.pdf).

Sarto, F., Cuccurullo, C., & Aria, M. (2014). Exploring healthcare governance literature: systematic review and paths for future research. *Mecosan* (https://www.francoangeli.it/Riviste/Scheda_Rivista.aspx?IDarticolo=52780&lingua=en).

Cuccurullo, C., Aria, M., & Sarto, F. (2013). Twenty years of research on performance management in business and public administration domains. In *Academy of Management Proceedings* (Vol. 2013, No. 1, p. 14270). Academy of Management (https://doi.org/10.5465/AMBPP.2013.14270abstract).

}

\concept{Science Mapping} 
\concept{Clarivate Analytics Web of Science}
\concept{Scopus}
\concept{citations}
\concept{co-citations}
\concept{network}
\concept{co-authors}
\concept{co-occurrences}
\concept{collaboration}
\concept{co-word analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assignEvolutionColors.R
\name{assignEvolutionColors}
\alias{assignEvolutionColors}
\title{Assign Colors to Thematic Evolution Nodes Based on Lineages}
\usage{
assignEvolutionColors(nexus, threshold = 0.5, palette = NULL, alpha = 0.5)
}
\arguments{
\item{nexus}{A list object returned by \code{\link{thematicEvolution}} containing:
\itemize{
  \item \code{Nodes}: data frame with node information (name, group, id, sum, freq, etc.)
  \item \code{Edges}: data frame with edge information (from, to, weight measures)
  \item \code{TM}: list of thematic maps for each period
}}

\item{threshold}{Numeric. The minimum weight value for an edge to be considered a
"strong connection" (default: 0.5). Edges with weights >= threshold will propagate
the same color to connected nodes across periods.}

\item{palette}{Character vector. Optional custom color palette as hex codes. If NULL,
uses a default palette of 50+ distinct colors. Colors are assigned sequentially
without reuse.}

\item{alpha}{Numeric. Balancing parameters between inclusion and importance measures. Default value is 0.5.}
}
\value{
Returns the modified \code{nexus} object with updated colors:
  \itemize{
    \item \code{Nodes$color}: updated with lineage-based colors
    \item \code{Edges$color}: edges connecting same-colored nodes receive the node color,
      others are grey
    \item \code{TM}: thematic maps updated with new cluster colors
  }
}
\description{
This function assigns colors to nodes in a thematic evolution analysis based on
their lineages across time periods. Nodes connected by strong edges (above a threshold)
receive the same color to visualize thematic continuity. Nodes with the same name
across periods that are not strongly connected to other nodes are also colored identically.
}
\details{
The function uses a multi-phase algorithm:
\enumerate{
  \item \strong{Phase 1}: Identifies lineages by following strong connections (weight >= threshold)
    from the first period forward. When a node has multiple strong connections, the
    strongest one determines the lineage.
  \item \strong{Phase 1.5}: Assigns the same lineage to nodes with identical names across
    periods if they are not already part of different strong connections.
  \item \strong{Phase 2}: Assigns unique colors from the palette to each identified lineage.
  \item \strong{Phase 3}: Assigns unique colors to isolated nodes (those without any lineage).
  \item \strong{Phase 4}: Colors edges based on their connected nodes - same color if both
    nodes share a color, grey otherwise.
  \item \strong{Final}: Updates thematic maps with the new color scheme.
}

Each lineage receives a unique color from the palette. No color is reused across
different lineages, ensuring clear visual distinction between independent thematic streams.
}
\examples{
\dontrun{
data(scientometrics, package = "bibliometrixData")
years <- c(2000, 2010)

nexus <- thematicEvolution(scientometrics, field = "ID",
                            years = years, n = 100, minFreq = 2)


# Use custom threshold and measure
nexus <- assignEvolutionColors(nexus, threshold = 0.6, use_measure = "weighted")

}

}
\seealso{
\code{\link{thematicEvolution}} to perform thematic evolution analysis.

\code{\link{plotThematicEvolution}} to visualize the colored evolution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtag.R
\name{bibtag}
\alias{bibtag}
\title{Tag list and bibtex fields.}
\format{
A data frame with 44 rows and 6 variables:
 \describe{
    \item{TAG}{Tag Fields}
    \item{SCOPUS}{Scopus bibtex fields}
    \item{ISI}{WOS/ISI bibtex fields}
    \item{GENERIC}{Generic bibtex fields}
    \item{DIMENSIONS_OLD}{DIMENSIONS cvs/xlsx old fields}
    \item{DIMENSIONS}{DIMENSIONS cvs/xlsx fields}
    }
}
\description{
Data frame containing a list of tags and corresponding: WoS, SCOPUS and generic bibtex fields; and Dimensions.ai csv and xlsx fields.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{applyReferenceMatching}
\alias{applyReferenceMatching}
\title{Apply reference normalization to bibliometrix data frame}
\usage{
applyReferenceMatching(
  M,
  threshold = 0.9,
  method = "jw",
  min_chars = 20,
  max_block_size = 100,
  use_iso4 = TRUE,
  use_doi = TRUE,
  use_exact = TRUE,
  fuzzy = TRUE,
  use_postproc = TRUE,
  title_guard = FALSE
)
}
\arguments{
\item{M}{A bibliometrix data frame, typically created by \code{\link{convert2df}}.
Must contain the columns:
\itemize{
  \item \code{SR}: Short reference identifier for each document
  \item \code{CR}: Cited references field (citations separated by semicolons)
  \item \code{DB}: (Optional) Database source identifier for format detection
}}

\item{threshold}{Numeric value between 0 and 1 indicating the similarity threshold
for matching citations. Default is 0.90. See \code{\link{normalize_citations}}
for details on selecting appropriate thresholds.}

\item{method}{String distance method to use for fuzzy matching. Options include:
\itemize{
  \item "jw" (default): Jaro-Winkler distance, optimized for bibliographic strings
  \item "lv": Levenshtein distance
  \item "osa": Optimal String Alignment distance
  \item "lcs": Longest Common Subsequence distance
  \item Other methods supported by \code{\link[stringdist]{stringdistmatrix}}
}}

\item{min_chars}{Minimum characters for valid citations (default: 20).}

\item{max_block_size}{Integer. Blocks with at least this many unique normalized
strings skip within-block fuzzy matching and fall back to exact matching only
(default: 100). Passed to \code{\link{normalize_citations}}.}

\item{use_iso4}{Logical. Enable ISO 4 / LTWA journal normalization (default \code{TRUE}).}

\item{use_doi}{Logical. Enable DOI-based exact matching (default \code{TRUE}).}

\item{use_exact}{Logical. Enable exact normalized-string and punctuation-invariant
matching (default \code{TRUE}).}

\item{fuzzy}{Logical. Enable within-block (WoS deterministic and Scopus fuzzy)
matching (default \code{TRUE}).}

\item{use_postproc}{Logical. Enable Phase 4.5 metadata-based post-processing
merge (default \code{TRUE}). These five logical switches are intended primarily
for ablation analyses and are passed through to \code{\link{normalize_citations}}.}

\item{title_guard}{Logical. Enable the optional Phase 4.6 series part-marker
cluster purification that splits over-merged multi-part works (default
\code{FALSE}). Passed through to \code{\link{normalize_citations}}.}
}
\value{
A list with four elements:
\describe{
  \item{full_data}{A data frame with columns:
    \itemize{
      \item \code{SR}: Source document identifier
      \item \code{CR}: Original citation string
      \item \code{CR_canonical}: Canonical (normalized) citation
      \item \code{cluster_id}: Unique cluster identifier
      \item \code{n_cluster}: Size of the citation cluster
      \item \code{first_author}, \code{year}, \code{journal}, \code{volume}: Extracted metadata
    }
  }
  \item{summary}{A data frame summarizing citation frequencies with columns:
    \itemize{
      \item \code{CR_canonical}: The canonical citation for each cluster
      \item \code{n}: Total number of times this work was cited
      \item \code{n_variants}: Number of different formatting variants found
      \item \code{variants_example}: Sample of variant formats (up to 3 examples)
    }
    Sorted by citation frequency (n) in descending order.
  }
  \item{matched_citations}{Complete output from \code{\link{normalize_citations}},
    useful for detailed analysis of the matching process.}
  \item{CR_normalized}{A data frame with columns:
    \itemize{
      \item \code{SR}: Source document identifier
      \item \code{CR}: Reconstructed CR field with normalized citations (semicolon-separated)
      \item \code{n_references}: Number of unique references after normalization
    }
    This can be merged back with M to replace the original CR field.
  }
}
}
\description{
This is a convenience wrapper function that applies \code{\link{normalize_citations}}
to a bibliometrix data frame (typically loaded with \code{\link{convert2df}}). It
extracts citations from the CR field, performs normalization and matching, and
returns comprehensive results including per-paper citation lists and summary statistics.
}
\details{
The function automatically handles the new Scopus citation format (where the year
appears at the end in parentheses) by converting it to the classic format before
processing.


The function performs the following steps:
\enumerate{
  \item Splits the CR field by semicolons to extract individual citations
  \item Detects and converts new Scopus format citations to classic format
  \item Trims whitespace from each citation
  \item Applies \code{\link{normalize_citations}} to identify duplicate citations
  \item Links normalized citations back to source documents (SR)
  \item Generates summary statistics and reconstructs normalized CR fields
}

The normalized CR field can be used to replace the original CR field in subsequent
bibliometric analyses, ensuring that citation counts and network analyses are not
inflated by duplicate citations with minor formatting differences.
}
\examples{
\dontrun{
# Load bibliometric data
file <- "https://www.bibliometrix.org/datasets/savedrecs.txt"
M <- convert2df(file, dbsource = "wos", format = "plaintext")

# Apply reference normalization
results <- applyReferenceMatching(M, threshold = 0.85)

# View top cited works (after normalization)
head(results$summary, 20)

# See how many variants were found for the top citation
top_citation <- results$summary$CR_canonical[1]
variants <- subset(results$full_data, CR_canonical == top_citation)
unique(variants$CR)

# Replace original CR with normalized CR in the data frame
M_normalized <- M \%>\%
  rename(CR_orig = CR) \%>\%
  left_join(results$CR_normalized, by = "SR")

# Compare citation counts before and after normalization
original_citations <- strsplit(M$CR, ";") \%>\%
  unlist() \%>\%
  trimws() \%>\%
  table() \%>\%
  length()

normalized_citations <- nrow(results$summary)

cat("Original unique citations:", original_citations, "\n")
cat("After normalization:", normalized_citations, "\n")
cat("Duplicates found:", original_citations - normalized_citations, "\n")

# Use normalized data for further analysis
CR_analysis <- citations(M_normalized, field = "article", sep = ";")
}

}
\references{
Aria, M. & Cuccurullo, C. (2017). bibliometrix: An R-tool for comprehensive
science mapping analysis. Journal of Informetrics, 11(4), 959-975.
}
\seealso{
\code{\link{normalize_citations}} for the underlying normalization algorithm
\code{\link{citations}} for citation analysis
\code{\link{localCitations}} for local citation analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{convert_scopus_new_to_classic}
\alias{convert_scopus_new_to_classic}
\title{Convert new Scopus citation format to classic format}
\usage{
convert_scopus_new_to_classic(citation)
}
\arguments{
\item{citation}{Character string containing a bibliographic citation}
}
\value{
Character string with citation in classic Scopus format
}
\description{
Scopus has introduced a new citation format where the publication year appears
at the end in parentheses instead of after the title. This function converts
citations from the new format to the classic format by moving the year from
the end to after the title.
}
\details{
New Scopus format: AUTHOR, TITLE, JOURNAL, VOLUME, ISSUE, PAGES, (YEAR)
Classic Scopus format: AUTHOR, TITLE (YEAR) JOURNAL, VOLUME, PAGES

The function uses a robust approach:
\itemize{
  \item Extracts year from end (YYYY)
  \item Extracts first author from beginning
  \item Extracts pages (PP. xxx-xxx or PP. xxx)
  \item Extracts volume and issue numbers
  \item Extracts journal name (text before volume/issue/pages)
  \item Deduces title as remaining text after author
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countries.R
\name{countries}
\alias{countries}
\title{Index of Countries.}
\format{
A data frame with 202 rows and 5 variables:
 \describe{
    \item{countries}{country names}
    \item{continent}{continent names}
    \item{iso2}{country ISO 3166-1 alpha-2 code}
    \item{Longitude}{country centroid longitude}
    \item{Latitude}{country centroid latitude}
    }
}
\description{
Data frame containing a normalized index of countries.\cr
Data are used by \code{\link{biblioAnalysis}} function
to extract Country Field of Cited References and Authors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biblioshiny.R
\name{biblioshiny}
\alias{biblioshiny}
\title{Shiny UI for bibliometrix package}
\usage{
biblioshiny(
  host = "127.0.0.1",
  port = 3838,
  launch.browser = TRUE,
  maxUploadSize = 500,
  max.rows = Inf
)
}
\arguments{
\item{host}{The IPv4 address that the application should listen on.
Defaults to the shiny.host option, if set, or "127.0.0.1" if not.}

\item{port}{is the TCP port that the application should listen on. If the port is not specified,
and the shiny.port option is set (with options(shiny.port = XX)), then that port will be used.
Otherwise, use the default port 3838.}

\item{launch.browser}{If true, the system's default web browser will be launched automatically
after the app is started. Defaults to true in interactive sessions only. This value of
this parameter can also be a function to call with the application's URL.}

\item{maxUploadSize}{is a integer. The max upload file size argument. Default value is 200 (megabyte)}

\item{max.rows}{is an integer or Inf. The maximum number of records (publications) allowed
when importing or loading a file. Files exceeding this limit will be rejected with an
informative message. Default value is \code{Inf} (no limit).}
}
\description{
\code{biblioshiny} performs science mapping analysis using the main functions of the bibliometrix package.
}
\examples{

# biblioshiny()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{create_journal_iso4_lookup}
\alias{create_journal_iso4_lookup}
\title{Create ISO4 journal normalization lookup table}
\usage{
create_journal_iso4_lookup(journal_vector, ltwa_db)
}
\arguments{
\item{journal_vector}{Character vector of journal names}

\item{ltwa_db}{LTWA database data frame}
}
\value{
Data frame with journal_original and journal_iso4 columns
}
\description{
Create ISO4 journal normalization lookup table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotThematicEvolution.R
\name{plotThematicEvolution}
\alias{plotThematicEvolution}
\title{Plot a Thematic Evolution Analysis}
\usage{
plotThematicEvolution(Nodes, Edges, measure = "lineage_strength", min.flow = 0)
}
\arguments{
\item{Nodes}{is a list of nodes obtained by \code{\link{thematicEvolution}} function.}

\item{Edges}{is a list of edges obtained by \code{\link{thematicEvolution}} function.}

\item{measure}{is a character. It is deprecated and has no effect. The weight of the flows is always calculated using lineage_strength.}

\item{min.flow}{is numerical. It indicates the minimum value of measure to plot a flow.}
}
\value{
a sankeyPlot
}
\description{
It plot a Thematic Evolution Analysis performed using the \code{\link{thematicEvolution}} function.
}
\examples{

\dontrun{
data(management, package = "bibliometrixData")
years=c(2004,2008,2015)

nexus <- thematicEvolution(management,field="DE",years=years,n=100,minFreq=2)

plotThematicEvolution(nexus$Nodes,nexus$Edges)
}

}
\seealso{
\code{\link{thematicMap}} function to create a thematic map based on co-word network analysis and clustering.

\code{\link{thematicEvolution}} function to perform a thematic evolution analysis.

\code{\link{networkPlot}} to plot a bibliographic network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeCItationScore.R
\name{normalizeCitationScore}
\alias{normalizeCitationScore}
\title{Calculate the normalized citation score metric}
\usage{
normalizeCitationScore(M, field = "documents", impact.measure = "local")
}
\arguments{
\item{M}{is a bibliographic data frame obtained by \code{\link{convert2df}} function.}

\item{field}{is a character. It indicates the unit of analysis on which calculate the NCS. It can be equal to \code{field = c("documents", "authors", "sources")}. Default is \code{field = "documents"}.}

\item{impact.measure}{is a character. It indicates the impact measure used to rank cluster elements (documents, authors or sources).
It can be \code{impact.measure = c("local", "global")}.\\
With \code{impact.measure = "local"}, \link{normalizeCitationScore} calculates elements impact using the Normalized Local Citation Score while
using \code{impact.measure = "global"}, the function uses the Normalized Global Citation Score to measure elements impact.}
}
\value{
a dataframe.
}
\description{
It calculates the normalized citation score for documents, authors and sources using both global and local citations.
}
\details{
The document Normalized Citation Score (NCS) of a document is calculated by dividing the actual count of citing items by the expected
citation rate for documents with the same year of publication.

The MNCS of a set of documents, for example the collected works of an individual, or published on a journal, is the average of the NCS values for all the documents in the set.

The NGCS is the NCS calculated using the global citations (total citations that a document received considering the whole bibliographic database).

The NLCS is the NCS calculated using the local citations (total citations that a document received from a set of documents included in the same collection).
}
\examples{
\dontrun{
data(management, package = "bibliometrixData")
NCS <- normalizeCitationScore(management, field = "authors", impact.measure = "local")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim.R
\name{trim}
\alias{trim}
\title{Deleting leading and ending white spaces}
\usage{
trim(x)
}
\arguments{
\item{x}{is a \code{character} object.}
}
\value{
an object of class \code{character}
}
\description{
Deleting leading and ending white spaces from a \code{character} object.
}
\details{
\code{tableTag} is an internal routine of \code{bibliometrics} package.
}
\examples{

char <- c("  Alfred", "Mary", " John")
char
trim(char)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findAuthorWorks.r
\name{findAuthorWorks}
\alias{findAuthorWorks}
\title{Find Author's Co-authored Works}
\usage{
findAuthorWorks(author_name, data, partial_match = TRUE, exact_match = FALSE)
}
\arguments{
\item{author_name}{Character. The author's name to search for (case-insensitive)}

\item{data}{Data.frame. The bibliometric dataframe with AU and DI columns}

\item{partial_match}{Logical. If TRUE, allows partial name matching (default: TRUE)}

\item{exact_match}{Logical. If TRUE, requires exact name matching (default: FALSE)}
}
\value{
A data.frame with columns:
  \itemize{
    \item doi: DOI of the work
    \item author_position: Numerical position of the author in the author list
    \item total_authors: Total number of authors in the work
    \item all_authors: Complete list of authors for reference
    \item matched_name: The exact name variant that was matched
  }
}
\description{
Searches for an author's name in a bibliometric dataframe and returns 
the DOIs and author positions of their co-authored works.
}
\details{
The function searches through the AU column which contains author names 
separated by semicolons. It identifies the position of the target author
and returns comprehensive information about each matching work.
}
\examples{
\dontrun{
# Find works by "ARIA M"
works <- findAuthorWorks("ARIA M", M)

# Find works with exact matching
works_exact <- findAuthorWorks("PESTANA MH", M, exact_match = TRUE)

# Find works with partial matching disabled
works_full <- findAuthorWorks("MASSIMO ARIA", M, partial_match = FALSE)
}

}
\author{
Your Name
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{mergeKeywords}
\alias{mergeKeywords}
\title{Merge DE and ID Fields into a Unified Keywords Column}
\usage{
mergeKeywords(M, force = FALSE)
}
\arguments{
\item{M}{A dataframe containing at least the `DE` and/or `ID` columns, typically generated by `convert2df()` from the `bibliometrix` package.}

\item{force}{Logical. If `TRUE`, an existing `KW_Merged` column will be overwritten. Default is `FALSE`.}
}
\value{
A dataframe with an added (or updated) `KW_Merged` column containing deduplicated and cleaned keyword strings.
}
\description{
This function creates a new column `KW_Merged` by combining the contents of the `DE` (author keywords) and `ID` (keywords plus) fields
in a bibliographic dataframe. Duplicate keywords within each record are removed, and leading/trailing spaces are trimmed.
The merged keywords are separated by a semicolon (`;`).
}
\details{
If the `KW_Merged` column already exists, it will not be overwritten unless `force = TRUE` is specified.
}
\examples{
\dontrun{
data(management, package = "bibliometrix")
M <- mergeKeywords(management)
head(M$KW_Merged)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{normalize_journal_to_iso4}
\alias{normalize_journal_to_iso4}
\title{Normalize journal names to ISO4 abbreviated form}
\usage{
normalize_journal_to_iso4(journal_name, ltwa_db)
}
\arguments{
\item{journal_name}{Character string with journal name}

\item{ltwa_db}{Data frame with LTWA database}
}
\value{
Normalized journal name in ISO4 abbreviated form
}
\description{
Converts all journal names to their ISO4 abbreviated form using LTWA.
Only uses English language entries from LTWA to avoid foreign word matches.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dominance.R
\name{dominance}
\alias{dominance}
\title{Authors' dominance ranking}
\usage{
dominance(results, k = 10)
}
\arguments{
\item{results}{is an object of the class '\code{bibliometrix}' for which the analysis of the authors' dominance ranking is desired.}

\item{k}{is an integer, used for table formatting (number of authors). Default value is 10.}
}
\value{
The function \code{dominance} returns a data frame with cases corresponding to the first \code{k} most productive authors and variables to typical field of a dominance analysis.

the data frame variables are:
\tabular{lll}{
\code{Author} \tab   \tab Author's name\cr
\code{Dominance Factor}  \tab   \tab Dominance Factor (DF = FAA / MAA)\cr
\code{Tot Articles}   \tab   \tab N. of Authored Articles (TAA)\cr
\code{Single Authored}   \tab   \tab N. of Single-Authored Articles (SAA)\cr
\code{Multi Authored}   \tab   \tab N. of Multi-Authored Articles (MAA=TAA-SAA)\cr
\code{First Authored} \tab   \tab N. of First Authored Articles (FAA)\cr
\code{Rank by Articles}    \tab   \tab Author Ranking by N. of Articles\cr
\code{Rank by DF}    \tab   \tab Author Ranking by Dominance Factor}
}
\description{
It calculates the authors' dominance ranking from an object of the class '\code{bibliometrix}' as proposed by Kumar & Kumar, 2008.
}
\examples{
data(scientometrics, package = "bibliometrixData")
results <- biblioAnalysis(scientometrics)
DF <- dominance(results)
DF

}
\seealso{
\code{\link{biblioAnalysis}} function for bibliometric analysis

\code{\link{summary}} method for class '\code{bibliometrix}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert2df.R
\name{convert2df}
\alias{convert2df}
\title{Import and Convert bibliographic export files and API objects.}
\usage{
convert2df(
  file,
  dbsource = "wos",
  format = "plaintext",
  remove.duplicates = TRUE
)
}
\arguments{
\item{file}{a character array containing a sequence of filenames coming from WoS, Scopus, Dimensions, Lens.org, OpenAlex and Pubmed. Alternatively, \code{file} can be
an object resulting from an API query fetched from Dimensions, and PubMed databases:
\tabular{lll}{
a)\tab 'wos' \tab Clarivate Analytics WoS (in plaintext '.txt', Endnote Desktop '.ciw', or bibtex formats '.bib');\cr
b)\tab 'scopus' \tab SCOPUS (exclusively in bibtex format '.bib');\cr
c)\tab 'dimensions' \tab Digital Science Dimensions (in csv '.csv' or excel '.xlsx' formats);\cr
d)\tab 'lens' \tab Lens.org (in csv '.csv');\cr
e)\tab 'pubmed' \tab an object of the class \code{pubmedR (package pubmedR)} containing a collection obtained from a query performed with pubmedR package;\cr
f)\tab 'dimensions' \tab an object of the class \code{dimensionsR (package dimensionsR)} containing a collection obtained from a query performed with dimensionsR package;\cr
g)\tab 'openalex' \tab OpenAlex .csv file;\cr
h)\tab 'openalex_api' \tab the filename and path to a list object returned by openalexR package, containing a collection of works resulting from a query fetched from OpenAlex database.}}

\item{dbsource}{is a character indicating the bibliographic database. \code{dbsource} can be \code{dbsource = c('cochrane','dimensions','generic','isi','openalex', 'pubmed','scopus','wos', 'lens')} . Default is \code{dbsource = "isi"}.}

\item{format}{is a character indicating the SCOPUS, Clarivate Analytics WoS, and other databases export file format. \code{format} can be \code{c('api', 'bibtex', 'csv', 'endnote','excel','plaintext', 'pubmed')}. Default is \code{format = "plaintext"}.}

\item{remove.duplicates}{is logical. If TRUE, the function will remove duplicated items checking by DOI and database ID.}
}
\value{
a data frame with cases corresponding to articles and variables to Field Tags in the original export file.

I.e We have three files download from Web of Science in plaintext format, file will be:

file <- c("filename1.txt", "filename2.txt", "filename3.txt")

data frame columns are named using the standard Clarivate Analytics WoS Field Tag codify. The main field tags are:

\tabular{lll}{
\code{AU}\tab   \tab Authors\cr
\code{TI}\tab   \tab Document Title\cr
\code{SO}\tab   \tab Publication Name (or Source)\cr
\code{JI}\tab   \tab ISO Source Abbreviation\cr
\code{DT}\tab   \tab Document Type\cr
\code{DE}\tab   \tab Authors' Keywords\cr
\code{ID}\tab   \tab Keywords associated by SCOPUS or WoS database \cr
\code{AB}\tab   \tab Abstract\cr
\code{C1}\tab   \tab Author Address\cr
\code{RP}\tab   \tab Reprint Address\cr
\code{CR}\tab   \tab Cited References\cr
\code{TC}\tab   \tab Times Cited\cr
\code{PY}\tab   \tab Year\cr
\code{SC}\tab   \tab Subject Category\cr
\code{UT}\tab   \tab Unique Article Identifier\cr
\code{DB}\tab   \tab Database\cr}

for a complete list of field tags see: \href{https://www.bibliometrix.org/documents/Field_Tags_bibliometrix.pdf}{Field Tags used in bibliometrix}
}
\description{
It converts a SCOPUS, Clarivate Analytics WoS, Dimensions, Lens.org, PubMed and COCHRANE Database export files or pubmedR and dimensionsR JSON/XML
objects into a data frame, with cases corresponding to articles and variables to Field Tags as used in WoS.
}
\examples{

# Example:
# Import and convert a Web of Science collection form an export file in plaintext format:

\dontrun{
files <- "https://www.bibliometrix.org/datasets/wos_plaintext.txt"

M <- convert2df(file = files, dbsource = "wos", format = "plaintext")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{abbreviate_journal_title}
\alias{abbreviate_journal_title}
\title{Abbreviate journal title to ISO 4 standard}
\usage{
abbreviate_journal_title(title, ltwa_lookup)
}
\arguments{
\item{title}{Journal title string}

\item{ltwa_lookup}{Pre-processed LTWA lookup tables (from prepare_ltwa_lookup)}
}
\value{
Abbreviated journal title in ISO 4 format (without periods)
}
\description{
Converts a full journal title to its ISO 4 abbreviated form using LTWA.
Removes articles, prepositions, and conjunctions according to ISO 4 rules.
Returns result WITHOUT periods (dots).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localCitations.R
\name{localCitations}
\alias{localCitations}
\title{Author local citations}
\usage{
localCitations(M, fast.search = FALSE, sep = ";", verbose = FALSE)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{fast.search}{is logical. If true, the function calculates local citations only for 25 percent top cited documents.}

\item{sep}{is the field separator character. This character separates citations in each string of CR column of the bibliographic data frame. The default is \code{sep = ";"}.}

\item{verbose}{is a logical.  If TRUE, results are printed on screen.}
}
\value{
an object of \code{class} "list" containing author local citations and document local citations.
}
\description{
It calculates local citations (LCS) of authors and documents of a bibliographic collection.
}
\details{
Local citations measure how many times an author (or a document) included in this collection have been cited by the documents also included in the collection.
}
\examples{

data(scientometrics, package = "bibliometrixData")

CR <- localCitations(scientometrics, sep = ";")

CR$Authors[1:10, ]
CR$Papers[1:10, ]

}
\seealso{
\code{\link{citations}} function for citation frequency distribution.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/authorBio.r
\name{authorBio}
\alias{authorBio}
\title{Retrieve Author Biographical Information from OpenAlex}
\usage{
authorBio(
  author_position = 1,
  doi = "10.1016/j.joi.2017.08.007",
  verbose = FALSE,
  return_all_authors = FALSE,
  sleep_time = 1,
  max_retries = 3,
  retry_delay = 2,
  mailto = NULL
)
}
\arguments{
\item{author_position}{Integer. The numerical position of the author in the authors list (default: 1)}

\item{doi}{Character. DOI of the article used to identify the authors}

\item{verbose}{Logical. Print informative messages during execution (default: FALSE)}

\item{return_all_authors}{Logical. If TRUE, returns information for all co-authors (default: FALSE)}

\item{sleep_time}{Numeric. Seconds to wait between API calls to respect rate limits (default: 1)}

\item{max_retries}{Integer. Maximum number of retry attempts for failed API calls (default: 3)}

\item{retry_delay}{Numeric. Base delay in seconds before retrying after an error (default: 2)}

\item{mailto}{Character. Email address for OpenAlex polite pool (default: NULL, uses openalexR.mailto option if set)}
}
\value{
If \code{return_all_authors = FALSE}, returns a tibble with comprehensive information 
  about the specified author including:
  \itemize{
    \item Basic information (name, ORCID, OpenAlex ID)
    \item Bibliometric indicators (works count, citations, h-index, i10-index)
    \item Affiliation details from both the paper and author profile
    \item Research topics and areas
    \item Paper-specific metadata (corresponding author status, position type)
  }
  If \code{return_all_authors = TRUE}, returns a list of tibbles, one for each co-author.
}
\description{
This function downloads comprehensive author information from OpenAlex based on a DOI 
and the numerical position of the author in the co-authors list. It provides detailed 
biographical data, bibliometric indicators, and affiliation information.
}
\details{
The function first retrieves the work information using the provided DOI, then extracts
author IDs from the authorships data, and finally fetches detailed author profiles from
OpenAlex. It enriches the author data with paper-specific information such as authorship
position, corresponding author status, and affiliations as listed in the paper.

The function implements automatic retry logic with exponential backoff to handle rate limiting
(HTTP 429 errors) and temporary network issues. It respects OpenAlex API rate limits by adding
configurable delays between requests.

IMPORTANT: For better rate limits, set your OpenAlex API key using:
Sys.setenv(openalexR_apikey = "YOUR_API_KEY")
Get a free API key at: https://openalex.org/
}
\examples{
\dontrun{
# Get information for the first author
first_author <- authorBio(doi = "10.1016/j.joi.2017.08.007")

# Get information for the second author with verbose output
second_author <- authorBio(
  author_position = 2, 
  doi = "10.1016/j.joi.2017.08.007", 
  verbose = TRUE
)

# Get information for all co-authors with custom rate limiting
all_authors <- authorBio(
  doi = "10.1016/j.joi.2017.08.007", 
  return_all_authors = TRUE,
  sleep_time = 0.5,
  max_retries = 5
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.bibliometrix_netstat.R
\name{summary.bibliometrix_netstat}
\alias{summary.bibliometrix_netstat}
\title{Summarizing network analysis results}
\usage{
\method{summary}{bibliometrix_netstat}(object, ...)
}
\arguments{
\item{object}{is the object for which a summary is desired.}

\item{...}{can accept two arguments:\cr
\code{k} integer, used for table formatting (number of rows). Default value is 10.\cr}
}
\value{
The function \code{summary} computes and returns on display several statistics both at network and vertex level.
}
\description{
\code{summary} method for class '\code{bibliometrix_netstat}'
}
\examples{

# to run the example, please remove # from the beginning of the following lines
# data(scientometrics, package = "bibliometrixData")

# NetMatrix <- biblioNetwork(scientometrics, analysis = "collaboration",
#                   network = "authors", sep = ";")
# netstat <- networkStat(NetMatrix, stat = "all", type = "degree")
# summary(netstat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hindex.R
\name{Hindex}
\alias{Hindex}
\title{h-index calculation}
\usage{
Hindex(M, field = "author", elements = NULL, sep = ";", years = Inf)
}
\arguments{
\item{M}{is a bibliographic data frame obtained by the converting function \code{\link{convert2df}}.
It is a data matrix with cases corresponding to manuscripts and variables to Field Tag in the original SCOPUS and Clarivate Analytics WoS file.}

\item{field}{is character. It can be equal to c("author", "source"). field indicates if H-index have to be calculated for a list of authors or for a list of sources. Default
value is \code{field = "author"}.}

\item{elements}{is a character vector. It contains the authors' names list or the source list for which you want to calculate the H-index. When the field is
"author", the argument has the form C("SURNAME1 N","SURNAME2 N",...), in other words, for each author: surname and initials separated by one blank space. If elements=NULL, the function calculates impact indices for all elements contained in the data frame.
i.e for the authors SEMPRONIO TIZIO CAIO and ARIA MASSIMO \code{elements} argument is \code{elements = c("SEMPRONIO TC", "ARIA M")}.}

\item{sep}{is the field separator character. This character separates authors in each string of AU column of the bibliographic data frame. The default is \code{sep = ";"}.}

\item{years}{is a integer. It indicates the number of years to consider for Hindex calculation. Default is Inf.}
}
\value{
an object of \code{class} "list". It contains two elements: H is a data frame with h-index, g-index and m-index for each author; CitationList is a list with the bibliographic collection for each author.
}
\description{
It calculates the authors' h-index and its variants.
}
\examples{

### EXAMPLE 1: ###

data(scientometrics, package = "bibliometrixData")

authors <- c("SMALL H", "CHEN DZ")

Hindex(scientometrics, field = "author", elements = authors, sep = ";")$H

Hindex(scientometrics, field = "source", elements = "SCIENTOMETRICS", sep = ";")$H

### EXAMPLE 2: Garfield h-index###

data(garfield, package = "bibliometrixData")

indices <- Hindex(garfield, field = "author", elements = "GARFIELD E", years = Inf, sep = ";")

# h-index, g-index and m-index of Eugene Garfield
indices$H

# Papers and total citations
head(indices$CitationList[[1]])

}
\seealso{
\code{\link{convert2df}} to import and convert an WoS or SCOPUS Export file in a bibliographic data frame.

\code{\link{biblioAnalysis}} function for bibliometric analysis.

\code{\link{summary}} to obtain a summary of the results.

\code{\link{plot}} to draw some useful plots of the results.
}
